      SUBROUTINE ANTENNA_TABLE (NS,STNAME,STX,STY,STZ,IER)
C-----------------------------------------------------------------------
C Append AIPS antenna table as a FITS table extension to the FITS file.
C Used by PHFITS and MERGEFITS.
C
C Arguments:
C  NS      (input, I*4)       Number of antennas.
C  STNAME  (input, C*8 array) Names of antennas.
C  STX     (input, R*8 array) Antenna x-coordinates (m).
C  STY     (input, R*8 array) Antenna y-coordinates (m).
C  STZ     (input, R*8 array) Antenna z-coordinates (m).
C  IER     (output, I*4)      Error flag: 0 => OK, non-zero indicates
C                             an error in the FITS routines.
C-----------------------------------------------------------------------
      INTEGER      IER, IS, NS
      CHARACTER*(*) STNAME(NS)
      DOUBLE PRECISION       STX(NS), STY(NS), STZ(NS)
      CHARACTER*80 HISTRY
C-----------------------------------------------------------------------
      CALL FITS_FLUSH(IER)
      CALL FITS_WSTR ('XTENSION', 'TABLE', 'EXTENSION TYPE', IER)
      CALL FITS_WINT ('BITPIX', 8, 'PRINTABLE ASCII CODES', IER)
      CALL FITS_WINT ('NAXIS',  2, 'TABLE IS A MATRIX', IER)
      CALL FITS_WINT ('NAXIS1', 80, 'WIDTH OF TABLE IN CHARACTERS', IER)
      CALL FITS_WINT ('NAXIS2', NS, 'NUMBER OF ENTRIES IN TABLE', IER)
      CALL FITS_WINT ('PCOUNT', 0, 'NO RANDOM PARAMETERS', IER)
      CALL FITS_WINT ('GCOUNT', 1, 'GROUP COUNT', IER)
      CALL FITS_WINT ('TFIELDS', 5, 'NUMBER OF FIELDS IN EACH ROW', IER)
      CALL FITS_WSTR ('EXTNAME', 'AIPS AN', 'AIPS ANTENNA TABLE', IER)
      CALL FITS_WINT ('EXTVER', 1, 'VERSION NUMBER OF TABLE', IER)
C
      CALL FITS_WINT ('TBCOL1', 1, 'STARTING COLUMN', IER)
      CALL FITS_WSTR ('TFORM1', 'I3', 'FORTRAN FORMAT', IER)
      CALL FITS_WSTR ('TTYPE1', 'ANT NO.', 'ANTENNA NUMBER', IER)
C
      CALL FITS_WINT ('TBCOL2', 7, 'STARTING COLUMN', IER)
      CALL FITS_WSTR ('TFORM2', 'A8', 'FORTRAN FORMAT', IER)
      CALL FITS_WSTR ('TTYPE2', 'STATION', 'ANTENNA NAME', IER)
C
      CALL FITS_WINT ('TBCOL3', 15, 'STARTING COLUMN', IER)
      CALL FITS_WSTR ('TFORM3', 'D20.10', 'FORTRAN FORMAT', IER)
      CALL FITS_WSTR ('TTYPE3', 'LX', 'ANTENNA X COORDINATE', IER)
      CALL FITS_WSTR ('TUNIT3', 'METERS', 'PHYSICAL UNITS', IER)
      CALL FITS_WREAL('TSCAL3', 1.0, ' ', IER)
      CALL FITS_WREAL('TZERO3', 0.0, ' ', IER)
C
      CALL FITS_WINT ('TBCOL4', 35,  'STARTING COLUMN', IER)
      CALL FITS_WSTR ('TFORM4', 'D20.10', 'FORTRAN FORMAT', IER)
      CALL FITS_WSTR ('TTYPE4', 'LY', 'ANTENNA Y COORDINATE', IER)
      CALL FITS_WSTR ('TUNIT4', 'METERS', 'PHYSICAL UNITS', IER)
      CALL FITS_WREAL('TSCAL4', 1.0, ' ', IER)
      CALL FITS_WREAL('TZERO4', 0.0, ' ', IER)
C
      CALL FITS_WINT ('TBCOL5', 55, 'STARTING COLUMN', IER)
      CALL FITS_WSTR ('TFORM5', 'D20.10', 'FORTRAN FORMAT', IER)
      CALL FITS_WSTR ('TTYPE5', 'LZ', 'ANTENNA Z COORDINATE', IER)
      CALL FITS_WSTR ('TUNIT5', 'METERS', 'PHYSICAL UNITS', IER)
      CALL FITS_WREAL('TSCAL5', 1.0, ' ', IER)
      CALL FITS_WREAL('TZERO5', 0.0, ' ', IER)
C
      CALL FITS_WEND (IER)
      DO 10 IS=1,NS
          WRITE (HISTRY,'(I3,3X,A8,3D20.10)')
     1            IS, STNAME(IS), STX(IS), STY(IS), STZ(IS)
          CALL FITS_WRAYB(%REF(HISTRY),80,IER)
   10 CONTINUE
C-----------------------------------------------------------------------
      END
