      SUBROUTINE FITS_WRAYI (DATA,NELM,STATUS)
C-----------------------------------------------------------------------
C     Writes an integer array to output file as part of a FITS image.
C
C     This routine simply copies the array element by element into a
C     buffer, writing the buffer out whenever it fills.  This means
C     that this routine may be called as many times as necessary - for
C     example, if all the data is not available in memory at one time.
C     This routine should be called for the first time immediately
C     following the FITS_WEND call that terminated the FITS header.  Once
C     all the data is output, FITS_CLOSE should be called to flush out
C     the data buffer.
C
C     Parameters -   (">" input, "W" workspace, "!" modified, "<" output)
C
C     (>) DATA     (Integer array DATA(NELM)) The data array to be output.
C                  DATA may be multi-dimensional, but it is treated
C                  as linear here for generality.
C     (>) NELM     (Integer) The number of elements in DATA.
C     (<) STATUS   (Integer) Returned status code.  0 => OK, non-zero
C                  values indicate an I/O error and can be
C                  decoded using FITS_ERROR.
C
C     Common variables used -
C
C     (W) FBUFF    (Byte array) The main FITS I/O buffer
C     (W) FBUFFI   (Integer array) The same buffer as FBUFF.
C     (!) FPTR     (Integer) Byte level pointer to FBUFF.
C
C     Subroutines / functions used -
C
C     FITS_WBSWAP   (FITS_ package) Swap order of bytes in longwords
C     FITS_MWRIT    (FITS_    "   ) Write FBUFF buffer to output file
C
C                                       KS / CIT 19th Oct 1983
C-----------------------------------------------------------------------
C
      INTEGER NELM,STATUS,DATA(NELM)
C
      INCLUDE 'fits1.inc'
C
      INTEGER FPTRI,FPTRO,I
C
C     Write out as signed 32 bit integers.  The call to FITS_WBSWAP
C     is needed because the VAX byte order is not that required
C     by the FITS standard.
C
      STATUS=0
      FPTRI=FPTR/4+1
      FPTRO=FPTRI
      DO I=1,NELM
         FBUFFI(FPTRI)=DATA(I)
         FPTRI=FPTRI+1
         IF (FPTRI.GT.720) THEN
            CALL FITS_WBSWAP(FBUFFI(FPTRO),FPTRI-FPTRO)
            CALL FITS_MWRIT(STATUS)
            FPTR=1
            FPTRI=1
            FPTRO=1
            IF (STATUS.NE.0) GO TO 600
         END IF
      END DO
      IF (FPTRI.GT.1) CALL FITS_WBSWAP(FBUFFI(FPTRO),FPTRI-FPTRO)
      FPTR=(FPTRI-1)*4+1
C
  600 CONTINUE
      END
