      SUBROUTINE FITS_WRAYS (DATA,NELM,STATUS)
C-----------------------------------------------------------------------
C     Writes an integer*2 array to output file as part of a FITS image.
C
C     This routine simply copies the array element by element into a
C     buffer, writing the buffer out whenever it fills.  This means
C     that this routine may be called as many times as necessary - for
C     example, if all the data is not available in memory at one time.
C     This routine should be called for the first time immediately
C     following the FITS_WEND call that terminated the FITS header.  Once
C     all the data is output, FITS_CLOSE should be called to flush out
C     the data buffer.
C
C     Parameters -   (">" input, "W" workspace, "!" modified, "<" output)
C
C     (>) DATA     (Integer*2 array DATA(NELM)) The data array to be output.
C                  DATA may be multi-dimensional, but it is treated
C                  as linear here for generality.
C     (>) NELM     (Integer) The number of elements in DATA.
C     (<) STATUS   (Integer) Returned status code.  0 => OK, non-zero
C                  values indicate an I/O error and can be
C                  decoded using FITS_ERROR.
C
C     Common variables used -
C
C     (W) FBUFF    (Byte array) The main FITS I/O buffer
C     (W) FBUFFS   (Integer*2 array) The same buffer as FBUFF.
C     (!) FPTR     (Integer) Byte level pointer to FBUFF.
C
C     Subroutines / functions used -
C
C     FITS_BSWAP    (FITS_ package) Swap order of bytes in words
C     FITS_MWRIT    (FITS_    "   ) Write FBUFF buffer to output
C
C                                       KS / CIT 19th Oct 1983
C-----------------------------------------------------------------------
C
      INTEGER NELM,STATUS
      INTEGER*2 DATA(NELM)
C
      INCLUDE 'fits1.inc'
C
      INTEGER FPTRO,FPTRS,I
C
C     Write out as signed 16 bit integers.  The call to FITS_BSWAP
C     is needed because the VAX byte order is not that required
C     by the FITS standard.
C
      STATUS=0
      FPTRS=FPTR/2+1
      FPTRO=FPTRS
      DO I=1,NELM
         FBUFFS(FPTRS)=DATA(I)
         FPTRS=FPTRS+1
         IF (FPTRS.GT.1440) THEN
            CALL FITS_BSWAP(FBUFFS(FPTRO),FPTRS-FPTRO)
            CALL FITS_MWRIT(STATUS)
            FPTR=1
            FPTRS=1
            FPTRO=1
            IF (STATUS.NE.0) GO TO 600
         END IF
      END DO
      IF (FPTRS.GT.1) CALL FITS_BSWAP(FBUFFS(FPTRO),FPTRS-FPTRO)
      FPTR=(FPTRS-1)*2+1
C
  600 CONTINUE
      END
