      SUBROUTINE FITS_WSTR (NAME,STRING,COMMENT,STATUS)
C-----------------------------------------------------------------------
C     Writes a string keyword into a FITS header.  Assumes 
C     FITS_INIT has been called to initialise the header routines.
C
C     Parameters -  (">" input, "<" output)
C
C     (>) NAME      (Character) The name of the keyword.
C     (>) STRING    (Character) The value of the keyword - this routine
C                   appends the quote marks required by the standard, so
C                   these should not be included in STRING.
C     (>) COMMENT   (Character) Any comment to be associated with the
C                   keyword.  Normally the comment is started at col 34
C                   and so should be limited to 46 characters.  If the
C                   keyword itself runs over past col 30, the comment
C                   will have to be shorter.
C     (>) STATUS    (Integer) Return status code. 0 => OK, non-zero
C                   values will be error codes caused by an
C                   I/O error.  These can be decoded by FITS_ERROR.
C
C     Note: The FITS standard says keyword names and values must be
C     output in upper case.  NAME & STRING can be passed in lower case, 
C     but will be output in upper.  COMMENT will be output as passed.
C
C     Common variables used -
C
C     (<) BUFF      (Character) Buffer for one header line
C
C     Functions / subroutines used -
C
C     FITS_BUFFO     (FITS_ package) Outputs BUFF to main I/O buffer.
C     UPCASE         Convert string to upper case.
C
C                                                KS / CIT  15th Nov 1983
C-----------------------------------------------------------------------
C
      INTEGER STATUS
      CHARACTER*(*) NAME,COMMENT,STRING
C
      INTEGER IBUF,ISTR,LPTR
      CHARACTER CHR
C
      INCLUDE 'fits1.inc'
C
      BUFF=NAME
      CALL UPCASE(BUFF(1:8))
      BUFF(9:11)='= '''
      ISTR=1
      IBUF=12
      LPTR=12
      DO WHILE ((ISTR.LE.LEN(STRING)).AND.(IBUF.LT.LEN(BUFF)))
         CHR=STRING(ISTR:ISTR)
         CALL UPCASE(CHR)
         BUFF(IBUF:IBUF)=CHR
         IF (CHR.NE.' ') LPTR=IBUF
         IBUF=IBUF+1
         ISTR=ISTR+1
      END DO
      BUFF(MAX(20,LPTR+1):)=''''
      BUFF(MAX(32,LPTR+1):)='/ '//COMMENT
      CALL FITS_BUFFO(STATUS)
C
      END
