C
C      ***************
C      *             *
C      *  T A P O S  *
C      *             *
C      ***************
C
C      This is a general purpose interactive program that
C      allows the user to move around a tape and examine
C      its contents.
C
C      The program prompts for the tape drive to be used,
C      assigns a channel to it, and then waits for commands.
C
C      The following commands are accepted -
C
C      RW    Rewind tape
C      EO n  Write n end-of-file marks
C      BF n  Backspace over n file marks
C      FF n  Forward space over n file marks
C      BR n  Backspace over n records
C      FR n  Forward space over n records
C      RL n  Read n records and list record lengths
C      RA n  Read n records and display in ASCII
C      RE n  Read n records and display in EBCDIC
C      RX n  Read n records and display in hex
C      LE n  Set max displayed length to n bytes
C      PR    List records on file for printing
C      XP    Revert to listing on terminal
C      DI    Display information about tape
C      HELP (HE)  Give this information
C      TAPE (TA)  Change the tape in use
C      QUIT (QU)  Quit the program
C
C      For most functions ''n'' defaults to 1'
C      For LE, n defaults to 0, ie list all of records'
C-
C      Subroutines / functions used -
C
C      LIB$SYS_TRNLOG  (VAX standard) Translate logical name
C      LIB$TRA_EBC_ASC ( "     "    ) Convert EBCDIC -> ASCII
C      MVBITS (VAX Fortran extension) Get bits from an integer
C      BTEST  ( "    "         "    ) Test a bit
C      INCOM          (KS - Appended) Decode input command
C      MTGETN    (MTPCKG) Get name of an available tape drive
C      MTOPID    (  "   ) Open a named tape drive
C      MTMESS    (  "   ) Produce an error message for MTPCKG
C      MTSENS    (  "   ) Sense tape status
C      MTREWI    (  "   ) Rewind tape
C      MTVMER    (  "   ) Get VMS System service error message
C      MTWEOF    (  "   ) Write a file mark on tape
C      MTNAME    (  "   ) Get name of an opened tape
C      MTSKFI    (  "   ) Skip over file marks on tape
C      MTREAD    (  "   ) Read a record from tape
C      ICH_CLEAN (ICH_ package) Clean up a character string
C      ICH_FOLD  ( "     "    ) Convert string to upper case
C      ICH_KEY   ( "     "    ) Look for a keyword in string
C      ICN_TIDY  ( "     "    ) Tidy up a characters in a numeric array
C      ICH_TIDY  ( "     "    ) Tidy up a character string
C
C                                    KS / AAO 18th Oct 1984
C      Modified:
C
C      17th Mar 1988  CKL / CIT. Converted to Convex Unix.
C      17th Oct 1988  KS/AAO Buffer size increased, VT terminals allowed.
C      13th Dec 1989  SNS / CIT. Convex & VMS versions merged.
C      29th Mar 1990  SNS / CIT. Now uses unix version numbers */
C+
C
C     MTPCKG error codes
C
#ifdef VMS
      INCLUDE 'MTERCS.INS'
#else
#include "mtercs.ins"
#endif
C
C     Local variables
C
#ifdef VMS
      BYTE BUFFER(28800)
#else
      BYTE BUFFER(32767)
#endif
      LOGICAL TPEASS,NOFILE,PRSET,BTEST
      CHARACTER*(20) LINE
      INTEGER CR,STATUS,SIZE,RECORD,FLAGS,PRFILE
      CHARACTER TCHAR
      CHARACTER*(64) ERROR,NAMEIN,NAMEOUT
      CHARACTER*(6) NAME
      CHARACTER*(3) TTCHS
      CHARACTER*(80) STRING
      CHARACTER*(1024) NEWFILE
C
C        Buffer size (bytes)
#ifdef VMS
      DATA SIZE/28800/
#else
      DATA SIZE/32767/
#endif
C        Flag value for error message decoding
      DATA FLAGS/3/
C        Printer file logical unit
      DATA PRFILE/3/
C
      PRINT *,' '
      PRINT *,'     Tape Positioning Program   - Version: 17th Oct 1988'
      PRINT *,' '
C
C     See if in batch mode by translating 'SYS$INPUT'
C     and allow for all the silly little VAX conventions
C     for process permanent files and concealed devices.
C     Assume batch mode if result doesn't start '_TT', '_RT',
C     '_TX' or '_OP' - this is rather a crude assumption.  Could
C     be done properly with $GETJPI now.
C
#ifdef VMS
      NAMEIN='SYS$INPUT'
      LENGTH=9
  300 CONTINUE
         STATUS=LIB$SYS_TRNLOG(NAMEIN(:LENGTH),LENGTH,NAMEOUT)
         NAMEIN=NAMEOUT
         IF (STATUS.EQ.1)  GO TO 300
C
C     Note that if this section of code is ever converted to run under Unix,
C     specifically Sun Fortran 1.3.1, the hexidecimal constant below will
C     need to be changed.
C
      IF (NAMEOUT(1:1).EQ.CHAR('1B'X))  THEN
         NAMEOUT=NAMEOUT(5:)
         IF (NAMEOUT(:2).EQ.'__') THEN
            NAMEOUT=NAMEOUT(2:)
         END IF
      END IF 
      TTCHS=NAMEOUT(:3)
      IF ((TTCHS.EQ.'_TT').OR.(TTCHS.EQ.'_RT').OR.
     :                 (TTCHS.EQ.'_TX').OR.(TTCHS.EQ.'_OP')
     :                  .OR.(TTCHS.EQ.'_VT')) THEN
         KBATCH=0
      ELSE
         KBATCH=1
      END IF
#else
      KBATCH=0
#endif
C
C     Some initial values
C
C     Note use of some variables - RECORD is last record number
C     read in file, MTCHAN is the channel assigned to the tape,
C     KBATCH is 1 or 0 depending on the program mode, TPEASS is
C     set if a tape has been assigned, LENLIM is the number of 
C     bytes to be listed by the RA,RX commands. NOFILE is set if
C     a printer file has not yet been opened, PRSET is set if
C     output is being routed to a printer file.
C
      RECORD=0
      MTCHAN=0
      LENLIM=0
      PRSET=.FALSE.
      NOFILE=.TRUE.
      TPEASS=.FALSE.
C
C     In interactive mode, find out which tape to use
C
  310 CONTINUE
      IF (KBATCH.NE.1) THEN
         CALL MTGETN(1,NAME,STATUS)
         IF (STATUS.EQ.0) THEN
            PRINT *,'Tape drives are -'
            PRINT *,' '
            IDRIVE=2
            DO WHILE (STATUS.EQ.0)
               PRINT *,NAME 
               CALL MTGETN(IDRIVE,NAME,STATUS)
               IDRIVE=IDRIVE+1
            END DO
         ELSE
#ifdef VMS
            PRINT *,'SYS$TAPES has not been defined.'
#else
            PRINT *,'TAPES has not been defined.'
#endif
            PRINT *,'Cannot get list of available drives.'
         END IF
         PRINT *,' '
  320    CONTINUE
         PRINT 325,' Which tape unit? - '
  325    FORMAT(A,$)
         READ  (*,'(A)') LINE
         INVOKE=ICH_CLEAN(LINE)
         LENGTH=ICH_FOLD(LINE)
         KEY=ICH_KEY(LINE,1,' ;','NONE.','EXACT',NEXT)
         IF (KEY.EQ.1)  THEN
            PRINT *,'OK, but you won''t be able to do much'
         ELSE
            CALL MTGETN(1,NAME,STATUS)
#ifdef VMS
            IF (STATUS.NE.0) NAME='_MTA0:'
            IF (LENGTH.GT.0) THEN
               IF (LINE(LENGTH:LENGTH).EQ.':') LENGTH=LENGTH-1
               NAME(6-LENGTH:5)=LINE
            END IF
#else
            IF (STATUS.NE.0) NAME='/dev/rmt16'
            IF (LENGTH.GT.0) NAME=LINE
#endif
            PRINT *,' '
            PRINT *,'Will use tape ',NAME
            CALL MTOPID(NAME,MTCHAN,STATUS)
            CALL MTMESS(STATUS,'MTOPID',ERROR)
            IF (STATUS.NE.0) THEN
               PRINT *,'Failed to assign channel to tape drive'
               PRINT *,ERROR
               GO TO 320
            END IF
            PRINT *,' '
            LENGTH=ICH_LEN(NAME)
            PRINT *,NAME(:LENGTH),' assigned'
            TPEASS=.TRUE.
         END IF
      END IF
C
C     Quick reminder..
C
      PRINT *,' '
      IF (KBATCH.NE.1) THEN
         PRINT *,'Type "HELP" for a full list of commands'
         PRINT *,' '
      END IF
C
C     Main node.  Get a command.
C
  360 CONTINUE
      PRINT 325,' >'
      CALL INCOM(KEY,VALUE,NSPEC,
     +     'RW,EO,BF,FF,BR,FR,RL,RA,RE,RX,DI,LE,HELP,HE,TAPE,TA,'
     +     //'PR,XP,QUIT,QU,',
     +     KBATCH,IERR)
C
C     Check for batch mode error
C
      IF (IERR.EQ.1) THEN
         PRINT *,'Cannot understand command.  Program aborting.'
         GO TO 910
      ELSEIF (IERR.EQ.2) THEN
         PRINT *,'I/O error from command stream. Program aborting.'
         GO TO 910
      ELSEIF (IERR.EQ.3) THEN
         PRINT *,'End of command stream.  Program terminating.'
         GO TO 910
      END IF
C
C     For all actual tape moving commands, check that we have
C     a tape assigned.
C
      IF ((KEY.LE.10).AND.(.NOT.TPEASS)) THEN
         IF (KBATCH.EQ.0)  THEN
            PRINT *,'You don''t have a tape assigned'
            PRINT *,'Use the TAPE command to assign one.'
            GO TO 360
         ELSE
            PRINT *,'No tape assigned.  Program aborting.'
            GO TO 910
         END IF
      END IF
C
C     Act on command
C
      GO TO (410,420,430,440,450,460,470,470,470,470,480,490,500,500,
C             rw  eo  bf  ff  br  fr  rl  ra  re  rx  di  le   help
     +       510,510,520,530,910,910),KEY
C              tape   pr  xp   quit
C
C     Rewind  (RW)
C
  410 CONTINUE
      IF (NSPEC.EQ.1) THEN
         PRINT *,'A tape can only be rewound once.'
         PRINT *,'Numeric parameter ignored.'
      END IF
      CALL MTREWI(MTCHAN,STATUS)
      CALL MTMESS(STATUS,'MTREWI',ERROR)
      IF (STATUS.NE.0) THEN
         PRINT *,'Error rewinding tape'
         PRINT *,ERROR
         CALL MTVMER(FLAGS,STATUS,ERROR)
         PRINT *,ERROR
      ELSE
         PRINT *,'Tape rewound'
      END IF
      RECORD=0
      GO TO 360
C
C     Write end of file mark (EO)
C
  420 CONTINUE
      KEOFS=VALUE
      IF (NSPEC.EQ.0)  KEOFS=1
      IF (KEOFS.EQ.0)  THEN
         PRINT *,'You can''t write zero file marks'
         GO TO 360
      END IF
      IF (KEOFS.LT.0)  THEN
         PRINT *,'You can''t write a negative number of file marks'
         GO TO 360
      END IF
      DO NEOF=1,KEOFS
         CALL MTWEOF(MTCHAN,STATUS)
         CALL MTMESS(STATUS,'MTWEOF',ERROR)
         IF (STATUS.NE.0) THEN
            PRINT *,'Error writing file mark'
            PRINT *,ERROR
            CALL MTVMER(FLAGS,STATUS,ERROR)
            PRINT *,ERROR
            IF (NEOF.GT.1) THEN
               PRINT '(I4,A)',NEOF-1,' file marks written O.K.'
            END IF
            GO TO 360
         END IF
         RECORD=0
      END DO
      GO TO 360
C
C     Back over file marks (BF)
C
  430 CONTINUE
      KFILES=VALUE
      IF (NSPEC.EQ.0)  KFILES=1
      IF (KFILES.EQ.0)  THEN
         PRINT *,'You can''t skip zero files'
         GO TO 360
      END IF
      IF (KFILES.LT.0) THEN
         PRINT *,'If you want to go forward, use the FF command'
         PRINT *,'This command ignored'
         GO TO 360
      END IF
      RECORD=0
      DO NFILE=1,KFILES
         CALL MTSKFI(MTCHAN,-1,ICOUNT,STATUS)
         CALL MTMESS(STATUS,'MTSKFI',ERROR)
         IF (STATUS.EQ.0) THEN
            PRINT '(A,I5,A)',' File',NFILE,' skipped o.k.'
         ELSE
            PRINT '(A,I5)',' Error skipping file',NFILE
            PRINT *,ERROR
            CALL MTVMER(FLAGS,STATUS,ERROR)
            PRINT *,ERROR
            GO TO 360
         END IF
      END DO
      GO TO 360
C
C     Forward over file marks (FF)
C
  440 CONTINUE
      KFILES=VALUE
      IF (NSPEC.EQ.0)  KFILES=1
      IF (KFILES.EQ.0) THEN
         PRINT *,'You can''t skip zero files'
         GO TO 360
      END IF
      IF (KFILES.LT.0) THEN
         PRINT *,'If you want to go backward, use the BF command'
         PRINT *,'This command ignored'
         GO TO 360
      END IF
      RECORD=0
      DO NFILE=1,KFILES
         CALL MTSKFI(MTCHAN,1,ICOUNT,STATUS)
         CALL MTMESS(STATUS,'MTSKFI',ERROR)
         IF (STATUS.EQ.0) THEN
            PRINT '(A,I5,A)',' File',NFILE,' skipped o.k.'
         ELSE
            PRINT '(A,I5)',' Error skipping file',NFILE
            PRINT *,ERROR
            CALL MTVMER(FLAGS,STATUS,ERROR)
            PRINT *,ERROR
            GO TO 360
         END IF
      END DO
      GO TO 360
C
C     Back over records (BR)
C
  450 CONTINUE
      KRECS=VALUE
      IF (NSPEC.EQ.0)  KRECS=1
      IF (KRECS.EQ.0)  THEN
         PRINT *,'You can''t skip zero records'
         GO TO 360
      END IF
      IF (KRECS.LT.0) THEN
         PRINT *,'If you want to go forward, use the FR command'
         PRINT *,'This command ignored'
         GO TO 360
      END IF
      DO NREC=1,KRECS
         CALL MTSKRE(MTCHAN,-1,ICOUNT,STATUS)
         CALL MTMESS(STATUS,'MTSKRE',ERROR)
         IF (STATUS.NE.0) THEN
            IF (STATUS.EQ.MTREOF) THEN
               PRINT *,'End of file'
               RECORD=1
            ELSE
               PRINT '(A,I5)',' Error skipping record',NREC
               PRINT *,ERROR
               CALL MTVMER(FLAGS,STATUS,ERROR)
               PRINT *,ERROR
               GO TO 360
            END IF
         END IF
         RECORD=RECORD-1
      END DO
      GO TO 360
C
C     Forward over records (FR)
C
  460 CONTINUE
      KRECS=VALUE
      IF (NSPEC.EQ.0)  KRECS=1
      IF (KRECS.EQ.0) THEN
         PRINT *,'You can''t skip zero records'
         GO TO 360
      END IF
      IF (KRECS.LT.0) THEN
         PRINT *,'If you want to go backward, use the BR command'
         PRINT *,'This command ignored'
         GO TO 360
      END IF
      DO NREC=1,KRECS
         CALL MTSKRE(MTCHAN,1,ICOUNT,STATUS)
         CALL MTMESS(STATUS,'MTSKRE',ERROR)
         IF (STATUS.NE.0) THEN
            IF (STATUS.EQ.MTREOF) THEN
               PRINT *,'End of file'
               RECORD=-1
            ELSE
               PRINT '(A,I5)',' Error skipping record',NREC
               PRINT *,ERROR
               CALL MTVMER(FLAGS,STATUS,ERROR)
               PRINT *,ERROR
               GO TO 360
            END IF
         END IF
         RECORD=RECORD+1
      END DO
      GO TO 360
C
C     Read records and display length (RL), or in
C     ASCII (RA), or in hex (RX).
C
C     Note - operation depends on value of KEY
C     (7,8,9 or 10 for RL,RA,RE,RX).
C
  470 CONTINUE
      KOP=KEY-6
      IRCNT=0
      KRECS=VALUE
      IF (NSPEC.EQ.0)  KRECS=1
      IF (KRECS.EQ.0)  THEN
         PRINT *,'You can''t read zero records'
         GO TO 360
      END IF
      IF (KRECS.LT.0) THEN
         PRINT *,'Sorry, you can''t read records in reverse order'
         PRINT *,'This command ignored'
         GO TO 360
      END IF
      DO NREC=1,KRECS
         CALL MTREAD(MTCHAN,SIZE,BUFFER,LENGTH,STATUS)
         LBYTES=LENGTH
         IF (LENLIM.GT.0)  LBYTES=MIN(LENLIM,LBYTES)
         CALL MTMESS(STATUS,'MTREAD',ERROR)
         IF (STATUS.EQ.MTREOF) THEN
            PRINT *,'End of File.'
            IF (IRCNT.EQ.0) THEN
               PRINT *,'No records read'
            END IF
            RECORD=0
            GO TO 360
         ELSE
            IF (STATUS.NE.0) THEN
               PRINT '(A,I5)',' Error reading record',NREC
               PRINT *,ERROR
               CALL MTVMER(FLAGS,STATUS,ERROR)
               PRINT *,ERROR
            END IF
C
C           How data is displayed depends on which
C           operation was requested.  PRSET = true indicates
C           that the PR command is in effect.
C
            IRCNT=IRCNT+1
            RECORD=RECORD+1
            IF (KOP.EQ.1) THEN
C
C              Just list record length (RL).
C
               IF (PRSET) THEN
                  WRITE(PRFILE,'(A,I5,A,I6,A)',ERR=535)
     +               ' Record',RECORD,' has',LENGTH,' bytes'
               ELSE
                  PRINT '(A,I5,A,I6,A)',' Record',RECORD,' has',
     +                                         LENGTH,' bytes'
               END IF
            ELSEIF (KOP.EQ.2) THEN
C
C              Output record in ASCII (RA).
C
               CALL ICN_TIDY(BUFFER,LBYTES)
               IF (PRSET) THEN
                  WRITE(PRFILE,'(80A1)',ERR=535)
     +                           (BUFFER(I),I=1,LBYTES)
               ELSE
                  PRINT '(1X,80A1)',(BUFFER(I),I=1,LBYTES)
               END IF
            ELSEIF (KOP.EQ.3) THEN
C
C              Output record in EBCDIC. (RE)
C
               ISB=1
               DO WHILE (ISB.LE.LBYTES)
                  ILB=MIN(ISB+79,LBYTES)
                  CALL ICH_NTOCS(STRING,BUFFER(ISB),ILB-ISB+1)
#ifdef VMS
                  CALL LIB$TRA_EBC_ASC(STRING,STRING)
#endif
                  CALL ICH_TIDY(STRING)
                  IF (PRSET) THEN
                     WRITE (PRFILE,'(A)',ERR=535) 'No translation done'
                     WRITE (PRFILE,'(A)',ERR=535) STRING
                  ELSE
                     PRINT '(1X,A)','No translation done'
                     PRINT '(1X,A)',STRING
                  END IF
                  ISB=ISB+80
               END DO
            ELSEIF (KOP.EQ.4) THEN
C
C              Output record in Hex (RX). (Also puts ASCII
C              translation below the hex)
C
               ISB=1
               DO WHILE (ISB.LE.LBYTES)
                  ILB=MIN(ISB+39,LBYTES)
                  IF (PRSET) THEN
                     WRITE(PRFILE,'(40Z2)',ERR=535)
     :                              (BUFFER(I),I=ISB,ILB)
                  ELSE
                     PRINT '(1X,40Z2)',(BUFFER(I),I=ISB,ILB)
                  END IF
                  CALL ICN_TIDY(BUFFER(ISB),ILB-ISB+1)
                  IF (PRSET) THEN
                     WRITE(PRFILE,'(40A2)',ERR=535)
     :                              (BUFFER(I),I=ISB,ILB)
                  ELSE
                     PRINT '(1X,40A2)',(BUFFER(I),I=ISB,ILB)
                  END IF
                  ISB=ISB+40
               END DO
            END IF
         END IF
      END DO
      GO TO 360
C
C     Display information about tape (DI)
C
  480 CONTINUE
      IF (.NOT.TPEASS) THEN
         Print *,'You don''t have a tape assigned'
         GO TO 360
      END IF
      CALL MTSENS(MTCHAN,NSTAT,STATUS)
      IF (STATUS.EQ.MTRVIN) THEN
         PRINT *,'Volume has been off-line'
      ELSE
         IF (STATUS.NE.0) THEN
            PRINT *,'Error obtaining tape status information'
            CALL MTMESS(STATUS,'MTSENS',ERROR)
            PRINT *,ERROR
         ELSE
            CALL MTNAME(MTCHAN,NAME,LENGTH,STATUS)
            PRINT *,' '
            PRINT *,'Tape ',NAME
            PRINT *,' '
            CALL MVBITS(NSTAT,8,4,NDEN,0)
            IF (NDEN.EQ.3) PRINT *,'800 bpi'
            IF (NDEN.EQ.4) PRINT *,'1600 bpi'
            IF (NDEN.EQ.5) PRINT *,'6250 bpi'
            IF (BTEST(NSTAT,20)) PRINT *,'Tape position lost'
            IF (BTEST(NSTAT,19)) THEN
               PRINT *,'Write protected'
            ELSE
               PRINT *,'Write enabled'
            END IF
            IF (BTEST(NSTAT,18)) PRINT *,'End of tape'
            IF (BTEST(NSTAT,17)) PRINT *,'End of file'
            IF (BTEST(NSTAT,16)) PRINT *,'Load point'
            IF (BTEST(NSTAT,3)) THEN
               PRINT *,'Even parity'
            ELSE
               PRINT *,'Odd parity'
            END IF
         END IF
      END IF
      GO TO 360
C
C     Set record length for display (LE)
C
  490 CONTINUE
      LENLIM=VALUE
      IF (NSPEC.EQ.0)  LENLIM=0
      IF (LENLIM.LT.0) THEN
         PRINT *,'You can''t have a negative byte count'
         PRINT *,'Will assume zero - ie display all of each record'
         LENLIM=0
      END IF
      GO TO 360
C
C     Explain commands  (HELP)
C       
  500 CONTINUE
      PRINT *,' '
      PRINT *,' Commands accepted - '
      PRINT *,' '
      PRINT *,'RW    Rewind tape'
      PRINT *,'EO n  Write n end-of-file marks'
      PRINT *,'BF n  Backspace over n file marks'
      PRINT *,'FF n  Forward space over n file marks'
      PRINT *,'BR n  Backspace over n records'
      PRINT *,'FR n  Forward space over n records'
      PRINT *,'RL n  Read n records and list record lengths'
      PRINT *,'RA n  Read n records and display in ASCII'
      PRINT *,'RE n  Read n records and display in EBCDIC'
      PRINT *,'RX n  Read n records and display in hex'
      PRINT *,'LE n  Set max displayed length to n bytes'
      PRINT *,'PR    List records on file for printing'
      PRINT *,'XP    Revert to listing on terminal'
      PRINT *,'DI    Display information about tape'
      PRINT *,'HELP (HE)  Give this information'
      PRINT *,'TAPE (TA)  Change the tape in use'
      PRINT *,'QUIT (QU)  Quit the program'
      PRINT *,' '
      PRINT *,'For most functions ''n'' defaults to 1'
      PRINT *,'For LE, n defaults to 0, ie list all of records'
      PRINT *,' '
      GO TO 360
C
C     Set tape drive to use (TAPE, TA)
C
  510 CONTINUE
      IF (NSPEC.EQ.0) THEN
         IF (KBATCH.EQ.0) THEN
            GO TO 310
         ELSE
            PRINT *,'You MUST specify a tape drive number'
         END IF
      ELSE
         ITAPE=VALUE
         CALL MTGETN(ITAPE,NAME,STATUS)
         IF (STATUS.NE.0) THEN
            PRINT *,'Cannot get name for this tape'
         ELSE
            PRINT *,'Tape ',ITAPE,' is ',NAME
            CALL MTOPID(NAME,MTCHAN,STATUS)
            IF (STATUS.NE.0) THEN
               CALL MTMESS(STATUS,'MTOPID',ERROR)
               PRINT *,'Failed to open tape'
               PRINT *,ERROR
            ELSE
               TPEASS=.TRUE.
            END IF
         END IF
      END IF
      GO TO 360
C
C     PR - open file and set so that all RA,RX,RL commands
C     list on this file rather than the terminal.
C
  520 CONTINUE
      IF (NOFILE) THEN
         CALL VERSNAME('tapos.lis','NEW',NEWFILE)
         OPEN(UNIT=PRFILE,FILE=NEWFILE,STATUS='NEW',
     +        CARRIAGECONTROL='LIST',ERR=525)
         NOFILE=.FALSE.
      END IF
      PRSET=.TRUE.
      GO TO 360
C
  525 CONTINUE
      PRINT *,'Unable to create printer file "TAPOS.LIS"'
      GO TO 360
C
C     XP - change flag so records are listed on the terminal
C          
  530 CONTINUE
      IF (PRSET) THEN
         PRSET=.FALSE.
      ELSE
         PRINT *,'You weren''t printing'
      END IF
      GO TO 360
C
C     Printer file I/O errors come here
C
  535 CONTINUE
      PRINT *,'I/O error from print file "TAPOS.LIS"'
      PRSET=.FALSE.
      GO TO 360
C
C     Quit
C
  910 CONTINUE
      END
      SUBROUTINE INCOM(KEY,VALUE,NSPEC,LIST,KBATCH,IERR)
C
C     I N C O M
C
C     Reads a commmand from the input device and interprets
C     it, assuming that it has the format
C     COMMAND,[delimiter,numeric parameter]
C
C     Passed to INCOM -
C
C     LIST   is a character array, giving the possible
C            commmands, separated by and terminated by the
C            same character.
C
C     KBATCH is an integer set to 1 if the program is being
C            run in batch mode, 0 if interactive.
C
C     Returned by INCOM -
C
C     KEY    is an integer giving the number in LIST of the
C            command given. (from 1).
C
C     VALUE  is the value of the numeric parameter, if one is
C            specified.
C
C     NSPEC  is an integer set to 1 if a parameter was specified,
C            zero otherwise.
C
C     IERR   is an integer giving the batch mode error code.
C            0 => no error
C            1 => illegal command or parameter
C            2 => I/O error from input device
C            3 => end of file on command device
C
C     Example -
C
C     CALL INCOM(KEY,VALUE,NSPEC,'COM1,COM2,',0,IERR)
C
C     if given the command 'COM1 5' will return KEY=1,
C     VALUE=5., NSPEC=1.
C
C     Note that all input is folded into upper case - LIST
C     should therefore be given in upper case.
C
C     Subroutines / functions used -
C
C     ICH_CLEAN    (ICH_ package) Clean a character string
C     ICH_FOLD     ( "     "    ) Convert string to upper case
C     ICH_KEY      ( "     "    ) Look for a keyword at start of string
C     ICH_NUMBR    ( "     "    ) Evaluate a free-format number
C
C                                       KS / CIT 14th Dec 1982
C+
      CHARACTER*(*) LIST
      CHARACTER LINE*80   
      LOGICAL LEGAL,ERROR,RETRY
C
      IERR=0
C
C     Read in command line
C
  320 CONTINUE
      KEY=0
      RETRY=.FALSE.
      VALUE=0.
      READ (*,'(A)',ERR=520,END=530) LINE
      ERROR=.FALSE.
      INVOKE=ICH_CLEAN(LINE)
C
C     In batch mode, log command
C
      IF (KBATCH.EQ.1) THEN
         PRINT '(1X,A)',LINE
      END IF
C
C     Fold into upper case, 
C
      INVOKE=ICH_FOLD(LINE)
C
C     Look for command
C
      KEY=ICH_KEY(LINE,1,' ;',LIST,'EXACT',NEXT)
      IF (KEY.EQ.0) THEN
C
C        Unrecognised command
C
         ERROR=.TRUE.
C
C        Recognised command, look for parameter.
C
      ELSE
         IF (NEXT.EQ.0) THEN
            NSPEC=0
         ELSE
            ISNUMB=ICH_NUMBR(LINE,NEXT,'; ',VALUE,NDUM)
            ERROR=ISNUMB.EQ.1
            IF (ISNUMB.EQ.-1) THEN
               NSPEC=0
            ELSE
               NSPEC=1
            END IF
         END IF
      END IF
C
C     Have there been any errors, and can we retry?
C
      IF (ERROR) THEN
         IF (KBATCH.EQ.1) THEN
            IERR=1
            RETRY=.FALSE.
         ELSE
            PRINT *,' Pardon?'
            RETRY=.TRUE.
         END IF
      END IF
C
C     Retry if necessary and possible
C
      IF (RETRY) GO TO 320
C
C     Normal return
C
      GO TO 560
C
C     I/O errors..
C
  520 IERR=2
      GO TO 560
  530 IERR=3
  560 CONTINUE
C
      END
