      PROGRAM AMPHI
C
C Main Program for AMPHI. This routine determines
C the array sizes, allocates the arrays dynamically
C (VAX specific) and calls subroutine AMPHI1 to do
C the work.
C
C S.C. Unwin   1982 Nov  5; based on TJP's PHASE
C D.L. Meier   1983 Jul 21; MERGE version 2 files
C S.C. Unwin   1984 Jun 17; Add UCUT, UVRATIO
C Version 7.4: 1984 Aug  9; Use IMSL routine LEQT1F instead of
C                           EQSOV; introduce FLAGFAIL for failed
C                           solution (SCU).
C              1984 Dec  5; Fix bug in call to PNORM (SCU).
C Version 7.5: 1985 Jun 15; Improve debug output (LONG=3)
C Version 7.6: 1986 Aug 28; Increase MAXS to 20.
C Version 8.0: 1988 Apr  8; all input by parameters; eliminate
C                           command qualifiers; test on Convex (TJP).
C Version 8.1: 1988 Aug  2: remove BASERR parameter; add execution
C                           summary (TJP).
C Version 8.2: 1989 Jun 26: record method in output history (TJP).
C version 8.3: 1989 Dec  2: read the file to determine MAXS and MAXSECS;
C                           ignore user input (TJP).
C Version 8.4: 1990 Jan 16: changes for Sun: remove illegal use of
C                           SCALEX as common block name and INT as
C                           variable (DLM/TJP).
C Version 9.0: 1992 Feb 12: correct bug in closure-phase assignment
C                           (TJP).
C Version 9.1: 1992 Jun 18: correct bug in phase cycling (TJP).
C-----------------------------------------------------------------------
      CHARACTER*(*) VERSN
      PARAMETER (VERSN='9.1 - 1992 Jun 18')
      INTEGER    INC, OUTC, LIST
      PARAMETER (INC=5, OUTC=6)
      PARAMETER (LIST=1)
      INTEGER    MAXS, MAXB, MAXT, MAXWPB, MAXL, NPARS, MAXMOD
      PARAMETER (MAXS=20)
      PARAMETER (MAXB=MAXS*(MAXS-1)/2)
      PARAMETER (MAXT=MAXB*(MAXS-2)/3)
      PARAMETER (MAXWPB=7)
      PARAMETER (MAXL=MAXWPB*MAXB+2*MAXT)
      PARAMETER (NPARS=36+MAXS*3)
      PARAMETER (MAXMOD=10000)
C
      INTEGER   MXSECS, MXS, MXL, I, IER, MODE, NB, NT, NP
      INTEGER   LIB$GET_VM, LEN1, VLBOPE
      INTEGER   NBYTES(3), LOC(3)
      CHARACTER*128 LISFIL, INFILE, TEXT
      DOUBLE PRECISION NAMES(NPARS), PARS(NPARS), ENDMRK
      DATA NAMES/ 'CGnum','XCg','YCg', 'NCprint', 'INput',
     1         4*' ', 'OUtput',4*' ', 'MOdfile',4*' ',
     2         'LOng', 'CLamp', 'NOLookup', 'MAXSEcs', 'MAXS',
     3         'FLOatgai', 'SCale', 'FLAgfail', 'NUmber', 'CUtoff',
     4         'UCutoff', 'UVratio', 'LIstfile', 4*' ',
     5          MAXS*'PHSerr', MAXS*'GAInerr',
     6          MAXS*'TImescal'/
      DATA PARS / 0D0, 0D0, 0D0, 10D0, 'FROM',
     1         4*' ', 'TO',4*' ', 'MODEL', 4*' ',
     2         -1D0,-1D0,-1D0, 400D0, 10D0,
     3         -1D0,-1D0,-1D0,MAXMOD,0.1D0,
     4         0D0,1D0, 'amphi.li', 's', 3*' ',
     5         MAXS*-1D0, MAXS*-1D0,
     6         MAXS*-1D0/
      DATA ENDMRK/'/'/
C-----------------------------------------------------------------------
C Format statements.
C
  630 FORMAT ('Program AMPHI  --  Visibility amplitude and phase',
     1        ' correction'/'(Version ',A,')'//
     2        'Control parameters:'/)
  640 FORMAT ('Allocating memory for',I3,' stations and',I5,
     1        ' data records;',F8.2,' Mbytes required.')
C-----------------------------------------------------------------------
C
C Introduction.
C
      CALL PUTOUT('AMPHI self-calibrates MERGE files')
      CALL PUTOUT('(Version '//VERSN//')')
      CALL PUTOUT(' ')
C
C Read the control parameters.
C
      CALL PUTOUT('Control parameters (Input, Output, Modfile, etc):')
      MODE = 0
      CALL KEYIN (NAMES,PARS,NPARS,ENDMRK,MODE,INC,OUTC)
C
C Determine the size of the input file. The critical parameters
C are the number of stations (MXS) and the number of data records
C (MXSECS).
C
      WRITE (INFILE,'(5A8)') (PARS(I),I= 5,9)
      IER = VLBOPE(17, INFILE, 'MERGE', 'OLD', INFILE)
      IF (IER.NE.1) STOP
      CALL MRGSIZ(17, MXS, NB, NT, MXSECS, NP)
      CLOSE (UNIT=17)
C     -- Add 1 to MXSECS (needed for workspace).
      MXSECS = MXSECS+1
      MXL = 7*MAXB
C
C Allocate memory.
C
      NBYTES(1) = MXSECS*MXL*4
      NBYTES(2) = MXSECS*MXS*MXS*4
      NBYTES(3) = MXSECS*MXS*MXS*4
      WRITE (TEXT, 640) MXS, MXSECS,
     1      REAL(NBYTES(1)+NBYTES(2)+NBYTES(3))/(1024.0*1024.0)
      CALL PUTOUT(TEXT(1:LEN1(TEXT)))
      IF (MXS.GT.MAXS) CALL ERROR
     1       ('Actual number of stations exceeds maximum (20)')
      IF(MXSECS.GT.2000) CALL ERROR
     1       ('Actual number of data records exceeds maximum (2000)')
      DO 30 I=1,3
          IER = LIB$GET_VM(NBYTES(I),LOC(I))
          IF (IER.NE.1) CALL
     1       ERROR('Dynamic allocation failed: arrays too big')
   30 CONTINUE
C
C Open the listing file.
C
      WRITE (LISFIL,'(5A8)') (PARS(I),I=32,36)
      OPEN (UNIT=LIST, FILE=LISFIL, STATUS='NEW',
     1      CARRIAGECONTROL='LIST', IOSTAT=IER)
      IF (IER.NE.0) CALL ERROR('Cannot open LISTFILE')
      WRITE (LIST,630) VERSN
      CALL KEYOUT(NAMES,PARS,NPARS,LIST)
C
C Call a subroutine to do all the work.
C
      CALL AMPHI1(MXSECS,MXS,MXL,
     1           %VAL(LOC(1)),%VAL(LOC(2)),%VAL(LOC(3)),
     2           PARS, NPARS, VERSN)
C
C Execution summary.
C
      WRITE (LIST,'(/A,T20,A)') 'AMPHI version:',VERSN
      CALL REPORT(LIST)
      CALL REPCPU(LIST)
C
C Close the listing file.
C
      INQUIRE (UNIT=LIST, NAME=LISFIL)
      CLOSE (UNIT=LIST)
      CALL PUTOUT('AMPHI completed; listing in '//
     1            LISFIL(1:LEN1(LISFIL)))
C
      END

      SUBROUTINE AMPHI1(MXSECS,MXS,MXL,DATA,BSTORE,FSTORE,PARS,NPARS,
     :                  VERSN)
      INTEGER MXSECS, MXS, MXL, NPARS
      REAL*8 PARS(NPARS)
      CHARACTER*(*) VERSN
C
C       Subroutine AMPHI1 called by AMPHI, which uses dynamic memory
C       allocation for the largest arrays.
C
C       Station amplitude and phase solution for Hybrid-mapping.
C       Uses method of Cornwell and Wilkinson (1981) MNRAS, 196, 1067.
C       Observed baseline amplitudes and phases are adjusted
C       station-by-station, minimizing differences between the
C       observed and input-model visibilities. Loosely based on PHASE
C       program of T.J. Pearson (1981 March 5).
C
C       S.C. Unwin      VAX-11 Fortran      20 stations
C-----------------------------------------------------------------------
      INTEGER MAXS,MAXB,MAXT,MAXL,MAXMOD,INDAT,OUTDAT,INMOD,
     1          MAXWPB,LIST,MAXSECS,MAXS450
      PARAMETER (MAXS=20)
C     ! maximum number of stations
      PARAMETER (MAXB=MAXS*(MAXS-1)/2)
C     !  baselines
      PARAMETER (MAXT=MAXB*(MAXS-2)/3)
C     !  triangles
      PARAMETER (MAXWPB=7)
C     ! maximum data words per baseline
      PARAMETER (MAXL=MAXWPB*MAXB+2*MAXT)
C     !  data record length with clos. phases
      PARAMETER (MAXMOD=10000)
C     !  model points
      PARAMETER (MAXSECS=2000)
C     ! no of UT times
      PARAMETER (MAXS450=50+MAXS*4)
C     ! for CHANGE in HEADX20
      PARAMETER (INDAT=9)
      PARAMETER (OUTDAT=10)
      PARAMETER (INMOD=4)
      PARAMETER (LIST=1)
C
      DOUBLE PRECISION PI, RPDEG, RPMAS, VELC, SIDR
      PARAMETER (PI=3.141592654D0)
      PARAMETER (RPDEG=PI/180.0D0)
      PARAMETER (RPMAS=RPDEG/3.6D6)
      PARAMETER (VELC=2.997925D8)
C
C     ! radians of ST per sec/60 of UT
      PARAMETER (SIDR=(PI/2592000.0D0)*1.00273791D0 )
C
C     ! Dynamic allocation
      REAL BSTORE(MXSECS,MXS,MXS),FSTORE(MXSECS,MXS,MXS)
      REAL DATA(MXSECS,MXL)
C
      REAL REC(MAXL)
      REAL*8 BBB(MAXS,MAXS),FFF(MAXS,MAXS),FSUM(MAXS,MAXS)
      REAL*8 CCC(MAXS),EEE(MAXS),ESUM(MAXS),XXX(MAXS),YYY(MAXS)
      REAL CSTORE(MAXSECS,MAXS),ESTORE(MAXSECS,MAXS)
      REAL*8 BASERR(MAXB),AERRIN(MAXS),PERRIN(MAXS)
      REAL*8 DELSQA,DELSQP,ADATA,PDATA,AMPMOD,PHSMOD
      INTEGER BASE(MAXB),TRIG,ITYPE(MAXMOD),NCOMP,STATYP
      INTEGER CONST(MAXT,3),B1,B2,B3,IUT(MAXSECS)
      INTEGER TFIND
      INTEGER NCPRIN, LONG, NUMBER
      INTEGER I, IS, IZ, ID, IY, KFLAG, IFEOF, LENG, NDWPB, NB, NT
      INTEGER LENGB, NBASE, NSTA, NS, IB, NB1, NB2, NTX, NTIND, NCLAMP
      INTEGER NCLAMPI, IT, I1, I2, I3, IS1, IS2, J, NCS, NSECS, ISECS
      INTEGER NVER
      LOGICAL CLAMP,SCALE,FLOATGN,SMOOTH,USELOOK
      LOGICAL TUSED(MAXT),PCHECK(MAXB),AUSABLE(MAXB),PUSABLE(MAXB)
      LOGICAL FLAGFAIL
      LOGICAL*1 UVFLAG(MAXSECS,MAXB)
      CHARACTER*8 ELAPSE,UTP,TIMEC
      CHARACTER*8 FIELD(MAXS),STAR,ZERO,BLANK,FORM1,FORM2,FORM3
      CHARACTER*8 TDATA(MAXL),STAR2,FORM4
      CHARACTER NEWHIS*240, MODE*80
      REAL CGNUM, XCG, YCG, SPARE1, SPARE2, CUTOFF, UCUT, RATIO
      REAL XSUM, YSUM, TOTFLX, FLXSUM, RAD, THETA, XSHIFT, YSHIFT
      REAL CPU, UU, VV, UVSIGN
      REAL    FLUX(MAXMOD),XC(MAXMOD),YC(MAXMOD),AA(MAXMOD),
     1          R(MAXMOD),PHI(MAXMOD),
     2          WT(MAXS),TIMESCAL(MAXS),LOOK(4096)
      REAL    AMPM(MAXSECS,MAXB),PHSM(MAXSECS,MAXB)
      REAL*8  BX,BY,BZ(MAXB),BXY(MAXB),BOFF(MAXB),XI, WAVE
      REAL*8  STNAME,STX,STY,STZ,SOURCE(2),RA,DEC,GASTM,FREQ,TOTFL,
     1          TINTG,AVG,BW,AEIOWM,
     2          COSDEC,SINDEC,TIHA,CHANGE(MAXS450)
      REAL    RMSIN,RMSINJ,RMSOUT,RMSOUTJ
      CHARACTER*64  FFROM,FTO,FMODEL
      COMMON  /MODEL/ NCOMP,FLUX,XC,YC,AA,R,PHI,ITYPE
      COMMON  /LOOKUP/ USELOOK,LOOK
      COMMON  /PARMS/ STNAME(MAXS),STX(MAXS),STY(MAXS),STZ(MAXS),
     1                  SOURCE,RA,DEC,GASTM,FREQ,TOTFL,TINTG,AVG,BW,
     2                  AEIOWM(6,MAXS),STATYP(MAXS),ORBITERS
      COMMON  /ORDER/ NS,NB,NT,NB1(MAXB),NB2(MAXB),
     1                  NT1(MAXT),NT2(MAXT),NT3(MAXT),NVER,NDWPB
      COMMON  /ERR/ NSECS,NBASE,AMPM,PHSM
      COMMON  /SCACOM/ XXX,AERRIN,NSTA
      DATA    STAR/'     *  '/,ZERO/'    0   '/,BLANK/'        '/,
     1          FORM1/'(F8.4)'/,FORM2/'(A8)'/,FORM3/'(F8.3)'/,
     2          STAR2/'    *   '/,FORM4/'(F8.2)'/
      DATA    CHANGE/MAXS450*0D0/
C-----------------------------------------------------------------------
  596 FORMAT(/' Input file:  ',A)
  597 FORMAT( ' Input model: ',A)
  598 FORMAT( ' Output file: ',A/)
  601 FORMAT(' Amplitude and phase correction of ',
     1   2A8,' with',I3,' stations.'/
     2   1X,I4,' baselines,',I4,' triangles (',I4,' independent),'/
     3   1X,I4,' closure amplitudes  (',I4,' independent).')
  602 FORMAT(/' The baselines are labelled:'/(1X,15I5))
  603 FORMAT(/' Model:',5X,'Flux',3X,'Radius',4X,'Theta',5X,
     2         'Axis',4X,'Ratio',6X,'Phi',2X,'Type'/)
  604 FORMAT('+',21X,' - TIMESCALE for amplitude averaging ',
     1     '(minutes):'/' FWHM:   ',7(F10.1)/(9X,7(F10.1)))
  605 FORMAT(1X,I5,')',6F9.3,I5)
  606 FORMAT(/' Centroid of model is at X =',F10.5,', Y =',F10.5,
     1         ' (milliarcsec)'/
     2         ' Centroid shifted to     X =',F10.5,
     3         ', Y =',F10.5/' First',I5,' components',
     4         ' used for centroid calculation')
  607 FORMAT(/' Model consists of',I5,' components'/
     1          ' Total flux density = ',F8.3,' Jy')
  608 FORMAT(/' Only first',I5,' model components',
     1          ' will be used;  flux density = ',F8.3,' Jy')
  609 FORMAT(//' Station gain and phase solution:'//
     1            1X,'Section    UT',14X,12A8/(1X,27X,12A8))
  610 FORMAT(1X,'Baselines:      ',14I8/(1X,16I8))
  611 FORMAT(/' Stations:  ',7(2X,A8)/(12X,7(2X,A8)))
  612 FORMAT(//,' N =',I4,2X,A8,' UT')
  613 FORMAT(' CUTOFF level =',F5.2,' of peak')
  615 FORMAT(' CLAMP option selected')
  616 FORMAT(1X)
  618 FORMAT(/' Sin/cos lookup tables used in model visibility',
     1          ' calculation')
  619 FORMAT(/' Machine subroutines for sin/cos used in model',
     1          ' visibility calculation')
  620 FORMAT(' ++WARNING++ Data have not been calibrated')
  622 FORMAT('- RMS difference model:observed =',F9.3,
     1          ' Jy,  (',F8.3,' sigma)')
  631 FORMAT(1X,'Observed  amp:  ',14A8/(1X,16A8))
  632 FORMAT(1X,'Model     amp:  ',14F8.3/(1X,16F8.3))
  633 FORMAT(1X,'Corrected amp:  ',14A8/(1X,16A8))
  634 FORMAT(1X,'Observed  phs:  ',14A8/(1X,16A8))
  635 FORMAT(1X,'Model     phs:  ',14F8.2/(1X,16F8.2))
  636 FORMAT(1X,'Corrected phs:  ',14A8/(1X,16A8))
  637 FORMAT(1X,I4,3X,A8,'   gain:  ',12A8/(26X,12A8))
  638 FORMAT(1X,15X,     '  phase:  ',12A8/(26X,12A8))
  639 FORMAT(1X,I4,3X,A8,'  phase:  ',12A8/(26X,12A8))
  640 FORMAT(5A8)
  642 FORMAT(' Model visibility calculation: cpu time = ',F8.2,' s')
  650 FORMAT(/' ++WARNING++ Model truncated after ',I4,' components.')
  651 FORMAT(/' ++WARNING++ Input data truncated after ',
     1         I4,'UT sections.')
  652 FORMAT(/' RMS difference between observed and model',
     1     ' visibilities:'
     2/2X,F9.3,' Jy   (',F8.3,' sigma)   before correction'
     3/2X,F9.3,' Jy   (',F8.3,' sigma)   after  correction'/)
  653 FORMAT(/' SCALE option selected  - gain corrections applied:')
  654 FORMAT(' Gain:    ',7(2X,A8)/(10X,7(2X,A8)))
  655 FORMAT(' Station gain solutions multiplied by',F11.4,
     1' in calculating these gains (FLOATGAIN not specified)')
  656 FORMAT(/' Each station gain also multiplied by',F11.4,
     1          '  (FLOATGAIN not specified)')
  657 FORMAT(/' U,V data within an ellipse with semi-axes U =',
     1F8.2,', V =',F8.2,' million wavelengths excluded from fit')
  658 FORMAT(/' U,V data outside an ellipse with semi-axes U =',
     1F8.2,', V =',F8.2,' million wavelengths excluded from fit')
  661 FORMAT(1X,I4,3X,A8,' UT,  solution failed, IER =',
     1          I4,' - no corrections applied')
  662 FORMAT(' ++WARNING++ Solution failed, IER =',
     1          I4,' - no gain corrections applied')
C-----------------------------------------------------------------------
C
      CALL SCLOCK
C     ! set clock
      CALL MKHIST('AMPHI', VERSN, NEWHIS(1:80))
C
C Interpret parameters.
C
      CGNUM = PARS(1)
      XCG   = PARS(2)
      YCG   = PARS(3)
      NCPRIN= PARS(4)
      LONG  = PARS(20)
      CLAMP = PARS(21).NE.-1D0
      USELOOK  = PARS(22).EQ.-1D0
      SPARE1= PARS(23)
C     ! old NIT
      SPARE2= PARS(24)
C     ! old CONV
      FLOATGN  = PARS(25).NE.-1D0
      SCALE    = PARS(26).NE.-1D0
      FLAGFAIL = PARS(27).NE.-1D0
      NUMBER   = PARS(28)
      CUTOFF   = PARS(29)
      UCUT     = PARS(30)
      RATIO    = PARS(31)
C
      IF (USELOOK) THEN
C         -- Set up lookup table
          DO 50 I=1,4096
              XI = DBLE(I-1)*1.533980788D-3
              LOOK(I) = DSIN(XI)
   50     CONTINUE
      END IF
C
C
      DO 99 IS=1,MAXS
          IF (PARS(37).EQ.-1D0) THEN
              PERRIN(IS) = 360.0*RPDEG
C             ! Def PERRIN(IS) = 360 deg
          ELSE
              PERRIN(IS) = PARS(IS+36)*RPDEG
          END IF
          IF (PARS(MAXS+37).EQ.-1D0) THEN
              AERRIN(IS) = 2.0
C             ! Def AERRIN(IS) = 2.0
          ELSE
              AERRIN(IS) = PARS(IS+MAXS+36)
          END IF
          IF (PARS(2*MAXS+37).EQ.-1D0) THEN
C             ! Def SMOOTH = .FALSE. if TIMESCAL(1) is unset
              SMOOTH = .FALSE.
          ELSE
              SMOOTH = .TRUE.
              TIMESCAL(IS) = PARS(IS+2*MAXS+36)*3600.0
C             ! Unit = frames
              IF (TIMESCAL(IS).LT.0.001) TIMESCAL(IS) = 1.0
          END IF
 99   CONTINUE
C
C Open and check file names
C
      WRITE (FFROM,640) (PARS(I),I= 5, 9)
      WRITE (FTO,640)   (PARS(I),I=10,14)
      WRITE (FMODEL,640)(PARS(I),I=15,19)
      OPEN    (UNIT=INDAT,NAME=FFROM,TYPE='OLD',READONLY,
     1         FORM='UNFORMATTED',ERR=510)
      INQUIRE (UNIT=INDAT,NAME=FFROM)
      OPEN    (UNIT=INMOD,NAME=FMODEL,TYPE='OLD',READONLY,
     1         FORM='FORMATTED',ERR=520)
      INQUIRE (UNIT=INMOD,NAME=FMODEL)
      OPEN    (UNIT=OUTDAT,NAME=FTO,TYPE='NEW',FORM='UNFORMATTED',
     1         ERR=530)
      INQUIRE (UNIT=OUTDAT,NAME=FTO)
      WRITE (LIST,596) FFROM
      WRITE (LIST,597) FMODEL
      WRITE (LIST,598) FTO
C
C Read header from input file
C
      CALL HEADX20(.FALSE.,INDAT,IZ,ID,IY,
     1          %REF(NEWHIS),2,0,KFLAG,CHANGE,IFEOF,0)
      LENG=NDWPB*NB + 2*NT
      LENGB=NDWPB*NB
      NBASE=NB
      NSTA=NS
      IF(NS.GT.MXS) CALL ERROR('Too many stations.')
C     ! Limit from D.M.A.
      IF(KFLAG.NE.1) WRITE (LIST,620)
      IF(NB.LT.1 .OR. NS.LT.2) CALL ERROR('Not enough baselines.')
C
C     -- Baseline labels
      DO 100 IB=1,NB
          BASE(IB) = 100*NB1(IB) + NB2(IB)
  100 CONTINUE
C     -- Maximum number of closure phases for NS stations
      NTX = NS*(NS-1)*(NS-2)/6
C     -- Number of independent ones
      NTIND = NS*(NS-1)/2 - NS + 1
      IF(NS.LE.2) NTIND=0
C     -- Maximum number of closure amplitudes
      NCLAMP = NS*(NS-1)*(NS-2)*(NS-3)/8
C     -- Number of independent ones
      NCLAMPI = NS*(NS-1)/2 - NS
      IF(NS.LE.3) NCLAMPI=0
      WRITE (LIST,601) SOURCE,NS,NB,NTX,NTIND,NCLAMP,NCLAMPI
C
      IF(CLAMP.AND.SCALE) CLAMP = .FALSE.
C     ! Prefer SCALE over CLAMP
      IF(SMOOTH.AND.(.NOT.SCALE)) CLAMP = .TRUE.
C     ! SMOOTH => CLAMP
      IF(SCALE.AND.SMOOTH) SMOOTH = .FALSE.
C     ! SCALE =>  SMOOTH = .FALSE.
      IF(.NOT.SCALE) WRITE (LIST,611) (STNAME(IS),IS=1,NS)
      IF(CLAMP) THEN
          WRITE (LIST,615)
          IF(SMOOTH) WRITE (LIST,604) (TIMESCAL(IS)/3600.0,IS=1,NS)
          IF(SMOOTH) WRITE (LIST,613) CUTOFF
      END IF
      WRITE (LIST,602) (BASE(IB),IB=1,NB)
C-----------------------------------------------------------------------
C Figure out which of the observed NB baselines contribute to each of
C the NTX possible closure triangles (array CONST).
C-----------------------------------------------------------------------
      IT = 0
      DO 110 I3=3,NS
        DO 110 I2=2,I3-1
          DO 110 I1=1,I2-1
            IT = IT+1
            TRIG = 10000*I1 + 100*I2 + I3
            CONST(IT,1) = TFIND(100*I1+I2,BASE,NB)
            CONST(IT,2) = TFIND(100*I2+I3,BASE,NB)
            CONST(IT,3) = TFIND(100*I1+I3,BASE,NB)
            IF (LONG.GE.3 .AND.
     1          CONST(IT,1)*CONST(IT,2)*CONST(IT,3).NE.0)
     2            WRITE(LIST,'(4I6)') TRIG, (CONST(IT,J),J=1,3)
  110 CONTINUE
C-----------------------------------------------------------------------
C Compute baseline parameters.
C-----------------------------------------------------------------------
      WAVE=VELC/FREQ
      COSDEC=DCOS(DEC)
      SINDEC=DSIN(DEC)
      DO 120 IB=1,NB
        IS1 = NB1(IB)
        IS2 = NB2(IB)
        IF ((STATYP(IS1).LE.1) .AND. (STATYP(IS2).LE.1)) THEN
           BX=(STX(IS1)-STX(IS2))/WAVE
           BY=(STY(IS1)-STY(IS2))/WAVE
           BZ(IB)=(STZ(IS1)-STZ(IS2))/WAVE
           BOFF(IB)=GASTM-RA-PI/2.0-DATAN2(BY,BX)
           BXY(IB)=DSQRT(BX*BX + BY*BY)
        ELSE
           BZ(IB)   = 0D0
           BOFF(IB) = 0D0
           BXY(IB)  = 0D0
        END IF
  120 CONTINUE
C
C-----------------------------------------------------------------------
C Read the model parameters.
C-----------------------------------------------------------------------
      IF(NCPRIN.GT.0) WRITE (LIST,603)
      NCS=0
      XSUM=0.0
      YSUM=0.0
      TOTFLX=0.0
      FLXSUM=0.0
      DO 150 I=1,MAXMOD
        CALL GTMODL(INMOD,ITYPE(I),FLUX(I),RAD,THETA,AA(I),R(I),
     1            PHI(I), *155, 0)
        IF(FLUX(I).EQ.0.0) GOTO 155
        IF(I.GT.NUMBER)    GOTO 155
        FLXSUM=FLXSUM+FLUX(I)
        IF(I.LE.NCPRIN) WRITE (LIST,605) I,FLUX(I),RAD,THETA,AA(I),
     1          R(I),PHI(I),ITYPE(I)
        THETA=THETA*RPDEG
        XC(I) = RAD*SIN(THETA)
        YC(I) = RAD*COS(THETA)
        IF(I.LE.CGNUM) THEN
            NCS = NCS+1
            TOTFLX = TOTFLX+FLUX(I)
            XSUM = XSUM + XC(I)*FLUX(I)
            YSUM = YSUM + YC(I)*FLUX(I)
        END IF
  150 CONTINUE
      WRITE (LIST,650) MAXMOD
      I = MAXMOD+1
  155 NCOMP = I-1
      IF(NUMBER.GT.NCOMP) WRITE (LIST,607) NCOMP,FLXSUM
      IF(NUMBER.EQ.NCOMP) WRITE (LIST,608) NCOMP,FLXSUM
      CLOSE (UNIT=INMOD)
      IF(FLXSUM.EQ.0.0) CALL ERROR('Model has zero total flux')
C
C     ! Find centroid of first CGNUM components
      XSHIFT=0.0
      YSHIFT=0.0
      IF(CGNUM.GT.0) THEN
          XSUM = XSUM/TOTFLX
          YSUM = YSUM/TOTFLX
          WRITE (LIST,606) XSUM,YSUM,XCG,YCG,NCS
          IF(NCPRIN.GT.0) WRITE (LIST,603)
          XSHIFT = XSUM - XCG
          YSHIFT = YSUM - YCG
      END IF
      DO 170 I=1,NCOMP
          XC(I) = XC(I) - XSHIFT
          YC(I) = YC(I) - YSHIFT
          RAD = SQRT(XC(I)*XC(I)+YC(I)*YC(I))
          IF(RAD.GT.0.0) THETA = ATAN2(XC(I),YC(I))/RPDEG
          IF (CGNUM.GT.0 .AND. I.LE.NCPRIN) WRITE (LIST,605)
     1          I,FLUX(I),RAD,THETA,AA(I),R(I),PHI(I),ITYPE(I)
C         ! Unit = radians*4096 for use with lookup tables
          XC(I) = XC(I)*RPMAS*4096.0
          YC(I) = YC(I)*RPMAS*4096.0
          IF( (ABS(AA(I)).GT.0.001) .OR.
     1          (ITYPE(I).GE.1) ) USELOOK = .FALSE.
C               ! Gaussians etc.
          AA(I) = AA(I)*RPMAS
          PHI(I) = PHI(I)*RPDEG
  170 CONTINUE
C
C-----------------------------------------------------------------------
C Read input data into array DATA().
C-----------------------------------------------------------------------
      NSECS=1
 200  READ(INDAT,END=210) IUT(NSECS),(DATA(NSECS,J),J=1,LENGB)
      IF(IUT(NSECS).LT.0) GOTO 210
      NSECS=NSECS+1
      IF(NSECS.GT.MXSECS) GOTO 209
C     ! Limit set by dynamic allocation
      GOTO 200
 209  WRITE (LIST,651) MXSECS
 210  NSECS=NSECS-1
      CLOSE(UNIT=INDAT)
C-----------------------------------------------------------------------
C Calculate model amp and phase, AMPM() and PHSM().
C-----------------------------------------------------------------------
      CALL RCLOCK(ELAPSE,CPU)
      IF(USELOOK)      WRITE (LIST,618)
      IF(.NOT.USELOOK) WRITE (LIST,619)
      DO 220 ISECS=1,NSECS
          DO 219 IB=1,NB
C             -- Get (u,v) coordinates
              IF (NVER.EQ.1) THEN
                  TIHA=(IUT(ISECS)-IZ)*SIDR + BOFF(IB)
                  UU=BXY(IB)*DCOS(TIHA)
                  VV=BZ(IB)*COSDEC + BXY(IB)*SINDEC*DSIN(TIHA)
              ELSE IF (NVER.EQ.2) THEN
                  UU=DATA(ISECS,NDWPB*(IB-1)+5)
                  VV=DATA(ISECS,NDWPB*(IB-1)+6)
              END IF
C             -- Flag data outside u-v limits
              UVFLAG(ISECS,IB) = .FALSE.
C             ! false/true = use/reject data
              IF(UCUT.NE.0) THEN
                  UVSIGN = -1.0
C                 ! neg/pos = 'inside'/'outside'
                  IF( SQRT( (UU*1.0E-6)**2 + (RATIO*VV*1.0E-6)**2 )
     1                   .GT. ABS(UCUT) )  UVSIGN = +1.0
                  IF(UVSIGN*UCUT .LT. 0.0) UVFLAG(ISECS,IB) = .TRUE.
              END IF
              CALL PHSAMP(UU,VV,AMPM(ISECS,IB),PHSM(ISECS,IB))
  219     CONTINUE
  220 CONTINUE
      CALL RCLOCK(ELAPSE,CPU2)
      WRITE (LIST,642) (CPU2-CPU)
      IF(UCUT.GT.0) WRITE (LIST,657)  UCUT, UCUT*RATIO
      IF(UCUT.LT.0) WRITE (LIST,658) -UCUT,-UCUT*RATIO
C
C-----------------------------------------------------------------------
C Compute fit of input data to model.
C-----------------------------------------------------------------------
      CALL RMSERR(DATA,MXSECS,MXL,NDWPB,RMSIN,RMSINJ)
C
C-----------------------------------------------------------------------
C Solve for amp and phase correction for each UT.
C-----------------------------------------------------------------------
      DO 300 ISECS=1,NSECS
C
C         Which baselines are usable at this time?
C
          DO 320 IB=1,NB
              AMPERR = DATA(ISECS,NDWPB*(IB-1)+2)
              PHSERR = DATA(ISECS,NDWPB*(IB-1)+4)
              AUSABLE(IB)= AMPERR.GT.1E-10
              PUSABLE(IB)=(AMPERR.GT.1E-10) .AND. (PHSERR.GT.1E-10)
              IF(PUSABLE(IB)) DATA(ISECS,NDWPB*(IB-1)+3) =
     1                            PNORM(DATA(ISECS,NDWPB*(IB-1)+3),0.0)
              BASERR(IB) = AMPERR
C             -- Flag data rejected by u-v limits
              IF(UVFLAG(ISECS,IB)) THEN
                  AUSABLE(IB) = .FALSE.
                  PUSABLE(IB) = .FALSE.
              END IF
              PCHECK(IB) = .FALSE.
  320     CONTINUE
C
C         Compute (observed-model) closure phase, and adjust observed
C         phase until |diff| < 180deg.
C
          NTAKE = 0
 730      NTAKE = NTAKE+1
          IF(NTAKE.GE.NB+1) GOTO 310
          IF(PCHECK(NTAKE)) GOTO 730
          IF(.NOT.PUSABLE(NTAKE)) GOTO 730
          IB=NTAKE
          DATA(ISECS,NDWPB*(IB-1)+3) =
     1                 PNORM(DATA(ISECS,NDWPB*(IB-1)+3),PHSM(ISECS,IB))
 740      PCHECK(IB)=.TRUE.
          DO 700 IT=1,NTX
              TUSED(IT)=.FALSE.
              IF(TUSED(IT)) GOTO 700
              B1=CONST(IT,1)
              B2=CONST(IT,2)
              B3=CONST(IT,3)
              IF(B1*B2*B3.EQ.0) GOTO 700
              IF( (.NOT.PUSABLE(B1)) .OR. (.NOT.PUSABLE(B2)) .OR.
     1                                   (.NOT.PUSABLE(B3)) )  GOTO 700
              IF(PCHECK(B1)) GOTO 701
              IF((.NOT.PCHECK(B2)).OR.(.NOT.PCHECK(B3))) GOTO 700
              IB=B1
              GOTO 710
 701          IF(PCHECK(B2)) GOTO 702
              IF((.NOT.PCHECK(B1)).OR.(.NOT.PCHECK(B3))) GOTO 700
              IB=B2
              GOTO 710
 702          IF(PCHECK(B3)) GOTO 700
              IF((.NOT.PCHECK(B1)).OR.(.NOT.PCHECK(B2))) GOTO 700
              IB=B3
 710          CLS = DATA(ISECS,NDWPB*(B1-1)+3) +
     1              DATA(ISECS,NDWPB*(B2-1)+3) -
     2              DATA(ISECS,NDWPB*(B3-1)+3)
              CLSM=PHSM(ISECS,B1) + PHSM(ISECS,B2) - PHSM(ISECS,B3)
              DIFF = CLS - CLSM
              TUSED(IT)=.TRUE.
              SIGN =  +1.0
              IF(DIFF.GE.180.0) SIGN = -1.0
              SIGN3 = +1.0
              IF (IB.EQ.B3) SIGN3 = -1.0
 711          IF((DIFF.LT. 180.0) .AND. (DIFF.GE.-180.0)) GOTO 740
              DATA(ISECS,NDWPB*(IB-1)+3) = DATA(ISECS,NDWPB*(IB-1)+3) +
     1                               SIGN3*SIGN*360.0
              DIFF    = DIFF    + SIGN*360.0
              GOTO 711
 700      CONTINUE
          GOTO 730
 310      CONTINUE
C
C         Initialize matrix arrays.
C
          DO 330 IS2=1,MAXS
              CCC(IS2)=0.0D0
              EEE(IS2)=0.0D0
              XXX(IS2)=0.0D0
              YYY(IS2)=0.0D0
              DO 329 IS1=1,MAXS
                  BBB(IS1,IS2)=0.0D0
                  FFF(IS1,IS2)=0.0D0
  329         CONTINUE
  330     CONTINUE
C
C         Fill matrix arrays BBB,CCC,FFF,EEE.
C
          DO 340 IB=1,NB
              IS1=NB1(IB)
              IS2=NB2(IB)
              DELSQA=0.0D0
              DELSQP=0.0D0
              ADATA=DATA(ISECS,NDWPB*(IB-1)+1)
C             ! data  amplitude
              PDATA=DATA(ISECS,NDWPB*(IB-1)+3)
C             ! data  phase
              AMPMOD=AMPM(ISECS,IB)
C             ! model amplitude
              PHSMOD=PHSM(ISECS,IB)
C             ! model phase
              IF (PUSABLE(IB))
     1          DELSQP =  (ADATA/BASERR(IB))*(AMPMOD/BASERR(IB))
              IF (AUSABLE(IB))
     1          DELSQA = (AMPMOD/BASERR(IB))*(AMPMOD/BASERR(IB))
              BBB(IS1,IS2) = DELSQP
              BBB(IS2,IS1) = DELSQP
              BBB(IS1,IS1) = BBB(IS1,IS1) - DELSQP
              BBB(IS2,IS2) = BBB(IS2,IS2) - DELSQP
              FFF(IS1,IS2) = -DELSQA
              FFF(IS2,IS1) = -DELSQA
              FFF(IS1,IS1) = FFF(IS1,IS1) - DELSQA
              FFF(IS2,IS2) = FFF(IS2,IS2) - DELSQA
              CCC(IS1) = CCC(IS1) - DELSQP*(PDATA-PHSMOD)*RPDEG
              CCC(IS2) = CCC(IS2) + DELSQP*(PDATA-PHSMOD)*RPDEG
C             -- If AMPMOD = 0, then DELSQA = 0, so trap zero-divide:-
              IF(AMPMOD.GT.1.0E-12) THEN
                  EEE(IS1) = EEE(IS1) - DELSQA*(ADATA/AMPMOD - 1.0D0)
                  EEE(IS2) = EEE(IS2) - DELSQA*(ADATA/AMPMOD - 1.0D0)
              END IF
  340     CONTINUE
C
          DO 341 IS1=1,NS
              CCC(IS1) = CCC(IS1)*PERRIN(IS1)*PERRIN(IS1)
              EEE(IS1) = EEE(IS1)*AERRIN(IS1)*AERRIN(IS1)
              DO 342 IS2=1,NS
                  BBB(IS1,IS2) = BBB(IS1,IS2)*PERRIN(IS1)*PERRIN(IS1)
                  FFF(IS1,IS2) = FFF(IS1,IS2)*AERRIN(IS1)*AERRIN(IS1)
  342         CONTINUE
              BBB(IS1,IS1) = BBB(IS1,IS1) - 1.0D0
              FFF(IS1,IS1) = FFF(IS1,IS1) - 1.0D0
  341     CONTINUE
C
          DO 344 IS1=1,NS
              CSTORE(ISECS,IS1) = CCC(IS1)
              ESTORE(ISECS,IS1) = EEE(IS1)
              DO 343 IS2=1,NS
                  BSTORE(ISECS,IS1,IS2) = BBB(IS1,IS2)
                  FSTORE(ISECS,IS1,IS2) = FFF(IS1,IS2)
  343         CONTINUE
  344     CONTINUE
C     -- end ISECS loop
  300 CONTINUE
C
C-----------------------------------------------------------------------
C Write headings in listing file.
C-----------------------------------------------------------------------
      IF((LONG.GE.1).OR.((LONG.GE.0).AND.CLAMP))
     1                 WRITE (LIST,609) (STNAME(I),I=1,NS)
      IF(LONG.GE.3)  WRITE (LIST,610) (BASE(IB),IB=1,NB)
      IF((LONG.GE.1).OR.((LONG.GE.0).AND.CLAMP))
     1                 WRITE (LIST,616)
C
C-----------------------------------------------------------------------
C Loop to evaluate amp and phs errors.
C-----------------------------------------------------------------------
      GAINSUM = 0.0
      NGAINS  = 0
      DO 800 ISECS=1,NSECS
          UTP=TIMEC(MOD(IUT(ISECS)/60,86400))
C
C Solve for amp errors XXX and apply to DATA(ISECS,NDWPB*(IB-1)+1).
C If SMOOTH set, smooth arrays EEE and FFF first.
C
          IF(CLAMP) THEN
              IF(LONG.GE.3) THEN
C               ! write diagnostics (deleted data printed as *)
                WRITE (LIST,612) ISECS,UTP
                DO IX=1,NB
                  WRITE(TDATA(IX),FORM3) DATA(ISECS,NDWPB*(IX-1)+1)
                  IF(DATA(ISECS,NDWPB*(IX-1)+2).LE.0.0)
     1                  WRITE(TDATA(IX),FORM2) STAR
                END DO
                WRITE (LIST,631) (TDATA(IB),IB=1,NB)
                WRITE (LIST,632) (AMPM(ISECS,IB),IB=1,NB)
              END IF
C
              IF(.NOT.SMOOTH) THEN
C             ! Unpack data for section ISECS
              DO 805 IS1=1,NS
                  ESUM(IS1) = ESTORE(ISECS,IS1)
                  DO 804 IS2=1,NS
                      FSUM(IS1,IS2) = FSTORE(ISECS,IS1,IS2)
  804             CONTINUE
  805         CONTINUE
          END IF
C
          IF (SMOOTH) THEN
C             ! Smooth amp data over index JSECS
              DO 810 IS1=1,NS
                  ESUM(IS1) = 0.0D0
                  DO 809 IS2=1,NS
                      FSUM(IS1,IS2) = 0.0D0
  809             CONTINUE
  810         CONTINUE
C
              TIMEI = IUT(ISECS)
              DO 820 JSECS=1,NSECS
                DO 820 IS1=1,NS
                  TIMEJ = IUT(JSECS)
                  WT(IS1) = WEIGHT((TIMEJ-TIMEI)/TIMESCAL(IS1))
                  IF(WT(IS1).GT.CUTOFF) THEN
                      ESUM(IS1) = ESUM(IS1) + WT(IS1)*ESTORE(JSECS,IS1)
                      DO 821 IS2=1,NS
                          FSUM(IS1,IS2) = FSUM(IS1,IS2) +
     1                                    WT(IS1)*FSTORE(JSECS,IS1,IS2)
  821                 CONTINUE
                  END IF
  820         CONTINUE
          END IF
C
          GAIN = 1.0
          NZERO=0
          DO 802 IS=1,NS
              XXX(IS)=0.0
              IF(ABS(ESUM(IS)).LT.1.0E-12) NZERO=NZERO+1
  802     CONTINUE
          IF(NZERO.GE.NS) GOTO 807
C     ! In LEQT1F ESUM is overwritten by the soln XXX
      DO IK=1,NS
      XXX(IK) = ESUM(IK)
      END DO
      CALL GAUSSJ(FSUM,NS,MAXS,XXX,1,1,IER)
      IF(IER.EQ.0) THEN
      IF(.NOT.FLOATGN) CALL SCALEX(GAIN)
C     ! GAIN = average 1/(1+XXX)
      NGAINS = NGAINS + 1
      GAINSUM = GAINSUM + GAIN*GAIN
C     ! For average value of GAIN**2
      DO 806 IB=1,NB
      FACTOR = 1.0/((1.0 + XXX(NB1(IB)))*(1.0 + XXX(NB2(IB))))
      IF(ABS(FACTOR).GT.100.0) GOTO 806
      DATA(ISECS,NDWPB*(IB-1)+1) = DATA(ISECS,NDWPB*(IB-1)+1)*FACTOR
C     ! amplitudes
      DATA(ISECS,NDWPB*(IB-1)+2) = DATA(ISECS,NDWPB*(IB-1)+2)*FACTOR
C     ! amplitude errors
 806  CONTINUE
                ELSE
                DO IK=1,NS
                XXX(IK) = 0.0
                END DO
                WRITE (LIST,661) ISECS,UTP,IER
                        IF(FLAGFAIL) THEN
                        DO IB=1,NB
                        ERRA = DATA(ISECS,NDWPB*(IB-1)+2)
                        ERRP = DATA(ISECS,NDWPB*(IB-1)+4)
                        IF(ERRA.GT.0) DATA(ISECS,NDWPB*(IB-1)+2)=-ERRA
                        IF(ERRP.GT.0) DATA(ISECS,NDWPB*(IB-1)+4)=-ERRP
                        END DO
                        ENDIF
                ENDIF
 807  IF(LONG.GE.3) THEN
C               ! write diagnostics
                DO IX=1,NB
                WRITE(TDATA(IX),FORM3) DATA(ISECS,NDWPB*(IX-1)+1)
                IF(DATA(ISECS,NDWPB*(IX-1)+2).LE.0.0)
     1                  WRITE(TDATA(IX),FORM2) STAR
                END DO
                WRITE (LIST,633) (TDATA(IB),IB=1,NB)
                ENDIF
      DO 808 IS=1,NS
      IF(ABS(1.0 + XXX(IS)).LT.0.01) XXX(IS) = 0.0
      XXX(IS) = 1.0/(1.0 + XXX(IS))
      WRITE(FIELD(IS),FORM1,ERR=808) XXX(IS)
      IF(ABS(ESUM(IS)).LT.1.0E-12) WRITE(FIELD(IS),FORM2) STAR2
 808  CONTINUE
      IF(LONG.GE.0) WRITE (LIST,637) ISECS,UTP,(FIELD(IS),IS=1,NS)
      ENDIF
C-----------------------------------------------------------------------
C PHASE SOLUTION:
C Solve for phs errors YYY and apply to DATA(ISECS,NDWPB*(IB-1)+3)
C-----------------------------------------------------------------------
C     -- Write diagnostics
      IF (LONG.GE.3) THEN
          WRITE (LIST,616)
          DO IX=1,NB
              WRITE (TDATA(IX),FORM4) DATA(ISECS,NDWPB*(IX-1)+3)
              IF (DATA(ISECS,NDWPB*(IX-1)+4).LE.0.0) TDATA(IX) = STAR
          END DO
          WRITE (LIST,634) (TDATA(IB),IB=1,NB)
          WRITE (LIST,635) (PHSM(ISECS,IB),IB=1,NB)
      END IF
      NZERO=0
      IFIXPHS=0
      DO 381 IS1=1,NS
          YYY(IS1) = 0.0
          CCC(IS1) = CSTORE(ISECS,IS1)
          IF (ABS(CCC(IS1)).LT.1.0E-12) NZERO = NZERO+1
          IF (ABS(CCC(IS1)).GT.1.0E-12) IFIXPHS = IS1
          DO 380 IS2=1,NS
              BBB(IS1,IS2) = BSTORE(ISECS,IS1,IS2)
  380     CONTINUE
  381 CONTINUE
      IF ((NZERO.GE.NS-1).OR.(IFIXPHS.EQ.0)) GOTO 382
C
C       Phase at one station can be set to 0.0
C       But must have at least two stations with CCC(IS) .NE. 0.0
C       IFIXPHS = number of last such station
C
          CCC(IFIXPHS) = 0.0
          DO 384 IS=1,NS
              BBB(IS,IFIXPHS) = 0.0
              BBB(IFIXPHS,IS) = 0.0
  384     CONTINUE
          BBB(IFIXPHS,IFIXPHS) = -1.0
C         -- CCC is overwritten by the solution YYY
          DO IK=1,NS
              YYY(IK) = CCC(IK)
          END DO
          CALL GAUSSJ(BBB,NS,MAXS,YYY,1,1,IER)
          IF (IER.EQ.0) THEN
C             -- Successful solution: apply it
              DO 383 IB=1,NB
                  DATA(ISECS,NDWPB*(IB-1)+3) =
     1                DATA(ISECS,NDWPB*(IB-1)+3) -
     2                (YYY(NB1(IB))-YYY(NB2(IB)))/RPDEG
  383         CONTINUE
          ELSE
C             -- Solution failed: set to zero (no change) and flag
C             -- data if requested.
              DO IK=1,NS
                  YYY(IK) = 0.0
              END DO
              WRITE (LIST,661) ISECS,UTP,IER
              IF (FLAGFAIL) THEN
                  DO IB=1,NB
                      ERRA = DATA(ISECS,NDWPB*(IB-1)+2)
                      ERRP = DATA(ISECS,NDWPB*(IB-1)+4)
                      IF(ERRA.GT.0) DATA(ISECS,NDWPB*(IB-1)+2)=-ERRA
                      IF(ERRP.GT.0) DATA(ISECS,NDWPB*(IB-1)+4)=-ERRP
                  END DO
              END IF
          END IF
C
 382      IF (LONG.GE.3) THEN
C            ! write diagnostics
             DO IX=1,NB
                WRITE(TDATA(IX),FORM4) DATA(ISECS,NDWPB*(IX-1)+3)
                IF(DATA(ISECS,NDWPB*(IX-1)+4).LE.0.0)
     1                  WRITE(TDATA(IX),FORM2) STAR
             END DO
             WRITE (LIST,636) (TDATA(IB),IB=1,NB)
          END IF
          IF (LONG.GE.1) THEN
             DO 385 IS=1,NS
                 WRITE(FIELD(IS),FORM3,ERR=385) YYY(IS)/RPDEG
                 IF (ABS(CCC(IS)).LT.1.0E-12) FIELD(IS) = STAR2
                 IF (IS.EQ.IFIXPHS) FIELD(IS) = ZERO
  385        CONTINUE
             IF (CLAMP) THEN
                 WRITE (LIST,638)           (FIELD(IS),IS=1,NS)
             ELSE
                 WRITE (LIST,639) ISECS,UTP,(FIELD(IS),IS=1,NS)
             END IF
          END IF
C     -- End main loop for phase and CLAMP corrections
  800 CONTINUE
C
C-----------------------------------------------------------------------
C Constant amplitude corrections -- SCALE option.
C-----------------------------------------------------------------------
      IF(SCALE) THEN
          DO 850 IS1=1,NS
          ESUM(IS1)=0.0
          DO 850 IS2=1,NS
 850      FSUM(IS1,IS2)=0.0
C
          DO 851 ISECS=1,NSECS
          DO 851 IS1=1,NS
          ESUM(IS1) = ESUM(IS1) + ESTORE(ISECS,IS1)
          DO 851 IS2=1,NS
 851      FSUM(IS1,IS2) = FSUM(IS1,IS2) + FSTORE(ISECS,IS1,IS2)
C
          NZERO=0
          DO 852 IS=1,NS
          XXX(IS)=0.0
 852      IF(ABS(ESUM(IS)).LT.1.0E-12) NZERO=NZERO+1
          GAIN = 1.0
          IF(NZERO.GE.NS) GOTO 854
          DO IK=1,NS
C           ! In LEQT1F ESUM is overwritten by solution XXX
          XXX(IK) = ESUM(IK)
          END DO
          CALL GAUSSJ(FSUM,NS,MAXS,XXX,1,1,IER)
          IF((.NOT.FLOATGN) .AND. (IER.EQ.0))
     1          CALL SCALEX(GAIN)
C               ! GAIN = average 1/(1+XXX)
 854      CONTINUE
          IF(IER.EQ.0) THEN
          DO 855 ISECS=1,NSECS
          UTP=TIMEC(MOD(IUT(ISECS)/60,86400))
          IF(LONG.GE.3)  THEN
C               ! write diagnostics
                WRITE (LIST,612) ISECS,UTP
                DO IX=1,NB
                WRITE(TDATA(IX),FORM3) DATA(ISECS,NDWPB*(IX-1)+1)
                IF(DATA(ISECS,NDWPB*(IX-1)+2).LE.0.0)
     1                  WRITE(TDATA(IX),FORM2) STAR
                END DO
                WRITE (LIST,631) (TDATA(IB),IB=1,NB)
                WRITE (LIST,632) (AMPM(ISECS,IB),IB=1,NB)
                ENDIF
          DO 853 IB=1,NB
          FACTOR = 1.0/((1.0 + XXX(NB1(IB)))*(1.0 + XXX(NB2(IB))))
          IF(ABS(FACTOR).GT.100.0) GOTO 853
          IF(.NOT.FLOATGN) FACTOR = FACTOR/(GAIN*GAIN)
          DATA(ISECS,NDWPB*(IB-1)+1) = DATA(ISECS,NDWPB*(IB-1)+1)*FACTOR
C         ! amplitudes
          DATA(ISECS,NDWPB*(IB-1)+2) = DATA(ISECS,NDWPB*(IB-1)+2)*FACTOR
C         ! amplitude errors
 853      CONTINUE
          IF(LONG.GE.3)   THEN
C               ! write diagnostics
                DO IX=1,NB
                WRITE(TDATA(IX),FORM3) DATA(ISECS,NDWPB*(IX-1)+1)
                IF(DATA(ISECS,NDWPB*(IX-1)+2).LE.0.0)
     1                  WRITE(TDATA(IX),FORM2) STAR
                END DO
                WRITE (LIST,633) (TDATA(IB),IB=1,NB)
                ENDIF
 855      CONTINUE
                ELSE
C                ! IER .GE. 64 in LEQT1F
                DO IK=1,NS
                XXX(IK) = 0.0
                END DO
                WRITE (LIST,662) IER
                IF(FLAGFAIL) CALL ERROR('SCALE gain solution failed -
     1all data flagged bad')
                ENDIF
          DO 856 IS=1,NS
              IF(ABS(1.0 + XXX(IS)).LT.0.01) XXX(IS) = 0.0
              XXX(IS) = 1.0/(1.0 + XXX(IS))
              XXX(IS) = XXX(IS)/GAIN
              WRITE(FIELD(IS),FORM1,ERR=856) XXX(IS)
              IF(ABS(ESUM(IS)).LT.1.0E-12) WRITE(FIELD(IS),FORM2) STAR2
 856      CONTINUE
          WRITE (LIST,653)
          WRITE (LIST,611) (STNAME(IS),IS=1,NS)
          WRITE (LIST,654) (FIELD(IS),IS=1,NS)
C         --  Overall gain correction
          IF(.NOT.FLOATGN) WRITE (LIST,655) 1.0/GAIN
      END IF
C
C-----------------------------------------------------------------------
C Apply overall scaling if CLAMP specified, FLOATGAIN omitted.
C Multiply amp and amperr by 1/(mean GAIN**2).
C-----------------------------------------------------------------------
      IF (CLAMP .AND. (.NOT.FLOATGN)) THEN
          FACTOR = NGAINS
          FACTOR = FACTOR/GAINSUM
          WRITE (LIST,656) SQRT(FACTOR)
          DO ISECS=1,NSECS
              DO IB=1,NB
                  DATA(ISECS,NDWPB*(IB-1)+1) =
     1                                 DATA(ISECS,NDWPB*(IB-1)+1)*FACTOR
                  DATA(ISECS,NDWPB*(IB-1)+2) =
     1                                 DATA(ISECS,NDWPB*(IB-1)+2)*FACTOR
              END DO
          END DO
      END IF
C
C-----------------------------------------------------------------------
C Compute fit of output data versus model.
C-----------------------------------------------------------------------
      CALL RMSERR(DATA,MXSECS,MXL,NDWPB,RMSOUT,RMSOUTJ)
      WRITE (LIST,652) RMSINJ,RMSIN,RMSOUTJ,RMSOUT
C
C-----------------------------------------------------------------------
C Reread the input data file and write the output data file.
C-----------------------------------------------------------------------
C     -- Set up new history records.
      IF (CLAMP) THEN
           MODE = '- CLAMP: phase and amplitude'
      ELSE IF (SCALE) THEN
           MODE = '- SCALE'
      ELSE
           MODE = '- Phase only'
      END IF
      IF (SMOOTH) THEN
           NEWHIS(81:160) = MODE(1:LEN1(MODE))//' (smoothed)'
      ELSE
           NEWHIS(81:160) = MODE
      END IF
      WRITE(NEWHIS(161:240),622) RMSOUTJ, RMSOUT
C     -- Open input file, skip header.
      OPEN (UNIT=INDAT,NAME=FFROM,TYPE='OLD',READONLY,
     1        FORM='UNFORMATTED',ERR=510)
      CALL HEADX20(.FALSE.,INDAT,IZ,ID,IY,
     1          %REF(NEWHIS),3,OUTDAT,KFLAG,CHANGE,IFEOF,0)
C     -- Read and rewrite data (including closure phases).
      DO 400 ISECS=1,NSECS
          READ(INDAT) IUTCHK,(REC(J),J=1,LENG)
          IF (IUTCHK.NE.IUT(ISECS)) CALL ERROR(
     1        'Error writing output file. '//
     2        'Input and output records do not match.')
          WRITE(OUTDAT) IUT(ISECS),(DATA(ISECS,J),J=1,LENGB),(REC(J),
     1          J=LENGB+1,LENG)
  400 CONTINUE
      WRITE(OUTDAT) -1,(0.0,I=1,LENG)
      CLOSE (UNIT=OUTDAT)
      CLOSE (UNIT=INDAT)
C-----------------------------------------------------------------------
C Finished.
C-----------------------------------------------------------------------
      RETURN
C-----------------------------------------------------------------------
C Error exits.
C-----------------------------------------------------------------------
  510 CALL ERROR('Cannot open file INPUT: '  //FFROM)
  520 CALL ERROR('Cannot open file MODFILE: '//FMODEL)
  530 CALL ERROR('Cannot open file OUTPUT: ' //FTO)
C-----------------------------------------------------------------------
      END

      REAL FUNCTION WEIGHT(DTIME)
      REAL DTIME
C-----------------------------------------------------------------------
      INTEGER INTVAL
      REAL TIME
      REAL    VALS(27)
      DATA    VALS/1.000,.993,.973,.940,.895,.841,.779,.712,.642,
     1          .570,.5000,.432,.369,.310,.257,.210,.170,.135,.106,
     2          .0819,.0625,.0470,.0349,.0256,.0185,.0131,.0092 /
C-
      WEIGHT = 0.0
      TIME   = 20*ABS(DTIME) + 1.00
      INTVAL = INT(TIME)
      IF (INTVAL.GE.27) RETURN
      WEIGHT = VALS(INTVAL) +
     1         (TIME-AINT(TIME))*(VALS(INTVAL+1)-VALS(INTVAL))
      END

      SUBROUTINE SCALEX(GAIN)
C-----------------------------------------------------------------------
C       Calculate gain correction GAIN = mean value of 1/(1 + XXX),
C       weighted by the user-supplied AERRIN.
C       (Original version - 1982 March 23)
C       S.C. Unwin              1984 Jun 16
C       Revised version - do not APPLY corrections to XXX
C       Version 7.6     1986 Aug 28; Increase MAXS to 20
C-----------------------------------------------------------------------
      INTEGER MAXS
      PARAMETER (MAXS=20)
      INTEGER IS, NS
      DOUBLE PRECISION XXX(MAXS),AERRIN(MAXS)
      COMMON  /SCACOM/ XXX,AERRIN,NS
      REAL SUM1, SUM2, XNEW, GAIN, WEIGHT
C
      SUM1 = 0.0
      SUM2 = 0.0
      DO 2 IS=1,NS
          IF(ABS(1.0 + XXX(IS)).LT.0.01) XNEW = 1.0
          IF(ABS(1.0 + XXX(IS)).GE.0.01) XNEW = 1.0/(1.0 + XXX(IS))
          GAIN = XNEW
          IF(AERRIN(IS).LT.0.0001) GOTO 3
          WEIGHT = 1.0/(AERRIN(IS)*AERRIN(IS))
          IF(ABS(XXX(IS)).LT.0.0001) WEIGHT = 0.0
          SUM1 = SUM1 +  1.0*WEIGHT
          SUM2 = SUM2 + XNEW*WEIGHT
    2 CONTINUE
      GAIN=1.0
      IF(SUM1.LT.0.0001)    RETURN
      GAIN = SUM2/SUM1
  3   IF(ABS(GAIN).LT.0.01) GAIN = 1.0
      RETURN
      END

      SUBROUTINE RMSERR(DATA,MXSECS,MXL,NDWPB,RMS,RMSJ)
C-----------------------------------------------------------------------
C Calculate rms difference between observed and model visibilities
C S.C. Unwin              1981 September 24
C Fix bug in RMS calculation; USABLE handled wrongly.  1982 May 3.
C D.L. Meier
C Extended to include MERGE format version 2 files.  1983 Jul 23.
C               S.C. Unwin    1984 Apr 16; Increase max MAXSECS to 2000
C Version 7.6     1986 Aug 28; Increase MAXS to 20
C-----------------------------------------------------------------------
      INTEGER MAXSEC, MAXS, MAXB, MAXT, MAXWPB
      PARAMETER (MAXSEC=2000)
      PARAMETER (MAXS=20)
      PARAMETER (MAXB=MAXS*(MAXS-1)/2)
      PARAMETER (MAXT=MAXB*(MAXS-2)/3)
      PARAMETER (MAXWPB=7)
      INTEGER MXL, NERR, MXSECS
      REAL DATA(MXSECS,MXL), AMPM(MAXSEC,MAXB), PHSM(MAXSEC,MAXB)
      LOGICAL USABLE
      COMMON /ERR/ NSECS,NB,AMPM,PHSM
      REAL RMS, RMSJ, AMPERR, PHSERR, SQ, SNR, PM, PD, CALC
      INTEGER ISECS, NSECS, IB, NB, NDWPB
      REAL RPDEG
      PARAMETER (RPDEG=3.141592654/180.0)
C
      NERR = 0
      RMS  = 0.0
      RMSJ = 0.0
      DO 20 ISECS=1,NSECS
          DO 10 IB=1,NB
              AMPERR = DATA(ISECS,NDWPB*(IB-1)+2)
              PHSERR = DATA(ISECS,NDWPB*(IB-1)+4)
              USABLE = (AMPERR.GT.1.0E-10) .AND. (PHSERR.GT.1.0E-10)
              IF (USABLE) THEN
                  SQ = DATA(ISECS,NDWPB*(IB-1)+1)*PHSERR*RPDEG
                  SNR = 1.0/(AMPERR*AMPERR + SQ*SQ)
                  PD = DATA(ISECS,NDWPB*(IB-1)+3)*RPDEG
C                 ! changed from PD=PNORM(....)
                  PM = PHSM(ISECS,IB)*RPDEG
                  NERR = NERR+1
                  CALC = ((DATA(ISECS,NDWPB*(IB-1)+1)*COS(PD) -
     1                     AMPM(ISECS,IB)*COS(PM))**2 +
     2                    (DATA(ISECS,NDWPB*(IB-1)+1)*SIN(PD) -
     3                     AMPM(ISECS,IB)*SIN(PM))**2)
                  RMSJ = RMSJ + CALC
                  RMS  = RMS  + CALC*SNR
              END IF
   10     CONTINUE
   20 CONTINUE
C     -- Agreement factor
      RMS  = SQRT(RMS/NERR)
C     -- RMS difference in Jy
      RMSJ = SQRT(RMSJ/NERR)
      RETURN
      END

      REAL FUNCTION PNORM(P,A)
      REAL P, A
C-----------------------------------------------------------------------
C Normalize P to +/- 180deg of A.
C-----------------------------------------------------------------------
      PNORM = AMOD(P-A,360.0)
      IF (PNORM.LT.-180.0) PNORM = PNORM+360.0
      IF (PNORM.GT. 180.0) PNORM = PNORM-360.0
      PNORM = PNORM+A
      RETURN
      END

      SUBROUTINE GAUSSJ(A,N,NP,B,M,MP,IER)
      INTEGER NMAX
      PARAMETER (NMAX=50)
      INTEGER N, NP, M, MP, IER
      DOUBLE PRECISION A(NP,NP), B(NP,MP)
      INTEGER IPIV(NMAX), INDXR(NMAX), INDXC(NMAX)
      INTEGER IROW, ICOL, L, LL, J, I, K
      DOUBLE PRECISION BIG, DUM, PIVINV
C
      IER = 0
      DO 11 J=1,N
        IPIV(J)=0
11    CONTINUE
      DO 22 I=1,N
        BIG=0.
        DO 13 J=1,N
          IF(IPIV(J).NE.1)THEN
            DO 12 K=1,N
              IF (IPIV(K).EQ.0) THEN
                IF (ABS(A(J,K)).GE.BIG)THEN
                  BIG=ABS(A(J,K))
                  IROW=J
                  ICOL=K
                ENDIF
              ELSE IF (IPIV(K).GT.1) THEN
                IER = 1
                RETURN
C               CALL ERROR('Singular matrix')
              ENDIF
12          CONTINUE
          ENDIF
13      CONTINUE
        IPIV(ICOL)=IPIV(ICOL)+1
        IF (IROW.NE.ICOL) THEN
          DO 14 L=1,N
            DUM=A(IROW,L)
            A(IROW,L)=A(ICOL,L)
            A(ICOL,L)=DUM
14        CONTINUE
          DO 15 L=1,M
            DUM=B(IROW,L)
            B(IROW,L)=B(ICOL,L)
            B(ICOL,L)=DUM
15        CONTINUE
        ENDIF
        INDXR(I)=IROW
        INDXC(I)=ICOL
        IF (A(ICOL,ICOL).EQ.0.) THEN
          IER = 1
          RETURN
C         CALL ERROR('Singular matrix.')
        END IF
        PIVINV=1./A(ICOL,ICOL)
        A(ICOL,ICOL)=1.
        DO 16 L=1,N
          A(ICOL,L)=A(ICOL,L)*PIVINV
16      CONTINUE
        DO 17 L=1,M
          B(ICOL,L)=B(ICOL,L)*PIVINV
17      CONTINUE
        DO 21 LL=1,N
          IF(LL.NE.ICOL)THEN
            DUM=A(LL,ICOL)
            A(LL,ICOL)=0.
            DO 18 L=1,N
              A(LL,L)=A(LL,L)-A(ICOL,L)*DUM
18          CONTINUE
            DO 19 L=1,M
              B(LL,L)=B(LL,L)-B(ICOL,L)*DUM
19          CONTINUE
          ENDIF
21      CONTINUE
22    CONTINUE
      DO 24 L=N,1,-1
        IF(INDXR(L).NE.INDXC(L))THEN
          DO 23 K=1,N
            DUM=A(K,INDXR(L))
            A(K,INDXR(L))=A(K,INDXC(L))
            A(K,INDXC(L))=DUM
23        CONTINUE
        ENDIF
24    CONTINUE
      RETURN
      END

C*TFIND -- find entry in table of integers
C+
      INTEGER FUNCTION TFIND (ENTRY,TABLE,N)
      INTEGER ENTRY
      INTEGER N
      INTEGER TABLE(N)
C
C Find an entry in a table of integers by simple linear search.
C
C Arguments:
C  ENTRY  (input) : the value to be found.
C  TABLE  (input) : the table to be searched.
C  N      (input) : number of elements in TABLE.
C
C Returns:
C  TFIND          : if entry is found, TFIND receives the first
C                   subscript I such that ENTRY = TABLE(I); otherwise
C                   it receives 0.
C
C History:
C  1978 Mar  7 - TJP.
C  1987 Nov 12 - convert to standard Fortran-77 (TJP).
C-----------------------------------------------------------------------
      INTEGER I
C
      DO 10 I=1,N
          IF (ENTRY.EQ.TABLE(I)) GOTO 20
   10 CONTINUE
      I = 0
   20 TFIND = I
C
      END
