      PROGRAM ARRGEN
C-----------------------------------------------------------------------
C ARRGEN generates station positions on the surface of a planet from
C  an input radius, lat, lon, and X-Y positions in meters
C
C D. L. Meier,  VAX-11 Fortran
C
C Version 1.0:  1990 Sep 25.  New program (DLM).
C Version 1.1:  1990 Nov 21.  Add XYPLOT of station positions (DLM).
C Version 1.2:  1990 Dec  6.  Add computation of standard Wye arrays
C                             (DLM).
C Version 1.3:  1990 Dec 14.  Add computation of standard crystalline
C                             (Cornwell) ring arrays. Fix plot bugs and
C                             add snapshot UV coverage plot (DLM).
C Version 1.4:  1991 Mar 18.  Convert lat/long from decimal degrees
C                             to decimal arcseconds (a la KEYIN) (DLM).
C Version 1.5:  1993 Jan  5.  Divide XYPLOT into XYPLOT & UVPLOT (DLM)
C Version 1.6:  1993 Jul 15.  Use PGPLOT routine PGCIRC (TJP).
C Version 1.7:  1993 Nov 11.  Modified to conform to Fortran-77 (TJP).
C-----------------------------------------------------------------------
C     IMPLICIT NONE
      CHARACTER*(*)  PROGNM, VERSN
      PARAMETER  (PROGNM='ARRGEN')
      PARAMETER  (VERSN='1.7 - 1993 Nov 11')
C
      INTEGER    MAXS,MAXS2,NPARS,INC,OUTC,OUTST,MAXORD
C     ! maximum number of stations
      PARAMETER  (MAXS=40)
      PARAMETER  (MAXS2=MAXS-2)
C     ! number of input parameters
      PARAMETER  (NPARS=40+2*MAXS)
C     ! units for input, output, STATIONS file
      PARAMETER  (INC=5)
      PARAMETER  (OUTC=6)
      PARAMETER  (OUTST=10)
C     ! maximum order of Cornwell ring
      PARAMETER  (MAXORD=12)
C
      DOUBLE PRECISION PI,TWOPI,RPDEG,RPASEC,RE
      PARAMETER  (PI=3.141592653589793 D0)
      PARAMETER  (TWOPI=2D0*PI)
      PARAMETER  (RPDEG=PI/180.D0)
      PARAMETER  (RPASEC=RPDEG/3600D0)
      PARAMETER  (RE=6370000.D0)
C
      CHARACTER*3  NUMBER
      CHARACTER*8  ARRNAM, ARRTYP
      CHARACTER*8  NAME(MAXS)
      CHARACTER*20 STRING
      CHARACTER*40 LEGEN1, LEGEN2
      CHARACTER*80 OUTFIL,PLTFIL
      INTEGER I, IS, IC, ICB, L, NS, IDEL, IER, LEN1, MODE, J
      INTEGER IS1, IS2, NS1, VLBOPE
      INTEGER MARK, LINEWD, NFONT, NDOTSZ, PGBEG
      INTEGER RNGORD, NARM(MAXORD)
      LOGICAL XYPLOT, UVPLOT
      REAL   PL, XP(MAXS), YP(MAXS), DRRMAX, ANTRAD
      REAL   UMAX, VMAX, UMAXC, VMAXC, UU, VV
      DOUBLE PRECISION PARS(NPARS), VALS(NPARS), ENDMRK
      DOUBLE PRECISION SX(MAXS), SY(MAXS), SX0(MAXS), SY0(MAXS), SR
      DOUBLE PRECISION RR, LAT0, LON0, CLAT0, SLAT0, CLON0, SLON0
      DOUBLE PRECISION CLAT, SLAT
      DOUBLE PRECISION ZENANG, LAT, LON, XX, YY, ZZ, DXX, DYY, DZZ, RHO
      DOUBLE PRECISION ROTATE, COSROT, SINROT
      DOUBLE PRECISION ARRDIA, EXPON, RMAX, SCLFAC, ARMANG
      DOUBLE PRECISION COSANG, SINANG
      DOUBLE PRECISION RINGX(MAXORD,MAXORD),RINGY(MAXORD,MAXORD)
      DOUBLE PRECISION STX(MAXS), STY(MAXS), STZ(MAXS)
      DOUBLE PRECISION BXIB, BYIB, BZIB, BEIB
C
      DATA ENDMRK/'/'/
      DATA PARS /'Radius', 'LATitude', 'LONgitud', 'NSTAT', 
     1         MAXS*'EWoffset', MAXS*'NSoffset', 
     2         'OUTPUT',9*' ', 'ARRNAMe', 'ZENANG', 
     3         'XYPLOT', 'PLOTFile',9*' ', 'MARK', 'LINEWidt', 
     4         'FONT', 'DOTSIZE', 'DIAMeter', 'ROTate', 'ARRTYPe',
     5         'ARRDIAM', 'EXPON', 'RINGORD', 'EAST', 'WEST', 
     6         'UVPLOT' /
      DATA VALS /RE, 0D0, 0D0, 2D0, 
     1         MAXS*0D0, MAXS*0D0, 
     2         'STATIONS',9*' ', 'ARRAY', 89.9D0, 
     3         -1D0, '?', 9*' ', -1D0, 1D0, 
     4          1D0, 0D0, 0D0, 0D0, 'SPECIFY',
     5         -1D0, 1D0, 3D0, -1D0, -1D0, 
     6         -1D0 /
      DATA RINGX(1,1)/0D0/, RINGY(1,1)/0D0/
      DATA (RINGX(I,2),I=1,2)/-0.5D0, 0.5D0/
      DATA (RINGY(I,2),I=1,2)/   0D0,   0D0/
C
C     The following DATA statements are generated by subroutine
C     SRTRNS() (more or less) when it is un-commented out
C
      DATA (RINGX(I,3),I=1,3)/
     1    0.0000000D+00,  0.4320858D+00, -0.4328533D+00/
      DATA (RINGY(I,3),I=1,3)/
     1    0.5000000D+00, -0.2515986D+00, -0.2502755D+00/
      DATA (RINGX(I,4),I=1,4)/
     1   -0.2289764D+00,  0.4152750D+00, -0.4189747D+00,  0.2398397D+00/
      DATA (RINGY(I,4),I=1,4)/
     1    0.4444882D+00, -0.2784720D+00, -0.2728738D+00,  0.4387220D+00/
      DATA (RINGX(I,5),I=1,5)/
     1    0.0000000D+00,  0.2935914D+00, -0.4751981D+00,  0.4755248D+00,
     2   -0.2941984D+00/
      DATA (RINGY(I,5),I=1,5)/
     1    0.5000000D+00, -0.4047270D+00,  0.1555202D+00,  0.1545189D+00,
     2   -0.4042856D+00/
      DATA (RINGX(I,6),I=1,6)/
     1    0.3151123D+00, -0.1666702D+00, -0.3241841D+00,  0.1795142D+00,
     2   -0.4936455D+00,  0.4915545D+00/
      DATA (RINGY(I,6),I=1,6)/
     1    0.3882064D+00, -0.4714030D+00,  0.3806634D+00, -0.4666631D+00,
     2    0.7946069D-01,  0.9150983D-01/
      DATA (RINGX(I,7),I=1,7)/
     1    0.0000000D+00, -0.1796049D+00,  0.3989847D+00, -0.4946291D+00,
     2    0.4948349D+00, -0.3999324D+00,  0.1810795D+00/
      DATA (RINGY(I,7),I=1,7)/
     1    0.5000000D+00, -0.4666281D+00,  0.3013487D+00, -0.7308832D-01,
     2   -0.7168254D-01,  0.3000896D+00, -0.4660578D+00/
      DATA (RINGX(I,8),I=1,8)/
     1    0.0000000D+00, -0.1328101D-02,  0.2965855D+00, -0.4024266D+00,
     2    0.4935313D+00, -0.4938155D+00,  0.4014871D+00, -0.2949823D+00/
      DATA (RINGY(I,8),I=1,8)/
     1    0.5000000D+00, -0.4999981D+00,  0.4025382D+00, -0.2967358D+00,
     2   -0.8016437D-01, -0.7839692D-01, -0.2980067D+00,  0.4037142D+00/
      DATA (RINGX(I,9),I=1,9)/
     1    0.0000000D+00, -0.2064091D+00,  0.2867274D+00, -0.4332467D+00,
     2    0.4976544D+00, -0.4982555D+00,  0.4330290D+00, -0.2910378D+00,
     3    0.2119921D+00/
      DATA (RINGY(I,9),I=1,9)/
     1    0.5000000D+00, -0.4554068D+00,  0.4096187D+00, -0.2495928D+00,
     2    0.4837120D-01,  0.4173138D-01, -0.2499709D+00,  0.4065673D+00,
     3   -0.4528345D+00/
      DATA (RINGX(I,10),I=1,10)/
     1    0.0000000D+00, -0.3283518D-02,  0.3462886D+00, -0.2445619D+00,
     2    0.4496760D+00, -0.4904117D+00,  0.4888961D+00, -0.4520736D+00,
     3    0.2425469D+00, -0.3447906D+00/
      DATA (RINGY(I,10),I=1,10)/
     1    0.5000000D+00, -0.4999890D+00,  0.3606716D+00, -0.4361070D+00,
     2    0.2186115D+00, -0.9744845D-01, -0.1047875D+00,  0.2136100D+00,
     3   -0.4372308D+00,  0.3621033D+00/
      DATA (RINGX(I,11),I=1,11)/
     1    0.0000000D+00,  0.1667254D+00, -0.2899925D+00,  0.3457167D+00,
     2   -0.4568465D+00,  0.4989131D+00, -0.4990395D+00,  0.4572277D+00,
     3   -0.3457744D+00,  0.2912835D+00, -0.1671764D+00/
      DATA (RINGY(I,11),I=1,11)/
     1    0.5000000D+00, -0.4713836D+00,  0.4073131D+00, -0.3612197D+00,
     2    0.2032023D+00, -0.3294844D-01, -0.3096992D-01,  0.2023420D+00,
     3   -0.3611643D+00,  0.4063911D+00, -0.4712238D+00/
      DATA (RINGX(I,12),I=1,12)/
     1    0.0000000D+00,  0.5587120D-03, -0.2082298D+00,  0.2959012D+00,
     2   -0.4542729D+00,  0.4033182D+00, -0.4983997D+00,  0.4979849D+00,
     3   -0.4024793D+00,  0.4527406D+00, -0.2927386D+00,  0.2072576D+00/
      DATA (RINGY(I,12),I=1,12)/
     1    0.5000000D+00, -0.4999995D+00,  0.4545767D+00, -0.4030411D+00,
     2    0.2088913D+00, -0.2955227D+00,  0.3996766D-01,  0.4483931D-01,
     3   -0.2966645D+00,  0.2121913D+00, -0.4053441D+00,  0.4550205D+00/
C-----------------------------------------------------------------------
C Formats
C-----------------------------------------------------------------------
 6000 FORMAT(' ',A,' generates a STATIONS file from LAT, LON, and',
     1         ' X-Y coordinates'/' (Version ',A,')'/)
 6010 FORMAT(A)
 6020 FORMAT (A8,2X,F10.1,2(F16.10,9X),F5.1)
 6030 FORMAT (/1X,A,/1X,A)
C
C---------------------------------------------------------------------
C Re-order Crystalline Ring antennas; write out DATA statements
C (only necessary when adding more rings to program)
C---------------------------------------------------------------------
C
C     CALL SRTRNS(RINGX,RINGY,MAXORD)
C     DO 2 J=1,MAXORD
C       DO 1 I=1,J
C         SR = RINGX(I,J)**2 + RINGY(I,J)**2
C         SR = SQRT(SR)
C         WRITE (OUTC,'(A,I2,A,I2,A,F9.6)') 'R(',I,',',J,') =',SR
C   1   CONTINUE
C       WRITE (OUTC,'(/)')
C   2 CONTINUE
C
C---------------------------------------------------------------------
C Read control parameters
C---------------------------------------------------------------------
C
      WRITE (OUTC,6000) PROGNM, VERSN
    5 MODE = 0
      WRITE (OUTC,6010) ' Control parameters (end with /):'
      CALL KEYIN(PARS,VALS,NPARS,ENDMRK,MODE,INC,OUTC)
C
C---------------------------------------------------------------------
C     Deterimine station coordinates and antenna radius
C---------------------------------------------------------------------
      RR     = VALS(1)
      LAT0   = VALS(2)*RPASEC
      LON0   = VALS(3)*RPASEC
C     -- Set input longitude sense; default is EAST longitude
C     -- Note that parameter EAST is dummy
      IF (VALS(39+2*MAXS) .EQ. 0D0) LON0 = -LON0
      CLAT0  = COS(LAT0)
      SLAT0  = SIN(LAT0)
      CLON0  = COS(LON0)
      SLON0  = SIN(LON0)
      NS      = VALS(4)
      DO 8 IS=1,NS
        SX0(IS) = VALS(4+IS)
        SY0(IS) = VALS(4+MAXS+IS)
    8 CONTINUE
      ARRDIA  = VALS(35+2*MAXS)
      EXPON   = VALS(36+2*MAXS)
      RNGORD  = VALS(37+2*MAXS) + 0.1D0
      IF (RNGORD .GT. MAXORD) THEN
        WRITE (OUTC,'(A,A,I2)') ' +++ WARNING +++ ',
     1    'Requested ring order exceeds maximum = ',MAXORD
        GO TO 5
      END IF
C
C     -- Determine array type and act accordingly
      WRITE (ARRTYP, '(10A8)') VALS(34+2*MAXS)
C
      IF (ARRTYP .EQ. 'SPECIFY') THEN
C
C     -- Check that enough coordinates have been specified
C     -- Default:  specify station coordinates in meters
        DO 10 IS=MAXS,1,-1
          IF ((SX0(IS).NE.0D0) .AND. (SY0(IS).NE.0D0))
     1      GO TO 15
   10   CONTINUE
   15   CONTINUE
        IF (IS .NE. NS) THEN
          WRITE (OUTC,'(A,A)') ' +++ WARNING +++ ',
     1      'Number of stations not consistent with coordinates'
          GO TO 5
        END IF

      ELSE IF (ARRTYP .EQ. 'WYE') THEN
C
C       -- VLA-like WYE configuration
C       -- Distribute number of antennas as evenly as possible
        DO 20 I=1,3
          NARM(I) = 0
   20   CONTINUE
        IS = 0
        RMAX = 0D0
        DO 25 J=1,(1+NS/3)
          DO 25 I=1,3
            IS = IS + 1
            NARM(I) = NARM(I) + 1
            RMAX = MAX(RMAX,DBLE(J)**EXPON)
            IF (IS .GE. NS) GO TO 30
   25   CONTINUE
   30   CONTINUE
C
C       -- Create each arm of the WYE
        SCLFAC = ARRDIA/(2D0*RMAX)
        IS = 0
        DO 35 I=1,3
          ARMANG = (I-1)*TWOPI/3D0
          COSANG = COS(ARMANG)
          SINANG = SIN(ARMANG)
          DO 35 J=1,NARM(I)
            IS = IS + 1
            SR = SCLFAC*(DBLE(J)**EXPON)
            SX0(IS) = SR*SINANG
            SY0(IS) = SR*COSANG
   35     CONTINUE
C
      ELSE IF (ARRTYP .EQ. 'CRING') THEN
C
C       -- Crystalline (Cornwell) ring configuration
C       -- Distribute number of antennas as evenly as possible,
C       -- starting with first at north and remaining with decreasing
C       -- baseline
        DO 40 I=1,RNGORD
          NARM(I) = 0
   40   CONTINUE
        IS = 0
        RMAX = 0D0
        DO 45 J=1,(1+NS/RNGORD)
          DO 45 I=1,RNGORD
            IS = IS + 1
            NARM(I) = NARM(I) + 1
            SR = RINGX(I,RNGORD)**2 + RINGY(I,RNGORD)**2
            SR = SQRT(SR)*(DBLE(J)**EXPON)
            RMAX = MAX(RMAX,SR)
            IF (IS .GE. NS) GO TO 50
   45   CONTINUE
   50   CONTINUE
C
C       -- Create each arm of the concentric RINGs
C       -- Simple rings are created by setting NSTAT=RNGORD
        SCLFAC = ARRDIA/(2D0*RMAX)
        IS = 0
        DO 55 I=1,RNGORD
          DO 55 J=1,NARM(I)
            IS = IS + 1
            SR = SCLFAC*(DBLE(J)**EXPON)
            SX0(IS) = SR*RINGX(I,RNGORD)
            SY0(IS) = SR*RINGY(I,RNGORD)
   55     CONTINUE
C
      END IF
C
C     -- Compute antenna positions and
C        maximum distance from array origin, rotating if necessary
      ROTATE   = VALS(33+2*MAXS)*RPDEG
      COSROT   = COS(ROTATE)
      SINROT   = SIN(ROTATE)
      DRRMAX = 0.0
      DO 60 IS=1,NS
        SX(IS) = SX0(IS)*COSROT - SY0(IS)*SINROT
        XP(IS) = SX(IS)
        SY(IS) = SX0(IS)*SINROT + SY0(IS)*COSROT
        YP(IS) = SY(IS)
        DRRMAX = MAX(DRRMAX,(XP(IS)*XP(IS)+YP(IS)*YP(IS)))
   60 CONTINUE
      DRRMAX = 2.0*SQRT(DRRMAX)
      ANTRAD = VALS(32+2*MAXS)/2D0
C
C---------------------------------------------------------------------
C     Check station list for duplicates
C---------------------------------------------------------------------
      IDEL = 0
   70 CONTINUE
      DO 80 IS=1,NS
        CALL STACHK(IS,NS,SX,SY,OUTC,IDEL,IER)
        IF (IER .NE. 0) THEN
          IDEL = IDEL + 1
          GO TO 70
        END IF
   80 CONTINUE
C
C---------------------------------------------------------------------
C     Set character strings
C---------------------------------------------------------------------
      WRITE (OUTFIL,  '(10A8)') (VALS(I),I=4+2*MAXS+1,4+2*MAXS+10)
C
C---------------------------------------------------------------------
C     Determine array name; truncate to 5 characters
C     Set station names
C---------------------------------------------------------------------
      WRITE (ARRNAM, '(10A8)') VALS(15+2*MAXS)
      L = LEN1(ARRNAM)
      IF (L .GT. 5) L = 5
      DO 90 IS=1,NS
        WRITE (NUMBER,'(I3)') IS
        ICB = 1
        DO 85 IC=1,2
          IF (NUMBER(IC:IC) .EQ. ' ') ICB = ICB + 1
   85 CONTINUE
        NAME(IS) = ARRNAM(1:L)//'-'//NUMBER(ICB:3)
   90 CONTINUE
C
C---------------------------------------------------------------------
C     Set Zenith Angle limit for telescopes
C---------------------------------------------------------------------
      ZENANG = VALS(16+2*MAXS)
C
C---------------------------------------------------------------------
C     Set XYPLOT parameters
C---------------------------------------------------------------------
      XYPLOT = VALS(17+2*MAXS)  .EQ. 0D0
      UVPLOT = VALS(40+2*MAXS)  .EQ. 0D0
      WRITE(PLTFIL,'(10A8)') (VALS(I),I=18+2*MAXS, 27+2*MAXS)
      MARK   = VALS(28+2*MAXS)
      LINEWD = VALS(29+2*MAXS) + 0.1D0
      NFONT  = VALS(30+2*MAXS) + 0.1D0
      NDOTSZ = VALS(31+2*MAXS) + 0.1D0
C
      IF (XYPLOT .AND. UVPLOT) THEN
          IF (NDOTSZ.LT.1) NDOTSZ = LINEWD
          IER = PGBEG(PL,PLTFIL,2,1)
          IF (IER.NE.1) THEN
              CALL ERROR('Cannot open file PLOTFILE: '//PLTFIL)
              XYPLOT = .FALSE.
          ELSE 
              CALL PGQINF('DEV/TYPE', PLTFIL, L)
              WRITE (OUTC,'(A)') ' Plot will go to '//PLTFIL(:L)
          END IF
      ELSE IF (XYPLOT) THEN
          IF (NDOTSZ.LT.1) NDOTSZ = LINEWD
          IER = PGBEG(PL,PLTFIL,1,1)
          IF (IER.NE.1) THEN
              CALL ERROR('Cannot open file PLOTFILE: '//PLTFIL)
              XYPLOT = .FALSE.
          ELSE 
              CALL PGQINF('DEV/TYPE', PLTFIL, L)
              WRITE (OUTC,'(A)') ' Plot will go to '//PLTFIL(:L)
          END IF
      ELSE IF (UVPLOT) THEN
          IF (NDOTSZ.LT.1) NDOTSZ = LINEWD
          IER = PGBEG(PL,PLTFIL,1,1)
          IF (IER.NE.1) THEN
              CALL ERROR('Cannot open file PLOTFILE: '//PLTFIL)
              XYPLOT = .FALSE.
          ELSE 
              CALL PGQINF('DEV/TYPE', PLTFIL, L)
              WRITE (OUTC,'(A)') ' Plot will go to '//PLTFIL(:L)
          END IF
      END IF
C
C-----------------------------------------------------------------------
C XYPLOT Section
C-----------------------------------------------------------------------
C
      IF (XYPLOT) THEN
C
C         -- Initialize plot
          LEGEN1 = ARRNAM
          I = LEN1(LEGEN1)+2
          WRITE (LEGEN1(I:),'(A)') 'STATIONS'
          WRITE (LEGEN2,'(A,F6.1,A)') 'ROTATION ANGLE = ',
     1        ROTATE/RPDEG, ' DEGREES'
          WRITE (OUTC,6030) LEGEN1(1:LEN1(LEGEN1)),
     1                      LEGEN2(1:LEN1(LEGEN2))
          CALL PGQINF('TERMINAL', STRING, L)
Cvms      IF (STRING.EQ.'YES')
Cvms 1        CALL LIB$GET_COMMAND(STRING, 'Type <CR> to see plot: ', L)
          CALL PGBBUF
          CALL PGSLW(LINEWD)
          CALL PGSCF(NFONT)
          CALL PGSCI(5)
          CALL PGENV(-DRRMAX,DRRMAX,-DRRMAX,DRRMAX,1,0)
          CALL PGLAB('X  (meters)','Y  (meters)',' ')
          CALL PGMTXT('T',2.5,0.5,0.5,LEGEN1)
          CALL PGMTXT('T',0.8,0.5,0.5,LEGEN2)
          CALL PGSLW(NDOTSZ)
C
C         -- Plot antenna positions
          CALL PGSCI(3)
          CALL PGSFS(2)
          DO 100 IS=1,NS
              CALL PGCIRC(XP(IS),YP(IS),ANTRAD)
  100     CONTINUE
C         CALL PGPT(NS,XP,YP,MARK)
          CALL PGEBUF
          CALL PGQINF('TERMINAL', STRING, L)
Cvms      IF (STRING.EQ.'YES')
Cvms 1        CALL LIB$GET_COMMAND(STRING, 'Type <CR> to continue: ', L)
      END IF
C
C---------------------------------------------------------------------
C     Compute station coordinates and write to file
C---------------------------------------------------------------------
      IER = VLBOPE(OUTST, OUTFIL, 'TEXT', 'NEW', OUTFIL)
      IF (IER.NE.1) CALL ERROR('Cannot create file OUTPUT: '//
     1   OUTFIL(1:LEN1(OUTFIL)))
      DO 110 IS=1,NS
        XX    = RR*CLAT0*CLON0
        YY    = RR*CLAT0*SLON0
        ZZ    = RR*SLAT0
        DXX   = -SX(IS)*SLON0 - SY(IS)*SLAT0*CLON0
        DYY   =  SX(IS)*CLON0 - SY(IS)*SLAT0*SLON0
        DZZ   =  SY(IS)*CLAT0
        XX    = XX + DXX
        YY    = YY + DYY
        ZZ    = ZZ + DZZ
        RHO   = XX*XX + YY*YY
        RHO   = SQRT(RHO)
        LAT = ATAN2(ZZ,RHO)
        IF (LAT .GT.  PI/2D0) LAT = LAT - TWOPI
        IF (LAT .LT. -PI/2D0) LAT = LAT + TWOPI
C       -- NOTE:  output in STATIONS file is WEST longitude between
C       -- -PI and +PI
        LON   = -ATAN2(YY,XX)
        IF (LON .GT.  PI)   LON = LON - TWOPI
        IF (LON .LT. -PI)   LON = LON + TWOPI
        WRITE (OUTST,6020) NAME(IS), RR, LAT/RPDEG, LON/RPDEG, ZENANG
        SLAT = SIN(LAT)
        CLAT = COS(LAT)
        STX(IS) = RR*CLAT*COS(LON)
        STY(IS) = RR*CLAT*SIN(LON)
        STZ(IS) = RR*SLAT
  110   CONTINUE
      CLOSE(UNIT=OUTST)
C
C-----------------------------------------------------------------------
C UVPLOT Section
C-----------------------------------------------------------------------
C
      IF (UVPLOT) THEN
C
C           Initialization and Plot Limits
C
        UMAXC = 0.0
        VMAXC = 0.0
        DO 130 IS2=2,NS
          NS1 = IS2-1
          DO 120 IS1=1,NS1
            BXIB = (STX(IS1)-STX(IS2))
            BYIB = (STY(IS1)-STY(IS2))
            BZIB = (STZ(IS1)-STZ(IS2))
            BEIB = SQRT(BXIB**2+BYIB**2)
            UMAXC = MAX(UMAXC,REAL(BEIB))
            VMAXC = MAX(VMAXC, REAL(ABS(BZIB*CLAT0)) +
     1                                 REAL(ABS(BEIB*SLAT0)) )
  120     CONTINUE
  130   CONTINUE
        UMAX  = 1.2*UMAXC
        VMAX  = 1.2*VMAXC
C
C           Plot Legends
C
        WRITE (LEGEN2,'(A,F6.1,A)') 'UVPLOT'
        CALL PGBBUF
        CALL PGSLW(LINEWD)
        CALL PGSCF(NFONT)
        CALL PGSCI(5)
        CALL PGENV(UMAX,-UMAX,-VMAX,VMAX,1,1)
        CALL PGLAB('U  (meters)','V  (meters)',' ')
        CALL PGMTXT('T',2.5,0.5,0.5,LEGEN1)
        CALL PGMTXT('T',0.8,0.5,0.5,LEGEN2)
        CALL PGSLW(NDOTSZ)
C
C           Plot (U,V) Points
C
        DO 150 IS1=1,NS-1
          DO 140 IS2=IS1+1,NS
            BXIB = STX(IS1) - STX(IS2)
            BYIB = STY(IS1) - STY(IS2)
            BZIB = STZ(IS1) - STZ(IS2)
            UU = -SLON0*BXIB + CLON0*BYIB
            VV = -SLAT0*CLON0*BXIB - SLAT0*SLON0*BYIB 
     1          + CLAT0*BZIB
            CALL PGSCI(3)
            CALL PGPT(1,UU,VV,MARK)
            CALL PGSCI(2)
            CALL PGPT(1,-UU,-VV,MARK)
  140     CONTINUE
  150   CONTINUE
        CALL PGEBUF
C
      END IF
C
      IF (XYPLOT .OR. UVPLOT) THEN
        CALL PGEND
      END IF
C
C-----------------------------------------------------------------------
      END
C-----------------------------------------------------------------------
      SUBROUTINE STACHK(I,NS,SX,SY,OUTC,IDEL,IER)
C
C     IMPLICIT NONE
C
      INTEGER I, NS, IER, OUTC, IDEL
      DOUBLE PRECISION  SX(NS), SY(NS)
C
      INTEGER J
      DOUBLE PRECISION  XCHK, YCHK
C
      IER  = 0
      XCHK = SX(I)
      YCHK = SY(I)
      DO 10 J=I+1,NS
        IF ((XCHK .EQ. SX(J)) .AND. (YCHK .EQ. SY(J))) THEN
          IER = 1
          WRITE(OUTC,200) I+IDEL
          CALL STADEL(I,NS,SX,SY)
          RETURN
        END IF
   10 CONTINUE
      RETURN
  200 FORMAT('Deleting Station #',I3,' because it has a duplicate',
     *       ' later in list')
      END
C-----------------------------------------------------------------------
      SUBROUTINE STADEL(I,NS,SX,SY)
C
C     IMPLICIT NONE
C
      INTEGER I, NS
      DOUBLE PRECISION  SX(NS), SY(NS)
C
      INTEGER J
C
      DO 10 J=I+1,NS
        SX(J-1) = SX(J)
        SY(J-1) = SY(J)
   10 CONTINUE
      NS = NS - 1
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE SRTRNS(RINGX,RINGY,ORDMAX)
      INTEGER ORDMAX
      DOUBLE PRECISION RINGX(ORDMAX,ORDMAX), RINGY(ORDMAX,ORDMAX)
C
C Sort antennas in crystalline rings according to decreasing baseline
C and write out DATA statements for main program.  This allows odd
C odd antennas to be placed in intelligent locations and the RINGX,RINGY
C DATA statements to be presorted.
C
      INTEGER    OUTC,MAXORD
C     ! unit for output
      PARAMETER  (OUTC=6)
C     ! maximum order of Cornwell ring
      PARAMETER  (MAXORD=12)
C
      INTEGER I, J
      DOUBLE PRECISION RINGX0(MAXORD,MAXORD), RINGY0(MAXORD,MAXORD)
C
      DATA (RINGX0(I,3),I=1,3)/
     1      0.0000000D+0,  0.4320858D+0, -0.4328533D+0/
      DATA (RINGY0(I,3),I=1,3)/
     1      0.5000000D+0, -0.2515986D+0, -0.2502755D+0/
      DATA (RINGX0(I,4),I=1,4)/
     1     -0.4189747D+0,  0.2398397D+0, -0.2289764D+0,  0.4152750D+0/
      DATA (RINGY0(I,4),I=1,4)/
     1     -0.2728738D+0,  0.4387220D+0,  0.4444882D+0, -0.2784720D+0/
      DATA (RINGX0(I,5),I=1,5)/
     1      0.0000000D+0,  0.4755248D+0, -0.2941984D+0, -0.4751981D+0,  
     2      0.2935914D+0/
      DATA (RINGY0(I,5),I=1,5)/
     1      0.5000000D+0,  0.1545189D+0, -0.4042856D+0,  0.1555202D+0, 
     2     -0.4047270D+0/
      DATA (RINGX0(I,6),I=1,6)/
     1      0.3151123D+0, -0.4936455D+0, -0.3241841D+0,  0.1795142D+0, 
     2     -0.1666702D+0,  0.4915545D+0/
      DATA (RINGY0(I,6),I=1,6)/
     1      0.3882064D+0,  0.79460695D-1, 0.3806634D+0, -0.4666631D+0, 
     2     -0.4714030D+0,  0.91509834D-1/
      DATA (RINGX0(I,7),I=1,7)/
     1      0.0000000D+0,  0.4948349D+0,  0.3989847D+0,  0.1810795D+0, 
     2     -0.3999324D+0, -0.1796049D+0, -0.4946291D+0/
      DATA (RINGY0(I,7),I=1,7)/
     1      0.5000000D+0, -0.71682535D-1, 0.3013487D+0, -0.4660578D+0,  
     2      0.3000896D+0, -0.4666281D+0, -0.73088318D-1/
      DATA (RINGX0(I,8),I=1,8)/
     1      0.0000000D+0,  0.2965855D+0, -1.3281007D-3, -0.4938155D+0, 
     2     -0.4024266D+0,  0.4935313D+0,  0.4014871D+0, -0.2949823D+0/
      DATA (RINGY0(I,8),I=1,8)/
     1      0.5000000D+0,  0.4025382D+0, -0.4999981D+0, -0.78396916D-1, 
     2     -0.2967358D+0, -0.80164373D-1,-0.2980067D+0,  0.4037142D+0/
      DATA (RINGX0(I,9),I=1,9)/
     1      0.0000000D+0,  0.2119921D+0,  0.4330290D+0, -0.4332467D+0,  
     2      0.2867274D+0, -0.4982555D+0,  0.4976544D+0, -0.2064091D+0,
     3     -0.2910378D+0/
      DATA (RINGY0(I,9),I=1,9)/
     1      0.5000000D+0, -0.4528345D+0, -0.2499709D+0, -0.2495928D+0,  
     2      0.4096187D+0,  0.41731380D-1, 0.48371203D-1,-0.4554068D+0,
     3      0.4065673D+0/
      DATA (RINGX0(I,10),I=1,10)/
     1      0.0000000D+0,  0.4888961D+0, -0.2445619D+0,  0.2425469D+0,  
     2      0.4496760D+0,  0.3462886D+0, -0.3447906D+0, -0.32835179D-2,
     3     -0.4520736D+0, -0.4904117D+0/
      DATA (RINGY0(I,10),I=1,10)/
     1      0.5000000D+0, -0.1047875D+0, -0.4361070D+0, -0.4372308D+0,  
     2      0.2186115D+0,  0.3606716D+0,  0.3621033D+0, -0.4999890D+0,
     3      0.2136100D+0, -0.97448446D-1/
      DATA (RINGX0(I,11),I=1,11)/
     1      0.0000000D+0,  0.2912835D+0, -0.4568465D+0, -0.3457744D+0,  
     2      0.3457167D+0,  0.4572277D+0,  0.4989131D+0, -0.4990395D+0,
     3     -0.1671764D+0, -0.2899925D+0,  0.1667254D+0/
      DATA (RINGY0(I,11),I=1,11)/
     1      0.5000000D+0,  0.4063911D+0,  0.2032023D+0, -0.3611643D+0, 
     2     -0.3612197D+0,  0.2023420D+0, -0.32948442D-1,-0.30969921D-1,
     3     -0.4712238D+0,  0.4073131D+0, -0.4713836D+0/
      DATA (RINGX0(I,12),I=1,12)/
     1      0.0000000D+0,  0.4527406D+0, -0.4983997D+0,  0.4033182D+0, 
     2     -0.2082298D+0, -0.4024793D+0, -0.4542729D+0,  0.55871205D-3, 
     3      0.2072576D+0,  0.4979849D+0,  0.2959012D+0, -0.2927386D+0/
      DATA (RINGY0(I,12),I=1,12)/
     1      0.5000000D+0,  0.2121913D+0,  0.39967656D-1,-0.2955227D+0,  
     2      0.4545767D+0, -0.2966645D+0,  0.2088913D+0, -0.4999995D+0,  
     3      0.4550205D+0,  0.44839311D-1,-0.4030411D+0, -0.4053441D+0/
C
      DO 20 J=1,MAXORD
        DO 10 I=1,J
          RINGX(I,J) = RINGX0(I,J)
          RINGY(I,J) = RINGY0(I,J)
   10   CONTINUE
   20 CONTINUE
      DO 30 J=3,MAXORD
        CALL SRTRNG(RINGX(1,J),RINGY(1,J),J)
        WRITE (OUTC,'(A,I2,A,I2,A)')
     1    '      DATA (RINGX(I,', J,'),I=1,', J, ')/'
        WRITE (OUTC,'(5X,''*'',1X,4(D16.7,'', ''))') (RINGX(I,J),I=1,J)
        WRITE (OUTC,'(A,I2,A,I2,A)')
     1    '      DATA (RINGY(I,', J,'),I=1,', J, ')/'
        WRITE (OUTC,'(5X,''*'',1X,4(D16.7,'', ''))') (RINGY(I,J),I=1,J)
   30 CONTINUE
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE SRTRNG(RINGX,RINGY,RNGORD)
      INTEGER RNGORD
      DOUBLE PRECISION RINGX(RNGORD), RINGY(RNGORD)
C
C Sort antennas in a crystalline ring according to decreasing baseline
C This allows odd antennas to be placed in intelligent locations
C
      DOUBLE PRECISION     PI,TWOPI
      PARAMETER  (PI=3.141592653589793 D0)
      PARAMETER  (TWOPI=2D0*PI)
C
      INTEGER I, J, IMIN, JMAX
      DOUBLE PRECISION ANGMIN, NTHANG
      DOUBLE PRECISION XTMP, YTMP
      DOUBLE PRECISION BASE2, BASE2M
C
C Find antenna closest to north
      ANGMIN = PI
      IMIN = 1
      DO 10 I=1,RNGORD
        NTHANG = ATAN2(RINGX(I),RINGY(I))
        IF (NTHANG .GT.  PI) NTHANG = NTHANG - TWOPI
        IF (NTHANG .LT. -PI) NTHANG = NTHANG + TWOPI
        NTHANG = ABS(NTHANG)
        IF (NTHANG .LT. ANGMIN) THEN
          ANGMIN = NTHANG
          IMIN = I
        END IF
   10 CONTINUE
C
C Rearrange so antenna closest to north is first in array
      XTMP = RINGX(IMIN)
      YTMP = RINGY(IMIN)
      RINGX(IMIN) = RINGX(1)
      RINGY(IMIN) = RINGY(1)
      RINGX(1) = XTMP
      RINGY(1) = YTMP
C
C Sort remaining antennas according to decreasing baseline
C
      DO 30 I=2,RNGORD
C
C Find antenna furthest away from I-1
        BASE2M = 0.0
        JMAX = I
        DO 20 J=I,RNGORD
          BASE2 = (RINGX(J)-RINGX(I-1))**2 + (RINGY(J)-RINGY(I-1))**2
          IF (BASE2 .GT. BASE2M) THEN
            BASE2M = BASE2
            JMAX = J
          END IF
   20   CONTINUE
C
C Rearrange so antenna furthest away is next in array
        XTMP = RINGX(JMAX)
        YTMP = RINGY(JMAX)
        RINGX(JMAX) = RINGX(I)
        RINGY(JMAX) = RINGY(I)
        RINGX(I) = XTMP
        RINGY(I) = YTMP
   30 CONTINUE
      RETURN
      END
