C*B2FITS
C+
************************************************************************
*    B2FITS internals
*
* The file B2FITS.FOR defines the main program, B2FITS,
* and the following subroutines:
*
* READ_SCRATCH    
* FIND_FILES      
* MAKE_NAME       
* READ_PCR        
* PR_PRC          
* PR_GRP          
* PR_DLY          
* PR_PHS          
* PR_CDO          
* PR_OPR
* PUTSCR          
* ASCCNV          
* ASCCNL
* BITCNV
* ANTABL
* SUTABL
* 
* Four INCLUDE modules are required to define the format of the PCR 
* file and for common blocks:
*
* CORDEF
* PCRDEF
* B2FDEF
* B2FSEL
*
* The "standard" version of these can be found in VLBSOURCE:xxxxxxx.FOR.
* 
* Additional subroutines are required from the VLBLIB, FITSLIB, and
* SLALIB libraries.
************************************************************************
C--

C*B2FITS
C+
      PROGRAM B2FITS
C
C B2FITS converts output PCR files from  Caltech/JPL Block-II VLBI 
C into  AIPS-compatible  FITS format.  
C
C Program author:
C  Tim Pearson (TJP).
C
C History:
C 
C Version 1.0:  1986 Jun  1 - new program, based on PHFITS v3.1.
C Version 1.1:  1986 Aug 27 - do not quit on invalid configuration.
C Version 1.2:  1986 Aug 28 - convert character parameters to uppercase.
C Version 1.3:  1986 Oct  6 - skip records to find PRC table.
C Version 1.4:  1986 Oct 15 - more information about spikes.
C               1986 Oct 16 - correct format bug.
C Version 1.5:  1986 Oct 22 - add LIST parameter.
C Version 1.6:  1986 Dec 19 - add TAI-UTC correction.
C Version 1.7:  1986 Dec 22 - change scaling of time tags in FITS file;
C                             unit is now 1/20 sec.
C Version 1.8:  1986 Dec 23 - new format for phase model table.
C Version 1.9:  1987 Jan 13 - better error checking on output.
C Version 2.0:  1987 Feb 17 - do not write uvw in scratch file.
C Version 2.1:  1987 Apr 20 - fix bug: TAI-UTC was ignored.
C Version 2.2:  1987 Jun 29 - report correlator program version; handle
C                             new phase model format.
C Version 2.3:  1987 Nov  5 - new selection of data in READ_SCRATCH.
C Version 2.4:  1987 Nov 24 - add BADCHAN parameter.
C Version 2.5:  1988 Jul  5 - hardware baseline order changed from
C                             12 13 14 23 24 34 to 12 13 23 14 24 34.
C Version 2.6:  1988 Sep 12 - add EPOCH parameter.
C Version 2.7:  1989 Jan 31 - add correln time to output, etc.
C Version 2.8:  1990 Apr  5 - allow antenna names longer than 8 chars;
C                             use correct TAI-UTC; check COFFLCD.
C Version 2.9:  1990 Aug  3 - allow up to 14 BADCHANs.
C Version 2.10: 1990 Oct 21 - add INVALIDS parameters.
C Version 2.11: 1990 Nov 27 - display operator comments.
C Version 3.0:  1992 Apr 29 - default OBSERVER=expt; check UT1-UTC etc.
C Version 4.0:  1992 Nov 12 - write multisource file.
C Version 4.1:  1993 Jan  7 - correct bug in source selection.
C Version 4.2:  1993 Jan 27 - correct error in labeling WW parameter
C                             (introduced in v4.0).
C Version 4.3:  1993 Nov 11 - remove use of uninitialized variable BW
C                             (this was a bug, but harmless).
C Version 4.4:  1993 Dec  2 - display breakdown by baseline correctly
C                             when there are more than 20 antennas.
C-----------------------------------------------------------------------
C
C Constants.
C
      CHARACTER*(*) PROGNM, VERSN, DASH, LOGNM
      PARAMETER (PROGNM='B2FITS')
      PARAMETER (VERSN='4.4 - 1993 Dec 2')
      PARAMETER (DASH='------------------------------------'//
     1                '------------------------------------')
      PARAMETER (LOGNM=PROGNM//'_LOG.LIS')
      INTEGER   INC,OUTC,LOG,SCR
      PARAMETER (INC=5,OUTC=6,LOG=1,SCR=2)
      INTEGER   MAXS,MAXB,MAXS1,NPARS,MAXF
C     -- maximum number of stations
      PARAMETER (MAXS=32)
      PARAMETER (MAXS1=MAXS-1)
      PARAMETER (MAXB=MAXS*MAXS1/2)
      PARAMETER (NPARS=90+MAXS)
C     -- maximum number of input files
      PARAMETER (MAXF=100)
      REAL*8    PI,RPDEG,DCORIN
      PARAMETER (PI=3.1415926535897932846D0)
      PARAMETER (RPDEG=PI/180D0)
C
C Storage.
C
      INCLUDE 'b2fsel.inc'
      INCLUDE 'b2fspi.inc'
C
C Variables.
C
      CHARACTER*8      BUF,DATMAP,DATOBS
      CHARACTER*16     OBSERV,IMCLAS,TEST
      CHARACTER*80     HISTORY,IMNAME
      CHARACTER*40     TRIMIT
      CHARACTER*20     CORTIM(MAXF)
      CHARACTER*128    INDSN,OUTDSN,FILES(MAXF),LOGDSN
      CHARACTER*2880   FTSCRD
      CHARACTER*(MAXS) TELESC
      CHARACTER*16     CHRA, CHDEC
      CHARACTER*20     RUNTIM
      INTEGER   I,ID,IER,IMON,IS,IY,JD,IF,NF,L,LIN,IPP, IS1,IS2,
     1          LOUT,MONTH,STOKES,USERNO,IMSEQ,MODE,DUM1,
     2          NRCL,YEAR,REFDAY, NDATA(MAXF), DELTAT, EPOSEL
      LOGICAL   DISMNT
      REAL*8    EPOCH
      REAL*4    WTSCAL,UVWSCL,BSCALE
      REAL*8    PARS(NPARS),VALS(NPARS),ENDMRK
      REAL*8    FREQ,DFREQ,DJM
      INTEGER   IP,NS,NREFCH,NFRST,NLAST, D1,D2, T1,T2, NQBAD, N1
      INTEGER   COUNT(MAXS,MAXS)
      INTEGER   SRCCNT(MAXSRC)
      INTEGER*2 KEYBUF(9), LRL
C
C Functions called.
C
      CHARACTER*1 STNABR
      CHARACTER*8 TIMEC
      INTEGER   LEN1
      INTEGER   SOR$BEGIN_SORT, SOR$SORT_MERGE, SOR$END_SORT
      DOUBLE PRECISION SLA_DAT
      EXTERNAL  DSC$K_DTYPE_D, DSC$K_DTYPE_L
C
C KEYIN parameters.
C
      DATA PARS / 'INput', 9*' ',
     1            'OUTput', 9*' ',
     2            'STOKes',
     3            'OBServer', ' ',
     4            'USerno',
     5            'IMName', 9*' ',
     6            'IMClass', ' ',
     7            'IMSeq',
     8            'SOurce', ' ',
     9            2*'STARt',2*'STOP',
     A            'DISMount',
     B            'REFday',
     C            'EXPerime', ' ',
     D            'SPikelim',
     E            'LIstfile', 9*' ',
     F            'TAI-UTC',
     G            'EPoch',' ',
     H            28*'BADchan',
     I            'INValids',
     J            MAXS*'STATions'/
      DATA VALS / '***UNSET', 9*' ',
     1            10*' ',
     2            -2D0,
     3            2*' ',
     4            0D0,
     5            10*' ',
     6            2*' ',
     7            0D0,
     8            '*', ' ',
     9            1D0,0D0,366D0,86399D0,
     A            1D0,
     B            0D0,
     C            '*', ' ',
     D            1D0,
     E            10*' ',
     F            0D0,
     G            '1950',' ',
     H            28*0D0,
     I            0.5D0,
     J            '*', MAXS1*' '/
C
      DATA ENDMRK / '/' /
C-----------------------------------------------------------------------
 1000 FORMAT(I2.2,'/',I2.2,'/',I2.2)
 1010 FORMAT(1X,A,' converts Block-II PCR files'
     1 ' into AIPS-compatible UVFITS format'/' (Version ',A,')'//
     2 ' Control parameters (INPUT, OUTPUT, SOURCE, STATIONS,...):')
 1020 FORMAT('  Umax = ',F10.2,' Vmax = ',F10.2,' Wmax =',F10.2)
 1030 FORMAT('Maximum amplitude = ',F12.6)
 1040 FORMAT('Time range: ',I4,',',I3.3,',',A8,' to ',I3.3,',',A8)
 1050 FORMAT('Reference day number =',I4,'   (',A,')')
 1051 FORMAT('TAI-UTC =',I4,' s')
 1052 FORMAT('UT1-UTC =',F8.4, ' s; Xpole =',F8.4,' Ypole =',
     1       F8.4,' arcsec')
 1060 FORMAT('AIPS WTSCAL= ',E17.10)
 1090 FORMAT(1X,I8,' data groups written in file'/1X,A//
     1       1X,A,' completed successfully')
 1100 FORMAT(A)
 1200 FORMAT(' Error closing ',A,' - please check/delete later.')
C-----------------------------------------------------------------------
C
C Set cpu timer and obtain start time.
C
      CALL LIB$INIT_TIMER
      CALL SYS$ASCTIM(,RUNTIM,,)
C
C Today's date in FITS format.
C
      CALL IDATE(IMON,ID,IY)
      WRITE (DATMAP,1000) ID,IMON,IY
C
C Introduction.
C
      WRITE (OUTC,1010) PROGNM, VERSN
C
C Read parameters.
C
      MODE = 0
      CALL KEYIN(PARS,VALS,NPARS,ENDMRK,MODE,INC,OUTC)
      WRITE (INDSN,'(10A8)') (VALS(I),I=1,10)
      CALL UPCASE(INDSN)
      WRITE (OUTDSN,'(10A8)') (VALS(I),I=11,20)
      CALL UPCASE(OUTDSN)
      WRITE (LOGDSN,'(10A8)') (VALS(I),I=49,58)
      CALL UPCASE(LOGDSN)
      STOKES = VALS(21)
      WRITE (OBSERV,'(2A8)') VALS(22),VALS(23)
      CALL UPCASE(OBSERV)
      USERNO = VALS(24)
      WRITE (IMNAME,'(10A8)') (VALS(I),I=25,34)
      CALL UPCASE(IMNAME)
      WRITE (IMCLAS,'(2A8)') VALS(35),VALS(36)
      CALL UPCASE(IMCLAS)
      IMSEQ = VALS(37)
      WRITE (QSRC(1),'(2A8)') VALS(38),VALS(39)
      CALL UPCASE(QSRC(1))
      IF (QSRC(1).EQ.'*') THEN
          QAUSRC = .TRUE.
          QSRCN = 0
      ELSE
          QAUSRC = .FALSE.
          QSRCN = 1
      END IF
      WRITE (QEXP,'(2A8)') VALS(46),VALS(47)
      CALL UPCASE(QEXP)
      QAUEXP = QEXP.EQ.'*'
      SPIKEL = VALS(48)
      QINVAL = VALS(90)
      DO I=1,28
          QBAD(I) = VALS(61+I)
      END DO
      NQBAD = 0
      DO I=28,1,-1
          IF (QBAD(I).NE.0) THEN
              NQBAD = I
              GOTO 17
          END IF
      END DO
   17 CONTINUE
      NS = 0
      DO IS=1,MAXS
          NS = NS+1
          WRITE (QANT(NS),'(A8)') VALS(90+IS)
          CALL UPCASE(QANT(NS))
          IF (QANT(NS).EQ.' ') NS = NS-1
          QANTX(IS) = 0D0
          QANTY(IS) = 0D0
          QANTZ(IS) = 0D0
      END DO
      IF (QANT(1).EQ.'*') THEN
          QAUTO = .TRUE.
          QANTN = 0
      ELSE
          QAUTO = .FALSE.
          QANTN = NS
      END IF
C               Dismount drive ???
      DISMNT = VALS(44).LT.1D0
C               Start and stop time. Convert to days
      QSTART = (VALS(40)-1.0D0) + VALS(41)/86400.0D0
      QSTOP  = (VALS(42)-1.0D0) + VALS(43)/86400.0D0
C Reference day (day of year).
      REFDAY = VALS(45)
C TAI-UTC correction.
      DELTAT = NINT(VALS(59))
C Epoch.
      WRITE (TEST,'(2A8)') VALS(60),VALS(61)
      CALL UPCASE(TEST)
      IF (TEST.EQ.'DATE') THEN
          EPOSEL = 0
      ELSE IF (TEST.EQ.'1950') THEN
          EPOSEL = 1
      ELSE
          CALL ERROR('Illegal value: EPOCH='//TEST)
      END IF
      QUT1TC = 0D0
      QXPOLE = 0D0
      QYPOLE = 0D0
C
C Expand wild cards and check input files exist.
C
      CALL FIND_FILES(INDSN,'VLB_PCR:.PCR',NF,FILES,MAXF)
      IF (NF.LT.1) CALL ERROR('Cannot find input file(s): '//
     1                            INDSN(1:LEN1(INDSN)))
C
C Initialize VMS SORT utility.
C
      NFREQ = 4
      NRCL = 5 + 2*NFREQ
      KEYBUF(1) = 2			! two keys
      KEYBUF(2) = %LOC(DSC$K_DTYPE_D)	! key 1: REAL*8 UT
      KEYBUF(3) = 0			! ascending
      KEYBUF(4) = 0			! position
      KEYBUF(5) = 8			! size
      KEYBUF(6) = %LOC(DSC$K_DTYPE_L)	! key 2: INTEGER BASE
      KEYBUF(7) = 0			! ascending
      KEYBUF(8) = 8			! position
      KEYBUF(9) = 4			! size
      LRL = NRCL*4
      IER = SOR$BEGIN_SORT(KEYBUF,LRL)
      IF (.NOT.IER) CALL EXIT(IER)
C
C Open the output log file, FITS file.
C
      IF (OUTDSN.EQ.' ') THEN
          IF (QAUSRC) THEN
              CALL MAKE_NAME('MULTI',OUTDSN,LOUT)
          ELSE
              CALL MAKE_NAME(QSRC(1),OUTDSN,LOUT)
          END IF
          OUTDSN = OUTDSN(1:LOUT)//'.UVF'
      END IF
      CALL FITS_INIT(OUTDSN,OUTDSN,LOUT,IER)
      IF (IER.NE.0) CALL ERROR('Cannot open output FITS file.')
      OPEN (UNIT=LOG, FILE=LOGDSN, FORM='FORMATTED',
     1      STATUS='NEW', DISPOSE='KEEP', IOSTAT=IER,
     2      CARRIAGECONTROL='LIST', DEFAULTFILE=LOGNM)
      IF (IER.NE.0) CALL ERROR('Cannot open log file.')
C
C List the files.
C
      WRITE (OUTC,1100) ' Input files:'
      DO IF=1,NF
          L = MIN(126,LEN1(FILES(IF)))
          WRITE (OUTC,'(1X,I4,1X,A)') IF, FILES(IF)(1:L)
      END DO
C-----------------------------------------------------------------------
C Summarize data selection parameters in log file.
C-----------------------------------------------------------------------
      WRITE (LOG,'(''Log file from ''A,T60,A)') PROGNM, RUNTIM
      WRITE (LOG,'(/''Requested input files: '',A)') 
     1                                           INDSN(1:LEN1(INDSN))
      WRITE (LOG,'( ''Requested experiment:  '',A)') QEXP
      IF (QAUSRC) THEN
          WRITE (LOG,'(''Requested sources:     *'')')
      ELSE
          WRITE (LOG,'(''Requested sources: '',4X,5A16)')
     1           (QSRC(I),I=1,QSRCN)
      END IF
      IF (QAUTO) THEN
          WRITE (LOG,'(''Requested antennas:    *'')')
      ELSE
          WRITE (LOG,'(''Requested antennas:'',4X,5A10)')
     1           (QANT(I),I=1,QANTN)
      END IF
      D1 = QSTART
      T1 = NINT(86400D0*(QSTART-D1))
      D2 = QSTOP
      T2 = NINT(86400D0*(QSTOP-D2))
      WRITE (LOG,'( ''Requested start/stop:  '',I3.3,'','',A8,2X,
     1           I3.3,'','',A8)') D1+1, TIMEC(T1), D2+1, TIMEC(T2)
      IF (EPOSEL.EQ.1) THEN
          WRITE (LOG,'(''Requested epoch:  1950'')')
      ELSE
          WRITE (LOG,'(''Requested epoch:  DATE'')')
      END IF
      IF (NQBAD.GT.0) THEN
          WRITE (LOG,'(''Requested BADCHAN:'',5X,14(I3,'',''))')
     1         (QBAD(I),I=1,NQBAD)
      END IF
      WRITE (LOG,'(''Requested Invalids threshold:'',F7.3)') QINVAL
C
C-----------------------------------------------------------------------
C First phase: read the input files and pass data to sort utility.
C-----------------------------------------------------------------------
C
      WRITE (OUTC,1100) ' Beginning phase 1: reading input files.'
      CALL LIB$SHOW_TIMER
      AMAX = 0.0
      UMAX = 0.0
      VMAX = 0.0
      WMAX = 0.0
      TOTEPC = 0.0
      UTMAX = 0D0
      UTMIN = 1D37
      IP = 0
C
C Read input files.
C
      DO IF=1,NF
          LIN = LEN1(FILES(IF))
          CALL READ_PCR(LOG, FILES(IF)(1:LIN), NDATA(IF), CORTIM(IF))
          IP = IP+NDATA(IF)
      END DO
C
C Dismount tape drive if requested.
C
      IF (DISMNT) CALL SYS$DISMOU(FILES(1),)
C
C Derive "telescope name" from antenna names.
C Summarize source and antenna parameters in log file.
C
      TELESC = ' '
      DO IS=1,QANTN
          TELESC(IS:IS) = STNABR(QANT(IS))
      END DO
      WRITE (LOG,'(/''Experiment: '',A)') QEXP
      WRITE (LOG,'( ''Telescope:  '',A)') TELESC
      DO I=1,QSRCN
          WRITE (LOG,'(/''Source:     '',A)') QSRC(I)
          CALL PANGLE('H', QRA(I), CHRA, 4)
          CALL PANGLE('D', QDEC(I), CHDEC, 3)
          WRITE (LOG,'(''RA:  '',A,''  Dec:  '',A)') CHRA, CHDEC
          CALL PANGLE('H', QRA0(I), CHRA, 4)
          CALL PANGLE('D', QDEC0(I), CHDEC, 3)
          WRITE (LOG,'(''RA0: '',A,''  Dec0: '',A)') CHRA, CHDEC
      END DO
      WRITE (LOG,'(/''Antennas:'',24X,''X'',15X,''Y'',15X,''Z'')') 
      DO I=1,QANTN
          WRITE (LOG,'(I6,2X,A16,3F16.3)') 
     1             I, QANT(I), QANTX(I), QANTY(I), QANTZ(I)
      END DO
      WRITE (LOG,'(/''Total number of uv-points:      '',I12)') IP
      WRITE (LOG,'(/''Reference (LO) frequency (MHz): '',F12.4)') 
     1       QFREF*1D-6
      WRITE (LOG,'( ''Number of bins:                 '',I7)') NFREQ
      WRITE (LOG,'( ''Bin spacing (MHz):              '',F12.4)') 
     1       QFDEL*1D-6
      WRITE (LOG,'( ''Bin centers (MHz):              '',4F12.4)') 
     1       ((QFREF + (I-QFPIX)*QFDEL)*1D-6, I=1,NFREQ)
C
      IF (IP.LE.0) CALL ERROR('No data found for requested source '//
     1                        'and stations.')
C
      EPOCH = TOTEPC/FLOAT(IP)
      YEAR = INT(EPOCH)
C
C Date of first observation (reference date).
C
      IY = YEAR
      IMON = 1
      IF (REFDAY.LE.0) THEN
          ID = UTMIN+1
          REFDAY = ID
      ELSE
          ID = REFDAY
      END IF
      CALL TDATE(IY,IMON,ID,JD,MONTH,DUM1)
      WRITE (DATOBS,1000) ID,IMON,MOD(IY,100)
      IF (DELTAT.EQ.0) THEN
          CALL SLA_CALDJ(IY, IMON, ID, DJM, IER)
          DELTAT = SLA_DAT(DJM)
      END IF
C
C Julian date at beginning of year.
C
      IY = YEAR
      IMON = 1
      ID = 1
      CALL TDATE(IY,IMON,ID,JD,MONTH,DUM1)
C
C Tell VMS to finish sorting.
C
      IER = SOR$SORT_MERGE()
      IF (.NOT.IER) CALL EXIT(IER)
C
C-----------------------------------------------------------------------
C Second phase: get the data back from SORT and write the FITS file.
C-----------------------------------------------------------------------
C
      WRITE (OUTC,1100) ' Beginning phase 2: copying data to output.'
      CALL LIB$SHOW_TIMER
C
C Scaling factors to fit into I*2.
C
C>      UVWSCL = MAX(UMAX,VMAX,WMAX)/32767.0
      UVWSCL = 1E10/32767.0
      BSCALE = AMAX/32767.0
      WTSCAL = 0.1/BSCALE
C
C Write FITS header.
C
      CALL FITS_WLOG ('SIMPLE',.TRUE.,'Standard UVFITS',IER)
      CALL FITS_WINT ('BITPIX',    16,' ',IER)
      CALL FITS_WINT ('NAXIS' ,     6,' ',IER)
      CALL FITS_WINT ('NAXIS1',     0,'Groups only',IER)
      CALL FITS_WINT ('NAXIS2',     3,' ',IER)
      CALL FITS_WINT ('NAXIS3',     1,' ',IER)
      CALL FITS_WINT ('NAXIS4', NFREQ,' ',IER)
      CALL FITS_WINT ('NAXIS5',     1,' ',IER)
      CALL FITS_WINT ('NAXIS6',     1,' ',IER)
      CALL FITS_WLOG ('EXTEND',.TRUE.,'Tables follow',IER)
      CALL FITS_WLOG ('BLOCKED',.TRUE.,'Tape may be blocked',IER)
C
      IF (QSRCN.GT.1) THEN
          CALL FITS_WSTR ('OBJECT',  'MULTI','Multi-source file',IER)
      ELSE
          CALL FITS_WSTR ('OBJECT',  QSRC(1),'Source name',IER)
      END IF
      CALL FITS_WSTR ('TELESCOP',TELESC,'VLBI antennas',IER)
      CALL FITS_WSTR ('INSTRUME', 'VLBI', ' ',IER)
      IF (OBSERV.EQ.' ') OBSERV = QEXP
      CALL FITS_WSTR ('OBSERVER', OBSERV,   ' ',IER)
      CALL FITS_WSTR ('DATE-OBS',DATOBS,'First day of run (dd/mm/yy)',
     1                IER)
      CALL FITS_WSTR ('DATE-MAP',DATMAP,'dd/mm/yy',IER)
C
      CALL FITS_WREAL('BSCALE',BSCALE,'Real = Tape*BSCALE + BZERO',
     1                IER)
      CALL FITS_WREAL('BZERO',0.0,' ',IER)
      CALL FITS_WSTR ('BUNIT', 'COR.COEF', 
     1                'Correlation coefficient', IER)
      IF (EPOSEL.EQ.1) THEN
          CALL FITS_WREAL('EPOCH',1950.0,'Epoch of RA/Dec (B1950.0)',
     1                    IER)
      ELSE
          CALL FITS_WDBLE('EPOCH',EPOCH,'Epoch of RA/Dec (approx)',IER)
      ENDIF
      CALL FITS_WSTR ('ORIGIN', 'BLOCK-2',
     1                'Caltech/JPL VLBI Processor', IER)
      CALL FITS_WSTR ('DATE',DATMAP,'Creation date (dd/mm/yy)',IER)
C
C FITS coordinate parameters.
C
      CALL FITS_WSTR ('CTYPE2','COMPLEX','1=real, 2=imag, 3=weight',
     1                IER)
      CALL FITS_WINT ('CRVAL2',        1,' ',IER)
      CALL FITS_WINT ('CDELT2',        1,' ',IER)
      CALL FITS_WINT ('CRPIX2',        1,' ',IER)
      CALL FITS_WSTR ('CTYPE3', 'STOKES',
     1                'Correlator: -1=RR, -2=LL, -3=RL, -4=LR',IER)
      CALL FITS_WINT ('CRVAL3',   STOKES,' ',IER)
      CALL FITS_WINT ('CDELT3',       -1,' ',IER)
      CALL FITS_WINT ('CRPIX3',        1,' ',IER)
      CALL FITS_WSTR ('CTYPE4',   'FREQ','Frequency',IER)
      CALL FITS_WDBLE('CRVAL4',    QFREF,'LO frequency, Hz',IER)
      CALL FITS_WDBLE('CDELT4',    QFDEL,'Channel width, Hz',IER)
      CALL FITS_WDBLE('CRPIX4',    QFPIX,' ',IER)
      CALL FITS_WSTR ('CTYPE5','RA','Right ascension, degrees',IER)
      IF (EPOSEL.EQ.1) THEN
          CALL PANGLE('H', QRA0(1), CHRA, 4)
          CALL FITS_WDBLE('CRVAL5', QRA0(1)/RPDEG, CHRA, IER)
      ELSE
          CALL PANGLE('H', QRA(1), CHRA, 4)
          CALL FITS_WDBLE('CRVAL5', QRA(1)/RPDEG, CHRA,IER)
      END IF
      CALL FITS_WINT ('CDELT5',         0,' ',IER)
      CALL FITS_WINT ('CRPIX5',         1,' ',IER)
      CALL FITS_WSTR ('CTYPE6','DEC','Declination, degrees',IER)
      IF (EPOSEL.EQ.1) THEN
          CALL PANGLE('D', QDEC0(1), CHDEC, 3)
          CALL FITS_WDBLE('CRVAL6', QDEC0(1)/RPDEG, CHDEC, IER)
      ELSE
          CALL PANGLE('D', QDEC(1), CHDEC, 3)
          CALL FITS_WDBLE('CRVAL6', QDEC(1)/RPDEG, CHDEC, IER)
      END IF
      CALL FITS_WINT ('CDELT6',         0,' ',IER)
      CALL FITS_WINT ('CRPIX6',         1,' ',IER)
C
C FITS random parameters.
C
      CALL FITS_WLOG ('GROUPS',.TRUE.,'Groups format',IER)
      CALL FITS_WINT ('GCOUNT',IP,'Number of uv samples',IER)
      CALL FITS_WINT ('PCOUNT',10,'True value = tape value * PSCALn '//
     1                '+ PZEROn.',IER)
      CALL FITS_WSTR ('PTYPE1','UU',
     1                'baseline u projection, seconds',IER)
      CALL FITS_WREAL('PSCAL1',UVWSCL,' ',IER)
      CALL FITS_WINT ('PZERO1',0,' ',IER)
      CALL FITS_WSTR ('PTYPE2','UU',' ',IER)
      CALL FITS_WREAL('PSCAL2',UVWSCL/65400.0,' ',IER)
      CALL FITS_WINT ('PZERO2',0,' ',IER)
      CALL FITS_WSTR ('PTYPE3','VV',
     1                'baseline v projection, seconds',IER)
      CALL FITS_WREAL('PSCAL3',UVWSCL,' ',IER)
      CALL FITS_WINT ('PZERO3',0,' ',IER)
      CALL FITS_WSTR ('PTYPE4','VV',' ',IER)
      CALL FITS_WREAL('PSCAL4',UVWSCL/65400.0,' ',IER)
      CALL FITS_WINT ('PZERO4',0,' ',IER)
      CALL FITS_WSTR ('PTYPE5','WW',
     1                'baseline w projection, seconds',IER)
      CALL FITS_WREAL('PSCAL5',UVWSCL,' ',IER)
      CALL FITS_WINT ('PZERO5',0,' ',IER)
      CALL FITS_WSTR ('PTYPE6','WW',' ',IER)
      CALL FITS_WREAL('PSCAL6',UVWSCL/65400.0,' ',IER)
      CALL FITS_WINT ('PZERO6',0,' ',IER)
      CALL FITS_WSTR ('PTYPE7','BASELINE','256*ANT1 + ANT2',IER)
      CALL FITS_WINT ('PSCAL7',1,' ',IER)
      CALL FITS_WINT ('PZERO7',0,' ',IER)
      CALL FITS_WSTR ('PTYPE8','DATE','TAI Julian Date part 1',IER)
      CALL FITS_WDBLE('PSCAL8',1D0/60D0,'Days/60',IER)
      CALL FITS_WDBLE('PZERO8',JD-0.5D0,' ',IER)
      CALL FITS_WSTR ('PTYPE9','DATE','TAI Julian Date part 2',IER)
      CALL FITS_WDBLE('PSCAL9',1D0/1728000D0,'Days/1728000 (1/20 s)',
     1                IER)
      CALL FITS_WINT ('PZERO9',0,' ',IER)
      CALL FITS_WSTR ('PTYPE10','SOURCE','Source id number',IER)
      CALL FITS_WINT ('PSCAL10',1,' ',IER)
      CALL FITS_WINT ('PZERO10',0,' ',IER)
C
C FITS history records.
C
      CALL FITS_WCMT('HISTORY',DASH,IER)
      CALL FITS_WCMT('HISTORY', PROGNM//' versn:    '//VERSN, IER)
      CALL FITS_HIS(IER)
      CALL FITS_WCMT('HISTORY', 'Input file(s):   '//INDSN, IER)
      DO 125 IF=1,NF
          IF (NDATA(IF).GT.0) THEN
              CALL LIB$TRIM_FILESPEC(FILES(IF), TRIMIT)
              WRITE (HISTORY, '(4X,A,1X,I8,'' points'')') 
     1                         TRIMIT, NDATA(IF)
              CALL FITS_WCMT('HISTORY',HISTORY,IER)
              WRITE (HISTORY, '(4X,''Correlated on '',A)') CORTIM(IF)
              CALL FITS_WCMT('HISTORY',HISTORY,IER)
          END IF
  125 CONTINUE
      CALL FITS_WCMT('HISTORY', 'Output file:     '//OUTDSN,IER)
      CALL FITS_WCMT('HISTORY','Maximum baseline in millions of'//
     1               ' wavelengths:',IER)
      WRITE (HISTORY,1020) UMAX*1E-6,VMAX*1E-6,WMAX*1E-6
      CALL FITS_WCMT('HISTORY',HISTORY,IER)
      WRITE (HISTORY,1030) AMAX
      CALL FITS_WCMT('HISTORY',HISTORY,IER)
      WRITE (LOG,1100) HISTORY
      D1 = UTMIN
      T1 = NINT(86400D0*(UTMIN-D1))
      D2 = UTMAX
      T2 = NINT(86400D0*(UTMAX-D2))
      WRITE (HISTORY,1040) YEAR,D1+1, TIMEC(T1), D2+1, TIMEC(T2)
      CALL FITS_WCMT('HISTORY',HISTORY,IER)
      WRITE (LOG,1100) HISTORY
      WRITE (HISTORY,1050) REFDAY, DATOBS
      CALL FITS_WCMT('HISTORY',HISTORY,IER)
      WRITE (LOG,1100) HISTORY
      WRITE (HISTORY,1051) DELTAT
      CALL FITS_WCMT('HISTORY',HISTORY,IER)
      WRITE (LOG,1100) HISTORY
      WRITE (HISTORY,1052) QUT1TC, QXPOLE, QYPOLE
      CALL FITS_WCMT('HISTORY',HISTORY,IER)
      WRITE (LOG,1100) HISTORY
C
C History records to confuse AIPS.
C
      IF (IMNAME.NE.' ') 
     1    CALL FITS_WCMT('HISTORY','AIPS IMNAME='''//
     2                   IMNAME(1:LEN1(IMNAME))//'''',IER)
      IF (IMCLAS.NE.' ') 
     1    CALL FITS_WCMT('HISTORY','AIPS IMCLASS='''//
     2                   IMCLAS(1:LEN1(IMCLAS))//'''',IER)
      IF (IMSEQ.NE.0) THEN
          WRITE (BUF,'(I8.8)',IOSTAT=IER) IMSEQ
          CALL FITS_WCMT('HISTORY','AIPS IMSEQ='//BUF,IER)
      END IF
      IF (USERNO.NE.0) THEN
          WRITE (BUF,'(I8.8)',IOSTAT=IER) USERNO
          CALL FITS_WCMT('HISTORY','AIPS USERNO='//BUF,IER)
      END IF
      WRITE (HISTORY,1060) WTSCAL
      CALL FITS_WCMT('HISTORY',HISTORY,IER)
      CALL FITS_WCMT('HISTORY','AIPS SORT ORDER=''TB'' ',IER)
      CALL FITS_WCMT('HISTORY',DASH,IER)
      CALL FITS_WEND(IER)
C
C Write the data records.
C
      DO IS2=1,QANTN
          DO IS1=1,QANTN
              COUNT(IS1,IS2) = 0
          END DO
      END DO
      DO I=1,QSRCN
          SRCCNT(I) = 0
      END DO
      CALL READ_SCRATCH(OUTC,IP,NFREQ,UVWSCL,BSCALE,DCORIN,DELTAT,
     1                  COUNT, MAXS, SRCCNT, MAXSRC)
      IER = SOR$END_SORT()
      IF (.NOT.IER) CALL EXIT(IER)
C
C Write FITS-AIPS antenna table.
C
      CALL ANTABL(TELESC, DELTAT, DATOBS, IER)
      IF (IER.NE.0) CALL ERROR('Error while writing antenna table.')
C
C Write FITS-AIPS source table.
C
      CALL SUTABL(IER)
      IF (IER.NE.0) CALL ERROR('Error while writing source table.')
C
C Close output file.
C
      CALL FITS_CLOSE(IER)
      IF (IER.NE.0) CALL ERROR('Error while writing FITS file.')
C
C Insert new IP (number of uv-points). If you change the position
C of GCOUNT in the header, beware!
C
      OPEN (UNIT=SCR, FILE=OUTDSN, STATUS='OLD',
     1      FORM='UNFORMATTED', RECORDTYPE='FIXED',
     2      RECL=720, ACCESS='DIRECT', IOSTAT=IER)
      READ (SCR'2) FTSCRD
      WRITE (FTSCRD(661:670),'(I10)') IP
      WRITE (SCR'2) FTSCRD
      CLOSE (UNIT=SCR)
C
C Write output file name and size in log file.
C
      LOUT = MIN(LOUT,80)
      WRITE (OUTC,1090) IP, OUTDSN(1:LOUT), PROGNM
      CALL LIB$SHOW_TIMER
      WRITE (LOG, '(/''Output file: '',A)') OUTDSN(1:LOUT)
      WRITE (LOG, '( ''Number of distinct uv-points:  '',I12)') IP
C
C Write a summary of data points by stations.
C
      WRITE (LOG, '(''Breakdown by baseline:'')')
      N1 = MIN(20,QANTN)
      WRITE (LOG, '(12X,20(3X,A1,1X))') (TELESC(IS2:IS2),IS2=1,N1)
      DO IS1=1,QANTN
          WRITE (LOG, '(A8,1X,A1,'': '',20I5.0)') QANT(IS1), 
     1        TELESC(IS1:IS1), (COUNT(IS1,IS2),IS2=1,N1)
      END DO
      IF (QANTN.GT.20) THEN
          N1 = MIN(40,QANTN)
          WRITE (LOG, '(12X,20(3X,A1,1X))') (TELESC(IS2:IS2),IS2=21,N1)
          DO IS1=1,QANTN
              WRITE (LOG, '(A8,1X,A1,'': '',20I5.0)') QANT(IS1), 
     1            TELESC(IS1:IS1), (COUNT(IS1,IS2),IS2=21,N1)
          END DO
      END IF
C
C Write a summary of data points by source.
C
      WRITE (LOG, '(''Breakdown by source:'')')
      DO I=1,QSRCN
          WRITE (LOG, '(2X,A16'': '',I12)') QSRC(I), SRCCNT(I)
      END DO
C
C Write execution summary in log file.
C
      WRITE (LOG, '(/A/)') PROGNM//' execution summary:'
      WRITE (LOG, '(A,T20,A)') PROGNM//' version:', VERSN
      CALL REPORT(LOG)
      CALL REPCPU(LOG)
C-----------------------------------------------------------------------
      END

C*READ_SCRATCH
C+
      SUBROUTINE READ_SCRATCH (OUTC, IP, NFREQ, UVWSCL, BSCALE,
     1                         DCORIN, DELTAT, COUNT, NS, SRCCNT,
     2                         NSRC)
      INTEGER       OUTC, IP, NFREQ, DELTAT, NS, NSRC
      REAL*4        UVWSCL, BSCALE
      INTEGER       COUNT(NS,NS), SRCCNT(NSRC)
C
C READ_SCRATCH: Read the scratch file, scale the data, and write them
C into the output FITS file.
C
C Arguments:
C  OUTC    (input, I*4)  Fortran unit number for messages to user.
C  IP      (in/out,I*4)  Number of data records.
C  NFREQ   (input, I*4)  Number of frequency channels.
C  UVWSCL  (input, R*4)  Scale factor for u,v,w.
C  BSCALE  (input, R*4)  Scale factor for amplitudes.
C  DCORIN
C  DELTAT  (input, I*4)  TAI-UTC correction (s).
C  COUNT   (output)      Count of data points by baseline.
C  SRCCNT  (output)      Count of points by source.
C-----------------------------------------------------------------------
C
C Local variables.
C
      INTEGER       I,J,IER,K1,K2,K3,K4,K5,LENERR,IS1,IS2
      INTEGER*2     I2DATA(10)
      LOGICAL       LAST
      REAL*8        UTOLD,DUT,EPS,DEPS,DCORIN
      REAL*4        AMP,AMPOLD,WTOLD
      INTEGER       IGOOD,IOLD,ISROLD
      COMPLEX*8     CCOLD(4)
      DATA EPS/0.10E-05/
      INTEGER       SOR$RETURN_REC, LEN1
      REAL*8        DAT
      CHARACTER*80  TXTERR
C
C Buffer for return of data from sort; sized assuming NFREQ=4.
C
      CHARACTER*52  BUFFER
      REAL*8        UT
      INTEGER*4     IBASE, ISRC
      REAL*4        WT
      COMPLEX*8     CC(4)
      COMMON        /B2FREC/ UT,IBASE,ISRC,WT,CC
      EQUIVALENCE   (BUFFER, UT)
C-----------------------------------------------------------------------
      DAT = DELTAT/86400D0
      IGOOD = 0
      DO 100 I=1,IP
          IER = SOR$RETURN_REC(BUFFER)
          IF (IER.NE.1) CALL EXIT(IER)
C         -- convert UTC to TAI by adding DELTAT
          UT = UT + DAT
C         -- compute amplitude
          AMP = 0.0E0
          DO J = 1,NFREQ
              AMP = AMP + SQRT(REAL(CC(J))*REAL(CC(J))+
     1                 AIMAG(CC(J))*AIMAG(CC(J)))
          END DO
          AMP = AMP/FLOAT(NFREQ)
          IF (I.EQ.1) GOTO 90
C
C Look for new time/baseline tag.
C
          DUT = DABS(UT-UTOLD)
          DEPS = DUT*60.0D0*24.0D0*3600.0D0
          IF((DEPS.GT.0.20D0*DCORIN).OR.(IBASE.NE.IOLD)) GOTO 10
C
C Same tags. Look for best data.
C Accept the new point if weight is >1.2 times previous one, or
C if weight is > previous weight and amplitude > previous.
C
          IF (WT.GT.(1.2*WTOLD)) GOTO 90
          IF (WT.GT.WTOLD .AND. AMP.GT.AMPOLD) GOTO 90
          GOTO 100
C
C Write best data for previous time/baseline tag.
C
   10     I2DATA(1) = 0		! u
          I2DATA(2) = 0
          I2DATA(3) = 0		! v
          I2DATA(4) = 0
          I2DATA(5) = 0		! w
          I2DATA(6) = 0
          I2DATA(7) = IOLD
          I2DATA(8) = INT(UTOLD*60D0)
          I2DATA(9) = NINT((UTOLD-I2DATA(8)/60D0)*1728000D0)
          I2DATA(10)= ISROLD		! source number
          CALL FITS_WRAYS(I2DATA,10,IER)
          IF (IER.NE.0) GOTO 300
          I2DATA(3) = NINT(10.0*WTOLD)
          DO 80 J=1,NFREQ
            I2DATA(1) = NINT(REAL(CCOLD(J))/BSCALE)
            I2DATA(2) = NINT(AIMAG(CCOLD(J))/BSCALE)
            CALL FITS_WRAYS(I2DATA,3,IER)
            IF (IER.NE.0) GOTO 300
   80     CONTINUE
          IGOOD = IGOOD + 1
          IS1 = IOLD/256
          IS2 = IOLD - 256*IS1
          COUNT(IS1,IS2) = COUNT(IS1,IS2)+1
          SRCCNT(ISROLD) = SRCCNT(ISROLD)+1
C
C                          Save this data record for future use.
C
   90   CONTINUE
        UTOLD   = UT
        WTOLD   = WT
        IOLD    = IBASE
        ISROLD  = ISRC
        AMPOLD  = AMP
        DO J = 1,NFREQ
           CCOLD(J) = CC(J)
        END DO
  100 CONTINUE
C
C Last data record.
C
      I2DATA(1) = 0
      I2DATA(2) = 0
      I2DATA(3) = 0
      I2DATA(4) = 0
      I2DATA(5) = 0
      I2DATA(6) = 0
      I2DATA(7) = IOLD
      I2DATA(8) = INT(UTOLD*60D0)
      I2DATA(9) = NINT((UTOLD-I2DATA(8)/60D0)*1728000D0)
      I2DATA(10)= ISROLD	! source number
      CALL FITS_WRAYS(I2DATA,10,IER)
      IF (IER.NE.0) GOTO 300
      I2DATA(3) = NINT(10.0*WTOLD)
      DO 180 J=1,NFREQ
          I2DATA(1) = NINT(REAL(CCOLD(J))/BSCALE)
          I2DATA(2) = NINT(AIMAG(CCOLD(J))/BSCALE)
          CALL FITS_WRAYS(I2DATA,3,IER)
          IF (IER.NE.0) GOTO 300
  180 CONTINUE
      IS1 = IOLD/256
      IS2 = IOLD - 256*IS1
      COUNT(IS1,IS2) = COUNT(IS1,IS2)+1
      SRCCNT(ISROLD) = SRCCNT(ISROLD)+1
      IP = IGOOD + 1
      RETURN
C-----------------------------------------------------------------------
C Error exit.
C
  300 WRITE (OUTC,'(/1X,A,I6)') '+++ERROR+++ FITS error',IER
      CALL ERRSNS(K1,K2,K3,K4,K5)
      IF (K2.NE.0) THEN
          CALL SYS$GETMSG(%VAL(K2),LENERR,TXTERR,,)
          WRITE (OUTC,'(1X,A)') TXTERR(1:LENERR)
      END IF
      IF (K3.NE.0) THEN
          CALL SYS$GETMSG(%VAL(K3),LENERR,TXTERR,,)
          WRITE (OUTC,'(1X,A)') TXTERR(1:LENERR)
      END IF
      CALL ERROR('Error writing FITS file.')
C-----------------------------------------------------------------------
      END

C*FIND_FILES
C+
      SUBROUTINE FIND_FILES (INPUT, DEFAUL, NF, FILES, MAXF)
C
C B2FITS: expand wild cards in input file list.
C
C Arguments:
C  INPUT  (input, character)  Requested file name, possibly including
C                             wild cards.
C  DEFAUL (input, character)  Default file name for VMS parsing.
C  NF     (output, integer)   Number of files found.
C  FILES  (output, character) List of expanded file names.
C  MAXF   (input, integer)    Maximum number of files.
C-----------------------------------------------------------------------
      INTEGER RMS$_NMF
      PARAMETER  (RMS$_NMF = '000182CA'X)
      INTEGER NF,MAXF,CONTEXT,IER
      INTEGER LIB$FIND_FILE
      CHARACTER*(*) FILES(MAXF)
      CHARACTER*(*) INPUT, DEFAUL
      CHARACTER*128 OUTPUT
C-----------------------------------------------------------------------
      NF = 0
      CONTEXT = 0
   20 IER = LIB$FIND_FILE(INPUT,OUTPUT,CONTEXT,DEFAUL)
      IF (IER.EQ.RMS$_NMF) GOTO 10
      IF (.NOT.IER) GOTO 10
      IF (NF.GE.MAXF) THEN
          CALL ERROR('Maximum number of input files exceeded.')
          GOTO 10
      END IF
      NF = NF+1
      FILES(NF) = OUTPUT
      GOTO 20
   10 RETURN
C-----------------------------------------------------------------------
      END

C*MAKE_NAME
C+
      SUBROUTINE MAKE_NAME (SRC, NAME, L)
      CHARACTER*(*)  SRC, NAME
      INTEGER  L
C
C Generate a filename NAME derived from a source name SRC.  This 
C involves removing or truncating illegal characters.  IAU-style source
C names are truncated at the + or - sign.
C
C Arguments:
C  SRC (input, character): the source name.
C  NAME (output, character): a valid file name (9 or less alphanumeric
C                            characters).
C  L (output, integer): the number of characters in NAME.
C-----------------------------------------------------------------------
      CHARACTER     C
      INTEGER I
C-----------------------------------------------------------------------
      I = 0
      L = 0
   10 I = I+1
      IF (I.GT.LEN(SRC)) GOTO 20
      C = SRC(I:I)
      IF (C.EQ.'+' .OR. C.EQ.'-') GOTO 20
      IF ( ('A'.LE.C .AND. C.LE.'Z') .OR.
     1     ('0'.LE.C .AND. C.LE.'9')) THEN
          L = L+1
          NAME(L:L) = C
          IF (L.GE.LEN(NAME)) GOTO 20
      END IF
      GOTO 10
   20 L = MIN(L,9)
C-----------------------------------------------------------------------
      END

C*READ_PCR
C+
      SUBROUTINE READ_PCR (LOG, FILE, NDATA, CORTIM)
      INTEGER LOG, NDATA
      CHARACTER*(*) FILE, CORTIM
C
C Process a PCR file (B2FITS).
C
C Arguments:
C  LOG    (input)  : Fortran unit number of the log file.
C  FILE   (input)  : name of input file.
C  NDATA  (output) : number of uv-data points accepted.
C  CORTIM (output) : correlation time.
C-----------------------------------------------------------------------
      INCLUDE 'cordef.inc'
      INCLUDE 'pcrdef.inc'
      INCLUDE 'b2fspi.inc'
C
C Define an array to hold the input record, and the Group and Class
C fields in the record.
C
      INTEGER   MAXREC
      PARAMETER (MAXREC=32000)
      INTEGER*2 INREC(MAXREC/2), GROUP, CLASS
      EQUIVALENCE (INREC(1),GROUP)
      EQUIVALENCE (INREC(2),CLASS)
C
C Other variables.
C
      INTEGER IOS, IPTR, LENGTH, NPT, NCDO, NREJEC, NBAD, NSPIKE, JUNK
      INTEGER STATUS, NUNSYN, NSKIP, I, J, PRVID, VALID
      INTEGER NSPBY(28,6), T
      CHARACTER*255 JUNKN
C-----------------------------------------------------------------------
  100 FORMAT (/'File: ',A/)
  105 FORMAT ( 'Correlator program version: ',I7)
  106 FORMAT ( 'Correlation time: ',A20)
  110 FORMAT (/'Number of CDO records accepted:              ',I10/
     1         'Number of CDO records rejected by START/STOP:',I10/
     1         'Number of CDO records without valid config:  ',I10/
     2         'Number of uv-points accepted:                ',I10/
     3         'Number of spikes rejected (threshold ',F6.4,'):',I10)
  115 FORMAT (/'Number of spikes in each channel of each baseline:')
  116 FORMAT (4X,I2,':',28I4)
  120 FORMAT (A/)
  650 FORMAT ('***',I10,' records skipped while looking for Process',
     1        ' Configuration table  ***')
  660 FORMAT ('*** Process Configuration table not found: this is not',
     1        ' a PCR file **************')
C-----------------------------------------------------------------------
C
C Display the file name.
C
      WRITE (LOG, 100) FILE
C
C Open the file.
C
      CALL SQOPEN(FILE, JUNKN, JUNK, IOS)
      IF (IOS.NE.1) THEN
          WRITE (LOG, 120) '*** cannot open file ***'
          RETURN
      END IF
C
C Read the first record.
C
      NCDO = 0
      NREJEC = 0
      NUNSYN = 0
      NDATA = 0
      NSPIKE = 0
      CLASS = 0
      DO 12 I=1,6
          DO 11 J=1,28
              NSPBY(J,I) = 0
   11     CONTINUE
   12 CONTINUE
      CALL SQREAD(INREC,IPTR,MAXREC,LENGTH,IOS)
C
C Skip records until a PRC record is found.
C
      NSKIP = 0
      DO WHILE (IOS.EQ.1 .AND. CLASS.NE.PRCLBL)
          NSKIP = NSKIP+1
C         -- read another record
          CALL SQREAD(INREC,IPTR,MAXREC,LENGTH,IOS)
      END DO
      IF (NSKIP.GT.0) WRITE (LOG, 650) NSKIP
      IF (IOS.NE.1) THEN
          WRITE (LOG, 660)
          RETURN
      END IF
C
      STATUS = 0
C
C Decode this record and read another one, until an error occurs 
C or eof is found.  STATUS is 3 if we have valid PRC and GRP
C table.
C
      DO WHILE (IOS.EQ.1)
C         -- correlator data output table
          IF (CLASS.EQ.CDOLBL) THEN
            IF (STATUS.EQ.3) THEN
              CALL PR_CDO (INREC, NPT, NBAD, NSPBY)
              IF (NPT.LT.0) THEN
                  NREJEC = NREJEC+1
              ELSE
                  NCDO = NCDO + 1
                  NDATA = NDATA + NPT
                  NSPIKE = NSPIKE + NBAD
              END IF
            ELSE
              NUNSYN = NUNSYN+1
            END IF
C         -- delay model table
          ELSE IF (CLASS.EQ.DLYLBL) THEN
              CALL PR_DLY (INREC, LOG)
C         -- phase model table
          ELSE IF (CLASS.EQ.PHSLBL) THEN
              IF (PRVID.GE.1987131) THEN
                  CALL PR_PHS (INREC, LOG)
              ELSE
                  CALL PR_OLDPHS (INREC, LOG)
              ENDIF
C         -- process configuration table
          ELSE IF (CLASS.EQ.PRCLBL) THEN
              CALL PR_PRC (INREC, LOG, PRVID)
              STATUS = STATUS .OR. 1
              WRITE (LOG,105) PRVID
C         -- group configuration table
          ELSE IF (CLASS.EQ.GRPLBL) THEN
              CALL PR_GRP (INREC, LOG, VALID, CORTIM)
              WRITE (LOG,106) CORTIM
              IF (VALID.NE.1) THEN
C                 -- this is not an acceptable group configuration
                  STATUS = STATUS .AND. (.NOT.2)
              ELSE
                  STATUS = STATUS .OR. 2
              END IF
C         -- operator comment table
          ELSE IF (CLASS.EQ.OPRLBL) THEN
              CALL PR_OPR (INREC, LOG)
          END IF
C         -- read another record
          CALL SQREAD(INREC,IPTR,MAXREC,LENGTH,IOS)
      END DO
C
C Close the file and record the number of CDO records and uv points.
C
  200 CALL SQCLOS (IOS)
      IF (NCDO.GT.0 .OR. NREJEC.GT.0) WRITE (LOG, 110) NCDO, NREJEC,
     1                                NUNSYN, NDATA, SPIKEL, NSPIKE
      IF (NSPIKE.GT.0) THEN
          WRITE (LOG, 115)
          DO 210 I=1,6
              T = 0
              DO 205 J=1,28
                 T = T + NSPBY(J,I)
  205         CONTINUE
              IF (T.GT.0) WRITE (LOG, 116) I, (NSPBY(J,I), J=1,28)
  210     CONTINUE
      END IF
      RETURN
C-----------------------------------------------------------------------
      END

C*PR_PRC
C+
      SUBROUTINE PR_PRC (REC, LOG, IVER)
      INCLUDE 'cordef.inc'
      INCLUDE 'pcrdef.inc'
      RECORD /PRC_GO/ REC
      INTEGER  LOG, IVER
C
C Process a PCR process configuration table. This subroutine 
C saves a copy of the table in common, checks it for acceptability,
C and resets all validity flags.
C
C Arguments:
C  REC    (input)  : the record read from the PCR file.
C  LOG    (input)  : Fortran unit number of the log file.
C  IVER   (output) : correlator program version.
C-----------------------------------------------------------------------
      INCLUDE 'b2fdef.inc'
      INTEGER I, J
C-----------------------------------------------------------------------
      PRC = REC
      DO J=1,MAXCHN
          DO I=1,MAXBAS
              OBASE(I,J) = 0
          END DO
      END DO
      IVER = REC.PRVIDGO
C-----------------------------------------------------------------------
      END

C*PR_GRP
C+
      SUBROUTINE PR_GRP (REC, LOG, IOS, CORTIM)
      INCLUDE 'cordef.inc'
      INCLUDE 'pcrdef.inc'
      RECORD /GRP_GO/ REC
      INTEGER LOG, IOS
      CHARACTER*(*) CORTIM
C
C Process a PCR group configuration table. This subroutine 
C saves a copy of the table in common, checks it for acceptability,
C and resets all validity flags.
C
C Arguments:
C  REC    (input)  : the record read from the PCR file.
C  LOG    (input)  : Fortran unit number of the log file.
C  IOS    (output) : status; 1 => OK, 0 => unacceptable configuration.
C  CORTIM (output) : correlation time.
C-----------------------------------------------------------------------
      INCLUDE 'b2fdef.inc'
      INTEGER I, J, NLAGS, L
      CHARACTER*255 CCRNAM
C-----------------------------------------------------------------------
  100 FORMAT('Config: ',I2,' stations,',I3,' channels,',
     1       I3,' lags/chan, ', A,' domain.')
  110 FORMAT('++WARNING++ unacceptable configuration: ',A)
  120 FORMAT('CCR file: ',A)
C-----------------------------------------------------------------------
      GRP = REC
      DO J=1,MAXCHN
          DO I=1,MAXBAS
              OBASE(I,J) = 0
          END DO
      END DO
C
C Log configuration.
C
      CALL ASCCNV(REC.CCRNMGO, CCRNAM, L)
      WRITE (LOG,120) CCRNAM(1:L)
      CALL LIB$SYS_ASCTIM(L, CORTIM, GRP.MSYSTGO)
      NLAGS = 2**GRP.L2NPTGO
      IF (GRP.EODLAGO .EQ. 1) NLAGS = NLAGS-1
      IF (GRP.LAGFLGO) THEN
          WRITE (LOG,100) PRC.NSTNSGO, GRP.NCHANGO, NLAGS, 'lag'
      ELSE
          WRITE (LOG,100) PRC.NSTNSGO, GRP.NCHANGO, NLAGS, 'frequency'
      END IF
C
C Check for acceptable configuration.
C
      IOS = 1
      IF (PRC.NDMODGO .LT. 1) THEN
          WRITE (LOG,110) 'No delay models'
          IOS = 0
      END IF
      IF (PRC.NPMODGO .LT. 1) THEN
          WRITE (LOG,110) 'No phase models'
          IOS = 0
      END IF
      IF (PRC.NSTNSGO .LT. 2) THEN 
          WRITE (LOG,110) 'Less than 2 stations'
          IOS = 0
      END IF
      IF (PRC.NSTNSGO .GT. 4) THEN 
          WRITE (LOG,110) 'More than 4 stations'
          IOS = 0
      END IF
      IF (GRP.L2NPTGO .NE. 3) THEN
          WRITE (LOG,110) 'Lags per channel is not 8'
          IOS = 0
      END IF
      IF (GRP.LAGFLGO) THEN
          WRITE (LOG,110) 'Data are in lag domain'
          IOS = 0
      END IF
      IF (GRP.DFRFCGO .NE. 1) THEN
          WRITE (LOG,110) 'Digital filter in use'
          IOS = 0
      END IF
C-----------------------------------------------------------------------
      END

C*PR_DLY
C+
      SUBROUTINE PR_DLY (REC, LOG)
      INCLUDE 'cordef.inc'
      INCLUDE 'pcrdef.inc'
      RECORD /DLY_DM/ REC
      INTEGER  LOG
C
C Process a PCR delay model table.  This subroutine compares
C the delay model table with the data selection criteria (experiment,
C source name, and antenna names) and updates the array OBASE which
C indicates which baselines on which channels are accepted.  It also
C updates source and antenna coordinates for the requested source and
C antennas.
C
C Arguments:
C  REC    (input)  : the record read from the PCR file.
C  LOG    (input)  : Fortran unit number of the log file.
C
C Data order changed: 1988 Jul 5
C UT1-UTC etc added:  1992 Apr 23
C sources added:      1992 Oct 19
C bug corrected:      1993 Jan 7
C-----------------------------------------------------------------------
      INTEGER MAXS, MAXB
      PARAMETER (MAXS=32, MAXB=MAXS*(MAXS-1)/2)
C     -- maximum number of antennas: must match main program
      INCLUDE 'b2fdef.inc'
      INCLUDE 'b2fsel.inc'
      CHARACTER*16 MEXP, MSRC, MANT(4)
      CHARACTER*8  WORD
      INTEGER I, J, DM, NS, NBAS, LEXP, LSRC, LANT(4), MANTN(4), NF
      INTEGER IS1, IS2, I1, I2, SCAN, IX, MSRCN, LQEXP, LMEXP, L
      INTEGER LEN1
      LOGICAL REJECT
      REAL*8  MANTX(4), MANTY(4), MANTZ(4)
C-----------------------------------------------------------------------
  100 FORMAT ('++WARNING++ Wrong source coordinates in delay model',I3)
  110 FORMAT ('DM',I3,':',I10,1X,A,1X,A,1X,A,3X,4(I3,'=',A))
  120 FORMAT ('++WARNING++ Wrong year [ODATE] in delay model',I3)
  130 FORMAT ('BADCHAN: ',I4,',',I4)
  140 FORMAT ('++WARNING++ Value of UT1-UTC has changed from',F8.4,
     1        ' to',F8.4)
  150 FORMAT ('++WARNING++ Value of XPOLE has changed from',F8.4,
     1        ' to',F8.4)
  160 FORMAT ('++WARNING++ Value of YPOLE has changed from',F8.4,
     1        ' to',F8.4)
C-----------------------------------------------------------------------
      NS = PRC.NSTNSGO
      NBAS = NS*(NS-1)/2
C
C Find delay model number, experiment name, source names, antenna
C names, and antenna coordinates.
C
      DM = REC.DMODNDM
      CALL ASCCNV(REC.EXNAMDM, MEXP, LEXP)
      CALL ASCCNV(REC.SNAMEDM, MSRC, LSRC)
      DO I=1,PRC.NSTNSGO
          CALL ASCCNV(REC.STATION(I).STNAMDO, MANT(I), L)
          L = LEN1(MANT(I)(1:L))
          LANT(I)  = MAX(8, L)
          MANTX(I) = REC.STATION(I).RSPINDO*COS(REC.STATION(I).WLONGDO)
          MANTY(I) = REC.STATION(I).RSPINDO*SIN(REC.STATION(I).WLONGDO)
          MANTZ(I) = REC.STATION(I).ZPROJDO
      END DO
C
C Run through PCTAB to see which channels use this delay model.  Save
C the delay model in the slots corresponding to these channels.  Mark
C all baselines on these channels as invalid.
C
      DO J=1,GRP.NCHANGO
          IF (DM .EQ. GRP.PCTABGO(J).DMODEL) THEN
              DO I=1,NBAS
                  OBASE(I,J) = 0
              END DO
          END IF
      END DO
C
C Record the scan number.
C
      SCAN = REC.SCANNDM
C
C Check the experiment name against the requested experiment.  If it
C matches, or "*" was requested, proceed.  Otherwise, issue warning
C and exit (ignore this table).
C
C>    write (*,*) qauexp,qexp,mexp
      IF (QAUEXP) THEN
C         -- accept all experiments
          IF (QEXP.EQ.'*') THEN
              QEXP = MEXP
          ELSE
              LQEXP = LEN1(QEXP)
              LMEXP = LEN1(MEXP)
              IF (INDEX(QEXP,MEXP(1:LMEXP)).EQ.0) 
     :            QEXP = QEXP(1:LQEXP)//','//MEXP
          END IF
      ELSE IF (QEXP.NE.MEXP) THEN
C         -- reject if not requested experiment
          GOTO 1000
      END IF
C
C Check the source name against the requested source names.  If it
C matches, or "*" was requested, proceed.  Otherwise, issue warning
C and exit (ignore this table).
C
      DO J=1,QSRCN
        IF (MSRC.EQ.QSRC(J)) THEN
          IF (QRA0(J).EQ.REC.RTAS0DM .AND. QDEC0(J).EQ.REC.DECL0DM) THEN
              MSRCN = J
              GOTO 5
          ELSE IF (QRA0(J).EQ.0D0 .AND. QDEC0(J).EQ.0D0) THEN
              QRA(J)   = REC.RTASCDM
              QDEC(J)  = REC.DECLNDM
              QRA0(J)  = REC.RTAS0DM
              QDEC0(J) = REC.DECL0DM
              COSDEC(J) = COS(QDEC(J))
              SINDEC(J) = SIN(QDEC(J))
              MSRCN = J
              GOTO 5
          END IF
        END IF
      END DO
      IF (QAUSRC) THEN
              QSRCN = QSRCN+1
              IF (QSRCN.GT.MAXSRC) CALL ERROR('Too many sources')
              MSRCN = QSRCN
              QSRC(QSRCN)  = MSRC
              QRA(QSRCN)   = REC.RTASCDM
              QDEC(QSRCN)  = REC.DECLNDM
              QRA0(QSRCN)  = REC.RTAS0DM
              QDEC0(QSRCN) = REC.DECL0DM
              COSDEC(QSRCN) = COS(QDEC(QSRCN))
              SINDEC(QSRCN) = SIN(QDEC(QSRCN))
      ELSE
              MSRCN = 0
              GOTO 1000
      END IF
    5 CONTINUE
C
C
      IF (UTYEAR .EQ. 0) THEN
          UTYEAR = REC.ODATEDM
      ELSE IF (UTYEAR .NE. REC.ODATEDM) THEN
          WRITE (LOG,120) DM
          GOTO 1000
      END IF
C
C Identify the antennas in this table by their numbers in the list
C of requested antennas, QANT.  If QAUTO is set, the user wants all
C antennas regardless of name (excluding dummy antennas), so we add
C the antenna to the requested list if it isn't already there.
C
      DO I=1,NS
          DO J=1,QANTN
              IF (MANT(I)(1:8).EQ.QANT(J)(1:8)) THEN
                  IF (MANTX(I).EQ.QANTX(J) .AND. MANTY(I).EQ.QANTY(J)
     1                .AND. MANTZ(I).EQ.QANTZ(J)) THEN
                      MANTN(I) = J
                      GOTO 10
                  ELSE IF (QANTX(J).EQ.0D0) THEN
                      QANT(J)  = MANT(I)
                      QANTX(J) = MANTX(I)
                      QANTY(J) = MANTY(I)
                      QANTZ(J) = MANTZ(I)
                      MANTN(I) = J
                      GOTO 10
                  END IF
              END IF
          END DO
          IF (QAUTO .AND. MANT(I).NE.'DUMMY') THEN
              QANTN = QANTN+1
              IF (QANTN.GT.MAXS) CALL ERROR('Too many antennas')
              MANTN(I) = QANTN
              QANT(QANTN) = MANT(I)
              QANTX(QANTN) = MANTX(I)
              QANTY(QANTN) = MANTY(I)
              QANTZ(QANTN) = MANTZ(I)
          ELSE
              MANTN(I) = 0
          END IF
   10     CONTINUE
      END DO
C
C Run through PCTAB to see which channels use this delay model.
C Record the 'output baseline number' for each baseline in each channel.
C Note that the baseline must be reversed if it is not in the canonical 
C order.
C
      NF = 0
      DO J=1,GRP.NCHANGO
          IF (DM .EQ. GRP.PCTABGO(J).DMODEL) THEN
              I= 0
              DO I2=2,NS         !!! was I1=1,NS-1
              DO I1=1,I2-1       !!! was I2=I1+1,NS
                  I = I+1
                  IS1 = MANTN(I1)
                  IS2 = MANTN(I2)
                  REJECT = .FALSE.
                  DO IX=1,28,2
                      IF (I.EQ.QBAD(IX) .AND. J.EQ.QBAD(IX+1))
     1                    REJECT = .TRUE.
                  END DO
                  IF (IS1.EQ.0 .OR. IS2.EQ.0 .OR. IS1.EQ.IS2) THEN
                      OBASE(I,J) = 0
                      OSRC(I,J) = 0
                  ELSE IF (REJECT) THEN
                      OBASE(I,J) = 0
                      OSRC(I,J) = 0
                      WRITE (LOG, 130) I, J
                  ELSE IF (IS1.LT.IS2) THEN
                      OBASE(I,J) = 256*IS1+IS2
                      OSRC(I,J) = MSRCN
                      NF = NF+1
                  ELSE 
                      OBASE(I,J) = -(256*IS2+IS1)
                      OSRC(I,J) = MSRCN
                      NF = NF+1
                  END IF
              END DO
              END DO
C>            write (log,'(7I10)') J,(OBASE(I,J),I=1,NBAS)
          END IF
      END DO
C
C Get UT1-UTC and polar motion parameters.
C
      IF (QUT1TC.EQ.0D0) THEN
          QUT1TC = REC.UTMTCDM
      ELSE IF (QUT1TC.NE.REC.UTMTCDM) THEN
          WRITE (LOG,140) QUT1TC, REC.UTMTCDM
      END IF
      IF (QXPOLE.EQ.0D0) THEN
          QXPOLE = REC.XPOLEDM
      ELSE IF (QXPOLE.NE.REC.XPOLEDM) THEN
          WRITE (LOG,150) QXPOLE, REC.XPOLEDM
      END IF
      IF (QYPOLE.EQ.0D0) THEN
          QYPOLE = REC.YPOLEDM
      ELSE IF (QYPOLE.NE.REC.YPOLEDM) THEN
          WRITE (LOG,160) QYPOLE, REC.YPOLEDM
      END IF
C 
C Tell the user we have something for her, and return.
C
      IF (NF.LT.1) THEN
          WORD = 'not used'
      ELSE
          WORD = 'accepted'
      END IF
      WRITE (LOG,110) DM, SCAN, MEXP(1:LEXP), MSRC(1:LSRC), WORD,
     1                 (MANTN(I), MANT(I)(1:LANT(I)), I=1,NS)
      RETURN
C
C Reject this Delay Model Table.
C
 1000 WRITE (LOG,110) DM, SCAN, MEXP(1:LEXP), MSRC(1:LSRC), 'rejected'
      RETURN
C-----------------------------------------------------------------------
      END

C*PR_PHS
C+
      SUBROUTINE PR_PHS (REC, LOG)
      INCLUDE 'cordef.inc'
      INCLUDE 'pcrdef.inc'
      RECORD /PHS_PM/ REC
      INTEGER  LOG
C
C Process a PCR phase model table. 
C -- this version is very crude -- it doesn't do any real checking,
C just picks out the frequency first time through
C
C Arguments:
C  REC    (input)  : the record read from the PCR file.
C  LOG    (input)  : Fortran unit number of the log file.
C-----------------------------------------------------------------------
      INTEGER MAXS, MAXB
      PARAMETER (MAXS=32, MAXB=MAXS*(MAXS-1)/2)
C     -- maximum number of antennas: must match main program
      INCLUDE 'b2fdef.inc'
      INCLUDE 'b2fsel.inc'
      REAL*8 PI,VELC
      PARAMETER (PI=3.1415926535897932846D0)
      PARAMETER (VELC=2.9979250D8)
      CHARACTER*3 SB
      CHARACTER*8 WORD
      INTEGER PM, NS, I
      REAL*8 FREF, FDEL, FPIX, EXPDEL
C-----------------------------------------------------------------------
  100 FORMAT ('PM',I3,':  NFREQ',F12.4,' DFREQ',F8.4,1X,A3)
  110 FORMAT (8X,'++WARNING++ expected DFREQ = ',F8.4)
  120 FORMAT (8X,'++WARNING++ SFREQ(',I1,') = ',F12.4)
C-----------------------------------------------------------------------
C Parameters in the station-independent part of the phase model table:
C   FQCENPM : nominal centroid frequency, MHz - ignored
C   FQNLOPM : LO frequency, MHz (NFREQ) - checked and saved
C   FQDELPM : shift delta frequency, MHz (DFREQ) - checked
C   FQSBFPM : sideband flag (USB/LSB) - checked
C Parameters in the station-dependent part:
C   FQSLOPO : station LO frequency, MHz (SFREQ) - checked = NFREQ
C   FQOFFPO : station LO offset, Hz (FOFF) - ignored
C-----------------------------------------------------------------------
      NS = PRC.NSTNSGO
      PM = REC.PMODNPM
      IF (REC.FQSBFPM) THEN
          SB = 'USB'
          FPIX = 0.5D0
          EXPDEL = 0.75D0
      ELSE
          SB = 'LSB'
          FPIX = 4.5D0
          EXPDEL = -1.25D0
      END IF
      FREF = REC.FQNLOPM*1D6 ! nominal LO
      FDEL = 0.5D6 	! spacing of frequency bins
      IF (QFREF .EQ. 0D0) THEN
          QFREF = FREF
          QWAVE = VELC/QFREF
          QFDEL = FDEL
          QFPIX = FPIX
      END IF
      WRITE (LOG, 100) PM, REC.FQNLOPM, 
     1                 REC.FQDELPM, SB
      IF (REC.FQDELPM .NE. EXPDEL) WRITE (LOG,110) EXPDEL
      DO I=1,NS
         IF ( (REC.STATION(I).FQSLOPO) .NE. (REC.FQNLOPM) )
     1        WRITE (LOG,120) I, REC.STATION(I).FQSLOPO
      END DO
C-----------------------------------------------------------------------
      END

C*PR_OLDPHS
C+
      SUBROUTINE PR_OLDPHS (REC, LOG)
      INCLUDE 'cordef.inc'
      INCLUDE 'pcrdef.inc'
	STRUCTURE /OLDPHS_PO/	! phase model station table
	REAL*8 FQSLOPO			! station LO frequency (MHz)
	REAL*8 FQDELPO			! shift delta frequency (+/-MHz)
	REAL*8 PHOFFPO			! phase offset (cyc)
	REAL*8 FQOFFPO			! LO offset for model (Hz)

	REAL*8 FQCENPO			! nominal centroid freqency (MHz)
	BYTE %FILL(24)			! (reserved)
	END STRUCTURE ! /OLDPHS_PO/

	STRUCTURE /OLDPHS_PM/	! phase model table
C table identification
	INTEGER*2 GROUPPM		! process group #
	INTEGER*2 CLASSPM		! table type #
	INTEGER*4 PMODNPM		! tone model #
	RECORD /INT_8/ PSYSTPM		! model synchronization time
	BYTE %FILL(16)			! (reserved)
C global model information
	BYTE %FILL(32)			! (reserved)

	LOGICAL*4 FQSBFPM		! upper sideband?
	BYTE %FILL(28)			! (reserved)
	RECORD /OLDPHS_PO/ STATION(MAXSTN)
	END STRUCTURE ! /OLDPHS_PM/
      RECORD /OLDPHS_PM/ REC
      INTEGER  LOG
C
C Process a PCR phase model table (versions prior to 1987131).
C -- this version is very crude -- it doesn't do any real checking,
C just picks out the frequency first time through
C
C Arguments:
C  REC    (input)  : the record read from the PCR file.
C  LOG    (input)  : Fortran unit number of the log file.
C-----------------------------------------------------------------------
      INTEGER MAXS, MAXB
      PARAMETER (MAXS=32, MAXB=MAXS*(MAXS-1)/2)
C     -- maximum number of antennas: must match main program
      INCLUDE 'b2fdef.inc'
      INCLUDE 'b2fsel.inc'
      REAL*8 PI,VELC
      PARAMETER (PI=3.1415926535897932846D0)
      PARAMETER (VELC=2.9979250D8)
      CHARACTER*3 SB
      CHARACTER*8 WORD
      INTEGER PM
      REAL*8 FREF, FDEL, FPIX, EXPDEL
C-----------------------------------------------------------------------
  100 FORMAT ('PM',I3,':  LO',F12.4,' DEL',F8.4,1X,A3)
C-----------------------------------------------------------------------
      PM = REC.PMODNPM
      IF (REC.FQSBFPM) THEN
          SB = 'USB'
          FPIX = 0.5D0
          EXPDEL = 0.75D0
      ELSE
          SB = 'LSB'
          FPIX = 4.5D0
          EXPDEL = 1.25D0
      END IF
      FREF = REC.STATION(1).FQSLOPO*1D6 ! LO (first station only...)
      FDEL = 0.5D6		! spacing of frequency bins
      IF (QFREF .EQ. 0D0) THEN
          QFREF = FREF
          QWAVE = VELC/QFREF
          QFDEL = FDEL
          QFPIX = FPIX
          WORD = 'accepted'
      ELSE IF ( (ABS(QFREF-FREF).GT.1.0) .OR. 
     1          (QFDEL.NE.FDEL) .OR.
     2          (QFPIX.NE.FPIX) .OR. 
     3          (REC.STATION(1).FQDELPO .NE. EXPDEL)) THEN
          WORD = 'rejected'
      ELSE 
          WORD = 'accepted'
      END IF
      WRITE (LOG, 100) PM, REC.STATION(1).FQSLOPO, 
     1                 REC.STATION(1).FQDELPO, SB
C-----------------------------------------------------------------------
      END

C*PR_CDO
C+
      SUBROUTINE PR_CDO (REC, NPT, NBAD, NSPBY)
      INCLUDE 'cordef.inc'
      INCLUDE 'pcrdef.inc'
      RECORD /CDO_CD/ REC
      INTEGER  NPT, NBAD, NSPBY(28,6)
C
C Process a PCR correlator data output table. The 4 useful lag/freq
C bins for each active channel and baseline are extracted and passed
C to subroutine PUTSCR for output.  This routine regards the CDO table
C as a 32-byte header, containing the reference time, followed by an
C unstructured array REC.COUNTCD of integers. The length of this array
C should be NS*MAXSS4 + NBAS*MAXCD4 where NS is the number of correlator
C stations, NBAS = NS*(NS-1)/2 is the number of baselines, and MAXSS4
C and MAXCD4 are 32 and 596 (declared in the include file).
C
C Data with a correlation coefficient >1 in any frequency bin are 
C rejected ("spikes").
C Phase is inverted to change from Block-2/Block-0 correlator convention
C to AIPS convention.
C
C Arguments:
C  REC    (input)  : the record read from the PCR file.
C  NPT    (output) : the number of data uv-points accepted; -1 if the
C                    record was rejected by time selection.
C  NBAD   (output) : the number of spikes rejected.
C  NSPBY  (in/out) : counts of spikes by channel and baseline.
C-----------------------------------------------------------------------
      INTEGER MAXS, MAXB
      PARAMETER (MAXS=32, MAXB=MAXS*(MAXS-1)/2)
C     -- maximum number of antennas: must match main program.
      INCLUDE 'b2fsel.inc'
      INCLUDE 'b2fdef.inc'
      INCLUDE 'b2fspi.inc'
      INTEGER NS, IBAS, NBAS, ICHAN, NCHAN, IBIN, INVAL, P, Q, NBITPT
      REAL XC(4), XS(4), RNORMAL, CNORMAL, SNORMAL, RMAX, WT
      REAL*8  TIME
      INTEGER MASK(6)
      DATA MASK/2, 4, 8, 16, 32, 64/
C-----------------------------------------------------------------------
C Number of stations, baselines, channels.
      NS = PRC.NSTNSGO
      NBAS = NS*(NS-1)/2
      NCHAN = GRP.NCHANGO
C Bits per integration.
      NBITPT = GRP.CDADIGO * GRP.BITRIGO 
C Data reference time. Shift to center of integration, and convert units
C to days.
      TIME = (REC.RTIMECD + NBITPT/2)/GRP.BITSCGO/86400D0
      NPT = 0
      NBAD = 0
C
C Check TIME against requested start/stop.
C
      IF (TIME.LT.QSTART .OR. TIME.GE.QSTOP) THEN
          NPT = -1
          RETURN
      END IF
C
C Loop through baselines.
      DO IBAS=1,NBAS
C         -- pointer to this baseline dump.
          P = NS*MAXSS4 + (IBAS-1)*MAXCD4
C         -- check correlator flags.
          IF ( ((REC.COFFLCD).AND.MASK(IBAS)).EQ.0) THEN
C
C Loop through channels, skipping unwanted ones.
            DO ICHAN=1,NCHAN
              IF (OBASE(IBAS,ICHAN).NE.0) THEN
                  INVAL = REC.COUNTCD(P+512+(ICHAN-1)*3+3)
C>                type *, ibas, ichan, inval
                  IF (INVAL.GE.0 .AND. 
     1                INVAL.LT.(QINVAL*0.5*NBITPT)) THEN
C
C Loop through bins for this baseline-channel (bins 8, 1, 2, 3 are
C retained.)
                      RNORMAL = (NBITPT/2-INVAL)/2.0
                      WT = 10.0*FLOAT(NBITPT-2*INVAL)/NBITPT
                      CNORMAL = 1.0/RNORMAL
                      SNORMAL = -CNORMAL
                      IF (OBASE(IBAS,ICHAN).LT.0) SNORMAL = -SNORMAL
                      IBIN = 8
                      Q = P + (ICHAN-1)*16 + (IBIN-1)*2 + 1
                      XC(1) = CNORMAL*REC.COUNTCD(Q)
                      XS(1) = SNORMAL*REC.COUNTCD(Q+1)
                      RMAX = MAX(ABS(XC(1)),ABS(XS(1)))
                      DO IBIN=1,3
                          Q = P + (ICHAN-1)*16 + (IBIN-1)*2 + 1
                          XC(IBIN+1) = CNORMAL*REC.COUNTCD(Q)
                          XS(IBIN+1) = SNORMAL*REC.COUNTCD(Q+1)
                          RMAX = MAX(RMAX,ABS(XC(IBIN+1)),
     1                                    ABS(XS(IBIN+1)))
                      END DO
                      IF (RMAX.LT.SPIKEL) THEN	!*** SPIKE DETECTION?
                          CALL PUTSCR(TIME, OBASE(IBAS,ICHAN),
     1                                OSRC(IBAS,ICHAN), XC, XS, WT)
                          NPT = NPT + 1
                      ELSE
                          NBAD = NBAD + 1
                          NSPBY(ICHAN,IBAS) = NSPBY(ICHAN,IBAS) + 1
                      END IF
                  END IF
              END IF
            END DO
          END IF
      END DO
C-----------------------------------------------------------------------
      END

C*PR_OPR
C+
      SUBROUTINE PR_OPR (REC, LOG)
      INCLUDE 'cordef.inc'
      INCLUDE 'pcrdef.inc'
      RECORD /OPR_OP/ REC
      INTEGER  LOG
C
C Process an Operator Comment table.
C
C Arguments:
C  REC    (input)  : the record read from the PCR file.
C  LOG    (input)  : Fortran unit number of the log file.
C-----------------------------------------------------------------------
      INTEGER L1, L2, L3, L4, L5
      INTEGER LEN1
      CHARACTER*32  TIME, RTIME, OPER
      CHARACTER*255 TEXT
C
  100 FORMAT(A, 1X, A, 1X, A, 1X, '"',A,'"')
  110 FORMAT('"',A,'"')
C-----------------------------------------------------------------------
      CALL LIB$SYS_ASCTIM(L1, TIME, REC.OSYSTOP)
      L1 = MIN(L1,20)
      CALL BITCNV (REC.RTIMEOP, RTIME, L5)
      CALL ASCCNL(REC.OPNAMOP, OPER, L2)
      L2 = LEN1(OPER(:L2))
      CALL ASCCNL(REC.OPCOMOP, TEXT, L3)
      IF (L1+L2+L5+L3+5 .LE. 132) THEN
          WRITE (LOG,100) TIME(:L1), OPER(:L2), RTIME(:L5), TEXT(:L3)
      ELSE
          WRITE (LOG,100) TIME(:L1), OPER(:L2), RTIME(:L5)
          L4 = MIN(128,L3)
          WRITE (LOG,110) TEXT(:L4)
          L4 = MIN(256,L3)
          IF (L3.GT.L4) WRITE (LOG,110) TEXT(L4+1:L3)
      END IF
C-----------------------------------------------------------------------
      END

C*PUTSCR
C+
      SUBROUTINE PUTSCR (UT, BASE, SRC, XC, XS, WT)
      REAL*8 UT
      INTEGER BASE, SRC
      REAL XC(4), XS(4), WT
C
C Write the data for one uv-point into the scratch file.
C
C Arguments:
C  UT     (input)  : the UT time label for the point (days since
C                    beginning of year).
C  BASE   (input)  : the encoded baseline number.
C  XC     (input)  : real part of correlation coefficient.
C  XS     (input)  : imaginary part of correlation coefficient.
C  WT     (input)  : weight (in range 0-10, at present, 10* fraction of
C                    bits that were valid).
C-----------------------------------------------------------------------
      INTEGER MAXS, MAXB
      PARAMETER (MAXS=32, MAXB=MAXS*(MAXS-1)/2)
C     -- maximum number of antennas: must match main program.
      INCLUDE 'b2fsel.inc'
      INTEGER J, IER, SOR$RELEASE_REC
C
C Buffer for passing data to sort; sized assuming NFREQ=4.
C
      CHARACTER*52  BUFFER
      REAL*8        UTT
      INTEGER*4     IBASE,ISRC
      REAL*4        WTT
      COMPLEX*8     CC(4)
      COMMON        /B2FREC/ UTT,IBASE,ISRC,WTT,CC
      EQUIVALENCE   (BUFFER, UTT)
C-----------------------------------------------------------------------
      UTT = UT
      IBASE = ABS(BASE)
      ISRC = SRC
      WTT = WT
      DO J=1,4
          CC(J) = CMPLX(XC(J),XS(J))
      END DO
      IER = SOR$RELEASE_REC(BUFFER)
      IF (IER.NE.1) CALL EXIT(IER)
C
C Accumulate statistics.
C
C  UTMIN   (R*8)         Minimum UT (days from beginning of year).
C  UTMAX   (R*8)         Maximum UT (days from beginning of year).
C  TOTEPC  (R*8)         Total epoch (years; to derive mean epoch).
C  AMAX    (R*4)         Maximum amplitude.
C
      UTMIN = MIN(UT,UTMIN)
      UTMAX = MAX(UT,UTMAX)
      TOTEPC = TOTEPC+UTYEAR+UT/365.25D0
      DO J=1,4
          AMAX = MAX(AMAX,ABS(XC(J)),ABS(XS(J)))
      END DO
C-----------------------------------------------------------------------
      END


C*ASCCNV
C+
      SUBROUTINE ASCCNV(B,S,L)
      BYTE B(*)
      CHARACTER*(*) S
      INTEGER L
C
C ASCCNV: copy a 'counted byte string' into a Fortran character
C string, converting non-printing characters to space and lower case
C letters to upper case.
C-----------------------------------------------------------------------
      INTEGER I, NB
C
      S = ' '
      NB = B(1)
      L = MIN(NB,LEN(S))
      DO 10 I=1,L
          IF (B(I+1).LT.32 .OR. B(I+1).GT.127) THEN
              S(I:I) = ' '
          ELSE
              S(I:I) = CHAR(B(I+1))
          END IF
   10 CONTINUE
      CALL UPCASE(S)
      END

C*ASCCNL
C+
      SUBROUTINE ASCCNL(B,S,L)
      BYTE B(*)
      CHARACTER*(*) S
      INTEGER L
C
C ASCCNL: copy a 'counted byte string' into a Fortran character
C string, converting non-printing characters to space and lower case
C letters to upper case.
C-----------------------------------------------------------------------
      INTEGER I, NB
C
      S = ' '
      NB = B(1)
      L = MIN(NB,LEN(S))
      DO 10 I=1,L
          IF (B(I+1).LT.32 .OR. B(I+1).GT.127) THEN
              S(I:I) = ' '
          ELSE
              S(I:I) = CHAR(B(I+1))
          END IF
   10 CONTINUE
      END

C*BITCNV
C+
      SUBROUTINE BITCNV (BITTIM, TEXT, L)
      REAL*8 BITTIM
      CHARACTER*(*) TEXT
      INTEGER L
C
C Convert a Bit Time in the Block-II format into a printable
C character string 'ddd hh:mm:ss.sss'.  CAUTION: it assumes the bit rate
C is 4 MHz.
C-----------------------------------------------------------------------
      REAL*8 SEC
      INTEGER MN, HR, DAY, IS, IMS
C
      SEC = BITTIM/4D6
      DAY = INT(SEC)/86400 + 1
      SEC = SEC - (DAY-1)*86400
      HR = INT(SEC)/3600
      SEC = SEC - HR*3600
      MN = INT(SEC)/60
      SEC = SEC - MN*60
      IMS = NINT(SEC*1000D0)
      IS = IMS/1000
      IMS = IMS - IS*1000
      WRITE (TEXT,'(I3.3,I3.2,'':'',I2.2,'':'',I2.2,''.'',I3.3)')
     2                    DAY, HR, MN, IS, IMS
      L = 16
      END

C*ANTABL
C+
      SUBROUTINE ANTABL(TELESC, DELTAT, DATOBS, IER)
      CHARACTER*(*) TELESC
      INTEGER DELTAT
      CHARACTER*(*) DATOBS
      INTEGER IER
C
C Append AIPS antenna table as a FITS binary table extension to the 
C FITS file.
C-----------------------------------------------------------------------
      INTEGER MAXS, MAXB
      PARAMETER (MAXS=32, MAXB=MAXS*(MAXS-1)/2)
      DOUBLE PRECISION PI
      PARAMETER (PI=3.1415926535897932846D0)
      INCLUDE 'b2fsel.inc'
      INTEGER      IS, J, K, ID, IM, IY
      CHARACTER*80 HISTRY
      CHARACTER*8 STR
      DOUBLE PRECISION DEGPDY, DJM, GMST, GAST
      DOUBLE PRECISION SLA_GMST, SLA_EQEQX
      INTEGER ZERO, I4
      BYTE RCP, LCP, LINE(8)
      DATA ZERO/0/
      DATA RCP/'R'/,LCP/'L'/
C-----------------------------------------------------------------------
C
C Compute GAST at TAI 0 on reference day. DJM is modified Julian date of
C reference date.
C
      READ (DATOBS, '(I2,1X,I2,1X,I2)') ID, IM, IY
      CALL SLA_CALDJ(IY, IM, ID, DJM, IER)
      IF (IER.NE.0) CALL ERROR('B2FITS error in ANTABL')
C     -- convert TAI to UTC
      DJM = DJM - DBLE(DELTAT)/86400D0
C     -- convert UTC to UT1
      DJM = DJM + QUT1TC/86400D0
C     -- find Greenwich Mean Sidereal Time
      GMST = SLA_GMST(DJM)
C     -- find Greenwich Apparent Sidereal Time
      GAST = GMST + SLA_EQEQX(DJM)
C     -- convert from radians to degrees
      GAST = GAST*180D0/PI
C
C Compute earth rotation rate (degrees per day).
C
      DEGPDY = PI*2D0 + (SLA_GMST(DJM+1D0) - SLA_GMST(DJM)) + 
     1         (SLA_EQEQX(DJM+1D0) - SLA_EQEQX(DJM))
C     -- convert from radians to degrees
      DEGPDY = DEGPDY*180D0/PI
C
C Write table header. Note use of FITS_WDBLD for values that AIPS
C requires to have D exponent (double type).
C
      CALL FITS_FLUSH(IER)
      CALL FITS_WSTR ('XTENSION', 'A3DTABLE', 'Extension type',     IER)
      CALL FITS_WINT ('BITPIX',   8,    'Binary data',              IER)
      CALL FITS_WINT ('NAXIS',    2,    'Table is a matrix',        IER)
      CALL FITS_WINT ('NAXIS1',   78,   'Width of table in bytes',  IER)
      CALL FITS_WINT ('NAXIS2',   QANTN,'Number of entries',        IER)
      CALL FITS_WINT ('PCOUNT',   0,    'No random parameters',     IER)
      CALL FITS_WINT ('GCOUNT',   1,    'Group count',              IER)
      CALL FITS_WINT ('TFIELDS',  12,   'Fields in each row',       IER)
      CALL FITS_WSTR ('EXTNAME',  'AIPS AN', 'AIPS antenna table',  IER)
      CALL FITS_WINT ('EXTVER',   1,    'Version number',           IER)
C
      CALL COLHED( 1, 'ANNAME', '8A', ' ', 'Station (antenna) name',IER)
      CALL COLHED( 2, 'STABXYZ', '3D', 'METERS',
     1                            'X,Y,Z offset from array center', IER)
      CALL COLHED( 3, 'ORBPARM', '0D', ' ', 
     1                            'Orbital parameters (none)',      IER)
      CALL COLHED( 4, 'NOSTA',  '1J', ' ', 'Station number',        IER)
      CALL COLHED( 5, 'MNTSTA', '1J', ' ', 'Mount type',            IER)
      CALL COLHED( 6, 'STAXOF', '1E', 'METERS', 'Axis offset',      IER)
      CALL COLHED( 7, 'POLTYA', '1A', ' ',
     1                     'Feed A poln. type (R, L, X, or Y)',     IER)
      CALL COLHED( 8, 'POLAA',  '1E', 'DEGREES',
     1                     'Feed A position angle',                 IER)
      CALL COLHED( 9, 'POLCALA', '3E', ' ', 'Feed A poln cal parms',IER)
      CALL COLHED(10, 'POLTYB', '1A', ' ',
     1                     'Feed B poln. type (R, L, X, or Y)',     IER)
      CALL COLHED(11, 'POLAB',  '1E', 'DEGREES',
     1                     'Feed B position angle',                 IER)
      CALL COLHED(12, 'POLCALB', '3E', ' ', 'Feed B poln cal parms',IER)
C
      CALL FITS_WDBLD ('ARRAYX', 0D0, 'Array center x-coord (m)',   IER)
      CALL FITS_WDBLD ('ARRAYY', 0D0, 'Array center y-coord (m)',   IER)
      CALL FITS_WDBLD ('ARRAYZ', 0D0, 'Array center z-coord (m)',   IER)
      CALL FITS_WDBLD ('GSTIA0', GAST,
     1                     'Greenwich App. ST at TAI=0 (deg)',      IER)
      CALL FITS_WDBLD ('DEGPDY', DEGPDY, 
     1          'Earth rotation rate (deg of GAST per UT day)',     IER)
      CALL FITS_WDBLD ('FREQ',   QFREF,  'Reference frequency (Hz)',IER)
      CALL FITS_WSTR  ('RDATE',  DATOBS, 'Reference date',          IER)
      CALL FITS_WDBLE ('POLARX', QXPOLE, 'Polar x pos. (arcsec)',   IER)
      CALL FITS_WDBLE ('POLARY', QYPOLE, 'Polar y pos. (arcsec)',   IER)
      CALL FITS_WDBLE ('UT1UTC', QUT1TC, 'UT1-UTC (sec)',           IER)
      CALL FITS_WDBLE ('IATUTC', DBLE(DELTAT), 'TAI-UTC (sec)',     IER)
      CALL FITS_WSTR  ('ARRNAM', TELESC, 'Array name',              IER)
      CALL FITS_WINT  ('NUMORB', 0,      'No orbital parameters',   IER)
      CALL FITS_WINT  ('NOPCAL', 3,      'No pol cal parameters',   IER)
C
      CALL FITS_WEND (IER)
C
C Write table.
C
      DO 40 J=1,QANTN
C        -- ANNAME (8 bytes)
         STR = QANT(J)
         CALL FITS_WRAYB(%REF(STR),8,IER)
C        -- STABXYZ (3x8 bytes)
         CALL ZRLR64(1,1,QANTX(J),LINE)
         CALL FITS_WRAYB(LINE,8,IER)
         CALL ZRLR64(1,1,QANTY(J),LINE)
         CALL FITS_WRAYB(LINE,8,IER)
         CALL ZRLR64(1,1,QANTZ(J),LINE)
         CALL FITS_WRAYB(LINE,8,IER)
   10    CONTINUE
C        -- ORBPARM (0 bytes)
C        -- NOSTA (4 byte integer)
         I4 = J
         CALL FITS_WBSWAP(I4,1)
         CALL FITS_WRAYB(I4,4,IER)
C        -- MNTSTA (4 byte integer)
         I4 = 0
         CALL FITS_WBSWAP(I4,1)
         CALL FITS_WRAYB(I4,4,IER)
C        -- STAXOF (4 byte float)
         CALL FITS_WRAYB(ZERO,4,IER)
C        -- POLTYA (1 byte)
         CALL FITS_WRAYB(RCP,1,IER)
C        -- POLAA (4 bytes)
         CALL FITS_WRAYB(ZERO,4,IER)
C        -- POLCALA (3x4 bytes)
         DO 20 K=1,3
             CALL FITS_WRAYB(ZERO,4,IER)
   20    CONTINUE
C        -- POLTYB (1 byte)
         CALL FITS_WRAYB(LCP,1,IER)
C        -- POLAB (4 bytes)
         CALL FITS_WRAYB(ZERO,4,IER)
C        -- POLCALB (3x4 bytes)
         DO 30 K=1,3
             CALL FITS_WRAYB(ZERO,4,IER)
   30    CONTINUE
   40 CONTINUE
C
      CALL FITS_FLUSH(IER)
C-----------------------------------------------------------------------
      END

C*SUTABL
C+
      SUBROUTINE SUTABL(IER)
      INTEGER IER
C
C Append AIPS source table as a FITS binary table extension to the 
C FITS file.
C-----------------------------------------------------------------------
      INTEGER MAXS, MAXB
      PARAMETER (MAXS=32, MAXB=MAXS*(MAXS-1)/2)
      REAL*8    PI,RPDEG
      PARAMETER (PI=3.1415926535897932846D0)
      PARAMETER (RPDEG=PI/180D0)
      INCLUDE 'b2fsel.inc'
      INTEGER J, K, I4, ZERO
      CHARACTER*16 STR
      BYTE LINE(8)
      DATA ZERO/0/
C-----------------------------------------------------------------------
C
C Write table header. 
C
      CALL FITS_FLUSH(IER)
      CALL FITS_WSTR ('XTENSION', 'A3DTABLE', 'Extension type',     IER)
      CALL FITS_WINT ('BITPIX',   8,    'Binary data',              IER)
      CALL FITS_WINT ('NAXIS',    2,    'Table is a matrix',        IER)
      CALL FITS_WINT ('NAXIS1',   132,  'Width of table in bytes',  IER)
      CALL FITS_WINT ('NAXIS2',   QSRCN,'Number of entries',        IER)
      CALL FITS_WINT ('PCOUNT',   0,    'No random parameters',     IER)
      CALL FITS_WINT ('GCOUNT',   1,    'Group count',              IER)
      CALL FITS_WINT ('TFIELDS',  19,   'Fields in each row',       IER)
      CALL FITS_WSTR ('EXTNAME',  'AIPS SU', 'AIPS source table',   IER)
      CALL FITS_WINT ('EXTVER',   1,    'Version number',           IER)
C
      CALL COLHED( 1, 'ID. NO.',  '1J',  ' ', 'Source number',      IER)
      CALL COLHED( 2, 'SOURCE',   '16A', ' ', 'Source name',        IER)
      CALL COLHED( 3, 'QUAL',     '1J',  ' ', 'Qualifier',          IER)
      CALL COLHED( 4, 'CALCODE',  '4A',  ' ', 'Cal code ',          IER)
      CALL COLHED( 5, 'IFLUX',    '1E',  'JY', 'Flux density',      IER)
      CALL COLHED( 6, 'QFLUX',    '1E',  'JY', 'Flux density',      IER)
      CALL COLHED( 7, 'UFLUX',    '1E',  'JY', 'Flux density',      IER)
      CALL COLHED( 8, 'VFLUX',    '1E',  'JY', 'Flux density',      IER)
      CALL COLHED( 9, 'FREQOFF',  '1D',  'HZ',      '   ',          IER)
      CALL COLHED(10, 'BANDWIDTH','1D',  'HZ',      '   ',          IER)
      CALL COLHED(11, 'RAEPO',    '1D',  'DEGREES', '   ',          IER)
      CALL COLHED(12, 'DECEPO',   '1D',  'DEGREES', '   ',          IER)
      CALL COLHED(13, 'EPOCH',    '1D',  'YEARS',   '   ',          IER)
      CALL COLHED(14, 'RAAPP',    '1D',  'DEGREES', '   ',          IER)
      CALL COLHED(15, 'DECAPP',   '1D',  'DEGREES', '   ',          IER)
      CALL COLHED(16, 'LSRVEL',   '1D',  'M/SEC',   '   ',          IER)
      CALL COLHED(17, 'RESTFREQ', '1D',  'HZ',      '   ',          IER)
      CALL COLHED(18, 'PMRA',     '1D',  'DEG/DAY', '   ',          IER)
      CALL COLHED(19, 'PMDEC',    '1D',  'DEG/DAY', '   ',          IER)
C
      CALL FITS_WINT ('NO_IF',   1,         ' ', IER)
      CALL FITS_WSTR ('VELTYP',  ' ',       ' ', IER)
      CALL FITS_WSTR ('VELDEF',  'OPTICAL', ' ', IER)
      CALL FITS_WINT ('FREQID',  -1,        ' ', IER)
C
      CALL FITS_WEND (IER)
C
C Write table.
C
      DO 40 J=1,QSRCN
C        -- (1) ID. NO. (4 byte integer)
         I4 = J
         CALL FITS_WBSWAP(I4,1)
         CALL FITS_WRAYB(I4,4,IER)
C        -- (2) SOURCE (16 byte string)
         STR = QSRC(J)
         CALL FITS_WRAYB(%REF(STR),16,IER)
C        -- (3) QUAL (4 byte integer)
         I4 = 1
         CALL FITS_WBSWAP(I4,1)
         CALL FITS_WRAYB(I4,4,IER)
C        -- (4) CALCODE (4 byte string)
         STR = ' '
         CALL FITS_WRAYB(%REF(STR),4,IER)
C        -- (5-8) *FLUX (4 4-byte reals)
         DO 20 K=5,8
             CALL FITS_WRAYB(ZERO,4,IER)
   20    CONTINUE
C        -- (9) FREQOFF (8-byte real)
         CALL ZRLR64(1,1,0D0,LINE)
         CALL FITS_WRAYB(LINE,8,IER)
C        -- (10) BANDWIDTH (8-byte real)
         CALL ZRLR64(1,1,QFDEL,LINE)
         CALL FITS_WRAYB(LINE,8,IER)
C        -- (11) RAEPO (8-byte real)
         CALL ZRLR64(1,1,QRA0(J)/RPDEG,LINE)
         CALL FITS_WRAYB(LINE,8,IER)
C        -- (12) DECEPO (8-byte real)
         CALL ZRLR64(1,1,QDEC0(J)/RPDEG,LINE)
         CALL FITS_WRAYB(LINE,8,IER)
C        -- (13) EPOCH (8-byte real)
         CALL ZRLR64(1,1,1950D0,LINE)
         CALL FITS_WRAYB(LINE,8,IER)
C        -- (14) RAAPP (8-byte real)
         CALL ZRLR64(1,1,QRA(J)/RPDEG,LINE)
         CALL FITS_WRAYB(LINE,8,IER)
C        -- (15) DECAPP(8-byte real)
         CALL ZRLR64(1,1,QDEC(J)/RPDEG,LINE)
         CALL FITS_WRAYB(LINE,8,IER)
C        -- (16) LSRVEL (8-byte real)
C        -- (17) RESTFREQ (8-byte real)
C        -- (18) PMRA (8-byte real)
C        -- (19) PMDEC (8-byte real)
         CALL ZRLR64(1,1,0D0,LINE)
         DO 30 K=16,19
             CALL FITS_WRAYB(LINE,8,IER)
   30    CONTINUE
   40 CONTINUE
C
      CALL FITS_FLUSH(IER)
C-----------------------------------------------------------------------
      END

C*COLHED
C+
      SUBROUTINE COLHED(N, TYPE, FORM, UNIT, COMM, IER)
      INTEGER N
      CHARACTER*(*) TYPE, FORM, UNIT, COMM
      INTEGER IER
C
C Write header keywords for a table column.
C-----------------------------------------------------------------------
      CHARACTER*3 CODE
      IF (N.GT.99) THEN
          WRITE (CODE, '(I3)') N
      ELSE IF (N.GT.9) THEN
          WRITE (CODE, '(I2,1X)') N
      ELSE
          WRITE (CODE, '(I1,2X)') N
      END IF
      CALL FITS_WSTR ('TTYPE'//CODE, TYPE, COMM, IER)
      CALL FITS_WSTR ('TFORM'//CODE, FORM, ' ',  IER)
      CALL FITS_WSTR ('TUNIT'//CODE, UNIT, ' ',  IER)
C-----------------------------------------------------------------------
      END
