C*B2PS
C+
************************************************************************
*    B2PS internals
*
* The file B2PS.FOR defines the main program, B2PS,
* and the following subroutines:
*
* TSRPCR          - read one file
* SHOW_PRC        - list PRC table
* SHOW_GRP        - list GRP table
* SHOW_DLY        - list DLY table
* SHOW_PHS        - list PHS table
* SHOW_TON        - list TON table
* SHOW_IAO        - list IAO table
* SHOW_OPR        - list OPR table
* GET_NS          - get parameters from PRC table
* CONFIG          - get parameters from GRP table
* GETSS           - get parameters from DLY table
* ASCCNV          - format a counted byte string
* BITCNV          - format a REAL*8 bit time
*
* The main program handles user control parameters and wild-card 
* expansion. It calls TSRPCR once for each input file. TSRPCR reads
* each record from the file and passes it to an appropriate subroutine
* for processing and/or listing.
* 
* Two INCLUDE modules are required to define the format of the PCR file:
*
* CORDEF
* PCRDEF
*
* The "standard" version of these can be found in VLBSOURCE:CORDEF.FOR,
* PCRDEF.FOR.  These files were obtained from Tom Lockhart.
* 
* Additional subroutines required from the VLBLIB library are:
*
* ERROR FRDT KEYIN LEN1 PACK PANGLE PROGNM REPCPU REPORT SCLOCK SKIPBL
* SQREAD SYMSUB COMPST CTOI CTOR EXPRES GETARG KEYHLP KEYOUT KEYPUT CTOD
* KEYCHK 
************************************************************************
C--

C*B2PS
C+
      PROGRAM B2PS
C
C B2PS (Block-II Summary) summarizes the contents of output files (PCR
C files) from the JPL/Caltech Block-II VLBI Correlator.
C
C Parameters:
C
C Parameter INPUT must be specified; the others are optional.
C
C Input = "filename" :  the name of the input file, which must
C           exist.  Wild cards are permitted (e.g.,
C           INPUT = CITSCR:[TJP...]*.FPR).
C
C DEFault = "filename" :  the default input file name; this file
C           name is used to supply default fields (disk, directory,
C           extension) for the Input filename.
C
C Listfile = "filename" :  a name for the listing file.  If this
C           is not specified, "SYS$OUTPUT" is used.
C
C DUMp (no value) : if specified, display contents of setup tables.
C
C Tsys (no value) : if specified, dump the contents of the "Mark-II
C           interface ancillary data output records" (which normally
C           contain Tsys information from the Mark-II tapes) in a
C           file called B2PS_TSYS.DAT.
C
C Author:
C  T.J. Pearson, VAX-11 Fortran.
C
C Program History:
C  Version 1.0:  1985 Oct 19 - new program (TJP).
C  Version 1.1:  1986 Feb  2 - additional output (TJP).
C  Version 1.2:  1986 Feb 27 - add Default (TJP).
C  Version 2.1:  1986 Jun  7 - new format for PCR files (TJP).
C  Version 2.2:  1986 Oct  6 - do not insist that first record be a
C                              process configuration table (TJP).
C  Version 2.3:  1986 Dec 22 - new format for phase model table (TJP).
C  Version 2.4:  1987 Jul  2 - new format for phase model table (TJP).
C  Version 2.5:  1989 Jan 31 - add EODLAG (TJP).
C  Version 2.6:  1990 Nov 27 - add operator tables (TJP).
C-----------------------------------------------------------------------
C
C Constants (RMSNMF is the VMS parameter RMS$_NMF).
C
      CHARACTER*(*) VERSN
      PARAMETER (VERSN='2.6 - 1990 Nov 27')
      INTEGER   INC,OUTC,LIST,LTSYS,NPARS
      PARAMETER (INC=5,OUTC=6,LIST=9,LTSYS=1)
      PARAMETER (NPARS=32)
      INTEGER   RMSNMF
      PARAMETER (RMSNMF='000182CA'X)
C
C Variables.
C
      CHARACTER*128    INPUT,DEFAUL,LISTFI,FILE,NAME
      CHARACTER*17     REVTIM
      INTEGER          I,IER,NF,LIN,MODE,CONTXT,LNAME
      LOGICAL          DUMP, DTSYS
      DOUBLE PRECISION PARS(NPARS),VALS(NPARS),ENDMRK
C
C Functions called.
C
      INTEGER   LEN1, LIB$FIND_FILE
C
C KEYIN parameters.
C
      DATA PARS / 'Input   ', 9*' ',
     1            'DEFault ', 9*' ',
     2            'Listfile', 9*' ',
     3            'DUMp',
     4            'Tsys' /
      DATA VALS / '***UNSET', 9*' ',
     1            'VLB_PCR:', '.PCR', 8*' ',
     2            'SYS$OUTP', 'UT', 8*' ',
     3            -1D0,
     4            -1D0 /
      DATA ENDMRK / '/' /
C-----------------------------------------------------------------------
 1010 FORMAT (' B2PS lists scans in Block-II output PCR files'/
     1        ' (Version ',A,')'//
     2        ' Control parameters (INPUT, LISTFILE, DUMP, TSYS):')
 1020 FORMAT (A,T20,A)
 1030 FORMAT (80('-'))
 1040 FORMAT (2A)
 1050 FORMAT (10A8)
 1070 FORMAT ('*** Cannot read file ***')
C-----------------------------------------------------------------------
C
C Introduction.
C
      CALL SCLOCK
      WRITE (OUTC,1010) VERSN
C
C Read parameters.
C
      MODE = 0
      CALL KEYIN(PARS,VALS,NPARS,ENDMRK,MODE,INC,OUTC)
      WRITE (INPUT,1050)  (VALS(I),I=1,10)
      WRITE (DEFAUL,1050) (VALS(I),I=11,20)
      WRITE (LISTFI,1050) (VALS(I),I=21,30)
      DUMP = VALS(31).EQ.0D0
      DTSYS = VALS(32).EQ.0D0
C
C Open the output list file.
C
      OPEN (UNIT=LIST, FILE=LISTFI, FORM='FORMATTED',
     1      STATUS='NEW', DISPOSE='DELETE', IOSTAT=IER,
     2      DEFAULTFILE='B2PS.LIS', CARRIAGECONTROL='LIST')
      IF (IER.NE.0) CALL ERROR('Cannot open output file.')
      INQUIRE (UNIT=LIST, NAME=LISTFI)
C
C Open the output TSYS file.
C
      IF (DTSYS) THEN
          OPEN (UNIT=LTSYS, FILE='B2PS_TSYS', FORM='FORMATTED',
     1          STATUS='NEW', DISPOSE='KEEP', IOSTAT=IER,
     2          DEFAULTFILE='B2PS_TSYS.DAT', CARRIAGECONTROL='LIST',
     3          RECL=256)
          IF (IER.NE.0) CALL ERROR('Cannot open output file B2PS_TSYS.')
      END IF
C
C Initialize counters.
C
      WRITE (LIST,1040) 'Listing from program B2PS'
C
C Read input files.
C
      NF = 0
      CONTXT = 0
   20 IER = LIB$FIND_FILE(INPUT,FILE,CONTXT,DEFAUL)
      IF (IER.EQ.RMSNMF) GOTO 10
      LIN = LEN1(FILE)
      IF (.NOT.IER) CALL ERROR('Cannot find input file: '//
     1                          FILE(1:LIN))
      NF = NF+1
      WRITE (LIST,1030)
      WRITE (LIST,1040) 'File: ',FILE(1:LIN)
      CALL FRDT(FILE(1:LIN), REVTIM)
      WRITE (LIST,1040) 'Last modified: ',REVTIM
      CALL SQOPEN(FILE(1:LIN),NAME,LNAME,IER)
      IF (IER.NE.1) THEN
          WRITE (LIST,1070) 
      ELSE
          CALL TSRPCR(LIST,DUMP,DTSYS)
          CALL SQCLOS(IER)
      END IF
      GOTO 20
C
C End.
C
   10 WRITE (LIST,1030)
      WRITE (LIST,1020) 'B2PS version:',VERSN
      CALL REPORT(LIST)
      CALL REPCPU(LIST)
      WRITE (LIST,1020) 'Input:',INPUT(1:LEN1(INPUT))
      WRITE (LIST,1020) 'Listfile:',LISTFI(1:LEN1(LISTFI))
      CLOSE (UNIT=LIST, DISPOSE='KEEP')
C-----------------------------------------------------------------------
      END

C*TSRPCR
C+
      SUBROUTINE TSRPCR (OUTC, DUMP, DTSYS)
      INTEGER OUTC
      LOGICAL DUMP, DTSYS
C
C TSRPCR: Read and analyze an output file from the Block 2 correlator.
C
C Arguments:
C  OUTC   (input)  : Fortran unit number on which the output (listing)
C                    file has been opened, with LIST carriage control.
C  DUMP   (input)  : if TRUE, display contents of data records.
C  DTSYS  (input)  : if TRUE, dump IAO records in file.
C Implicit input:
C  The data (PCR) file is assumed to have been opened using SQOPEN.
C
C 16-May-1986
C  6-Oct-1986  Skip records up to first PRC record.
C-----------------------------------------------------------------------
      INCLUDE 'cordef.inc'
      INCLUDE 'pcrdef.inc'
C
C Define an array to hold the input record, and the Group and Class
C fields in the record.
C
      INTEGER   MAXREC
      PARAMETER (MAXREC=32000)
      INTEGER*2 INREC(MAXREC/2), GROUP, CLASS
      REAL*8    RTIM
      EQUIVALENCE (INREC(1),GROUP)
      EQUIVALENCE (INREC(2),CLASS)
      EQUIVALENCE (INREC(9),RTIM)
C
C Other variables.
C
      INTEGER MAXCLA
      PARAMETER (MAXCLA=200)
      CHARACTER*40 TEXT
      INTEGER NT(MAXCLA), NBYTE(MAXCLA)
      INTEGER IOS, IPTR, LENGTH, I, N, BYTES, NS, L, J, NSKIP
      REAL*8  TIM1, TIM2
      CHARACTER*16 SRC(50), STN(50), EXP(50)
      INTEGER NSRC, NSTN, NEXP, NSCN, SCN(50)
      LOGICAL LAGFL, ODDLAG
      INTEGER NCHAN, NLAGS, PRVID
      REAL INTEG
C-----------------------------------------------------------------------
  100 FORMAT('The correlator was configured with',I3,' stations and',I3,
     1       ' channels, each with')
  110 FORMAT(I3,' lags.  The integration time was',F7.2,' seconds.')
  120 FORMAT('Data output was in the ',A,' domain.')
  130 FORMAT('The program version was',I10,'.')
  600 FORMAT (A14,2X,4A16/(16X,4A16))
  610 FORMAT (T7,5X,A40,I8,I10)
  620 FORMAT (T7,Z4.4,1X,A40,I8,I10)
  630 FORMAT (/'Summary of records in this file:'//
     1        T15,'Record Type',T55,'Number',4X,'Size'/)
  640 FORMAT (A14,2X,4I16/(16X,4I16))
  650 FORMAT ('***',I10,' records skipped while looking for Process',
     1        ' Configuration table  ***')
  660 FORMAT ('*** Process Configuration table not found: this is not',
     1        ' a PCR file **************')
C-----------------------------------------------------------------------
C
C Initialize the record counters.
C
      N = 0
      BYTES = 0
      NSRC = 0
      NSTN = 0
      NEXP = 0
      NSCN = 0
      DO I=1,MAXCLA
          NT(I) = 0
          NBYTE(I) = 0
      END DO
      TIM1 = 1D38
      TIM2 = 0D0
C
C Read the first record.
C
      CLASS = 0
      CALL SQREAD(INREC,IPTR,MAXREC,LENGTH,IOS)
C
C Skip records until a PRC record is found.
C
      NSKIP = 0
      DO WHILE (IOS.EQ.1 .AND. CLASS.NE.PRCLBL)
          NSKIP = NSKIP+1
C         -- read another record
          CALL SQREAD(INREC,IPTR,MAXREC,LENGTH,IOS)
      END DO
      IF (NSKIP.GT.0) WRITE (OUTC, 650) NSKIP
      IF (IOS.NE.1) THEN
          WRITE (OUTC, 660)
          RETURN
      END IF
C
C Decode this record and read another one, until an error occurs 
C or eof is found. 
C
      DO WHILE (IOS.EQ.1)
C         -- count the record
          N = N + 1
          BYTES = BYTES + LENGTH
          IF (CLASS.LT.1) CLASS = 1
          IF (CLASS.GT.MAXCLA) CLASS = MAXCLA
          NT(CLASS) = NT(CLASS) + 1
          NBYTE(CLASS) = LENGTH
          IF (DUMP) THEN
C             -- delay model table
              IF (CLASS.EQ.DLYLBL) THEN
                  CALL SHOW_DLY (N,INREC,LENGTH,OUTC,NS)
C             -- phase model table
              ELSE IF (CLASS.EQ.PHSLBL) THEN
                  CALL SHOW_PHS (N,INREC,LENGTH,OUTC,NS,PRVID)
C             -- tone model table
              ELSE IF (CLASS.EQ.TONLBL) THEN
                  CALL SHOW_TON (N,INREC,LENGTH,OUTC,NS)
C             -- process configuration table
              ELSE IF (CLASS.EQ.PRCLBL) THEN
                  CALL SHOW_PRC (N,INREC,LENGTH,OUTC,NS)
C             -- group configuration table
              ELSE IF (CLASS.EQ.GRPLBL) THEN
                  CALL SHOW_GRP (N,INREC,LENGTH,OUTC,NS)
C             -- operator table
              ELSE IF (CLASS.EQ.OPRLBL) THEN
                  CALL SHOW_OPR (N,INREC,LENGTH,OUTC,NS)
              END IF
          END IF
C         -- TSYS option
          IF (DTSYS .AND. CLASS.EQ.IAOLBL) CALL
     1                  SHOW_IAO (N,INREC,LENGTH,1)
C         -- timecode from raw data table
          IF (CLASS.EQ.CDOLBL) THEN
              TIM1 = MIN(TIM1,RTIM)
              TIM2 = MAX(TIM2,RTIM)
C         -- number of stations from process configuration table
          ELSE IF (CLASS.EQ.PRCLBL) THEN
              CALL GET_NS (N,INREC,LENGTH,OUTC,NS,PRVID)
C         -- correlator configuration from group table
          ELSE IF (CLASS.EQ.GRPLBL) THEN
              CALL CONFIG (N,INREC,LENGTH,NS,NCHAN,NLAGS,INTEG,LAGFL,
     1                     ODDLAG)
C         -- experiment details from delay model table
          ELSE IF (CLASS.EQ.DLYLBL) THEN
              CALL GETSS (INREC, NS, SRC, NSRC, STN, NSTN, EXP, NEXP,
     1                    SCN, NSCN)
          END IF
C         -- read another record
          CALL SQREAD(INREC,IPTR,MAXREC,LENGTH,IOS)
      END DO
C
C Print a table summarizing number of each kind of record.
C
      WRITE (OUTC, 630)
      DO I=1,MAXCLA
          IF (NT(I).GT.0) THEN
              TEXT = 'Unknown'
              IF (I.EQ.PRCLBL) TEXT = 'process configuration table'
              IF (I.EQ.GRPLBL) TEXT = 'group configuration table'
              IF (I.EQ.ICMLBL) TEXT = 'mark-2 interface configuration'
              IF (I.EQ.TONLBL) TEXT = 'tone model'
              IF (I.EQ.TMLLBL) TEXT = 'tone model lookup table header'
              IF (I.EQ.TMOLBL) TEXT = 'tone (phase cal) model output'
              IF (I.EQ.DLYLBL) TEXT = 'delay model'
              IF (I.EQ.GEOLBL) TEXT = 'geometric model output'
              IF (I.EQ.PHSLBL) TEXT = 'phase model'
              IF (I.EQ.TNSLBL) TEXT = 'tensor configuration'
              IF (I.EQ.TCOLBL) 
     1            TEXT = 'tensor current configuration output'
              IF (I.EQ.CDOLBL) TEXT = 'cross-correlation data'
              IF (I.EQ.CSOLBL) TEXT = 'cross-correlation status'
              IF (I.EQ.SDOLBL) TEXT = 'tone (phase cal) data'
              IF (I.EQ.SSOLBL) TEXT = 'tone (phase cal) status'
              IF (I.EQ.IAOLBL) TEXT = 'interface ancillary data'
              IF (I.EQ.OPRLBL) TEXT = 'operator comment'
              IF (I.EQ.TDOLBL) TEXT = 'tensor (segment) data'
              IF (I.EQ.FDOLBL) TEXT = 'digital filter data'
              WRITE (OUTC, 620) I, TEXT, NT(I), NBYTE(I)
          END IF
      END DO
      WRITE (OUTC,610) ' Total:', N, BYTES
      WRITE (OUTC,100) NS, NCHAN
      IF (ODDLAG) NLAGS = NLAGS-1
      WRITE (OUTC,110) NLAGS, INTEG
      IF (LAGFL) THEN
          WRITE (OUTC,120) 'lag'
      ELSE
          WRITE (OUTC,120) 'frequency'
      END IF
      WRITE (OUTC,130) PRVID
      IF (NEXP.GT.0) WRITE(OUTC,600) 'Experiments:  ', (EXP(J),J=1,NEXP)
      IF (NSCN.GT.0) WRITE(OUTC,640) 'Scans:        ', (SCN(J),J=1,NSCN)
      IF (NSRC.GT.0) WRITE(OUTC,600) 'Sources:      ', (SRC(J),J=1,NSRC)
      IF (NSTN.GT.0) WRITE(OUTC,600) 'Antennas:     ', (STN(J),J=1,NSTN)
      IF (NT(CDOLBL).GT.0) THEN
          CALL BITCNV (TIM1, TEXT, L)
          WRITE (OUTC, 600) 'Earliest time:', TEXT(1:L)
          CALL BITCNV (TIM2, TEXT, L)
          WRITE (OUTC, 600) 'Latest time:  ', TEXT(1:L)
      END IF
C-----------------------------------------------------------------------
      END

C*ASCCNV
C+
      SUBROUTINE ASCCNV(B,S,L)
      BYTE B(*)
      CHARACTER*(*) S
      INTEGER L
C
C ASCCNV: copy a 'counted byte string' into a Fortran character
C string, converting non-printing characters to space.
C-----------------------------------------------------------------------
      INTEGER I, NB
C
      S = ' '
      NB = B(1)
      L = MIN(NB,LEN(S))
      DO 10 I=1,L
          IF (B(I+1).LT.32 .OR. B(I+1).GT.127) THEN
              S(I:I) = ' '
          ELSE
              S(I:I) = CHAR(B(I+1))
          END IF
   10 CONTINUE
      END

C*SHOW_PRC
C+
      SUBROUTINE SHOW_PRC (NREC,REC,LREC,OUTC,NS)
      INCLUDE 'cordef.inc'
      INCLUDE 'pcrdef.inc'
      RECORD /PRC_GO/ REC
      INTEGER NREC, LREC, OUTC, NS
C
C SHOW_PRC: list contents of a PCR process table in output file.
C
C Arguments:
C  NREC   (input)  : the sequence number of the record in the file.
C  REC    (input)  : the record (table).
C  LREC   (input)  : length of record in bytes.
C  OUTC   (input)  : Fortran unit number on which data are to be listed.
C  NS     (output) : number of stations.
C
C 29-Apr-1986
C-----------------------------------------------------------------------
      INTEGER I, L
      CHARACTER*80 TEXT
C
  100 FORMAT (/'(',I6.6,')   Process Table,',I6,' bytes'/)
  110 FORMAT (2X,A8,' = ',I16,3(:',',I16))
  115 FORMAT (2X,A8,' = ',L16)
  120 FORMAT (2X,A8,' = "',A,'"')
C-----------------------------------------------------------------------
      WRITE (OUTC,100) NREC, LREC
      WRITE (OUTC,110) 'PRVID', REC.PRVIDGO
      WRITE (OUTC,110) 'MXCHN', REC.MXCHNGO
      WRITE (OUTC,110) 'MXTMC', REC.MXTMCGO
      WRITE (OUTC,110) 'MXCYC', REC.MXCYCGO
      WRITE (OUTC,110) 'NSTNS', REC.NSTNSGO
      WRITE (OUTC,110) 'NTMOD', REC.NTMODGO
      WRITE (OUTC,110) 'NTCOE', REC.NTCOEGO
      WRITE (OUTC,110) 'NDMOD', REC.NDMODGO
      WRITE (OUTC,110) 'NPMOD', REC.NPMODGO
      WRITE (OUTC,115) 'INFLA', REC.INFLAGO
      CALL ASCCNV(REC.TCFNMGO, TEXT, L)
      WRITE (OUTC,120) 'TCFNM', TEXT(1:L)
      CALL ASCCNV(REC.ICFNMGO, TEXT, L)
      WRITE (OUTC,120) 'ICFNM', TEXT(1:L)
      NS = REC.NSTNSGO
      WRITE (OUTC,110) 'PSTNS', (REC.STATION(I).PSTNSSO, I=1,NS)
      END

C*GET_NS
C+
      SUBROUTINE GET_NS (NREC,REC,LREC,OUTC,NS,PRVID)
      INCLUDE 'cordef.inc'
      INCLUDE 'pcrdef.inc'
      RECORD /PRC_GO/ REC
      INTEGER NREC, LREC, OUTC, NS, PRVID
C
C GET_NS: get number of stations from PRC record.
C
C Arguments:
C  NREC   (input)  : the sequence number of the record in the file.
C  REC    (input)  : the record (table).
C  LREC   (input)  : length of record in bytes.
C  OUTC   (input)  : Fortran unit number on which data are to be listed.
C  NS     (output) : number of stations.
C  PRVID  (output) : correlator program ID
C
C 19-May-1986
C-----------------------------------------------------------------------
      NS = REC.NSTNSGO
      PRVID = REC.PRVIDGO
C-----------------------------------------------------------------------
      END

C*SHOW_GRP
C+
      SUBROUTINE SHOW_GRP (NREC,REC,LREC,OUTC,NS)
      INCLUDE 'cordef.inc'
      INCLUDE 'pcrdef.inc'
      RECORD /GRP_GO/ REC
      INTEGER NREC, LREC, OUTC, NS
C
C SHOW_GRP: list contents of a group configuration table in output file.
C
C Arguments:
C  NREC   (input)  : the sequence number of the record in the file.
C  REC    (input)  : the record (table).
C  LREC   (input)  : length of record in bytes.
C  OUTC   (input)  : Fortran unit number on which data are to be listed.
C  NS     (input)  : number of stations.
C
C 19-May-1986
C-----------------------------------------------------------------------
      INTEGER I, J, L, LSTN(MAXSTN), NC
      CHARACTER*80 TEXT
      CHARACTER*16 NAME(MAXSTN)
C
  100 FORMAT (/'(',I6.6,')   Group Configuration Table,',I6,' bytes'/)
  110 FORMAT (2X,A8,' = ',I16,5(:',',I16))
  115 FORMAT (2X,A8,' = ',L16)
  120 FORMAT (2X,A8,' = "',A,'"')
  121 FORMAT (2X,A8,' = ',A16,5(:',',A16))
  130 FORMAT (2X,A8,' = ',F16.4,5(:',',F16.4))
  140 FORMAT (2X,A8,' = ',I3,27(:',',I3))
  150 FORMAT (2X,A6,I2.2, ' = ',I16,5(:',',I16))
C-----------------------------------------------------------------------
      WRITE (OUTC,100) NREC, LREC
      CALL LIB$SYS_ASCTIM(L, TEXT, REC.MSYSTGO)
      WRITE (OUTC,120) 'MSYST', TEXT(1:L)
      WRITE (OUTC,130) 'BITSC', REC.BITSCGO
      WRITE (OUTC,130) 'BITRI', REC.BITRIGO
      WRITE (OUTC,130) 'BTCOI', REC.BTCOIGO
      WRITE (OUTC,130) 'BTTFR', REC.BTTFRGO
      WRITE (OUTC,130) 'BTHFR', REC.BTHFRGO
      WRITE (OUTC,130) 'BITMT', REC.BITMTGO
      WRITE (OUTC,110) 'MYEAR', REC.MYEARGO
      WRITE (OUTC,130) 'ETA00', REC.ETA00GO
      WRITE (OUTC,130) 'HODEL', REC.HODELGO
      WRITE (OUTC,115) 'CPFLA', REC.CPFLAGO
      WRITE (OUTC,115) 'TMLFL', REC.TMLFLGO
      WRITE (OUTC,110) 'TMMUI', REC.TMMUIGO
      WRITE (OUTC,110) 'GMMUI', REC.GMMUIGO
      WRITE (OUTC,115) 'LAGFL', REC.LAGFLGO
      WRITE (OUTC,110) 'SDADI', REC.SDADIGO
      WRITE (OUTC,110) 'CDADI', REC.CDADIGO
      WRITE (OUTC,110) 'DFRFC', REC.DFRFCGO
      WRITE (OUTC,110) 'EODLA', REC.EODLAGO
      WRITE (OUTC,110) 'NCHAN', REC.NCHANGO
      NC = REC.NCHANGO
      WRITE (OUTC,110) 'L2NPT', REC.L2NPTGO
      WRITE (OUTC,110) 'SYCHN', REC.SYCHNGO
      WRITE (OUTC,140) 'PCTAB.DM', (REC.PCTABGO(J).DMODEL, J=1,NC)
      WRITE (OUTC,140) 'PCTAB.PM', (REC.PCTABGO(J).PMODEL, J=1,NC)
      WRITE (OUTC,140) 'ICTAB', REC.ICTABGO
      WRITE (OUTC,115) 'RERRS', REC.RERRSGO
      WRITE (OUTC,115) 'STEMP', REC.STEMPGO
      WRITE (OUTC,110) 'INVCR', REC.INVCRGO
      WRITE (OUTC,140) 'IPVCR', REC.IPVCRGO
      CALL ASCCNV(REC.OPRIDGO, TEXT, L)
      WRITE (OUTC,120) 'OPRID', TEXT(1:L)
      CALL ASCCNV(REC.CCRNMGO, TEXT, L)
      WRITE (OUTC,120) 'CCRNM', TEXT(1:L)
      CALL ASCCNV(REC.PCRNMGO, TEXT, L)
      WRITE (OUTC,120) 'PCRNM', TEXT(1:L)
      CALL ASCCNV(REC.TMLNMGO, TEXT, L)
      WRITE (OUTC,120) 'TMLNM', TEXT(1:L)
      DO J=1,NC
           WRITE (OUTC,150) 'ECLCH', J, (REC.STATION(I).ECLCHSO(J), 
     1                                    I=1,NS)
      END DO
      WRITE (OUTC,110) 'ECLBS', (REC.STATION(I).ECLBSSO, I=1,NS)
      WRITE (OUTC,110) 'ANABD', (REC.STATION(I).ANABDSO, I=1,NS)
      WRITE (OUTC,110) 'TSTMD', (REC.STATION(I).TSTMDSO, I=1,NS)
      WRITE (OUTC,110) 'PCINT'
      DO I=1,NS
          CALL ASCCNV(REC.STATION(I).MTSEQSO, NAME(I), LSTN(I))
      END DO
      WRITE (OUTC,121) 'MTSEQ', (NAME(I)(1:LSTN(I)), I=1,NS)
      DO I=1,NS
          CALL ASCCNV(REC.STATION(I).MTTAGSO, NAME(I), LSTN(I))
      END DO
      WRITE (OUTC,121) 'MTTAG', (NAME(I)(1:LSTN(I)), I=1,NS)
      END

C*CONFIG
C+
      SUBROUTINE CONFIG (NREC,REC,LREC,NS,
     1                   NCHAN,NLAGS,INTEG,LAGFL,ODDLAG)
      INCLUDE 'cordef.inc'
      INCLUDE 'pcrdef.inc'
      RECORD /GRP_GO/ REC
      INTEGER NREC, LREC, OUTC, NS, NCHAN, NLAGS
      REAL INTEG
      LOGICAL LAGFL, ODDLAG
C
C CONFIG : summarize correlator setup, based on a group table.
C
C Arguments:
C  NREC   (input)  : the sequence number of the record in the file.
C  REC    (input)  : the record (table).
C  LREC   (input)  : length of record in bytes.
C  NS     (input)  : number of stations.
C
C 19-May-1986
C-----------------------------------------------------------------------
      NCHAN = REC.NCHANGO
      NLAGS = 2**REC.L2NPTGO
      INTEG = REC.CDADIGO * REC.BITRIGO / REC.BITSCGO
      LAGFL = REC.LAGFLGO
      ODDLAG = (REC.EODLAGO.EQ.1)
C-----------------------------------------------------------------------
      END

C*SHOW_DLY
C+
      SUBROUTINE SHOW_DLY (NREC,REC,LREC,OUTC,NS)
      INCLUDE 'cordef.inc'
      INCLUDE 'pcrdef.inc'
      RECORD /DLY_DM/ REC
      INTEGER NREC, LREC, OUTC, NS
C
C SHOW_DLY: list contents of a PCR delay model table in output file.
C
C Arguments:
C  NREC   (input)  : the sequence number of the record in the file.
C  REC    (input)  : the record (table).
C  LREC   (input)  : length of record in bytes.
C  OUTC   (input)  : Fortran unit number on which data are to be listed.
C  NS     (input)  : number of stations.
C
C 29-Apr-1986
C-----------------------------------------------------------------------
      REAL*8 PI, DEGREE
      PARAMETER (PI = 3.141592653589793 D0)
      PARAMETER (DEGREE=180D0/PI)
      INTEGER I, L, LSTN(MAXSTN)
      CHARACTER*80 TEXT
      REAL*8 X(MAXSTN), Y(MAXSTN)
      CHARACTER*16 STNAME(MAXSTN)
C
  100 FORMAT (/'(',I6.6,')   Delay Model Table,',I6,' bytes'/)
  110 FORMAT (2X,A8,' = ',I16,5(:',',I16))
  115 FORMAT (2X,A8,' = ',L16)
  120 FORMAT (2X,A8,' = "',A,'"')
  121 FORMAT (2X,A8,' = ',A16,5(:',',A16))
  130 FORMAT (2X,A8,' = ',F16.4,5(:',',F16.4))
C-----------------------------------------------------------------------
      WRITE (OUTC,100) NREC, LREC
C -- table identification
      WRITE (OUTC,110) 'DMODN', REC.DMODNDM
      CALL LIB$SYS_ASCTIM(L, TEXT, REC.DSYSTDM)
      WRITE (OUTC,120) 'DSYST', TEXT(1:L)
C -- data identification
      CALL ASCCNV(REC.EXNAMDM, TEXT, L)
      WRITE (OUTC,120) 'EXNAM', TEXT(1:L)
      CALL ASCCNV(REC.SNAMEDM, TEXT, L)
      WRITE (OUTC,120) 'SNAME', TEXT(1:L)
      WRITE (OUTC,110) 'SCANN', REC.SCANNDM
C -- orientation model information
      WRITE (OUTC,115) 'RSFLG', REC.RSFLGDM
      WRITE (OUTC,130) 'XPOLE', REC.XPOLEDM
      WRITE (OUTC,130) 'YPOLE', REC.YPOLEDM
C -- source model information
      WRITE (OUTC,110) 'PDATE', REC.PDATEDM
      CALL PANGLE('H', REC.RTAS0DM, TEXT(1:16), 4)
      WRITE (OUTC,120) 'RTAS0', TEXT(1:16) 
      CALL PANGLE('D', REC.DECL0DM, TEXT(1:15), 3)
      WRITE (OUTC,120) 'DECL0', TEXT(1:15) 
      CALL PANGLE('H', REC.RTASCDM, TEXT(1:16), 4)
      WRITE (OUTC,120) 'RTASC', TEXT(1:16) 
      CALL PANGLE('D', REC.DECLNDM, TEXT(1:15), 3)
      WRITE (OUTC,120) 'DECLN', TEXT(1:15) 
C -- timing model information
      CALL BITCNV(REC.EXTIMDM, TEXT, L)
      WRITE (OUTC,120) 'EXTIM', TEXT(1:L)
      WRITE (OUTC,130) 'UTMTC', REC.UTMTCDM
      WRITE (OUTC,130) 'EQOEQ', REC.EQOEQDM
C -- sequencing information
      WRITE (OUTC,110) 'ODATE', REC.ODATEDM
      CALL BITCNV(REC.BEGTMDM, TEXT, L)
      WRITE (OUTC,120) 'BEGTM', TEXT(1:L)
      CALL BITCNV(REC.ENDTMDM, TEXT, L)
      WRITE (OUTC,120) 'ENDTM', TEXT(1:L)
C -- station section
      DO I=1,NS
          CALL ASCCNV(REC.STATION(I).STNAMDO, STNAME(I), LSTN(I))
          X(I) = REC.STATION(I).RSPINDO * COS(REC.STATION(I).WLONGDO)
          Y(I) = REC.STATION(I).RSPINDO * SIN(REC.STATION(I).WLONGDO)
      END DO
      WRITE (OUTC,121) 'STNAM', (STNAME(I)(1:LSTN(I)), I=1,NS)
C -- geometric model information
      WRITE (OUTC,130) 'RSPIN', (REC.STATION(I).RSPINDO, I=1,NS)
      WRITE (OUTC,130) '  X  ', (X(I), I=1,NS)
      WRITE (OUTC,130) '  Y  ', (Y(I), I=1,NS)
      WRITE (OUTC,130) 'ZPROJ', (REC.STATION(I).ZPROJDO, I=1,NS)
      WRITE (OUTC,130) 'WLONG', (REC.STATION(I).WLONGDO*DEGREE, I=1,NS)
      WRITE (OUTC,130) 'ZTROP', (REC.STATION(I).ZTROPDO, I=1,NS)
C -- antenna axis offsets
      WRITE (OUTC,110) 'AXTYP', (REC.STATION(I).AXTYPDO, I=1,NS)
      WRITE (OUTC,130) 'AXOF1', (REC.STATION(I).AXOF1DO, I=1,NS)
      WRITE (OUTC,130) 'AXOF2', (REC.STATION(I).AXOF2DO, I=1,NS)
C -- timing model information
      WRITE (OUTC,130) 'CLCOR', (REC.STATION(I).CLCORDO, I=1,NS)
      WRITE (OUTC,130) 'CLRAT', (REC.STATION(I).CLRATDO, I=1,NS)
      WRITE (OUTC,130) 'PCCOR', (REC.STATION(I).PCCORDO, I=1,NS)
      WRITE (OUTC,130) 'PCRAT', (REC.STATION(I).PCRATDO, I=1,NS)
C -- sequencing information
      DO I=1,NS
          CALL BITCNV(REC.STATION(I).STARTDO, STNAME(I), LSTN(I))
      END DO
      WRITE (OUTC,121) 'START', (STNAME(I), I=1,NS)
      DO I=1,NS
          CALL BITCNV(REC.STATION(I).STOPTDO, STNAME(I), LSTN(I))
      END DO
      WRITE (OUTC,121) 'STOPT', (STNAME(I), I=1,NS)
      END

C*BITCNV
C+
      SUBROUTINE BITCNV (BITTIM, TEXT, L)
      REAL*8 BITTIM
      CHARACTER*(*) TEXT
      INTEGER L
C
C Convert a Bit Time in the Block-II format into a printable
C character string 'ddd hh:mm:ss.sss'.  CAUTION: it assumes the bit rate
C is 4 MHz.
C-----------------------------------------------------------------------
      REAL*8 SEC
      INTEGER MN, HR, DAY, IS, IMS
C
      SEC = BITTIM/4D6
      DAY = INT(SEC)/86400 + 1
      SEC = SEC - (DAY-1)*86400
      HR = INT(SEC)/3600
      SEC = SEC - HR*3600
      MN = INT(SEC)/60
      SEC = SEC - MN*60
      IMS = NINT(SEC*1000D0)
      IS = IMS/1000
      IMS = IMS - IS*1000
      WRITE (TEXT,'(I3.3,I3.2,'':'',I2.2,'':'',I2.2,''.'',I3.3)')
     2                    DAY, HR, MN, IS, IMS
      L = 16
      END

C*SHOW_PHS
C+
      SUBROUTINE SHOW_PHS (NREC,REC,LREC,OUTC,NS,PRVID)
      INCLUDE 'cordef.inc'
      INCLUDE 'pcrdef.inc'
      RECORD /PHS_PM/ REC
      INTEGER NREC, LREC, OUTC, NS, PRVID
C
C SHOW_PHS: list contents of a PCR phase model table in output file.
C
C Arguments:
C  NREC   (input)  : the sequence number of the record in the file.
C  REC    (input)  : the record (table).
C  LREC   (input)  : length of record in bytes.
C  OUTC   (input)  : Fortran unit number on which data are to be listed.
C  NS     (input)  : number of stations.
C  PRVID  (input)  : correlator program ID.
C
C 2-Jul-1987
C-----------------------------------------------------------------------
      INTEGER I, L
      CHARACTER*80 TEXT
C
  100 FORMAT (/'(',I6.6,')   Phase Model Table,',I6,' bytes'/)
  110 FORMAT (2X,A8,' = ',I16,5(:',',I16))
  115 FORMAT (2X,A8,' = ',L16)
  120 FORMAT (2X,A8,' = "',A,'"')
  130 FORMAT (2X,A8,' = ',F16.4,5(:',',F16.4))
  140 FORMAT (2X,'Data format incompatible with this version of B2PS.')
C-----------------------------------------------------------------------
      WRITE (OUTC,100) NREC, LREC
C -- table identification
      WRITE (OUTC,110) 'PMODN', REC.PMODNPM
      CALL LIB$SYS_ASCTIM(L, TEXT, REC.PSYSTPM)
      WRITE (OUTC,120) 'PSYST', TEXT(1:L)
      IF (PRVID.LT.1987131) THEN
          WRITE (OUTC,140)
          RETURN
      END IF
C -- global model information
      WRITE (OUTC,130) 'FQCEN', REC.FQCENPM
      WRITE (OUTC,130) 'FQNLO', REC.FQNLOPM
      WRITE (OUTC,130) 'FQDEL', REC.FQDELPM
      WRITE (OUTC,115) 'FQSBF', REC.FQSBFPM
C -- station section
      WRITE (OUTC,130) 'FQSLO', (REC.STATION(I).FQSLOPO, I=1,NS)
      WRITE (OUTC,130) 'FQOFF', (REC.STATION(I).FQOFFPO, I=1,NS)
      WRITE (OUTC,130) 'PHREP', (REC.STATION(I).PHREPPO, I=1,NS)
      WRITE (OUTC,130) 'PDREP', (REC.STATION(I).PDREPPO, I=1,NS)
      WRITE (OUTC,130) 'PDDRE', (REC.STATION(I).PDDREPO, I=1,NS)
      WRITE (OUTC,130) 'PDDDR', (REC.STATION(I).PDDDRPO, I=1,NS)
      WRITE (OUTC,130) 'DPREP', (REC.STATION(I).DPREPPO, I=1,NS)
      END

C*SHOW_TON
C+
      SUBROUTINE SHOW_TON (NREC,REC,LREC,OUTC,NS)
      INCLUDE 'cordef.inc'
      INCLUDE 'pcrdef.inc'
      RECORD /TON_TM/ REC
      INTEGER NREC, LREC, OUTC, NS
C
C SHOW_TON: list contents of a PCR tone model table in output file.
C
C Arguments:
C  NREC   (input)  : the sequence number of the record in the file.
C  REC    (input)  : the record (table).
C  LREC   (input)  : length of record in bytes.
C  OUTC   (input)  : Fortran unit number on which data are to be listed.
C  NS     (input)  : number of stations.
C
C 16-May-1986
C-----------------------------------------------------------------------
      INTEGER I, J, L
      CHARACTER*80 TEXT
C
  100 FORMAT (/'(',I6.6,')   Tone Model Table,',I6,' bytes'/)
  110 FORMAT (2X,A8,' = ',I16,5(:',',I16))
  120 FORMAT (2X,A8,' = "',A,'"')
  130 FORMAT (2X,A8,' = ',F16.4,5(:',',F16.4))
C-----------------------------------------------------------------------
      WRITE (OUTC,100) NREC, LREC
C -- table identification
      WRITE (OUTC,110) 'TMODN', REC.TMODNTM
      CALL LIB$SYS_ASCTIM(L, TEXT, REC.TSYSTTM)
      WRITE (OUTC,120) 'TSYST', TEXT(1:L)
C -- station section
      WRITE (OUTC,130) 'TCPH0', (REC.STATION(I).TCPH0TO, I=1,NS)
      WRITE (OUTC,130) 'TCPD0', (REC.STATION(I).TCPD0TO, I=1,NS)
      WRITE (OUTC,130) 'TFREQ', (REC.STATION(I).TFREQTO, I=1,NS)
      WRITE (OUTC,130) 'TFQOF', (REC.STATION(I).TFQOFTO, I=1,NS)
      DO J=1,MAXTMC
          WRITE (OUTC,130) 'TPOLY', (REC.STATION(I).TPOLYTO(J), I=1,NS)
      END DO
      END

C*SHOW_IAO
C+
      SUBROUTINE SHOW_IAO (NREC,REC,LREC,OUTC)
      INCLUDE 'cordef.inc'
      INCLUDE 'pcrdef.inc'
      RECORD /IAO_IA/ REC
      INTEGER NREC, LREC, OUTC
C
C SHOW_DLY: list contents of an IAO Interface Ancillary Data table in
C output file.
C
C Arguments:
C  NREC   (input)  : the sequence number of the record in the file.
C  REC    (input)  : the record (table).
C  LREC   (input)  : length of record in bytes.
C  OUTC   (input)  : Fortran unit number on which data are to be listed.
C
C 22-May-1986
C-----------------------------------------------------------------------
      BYTE DUMP(4*MAXIAF)
      INTEGER I, J, L, NBYTES
      CHARACTER*16 ITIME

      CALL BITCNV (REC.RTIMEIA,ITIME,L)
      L = 0
      NBYTES = LREC-64
      DO J=1,MAXIAF*4
              IF (L.GE.NBYTES) GOTO 10
              L = L+1
              DUMP(L) = REC.VDATAIA(J)
              IF (DUMP(L).LT.32 .OR. DUMP(L).GT.126) 
     1            DUMP(L) = ICHAR('.')
      END DO
   10 WRITE (OUTC,'(A16,1X,6A1,1X,Z8.8,1X,128A1)') ITIME, REC.VTIMEIA, 
     1        REC.VSTATIA, (DUMP(I),I=1,L)
C-----------------------------------------------------------------------
      END

C*SHOW_OPR
C+
      SUBROUTINE SHOW_OPR (NREC,REC,LREC,OUTC,NS)
      INCLUDE 'cordef.inc'
      INCLUDE 'pcrdef.inc'
      RECORD /OPR_OP/ REC
      INTEGER NREC, LREC, OUTC, NS
C
C SHOW_OPR: list contents of an operator table in output file.
C
C Arguments:
C  NREC   (input)  : the sequence number of the record in the file.
C  REC    (input)  : the record (table).
C  LREC   (input)  : length of record in bytes.
C  OUTC   (input)  : Fortran unit number on which data are to be listed.
C  NS     (input)  : number of stations.
C
C 27-Nov-90
C-----------------------------------------------------------------------
      INTEGER L
      CHARACTER*255 TEXT
C
  100 FORMAT (/'(',I6.6,')   Operator Comment Table,',I6,' bytes'/)
  120 FORMAT (2X,A8,' = "',A,'"')
C-----------------------------------------------------------------------
      WRITE (OUTC,100) NREC, LREC
      CALL LIB$SYS_ASCTIM(L, TEXT, REC.OSYSTOP)
      WRITE (OUTC,120) 'OSYST', TEXT(1:L)
      CALL BITCNV (REC.RTIMEOP,TEXT,L)
      WRITE (OUTC,120) 'RTIME', TEXT(1:L)
      CALL ASCCNV(REC.OPNAMOP, TEXT, L)
      WRITE (OUTC,120) 'OPNAM', TEXT(1:L)
      CALL ASCCNV(REC.OPCOMOP, TEXT, L)
      WRITE (OUTC,120) 'OPCOM', TEXT(1:L)
      END

C*GETSS
C+
      SUBROUTINE GETSS (REC, NS, SRC, NSRC, STA, NSTA, EXP, NEXP,
     1                  SCN, NSCN)
      INCLUDE 'cordef.inc'
      INCLUDE 'pcrdef.inc'
      RECORD /DLY_DM/ REC
      INTEGER NS, NSRC, NSTA, NEXP, NSCN, SCN(50)
      CHARACTER*16 SRC(50), STA(50), EXP(50)
C
C GETSS : read a PCR delay model table and update the source and
C station lists SRC() and STA().
C
C Arguments:
C  REC    (input)  : the record (table).
C  NS     (input)  : number of stations.
C  SRC    (in/out) : source list.
C  NSRC   (in/out) : number of sources.
C  STA    (in/out) : station list.
C  NSTA   (in/out) : number of stations.
C  EXP    (in/out) : experiment list.
C  NEXP   (in/out) : number of experiments.
C
C 15-May-1986
C-----------------------------------------------------------------------
      CHARACTER*16 TEXT
      INTEGER I,J,L
C
C Save source name.
C
      CALL ASCCNV(REC.SNAMEDM, TEXT, L)
      IF (TEXT.EQ.' ') GOTO 10
      DO J=1,NSRC
          IF (TEXT .EQ. SRC(J)) GOTO 10
      END DO
      NSRC = NSRC+1
      SRC(NSRC) = TEXT
   10 CONTINUE
C
C Save station names.
C
      DO I=1,NS
          CALL ASCCNV(REC.STATION(I).STNAMDO, TEXT, L)
          IF (TEXT.EQ.' ') GOTO 20
          DO J=1,NSTA
              IF (TEXT .EQ. STA(J)) GOTO 20
          END DO
          NSTA = NSTA + 1
          STA(NSTA) = TEXT
   20     CONTINUE
      END DO
C
C Save experiment name.
C
      CALL ASCCNV(REC.EXNAMDM, TEXT, L)
      IF (TEXT.EQ.' ') GOTO 30
      DO J=1,NEXP
          IF (TEXT .EQ. EXP(J)) GOTO 30
      END DO
      NEXP = NEXP+1
      EXP(NEXP) = TEXT
   30 CONTINUE
C
C Save scan numbers.
C
      L = REC.SCANNDM
      IF (L.EQ.0) GOTO 40
      DO J=1,NSCN
          IF (L .EQ. SCN(J)) GOTO 40
      END DO
      NSCN = NSCN+1
      SCN(NSCN) = L
   40 CONTINUE
C-----------------------------------------------------------------------
      END
