      PROGRAM BASEDT
C-----------------------------------------------------------------------
C BASEDT: to copy one MERGE-format file to another, selecting only the
C baselines and triangles that have data.
C
C Parameters:
C      INPUT = input file name
C      OUTPUT = output file name
C      HISTORY = text for history (optional)
C
C T.J. Pearson, standard Fortran 77.
C
C Subroutines required:
C      ERROR, KEYIN, MKHIST.
C
C Version 1.0:  1985 Jun 20.
C Version 1.1:  1987 Dec 15 - add a history record.
C Version 1.2:  1988 Jun 21 - Convex changes.
C Version 1.3:  1990 Nov 13 - remove variables declared but not used.
C-----------------------------------------------------------------------
C
C Constants.
C
      CHARACTER*(*) PROGNM, VERSN
      INTEGER       INC,OUTC,INDAT,OUTDAT,MAXS,MAXB,MAXT,NPARS
      PARAMETER     (PROGNM='BASEDIT')
      PARAMETER     (VERSN='1.3 - 1990 Nov 13')
      PARAMETER     (INC=5,OUTC=6,INDAT=8,OUTDAT=9)
      PARAMETER     (MAXS=20)
      PARAMETER     (MAXB=(MAXS-1)*MAXS/2)
      PARAMETER     (MAXT=MAXB*(MAXS-2)/3)
      PARAMETER     (NPARS=30)
C
C Variables.
C
      CHARACTER*8  STNAME(MAXS)
      CHARACTER*16 SOURCE
      INTEGER   IUT, IS, IB, IT, NS, NB, NT, I, J, IER, MODE
      INTEGER   IZ, IY, NHIST
      INTEGER   NBO, NTO, LENGO, NBGONE, NTGONE, NNEW
      INTEGER   NBI(MAXB), NB1(MAXB), NB2(MAXB), BCOUNT(MAXB)
      INTEGER   BGONE(MAXB)
      INTEGER   NTI(MAXT), TCOUNT(MAXT), TGONE(MAXT)
      INTEGER   REC1(20), IDUM(8)
      INTEGER   JPOS(4*MAXB+2*MAXT)
      INTEGER   VLBOPE
      REAL      INREC(4*MAXB+2*MAXT)
      DOUBLEPRECISION STX(MAXS), STY(MAXS), STZ(MAXS)
      DOUBLEPRECISION RA, DEC, RAP, DECP, GASTM, FREQ, FLUX
      DOUBLEPRECISION TINTG, AVG, BW
      DOUBLEPRECISION PARS(NPARS),VALS(NPARS),ENDMRK
      LOGICAL   BWANT(MAXB), TWANT(MAXT)
      CHARACTER*80 INDSN, OUTDSN, HISTRY, ADDHIS(2)
C
      DATA      PARS/ 'Input   ', 9*' ',
     1                'Output  ', 9*' ',
     2                'History ', 9*' '/
      DATA      VALS/ '***UNSET', 9*' ',
     1                '***UNSET', 9*' ',
     2                10*' '           /
      DATA      ENDMRK/ '/' /
C
C Format statements.
C
  610 FORMAT(/1X,I3,' of the original ',I3,
     1       ' baselines will be retained')
  611 FORMAT(' Baselines deleted:'/(1X,10(1X,I4)))
  620 FORMAT(/1X,I3,' of the original ',I3,
     1       ' triangles will be retained')
  621 FORMAT(' Triangles deleted:'/(1X,10(1X,I6)))
C-----------------------------------------------------------------------
C
C Introduction.
C
      CALL PUTOUT(PROGNM//
     1            ' removes unobserved baselines from MERGE files')
      CALL PUTOUT('(Version '//VERSN//')')
C
C Control parameters.
C
      MODE = 0
  100 IF (MODE.NE.0) CALL ERROR('Bad input parameters in '//PROGNM)
      CALL PUTOUT(' ')
      CALL PUTOUT('Parameters (INPUT,OUTPUT,HISTORY):')
      CALL KEYIN(PARS,VALS,NPARS,ENDMRK,MODE,INC,OUTC)
      WRITE (INDSN,'(10A8)') (VALS(I),I=1,10)
      WRITE (OUTDSN,'(10A8)') (VALS(I),I=11,20)
      WRITE (HISTRY,'(10A8)') (VALS(I),I=21,30)
C
C Open input Merge file.
C
      IER = VLBOPE(INDAT, INDSN, 'MERGE', 'OLD', INDSN)
      IF (IER.NE.1) GOTO 100
C
C Open output Merge file.
C
      IER = VLBOPE(OUTDAT, OUTDSN, 'MERGE', 'NEW', OUTDSN)
      IF (IER.NE.1) THEN
          CLOSE (UNIT=INDAT)
          GOTO 100
      END IF
C
C Make the new history record.
C
      CALL MKHIST(PROGNM, VERSN, ADDHIS(1))
      IF (HISTRY.NE.' ') THEN
          NNEW = 2
          ADDHIS(2) = '- '//HISTRY(1:78)
      ELSE
          NNEW = 1
      END IF
C
C Read input header; check format=1.
C
      READ (INDAT,END=82,ERR=82) (REC1(J),J=1,20)
      IF (REC1(1).NE.1) GOTO 82
      NHIST = REC1(2)
      NS    = REC1(4)
      NB    = REC1(5)
      NT    = REC1(6)
      IF (NS.LT.2) CALL ERROR('Not enough stations.')
      IF (NS.GT.MAXS) CALL ERROR('Too many stations.')
      IF (NB.GT.0) THEN
          READ (INDAT,END=82,ERR=82) (NBI(IB),IB=1,NB)
          DO 110 IB=1,NB
              NB1(IB) = NBI(IB)/100
              NB2(IB) = NBI(IB) -  NB1(IB)*100
  110     CONTINUE
      END IF
      IF (NT.GT.0) THEN
          READ (INDAT,END=82,ERR=82) (NTI(IT),IT=1,NT)
      END IF
      DO 120 I=1,NHIST
          READ (INDAT,END=82,ERR=82) HISTRY
  120 CONTINUE
      READ (INDAT,END=82,ERR=82) SOURCE,RA,DEC,RAP,DECP,FLUX
      DO 130 IS=1,NS
          READ (INDAT,END=82,ERR=82) STNAME(IS),STX(IS),STY(IS),STZ(IS)
  130 CONTINUE
      READ (INDAT,END=82,ERR=82) IY, IZ, GASTM, FREQ, TINTG,
     1            AVG, BW, (IDUM(I),I=1,8)
C
C Read data records and count data.
C
      DO 140 IB=1,NB
          BCOUNT(IB) = 0
  140 CONTINUE
      DO 150 IT=1,NT
          TCOUNT(IT) = 0
  150 CONTINUE
  160 READ (INDAT,END=190) IUT,(INREC(J),J=1,4*NB+2*NT)
      IF (IUT.LT.0) GOTO 190
      DO 170 IB=1,NB
          IF (INREC(4*IB-2).GT.0.0) BCOUNT(IB) = BCOUNT(IB) + 1
  170 CONTINUE
      DO 180 IT=1,NT
          IF (INREC(2*IT+4*NB).GT.0.0) TCOUNT(IT) = TCOUNT(IT) + 1
  180 CONTINUE
      GOTO 160
C
C Rewind input file for re-reading; select which data items are to be
C kept.
C
  190 REWIND (UNIT=INDAT)
      NBO = 0
      DO 200 IB=1,NB
          BWANT(IB) = BCOUNT(IB).GT.0
          IF (BWANT(IB)) THEN
              NBO = NBO+1
              JPOS(4*NBO-3) = 4*IB-3
              JPOS(4*NBO-2) = 4*IB-2
              JPOS(4*NBO-1) = 4*IB-1
              JPOS(4*NBO)   = 4*IB
          ELSE
              NBGONE = NBGONE+1
              BGONE(NBGONE) = NBI(IB)
          END IF
  200 CONTINUE
      NTO = 0
      DO 210 IT=1,NT
          TWANT(IT) = TCOUNT(IT).GT.0
          IF (TWANT(IT)) THEN
              NTO = NTO+1
              JPOS(4*NBO + 2*NTO -1) = 4*NB + 2*IT - 1
              JPOS(4*NBO + 2*NTO)    = 4*NB + 2*IT
          ELSE 
              NTGONE = NTGONE+1
              TGONE(NTGONE) = NTI(IT)
          END IF
  210 CONTINUE
                       WRITE (OUTC,610) NBO, NB 
      IF (NBGONE.GT.0) WRITE (OUTC,611) (BGONE(I),I=1,NBGONE)
      IF (NT.GT.0)     WRITE (OUTC,620) NTO, NT 
      IF (NTGONE.GT.0) WRITE (OUTC,621) (TGONE(I),I=1,NTGONE)
C
C Read and re-write header.
C
      READ (INDAT,END=82,ERR=82) (REC1(J),J=1,20)
      IF (REC1(1).NE.1) GOTO 82
      NHIST = REC1(2)
      NS    = REC1(4)
      NB    = REC1(5)
      NT    = REC1(6)
      IF (NS.LT.2) CALL ERROR('Not enough stations.')
      IF (NS.GT.MAXS) CALL ERROR('Too many stations.')
      REC1(2) = NHIST + NNEW
      REC1(3) = NHIST + 2 + NS + NNEW
      IF (NBO.GT.0) REC1(3) = REC1(3) + 1
      IF (NTO.GT.0) REC1(3) = REC1(3) + 1
      REC1(5) = NBO
      REC1(6) = NTO
      LENGO = 4*NBO + 2*NTO
      REC1(7) = LENGO+1
      WRITE (OUTDAT) (REC1(J),J=1,20)
C
      IF (NB.GT.0) THEN
          READ (INDAT,END=82,ERR=82) (NBI(IB),IB=1,NB)
          IF (NBO.GT.0) THEN
              NBO = 0
              DO 220 IB=1,NB
                  IF (BWANT(IB)) THEN
                      NBO = NBO+1
                      NBI(NBO) = NBI(IB)
                  END IF
  220         CONTINUE
              WRITE (OUTDAT) (NBI(IB),IB=1,NBO)
          END IF
      END IF
C
      IF (NT.GT.0) THEN
          READ (INDAT,END=82,ERR=82) (NTI(IT),IT=1,NT)
          IF (NTO.GT.0) THEN
              NTO = 0
              DO 230 IT=1,NT
                  IF (TWANT(IT)) THEN
                      NTO = NTO+1
                      NTI(NTO) = NTI(IT)
                  END IF
  230         CONTINUE
              WRITE (OUTDAT) (NTI(IT),IT=1,NTO)
          END IF
      END IF
C
      DO 240 I=1,NHIST
          READ (INDAT,END=82,ERR=82) HISTRY
          WRITE (OUTDAT) HISTRY
  240 CONTINUE
      DO 250 I=1,NNEW
          WRITE (OUTDAT) ADDHIS(I)
  250 CONTINUE
C
      READ (INDAT,END=82,ERR=82) SOURCE,RA,DEC,RAP,DECP,FLUX
      WRITE (OUTDAT) SOURCE,RA,DEC,RAP,DECP,FLUX
C
      DO 260 IS=1,NS
          READ (INDAT,END=82,ERR=82) STNAME(IS),STX(IS),STY(IS),STZ(IS)
          WRITE(OUTDAT) STNAME(IS),STX(IS),STY(IS),STZ(IS),
     1            (0.D0,J=5,10),0
  260 CONTINUE
C
      READ (INDAT,END=82,ERR=82) IY, IZ, GASTM, FREQ, TINTG,
     1            AVG, BW, (IDUM(I),I=1,8)
      WRITE (OUTDAT) IY, IZ, GASTM, FREQ, TINTG,
     1            AVG, BW, (IDUM(I),I=1,8)
C
C Read and re-write data records.
C
  310 READ(INDAT,END=410) IUT,(INREC(J),J=1,4*NB+2*NT)
      IF (IUT.LT.0) GOTO 410
      WRITE (OUTDAT) IUT,(INREC(JPOS(J)),J=1,LENGO)
      GOTO 310
C
C Error warning.
C
   82 CALL PUTOUT('Something wrong with input MERGE file.')
      CLOSE (UNIT=OUTDAT)
      CLOSE (UNIT=INDAT)
      GOTO 100
C
C Write final record and quit.
C
  410 WRITE (OUTDAT) -1,(0.0,J=1,LENGO)
      CLOSE (UNIT=INDAT)
      CLOSE (UNIT=OUTDAT)
C
      END
