      PROGRAM CLEAN
C-----------------------------------------------------------------------
C CLEAN: Main Program. This routine determines the array sizes,
C allocates the arrays dynamically (VAX specific) and calls
C subroutine DOCLEAN to do the work.
C
C T.J. Pearson.
C
C       Version 1:    1979 July 5
C       Version 2:    1979 September 8
C         (1) Print cleaning window above clean map
C         (2) Change XYINT to floating-point, milliarcsec
C         (3) Change transfer-format for maps to binary
C       Version 3:    1980 Feb 18
C         (4) New format for contour maps
C         (5) Default contours 5,15...%; list contours
C         (6) Plot window if requested
C         (7) Variable map size
C         (8) NORESTORE switch
C         (9) Specify window (LRTB) in milliarcsec
C       Version 4:    1980 May 30
C          (10) Revised for new INVERT.
C          (11) Dynamic allocation.
C       Version 5:      1981 March 5
C          (12) NORESTORE inhibits PRINTBEAM
C          (13) Add comments in output model file
C          (14) Change to Fortran-77 syntax
C       Version 6:      1981 July 16
C          (15) Plot x-axis right to left
C          (16) Squash model
C       Version 7:      1981 November 27
C          (17) Add NOSQUASH parameter to suppress (16)
C          (18) Change format of gray-scale plots
C          (19) Remove PRINTBEAM option
C       Version 7.1:    1982 March 4
C          (20) Correct bug in error message
C          (21) More decimal places in flux output
C       Version 8.0:    1983 March 30
C          (22) Add COMPLOT option.
C          (23) Add PLOTFILE parameter.
C          (24) Change contour-level specification to percent.
C          (25) Add CLEANMAP output file parameter.
C Version 9.0:   1984 Jan 22 - Change format of clean map to FITS;
C                              change all flux scales to Jy/beam.
C Version 9.1:   1984 Jun 11 - Add clean components as a Tables
C                              extension to the clean map; display max
C                              and min of clean map.
C Version 9.2:   1985 Jun 26 - Pick up array name and epoch from INVERT.
C Version 9.3:   1986 Apr  4 - Pick up map center from INVERT.
C Version 9.4:   1986 Sep 25 - Add ADDSOURCE parameter.
C Version 10.0:  1988 Apr 10 - eliminate command qualifiers; add
C                              LISTFILE; test on Convex (TJP).
C Version 10.1:  1988 Jul  7 - multiple windows (TJP).
C Version 10.2:  1988 Aug  9 - add epoch to header of clean map.
C Version 10.3:  1988 Oct 22 - add explicit 'D0' to real*8 parameters.
C Version 10.4:  1988 Nov  1 - if NORESTORE specified, still write
C                              out clean map (i.e. map of residuals).
C Version 10.5:  1988 Dec 16 - slight changes to plotting.
C Version 10.6:  1988 Dec 30 - correct error in window calculation;
C                              add PLOTDELTA option.
C Version 10.7:  1989 Mar  3 - changes to plotting; additional 
C                              annotation.
C Version 10.8:  1989 Jun  6 - put FITS image the right way up.
C Version 10.9:  1989 Aug 23 - increase size of clean beam array.
C Version 11.0:  1991 Apr 17 - put rotation angle in header of cleanmap.
C Version 11.1:  1991 Nov  7 - pick up default beam from INVERT.
C Version 11.2:  1991 Dec 17 - correct error in CC table.
C Version 11.3:  1992 Feb 20 - prevent underflow.
C Version 11.4:  1994 May 13 - fix bug: crashed if first cmpt was
C                              negative and PLOTCOMP requested.
C-----------------------------------------------------------------------
      CHARACTER*(*) VERSN
      PARAMETER (VERSN='11.4 - 1994 May 13')
      INTEGER   INC, OUTC, LIST, NPARS
      PARAMETER (INC=5, OUTC=6, LIST=1, NPARS=180)
C
      CHARACTER*255 LISFIL
      INTEGER   I, MNDT, MAXIT, MODE, IER
      INTEGER   LIB$GET_VM, LEN1
      INTEGER   NBYTES(5),LOC(5)
C
C Input control parameter definitions.
C
      DOUBLE PRECISION PARS(NPARS),VALS(NPARS),ENDMRK
      DATA PARS/'NITER','TMIN',3*'BEAM','LOOPGAIN','NCPRINT',
     1          'PLOTComp','PRINTMap','PRINTRes','PLOTMap','PLOTWind',
     2          'NORESTor','PLOTDelt', 3*'*Unused*',20*'CONTours',
     3          'TITLE',9*' ', 'MAPFILE',4*' ', 'BEAMFile',4*' ',
     4          'MODEL',4*' ','HYBRID','ARCSEC','MILLIARC',
     5          'NOSQUASH', 'PLOTFile',9*' ',
     6          'CLEANMAP', 9*' ', 3*'ADdsourc',
     7          'SIZE', 'LIStfile', 9*' ', 80*'LRTB'/
      DATA VALS/100D0,0D0,3*0D0,0.3D0,20D0,7*-1D0,
     1          3*0D0, 20*1D37,
     2          '\fiClean',' Map',8*' ', 'MAP',4*' ',
     3          'BEAM',4*' ', 'MODEL',4*' ',4*-1D0,
     4          'PLOTOUT', 9*' ', '  ', 9*' ', 3*0D0,
     5          128D0, 'clean.li', 's', 8*' ', 4*1D37, 76*0D0/
      DATA ENDMRK/'/'/
C-----------------------------------------------------------------------
C Format statements.
C
  630 FORMAT('Program CLEAN cleans aperture-synthesis images'/
     1       '(Version ',A,')'//
     2       'Control parameters:'/)
C-----------------------------------------------------------------------
C Introduction.
C
      CALL PUTOUT('CLEAN cleans aperture-synthesis images')
      CALL PUTOUT('(Version '//VERSN//')')
      CALL PUTOUT(' ')
C
C Read the control parameters.
C
      CALL PUTOUT('Control parameters:')
      MODE = 0
      CALL KEYIN(PARS,VALS,NPARS,ENDMRK,MODE,INC,OUTC)
C
C Evaluate the parameters used for memory allocation.
C
      MNDT = VALS(90)
      MAXIT = ABS(VALS(1))
      IF (MNDT.GT.2048) CALL ERROR('SIZE > 2048')
C
C Allocate memory for 5 arrays:
C     REAL*4 B(MNDT,2*MNDT)
C     REAL*4 T(MNDT,MNDT+2)
C     INTEGER*2 ICOMP(MAXIT)
C     INTEGER*2 JCOMP(MAXIT)
C     REAL*4 TCOMP(MAXIT)
C
      NBYTES(1) = MNDT*MNDT*2*4
      NBYTES(2) = MNDT*(MNDT+2)*4
      NBYTES(3) = MAXIT*2
      NBYTES(4) = MAXIT*2
      NBYTES(5) = MAXIT*4
      DO 30 I=1,5
          IER = LIB$GET_VM(NBYTES(I),LOC(I))
          IF (IER.NE.1) CALL
     1        ERROR('Insufficient virtual memory for dynamic arrays.')
   30 CONTINUE
C
C Open the listing file.
C
      WRITE (LISFIL, '(10A8)') (VALS(I),I=91,100)
      OPEN (UNIT=LIST, FILE=LISFIL, STATUS='NEW',
     1      CARRIAGECONTROL='LIST', IOSTAT=IER)
      IF (IER.NE.0) CALL ERROR('Cannot open LISTFILE='//
     1                         LISFIL(1:LEN1(LISFIL)))
      WRITE (LIST, 630) VERSN
      CALL KEYOUT(PARS, VALS, NPARS, LIST)
C
C Call a subroutine to do the work.
C
      CALL DOCLEAN(MNDT,MAXIT,%VAL(LOC(1)),%VAL(LOC(2)),
     1             %VAL(LOC(3)),%VAL(LOC(4)),%VAL(LOC(5)),
     2             VALS, NPARS, LIST, VERSN)
C
C Close the listing file.
C
      INQUIRE (UNIT=LIST, NAME=LISFIL)
      CLOSE (UNIT=LIST)
      CALL PUTOUT('CLEAN completed; listing in '//
     1            LISFIL(1:LEN1(LISFIL)))
C-----------------------------------------------------------------------
      END

      SUBROUTINE DOCLEAN(MNDT,MAXIT,B,T,ICOMP,JCOMP,TCOMP,
     1                   VALS, NPARS, LIST, VERSN)
      INTEGER NPARS, LIST
      DOUBLE PRECISION VALS(NPARS)
      CHARACTER*(*) VERSN
C-----------------------------------------------------------------------
C       Clean for Hybrid Mapping
C       T.J.Pearson    VAX-11 Fortran
C-----------------------------------------------------------------------
      INTEGER IUNTM,IUNTC,MAXCB
      REAL*8  PI,RPDEG,RPSA,RPMAS,VELC,BOLTZ,FLOG2
      PARAMETER (IUNTM=2,IUNTC=4)
      PARAMETER (MAXCB=99)            ! size of clean beam array (odd)
      PARAMETER (PI=3.14159265359D0)
      PARAMETER (RPDEG=PI/180D0)
      PARAMETER (RPSA=RPDEG/3600D0)
      PARAMETER (RPMAS=RPSA/1000D0)
      PARAMETER (VELC=2.997925D8)     ! m/s
      PARAMETER (BOLTZ=1.38044D3)     ! Boltzmann constant, Jy m**2/K
      PARAMETER (FLOG2=0.693147181D0) ! ln 2
C
C     -- Maximum number of windows
      INTEGER MAXWIN
      PARAMETER (MAXWIN=20)
C
      INTEGER MAXIT           ! maximum number of iterations
      INTEGER MNDT            ! dimension of first subscript
C                               ! of map and beam arrays
C
      CHARACTER*64 FMAP,FBEAM,FMODEL
      CHARACTER*80 TITLE,FPLOT,MESSAGE,FCMAP,LEGEND,HISTRY
      CHARACTER*128 TEXT
      CHARACTER*20 XYUNIT
      CHARACTER*16 SNAME,BNAME,OBSERVER
      CHARACTER*8  FTYPE, DATMAP, DATOBS
      CHARACTER    ELAPSE*12
      CHARACTER*32 TELESC
      CHARACTER*23 STRING
      REAL EPOCH
      REAL FITBM(3)
      REAL*8 RA0, DEC0
      INTEGER*2 ICOMP,JCOMP
      INTEGER  PGBEGIN, BITPIX, NAXISN(4)
      LOGICAL  PRTMAP,PRTRES,PLTMAP,PLTWIN,NOREST,NONEG
      LOGICAL  HYBRID,SQUASH,PLTCOM,STOPNEG,PLTDEL
      DIMENSION B(MNDT,2*MNDT),T(MNDT,MNDT+2)
C                       ! +2 for smoothing!
      DIMENSION ICOMP(MAXIT),JCOMP(MAXIT),TCOMP(MAXIT)
      DIMENSION CB(MAXCB,MAXCB),VCN(20),VCNDEF(20)
      DIMENSION TR(6)
      DATA VCNDEF / -75., -65., -55., -45., -35., -25., -15., -5.,
     1              -2., 2., 5., 15., 25., 35., 45., 55., 65., 75.,
     2              85., 95. /
C
      INTEGER NWIN
      INTEGER MTLO(MAXWIN), MTHI(MAXWIN), NTLO(MAXWIN), NTHI(MAXWIN)
      REAL    WX1(MAXWIN), WX2(MAXWIN), WY1(MAXWIN), WY2(MAXWIN)
C-----------------------------------------------------------------------
      CALL SCLOCK     ! set clock
C
C Decode parameters.
C
      NIT = ABS(VALS(1))
      STOPNEG = VALS(1).LT.0D0
      TMIN = VALS(2)
      GAIN = VALS(6)
          IF (GAIN.GT.1.0 .OR. GAIN.LE.0.0) GAIN = 0.3
C
C Restoring beam: specified by BEAM = major,minor,pa.
C The clean beam is an elliptical Gaussian with FWHM
C axes 'major' and 'minor' (mas), with the major axis in
C position angle (N through E) 'pa' (deg). If BEAM is
C assigned only one value, it is the FWHM of a circular
C Gaussian.  The order of 'major' and 'minor' is allowed
C to be reversed.
C
      BW1 = VALS(3)
      BW2 = VALS(4)
      IF (BW2.LE.0.0) BW2 = BW1
      BMIN = MIN(BW1,BW2)
      BMAJ = MAX(BW1,BW2)
      BPA  = VALS(5)
C
C               Printing/Plotting switches
C
      PLTCOM = VALS(8).NE.-1D0        ! PLOTCOM
      PRTMAP = VALS(9).NE.-1D0        ! PRINTMAP
      PRTRES = VALS(10).NE.-1D0       ! PRINTRESIDUALS
      PLTMAP = VALS(11).NE.-1D0       ! PLOTMAP
      PLTWIN = VALS(12).NE.-1D0       ! PLOTWINDOW
      NOREST = VALS(13).NE.-1D0       ! NORESTORE
      PLTDEL = VALS(14).NE.-1D0
      IF (NOREST) PLTMAP = .FALSE.
      IF (NOREST) PRTMAP = .FALSE.
      NCP1   = VALS(7)                ! NCPRINT
      NCP2   = NIT-NCP1+1
      HYBRID = VALS(63).NE.-1D0       ! HYBRID
      SQUASH = VALS(66).EQ.-1D0       ! NOSQUASH
C
C Cleaning windows.
C
      NWIN = 0
      DO IWIN=1,MAXWIN
          IF (VALS(101 + (IWIN-1)*4).EQ.0D0 .AND.
     1        VALS(102 + (IWIN-1)*4).EQ.0D0 .AND.
     2        VALS(103 + (IWIN-1)*4).EQ.0D0 .AND.
     3        VALS(104 + (IWIN-1)*4).EQ.0D0 ) GOTO 57
          NWIN = NWIN+1
          WX1(NWIN) = VALS(101 + (IWIN-1)*4)
          WX2(NWIN) = VALS(102 + (IWIN-1)*4)
          WY1(NWIN) = VALS(103 + (IWIN-1)*4)
          WY2(NWIN) = VALS(104 + (IWIN-1)*4)
      END DO
   57 CONTINUE
      IF (NWIN.LT.1) CALL ERROR('No valid windows specified')
C
C Contours.
C
      NCONS = 0
      DO I=1,20
            IF (VALS(17+I).GE.1D37) GOTO 147
            NCONS = NCONS+1
            VCN(I) = VALS(17+I)
      END DO
  147 CONTINUE
      IF (NCONS.EQ.0) THEN
            NCONS = 20
            DO I=1,20
                VCN(I) = VCNDEF(I)
            END DO
      END IF
C
C Title and file names
C
      WRITE (TITLE,610)  (VALS(I),I=38,47)
      WRITE (FMAP,610)   (VALS(I),I=48,52)
      WRITE (FBEAM,610)  (VALS(I),I=53,57)
      WRITE (FMODEL,610) (VALS(I),I=58,62)
      WRITE (FPLOT,610)  (VALS(I),I=67,76)
      WRITE (FCMAP,610)  (VALS(I),I=77,86)
C
C ADDSOURCE parameter.
C
      ADDFL = VALS(87)
      ADDX  = VALS(88)
      ADDY  = VALS(89)
C
C If units were specified as ARCSEC, convert to milliarcsec.
C
      IF (VALS(64).NE.-1D0) THEN
            BMIN = BMIN*1000.0
            BMAJ = BMAJ*1000.0
            DO IWIN=1,NWIN
                WX1(IWIN) = WX1(IWIN)*1000.0
                WX2(IWIN) = WX2(IWIN)*1000.0
                WY1(IWIN) = WY1(IWIN)*1000.0
                WY2(IWIN) = WY2(IWIN)*1000.0
            END DO
      END IF
C
C Open output files.
C
      IF (FCMAP.NE.' ') THEN
          CALL FITS_INIT(FCMAP,FCMAP,JUNK,IER)
          IF (IER.NE.0) GOTO 340
      END IF
      OPEN (UNIT=IUNTC,NAME=FMODEL,STATUS='NEW',
     1      CARRIAGECONTROL='LIST',FORM='FORMATTED',ERR=330)
      INQUIRE(UNIT=IUNTC,NAME=FMODEL)
C
C Read Beam from file FBEAM
C
      CALL PUTOUT('CLEAN - reading files')
      OPEN (UNIT=IUNTM, NAME=FBEAM, STATUS='OLD', READONLY,
     1        FORM='UNFORMATTED',  ERR=310)
      INQUIRE(UNIT=IUNTM,NAME=FBEAM)
      READ(IUNTM) FTYPE,BNAME,XYINTB,ROTATB,FREQB,DXB,DYB,IDIMB,
     1              MBLO,MBHI,NBLO,NBHI,MBCENT,NBCENT
      IF (FTYPE.NE.'BEAM') CALL ERROR('File is not a beam: '//FBEAM)
      IF (MBHI.GT.MNDT .OR. NBHI.GT.2*MNDT) CALL ERROR(
     1          'Beam too big for array')
      DO 1010 N=NBLO,NBHI
 1010 READ(IUNTM) (B(M,N),M=MBLO,MBHI)
      CLOSE (UNIT=IUNTM)
      BMAX = B(MBCENT,NBCENT)
      WRITE(LIST,614) FBEAM,MBLO,MBHI,NBLO,NBHI,MBCENT,NBCENT
C
C Read Map from file FMAP.
C
      OPEN (UNIT=IUNTM,NAME=FMAP,STATUS='OLD',READONLY,
     1        FORM='UNFORMATTED',  ERR=320)
      INQUIRE (UNIT=IUNTM,NAME=FMAP)
      READ(IUNTM) FTYPE,SNAME,XYINT,ROTATE,FREQ,DX,DY,IDIMM,
     1              MLO,MHI,NLO,NHI,MCENT,NCENT,
     2              TELESC,DATOBS,EPOCH,RA0,DEC0,FITBM
      IF (FTYPE.NE.'MAP') CALL ERROR('File is not a map: '//FMAP)
      IF (MHI.GT.MNDT .OR. NHI.GT.MNDT) CALL ERROR(
     1          'Map too big for array')
      DO 1040 N=NLO,NHI
 1040 READ(IUNTM) (T(M,N),M=MLO,MHI)
      CLOSE (UNIT=IUNTM)
      XC = MCENT
      YC = NCENT
      WRITE(LIST,615) FMAP,SNAME,XYINT,ROTATE,MLO,MHI,NLO,NHI
      WRITE (LEGEND,'(A,F10.3,'' GHz'',F9.2)')
     1          TELESC(1:LEN1(TELESC)),FREQ*1E-9,EPOCH
      IF (XYINT.NE.XYINTB .OR. ROTATE.NE.ROTATB .OR. FREQ.NE.FREQB
     1     .OR. IDIMM.NE.IDIMB) CALL ERROR('Map and Beam don''t match!')
      MRANGE = MHI-MLO
      NRANGE = NHI-NLO
      IF ( (MBHI-MBCENT).LT.MRANGE .OR.
     1     (NBHI-NBCENT).LT.NRANGE .OR.
     2     (NBCENT-NBLO).LT.NRANGE ) CALL ERROR('Beam too small')
C
C               Input beam overrides map beam.
C
      IF (BMIN.EQ.0.0 .OR. BMAJ.EQ.0.0) THEN
          BMAJ = FITBM(1)
          BMIN = FITBM(2)
          BPA  = FITBM(3)
      END IF
C
C               Calculate Clean Beam (Gaussian) - array
C               CB(1:MAXCB,1:MAXCB) (not needed if NORESTORE requested).
C               The beam is normalized to total amplitude of unity.
C
      WAVE = VELC/FREQ        ! wavelength, m
      BMAREA = PI*BMIN*BMAJ*RPMAS*RPMAS/4.0/FLOG2
C                               ! beam area, sr
      BTEMP = (WAVE*WAVE)/(2.0*BOLTZ*BMAREA)*1E-8
C               ! peak temperature of a 1-Jy point source
C               ! convolved with clean beam (1E8 K)
C
      IF (.NOT.NOREST) THEN
            SIGU = FLOG2*(XYINT*2./BMIN)**2
            SIGV = FLOG2*(XYINT*2./BMAJ)**2
            PSIN = SIN((BPA+ROTATE)*RPDEG)
            PCOS = COS((BPA+ROTATE)*RPDEG)
            MCCB = MAXCB/2 + 1
            NCCB = MAXCB/2 + 1
Cx            BTOT = 0.0
            DO N=1,MAXCB
                Y = NCCB-N
                DO M=1,MAXCB
                    X = M-MCCB
                    XU = X*PCOS+Y*PSIN
                    YV = Y*PCOS-X*PSIN
                    ARG = -SIGU*XU*XU-SIGV*YV*YV
                    CB(M,N) = 0.0
                    IF (ARG.GT.-40.0) CB(M,N) = EXP(ARG)
                END DO
            END DO
      END IF
C
C Check windows lie inside map.
C
      WRITE (LIST, 629) NWIN
      DO IWIN=1,NWIN
          MTLO(IWIN) = MLO
          MTHI(IWIN) = MHI
          NTLO(IWIN) = NLO
          NTHI(IWIN) = NHI
          IF (WX1(IWIN).LT.1D30) MTLO(IWIN) =
     1                             INT(WX1(IWIN)/XYINT) + MCENT
          IF (WX2(IWIN).LT.1D30) MTHI(IWIN) =
     1                             INT(WX2(IWIN)/XYINT) + MCENT
          IF (WY1(IWIN).LT.1D30) NTLO(IWIN) =
     1                             NCENT - INT(WY1(IWIN)/XYINT)
          IF (WY2(IWIN).LT.1D30) NTHI(IWIN) =
     1                             NCENT - INT(WY2(IWIN)/XYINT)
          MTLO(IWIN) = MAX0(MLO,MTLO(IWIN))
          MTHI(IWIN) = MIN0(MHI,MTHI(IWIN))
          NTLO(IWIN) = MAX0(NLO,NTLO(IWIN))
          NTHI(IWIN) = MIN0(NHI,NTHI(IWIN))
          WX1(IWIN) = (MTLO(IWIN)-MCENT-0.5)*XYINT
          WX2(IWIN) = (MTHI(IWIN)-MCENT+0.5)*XYINT
          WY1(IWIN) = (NCENT-NTLO(IWIN)+0.5)*XYINT
          WY2(IWIN) = (NCENT-NTHI(IWIN)-0.5)*XYINT
          WRITE (LIST,630) WX1(IWIN), WX2(IWIN), WY1(IWIN), WY2(IWIN),
     1           MTLO(IWIN), MTHI(IWIN), NTLO(IWIN), NTHI(IWIN)
          IF (NTLO(IWIN).GE.NTHI(IWIN) .OR. MTLO(IWIN).GE.MTHI(IWIN))
     1           CALL ERROR('in LRTB values')
      END DO
      WRITE (LIST,631) NIT,TMIN,GAIN,BMAJ,BMIN,BPA,BTEMP*1E8,FREQ*1E-9
C
C               Open plot file if necessary
C
      IF (PRTRES.OR.PRTMAP.OR.PLTMAP.OR.PLTCOM.OR.PLTDEL) THEN
            IER = PGBEGIN(0,FPLOT,1,1)
            IF (IER.NE.1) THEN
                PRTRES = .FALSE.
                PRTMAP = .FALSE.
                PLTMAP = .FALSE.
                PLTCOM = .FALSE.
                PLTDEL = .FALSE.
            ELSE
                CALL PGSFS(2)
            END IF
      END IF
C
C               Prepare to start Clean loop
C
      FSUM = 0.0
      ITER = 0
      TCOMP(1) = 0.0
      NONEG = .TRUE.
      CALL PUTOUT('CLEAN - starting subtraction phase')
      CALL RCLOCK(ELAPSE,CPU1)
      WRITE(LIST,641)
C
C               Start Clean Loop: scan map for peak
C
   50 ABSMAX = 0.0
      ITP = 1
      JTP = 1
      DO 82 IWIN=1,NWIN
        DO 80 N=NTLO(IWIN),NTHI(IWIN)
          DO 80 M=MTLO(IWIN),MTHI(IWIN)
            TEMP = ABS(T(M,N))
            IF (TEMP.GT.ABSMAX) THEN
              ABSMAX = TEMP
              ITP = M
              JTP = N
            END IF
   80     CONTINUE
   82 CONTINUE
      IF (ABSMAX/BMAX.LE.TMIN) GOTO 110      ! converged!
      ITER = ITER+1
      TMAX = T(ITP,JTP)
      FACT = GAIN*TMAX/BMAX
C
C               If this is the first negative component, report the
C               iteration number. If negative NIT was requested, we
C               have gone one iteration too far.
C
      IF (NONEG.AND.FACT.LE.0.0) THEN
            NONEG = .FALSE.
            WRITE (LIST,662) ITER
            IF (STOPNEG) THEN
                ITER = ITER-1
                GOTO 110
            END IF
      END IF
C
C               Subtract component from map.
C
      MILO = ITP-1
      MM1 = MBCENT+ITP
      MM2 = MBCENT-ITP
      NN1 = NBCENT+JTP
      NN2 = NBCENT-JTP
      DO 100 N=NLO,NHI
          DO 92 M=MLO,MILO
              T(M,N) = T(M,N)-B(MM1-M,NN1-N)*FACT
   92     CONTINUE
          DO 96 M=ITP,MHI
              T(M,N) = T(M,N)-B(MM2+M,NN2+N)*FACT
   96     CONTINUE
  100 CONTINUE
C
C               Save information about component. If HYBRID set, and
C               if it is more negative than 10% of the peak temp. in the
C               whole dirty map, zero it. This is intended to help the
C               convergence of Hybrid Mapping, and should not be used
C               for normal cleaning.
C
      IF (HYBRID .AND. FACT.LT.-0.1*TCOMP(1)) FACT = 0.00
      TCOMP(ITER) = FACT      ! Jy
      ICOMP(ITER) = ITP
      JCOMP(ITER) = JTP
      FSUM = FSUM+FACT
      IF (ITER.LE.NCP1 .OR. ITER.GE.NCP2 .OR.
     1          MOD(ITER,50).EQ.0) THEN
            RITP = XC-ITP
            RJTP = YC-JTP
            RCOORD = XYINT*SQRT(RITP*RITP+RJTP*RJTP)
            PCOORD = 0.0
            IF (RITP.NE.0.0.OR.RJTP.NE.0.0) PCOORD =
     1          (ATAN2(RITP,RJTP))/RPDEG
            PCOORD = PCOORD-ROTATE
            IF (PCOORD.GT.180.) PCOORD = PCOORD-360.
            IF (PCOORD.LT.-180.) PCOORD = PCOORD+360.
            WRITE(LIST,660) ITER,ITP,JTP,FACT,RCOORD,PCOORD,FSUM
      END IF
      IF (MOD(ITER,50).EQ.0) THEN
            WRITE (MESSAGE,661) ITER,FSUM
            CALL PUTOUT(MESSAGE)
      END IF
      IF (ITER.LT.NIT) GO TO 50
C
C               End of clean loop
C
  110 CALL RCLOCK(ELAPSE,CPU2)
      IF (NONEG) WRITE(LIST,664)
      WRITE (LIST,670) FSUM
      CALL PUTOUT('CLEAN - subtraction phase completed')
C
C               Find largest residual and rms deviation of residuals.
C
      FSUM = 0.0
      FSSUM = 0.0
      TMAX = 0.0
      TMIN = 0.0
      DO N=NLO,NHI
          DO M=MLO,MHI
              FSUM = FSUM+T(M,N)
              FSSUM = FSSUM+T(M,N)**2
              TMAX = MAX(TMAX,T(M,N))
              TMIN = MIN(TMIN,T(M,N))
          END DO
      END DO
      NSUM = (NHI-NLO+1)*(MHI-MLO+1)
      FSSUM = SQRT(ABS(FSSUM/NSUM-(FSUM/NSUM)**2))
      TMAX = MAX(ABS(TMAX),ABS(TMIN))
      WRITE (LIST,680) TMAX,FSSUM
C
C               Plot convergence if requested (before squashing)!
C
      IF (PLTCOM) CALL COMPLT(ITER,TCOMP)
C
C               Squash model.
C
      IF (SQUASH) THEN
            DO I=2,ITER
            DO J=1,I-1
                IF (ICOMP(I).EQ.ICOMP(J).AND.JCOMP(I).EQ.JCOMP(J)) THEN
                    TCOMP(J) = TCOMP(J) + TCOMP(I)
                    TCOMP(I) = 0.0
                    GOTO 111
                END IF
            END DO
  111       CONTINUE
            END DO
      END IF
C
C               Open model output file FMODEL and write header comments.
C               Write components in the output model file, skipping
C               zeroed components.
C
      CALL MKHIST('CLEAN', VERSN, HISTRY)
      WRITE(IUNTC,730) HISTRY(1:LEN1(HISTRY))
      WRITE(IUNTC,730) TITLE(1:LEN1(TITLE))
      WRITE(IUNTC,740) SNAME(1:LEN1(SNAME))
      WRITE(IUNTC,741) TELESC(1:LEN1(TELESC))
      WRITE(IUNTC,742) DATOBS
      WRITE(IUNTC,743) EPOCH
      WRITE(IUNTC,750) FREQ*1E-6
      WRITE(IUNTC,760) BMAJ,BMIN,BPA
      WRITE(IUNTC,770) XYINT,ROTATE
      NCOMPS = 0
      DO I=1,ITER
            RITP = XC-ICOMP(I)
            RJTP = YC-JCOMP(I)
            RCOORD = XYINT*SQRT(RITP*RITP+RJTP*RJTP)
            PCOORD = 0.0
            IF (RITP.NE.0.0.OR.RJTP.NE.0.0) PCOORD =
     1          (ATAN2(RITP,RJTP))/RPDEG
            PCOORD = PCOORD-ROTATE
            IF (PCOORD.GT.180.) PCOORD = PCOORD-360.
            IF (PCOORD.LT.-180.) PCOORD = PCOORD+360.
            IF (TCOMP(I).NE.0.0) THEN
                WRITE(IUNTC,650) TCOMP(I),RCOORD,PCOORD
                NCOMPS = NCOMPS+1
            END IF
      END DO
      CLOSE (UNIT=IUNTC)
      IF (SQUASH) THEN
                WRITE(LIST,780) NCOMPS,'after squashing'
      ELSE
                WRITE(LIST,780) NCOMPS,'(model not squashed)'
      END IF
C
C Set up for plotting.
C
      X1 = (MLO-XC)*XYINT
      X2 = (MHI-XC)*XYINT
      Y1 = (YC-NLO)*XYINT
      Y2 = (YC-NHI)*XYINT
      IF (AMAX1(X2-X1,Y1-Y2).LT.1000.0) THEN
            XYUNIT = '(milliarcsec)'
            FACTOR = 1.0
      ELSE
            XYUNIT = '(arcsec)'
            FACTOR = 0.001
      END IF
C     -- plot window, world coordinates
      X1 = X1*FACTOR
      X2 = X2*FACTOR
      Y1 = Y1*FACTOR
      Y2 = Y2*FACTOR
C     -- transformation matrix for contouring
      TR(1) = XC*XYINT*FACTOR
      TR(2) = -XYINT*FACTOR
      TR(3) = 0.0
      TR(4) = YC*XYINT*FACTOR
      TR(5) = 0.0
      TR(6) = -XYINT*FACTOR
C     -- clean windows, world coordinates
      DO IWIN=1,NWIN
          WX1(IWIN) = -FACTOR*WX1(IWIN)
          WX2(IWIN) = -FACTOR*WX2(IWIN)
          WY1(IWIN) = FACTOR*WY1(IWIN)
          WY2(IWIN) = FACTOR*WY2(IWIN)
      END DO
C
C Plot residuals if requested.
C
      IF (PRTRES) THEN
          CALL PUTOUT('CLEAN - plotting residuals')
          CALL PGADVANCE
          CALL SETVP
          CALL PGWNAD(X2,X1,Y2,Y1)
          CALL PGSCI(1)
          CALL PGGRAY(T,MNDT,MNDT,MLO,MHI,NLO,NHI,TMAX,0.0,TR)
          CALL PGSCI(9)
          CALL ANNOT(XYUNIT, XYUNIT, '\fiResiduals', SNAME, LEGEND)
          IF (PLTWIN) THEN
              DO IWIN=1,NWIN
                  CALL PGRECT(WX1(IWIN),WX2(IWIN), WY1(IWIN),WY2(IWIN))
              END DO
          END IF
          CALL PGSCI(1)
      END IF
C
C Plot component positions if requested.
C
      IF (PLTDEL) THEN
          CALL PUTOUT('CLEAN - plotting deltas')
          CALL PGADVANCE
          CALL PGBBUF
          CALL SETVP
          CALL PGWNAD(X2,X1,Y2,Y1)
          CALL PGSCI(1)
          DO I=1,ITER
              X = TR(1) + TR(2)*ICOMP(I)
              Y = TR(4) + TR(6)*JCOMP(I)
              IF (TCOMP(I).GT.0.0) THEN
                  CALL PGPOINT(1, X, Y, 17)
              ELSE IF (TCOMP(I).LT.0.0) THEN
                  CALL PGPOINT(1, X, Y, 21)
              END IF
          END DO
          CALL PGSCI(9)
          CALL ANNOT(XYUNIT, XYUNIT, '\fiComponent Positions', SNAME,
     1               LEGEND)
          IF (PLTWIN) THEN
              DO IWIN=1,NWIN
                  CALL PGRECT(WX1(IWIN),WX2(IWIN), WY1(IWIN),WY2(IWIN))
              END DO
          END IF
          CALL PGSCI(1)
          CALL PGEBUF
      END IF
C
C Restoration phase.
C
      CALL RCLOCK(ELAPSE,CPU3)
      IF (.NOT.NOREST) THEN
          CALL PUTOUT('CLEAN - starting restoration phase')
C
C Smooth what's left.
C
          CALL SMTH(T,MNDT,MLO,MHI,NLO,NHI)
C
C Add components back into map.
C
          IF (ADDFL.NE.0.0) THEN
              ITER = ITER+1
              TCOMP(ITER) = ADDFL
              ICOMP(ITER) = XC
              JCOMP(ITER) = YC
          END IF
          IF (ITER.GT.0) THEN
              DO 125 I=1,ITER
                  FACT = TCOMP(I)
                  IF (FACT.EQ.0.0) GOTO 125
                  ITP = ICOMP(I)
                  JTP = JCOMP(I)
                  N1 = MAX0(JTP-NCCB+1,NLO)
                  N2 = MIN0(JTP+NCCB-1,NHI)
                  DO 120 N=N1,N2
                      NN = N+NCCB-JTP
                      M1 = MAX0(ITP-MCCB+1,MLO)
                      M2 = MIN0(ITP+MCCB-1,MHI)
                      DO 115 M=M1,M2
                          MM = M+MCCB-ITP
                          T(M,N) = T(M,N)+CB(MM,NN)*FACT
  115                 CONTINUE
  120             CONTINUE
  125         CONTINUE
          END IF
      END IF
      CALL RCLOCK(ELAPSE,CPU4)
C
C Find maximum.
C
      TMMAX = T(NLO,MLO)
      TMMIN = TMMAX
      DO N=NLO,NHI
            DO M=MLO,MHI
                TMMAX = MAX(TMMAX,T(M,N))
                TMMIN = MIN(TMMIN,T(M,N))
            END DO
      END DO
      WRITE (LIST,781) TMMAX,TMMIN
      TMAX = MAX(ABS(TMMAX),ABS(TMMIN))
C
C Write clean map to disk in FITS format.
C
      IF (FCMAP.EQ.' ') THEN
            FCMAP = 'not requested'
      ELSE
            CALL PUTOUT('CLEAN - writing clean map')
            BITPIX = 32
            NAXIS = 4
            NAXISN(1) = (MHI-MLO+1)
            NAXISN(2) = (NHI-NLO+1)
            NAXISN(3) = 1
            NAXISN(4) = 1
            BZERO = 0.0
            RMAX = 32000.
            IF (BITPIX.EQ.32) RMAX = 2E9
            BSCALE = TMAX/RMAX
            CALL USERNM(OBSERVER)
            CALL IDATE(IMON,ID,IY)
            WRITE (DATMAP,1000) ID,IMON,IY
 1000       FORMAT(I2.2,'/',I2.2,'/',I2.2)
            CALL FITS_HSTAN(BITPIX,NAXIS,NAXISN,0D0,0D0,IER)
            CALL FITS_WLOG ('EXTEND',.TRUE.,
     1                      'Clean components follow',IER)
            CALL FITS_WLOG ('BLOCKED',.TRUE.,
     1                      'Tape may be blocked',IER)
            CALL FITS_WREAL('BSCALE', BSCALE,
     1                      'Real = Tape*BSCALE + BZERO',IER)
            CALL FITS_WREAL('BZERO', BZERO, ' ', IER)
            CALL FITS_WSTR ('BUNIT',    'JY/BEAM',
     1                      'Brightness units', IER)
            CALL FITS_WSTR ('OBJECT', SNAME, 'Object name', IER)
            CALL FITS_WSTR ('INSTRUME', 'VLBI', ' ', IER)
            L = LEN1(TELESC)
            CALL FITS_WSTR ('TELESCOP', TELESC(1:L), ' ', IER)
            CALL FITS_WSTR ('DATE-OBS', DATOBS,
     1                      'Date of observation dd/mm/yy', IER)
            CALL FITS_WSTR ('DATE-MAP', DATMAP, 'Date map created', IER)
            CALL FITS_WSTR ('DATE',     DATMAP, 'Date map created', IER)
            CALL FITS_WSTR ('OBSERVER', OBSERVER, ' ', IER)
            CALL FITS_WSTR ('ORIGIN', 'CLEAN (Caltech)',
     1                      'Version '//VERSN, IER)
            CALL FITS_WREAL('DATAMAX', TMMAX, 'Map maximum (Jy/beam)',
     1                      IER)
            CALL FITS_WREAL('DATAMIN', TMMIN, 'Map minimum (Jy/beam)',
     1                      IER)
            CALL FITS_WREAL('EPOCH',   EPOCH, 'Epoch of RA/Dec', IER)
            CALL FITS_WREAL('BMAJ', BMAJ/3600000.,
     1                      'Beam major axis FHWM (deg)', IER)
            CALL FITS_WREAL('BMIN', BMIN/3600000.,
     1                      'Beam minor axis FHWM (deg)', IER)
            CALL FITS_WREAL('BPA', BPA+ROTATE,
     1                      'Beam major axis position angle (deg)', IER)
            CALL FITS_WINT('NITER', ITER,
     1                      'Number of CLEAN iterations', IER)
C
C               Axis 1 - RA
C
            PIX1 = XC-MLO+1
            DEL1 = -XYINT/3600E3
            STRING = 'RA  = '
            CALL PANGLE('H', RA0, STRING(6:23), 4)
            CALL FITS_WDBLE('CRVAL1',   RA0/RPDEG, STRING, IER)
            CALL FITS_WREAL('CRPIX1',   PIX1,      ' ', IER)
            CALL FITS_WSTR ('CTYPE1',   'RA---SIN',' ', IER)
            CALL FITS_WREAL('CDELT1',   DEL1,      ' ', IER)
            CALL FITS_WREAL('CROTA1',   ROTATE,    ' ', IER)
C
C               Axis 2 - DEC
C
            PIX2 = NHI-YC+1
            DEL2 = XYINT/3600E3
            STRING = 'Dec = '
            CALL PANGLE('D', DEC0, STRING(6:22), 3)
            CALL FITS_WDBLE('CRVAL2',   DEC0/RPDEG, STRING, IER)
            CALL FITS_WREAL('CRPIX2',   PIX2,      ' ', IER)
            CALL FITS_WSTR ('CTYPE2',   'DEC--SIN',' ', IER)
            CALL FITS_WREAL('CDELT2',   DEL2,      ' ', IER)
            CALL FITS_WREAL('CROTA2',   ROTATE,    ' ', IER)
C
C               Axis 3 - Frequency (degenerate).
C
            CALL FITS_WREAL('CRVAL3',   FREQ,      ' ', IER)
            CALL FITS_WREAL('CRPIX3',   1.0,       ' ', IER)
            CALL FITS_WSTR ('CTYPE3',   'FREQ',    ' ', IER)
            CALL FITS_WREAL('CDELT3',   0.0,       ' ', IER)
            CALL FITS_WREAL('CROTA3',   0.0,       ' ', IER)
C
C               Axis 4 - Stokes parameter (degenerate).
C
            CALL FITS_WREAL('CRVAL4',   1.0,       ' ', IER)
            CALL FITS_WREAL('CRPIX4',   1.0,       ' ', IER)
            CALL FITS_WSTR ('CTYPE4',   'STOKES',  ' ', IER)
            CALL FITS_WREAL('CDELT4',   1.0,       ' ', IER)
            CALL FITS_WREAL('CROTA4',   0.0,       ' ', IER)
C
C               History.
C
            DO I=1,LEN(MESSAGE)
                MESSAGE(I:I) = '-'
            END DO
            CALL FITS_WCMT('HISTORY', MESSAGE, IER)
            CALL FITS_WCMT('HISTORY', 'CLEAN version:   '//VERSN, IER)
            CALL FITS_HIS(IER)
            CALL FITS_WCMT('HISTORY', 'Dirty map file:  '//FMAP,  IER)
            CALL FITS_WCMT('HISTORY', 'Dirty beam file: '//FBEAM, IER)
            CALL FITS_WCMT('HISTORY', 'Clean map file:  '//FCMAP, IER)
            CALL FITS_WCMT('HISTORY', 'Model file:      '//FMODEL,IER)
C
C               Hidden parameters for AIPS.
C
            IF (NOREST) CALL FITS_WCMT('HISTORY',
     1               'AIPS PRODUCT=3   / Residuals', IER)
C
C               End FITS header; write FITS data, starting at bottom.
C
            CALL FITS_WEND(IER)
            DO N=NHI,NLO,-1
                CALL FITS_WRAYF(T(MLO,N),MHI-MLO+1,BITPIX,BSCALE,
     1                          BZERO,IER)
            END DO
C
C               Write clean components extension.
C
            CALL FITS_FLUSH(IER)
            CALL FITS_WSTR ('XTENSION','TABLE',' ',IER)
            CALL FITS_WINT ('BITPIX',8,'ASCII characters',IER)
            CALL FITS_WINT ('NAXIS',2,' ',IER)
            CALL FITS_WINT ('NAXIS1',48,
     1                  'Each entry is 48 characters wide',IER)
            CALL FITS_WINT ('NAXIS2',NCOMPS,'Number of entries',IER)
            CALL FITS_WINT ('PCOUNT',0,' ',IER)
            CALL FITS_WINT ('GCOUNT',1,' ',IER)
            CALL FITS_WINT ('TFIELDS',3,
     1                  'Number of fields in each entry',IER)
            CALL FITS_WSTR ('EXTNAME','AIPS CC',
     1                      'AIPS clean components',IER)
            CALL FITS_WINT ('EXTVER',1,' ',IER)
            CALL FITS_WINT ('TBCOL1',1,' ',IER)
            CALL FITS_WSTR ('TFORM1','E15.6',' ',IER)
            CALL FITS_WSTR ('TTYPE1','FLUX',' ',IER)
            CALL FITS_WSTR ('TUNIT1','JY',' ',IER)
            CALL FITS_WREAL('TSCAL1',1.0,' ',IER)
            CALL FITS_WREAL('TZERO1',0.0,' ',IER)
            CALL FITS_WINT ('TBCOL2',17,' ',IER)
            CALL FITS_WSTR ('TFORM2','E15.6',' ',IER)
            CALL FITS_WSTR ('TTYPE2','DELTAX',' ',IER)
            CALL FITS_WSTR ('TUNIT2','DEGREES',' ',IER)
            CALL FITS_WREAL('TSCAL2',1.0,' ',IER)
            CALL FITS_WREAL('TZERO2',0.0,' ',IER)
            CALL FITS_WINT ('TBCOL3',33,' ',IER)
            CALL FITS_WSTR ('TFORM3','E15.6',' ',IER)
            CALL FITS_WSTR ('TTYPE3','DELTAY',' ',IER)
            CALL FITS_WSTR ('TUNIT3','DEGREES',' ',IER)
            CALL FITS_WREAL('TSCAL3',1.0,' ',IER)
            CALL FITS_WREAL('TZERO3',0.0,' ',IER)
            CALL FITS_WEND (IER)
            DO I=1,ITER
                RITP = (ICOMP(I)-XC)*DEL1
                RJTP = (YC-JCOMP(I))*DEL2
                IF (TCOMP(I).NE.0.0) THEN
                    WRITE (MESSAGE,'(3(E15.6,1X))') TCOMP(I),RITP,RJTP
                    CALL FITS_WRAYB(%REF(MESSAGE),48,IER)
                END IF
            END DO
C
C               Close file.
C
            CALL FITS_CLOSE(IER)
            IF (IER.NE.0) CALL ERROR(
     1                'Something went wrong writing FITS file.')
      END IF
C
C               Plot clean map (contour/grayscale) if requested.
C
      IF (PLTMAP.OR.PRTMAP) THEN
          CALL PUTOUT('CLEAN - plotting clean map')
          DO ICON=1,NCONS
                VCN(ICON) = VCN(ICON)*TMAX*0.01
          END DO
          IF (PLTMAP) THEN
              CALL PGADVANCE
              CALL SETVP
              CALL PGWNAD(X2,X1,Y2,Y1)
              CALL PGSCI(1)
              CALL PGCONT(T,MNDT,MNDT,MLO,MHI,NLO,NHI,VCN,NCONS,TR)
              CALL PGSCI(9)
              CALL ANNOT(XYUNIT, XYUNIT, TITLE, SNAME, LEGEND)
              IF (PLTWIN) THEN
                  DO IWIN=1,NWIN
                      CALL PGRECT(WX1(IWIN),WX2(IWIN),
     1                            WY1(IWIN),WY2(IWIN))
                  END DO
              END IF
              CALL PGSCH(0.6)
              DY = 1.5
              Y = 9.0
              WRITE (TEXT,7330) TMAX, 'Jy/beam'
              CALL SSPACE(TEXT,L)              
              CALL PGMTEXT('B',Y,0.0,0.0,TEXT(1:L))
              N = MIN(10,NCONS)
              WRITE(TEXT,7340) (VCN(ICON)*100.0/TMAX,ICON=1,N)
              CALL SSPACE(TEXT,L)
              Y = Y+DY
              CALL PGMTEXT('B',Y,0.0,0.0,TEXT(1:L))
              IF (NCONS.GT.10) THEN
                  N = MIN(20,NCONS)
                  WRITE(TEXT,7340) (VCN(ICON)*100.0/TMAX,ICON=11,N)
                  CALL SSPACE(TEXT,L)
                  Y = Y+DY
                  CALL PGMTEXT('B',Y,0.0,0.0,TEXT(1:L))
              END IF
              Y = Y+DY
              CALL PGMTEXT('B',Y,0.0,0.0,HISTRY)
              CALL PGSCH(1.0)
              CALL PGSCI(1)
              WRITE (LIST,710) (VCN(ICON),ICON=1,NCONS)
              WRITE (LIST,711) (VCN(ICON)/TMAX*100.0, ICON=1,NCONS)
          END IF
          IF (PRTMAP) THEN
              CALL PGADVANCE
              CALL SETVP
              CALL PGWNAD(X2,X1,Y2,Y1)
              CALL PGSCI(1)
              CALL PGGRAY(T,MNDT,MNDT,MLO,MHI,NLO,NHI,1.3*TMAX,0.0,TR)
              CALL PGSCI(9)
              CALL ANNOT(XYUNIT, XYUNIT, TITLE, SNAME, LEGEND)
              IF (PLTWIN) THEN
                  DO IWIN=1,NWIN
                      CALL PGRECT(WX1(IWIN),WX2(IWIN),
     1                            WY1(IWIN),WY2(IWIN))
                  END DO
              END IF
              CALL PGSCI(1)
          END IF
      END IF
C
      IF (PLTMAP.OR.PRTMAP.OR.PRTRES.OR.PLTCOM) CALL PGEND
      CALL RCLOCK(ELAPSE,CPU)
      WRITE (LIST,'(//A/)') 'CLEAN execution summary:'
      WRITE (LIST,640) 'CLEAN version:', VERSN
      CALL REPORT(LIST)
      WRITE (LIST,640) 'Dirty map:',          FMAP(1:LEN1(FMAP))
      WRITE (LIST,640) 'Dirty beam:',         FBEAM(1:LEN1(FBEAM))
      WRITE (LIST,640) 'Clean map:',          FCMAP(1:LEN1(FCMAP))
      WRITE (LIST,640) 'Model file:',         FMODEL(1:LEN1(FMODEL))
      WRITE (LIST,640) 'Elapsed time:',       ELAPSE
      WRITE (LIST,642) 'Total cpu time:',     CPU
      WRITE (LIST,642) 'Subtraction phase:',  CPU2-CPU1
      WRITE (LIST,642) 'Restoration phase:',  CPU4-CPU3
      CALL PUTOUT('CLEAN completed')
      STOP
C
C               File open errors
C
  310 CALL ERROR('Can''t open BEAMFILE: '//FBEAM)
  320 CALL ERROR('Can''t open MAPFILE: '//FMAP)
  330 CALL ERROR('Can''t create file MODEL: '//FMODEL)
  340 CALL ERROR('Can''t create file CLEANMAP: '//FCMAP)
C-----------------------------------------------------------------------
C Format statements.
C-----------------------------------------------------------------------
  610 FORMAT(10A8)
  614 FORMAT(//'Dirty beam: ',A//
     1          6X,'Beam limits',3(I4,','),I4,' (gridpoints)'/
     2          6X,'Center at  ',I4,',',I4)
  615 FORMAT(//'Dirty map: ',A//
     1          6X,'Source:  ',A/
     2          6X,'X,Y-interval  ',F8.3,' milliarcsec'/
     3          6X,'Map rotated anticlockwise through  ',F6.1,' deg'/
     4          6X,'Map limits',3(I4,','),I4,'  (gridpoints)')
  629 FORMAT(//'CLEAN control parameters:'//
     1       6X,'Number of search windows:',I3)
  630 FORMAT(6X,' LRTB',3(F9.3,','),F9.3,'  (milliarcsec)'/
     1       6X,'   equivalent to ',3(I4,','),I4, '  (gridpoints)')
  631 FORMAT(/
     1      6X,'Max. no. of iterations [NITER]  ',I4/
     2      6X,'Temperature cut-off [TMIN]',F13.6,' Jy/beam'/
     3      6X,'Loop gain [LOOPGAIN]',8X,F8.3/
     4      6X,'Restoring beam FWHM ',F9.3,' x ',F9.3,' milliarcsec'/
     5      6X,'Position angle of long axis ',F6.1,' deg E of N'/
     6      6X,'Peak temp. of a 1-Jy point source'/
     7      12X,'(convolved with Clean Beam)  ',1PE13.4,' K'/
     8      6X,'Frequency ',1PE13.3,' GHz')
  640 FORMAT(A,T20,A)
  642 FORMAT(A,T20,F8.1,' seconds')
  641 FORMAT(//'Subtraction phase:'/
     1         /6X,'Iteration',4X,'I ',5X,'J ',12X,'Flux (Jy)',
     2         3X,'Radius',4X,'P.A.',3X,'Total (Jy)'/)
  650 FORMAT(1X,F11.6,2F10.3)
  660 FORMAT(10X,I4,2(4X,I3),9X,F11.6,2F10.3,F11.6)
  661 FORMAT('CLEAN - iteration',I5,',   total flux',F11.6)
  662 FORMAT(' *** First negative component at iteration',I5)
  664 FORMAT(//6X,'*** No negative components found ***')
  670 FORMAT(//6X,'Sum of components = ',F12.6,' Jy')
  680 FORMAT(  6X,'Maximum residual =  ',F12.6,' Jy/beam'/
     1         6X,'rms of residuals =  ',F12.6,' Jy/beam')
  710 FORMAT(/'Contour plot of clean map:'//
     1        'Contour levels in Jy/beam'//(5F12.6))
  711 FORMAT(/'Contour levels in percent'//(5F7.2))
  730 FORMAT('!  ',A)
  740 FORMAT('!  SOURCE = ',A)
  741 FORMAT('!  TELESCOPE = ',A)
  742 FORMAT('!  DATE-OBS = ',A)
  743 FORMAT('!  EPOCH = ',F10.2)
  750 FORMAT('!  FREQUENCY = ',F13.6)
  760 FORMAT('!  BEAM = ',F8.3,',',F8.3,',',F7.2)
  770 FORMAT('!  XYINT = ',F8.3,'  ROTATE = ',F7.2)
  780 FORMAT(6X,'Model consists of',I6,' non-zero components ',A/)
  781 FORMAT(/6X,'Maximum of clean map = ',F12.6,' Jy/beam'/
     1        6X,'Minimum of clean map = ',F12.6,' Jy/beam')
 7330 FORMAT('Maximum: ', 1PG12.4,' ',A)
 7340 FORMAT('Contours (%): ',10F7.2)
C-----------------------------------------------------------------------
      END

C*SMTH -- smooth array (used by CLEAN)
C+
      SUBROUTINE SMTH(T,MDIM,MLO,MHI,NLO,NHI)
      INTEGER MDIM, MLO, MHI, NLO, NHI
      REAL    T(MDIM,*)
C
C SMTH smoothes array T by convolving with (fixed)
C array SM. +WARNING+ it needs two spare rows at NHI+1, NHI+2!
C-----------------------------------------------------------------------
      INTEGER I, J, N, M, NT, NN, MM
      REAl SM(3,3)
      DATA SM/0.0625,0.125,0.0625,0.125,0.25,0.125,0.0625,0.125,0.0625/
C
C Shift data in array.
C
      NT=NHI+NLO
      DO 20 N=NLO,NHI
          NN=NT-N
          DO 20 M=MLO,MHI
   20         T(M,NN+2)=T(M,NN)
C
C Smooth array
C
      DO 60 N=NLO,NHI
          DO 60 M=MLO,MHI
              SUM=0.0
              DO 40 J=1,3
                  NN=N+J-2
                  NN=MAX0(NN,NLO)
                  NN=MIN0(NN,NHI)
                  DO 40 I=1,3
                      MM=M+I-2
                      MM=MAX0(MM,MLO)
                      MM=MIN0(MM,MHI)
   40                 SUM=SUM+SM(I,J)*T(MM,NN+2)
   60     T(M,N)=SUM
      RETURN
      END

      SUBROUTINE COMPLT(N,T)
C-----------------------------------------------------------------------
C CLEAN: plot convergence (component amplitude v. iteration).
C
C Arguments:
C
C N (input, integer): number of components.
C T (input, real array, dimension at least N): component fluxes.
C-----------------------------------------------------------------------
      IMPLICIT NONE
      INTEGER I,N
      REAL    T(*),TMAX,TMIN,F,X,Y
C
      F(X) = SIGN(1.0,X)*ALOG10(ABS(X)*10.0/TMIN)
C
      IF (N.LT.2) RETURN
      TMIN = ABS(T(1))
      TMAX = ABS(T(1))
      DO I=2,N
            IF (T(I).NE.0.) TMIN = MIN(TMIN,ABS(T(I)))
            TMAX = MAX(TMAX,ABS(T(I)))
      END DO
      IF (TMAX.LE.0.0) RETURN
      TMAX = F(TMAX)
      CALL PGENV(0.,FLOAT(N),-TMAX,TMAX,0,1)
      CALL PGLABEL('Number of iteration',
     1          'S*log[S*flux/(0.1*fluxmin)] [where S=sign(flux)]',
     2          ' ')
      CALL PGMOVE(1.0,0.0)
      DO I=1,N
            Y = 0.0
            IF (T(I).NE.0.0) Y = F(T(I))
            CALL PGDRAW(FLOAT(I),Y)
      END DO
      RETURN
      END

      SUBROUTINE SETVP
C
C Set viewport for PGPLOT.
C
      REAL VPX1, VPX2, VPY1, VPY2, D
C
      CALL PGVPORT(0.0, 1.0, 0.0, 1.0)
      CALL PGQVP(1, VPX1, VPX2, VPY1, VPY2)
      D = MIN(VPX2-VPX1, VPY2-VPY1)/40.0
      VPX1 = VPX1 + 5.0*D
      VPX2 = VPX2 - 2.0*D
      VPY1 = VPY1 + 9.0*D
      VPY2 = VPY2 - 4.0*D
      CALL PGVSIZE(VPX1, VPX2, VPY1, VPY2)
      RETURN
      END

      SUBROUTINE ANNOT(XUNIT, YUNIT, TITLE, SNAME, LEGEND)
      CHARACTER*(*) XUNIT, YUNIT, TITLE, SNAME, LEGEND
C
      CALL PGBOX('BCNST', 0.0, 0, 'BCNSTV', 0.0, 0)
      CALL PGMTEXT('B', 3.0, 0.5, 0.5, 'x '//XUNIT)
      CALL PGMTEXT('L', 3.0, 0.5, 0.5, 'y '//YUNIT)
      CALL PGMTEXT('T',2.5,0.0,0.0,TITLE)
      CALL PGMTEXT('T',0.8,0.0,0.0,SNAME)
      CALL PGMTEXT('T',0.8,1.0,1.0,LEGEND)
      RETURN
      END
