c
c  a group of subroutines to transform vax numbers to convex
c  numbers and vice versa.  in every case, the result replaces
c  the argument.  there are two arguments in each subroutine;
c  the first is the address of the first value to be converted,
c  the second is the number of contiguous values in the array.
c
c  written by Bill Meredith, NRAO, Apr 1986
c ___________________________________________________________________________
c
c  converts vax r*8 to convex r*8
c
c  first:   the bytes in each halfword are swapped.
c  second:  the exponent is masked out.
c  third:   896 (decimal) is added to the exponent;
c           the new (convex) exponent is thus excess 1024, rather than 128.
c  fourth:  the new exponent is shifted right 3 bits to become an 11 bit exp.
c  fifth:   the vax 23 bit mantissa is masked out, shifted right 2 bits,
c           one added to it for rounding, then shifted right one more bit,
c           then exponent and mantissa are merged.
c  sixth:   if the argument was negative, the result is negated.
c  seventh: the result replaces the argument.
c
      subroutine v8tc8 (v8d8,n)
c
      r e a l*8 v8d8(1),v8r8
      integer*8 v8i8,v8iw8
      logical*8 vaxmsk/'7f80000000000000'x/
      logical*8 vaxman/'007fffffffffffff'x/
      logical*8 v8l8,v8lw8,signbt/'0080000000000000'x/,sign
      integer*2 v8i2(4)
c
      equivalence (v8lw8,v8iw8),(v8r8,v8l8,v8i8,v8i2(1))
c
      do i   = 1,n
      v8r8   = v8d8(i)
      sign   = .false.
      if (v8r8 .ne. '0000000000000000'x)  then
        sign = v8l8 .and. signbt
        do j = 1,4
        call bswap (v8i2(j))
        enddo
      v8lw8  = v8l8 .and. vaxmsk
      v8iw8  = v8iw8 / 8 + '3800000000000000'x
      v8l8   = v8l8 .and. vaxman
      v8i8   = v8iw8 + ((v8i8 / 4 + 1) / 2)
               if (sign)  v8r8 = -v8r8
      endif
      v8d8(i) = v8r8
      enddo
      r e t u r n
      e  n  d
c
c  _______________________________________________________________________
c  converts convex r*8 to vax r*8.
c  convex exponents > 255 will not work for vax.
c  if convex exponent is > 255, vax infinity  (+ or -) is returned
c
c  first:   the convex exponent is masked out.
c  second:  convert 11 bit convex exponent to 8 bit vax exponent.
c  third:   mask out the convex mantissa.
c  fourth:  shift mantissa left 3 bits and merge with exponent.
c  fifth:   swap bytes in each halfword.
c  sixth:   replace argument with result.
c
      subroutine c8tv8 (c8d8,n)
c
      r e a l*8 c8r8,c8d8(1)
      integer*8 r8i8,c8iw8
      logical*8 conexp/'7ff0000000000000'x/
      logical*8 conman/'000fffffffffffff'x/
      logical*8 c8l8,c8lw8
      integer*2 c8i2(4)
c
      equivalence (c8lw8,c8iw8),(c8r8,c8l8,r8i8,c8i2(1))
      do i   = 1,n
      c8r8   = c8d8(i)
        if (c8r8 .ne. '0000000000000000'x)  then
        c8lw8  = c8l8 .and. conexp
               if (c8iw8 .lt. '4800000000000000'x)  then
               c8iw8  = (c8iw8 - '3800000000000000'x) * 8
               c8l8   = c8l8 .and. conman
               r8i8   = c8iw8 + 8 * r8i8
               else
               c8r8   = '7fffffffffffffff'x
               endif
               if (c8d8(i) .lt. 0.0d0)  c8r8 = -c8r8
          do j = 1,4
          call bswap (c8i2(j))
          enddo
        endif
      c8d8(i) = c8r8
      enddo
      r e t u r n
      e  n  d
c
c  _______________________________________________________________________
c  subroutine to convert i*4 from vax to convex and vice versa
c
c  first:   the two halfwords are swapped.
c  second:  the bytes in each halfword are swapped.
c  third:   the result replaces the argument.
c
      subroutine i4ti4 (i4i4,n)
c
      integer*4 i4i4(1),i4j4
      integer*2 i4i2(2)
      equivalence (i4j4,i4i2(1))
c
        do i = 1,n
        i4j4   = i4i4(i)
        call wswap (i4j4)
        call bswap (i4i2(1))
        call bswap (i4i2(2))
        i4i4(i) = i4j4
        enddo
c
      r e t u r n
      e  n  d
c
c  ________________________________________________________________________
c  subroutine to convert convex r*4 to vax r*4 and vice versa
c
c  first:   the bytes in each halfword are swapped.
c  second:  the result replaces the argument.
c
      subroutine r4tr4 (r4r4,n)
c
      real*4 r4r4(1),r4c4
      integer*2 r4i2(2)
c
      equivalence (r4c4,r4i2(1))
c
        do i = 1,n
        r4c4   = r4r4(i)
        call bswap (r4i2(1))
        call bswap (r4i2(2))
c
        r4r4(i) = r4c4
        enddo
c
      r e t u r n
      e  n  d
c
c  ______________________________________________________________________
c
c  converts vax integer*2 words to convex integer*2 words, and vice versa.
c
      subroutine i2ti2 (i2i2,n)
      integer*2 i2i2(1)
      do i=1,n
      call bswap (i2i2(i))
      enddo
      r e t u r n
      e  n  d
c
c  _______________________________________________________________________
c  swaps bytes in an integer*2 word
c
      subroutine bswap (bsi2)
c
      integer*2 bsi2
c
      bsi2 = iishftc(bsi2,8,16)
c
      r e t u r n
      e  n  d
c
c  _______________________________________________________________________
c  swaps halfwords in an integer*4 word
c
      subroutine wswap (wsi4)
c
      integer*4 wsi4
c
      wsi4 = jishftc(wsi4,16,32)
c
      r e t u r n
c
      e  n  d
c
c  ===================================================================
      subroutine i4tc4 (i4r4,n)
c  ===================================================================
c
converts ibm real*4 to convex real*4
c
      real*4    i4r4(1),i4f4
      integer*4 i4man/'00ffffff'x/,i4exp/'7f000000'x/
      integer*4 i4hid/'01000000'x/,i4sgn/'80000000'x/
      integer*4 i4msk/'007fffff'x/
c
      equivalence (i4i4,i4f4)
c
      do i = 1,n
        if (i4r4(i) .ne. 0.0)  then
          i4f4 = i4r4(i)
          i4norm = 0
          i4dum = jiand (i4i4, i4man)
          dowhile (bjtest (i4dum, 24) .eq. 0)
            i4dum = jishft (i4dum, 1)
            i4norm = i4norm + 1
          enddo
          i4i4 = jior (jiand (i4i4, i4sgn), jior (jiand (i4dum, i4msk)
     +           , jishft (jishft (jishft (jiand (i4i4,i4exp), -24) -
     +           64, 2) + 129 - i4norm, 23)))
          i4r4(i) = i4f4
        endif
      enddo
      r e t u r n
      e  n  d
