C CORDEF
C Block-2 Correlator Fundamental Definitions
C Thomas G. Lockhart, JPL m/s 138-208, Pasadena CA 91109
C written February 20, 1986
C modified September 19, 1988
C - define xSIT as parameters for logical stations (sites)
C - define xCAL as parameters for logical tone extractors
C modified July 6, 1988
C - reduce MXCYCVRC to 14 from 16 to match change in VTABLES.NTH;526
C (this requires a corresponding change in PCRDEF)

C Dimensions and limits defined in the block-2 correlator FORTH code.
C
C The quantities in this section should not be referenced in FORTRAN
C programs, but are used to derive other parameters within this module.

	INTEGER MXGROUPS, MXSTATNS
	INTEGER MXLVRECS, MXPVRECS, MXCYCVRC
	INTEGER MXTMODEL, MXTNCOEF, MXDMODEL, MXPMODEL
	INTEGER MXPRCHAN, MXTTCHAN, MXPTCHAN
	INTEGER MXFTBINS, MXFQBAND, MXWINDOW, MXTDPNTS
	INTEGER MXTNSBUF, MXPCRTBL

	PARAMETER (MXGROUPS = 2)	! # processor groups
	PARAMETER (MXSTATNS = 4)	! # mark-3 stations
	PARAMETER (MXLVRECS = 16)	! # logical mark-2 streams
	PARAMETER (MXPVRECS = 18)	! # physical mark-2 VCRs
	PARAMETER (MXCYCVRC = 14)	! # mark-2 time mux cycles

	PARAMETER (MXTMODEL = 112)	! # tone models
	PARAMETER (MXTNCOEF = 9)	! # tone model polynomial coeff's
	PARAMETER (MXDMODEL = 28)	! # delay models
	PARAMETER (MXPMODEL = 28)	! # phase models

	PARAMETER (MXPRCHAN = 28)	! # processor data channels
	PARAMETER (MXTTCHAN = 4)	! # tone extractors per channel
	PARAMETER (MXPTCHAN = (2**8))	! # points (lags) per channels

	PARAMETER (MXTDPNTS = (2**15))	! # TENSOR data points
	PARAMETER (MXFTBINS = (2**11))	! FFT size limit
	PARAMETER (MXFQBAND = 4)	! # frequency (channel) bands
	PARAMETER (MXWINDOW = 8)	! # TENSOR output windows

	PARAMETER (MXTNSBUF = 255)		! # TENSOR output buffers
	PARAMETER (MXPCRTBL = (2**16/2-1))	! PCR record size limit

C The following constants should be used to define the size of data
C areas in the PCR tables, internal program storage, and loop indices.

	INTEGER MINGRP, MAXGRP, NULGRP
	INTEGER MINSTN, MAXSTN, NULSTN
	INTEGER MINCHN, MAXCHN, NULCHN
	INTEGER MINTEX, MAXTEX, NULTEX

	INTEGER MINLAG, MAXLAG, NULLAG
	INTEGER MINBIN, MAXBIN, NULBIN

	INTEGER MINDEV, MAXDEV, NULDEV
	INTEGER MINTRK, MAXTRK, NULTRK
	INTEGER MINPAS, MAXPAS, NULPAS

	INTEGER MINVCR, MAXVCR, NULVCR
	INTEGER MINCYC, MAXCYC, NULCYC

	INTEGER MINBAS, MAXBAS, NULBAS
	INTEGER MINSIT, MAXSIT, NULSIT
	INTEGER MINCAL, MAXCAL, NULCAL

	INTEGER MINMOD, MAXMOD
	INTEGER MINTME, MAXTME
	INTEGER MINGME, MAXGME
                   
	INTEGER MINTON, MAXTON, NULTON
	INTEGER MINDLY, MAXDLY, NULDLY
	INTEGER MINPHS, MAXPHS, NULPHS

	INTEGER MINTMC, MAXTMC, NULTMC
	INTEGER MINCLK, MAXCLK, NULCLK
	INTEGER MINAXS, MAXAXS, NULAXS

	INTEGER MINMNT, MAXMNT, NULMNT

	INTEGER MINTCH, MAXTCH, NULTCH
	INTEGER MINBND, MAXBND, NULBND
	INTEGER MINWND, MAXWND, NULWND
	INTEGER MINTDP, MAXTDP, NULTDP

	INTEGER MINPCR, MAXPCR, NULPCR

C H/W configuration
	PARAMETER (MINGRP = 1)
	PARAMETER (MAXGRP = MXGROUPS)
	PARAMETER (NULGRP = 0)
	PARAMETER (MINSTN = 1)
	PARAMETER (MAXSTN = MXSTATNS)	! # processor stations
	PARAMETER (NULSTN = 0)
	PARAMETER (MINCHN = 1)
	PARAMETER (MAXCHN = MXPRCHAN)	! # processor channels
	PARAMETER (NULCHN = 0)
	PARAMETER (MINTEX = 1)
	PARAMETER (MAXTEX = MXTTCHAN)	! # tone extractors per channel
	PARAMETER (NULTEX = 0)

	PARAMETER (MINLAG = 1)
	PARAMETER (MAXLAG = MXPTCHAN)	! # lags per channel
	PARAMETER (NULLAG = 0)
	PARAMETER (MINBIN = 1)
	PARAMETER (MAXBIN = MAXLAG/2)	! # transform bins
	PARAMETER (NULBIN = 0)
C mark-3 interface constants
	PARAMETER (MINDEV = 1)
	PARAMETER (MAXDEV = MXSTATNS)
	PARAMETER (NULDEV = 0)
	PARAMETER (MINTRK = 1)
	PARAMETER (MAXTRK = 31)
	PARAMETER (NULTRK = 0)
	PARAMETER (MINPAS = 1)
	PARAMETER (MAXPAS = 20)
	PARAMETER (NULPAS = 0)
C mark-2 interface constants
	PARAMETER (MINVCR = 1)
	PARAMETER (MAXVCR = MXLVRECS)	! # video recorders
	PARAMETER (NULVCR = 0)
	PARAMETER (MINCYC = 1)
	PARAMETER (MAXCYC = MXCYCVRC)	! # time multiplexed channels
	PARAMETER (NULCYC = 0)
C logical entities
	PARAMETER (MINBAS = 1)		! # baselines
	PARAMETER (MAXBAS = ((MAXSTN-1)*MAXSTN/2))
	PARAMETER (NULBAS = 0)
	PARAMETER (MINSIT = 1)		! logical sites
	PARAMETER (MAXSIT = MXLVRECS)
	PARAMETER (NULSIT = 0)
	PARAMETER (MINCAL = 1)		! logical phase calibrators
	PARAMETER (MAXCAL = MAXCHN*MAXTEX)
	PARAMETER (NULCAL = 0)
C table dimensions
	PARAMETER (MINMOD = 1)		! # values per entry
	PARAMETER (MAXMOD = 4)
	PARAMETER (MINTME = 1)		! # tone model entries
	PARAMETER (MAXTME = (MAXSTN*MAXTEX*MAXCHN))
	PARAMETER (MINGME = 1)		! # geo. model entries
	PARAMETER (MAXGME = (MAXSTN*MAXCHN))
C configuration table dimensions
	PARAMETER (MINTON = 1)
	PARAMETER (MAXTON = MXTMODEL)	! tone models
	PARAMETER (NULTON = 0)
	PARAMETER (MINDLY = 1)
	PARAMETER (MAXDLY = MXDMODEL)	! delay models
	PARAMETER (NULDLY = 0)
	PARAMETER (MINPHS = 1)
	PARAMETER (MAXPHS = MXPMODEL)	! phase models
	PARAMETER (NULPHS = 0)
C table sub-dimensions
	PARAMETER (MAXTMC = MXTNCOEF)	! tone model coefficients
	PARAMETER (MINCLK = 0)
	PARAMETER (MAXCLK = 1)
	PARAMETER (MINAXS = 1)
	PARAMETER (MAXAXS = 2)
C table contents
	PARAMETER (MINMNT = 1)
	PARAMETER (MAXMNT = 5)
	PARAMETER (NULMNT = 0)
C TENSOR parameters (per baseline)
	PARAMETER (MINTCH = 1)
	PARAMETER (MAXTCH = 28)		! # logical channels
	PARAMETER (NULTCH = 0)
	PARAMETER (MINBND = 1)
	PARAMETER (MAXBND = 4)		! # frequency bands
	PARAMETER (NULBND = 0)
	PARAMETER (MINWND = 1)
	PARAMETER (MAXWND = 8)		! # segment windows
	PARAMETER (NULWND = 0)
	PARAMETER (MINTDP = 1)
	PARAMETER (MAXTDP = MXTDPNTS)	! # tensor data points
	PARAMETER (NULTDP = 0)
C table parameters
	PARAMETER (MINPCR = 1)
	PARAMETER (MAXPCR = MXPCRTBL)	! PCR record size limit
	PARAMETER (NULPCR = 0)
