      PROGRAM ERRFIT
C-----------------------------------------------------------------------
C Program ERRFIT          June 1985
C
C Written by G. Comoretto
C
C This program, derived from the program QFIT, tries to estimate
C the formal errors associated to model parameters.
C It asks interactively for the parameters to be tested,
C and for a reasonable assumption about the degrees of freedom
C in the data set
C 
C Then the error matrix is computed on these parameters.
C The algorithm is based on a linear approx. to the chi-squared
C function. The second derivatives are evaluated for parameter
C variations close to the 1-sigma level.
C
C The error matrix is scaled using the given number of independent
C data points, and enlarging the error bars to obtain chi-squared=1.
C (measuring the unknown errors from the chi-squared)
C
C Version  1:     may 27,  1985
C Version  2:     Apr 8, 1993 (Minor reformatting for Unix; TJP).
C-----------------------------------------------------------------------
C Common blocks, etc. are contained in a separate file:
      INCLUDE 'errfit.inc'
C
      CHARACTER*6 ANSWER(MXMOD)
C
      WRITE (OUTC,10000) ' Program ERRFIT               June  3, 1985',
     1                   '                G. Comoretto'
      WRITE (OUTC,10000) ' '
      WRITE (OUTC,10000) ' ERRFIT finds the errors for the ',
     1                   'fit of an optimized model to VLB data.'
      WRITE (OUTC,10000) ' '
C
C Get the file names for data, model, and output:
C
   10 WRITE (OUTC,10001) ' Input data file name: '
      READ  (INC, 10000, ERR=99) INDSN
      OPEN  (UNIT=INDAT,NAME=INDSN,STATUS='OLD',READONLY,
     1       FORM='UNFORMATTED',ERR=10)
C
      WRITE (OUTC,10000) ' '
   20 WRITE (OUTC,10001) ' Input model file name: '
      READ  (INC, 10000, ERR=99) MODDSN
      OPEN  (UNIT=INMOD,NAME=MODDSN,STATUS='OLD',READONLY,
     1       FORM='FORMATTED',ERR=20)
C
      WRITE (OUTC,10000) ' '
   30 WRITE (OUTC,10001) ' Listing file name (default=stdout): '
      READ  (INC, 10000, ERR=99) OUTDSN
      IF (OUTDSN .EQ. ' ') THEN
          OUTPR = OUTC
      ELSE
          OUTPR = PR
          OPEN  (UNIT=OUTPR,NAME=OUTDSN,STATUS='NEW',
     1           FORM='FORMATTED',ERR=30)
      ENDIF
C
C Get the input data and compute the arrays UDATA and VDATA:
C
      CALL QGETDAT
C
C Determine which triangles are independent for each data record:
C
      CALL TRIANGLE
C
C Compute the non-zero data points and the weights for the data.
C
      CALL WTDATA
C
C Read in the model file:
C
      CALL QGETMOD
C
C Compute the model amplitudes and closure phases:
C
      CALL QMODAP
C
C Compute the agreement factors:
C
      CALL AGRFAC
      WRITE (OUTC,*) 'Initial agreement factor =', TOTLAF
      AMPLA0 = AMPLAF
      PHASA0 = PHASAF
      TOTLA0 = TOTLAF
C
C Ask for degrees of freedom:
C
  100 WRITE (OUTC,10010)
      READ (INC,*,ERR=100) NU0
      IF (NU0.LE.0.OR.NU0.GT.100000) GO TO 100
C
C Parameters are varied at a 2 sigma confidence level:
C
      CONFID = 2.0
      DELTAF = (CONFID**2)*0.5/NU0
      NUMPAR = 0
C
C Asks for parameters to be varied:
C
      WRITE (OUTC,10020)
      DO I=1,NCOMP
  210     WRITE (OUTC,10025) I,FLUX(I),RADIUS(I),THETA(I),
     +                       AXIS(I),RATIO(I),PHI(I),TYPE(I)
          READ (INC,10000,ERR=210,END=300) ANSWER(I)
      END DO
  300 DO NMOD=1,NCOMP
          DO NPAR1=1,6
              NPAR=ICHAR(ANSWER(NMOD)(NPAR1:NPAR1))-ICHAR('0')
              IF (NPAR.GE.1.AND.NPAR.LE.6) THEN
                  DELTAP=0.1
                  CALL DEALPR (NMOD,NPAR,DELTAP,IERR)
                  IF (IERR.GT.0) WRITE (OUTPR,10027)NPAR,NMOD
              END IF
          END DO
      END DO
C
C Normalization constant:
C Errors scale as agreement factor of input model (worse model=larger
C errors) and as 1/sqrt(correlation factor) (# of data points per
C degree of freedom) (less degrees of freedom=larger errors).
C
      ANORM = FLOAT(NU0)/((NZAMP+TNZCLO) * TOTLA0 * TOTLA0)
C
C Calculate error matrix
C
      DO IX=1,NUMPAR
          DO IY=IX,NUMPAR
              SUM = 0.0
              DO IR=1,NUMREC
                  DO IB=1,NUMBAS
                      IF (WTVIS(IR,IB).GT.0.0)
     +                    SUM = SUM + WTVIS(IR,IB) *
     +                        VISDEL(IR,IB,IX) * VISDEL(IR,IB,IY)
                  END DO
                  DO IT=1,NUMTRI
                      IF (WTCLO(IR,IB).GT.0.0)
     +                    SUM = SUM + WTCLO(IR,IB) * DOWNWT(IR) *
     +                        CLODEL(IR,IB,IX) * CLODEL(IR,IB,IY)
                  END DO
              END DO
              SUM=SUM*ANORM
              AMAT(IX,IY)=SUM
              IF (IX.NE.IY) AMAT (IY,IX)=SUM
          END DO
      END DO
      CALL MATINV (AMAT,MXPAR,NUMPAR,DET)
      IF (DET.EQ.0.0) THEN
            WRITE (OUTC,10030)
      END IF
      CALL LISERR
   99 STOP
C
10000 FORMAT (A,A,A,A,A)
10001 FORMAT (A,$)
10010 FORMAT (' Now enter an estimate for the number of',
     +        ' independent data points: ',$)
10020 FORMAT (' Enter parameters to be varied fore each component'/
     +        ' Example: to vary flux and axial ratio, enter ''15'' '/
     +        ' ',12X,'Flux',6X,'Radius  Theta',7X,'Axis',6X,
     +        'Ratio     Phi    Type')
10025 FORMAT (' Comp.',I2,4F10.3,F10.4,F10.3,I3,' : ',$)
10027 FORMAT (' I have problems with parameter',I3,
     +        ' in component',I3,': errors not calculated')
10030 FORMAT (' Error matrix singlar:',
     +        ' Some parameter is not constrained by data.',
     +        ' Try a model wth fewer free parameters. Good bye')
C
      END

	subroutine liserr
C
C OUTPUT SUBROUTINE, THAT DISPLAYS EVERYTHING THERE IS TO BE DISPLAYED
C
      	include 'errfit.inc'
	character*9 errpri(mxmod,6)
	character*2 achar(6)
	real modpar
	dimension modpar(mxmod,6)
	DIMENSION CORMAT(MXPAR,MXPAR),sigma(mxpar)
	equivalence (modpar,flux)
	data achar/'Fl','Rd','Th','Ax','Rt','Ph'/
	if (outpr.ne.outc) then
	   inquire (file=outdsn,name=outdsn)
	   write (outc,1000)'0Results to file ',outdsn(1:50)
	   write (outpr,1000) ' This is an output file from program ERRFIT.'
	end if
	inquire (file=indsn,name=indsn)
	inquire (file=moddsn,name=moddsn)
	write (outpr,1000) ' Input data file:    ',indsn(1:50)
	write (outpr,1000) ' Input model file:   ',moddsn(1:50)
	freq1=freq * 1.0e-6
	write (outpr,1001) source,iday,iyear,freq1
	itotalp=nzamp+tnzclo
	write (outpr,1002) itotalp,nu0
	write (outpr,1003) ' Total amplitude agreement factor:',ampla0
	write (outpr,1003) ' Total phase clo agreement factor:',phasa0
	write (outpr,1003) ' Total agreement factor:          ',totla0
	af1sigma=totla0 * (1.0 + 0.5/nu0)
	write (outpr,1003) ' Agr. factor at 1 sigma confidence level:',af1sigma
	do i=1,ncomp
	    do j=1,6
		errpri(i,j)='   ---   '
	    end do
	end do
	do i=1,numpar
	    sigma(i) = sqrt(amat(i,i))
	    nmod=parwho(i,1)
	    npar=parwho(i,2)
	    write (errpri(nmod,npar),'(f9.4)',err=100) sigma(i)
	    go to 150
100	    errpri(nmod,npar) = '****.****'
150	    continue
	end do
	write (outpr,1010) 
	do i=1,ncomp
	    write (outpr,1020) i, (modpar(i,j),j=1,6), type(i)
	    write (outpr,1030)    (errpri(i,j),j=1,6)
	end do
	numpar1=numpar
	if (numpar1.gt.20) numpar1=20
	write (outpr,1100)(achar(parwho(i,2)),parwho(i,1),i=1,numpar1) 
	do ix=1,numpar
	    cormat (ix,ix)=1.
	    do iy=ix+1,numpar
		sum=amat(ix,iy)/(sigma(ix)*sigma(iy))
		cormat(ix,iy)=sum
		cormat(iy,ix)=sum
	    end do
	    write (outpr,1150) achar(parwho(ix,2)),parwho(ix,1),
     +		(cormat(ix,iy),iy=1,numpar1)
	end do
	write (outpr,1200)(achar(parwho(i,2)),parwho(i,1),i=1,numpar1) 
	do ix=1,numpar
	    do iy=1,numpar
		cormat (ix,iy)=amat(ix,iy) / sigma(ix)
	    end do
	    write (outpr,1150) achar(parwho(ix,2)),parwho(ix,1),
     +		(cormat(ix,iy),iy=1,numpar1)
	end do
	return
1000	FORMAT (a,a,a,a,a)
1001	FORMAT (' Source: ',2a8,'     Day',i4,'   Year',i5,f12.2,' MHz',/)
1002	FORMAT (' Equivalent number of independent data points:    ',i6,/
     +		' Degrees of freedom used to calculate chi squared:',i6)
1003	FORMAT (a,f10.3)
1010	FORMAT (/,' Errors for the model: ',/,
     +		' ',10x,'Flux',5x,'Radius',
     +		6x,'Theta',7x,'Axis',6x,'Ratio',8x,'Phi',4x,'Type')
1020	FORMAT (' #',i2,6(1x,f10.4),6x,i2)
1030	FORMAT (' +/-',6(2x,a9),/)
1100	FORMAT (/,' Cross correlation matrix:'/,
     +		  ' J    I=',a2,i2,19(2x,a2,i2))
1150	FORMAT (' ',a2,i2,1x,20f6.2)
1200	FORMAT (/,' Expected variation of p[i] when p[j] is varied at',
     +		  ' 1 sigma level:'/,
     +		  ' J    I=',a2,i2,19(2x,a2,i2))
	end

	subroutine dealpr(nmod,npar,deltap,ierr)
C
C Subroutine that calculates d(x[i]) / d([p[j]),
C that is, the linear dipendence of visibilities and closure phases
C with respect to parameter (npar) of component (nmod). 
C Input:
C	nmod	- # of component
C	npar	- # of parameter (1=flux, etc)
C	deltap  - initial guess for variation of parameter
C in common:
C	totlaf0	- Agreement factor for optimized model
C	numpar	- sequential number of parameter (initially = 0)
C Output:
C	ierr	- error flag: 0 = O.K., 1 = chi^2 not parabolic, cannot guess
C						a reasonable d(p[j])
C in common:
C	numpar	- incremented if ierr=0, not modified if ierr=1
C	visdel(ir,ib,numpar) - deriv. of visib. in record (ir), baseline (ib),
C	clodel(ir,it,numpar) - and of closure phase in trangle (it)
C	parwho(numpar,1)     - nmod  | Used to identify parameter 
C	parwho(numpar,2)     - npar  |   in outpt routine
C
      	include 'errfit.inc'
c
c calculate contribution of all other components
c
	call qmodmp0 (nmod)
c
c find a reasonable model with positive delta parameter
c
	call guess (nmod, npar, deltap, ierr)
	if (ierr.gt.0) return
c
c save model results
c
	do id=1,numrec
	    do ib=1,numbas
		visplu (id,ib) = modvis (id,ib)
	    end do
	    do it=1,numtri
		cloplu (id,it) = modclo (id,it)
	    end do
	end do
c
c now finds a model on the other side of the parameter
c
	delta1=deltap
	deltap=-deltap
	call guess (nmod, npar, deltap, ierr)
	if (ierr.gt.0) return
c
c calculates all derivatives with respect to this parameter
c
	numpar=numpar+1
	parwho(numpar,1)=nmod
	parwho(numpar,2)=npar
	do id=1,numrec
	    do ib=1,numbas
		visdel(id,ib,numpar) = 
     +			(visplu(id,ib)-modvis(id,ib))/(delta1-deltap)
	    end do
	    do it=1,numtri
		clodel(id,it,numpar) = 
     +			(cloplu(id,it)-modclo(id,it))/(delta1-deltap)
	    end do
	end do
	return
	end

	subroutine guess (nmod, npar, deltap, ierr)
C
C Subroutine that finds a 'reasonable' offset in parameter.
C Agreement factor is calculated, and the offset adjusted until the A.F.
C is approx. that specified
C
C Input:
C 	nmod	- # of model component
C	npar	- # of parameter in model (1=flux, etc.)
C	deltap	- initial guessed offset for parameter
C in common:
C	totlaf0	- A.F. for optimized model
C	deltaf	- expected fractional increase in A.F.
C			(expected A.F.=totlaf0 *(1 + deltaf)  )
C	data calculated by qmodmp0
C Output:
C	deltap	- optimum offset for parameter 
C	ierr	- error code: 0 = O.K. - 1 = cannot find a reasonable value
C						for deltap (chi^2 not parabolic)
C	modvis, modclo - visibilities and closure phases for offsetted model
C
      	include 'errfit.inc'
	real modprm
	dimension parms(7),modprm(mxmod,6)
	equivalence (modprm,flux)
	data powrt/-0.3/
c
c copy parameters relative tothis component n vector parms
c
	do i=1,6
	    parms(i)=modprm(nmod,i)
	end do	
	parms(7)=type(nmod)
	ierr=0
c
c try to find optimum deltap (maximum 10 times)
c
	do i=1,10
	    parms(npar)=modprm(nmod,npar)+deltap
c
c calculate offsetted model
c
	    call qmodmp1 (parms)
c
c and its agreement factor
c
	    call agrfac
	    afdel=(totlaf-totla0)/(totla0*deltaf)
c
c afdel is: (delta A.F. observed) / (delta A.F. expected)
c
	    if (afdel.gt.0.7.and.afdel.lt.1.4) return
	    if (afdel.lt.0.02) then
c
c if afdel is too small, increment deltap by a big factor
c
		deltap=deltap*3.0
	    else
C
C else, calculate directly the expected deltap
C
		deltap=deltap*(afdel**powrt)
	    end if
	end do
c
c if this does not converge after ten tries, something is wrong
c
	ierr = 1
	return
	end

	subroutine qmodmp0 (inot)
c
c This computes the constant part of the model (all but component #inot)
c The (complex) visibilities are stored in vector moddt0, to be used later
C    by subrotine qmodmp1
c
      	include 'errfit.inc'
	REAL C1, IMVIS
	PARAMETER (C1=-3.559707)
C
	Q(A,SPHI,CPHI,R,U,V)= TWOPI*SQRT((V*CPHI+U*SPHI)**2
     1	+ (R*(U*CPHI-V*SPHI))**2) * A
	DO 105 IB = 1, NUMBAS
	  DO 100 ID = 1, NUMREC
	    IF (WTVIS(ID,IB).LE.0.) GO TO 100
	    REVIS = 0.0
	    IMVIS = 0.0
	    DO 5  J=1,NCOMP
	      if (j.eq.inot) go to 5
c
c all components are symmetric, then phase depends only on position
c
	      COMPHA = TWOPI*RADIUS(J) * (UDATA(ID,IB)*SIN(RTHETA(J))
     1	      + VDATA(ID,IB)*COS(RTHETA(J)))
C
C	      Find the complex visibility amplitude:
c
	      IF (AXIS(J) .EQ. 0.0) THEN
		COMVIS = FLUX(J)
	      ELSEIF (TYPE(J) .EQ. 1) THEN
	        SPHI   = SIN(RPHI(J))
	        CPHI   = COS(RPHI(J))
		COMVIS = FLUX(J) * (EXP (C1 * AXIS(J)**2 * 
     1		((VDATA(ID,IB)*CPHI + UDATA(ID,IB)*SPHI)**2 +
     2          (RATIO(J) * (UDATA(ID,IB)*CPHI-VDATA(ID,IB)*SPHI))**2)))
	      ELSEIF (TYPE(J) .EQ. 2) THEN
	        SPHI   = SIN(RPHI(J))
	        CPHI   = COS(RPHI(J))
		QQ = Q(AXIS(J),SPHI,CPHI,RATIO(J),UDATA(ID,IB),VDATA(ID,IB))
	        QQ = QQ / 2.0	! Thus, axis is diameter
	        IF (ABS(QQ).LT.0.01) THEN
	          COMVIS = FLUX(J)*(1.0 - QQ*QQ/8. + QQ*QQ*QQ*QQ/192.)
	        ELSE
		  COMVIS = 2. * FLUX(J) * BESJ1(QQ)/QQ
	        END IF
	      ELSEIF (TYPE(J) .EQ. 3) THEN
	        SPHI   = SIN(RPHI(J))
	        CPHI   = COS(RPHI(J))
		QQ = Q(AXIS(J),SPHI,CPHI,RATIO(J),UDATA(ID,IB),VDATA(ID,IB))
	        QQ = QQ / 2.0	! Thus, axis is diameter
	        IF (ABS(QQ).LT.0.01) THEN
	          COMVIS = FLUX(J)*(1.0 - QQ*QQ/10. + QQ*QQ*QQ*QQ/280.)
	        ELSE
	          COMVIS = 3. * FLUX(J) * (SIN(QQ)-QQ*COS(QQ)) /
     1	             (QQ*QQ*QQ)
	        END IF
	      ELSEIF (TYPE(J) .EQ. 4) THEN
	        SPHI   = SIN(RPHI(J))
	        CPHI   = COS(RPHI(J))
		COMVIS = FLUX(J) * BESJ0(Q(AXIS(J),SPHI,
     1		CPHI,RATIO(J),UDATA(ID,IB),VDATA(ID,IB)))
	      ELSEIF (TYPE(J) .EQ. 5) THEN
	        SPHI   = SIN(RPHI(J))
	        CPHI   = COS(RPHI(J))
		XX = (UDATA(ID,IB)*SPHI + VDATA(ID,IB)*CPHI) * PI * AXIS(J)
		IF (ABS(XX).LT.0.01) THEN
		  COMVIS = FLUX(J)*(1.0 - XX*XX/6. + XX*XX*XX*XX/120.)
		ELSE
        	  COMVIS = FLUX(J) * SIN(XX)/XX
		END IF
	      ELSE
		CALL ERROR (' Illegal component type in model.  Bye-bye!')
	      ENDIF
C
	      REVIS  = REVIS + COMVIS*COS(COMPHA)
	      IMVIS  = IMVIS + COMVIS*SIN(COMPHA)
    5       CONTINUE
	    moddt0(id,ib)=cmplx(revis,imvis)
  100	  CONTINUE
  105	CONTINUE
	return
	end
c

	subroutine qmodmp1(parms)
      	include 'errfit.inc'
	dimension parms(7)
c
c subrotine that calculates the visibilities and the closure amplitudes
c    of a model.
c It assumes that the contributions of all components have already been 
c   calculated by QMODMP0, apart from the component described in vector
c   PARMS.
c Output goes to the matrices:
c 	MODVIS (IR,IB) - visibility amplitudes
c	MODPHA (IR,IB) - visibility phases
c	MODCLO (IR,IT) - closure phases
c
	REAL C1, IMVIS, MODPHA(MXREC,MXBAS)
	PARAMETER (C1=-3.559707)
C
	Q(A,SPHI,CPHI,R,U,V)= TWOPI*SQRT((V*CPHI+U*SPHI)**2
     1	+ (R*(U*CPHI-V*SPHI))**2) * A
	flux1  = parms(1)
	radius1= parms(2)
	rtheta1= parms(3) * degrad
	axis1  = parms(4)
	ratio1 = parms(5)
	rphi1  = parms(6) * degrad
	itype1 = parms(7)
c
	DO 105 IB = 1, NUMBAS
	  DO 100 ID = 1, NUMREC
	    IF (WTVIS(ID,IB).LE.0.) GO TO 100
c
c calculate this component
c
	      COMPHA = TWOPI*RADIUS1 * (UDATA(ID,IB)*SIN(RTHETA1)
     1	      + VDATA(ID,IB)*COS(RTHETA1))
C
C	      Find the complex visibility amplitude:
	      IF (AXIS1 .EQ. 0.0) THEN
		COMVIS = FLUX1
	      ELSEIF (iTYPE1 .EQ. 1) THEN
	        SPHI   = SIN(RPHI1)
	        CPHI   = COS(RPHI1)
		COMVIS = FLUX1 * (EXP (C1 * AXIS1**2 * 
     1		((VDATA(ID,IB)*CPHI + UDATA(ID,IB)*SPHI)**2 +
     2          (RATIO1 * (UDATA(ID,IB)*CPHI-VDATA(ID,IB)*SPHI))**2)))
	      ELSEIF (ITYPE1 .EQ. 2) THEN
	        SPHI   = SIN(RPHI1)
	        CPHI   = COS(RPHI1)
		QQ = Q(AXIS1,SPHI,CPHI,RATIO1,UDATA(ID,IB),VDATA(ID,IB))
	        QQ = QQ / 2.0	! Thus, axis is diameter
	        IF (ABS(QQ).LT.0.01) THEN
	          COMVIS = FLUX1*(1.0 - QQ*QQ/8. + QQ*QQ*QQ*QQ/192.)
	        ELSE
		  COMVIS = 2. * FLUX1 * BESJ1(QQ)/QQ
	        END IF
	      ELSEIF (ITYPE1 .EQ. 3) THEN
	        SPHI   = SIN(RPHI1)
	        CPHI   = COS(RPHI1)
		QQ = Q(AXIS1,SPHI,CPHI,RATIO1,UDATA(ID,IB),VDATA(ID,IB))
	        QQ = QQ / 2.0	! Thus, axis is diameter
	        IF (ABS(QQ).LT.0.01) THEN
	          COMVIS = FLUX1*(1.0 - QQ*QQ/10. + QQ*QQ*QQ*QQ/280.)
	        ELSE
	          COMVIS = 3. * FLUX1 * (SIN(QQ)-QQ*COS(QQ)) /
     1	             (QQ*QQ*QQ)
	        END IF
	      ELSEIF (ITYPE1 .EQ. 4) THEN
	        SPHI   = SIN(RPHI1)
	        CPHI   = COS(RPHI1)
		COMVIS = FLUX1 * BESJ0(Q(AXIS1,SPHI,
     1		CPHI,RATIO1,UDATA(ID,IB),VDATA(ID,IB)))
	      ELSEIF (ITYPE1 .EQ. 5) THEN
	        SPHI   = SIN(RPHI1)
	        CPHI   = COS(RPHI1)
		XX = (UDATA(ID,IB)*SPHI + VDATA(ID,IB)*CPHI) * PI * AXIS1
		IF (ABS(XX).LT.0.01) THEN
		  COMVIS = FLUX1*(1.0 - XX*XX/6. + XX*XX*XX*XX/120.)
		ELSE
        	  COMVIS = FLUX1 * SIN(XX)/XX
		END IF
	      ELSE
		CALL ERROR (' Illegal component type in model.  Bye-bye!')
	      ENDIF
C
	      REVIS  = COMVIS*COS(COMPHA) + real (moddt0(id,ib))
	      IMVIS  = COMVIS*SIN(COMPHA) + aimag(moddt0(id,ib))
c
	    MODVIS(ID,IB) = SQRT(REVIS*REVIS + IMVIS*IMVIS)
	    IF (REVIS .EQ. 0. .AND. IMVIS .EQ. 0.) THEN
	      MODPHA(ID,IB) = 0.
	    ELSE
	      MODPHA(ID,IB) = -ATAN2(IMVIS,REVIS)
	    ENDIF
  100	  CONTINUE
  105	CONTINUE
C
C	Calculate the closure phases for the model:
	DO 120 IC = 1,NUMTRI
	  DO 110 ID = 1,NUMREC
	    MODCLO(ID,IC) = MODPHA(ID,ICBSNO(1,IC)) +
     1	    MODPHA(ID,ICBSNO(3,IC)) - MODPHA(ID,ICBSNO(2,IC))
	    IF (MODCLO(ID,IC) .LT. -PI .OR. MODCLO(ID,IC) .GT. PI) THEN
	      MODCLO(ID,IC) = AMOD( MODCLO(ID,IC), TWOPI )
	      IF (MODCLO(ID,IC) .LT. -PI) MODCLO(ID,IC) = MODCLO(ID,IC)
     +	      + TWOPI
	      IF (MODCLO(ID,IC) .GT.  PI) MODCLO(ID,IC) = MODCLO(ID,IC)
     +	      - TWOPI
	    ENDIF
  110	  CONTINUE
  120	CONTINUE
C
	RETURN
	END

	Subroutine AGRFAC
C
C	This subroutine finds the agreement factors for the fit of a 
C	modle to VLB data.
C
      	INCLUDE 'errfit.inc'
	REAL CSUM(MXTRI)
C
C	Amplitude agreement factors:
	AMPSUM = 0.
	DO 20 IB = 1, NUMBAS
	  IF (NZVIS(IB) .GT. 0) THEN
	    VSUM = 0.
	    DO 10 ID = 1, NUMREC
	      IF (WTVIS(ID,IB) .GT. 0.0) THEN
		DELVIS = MODVIS(ID,IB) - OBSVIS(ID,IB)
		VSUM   = VSUM + WTVIS(ID,IB)*DELVIS*DELVIS
	      ENDIF
   10	    CONTINUE
	    VISAF(IB) = SQRT (VSUM / NZVIS(IB))
	    AMPSUM = AMPSUM + VSUM
	  ELSE
	    VISAF(IB) = -1.
	  ENDIF
   20	CONTINUE
	IF (NZAMP .GT. 0) THEN
	  AMPLAF = SQRT (AMPSUM / NZAMP)
	ELSE
	  AMPLAF = -1.
	ENDIF
C
C	Closure phase agreement factors, for each triangle
	DO 25 IC = 1, NUMTRI
	  CSUM(IC) = 0.
   25	CONTINUE
	CLOSUM = 0.
	IF (NUMTRI .GT. 0) THEN
	  DO 40 ID = 1,NUMREC
	    IF (NINTRI(ID) .GT. 0) THEN
	      DO 30 IC = 1,NUMTRI
	        IF (WTCLO(ID,IC) .GT. 0.) THEN
		  DELCLO = ABS (MODCLO(ID,IC) - OBSCLO(ID,IC))
		  IF (DELCLO .GT. PI) DELCLO = DELCLO - TWOPI
		  CSUM(IC) = CSUM(IC) + WTCLO(ID,IC)*DELCLO*DELCLO
		  CLOSUM = CLOSUM+DOWNWT(ID)*WTCLO(ID,IC)*DELCLO*DELCLO
	        ENDIF
   30	      CONTINUE
	    ENDIF
   40	  CONTINUE
	ENDIF
	DO 50 IC = 1,NUMTRI
	  IF (NZCLO(IC) .GT. 0) THEN
	    CLOAF(IC) = SQRT (CSUM(IC)/NZCLO(IC))
	  ELSE
	    CLOAF(IC) = -1.
	  ENDIF
   50	CONTINUE
C
	IF (TNZCLO .GT. 0.) THEN
	  PHASAF = SQRT (CLOSUM/TNZCLO)
	ELSE
	  PHASAF = -1.
	ENDIF
C
C	Total agreement factor for all data:
	TOTLAF = SQRT ((AMPSUM+CLOSUM) / (NZAMP+TNZCLO))
C
	RETURN
	END

	SUBROUTINE QGETMOD
C
C	By Richard S. Simon
C
C	This routine, based on TJP's GETMOD in UVPLOT, retrieves a model.
C
      	INCLUDE 'errfit.inc'
C
	FLXSUM = 0.
	DO 150 I = 1, MXMOD
	  CALL GTMODL (INMOD, ITYPE, FLUX(I), RADIUS(I), THETA(I),
     1	  AXIS(I), RATIO(I), PHI(I), *155, 0)
	  IF (FLUX(I) .EQ. 0.0) GO TO 155
	  FLXSUM = FLXSUM + FLUX(I)
	  RTHETA(I) = THETA(I) * DEGRAD
	  RPHI(I)   = PHI(I) * DEGRAD
	  TYPE(I)   = ITYPE
	  IF (TYPE(I) .EQ. 0) THEN
	    TYPE(I) = 1
	  ELSEIF (TYPE(I) .LT. 1 .OR. TYPE(I) .GT. 5) THEN
	    WRITE (OUTC,660) TYPE(I)
	    TYPE(I) = 1
	  ENDIF
  150	CONTINUE
	WRITE (OUTC,650) MXMOD
	I = MXMOD + 1
  155	NCOMP = I-1
	IF (FLXSUM .EQ. 0.0) CALL ERROR('Model has zero total flux')
	RETURN
C
  650	FORMAT (' ++WARNING++ Model truncated after ',I5,' components.')
  660	FORMAT (' ++WARNING++ Invalid component type for component ',I4,
     1	'; Gaussian used')
C
	END

	SUBROUTINE QGETDAT
C
C	By Richard S. Simon
C
C	Version 1:	??-June-1980	
C	Version 2:	20-July-1983	Extend to include Merge
C				version 2 files and compute U and V;
C				fix bug in TAKE dimension
C
C  This subroutine does the following:
C	1) Calls subroutine HEADX15 to get data parameters.
C	2) Reads in amplitude and closure phase data from data file.
C	3) Computes (or reads in) the U's and V's for a data set.
C
      	INCLUDE 'errfit.inc'
C
	REAL	TAKE(MAXWPB*MXBAS + 2*MXTRI), SIDINT(MXREC), BZ(MXBAS),
     1	  BXY(MXBAS), XIHA(MXBAS)
C
D	WRITE (OUTC,1000)' D lines being used in subr QGETDAT;',
D    1	'  see FOR069.DAT'
C
C	Call headx15 to get data parameters
	IOUT = 0
	IPROUT = 0
	CALL HEADX15(.FALSE., INDAT, IZERO, IDAY, IYEAR, IDUMMY, IDUM,
     +		  IOUT, IUNITS, DUMMY, IFILE, IPROUT)
	IF (IFILE .NE. 0) THEN
	  WRITE (OUTC,1000) ' Data file does not have header record at',
     1	  ' start of file.  Bye-bye!'
	  STOP
	END IF
C
C	Certify that data is in flux units before proceeding.
        IF (IUNITS .NE. 1) THEN
	  WRITE (OUTC,1000) ' Data not in flux units, so Bye-bye!'
	  STOP
	END IF
C	
	WRITE (OUTC,1007) SOURCE(1),SOURCE(2)
	LAMBDA = VELC/FREQ
C	
C	Load NSTA, NBAS, etc into COMMON
	NUMSTA = NSTA
	NUMBAS = NBAS
	NUMTRI = NTRI
	NINDCL = (NUMSTA-1)*(NUMSTA-2)/2
C
	COSDEC = COS(DECP)
	SINDEC = SIN(DECP)
	IF (NVER.EQ.1) THEN
	  DO IB = 1, NUMBAS
	    BX       = (STX(NBVIS(IB))-STX(NAVIS(IB))) / LAMBDA
	    BY       = (STY(NBVIS(IB))-STY(NAVIS(IB))) / LAMBDA
	    BZ(IB)   = (STZ(NBVIS(IB))-STZ(NAVIS(IB))) / LAMBDA
	    BHA      = ATAN2(BY,BX)
	    BXY(IB)  = SQRT (BX*BX + BY*BY)
	    XIHA(IB) = (GASTM-RAP-BHA+HALFPI) * RADHRS
	  ENDDO
	ENDIF
C
C	Read in the data, compute UT
	NWORDS = NDWPB*NUMBAS + 2*NUMTRI
	DO 150 ID = 1, MXREC
	  READ (INDAT, END=160)  IT, (TAKE(I), I = 1, NWORDS)
	  IF (IT .EQ. -1) GO TO 160
C	  Compute UT in hours
	  UT(ID) = (IT - IZERO) / 216000.
	  SIDINT(ID) = UT(ID) * SIDR
C	  Get amplitudes and errors and UV values
	  DO 130 IB = 1, NUMBAS
	    K = NDWPB*(IB-1)
	    OBSVIS(ID,IB) = TAKE(K+1)
	    WTVIS(ID,IB)= TAKE(K+2)
	    IF (NVER.EQ.1) THEN
	      OBSIHA = XIHA(IB) + SIDINT(ID)
	      RADIHA = OBSIHA / RADHRS
	      UDATA(ID,IB) = (-BXY(IB)*COS(RADIHA)) / CONV1
	      VDATA(ID,IB) = (BZ(IB)*COSDEC -
     1		BXY(IB)*SINDEC*SIN(RADIHA)) / CONV1
	    ELSEIF (NVER.EQ.2) THEN
	      UDATA(ID,IB) = - TAKE(K+5) / CONV1
	      VDATA(ID,IB) = - TAKE(K+6) / CONV1
	    ENDIF
  130	  CONTINUE
C	  Get closure phases and errors
	  IF (NUMTRI .GT. 0.) THEN
	    DO 140 IC = 1, NUMTRI
	      K = NDWPB*NUMBAS + 2*(IC-1)
	      OBSCLO(ID,IC)  = TAKE(K+1) * DEGRAD
	      WTCLO(ID,IC) = TAKE(K+2)   * DEGRAD
  140	    CONTINUE
	  ENDIF
  150	CONTINUE
C	Not all the data was read in, so issue a warning:
	WRITE (OUTC,1008) MXREC,MXREC,MXREC
	ID = MXREC + 1
C	
  160	CONTINUE
	NUMREC = ID - 1
C	
 1000   FORMAT (A,A,A,A)
 1007	FORMAT (/' Source:',2A8)
 1008	FORMAT(/' Well, it looks like you have blown it.  This version',
     +	' of the program will',
     +	/' only handle',I4,' separate data records, i.e.,',I4,
     +	' separate UT''s.  Thus,',
     +	/' only the first',I4,' times will be used in the fit.  I',
     +	' suggest you use the program',
     +	/' SIA0:[VLB]AVERAGE to average your data somewhat.  Good',
     +	' Luck.'/50X,'- RSS'/)
C	
C	Print out the data during program checkout.
D	WRITE (69,9003)
D	DO 8100 IB = 1, NUMBAS
D	  WRITE (69,9002) STNAME(NAVIS(IB)), STNAME(NBVIS(IB))
D	  DO 8050 ID = 1,NUMREC
D	    WRITE (69,9004) UT(ID), OBSVIS(ID,IB), WTVIS(ID,IB)
D8050	  CONTINUE
D8100	CONTINUE
D	WRITE (69,9005)
D	IF (NUMTRI .LE. 0) THEN
D	  WRITE (69,9008)
D	  GO TO 8300
D	END IF
D	DO 8200 IC = 1, NUMTRI
D	  WRITE (69,9006) STNAME(NACLO(IC)), STNAME(NBCLO(IC)),
D    +			 STNAME(NCCLO(IC))
D	  WRITE (69,9007)
D	  DO 8150 ID = 1, NUMREC
D	    CLOSE = OBSCLO(ID,IC) / DEGRAD
D	    ERROR = WTCLO(ID,IC) / DEGRAD
D	    WRITE (69,9004) UT(ID),CLOSE, ERROR
D8150	  CONTINUE
D8200	CONTINUE
D8300	CONTINUE
C	
D9002	FORMAT (' Baseline: ',A4,' to ',A4)
D9003	FORMAT (/' Amplitude data and errors:'/)
D9004	FORMAT (' ',3F12.3)
D9005	FORMAT (//' Closure phase data and errors:'//)
D9006	FORMAT (' The triangle consists of stations ',A4,'-',A4,'-',A4/)
D9007	FORMAT ('      UT Time   Cl. Phase       Error'/)
D9008	FORMAT (' No closure phase information to be used'//)
C	
   	RETURN
	END

	SUBROUTINE QMODAP
C
      	INCLUDE 'errfit.inc'
C
	REAL C1, IMVIS, MODPHA(MXREC,MXBAS)
	PARAMETER (C1=-3.559707)
C
	Q(A,SPHI,CPHI,R,U,V)= TWOPI*SQRT((V*CPHI+U*SPHI)**2
     1	+ (R*(U*CPHI-V*SPHI))**2) * A
	DO 105 IB = 1, NUMBAS
	  DO 100 ID = 1, NUMREC
	    IF (WTVIS(ID,IB).LE.0.) GO TO 100
	    REVIS = 0.0
	    IMVIS = 0.0
	    DO 5  J=1,NCOMP
	      COMPHA = TWOPI*RADIUS(J) * (UDATA(ID,IB)*SIN(RTHETA(J))
     1	      + VDATA(ID,IB)*COS(RTHETA(J)))
C
C	      Find the complex visibility amplitude:
	      IF (AXIS(J) .EQ. 0.0) THEN
		COMVIS = FLUX(J)
	      ELSEIF (TYPE(J) .EQ. 1) THEN
	        SPHI   = SIN(RPHI(J))
	        CPHI   = COS(RPHI(J))
		COMVIS = FLUX(J) * (EXP (C1 * AXIS(J)**2 * 
     1		((VDATA(ID,IB)*CPHI + UDATA(ID,IB)*SPHI)**2 +
     2          (RATIO(J) * (UDATA(ID,IB)*CPHI-VDATA(ID,IB)*SPHI))**2)))
	      ELSEIF (TYPE(J) .EQ. 2) THEN
	        SPHI   = SIN(RPHI(J))
	        CPHI   = COS(RPHI(J))
		QQ = Q(AXIS(J),SPHI,CPHI,RATIO(J),UDATA(ID,IB),VDATA(ID,IB))
	        QQ = QQ / 2.0	! Thus, axis is diameter
	        IF (ABS(QQ).LT.0.01) THEN
	          COMVIS = FLUX(J)*(1.0 - QQ*QQ/8. + QQ*QQ*QQ*QQ/192.)
	        ELSE
		  COMVIS = 2. * FLUX(J) * BESJ1(QQ)/QQ
	        END IF
	      ELSEIF (TYPE(J) .EQ. 3) THEN
	        SPHI   = SIN(RPHI(J))
	        CPHI   = COS(RPHI(J))
		QQ = Q(AXIS(J),SPHI,CPHI,RATIO(J),UDATA(ID,IB),VDATA(ID,IB))
	        QQ = QQ / 2.0	! Thus, axis is diameter
	        IF (ABS(QQ).LT.0.01) THEN
	          COMVIS = FLUX(J)*(1.0 - QQ*QQ/10. + QQ*QQ*QQ*QQ/280.)
	        ELSE
	          COMVIS = 3. * FLUX(J) * (SIN(QQ)-QQ*COS(QQ)) /
     1	             (QQ*QQ*QQ)
	        END IF
	      ELSEIF (TYPE(J) .EQ. 4) THEN
	        SPHI   = SIN(RPHI(J))
	        CPHI   = COS(RPHI(J))
		COMVIS = FLUX(J) * BESJ0(Q(AXIS(J),SPHI,
     1		CPHI,RATIO(J),UDATA(ID,IB),VDATA(ID,IB)))
	      ELSEIF (TYPE(J) .EQ. 5) THEN
	        SPHI   = SIN(RPHI(J))
	        CPHI   = COS(RPHI(J))
		XX = (UDATA(ID,IB)*SPHI + VDATA(ID,IB)*CPHI) * PI * AXIS(J)
		IF (ABS(XX).LT.0.01) THEN
		  COMVIS = FLUX(J)*(1.0 - XX*XX/6. + XX*XX*XX*XX/120.)
		ELSE
        	  COMVIS = FLUX(J) * SIN(XX)/XX
		END IF
	      ELSE
		CALL ERROR (' Illegal component type in model.  Bye-bye!')
	      ENDIF
C
	      REVIS  = REVIS + COMVIS*COS(COMPHA)
	      IMVIS  = IMVIS + COMVIS*SIN(COMPHA)
    5       CONTINUE
	    MODVIS(ID,IB) = SQRT(REVIS*REVIS + IMVIS*IMVIS)
	    IF (REVIS .EQ. 0. .AND. IMVIS .EQ. 0.) THEN
	      MODPHA(ID,IB) = 0.
	    ELSE
	      MODPHA(ID,IB) = -ATAN2(IMVIS,REVIS)
	    ENDIF
  100	  CONTINUE
  105	CONTINUE
C
C	Calculate the closure phases for the model:
	DO 120 IC = 1,NUMTRI
	  DO 110 ID = 1,NUMREC
	    MODCLO(ID,IC) = MODPHA(ID,ICBSNO(1,IC)) +
     1	    MODPHA(ID,ICBSNO(3,IC)) - MODPHA(ID,ICBSNO(2,IC))
	    IF (MODCLO(ID,IC) .LT. -PI .OR. MODCLO(ID,IC) .GT. PI) THEN
	      MODCLO(ID,IC) = AMOD( MODCLO(ID,IC), TWOPI )
	      IF (MODCLO(ID,IC) .LT. -PI) MODCLO(ID,IC) = MODCLO(ID,IC)
     +	      + TWOPI
	      IF (MODCLO(ID,IC) .GT.  PI) MODCLO(ID,IC) = MODCLO(ID,IC)
     +	      - TWOPI
	    ENDIF
  110	  CONTINUE
  120	CONTINUE
C
	RETURN
	END

	SUBROUTINE TRIANGLE
C
C	Finds which triangles are independent in a VLB data set.
C	Hypothesis:  A triangle is independent if all three of it's baselines
C		     have NOT appeared in an earlier triangle as the third
C		     baseline.
C
      	INCLUDE 'errfit.inc'
C
	INTEGER ISET3B(MXICLO)
C
C	Find the 3 baseline numbers ICBSNO(3,NTRI) for triangle NTRI:
	NTRI1  = 0
	NTRI23 = 0
	DO 40 I = 1, NUMSTA-2
	  JSTOP = I*(I+1) / 2
	  DO 10 J = 1, JSTOP
	    NTRI1 = NTRI1 + 1
	    ICBSNO(1,NTRI1) = J
   10	  CONTINUE
	  DO 30 J = 1, I
	    ICBAS3 = JSTOP + 1 + J
	    DO 20 K = 1, J
	      NTRI23 = NTRI23 + 1
	      ICBSNO(2,NTRI23) = JSTOP + K
	      ICBSNO(3,NTRI23) = ICBAS3
   20	    CONTINUE
   30	  CONTINUE
   40	CONTINUE
C
C	Find the independent triangles:
	NZCLOS = 0
	NUM3B = NINDCL
	DO 100 ID = 1, NUMREC
	  DO 50 IC=1,NUM3B
	    ISET3B(IC) = 0
	  NUM3B = 0
   50	  CONTINUE
	  DO 90 IT = 1, NUMTRI
	    IF (WTCLO(ID,IT) .GT. 0.0) THEN
	      DO 80 IB = 1,3
	        DO 70 I3B = 1, NUM3B
		  IF (ISET3B(I3B) .EQ. ICBSNO(IB,IT)) THEN
C		    Baseline in this triangle IS an element of set of 3rd base:
		    GO TO 80	! Test next baseline in this triangle
		  ENDIF
   70	        CONTINUE
C	        A baseline in triangle IT is not element of set of 3rd base
C	        So, add 3rd baseline to ISET3B, and triangle to INDTRI
	        NUM3B = NUM3B + 1
	        ISET3B(NUM3B) = ICBSNO(3,IT)
C	        INDTRI(ID,NUM3B) = IT
	        NINTRI(ID) = NUM3B		! cumulative number of ind tri
	        IF (NUM3B .EQ. NINDCL)GO TO 100	! All ind tri found for this ID
	        GO TO 90				! Go to next triangle
   80	      CONTINUE
	    ENDIF
   90	  CONTINUE
  100	CONTINUE
C
	RETURN
	END

	SUBROUTINE WTDATA
C
C	By Richard S. Simon
C
C	This subroutine is part of program QFIT, and does the following:
C	1)  Compute the weights for each data point.
C	2)  Find the number of non-zero points in the data set.
C
C	Version N+1:	??-Jun-1981

C	Version N+2:	Extend to include MERGE format Version 2 files
C
      	INCLUDE 'errfit.inc'
C
	DO 30 ID = 1, NUMREC
	  NUMCLO(ID) = 0
   30	CONTINUE
C
C	Amplitude weights:
	NZAMP = 0
	DO 60 IB = 1, NUMBAS
	  NZVIS(IB) = 0
	  DO 50 ID = 1, NUMREC
	    IF (WTVIS(ID,IB) .GT. 0.0) THEN
	      NZVIS(IB) = NZVIS(IB) + 1
	      WTVIS(ID,IB) = 1. / (WTVIS(ID,IB))**2
	    ENDIF
   50	  CONTINUE
	  NZAMP = NZAMP + NZVIS(IB)
   60	CONTINUE
C
C	Find the weights for the closure phase data, if any:
	DO 80 IC = 1, NUMTRI
	  NZCLO(IC) = 0
	  DO 70 ID = 1, NUMREC
	    IF (WTCLO(ID,IC) .GT. 0.0) THEN
	      NZCLO(IC) = NZCLO(IC) + 1
	      NUMCLO(ID) = NUMCLO(ID) + 1
	      WTCLO(ID,IC) = 1./ (WTCLO(ID,IC))**2
	    ENDIF
   70	  CONTINUE
   80	CONTINUE
	TNZCLO = 0.
	DO 90 ID = 1,NUMREC
	  IF (NUMCLO(ID) .GT. 0) THEN
	    DOWNWT(ID) = REAL(NINTRI(ID)) / REAL(NUMCLO(ID))
	    TNZCLO = TNZCLO + NINTRI(ID)
	  ELSE
	    DOWNWT(ID) = 0
	  ENDIF
   90	CONTINUE
C
	RETURN
	END

Cvms	OPTIONS/G_FLOATING
	subroutine matinv(a,n,norder,det)
c 
c matrix inversion of real matrix a(n,n)
c  Only the first norder rows and columns are used
c  Det = matrix determinant on output
C
C WARNING: This subroutine MUST compile g-float
C --------                 ----
c
	dimension a(n,n),ik(50),jk(50)
	real*8 array(50,50),amax,save,det1
c
c first copies matrix in double precision array
c
	do i=1,norder
	    do j=1,norder
		array(i,j)=a(i,j)
	    end do
	end do
	det1=1.0d0
	do k=1,norder
c
c find largest element in array
c
	    amax=0.
	    do i=k,norder
		do j=k,norder
		if (abs(amax).lt.abs(array(i,j)) ) then
		    amax=array(i,j)
		    ik(k)=i
		    jk(k)=j
		end if
		end do
	    end do
c
c interchange row & columns so that pivot is in array(k,k)
c
	    if (amax.eq.0.d0) then
		det=0.
		return
	    end if
	    i=ik(k)
	    if (i.gt.k) then
		do j=1,norder
		    save=array(k,j)
		    array(k,j)=array(i,j)
		    array(i,j)=-save
		end do
	    end if
	    j=jk(k)
	    if (j.gt.k) then
		do i=1,norder
		    save=array(i,k)
		    array(i,k)=array(i,j)
		    array(i,j)=-save
		end do
	    end if
c
c Accumulate elements of inverse matrix
c
	    do i=1,norder
		if (i.ne.k) array(i,k)=-array(i,k) / amax
	    end do
	    do i=1,norder
		do j=1,norder
		    if (i.ne.k.and.j.ne.k) 
     +			array(i,j)=array(i,j) + array(i,k) * array(k,j)
		end do
	    end do
	    do j=1,norder
		if (j.ne.k) array(k,j)=array(k,j)/amax
	    end do
	    array(k,k)=1.0d0/amax
	    det1=det1 * amax
	end do
c
c Restore matrix order
c
	do l=1,norder
	k=norder-l+1
	j=ik(k)
	if (j.gt.k) then
	    do i=1,norder
		save=array(i,k)
		array(i,k)=-array(i,j)
		array(i,j)=save
	    end do
	end if
	i=jk(k)
	if (i.gt.k) then
	    do j=1,norder
		save=array(k,j)
		array(k,j)=-array(i,j)
		array(i,j)=save
	    end do
	end if
	end do
	do i=1,norder
	    do j=1,norder
		a(i,j)=array(i,j)
	    end do
	end do
	det=det1
	return
	end
