      PROGRAM FAKE
C-----------------------------------------------------------------------
C FAKE generates artificial data in the form of a MERGE dataset.
C
C T.J. Pearson,  VAX-11 Fortran
C
C Version 1.0:  1979 Jul  5.
C Version 2.0:  1979 Oct 16 - Table lookup for stations, 15 stations.
C Version 3.0:  1980 Jan 19 - Add precession; add random scaling errors;
C                             take account of telescope limits.
C Version 3.1:  1980 Mar 27 - Add NUP parameter.
C Version 3.2:  1980 Jun 24 - Correct bug ERRADD or ERRMULT = 0.
C Version 3.8:  1981 May 23 - Permit comments in model file.
C Version 4.0:  1981 Dec  9 - Add telescope phase errors and
C                             parameters PHASES, MODEL, OUTPUT.
C Version 4.1:  1981 Dec 19 - Correct bug in ERRSCALE handling.
C Version 4.2:  1982 Feb  1 - Improved SCALE errors (RPL).
C Version 4.3:  1982 Jun  8 - Up to 2000 model components.
C Version 4.4:  1982 Nov 15 - Add parameter PERFECT; all model types.
C Version 4.5:  1983 Apr 26 - Correct divide by zero bug.
C Version 5.0:  1983 Jul  5 - Extended to include Merge format version 2.
C                             D. L. Meier
C Version 5.1:  1983 Aug  3 - Included W in version 2 files (DLM)
C Version 5.2:  1983 Aug 16 - Make ERRAD an array, one for each stn.
C Version 5.3:  1983 Aug 17 - Add LONG parameter (DLM)
C Version 5.4:  1983 Aug 21 - Added TELEMETRY stations option (DLM);
C                             Delete station type 0 (all baselines now
C                             computed with ROT(i,j) )
C Version 5.5:  1983 Aug 28 - Add NONOFF parameter (on/off source) and
C                             increase MAXS to 20 (DLM)
C Version 5.6:  1983 Oct 17 - Add history record specifying ERRSCALE and
C                             PHASES; add checks for errors in TELEMETRY.
C                             Allow FAKE to create a Merge format version 1
C                             file when no orbiters are present (DLM).
C Version 5.7:  1983 Dec  2 - Increase MAXS to 40 (DLM).
C Version 6.0:  1984 Jan  4 - Add NODATA and UVPLOT options with
C                             associated parameters (DLM). Add NGRDIL.
C Version 6.1:  1984 Jan 17 - Fix bugs introduced into V6.0. (DLM)
C Version 6.2:  1984 Apr  5 - Fix bug in UVPLOT axis-choosing routine
C                             (DLM)
C Version 6.3:  1984 Jul 17 - Add ability to do VLBI at more than one
C                             frequency (FREQ=freq1, freq2,...). (DLM)
C Version 6.4:  1984 Jul 25 - Add ability to compute ERRADD from TSYS,
C                             EFF, DIAM, POINT, BANDW, and INTEG. (DLM)
C Version 6.5:  1984 Aug 13 - Fix bugs introduced into 6.3 and 6.4 (DLM)
C Version 6.6:  1984 Oct 17 - Change default ERRMULT to 0, to agree
C                             with help file (TJP/DLM).
C Version 6.7:  1987 Feb  2 - Add parameter TELANGLE to suppress data
C                             when spacecraft telemetry antenna cannot
C                             point to a ground telemetry station (DJ).
C Version 6.8:  1987 Sep  4 - replace all GRPCKG calls with PGPLOT (TJP)
C Version 6.9:  1987 Oct 23 - add prompts when plotting on command
C                             terminal to avoid interference (TJP).
C Version 7.0:  1988 Apr  7 - changes for Convex; add SEED parameter
C                             (TJP).
C Version 7.1:  1989 Jun  9 - add ROT parameter to allow for space
C                             optical interferometry (apertures not
C                             mounted on earth but rather slowly
C                             rotating, inertially fixed platform) (DLM)
C Version 7.2:  1990 May  8 - fixed bugs in ROT implementation;
C                             allowed stations to reside at radii as small
C                             as one lunar radius; generate Merge version 2
C                             file in that case
C Version 7.3:  1991 Jan 23 - add photon statistical noise for optical
C                             interferometric observations (DLM)
C Version 7.4:  1991 Apr 22 - add zodiacal light, thermal noise, and
C                             quantum efficiency for optical
C                             interferometric observations;  also add
C                             ability to cancel real part of visibility
C                             (NULLING) with nulling interferometer (DLM)
C Version 7.5:  1994 Mar  9 - fix minor bug (TJP).
C-----------------------------------------------------------------------
      CHARACTER*(*)  PROGNM, VERSN
      INTEGER    MAXS,MAXB,MAXT,MAXMOD,NPARS,INC,OUTC,INSTA,INMOD,OUT
      INTEGER    MAXWPB,MAXTE,MAXS2,MAXFRQ,MAXFRQ1
      DOUBLE PRECISION  PI,TWOPI,RPSA,RPST,RPDEG,RPMAS,VELC,SIDR,
     1           SIDDAY,PLANCK,BOLTZ
      PARAMETER  (PROGNM='FAKE')
      PARAMETER  (VERSN='7.5 - 1994 Mar 9')
      PARAMETER  (MAXS=40)
C                ! maximum number of stations
      PARAMETER  (MAXS2=MAXS-2)
      PARAMETER  (MAXB=MAXS*(MAXS-1)/2)
      PARAMETER  (MAXT=MAXB*(MAXS-2)/3)
      PARAMETER  (MAXTE=15)
C                ! maximum number of telem. stations
      PARAMETER  (MAXWPB=7)
C                ! maximum no. of words per baseline
      PARAMETER  (MAXMOD=2000)
C                ! maximum number of model components
      PARAMETER  (MAXFRQ=10)
      PARAMETER  (MAXFRQ1=MAXFRQ-1)
      PARAMETER  (NPARS=86+7*MAXS+MAXTE+MAXFRQ)
C                ! number of input parameters
      PARAMETER  (INC=5)
C                ! unit for input
      PARAMETER  (OUTC=6)
C                ! unit for output
      PARAMETER  (INSTA=90)
C                ! unit for stations catalog
      PARAMETER  (INMOD=4)
C                ! unit for input model
      PARAMETER  (OUT=10)
C                ! unit for output MERGE file
      PARAMETER  (PI=3.141592653589793 D0)
      PARAMETER  (TWOPI=2D0*PI)
      PARAMETER  (RPSA=PI/648000 D0)
      PARAMETER  (RPST=PI/43200 D0)
      PARAMETER  (RPDEG=PI/180.D0)
      PARAMETER  (RPMAS=RPSA/1D3)
      PARAMETER  (VELC=2.99792458D8)
      PARAMETER  (SIDR=(PI/2592000D0)*1.0027379093D0)
C                (radians of ST per sec/60 of UT)
      PARAMETER  (SIDDAY=24D0/1.0027379093D0)
C                (length of sidereal day in hours of UT)
      PARAMETER  (PLANCK=6.6260755D-34)
C                (Planck's constant in J/Hz)
      PARAMETER  (BOLTZ=1.380658D-23)
C                (Boltzmann's constant in J/K)
C
      DOUBLE PRECISION SCAL,TOTFLX,RAD,THETA,
     1   A,PH,ARG,PHASES,ERRADD,ERRMUL,ERRSCA,COSHAL,HA,HAL,
     2   SCALB,U,V,W,VISR,VISI,ERR1,ERR2,ERR3,AMP0,AMP,AMPERR,CPHASE,
     3   YNTERP,Q5,Q6,UTH,UTM,UTS,UMAXC,VMAXC,UMAXP,VMAXP,UMAX,
     4   VMAX,EPOCH,DAY,UMX1,VMX1,UMX2,VMX2,UU,VV,PHS,ERRPHS,ERRAMP,
     5   HNU,VSRSUB,VSISUB
      INTEGER MODE,IM,ID,IY,IER,NS,NB,NT,M,LEN1,NCPRIN,NCOMP,NUPMIN,
     1   IUT1,IUT2,IDAY,IUT3,IUT4,IS,NTIME,IUT,NVER,NHIST,IB,IS1,IS2,
     2   IT,IS3,JMAX,LENG,NHEAD,KFLAG,I0,J,ISEED,NUP,IPOS,I,STTYPE,
     3   STATYP(MAXS),NST,IST,NSTOP,NFIRST,NPRT,J1,J2,NON,NOFF,NTOT,
     4   NTYME,JTYME,NDWPB,MARK,LINEWD,NFONT,NDOTSZ,L,L1,L2,JD,MONTH,
     5   NS1,NS2,NS3,NS4,BCOUNT(MAXB),IP,M1,M2,PGBEG,NGRDIL,NFRQ,
     6   NFREQ,NCSUB,NCSAV,VLBOPE
      CHARACTER*3  FUNIT
      CHARACTER*80 MODFIL,OUTFIL,HISTRY,PLTFIL
      CHARACTER*20 NOTE, STRING
      CHARACTER*16 SOURCE
      CHARACTER*17 BNAME(MAXB)
      CHARACTER*8  STNAME(MAXS),XON, Q,TIMEC, TENAME(MAXTE)
      CHARACTER*64 LEGEN1
      CHARACTER*(9*14) LEGEN2
      CHARACTER*(9*14) LEGEN3
      CHARACTER*(9*14) LEGEN4
      DOUBLE PRECISION PARS(NPARS), VALS(NPARS), ENDMRK, D0,
     1       RA, DEC, RA50, DEC50, FLUX, SRCFLX,
     2       COSRA, SINRA, COSDEC, SINDEC, SVEC(3), ROT(3,3),
     3       WAVE, WAVEMIN, FREQ, FREQU(MAXFRQ),
     4       RR, TT, PP, CLAT(MAXS), SLAT(MAXS), COSZAL,
     5       STX(MAXS), STY(MAXS), STZ(MAXS),
     6       RHST0(6,MAXS), RHST(6,MAXS), AEIOWM(6,MAXS), UTORB,
     7       EPOCHJD(MAXS), TODAYJD, DUMGST,
     8       GASTM, INTEG, GST, GSTM, COST, SINT, DT,
     9       BXIB, BYIB, BZIB, BEIB,
     *       RE, GM, HJ2,
     *       SATRA, COSSATDEC, SINSATDEC, CLATTE(MAXTE),
     *       SLATTE(MAXTE), TEX(MAXTE), TEY(MAXTE), TEZ(MAXTE),
     *       RHTE0(3,MAXTE), RHTE(3), RHSTP(3),
     *       TSYS(MAXS), EFF(MAXS), DIAM(MAXS), POINT(MAXS),
     *       BANDW, FREQSUM, TELANGLE, TANG, FACTOR, OMEGA, PERIOD,
     *       TD,TM,TS,PD,PM,PS,XZL,XHL,PHPERJY,NPHTOT,VSQ,ERRVSQ,REDOUT,
     *       TZODI,EZODI,QE,DETFLX,NPHZOD,NPHSRC,NPHOP1,NPHOP2,
     *       REALCANC,DELPHI0,DELPHI,CDLPHI,SDLPHI,FLXSUB,SUBFAC,SUBFLX
      DOUBLE PRECISION BNU
      REAL   REC(MAXWPB*MAXB+2*MAXT), RAN, RNGAUS
      DOUBLE PRECISION FLXC(MAXMOD),X(MAXMOD),Y(MAXMOD),AA(MAXMOD),
     1       RAT(MAXMOD),PHI(MAXMOD),PHASE(MAXB),
     2       PHSERR(MAXB),FTIME(500),ERRSCL(500,MAXS),SCLFAC(MAXS),
     3       STAPHA(MAXS),HALIM(MAXS),ZALIM(MAXS),STLONG(MAXS),
     4       ERRSTN(MAXS),HALIMTE(MAXTE),ZALIMTE(MAXTE),TELONG(MAXTE),
     5       EXPECT(MAXS)
      INTEGER   ITYPE(MAXMOD)
      INTEGER   YEAR,UT,STIME(6),TYPE,LABELB(MAXB),LABELT(MAXT)
      INTEGER*2 IB1(MAXT),IB2(MAXT),IB3(MAXT)
      LOGICAL   FOUND(MAXS),UP(MAXS),OK,SCREWY, PERFCT,SRCUP,
     1          ORBITERS,LONG,TELEMETRY,TEFOUND(MAXTE),VIS,HIST,
     2          ONSOURCE,DATA,UVPLOT,INVERT,COUNT,GRSKIP,
     3          CALCERR,FREQSYNTH,OPTICL,NULLNG,SUBMOD
      COMMON /DMODEL/ FLXC,X,Y,AA,RAT,PHI,ITYPE,NCOMP
      COMMON /EARTH/ RE,GM,HJ2
C
      DATA ENDMRK/'/'/
      DATA STIME/6*0/, I0/0/, D0/0D0/, RE/6370000.D0/, GM/3.986012D14/,
     1    HJ2/1.083D-3/
      DATA PARS/'RA','DEC','TIMESCAL','NCPRINT','FLUX','YEAR','LONG',
     1         'ERRMult','*','INTEG','SOUrce',' ',MAXS*'STATions',
     2         'TITle',9*' ','NOCLOSE','DAY','STARt','STOP',
     3         MAXS*'ERRScale','NUP', 'PHASES', 'PERFECT','*',
     4         'OUTPUT',9*' ', 'MODEL',9*' ', MAXS*'ERRAdd',
     5         MAXTE*'TELEM???', 2*'NONOFF', 'NODATA', 'UVPLOT',
     6         'PLOTFile',9*' ', 'UMAX', 'VMAX', 'MARK', 'NOINVERT',
     7         'LINEWidt', 'FONT', 'DOTSIZE', 'COUNT', 'NGRDIL',
     8         MAXFRQ*'FREQuenc', MAXS*'TSYS', MAXS*'EFFicien',
     9         MAXS*'DIAMeter', MAXS*'POINTing', 'BANDwidt', 'TELANgle',
     A         'SEED', 'ROTation', 'OPTICAL', 'NPIX', 'READout',
     B         'TZODI', 'EZODI', 'QE', 'NULLING', 'NCSUB','SUBFLux' /
      DATA VALS/2*'***UNSET', 60D0, 25D0, 0D0, 0D0, -1D0,
     1         0D0, -1D0, '***UNSET', 2*' ', 2*'***UNSET', MAXS2*' ',
     2         10*' ', -1D0, 0D0, 2*'***UNSET',
     3         MAXS*0D0, 2D0, 360D0, -1D0, -1D0,
     4         'OUTPUT',9*' ', 'MODEL',9*' ',MAXS*0D0,
     5         MAXTE*' ', 1D0, 0D0, -1D0, -1D0,
     6         '?', 9*' ', 0D0, 0D0, -1D0, -1D0,
     7         1D0, 1D0, 0D0, -1D0, 1D0,
     8         '***UNSET', MAXFRQ1*0D0, MAXS*0D0, MAXS*0D0,
     9         MAXS*0D0, MAXS*0D0, 0D0, 180D0,
     A         3456757D0, SIDDAY, -1D0, 1024D0, 1D0,
     B         275D0, 1.D-7, 0.5D0, -1D0, -1D0, 0D0 /
C-----------------------------------------------------------------------
C Formats
C-----------------------------------------------------------------------
  480 FORMAT(10A8)
  510 FORMAT(A)
  515 FORMAT(' +++ERROR+++ ',A8,' cannot be used as a telemetry',/,
     1       ' station and an observatory at the same time.',/)
  520 FORMAT(A8,2X,F10.0,2(F16.0,F3.0,F6.3),4F5.0,I5,I3,I4,
     1       (F16.0,F3.0,F6.3),I5)
  525 FORMAT(' +++ERROR+++ ',A8,' cannot be used as a telemetry',/,
     1       ' station.  Only ground stations may be telemetry',/,
     1       ' stations in this version of FAKE ',/)
  530 FORMAT(' +++ERROR+++ Station not found in catalog: ',A8)
  535 FORMAT(' +++ERROR+++ No telemetry stations given.  Specify',
     1       ' list or "NONE"')
  600 FORMAT(' ',A,' generates artificial MERGE-format visibility',
     1         ' files from models'/' (Version ',A,')'/)
  604 FORMAT(///' Model parameters from:  ',A/)
  614 FORMAT(1X,'      Flux','      Sepn.','   p.a.','  Axis Ratio',
     1          '    p.a.   Type'/)
  605 FORMAT(1X,1P,E10.3,1X,0P,F10.5,1X,F6.1,1X,F6.2,1X,F4.2,1X,F7.2,I6)
  606 FORMAT(' ++WARNING++ Invalid "type": type=1 assumed')
  607 FORMAT(/' Model consists of',I4,' components'/
     1        ' Total model flux = ',1PE10.3,' Jy')
  610 FORMAT(/' ++WARNING++  Model truncated: too many components.')
  620 FORMAT(//' Stations:'//4X,'Name',16X,'X',15X,'Y',15X,'Z',
     1           6X,'HA limit'/)
  625 FORMAT(//' Orbiter Parameters at 0 hr UT on Start Date:'//
     1      4X,'Name',10X,'A',14X,'E',12X,'I',12X,'O',12X,'W',12X,'M')
  627 FORMAT(//' Telemetry Stations:'//4X,'Name',13X,'X',12X,'Y',
     1           12X,'Z',/)
  630 FORMAT(2X,A8,2X,3F16.4,2X,F5.2,2X,A)
  635 FORMAT(2X,A8,2X,F13.1,5X,F6.3,4(5X,F8.3))
  645 FORMAT(' TELEMETRY="NONE" specified:',/,
     1      ' orbiter(s) assumed to have a VLBI terminal on board.',/)
  650 FORMAT(//' Random scaling factors (by station):'/)
  660 FORMAT(1X,A8,2X,15F7.3)
  670 FORMAT(/' Output in file ',A)
  680 FORMAT('- Average ERRADD =',E9.2,' Jy',1X,'ERRMULT =',E9.2)
  690 FORMAT('- Average ERRSCAL=',E9.2,      4X,'PHASES  =',F9.2)
  700 FORMAT(' Umax = ',1PE10.2,'       Vmax = ',1PE10.2,
     1          '  million wavelengths'/
     2          ' Number of (u,v)-points = ',I6)
  701 FORMAT(' Mean epoch = ',F10.2)
  705 FORMAT(1X,A,I6,' points')
  720 FORMAT(/1X,A/1X,A)
  725 FORMAT('(',A3,I5,', ',F7.3, 1X, A,')')
  800 FORMAT(/,' ERRADD is being calculated from thermal noise. ',
     1         'ERRADD  parameter is disabled.',/)
  810 FORMAT(' +++ERROR+++ Station system temperatures (TSYS) ',
     1         'unset')
  820 FORMAT(' +++ERROR+++ Station antenna efficiencies (EFF) ',
     1         'unset')
  830 FORMAT(' +++ERROR+++ Station antenna diameters (DIAM) ',
     1         'unset')
  840 FORMAT(' +++ERROR+++ Station antenna pointing efficiencies ',
     1         '(POINT) unset')
  850 FORMAT(' +++ERROR+++ Bandwidth cannot be zero or negative')
  860 FORMAT(' +++ERROR+++ Integration time cannot be zero or ',
     1       'negative')
C---------------------------------------------------------------------
C Read control parameters and set flags
C---------------------------------------------------------------------
C
      WRITE (OUTC,600) PROGNM, VERSN
      CALL IDATE(IM,ID,IY)
      VALS(6) = IY+1900
C                ! default year
      VALS(12+MAXS+12) = 1
C                ! default day
    6 MODE = 0
      OK = .TRUE.
      WRITE (OUTC,510) ' Control parameters (end with /):'
      CALL KEYIN(PARS,VALS,NPARS,ENDMRK,MODE,INC,OUTC)
C
      RA50   = VALS(1)*RPST
      DEC50  = VALS(2)*RPSA
      IUT4   = VALS(3)*60*60
C                ! error timescale (frames)
      NCPRIN = VALS(4)
      FLUX   = VALS(5)
      YEAR   = VALS(6)
      LONG   = VALS(7).NE.-1D0
      ERRMUL = VALS(8)
C                ! ERRMULT, default 0.0
      IUT3   = VALS(10)*60
C                ! stepping interval
      WRITE(SOURCE,'(2A8)') VALS(11),VALS(12)
C
      NS=0
      DO 10 I=1,MAXS
      WRITE(STNAME(I),'(A)') VALS(12+I)
      IF (STNAME(I).EQ.' ') GOTO 20
   10 NS=NS+1
   20 IF (NS.LE.1) CALL ERROR('No stations specified.')
      NB=NS*(NS-1)/2
      NT=NB*(NS-2)/3
C
      WRITE (HISTRY, 480) (VALS(I),I=12+MAXS+1,12+MAXS+10)
      IF (VALS(12+MAXS+11).EQ.0D00) NT = 0
C                ! NOCLOSE parameter
      IDAY = VALS(12+MAXS+12)
      IUT1 = VALS(12+MAXS+13)*60
C                ! start
      IUT2 = VALS(12+MAXS+14)*60
C                ! stop
C
      DO IS=1,NS
            EXPECT(IS) = VALS(26+MAXS+IS)
      END DO
C
      NUPMIN  = VALS(26+2*MAXS+1)
      PHASES  = VALS(26+2*MAXS+2)
      PERFCT = VALS(26+2*MAXS+3).NE.-1D0
      WRITE(OUTFIL,'(10A8)') (VALS(I),I=26+2*MAXS+5,26+2*MAXS+14)
      WRITE(MODFIL,'(10A8)') (VALS(I),I=26+2*MAXS+15,26+2*MAXS+24)
C
      DO IS=1,NS
            ERRSTN(IS) = VALS(50+2*MAXS+IS)
C                ! ERRADD parameters
      END DO
C
      NST=0
      DO I=1,MAXTE
            WRITE(TENAME(I),'(A)') VALS(50+3*MAXS+I)
            IF ((TENAME(I).EQ.' ').OR.(TENAME(I).EQ.'NONE')) GOTO 2150
            DO M=1,NS
                IF (TENAME(I).EQ.STNAME(M)) THEN
                    WRITE (OUTC,515) TENAME(I)
                    OK = .FALSE.
                END IF
            END DO
            NST=NST+1
      END DO
 2150 CONTINUE
C
      NON    = VALS(50+3*MAXS+MAXTE+1)
C                ! # intervals on source
      NOFF   = VALS(50+3*MAXS+MAXTE+2)
C                ! # intervals off source
      DATA   = VALS(50+3*MAXS+MAXTE+3)  .EQ. -1D0
      UVPLOT = VALS(50+3*MAXS+MAXTE+4)  .EQ. 0D0
      WRITE(PLTFIL,'(10A8)') (VALS(I),I=50+3*MAXS+MAXTE+5,
     1      50+3*MAXS+MAXTE+14)
      UMAXP  = VALS(50+3*MAXS+MAXTE+15)*1D-6
C                ! Millions of wavelengths
      VMAXP  = VALS(50+3*MAXS+MAXTE+16)*1D-6
      MARK   = VALS(50+3*MAXS+MAXTE+17)
      INVERT = VALS(50+3*MAXS+MAXTE+18) .EQ. -1D0
      LINEWD = VALS(50+3*MAXS+MAXTE+19)
      NFONT  = VALS(50+3*MAXS+MAXTE+20)
      NDOTSZ = VALS(50+3*MAXS+MAXTE+21)
      COUNT  = VALS(50+3*MAXS+MAXTE+22) .EQ. 0D0
      NGRDIL = VALS(50+3*MAXS+MAXTE+23)
C                If UVPLOT, plot only every NGRDIL ground-ground points
      IF (NGRDIL.LE.0) NGRDIL = 1
C
      NFREQ=0
      FREQSUM = 0D0
      FREQSYNTH = .FALSE.
      DO NFRQ=1,MAXFRQ
            IF (VALS(73+3*MAXS+MAXTE+NFRQ).NE.0D0) THEN
                NFREQ = NFREQ+1
                FREQU(NFREQ) = VALS(73+3*MAXS+MAXTE+NFRQ)*1D6
                FREQSUM = FREQSUM + FREQU(NFREQ)
            END IF
      END DO
      FREQ = FREQSUM/NFREQ
      IF (NFREQ.GT.1) FREQSYNTH = .TRUE.
C
      CALCERR = .FALSE.
      DO IS=1,NS
            TSYS(IS)  = VALS(73+3*MAXS+MAXTE+MAXFRQ+IS)
            EFF(IS)   = VALS(73+4*MAXS+MAXTE+MAXFRQ+IS)
            DIAM(IS)  = VALS(73+5*MAXS+MAXTE+MAXFRQ+IS)
            POINT(IS) = VALS(73+6*MAXS+MAXTE+MAXFRQ+IS)
            CALCERR = CALCERR .OR. ((ABS(TSYS(IS))+ABS(EFF(IS))+
     1          ABS(DIAM(IS))+ABS(POINT(IS))) .NE. 0D0)
      END DO
C
C     Distribute total bandwidth over frequency bands
      BANDW = VALS(74+7*MAXS+MAXTE+MAXFRQ)*1D6/NFREQ
C
C     Determine if data are optical data and force calculation of error
      OPTICL = VALS(78+7*MAXS+MAXTE+MAXFRQ) .NE. -1D0
      IF (OPTICL) CALCERR = .TRUE.
C
      IF (CALCERR .AND. DATA) THEN
C                ! If user wants to calculate error
            IF (.NOT.OPTICL) WRITE (OUTC,800)
C                ! make sure he has entered all
            IF (TSYS(1).LE.0D0) THEN
C                ! information needed
                WRITE (OUTC,810)
                OK = .FALSE.
            END IF
            IF (EFF(1).LE.0D0) THEN
                WRITE (OUTC,820)
                OK = .FALSE.
            END IF
            IF (DIAM(1).LE.0D0) THEN
                WRITE (OUTC,830)
                OK = .FALSE.
            END IF
            IF ((POINT(1).LE.0D0) .AND. .NOT.OPTICL) THEN
                WRITE (OUTC,840)
                OK = .FALSE.
            END IF
            IF (BANDW.LE.0D0) THEN
                WRITE (OUTC,850)
                OK = .FALSE.
            END IF
            IF (IUT3.LE.0) THEN
                WRITE (OUTC,860)
                OK = .FALSE.
            END IF
      END IF
C                ! Spacecraft telemetry antenna angle limit.
      TELANGLE = VALS(75+7*MAXS+MAXTE+MAXFRQ)
C                ! Integer seed for random number generators.
      ISEED = VALS(76+7*MAXS+MAXTE+MAXFRQ)
C
C Compute rotation rate for "earth";  used for optical
C interferometry simulations when we imagine the apertures to be
C mounted on a spacecraft with a rotation period different from
C 24 sidereal hours
C
      PERIOD = VALS(77+7*MAXS+MAXTE+MAXFRQ)
      OMEGA  = SIDR*SIDDAY/PERIOD
C
C     Set CCD, zodiacal light, and other interferometer parameters
C
C     Compute effective readout noise from product of # pixels and
C     square of read noise per pixel
      REDOUT = VALS(79+7*MAXS+MAXTE+MAXFRQ)*
     1         VALS(80+7*MAXS+MAXTE+MAXFRQ)**2
C     Zodiacal light temperature and emissivity
      TZODI =  VALS(81+7*MAXS+MAXTE+MAXFRQ)
      EZODI =  VALS(82+7*MAXS+MAXTE+MAXFRQ)
C     Detector quantum efficiency
      QE    =  VALS(83+7*MAXS+MAXTE+MAXFRQ)
C     Cancelation factor for real part of visibility (nulling interf.)
      REALCANC =  VALS(84+7*MAXS+MAXTE+MAXFRQ)
      NULLNG  =  REALCANC .GE. 0D0
      IF (REALCANC .GT. 1D0) THEN
        WRITE(OUTC,'(A)') '+++ERROR+++ NULLING must be between 0 and 1'
        OK = .FALSE.
      END IF
      IF (NULLNG) DELPHI0 = 
     1             DASIN(SQRT(2.D0*REALCANC-REALCANC*REALCANC))
      NCSUB   = VALS(85+7*MAXS+MAXTE+MAXFRQ) + 0.1
      SUBMOD  = NCSUB .GT. 0
      SUBFLX  = VALS(86+7*MAXS+MAXTE+MAXFRQ)
C
C---------------------------------------------------------------------
C Time and source parameters
C---------------------------------------------------------------------
C
      IF (NUPMIN.GT.NS) CALL ERROR('NUP > number of stations')
      IF (IUT2.LE.IUT1) IUT2 = IUT2 + 86400*60
      IUT1 = (IDAY-1)*86400*60 + IUT1
      IUT2 = (IDAY-1)*86400*60 + IUT2
      IF (NOFF.LE.0) NON = 1
      NTOT = NON + NOFF
      CALL JULDA(YEAR,1,IDAY,TODAYJD,GASTM)
      GASTM = GASTM*RPST
C
      UT = (IDAY-1)*86400*60
C                ! 60ths of a second since beginning of
C                ! this year at midnight before start
      STIME(1) = YEAR-1900
      STIME(2) = 1
      STIME(3) = IDAY
      CALL SRCPAN(STIME,RA50,DEC50,RA,DEC)
C                ! precession to midnight before start
      WAVEMIN = 1D32
      DO NFRQ=1,NFREQ
            WAVEMIN = MIN(VELC/FREQU(NFRQ), WAVEMIN)
      END DO
      COSDEC = COS(DEC)
      SINDEC = SIN(DEC)
      COSRA  = COS(RA)
      SINRA  = SIN(RA)
      SVEC(1) = COSDEC*COSRA
      SVEC(2) = COSDEC*SINRA
      SVEC(3) = SINDEC
      ROT(1,1) = -SINRA
      ROT(1,2) = COSRA
      ROT(1,3) = 0.D0
      ROT(2,1) = -SINDEC*COSRA
      ROT(2,2) = -SINDEC*SINRA
      ROT(2,3) = COSDEC
      ROT(3,1) = COSDEC*COSRA
      ROT(3,2) = COSDEC*SINRA
      ROT(3,3) = SINDEC
C
C
C-----------------------------------------------------------------------
C File names
C-----------------------------------------------------------------------
C
      IF (DATA) THEN
          IER = VLBOPE(OUT, OUTFIL, 'MERGE', 'NEW', OUTFIL)
          IF (IER.NE.1) CALL ERROR('Cannot create file OUTPUT.')
          IER = VLBOPE(INMOD, MODFIL, 'MODEL', 'OLD', MODFIL)
          IF (IER.NE.1) CALL ERROR('Cannot find file MODEL.')
      END IF
C
      IF (UVPLOT) THEN
          IF (NDOTSZ.LT.1) NDOTSZ = LINEWD
          IER = PGBEG(0,PLTFIL,1,1)
          IF (IER.NE.1) CALL ERROR('Cannot open file PLOTFILE: '//
     1                               PLTFIL)
          CALL PGQINF('DEV/TYPE', PLTFIL, L)
          WRITE (OUTC,'(A)') ' Plot will go to '//PLTFIL(:L)
      END IF
C
C---------------------------------------------------------------------
C Station coordinates: search catalog
C---------------------------------------------------------------------
C
C
C Baseline and triangle labels for header,
C and constituent baselines of each triangle
      IB=0
      DO 31 IS2=2,NS
      DO 31 IS1=1,IS2-1
          IB = IB+1
   31     LABELB(IB) = 100*IS1 + IS2
      IT = 0
      DO 32 IS3=3,NS
      DO 32 IS2=2,IS3-1
      DO 32 IS1=1,IS2-1
          IT = IT+1
          IB1(IT) = IS1 + (IS2-1)*(IS2-2)/2
          IB2(IT) = IS1 + (IS3-1)*(IS3-2)/2
          IB3(IT) = IS2 + (IS3-1)*(IS3-2)/2
   32 LABELT(IT) = 10000*IS1 + 100*IS2 + IS3
C
      DO 2100 I=1,NS
          FOUND(I) = .FALSE.
 2100 CONTINUE
C
      DO I=1,NST
          TEFOUND(I) = .FALSE.
      END DO
C
C
      OPEN ( UNIT=INSTA, NAME='STATIONS', READONLY,
     1       FORM='FORMATTED', STATUS='OLD', ERR=2900 )
C
      ORBITERS = .FALSE.
2200  READ(INSTA,520,END=2400) XON, RR, TD,TM,TS, PD,PM,PS, XZL,XHL,
     1   Q5, Q6, IY, IM, ID, UTH, UTM, UTS, STTYPE
      IF (STTYPE.LE.1) STTYPE = 1
      DO 2300 M=1,NS
          IF (XON.EQ.STNAME(M)) THEN
                STATYP(M) = STTYPE
                IF (STTYPE.EQ.1) THEN
                    TT = TD + TM/60D0 + TS/3600D0
C                ! latitude(deg)
                    PP = PD + PM/60D0 + PS/3600D0
C                ! w longitude(deg)
                    IF (RR.LT.1D6) CALL GEOID(RR,TT)
                    SLAT(M) = SIN(TT*RPDEG)
                    CLAT(M) = COS(TT*RPDEG)
                    STLONG(M) = PP*RPDEG
                    STX(M) = RR*CLAT(M)*COS(PP*RPDEG)
                    STY(M) = RR*CLAT(M)*SIN(PP*RPDEG)
                    STZ(M) = RR*SLAT(M)
                    IF (XZL.EQ.0.0) XZL = 90.0
                    IF (XHL.EQ.0.0) XHL = 12.0
                    ZALIM(M) = XZL
                    HALIM(M) = XHL
                ELSE IF (STTYPE.EQ.2) THEN
                    ORBITERS = .TRUE.
                    AEIOWM(1,M) = RR
                    AEIOWM(2,M) = TD
                    AEIOWM(3,M) = PD*RPDEG
                    AEIOWM(4,M) = XZL*RPDEG
                    AEIOWM(5,M) = Q5*RPDEG
                    AEIOWM(6,M) = Q6*RPDEG
                    UTORB = UTH + UTM/60.D0 + UTS/3600.D0
                    CALL JULDA(IY,IM,ID,EPOCHJD(M),DUMGST)
                    EPOCHJD(M) = EPOCHJD(M) + UTORB/24.D0
C                   Propagate orbits from epoch date to 0 hr UT today
                    DT = (TODAYJD - EPOCHJD(M))*86400.D0
                    CALL PROPGAT(AEIOWM(1,M),DT)
                END IF
                FOUND(M) = .TRUE.
                GOTO 2300
          END IF
 2300 CONTINUE
      DO 2350 M=1,NST
          IF (XON.EQ.TENAME(M)) THEN
                IF (STTYPE.EQ.1) THEN
                    TT = TD + TM/60D0 + TS/3600D0
C                ! latitude(deg)
                    PP = PD + PM/60D0 + PS/3600D0
C                ! w longitude(deg)
                    IF (RR.LT.6D6) CALL GEOID(RR,TT)
                    SLATTE(M) = SIN(TT*RPDEG)
                    CLATTE(M) = COS(TT*RPDEG)
                    TELONG(M) = PP*RPDEG
                    TEX(M) = RR*CLATTE(M)*COS(PP*RPDEG)
                    TEY(M) = RR*CLATTE(M)*SIN(PP*RPDEG)
                    TEZ(M) = RR*SLATTE(M)
                    IF (XZL.EQ.0.0) XZL = 90.0
                    IF (XHL.EQ.0.0) XHL = 12.0
                    ZALIMTE(M) = XZL
                    HALIMTE(M) = XHL
                ELSE IF (STTYPE.EQ.2) THEN
                    WRITE (OUTC,525) TENAME(M)
                    OK = .FALSE.
                END IF
                TEFOUND(M) = .TRUE.
                GOTO 2350
          END IF
 2350 CONTINUE
      GOTO 2200
C
 2400 CLOSE(UNIT=INSTA)
      DO 2500 M=1,NS
          IF (.NOT.FOUND(M)) THEN
              WRITE(OUTC,530) STNAME(M)
              OK = .FALSE.
          END IF
 2500 CONTINUE
      IF (ORBITERS) THEN
            IF (TENAME(1).EQ.' ') THEN
                WRITE(OUTC,535)
                OK = .FALSE.
            ELSE IF (TENAME(1).EQ.'NONE') THEN
                TELEMETRY = .FALSE.
            ELSE
                TELEMETRY = .TRUE.
                DO 2550 M=1,NST
                    IF (TEFOUND(M)) GOTO 2550
                    WRITE(OUTC,530) TENAME(M)
                    OK = .FALSE.
 2550           CONTINUE
            END IF
      END IF
      IF ((.NOT.OK).AND.MODE.EQ.1)
     1      CALL ERROR('Parameter entry failed.')
      IF (.NOT.OK) GOTO 6
C                ! ask user to try again
      GOTO 3000
C
 2900 CALL ERROR('Cannot read STATIONS catalog')
 3000 CONTINUE
C
C---------------------------------------------------------------------
C DATA Section
C---------------------------------------------------------------------
C
      IF (DATA) THEN
C
C Read the model parameters from MODFIL
C
            WRITE(OUTC,604) MODFIL(1:LEN1(MODFIL))
            WRITE(OUTC,614)
            TOTFLX = 0.0D0
            FLXSUB = 0D0
            DO I=1,MAXMOD
                CALL DGTMDL(INMOD,TYPE,FLXC(I),RAD,THETA,A,
     1                      RAT(I),PH,*2,OUTC)
                IF (FLXC(I).EQ.0.0) GOTO 2
                IF (I.LE.NCPRIN) WRITE(OUTC,605) FLXC(I),RAD,THETA,A,
     1              RAT(I),PH,TYPE
                IF (TYPE.LT.0 .OR. TYPE.GT.5) THEN
                    WRITE(OUTC,606)
                    TYPE = 1
                END IF
                IF (I.LE.NCSUB) THEN
                  FLXSUB = FLXSUB+FLXC(I)
                END IF
                ITYPE(I) = TYPE
                TOTFLX = TOTFLX+FLXC(I)
                ARG = THETA*RPDEG
                X(I) = SIN(ARG)*RAD*RPMAS
                Y(I) = COS(ARG)*RAD*RPMAS
                AA(I) = A*RPMAS
                PHI(I) = PH*RPDEG
            END DO
            WRITE(OUTC,610)
            I = MAXMOD+1
    2       NCOMP = I-1
            WRITE(OUTC,607) NCOMP,TOTFLX
            IF (SUBMOD) THEN
            IF (SUBFLX .NE. 0D0) THEN
C             Use user-supplied flux, if available, to scale
C             subtracted model
              SUBFAC = SUBFLX/FLXSUB
              WRITE(OUTC,'(2A)') 'Using user-supplied flux',
     1                    ' to scale subtracted model'
            ELSE
C             Otherwise, use total flux of model
              SUBFAC = TOTFLX/FLXSUB
              WRITE(OUTC,'(2A)') 'Using total model flux',
     1                    ' to scale subtracted model'
            END IF
            END IF
            IF (TOTFLX.EQ.0.0) CALL ERROR('Model has zero total flux.')
            CLOSE (UNIT=INMOD)
            IF (FLUX.LE.0D0) FLUX = TOTFLX
C
C Determine Error Parameters
C
            IF (OPTICL) THEN
C             If optical data, assume TSYS, etc. data are thermal
C             data on the mirrors; efficiency is the reflectance.
C             Allow user to still employ ERRADD and ERRMUL parameters
C             by not replacing these with a system noise calculation
              DO IS=2,NS
                  IF (TSYS(IS).LE.0D0)  TSYS(IS)  = TSYS(IS-1)
                  IF (EFF(IS).LE.0D0)   EFF(IS)   = EFF(IS-1)
                  IF (DIAM(IS).LE.0D0)  DIAM(IS)  = DIAM(IS-1)
                  IF (ERRSTN(IS).LE.0D0) ERRSTN(IS) = ERRSTN(IS-1)
              END DO
            ELSE
C             Calculate error for radio data from ERRADD or TSYS, etc.
C             parameters; if TSYS, etc. are specified, replace ERRADD
C             with radio receiver noise computation
              IF (ERRMUL.LT.0.0) CALL
     1            ERROR('ERRMULT may not be negative')
              IF (ERRSTN(1).LE.0D0) ERRSTN(1) = 0.02*FLUX
              DO IS=2,NS
                  IF (ERRSTN(IS).LE.0D0) ERRSTN(IS) = ERRSTN(IS-1)
              END DO
              IF (CALCERR) THEN
C                ! Calculate station errors and
C                ! turn off ERRADD parameter
                  DO IS=2,NS
                      IF (TSYS(IS).LE.0D0)  TSYS(IS)  = TSYS(IS-1)
                      IF (EFF(IS).LE.0D0)   EFF(IS)   = EFF(IS-1)
                      IF (DIAM(IS).LE.0D0)  DIAM(IS)  = DIAM(IS-1)
                      IF (POINT(IS).LE.0D0) POINT(IS) = POINT(IS-1)
                  END DO
                  DO IS=1,NS
                      ERRSTN(IS) = TSYS(IS)/
     1                  (2D-4*SQRT(BANDW*IUT3/60)*EFF(IS)*
     2                  DIAM(IS)*DIAM(IS)*POINT(IS))
                  END DO
              END IF
            END IF
C
            SCREWY = .FALSE.
            ERRSCA = 0.0
            DO IS=1,NS
              ERRSCA = ERRSCA + EXPECT(IS)
              IF (EXPECT(IS).NE.0D0) SCREWY = .TRUE.
            END DO
            ERRSCA = ERRSCA/NS
            IF (PERFCT) THEN
              SCREWY = .FALSE.
              ERRSCA = 0.0
              PHASES = 0.0
            END IF
            IF (SCREWY) THEN
              NTIME = 0
              DO 100 IUT=IUT1,IUT2+IUT4,IUT4
              NTIME = NTIME+1
              IF (NTIME.GT.500) CALL ERROR('TIMESCALE too short')
              FTIME(NTIME) = IUT - IUT1
              DO 100 IS=1,NS
              SCAL = 1.0
              IF (EXPECT(IS).GT.0.0) CALL FAKSCL(SCAL,EXPECT(IS),ISEED)
  100         ERRSCL(NTIME,IS) = SCAL
            END IF
C
C Write out Header Records
C
            NVER = 1
            NDWPB = 4
            IF (ORBITERS .OR. ABS(PERIOD-SIDDAY).GT.1D-5) THEN
                NVER = 2
C                ! MERGE-format 2 has U,V, and
C                ! W stored for each baseline
                NDWPB = MAXWPB
            END IF
            IF (FREQSYNTH) THEN
C                ! MERGE-format 3 is just like
C                ! format 2 except that more
C                ! than one frequency band has
C                ! been specified
                NVER = 3
                NDWPB = MAXWPB
            END IF
            NHIST = 5
            HIST = .FALSE.
            IF (HISTRY.NE.' ') THEN
                NHIST = NHIST + 1
                HIST = .TRUE.
            END IF
            IF (ORBITERS) NHIST = NHIST + 1
            JMAX = NDWPB*NB+2*NT
            LENG = JMAX+1
            NHEAD = 2+NHIST+NS
            IF (NB.GT.0) NHEAD = NHEAD+1
            IF (NT.GT.0) NHEAD = NHEAD+1
            KFLAG = 1
            WRITE(OUT) NVER,NHIST,NHEAD,NS,NB,NT,LENG,KFLAG,(I0,J=9,20)
            IF (NB.GT.0) WRITE(OUT) (LABELB(IB),IB=1,NB)
            IF (NT.GT.0) WRITE(OUT) (LABELT(IT),IT=1,NT)
C
C               History (5, 6, or 7 records)
            IF (HIST) WRITE(OUT) HISTRY
            CALL MKHIST(PROGNM, VERSN, HISTRY)
            WRITE(OUT) HISTRY
            HISTRY = '- Input model: '//MODFIL
            WRITE(OUT) HISTRY
            IF (PERFCT) THEN
                HISTRY = '- No errors added to model'
            ELSE
                ERRADD = 1.0
                DO IS=1,NS
                    ERRADD = ERRADD*(ERRSTN(IS)**(1.0/NS))
                END DO
                WRITE (HISTRY, 680) ERRADD,ERRMUL
            END IF
            WRITE(OUT) HISTRY
            WRITE (HISTRY, 690) ERRSCA,PHASES
            WRITE(OUT) HISTRY
            IF (ORBITERS) THEN
                NSTOP = MAX(MIN(NST,15),1)
                NPRT = 5
                IF (NSTOP.GT.12) NPRT = 4
                HISTRY = '- TELEM='
                NFIRST = 8
                IF (NSTOP.EQ.15) THEN
                    HISTRY = '- TLM='
                    NFIRST = 6
                END IF
                DO IST=1,NSTOP
                    J1 = NFIRST + (IST-1)*(NPRT+1) + 1
                    J2 = J1 + NPRT - 1
                    WRITE(HISTRY(J1:J2),'(A)') TENAME(IST)(1:NPRT)
                    IF (IST.NE.NSTOP)
     1                  WRITE(HISTRY(J2+1:J2+1),'(A1)') ','
                END DO
                WRITE(OUT) HISTRY
            END IF
            HISTRY = '- Output file: '//OUTFIL
            WRITE(OUT) HISTRY
C
C               Source
            WRITE(OUT) SOURCE,RA50,DEC50,RA,DEC,FLUX,(D0,J=8,10)
C
C               Stations
            IF (NVER.EQ.1) THEN
                DO I=1,NS
                    WRITE(OUT)
     1                STNAME(I),STX(I),STY(I),STZ(I),(D0,J=5,10)
                END DO
            ELSE
                DO I=1,NS
                    IF (STATYP(I).EQ.1) WRITE(OUT) STNAME(I),STX(I),
     1                  STY(I),STZ(I),(D0,J=5,10),STATYP(I)
                    IF (STATYP(I).EQ.2) THEN
                        WRITE(OUT) STNAME(I),(AEIOWM(J,I),J=1,6),
     1                      (D0,J=8,10),STATYP(I)
                    END IF
                END DO
            END IF
C
C               Miscellaneous
            INTEG = IUT3/60D0
            WRITE(OUT) YEAR,UT,GASTM,FREQ,INTEG,D0,BANDW,(D0,J=7,10)
      END IF
C                ! End DATA Section
C
C---------------------------------------------------------------------
C UV and Visibility Calculation
C---------------------------------------------------------------------
C
C HA and ZA limits; Change to R.H. coordinate system
C
      WRITE(OUTC,620)
      DO 200 IS=1,NS
            IF (STATYP(IS).EQ.1) THEN
                COSZAL = COS(ZALIM(IS)*RPDEG)
                COSHAL = (COSZAL-SLAT(IS)*SINDEC)/(CLAT(IS)*COSDEC)
                IF (COSHAL.GE.1.0) THEN
                    NOTE = 'Source never visible'
                    HAL = 0.0
                ELSE IF (COSHAL.LE.-1.0) THEN
                    NOTE = 'Circumpolar source'
                    HAL = 12.0
                ELSE
                    NOTE = ' '
                    HAL = DACOS(COSHAL)/RPDEG/15.0D0
                END IF
                HALIM(IS) = MIN(HALIM(IS),HAL)
                WRITE(OUTC,630) STNAME(IS),STX(IS),STY(IS),STZ(IS),
     1                    HALIM(IS),NOTE
                RHST0(1,IS) = STX(IS)
                RHST0(2,IS) = -STY(IS)
                RHST0(3,IS) = STZ(IS)
            END IF
  200 CONTINUE
      IF (ORBITERS) THEN
            WRITE(OUTC,625)
            DO IS=1,NS
                IF (STATYP(IS).EQ.2) WRITE (OUTC,635) STNAME(IS),
     1              (AEIOWM(J,IS),J=1,2),((AEIOWM(J,IS)/RPDEG),J=3,6)
            END DO
            WRITE(OUTC,627)
            IF (TELEMETRY) THEN
                DO IST=1,NST
                    WRITE(OUTC,630) TENAME(IST),TEX(IST),TEY(IST),
     1                  TEZ(IST)
                    RHTE0(1,IST) = TEX(IST)
                    RHTE0(2,IST) = -TEY(IST)
                    RHTE0(3,IST) = TEZ(IST)
                END DO
            ELSE
                WRITE (OUTC,645)
            END IF
      END IF
C
      DO 220 IS=1,NS
  220     SCLFAC(IS) = 1.0
      IF (SCREWY.AND.LONG) WRITE(OUTC,650)
C
C-----------------------------------------------------------------------
C UVPLOT Section
C-----------------------------------------------------------------------
C
      IF (UVPLOT) THEN
C
C           Plot Legends
C
            IY = YEAR
            IM = 1
            ID = IDAY
            CALL TDATE(IY,IM,ID,JD,MONTH,DAY)
            LEGEN1 = SOURCE
            I = LEN1(LEGEN1)+3
            CALL PRFREQ(FREQ, FACTOR, FUNIT)
            WRITE (LEGEN1(I:),725) MONTH, IY, FREQ/FACTOR, FUNIT
            LEGEN2 = STNAME(1)
            NS2 = MIN(14,NS)
            DO IS=2,NS2
                I = LEN1(LEGEN2)+2
                LEGEN2(I:) = STNAME(IS)
            END DO
            IF (NS.GE.15) LEGEN3 = STNAME(15)
            NS3 = MIN(28,NS)
            DO IS=NS2+2,NS3
                I = LEN1(LEGEN3)+2
                LEGEN3(I:) = STNAME(IS)
            END DO
            IF (NS.GE.29) LEGEN4 = STNAME(29)
            NS4 = MIN(42,NS)
            DO IS=NS3+2,NS4
                I = LEN1(LEGEN4)+2
                LEGEN4(I:) = STNAME(IS)
            END DO
            WRITE(OUTC,720) LEGEN1(1:LEN1(LEGEN1)),
     1                      LEGEN2(1:LEN1(LEGEN2)),
     2                      LEGEN3(1:LEN1(LEGEN3)),
     3                      LEGEN4(1:LEN1(LEGEN4))
C
C           Initialization and Plot Limits
C
            EPOCH = 0.0
            DO IB=1,NB
                BCOUNT(IB) = 0
                IS1 = LABELB(IB)/100
                IS2 = LABELB(IB)-100*IS1
                L1 = LEN1(STNAME(IS1))
                L2 = LEN1(STNAME(IS2))
                BNAME(IB) = STNAME(IS1)(1:L1)//'-'//STNAME(IS2)(1:L2)
            END DO
C
            UMAXC = 0.0
            VMAXC = 0.0
            DO M2=2,NS
            NS1 = M2-1
            CALL UVMX(STX(M2),STY(M2),STZ(M2),AEIOWM(1,M2),COSRA,
     1          SINRA,COSDEC,SINDEC,STATYP(M2),UMX2,VMX2)
            DO M1=1,NS1
                CALL UVMX(STX(M1),STY(M1),STZ(M1),AEIOWM(1,M1),COSRA,
     1              SINRA,COSDEC,SINDEC,STATYP(M1),UMX1,VMX1)
                IF ((STATYP(M1).EQ.1) .AND. (STATYP(M2).EQ.1)) THEN
                    BXIB = (STX(M1)-STX(M2))
                    BYIB = (STY(M1)-STY(M2))
                    BZIB = (STZ(M1)-STZ(M2))
                    BEIB = SQRT(BXIB**2+BYIB**2)
                    UMAXC = MAX(UMAXC,BEIB)
                    VMAXC = MAX(VMAXC, ABS(BZIB*COSDEC) +
     1                                 ABS(BEIB*SINDEC) )
                ELSE
                    UMAXC = MAX(UMAXC,(ABS(UMX1)+ABS(UMX2)))
                    VMAXC = MAX(VMAXC,(ABS(VMX1)+ABS(VMX2)))
                END IF
            END DO
            END DO
            UMAXC = UMAXC*1E-6/WAVEMIN
C                ! Millions of wavelengths
            VMAXC = VMAXC*1E-6/WAVEMIN
            UMAX  = 1.2*UMAXC
            VMAX  = 1.2*VMAXC
            IF (UMAXP.GT.0.0) UMAX = UMAXP
            IF (VMAXP.GT.0.0) VMAX = VMAXP

          CALL PGQINF('TERMINAL', STRING, L)
Cvms      IF (STRING.EQ.'YES')
Cvms 1        CALL LIB$GET_COMMAND(STRING, 'Type <CR> to see plot: ', L)
          CALL PGBBUF
          CALL PGSLW(LINEWD)
          CALL PGSCF(NFONT)
          CALL PGSCI(5)
          CALL PGENV(SNGL(UMAX),SNGL(-UMAX),SNGL(-VMAX),SNGL(VMAX),
     1              1,1)
          CALL PGLAB('u  (10\u6\d \gl)','v  (10\u6\d \gl)',' ')
          CALL PGMTXT('T',2.5,0.5,0.5,LEGEN1)
          CALL PGMTXT('T',0.8,0.5,0.5,LEGEN2)
          CALL PGSLW(NDOTSZ)
      END IF
C
C---------------------------------------------------------------------
C Generate the data records
C---------------------------------------------------------------------
C
C Propagate the orbital parameters to beginning of experiment
C
      DT = (IUT1-UT)/60D0
      NTYME = 0
C
      DO 400 IUT=IUT1,IUT2,IUT3
      IF (IUT.GT.IUT1) DT = IUT3/60D0
C                ! Propagate orbiters forward
C
C Is telescope on source?
C
      NTYME = NTYME + 1
      JTYME = MOD(NTYME,NTOT)
      IF (JTYME.EQ.0) JTYME = NTOT
      ONSOURCE = (JTYME.GE.1) .AND. (JTYME.LE.NON)
C
C Do we want to skip ground-ground baselines now?
C
      GRSKIP = MOD(NTYME,NGRDIL) .NE. 0
C
C Calculate scaling factors; YNTERP is a CIT Computing Center subroutine
C
      IF (SCREWY) THEN
          DO 310 IS=1,NS
  310     SCLFAC(IS)=YNTERP(FTIME,ERRSCL(1,IS),DBLE(IUT-IUT1),NTIME,2)
          Q = TIMEC(MOD(IUT,86400*60)/60)
          IF (LONG) WRITE(OUTC,660) Q,(SCLFAC(IS),IS=1,NS)
      END IF
C
C Random station phases (uniform in 0-PHASES; eg PHASES=0
C gives no errors, PHASES=360 gives completely random phases)
C
      DO IS=1,NS
            IF (PERFCT) THEN
                STAPHA(IS) = 0.0
            ELSE
                STAPHA(IS) = PHASES*RAN(ISEED)
            END IF
      END DO
C
C Is source up?
C
      NUP = 0
      GST = (IUT-UT)*OMEGA + GASTM
      GSTM = MOD(GST,TWOPI)
      COST = COS(GSTM)
      SINT = SIN(GSTM)
      DO 315 IS=1,NS
            IF (STATYP(IS).EQ.1) THEN
                HA = (GST-RA-STLONG(IS))/RPDEG/15.0D0
                HA = MOD(HA,24.0D0)
                IF (HA.LT.-12.0) HA = HA+24.0D0
                IF (HA.GT.12.0)  HA = HA-24.0D0
                UP(IS) = ABS(HA).LT.HALIM(IS) .AND. ONSOURCE
                RHST(1,IS) = RHST0(1,IS)*COST - RHST0(2,IS)*SINT
                RHST(2,IS) = RHST0(1,IS)*SINT + RHST0(2,IS)*COST
                RHST(3,IS) = RHST0(3,IS)
            ELSE IF (STATYP(IS).EQ.2) THEN
C                ! propagate orbiters even
C                ! if .NOT. ONSOURCE
                CALL PROPGAT(AEIOWM(1,IS),DT)
                CALL CARTES(RHST(1,IS),GM,AEIOWM(1,IS),AEIOWM(2,IS),
     1              AEIOWM(3,IS),AEIOWM(4,IS),AEIOWM(5,IS),
     2              AEIOWM(6,IS))
C                ! Check for Earth occultation.
                UP(IS) = SRCUP(RHST(1,IS),SVEC(1)) .AND. ONSOURCE
                IF ((TELEMETRY) .AND. UP(IS)) THEN
C                ! Satellite can see source --
C                ! now check that a telemetry
C                ! station can see satellite.
                    VIS = .FALSE.
                    DO IST=1,NST
                        RHTE(1) = RHTE0(1,IST)*COST - RHTE0(2,IST)*SINT
                        RHTE(2) = RHTE0(1,IST)*SINT + RHTE0(2,IST)*COST
                        RHTE(3) = RHTE0(3,IST)
                        DO J=1,3
                            RHSTP(J) = RHST(J,IS) - RHTE(J)
                        END DO
C
C                       THE PROCEDURE BELOW IS NECESSARY TO
C                       HANDLE EQUATORIAL AS WELL AS ALTAZ TELEMETRY
C                       STATIONS. THE "RA" AND "DEC" OF THE SATELITE
C                       ARE USED IN THE COMPUTATION. "RHSTP" IS THE
C                       POSITION VECTOR OF THE SATELITE RELATIVE TO THE
C                       TELEMETRY STATION.
C
                        IF (RHSTP(1).EQ.0D0) THEN
                            IF (RHSTP(2).GT.0D0) SATRA = PI/2D0
                            IF (RHSTP(2).LT.0D0) SATRA = -PI/2D0
                        ELSE
                            SATRA = DATAN2(RHSTP(2),RHSTP(1))
                            IF (SATRA.LT.0D0) SATRA = SATRA + 2D0*PI
                        END IF
                        SINSATDEC = RHSTP(3)/
     1                   SQRT(RHSTP(1)**2 + RHSTP(2)**2 + RHSTP(3)**2)
                        COSSATDEC = SQRT(1.D0 - SINSATDEC**2)
                        COSZAL = COS(ZALIMTE(IST)*RPDEG)
                        COSHAL = (COSZAL - SLATTE(IST)*SINSATDEC)/
     1                      (CLATTE(IST)*COSSATDEC)
                        IF (COSHAL.GE.1.0) THEN
                            HAL = 0.0
                        ELSE IF (COSHAL.LE.-1.0) THEN
                            HAL = 12.0
                        ELSE
                            HAL = DACOS(COSHAL)/RPDEG/15.0D0
                        END IF
                        HAL = MIN(HALIMTE(IST),HAL)
                        HA = (GST-SATRA-TELONG(IST))/RPDEG/15.0D0
                        HA = MOD(HA,24.0D0)
                        IF (HA.LT.-12.0) HA = HA+24.0D0
                        IF (HA.GT.12.0)  HA = HA-24.0D0
C                ! Check pointing limits
C                ! for the spacecraft's
C                ! telemetry ant.  TANG
C                ! is the angle between
C                ! the spacecraft axis
C                ! and the direction of
C                ! the telemetry ant.
C                ! TANG = 0 if telem ant
C                ! points in opposite
C                ! direction from main
C                ! spacecraft ant.
                        TANG = (RHSTP(1)*SVEC(1) + RHSTP(2)*SVEC(2) +
     1                         RHSTP(3)*SVEC(3)) /
     2                         (SQRT(RHSTP(1)**2 + RHSTP(2)**2 +
     3                         RHSTP(3)**2))
C                ! Dot product.
                        TANG = DACOS(TANG)
C                ! Convert to radians.
                        TANG = TANG / RPDEG
C                ! Convert to degrees.
                        TANG = 180.D0 - TANG
                        IF (TANG .LT. 0.D0) TANG = -1.0D0 * TANG
C                ! Set VIS true if
C                ! everything is OK.
                        VIS = (ABS(HA).LT.HAL) .AND. (TANG.LE.TELANGLE)
                        IF (VIS) GO TO 312
                    END DO
  312               CONTINUE
                    UP(IS) = UP(IS).AND.VIS
                END IF
            END IF
            IF (UP(IS)) NUP = NUP+1
  315 CONTINUE
      IF (NUP.LT.NUPMIN) GOTO 400
C
C Calculate amp and phase
C
      DO 390 NFRQ=1,NFREQ
      WAVE = VELC/FREQU(NFRQ)
      IF (OPTICL) THEN
C
C       Compute total number of photons contributed by Zodiacal
C       light. Note that diameter of mirror squared appears in
C       mirror area factor, but also appears in denominator
C       of beam area, leaving a factor of lambda**2 only.
        HNU = PLANCK*FREQU(NFRQ)
        NPHZOD  = EZODI*BNU(FREQU(NFRQ),TZODI)*WAVE*WAVE*BANDW*IUT3
     1          / (60D0*HNU*(NS-1))
      END IF
      IPOS = 0
C
      DO 320 IB=1,NB
      IS1 = LABELB(IB)/100
      IS2 = LABELB(IB)-100*IS1
      IF (UP(IS1).AND.UP(IS2)) THEN
            BXIB = RHST(1,IS1) - RHST(1,IS2)
C                ! Compute intantaneous baseline
            BYIB = RHST(2,IS1) - RHST(2,IS2)
            BZIB = RHST(3,IS1) - RHST(3,IS2)
            U = (ROT(1,1)*BXIB + ROT(1,2)*BYIB)/WAVE
            V = (ROT(2,1)*BXIB + ROT(2,2)*BYIB +ROT(2,3)*BZIB)/WAVE
            IF (UVPLOT .AND.
     1        .NOT.(GRSKIP .AND. (STATYP(IS1).EQ.1) .AND.
     2        (STATYP(IS2).EQ.1))) THEN
C                ! Skip if GRSKIP and ground-ground
                UU = U*1E-6
                VV = V*1E-6
                BCOUNT(IB) = BCOUNT(IB)+1
                EPOCH = EPOCH + DBLE(IUT)/(365.25D0*24*3600*60)
                CALL PGSCI(3)
                CALL PGPT(1,SNGL(UU),SNGL(VV),MARK)
                IF (INVERT) THEN
                    CALL PGSCI(2)
                    CALL PGPT(1,SNGL(-UU),SNGL(-VV),MARK)
                END IF
            END IF
            IF (DATA) THEN
                W = (ROT(3,1)*BXIB + ROT(3,2)*BYIB +ROT(3,3)*BZIB)/WAVE
                ERRADD = SQRT(ERRSTN(IS1)*ERRSTN(IS2))
C
C                ! Needed for error bars
                CALL CMODEL(U,V,VISR,VISI)
                IF (OPTICL .AND. .NOT. PERFCT) THEN
C
C                 Compute the visibility amplitude for an optical
C                 interferometer.  The observables are, as in the
C                 radio case, just VISR and VISI. (That is, the D.C.
C                 component of the total intensity has been subtracted).
C                 However, the noise depends on the total number
C                 of photons incident on the detector, including
C                 the number from the source,
C                 the number from the warm optics, and
C                 the number from the zodiacal light
C
C                 Compute number of photons per Jy, assuming photons are
C                 divided evenly among the NS antennas;  note: 240 = 60
C                 (from IUT3 in 60ths of sec) times 4 (from 1/2 DIAM**2)
                  PHPERJY = PI*(DIAM(IS1)**2+DIAM(IS2)**2)*1.D-26*
     1                      BANDW*IUT3 / (240D0*HNU*(NS-1))
C
C                 Number of photons detected from source:
C                 For a normal (adding) interferometer, this is just the
C                 TOTAL number of photons from the source (as both the real
C                 and imaginary part of the fringe are scanned)
                  SRCFLX = FLUX
C
                  IF (NULLNG) THEN
C
C                   The visibility amplitude is different for a nulling
C                   (or subtracting) interferometer.
C                   Rotate fringes by cancellation error; this must be done
C                   before computation of detected flux for a nulling
C                   interferometer
                    DELPHI = DELPHI0*RNGAUS(ISEED)
                    CDLPHI = COS(DELPHI)
                    SDLPHI = SIN(DELPHI)
                    VISR = VISR*CDLPHI - VISI*SDLPHI
C                   VISI = VISR*SDLPHI + VISI*CDLPHI
C                   For a nulling interferometer the total detected flux
C                   is cancelled by the real part of the visibility.
C                   Note that, because of the cancellation errors, the
C                   cancellation is not exact for a point source at the
C                   phase center.
                    SRCFLX = FLUX - VISR
C                   For a nulling interferometer, both real and imaginary
C                   parts of the visibility are not scanned, only the
C                   negative of the real part plus a constant (the FLUX)
C                   is measured.  The real part is derived from this
C                   measurement.
                    VISR = -SRCFLX
                    VISI = 0D0
                  END IF
                  NPHSRC = PHPERJY*SRCFLX
C
C                 Number of photons contributed by warm optics:
C                 Note that focal ratio of mirror (focal length divided
C                 by diameter) squared appears in factor involving spot
C                 area on detector, but also appears in denominator of
C                 solid angle of mirror subtended at detector, leaving
C                 a factor of lambda**2 only.
                  NPHOP1 = (1D0-EFF(IS1))*BNU(FREQU(NFRQ),TSYS(IS1))*
     1                      WAVE*WAVE*BANDW*IUT3/(60D0*HNU*(NS-1))
                  NPHOP2 = (1D0-EFF(IS2))*BNU(FREQU(NFRQ),TSYS(IS2))*
     1                      WAVE*WAVE*BANDW*IUT3/(60D0*HNU*(NS-1))
C
C                 Compute total number of photons taking part in visibility,
C                 including zodiacal light from both mirrors (computed
C                 earlier) and taking into account quantum efficiency.
C                 Also compute total inferred flux (in Jy) on detector
                  NPHTOT = QE*(NPHSRC + NPHOP1 + NPHOP2 + 2D0*NPHZOD)
                  DETFLX = NPHTOT/(QE*PHPERJY)
C
C                 Compute mean amplitude and phase
                  AMP = SQRT(VISR**2+VISI**2)
                  PHS = 0.0
                  IF (AMP.GT.0.0) PHS = DATAN2(VISI,VISR)/RPDEG +
     1                STAPHA(IS1) - STAPHA(IS2) + 720.0D0
                  PHS = MOD(PHS,360.0D0)
C
C
C                 Apply photon statistical noise to incoming photons
C
                  IF (NULLNG) THEN
C                   For a nulling interferometer, noise is determined by
C                   total detected flux; there is no photon-starved term
C                   (according to M. Colavita ????)
                    ERRAMP = DETFLX*SQRT(PI*PI*REDOUT
     1                      /(4D0*NPHTOT*NPHTOT) + 1D0/NPHTOT)
                    AMP = AMP + ERRAMP*RNGAUS(ISEED)
                    AMP = MAX(AMP,0D0)
C
                    ERRPHS = 180.0D0
                  ELSE
C                   For a regular interferometer, noise is determined by
C                   the measured fringe visibility. The latter is computed
C                   by dividing visibility amplitude by total detected flux.
                    VSQ = AMP*AMP/(DETFLX*DETFLX)
                    ERRVSQ = SQRT(PI*PI*(1D0+REDOUT)
     1                      /(4D0*NPHTOT*NPHTOT)+VSQ/NPHTOT)
                    VSQ = VSQ + ERRVSQ*RNGAUS(ISEED)
                    VSQ = MAX(VSQ,0D0)
                    AMP = SQRT(VSQ)*DETFLX
                    ERRAMP = 0.5*AMP*ERRVSQ/VSQ
C
                    ERRPHS = PI/(2D0*RPDEG*SQRT(NPHTOT*VSQ))
                    PHS = PHS + ERRPHS*RNGAUS(ISEED)
                  END IF
                  VISR = AMP*COS(RPDEG*PHS)
                  VISI = AMP*SIN(RPDEG*PHS)
                ELSE
                  ERRAMP = 0.0
                  ERRPHS = 0.0
                END IF
C
                ERR1 = 1.0
                ERR2 = 0.0
                ERR3 = 0.0
                IF (.NOT.PERFCT) THEN
                    IF (ERRMUL.GT.0.0) ERR1 = 1.0 + ERRMUL*RNGAUS(ISEED)
                    IF (ERRADD.GT.0.0) THEN
                        ERR2 = ERRADD*RNGAUS(ISEED)
                        ERR3 = ERRADD*RNGAUS(ISEED)
                    END IF
                END IF
                SCALB = SCLFAC(IS1)*SCLFAC(IS2)
                AMP0 = SQRT(VISR**2+VISI**2)*SCALB
                VISR = VISR*ERR1+ERR2
                VISI = VISI*ERR1+ERR3
C
C                    A Poor-Man's Modelfit and "UVSUB"
C
C               The following code assumes that the observer has
C               fit a preliminary model to the data already and wants
C               to subtract it from the data. The code uses the first NCSUB
C               components to subtract, scaled to the source's total flux.
C               (That way, the model can be simple, yet accurate in flux,
C               just as a preliminary model would be.)
C               Note that errors in data are retained.
                IF (SUBMOD) THEN
                  NCSAV = NCOMP
                  NCOMP = NCSUB
                  CALL CMODEL(U,V,VSRSUB,VSISUB)
                  NCOMP = NCSAV
                  IF (NULLNG) THEN
                    VISR = VISR - SUBFAC*(VSRSUB-FLXSUB)
                  ELSE
                    VISR = VISR - SUBFAC*VSRSUB
                    VISI = VISI - SUBFAC*VSISUB
                  END IF
                END IF
                AMP = SQRT(VISR**2+VISI**2)*SCALB
                PHASE(IB) = 0.0
                IF (AMP.GT.0.0) PHASE(IB) = DATAN2(VISI,VISR)/RPDEG +
     1              STAPHA(IS1) - STAPHA(IS2) + 720.0D0
                PHASE(IB) = MOD(PHASE(IB),360.0D0)
                AMPERR = SQRT((AMP*ERRMUL)**2 + ERRADD**2 + ERRAMP**2)
                IF (AMP.LE.0.0) AMPERR = 0.0
                IF (AMP.LE.0.0) THEN
                    PHSERR(IB) = 0.0
                ELSE IF (AMP0.LE.0.0) THEN
                    PHSERR(IB) = 180.0
                ELSE
                    PHSERR(IB) = (ERRADD/AMP0)/RPDEG + ERRPHS
                END IF
                REC(IPOS+1) = AMP
                REC(IPOS+2) = AMPERR
                REC(IPOS+3) = PHASE(IB)
                REC(IPOS+4) = PHSERR(IB)
                IF (NVER.GE.2) THEN
                    REC(IPOS+5) = U
                    REC(IPOS+6) = V
                    REC(IPOS+7) = W
                END IF
            END IF
      ELSE
C                ! not up
            IF (DATA) THEN
                REC(IPOS+1) = 0.0
                REC(IPOS+2) = 0.0
                REC(IPOS+3) = 0.0
                REC(IPOS+4) = 0.0
                IF (NVER.GE.2) THEN
                    REC(IPOS+5) = 0.0
                    REC(IPOS+6) = 0.0
                    REC(IPOS+7) = 0.0
                END IF
                PHSERR(IB) = 0.0
            END IF
      END IF
C                ! end test for UP(IS1) and UP(IS2)
      IPOS = IPOS+NDWPB
  320 CONTINUE
C
C Calculate closure phases
C
      IF (DATA) THEN
          DO IT=1,NT
              CPHASE = PHASE(IB1(IT))-PHASE(IB2(IT))+PHASE(IB3(IT))
              REC(IPOS+1) = MOD(CPHASE+180D0,360D0)-180D0
              REC(IPOS+2) = MIN(180D0, SQRT( PHSERR(IB1(IT))**2
     1                   + PHSERR(IB2(IT))**2 + PHSERR(IB3(IT))**2 ))
C                ! delete unmeasured closure phases
              IF (PHSERR(IB1(IT)).LE.0.0 .OR.
     1            PHSERR(IB2(IT)).LE.0.0 .OR.
     2            PHSERR(IB3(IT)).LE.0.0) THEN
                    REC(IPOS+1) = 0.0
                    REC(IPOS+2) = 0.0
              END IF
              IPOS = IPOS+2
          END DO
          WRITE(OUT) IUT,(REC(J),J=1,JMAX)
      END IF
  390 CONTINUE
  400 CONTINUE
C
C End record
C
      IF (UVPLOT) THEN
          CALL PGEBUF
          CALL PGQINF('TERMINAL', STRING, L)
Cvms      IF (STRING.EQ.'YES')
Cvms 1        CALL LIB$GET_COMMAND(STRING, 'Type <CR> to continue: ', L)
          CALL PGEND
          IP = 0
          DO IB=1,NB
              IP = IP + BCOUNT(IB)
          END DO
          EPOCH = EPOCH/DBLE(IP) + YEAR
          WRITE (OUTC,700) UMAXC,VMAXC,IP
          WRITE (OUTC,701) EPOCH
          IF (COUNT) WRITE (OUTC,705) (BNAME(IB),BCOUNT(IB),IB=1,NB)
      END IF
      IF (DATA) THEN
          IUT = -1
          WRITE(OUT) (IUT,J=1,LENG)
          CLOSE (UNIT=OUT,STATUS='KEEP')
          WRITE(OUTC,670) OUTFIL(1:LEN1(OUTFIL))
      END IF
C
      END

C*CMODEL -- compute model visibility at given (u,v) point
C+
      SUBROUTINE CMODEL(UU,VV,VISR,VISI)
      DOUBLE PRECISION UU, VV
      DOUBLE PRECISION VISR, VISI
C
C Calculate model visibility at point (uu,vv).
C
C  7-Dec-1984: correct bug for small QQ in model type 3 [TJP].
C              Further study of numerical accuracy would be useful.
C-----------------------------------------------------------------------
      INTEGER   MAXMOD
      DOUBLE PRECISION    PI
      PARAMETER (MAXMOD=2000)
      PARAMETER (PI=3.14159265359)
      INTEGER NCOMP
      DOUBLE PRECISION  FLUX(MAXMOD),X(MAXMOD),Y(MAXMOD),AA(MAXMOD),
     1        R(MAXMOD),PHI(MAXMOD)
      INTEGER ITYPE(MAXMOD)
      COMMON /DMODEL/ FLUX,X,Y,AA,R,PHI,ITYPE,NCOMP
C
      INTEGER J
      DOUBLE PRECISION COMPHA, COMVIS, SPHI, CPHI, Q, QQ, XX
      DOUBLE PRECISION DBESJ0, DBESJ1
C
      VISI = 0.0D0
      VISR = 0.0D0
      IF (NCOMP.LT.1) GOTO 40
      DO 30 J=1,NCOMP
          COMPHA = 2.0D0*PI*(UU*X(J)+VV*Y(J))
C
C -- Delta function --
C
          IF (AA(J).EQ.0.0) THEN
              COMVIS = 1.0D0
          ELSE
              SPHI = SIN(PHI(J))
              CPHI = COS(PHI(J))
              Q = (VV*CPHI + UU*SPHI)**2 + (R(J)*(UU*CPHI-VV*SPHI))**2
              QQ = PI*AA(J)*SQRT(Q)
C
C -- Gaussian --
C
              IF (ITYPE(J).LE.1) THEN
                  COMVIS = DEXP(-3.559707D0*Q*AA(J)**2)
C
C -- Uniform disk --
C
              ELSE IF (ITYPE(J).EQ.2) THEN
                  COMVIS = 2.0D0*DBESJ1(QQ)/QQ
C
C -- Optically thin sphere --
C
              ELSE IF (ITYPE(J).EQ.3) THEN
                  IF (ABS(QQ).LT.1D-5) THEN
                      COMVIS = 1.0D0
                  ELSE
                      COMVIS = 3.0D0*(SIN(QQ)-QQ*COS(QQ))/(QQ*QQ*QQ)
C                     [For small QQ, COMVIS = 1.0 + o(QQ**2)]
                  END IF
C
C -- Ring --
C
              ELSE IF (ITYPE(J).EQ.4) THEN
                  COMVIS = DBESJ0(QQ)
C
C -- Line --
C
              ELSE IF (ITYPE(J).EQ.5) THEN
                  XX = AA(J)*PI*(UU*SPHI+VV*CPHI)
                  IF (ABS(XX).LT.1D-5) THEN
                      COMVIS = 1.0D0
                  ELSE
                      COMVIS = SIN(XX)/XX
                  END IF
              ELSE
                  CALL ERROR('Bugcheck')
              END IF
          END IF
          COMVIS = COMVIS*FLUX(J)
          VISI = VISI+COMVIS*SIN(COMPHA)
          VISR = VISR+COMVIS*COS(COMPHA)
   30 CONTINUE
C
   40 RETURN
      END

C*FAKSCL -- used by FAKE
C+
      SUBROUTINE FAKSCL(SCAL,EXPECT,ISEED)
      DOUBLE PRECISION SCAL, EXPECT
      INTEGER ISEED
C-----------------------------------------------------------------------
      REAL RAN
      DOUBLE PRECISION X, Y
      X = RAN(ISEED)
      CALL SOLVE(Y,X)
      SCAL = DEXP(-Y*EXPECT/2.)
      END

C*SOLVE
C+
      SUBROUTINE SOLVE(Y,X)
      DOUBLE PRECISION Y, X
C-----------------------------------------------------------------------
      DOUBLE PRECISION XP,YU,YL,YN
      IF (1.-X.LT.1.0E-7) GO TO 30
      Y=1.
2     XP=(1+Y)*DEXP(-Y)
      IF (XP.LT.X) GO TO 10
          Y=Y*2.
          GO TO 2
10    YU=Y
      YL=Y/2.
      IF (Y.EQ.1.) YL=1.0E-4
15    YN=SQRT(YU*YL)
      XP=(1.+YN)*DEXP(-YN)
      IF (ABS(X-XP).LT.1.E-5) GO TO 20
          IF (XP.LT.X) YU=YN
          IF (XP.GT.X) YL=YN
          GO TO 15
20    Y=YN
      RETURN
30    Y=0.
      RETURN
      END

C*SRCUP
C+
      LOGICAL FUNCTION SRCUP(X,S)
      DOUBLE PRECISION X(3), S(3)
C
C Determine if source in (unit) direction S is visible to
C satellite in direction X.  Vectors measured from center of
C earth.
C-----------------------------------------------------------------------
      DOUBLE PRECISION  RE,GM,HJ2,DOT
      COMMON /EARTH/ RE,GM,HJ2
C
C       Source visible if satellite is in front of plane perpendicular
C       to source vector passing through earth.  (This assumes orbit is
C       above earth.)
C
      DOT = X(1)*S(1) + X(2)*S(2) + X(3)*S(3)
      IF (DOT.GT.0.) THEN
          SRCUP = .TRUE.
C
C       If satellite behind plane, source is still visible if satellite
C       is not shadowed by earth (|X x S| > earth radius).
C
      ELSE IF (SQRT(X(1)*X(1)+X(2)*X(2)+X(3)*X(3) - DOT*DOT) .GT. RE)
     1  THEN
          SRCUP = .TRUE.
      ELSE
          SRCUP = .FALSE.
      END IF
      END

      SUBROUTINE UVMX(X,Y,Z,AEIOWM,COSRA,SINRA,COSDEC,
     1      SINDEC,STATYP,UMX,VMX)
      DOUBLE PRECISION X,Y,Z,AEIOWM(6),COSRA,SINRA,COSDEC,SINDEC,
     1      COSI,SINI,COSO,SINO,SINOMA,COSTH,E,E2,EM,COSP,SINP,A,
     2      TANFI,COSFI,SINFI,BE
      DOUBLE PRECISION UMX,VMX
      INTEGER STATYP
      IF (STATYP.EQ.2) THEN
            A = (1D0+AEIOWM(2))*AEIOWM(1)
C                ! Apoapsis; ass. orbit circluar
            SINI = SIN(AEIOWM(3))
C                ! Inclination
            COSI = COS(AEIOWM(3))
            SINO = SIN(AEIOWM(4))
C                ! Ascending Node
            COSO = COS(AEIOWM(4))
            SINOMA = SINO*COSRA - COSO*SINRA
C                ! (O-RA) = Viewing angle + 90
            COSTH = SINI*COSDEC*SINOMA + COSI*SINDEC
C                ! TH = Angle between
            E = SQRT(1D0 - COSTH*COSTH)
C                ! source and orbital plane
            IF (E.LE.1D-2) THEN
C                ! vectors  (E = SINTH =
                UMX = A
C                ! eccentricity of ellipse
                VMX = A
C                ! in UV plane)
                RETURN
            END IF
            COSP = (-SINI*SINDEC*SINOMA + COSI*COSDEC)/E
C                ! P = Angle of
            SINP = SQRT(1D0-COSP*COSP)
C                ! orientation of ellipse in
            IF (E.GT.0.99) THEN
C                ! UV plane
                UMX = A*COSP
                VMX = A*SINP
                RETURN
            END IF
            E2 = E*E
            EM = SQRT(1D0-E2)
            TANFI = E2*SINP*COSP/(1D0-E2*SINP*SINP)
C                ! FI is position
            COSFI = 1D0/SQRT(1D0+TANFI*TANFI)
C                ! in orbit where max U (or
            SINFI = SQRT(1D0-COSFI*COSFI)
C                ! max V) occurs
            UMX = A*EM*COSFI/SQRT(1D0-E2*(COSFI*COSP+SINFI*SINP)**2)
            TANFI = (1D0-E2*COSP*COSP)/(E2*SINP*COSP)
            COSFI = 1D0/SQRT(1D0+TANFI*TANFI)
            SINFI = SQRT(1D0-COSFI*COSFI)
            VMX = A*EM*SINFI/SQRT(1D0-E2*(COSFI*COSP+SINFI*SINP)**2)
            RETURN
      ELSE IF (STATYP.EQ.1) THEN
            BE = SQRT(X*X+Y*Y)
            UMX = BE
            VMX = ABS(BE*SINDEC) + ABS(Z*COSDEC)
            RETURN
      ELSE
            CALL ERROR('ILLEGAL STATION TYPE')
      END IF
      END

      DOUBLE PRECISION FUNCTION YNTERP(XA,YA,X,L,N)
      DOUBLE PRECISION XA(1),YA(1),ZA(21)
      LOGICAL SMX
      INTEGER I,IA,IB,IU,IL,IFD,IFLAG,J,JA,L,N,NIN,NP1
      DOUBLE PRECISION SG1,SG2,T1,T2,X
      DATA SG1/0.0D0/
      IF (SG1.NE.0.0) GO TO 12
      SG1=1.0
      SMX=.FALSE.
      IFD=0
12    SG2=0
      IF (SMX) GO TO 44
      IU=L+1
      IF (L.LT.2) GO TO 43
      NIN=N
      IF (N.LT.1.OR.N.GT.20) GO TO 40
      NP1=N+1
      IF (NP1-L) 14,48,43
14    IF (XA(L)-XA(1)) 16,43,18
16    SG2=1.0
      IL=IU
      IU=0
      GO TO 20
18    IL=0
20    IA=(IU+IL)/2
      IF (IABS(IU-IL).LT.3) GO TO 26
      IF (X-XA(IA)) 22,50,24
22    IU=IA
      GO TO 20
24    IL=IA
      GO TO 20
26    IF (X.EQ.XA(IA)) GO TO 50
      IFD=IA
      IA=IA-NP1/2
      IF (MOD(NIN,2).EQ.0) GO TO 30
      IF (X.GT.XA(IFD)) GO TO 28
      IF (SG2.NE.0.0) IA=IA+1
      GO TO 30
28    IF (SG2.EQ.0.0) IA=IA+1
30    IF (IA.LT.1) IA=1
      IF (L-NIN.LT.IA) IA=L-NIN
32    JA=IA
      DO 34 J=1,NP1
      ZA(J)=YA(JA)
34    JA=JA+1
      IB=IA
      DO 38 I=1,NIN
      T1=X-XA(IB)
      DO 36 J=I,NIN
      JA=J+IA
      T2=XA(JA)-XA(IB)
36    ZA(J+1)=ZA(I)+T1*(ZA(J+1)-ZA(I))/T2
38    IB=IB+1
      IFLAG=1
      YNTERP=ZA(NIN+1)
      RETURN
40    WRITE(6,42)
42    FORMAT(/'ERROR RETURN FROM ''YNTERP''--N IS OUT OF RANGE.')
43    IFD=1
      IFLAG=3
      YNTERP=YA(1)
      RETURN
44    SMX=.FALSE.
      IF (IFLAG-2) 32,46,43
46    YNTERP=YA(IA)
      IFD=IA
      RETURN
48    IA=1
      IFD=1
      GO TO 32
50    IFLAG=2
      GO TO 46
      END

      DOUBLE PRECISION FUNCTION BNU(FREQ,TEMP)
      DOUBLE PRECISION FREQ, TEMP
C
C     Compute surface brightness of a black body of a given Temperature
C     (in units of K) at a given frequency (in Hz)
C
      DOUBLE PRECISION  VELC, PLANCK, BOLTZ
      PARAMETER  (VELC=2.99792458D8)
      PARAMETER  (PLANCK=6.6260755D-34)
C                (Planck's constant in J/Hz)
      PARAMETER  (BOLTZ=1.380658D-23)
C                (Boltzmann's constant in J/K)
C
      DOUBLE PRECISION HNUOKT
      HNUOKT = PLANCK*FREQ/(BOLTZ*TEMP)
      IF (HNUOKT .LE. 1.D-5) THEN
        BNU = 1.0D0
      ELSE IF (HNUOKT .GT. 200.D0) THEN
        BNU = 0.0D0
      ELSE
        BNU = HNUOKT/(DEXP(HNUOKT)-1.0D0)
      END IF
      BNU = 2D0*BOLTZ*TEMP*(FREQ*FREQ)*BNU/(VELC*VELC)
      RETURN
      END

C*DGTMDL -- read one model component from file
C+
      SUBROUTINE DGTMDL(INMOD,ITYPE,P1,P2,P3,P4,P5,P6,*,OUTMOD)
      INTEGER INMOD, ITYPE, OUTMOD
      DOUBLE PRECISION P1, P2, P3, P4, P5, P6
C
C T.J.Pearson   1979 FEB 18
C Modified for VAX version  1979 JULY 5
C Modified to allow comments 1981 February 9
C Modified to copy input comments to an output file 1982 September 22
C
C Read one model component (card) from unit 'INMOD'.
C Each card carries up to 7 numbers:
C 6 floating-point parameters P1-P6, and an optional integer ITYPE.
C Copy comments following a '!' character to unit 'OUTMOD'
C 	Ignore zero-flux components (or blank lines)
C-----------------------------------------------------------------------
      DOUBLE PRECISION PARS(7)
      DOUBLE PRECISION KCTOR
      CHARACTER*80 CARD
      CHARACTER BLANK, EXCL
      INTEGER I, J, L
      CHARACTER*64 FNAME
      DATA BLANK/' '/
      DATA EXCL/'!'/
C
   5  READ(INMOD, '(A)', END=40) CARD
      L = 80
   10 IF (CARD(L:L).EQ.BLANK) THEN
            L = L-1
            IF (L.LT.1) GOTO 5
        GOTO 10
      END IF
C
      I = 1
      DO 30 J=1,7
          CALL KSKIPB(CARD,I)
          PARS(J) = KCTOR(CARD,I)
   30 CONTINUE
      IF (I.GT.L) GOTO 50
      IF (CARD(I:I).EQ.BLANK) GOTO 50
      IF (CARD(I:I).EQ.EXCL) THEN
            IF (OUTMOD.GT.0) WRITE (OUTMOD,'(A)') CARD(I:L)
            GOTO 50
      END IF
      INQUIRE(UNIT=INMOD,NAME=FNAME)
      CALL ERROR('Invalid model format: file '//FNAME)
   50 CONTINUE
      P1 = PARS(1)
      IF (P1.EQ.0D0) GOTO 5
      P2 = PARS(2)
      P3 = PARS(3)
      P4 = PARS(4)
      P5 = PARS(5)
      P6 = PARS(6)
      ITYPE = PARS(7)
      RETURN
C
   40 RETURN 1
      END
