      PROGRAM FBSCOR
C-----------------------------------------------------------------------
C FBSCOR applies a correction to the amplitude and
C anmplitude error.  This is to compensate for the error in applying
C the fractional bitshift correction (FBS) that affected data correlated
C prior to June 10, 1991.
C
C Parameters:
C      INPUT = input file name
C      OUTPUT = output file name
C      HISTORY = text to be written in header
C      BANDPASS = weights to assign to the four pieces of the band
C      CORREXP = apply correction (+1), apply error or uncorrect (-1)
C      LISTFIL = correction listing file name
C
C Subroutines required:
C      KEYIN, LEN1, MKHIST, PUTOUT, VLBOPE.
C
C Version 1.0   1991 Nov     Rename to MRGCOPFBS: modification of
C                            MRGCOPY for FBS corrections with BANDPASS
C                            parameters.     (GLB)
C Version 1.1   1991 Dec     Add CORREXP parameter to choose between
C                            applying correction (+1) or applying error,
C                            i.e. uncorrect (-1)
C Version 1.2   1992 Jan     include LISTFIL as KEYIN parameter; no
C                            longer writes correction values to 6  (KLT)
C               1992 Feb 5   Minor modifications; change name to
C                            FBSCOR (SCU)
C-----------------------------------------------------------------------
C
C Constants.
C
      CHARACTER*(*) PROGNM, VERSN
      INTEGER INC,OUTC,INDAT,OUTDAT,MAXS,MAXB,MAXT,NPARS
      DOUBLE PRECISION PI,SIDR,VELC
C     -- Program version and date:
      PARAMETER (PROGNM='FBSCOR')
      PARAMETER (VERSN='1.2 - 1992 Feb 6')
C     -- Fortran unit numbers used:
      PARAMETER (INC=5,OUTC=6,INDAT=8,OUTDAT=9)
C     -- Maximum numbers of stations, baselines, and triangles:
      PARAMETER (MAXS=20)
      PARAMETER (MAXB=(MAXS-1)*MAXS/2)
      PARAMETER (MAXT=MAXB*(MAXS-2)/3)
C     -- Number of input parameters:
      PARAMETER (NPARS=45)
C     -- Pi:
      PARAMETER (PI=3.141592653589793D0)
C     -- Radians of ST per sec of UT:
      PARAMETER (SIDR=(PI/2592000.0D0)*1.0027379093D0)
C     -- Speed of light:
      PARAMETER (VELC=2.997925D8)
C-----------------------------------------------------------------------
C Variables.
C
      CHARACTER*8  STNAME(MAXS)
      CHARACTER*8  PERCNT, UTP, TIMEC
      CHARACTER*16 SOURCE
      INTEGER   IUT, IS, IB, IT, NS, NB, NT, I, J, IER, MODE
      INTEGER   IZ, IY, IS1, IS2, NHIST
      INTEGER   NNEW, NDWPB
      INTEGER   NBI(MAXB), NB1(MAXB), NB2(MAXB)
      INTEGER   NTI(MAXT)
      INTEGER   REC1(20), IDUM(8)
      INTEGER   CEXP
      REAL      INREC(7*MAXB+2*MAXT),W1,W2,W3,W4
      REAL      MNCOR, COUNT, DRCOEF, UMAX, SUMCOR, TIHA, U, ABSU
      REAL      DR, FCOR, AVCOR
      DOUBLE PRECISION STX(MAXS), STY(MAXS), STZ(MAXS),
     1          RA, DEC, RAP, DECP, GASTM, FREQ, FLUX,
     2          WAVE,BX,BY,BZ(MAXB),BOFF(MAXB),BXY(MAXB),
     3          COSDEC,SINDEC, TINTG, AVG, BW,
     4          PARS(NPARS),VALS(NPARS),ENDMRK
      CHARACTER*80 INDSN, OUTDSN, OLDHIS, HISTRY, ADDHIS(3), LISFIL
C-----------------------------------------------------------------------
C Functions called.
C
      INTEGER   LEN1, VLBOPE
C-----------------------------------------------------------------------
C Names and default values of KEYIN parameters.
C
      DATA  PARS/ 'Input',   9*' ', 'Output',  9*' ', 'History',9*' ',
     1          4*'Bandpass', 'Correxp', 'Listfile',  9*' ' /
      DATA  VALS/ '***UNSET',9*' ', '***UNSET',9*' ', 10*' ' ,0.8D0,
     1          1.0D0, 1.0D0, 0.6D0, +1.0, 'fbscor.l', 'is', 8*' ' /
      DATA  ENDMRK/ '/' /
C-----------------------------------------------------------------------
C
C Introduction.
C
      CALL PUTOUT(PROGNM//' corrects for fractional-bitshift error.')
      CALL PUTOUT('For use ONLY on Block II data correlated'
     1 //' before 10 June 1991')
      CALL PUTOUT('(Version '//VERSN//')')
C
C Control parameters.
C
      MODE = 0
  100 IF (MODE.NE.0) CALL ERROR('Bad input parameters in '//PROGNM)
      CALL PUTOUT(' ')
      CALL PUTOUT('Control Parameters'//
     1' (INPUT,OUTPUT,HISTORY,BANDPASS,CORREXP,LISTFILE):')
      CALL KEYIN(PARS,VALS,NPARS,ENDMRK,MODE,INC,OUTC)
      WRITE (INDSN,'(10A8)') (VALS(I),I=1,10)
      WRITE (OUTDSN,'(10A8)') (VALS(I),I=11,20)
      WRITE (HISTRY,'(10A8)') (VALS(I),I=21,30)
      W1 = VALS(31)
      W2 = VALS(32)
      W3 = VALS(33)
      W4 = VALS(34)
      CEXP = VALS(35)
      WRITE (LISFIL,'(10A8)') (VALS(I),I=36,45)
C
C Open input Merge file.
C
      IER = VLBOPE(INDAT, INDSN, 'MERGE', 'OLD', INDSN)
      IF (IER.NE.1) GOTO 100
C
C Open output Merge file.
C
      IER = VLBOPE(OUTDAT, OUTDSN, 'MERGE', 'NEW', OUTDSN)
      IF (IER.NE.1) THEN
          CLOSE (UNIT=INDAT)
          GOTO 100
      END IF
C
C Open output Log file.
C
      IER = VLBOPE(10, LISFIL, 'TEXT', 'NEW', LISFIL)
      IF (IER.NE.1) THEN
          CLOSE (UNIT=INDAT)
          CLOSE (UNIT=OUTDAT, STATUS='DELETE')
          GOTO 100
      ENDIF
C
C Make the new history records.
C
      IF (CEXP.NE.1.0 .AND. CEXP.NE.-1.0) 
     1         CALL ERROR('Illegal CORREXP')
      CALL MKHIST(PROGNM, VERSN, ADDHIS(1))
      IF (HISTRY.NE.' ') THEN
          NNEW = 3
          ADDHIS(2) = '- '//HISTRY(1:78)
          IF(CEXP .EQ. 1.0) THEN
          ADDHIS(3) = '- Correction for Block II FBS error applied.'
          ELSE IF(CEXP .EQ. -1.0) THEN
          ADDHIS(3) = 
     1'** WARNING ** Block II FBS error introduced; for simulation ONLY'
          END IF
      ELSE
          NNEW = 2
          IF(CEXP .EQ. 1.0) THEN
          ADDHIS(2) = '- Correction for Block II FBS error applied.'
          ELSE IF(CEXP .EQ. -1.0) THEN
          ADDHIS(2) = 
     1'** WARNING ** Block II FBS error introduced; for simulation ONLY'
          END IF
      END IF
      DO 105 I=1,NNEW
          CALL PUTOUT(ADDHIS(I)(1:LEN1(ADDHIS(I))))
  105 CONTINUE
C
      WRITE(10,402)
  402 FORMAT('   UT           Umax     Average correction'/)
C
C Read and re-write header. Merge format level must be 1.
C
      READ (INDAT,END=82,ERR=82) (REC1(J),J=1,20)
      IF (REC1(1).NE.1) GOTO 82
      NDWPB = 4
      NHIST = REC1(2)
      NS    = REC1(4)
      NB    = REC1(5)
      NT    = REC1(6)
      IF (NS.LT.2) GOTO 82
      IF (NS.GT.MAXS) GOTO 82
      REC1(2) = REC1(2) + NNEW
      REC1(3) = REC1(3) + NNEW
      WRITE (OUTDAT) (REC1(J),J=1,20)
C
      IF (NB.GT.0) THEN
          READ (INDAT,END=82,ERR=82) (NBI(IB),IB=1,NB)
          WRITE (OUTDAT) (NBI(IB),IB=1,NB)
          DO 110 IB=1,NB
              NB1(IB) = NBI(IB)/100
              NB2(IB) = NBI(IB) -  NB1(IB)*100
  110     CONTINUE
      END IF
C
      IF (NT.GT.0) THEN
          READ (INDAT,END=82,ERR=82) (NTI(IT),IT=1,NT)
          WRITE (OUTDAT) (NTI(IT),IT=1,NT)
      END IF
C
      DO 120 I=1,NHIST
          READ (INDAT,END=82,ERR=82) OLDHIS
          WRITE (OUTDAT) OLDHIS
  120 CONTINUE
      DO 125 I=1,NNEW
          WRITE (OUTDAT) ADDHIS(I)
  125 CONTINUE
C
      READ (INDAT,END=82,ERR=82) SOURCE,RA,DEC,RAP,DECP,FLUX
      WRITE (OUTDAT) SOURCE,RA,DEC,RAP,DECP,FLUX
C
      DO 130 IS=1,NS
          READ (INDAT,END=82,ERR=82) STNAME(IS),STX(IS),STY(IS),STZ(IS)
          WRITE(OUTDAT) STNAME(IS),STX(IS),STY(IS),STZ(IS),
     1            (0.D0,J=5,10),0
  130 CONTINUE
C
      READ (INDAT,END=82,ERR=82) IY, IZ, GASTM, FREQ, TINTG,
     1            AVG, BW, (IDUM(I),I=1,8)
      WRITE (OUTDAT) IY, IZ, GASTM, FREQ, TINTG,
     1            AVG, BW, (IDUM(I),I=1,8)
C
C Baseline parameters (BXY and BZ are components in wavelengths;
C BOFF is "hour angle").
C
      WAVE   = VELC/FREQ
      COSDEC = DCOS(DECP)
      SINDEC = DSIN(DECP)
      DO 140 IB=1,NB
          IS1 = NB1(IB)
          IS2 = NB2(IB)
          BX  = (STX(IS1)-STX(IS2))/WAVE
          BY  = (STY(IS1)-STY(IS2))/WAVE
          BZ(IB) = (STZ(IS1)-STZ(IS2))/WAVE
          BXY(IB) = DSQRT(BX*BX+BY*BY)
          IF (BXY(IB).EQ.0D0) THEN
            CALL PUTOUT('Station coordinates missing.')
            GOTO 82
          END IF
          BOFF(IB) = GASTM-RAP-PI/2.0-DATAN2(BY,BX)
  140 CONTINUE
C
      COUNT = 0.0
C     -- Accumulate AVCOR values
      MNCOR = 0.0
C
C Read and re-write data records.
C
  300 READ(INDAT,END=400) IUT,(INREC(J),J=1,NDWPB*NB+2*NT)
      IF (IUT.LT.0) GOTO 400
C
C       This section inserted to correct fringe amplitudes for
C       the misapplied FBS correction prior to June 10, 1991.
C       Single precision is used since it is more than sufficiently
C       accurate, and the data INREC(IB) are are already REAL*4.
C
      DRCOEF = 2.42E-8
      UMAX = 0.0
      SUMCOR = 0.0
      DO 350 IB=1,NB
C           Calculate the delay rate in units of bins/COI.
          TIHA = (IUT-IZ)*SIDR + BOFF(IB)
C         -- in meters
          U = WAVE*BXY(IB)*COS(TIHA)
          ABSU = ABS(U)
C         -- in bins/COI
          DR = DRCOEF * ABSU * COSDEC
C
          CALL FBS(DR,W1,W2,W3,W4,FCOR)
          MNCOR = MNCOR + 1.0/FCOR
          COUNT = COUNT + 1.0
          FCOR = FCOR**CEXP
          INREC(4*IB-3) = FCOR*INREC(4*IB-3)
          INREC(4*IB-2) = FCOR*INREC(4*IB-2)
          SUMCOR = SUMCOR + FCOR
          IF (ABSU .GT. UMAX) UMAX = ABSU
  350 CONTINUE
      AVCOR = SUMCOR/NB
C
      UTP = TIMEC(MOD(IUT/60,86400))
      WRITE (10,'(A8,4X,F12.2,F12.4)') UTP,UMAX,AVCOR
C
C       End of correction section; now back to original program.
C
      WRITE (OUTDAT) IUT,(INREC(J),J=1,NDWPB*NB+2*NT)
      GOTO 300
C
C Write final record and quit.
C
  400 CONTINUE
      MNCOR = 1.0 - MNCOR/COUNT
      WRITE(PERCNT,'(F8.4)') MNCOR*100.0
      CALL PUTOUT
     1  ('Mean point-source sensitivity loss ='//PERCNT//' %')
      WRITE (10,401)   MNCOR*100.0
  401 FORMAT(/'Mean point-source sensitivity loss =',F8.4,' %')
C
      WRITE (OUTDAT) -1,(0.0,J=1,NDWPB*NB+2*NT)
      CLOSE (UNIT=INDAT)
      CLOSE (UNIT=OUTDAT)
      STOP
C
C Error exit. Delete the nascent output file.
C
   82 CLOSE (UNIT=OUTDAT, STATUS='DELETE')
      CLOSE (UNIT=INDAT)
      CALL ERROR('Something wrong with input MERGE file.')
      STOP
C-----------------------------------------------------------------------
      END

      SUBROUTINE FBS(D,W1,W2,W3,W4,FCOR)
C
C       From program fbs.for by SCU (1991 Jul 9) to evaluate fractional
C       bitshift error for Block II Correlator.
C
C       Changed to single precision for speed.
C-----------------------------------------------------------------------
      REAL P,D,W1,W2,W3,W4
      REAL A1,A2,A3,A4,P1,P2,P3,P4,CSUM,SSUM,ATOT,FCOR, ARG
      REAL PI, RPDEG
      PARAMETER  (PI=3.1415926536)
      PARAMETER  (RPDEG=PI/180.0)
C
C     -- p = delay period in COI
      P = 1.0/D
C
C     bin 1
      CALL AMP(P,-45.0,A1,P1)
C     bin 2 (NFREQ+DFREQ=0 bin)
      CALL AMP(P,  0.0,A2,P2)
C     bin 3
      CALL AMP(P, 45.0,A3,P3)
C     bin 4
      CALL AMP(P, 90.0,A4,P4)
C
C         Assume fitted phase slope is that determined by bins
C         1,2,3 only
C
      ARG = (P4-2.0*P3)*RPDEG
      CSUM = W1*A1 + W2*A2 + W3*A3 + W4*A4*COS(ARG)
      CSUM = CSUM / (W1+W2+W3+W4)
      SSUM = W4*A4*SIN(ARG)
      SSUM = SSUM / (W1+W2+W3+W4)
      ATOT = SQRT(CSUM*CSUM + SSUM*SSUM)
      FCOR = 1.0/ATOT
      RETURN
      END

      SUBROUTINE AMP(P,PHI,AR,PR)
      REAL P, PHI, A1, P1, A2, P2, AR, PR, ACS, ASN
      REAL PI, RPDEG
      PARAMETER  (PI=3.1415926536)
      PARAMETER  (RPDEG=PI/180.0)
C
      A1 = (P - 2.0)/P
      P1 = -PHI/P
      A2 = 2.0/P
      P2 = -PHI/P  + PHI
      ACS = A1 * COS(P1*RPDEG) + A2 * COS(P2*RPDEG)
      ASN = A1 * SIN(P1*RPDEG) + A2 * SIN(P2*RPDEG)
      AR = SQRT(ACS*ACS + ASN*ASN)
      PR = ATAN2(ASN,ACS)/RPDEG
C
      RETURN
      END
