      PROGRAM FITSLS
C-----------------------------------------------------------------------
C FITSLIST prints the headers of one or more FITS-format files, e.g.,
C VLBI maps. The input may be either a FITS-format tape or a set of
C disk files. In either case the input files must have the standard
C FITS format, with a recordsize and blocksize of 2880 bytes.
C
C FITSLIST uses standard KEYIN control parameters:
C   TAPE:     the name of the tape drive on which the input tape
C             has been mounted (/FOREIGN); if not specified, the input
C             is assumed to be disk files.
C   INPUT:    the name of the input disk file(s), which may include
C             VMS wild-cards; no default (ignored if TAPE is specified).
C   LISTFILE: the name for the long listing; default FITSLIST.LIS.
C   INDEX:    the name for the brief index; default standard output.
C   SKIP:     the number of files to skip at the beginning of the tape
C             (only if TAPE is specified).
C
C Author:     T.J. Pearson (TJP)
C Language:   Fortran-77, with BYTE extension.
C
C Modifications:
C   Version 1.0:  1984 Jan 19 - (TJP).
C   Version 1.1:  1984 Feb 23 - add TAPE parameter (TJP).
C   Version 1.2:  1984 Jun 11 - display FITS XTENSION file headers (TJP).
C   Version 1.3:  1984 Sep  9 - add SKIP parameter (TJP).
C   Version 1.4:  1986 Apr 22 - cosmetic changes to source code (TJP).
C   Version 1.5:  1987 Jan  7 - change 'FILE' to 'INPUT'; 'FILE'
C                               retained as an alias (TJP).
C   Version 1.6:  1987 Dec 10 - add INDEX parameter.
C   Version 2.0:  1988 Apr 23 - Convex version, uses MTPCKG (TJP).
C   Version 2.1:  1990 May 14 - use RDFITS routine (TJP).
C   Version 2.2:  1990 Aug 24 - display CRVAL4 for uvdata (TJP).
C   Version 3.0:  1992 May  5 - accept blocked FITS tapes (TJP).
C   Version 3.1:  1992 Oct  8 - improved error message (TJP).
C   Version 3.2:  1992 Oct 19 - put file names in index (TJP).
C-----------------------------------------------------------------------
C
C Constants.
C
      CHARACTER*(*) VERSN
      INTEGER    INC,OUTC,INDAT,PR,NPARS,MAGIC
      PARAMETER  (VERSN='3.2 - 1992 Oct 19')
C     -- I/O unit numbers
      PARAMETER  (INC=5,OUTC=6,INDAT=1,PR=2)
C     -- number of KEYIN parameters
      PARAMETER  (NPARS=51)
C     -- size of FITS records
      PARAMETER  (MAGIC=2880)
C     -- maximum number of input files
      INTEGER NMAX
      PARAMETER (NMAX=100)
C
C Variables.
C
      CHARACTER  FITFIL*128, INDSN*128, LISFIL*128, TDRIVE*128
      CHARACTER*128 RESULT(NMAX)
      CHARACTER  BLOCK*(MAGIC), RECORD*80, NOW*20, IDX*80
      CHARACTER  KEYW*8, CVAL*32, INDFIL*128, TEXT*32
      BYTE       BUFFER(MAGIC)
      INTEGER    I, IER, J, LDSN, MODE, NDREC, NREC, CHAN, NDONE
      INTEGER    BLKLEN, NFIL, NFSKIP, TYPE, K1, K2, JUNK, PR2
      INTEGER    NFOUND, ISTAT
      LOGICAL    PROPEN, TAPE, IMAGE
      DOUBLE PRECISION RVAL
      DOUBLE PRECISION ENDMRK, PARS(NPARS), VALS(NPARS)
      EQUIVALENCE (BUFFER,BLOCK)
C
C External functions.
C
      INTEGER    LEN1, VLBOPE
      LOGICAL    MTSEOF
C
C Initialize KEYIN parameters.
C
      DATA       ENDMRK/'/'/
      DATA       PARS/ 'Tape', 9*' ', 
     1                 'INPut', 9*' ', 
     2                 'File', 9*' ',
     3                 'Listfile', 9*' ',
     4                 'INDex', 9*' ',
     5                 'Skip' /
      DATA       VALS/ 10*' ',
     1                 10*' ', 
     2                 10*' ',
     3                 'fitslist', '.lis', 8*' ',
     4                 10*' ',
     5                 0D0 /
C
C Formats.
C
  610 FORMAT (A,T20,A)
  699 FORMAT ('FITS header for tape file number',I4/A)
  700 FORMAT ('FITS header for ',A/A)
  701 FORMAT (80('-'))
  702 FORMAT (//)
  703 FORMAT (I6,' additional records following header')
C-----------------------------------------------------------------------
C
C Introduction.
C
      CALL DATTIM(NOW)
      CALL PUTOUT('FITSLIST displays headers of FITS-format tapes or'//
     1            ' disk files')
      CALL PUTOUT('(Version '//VERSN//')')
      CALL PUTOUT(' ')
C
C Control parameters.
C
      CALL PUTOUT('Control parameters (TAPE or INPUT, LISTFILE;'//
     1            ' end with /):')
   10 MODE = 0
      CALL KEYIN(PARS,VALS,NPARS,ENDMRK,MODE,INC,OUTC)
      IF (MODE.NE.0) GOTO 80
      WRITE (TDRIVE,'(10A8)') (VALS(J),J= 1,10)
      WRITE (FITFIL,'(10A8)') (VALS(J),J=11,20)
      IF (FITFIL.EQ.' ') WRITE (FITFIL,'(10A8)') (VALS(J),J=21,30)
      WRITE (LISFIL,'(10A8)') (VALS(J),J=31,40)
      WRITE (INDFIL,'(10A8)') (VALS(J),J=41,50)
      NFSKIP = VALS(51)
      IF (NFSKIP.LT.0) NFSKIP = 0
C
C Tape or disk?
C
      IF (TDRIVE.EQ.' ' .AND. FITFIL.EQ.' ') THEN
          CALL PUTOUT('Please specify either TAPE or INPUT.')
          GOTO 10
      END IF
      TAPE = TDRIVE.NE.' '
C
C Open tape stream.
C
      IF (TAPE) THEN
          CALL PUTOUT('Reading tape: '//
     1           TDRIVE(1:LEN1(TDRIVE)))
          CALL MTOPID(TDRIVE(1:LEN1(TDRIVE)), CHAN, IER)
          IF (IER.NE.0) THEN
              CALL MTMESS(IER, 'MTOPID', TEXT)
              CALL PUTOUT('Cannot access tape: '//TEXT(1:LEN1(TEXT)))
              GOTO 10
          END IF
          CALL MTREWI(CHAN,IER)
          CALL ERRCHK(IER,'MTREWI','Error rewinding input tape.')
          CALL MTSKFI(CHAN,NFSKIP,NDONE,IER)
          CALL ERRCHK(IER,'MTSKFI','Error skipping input tape')
          NFIL = NFSKIP
C
C Expand wild cards.
C
      ELSE
          CALL FFILES(FITFIL, NMAX, RESULT, NFOUND)
          NFIL = 0
      END IF
C
C Open input file.
C
      PROPEN = .FALSE.
   20 NFIL = NFIL+1
      IF (TAPE) THEN
          CALL MTREAD(CHAN,MAGIC,BUFFER,BLKLEN,IER)
          IF (MTSEOF(IER)) GOTO 80
          CALL MTSKRE(CHAN,-1,NDONE,IER)
          IF (IER.NE.0) GOTO 10
      ELSE
          IF (NFIL.GT.NFOUND) GOTO 80
          INDSN = RESULT(NFIL)
          IER = VLBOPE(INDAT, INDSN, 'FITS', 'OLD', INDSN)
          IF (IER.NE.1) GOTO 10
          LDSN = MIN(132,LEN1(INDSN))
      END IF
C
C Open output file (first time only).
C
      IF (.NOT.PROPEN) THEN
          IER = VLBOPE(PR, LISFIL, 'TEXT', 'NEW', LISFIL)
          IF (IER.NE.1) THEN
              CLOSE (UNIT=INDAT)
              GOTO 10
          END IF
          PROPEN = .TRUE.
          IF (INDFIL.EQ.' ') THEN
              PR2 = OUTC
          ELSE
              PR2 = 3
              IER = VLBOPE(PR2, INDFIL, 'TEXT', 'NEW', INDFIL)
              IF (IER.NE.1) THEN
                CLOSE (UNIT=INDAT)
                GOTO 10
               END IF
          END IF
          IDX = ' '
          IDX(01:08) = 'OBJECT  '
          IDX(10:17) = 'TELESCOP'
          IDX(19:26) = 'DATE-OBS'
          IDX(28:35) = 'DATE-MAP'
          IDX(37:44) = 'FREQ    '
          IDX(46:53) = 'DATE    '
          IDX(55:62) = 'ORIGIN  '
          IDX(64:75) = 'IMNAME      '
          WRITE (PR2,'(4X,A)') IDX(1:LEN1(IDX))
          IDX(01:08) = '--------'
          IDX(10:17) = '--------'
          IDX(19:26) = '--------'
          IDX(28:35) = '--------'
          IDX(37:44) = '--------'
          IDX(46:53) = '--------'
          IDX(55:62) = '--------'
          IDX(64:75) = '------------'
          WRITE (PR2,'(4X,A)') IDX(1:LEN1(IDX))
      END IF
C
C Read and print Header Records.
C
      NREC = 0
      IDX = ' '
   30 IF (TAPE) THEN
          CALL MTRECD(CHAN,MAGIC,BUFFER,BLKLEN,IER)
          IF (IER.NE.0) GOTO 70
          IF (BLKLEN.NE.MAGIC) GOTO 70
      ELSE
          CALL RDFITS(INDAT, ISTAT, BUFFER)
          IF (ISTAT.NE.0) GOTO 70
      END IF
      IF (NREC.EQ.0) THEN
          IF (BLOCK(1:8).NE.'SIMPLE  ') GOTO 70
          IF (TAPE) THEN
              WRITE (PR,699) NFIL,NOW
          ELSE
              WRITE (PR,700) INDSN(:LDSN), NOW
              WRITE (PR2,'(1X,A)') INDSN(1:LDSN)
          END IF
          WRITE (PR,701)
      END IF
      NREC = NREC+1
   35 DO 36 I=1,36
          RECORD = BLOCK(80*I-79:80*I)
          WRITE (PR,'(A)') RECORD(1:LEN1(RECORD))
          CALL FITKEY(RECORD,KEYW,TYPE,RVAL,CVAL)
          IF (KEYW.EQ.'NAXIS1') THEN
              IMAGE = RVAL .NE. 0.0
          ELSE IF (KEYW.EQ.'EXTNAME') THEN
              IDX(1:8) = CVAL
          ELSE IF (KEYW.EQ.'OBJECT') THEN
              IDX(1:8) = CVAL
          ELSE IF (KEYW.EQ.'TELESCOP') THEN
              IDX(10:17) = CVAL
          ELSE IF (KEYW.EQ.'DATE-OBS') THEN
              IDX(19:26) = CVAL
          ELSE IF (KEYW.EQ.'DATE-MAP') THEN
              IDX(28:35) = CVAL
          ELSE IF ((KEYW.EQ.'CRVAL3' .AND. IMAGE) .OR.
     1             (KEYW.EQ.'CRVAL4' .AND. .NOT.IMAGE) ) THEN
              WRITE (IDX(37:44), '(1PE8.2)', IOSTAT=JUNK) RVAL
          ELSE IF (KEYW.EQ.'DATE    ') THEN
              IDX(46:53) = CVAL
          ELSE IF (KEYW.EQ.'ORIGIN  ') THEN
              IDX(55:62) = CVAL
          ELSE IF (KEYW.EQ.'HISTORY ') THEN
              K1 = INDEX(RECORD,'IMNAME=''')
              K2 = INDEX(RECORD(K1+8:), '''')
              IF (K1.NE.0 .AND. K2.NE.0) 
     1           IDX(64:75) = RECORD(K1+8:K1+6+K2)          
          ELSE IF (KEYW.EQ.'FILENAME') THEN
              IDX(64:75) = CVAL
          ELSE IF (KEYW.EQ.'END     ') THEN
              WRITE (PR,701)
              IF (IDX.NE.' ') WRITE (PR2,'(I3,1X,A)')
     1                          NFIL,IDX(1:LEN1(IDX))
              IDX = ' '
              GOTO 40
          END IF
   36 CONTINUE
      GOTO 30
C
C Count data records.
C
   40 NDREC = 0
      IF (TAPE) THEN
   45     CALL MTRECD(CHAN,MAGIC,BUFFER,BLKLEN,IER)
          IF (MTSEOF(IER)) GOTO 60
          IF (IER.NE.0) GOTO 70
          IF (BLKLEN.NE.MAGIC) GOTO 70
          IF (BLOCK(1:8).EQ.'XTENSION') THEN
              WRITE (PR,703) NDREC
              WRITE (PR,701)
              NREC = 1
              GOTO 35
          END IF
          NDREC = NDREC+1
          GOTO 45
      ELSE
   50     CALL RDFITS (INDAT, ISTAT, BUFFER)
          IF (ISTAT.LT.0) GOTO 60
          IF (ISTAT.GT.0) GOTO 70
          IF (BLOCK(1:8).EQ.'XTENSION') THEN
              WRITE (PR,703) NDREC
              WRITE (PR,701)
              NREC = 1
              GOTO 35
          END IF
          NDREC = NDREC+1
          GOTO 50
      END IF
   60 WRITE (PR,703) NDREC
      WRITE (PR,701)
      WRITE (PR,702)
C
C End of file: go and find another one.
C
      CLOSE (UNIT=INDAT)
      GOTO 20
C
C Error in FITS file. Tape errors are fatal; for disk errors, we try
C the next file.
C
   70 IF (TAPE) THEN
          IF (BLKLEN.NE.MAGIC) THEN
              WRITE (OUTC,'(A,I4,A)') ' File',NFIL,' is not a FITS file'
          ELSE
              WRITE (OUTC,'(A,I4)') ' Error reading tape file', NFIL
          END IF
          GOTO 80
      ELSE
          CALL PUTOUT('Error reading file '//INDSN(1:LDSN))
          WRITE (*,*) 'IOSTAT=',ISTAT
          CLOSE (UNIT=INDAT)
          GOTO 20
      END IF
C
C End of job: print Execution Summary in listing file.
C
   80 IF (PROPEN) THEN
          WRITE (PR,610) 'FITSLIST version:', VERSN
          CALL REPORT(PR)
          IF (TAPE) THEN
              WRITE (PR,610) 'Tape:    ', TDRIVE(1:LEN1(TDRIVE))
          ELSE
              WRITE (PR,610) 'Input:   ', FITFIL(1:LEN1(FITFIL))
          END IF
          WRITE (PR,610) 'Listfile:', LISFIL(1:LEN1(LISFIL))
          WRITE (PR,610) 'Index:   ', INDFIL(1:LEN1(INDFIL))
      END IF
      IF (PROPEN) CLOSE (UNIT=PR)
      IF (PROPEN) CLOSE (UNIT=PR2)
C-----------------------------------------------------------------------
      END

      SUBROUTINE ERRCHK(STATUS, ROUTIN, USRMSG)
*-----------------------------------------------------------------------
* TIO_ERRCHK - Check status from tape I/O operation
*-----------------------------------------------------------------------
      CHARACTER*(*) ROUTIN, USRMSG
      INTEGER       STATUS
*
      INTEGER       LEN1
      CHARACTER*76  ERRMSG
*
*
      IF (STATUS.NE.0) THEN
            CALL MTMESS(STATUS,ROUTIN,ERRMSG)
            CALL PUTOUT(USRMSG)
            CALL PUTOUT(ERRMSG(:LEN1(ERRMSG)))
            STOP
      END IF
*
      END

      SUBROUTINE MTRECD(CHAN,N,BUFFER,RECLEN,IER)
      INTEGER CHAN, N
      BYTE BUFFER(*)
      INTEGER RECLEN
      INTEGER IER
*-----------------------------------------------------------------------
* Read a logical record of data from the tape. The tape may be blocked
* in blocks of up to 28800 bytes.
*-----------------------------------------------------------------------
      INTEGER BLKSIZ
      PARAMETER (BLKSIZ=28800)
      BYTE BLOCK(BLKSIZ)
      INTEGER I, FIRST, BLKLEN, IERR
      SAVE BLOCK, FIRST, BLKLEN, IERR
      DATA FIRST/0/, BLKLEN/0/, IERR/0/
*
      RECLEN = 0
      DO 10 I=1,N
          FIRST = FIRST+1
          IF (FIRST.GT.BLKLEN) THEN
              CALL MTREAD(CHAN,BLKSIZ,BLOCK,BLKLEN,IERR)
C             WRITE (*,*) 'Read',BLKLEN,' bytes, status=',IERR
              FIRST = 1
              IF (BLKLEN.LT.0) GOTO 20
              IF (IERR.NE.0) GOTO 20
          END IF
          BUFFER(I) = BLOCK(FIRST)
          RECLEN = RECLEN+1
   10 CONTINUE
   20 IER = IERR
C     WRITE (*,*) 'Returning',RECLEN,' bytes, status=',IER
      RETURN
      END
