      PROGRAM HAZI
C-----------------------------------------------------------------------
C HAZI calculates visibility of sources at VLBI telescopes and plots
C (u,v)-tracks. Loosely based on Jon Romney's program HAZI.
C
C T.J.Pearson,  VAX-11 Fortran
C
C Version 1.0:  1979 May 10 - new program (T.J. Pearson)
C Version 1.1:  1979 Oct 15
C Version 1.2:  1979 Nov  6 - more than 1 source.
C Version 1.3:  1980 Mar 14 - ask questions.
C Version 1.4:  1980 Mar 25 - options NOTICKS, DOTS, GST.
C Version 1.5:  1980 Jun 30 - increase to 15 stations.
C Version 1.6:  1980 Nov 23 - cure bug in tick marks.
C Version 1.7:  1981 Jun 11 - Fortran-77 standard.
C Version 1.8:  1982 Jun  9 - new graphics devices.
C Version 2.0:  1982 Dec 23 - correct bug for invisible sources
C                             or sources circumpolar about S pole;
C                             add parameter NUP; compare names in upper
C                             case; scale in km if frequency not given.
C Version 2.1:  1984 Sep 25 - add FONT and LINEWIDTH parameters
C                             to match UVPLOT (TJP).
C Version 3.0:  1987 Sep  1 - use parameter input for file names;
C                             remove antiquated GRPCKG calls; increase
C                             to 20 stations (TJP).
C Version 3.1:  1988 Apr  5 - changes for Convex (TJP).
C Version 3.2:  1991 Feb  1 - new catalog format; correct labelling
C                             bugs (TJP).
C Version 3.3:  1993 Jan  4 - avoid crash on zero-length baseline (TJP).
C Version 4.0:  1993 Nov 23 - new parameters NPXY, STCODE, EXIT; 
C                             use TITLE; "*NOLIST*" option  (AKT).
C Version 4.1:  1994 Nov 17 - 12-char source names, aliases (TJP).
C-----------------------------------------------------------------------
      CHARACTER*(*) VERSN
      INTEGER       OUTC,INC,INCAT,PR,NPARS,MAXSRC,MAXS,MAXB
      REAL          IMPOSS
      DOUBLE PRECISION VELC,SIDR,PI,RPDEG,RPHR
C
      PARAMETER (VERSN='4.1 - 1994 Nov 17')
C     -- terminal output unit
      PARAMETER (OUTC=6)
C     -- terminal input unit
      PARAMETER (INC=5)
C     -- unit for source/station catalog
      PARAMETER (INCAT=1)
C     -- printer output
      PARAMETER (PR=8)
C     -- number of control parameters
      PARAMETER (NPARS=104)
C     -- maximum number of sources
      PARAMETER (MAXSRC=10)
C     -- maximum no of stations
      PARAMETER (MAXS=20)
C                        -- NB dimension of FOUND should exceed
C                        -- MAXS and MAXSRC
C     -- maximum no of baselines
      PARAMETER (MAXB=MAXS*(MAXS-1)/2)
C     -- flag for source-not-up
      PARAMETER (IMPOSS=1E10)
C     -- velocity of light m/s
      PARAMETER (VELC=2.99792458D8)
C     -- sidereal-solar ratio
      PARAMETER (SIDR=1.0027379093D0) 
      PARAMETER (PI=3.14159265359D0)
      PARAMETER (RPDEG=PI/180D0)
      PARAMETER (RPHR=PI/12D0)
C
      LOGICAL PRINT1,PRINT2,PLOT,INVERT,BLID,OK,FOUND(20)
      LOGICAL UP(MAXS),LIMITS,TICKS,DOTS,MOVE,DOLAB,LIST,STCODE
C
      CHARACTER*4   LABEL(MAXB)
      CHARACTER*8   FMT,TIMEC
      CHARACTER*12  SNAME(MAXSRC),XSNAME
      CHARACTER*8   STNAME(MAXS),XSTNAM
      CHARACTER*8   STACODE(MAXS)
      CHARACTER*165 LEGND1
      CHARACTER*30  LEGND2
      CHARACTER*32  TITLE
      CHARACTER*64  UNITS
      CHARACTER*128 PLTFIL, LSTFIL, PLTOLD, LSTOLD
      CHARACTER*16  LRA, LDEC
      CHARACTER*80  STAFIL, SRCFIL
      CHARACTER*5   EPOCH(MAXSRC)
C
      INTEGER   MNAME,DNAME,IS1(MAXB),IS2(MAXB),PGBEG
      INTEGER   IB, ID, IER, IM, IPAGE, ALIAS
      INTEGER   ISRC, IS, IY, I, J1, J2, JD, JUNK, J, K
      INTEGER   LINEWD, LJP, LJ, LK, M1, M2, MDATE, MINUP
      INTEGER   MODE, M, NFONT, NL, NPAGE, NS1, NSP, NSRC, NS, NUP
      INTEGER   LEN1, RDSTA, RDSRC
      INTEGER   NPX, NPY, NPXOLD, NPYOLD
C
      REAL      AZX, BX, BY, CHA, COSHAL, COSZAL, DEN, DT, GHA
      REAL      GST0, GST1, GST2, GST, HAIR, HAI
      REAL      U1, UMAX, UTIK
      REAL      UT, UVMAX, U, V0, V1, VMAX, VTIK, V
      REAL      HL(MAXS),HAL(MAXS),ZL(MAXS),CLCD(MAXS),CLSD(MAXS)
      REAL      SLCD(MAXS)
      REAL      SLSD(MAXS),H(MAXB,240),HA(MAXS),ZA(MAXS),AZ(MAXS)
      REAL      T(MAXS),P(MAXS),BZ(MAXB),BE(MAXB),BH(MAXB),BVEC(MAXB)
      REAL      CDEC(MAXSRC),SDEC(MAXSRC)
C
      DOUBLE PRECISION DATE,GMST,GMSTP,RR,TT,PP,FREQ,WL
      DOUBLE PRECISION RA(MAXSRC),DEC(MAXSRC)
      DOUBLE PRECISION CLAT(MAXS),SLAT(MAXS),X(MAXS),Y(MAXS),Z(MAXS)
      DOUBLE PRECISION PARS(NPARS),VALS(NPARS),ENDMRK,BLANK
C
      INCLUDE 'rdcat.inc'
C
      DATA   PARS/ 'PRint', 'NOTicks', 'NOInvert', 'NOBlid',
     1             'TItle',3*' ','NUp','Frequenc', 'UVMax', 'Date',
     2             20*'?', 'RA', 'DEC', 'DOTs',
     3             20*'STATions', 'DT', 2*'GST', 'LINewidt',
     4             'FOnt', 'PLotfile', 9*' ', 'LIStfile', 9*' ',
     5             'STAFile', 9*' ', 'SRCfile', 9*' ', 
     6             2*'NPxy', 'STCode', 'EXit' /
      DATA   VALS/ -1D0, -1D0, -1D0, -1D0,
     1             ' ',3*' ', 2D0, 0D0, 0D0, 0D0,
     2             20*'  ', 0D0, 0D0, -1D0,
     3             2*'***UNSET',18*' ', 0.1D0,2*-1D0, 1D0,
     4             1D0, '*NOPLOT*', 9*' ', 'HAZI.LST', 9*' ',
     5             'VLB_STAT', 'IONS', 8*' ', 'VLB_SOUR', 'CES', 8*' ',
     6              1D0, 1D0, -1D0, -1D0 /
      DATA   ENDMRK/ '/' /,BLANK/' '/

C-----------------------------------------------------------------------
C            Format statements
C-----------------------------------------------------------------------
  500 FORMAT(' HAZI computes u,v-tracks for VLBI experiments'/
     1       ' (Version ',A,')'/)
  501 FORMAT(A)
  601 FORMAT(/' Program HAZI'/' (Version ',A,')'/)
 6021 FORMAT('(million wavelengths,',I6,' MHz)')
 6022 FORMAT(/5X,'Frequency',F11.3,' MHz'/
     1       5X,'Wavelength',-4PF10.3,' cm')
  610 FORMAT(//5X,'Date: ',I4,1X,A3,I3,' (',A3,')'/
     1       5X,'Julian date: ',F12.1/
     2       5X,'Greenwich Mean S.T. at 0h U.T.: ',A8,F4.3)
  640 FORMAT(5X,'Source: ',A12,1X,'RA:',A16,5X,'Dec:',A16)
  620 FORMAT(//' Stations:'//4X,'Name',9X,'Rad./elev.',
     1       6X,'Latitude',17X,'Longitude',19X,'X',12X,'Y',12X,'Z',
     2       7X,'HA/ZA limits')
  621 FORMAT(2X,A8,5X,F9.1,F14.6,10X,F14.6,13X,3F13.1,T121,2F6.1)
  630 FORMAT(//' Baselines (millions of wavelengths - ',
     1       'kms if no frequency is specified):'
     2       //5X,'#',15X,'Stations',20X,
     3       'Bx',13X,'By',13X,'Bz',14X,'Beq',11X,'GHA (h)',
     4        8X,'Bvec')
  631 FORMAT(3X,I3,8X,A8,' -- ',A8,7X,3F15.6,2X,3F14.6)
  651 FORMAT(//3X,'UT',5X,'GST',3X, 100(9X,A8,:,5X))
  652 FORMAT(16X,          100(6X,'HA',4X,'ZA',5X,'AZ',:,1X))
  655 FORMAT(1X,F5.1,2X,F5.1,3X,100(4X,F5.1,2X,F4.1,2X,F5.1,:))
  851 FORMAT('   GST',10X,'Interferometer Hour Angles'
     1       //10X,100(I4,:,3X))
  855 FORMAT(1X,F5.1,4X,100(F5.1,:,2X))
   90 FORMAT(1X)
   91 FORMAT(1H1,10X,A)
  702 FORMAT('DEC =',F6.1)
  703 FORMAT('GST =',F5.1,',',F5.1)
C
C-----------------------------------------------------------------------
C            Read file names and parameters
C-----------------------------------------------------------------------
C
      WRITE (OUTC,500) VERSN
      LSTOLD = '*** UNSPECIFIED ***'
      PLTOLD = '*** UNSPECIFIED ***'
      NPXOLD = 1
      NPYOLD = 1
C
C Control parameters.
C
      MODE = 0
      DO 1 ISRC=1,MAXSRC
         J = 12+ISRC*2-1
         PARS(J) = 'Source'
         PARS(J+1) = ' '
 1    CONTINUE
   10 IF (MODE.NE.0) CALL ERROR('Bad input parameters in HAZI.')
      WRITE (OUTC,501) ' Parameters (type SHOW for complete list)'
      WRITE (OUTC,501) ' eg PLOTFILE, LISTFILE, SOURCES (or RA, DEC),'
     1                //' STATIONS, FREQ:'
  100 MODE = 0
      CALL KEYIN(PARS,VALS,NPARS,ENDMRK,MODE,INC,OUTC)
C     -- test for end-of-file or EXIT
      IF (MODE.EQ.1) GOTO 9000
      IF (VALS(104).NE.-1D0) GOTO 9000
C
C
C Plotfile.
C
      WRITE (PLTFIL,'(10A8)') (VALS(J),J=61,70)
      PLOT = PLTFIL.NE.'*NOPLOT*'
      LINEWD = VALS(59)
      NFONT  = VALS(60)
      NPX    = VALS(101)
      NPY    = VALS(102)
      STCODE = VALS(103).GE.0D0
      IF (PLOT) THEN
        IF (PLTFIL.NE.PLTOLD .OR. NPX.NE.NPXOLD .OR. NPY.NE.NPYOLD) THEN
          IER = PGBEG(9,PLTFIL,NPX,NPY)
          IF (IER.NE.1) GOTO 10
          CALL PGSLW(LINEWD)
          CALL PGSCF(NFONT)
          PLTOLD = PLTFIL
          NPXOLD = NPX
          NPYOLD = NPY
        END IF
      END IF
C
C Listfile.
C
      WRITE (LSTFIL,'(10A8)') (VALS(J),J=71,80)
      LIST = LSTFIL.NE.'*NOLIST*'
      IF (LIST .AND. LSTFIL.NE.LSTOLD) THEN
         OPEN (UNIT=PR, FILE=LSTFIL, STATUS='NEW', FORM='FORMATTED',
     1      IOSTAT=IER)
         IF (IER.NE.0) THEN
             WRITE (OUTC,501) ' +++ERROR+++ Cannot open LISTFILE'
             GOTO 10
         END IF
      END IF
C
C Other parameters.
C
      WRITE (STAFIL,'(10A8)') (VALS(J),J=81,90)
      WRITE (SRCFIL,'(10A8)') (VALS(J),J=91,100)
      PRINT1 = VALS(1).EQ.0D0 .OR. VALS(1).EQ.1D0
      PRINT2 = VALS(1).EQ.0D0 .OR. VALS(1).EQ.2D0
      TICKS  = VALS(2).EQ.-1D0
      DOTS   = VALS(35).EQ.0D0
      INVERT = VALS(3).EQ.-1D0
      BLID   = VALS(4).EQ.-1D0
      WRITE (TITLE,'(4A8)') (VALS(J),J=5,8)
      MINUP  = VALS(9)
C     -- frequency/MHz
      FREQ   = VALS(10)
      IF (FREQ.LE.0D0) THEN
          FREQ = 0D0
          WL = 1000
C         -- 1 km/m
          UNITS = '(kilometers)'
      ELSE
          WL = VELC/FREQ
C         -- 1 million wavelengths /m
          WRITE (UNITS,6021) NINT(FREQ)
      END IF
      UVMAX  = VALS(11)
      DT     = VALS(56)
C     -- time between points (hours)
      IF(DT.LT.0.1) DT=0.1
      GST1   = VALS(57)
      GST2   = VALS(58)
      LIMITS = GST1.GT.0.0 .AND. GST2.GT.0.0
      IF (GST2.LT.GST1) GST2 = GST2+24.0
      IF (GST2-GST1.GT.24.0) LIMITS = .FALSE.
C
      IF(LIST) THEN
         WRITE (PR,601) VERSN
         IF (FREQ.GT.0.0) WRITE (PR,6022) FREQ,WL
      END IF
C-----------------------------------------------------------------------
C      U.T. - G.S.T. conversion
C      If date not supplied, use today's date
C      DATE = Julian Date
C      GMST = Greenwich Mean S.T. at 0 U.T. (sec)
C-----------------------------------------------------------------------
      MDATE = VALS(12)
      IY = MOD(MDATE/3600,100)
      IM = MOD(MDATE,3600)/60
      ID = MOD(MDATE,60)
      IF (MDATE.LE.0) CALL IDATE(IM,ID,IY)
      IY = 1900 + IY
      CALL TDATE(IY,IM,ID,JD,MNAME,DNAME)
      CALL JULDA(IY,IM,ID,DATE,GMST)
      GMSTP = DMOD(GMST,1D0)
      IF(LIST) THEN
         WRITE(PR,610) IY,MNAME,ID,DNAME,DATE,TIMEC(IDINT(GMST)),GMSTP
      END IF
      GMST = GMST/3600D0
C-----------------------------------------------------------------------
C      Sources:
C      If RA, DEC not supplied, look up source in catalog
C-----------------------------------------------------------------------
      NSRC = 0
      SNAME(1) = ' '
      DO 1040 ISRC=1,MAXSRC
         J = 12+ISRC*2-1
         IF (VALS(J).NE.BLANK) THEN
            NSRC = NSRC+1
            WRITE (SNAME(NSRC),'(A8,A4)') VALS(J),VALS(J+1)
            CALL UPCASE(SNAME(ISRC))
            FOUND(NSRC) = .FALSE.
         END IF
 1040 CONTINUE
C
      IF(VALS(33).NE.0D0 .OR. VALS(34).NE.0D0) THEN
          FOUND(1) = .TRUE.
          RA(1) = VALS(33)/3600.0
          DEC(1) = VALS(34)/3600.0
          NSRC = MAX(NSRC,1)
      END IF
      IF(NSRC.EQ.1 .AND. FOUND(1)) GOTO 1200
C
 1100 IER = RDSRC(INCAT, .TRUE., SRCFIL)
      IF (IER.EQ.1) GOTO 1800
      XSNAME = SRCNAM(1)
      CALL UPCASE(XSNAME)
      DO 1150 ISRC=1,NSRC
         DO 1140 ALIAS=1,5
            XSNAME = SRCNAM(ALIAS)
            CALL UPCASE(XSNAME)
            IF(SNAME(ISRC).EQ.XSNAME) THEN
               RA(ISRC) = SRCRA*12D0/PI
               DEC(ISRC) = SRCDEC*180D0/PI
               EPOCH(ISRC) = SRCEPO
               FOUND(ISRC) = .TRUE.
            END IF
 1140    CONTINUE
 1150 CONTINUE
      GOTO 1100
 1800 CLOSE (UNIT=INCAT)
C
 1200 OK = .TRUE.
      DO 1250 ISRC=1,NSRC
        IF (FOUND(ISRC)) THEN
          CALL PANGLE('H', RA(ISRC)*PI/12D0, LRA, 3)
          CALL PANGLE('D', DEC(ISRC)*PI/180D0, LDEC, 3)
          IF(LIST) WRITE(PR,640) SNAME(ISRC),LRA,LDEC
          CDEC(ISRC) = DCOS(DEC(ISRC)*PI/180D0)
          SDEC(ISRC) = DSIN(DEC(ISRC)*PI/180D0)
        ELSE
          WRITE (OUTC,501) ' +++ERROR+++ Source not found'//
     1                  ' in catalog: '//SNAME(ISRC)
          OK = .FALSE.
        END IF
 1250 CONTINUE
C
C-----------------------------------------------------------------------
C            Station coordinates
C-----------------------------------------------------------------------
      NS = 0
      DO 2100 I=1,MAXS
        IF(VALS(35+I).EQ.BLANK) GOTO 2100
          NS = NS+1
          WRITE (STNAME(NS),501) VALS(35+I)
          CALL UPCASE(STNAME(NS))
          FOUND(NS) = .FALSE.
 2100 CONTINUE
      IF (NS.LT.1) CALL ERROR('No stations')
C
      IF(LIST) WRITE (PR,620)
C
 2200 IER = RDSTA(INCAT, .TRUE., STAFIL)
      IF (IER.EQ.1) GOTO 2400
      XSTNAM = STANAM
      CALL UPCASE(XSTNAM)
      DO 2300 M=1,NS
      IF (XSTNAM.NE.STNAME(M)) GOTO 2300
          STACODE(M) = STACOD
          ZL(M) = STAZAL
          HL(M) = STAHAL
          IF (HL(M).EQ.0.0) HL(M)=12.0
          HL(M) = ABS(HL(M))
          IF (ZL(M).EQ.0.0) ZL(M)=90.0
C         -- latitude (deg)
          TT = STALAT/RPDEG
C         -- w longitude (deg)
          PP = STALON/RPDEG
          RR = STAEL
          IF (STAEL.LT.6D6) CALL GEOID(RR,TT)
          SLAT(M) = SIN(STALAT)
          CLAT(M) = COS(STALAT)
          T(M) = TT
          P(M) = PP/15D0
          X(M) = RR*CLAT(M)*COS(STALON)
          Y(M) = RR*CLAT(M)*SIN(STALON)
          Z(M) = RR*SLAT(M)
          IF(LIST) THEN
            WRITE(PR,621) XSTNAM,RR,TT,PP,X(M),Y(M),Z(M),HL(M),ZL(M)
          END IF
          FOUND(M) = .TRUE.
 2300 CONTINUE
      GOTO 2200
C
 2400 CLOSE(UNIT=INCAT)
      DO 2500 M=1,NS
          IF (FOUND(M)) GOTO 2500
          WRITE (OUTC,501) ' +++ERROR+++ Station not found'//
     1              ' in catalog: '//STNAME(M)
          OK = .FALSE.
 2500 CONTINUE
      IF (.NOT.OK) CALL ERROR('Unknown source(s) or stations(s)')
C-----------------------------------------------------------------------
C            Baselines
C-----------------------------------------------------------------------
      J = 0
      IF(LIST) WRITE(PR,630)
      DO 3100 M2=2,NS
      NS1 = M2-1
      DO 3100 M1=1,NS1
          J = J+1
          IS1(J) = M1
          IS2(J) = M2
          BX = (X(M1)-X(M2))/WL
          BY = (Y(M1)-Y(M2))/WL
          BZ(J) = (Z(M1)-Z(M2))/WL
          BVEC(J) = SQRT(BX**2+BY**2+BZ(J)**2)
          BE(J) = SQRT(BX**2+BY**2)
          IF (BE(J).EQ.0.0) THEN
              BH(J) = 0.0
          ELSE
              BH(J) = ATAN2(BY,BX)/RPHR
          END IF
          IF(LIST) WRITE(PR,631) J,STNAME(M1),STNAME(M2),
     1                     BX,BY,BZ(J),BE(J),BH(J),BVEC(J)
          FMT = '(I2,2X)'
          IF (J.GT.9) FMT='(I3)'
          WRITE (LABEL(J),FMT) J
 3100 CONTINUE
      LJ=J
C
C-----------------------------------------------------------------------
C            Begin loop for sources
C-----------------------------------------------------------------------
C
      DO 8000 ISRC=1,NSRC
C
C            Determine hour-angle limits at each station
C
      DO 4200 IS=1,NS
          CLCD(IS) = CLAT(IS)*CDEC(ISRC)
          CLSD(IS) = CLAT(IS)*SDEC(ISRC)
          SLCD(IS) = SLAT(IS)*CDEC(ISRC)
          SLSD(IS) = SLAT(IS)*SDEC(ISRC)
          COSZAL   = DCOS(ZL(IS)*RPDEG)
          COSHAL   = (COSZAL-SLSD(IS))/CLCD(IS)
          IF (COSHAL.GE.1.0) THEN
C            -- Source is never visible
            HAL(IS) = -0.01
            IF(LIST) THEN
              WRITE (PR,501) ' ++WARNING++ Source '//SNAME(ISRC)//
     1                  ' cannot be seen from station '//STNAME(IS)
            END IF
            WRITE (OUTC,501) ' ++WARNING++ Source '//SNAME(ISRC)//
     1                  ' cannot be seen from station '//STNAME(IS)
          ELSE IF (COSHAL.LE.-1.0 .AND. HL(IS).GE.12.0) THEN
C            -- Source is circumpolar
            HAL(IS) = 12.0
          ELSE IF (COSHAL.LE.-1.0) THEN
C            -- Circumpolar, but HA limit is effective
            HAL(IS) = HL(IS)
          ELSE
C            -- ZA limit is effective; HA limit may be
            HAL(IS) = ACOS(COSHAL)/RPDEG/15.0
            HAL(IS) = MIN(HAL(IS),HL(IS))
          END IF
 4200 CONTINUE
C
      GST = DMOD(12.0+RA(ISRC)-DT,24D0)
      IF (GST.LT.0.0) GST = GST+24.0
      IF (LIMITS) GST = GST1
C     -- supplied start time
      GST0 = IFIX(GST/DT)*DT
C
C-----------------------------------------------------------------------
C            Print Section 1:  HA, ZA, AZ
C-----------------------------------------------------------------------
C
      LK = 24.0/DT
      IF (LIMITS) LK = (GST2-GST1)/DT
C     -- supplied time range
      NPAGE = NS/5
      IF(NS.GT.NPAGE*5) NPAGE=NPAGE+1
      IF(.NOT.PRINT1) NPAGE = 1
      DO 5100 IPAGE=1,NPAGE
      M1 = (IPAGE-1)*5 + 1
      M2 = MIN0(M1+4,NS)
      NSP = M2 - M1 + 1
      NL = 0
      DO 5100 K=1,LK
          GST = GST0 + K*DT
          IF(GST.GE.24.0) GST=GST-24.0
          GHA = GST-RA(ISRC)
C         -- Greenwich Hour Angle of source
          NUP = 0
C
C            Compute HA, ZA, AZ for each station
C
          DO 5200 IS=1,NS
            HA(IS) = AMOD(GHA-P(IS),24.0)
            IF (HA(IS).LT.-12.0) HA(IS) = HA(IS)+24.0
            IF (HA(IS).GT.12.0)  HA(IS) = HA(IS)-24.0
            CHA = COS(HA(IS)*RPHR)
            IF (ABS(HA(IS)).LE.HAL(IS)) THEN
                ZA(IS) = ACOS(SLSD(IS)+CLCD(IS)*CHA)/RPDEG
                DEN = CDEC(ISRC)*DSIN(HA(IS)*RPHR)
                AZX = ATAN2(CLSD(IS)-SLCD(IS)*CHA, DEN)/RPDEG
                AZ(IS) = AMOD(AZX-90.0,360.0)
                IF (AZ(IS).LT.0.0) AZ(IS)=AZ(IS)+360.0
                UP(IS) = .TRUE.
                NUP = NUP+1
            ELSE
                HA(IS) = IMPOSS
                ZA(IS) = IMPOSS
                AZ(IS) = IMPOSS
                UP(IS) = .FALSE.
            END IF
 5200     CONTINUE
C
C            Compute IHA for each baseline
C
          DO 5300 IB=1,LJ
            IF (NUP.GE.MINUP .AND. UP(IS1(IB)).AND.UP(IS2(IB))) THEN
                H(IB,K) = AMOD(GHA-BH(IB)-6.0, 24.0)
                IF (H(IB,K).LT.-12.0) H(IB,K) = H(IB,K)+24.0
                IF (H(IB,K).GT.12.0)  H(IB,K) = H(IB,K)-24.0
            ELSE
                H(IB,K) = IMPOSS
            END IF
 5300     CONTINUE
C
C            Print section 1: HA, ZA, AZ for each station
C
          IF(.NOT.LIST) GOTO 5100
          IF (PRINT1 .AND. NUP.GE.MINUP) THEN
            UT = DMOD((GST-GMST)/SIDR,24D0)
            IF (UT.LT.0.0) UT = UT+24.0
            NL = NL+1
            IF (MOD(NL,50).EQ.1) THEN
                WRITE (PR,91)  SNAME(ISRC)
                WRITE (PR,651) (STNAME(M),M=M1,M2)
                WRITE (PR,652)
            END IF
            IF (MOD(NL,10).EQ.1) WRITE(PR,90)
            WRITE (PR,655,IOSTAT=JUNK) UT,GST,
     1                  (HA(M),ZA(M),AZ(M),M=M1,M2)
          END IF
 5100 CONTINUE
C
C-----------------------------------------------------------------------
C            Print Section 2: IHA
C-----------------------------------------------------------------------
      IF (PRINT2 .AND. LIST) THEN
          NPAGE = LJ/15
          IF (LJ.GT.NPAGE*15) NPAGE = NPAGE + 1
          DO 6500 IPAGE = 1,NPAGE
            J1 = (IPAGE-1)*15 + 1
            J2 = MIN(J1+14,LJ)
            LJP = J2 - J1 + 1
            NL = 0
            DO 6400 K=1,LK
                DO 6100 IB=1,LJ
                  IF (H(IB,K).NE.IMPOSS) GOTO 6200
 6100           CONTINUE
                GOTO 6400
C
 6200                GST = GST0 + K*DT
                IF (GST.GT.24.0) GST = GST-24.0
                NL = NL+1
                IF (MOD(NL,50).EQ.1) THEN
                  WRITE (PR,91) SNAME(ISRC)
                  WRITE (PR,851) (IB,IB=J1,J2)
                END IF
                IF (MOD(NL,5).EQ.1) WRITE(PR,90)
                WRITE (PR,855,ERR=6400) GST,(H(IB,K),IB=J1,J2)
 6400       CONTINUE
 6500     CONTINUE
      END IF
C-----------------------------------------------------------------------
C            Plot section
C-----------------------------------------------------------------------
C
      IF (.NOT.PLOT) GOTO 8000
      IF (UVMAX.NE.0.0) THEN
          UMAX = UVMAX
          VMAX = UVMAX
      ELSE
          UMAX = 0.0
          VMAX = 0.0
          DO 7100 IB=1,LJ
            UMAX = MAX(UMAX,BE(IB))
            VMAX = MAX(VMAX, ABS(BZ(IB)*CDEC(ISRC)) +
     1                       ABS(BE(IB)*SDEC(ISRC)) )
 7100     CONTINUE
          UMAX = 1.2*UMAX
          VMAX = 1.2*VMAX
      END IF
C
      IF(STCODE) THEN
        LEGND1 = STACODE(1)
        DO 7150 IS=2,NS
          I = LEN1(LEGND1)+2
          LEGND1(I:) = STACODE(IS)
 7150   CONTINUE
      ELSE
        LEGND1 = STNAME(1)
        DO 7160 IS=2,NS
          I = LEN1(LEGND1)+2
          LEGND1(I:) = STNAME(IS)
 7160   CONTINUE
      END IF
      WRITE (LEGND2,702) DEC(ISRC)
      IF (LIMITS) WRITE (LEGND2(15:),703) GST1,GST2
      CALL PGBBUF
      CALL PGSCI(5)
      CALL PGENV(UMAX,-UMAX,-VMAX,VMAX,1,1)
      CALL PGLAB(UNITS,' ', ' ')
      CALL PGMTXT('T', 2.0, 0.0, 0.0, SNAME(ISRC))
      CALL PGSCH(0.6)
      CALL PGSLW(1)
      CALL PGMTXT('T', 0.7, 0.0, 0.0, LEGND1)
      CALL PGSLW(LINEWD)
      CALL PGSCH(2.0)
      CALL PGMTXT('T', 1.0, 0.5, 0.5, TITLE)
      CALL PGSCH(1.0)
      DO 7200 J=1,LJ
          U1 = BE(J)
          V0 = BZ(J)*CDEC(ISRC)
          V1 = BE(J)*SDEC(ISRC)
          MOVE = .TRUE.
          DOLAB = BLID
          CALL PGSCI(3)
          DO 7300 K=1,LK
            HAI = H(J,K)
            IF(HAI.EQ.IMPOSS) THEN
                  MOVE = .TRUE.
                  GOTO 7300
            END IF
            HAIR = HAI*RPHR
            U = U1*COS(HAIR)
            V = V0 + V1*SIN(HAIR)
            IF (DOLAB .AND. ABS(U).LT.UMAX .AND.
     1                      ABS(V).LT.VMAX) THEN
C               -- write label
                CALL PGSCH(0.6)
                CALL PGSCI(5)
                CALL PGTEXT(U, V, LABEL(J))
                CALL PGSCI(3)
                CALL PGSCH(1.0)
                DOLAB = .FALSE.
            END IF
            IF (DOTS) THEN
                CALL PGPT(1, U, V, -1)
            ELSE IF (MOVE) THEN
                CALL PGMOVE(U, V)
                MOVE = .FALSE.
            ELSE
                CALL PGDRAW(U, V)
            END IF
            IF(TICKS .AND. ABS(HAI-NINT(HAI)).LT.DT/2.0) THEN
                HAIR = REAL(NINT(HAI))*RPHR
                UTIK = U1*COS(HAIR)
                VTIK = V0 + V1*SIN(HAIR)
                CALL PGPT(1, UTIK, VTIK, 2)
                IF (INVERT) CALL PGPT(1, -UTIK, -VTIK, 2)
                CALL PGMOVE(U, V)
                END IF
 7300     CONTINUE
          IF (INVERT) THEN
              DO 7310 K=1,LK
                  HAI = H(J,K)
                  IF (HAI.NE.IMPOSS) THEN
                      HAIR = HAI*RPHR
                      U = -U1*COS(HAIR)
                      V = -(V0 + V1*SIN(HAIR))
                      CALL PGPT(1, U, V, -1)
                  END IF
 7310         CONTINUE
          END IF
 7200 CONTINUE
      CALL PGEBUF
C
 8000 CONTINUE
C
C Loop back for another iteration
      GOTO 100
C
C Here for termination
 9000 CONTINUE 
      IF(PLOT) CALL PGEND
      IF(LIST) CLOSE (UNIT=PR)
      STOP
C-----------------------------------------------------------------------
      END
