      PROGRAM IED
C-----------------------------------------------------------------------
C      This program is a simple interactive editor for VLB data.
C
C      By Richard S. Simon
C
C      v2: 23-July-1982
C      v3: Add option of plotting on VT125 (/VT) and VT640 (/RE)
C          terminals; add commands X (erase text), V (refresh plot),
C          I (inquire), J (jump), C (cut data into windows), and
C          G (glue windows); make D and R commands advance cursor;
C          plot more than one window on screen
C          5-April-1983
C          David L. Meier
C      v5: Fold in Richard Simon's changes:
C          (v3: 16-FEB-1983   Add capability for ID100 terminal)
C          (v4: 7-Mar-1983        Add baseline phase editing)
C          3-November-1983
C          David L. Meier
C      v6: T. J. Pearson, 1986 July 13: remove ZOOM and UNZOOM commands;
C          add display of UT in Grinnell prompt; make S skip to the
C          beginning of the next hour; add K, A, and U commands.
C      v6.1: T.J. Pearson, 1986 Aug 28: increase from 15 to 20 stations.
C      v6.2: Add Z command (J. Biretta, T. Pearson) 1987 Mar 23
C      v6.3: fix for Merge format 2 (T. Pearson) 1987 Apr 15
C      v6.4: update PGBBUF/PGEBUF/PGUPDT calls; remove dependence on
C          PGPLOT internals [can now use shareable library]; change
C          G command to remove all cuts and redraw screen (T. Pearson)
C          1987 Apr 27
C      v6.5: changes for VAX workstations; eliminate multiple PGBEGIN
C          calls (T. Pearson) 1988 Feb 5
C      v7.0: changes for Convex (TJP 1988 Apr 19)
C      v7.1: add end commands to output file on exit (TJP 1988 Oct 21)
C      v7.2: detect whether VMS or Unix output required (TJP 1988Oct29)
C      v7.3: use PGQCOL to find device capability (TJP 1989 May 31)
C      v7.4: start at first baseline, not 1,2 (TJP 1990 Oct 3)
C      v8.0: Fortran standardization (TJP 1991 May 30)
C-----------------------------------------------------------------------
C
      CHARACTER*(*) VERSN
      PARAMETER (VERSN='8.0 - 1991 May 30')
C
C The following contains common blocks and parameters:
C
      INCLUDE 'ied.inc'
      CHARACTER*(LENDSN) DEFDSN
      INTEGER PGBEGIN, GETIN, VLBOPE, LEN1
      LOGICAL QUERY
      CHARACTER*32 OPSYS
      CHARACTER*32 ANSWER
      INTEGER I, K, L
C-----------------------------------------------------------------------
C
C Find the name of the data set, and retrieve the data:
C
      CALL PUTOUT('IED: interactive editor for MERGE-format '//
     1            'visibility files')
      CALL PUTOUT('(Version '//VERSN//')')
      CALL PUTOUT(' ')
   10 IER = GETIN(DATDSN, 'Input data file: ', L)
      IF (IER.NE.1) STOP
      IER = VLBOPE(INDAT, DATDSN, 'MERGE', 'OLD', DATDSN)
      IF (IER.NE.1) GOTO 10
      NPLOT = 0
      NPLOT0 = 0
      TRYONE = .TRUE.
      PLTAMP = .TRUE.
      CALL GETDAT
      IF (IER .NE. 1) THEN    
C         -- Error, give user another chance
          CALL PUTOUT('I had an error while reading data file.'//
     +                '  Please try again.')
          GOTO 10
      ENDIF
C
C Find the name of the output data set: find the input extension, and
C change it to '.EDT'.
C
      L = LEN1(DATDSN)
      K = -1
      I = 0
      DO WHILE (I. EQ. 0)
          K = K+1
          I = INDEX(DATDSN(L-K:L),'.')
      ENDDO
      DEFDSN = DATDSN(1:L-K)//'EDT'
      L = LEN1(DEFDSN)
      CALL PUTOUT(' ')
   15 IER = GETIN(EDIDSN, 'Output data file (def='//DEFDSN(1:L)//
     1            '): ', L)
      IF (IER.NE.1) STOP
      IF (L.LT.1) EDIDSN = DEFDSN
C      Test to see if file name entered will work:
      OPEN (UNIT=OUTDAT,FORM='UNFORMATTED',ERR=15,FILE=EDIDSN,
     +      STATUS='NEW')
      CLOSE (OUTDAT, STATUS='DELETE')
C
C Determine operating system (VMS or UNIX).
C
      CALL SYSVER(OPSYS)
      VMS = INDEX(OPSYS,'VMS').NE.0
C
C Open output parameter file.
C
      CALL PUTOUT(' ')
   20 IF (VMS) THEN
         IER = GETIN(PRMDSN,
     1         'Output VMS DCL command file [default: IED.COM]: ', L)
         IF (IER.NE.1) STOP
         IF (L.EQ.0) PRMDSN = 'IED.COM'
      ELSE
         IER = GETIN(PRMDSN,
     1         'Output UNIX csh command file [default: ied.csh]: ', L)
         IF (IER.NE.1) STOP
         IF (L.EQ.0) PRMDSN = 'ied.csh'
      END IF
      IER = VLBOPE(PARMU, PRMDSN, 'TEXT', 'NEW', PRMDSN)
      IF (IER.NE.1) GOTO 20
      CALL PUTOUT('Parameters for VLBEDIT will go to file '//
     +      PRMDSN(1:LEN1(PRMDSN)))
      CALL PRMINI
C
C Decide what plot device to use and open plot device.
C
      CALL PUTOUT(' ')
      CALL PUTOUT('What device do you want to use for plotting?')
      CALL PUTOUT('Supply a PGPLOT device/type specification:')
      CALL PGLDEV
   25 IER = GETIN(PLTOUT, 'Plot device/type: ', L)
      IF (IER.NE.1) STOP
      IER = PGBEGIN(0,PLTOUT,1,1)
      IF (IER.NE.1) GOTO 25
C
C Decide if plot will use error bars or not.
C
      CALL PUTOUT(' ')
      PLTBAR = QUERY('Plot error bars? (Y or N): ')
C
C Find starting baseline or triangle:
C
      CALL PUTOUT(' ')
   40 IER = GETIN(ANSWER,
     1   'Starting baseline or triangle? (e.g. 1,2,3; def=1,2): ', L)
      IF (IER.NE.1) STOP
      READ (ANSWER(:L),'(3I10)',ERR=40) ISTA1,ISTA2,ISTA3
      IF (ISTA1 .LE. 0 .OR. ISTA2 .LE. 0) THEN
          ISTA1 = NAVIS(1)
          ISTA2 = NBVIS(1)
          ISTA3 = 0
      ENDIF
C
C Find out if baseline amplitudes or baseline phases are to be edited.
C
      IF (ISTA3.LE.0) THEN
   60     CALL PUTOUT(' ')
          IER = GETIN(ANSWER, 'Will you be editing amplitudes OR '//
     1                'baseline phases ? (A or P; def=A) ', L)
          IF (IER.NE.1) STOP
          CALL UPCASE (ANSWER)
          IF (ANSWER(1:1) .EQ. 'A' .OR. ANSWER(1:1) .EQ. ' ') THEN
              PLTAMP = .TRUE.
              PLTPHS = .FALSE.
          ELSEIF (ANSWER(1:1) .EQ. 'P') THEN
              PLTAMP = .FALSE.
              PLTPHS = .TRUE.
          ELSE
              CALL PUTOUT('Please answer the question.')
              GO TO 60
          ENDIF
      ELSE
          PLTAMP = .FALSE.
          PLTPHS = .FALSE.
      ENDIF
C
C Do initial plot to device PLTOUT:.
C
      NEWPLT = .TRUE.
      CALL GETNPL(ISTA1,ISTA2,ISTA3)
      IF (IER.NE.0) THEN
          CALL PLTALL
      ELSE
          CALL PUTOUT('Didn''t work.  Please try again.')
          GOTO 40
      ENDIF
      IF (IER.EQ.2) THEN
C        No data for starting data subset, go to next one
        CALL NEXTPL
        IF (IER .EQ. 3) THEN
              CALL PRMEND
              GOTO 9999
        END IF
      ENDIF
C
C      Start editing:
      CALL IEDIT
C
      CALL PGEND
C
 9999 STOP
C-----------------------------------------------------------------------
      END

      SUBROUTINE IEDIT
C-----------------------------------------------------------------------
C This subroutine is the 'heart' of IED.
C-----------------------------------------------------------------------
      INCLUDE 'ied.inc'
      CHARACTER*1 LETTER
      CHARACTER*8 TIMEC
      CHARACTER*32 ANSWER, COLOR
      INTEGER CICUR, CICURD, CI1, CI2
      INTEGER I, NPT
      INTEGER ISTART, ID, ISTOP, IPSAV, ISTA01, ISTA02, ISTA03
      INTEGER IZPOLD, IZP, IZ
      INTEGER IZSAV, IZDEL, ISEC, ICMD, NARROW
      LOGICAL BAD, IEDQ
      REAL    XP1, YP1, THRESH
      REAL    A1, A2, A3, UTSTOP
      REAL    ZVPX1, ZVPX2, ZVPY1, ZVPY2, ZX1, ZX2, ZY1, ZY2
      REAL    FZXSCL, FZYSCL, ZX, ZY, FZDSAV, FZDX, FZDY, FZDIST
      REAL    WORK(MXREC)
C-----------------------------------------------------------------------
C
C The array ICOLOR() indicates the color to be used to display each data
C point: CYAN (if not yet visited), GREEN (if accepted), or RED (if
C deleted).  Special colors CICUR and CICURD are used to display the
C current point (the one under the cursor); CICUR is used if the point
C is not deleted, CICURD if it is.
C
      CICUR  = WHITE
      CICURD = MAGEN
C
C Some devices behave differently.
C
      CALL PGQCOL(CI1, CI2)
      WSDEV = (CI1.LE.0) .AND. (CI2.GE.7)
C
C on color devices, set color representation.
C
      IF (WSDEV) THEN
          OPEN (UNIT=17, FILE='ied.colors', READONLY, STATUS='OLD',
     1          ERR=57)
C         -- Background
          READ(17,'(A)',ERR=57) COLOR
          CALL PGSCRN(BLACK, COLOR, IER)
C         -- Cursor
          READ(17,'(A)',ERR=57) COLOR
          CALL PGSCRN(WHITE, COLOR, IER)
C         -- Deleted data
          READ(17,'(A)',ERR=57) COLOR
          CALL PGSCRN(RED, COLOR, IER)
C         -- Good data
          READ(17,'(A)',ERR=57) COLOR
          CALL PGSCRN(GREEN, COLOR, IER)
C         -- Unseen data
          READ(17,'(A)',ERR=57) COLOR
          CALL PGSCRN(CYAN, COLOR, IER)
C         -- Cursor (when on deleted point)
          READ(17,'(A)',ERR=57) COLOR
          CALL PGSCRN(MAGEN, COLOR, IER)
C         -- Frame and labels
          READ(17,'(A)',ERR=57) COLOR
          CALL PGSCRN(YELLOW, COLOR, IER)
          CLOSE (UNIT=17)
   57     CONTINUE
      END IF
C
C Plot the first point to be considered.
C
      IPOINT = 1
C
C Start plotting.
C
      CALL PGSCI(CICUR)
      CALL IEDPNT(XP1,YP1)
C
C Give instructions on initial call.
C
      ANSWER = 'H'
    3 CONTINUE
C
C Decide on action to be taken, and do it.
C
C ---- F command: move pointer to next point, if possible --------------
C
      IF ((ANSWER(1:1).EQ.'F') .OR. (ANSWER(1:1).EQ.CHAR(13))) THEN
C        Move pointer to next point, if possible
        IF (IPOINT .EQ. IDATA) THEN
          CALL PUTOUT('No more data for this subset'//
     +          ', use another command.')
          GO TO 700
        ELSE
          IF (ICOLOR(IPOINT) .EQ. CYAN) ICOLOR(IPOINT) = GREEN
          CALL PGSCI(ICOLOR(IPOINT))
          CALL IEDPNT(XP1,YP1)
          IPOINT = IPOINT + 1
          IF (ICOLOR(IPOINT) .EQ. RED) THEN
              CALL PGSCI(CICURD)
          ELSE
              CALL PGSCI(CICUR)
          END IF
          CALL IEDPNT(XP1,YP1)
        ENDIF
C
C ---- B command: move backwards, if possible --------------------------
C
      ELSE IF (ANSWER(1:1) .EQ. 'B') THEN
        IF (IPOINT .EQ. 1) THEN
          CALL PUTOUT('No earlier data for this subset'//
     +          ', use another command.')
          GO TO 700
        ELSE
          IF (ICOLOR(IPOINT) .EQ. GREEN) ICOLOR(IPOINT) = CYAN
          CALL PGSCI(ICOLOR(IPOINT))
          CALL IEDPNT(XP1,YP1)
          IPOINT = IPOINT - 1
          IF (ICOLOR(IPOINT) .EQ. RED) THEN
              CALL PGSCI(CICURD)
          ELSE
              CALL PGSCI(CICUR)
          END IF
          CALL IEDPNT(XP1,YP1)
        ENDIF
C
C ---- D command: delete current point and move to next one ------------
C
      ELSE IF (ANSWER(1:1) .EQ. 'D') THEN
        ICOLOR(IPOINT) = RED
        IF (IPOINT .EQ. IDATA) THEN
            CALL PGSCI(CICURD)
        ELSE
          CALL PGSCI(ICOLOR(IPOINT))
          CALL IEDPNT(XP1,YP1)
          IPOINT = IPOINT + 1
          IF (ICOLOR(IPOINT) .EQ. RED) THEN
              CALL PGSCI(CICURD)
          ELSE
              CALL PGSCI(CICUR)
          END IF
        ENDIF
        CALL IEDPNT(XP1,YP1)
C
C ---- R command: restore current point and move to next one -----------
C
      ELSEIF (ANSWER(1:1) .EQ. 'R') THEN
        ICOLOR(IPOINT) = GREEN
        IF (IPOINT .EQ. IDATA) THEN
            CALL PGSCI(CICUR)
        ELSE
          CALL PGSCI(ICOLOR(IPOINT))
          CALL IEDPNT(XP1,YP1)
          IPOINT = IPOINT + 1
          IF (ICOLOR(IPOINT) .EQ. RED) THEN
              CALL PGSCI(CICURD)
          ELSE
              CALL PGSCI(CICUR)
          END IF
        ENDIF
        CALL IEDPNT(XP1,YP1)
C
C ---- K command: delete points below threshold ------------------------
C
      ELSE IF (ANSWER(1:1) .EQ. 'K') THEN
          CALL PUTOUT('Move cursor to desired position, and '//
     +                      'then type any character')
          THRESH = YP1
          CALL PGCURS(XP1,THRESH,LETTER)
          CALL PGBBUF
          CALL PGSCI(ICOLOR(IPOINT))
          CALL IEDPNT(XP1,YP1)
          CALL PGSCI(RED)
          DO I=1,IDATA
              IF (AMP(NDATA(I)).LT.THRESH) THEN
                  IPOINT = I
                  ICOLOR(IPOINT) = RED
                  CALL IEDPNT(XP1,YP1)
              END IF
          END DO
          IPOINT = 1
          IF (ICOLOR(IPOINT) .EQ. RED) THEN
              CALL PGSCI(CICURD)
          ELSE
              CALL PGSCI(CICUR)
          END IF
          CALL IEDPNT(XP1,YP1)
          CALL PGEBUF
C
C ---- A command: 'automatic' editor -----------------------------------
C
      ELSE IF (ANSWER(1:1) .EQ. 'A') THEN
          CALL PGBBUF
          CALL PGSCI(ICOLOR(IPOINT))
          CALL IEDPNT(XP1,YP1)
          CALL PGSCI(RED)
C         -- find the median error bar of undeleted points.
          NPT = 0
          DO I=1,IDATA
              IF (ICOLOR(I).NE.RED) THEN
                  NPT = NPT+1
                  WORK(NPT) = ERRAMP(NDATA(I))
              END IF
          END DO
          CALL MEDIAN(WORK, NPT, THRESH)
C         -- delete all points with error more than 3x the median.
          THRESH = 3.0*THRESH
          DO I=1,IDATA
              IF (ERRAMP(NDATA(I)).GE.THRESH) THEN
                  IPOINT = I
                  ICOLOR(IPOINT) = RED
                  CALL IEDPNT(XP1,YP1)
              END IF
          END DO
C         -- display the cursor at the beginning again.
          IPOINT = 1
          IF (ICOLOR(IPOINT) .EQ. RED) THEN
              CALL PGSCI(CICURD)
          ELSE
              CALL PGSCI(CICUR)
          END IF
          CALL IEDPNT(XP1,YP1)
          CALL PGEBUF
C
C ---- O command: delete 'Outliers' ------------------------------------
C
      ELSE IF (ANSWER(1:1) .EQ. 'O') THEN
          CALL PGBBUF
          CALL PGSCI(ICOLOR(IPOINT))
          CALL IEDPNT(XP1,YP1)
          CALL PGSCI(RED)
C         -- find the median error bar of undeleted points.
          NPT = 0
          DO I=1,IDATA
              IF (ICOLOR(I).NE.RED) THEN
                  NPT = NPT+1
                  WORK(NPT) = ERRAMP(NDATA(I))
              END IF
          END DO
          CALL MEDIAN(WORK, NPT, THRESH)
          THRESH = 3.0*THRESH
C         -- delete discrepant points.
          DO I=2,IDATA-1
              IF (PLTAMP) THEN
                  A1 = AMP(NDATA(I-1))
                  A2 = AMP(NDATA(I))
                  A3 = AMP(NDATA(I+1))
              ELSE IF (PLTPHS) THEN
                  A1 = PHS(NDATA(I-1))
                  A2 = PHS(NDATA(I))
                  A3 = PHS(NDATA(I+1))
              ELSE IF (PLTAMP) THEN
                  A1 = CLOS(NDATA(I-1))
                  A2 = CLOS(NDATA(I))
                  A3 = CLOS(NDATA(I+1))
              END IF
              IF (ICOLOR(I-1).EQ.RED .AND. ICOLOR(I+1).EQ.RED) THEN
                  BAD = .TRUE.
              ELSE IF (ICOLOR(I-1).EQ.RED) THEN
                  BAD = ABS(A2-A3).GT.THRESH
              ELSE IF (ICOLOR(I+1).EQ.RED) THEN
                  BAD = ABS(A2-A1).GT.THRESH
              ELSE
                  BAD = (ABS(A2-A1).GT.THRESH).AND.
     1                  (ABS(A2-A3).GT.THRESH)
              END IF
              IF (BAD) THEN
                  IPOINT = I
                  ICOLOR(IPOINT) = RED
                  CALL IEDPNT(XP1,YP1)
              END IF
          END DO
C         -- display the cursor at the beginning again.
          IPOINT = 1
          IF (ICOLOR(IPOINT) .EQ. RED) THEN
              CALL PGSCI(CICURD)
          ELSE
              CALL PGSCI(CICUR)
          END IF
          CALL IEDPNT(XP1,YP1)
          CALL PGEBUF
C
C ---- U command: undo all deletions -----------------------------------
C
      ELSE IF (ANSWER(1:1) .EQ. 'U') THEN
          CALL PGBBUF
          CALL PGSCI(ICOLOR(IPOINT)) 
C         -- undisplay cursor
          CALL IEDPNT(XP1,YP1)
          CALL PGSCI(GREEN)
          DO I=1,IDATA
              IF (ICOLOR(I).EQ.RED) THEN
                  IPOINT = I
                  ICOLOR(IPOINT) = GREEN
                  CALL IEDPNT(XP1,YP1)
              END IF
          END DO
          IPOINT = 1
          IF (ICOLOR(IPOINT) .EQ. RED) THEN
              CALL PGSCI(CICURD)
          ELSE
              CALL PGSCI(CICUR)
          END IF
          CALL IEDPNT(XP1,YP1)
          CALL PGEBUF
C
C ---- S command: move forward one hour, or to end of data -------------
C
      ELSE IF (ANSWER(1:1) .EQ. 'S') THEN
          UTSTOP = INT(XDATA(IPOINT)) + 1.0
          ISTART = IPOINT
          CALL PGBBUF
        DO ID=ISTART,IDATA
          IPOINT = ID
          IF (ICOLOR(IPOINT) .EQ. CYAN) ICOLOR(IPOINT) = GREEN
          CALL PGSCI(ICOLOR(IPOINT))
          IF (WSDEV) CALL IEDPNT(XP1,YP1)
          IF (XDATA(IPOINT) .GT. UTSTOP) GO TO 30
        ENDDO
   30   IF (ICOLOR(IPOINT) .EQ. RED) THEN
            CALL PGSCI(CICURD)
        ELSE
            CALL PGSCI(CICUR)
        END IF
        CALL IEDPNT(XP1,YP1)
          CALL PGEBUF
C
C ---- P command: move back one hour, or to start of data --------------
C
      ELSE IF (ANSWER(1:1) .EQ. 'P') THEN
          UTSTOP = XDATA(IPOINT) - 1.0
          ISTOP = IPOINT
          CALL PGBBUF
        DO ID=1,ISTOP
          IPOINT = ISTOP - ID + 1
          IF (ICOLOR(IPOINT) .EQ. GREEN) ICOLOR(IPOINT) = CYAN
          CALL PGSCI(ICOLOR(IPOINT))
          IF (WSDEV) CALL IEDPNT(XP1,YP1)
          IF (XDATA(IPOINT) .LT. UTSTOP) GO TO 50
        ENDDO
   50     IF (ICOLOR(IPOINT) .EQ. RED) THEN
              CALL PGSCI(CICURD)
          ELSE
              CALL PGSCI(CICUR)
          END IF
          CALL IEDPNT(XP1,YP1)
          CALL PGEBUF
C
C ---- W command: write deletions, and move to next dataset ------------
C
      ELSE IF (ANSWER(1:1) .EQ. 'W') THEN
          IF (IEDQ('Are you sure you want to go to next data subset?'))
     1        THEN
              CALL IEDOUT
              CALL NEXTPL
              IF (IER .EQ. 3) THEN
                  CALL PRMEND
                  RETURN
              END IF
              IPOINT = 1
              IF (ICOLOR(IPOINT) .EQ. RED) THEN
                  CALL PGSCI(CICURD)
              ELSE
                  CALL PGSCI(CICUR)
              END IF
                  CALL IEDPNT(XP1,YP1)
          ELSE
              IF (SAMETT) CALL PGETXT
              GOTO 600
          END IF
C
C ---- E command: write deletions, and exit program -------------------
C
      ELSE IF (ANSWER(1:1) .EQ. 'E') THEN
          IF (IEDQ('Are you sure you want exit from IED?'))
     1        THEN
              CALL IEDOUT
              CALL PRMEND
              CALL IEDPNT(XP1,YP1)
              RETURN
          ELSE
              IF (SAMETT) CALL PGETXT
              GOTO 600
          END IF
C
C ---- Q command: quit -------------------------------------------------
C
      ELSEIF (ANSWER(1:1) .EQ. 'Q')THEN
          IF (IEDQ('Are you sure you want exit from IED with'//
     1        ' no further output?')) THEN
              CALL PRMEND
              CALL PUTOUT('IED aborted')
              STOP
          ELSE
              IF (SAMETT) CALL PGETXT
              GOTO 600
          END IF
C
C ---- X command: erase text from screen -------------------------------
C
      ELSE IF (ANSWER(1:1) .EQ. 'X') THEN
        CALL PGETXT
        CALL IEDPNT(XP1,YP1)
C
C ---- V command: refresh graphics display -----------------------------
C
      ELSE IF (ANSWER(1:1) .EQ. 'V') THEN
        NEWPLT = .FALSE.
        IPSAV = IPOINT
        CALL PLTALL
        IPOINT = IPSAV
        IF (ICOLOR(IPOINT) .EQ. RED) THEN
            CALL PGSCI(CICURD)
        ELSE
            CALL PGSCI(CICUR)
        ENDIF
        CALL IEDPNT(XP1,YP1)
C
C ---- I command: inquire cursor position ------------------------------
C
      ELSE IF (ANSWER(1:1) .EQ. 'I') THEN
        CALL IEDPNT(XP1,YP1)
        WRITE (OUTU,1000) ' Cursor position: '
        WRITE (OUTU,'(3X,12H POINT NO. =,I5)') NDATA(IPOINT)
        WRITE (OUTU,'(3X,4HT = ,1PG15.7,6H hours)') XP1
        IF (PLTAMP) THEN
          WRITE (OUTU,'(3X,6HAMP = ,1PG15.7,6H units)') YP1
        ELSE
          WRITE (OUTU,'(3X,6HPHS = ,1PG15.7,8H degrees)') YP1
        ENDIF
C
C ---- J command: jump to a new window or dataset ----------------------
C
      ELSE IF (ANSWER(1:1) .EQ. 'J') THEN
          WRITE (OUTU,1000)
   90     WRITE (OUTU,1005)
          READ  (INU,'(3I10)',ERR=90) ISTA01,ISTA02,ISTA03
          IF (ISTA01 .EQ. 0) ISTA01 = NWIND
          IF (ISTA01 .LE. 0 .OR. ISTA02 .LE. 0) THEN
C          Jump to beginning of a window
          IF ((ISTA01 .LT. 0) .OR. (ISTA01 .GT. NCUTS+1)) THEN
            CALL PUTOUT('+++ Error +++ Illegal window')
            GO TO 600
          ELSE IF (ISTA01 .EQ. 1) THEN
            ISTOP = 1
          ELSE
            DO ISTOP=1,IDATA
              IF (XDATA(ISTOP).GE.XMIN(ISTA01)) GO TO 92
            ENDDO
 92            CONTINUE
          ENDIF
          ISTART = IPOINT
          IF (ISTOP.LT.ISTART) THEN
            DO ID=ISTOP,ISTART
              IPOINT = ISTART - ID + ISTOP
              IF (ICOLOR(IPOINT) .EQ. GREEN) ICOLOR(IPOINT) = CYAN
              CALL PGSCI(ICOLOR(IPOINT))
              IF (WSDEV) CALL IEDPNT(XP1,YP1)
            ENDDO
          ELSE IF (ISTOP.GT.ISTART) THEN
            DO ID=ISTART,ISTOP
              IPOINT = ID
              IF (ICOLOR(IPOINT) .EQ. CYAN) ICOLOR(IPOINT) = GREEN
              CALL PGSCI(ICOLOR(IPOINT))
              IF (WSDEV) CALL IEDPNT(XP1,YP1)
            ENDDO
          ENDIF
        ELSE
C          Jump to new data subset if it is later than this one
          CALL GETNPL(ISTA01,ISTA02,ISTA03)
          IF (IER .EQ. 0) THEN
            CALL PUTOUT('+++ Error +++  Can only JUMP to a '//
     +            'data subset not yet edited')
            NPLOT = NPLOT0
            GO TO 600
          ELSE
            IF (IEDQ('Are you sure you want to go to '//
     +            'a new data subset?')) THEN
              CALL IEDOUT
              ISTA1 = ISTA01
              ISTA2 = ISTA02
              ISTA3 = ISTA03
              IF (ISTA3.GT.0) THEN
                PLTAMP = .FALSE.
                PLTPHS = .FALSE.
              ENDIF
              NEWPLT = .TRUE.
              CALL PLTALL
              IF (IER .EQ. 2) CALL NEXTPL
              IF (IER .EQ. 3) THEN
                  CALL PRMEND
                  RETURN
              END IF
            ELSE
              IF (SAMETT) CALL PGETXT
              NPLOT = NPLOT0
              GO TO 600
            ENDIF
C            Success 
            IPOINT = 1
          ENDIF
        ENDIF
        IF (ICOLOR(IPOINT) .EQ. RED) THEN
          CALL PGSCI(CICURD)
        ELSE
          CALL PGSCI(CICUR)
        ENDIF
        CALL IEDPNT(XP1,YP1)
        IF (SAMETT) CALL PGETXT
C
C ---- C command: cut display into windows -----------------------------
C
      ELSE IF (ANSWER(1:1) .EQ. 'C') THEN
          CALL PUTOUT('To cut: move cursor to desired position'//
     +                      ' and then type any character')
          CALL IEDCUT (MXCUT,NEWNCU,XCUT,YCUT,XP1,YP1)
          CALL IEDPNT(XP1,YP1)
C
C ---- G command: glue windows back together ---------------------------
C
      ELSE IF (ANSWER(1:1) .EQ. 'G') THEN
          NEWNCU = 0
          NEWPLT = .FALSE.
          IPSAV = IPOINT
          CALL PLTALL
          IPOINT = IPSAV
          IF (ICOLOR(IPOINT) .EQ. RED) THEN
              CALL PGSCI(CICURD)
          ELSE
              CALL PGSCI(CICUR)
          ENDIF
          CALL IEDPNT(XP1,YP1)
C
C ---- H command: print help message -----------------------------------
C
      ELSE IF (ANSWER(1:1) .EQ. 'H') THEN
          IF (WSDEV) THEN
              WRITE (OUTU,1001)
          ELSE
              WRITE (OUTU,1002)
          ENDIF
          WRITE (OUTU,1003)
          WRITE (OUTU,1004)
          GOTO 700
C
C ---- Z command: delete data point nearest cursor position ------------
C
      ELSE IF (ANSWER(1:1) .EQ. 'Z') THEN
C
C      Save old pointer value for last data point
        IZPOLD = IPOINT
C
C       Type message
C
        CALL PUTOUT('Position cursor and type any character (except S)')
        CALL PUTOUT('to delete the nearest datapoint.  Type S to exit.')
C
C
C       COMPUTE SCALE FACTOR FOR EACH AXIS
C       FZXSCL = NUMBER OF X UNITS PER MILLIMETER ON X AXIS
C       FZYSCL = NUMBER OF Y UNITS PER MILLIMETER ON Y AXIS
        CALL PGQVP(2,ZVPX1,ZVPX2,ZVPY1,ZVPY2)
        CALL PGQWIN(ZX1,ZX2,ZY1,ZY2)
        FZXSCL=(ZX1-ZX2)/(ZVPX1-ZVPX2)
        FZYSCL=(ZY1-ZY2)/(ZVPY1-ZVPY2)
        ZX = XP1
        ZY = YP1
C
C
C Start loop
C
C
8009    CONTINUE
C
C       SAVE OLD POINTER VALUE FOR LAST DATA POINT
        IZPOLD = IPOINT
C
C       GET CURSOR POSITION
C
        CALL PGCURS(ZX,ZY,LETTER)
C
C       TEST WHETHER TO QUIT
            CALL UPCASE (LETTER)
            IF (LETTER .EQ. 'S') GOTO 8099
C
C       FIND DATA ARRAY POINTER WHICH ROUGHLY CORRESPONDS TO THIS POSITION
C
C       SEARCH UPWARD FROM OLD CURSOR POSITION
        IF(ZX.GT.XDATA(IZPOLD)) THEN
        DO 8001 IZP=IZPOLD,IDATA
        IF(XDATA(IZP).GT.ZX) GOTO 8003
8001    CONTINUE
C       SEARCH DOWNWARD FROM OLD CURSOR POSITION
        ELSE
        DO 8002 IZP=IZPOLD,1,-1
        IF(XDATA(IZP).LT.ZX) GOTO 8003
8002    CONTINUE
        END IF
8003    CONTINUE
        IPOINT=IZP
C
C       IPOINT NOW SET TO POINTER VALUE FOR DATA POINT NUMBER
C       CLOSEST TO CURSOR POSITION IN X.
C
C
C       SEARCH 100 ADJACENT POINTS FOR ONE WHICH IS CLOSEST IN BOTH
C       X AND Y.  THIS WILL AVOID CHOOSING WRONG POINT IF DATA
C       POINTS ARE VERY CROWDED.  COMPUTE IZDEL WHICH IS POINTER
C       FOR POINT TO BE DELETED.
C
C
C       SEARCH
        FZDSAV=1.0E12
        DO 8010 IZ=MAX(1,IPOINT-50),MIN(IDATA,IPOINT+50)
        FZDX=(XDATA(IZ)-ZX)/FZXSCL
        IF (PLTBAR) THEN
        FZDY=((Y1DATA(IZ)+Y2DATA(IZ))/2 - ZY)/FZYSCL
        ELSE
        FZDY=(Y1DATA(IZ) - ZY)/FZYSCL
        END IF
        FZDIST= (FZDX)**2 +  (FZDY)**2
        IF(FZDIST.LT.FZDSAV) THEN
        IZSAV=IZ
        FZDSAV=FZDIST
        END IF
8010            CONTINUE
C
C       END SEARCH
C
        IZDEL=IZSAV
        IPOINT=IZDEL
C
C
C       SKIP TO POINT IZDEL WHICH WILL BE DELETED
C
C       SKIP FORWARD
        IF(IZDEL.GT.IZPOLD) THEN
          ISTART = IZPOLD
          CALL PGBBUF
        DO ID=ISTART,IDATA
          IPOINT = ID
          IF (ICOLOR(IPOINT) .EQ. CYAN) ICOLOR(IPOINT) = GREEN
          CALL PGSCI(ICOLOR(IPOINT))
          IF (WSDEV) CALL IEDPNT(XP1,YP1)
          IF (IPOINT .EQ. IZDEL) GO TO 8030
        ENDDO
 8030   IF (ICOLOR(IPOINT) .EQ. RED) THEN
            CALL PGSCI(CICURD)
        ELSE
            CALL PGSCI(CICUR)
        END IF
        CALL IEDPNT(XP1,YP1)
          CALL PGEBUF
C
C
C       SKIP BACKWARDS
        ELSE IF (IZDEL.LT.IZPOLD) THEN
          ISTOP = IZPOLD
          CALL PGBBUF
        DO ID=1,ISTOP
          IPOINT = ISTOP - ID + 1
          IF (ICOLOR(IPOINT) .EQ. GREEN) ICOLOR(IPOINT) = CYAN
          CALL PGSCI(ICOLOR(IPOINT))
          IF (WSDEV) CALL IEDPNT(XP1,YP1)
          IF (IPOINT .EQ. IZDEL) GO TO 8050
        ENDDO
 8050     IF (ICOLOR(IPOINT) .EQ. RED) THEN
              CALL PGSCI(CICURD)
          ELSE
              CALL PGSCI(CICUR)
          END IF
          CALL IEDPNT(XP1,YP1)
          CALL PGEBUF
C
C
C       DON'T NEED TO MOVE IF IZPOLD=IZDEL
        ELSE
        IPOINT=IZDEL
        END IF
C
C
C
C       DELETE THE POINT, CURSOR IS LEFT AT POSITION OF DELETED POINT
C
        ICOLOR(IPOINT) = RED
            CALL PGSCI(CICURD)
        CALL IEDPNT(XP1,YP1)
        GOTO 8009
C
C
8099    CONTINUE
C
C ---- Unknown command: ignore -----------------------------------------
C
      ELSE
          CALL PUTOUT('Illegal command. For help type "H".')
          GO TO 700
      ENDIF
C
C Fetch another command after plotting something.
C
  600 CONTINUE
C
C Fetch another command after writing a message.
C
  700 CONTINUE
      CALL PGUPDT
      IF (WSDEV) THEN
          ISEC = NINT(MOD(UT(NDATA(IPOINT)),24.0)*3600.0)
          WRITE (OUTU,1006) NDATA(IPOINT),TIMEC(ISEC)
          ICMD = NARROW()
          ANSWER(1:1) = CHAR(ICMD)
          CALL UPCASE (ANSWER)
          IF (.NOT.VMS) WRITE (OUTU,1007) ANSWER(1:1)
      ELSE
          CALL PGCURS(XP1,YP1,LETTER)
          ANSWER(1:1) = LETTER
          CALL UPCASE (ANSWER)
      ENDIF
      GOTO 3
C-----------------------------------------------------------------------
 1000 FORMAT (10A)
 1001 FORMAT ('In response to the ''>'' prompt, type one of the ',
     +        'following:')
 1002 FORMAT ('After the graphics cursor appears, type one of the ',
     +        'following:')
 1003 FORMAT (' (<RETURN> key is not necessary)'/
     +  'F [or <RETURN>]      ! FORWARD to next point'/
     +  'B            ! BACK to previous point'/
     +  'D            ! DELETE current point'/
     +  'R            ! RESTORE current point'/
     +  'S            ! SKIP ahead to start of next hour'/
     +  'P            ! move to PREVIOUS hour'/
     +  'K            ! delete all points below a cutoff level'/
     +  'A            ! invoke the AUTOMATIC editor'/
     +  'O            ! delete Outliers'/
     +  'Z            ! to delete points using cursor'/
     +  'U            ! UNDELETE all points deleted so far'/
     +  'W            ! WRITE list of deleted points, and move to ' )
 1004 FORMAT ('next plot'/,
     +  'E            ! EXIT the program, and write list of deleted',
     +  'points.'/
     +  'Q            ! QUIT, with no further output'/
     +  'X            ! erase text from terminal screen'/
     +  'V            ! VIEW current data subset (refresh screen'/
     +  '               after DELETING, CUTTING, or GLUING)'/
     +  'I            ! INQUIRE position of cursor (AMP/PHS,TIME)'/
     +  'J            ! JUMP to start of window or new data subset'/
     +  'C            ! CUT data into multiple windows'/
     +  'G            ! GLUE data back into one window'/
     +  'H            ! HELP (prints this message)' )
 1005 FORMAT(/'To jump to beginning of:            Type:'
     +       /'A window in current data subset     The window number ',
     +        '(def=current)'
     +       /'A new baseline                  Two numbers (e.g. 1,2)'
     +       /'A new triangle                  Three numbers (e.g. ',
     +        '1,2,3)')
 1006 FORMAT (I4,1X,A8,'> ',$)
 1007 FORMAT (A1)
C-----------------------------------------------------------------------
      END

      SUBROUTINE GETDAT
C
C      By Richard S. Simon
C
C      Version 1:      16-Jun-1982
C      Version 2:      15-Feb-1983      (D. Meier)
C      VERSION 6.3:    15-Apr-1987      (T. Pearson)
C
C        This subroutine does the following:
C      1) Calls subroutine HEADX20 to get data parameters.
C      2) Reads in amplitude and closure phase data from data file.
C      3) Loads data into the XDATA, Y1DATA, Y2DATA, and NDATA arrays.
C
      INCLUDE 'ied.inc'
C
      CHARACTER*16 SRC
      INTEGER IOUT, IPROUT, IZERO, NDAY, NYEAR, IDUMMY, IDUM, IUNITS
      INTEGER IFILE, NWORDS, ID, ITIME, I, K, IT
      REAL    DUMMY
      REAL    TAKE(MAXWPB*MXBAS + 2*MXTRI)
C-----------------------------------------------------------------------
C      Call head to get data parameters
      IOUT = 0
      IPROUT = 0
      REWIND (UNIT=INDAT,ERR=10)
   10 CALL HEADX20(.FALSE., INDAT, IZERO, NDAY, NYEAR, IDUMMY, IDUM,
     +              IOUT, IUNITS, DUMMY, IFILE, IPROUT)
      IF (IFILE .NE. 0) THEN
          CALL PUTOUT('Data file does not have header record at'//
     1        ' start of file.  Try again.')
          IER = 0
          RETURN
      END IF
C
      IF (TRYONE) THEN
          WRITE (SRC,'(2A8)') SOURCE(1),SOURCE(2)
          CALL PUTOUT('Source: '//SRC)
          IER = 1
          RETURN
      END IF
C
C Read in the data, compute UT in hours.
C
      NWORDS = NDWPB*NBAS + 2*NTRI
      DO ID = 1, MXREC
          READ (INDAT, END=160)  ITIME, (TAKE(I), I = 1, NWORDS)
          IF (ITIME .EQ. -1) GO TO 160
          UT(ID) = (ITIME - IZERO) / 216000.
C         -- get either Amplitudes or Phases or Closure Phases.
          IF (PLTAMP) THEN
              K = NDWPB*(NPLOT-1)
              AMP(ID) = TAKE(K+1)
              ERRAMP(ID)= TAKE(K+2)
              IF (ERRAMP(ID) .LT. 0.) ERRAMP(ID) = 0.
C             -- Ignore deleted data
          ELSE IF (PLTPHS) THEN
              K = NDWPB*(NPLOT-1)
              ERRPHS(ID)= TAKE(K+4)
              IF (ERRPHS(ID) .LT. 0.) THEN
                  ERRPHS(ID) = 0.
C                 -- Ignore deleted data
              ELSE
                  PHS(ID)  = AMOD(TAKE(K+3),360.)
                  IF (PHS(ID) .LT. -180.) PHS(ID) = PHS(ID) + 360.
                  IF (PHS(ID) .GT. +180.) PHS(ID) = PHS(ID) - 360.
              END IF
          ELSE
              K = NDWPB*NBAS + 2*(NPLOT-NBAS-1)
              CLOS(ID)  = AMOD(TAKE(K+1),360.)
              IF (CLOS(ID) .LT. -180.) CLOS(ID) = CLOS(ID) + 360.
              IF (CLOS(ID) .GT. +180.) CLOS(ID) = CLOS(ID) - 360.
              ERRCLO(ID) = TAKE(K+2)
              IF (ERRCLO(ID) .LT. 0.) ERRCLO(ID) = 0.
C             -- Ignore deleted data
          END IF
      END DO
C      Not all the data was read in, so issue a warning:
      WRITE (OUTU,1008) MXREC,MXREC,MXREC
      ID = MXREC + 1
C
  160 CONTINUE
      NUMREC = ID - 1
C
C      IF ((UT(NUMREC)-UT(1)) .GT. 24.0) THEN
C        The data spans more than 24 hours, so fold into 0-24 hours
C        DO ID=1,NUMREC
C          IF (UT(ID) .GT. 24.0) THEN
C            UT(ID) = AMOD(UT(ID),24.0)
C            IF (UT(ID) .LT. 0.) UT(ID) = UT(ID) + 24.0
C          ENDIF
C        ENDDO
C      ENDIF
C
C      Load up the XDATA, Y1DATA, Y2DATA, and NDATA arrays
      IDATA = 0
      IF (PLTBAR) THEN
        DO IT = 1,NUMREC
          IF (PLTAMP) THEN
            IF (ERRAMP(IT) .NE. 0.) THEN
              IDATA = IDATA + 1
              XDATA(IDATA) = UT(IT)
              Y1DATA(IDATA) = AMP(IT) - ERRAMP(IT)
              Y2DATA(IDATA) = AMP(IT) + ERRAMP(IT)
              NDATA(IDATA) = IT
              IF(ERRAMP(IT) .LT. 0.) THEN
                  ICOLOR(IDATA) = RED
C                 -- For use if deleted data is not ignored
              ELSE
                ICOLOR(IDATA) = CYAN
              ENDIF
            ENDIF
          ELSEIF (PLTPHS) THEN
            IF (ERRPHS(IT) .NE. 0.) THEN
              IDATA = IDATA + 1
              XDATA(IDATA) = UT(IT)
              Y1DATA(IDATA) = PHS(IT) - ERRPHS(IT)
              Y2DATA(IDATA) = PHS(IT) + ERRPHS(IT)
              NDATA(IDATA) = IT
              IF(ERRPHS(IT) .LT. 0.) THEN
                  ICOLOR(IDATA) = RED
C                 -- For use if deleted data is not ignored
              ELSE
                ICOLOR(IDATA) = CYAN
              ENDIF
            ENDIF
          ELSE
            IF (ERRCLO(IT) .NE. 0.) THEN
              IDATA = IDATA + 1
              XDATA(IDATA) = UT(IT)
              Y1DATA(IDATA) = CLOS(IT) - ERRCLO(IT)
              Y2DATA(IDATA) = CLOS(IT) + ERRCLO(IT)
              NDATA(IDATA) = IT
              IF(ERRCLO(IT) .LT. 0.) THEN
                  ICOLOR(IDATA) = RED            
C                 -- For use if deleted os not ignored
              ELSE
                ICOLOR(IDATA) = CYAN
              ENDIF
            ENDIF
          ENDIF
          ENDDO
C
      ELSE
        DO IT=1,NUMREC
          IF(PLTAMP) THEN
            IF (ERRAMP(IT) .NE. 0.) THEN
              IDATA = IDATA + 1
              XDATA(IDATA) = UT(IT)
              Y1DATA(IDATA) = AMP(IT)
              NDATA(IDATA) = IT
              IF(ERRAMP(IT) .LT. 0.) THEN
                  ICOLOR(IDATA) = RED
C                 -- For use if deleted data is not ignored
              ELSE
                ICOLOR(IDATA) = CYAN
              ENDIF
            ENDIF
          ELSEIF(PLTPHS) THEN
            IF (ERRPHS(IT) .NE. 0.) THEN
              IDATA = IDATA + 1
              XDATA(IDATA) = UT(IT)
              Y1DATA(IDATA) = PHS(IT)
              NDATA(IDATA) = IT
              IF(ERRPHS(IT) .LT. 0.) THEN
                  ICOLOR(IDATA) = RED
              ELSE
                ICOLOR(IDATA) = CYAN
              ENDIF
            ENDIF
          ELSE
            IF (ERRCLO(IT) .NE. 0.) THEN
              IDATA = IDATA + 1
              XDATA(IDATA) = UT(IT)
              Y1DATA(IDATA) = CLOS(IT)
                NDATA(IDATA) = IT
              IF(ERRCLO(IT) .LT. 0.) THEN
                  ICOLOR(IDATA) = RED
              ELSE
                ICOLOR(IDATA) = CYAN
              ENDIF
              ENDIF
          ENDIF
          ENDDO
      ENDIF
C
      IER = 1
C     Implies successful completion
      RETURN
C-----------------------------------------------------------------------
 1008 FORMAT(/' Well, it looks like you have blown it.  This version',
     +  ' of the program will',
     + /' only handle',I4,' separate data records, i.e.,',I4,
     +  ' separate UT''s.  Thus,',
     + /' only the first',I4,' times will be edited.  I',
     +  ' suggest you increase parameter '/' MXREC in ied.inc, and ',
     +  're-compile all the subroutines and re-link the program.'/
     +  ' Good luck.')
C-----------------------------------------------------------------------
      END

      SUBROUTINE PRMINI
C
C      Used by IED
C
      INCLUDE 'ied.inc'
      INTEGER L
      INTEGER LEN1
C-----------------------------------------------------------------------
C      Write the commands into PRMDSN to create a command procedure
C
      IF (VMS) THEN
          WRITE (PARMU,1000) '$! This is a command procedure to',
     1                       ' run VLBEDIT created by program IED.'
          WRITE (PARMU,1000) '$!'
          WRITE (PARMU,1000) '$ VLBEDIT'
      ELSE
          WRITE (PARMU,1000) '# This is a command procedure to',
     +                       ' run VLBEDIT created by program IED.'
          WRITE (PARMU,1000) '#'
          WRITE (PARMU,1000) 'vlbedit <<EOF'
      END IF
      INQUIRE (UNIT=INDAT,NAME=DATDSN)
      L = LEN1(DATDSN)
      WRITE (PARMU,1000) DATDSN(1:L)
      L = LEN1(EDIDSN)
      WRITE (PARMU,1000) EDIDSN(1:L)
      WRITE (PARMU,1000) '! Header control parameters:'
      WRITE (PARMU,1000) 'HISTORY=''Edit with IED and VLBEDIT.',
     +      '  Delete "bad" data points.''  /'
      WRITE (PARMU,1000) '!'
      WRITE (PARMU,1000) '! Begin editing commands:'
C
C      Finished with opening of parameter file; close for now.
      CLOSE (UNIT=PARMU)
      RETURN
C-----------------------------------------------------------------------
 1000 FORMAT (10A)
C-----------------------------------------------------------------------
      END

      SUBROUTINE IEDOUT
C-----------------------------------------------------------------------
C By Richard S. Simon.
C This subroutine writes out the VLBEDIT commands to delete data.
C-----------------------------------------------------------------------
      INCLUDE 'ied.inc'
      INTEGER NDELET(MXREC), ID, IDELET
C-----------------------------------------------------------------------
C First, find how many points are being deleted;  if none, return.
C
      IDELET = 0
      DO ID=1,IDATA
          IF (ICOLOR(ID) .EQ. RED) THEN
              IDELET = IDELET + 1
              NDELET(IDELET) = NDATA(ID)
          END IF
      END DO
C
C If data were deleted, write numbers into file and tell user.
C
      IF (IDELET.GT.0) THEN
          OPEN (UNIT=PARMU,STATUS='OLD',FORM='FORMATTED',
     +          FILE=PRMDSN,ACCESS='APPEND',CARRIAGECONTROL='LIST')
          IF (PLTAMP .OR. PLTPHS) THEN
              WRITE (OUTU,1001) IDELET,' data points were deleted on',
     +          ' baseline ',S1NAME(1:L1NAME),'-',S2NAME(1:L2NAME),'.'
              IF (PLTAMP) THEN
                  WRITE (PARMU,1000) '! The following points deleted',
     +                               ' due to bad amplitudes:'
              ELSE
                  WRITE (PARMU,1000) '! The following points deleted',
     +                               ' due to bad baseline phases:'
              END IF
              WRITE (PARMU,1000) 'DELETE  ',S1NAME(1:L1NAME),'  ',
     +                           S2NAME(1:L2NAME),'  ALL'
          ELSE
              WRITE (OUTU,1001) IDELET,' data points were deleted on',
     +          ' triangle ',S1NAME(1:L1NAME),'-',S2NAME(1:L2NAME),
     +          '-',S3NAME(1:L3NAME),'.'
              WRITE (PARMU,1000) 'DELETE  ',S1NAME(1:L1NAME),'  ',
     +          S2NAME(1:L2NAME),'  ',S3NAME(1:L3NAME),'  CLOSE'
          END IF
C
C Write out the record numbers of the deleted data.
C
          WRITE (PARMU,1002) (NDELET(ID), ID=1,IDELET)
          WRITE (PARMU,1000) '/'
C
C Close the parameter file for now.
C
          CLOSE (UNIT=PARMU)
      ELSE
C        No data points were deleted.
        IF (PLTAMP .OR. PLTPHS) THEN
          CALL PUTOUT('No data points were deleted from baseline '//
     +                S1NAME(1:L1NAME)//'-'//S2NAME(1:L2NAME))
        ELSE
          CALL PUTOUT('No data points were deleted from triangle '//
     +                S1NAME(1:L1NAME)//'-'//S2NAME(1:L2NAME)//'-'//
     +                S3NAME(1:L3NAME))
        ENDIF
      ENDIF
C
      RETURN
C-----------------------------------------------------------------------
 1000 FORMAT (A,A,A,A,A,A,A,A)
 1001 FORMAT (I5,A,A,A,A,A,A,A,A)
 1002 FORMAT ('N=',15(I4,:,',')/ 1(2X,15(I4,:,',')) )
C-----------------------------------------------------------------------
      END

      SUBROUTINE PRMEND
C
C Write terminating record in the output command file.
C-----------------------------------------------------------------------
      INCLUDE 'ied.inc'
      OPEN (UNIT=PARMU,STATUS='OLD',FORM='FORMATTED',
     +      FILE=PRMDSN,ACCESS='APPEND',CARRIAGECONTROL='LIST')
      IF (VMS) THEN
          WRITE (PARMU, '(A)') '$! End of IED commands'
      ELSE
          WRITE (PARMU, '(A)') 'EOF'
          WRITE (PARMU, '(A)') '#  End of IED commands'
      END IF
      CLOSE (UNIT=PARMU)
      RETURN
      END

      SUBROUTINE NEXTPL
C-----------------------------------------------------------------------
C This subroutine does the following:
C   1) Advances to the next available data subset number (baseline
C      or triangle)
C   2) Calls PLTALL and plots the data subset
C-----------------------------------------------------------------------
      INCLUDE 'ied.inc'
C
  100 NPLOT = NPLOT + 1
      IF (PLTAMP .OR. PLTPHS) THEN
        IF (NPLOT .LE. NBAS) THEN
          CALL PUTOUT('Going to next baseline.')
        ELSE
          IF (NTRI .GT. 0) THEN
              PLTAMP = .FALSE.
              PLTPHS = .FALSE.
              CALL PUTOUT('Going to first triangle.')
          ELSE
            CALL PUTOUT('There are no closure phase data,'//
     +            ' so it''s time to exit.  Goodbye.')
            IER = 3
            RETURN
          ENDIF
        ENDIF
      ELSE
        IF (NPLOT .LE. (NBAS+NTRI)) THEN
          CALL PUTOUT('Going to next triangle.')
        ELSE
          CALL PUTOUT('There are no more data,'//
     +            ' so it''s time to exit.  Goodbye.')
          IER = 3
          RETURN
        ENDIF
      ENDIF
      NEWPLT = .TRUE.
      CALL PLTALL
      IF (IER .EQ. 2) THEN
C        No data read in, so skip to next data subset, if any
        GO TO 100
      ENDIF
      RETURN
C-----------------------------------------------------------------------
      END

      SUBROUTINE GETNPL(ISTA01,ISTA02,ISTA03)
C-----------------------------------------------------------------------
C This subroutine does the following:
C      1) Determine NPLOT, the sequential data subset number for a
C         specified baseline or triangle
C-----------------------------------------------------------------------
      INCLUDE 'ied.inc'
      INTEGER ISTA01, ISTA02, ISTA03, IB, IC, IBMIN
C-----------------------------------------------------------------------
      IF (ISTA03.EQ.0) THEN
          IBMIN = MAX(NPLOT0,0)+1
          DO IB=IBMIN,NBAS
          IF (NAVIS(IB) .EQ. ISTA01 .AND. NBVIS(IB) .EQ. ISTA02) THEN
            NPLOT = IB
            GO TO 40
          ENDIF
          ENDDO
C        Error! so return
        IER = 0
        RETURN
      ELSE
        IBMIN = MAX(NPLOT0,NBAS)+1
        DO IB=IBMIN,NTRI+NBAS
          IC = IB-NBAS
          IF (NACLO(IC).EQ.ISTA01 .AND. NBCLO(IC).EQ.ISTA02 .AND.
     +          NCCLO(IC).EQ.ISTA03) THEN
            NPLOT = IB
            GO TO 40
          ENDIF
           ENDDO
C        Error! so return
        IER = 0
        RETURN
      ENDIF
   40 CONTINUE
      IER = 1
      RETURN
      END

      SUBROUTINE PLTALL
C-----------------------------------------------------------------------
C      By Richard S. Simon
C      Modified to work on other than Grinnell by D. Meier 15-Feb-1983
C      Modified again to plot windows by D. Meier 5-Apr-1983
C
C      This subroutine does the following:
C      1) Plots the entire data subset to device PLTOUT using PGPLOT
C-----------------------------------------------------------------------
      INCLUDE 'ied.inc'
      INTEGER LTYP, L, ID, IT, ICLR, NC, LEN1
      REAL    AMPMIN, AMPMAX, XD, AMPIT, YDELT, XP1, YP1
      CHARACTER*30 XLABEL, YLABEL, USERTT
      CHARACTER*45 TITLE
      DOUBLE PRECISION SNAME1,SNAME2,SNAME3
      EQUIVALENCE (S1NAME,SNAME1),(S2NAME,SNAME2),(S3NAME,SNAME3)
C-----------------------------------------------------------------------
      NPLOT0 = NPLOT
C
C Get type of plot device.
C
      CALL PGPAGE
      CALL PGQINF('TYPE',DEVTYP,LTYP)
      CALL PGASK(.FALSE.)
C
C Determine if plot is going to user's terminal.
C
      IF (TRYONE) THEN
          CALL PGQINF('TERMINAL',USERTT,L)
          SAMETT = USERTT.EQ.'YES'
      ENDIF
C
C Get the actual data to be plotted, for the current value of NPLOT.
C
      TRYONE = .FALSE.
      IF (NEWPLT) CALL GETDAT
C
C Determine the windows for the plot (based on data and cuts made).
C
      NCUTS = NEWNCU
      XMINN = 1E12
      XMAXX = -1E12
      AMPMIN = 1E12
      AMPMAX = -1E12
      DO 100 ID=1,IDATA
        XD = XDATA(ID)
        IF (XD .LT. XMINN) XMINN=XD
        IF (XD .GT. XMAXX) XMAXX=XD
        IF (PLTAMP) THEN
          IT = NDATA(ID)
          AMPIT = AMP(IT)
          IF (AMPIT .LT. AMPMIN) AMPMIN=AMPIT
          ICLR = ICOLOR(ID)
          IF ((ICLR.NE.RED) .AND. (ICLR.NE.MAGEN)) THEN
            IF (AMPIT .GT. AMPMAX) AMPMAX=AMPIT
          ENDIF
        ENDIF
  100 CONTINUE
      IF (XMINN .EQ. 1E12) THEN
        WRITE (OUTU,1000) '0No data for this data subset.  Skipping ',
     +    'to next subset, if any.'
        IER = 2
        RETURN
      ENDIF
      IF (PLTAMP) THEN
          IF (AMPMAX .EQ. -1E12) AMPMAX = AMPMIN
          YMAXX = 1.1*AMPMAX
          YMINN = 0.0
      ELSE
          YMAXX = 190.
          YMINN = -190.
      ENDIF
      XMIN(1) = XMINN
      YTOP(1) = 1.0
      YDELT   = 1.0/(NCUTS+1)
      DO NC=1,NCUTS
          XMAX(NC)   = XCUT(NC)
          YBOT(NC)   = YTOP(NC) - YDELT
          XMIN(NC+1) = XCUT(NC)
          YTOP(NC+1) = YBOT(NC)
      ENDDO
      XMAX(NCUTS+1) = XMAXX + 0.01
C     -- So last point will be plotted
      YBOT(NCUTS+1) = YTOP(NCUTS+1) - YDELT
      IF (NCUTS.EQ.0) YBOT(1) = 0.5
C
C Plot the boxes.
C
      CALL PGPAGE
      CALL PGBBUF
      DO NWIND=1,NCUTS+1
          CALL WINDOW
          CALL PGSCI(YELLOW)
          CALL PGBOX ( 'BCNST', 0.0, 0, 'BCNSTV', 0.0, 0)
C
C Get labels for plot
C
        IF (PLTAMP .OR. PLTPHS) THEN
          SNAME1 = STNAME(NAVIS(NPLOT))
          SNAME2 = STNAME(NBVIS(NPLOT))
          L1NAME = LEN1(S1NAME)
          L2NAME = LEN1(S2NAME)
          IF (L1NAME .LE. 0 .OR. L1NAME .GT. 8) L1NAME = 8
          IF (L2NAME .LE. 0 .OR. L2NAME .GT. 8) L2NAME = 8
          TITLE(1:L1NAME) = S1NAME(1:L1NAME)
          TITLE(L1NAME+1:L1NAME+3) = ' - '
          TITLE(L1NAME+4:L1NAME+3+L2NAME) = S2NAME(1:L2NAME)
          TITLE(L1NAME+L2NAME+4:30) = ' '
          IF (PLTAMP) YLABEL = 'Amplitude:'
          IF (PLTPHS) YLABEL = 'Visibility Phase:'
        ELSE
          SNAME1 = STNAME(NACLO(NPLOT-NBAS))
          SNAME2 = STNAME(NBCLO(NPLOT-NBAS))
          SNAME3 = STNAME(NCCLO(NPLOT-NBAS))
          L1NAME = LEN1(S1NAME)
          L2NAME = LEN1(S2NAME)
          L3NAME = LEN1(S3NAME)
          IF (L1NAME .LE. 0 .OR. L1NAME .GT. 8) L1NAME = 8
          IF (L2NAME .LE. 0 .OR. L2NAME .GT. 8) L2NAME = 8
          IF (L3NAME .LE. 0 .OR. L3NAME .GT. 8) L3NAME = 8
          TITLE(1:L1NAME) = S1NAME(1:L1NAME)
          TITLE(L1NAME+1:L1NAME+3) = ' - '
          TITLE(L1NAME+4:L1NAME+3+L2NAME) = S2NAME(1:L2NAME)
          TITLE(L1NAME+L2NAME+4:L1NAME+L2NAME+6) = ' - '
          TITLE(L1NAME+L2NAME+7:L1NAME+L2NAME+6+L3NAME)=S3NAME(1:L3NAME)
          IF (L1NAME+L2NAME+L3NAME+6 .LT. 30)
     +          TITLE(L1NAME+L2NAME+L3NAME+7:30) = ' '
          YLABEL = 'Closure Phase:'
        ENDIF
        XLABEL = 'Universal Time'
        TITLE(31:45) = ' '
        IF (NCUTS.GT.0) WRITE (TITLE(31:45),'(5X,8HWindow #,I2)')
     1          NWIND
        CALL PGSCI(YELLOW)
        CALL PGMTXT('B', 3.2, 0.5, 0.5, XLABEL)
        CALL PGMTXT('T',0.8,0.0,0.0,YLABEL(1:1+LEN1(YLABEL))//TITLE)
      ENDDO
C
C Plot the data.
C
      DO 160 IPOINT=1,IDATA
          CALL PGSCI(ICOLOR(IPOINT))
          CALL IEDPNT(XP1,YP1)
  160 CONTINUE
      CALL PGEBUF
C
C Successful!
C
      IER = 1
      RETURN
C-----------------------------------------------------------------------
 1000 FORMAT (4A)
C-----------------------------------------------------------------------
      END

      SUBROUTINE WINDOW
C-----------------------------------------------------------------------
C Establish "viewport" and "plot window" for current data window.
C-----------------------------------------------------------------------
      INCLUDE 'ied.inc'
      REAL XMARG
C-----------------------------------------------------------------------
      CALL VPORT (0.0, 1.0, YBOT(NWIND), YTOP(NWIND))
      XMARG = 0.05*(XMAX(NWIND)-XMIN(NWIND)) + 0.1
      XMINN = XMIN(NWIND) - XMARG
      XMAXX = XMAX(NWIND) + XMARG
      CALL PGSWIN (XMINN, XMAXX, YMINN, YMAXX)
      RETURN
C-----------------------------------------------------------------------
      END

      SUBROUTINE VPORT (XLEFT,XRIGHT,YBOT,YTOP)
C-----------------------------------------------------------------------
C Set Viewport for IED, leaving space for annotation. The margin for
C annotation is subtracted from the supplied arguments.
C
C Arguments:
C
C XLEFT (input, real) : x-coordinate of left hand edge of
C      viewport, in NDC.
C XRIGHT (input, real) : x-coordinate of right hand edge of
C      viewport, in NDC.
C YBOT (input, real) : y-coordinate of bottom edge of
C      viewport, in NDC.
C YTOP (input, real) : y-coordinate of top  edge of
C      viewport, in NDC.
C-----------------------------------------------------------------------
      REAL XLEFT, XRIGHT, YBOT, YTOP
      REAL XSZ, YSZ, YSP
      REAL X1, X2, Y1, Y2
C-----------------------------------------------------------------------
      CALL PGSVP(0.0, 1.0, 0.0, 1.0)
      CALL PGQVP(1, X1, X2, Y1, Y2)
      XSZ = X2-X1
      YSZ = Y2-Y1
      YSP = MIN(XSZ,YSZ)/40.0
      X1 = XLEFT*XSZ  + 4.0*YSP
      X2 = XRIGHT*XSZ
      Y1 = YBOT*YSZ   + 4.0*YSP
      Y2 = YTOP*YSZ   - 1.5*YSP
      CALL PGVSIZ(X1, X2, Y1, Y2)
C-----------------------------------------------------------------------
      END

      SUBROUTINE IEDPNT(XP1,YP1)
C-----------------------------------------------------------------------
C Used by subroutines of IED.
C-----------------------------------------------------------------------
      INCLUDE 'ied.inc'
      INTEGER ICLR
      REAL    XP1, YP1, YP2
C-----------------------------------------------------------------------
      ICLR = ICOLOR(IPOINT)
      IF ((.NOT.WSDEV) .AND.
     1    ((ICLR.EQ.RED) .OR. (ICLR.EQ.MAGEN))) CALL PGSCI(0)
      XP1 = XDATA(IPOINT)
      YP1 = Y1DATA(IPOINT)
      DO NWIND=1,NCUTS+1
          IF ((XMIN(NWIND).LE.XP1) .AND. (XP1.LT.XMAX(NWIND)))
     1          GO TO 10
      ENDDO
      NWIND = NCUTS+1
      RETURN
 10   CONTINUE
      CALL WINDOW
      IF (PLTBAR) THEN
        YP2 = Y2DATA(IPOINT)
        XP(1) = XP1
        XP(2) = XP1
        YP(1) = YP1
        YP(2) = YP2
        YP1 = 0.5*(YP1+YP2)
        IF (YP1 .GT. YMAXX) THEN
          CALL PGPT(1,XP1,YMAXX,-1)
        ELSEIF (YP1. LT. YMINN) THEN
          CALL PGPT(1,XP1,YMINN,-1)
        ELSE
          CALL PGLINE(2,XP,YP)
        END IF
      ELSE
        CALL PGPT(1,XP1,YP1,-1)
      ENDIF
      IF ((.NOT.WSDEV) .AND.
     1    ((ICLR.EQ.RED) .OR. (ICLR.EQ.MAGEN))) CALL PGSCI(ICLR)
      RETURN
C-----------------------------------------------------------------------
      END

      SUBROUTINE IEDCUT (MAXPT,NPT,X,Y,XPLT,YPLT)
C---------------------------------------------------------------------
C Interactive routine for user to enter cuts in Universal Time by
C use of the cursor.
C
C Arguments:
C
C MAXPT (input, integer): maximum number of cuts that may be
C      accepted.
C NPT (input/output, integer): number of cuts entered; should
C      be zero on first call.
C X (input/output, real array): array of x-coordinates.
C Y (input/output, real array): array of y-coordinates.
C
C Notes:
C
C (1) The dimension of arrays X and Y must be greater than or
C equal to MAXPT.
C
C (2) On return from the program, cursor cuts are returned in
C increasing order of X. Routine may be (re-)called with cuts
C already defined in X,Y (# in NPT).
C
C  (9-Feb-1983) TJP
C (15-Feb-1983) DLM
C (27-Apr-1987) TJP
C-----------------------------------------------------------------------
      INCLUDE 'ied.inc'
      CHARACTER*1 LETTER
      INTEGER  I, J, LINCOL, MAXPT, NPT
      REAL     X(*), Y(*), XPLT, YPLT
C
C Save current color.
C
      CALL PGQCI(LINCOL)
C
C Get new X and Y coordinates of cursor.
C
      CALL PGCURS(XPLT,YPLT,LETTER)
C-----------------------------------------------------------------------
C CUT command.
C-----------------------------------------------------------------------
      IF (NPT.GE.MAXPT) THEN
          CALL PGUPDT
          CALL PUTOUT('Command ignored (too many cuts).')
          RETURN
      END IF
C     -- Find what current cuts new cut is between.
      DO 120 J=1,NPT
          IF (XPLT.LT.X(J)) GOTO 122
  120 CONTINUE
C     -- New cut is beyond last current
      J = NPT + 1
  122 CONTINUE
C     -- J is vector location where new cut should be included.
      DO 140 I=NPT,J,-1
          X(I+1) = X(I)
          Y(I+1) = Y(I)
  140 CONTINUE
      NPT = NPT + 1
C     -- Draw the cut on the screen.
      X(J) = XPLT
      Y(J) = YPLT
      XP(1) = XPLT
      XP(2) = XPLT
      YP(1) = YMINN
      YP(2) = YMAXX
      CALL PGSCI(YELLOW)
      CALL PGLINE(2,XP,YP)
      CALL PGSCI(LINCOL)
C-----------------------------------------------------------------------
      END

      LOGICAL FUNCTION IEDQ (QUEST)
      CHARACTER*(*) QUEST
C
C Ask the user a question: get a yes/no answer.
C
      INCLUDE 'ied.inc'
      CHARACTER*1 ANS
      INTEGER NARROW
C
   95 WRITE (OUTU,'(1X,2A,$)') QUEST,' (Y or N; def=Y) '
      ANS = CHAR(NARROW())
      CALL UPCASE (ANS)
      IF (ANS .EQ. 'N') THEN
          IEDQ = .FALSE.
          IF (.NOT.VMS) WRITE (OUTU, '(A)') ' No'
      ELSE IF (ANS.EQ.'Y' .OR. ANS.EQ.CHAR(13)) THEN
          IEDQ = .TRUE.
          IF (.NOT.VMS) WRITE (OUTU, '(A)') ' Yes'
      ELSE
          IF (.NOT.VMS) WRITE (OUTU, '(/A)')
     1        'Please answer the question!'
          GO TO 95
      END IF
      RETURN
      END
