      PROGRAM LIST
C-----------------------------------------------------------------------
C LIST: prepare a VLB MERGE-format file for printing.
C
C  T.J.Pearson   (VAX-11 Fortran)
C
C Version 1:  1979 May 1
C        extended to 10 stations: 1979 October 16
C        supplementary printout: 1980 January 11
C Version 2.0:  1981 May 4
C        Suppress error message on format overflow
C        Adopt Fortran-77 standards
C Version 2.1:  1981 Nov 24
C        Add source name to page header
C Version 2.2:  1982 May 13
C        More decimal places for calibrated amplitudes
C        Add help routine
C Version 2.3:  1982 Sep 17
C        extended to 15 stations
C Version 2.4:  1982 Oct 12
C        add record count to listing, and wild cards
C Version 2.5:  1982 Dec 20
C        add GST, INT, BASeline and TRIAngle parameters;
C        display Bandwidth; prevent crash if frequency is 0.
C Version 3.0:  1983 Jul 5     D. Meier
C        Extended to include Merge format version 2 files
C Version 3.1:  1983 Aug 3     D. Meier
C        Added W to version 2 files; change so only visibility data
C        and errors are loaded into array DATA
C Version 3.2:  1984 Jul 10    TJP
C        Fix "output overflow" bug, and improve behavior if asked
C        to read a non-MERGE-format file.
C Version 3.3:  1984 Dec 15    TJP
C        Add display of Julian Date and Epoch to header.
C Version 3.4:  1986 Jan 29    TJP
C        Correct pagination and add LPP parameter.
C Version 3.5:  1986 Feb  3    TJP
C        Display polarization code.
C Version 3.6:  1986 Feb 13    TJP
C        Display date last modified instead of today's date; further 
C        correction to pagination; display LIST version in listing;
C        give warning instead of dying if input file is not Merge 
C        format; display corr-coeff with 1 decimal place only, and 
C        phases with 2.
C Version 3.7:  1986 Mar 18    TJP
C        Filter unprintable characters out of history records (yes, I
C        known there shouldn't be any, but...).
C Version 3.8:  1986 Aug 28    TJP
C        Increase from 15 to 20 stations.
C Version 4.0:  1986 Nov 15    TJP
C        Add parameters INPUT and LISTFILE and remove questions.
C Version 4.1:  1987 Nov 7     TJP
C        Changes for Convex compatibility.
C Version 4.2:  1988 Apr 14     TJP
C        Change listing file format for Unix compatibility.
C Version 4.3:  1989 Jan 10     TJP
C        Make no data the default (instead of AMP CLOSE); add count of
C        data points; add u, v, w; parameters HEADER and NONE now
C        ignored.
C Version 4.4:  1990 Jan 31     TJP
C        If frequency in header is 0, try not to divide by 0.
C Version 4.5:  1991 Mar 22     TJP
C        Allow up to 40 stations.
C Version 4.6:  1992 Apr 17     TJP
C        Add NOHISTORY option.
C Version 4.7:  1992 Sep 15     TJP
C        Handle stdout under Unix.
C Version 4.8:  1993 Apr 19     TJP
C        Increase MAXFIL to 200.
C-----------------------------------------------------------------------
      CHARACTER*(*) PROGNM, VERSN
      INTEGER    INC,OUTC,INDAT
      INTEGER    MAXS,MAXB,MAXT, MAXWPB, MAXL,NPARS, WIDTH
      DOUBLE PRECISION VELC,PI,RPHR,RPDEG,SIDR
      PARAMETER  (PROGNM='VLBLIST')
      PARAMETER  (VERSN='4.8 - 1993 Apr 19')
      PARAMETER  (INC=5,OUTC=6,INDAT=10) 
C ! I/O unit numbers
      PARAMETER  (MAXS=40) 
C ! maximum number of stations
      PARAMETER  (MAXB=MAXS*(MAXS-1)/2)
      PARAMETER  (MAXT=MAXB*(MAXS-2)/3)
      PARAMETER  (MAXWPB=7)
      PARAMETER  (MAXL=MAXWPB*MAXB+2*MAXT)
C ! number of parameters
      PARAMETER  (NPARS=78)
      PARAMETER  (WIDTH=8)
      PARAMETER  (VELC=2.99792458D8)     
C ! velocity of light m/s
      PARAMETER  (PI=3.141592653589793D0)
      PARAMETER  (RPHR = PI/12D0)      
C ! radians per hour
      PARAMETER  (RPDEG = PI/180D0)
C !  radians per degree
      PARAMETER (SIDR=(PI/2592000.0D0)*1.0027379093D0)
C ! radians of ST per 1/60 sec of UT
      CHARACTER*1 FORM
      PARAMETER  (FORM=CHAR(12))
      INTEGER    MAXFIL
      PARAMETER  (MAXFIL=200)
      REAL       NODATA
      PARAMETER  (NODATA=1.0E38)
C-----------------------------------------------------------------------
      CHARACTER*16 IA,JA,KA,LA
      INTEGER*2  IS1(MAXB),IS2(MAXB),JP(MAXL)
      INTEGER YEAR,UT,PAGE,IUT(50),RECN(50),LABELB(MAXB),LABELT(MAXT)
      INTEGER TFIRST,TLAST,STATYP(MAXS),MONTH,DAY,JD
      INTEGER STOKES, PR, NFIL, VLBOPE
      INTEGER IB, ICOLS, ID, IER, IFIL, IMON, ISECS, ISEC, IST1, IST2
      INTEGER IST3, IS, IT, IUTL, I, JUNK, J, KFLAG, LDSN, LEN1, LENG
      INTEGER LH, LINE, LPPAGE, LP, MCOLS, MODE, MPCOLS, M, NB, NCELLS
      INTEGER NDATA, NDREF, NDWPB, ND, NFIELD, NHEADX, NHEAD, NHIST
      INTEGER NLINES, NREC1, NREC2, NREC, NSECS, NS, NT, NVER, N
      LOGICAL FIRST, REQB(MAXB), REQT(MAXT), PGST
      LOGICAL PAMP,PPHA,PCLO,PERR,WANT(MAXL),EOF,FEED,BASE
      LOGICAL ORBIT, PU, PV, PW, GOOD(MAXB), NOHIST
      REAL    C, D, EPOCH, E, U, V, W
      REAL    DATA(MAXL,50), DATARC(MAXL)
      DOUBLE PRECISION RA50,DEC50,RAP,DECP,FLUX,X(MAXS),Y(MAXS),
     1          Z(MAXS),GST,GSTM,FREQ,TINTG,AVG,BW,ENDMRK,PARS(NPARS),
     2          VALS(NPARS),WAVE,BX,BY,BZ(MAXB),BEQ(MAXB),BHA(MAXB),
     3          AEIOWM(6,MAXS),WORD(9),DJD,EJD,JEPOCH,
     4          BOFF(MAXB),TIHA, COSDEC,SINDEC
      CHARACTER  INDSN*128,PRDSN*128,HIST*80,REVTIM*17,
     1      FIELD(MAXL)*(WIDTH),CHEAD1(MAXL)*(WIDTH),FCODE(MAXL)*12,
     2      STATN(MAXS)*8, CHEAD2(MAXL)*(WIDTH),BNAME(MAXB)*17,
     3      STNABR*1, STAB(MAXS)*1
      CHARACTER*1   CHPSTK(4)
      CHARACTER*2   CHNSTK(4)
      CHARACTER*16  SOURCE
      CHARACTER*6   TMODE
      CHARACTER*8   Q,TIMEC
      CHARACTER*128 INPUT,EXPAND(MAXFIL)
      CHARACTER*80  HISTRY
C-----------------------------------------------------------------------
      DATA CHPSTK/'I','Q','U','V'/, CHNSTK/'RR','LL','RL','LR'/
      DATA  ENDMRK/'/'/
      DATA  PARS/ 'INput', 9*' ',
     1            'LIStfile', 9*' ',
     2            'AMplitud','PHases','CLose','Errors','ALL','NONE',
     3            'COLs','NOFeed','BASE','Header','GST',20*'BASeline',
     4            20*'TRiangle', 2*'INT', 'LPpage', 'U', 'V', 'W',
     5            'NOHistor' /
      DATA  VALS/ '***UNSET', 9*' ',
     1            10*' ',
     2            6*-1D0, 12D0, 4*-1D0,
     3            20*0D0,20*0D0,1D0,1D5,60D0, 3*-1D0, -1D0 /
C-----------------------------------------------------------------------
  609 FORMAT(A1)
  610 FORMAT(' Page',I3,3(3X,A))
  611 FORMAT(/1X,A)
  620 FORMAT(/' Merge file header information:'//
     1         4X,'Format ',I5,T45,I5,' stations'/
     2         4X,I5,' History records',T45,I5,' baselines'/
     3         4X,I5,' Header records',T45,I5,' closure-phases'/
     4         4X,'Record length = ',I5)
  630 FORMAT(4X,'Amplitudes are Correlation Coefficients',
     1             ' ( x 100000 in listing)')
  640 FORMAT(4X,'Amplitudes are Correlated Flux Densities (Jy)')
  641 FORMAT(4X,'Dataset contains Fringe-Rates, not Amplitudes',
     1             ' (in mHz in listing)')
  642 FORMAT(4X,'Polarization undefined:  Stokes =',I8)
  643 FORMAT(4X,'Polarization code: ',I2,' (',A2,')')
  644 FORMAT(4X,'Stokes parameter:  ',I2,' (',A1,')')
  645 FORMAT(' Baselines: ',10I5/(12X,10I5))
  646 FORMAT(' Triangles: ',9I7/(5X,10I7))
  650 FORMAT(/' History:'/)
  670 FORMAT(/' Source and station parameters:'//
     1         4X,'Source:    ',A/
     2         4X,'RA    = ',A16,' (B1950)',A16,' (precessed)'/
     3         4X,'Dec   = ',A16,'        ',A16/
     4         4X,'Flux density = ',F8.3,' Jy'//
     5         10X,'Station',12X,'X',18X,'Y',18X,'Z'/)
  680 FORMAT(1X,I3,2X,A1,3X,A8,3(4X,F15.3))
  683 FORMAT(//7X,'Orbiter',9X,'A',13X,'E',12X,'I',12X,'O',12X,'W',
     1            12X,'M')
  686 FORMAT(1X,I3,3X,A8,4X,F12.3,5X,F6.3,4(5X,F8.3))
  690 FORMAT(/4X,'Reference date = ',I4,' day ',I3.3,
     2                 '  (',I4,1X,A3,1X,I2.2,', ',A3,')'/
     3         4X,'Julian Date = ',F12.2,',  Epoch J',F7.2/
     4         4X,'G.A.S.T. at UT ',A8,' on reference date is ',A16/
     5         4X,'Frequency = ',F16.9,' GHz  (wavelength',F8.3,' cm)'/
     6         4X,'Coherent integration time = ',F8.1,' sec'/
     7         4X,'Averaging time = ',F8.1,' sec'/
     8         4X,'Bandwidth = ',F16.6,' MHz')
  700 FORMAT(/' Computed baseline parameters:'//
     1             4X,'IHA = GHA(source) - GHA(baseline) - 6 h,',
     2             4X,'GHA(source) = GST - RA'//
     3             12X,'Baseline',18X,'Polar',10X,'Equatorial',
     4             8X,'GHA'/43X,'(wavelengths)',13X,'(hours)')
  710 FORMAT(4X,I4,2X,A1,A1,2X,A17,3X,F13.1,4X,F13.1,5X,F7.3)
  720 FORMAT(19X,100(A))
  730 FORMAT(1X)
  740 FORMAT(I4,3X,I3,'/',A8,100(A))
  750 FORMAT(/2X,'N',8X,A6,2X,100(A))
  760 FORMAT(I5,'=',2A1)
  770 FORMAT(I8)
  780 FORMAT(/' Dataset contains ',I6,' data records, ',I7,
     1        ' data points'
     2       /' UT range: ',I3,'/',A8, ' to ',I3,'/',A8
     3       /' Mean epoch:  JD',F12.3,' = J',F7.2)
C-----------------------------------------------------------------------
C
C Introduction
C
      CALL PUTOUT(PROGNM//
     1         ' displays contents of MERGE-format visibility files')
      CALL PUTOUT('(Version '//VERSN//')')
      CALL PUTOUT(' ')
      CALL MKHIST(PROGNM, VERSN, HISTRY)
C
C Control parameters.
C
      MODE = 0
   10 IF (MODE.NE.0) CALL ERROR('Bad input parameters in '//PROGNM)
      CALL PUTOUT('Parameters (INPUT, LISTFILE... end with /):')
      MODE = 0
      CALL KEYIN(PARS,VALS,NPARS,ENDMRK,MODE,INC,OUTC)
      WRITE (INPUT, '(10A8)') (VALS(J),J= 1,10)
      WRITE (PRDSN, '(10A8)') (VALS(J),J=11,20)
      PAMP = VALS(21).EQ.0D0 .OR. VALS(25).EQ.0D0
      PPHA = VALS(22).EQ.0D0 .OR. VALS(25).EQ.0D0
      PCLO = VALS(23).EQ.0D0 .OR. VALS(25).EQ.0D0
      PERR = VALS(24).EQ.0D0 .OR. VALS(25).EQ.0D0
      PU   = VALS(75).EQ.0D0
      PV   = VALS(76).EQ.0D0
      PW   = VALS(77).EQ.0D0
C     VALS(26) and (30) 'NONE' and 'HEADER' are ignored.
      MCOLS = VALS(27)
      FEED = VALS(28).EQ.-1
      BASE = VALS(29).NE.-1
      PGST = VALS(31).NE.-1
      IF (PGST) THEN
          TMODE = 'G.S.T.'
      ELSE
          TMODE = '  U.T.'
      END IF
      NREC1 = VALS(72)
      NREC2 = VALS(73)
      LPPAGE = VALS(74)
      NOHIST = VALS(78).EQ.0D0
      PAGE = 0
C
C Expand wild cards.
C
      CALL FFILES(INPUT, MAXFIL, EXPAND, NFIL)
      FIRST = .TRUE.
      IFIL = 1
C
C Open input file.
C
  20  IF (IFIL.GT.NFIL) STOP
      IER = VLBOPE(INDAT, EXPAND(IFIL), 'MERGE', 'OLD', INDSN)
      IF (IER.NE.1) THEN
          IFIL = IFIL+1
          GOTO 20
      END IF
      IFIL = IFIL+1
      LDSN = LEN1(INDSN)
      CALL FRDT(INDSN(1:LDSN), REVTIM)
C
C Open listing file.
C
      IF (FIRST) THEN
          FIRST = .FALSE.
          IF (PRDSN.EQ.' ') THEN
C             -- standard output
              PR = OUTC
          ELSE
              PR = 11
              IER = VLBOPE(PR, PRDSN, 'TEXT', 'NEW', PRDSN)
              IF (IER.NE.1) THEN
                  CALL PUTOUT('+++ERROR+++ Cannot open LISTFILE')
                  GOTO 10
              END IF
          END IF
      END IF
C
C Read and print Header Records.
C
      READ (INDAT, END=20, IOSTAT=IER) NVER,NHIST,NHEAD,NS,NB,NT,LENG,
     1      KFLAG,STOKES
      IF (IER.NE.0 .OR. NVER.LT.1 .OR. NVER.GT.2) THEN
          CALL PUTOUT(INDSN(1:LDSN)//' is not a Merge file')
          CLOSE (UNIT=INDAT)
          GOTO 20
      END IF
      PAGE = PAGE+1
      SOURCE = ' '
      IF (NVER.EQ.1) NDWPB = 4
      IF (NVER.EQ.2) NDWPB = 7
      IF (NS.GT.MAXS) THEN
          CALL PUTOUT(INDSN(1:LDSN)//' has too many stations')
          CLOSE (UNIT=INDAT)
          GOTO 20
      END IF
      NHEADX = NHIST + 2 + NS
      IF(NB.GT.0) NHEADX = NHEADX+1
      IF(NT.GT.0) NHEADX = NHEADX+1
      IF(NHEAD.NE.NHEADX) THEN
          CALL PUTOUT(INDSN(1:LDSN)//
     1         ' has wrong number of Header records')
          CLOSE (UNIT=INDAT)
          GOTO 20
      END IF
      IF (PAGE.NE.1) WRITE (PR,609) FORM
      WRITE (PR,610) PAGE,SOURCE,REVTIM,INDSN(1:LDSN)
      WRITE (PR,611) HISTRY(1:LEN1(HISTRY))
      WRITE (PR,620) NVER,NS,NHIST,NB,NHEAD,NT,LENG
      IF(KFLAG.EQ.0) WRITE(PR,630)
      IF(KFLAG.EQ.1) WRITE(PR,640)
      IF(KFLAG.EQ.2) WRITE(PR,641)
      IF (STOKES.EQ.0 .OR. ABS(STOKES).GT.4) THEN
          WRITE (PR,642) STOKES
      ELSE IF (STOKES.LT.0) THEN
          WRITE (PR,643) STOKES, CHNSTK(ABS(STOKES))
      ELSE 
          WRITE (PR,644) STOKES, CHPSTK(STOKES)
      END IF
C
C Read baseline labels and convert to standard form.
C
      IF  (NS.GT.0) THEN      
          READ (INDAT) (LABELB(IB),IB=1,NB)
          DO 101 IB=1,NB
            IS1(IB) = LABELB(IB)/100
            IS2(IB) = LABELB(IB) - 100*IS1(IB)
            IF(IS1(IB).LT.1 .OR. IS1(IB).GT.NS .OR.
     1            IS2(IB).LT.1 .OR. IS2(IB).GT.NS) CALL 
     2                  ERROR('in baseline labels')
            IST1 = IS1(IB)
            IST2 = IS2(IB)
            IF(IST2.GT.9) IST1 = IST1*10
              LABELB(IB) = IST1*10 + IST2
  101          CONTINUE
          WRITE(PR,645) (LABELB(IB),IB=1,NB)
      END IF
C
C Read triangle labels and convert to standard form.
C
      IF (NT.GT.0) THEN
          READ (INDAT) (LABELT(IT),IT=1,NT)
          DO 102 IT=1,NT
            IST1 = LABELT(IT)/10000
            IST2 = LABELT(IT)/100 - 100*IST1
            IST3 = LABELT(IT) - 10000*IST1 - 100*IST2
            IF (IST2.GT.9) IST1 = IST1*10
            IST2 = IST1*10 + IST2
            IF (IST3.GT.9) IST2 = IST2*10
            LABELT(IT) = IST2*10 + IST3
  102     CONTINUE
          WRITE (PR,646) (LABELT(IT),IT=1,NT)
      END IF
C
C Read and print history records.
C
      IF (.NOT.NOHIST) WRITE (PR,650)
      DO 110 I=1,NHIST
          READ (INDAT) HIST
          IF (.NOT.NOHIST) THEN
              CALL FILTER(HIST)
              LH = LEN1(HIST)
              IF (LH.GT.0) WRITE (PR,'(1X,A)') HIST(1:LH)
          END IF
  110 CONTINUE
C
C Read and print source, station, and miscellaneous
C parameters.
C
      READ (INDAT) SOURCE,RA50,DEC50,RAP,DECP,FLUX
      CALL PANGLE('H', RA50,  IA, 3)
      CALL PANGLE('D', DEC50, JA, 3)
      CALL PANGLE('H', RAP,   KA, 3)
      CALL PANGLE('D', DECP,  LA, 3)
      WRITE (PR,670) SOURCE,IA,KA,JA,LA,FLUX
      COSDEC = DCOS(DECP)
      SINDEC = DSIN(DECP)
      ORBIT = .FALSE.
      DO 140 IS=1,NS
          IF (NVER.EQ.1) THEN
              READ (INDAT) STATN(IS),X(IS),Y(IS),Z(IS)
              STATYP(IS) = 0
              STAB(IS) = STNABR(STATN(IS))
          ELSE IF (NVER.EQ.2) THEN
              READ (INDAT) STATN(IS),(WORD(J),J=1,9),STATYP(IS)
              STAB(IS) = STNABR(STATN(IS))
              IF (STATYP(IS).LE.1) THEN
                  X(IS) = WORD(1)
                  Y(IS) = WORD(2)
                  Z(IS) = WORD(3)
                  DO 120 J=1,6
                      AEIOWM(J,IS) = 0.D0
  120             CONTINUE
              ELSE IF (STATYP(IS).EQ.2) THEN
                  ORBIT = .TRUE.
                  X(IS) = 0.D0
                  Y(IS) = 0.D0
                  Z(IS) = 0.D0
                  DO 130 J=1,6
                      AEIOWM(J,IS) = WORD(J)
  130             CONTINUE
              END IF
          END IF
          IF (STATYP(IS).LE.1) WRITE (PR,680) IS, STAB(IS), STATN(IS),
     1            X(IS),Y(IS),Z(IS)
  140 CONTINUE
      IF (ORBIT) THEN
          WRITE (PR,683)
          DO 160 IS = 1,NS
            IF (STATYP(IS).EQ.2) THEN
                DO 150 J=3,6
                  WORD(J) = AEIOWM(J,IS)/RPDEG
  150           CONTINUE
                WRITE (PR,686) IS,STATN(IS),(AEIOWM(J,IS),J=1,2),
     1                  (WORD(J),J=3,6)
            END IF
  160     CONTINUE
      END IF
      READ (INDAT) YEAR,UT,GSTM,FREQ,TINTG,AVG,BW
      NDREF = UT/(60*86400)+1
      Q = TIMEC(MOD(UT/60,86400))
      CALL PANGLE('H', GSTM, IA, 3)
      IF (FREQ.EQ.0.0) THEN
          WAVE = 0.0
      ELSE
          WAVE = VELC/FREQ
      END IF
      IMON = 1
      ID = NDREF
      CALL TDATE(YEAR,IMON,ID,JD,MONTH,DAY)
      DJD = JD-0.5
      JEPOCH = 2000D0 + ((JD-2451545)-0.5D0)/365.25D0
      WRITE (PR,690) YEAR,NDREF,YEAR,MONTH,ID,DAY,DJD,JEPOCH,
     1                     Q,IA,FREQ*1D-9,WAVE*1D2,TINTG,AVG,BW*1D-6
C
C Compute and print baseline parameters (BASE option).
C
      IF (BASE) WRITE (PR,700)
      IF (WAVE.EQ.0.0) WAVE = 1.0
      DO 170 IB=1,NB
            BX = (X(IS1(IB))-X(IS2(IB)))/WAVE
            BY = (Y(IS1(IB))-Y(IS2(IB)))/WAVE
            BZ(IB)  = (Z(IS1(IB))-Z(IS2(IB)))/WAVE
            BEQ(IB) = DSQRT(BX*BX+BY*BY)
            BHA(IB) = 0D0
            IF (BEQ(IB).NE.0D0) BHA(IB) = DATAN2(BY,BX)
            BOFF(IB) = GSTM-RAP-PI/2.0-BHA(IB)
            BNAME (IB) = STATN(IS1(IB))(1:LEN1(STATN(IS1(IB))))//
     1                   '-'//STATN(IS2(IB))
            IF (BASE) WRITE (PR,710) LABELB(IB),STAB(IS1(IB)),
     1            STAB(IS2(IB)), BNAME(IB), BZ(IB),BEQ(IB),BHA(IB)/RPHR
  170 CONTINUE
C
C Baseline and triangle selection.
C
      IF (VALS(32).EQ.0D0 .AND. VALS(52).EQ.0D0) THEN
          DO 180 IB=1,NB
            REQB(IB) = .TRUE.
  180     CONTINUE
          DO 190 IT=1,NT
            REQT(IT) = .TRUE.
  190     CONTINUE
      ELSE
          DO 210 IB=1,NB
            REQB(IB) = .FALSE.
  210     CONTINUE
          DO 212 IT=1,NT
            REQT(IT) = .FALSE.
  212     CONTINUE
          DO 240 I=32,51
            DO 214 IB=1,NB
                IF (VALS(I).EQ.LABELB(IB)) REQB(IB) = .TRUE.
  214       CONTINUE
  240     CONTINUE
          DO 260 I=52,71
            DO 250 IT=1,NT
                IF (VALS(I).EQ.LABELT(IT)) REQT(IT) = .TRUE.
  250       CONTINUE
  260     CONTINUE
      END IF
C
C What is to be printed?
C Each data record contains up to MAXL cells;
C WANT(I) is .TRUE. if cell I is to be listed,
C FCODE(I) is the format for printing cell I,
C CHEAD1(I) is the first column heading for cell I
C           (data type),
C CHEAD2(I) is the second column heading for cell I
C           (baseline or triangle ID).
C
      IF(NB.GT.MAXB.OR.NT.GT.MAXT) CALL ERROR('Too many stations')
      I = 0
      DO 200 J = 1,NB        
          WANT(I+1) = (PAMP.AND.REQB(J))
          WANT(I+2) = (PAMP.AND.PERR.AND.REQB(J))
          WANT(I+3) = (PPHA.AND.REQB(J))
          WANT(I+4) = (PPHA.AND.PERR.AND.REQB(J))
          WANT(I+5) = (PU.AND.REQB(J))
          WANT(I+6) = (PV.AND.REQB(J))
          WANT(I+7) = (PW.AND.REQB(J))
          FCODE(I+1) = '(F8.3)'
          FCODE(I+2) = '(F7.3,1X)'
          IF(KFLAG.EQ.0) FCODE(I+1) = '(5PF8.1)'
          IF(KFLAG.EQ.0) FCODE(I+2) = '(5PF7.1,1X)'
          IF(KFLAG.EQ.2) FCODE(I+1) = '(3PF8.2)'
          IF(KFLAG.EQ.2) FCODE(I+2) = '(3PF7.2,1X)'
          FCODE(I+3) = '(F8.2)'
          FCODE(I+4) = '(F7.2,1X)'
          FCODE(I+5) = '(-6PF8.2)'
          FCODE(I+6) = '(-6PF8.2)'
          FCODE(I+7) = '(-6PF8.2)'
          CHEAD1(I+1) = '   Ampl '
          IF(KFLAG.EQ.2) CHEAD1(I+1) = '   Rate '
          CHEAD1(I+2) = '   +/-  '
          CHEAD1(I+3) = '   Phase'
          CHEAD1(I+4) = '   +/-  '
          CHEAD1(I+5) = ' u (Mwl)'
          CHEAD1(I+6) = ' v (Mwl)'
          CHEAD1(I+7) = ' w (Mwl)'
          WRITE(CHEAD2(I+1),760) LABELB(J),STAB(IS1(J)),STAB(IS2(J))
          CHEAD2(I+2) = ' '
          CHEAD2(I+3) = CHEAD2(I+1)
          CHEAD2(I+4) = ' '
          CHEAD2(I+5) = CHEAD2(I+1)
          CHEAD2(I+6) = CHEAD2(I+1)
          CHEAD2(I+7) = CHEAD2(I+1)
          I = I+7
  200 CONTINUE
      DO 220 J=1,NT
          WANT(I+1) = (PCLO.AND.REQT(J))
          WANT(I+2) = (PCLO.AND.PERR.AND.REQT(J))
          FCODE(I+1)='(F8.2)'
          FCODE(I+2)='(F7.2,1X)'
          CHEAD1(I+1) = '   Cl-Ph'
          CHEAD1(I+2) = '   +/-  '
          WRITE(CHEAD2(I+1),770) LABELT(J)
          CHEAD2(I+2) = '  '
          I = I+2
  220 CONTINUE
      NCELLS=I
C
C How many columns and sections? MPCOLS is the number of columns per
C page (=MCOLS if specified, otherwise 12); NSECS is the number of
C pages needed for each 50-line C chunk; 
C JP(ISEC) is the first cell that appears on page ISEC.
C
      ICOLS=0
      NSECS=0
      MPCOLS=MCOLS
      IF(MCOLS.LT.1 .OR. MCOLS.GT.14) MPCOLS=12
      DO 230 J=1,NCELLS
          IF (WANT(J)) THEN
            ICOLS = MOD(ICOLS,MPCOLS)+1
            IF (ICOLS.EQ.1) THEN
                NSECS = NSECS+1
                JP(NSECS)=J
            END IF
          END IF
  230 CONTINUE
      JP(NSECS+1) = NCELLS+1
C
C Read and print Data Records.
C
      LP = LPPAGE
      NREC = 0
      NDATA = 0
      EPOCH = 0.0
      LINE = 0
      EOF = .FALSE.
C
C Read data records up to end of data or end of file.
C
  310 CONTINUE
      READ(INDAT,END=330) IUTL,(DATARC(J),J=1,NDWPB*NB+2*NT)
      IF (IUTL.LT.0) GOTO 330
C     -- count records and data points
      NREC = NREC+1
      IF (NREC.EQ.1) TFIRST = IUTL
      TLAST = IUTL
      EPOCH = EPOCH + (FLOAT(IUTL)/(24.0*3600.0*60.0) - NDREF + 1)
      DO 315 IB=1,NB
          GOOD(IB) = (DATARC(NDWPB*(IB-1)+2).GT.0.0 .AND. 
     1                DATARC(NDWPB*(IB-1)+2).GT.0.0 )
          IF (GOOD(IB)) NDATA = NDATA+1
  315 CONTINUE
C     -- if this record is to be printed, copy it into the page buffer
      IF (NREC.GE.NREC1 .AND. NREC.LE.NREC2) THEN
          LINE = LINE+1
          IUT(LINE) = IUTL
          RECN(LINE) = NREC
          DO 324 IB=1,NB
            IF (GOOD(IB)) THEN
              DO 320 ND=1,NDWPB
                  DATA(MAXWPB*(IB-1)+ND,LINE) = DATARC(NDWPB*(IB-1)+ND)
  320         CONTINUE
              IF (NDWPB.EQ.4) THEN
                  TIHA = (IUTL-UT)*SIDR + BOFF(IB)
                  U = BEQ(IB)*DCOS(TIHA)
                  V = BZ(IB)*COSDEC + BEQ(IB)*SINDEC*DSIN(TIHA)
                  W = BZ(IB)*SINDEC - BEQ(IB)*COSDEC*DSIN(TIHA)
                  DATA(MAXWPB*(IB-1)+5,LINE) = U
                  DATA(MAXWPB*(IB-1)+6,LINE) = V
                  DATA(MAXWPB*(IB-1)+7,LINE) = W
              END IF
            ELSE
              DO 322 ND=1,MAXWPB
                  DATA(MAXWPB*(IB-1)+ND,LINE) = NODATA
  322         CONTINUE
            END IF
  324     CONTINUE
          DO 326 IT=1,NT
              C = DATARC(NDWPB*NB + 2*(IT-1) + 1)
              E = DATARC(NDWPB*NB + 2*(IT-1) + 2)
              IF (E.LE.0.0) THEN
                  C = NODATA
                  D = NODATA
              END IF
              DATA(MAXWPB*NB + 2*(IT-1) + 1, LINE) = C
              DATA(MAXWPB*NB + 2*(IT-1) + 2, LINE) = E
  326     CONTINUE
          IF (LINE.GE.50) GOTO 350
      END IF
      GOTO 310
C end loop.
C 
  330 EOF=.TRUE.
C
C Print NLINES records in NSECS sections.
C
  350 NLINES = LINE
      IF(NLINES.LT.1 .OR. NSECS.LT.1) GOTO 390
      DO 380 ISEC=1,NSECS
          IF (LP + (3+NLINES+(NLINES+9)/10) .GT. LPPAGE) THEN
              PAGE = PAGE+1
              LP = 1
              IF (FEED) WRITE (PR,609) FORM
              IF (FEED) WRITE (PR,610) PAGE,SOURCE,REVTIM,INDSN(1:LDSN)
          END IF
          LP = LP + (3+NLINES+(NLINES+9)/10)
          NFIELD = 0
          DO 355 J=JP(ISEC),JP(ISEC+1)-1
              IF (WANT(J)) THEN
                  NFIELD = NFIELD+1
                  FIELD(NFIELD) = CHEAD1(J)
              END IF
  355     CONTINUE
          WRITE (PR,750) TMODE,(FIELD(M),M=1,NFIELD)
          NFIELD = 0
          DO 356 J=JP(ISEC),JP(ISEC+1)-1
              IF (WANT(J)) THEN
                  NFIELD = NFIELD+1
                  FIELD(NFIELD) = CHEAD2(J)
              END IF
  356     CONTINUE
          WRITE (PR,720) (FIELD(M),M=1,NFIELD)
          DO 370 LINE=1,NLINES
C             ! record number
              N=RECN(LINE)
C             ! time calculation (UT or GST)
              ND=IUT(LINE)/(60*86400)+1
              ISECS = IUT(LINE)/60
              IF (PGST) THEN
                  GST = MOD(GSTM + SIDR*(IUT(LINE)-UT),2D0*PI)
                  IF (GST.LT.0D0) GST = GST + 2D0*PI
                  ISECS = NINT(GST*86400D0/(2D0*PI))
              END IF
              Q = TIMEC(MOD(ISECS,86400))
C             ! format the required data fields
              NFIELD = 0
              DO 360 J=JP(ISEC),JP(ISEC+1)-1
                  IF (WANT(J)) THEN
                      NFIELD=NFIELD+1
                      IF (DATA(J,LINE) .NE. NODATA) THEN
                          WRITE (FIELD(NFIELD),FCODE(J),IOSTAT=JUNK) 
     1                           DATA(J,LINE)
                      ELSE
                          FIELD(NFIELD) = '      * '
                      END IF
                  END IF
  360         CONTINUE
              IF (MOD(LINE,10).EQ.1) WRITE (PR,730)
              WRITE (PR,740) N,ND,Q,(FIELD(M),M=1,NFIELD)
  370     CONTINUE
  380 CONTINUE
C
C Finished printing NLINES records; go and get some more unless we 
C have reached the end of the file.
C
  390 IF (EOF) THEN
          IF (NREC.GT.0) THEN
              EJD = DJD + EPOCH/NREC
              EPOCH = JEPOCH + EPOCH/NREC/365.25
              WRITE(PR,780) NREC, NDATA, TFIRST/(60*86400)+1,
     1            TIMEC(MOD(TFIRST/60,86400)), TLAST/(60*86400)+1,
     2            TIMEC(MOD(TLAST/60,86400)), EJD, EPOCH
          ELSE
              WRITE(PR,780) NREC
          END IF
          GOTO 20
      ELSE
          LINE = 0
          GOTO 310
      END IF
C-----------------------------------------------------------------------
      END
