	PROGRAM MAKECCR
C-----------------------------------------------------------------------
C MAKECCR: generate blocks (CCRs) for MkII-format experiments on the 
C JPL/Caltech Block II Processor.
C
C S.C. Unwin, VAX-11 Fortran. (Based on TJP's MAKEBLOCK)
C
C Version 1.2:  17-Feb-1987 - fixes several minor format bugs, mostly in 
C		vicinity of PCROPEN etc.  Sets SDFREQ explicitly: do not
C		use with earlier FORTH than V1987020.
C		09-Apr-1987 - allow larger FOFF
C Version 1.3:  18-Apr-1987 - Remove SDFREQ parameter, add NFREQ,
C		for use with FORTH versions above V1987128
C 		26-Jun-1987 - check EXTIME is set when using DRIFTRATE
C Version 2.1	02-Jul-1987 - TDR - added the .log file
C		14-Aug-1987 - SCU - minor layout changes
C		22-Nov-1987 - SCU - minor layout changes
C Version 2.2	15-Mar-1988 - SCU - add AUTOLOAD, CYCSETUP, SCANSIZE, and 
C		reorganize scan blocks.; test $SOURCES and $SCANS syntax
C Version 2.3	28 Apr-1988 - SCU - NRACKS = 3 or 4, and re-order blocks
C		16-May-1988 - SCU - Add BADRACK = 1, 2, 3, or 4
C Version 2.4	16-Aug-1988 - SCU - Add subnetworks
C 		18-Nov-1988 - SCU - Allow seconds in start/stop times
C Version 2.5	24-Jan-1989 - SCU - Add NLAGS = 7 or 8 
C 		06-Apr-1989 - SCU - Fix scan count comment in Block #5
C 		08-Jun-1989 - SCU - Minor layout change in log file 
C Version 2.6	14-Jun-1989 - SCU - Flag unused DSOs in subnetwork scans
C 		27-Jun-1989 - SCU - Bug in NRACKS=3 case with unused DSOs
C Version 2.7	05-Aug-1989 - SCU - Allow NRACKS=2; add INTEG 
C 		18-Apr-1990 - SCU - Use 2-char stn abbreviations (STNABR)
C				  - Truncate station names at 8 characters
C Version 2.8	20-Dec-1990 - SCU - Fix bug with SECONDS in $SCANS section
C				  - Use Level II commands for most of setup
C		05-Mar-1991 - SCU - Remove LAG_DISPLAY (obsolete)
C Version 3.1 	16-Sep-1991 - SCU - Replace DMO/PMO with level II commands 
C				  - Use Level II commands in scan blocks
C				  - Remove CLOSE_AT_END block (obsolete)
C Version 3.2 	01-Sep-1992 - SCU - Enter NFREQ once only
C Version 3.3 	27-Oct-1992 - SCU - Allow LO settings in $SCANS via FSWITCH
C				    and replace DFREQ with SIDEBAND=USB,LSB
C		20-Feb-1993 - SCU - Fix minor bug in log output (for fswitch)
C-----------------------------------------------------------------------
	IMPLICIT  NONE
C
C		Parameters.
C
	CHARACTER*(*) VERSION
	INTEGER   INDAT, MAXSCN, MAXSRC, MAXSTA, NPARS
	INTEGER   OUTDAT, LIST, LOG
	PARAMETER (VERSION='3.3 - 1993 Feb 20')
	PARAMETER (INDAT=1)
	PARAMETER (MAXSRC=200)
	PARAMETER (MAXSCN=500)
	PARAMETER (MAXSTA=16)
	PARAMETER (NPARS=27)
	PARAMETER (OUTDAT=2)
	PARAMETER (LOG=4)
	PARAMETER (LIST=3)
C
C		Variables.
C
	CHARACTER*12 SOURCE(MAXSRC)
	CHARACTER*12 SSSCAN(MAXSCN),SSSCAN2(MAXSCN)
	CHARACTER*16 STATION(MAXSTA),STNAME,BAND
	CHARACTER*8  NOW, YESNO
	CHARACTER*9  TODAY
	CHARACTER*15 EXNAME
	CHARACTER*2  ABBREV(MAXSTA), STNABR
	CHARACTER*20 WORDS(20),WORD20
	CHARACTER*60 DEFAULTB,DEFAULTL,DEFAULTT
	CHARACTER*60 BLKHEAD2
	CHARACTER*64 BLKHEAD, BLOCK(16)
	CHARACTER*80 BLKFILE, INFILE, LISFILE, LOGFILE
	CHARACTER*132 RECORD
	CHARACTER*3  DMOVAL(MAXSCN,28), SIDEBAND
	INTEGER      DAY(MAXSCN), START(MAXSCN), STOP(MAXSCN), INTEG
	INTEGER      IER, NSTA, NSRC,X,I,J,K,Z, I1,I2, NWORDS,LEN1,MODE
	INTEGER      YEAR,NBLK, NSCN, NOFF, NSTAT, NCURR, NSCAN, NWRITE
	INTEGER      M1,MDAY,MSTART,MSTOP, PASSNUM, TRK29(4,28), NLAGS
	INTEGER      GETIN, BCOUNT, NN, NS, I100, LINE, NRACKS, BADRACK
	INTEGER	     NLOGSTN(MAXSCN), LOGSTN(MAXSCN,16), ID,IR,K2, SECS
	INTEGER	     DMOLIST(MAXSCN,28), NDMO(MAXSCN), DMOCOUNT, NTAB
	INTEGER      DMOFLAG(MAXSCN,28), NDMOFLAG(MAXSCN), JSUM, JTEMP
	INTEGER	     WF
	LOGICAL      EXP_FOUND, NOSPLIT, MORE_INP, AUTOLOAD, SUBNET
	LOGICAL	     SECONDS, JFLAG, SUBNETS
	REAL         UT1UTC, EQUINOX, CLOCK(MAXSTA), SCANSIZE
	REAL*8	     LOFF(MAXSTA), CLOCKRATE(MAXSTA), XPOLE, YPOLE
	REAL*8	     ZTROP(MAXSTA), XDAY, XSTART, XSTOP
	REAL*8	     YSTART, YSTOP, NFREQ, NEWFREQ(MAXSCN)
	REAL*8       XLOC(MAXSTA), YLOC(MAXSTA), ZLOC(MAXSTA), EXTIME
	REAL*8       RA(MAXSRC), DEC(MAXSRC), LO(MAXSTA)
	REAL*8       EXPPAR(NPARS), EXPVAL(NPARS), ENDMRK
	COMMON	     /M/ RECORD
	DATA         EXPPAR / 'YEAR', 'UT1UTC', 'EXNAME',' ',
     1			'TITLE', 6*' ', 'NFREQ', 
     2			'NOSPLIT','PASS???','EXTIME','XPOLE','YPOLE',
     3			'SIDE????','AUTO????','SCAN????','NRACKS',
     4			'BADRACK','SECONDS','NLAGS','INTEG',
     5			'FSW????','DFREQ' /
	DATA         EXPVAL / 5*'***UNSET', 6*' ',-1,
     1			-1,0,0,0,0,-1,-1,60,3,0,-1,7,2,
     2			-1,-1 /
	DATA         ENDMRK / '/' /
	DATA  TRK29/    1,2,3,4,
     2 			9,10,11,12,
     3 			1,2,5,6,
     4 			1,2,13,14,
     5 			5,6,3,4,
     6 			13,14,3,4,
     7 			1,2,7,8,
     8 			5,6,13,14,
     9 			7,8,3,4,
     1 			7,8,13,14,
     1 			5,6,7,8,
     2 			9,10,13,14,
     3 			1,2,9,10,
     4 			11,12,13,14,
     5 			9,10,3,4,
     6 			1,2,15,16,
     7 			5,6,9,10,
     8 			15,16,3,4,
     9 			7,8,9,10,
     1 			5,6,15,16,
     1 			1,2,11,12,
     2 			7,8,15,16,
     3 			11,12,3,4,
     4 			9,10,15,16,
     5 			5,6,11,12,
     6 			11,12,15,16,
     7 			7,8,11,12,
     8 			13,14,15,16/
C
C		Open the input and output files.
C
	CALL PUTOUT('MAKECCR generates blocks (CCRs) for the JPL'
     1		//'/CIT Block II VLBI Processor')
	CALL PUTOUT('(Version '//VERSION//')')
	CALL PUTOUT(' ')
	IER = 1
	DO WHILE (IER.NE.0)
	    IER = GETIN(INFILE, 
     1		'Input experiment file (default VLB_CCR:.EXP): ',K)
	    IF (.NOT.IER) CALL EXIT(IER)
	    OPEN (UNIT=INDAT,READONLY, STATUS='OLD', FORM='FORMATTED',
     1	    FILE=INFILE(1:K), IOSTAT=IER, DEFAULTFILE='VLB_CCR:.EXP')
	END DO
	INQUIRE (UNIT=INDAT, NAME=INFILE)
	I1 = INDEX(INFILE,']')+1
	I2 = INDEX(INFILE(I1:),'.')
	DEFAULTB = INFILE(I1:I1+I2-1)//'BLK'
	DEFAULTL = INFILE(I1:I1+I2-1)//'LIS'
	DEFAULTT = INFILE(I1:I1+I2-1)//'LOG'
	IER = 1
	DO WHILE (IER.NE.0)
	    IER = GETIN(BLKFILE, 'Output CCR file '//
     1		'(default '//DEFAULTB(1:LEN1(DEFAULTB))//'): ',K)
	    IF (.NOT.IER) CALL EXIT(IER)
	    OPEN (UNIT=OUTDAT, STATUS='NEW', FORM='UNFORMATTED',
     1		  RECORDTYPE='FIXED', RECL=256,
     2		  DEFAULTFILE=DEFAULTB,
     2		  FILE=BLKFILE(1:K), DISPOSE='DELETE', IOSTAT=IER)
	END DO
	INQUIRE (UNIT=OUTDAT, NAME=BLKFILE)
	IER = 1
	DO WHILE (IER.NE.0)
	    IER = GETIN(LISFILE, 'Listing file '//
     1		'(default '//DEFAULTL(1:LEN1(DEFAULTL))//'): ',K)
	    IF (.NOT.IER) CALL EXIT(IER)
	    OPEN (UNIT=LIST, STATUS='NEW', FORM='FORMATTED',
     1		  DEFAULTFILE=DEFAULTL,
     2		  FILE=LISFILE(1:K), IOSTAT=IER)
	END DO
	INQUIRE (UNIT=LIST, NAME=LISFILE)
	IER = 1
	DO WHILE (IER.NE.0)
	    IER = GETIN(LOGFILE, 'Correlator log file '//
     1		'(default '//DEFAULTT(1:LEN1(DEFAULTT))//'): ',K)
	    IF (.NOT.IER) CALL EXIT(IER)
	    OPEN (UNIT=LOG, STATUS='NEW', FORM='FORMATTED',
     1		  DEFAULTFILE=DEFAULTT,RECL=1024,
     2		  FILE=LOGFILE(1:K), IOSTAT=IER)
	END DO
	INQUIRE (UNIT=LOG, NAME=LOGFILE)
C
C		Read input file.
C
	EXP_FOUND = .FALSE.
   	MORE_INP  = .FALSE.	! Set only when > 1 $EXPERIMENT encountered
	SUBNETS   = .FALSE.	! Set when subnets read in $SCANS section
   10	CALL MKB_GET(INDAT,RECORD,IER)
	IF (IER.NE.0) GOTO 20
C
C		$EXPERIMENT group.
C
   11	IF (RECORD(1:4).EQ.'$EXP') THEN
	    IF (EXP_FOUND) THEN	    ! If NOT first $EXPERIMENT found: --
		MORE_INP = .FALSE.  ! issue warning, process input so far, 
		WRITE(RECORD,1501)  ! then stop
 1501	FORMAT(' Only single pass allowed - additional ',
     1		'$EXPERIMENT data ignored')
		CALL PUTOUT(RECORD(1:LEN1(RECORD)))
		GOTO 21
		ENDIF			
  211	    CONTINUE
C
	    EXP_FOUND = .TRUE.
	    MODE = 0
	    CALL KEYIN(EXPPAR,EXPVAL,NPARS,ENDMRK,MODE,INDAT,6)
	    YEAR = NINT(EXPVAL(1))
	    IF (YEAR.LT.1972 .OR. YEAR.GT.1995) CALL MKB_ERR ('Bad YEAR')
	    UT1UTC = EXPVAL(2)
	    IF (ABS(UT1UTC).GT.1.0) CALL MKB_ERR('Bad UT1UTC')
	    WRITE (WORD20,'(2A8)') EXPVAL(3),EXPVAL(4)
	    EXNAME = WORD20
	    CALL UPCASE(EXNAME)
	    WRITE (BLKHEAD,'(''( '',7A8)') (EXPVAL(I),I=5,11)
	    WRITE (BLKHEAD2,'(7A8)') (EXPVAL(I),I=5,11)
C--	    CALL UPCASE(BLKHEAD)
	    CALL STR$TRANSLATE(BLKHEAD(2:),BLKHEAD(2:),'[]????','(){}`|')
	    K = MIN(LEN1(BLKHEAD)+2,64)
	    BLKHEAD(K:K) = ')'
	    NOSPLIT = EXPVAL(13).NE.-1
	    PASSNUM = EXPVAL(14)
	    IF (PASSNUM.GT.99) CALL MKB_ERR ('Max PASSNUM = 99')
	    EXTIME = EXPVAL(15)
	    XPOLE = EXPVAL(16)
	    YPOLE = EXPVAL(17)

	    AUTOLOAD = EXPVAL(19).NE.-1
	    SCANSIZE = EXPVAL(20)		! Scan length in minutes
	    IF (SCANSIZE.GT.120) CALL MKB_ERR ('Max SCANSIZE = 120 min')
	    IF (SCANSIZE.LT.2)  SCANSIZE = 2.0
	    NRACKS = EXPVAL(21)	
	    IF(NRACKS.LE.1 .OR. NRACKS.GE.5) CALL MKB_ERR 
     1		('Bad NRACKS - allowed values are 2, 3, and 4')
	    BADRACK = EXPVAL(22)
	    IF(BADRACK.NE.0 .AND. NRACKS.LE.3) CALL MKB_ERR
     1		('BADRACK only allowed with NRACKS = 4')
	    SECONDS = EXPVAL(23).NE.-1		! allow seconds in start/stop
	    IF(SECONDS .AND. (PASSNUM.NE.0)) CALL PUTOUT 
     1		('+++WARNING+++ PASSNUM not placed in SCAN# '//
     1		'when SECONDS is set')
	    NLAGS =  EXPVAL(24)
	    IF((NLAGS .NE. 7) .AND. (NLAGS .NE. 8)) CALL MKB_ERR 
     1		('Bad NLAGS - allowed values are 7 (default) and 8')
	    INTEG = EXPVAL(25)
	    IF((INTEG .LE. 1) .OR. (INTEG .GT. 20)) CALL MKB_ERR 
     1		('Bad INTEG - allowed range 2 - 20')
C
C	Set NFREQ and SIDEBAND in $EXPERIMENT or in $SCANS?
C
	    WF = 0			! Word counter for $SCANS line
	    IF(EXPVAL(26).NE.-1) WF=1	! WF=1 if freq-switch; WF=0 otherwise
C
	    IF(WF.EQ.0) THEN		! FSWITCH not set: check NFREQ and
	    NFREQ = EXPVAL(12)		! SIDEBAND are set properly
	    IF(NFREQ .LE. -1) CALL MKB_ERR ('NFREQ unset')
	    WRITE (SIDEBAND,'(A3)') EXPVAL(18)
	    CALL UPCASE(SIDEBAND)
	    IF (SIDEBAND(1:3) .EQ. 'USB') THEN
		BAND = 'UPPER_SIDEBAND'
		ELSEIF (SIDEBAND(1:3) .EQ. 'LSB') THEN
		BAND = 'LOWER_SIDEBAND'
	    	ELSE
		CALL MKB_ERR ('Bad SIDEBAND - must be USB or LSB')
		ENDIF
	    ELSE			! FSWITCH set: don't use NFREQ,SIDEBAND
		IF(EXPVAL(12).NE.-1) CALL MKB_ERR('FSWITCH is set:'//
     1		' NFREQ must be specified only in $SCANS section')
		IF(EXPVAL(18).NE.-1) CALL MKB_ERR('FSWITCH is set:'//
     1		' SIDEBAND must be specified only in $SCANS section')
		ENDIF
	    IF(EXPVAL(27).NE.-1) CALL MKB_ERR
     1  ('DFREQ not supported by MAKECCR; use SIDEBAND = USB or LSB')
C
C		$STATIONS group
C
	ELSE IF (RECORD(1:4).EQ.'$STA') THEN
	    NSTA = 0
   12	    CALL MKB_GET(INDAT,RECORD,IER)
	    IF (IER.NE.0) GOTO 20
	    IF (RECORD(1:1).EQ.'$') THEN	! end of $STATIONS group
		IF (NSTA.NE.16) CALL MKB_ERR
     1		('Must specify exactly 16 stations')
		GOTO 11
		ENDIF
	    CALL MKB_PARSE(RECORD,WORDS,NWORDS)
	    NSTA = NSTA+1
	    IF (NSTA.GT.MAXSTA) CALL MKB_ERR ('Too many stations')
	    IF (NWORDS.LT.4 .OR. NWORDS.GE.10) 
     1		CALL MKB_ERR('in $STATIONS list')
	    IF (NWORDS.LT.5) WORDS(5) = '0.0'
	    IF (NWORDS.LT.6) WORDS(6) = '0.0'
C ** Now: DO NOT set LO by default - use NOMINAL_LO instead **
C		If unset, set LO = NFREQ for each station
C	    IF (NWORDS.LT.7) 
C     1		      WRITE (WORDS(7),'(BN,F20.9)',ERR=900) NFREQ
C **
	    IF (NWORDS.LT.7) WORDS(7) = '0.000'		! LO
	    IF (NWORDS.LT.8) WORDS(8) = '0.000'		! FOFF
	    IF (NWORDS.LT.9) WORDS(9) = '2.000'		! ZTROP
	    STATION(NSTA) = WORDS(1)
	    IF (LEN1(WORDS(1)).GT.8) STATION(NSTA) = WORDS(1)(1:8)
	    CALL UPCASE(STATION(NSTA))
	    ABBREV(NSTA) = STNABR(STATION(NSTA))
	    READ (WORDS(2),'(BN,F20.0)',ERR=900) XLOC(NSTA)
	    READ (WORDS(3),'(BN,F20.0)',ERR=900) YLOC(NSTA)
	    READ (WORDS(4),'(BN,F20.0)',ERR=900) ZLOC(NSTA)
	    READ (WORDS(5),'(BN,F20.0)',ERR=900) CLOCK(NSTA)
	    READ (WORDS(6),'(BN,F20.0)',ERR=900) CLOCKRATE(NSTA)
	    IF   (CLOCKRATE(NSTA).NE.0 .AND. EXTIME.EQ.0)
     1		CALL MKB_ERR ('EXTIME unset')
	    READ (WORDS(7),'(BN,F20.0)',ERR=900) LO(NSTA)
	    IF   (LO(NSTA).LT.0.0) CALL MKB_ERR ('Bad LO') 
	    IF   ((ABS(LO(NSTA)).NE.0.0) .AND. (WF.EQ.1))
     1	    CALL MKB_ERR ('FSWITCH set: cannot set LO in $STATIONS'//
     2		' - use BEDIT in scan blocks (SITE n LO x)')
	    READ (WORDS(8),'(BN,F20.0)',ERR=900) LOFF(NSTA)
	    IF   (ABS(LOFF(NSTA)).GT.2.0E6) CALL MKB_ERR ('Bad FOFF')
	    IF   ((ABS(LOFF(NSTA)).NE.0.0) .AND. (WF.EQ.1))
     1	    CALL PUTOUT ('+++WARNING+++ FSWITCH set:  any FOFF '//		
     2     'applies to ALL frequencies in $SCANS')
	    READ (WORDS(9),'(BN,F20.0)',ERR=900) ZTROP(NSTA)
	    GOTO 12
C
C		$SOURCES group.
C
	ELSE IF (RECORD(1:4).EQ.'$SOU') THEN
	    NSRC = 0
   13	    CALL MKB_GET(INDAT,RECORD,IER)
	    IF (IER.NE.0) GOTO 20
	    IF (RECORD(1:1).EQ.'$') GOTO 11
	    CALL MKB_PARSE(RECORD,WORDS,NWORDS)
	    IF (NWORDS.GE.4) CALL MKB_ERR 
     1		('in $SOURCES - too many entries on line')
	    NSRC = NSRC+1
	    SOURCE(NSRC) = WORDS(1)
	    READ (WORDS(2),'(BN,F20.0)',ERR=900) RA(NSRC)
	    READ (WORDS(3),'(BN,F20.0)',ERR=900) DEC(NSRC)
	    GOTO 13
C
C		$SCANS group.
C
	ELSE IF (RECORD(1:4).EQ.'$SCA') THEN
	    NSCN = 0
	    DO I=1,MAXSCN
	    DO J=1,16			! Initialize subnetwork arrays
	  	NLOGSTN(I) = 0
		LOGSTN(I,J) = 0
		END DO
		END DO
   14	    CALL MKB_GET(INDAT,RECORD,IER)
	    IF (IER.NE.0) GOTO 20
	    IF (RECORD(1:1).EQ.'$') GOTO 11
	    CALL MKB_PARSE(RECORD,WORDS,NWORDS)
	    IF (NWORDS.GT.15) CALL MKB_ERR	     ! see also NLOGSTN below
     1		('in $SCANS - too many entries on line')
	    READ (WORDS(2),'(BN,F20.0)',ERR=900) XDAY
	    READ (WORDS(3),'(BN,F20.0)',ERR=900) XSTART
	    READ (WORDS(4),'(BN,F20.0)',ERR=900) XSTOP
	    MDAY = NINT(XDAY)
	    SECS=100		! this adds '00' to 'input 'hhmm' format
	    IF (SECONDS) SECS=1   
	    MSTART=NINT(SECS*XSTART)
	    MSTOP =NINT(SECS*XSTOP)
	    IF (MDAY.LT.1.OR.MDAY.GT.366) CALL MKB_ERR('Bad day number')
	    IF (MSTART.GE.MSTOP) CALL MKB_ERR('Stoptime precedes start')
	    IF (MSTART.LT.0) CALL MKB_ERR('Bad start time')
	    IF (MSTOP.GT.240000) CALL MKB_ERR('Bad stop time')
	    M1=(MSTART/10000+1)*10000
   15	    	NSCN = NSCN+1
		IF (NSCN.GT.MAXSCN) CALL MKB_ERR('Too may scans')
		SSSCAN(NSCN) = WORDS(1)
C
		IF(WF.EQ.1) THEN
		    IF(NWORDS.LT.5) CALL MKB_ERR
     1		    ('in $SCANS - too few entries on line')
		    NEWFREQ(NSCN) = 0.0D0	! i.e. default is 'unset'
		    READ (WORDS(5),'(BN,F20.0)',ERR=900) NEWFREQ(NSCN)
		    IF(NEWFREQ(NSCN).EQ.0.0D0)
     1		    CALL MKB_ERR('Bad LO in $SCANS')
		    ENDIF
C
C--	Look for subnetworked scans (WF=1 if freq-switch; WF=0 otherwise)
C
	    IF(NWORDS.GE.5+WF) THEN	
		SUBNETS = .TRUE.
		SSSCAN2(NSCN) = WORDS(5+WF)	! second (subnetwork) source 
		NLOGSTN(NSCN) = NWORDS-(5+WF)
	    	IF (NLOGSTN(NSCN).GT.8) CALL MKB_ERR 
     1	('in $SCANS - too many subnetwork stations (max 8)')
	    	IF (MOD(NLOGSTN(NSCN),2) .NE. 0) CALL MKB_ERR
     1	('in $SCANS - odd number of subnetwork stations')
		DO I1 = 1,NLOGSTN(NSCN) 
		READ (WORDS(I1+5+WF),'(BN,I20)',ERR=900) LOGSTN(NSCN,I1)
		IF ((LOGSTN(NSCN,I1).LE.0)  .OR.
     1		    (LOGSTN(NSCN,I1).GT.16)) CALL MKB_ERR
     2	('in $SCANS - subnetwork station number out of range')
		END DO
C
C	Check syntax of LOGSTN list
C
		DO I2 = 1,NLOGSTN(NSCN),2
	  	IF ( (MOD(LOGSTN(NSCN,I2),2) .NE. 1)  .OR.
     1		(LOGSTN(NSCN,I2) .NE. (LOGSTN(NSCN,I2+1)-1)) )
     2    	CALL MKB_ERR
     3		('in $SCANS - subnetwork station number syntax')
	  	END DO	
C
		DO I2 = 1,NLOGSTN(NSCN)-1
		IF( LOGSTN(NSCN,I2) .GE. LOGSTN(NSCN,I2+1) ) 
     1		CALL MKB_ERR
     2		('in $SCANS - subnetwork stations out of sequence')
		END DO
		ENDIF		! end reading/checking of subnet scans
C
C	Set day number, start and stop times for scan
C	
		DAY(NSCN) = MDAY
		START(NSCN) = MSTART
		IF( (M1.LT.MSTOP) .AND. (.NOT.NOSPLIT) )  THEN
			STOP(NSCN) = M1
			MSTART = M1
			M1 = M1+10000
			GOTO 15
		ELSE
		        STOP(NSCN) = MSTOP
		END IF
	    GOTO 14
C
	END IF
	GOTO 10			! Read next input record
C
C		End of input file.
C
   20	CLOSE (UNIT=INDAT)
	MORE_INP = .FALSE.	! Don't try to read more input
	IF( ((NRACKS.LE.3) .OR. (BADRACK.NE.0)) .AND. SUBNETS ) THEN
	   CALL PUTOUT ('+++WARNING+++ inspect DMO and D12/D34 '//		
     1	'lists for subnetworked scans by hand.')
	   CALL PUTOUT ('Not all combinations are valid with '//
     1	'BADRACK, or NRACKS = 2 or 3.')
	   ENDIF
C
C		Write results.
C
   21	IF (.NOT.EXP_FOUND) CALL MKB_ERR('$EXPERIMENT data missing.')
C
C		Header block (#0)
C
	DO I=1,16
	   BLOCK(I) = ' '
	   END DO
	BLOCK(1) = BLKHEAD
C
	IF(NRACKS.EQ.4) THEN
	BLOCK(3) = 
     1	'CR ." Use initialization parameter file NEW4 (4 racks)"'
	ELSE IF(NRACKS.EQ.3) THEN
	BLOCK(3) = 
     1	'CR ." Use initialization parameter file NEW3 (3 racks)"'
	ELSE
	BLOCK(3) = 
     1	'CR ." Use initialization parameter file NEW2 (2 racks)"'
	ENDIF
C
	BLOCK(4) =
     1  'CR ." on main process terminal before loading CCR file. "'
	BLOCK(5) = 'CR ."   "'
	BLOCK(6) = 
     1	'CR ." WARNING: Must type # 1 LOAD on command (i.e. user)"'
	BLOCK(7) = 'CR ." window before loading scan block. "'
	BLOCK(8) = 'CR ."   "'
	WRITE(BLOCK(9),100) VERSION
 100	FORMAT('CR ." MAKECCR Version ',A,' "')
	CALL DATE(TODAY)			! Was called in .log section
	CALL TIME(NOW)
	WRITE(BLOCK(10),101) TODAY,NOW
 101	FORMAT('CR ." Run: ',A,1X,A,' " ?CCR CR')
	WRITE(BLOCK(11),102) BLKHEAD2(1:55)
 102	FORMAT('." ',A,'" CR')
	BLOCK(16) = ';S'
	CALL MKB_PUT(OUTDAT,LIST,BLOCK,BLKFILE(1:LEN1(BLKFILE)))
C
C		Experiment message block (#1)
C
	DO I=1,16
	  BLOCK(I) = ' '
	  END DO
	BLOCK(1) = BLKHEAD
	WRITE(BLOCK(3),399) EXNAME(1:LEN1(EXNAME)) 
399	FORMAT('." Loading setup for experiment:  ',A,'" CR')
	BLOCK(5) = 
     1	'\ This block must be loaded before any scan blocks.'
	BLOCK(6) = 
     1	'\ Loads Level II Interface commands which set PCR allocation'
	BLOCK(7) = 
     1	'\ etc., and replace SETCHANNEL, SETINTERVAL, TRKSELECT, etc.'
	BLOCK(8) =
     1	'\ Additional ''global'' parameters may be added here:'
	BLOCK(16) = ';P'
	CALL MKB_PUT(OUTDAT,LIST,BLOCK,BLKFILE(1:LEN1(BLKFILE)))
C
C		Number of lags, intervals block (#2)
C
	DO I=1,16
	  BLOCK(I) = ' '
	  END DO
	BLOCK(1) = BLKHEAD
	BLOCK(3) = 
     1	'#LAGS 8       \ standard 8-lag mode; cancels fringe search'
	WRITE(BLOCK(5),201) INTEG
 201	FORMAT    ('COR_DATA_INTERVAL    ',I2)
	BLOCK(6)= 'STN_DATA_INTERVAL     0'
	BLOCK(7)= 'MDL_UPDATE_INTERVAL  20'
	BLOCK(16) = ';P'
	CALL MKB_PUT(OUTDAT,LIST,BLOCK,BLKFILE(1:LEN1(BLKFILE)))
C
C		Model parameters (all sites) block (#3)
C
	DO I=1,16
	  BLOCK(I) = ' '
	  END DO
	BLOCK(1) = BLKHEAD
	BLOCK(3) = '\ Parameters applying to all sites (1-16), port 1'
	BLOCK(5) = 'PORT 1    ALL_SITES'
	WRITE(BLOCK(7),204) EXNAME(1:LEN1(EXNAME)),YEAR,EXTIME
 204	FORMAT('EXPERIMENT " ',A,'"'I6,F11.6)
	WRITE(BLOCK(8),202) UT1UTC
 202	FORMAT('UT1-UTC ',F7.4) 
	WRITE(BLOCK(9),203) XPOLE,YPOLE
 203	FORMAT('POLE_POSITION ',2F6.2)
	BLOCK(10) = 
     1	'RETARD_MODEL         \ Retarded station models'

	BLOCK(11) = 
     1	'PCAL_OFF             \ Turn off tone model application'
	IF(WF.EQ.0) THEN		! WF=0 implies FSWITCH not set
	WRITE(BLOCK(12),205) NFREQ
 205	FORMAT('NOMINAL_LO ',F18.9,'  \ FSWITCH not set:  NOMINAL_LO')
	WRITE(BLOCK(13),206) BAND
 206	FORMAT(A14,'       \ and SIDEBAND must be set here only.')
	ELSE
	BLOCK(11) = '\ FSWITCH set:  must set NOMINAL_LO and SIDEBAND'
	BLOCK(12) = '\ in scan blocks only.'
	ENDIF
	BLOCK(16) = ';P'
	CALL MKB_PUT(OUTDAT,LIST,BLOCK,BLKFILE(1:LEN1(BLKFILE)))
C
C		Data output setup etc. (Block #4)
C
	DO I=1,16
	  BLOCK(I) = ' '
	  END DO
	BLOCK(1) = BLKHEAD
	  IF (NLAGS.EQ.7) THEN
	  BLOCK(3) = 'ODDLAGS              \ 7 lags (zeroing lag 8)'
	  ELSE
	  BLOCK(3) = 'EVENLAGS             \ 8 lags (no zeroing)'
	  ENDIF
	BLOCK(4) = 
     1  'MTON                 \ Enable all MkIII stations'	
	BLOCK(5) = 
     1  'BIN_DATA             \ Output FFT bins'
	IF (NRACKS.EQ.2) SCANSIZE = 13002.0*SCANSIZE/60.0     	! 1 baselines
	IF (NRACKS.EQ.3) SCANSIZE = 32010.0*SCANSIZE/60.0     	! 3 baselines
	IF (NRACKS.EQ.4) SCANSIZE = 64009.0*SCANSIZE/60.0	! 6 baselines
	WRITE(BLOCK(6),504) JINT(SCANSIZE)
 504	FORMAT('PCRSIZE',I7,'       \ Set initial PCR file allocation')
	BLOCK(7) = 
     1	'PCRON                \ Enable writing to PCR file'
	BLOCK(8) = 
     1	'TSYSOFF  ERROFF      \ Disable Interface ancilliary records'
C
	IF (NRACKS.EQ.4) THEN
	 BLOCK(14) =
     1	 ': D12 DMOSET 1 DSOSET " DUMMY" _STNAME 2 DSOSET " DUMMY"'
	 BLOCK(15) =
     1	 '_STNAME ; : D34 DMOSET 3 DSOSET " DUMMY" _STNAME 4 DSOSET '
	 BLOCK(16) = '" DUMMY" _STNAME ;  ;P   \ for use in subnets'
	ELSE IF (NRACKS.EQ.3) THEN
	 BLOCK(14) =
     1	 ': D12 DMOSET 1 DSOSET " DUMMY" _STNAME 2 DSOSET " DUMMY"'
	 BLOCK(15) =
     1   '_STNAME ; : D34 DMOSET 3 DSOSET " DUMMY" _STNAME ;'
	 BLOCK(16) = ';P   \ for use in subnets' 
	ELSE
	 BLOCK(16) = ';P'
	ENDIF
	CALL MKB_PUT(OUTDAT,LIST,BLOCK,BLKFILE(1:LEN1(BLKFILE)))
C
C		Experiment message block (#5)
C
	DO I=1,16
	  BLOCK(I) = ' '
	  END DO
	BLOCK(1) = BLKHEAD
	WRITE(BLOCK(3),400) BLKFILE(1:LEN1(BLKFILE))
400	FORMAT('." CCR file:    ',A,'" CR')
	YESNO = 'off'
	IF (AUTOLOAD) YESNO = 'on'
	WRITE(BLOCK(4),401) EXNAME(1:LEN1(EXNAME)),YESNO(1:LEN1(YESNO))
401	FORMAT('." Experiment:  ',A,', with AUTOLOAD ',A,'" CR')
	WRITE(BLOCK(5),403) PASSNUM, 17+NSCN-1
403	FORMAT('." Pass number: ',I3,';  Scan blocks: 17 - ',I3,'" CR')
	BLOCK(6) = 
     1'." Logical:  1  2  3  4  5  6  7  8  9  10 11 12 13 14 15 16"'		
	BLOCK(7) = 'CR'
       	WRITE(BLOCK(8),501) (ABBREV(J),J=1,16)				
	BLOCK(9) = 'CR'
	WRITE(BLOCK(10),404) NRACKS, BADRACK
404	FORMAT('." NRACKS =',I2,';  unused rack =',I2,'" CR')
	BLOCK(11) = 
     1	'." Experiment setup completed.  Ready to load scan."'
	BLOCK(12) = 'CR'
	BLOCK(16) = 
     1	';S  \------------------------------'//
     2	'-----------------------------'
	CALL MKB_PUT(OUTDAT,LIST,BLOCK,BLKFILE(1:LEN1(BLKFILE)))
C
C		Load delay and phase models (Block #6)
C
	DO I=1,16
	   BLOCK(I) = ' '
	   END DO
	BLOCK(1) = BLKHEAD
	BLOCK(3) = '\ Load station-dependent parameters for all scans'
	BLOCK(4) = 
     1'\ using Level II Interface commands (replaces DMO/PMO words)'
	BLOCK(6) = '7 LOAD     \ Load station data (16 sites)'
	BLOCK(8) = 
     1'\ Insert additional (station-dependent) model parameters'
	BLOCK(9) = 
     1'\ here, if necessary:'
	BLOCK(16) = ';S'
	CALL MKB_PUT(OUTDAT,LIST,BLOCK,BLKFILE(1:LEN1(BLKFILE)))
C
C		Delay/phase models: blocks 7 - 14 (Use Level II commands)
C 		(previously blocks 7 - 34 and 35 - 62) 
C
	K = 0		
	DO I=7,14		! I = block number (8 model blocks)
		DO J=1,16
	  	BLOCK(J) = ' '
	  	END DO
		BLOCK(1) = BLKHEAD
	LINE = 3
	DO J=1,2
	   K = K + 1		! K = site number (16 sites)
	   WRITE (BLOCK(LINE),1000) K,ABBREV(K)
	   WRITE (BLOCK(LINE+1),1001) STATION(K)(1:LEN1(STATION(K))),
     1		XLOC(K),YLOC(K),ZLOC(K)
	   WRITE (BLOCK(LINE+2),1002) CLOCK(K),CLOCKRATE(K)
C	   WRITE (BLOCK(LINE+3),1003) BAND,LOFF(K)
	   WRITE (BLOCK(LINE+3),1003) LOFF(K)
		IF (LO(K).GT.0.0)  ! Only >0 if specfied in $STATIONS
     1	   WRITE (BLOCK(LINE+4),1004) LO(K)
	   WRITE (BLOCK(LINE+5),1005) ZTROP(K)
	   LINE = 10
	   END DO
		IF (K.EQ.16) THEN
	BLOCK(16) = 
     1		';S  \------------------------------'//
     2		'-----------------------------'
		ELSE
		BLOCK(16) = ';P'
		ENDIF
	CALL MKB_PUT(OUTDAT,LIST,BLOCK,BLKFILE(1:LEN1(BLKFILE)))
	END DO
1000	FORMAT('SITE ',I2,T46,' \ Site name = ',1A2)
1001	FORMAT('ANTENNA_XYZ " ',A,'" ',3F12.1)
1002	FORMAT('CLOCK_OFFSET ',F10.3,'  CLOCK_RATE     ',
     1		F10.4,' \ usec/day')
C1003	FORMAT(A16,9X,'FRINGE_RATE',F14.4,' \ Hz')
1003	FORMAT(16X,9X,'FRINGE_RATE',F14.4,' \ Hz')
1004	FORMAT('LO ',F18.9)
1005	FORMAT('ZENITH_TROPOSPHERE ',F8.2)
C
C		Additional scan parameters (block 15)
C
	DO I=1,16
	  BLOCK(I) = ' '
	  END DO
	BLOCK(1) = BLKHEAD
	BLOCK(3) = 
     1'\ Insert additional scan parameters here, if necessary:'
	BLOCK(16) = ';P'
	CALL MKB_PUT(OUTDAT,LIST,BLOCK,BLKFILE(1:LEN1(BLKFILE)))
C
C		'Scan setup completed' message (block #64)
C
	DO I=1,16
	  BLOCK(I) = ' '
	  END DO
	BLOCK(1) = BLKHEAD
	BLOCK(3) = 
     1  '\ Compute models, type scan times and setup message'
	BLOCK(4) = ' '
	BLOCK(5) = 'GO'
	WRITE(BLOCK(6),505) NRACKS, BADRACK, YESNO(1:LEN1(YESNO))
505	FORMAT('." NRACKS =',I2,';  unused rack =',I2,
     1		';  AUTOLOAD is ',A,'" CR')
	BLOCK(7) = ' '
	IF(WF.EQ.1) BLOCK(7) = 
     1	'." Frequency switching between scans is enabled" CR'
	BLOCK(8) = 
     1'." Logical:  1  2  3  4  5  6  7  8  9  10 11 12 13 14 15 16"'		
	BLOCK(9) = 'CR'
	WRITE(BLOCK(10),501) (ABBREV(J),J=1,16)				
501	FORMAT('." Physical:',16(1X,1A2),'"')
	BLOCK(11) = 'CR'
	WRITE(BLOCK(12),503) EXNAME(1:LEN1(EXNAME)) 
503	FORMAT('." Scan setup for expt ',A,' completed." CR')
	BLOCK(13) = 'SITE 1 ?SCAN ALL_SITES 2 BELL'
	BLOCK(16) = 
     1	';S  \------------------------------'//
     2	'-----------------------------'
	CALL MKB_PUT(OUTDAT,LIST,BLOCK,BLKFILE(1:LEN1(BLKFILE)))
C
C		Scan blocks (#17 onward)
C
	DO I=1,NSCN
	DO J=1,16
	  BLOCK(J) = ' '
	  END DO
	BLOCK(2) = '6 LOAD   ALL_SITES'
	IF(WF.EQ.1) THEN		! Frequency switching enabled
		BAND = 'UPPER_SIDEBAND'
		IF(NEWFREQ(I).LT.0.0D0) BAND = 'LOWER_SIDEBAND'
		WRITE(BLOCK(2),3000)  DABS(NEWFREQ(I)), BAND
 3000		FORMAT('6 LOAD  ALL_SITES  NOMINAL_LO',F18.9,1X,A14)
		ENDIF	
	IF(AUTOLOAD) THEN
		BLOCK(16) = '15 LOAD  SRC @ 1+ AUTOLOAD  ;S'	
		ELSE
		BLOCK(16) = '15 LOAD  ;S'
		ENDIF
	IF(I.EQ.NSCN) BLOCK(16) = '15 LOAD  ;S'	! Don't AUTOLOAD on last scan

	YSTART = 1D6*DFLOTJ(DAY(I)) + DFLOTJ(START(I))
	YSTOP  = 1D6*DFLOTJ(DAY(I)) + DFLOTJ(STOP(I))
	NSCAN = YSTOP/1D2 + DFLOTJ(PASSNUM)*1D7
C		Ignore passnumber if seconds specified (won't fit)
	IF (SECONDS) NSCAN = YSTOP
	WRITE (BLOCK(4),1020) NSCAN,(YSTART/1D6),(YSTOP/1D6)
 1020	FORMAT('SCAN ',I10,2F12.6)
C
C	Generate name for PCR file from EXNAME, PASSNUM, DAY, and STOP
C
	IF(PASSNUM.EQ.0) THEN
		NN = 3
	ELSE
	IF(PASSNUM.LE.9) THEN
		NN = 4
        ELSE
		NN = 5
	ENDIF
	ENDIF
		IF (SECONDS) THEN
		I100 = 1
		NS = 6
		ELSE
		I100 = 100
		NS = 4
		ENDIF
		K = 45 - LEN1(EXNAME) - NN - NS
		WRITE(BLOCK(1),1027) EXNAME(1:LEN1(EXNAME)),
     1	PASSNUM*1000+DAY(I),STOP(I)/I100,BLKHEAD2(1:K)
 1027	FORMAT('PCROPEN " ',A,'_',I<NN>.<NN>,'_',I<NS>.<NS>,
     1	'"  \ 'A)
C
	DO K=1,NSRC
	  IF (SOURCE(K).EQ.SSSCAN(I)) THEN
	    WRITE (BLOCK(3),1021) SOURCE(K)(1:LEN1(SOURCE(K))),
     1	    RA(K), DEC(K)
 1021	    FORMAT('SOURCE " ',A,'"  ',F10.3,2X,F10.2,'  1950')
	    GOTO 300
	    ENDIF
	  END DO
	CALL MKB_ERR('Undefined source: '//SSSCAN(I))
  300	CONTINUE
C
C++++++++++++	Begin subnetwork output (if any) for the I=1,NSCN scan
C
	IF(NLOGSTN(I).EQ.0) GOTO 301		! test for subnetwork
C
C		Determine flags to apply to partially-used delay models
C
	JFLAG = .FALSE.		! Flag for FIRST match (2-antenna subnet only)
C				
	DMOCOUNT = 0		! Initialize DMO lists and counts
	NDMO(I) = 0
	NDMOFLAG(I) = 0
	DO ID = 1,28		
	  DMOLIST(I,ID) = 0
	  DMOFLAG(I,ID) = 0
	  DMOVAL(I,ID) = '   '	  
	  END DO	
C-----
	DO ID = 1,28		! Loop through DMOs and search for matches
C
	  J = 0					
	  JSUM = 0
	  DO I1 = 1,NLOGSTN(I)
	  DO IR = 1,4	
	      	IF (LOGSTN(I,I1) .EQ. TRK29(IR,ID)) THEN
		  J = J + 1	
		  JSUM = JSUM + IR
		  ENDIF
		END DO
	      	END DO
C
          IF (J .EQ. 4) THEN			! Full use of delay model 
		NDMO(I) = NDMO(I) + 1		! set subnet source name
		DMOLIST(I,NDMO(I)) = ID
		DMOCOUNT = DMOCOUNT + 1
		ENDIF
C
	  IF ( (J .EQ. 2) .AND. (JSUM .NE. 3) .AND. (JSUM .NE. 7) )
     1	  CALL MKB_ERR ('in $SCANS: subnetwork DMO/DSO count')
C
	  IF ( (J. EQ. 2) .AND. (NLOGSTN(I) .EQ. 2) 
     1		          .AND. (.NOT.JFLAG) ) THEN
		JFLAG = .TRUE.			! mark FIRST occurrence of match
		NDMO(I) = NDMO(I) + 1		! set subnet source name
		DMOLIST(I,NDMO(I)) = ID
		DMOCOUNT = DMOCOUNT + 1
		IF (JSUM .EQ. 3) JTEMP = 7	! switch DSO 1,2 for DSO 3,4
		IF (JSUM .EQ. 7) JTEMP = 3
		JSUM = JTEMP			
		ENDIF
C
	  IF (J .EQ. 2) THEN			! Partial use of delay model
		NDMOFLAG(I) = NDMOFLAG(I) + 1
		DMOFLAG(I,NDMOFLAG(I)) = ID
		  IF (JSUM .EQ. 3) THEN
		    DMOVAL(I,NDMOFLAG(I)) = 'D12'
		  ELSE IF (JSUM .EQ. 7) THEN
		    DMOVAL(I,NDMOFLAG(I)) = 'D34'
		  ELSE
		    CALL MKB_ERR 
     1	('in $SCANS: subnetwork partial DMO use flag')
		  ENDIF			
	        ENDIF
	  END DO		! end of loop for DMO matches (ID = 1,28)
C-----
	IF(DMOCOUNT.EQ.0) 
     1	CALL MKB_ERR ('in $SCANS: subnetwork station list')
C
C		Check subnetwork source name
C
	DO K=1,NSRC
	  IF (SOURCE(K).EQ.SSSCAN2(I)) THEN
	    K2 = K
	    GOTO 303
	    ENDIF
	  END DO
	CALL MKB_ERR('Undefined subnetwork source: '//SSSCAN2(I))
  303	CONTINUE    
C
C 		Write subnetwork source/DMOs into scan block
C
	IF(NDMO(I).GT.6) CALL MKB_ERR
     1	('Too many DMOs in subnetwork for source '//SSSCAN2(I))
	WRITE (RECORD,1018) NSCAN,SOURCE(K2)(1:LEN1(SOURCE(K2))), 
     1		(DMOLIST(I,J),J=1,NDMO(I))
 1018	FORMAT ('SCAN# ',I10,' Source: ',A,' DMO: ',16I3)
	CALL PUTOUT(RECORD(1:LEN1(RECORD)))
	DO J=1,NDMO(I)
	WRITE (BLOCK(5+J),1019) 
     1	 DMOLIST(I,J),SOURCE(K2)(1:LEN1(SOURCE(K2))),RA(K2), DEC(K2)
 1019	FORMAT
     1	 ('DMO ',I2,'  NAME " ',A,'"  RA0 ',F10.3,' DEC0 ',F10.2)
	END DO
C
C 		Write subnetwork DMO partial use flags into scan block
C
	IF(NDMOFLAG(I).GT.16) CALL MKB_ERR
     1	('Too many DMO flags in subnetwork for source '//SSSCAN2(I))
	IF(NRACKS.GT.2) THEN		! Discard DMO flags for NRACKS=2
     	WRITE (BLOCK(6+NDMO(I)),1024)  ( (DMOFLAG(I,J), DMOVAL(I,J)), 
     1	J=1,MIN(NDMOFLAG(I),8) )
 1024	FORMAT(8(I2,1X,A3,1X))
	IF(NDMOFLAG(I).GT.8)
     1	WRITE (BLOCK(7+NDMO(I)),1024)  ( (DMOFLAG(I,J), DMOVAL(I,J)), 
     2	J=9,NDMOFLAG(I) ) 
	ENDIF
C
C++++++++++++	End subnetwork output
C
  301	CONTINUE
	CALL MKB_PUT(OUTDAT,LIST,BLOCK,BLKFILE(1:LEN1(BLKFILE)))
	END DO
C
C		Generate entry in 'log' file (unit 4)
C
	DATA BCOUNT /0/
	DO 1999 Z=1,NSCN
	IF (MOD(BCOUNT,2).EQ.0) THEN
		WRITE(4,2070)BLKHEAD2(1:50),TODAY,NOW
	END IF
	WRITE(4,2005) BLKFILE(1:LEN1(BLKFILE)),Z+16
 2005	FORMAT(/,2X,'CCR file:  ',A,',',T62,'Block #',I4)
C
	IF(PASSNUM.EQ.0) THEN
		NN = 3
	ELSE
	IF(PASSNUM.LE.9) THEN
	  	NN = 4
        ELSE
	  	NN = 5
	ENDIF
	ENDIF
		IF (SECONDS) THEN
		I100 = 1
		NS = 6
		ELSE
		I100 = 100
		NS = 4
		ENDIF
		WRITE(4,2000) 
     1		EXNAME(1:LEN1(EXNAME)),PASSNUM*1000+DAY(Z),STOP(Z)/I100
C
	YSTART = 1D6*DFLOTJ(DAY(Z)) + DFLOTJ(START(Z))
	YSTOP  = 1D6*DFLOTJ(DAY(Z)) + DFLOTJ(STOP(Z))
	NSCAN = YSTOP/1D2 + DFLOTJ(PASSNUM)*1D7
C		Ignore passnumber if seconds specified (won't fit)
	IF (SECONDS) NSCAN = YSTOP
C
	NTAB = 57 - 2*NS 	! set tab position for UT right-justified
	WRITE(4,2020) NRACKS,PASSNUM,
     1		DAY(Z),START(Z)/I100,STOP(Z)/I100
	IF (NLOGSTN(Z).GT.0) THEN
	  WRITE(4,2031)SSSCAN(Z)(1:LEN1(SSSCAN(Z))),
     1	               SSSCAN2(Z)(1:LEN1(SSSCAN2(Z))),
     2		       (DMOLIST(Z,I2),I2=1,NDMO(Z))	
	  ELSE
	  WRITE(4,2030)SSSCAN(Z)(1:LEN1(SSSCAN(Z)))
	  ENDIF
C
	IF(WF.EQ.1) THEN		! Frequency switching enabled
	IF(NEWFREQ(Z).GT.0.0D0) THEN
		WRITE(4,3010) NSCAN, NEWFREQ(Z)
 3010		FORMAT(2X,'Scan number: ',I10,
     1	',  nominal LO:',F13.6,' MHz, UPPER_SIDEBAND')
		ELSE
		WRITE(4,3011) NSCAN, -NEWFREQ(Z)
 3011		FORMAT(2X,'Scan number: ',I10,
     1	',  nominal LO:',F13.6,' MHz, LOWER_SIDEBAND')
		ENDIF
	ELSE				! Frequency switching not enabled
	WRITE(4,2010)NSCAN,NFREQ,BAND	
 2010	FORMAT(2X,'Scan number: ',I10,
     1	',  nominal LO:',F13.6,' MHz, ',A14)
	ENDIF	
	WRITE(4,2045)

	DO X=1,MAXSTA
	    STNAME = STATION(X)
	    IF (STNAME.EQ.'DUMMY') STNAME=' '
	    SUBNET = .FALSE. 			! X = subnetwork station?
	    IF(NLOGSTN(Z).GT.0) THEN		! subnetworking present
	      DO I1 = 1,NLOGSTN(Z)
	      IF(X.EQ.LOGSTN(Z,I1)) SUBNET = .TRUE.
	      ENDDO
	      ENDIF
	    IF (SUBNET) THEN
		 WRITE(4,2051)X,STNAME(1:8),ABBREV(X)	! subnetwork
		 ELSE
		 WRITE(4,2050)X,STNAME(1:8),ABBREV(X)	! no subnetwork
		 ENDIF
	END DO
C
	WRITE(4,2060)
	IF (MOD(BCOUNT,2).EQ.0) WRITE(4,2075)
C
	BCOUNT = BCOUNT + 1
 1999	CONTINUE
 2000	FORMAT(2X,'PCR file:  ',A,'_',
     1	I<NN>.<NN>,'_',I<NS>.<NS>,'.PCR')	
C
 2020	FORMAT(2X,'NRACKS =',I2,';  Pass',I3,';  scan times:',
     1	T<NTAB>,'Day ',I3,' . ',I<NS>.<NS>,' - ',I<NS>.<NS>,' UT')
 2030	FORMAT(2X,'Source:  ',A)
 2031	FORMAT(2X,'Source:  ',A,',  subnet:  ',A,
     1	' on DMO',<NDMO(Z)>I3)
C
 2045	FORMAT(/,2X,'Log   VCR   station  ',
     1	'name  pair    lag#     amp          comments'/)
C
 2050	FORMAT(2X,I2,'  _______',2X,A,1X,1A2,2X,
     1	'______   ____   _______   __________________')
 2051	FORMAT(2X,I2,'  _______',2X,A,1X,1A2,'+ ',
     1	'______   ____   _______   __________________')
C
 2060	FORMAT(/,2X,'Scan started (user,date,time):',40('_'),
     1	/2X,'Comments:',61('_'))
C
 2070	FORMAT('1 ',A,T55,A,1X,A)
C
 2075	FORMAT(/)
C
C		Finished with this pass
C
	IF (MORE_INP) GOTO 211	! Resume reading $EXPERIMENT group?
C
	WRITE (RECORD,1500) NSCN
 1500	FORMAT(I4,' scans')
	CALL PUTOUT('Output:  '//BLKFILE(1:LEN1(BLKFILE)))
	CALL PUTOUT(RECORD(1:LEN1(RECORD)))
	CALL PUTOUT('Listing: '//LISFILE(1:LEN1(LISFILE)))
	CALL PUTOUT('Log:     '//LOGFILE(1:LEN1(LOGFILE)))
	CLOSE (UNIT=OUTDAT, DISPOSE='KEEP')
	CLOSE (UNIT=LIST, DISPOSE='KEEP')
	CLOSE (UNIT=LOG, DISPOSE='KEEP')
	CALL EXIT
C
C		Errors.
C
  900	CALL MKB_ERR('Format error in input file')
	CALL EXIT
	END

	SUBROUTINE MKB_PUT(OUTC,LIST,BLOCK,NAME)
C-----------------------------------------------------------------------
C MAKECCR: write a block to the output file, swapping bytes for
C compatibility with the Mark-II Processor. Lower-case letters are
C forced to upper-case.
C ** Modified by SCU 1986 Oct 11 
C	- no byte-swapping for Block II
C	- no forcing to upper case (done in main prog where necessary)
C
C Arguments:
C
C OUTC (input, integer): unit number on which the output file is open.
C LIST (input, integer): unit number on which the listing file is open.
C BLOCK (input, character*1024): the block to be written; this is not
C	modified by MKB_PUT.
C NAME (input, character): header for listing.
C
C (25-Apr-1983) 
C-----------------------------------------------------------------------
	IMPLICIT NONE
	INTEGER  OUTC, IL, LIST, NB
	CHARACTER*1024 BLOCK, OUTPUT
	CHARACTER*(*) NAME
	CHARACTER*9 TODAY
	CHARACTER*8 NOW
	DATA NB /-1/
C
	OUTPUT = BLOCK
	WRITE (OUTC) OUTPUT
C
	CALL DATE(TODAY)
	CALL TIME(NOW)
	IF (MOD(NB+1,3).EQ.0) WRITE(LIST,100) NAME, TODAY, NOW
	NB = NB+1
	WRITE (LIST,110) NB
	DO IL=1,16
	    WRITE(LIST,120) IL, OUTPUT(64*(IL-1)+1:64*IL)
	END DO
	RETURN
C
  100	format('1',A,T60,A,1X,A)
  110	format(/' Block #',I4/)
  120	format(1X,I2,':    ',A)
	END

	SUBROUTINE MKB_GET(INDAT,RECORD,IER)
C-----------------------------------------------------------------------
C MAKECCR: read one record from the input file, skipping comments
C and blank lines, and converting to upper case. Leading blanks are
C removed and tabs are converted to spaces.
C
C Arguments:
C
C INDAT (input, integer): unit number on which the input file is open.
C RECORD (output, character): receives the next record.
C IER (output, integer): receives 0 if successful, non-zero otherwise
C	(eg, end of file).
C
C (28-Mar-1983)
C-----------------------------------------------------------------------
	IMPLICIT NONE
	INTEGER  I, INDAT, IER, L, L1
	CHARACTER*(*) RECORD
C
   10	READ (INDAT,'(A)',IOSTAT=IER) RECORD
	IF (IER.NE.0) THEN
	    RECORD = ' '
	ELSE
	    L = LEN(RECORD)
	    L1 = INDEX(RECORD,'!')
	    IF (L1.EQ.1) GOTO 10
	    IF (L1.GT.0) L = L1-1
	    DO I=1,L
		IF (RECORD(I:I).EQ.CHAR(9)) RECORD(I:I) = ' '
	    END DO
	    RECORD = RECORD(1:L)
	    IF (RECORD.EQ.' ') GOTO 10
	    I = 1
	    DO WHILE (RECORD(I:I).EQ.' ')
		I = I+1
	    END DO
	    RECORD = RECORD(I:)
	    CALL UPCASE(RECORD)
	END IF
	RETURN
	END

	SUBROUTINE MKB_PARSE( STRING, PARSEDARRAY, NWORDS)
C-----------------------------------------------------------------------
C Parse an input string into "words" delimited by a variable number
C of blanks.  The words (with no blanks) are returned in parsedarray,
C with nwords set to the total number of words recognized.
C
C If no input, return nwords = 0.
C-----------------------------------------------------------------------
	IMPLICIT INTEGER*4(A-Z)
	CHARACTER*(*) STRING, PARSEDARRAY(1)
	CHARACTER*120 INPUT
	input = string
	nwords = 0
	if (input .eq. ' ') then
		return
		endif
	l = len(input)			! # of characters remaining
C
C Discard initial blanks, if any.
C
100	if (input(1:1) .eq. ' ') then
		input=input(2:)
		l = l - 1
		if (l .le. 0) return
		go to 100
		endif
	eow = index(input,' ') - 1
	if (eow .le. 0) eow = len(input)
	nwords = nwords + 1
	parsedarray(nwords) = input(1:eow)
	input=input(min(len(input),eow+1):)
	l = l - eow
	go to 100
	end

	SUBROUTINE MKB_ERR (MESSAGE)
C-----------------------------------------------------------------------
C MAKECCR: report fatal error.
C
C Arguments:
C
C MESSAGE (input, character): message to user.
C
C (28-Mar-1983)
C-----------------------------------------------------------------------
	CHARACTER*(*) MESSAGE
	CHARACTER*132  RECORD
	COMMON        /M/ RECORD
	CALL PUTOUT('+++ERROR+++ '//MESSAGE)
	CALL PUTOUT(RECORD)
	CALL EXIT(44)
	END
