      PROGRAM MAPSTA
C-----------------------------------------------------------------------
C MAPSTAT:  Obtain Max, Min, Mean, and RMS of map.
C           Map must be a FITS-format image (on disk).
C
C Author:            R. C. Walker  (RCW)
C Language:          VAX-11 Fortran
C
C Modifications:
C Version 1.0:  1987 Mar 17 - adapted from MAPPLOT
C Version 1.1:  1987 Mar 23 - add LISTFILE, centroid (TJP).
C Version 1.2:  1988 Apr  4 - changes for Convex (TJP).
C Version 1.3:  1990 Jan 10 - remove MEM option (TJP).
C Version 1.4:  1992 Jan  9 - increase array size, standardize (TJP).
C Version 1.5:  1992 Jul  2 - change format statement (TJP).
C Version 1.6:  1993 Oct 30 - correct bug calling ANGLUN (TJP).
C-----------------------------------------------------------------------
C
C Constants
C
      CHARACTER*(*) VERSN
      INTEGER    INC,OUTC,NPARS,IDIM
      PARAMETER  (VERSN='1.6 - 1993 Oct 30')
      PARAMETER  (IDIM=1024*1024)
C                 array size
      PARAMETER  (INC=5,OUTC=6)
C                 unit numbers
      PARAMETER  (NPARS=26)
C                 number of parameters
C
C Variables.
C
      CHARACTER*255  PRDSN
      CHARACTER*128  MAPFIL,MAPOLD
      CHARACTER*40   XLABEL, YLABEL, XUNIT, YUNIT, BUNIT
      CHARACTER*128  TXTERR
      CHARACTER*8    MAPFMT
      DOUBLE PRECISION NAMES(NPARS),PARS(NPARS),ENDMRK
      REAL    MAPARR(IDIM), MAPMAX,MAPMIN
      REAL    RMAX, RMIN, SUM, SUMSQ, MEAN, RMS
      REAL    X1, X2, Y1, Y2, XFACT, YFACT, BFACT, BMAJ, BMIN, BAREA
      REAL    SUMX, SUMY
      LOGICAL PROPEN
      INTEGER I, K, L, N, M, IER, LENERR, MODE, MSLO, MSHI, MMAX, MMIN
      INTEGER NSLO, NSHI, NMAX, NMIN, NORM
      INTEGER PR, VLBOPE, LEN1
C
C Common blocks.
C
      INCLUDE 'mapcom.inc'
C
C Input parameters
C
      DATA    ENDMRK/ '/' /
      DATA    NAMES / 'MAPfile', 9*' ',
     1                'FORmat',
     2                2*'Xrange', 2*'Yrange',
     3                'EXit',
     4                'Listfile', 9*' '/
      DATA    PARS  / 'MAP',9*' ',
     1                'FITS', 
     2                2*0D0, 2*0D0 ,
     3                -1D0 ,
     4               10*' '/
C
C Format statements.
C
  600 FORMAT (' MAPSTAT calculates statistics of FITS-format maps'/
     1        ' (Version ',A,')'//
     2        ' Control parameters (end with /):')
  660 FORMAT (/1X,'File: ',A/
     1        5X,'Source: ',A/
     2        5X,'Type: ',A,'   Created by: ',A/
     3        5X,'Telescope: ',2A/
     4        5X,'Observer: ',A/
     5        5X,'Size in pixels (x,y): ',I5,',',I5/
     6        5X,'Center at ',F7.1,',',F7.1/
     7        5X,'Maximum of whole map ',1PG12.3,'  in cell',2I5/
     8        5X,'Minimum of whole map ',1PG12.3,'  in cell',2I5)
  750 FORMAT (5X,'Beam:  FWHM ',F8.3,' X ',F8.3,1X,A/
     1        8X,'P.A.',F7.1,' Degrees. ',
     2         '  Area:',F10.3,' Square pixels')
  760 FORMAT (1X)
  780 FORMAT('  ',A,' Axis interval - Requested pixels: ',I4,' to ',I4,
     1       /, '    Axis type: ',A/
     2       '    Range: ',F7.2,' to ',F7.2,1X,A)
  800 FORMAT('  Number of cells:   ',I10/
     1       '  Maximum in window: ',1P,G13.4,'  at cell ',2I5/
     2       '  Minimum in window: ',G13.4,'  at cell ',2I5/
     3       '  Mean in window:    ',G13.4/
     4       '  RMS in window:     ',G13.4/
     5       '  Centroid:          ',0P,2F10.3)
C-----------------------------------------------------------------------
C
C Start.
C
      WRITE (OUTC,600) VERSN
      PROPEN = .FALSE.
      MAPOLD = '*** UNSPECIFIED ***'
C
C Read control parameters.
C
 1000 MODE = 0
      CALL KEYIN(NAMES,PARS,NPARS,ENDMRK,MODE,INC,OUTC)
C     -- end-of-file or EXIT ?
      IF (MODE.EQ.1) GOTO 2000
      IF (PARS(16).NE.-1D0) GOTO 2000
C
C     -- MAPFILE
      WRITE (MAPFIL,'(10A8)') (PARS(I),I=1,10)
C     -- FORMAT
      WRITE (MAPFMT,'(A8)') PARS(11)
      CALL UPCASE(MAPFMT)
C     -- XRANGE
      MSLO = PARS(12)
      MSHI = PARS(13)
C     -- YRANGE
      NSLO = PARS(14)
      NSHI = PARS(15)
C     -- LISTFILE
      WRITE (PRDSN,'(10A8)') (PARS(I),I=17,26)
C
C Open listing file.
C
      IF (.NOT.PROPEN) THEN
        IF (PRDSN.EQ.' ') THEN
          PR = OUTC
        ELSE
          PR = 1
          IER = VLBOPE(PR, PRDSN, 'TEXT', 'NEW', PRDSN)
          IF (IER.NE.1) GOTO 1000
        END IF
        PROPEN = .TRUE.
      END IF
C
C Read map from file MAPFIL (if necessary), locate maximum and minimum
C and type header.
C
      IF (MAPFIL.NE.MAPOLD) THEN
          MAPOLD = MAPFIL
          IF (MAPFMT.EQ.'FITS') THEN
              IER = VLBOPE(2, MAPFIL, 'FITS', 'OLD', MAPFIL)
              IF (IER.NE.1) GOTO 3000
              CALL FITIN(2,MAPARR,IDIM,IER,TXTERR)
          ELSE
              IER = 1
              TXTERR = 'Unrecognized value for parameter FORMAT = "'//
     1                     MAPFMT//'"'
          END IF
          IF (IER.NE.0) THEN
              LENERR = LEN1(TXTERR)
              CALL PUTOUT(TXTERR(1:LENERR))
              GOTO 3000
          END IF
          CLOSE (UNIT=2)
C
C Locate maximum, minimum.
C
          MAPMAX = -1E37
          MAPMIN =  1E37
          DO 220 N=1,MPSIZ(2)
              K = (N-1)*MPSIZ(1)
              DO 210 M=1,MPSIZ(1)
                  IF (MAPARR(K+M).GT.MAPMAX) THEN
                      MAPMAX = MAPARR(K+M)
                      NMAX = N
                      MMAX = M
                  END IF
                  IF (MAPARR(K+M).LT.MAPMIN) THEN
                      MAPMIN = MAPARR(K+M)
                      NMIN = N
                      MMIN = M
                  END IF
  210         CONTINUE
  220     CONTINUE
          WRITE (PR,660) MAPFIL(1:LEN1(MAPFIL)),MPOBJ,MPTYP,MPCRT,
     1                  MPTEL,MPINS,MPOBS,MPSIZ(1),MPSIZ(2),MPRP(1),
     2                  MPRP(2),MAPMAX,MMAX,NMAX, MAPMIN,MMIN,NMIN
C
          BAREA = 1.132 * MPBMJ * MPBMN / (MPIN(1)*MPIN(2))
          CALL ANGLUN (REAL(MPBMJ), BUNIT, BFACT)
          BMAJ = MPBMJ * BFACT
          BMIN = MPBMN * BFACT
          WRITE (PR,750) BMAJ, BMIN, BUNIT, MPBPA, BAREA
      END IF
C
C
C Determine the ranges of the axes. The axis ranges are:
C MSLO to MSHI (horizontal) and NSLO to NSHI (vertical) in
C pixel numbers, and X1 to X2 (left to right) and Y1 to Y2
C (bottom to top) in real units.  We arrange for the 
C "real units" of the x-axis to increase from left to 
C right (except for RA-like quantities, which increase
C from right to left) and for the "real units" of the
C y-axis to increase from bottom to top.
C
      IF (MSLO.EQ.0 .AND. MSHI.EQ.0) THEN
          MSLO = 1
          MSHI = MPSIZ(1)
      ELSE IF (MSLO.LT.1 .OR. MSLO.GE.MSHI .OR. MSHI.GT.MPSIZ(1)) THEN
          WRITE (OUTC,'(A,I5)') ' +++ERROR+++ Requested XRANGE is '//
     1            'outside map pixel range of 1,',MPSIZ(1)
          GOTO 1000
      END IF
      IF (NSLO.EQ.0 .AND. NSHI.EQ.0) THEN
          NSLO = 1
          NSHI = MPSIZ(2)
      ELSE IF (NSLO.LT.1 .OR. NSLO.GE.NSHI .OR. NSHI.GT.MPSIZ(2)) THEN
          WRITE (OUTC,'(A,I5)') ' +++ERROR+++ Requested YRANGE is '//
     1            'outside map pixel range of 1,',MPSIZ(2)
          GOTO 1000
      END IF
      X1 = (MSLO-MPRP(1))*MPIN(1)
      X2 = (MSHI-MPRP(1))*MPIN(1)
      Y1 = (NSHI-MPRP(2))*MPIN(2)
      Y2 = (NSLO-MPRP(2))*MPIN(2)
      IF (MPCTY(1).EQ.'RA' .OR. MPCTY(1)(1:3).EQ.'RA-' .OR.
     1          MPCTY(1).EQ.'LL') THEN
          XLABEL = 'Relative R.A.'
          CALL ANGLUN(ABS(X2-X1),XUNIT,XFACT)
          MPRV(1) = 0.0
          IF (X1.LT.X2) CALL SWAP(X1,X2)
      ELSE
          XLABEL = MPCTY(1)
          XUNIT = ' '
          XFACT = 1.0
          IF (X1.GT.X2) CALL SWAP(X1,X2)
      END IF
      IF (MPCTY(2).EQ.'DEC' .OR. MPCTY(2)(1:4).EQ.'DEC-' .OR.
     1          MPCTY(2).EQ.'MM') THEN
          YLABEL = 'Relative Decl.'
          CALL ANGLUN(ABS(Y2-Y1),YUNIT,YFACT)
          MPRV(2) = 0.0
      ELSE
          YLABEL = MPCTY(2)
          YUNIT = ' '
          YFACT = 1.0
      END IF
      IF (Y1.GT.Y2) CALL SWAP(Y1,Y2)
      X1 = (X1+MPRV(1))*XFACT
      X2 = (X2+MPRV(1))*XFACT
      Y1 = (Y1+MPRV(2))*YFACT
      Y2 = (Y2+MPRV(2))*YFACT
C
      WRITE (PR, 760)
      WRITE (PR, 780) 'X', MSLO, MSHI, XLABEL, X1, X2, XUNIT
      WRITE (PR, 780) 'Y', NSLO, NSHI, YLABEL, Y1, Y2, YUNIT
C
C     Now get the data from map.
C
      RMAX = -1.E37
      RMIN =  1.E37
      SUM  =  0.0
      SUMSQ = 0.0
      SUMX = 0.0
      SUMY = 0.0
      NORM = 0
      DO 240 N = NSLO, NSHI
         K = (N-1)*MPSIZ(1)
         DO 230 M = MSLO, MSHI
            L = K + M
            IF (MAPARR(L).GT.RMAX) THEN
               RMAX = MAPARR(L)
               MMAX = M
               NMAX = N
            END IF
            IF (MAPARR(L).LT.RMIN) THEN
               RMIN = MAPARR(L)
               MMIN = M
               NMIN = N
            END IF
            SUM = SUM + MAPARR(L)
            SUMSQ = SUMSQ + MAPARR(L)**2
            SUMX = SUMX + M*MAPARR(L)
            SUMY = SUMY + N*MAPARR(L)
            NORM = NORM + 1
  230    CONTINUE
  240 CONTINUE
C
C     Calculate and write statistics
C
      MEAN = SUM / NORM
      RMS = SQRT ( (SUMSQ/NORM) - MEAN**2)
      SUMX = SUMX/SUM
      SUMY = SUMY/SUM
      WRITE (PR, 800)  NORM, RMAX, MMAX, NMAX, RMIN, MMIN, NMIN,
     1                 MEAN, RMS, SUMX, SUMY
C
C     Get next input
C
      GO TO 1000
C
C Error exits.
C
 3000 CLOSE (UNIT=2,IOSTAT=IER)
      MAPOLD = '*** UNSPECIFIED ***'
      GOTO 1000
C
C Exit from program.
C
 2000 CONTINUE
C-----------------------------------------------------------------------
      END
