      PROGRAM MERGE
C-----------------------------------------------------------------------
C Merge PHASOR, RESTORE and/or MERGE files into standard MERGE format.
C
C Version 1.0:  1979 May  1 - T.J.Pearson (VAX-11 Fortran).
C Version 2.0:  1980 Sep  2
C Version 3.0:  1981 Jun  9 - 15 stations, wild-card input (TJP).
C Version 3.1:  1981 Jul 21 - correct bug in IGNORE (TJP).
C Version 4.0:  1981 Dec  8 - merge MERGE-format files (TJP).
C Version 4.1:  1982 Feb 26 - new closure-phase options (TJP).
C Version 4.2:  1982 Jul  5 - correct bug in MRG_MAP (array dimn) (TJP).
C Version 5.0:  1982 Sep 12 - parameter CHANNEL for channel selection.
C Version 5.1:  1982 Sep 27 (D.L. Meier)
C        Add parameters FREQUENCY and BANDWIDTH for frequency
C        selection.
C Version 5.2:  1982 Oct 7 (D.L. Meier)
C        Add parameter NEWSTATIONS to allow possible renaming
C        of stations. Corrected bug that allowed 16 character
C        station names to be read from phasor file but only 8
C        characters in comparison list
C Version 5.3:  1982 Dec 21
C        Increase array size; add bandwidth parameter; add
C        SOURCE="ALL" kludge; test file type automatically (TJP).
C Version 5.4:  1984 Oct 30 - increase size of history storage;
C        apply phase corrections to phasor files for clock
C        offsets (suppressed by NOFIXPHS); fix bug in NEWSTAT for merge
C         files (TJP).
C Version 6.0*: 1984 Nov 10 - add ability to read RESTORE files (TJP).
C Version 6.0: 1985 Oct 1 - official version of 6.0 (TJP).
C Version 6.1: 1986 Feb 3 - remove BAND/CHANNEL check for RESTORE (TJP).
C Version 6.2: 1986 Mar 16 - correct phase error for RESTORE (TJP).
C Version 6.3: 1986 Aug 27 - increase to 20 stations (TJP).
C Version 7.0: 1988 Jun  1 - Convex version, MERGE files only (TJP).
C Version 7.1: 1990 Oct 15 - Fix bug: lower case stations ot recognized.
C Version 7.2: 1994 Jan 03 - Fix bug: MAXL=4*MAXB fixed to 4*MAXB+2*MAXT
C-----------------------------------------------------------------------
      INCLUDE 'merge64.inc'
      CHARACTER*(*) VERSN
      PARAMETER (VERSN='7.0 - 1988 Jun 1')
      INTEGER   INC,OUTC,INDAT,OUTDAT,MAXFIL
      PARAMETER (INC=5)         ! terminal input unit
      PARAMETER (OUTC=6)        ! terminal output unit
      PARAMETER (INDAT=10)      ! Phasor/Merge input unit
      PARAMETER (OUTDAT=12)     ! Merge output unit
      PARAMETER (MAXFIL=100)  ! Maximum number of input files
C
      CHARACTER*255 INDSN(MAXFIL),OUTDSN
      INTEGER       J, MODE, NFILE, NF, LDSN
      INTEGER       GETIN, LEN1, VLBOPE
C-----------------------------------------------------------------------
C
C File names.
C
      CALL PUTOUT('MERGE merges MERGE-format files')
      CALL PUTOUT('(Version '//VERSN//')')
      CALL PUTOUT(' ')
      IER = GETIN(OUTDSN, 'Output (MERGE) file name: ', L)
      IF (IER.NE.1) STOP
      IER = VLBOPE(OUTDAT, OUTDSN, 'MERGE', 'NEW', OUTDSN)
      IF (IER.NE.1) STOP
C
C Set up Header information.
C
      MODE = 0
      CALL MRG_PARAM(MODE,INC,OUTC,VERSN)
C
C Read PHASOR/MERGE/RESTORE files.
C
      CALL PUTOUT(' ')
      CALL PUTOUT('List input file names; end list with with blank '//
     1            'line or end-of-file.')
      NFILE = 0
      CALL MRG_FFIND(NF,INDSN,MAXFIL,INC,OUTC)
      DO J=1,NF
          LDSN = LEN1(INDSN(J))
          IER = VLBOPE(INDAT, INDSN(J), 'MERGE', 'OLD', INDSN(J))
          IF (IER.NE.1) THEN
              CALL PUTOUT('+++ERROR+++ Cannot read file '//
     1                    INDSN(J)(1:LDSN))
          ELSE
              CALL MRG_MERGE(INDSN(J),INDAT,OUTC)
              CLOSE (UNIT=INDAT)
          END IF
      END DO
C
C Write new dataset.
C
      IF (NFR.GT.0) THEN
          CALL MRG_OUTPUT(OUTDSN,OUTDAT,OUTC)
          CLOSE (UNIT=OUTDAT,DISPOSE='KEEP')
      ELSE
          CLOSE (UNIT=OUTDAT,DISPOSE='DELETE')
          CALL ERROR('No data found for requested source')
      END IF
C
      CALL REPORT(OUTC)
      END

      SUBROUTINE MRG_MERGE(INFILE,IN,PR)
C-----------------------------------------------------------------------
C MERGE: read in a MERGE-format file.
C
C Author:               T.J. Pearson (TJP)
C Language:             VAX-11 Fortran
C
C Modifications:
C   1984 Nov 12 - use new INCLUDE file (TJP).
C
C Arguments:
C   INFILE (input, character):  file name of input file.
C   IN (input, integer): unit number for input file.
C   PR (input, integer): unit number for output messages.
C-----------------------------------------------------------------------
      INCLUDE   'merge61.inc'
      INCLUDE   'merge62.inc'
      INCLUDE   'merge63.inc'
      INCLUDE   'merge64.inc'
C
      INTEGER   MAXL
      PARAMETER (MAXL=4*MAXB+2*MAXT)
C
      CHARACTER*(*) INFILE
C
      INTEGER ISMAP(MAXS)
      INTEGER IBMAP(MAXB), IBSGN(MAXB)
      INTEGER ITMAP(MAXT), ITSGN(MAXT)
      INTEGER UT, PR
      REAL*4  DATAIN(MAXL)
C
C               Input header variables
C
      INTEGER NVERIN,NHISTIN,NHEADIN,NSIN,NBIN,NTIN,
     1          GYEAR,GUTF,LBIN(MAXB),LTIN(MAXT)
      REAL*8  GRA50,GDEC50,GRAP,GDECP,GFLUX,
     1          GX(MAXS),GY(MAXS),GZ(MAXS),GGST,GFREQ,
     2          GINTEG,GAVG,GBW
      CHARACTER*16  GSRC
      CHARACTER*8   GSTN(MAXS)
      CHARACTER*80  GHIST(MAXHIS)
C
C               Read input header
C
      READ (IN,END=210,ERR=210) NVERIN,NHISTIN,NHEADIN,NSIN,NBIN,
     1          NTIN,LIN,KIN
      KFLAG = KIN
      IF (NBIN.GT.0) READ (IN,END=210,ERR=210) (LBIN(IB),IB=1,NBIN)
      IF (NTIN.GT.0) READ (IN,END=210,ERR=210) (LTIN(IT),IT=1,NTIN)
      IF (KLOSE.EQ.3) NTIN = 0
      IF (NHISTIN.GT.MAXHIS) THEN
            WRITE (PR,1030)
            GOTO 210
      END IF
      DO IHIST=1,NHISTIN
            READ (IN,END=210,ERR=210) GHIST(IHIST)
      END DO
      READ (IN,END=210,ERR=210) GSRC,GRA50,GDEC50,GRAP,GDECP,GFLUX
      CALL UPCASE(GSRC)
      DO IS=1,NSIN
            READ (IN,END=210,ERR=210) GSTN(IS),GX(IS),GY(IS),GZ(IS)
            CALL UPCASE(GSTN(IS))
      END DO
      READ (IN,END=210,ERR=210) GYEAR,GUTF,GGST,GFREQ,GINTEG,GAVG,GBW
C
C               Check source: do we want it?
C
      IF (HSRC.EQ.' ') HSRC = GSRC
      IF ((GSRC.NE.HSRC) .AND. (GSRC.NE.HALIAS) .AND.
     1          (HSRC.NE.'ALL')) GOTO 410
C
C               Determine input/output mapping
C
      NWANT = MRG_MAP(GSTN,NSIN, LBIN,NBIN, LTIN,NTIN,
     1              OSTATN,NS,   LABELB,NB, LABELT,NT,
     2              ISMAP, IBMAP, IBSGN, ITMAP, ITSGN)
      IF (NWANT.LT.0) GOTO 420
C
C               Update History
C
      CALL MRG_HISTORY('- Input file: '//INFILE,PR)
      WRITE (PR,1110) (GSTN(M),ISMAP(M),M=1,NSIN)
      WRITE (PR,1120) (IBMAP(M)*IBSGN(M),M=1,NBIN)
      WRITE (PR,1130) (ITMAP(M)*ITSGN(M),M=1,NTIN)
      DO IHIST=1,NHISTIN
            CALL MRG_HISTORY('- '//GHIST(IHIST),0)
      END DO
C
C               Update the header
C
      IF ( HRA50.EQ.0D0) HRA50  = GRA50
      IF (HDEC50.EQ.0D0) HDEC50 = GDEC50
      IF (  HRAP.EQ.0D0) HRAP   = GRAP
      IF ( HDECP.EQ.0D0) HDECP  = GDECP
      IF ( HFLUX.EQ.0D0) HFLUX  = GFLUX
      IF ( HYEAR.LE.0  ) HYEAR  = GYEAR
      IF (  HUTF.LE.0  ) HUTF   = GUTF
      IF (  HGST.EQ.0D0) HGST   = GGST
      IF ( HFREQ.EQ.0D0) HFREQ  = GFREQ
      IF (HINTEG.EQ.0D0) HINTEG = GINTEG
      IF (  HAVG.EQ.0D0) HAVG   = GAVG
      IF (   HBW.EQ.0D0) HBW    = GBW
      DO IS=1,NSIN
            JS = ISMAP(IS)
            IF (JS.GT.0) THEN
                IF (HX(JS).EQ.0D0) HX(JS) = GX(IS)
                IF (HY(JS).EQ.0D0) HY(JS) = GY(IS)
                IF (HZ(JS).EQ.0D0) HZ(JS) = GZ(IS)
            END IF
      END DO
C
C               Read the data records and merge
C               (note that an output record will be produced for
C               each UT in the input, even if there are no data for
C               that UT)
C
  300 READ (IN,ERR=210,END=320) UT,(DATAIN(K),K=1,4*NBIN+2*NTIN)
      IF (UT.LT.0) GOTO 320
      I0 = MRG_LOC(UT)
      DO 301 IBIN=1,NBIN
            IF (IBMAP(IBIN).EQ.0) GOTO 301      ! reject baseline?
            IW = I0 + 4*(IBMAP(IBIN)-1)
            IV = 4*(IBIN-1) + 1
            IF (DATAIN(IV+1) .GT. 0.0) THEN     ! amplitude
                STORE(IW) = DATAIN(IV)
                STORE(IW+1) = DATAIN(IV+1)
            END IF
            IF (DATAIN(IV+3) .GT. 0.0) THEN     ! phase
                STORE(IW+2) = DATAIN(IV+2)*IBSGN(IBIN)
                STORE(IW+3) = DATAIN(IV+3)
            END IF
301     CONTINUE
      IF (KLOSE.NE.2) GOTO 300
      DO 302 ITIN=1,NTIN
            IF (ITMAP(ITIN).EQ.0) GOTO 302
            IW = I0 + 4*NB + 2*(ITMAP(ITIN)-1)
            IV = 4*NBIN + 2*(ITIN-1) + 1
            IF (DATAIN(IV+1).GT.0.0) THEN
                STORE(IW) = DATAIN(IV)*ITSGN(ITIN)
                STORE(IW+1) = DATAIN(IV+1)
            END IF
302     CONTINUE
      GOTO 300
C
C               End of file
C
  320 CONTINUE
      RETURN
C
C               Rejection
C
  410 WRITE (PR,1010) GSRC
      GOTO 320
  420 WRITE (PR,1020) (GSTN(IS),IS=1,NSIN)
      GOTO 320
C
C               Errors
C
  210 WRITE (PR,1000)
      RETURN
C
C               Formats
C
 1000 FORMAT(' ++WARNING++ Error reading input Merge file')
 1010 FORMAT(' ++WARNING++ Merge file rejected: wrong source - ',A)
 1020 FORMAT(' ++WARNING++ Merge file rejected: wrong stations -'/
     1         (1X,5A))
 1030 FORMAT(' ++WARNING++ ',
     1          'Too many history records in input Merge file')
 1110 FORMAT(' Station',T20,'Sequence number'/(1X,A,T25,I4))
 1120 FORMAT(' Baseline map:'/(1X,15I4))
 1130 FORMAT(' Triangle map:'/(1X,15I4))
      END

      INTEGER FUNCTION MRG_MAP(STNIN,NSIN, LBIN,NBIN, LTIN,NTIN,
     1                    STN,NS,   LABELB,NB, LABELT,NT,
     2                    ISMAP, IBMAP, IBSGN, ITMAP, ITSGN)
C-----------------------------------------------------------------------
C       VLBMERGE: establish mapping between input Merge file and
C       output MERGE file.
C       Input file:
C               NSIN stations, named STNIN(ISIN), ISIN=1,NSIN
C               NBIN baselines, labelled LBIN(IBIN), IBIN=1,NBIN
C               NTIN triangles, labelled LTIN(ITIN), ITIN=1,NTIN
C       Output file:
C               NS   stations, named STN(IS), IS=1,NS
C               NB   baselines, labelled LABELB(IB), IB=1,NB
C               NT   traingles, labelled LABELT(IT), IT=1,NT
C       Baselines labels are 100*1S1 + IS2 (IS1<IS2)
C       Triangle labels are 10000*IS1 + 100*IS2 + IS3 (IS1<IS2<IS3)
C
C       Problem: to determine where each input baseline/triangle belongs
C       in the output file (if it is wanted at all), and whether the
C       phase should be inverted. We need to compute
C               ISMAP(ISIN), ISIN=1,NSIN : output station number of
C                       input station ISIN (0 if not wanted)
C               IBMAP(IBIN), IBIN=1,NBIN : output baseline number of
C                       input baseline IBIN (0 if not wanted)
C               IBSGN(IBIN), IBIN=1,NBIN : +/-1, indicating whether
C                       visibility phase must be inverted
C               ITMAP(ITIN), ITIN=1,NTIN : output triangle nunber of
C                       input triangle ITIN (0 if not wanted)
C               ITSGN(ITIN), ITIN=1,NTIN : +/-1, indicating whether
C                       closure phase must be inverted
C-----------------------------------------------------------------------
      CHARACTER*8  STNIN(NSIN), STN(NS)
      INTEGER      LBIN(NBIN), LTIN(*), LABELB(NB), LABELT(*),
     1               ISMAP(NSIN), IBMAP(NBIN), IBSGN(NBIN), ITMAP(*),
     2               ITSGN(*)
      INTEGER ISS(3)
C
C               Check stations
C
      DO ISIN=1,NSIN
            ISMAP(ISIN) = 0
            DO IS=1,NS
                IF (STNIN(ISIN).EQ.STN(IS)) THEN
                    ISMAP(ISIN) = IS
                END IF
            END DO
      END DO
C
C               Check baselines
C
      NWANT = 0
      DO IBIN=1,NBIN
            I1 = LBIN(IBIN)/100
            I2 = LBIN(IBIN) - 100*I1
            ISS(1) = ISMAP(I1)
            ISS(2) = ISMAP(I2)
            CALL MRG_ISORT(ISS,2,IBSGN(IBIN))
            LBTEST = 100*ISS(1) + ISS(2)
            IBMAP(IBIN) = 0
            DO IB=1,NB
                IF (LBTEST.EQ.LABELB(IB)) THEN
                    IBMAP(IBIN) = IB
                    NWANT = NWANT+1
                END IF
            END DO
      END DO
C
C               Check triangles
C
      DO ITIN=1,NTIN
            I1 = LTIN(ITIN)/10000
            I2 = LTIN(ITIN)/100 - 100*I1
            I3 = LTIN(ITIN) - 10000*I1 - 100*I2
            ISS(1) = ISMAP(I1)
            ISS(2) = ISMAP(I2)
            ISS(3) = ISMAP(I3)
            CALL MRG_ISORT(ISS,3,ITSGN(ITIN))
            LTTEST = 10000*ISS(1) + 100*ISS(2) + ISS(3)
            ITMAP(ITIN) = 0
            DO IT=1,NT
                IF (LTTEST.EQ.LABELT(IT)) THEN
                    ITMAP(ITIN) = IT
                    NWANT = NWANT+1
                END IF
            END DO
      END DO
C
      MRG_MAP = NWANT
      RETURN
C
      END

      SUBROUTINE MRG_ISORT(IN,N,PARITY)
C-----------------------------------------------------------------------
C       VLBMERGE: Sort array IN(I), I=1,N, into increasing order.
C       Return PARITY=+1 or -1 as the permutation applied
C       to IN is odd or even. Shell sort.
C-----------------------------------------------------------------------
      INTEGER IN(N)
      INTEGER PARITY
C
      PARITY = +1
      IGAP = N
    5 IF(IGAP.LE.1) RETURN
      IGAP=IGAP/2
      IMAX=N-IGAP
   10       IEX=0
            DO I=1,IMAX
            IF(IN(I).GT.IN(I+IGAP)) THEN
                ITEMP = IN(I)
                IN(I) = IN(I+IGAP)
                IN(I+IGAP) = ITEMP
                PARITY = -PARITY
                IEX = IEX+1
            END IF
            END DO
            IF(IEX.GT.0) GOTO 10
      GOTO 5
      END
