      PROGRAM MGFITS
C-----------------------------------------------------------------------
C MERGEFITS: to convert a standard MERGE-format VLB visibility file
C into AIPS-compatible UVFITS format.
C
C Limitations:
C   Up to 40 stations are accepted.
C   Deleted data are not entered in output file.
C   Closure phases, if present, are not entered in output file.
C   UVW coordinates in the output file are of date of observation,
C      abd are not rotated to B1950.0 according to the AIPS convention.
C
C Author: T.J. Pearson, Caltech.
C
C Version 1.0:  1984 Mar 19.
C Version 2.0:  1986 Apr 22 - use FITS format for antenna table.
C Version 2.1:  1988 Jul  9 - higher precision in time tags; add weight
C                             and TAI-UTC parameters; correct time
C                             calculation.
C Version 2.2:  1988 Jul 20 - change sign convention for phase, uvw.
C Version 2.3:  1988 Nov 22 - force IMNAME etc to upper case.
C Version 2.4:  1989 Feb 14 - correct error in SORT ORDER; reduce
C                             amount of history.
C Version 2.5:  1990 Feb  6 - put additional info in "CIT" keywords.
C Version 2.6:  1990 May 22 - use correct TAI-UTC by default; record
C                             B1950.0 position in header.
C Version 2.7:  1991 Feb 13 - increase to 30 stations.
C Version 2.8:  1991 Apr 22 - increase to 40 stations.
C Version 2.9:  1991 Aug 11 - add WT keywords for fitsmerge; default
C                             WTPOWER=1.
C Version 2.10: 1993 Apr 16 - adjust default WTFACT for uncalib data.
C version 2.11: 1993 Dec  1 - change UU-L to UU, etc.
C Version 3.0:  1994 Mar 24 - do not round weights.
C Version 4.0:  1995 Feb 21 - add BITPIX selection.
C Version 4.1:  1995 Sep 25 - remove unprintable history.
C-----------------------------------------------------------------------
C
C Constants.
C
      CHARACTER*(*) VERSN, DASH
      PARAMETER (VERSN='4.1 - 1995 Sep 25')
      PARAMETER (DASH='---------------------------------------------')
      INTEGER   INC,OUTC,INDAT,MAXS,MAXB,MAXT,MAXWPB,NPARS
      PARAMETER (INC=5,OUTC=6,INDAT=8)
C     -- maximum number of stations
      PARAMETER (MAXS=40)
      PARAMETER (MAXB=(MAXS-1)*MAXS/2)
      PARAMETER (MAXT=MAXB*(MAXS-2)/3)
C     -- maximum words per baseline
      PARAMETER (MAXWPB=7)
      PARAMETER (NPARS=41)
      DOUBLE PRECISION    PI,SIDR,VELC,RPDEG
      PARAMETER (PI=3.141592653589793D0)
C     -- radians of ST per sec of UT
      PARAMETER (SIDR=(PI/2592000.0D0)*1.0027379093D0)
C     -- speed of light
      PARAMETER (VELC=2.997925D8)
      PARAMETER (RPDEG=PI/180D0)
C
C Variables.
C
      CHARACTER*2      SORT
      CHARACTER*8      BUF,DATMAP,DATOBS,STNAME(MAXS)
      CHARACTER*16     SOURCE,OBSERV,IMCLAS
      CHARACTER*80     HISTRY,IMNAME,NEWHIS
      CHARACTER*128    INDSN,OUTDSN
      CHARACTER*(MAXS) TELESC
      CHARACTER*17     BNAME(MAXB)
      CHARACTER*24     STRING
      INTEGER   I,IB,ID,IER,IF,IMON,IP,IS,IS1,IS2,IS3,IS4,IUT,IY,IZ,J,
     1          JD,LOUT,MONTH,NB,NDWPB,NS,NT,NVER,STOKES,USERNO,IMSEQ,
     2          MODE,IUNITS,IUTP, DELTAT
      INTEGER   REC1(20)
      INTEGER   BCOUNT(MAXB),NB1(MAXB),NB2(MAXB)
      INTEGER   NC1(MAXT),NC2(MAXT),NC3(MAXT)
      INTEGER*2 I2DATA(9)
      INTEGER   I4DATA(9)
      REAL      R4DATA(9)
      INTEGER   BITPIX
      REAL      A,AMAX,BSCALE,DUM1,DUM3,PHS,U,UMAX,UVWSCL,
     1          V,VMAX,W,WMAX,WTSCAL,WTPOWR, WTFACT, AERR, FLAG
      REAL      WT, WTMIN, WTMAX
      REAL      REC(MAXWPB*MAXB)
      DOUBLE PRECISION    AVG,BW,BX,BY,COSDEC,DEC,FLUX,FREQ,
     1                    GASTM,RA,SINDEC,TIHA,TINTG,WAVE,REFDAT,T
      DOUBLE PRECISION    FIDMJD, RA50, DEC50
      DOUBLE PRECISION    BOFF(MAXB),BXY(MAXB),BZ(MAXB)
      DOUBLE PRECISION    STX(MAXS),STY(MAXS),STZ(MAXS)
      DOUBLE PRECISION    PARS(NPARS), VALS(NPARS), ENDMRK
      COMMON    /PARMS/ STNAME,STX,STY,STZ,SOURCE,RA,DEC,GASTM,
     1                  FREQ,FLUX,TINTG,AVG,BW
      COMMON    /PARM2/ RA50,DEC50
      COMMON    /ORDER/ NS,NB,NT,NB1,NB2,NC1,NC2,NC3,NVER,NDWPB
C
C Functions called.
C
      CHARACTER*1 STNABR
      INTEGER   LEN1, VLBOPE
      DOUBLE PRECISION SLA_DAT
C
C KEYIN parameters.
C
      DATA PARS / 'INput', 9*' ',
     1            'OUtput', 9*' ',
     2            'STokes',
     3            'OBserver', ' ',
     4            'Userno',
     5            'IMName', 9*' ',
     6            'IMClass', ' ',
     7            'IMSeq',
     8            'WTFactor', 'WTPower',
     9            'TAI-UTC',
     A            'BITPIX' /
      DATA VALS / '***UNSET', 9*' ',
     1            10*' ',
     2            -2D0,
     3            2*' ',
     4            0D0,
     5            10*' ',
     6            2*' ',
     7            0D0,
     8            1D0, 2D0,
     9            0D0,
     A            -32D0 /
      DATA ENDMRK / '/' /
C-----------------------------------------------------------------------
 1000 FORMAT(I2.2,'/',I2.2,'/',I2.2)
 1010 FORMAT(' MERGEFITS converts MERGE-format VLB visibility files'/
     1       ' into AIPS-compatible UVFITS format'/
     2       ' (Version ',A,')'//
     3       ' Control parameters (INPUT, OUTPUT, ...):')
 1020 FORMAT('- Umax = ',F10.2,' Vmax = ',F10.2,' Wmax =',F10.2,' Mwl')
 1030 FORMAT('- Maximum amplitude = ',F12.6)
 1040 FORMAT('- Number of visibilities:',I8)
 1051 FORMAT('- TAI-UTC =',I4,' s')
 1052 FORMAT('- WTFactor =',F10.3,'   WTPower =',F8.1)
 1045 FORMAT('AIPS WTSCAL= ',E17.10)
 1090 FORMAT(1X,I6,' data groups written in file ',A)
 1100 FORMAT(' Minimum weight: ',1PG12.4/' Maximum weight: ',1PG12.4)
C-----------------------------------------------------------------------
C
C Today's date in FITS format; new history record.
C
      CALL IDATE(IMON,ID,IY)
      WRITE (DATMAP,1000) ID,IMON,IY
      CALL MKHIST('MERGEFITS', VERSN, NEWHIS)
C
C Introduction.
C
      WRITE (OUTC,1010) VERSN
C
C Read parameters.
C
      MODE = 0
      CALL KEYIN(PARS,VALS,NPARS,ENDMRK,MODE,INC,OUTC)
      WRITE (INDSN,'(10A8)') (VALS(I),I=1,10)
      WRITE (OUTDSN,'(10A8)') (VALS(I),I=11,20)
      STOKES = VALS(21)
      WRITE (OBSERV,'(2A8)') VALS(22),VALS(23)
      IF (OBSERV.EQ.' ') CALL USERNM(OBSERV(1:12))
      USERNO = VALS(24)
      WRITE (IMNAME,'(10A8)') (VALS(I),I=25,34)
      WRITE (IMCLAS,'(2A8)') VALS(35),VALS(36)
      IMSEQ  = VALS(37)
C     -- weighting parameters
      WTFACT = VALS(38)
      WTPOWR = VALS(39)
C     -- TAI-UTC correction: zero => use SLALIB calculation
      DELTAT = NINT(VALS(40))
      BITPIX = NINT(VALS(41))
      IF (BITPIX.NE.16 .AND. BITPIX.NE.32 .AND. BITPIX.NE.-32) THEN
         CALL ERROR('Illegal BITPIX (must be 16, 32, or -32)')
      END IF
C
C Open input Merge file.
C
      IER = VLBOPE(INDAT, INDSN, 'MERGE', 'OLD', INDSN)
      IF (IER.NE.1) CALL ERROR('Cannot open input MERGE file.')
C
C Parse input file name (assuming VMS syntax) to get IMNAME etc.
C
      IS1 = INDEX(INDSN,']')
      IS2 = INDEX(INDSN(IS1+1:),'.')
      IS3 = INDEX(INDSN(IS1+IS2+1:),';')
      IS4 = INDEX(INDSN,' ')
      IF (IS1 .GT.0 .AND. IS2.GT.0 .AND. IS3.GT.0 .AND. IS4.GT.0) THEN
          IF (IMNAME.EQ.' ') IMNAME = INDSN(IS1+1:IS1+IS2-1)
          IF (IMCLAS.EQ.' ') IMCLAS = INDSN(IS1+IS2+1:IS1+IS2+IS3-1)
          IF (IMSEQ.EQ.0) READ (INDSN(IS1+IS2+IS3+1:IS4-1),'(I6)')
     1             IMSEQ
          IF (OUTDSN.EQ.' ') OUTDSN = INDSN(IS1+1:IS1+IS2-1)//'.uvf'
      ELSE
          IF (OUTDSN.EQ.' ') OUTDSN = 'fits.uvf'
      END IF
      CALL UPCASE(IMNAME)
      CALL UPCASE(IMCLAS)
C
C Read MERGE file header.
C
      CALL HEADX40(.FALSE.,INDAT,IZ,ID,IY,DUM1,0,0,IUNITS,DUM3,IF,0)
      IF (IF.NE.0) CALL ERROR('Bad input MERGE file.')
      IF (NS.LT.2) CALL ERROR('Not enough stations.')
      IF (NS.GT.MAXS) CALL ERROR('Too many stations.')
      IF (NVER.NE.1 .AND. NVER.NE.2) CALL 
     1     ERROR('Invalid format version number.')
      IF (IUNITS.NE.0 .AND. IUNITS.NE.1) CALL
     1     ERROR('MERGE file contains data of unknown type.')
      IF (WTFACT.EQ.0.0) THEN
          WTFACT = 1.0
          IF (IUNITS.EQ.0) WTFACT = 1E-3
      END IF
      IF (NT.GT.0) WRITE (OUTC,'(1X,A)') '++WARNING++ Closure phases'
     1          //' in MERGE file will not be copied to FITS file.'
      IMON = 1
      CALL TDATE(IY,IMON,ID,JD,MONTH,DUM1)
      WRITE (DATOBS,1000) ID,IMON,MOD(IY,100)
C     -- TAI-UTC correction: work out MJD of reference time, and
C        calculate TAI-UTC for that time.
      CALL SLA_CALDJ(IY,IMON,ID, FIDMJD, IER)
      IF (DELTAT.EQ.0) DELTAT = SLA_DAT(FIDMJD)
      TELESC = ' '
      DO IS=1,NS
          TELESC(IS:IS) = STNABR(STNAME(IS))
      END DO
C     -- Julian date at beginning of year.
      IMON = 1
      ID = 1
      CALL TDATE(IY,IMON,ID,JD,MONTH,DUM1)
      REFDAT = JD - 0.5D0
C
C Compute baseline parameters and check they are correctly sorted.
C
      WAVE = VELC/FREQ
      COSDEC = DCOS(DEC)
      SINDEC = DSIN(DEC)
      IUTP = 0
      SORT = 'TB'
      DO IB=1,NB
          IS1 = NB1(IB)
          IS2 = NB2(IB)
          IF (IS1.GE.IS2) CALL 
     1            ERROR('Baselines are numbered incorrectly.')
          IUT = 256*IS1+IS2
          IF (IUT.LT.IUTP) SORT(2:2) = '*'
          IUTP = IUT
          BNAME(IB) = STNAME(IS1)(1:LEN1(STNAME(IS1)))//'-'//
     1                      STNAME(IS2)(1:LEN1(STNAME(IS2)))
          IF (NVER.EQ.1) THEN
              BX  = (STX(IS1)-STX(IS2))/WAVE
              BY  = (STY(IS1)-STY(IS2))/WAVE
              BZ(IB) = (STZ(IS1)-STZ(IS2))/WAVE
              BXY(IB) = DSQRT(BX*BX+BY*BY)
              IF (BXY(IB).EQ.0D0) CALL 
     1                       ERROR('Station coordinates missing.')
              BOFF(IB) = GASTM-RA-PI/2.0-DATAN2(BY,BX)
          END IF
      END DO
C
C Having read the input header successfully, it is now safe to open 
C the output FITS file.
C
      CALL FITS_INIT(OUTDSN,OUTDSN,LOUT,IER)
      IF (IER.NE.0) CALL ERROR('Cannot open output FITS file.')
C
C Determine number of visibilities and maximum values.
C Check records are in time order.
C
      AMAX = 0.0
      UMAX = 0.0
      VMAX = 0.0
      WMAX = 0.0
      WTMAX = 0.0
      IP = 0
      DO IB=1,NB
          BCOUNT(IB) = 0
      END DO
      IUTP = 0
C
C (start loop)
C
  100 READ(INDAT,END=120) IUT,(REC(J),J=1,NDWPB*NB)
      IF (IUT.LT.0) GOTO 120
      IF (IUT.LT.IUTP) SORT(1:1) = '*'
      IUTP = IUT
      DO 110 IB=1,NB
          IF (REC(NDWPB*(IB-1)+2).EQ.0.0) GOTO 110
          A = REC(NDWPB*(IB-1)+1)
          FLAG = SIGN(1.0,REC(NDWPB*(IB-1)+2))
          AERR = ABS(REC(NDWPB*(IB-1)+2))
          IP = IP+1
          BCOUNT(IB) = BCOUNT(IB)+1
          IF (NVER.EQ.1) THEN
              TIHA = (IUT-IZ)*SIDR + BOFF(IB)
              U = BXY(IB)*DCOS(TIHA)
              V = BZ(IB)*COSDEC + BXY(IB)*SINDEC*DSIN(TIHA)
              W = BZ(IB)*SINDEC - BXY(IB)*COSDEC*DSIN(TIHA)
          ELSE
              U = REC(NDWPB*(IB-1)+5)
              V = REC(NDWPB*(IB-1)+6)
              W = REC(NDWPB*(IB-1)+7)
          END IF
          AMAX = MAX(A,AMAX)
          UMAX = MAX(ABS(U),UMAX)
          VMAX = MAX(ABS(V),VMAX)
          WMAX = MAX(ABS(W),WMAX)
          IF (WTPOWR.EQ.0.0) THEN
              WT = FLAG*WTFACT
          ELSE
              WT = FLAG*WTFACT/(AERR**WTPOWR)
          END IF
          WTMAX = MAX(ABS(WT),WTMAX)
  110 CONTINUE
      GOTO 100
C
C (end loop)
C
  120 CONTINUE
C
C Scaling factors to fit into I*2.
C
      IF (BITPIX.EQ.16) THEN
         UVWSCL = MAX(UMAX,VMAX,WMAX)/32767.0
         BSCALE = AMAX/32767.0
         WTSCAL = WTMAX/32767.0
      ELSE IF (BITPIX.EQ.32) THEN
         UVWSCL = MAX(UMAX,VMAX,WMAX)/2E9
         BSCALE = AMAX/2E9
         WTSCAL = WTMAX/2E9
      ELSE
         UVWSCL = 1.0
         BSCALE = 1.0
         WTSCAL = 1.0
      END IF
C
C Write FITS header.
C
      CALL FITS_WLOG ('SIMPLE',.TRUE.,'Standard UVFITS',IER)
      CALL FITS_WINT ('BITPIX', BITPIX,' ',IER)
      CALL FITS_WINT ('NAXIS' ,     6,' ',IER)
      CALL FITS_WINT ('NAXIS1',     0,'Groups only',IER)
      CALL FITS_WINT ('NAXIS2',     3,' ',IER)
      CALL FITS_WINT ('NAXIS3',     1,' ',IER)
      CALL FITS_WINT ('NAXIS4',     1,' ',IER)
      CALL FITS_WINT ('NAXIS5',     1,' ',IER)
      CALL FITS_WINT ('NAXIS6',     1,' ',IER)
      CALL FITS_WLOG ('EXTEND',.TRUE.,'Antenna table follows',IER)
C
      CALL FITS_WREAL('BSCALE',BSCALE,'Real = Tape*BSCALE + BZERO',
     1                IER)
      CALL FITS_WREAL('BZERO',0.0,' ',IER)
      IF (IUNITS.EQ.0) THEN
          CALL FITS_WSTR ('BUNIT', 'COR.COEF', 
     1                    'Correlation coefficient', IER)
      ELSE
          CALL FITS_WSTR ('BUNIT', 'JY',
     1                    'Correlated flux density', IER)
      END IF
      CALL FITS_WLOG ('GROUPS',.TRUE.,'Groups format',IER)
      CALL FITS_WINT ('GCOUNT',IP,'Number of uv samples',IER)
      CALL FITS_WSTR ('OBJECT',  SOURCE,'Source name',IER)
      CALL FITS_WSTR ('TELESCOP',TELESC,'VLBI antennas',IER)
      CALL FITS_WSTR ('INSTRUME','VLBI',' ',IER)
      CALL FITS_WSTR ('OBSERVER',OBSERV,' ',IER)
      CALL FITS_WREAL('EPOCH',1950.0,'Epoch of RA/Dec (B1950.0)',IER)
      CALL FITS_WSTR ('DATE-OBS',DATOBS,'dd/mm/yy',IER)
      CALL FITS_WSTR ('DATE-MAP',DATMAP,'dd/mm/yy',IER)
      CALL FITS_WSTR ('ORIGIN','MERGEFITS','Caltech VLBI program',IER)
      CALL FITS_WSTR ('DATE',DATMAP,'Creation date (dd/mm/yy)',IER)
C
C FITS coordinate parameters.
C
      CALL FITS_WSTR ('CTYPE2','COMPLEX','1=real, 2=imag, 3=weight',
     1                IER)
      CALL FITS_WINT ('CRVAL2',        1,' ',IER)
      CALL FITS_WINT ('CDELT2',        1,' ',IER)
      CALL FITS_WINT ('CRPIX2',        1,' ',IER)
      CALL FITS_WSTR ('CTYPE3', 'STOKES',
     1                'Correlator: -1=RR, -2=LL, -3=RL, -4=LR',IER)
      CALL FITS_WINT ('CRVAL3',   STOKES,' ',IER)
      CALL FITS_WINT ('CDELT3',       -1,' ',IER)
      CALL FITS_WINT ('CRPIX3',        1,' ',IER)
      CALL FITS_WSTR ('CTYPE4',   'FREQ','Frequency, Hz',IER)
      CALL FITS_WDBLE('CRVAL4',     FREQ,' ',IER)
      CALL FITS_WDBLE('CDELT4',       BW,' ',IER)
      CALL FITS_WINT ('CRPIX4',        1,' ',IER)
      CALL FITS_WSTR ('CTYPE5','RA','Right ascension, degrees',IER)
      STRING = 'RA  = '
      CALL PANGLE('H', RA50, STRING(6:23), 4)
      CALL FITS_WDBLE('CRVAL5', RA50/RPDEG,STRING(:23),IER)
      CALL FITS_WINT ('CDELT5',        0,' ',IER)
      CALL FITS_WINT ('CRPIX5',        1,' ',IER)
      CALL FITS_WSTR ('CTYPE6','DEC','Declination, degrees',IER)
      STRING = 'Dec = '
      CALL PANGLE('D', DEC50, STRING(6:22), 3)
      CALL FITS_WDBLE('CRVAL6',DEC50/RPDEG,STRING(:22),IER)
      CALL FITS_WINT ('CDELT6',        0,' ',IER)
      CALL FITS_WINT ('CRPIX6',        1,' ',IER)
C
C FITS random parameters.
C
      CALL FITS_WINT ('PCOUNT',6,'True value = tape value * PSCALn '//
     1                '+ PZEROn.',IER)
      CALL FITS_WSTR ('PTYPE1','UU',
     1                'baseline u projection, seconds',IER)
      CALL FITS_WDBLE('PSCAL1',UVWSCL/FREQ,' ',IER)
      CALL FITS_WINT ('PZERO1',0,' ',IER)
      CALL FITS_WSTR ('PTYPE2','VV',
     1                'baseline v projection, seconds',IER)
      CALL FITS_WDBLE('PSCAL2',UVWSCL/FREQ,' ',IER)
      CALL FITS_WINT ('PZERO2',0,' ',IER)
      CALL FITS_WSTR ('PTYPE3','WW',
     1                'baseline w projection, seconds',IER)
      CALL FITS_WDBLE('PSCAL3',UVWSCL/FREQ,' ',IER)
      CALL FITS_WINT ('PZERO3',0,' ',IER)
C
      CALL FITS_WSTR ('PTYPE4','BASELINE','256*ANT1 + ANT2',IER)
      CALL FITS_WINT ('PSCAL4',1,' ',IER)
      CALL FITS_WINT ('PZERO4',0,' ',IER)
C
      CALL FITS_WSTR ('PTYPE5','DATE','TAI Julian Date part 1',IER)
      CALL FITS_WDBLE('PSCAL5',1D0/60D0,'Days/60',IER)
      CALL FITS_WDBLE('PZERO5',REFDAT,' ',IER)
C
      CALL FITS_WSTR ('PTYPE6','DATE','TAI Julian Date part 2',IER)
      CALL FITS_WDBLE('PSCAL6',1D0/1728000D0,'Days/1728000 (1/20 s)',
     1                IER)
      CALL FITS_WINT ('PZERO6',0,' ',IER)
C
C Rewind input dataset to read History; skip remaining header records.
C
      REWIND (UNIT=INDAT)
      READ (INDAT) REC1
      IF (REC1(5).GT.0) THEN
         READ (INDAT)
         REC1(3) = REC1(3)-1
      END IF
      IF (REC1(6).GT.0) THEN
         READ (INDAT)
         REC1(3) = REC1(3)-1
      END IF
      DO I=1,REC1(2)
         READ (INDAT) HISTRY
         CALL FILTER(HISTRY)
         CALL FITS_WCMT('HISTORY',HISTRY,IER)
      END DO
      REC1(3) = REC1(3)-REC1(2)
      DO I=1,REC1(3)
         READ (INDAT)
      END DO
C
C Additional FITS history.
C
      CALL FITS_WCMT('HISTORY', NEWHIS, IER)
      CALL FITS_WCMT('HISTORY', '- Input file:      '//INDSN, IER)
      CALL FITS_WCMT('HISTORY', '- Output file:     '//OUTDSN,IER)
      WRITE (HISTRY,1020) UMAX*1E-6,VMAX*1E-6,WMAX*1E-6
      CALL FITS_WCMT('HISTORY',HISTRY,IER)
      WRITE (HISTRY,1030) AMAX
      CALL FITS_WCMT('HISTORY',HISTRY,IER)
      WRITE (HISTRY,1040) IP
      CALL FITS_WCMT('HISTORY',HISTRY,IER)
      WRITE (HISTRY,1051) DELTAT
      CALL FITS_WCMT('HISTORY',HISTRY,IER)
      WRITE (HISTRY,1052) WTFACT, WTPOWR
      CALL FITS_WCMT('HISTORY',HISTRY,IER)
C
C History records to confuse AIPS.
C
      CALL FITS_WCMT('COMMENT',DASH,IER)
      IF (IMNAME.NE.' ') 
     1          CALL FITS_WCMT('HISTORY','AIPS IMNAME='''//
     2                         IMNAME(1:LEN1(IMNAME))//'''',IER)
      IF (IMCLAS.NE.' ') 
     1          CALL FITS_WCMT('HISTORY','AIPS IMCLASS='''//
     2                         IMCLAS(1:LEN1(IMCLAS))//'''',IER)
      IF (IMSEQ.NE.0) THEN
          WRITE (BUF,'(I8.8)',IOSTAT=IER) IMSEQ
          CALL FITS_WCMT('HISTORY','AIPS IMSEQ='//BUF,IER)
      END IF
      IF (USERNO.NE.0) THEN
          WRITE (BUF,'(I8.8)',IOSTAT=IER) USERNO
          CALL FITS_WCMT('HISTORY','AIPS USERNO='//BUF,IER)
      END IF
      CALL FITS_WCMT('HISTORY','AIPS SORT ORDER = '''//SORT//'''',IER)
      WRITE (HISTRY,1045) WTSCAL/BSCALE
      CALL FITS_WCMT('HISTORY',HISTRY,IER)
      IF (FLUX.NE.0D0)
     1     CALL FITS_WDBLE('CITFLUX', FLUX,
     2     'Total flux density, Jy',IER)
      IF (TINTG.NE.0D0)
     1     CALL FITS_WDBLE('CITTINTG', TINTG, 
     2     'Coherent integration, sec',IER)
      IF (AVG.NE.0D0)
     1     CALL FITS_WDBLE('CITAVG', AVG, 
     2     'Incoherent integration, sec',IER)
      IF (BW.NE.0D0)
     1     CALL FITS_WDBLE('CITBW', BW, 
     2     'Bandwidth, Hz',IER)
      CALL FITS_WREAL('CITWTFAC', WTFACT,
     1     'AIPS weight = CITWTFAC / (amperr**CITWTPOW)', IER)
      CALL FITS_WREAL('CITWTPOW', WTPOWR,
     1     'AIPS weight = CITWTFAC / (amperr**CITWTPOW)', IER)
      CALL FITS_WCMT('COMMENT',DASH,IER)
      CALL FITS_WEND(IER)
C
C Re-read data and write out.
C
      IP = 0
      WTMIN = 20000
      WTMAX = -20000
  130 READ (INDAT,END=150) IUT,(REC(J),J=1,NDWPB*NB)
      IF (IUT.LT.0) GOTO 150
C     -- days since REFDAT (add on TAI-UTC)
      T = (IUT/5184000D0) + (DELTAT/86400D0)
      DO 140 IB=1,NB
          IF (REC(NDWPB*(IB-1)+2).EQ.0.0) GOTO 140
          A = REC(NDWPB*(IB-1)+1)
          FLAG = SIGN(1.0,REC(NDWPB*(IB-1)+2))
          AERR = ABS(REC(NDWPB*(IB-1)+2))
          PHS =  RPDEG*REC(NDWPB*(IB-1)+3)
          IF (NVER.EQ.1) THEN
              TIHA = (IUT-IZ)*SIDR + BOFF(IB)
              U = BXY(IB)*DCOS(TIHA)
              V = BZ(IB)*COSDEC + BXY(IB)*SINDEC*DSIN(TIHA)
              W = BZ(IB)*SINDEC - BXY(IB)*COSDEC*DSIN(TIHA)
          ELSE
              U = REC(NDWPB*(IB-1)+5)
              V = REC(NDWPB*(IB-1)+6)
              W = REC(NDWPB*(IB-1)+7)
          END IF
C         -- AIPS weight (in units of tenths)
          IF (WTPOWR.EQ.0.0) THEN
              WT = FLAG*WTFACT
          ELSE
              WT = FLAG*WTFACT/(AERR**WTPOWR)
          END IF
          WTMIN = MIN(WTMIN, ABS(WT))
          WTMAX = MAX(WTMAX, ABS(WT))
          IF (BITPIX.EQ.16) THEN
             I2DATA(1) = NINT(U/UVWSCL)
             I2DATA(2) = NINT(V/UVWSCL)
             I2DATA(3) = NINT(W/UVWSCL)
             I2DATA(4) = 256*NB1(IB) + NB2(IB)
             I2DATA(5) = INT(T*60D0)
             I2DATA(6) = NINT((T-I2DATA(5)/60D0)*1728000D0)
             I2DATA(7) = NINT(A*COS(PHS)/BSCALE)
             I2DATA(8) = NINT(A*SIN(PHS)/BSCALE)
             I2DATA(9) = NINT(WT/WTSCAL)
             CALL FITS_WRAYS(I2DATA,9,IER)
          ELSE IF (BITPIX.EQ.32) THEN
             I4DATA(1) = NINT(U/UVWSCL)
             I4DATA(2) = NINT(V/UVWSCL)
             I4DATA(3) = NINT(W/UVWSCL)
             I4DATA(4) = 256*NB1(IB) + NB2(IB)
             I4DATA(5) = INT(T*60D0)
             I4DATA(6) = NINT((T-I4DATA(5)/60D0)*1728000D0)
             I4DATA(7) = NINT(A*COS(PHS)/BSCALE)
             I4DATA(8) = NINT(A*SIN(PHS)/BSCALE)
             I4DATA(9) = NINT(WT/WTSCAL)
             CALL FITS_WRAYI(I4DATA,9,IER)
          ELSE
             R4DATA(1) = U
             R4DATA(2) = V
             R4DATA(3) = W
             R4DATA(4) = 256*NB1(IB) + NB2(IB)
             R4DATA(5) = INT(T*60D0)
             R4DATA(6) = ((T-R4DATA(5)/60D0)*1728000D0)
             R4DATA(7) = A*COS(PHS)
             R4DATA(8) = A*SIN(PHS)
             R4DATA(9) = WT
             CALL FITS_IEEER4(R4DATA,9)
             CALL FITS_WRAYI(R4DATA,9,IER)
          END IF
          IP = IP+1
  140 CONTINUE
      GOTO 130
  150 CLOSE (UNIT=INDAT)
C
C Write FITS-AIPS antenna table.
C
      CALL ANTENNA_TABLE(NS,STNAME,STX,STY,STZ,IER)
C
C Close output file.
C
      CALL FITS_CLOSE(IER)
      IF (IER.NE.0) CALL ERROR('Error while writing FITS file.')
      WRITE (OUTC,1090) IP,OUTDSN(1:MIN(100,LOUT))
      WRITE (OUTC,1100) WTMIN, WTMAX
C-----------------------------------------------------------------------
      END
