      PROGRAM MK3MRG
C-----------------------------------------------------------------------
C Merge Haystack 'A' files into standard MERGE format.
C
C T.J.Pearson     (Almost Fortran-77)
C Version 1.0:  1983 Jan 24
C Version 1.1:  1985 Jun 21 - add channel selection
C Version 2.0:  1986 Jul 23 - edited to conform to july 1986 haystack
C                             A file format with the exception that
C                             1950 source positions must still be
C                             entered manually!!! (Dave Hough).
C Version 2.1:  1989 Dec 22 - revised for Unix (Sun/Convex) (Dave 
C                             Meier).
C Version 2.2:  1992 Dec 22 - negative declinations were interpreted
C                             incorrectly (John Ellithorpe).
C Version 2.3:  1993 Nov  4 - fix NINT bug found on Sun/PC; closer
C                             to standard F77 (TJP).
C version 2.4:  1994 Oct  4 - fix bug in reading SNR>99 (M. Bietenholz).
C-----------------------------------------------------------------------
      CHARACTER*(*) VERSN,VERSD
      PARAMETER (VERSN='MK3,2.3',VERSD='1994 Oct 4')
      INTEGER   INC,OUTC,INDAT,OUTDAT,MAXFIL
C     -- terminal input unit
      PARAMETER (INC=5)
C     -- terminal output unit
      PARAMETER (OUTC=6)
C     -- Haystack input unit
      PARAMETER (INDAT=10)
C     -- Merge output unit
      PARAMETER (OUTDAT=12)
C     -- Maximum number of input files
      PARAMETER (MAXFIL=100)
C
      CHARACTER*128  INDSN(MAXFIL),OUTDSN, FILE
      INCLUDE 'merge64.inc'
      INTEGER IER, L, MODE, NFILE, NF, J, LDSN, K
      INTEGER GETIN, LEN1, VLBOPE
C-----------------------------------------------------------------------
C
C File names.
C
      CALL PUTOUT('MK3MERGE extracts visibility data for one source '//
     1            'from Haystack A-format files')
      CALL PUTOUT('(Version '//VERSN//' '//VERSD//')')
      CALL PUTOUT(' ')
      IER = GETIN(OUTDSN, 'Output (MERGE) file name: ', L)
      IF (IER.NE.1) STOP
      IER = VLBOPE(OUTDAT, OUTDSN, 'MERGE', 'NEW', OUTDSN)
      IF (IER.NE.1) STOP
C
C Set up Header information.
C
      MODE = 0
      CALL MRG_PARAM(MODE,INC,OUTC,VERSN)
C
C Read Haystack header file.
C
      CALL MRG_HAYHEAD(INC,OUTC)
C
C Read Haystack files.
C
      CALL PUTOUT(' ')
      CALL PUTOUT('List input file names - wild cards accepted.')
      CALL PUTOUT('End list with blank line or ^Z (VMS) or ^D (UNIX)')
      NFILE = 0
      CALL MRG_FFIND(NF,INDSN,MAXFIL,INC,OUTC)
      DO 10 J=1,NF
          LDSN = LEN1(INDSN(J))
          K = VLBOPE(INDAT, INDSN(J), 'TEXT', 'OLD', FILE)
          IF (K.EQ.1) THEN
              CALL MRG_HAYSTACK(INDSN(J),INDAT,OUTC,NFILE)
              CLOSE(UNIT=INDAT)
          END IF
   10 CONTINUE
C
C Write new dataset.
C
      IF (NFR.GT.0) THEN
          CALL MRG_OUTPUT(OUTDSN,OUTDAT,OUTC)
          CLOSE(UNIT=OUTDAT,STATUS='KEEP')
          WRITE (OUTC,40) OUTDSN(1:LEN1(OUTDSN)),NFR
      ELSE
          CLOSE(UNIT=OUTDAT,STATUS='DELETE')
          CALL ERROR('No data found for requested source')
      END IF
C
      STOP
C
C Format statements.
C
   40 FORMAT(/' Output in file: ',A/I6,' data records')
   70 FORMAT(' +++ERROR+++ Cannot read file ',A)
C
      END

      SUBROUTINE MRG_HAYSTACK(INFILE,IN,PR,NFILE)
C-----------------------------------------------------------------------
C      VLBMERGE: Read Haystack A files from unit 'IN'
C      and merge with existing data.
C
C      Format of file (items marked * are extracted here):
C
C      Field    Columns Format  Explanation
C
C       1       1:6     A6      File name
C       2       8:9     A2      Extent type (51,52,or 53)
C       3       10:11   I2      Extent#
C       4       14:17   I4      # of blocks (type 51,53) or # seconds in
C                               scan (type 52)
C       5       18:21   I3      Expt serial #
C       6       23:32   A10     Processing date/time
C       7*      34:35   I2      Year (-1900)
C       8*      37:44   A8      Scan ID (ddd-hhmm)
C       9*      46:53   A8      Source name
C       10*     55:57   A2      Baseline identifier
C       11*     58:62   F5.1    Decimal seconds of center time
C       11.1    63:66   I4      TAPE START TIME
C       12*     68:75   F8.2    Frequency (MHz)
C       13      77:78   A1      Frequency/polarization code
C       13.1    79:80   I2      # OF FREQUENCY CHANNELS
C       14*     81:86   F6.2    Correlated amplitude x 1E4
C       15*     87:91   F5.1    SNR (integer)
C       16*     93:97   F5.1    Residual phase (degrees)
C       17      99:103  F5.2    Incoherent amplitude
C       18      105:111 F7.2    Incoherent SNR
C       19      113:118 F6.2    Segment #
C       20      120:125 A6      Allan variance
C       21*     127:129 I3      Coherent integration (sec)
C
C       Notes on header parameters:
C
C       (1) The Greenwich MEAN sidereal time is computed for 0h UT
C       on the first day of observation (the format calls for APPARENT,
C       not mean, but the difference is not usually significant).
C-----------------------------------------------------------------------
      INCLUDE 'merge61.inc'
      INCLUDE 'merge62.inc'
      INCLUDE 'merge63.inc'
      INCLUDE 'merge64.inc'
      DOUBLE PRECISION  TWOPI
      PARAMETER (TWOPI=3.141592653589793 D0 * 2D0)
C
      CHARACTER*(*) INFILE
C
C            The following declarations apply to the data in
C            the disk file:
C
      CHARACTER*132 RECORD
      CHARACTER*8   SRC
      CHARACTER*1   STN(2)
      REAL          AMP,PHASE,SNR
      DOUBLE PRECISION        FREQ
C
C            Other declarations
C
      INTEGER   IN, PR, UT, NREC, NWANT, I, ICODE, IBASE, IP
      INTEGER   SIGN, ID, IH, IM, I0, MRG_LOC, IY, IAVG, NDAY, NFILE
      INTEGER   IS(2),STIME(8), LEN1
      REAL      RS
      DOUBLE PRECISION    DATE
C
C            Initialize.
C
      NREC = 0
      NWANT = 0
      WRITE (PR,640) INFILE(1:LEN1(INFILE))
C
C            Read next record; if first char is '*', it is a
C            comment.
C
  100 READ (IN,'(A)',END=210) RECORD
      NREC = NREC+1
      IF (RECORD(1:1).EQ.'*') GOTO 100
C
C            Check source
C
      SRC = RECORD(46:53)
      IF (HSRC.EQ.' ') HSRC = SRC
      CALL UPCASE(SRC)
      IF ((SRC.NE.HSRC).AND.(SRC.NE.HALIAS).AND.
     1    (HSRC.NE.'ALL')) GOTO 100
C
C            Check freq/pol code
C
      ICODE = ICHAR(RECORD(77:77))
      IF (NCHANL.NE.0 .AND. NCHANL.NE.1 .AND. ICODE.NE.NCHANL) GOTO 100
C
C            Check baseline
C
      STN(1) = RECORD(55:55)
      STN(2) = RECORD(56:56)
      IS(1) = 0
      IS(2) = 0
      CALL UPCASE(STN(1))
      CALL UPCASE(STN(2))
      DO 110 I=1,NS
          IF (STN(1).EQ.OSTATN(I)(1:1)) IS(1) = I
          IF (STN(2).EQ.OSTATN(I)(1:1)) IS(2) = I
  110 CONTINUE
      IF (IS(1).EQ.0 .OR. IS(2).EQ.0 .OR. IS(1).EQ.IS(2)) GOTO 100
      IF (IS(1).LT.IS(2)) THEN
          SIGN=+1
          IBASE=IS(1)+(IS(2)-1)*(IS(2)-2)/2
C
C            SIGN=-1 if baseline is reversed
C
      ELSE
          SIGN=-1
          IBASE=IS(2)+(IS(1)-1)*(IS(1)-2)/2
      END IF
C
C            Absorb data (if signal-to-noise ratio .ge. 1)
C
      IP = 0
      READ (RECORD,600)  ID,IH,IM,RS, AMP,SNR,PHASE
      IF (SNR.LT.1.0) GOTO 100
      NWANT = NWANT+1
      UT = 60*((ID-1)*86400 + IH*3600 +IM*60) + NINT(60.0*RS)
      I0 = MRG_LOC(UT) + 4*(IBASE-1)
      STORE(I0) = AMP*1E-4
      STORE(I0+1) = AMP*1E-4/SNR
      STORE(I0+2) = PHASE*SIGN
      STORE(I0+3) = 57.3/SNR
C
C            Update header parameters
C
      IF (NWANT.EQ.1) THEN
          READ (RECORD,610) IY, FREQ, IAVG
          IF ( HYEAR.LE.0  ) HYEAR = IY+1900
          IF ( HFREQ.EQ.0D0) HFREQ = FREQ*1E6
          IF (HINTEG.EQ.0D0) HINTEG = IAVG
          NDAY = UT/(60*86400) + 1
          HUTF = (60*86400)*(NDAY-1)
          CALL JULDA(HYEAR,1,NDAY,DATE,HGST)
          HGST = TWOPI*HGST/86400D0
          STIME(1) = IY
          STIME(2) = 1
          STIME(3) = NDAY
          CALL SRCPAN(STIME,HRA50,HDEC50,HRAP,HDECP)
      END IF
      GOTO 100
C
C
  210 WRITE (PR,650) NWANT, NREC
      RETURN
C
C            Format statements
C
  600 FORMAT (T37,I3,1X,I2,I2,T58,F5.0,T81,F6.0,T87,F5.0,T93,F5.0)
  610 FORMAT (T34,I2,T68,F8.2,T127,I3)
  640 FORMAT (/' File: ',A)
  650 FORMAT (I6,' records extracted from a total of',I6)
      END

      SUBROUTINE MRG_HAYHEAD(INC,OUTC)
C-----------------------------------------------------------------------
C      VLBMERGE: Read header file for Haystack Mark-3 data.
C-----------------------------------------------------------------------
      INCLUDE 'merge61.inc'
      INCLUDE 'merge62.inc'
      INCLUDE 'merge63.inc'
      DOUBLE PRECISION    PI,RPSA,RPST
      PARAMETER (PI=3.141592653589793 D0)
      PARAMETER (RPSA=PI/648000 D0)
      PARAMETER (RPST=PI/43200 D0)
      INTEGER INC,OUTC,INDAT
      PARAMETER (INDAT=10)
      CHARACTER*128 FILE
      CHARACTER*80 RECORD
      CHARACTER*20 WORDS(20)
      DOUBLE PRECISION  VALS(20)
      INTEGER IER, NWORDS, J, L
      INTEGER VLBOPE, GETIN
C
C            Open the header file.
C
    5 IER = GETIN(FILE, 'Name of Haystack header file: ', L)
      IF (IER.NE.1) STOP
      IER = VLBOPE(INDAT, FILE(1:L), 'TEXT', 'OLD', FILE)
      IF (IER.NE.1) GOTO 5
C
C      Read next record; interpret $EXP[ER], $STA[T], $SOU[RCE] groups.
C
   10 READ (INDAT,'(A)',END=200) RECORD
   11 IF (RECORD(1:4).EQ.'$EXP') THEN
          NHIS = NHIS+1
          IF (NHIS.LE.MAXHIS) HIS(NHIS) = RECORD
      ELSE IF (RECORD(1:4).EQ.'$STA' .AND. RECORD(1:7).NE.'$STATUS')
     1          THEN
   12     READ (INDAT,'(A)',END=200) RECORD
          IF (RECORD(1:1).EQ.'$') THEN
            GOTO 11
          ELSE IF (RECORD(1:1).NE.'*') THEN
            CALL PARSEB(RECORD,WORDS,NWORDS)
            DO 120 J=4,7
                READ (WORDS(J),'(BN,F20.0)') VALS(J)
  120       CONTINUE
            IF (VALS(4).NE.0D0) GOTO 12
            DO 130 J=1,NS
                IF (WORDS(1).EQ.OSTATN(J)) THEN
                  HX(J) = VALS(5)
                  HY(J) = -VALS(6)
                  HZ(J) = VALS(7)
                END IF
  130       CONTINUE
          END IF
          GOTO 12
      ELSE IF (RECORD(1:4).EQ.'$SOU') THEN
  140          READ (INDAT,'(A)',END=200) RECORD
          IF (RECORD(1:1).EQ.'$') THEN
            GOTO 11
          ELSE IF (RECORD(1:1).NE.'*') THEN
            CALL PARSEB(RECORD,WORDS,NWORDS)
            DO 150 J=3,9
                READ (WORDS(J),'(BN,F20.0)') VALS(J)
  150       CONTINUE
            IF (VALS(9).NE.1950D0) GOTO 140
            IF (HSRC.EQ.' ') HSRC = WORDS(1)
            IF (WORDS(1).EQ.HSRC .OR. WORDS(1).EQ.HALIAS .OR.
     1                WORDS(2).EQ.HSRC .OR. WORDS(2).EQ.HALIAS) THEN
                HRA50 = 3600D0*VALS(3) + 60D0*VALS(4) + VALS(5)
                HRA50 = HRA50*RPST
                IF (VALS(6).LT.0D0) THEN
                    HDEC50 = -3600D0*VALS(6) + 60D0*VALS(7) + VALS(8)
                    HDEC50 = -1D0*HDEC50*RPSA
                ELSE
                    HDEC50 = 3600D0*VALS(6) + 60D0*VALS(7) + VALS(8)
                    HDEC50 = HDEC50*RPSA
                END IF
            END IF
          END IF
          GOTO 140
      END IF
      GOTO 10
  200 CLOSE (UNIT=INDAT)
      RETURN
      END
