      PROGRAM MODFIT
C-----------------------------------------------------------------------
C Written and produced by Richard S. Simon, July 1981
C This program attempts to minimize the agreement factor for the
C fit of a model to VLB data.
C
C Version  1: July 28, 1981
C Version  2: Nov 16, 1981    Decrease max# of iterations
C Version  3: Dec 22, 1981
C Version  4: Feb  4, 1982    Add underlying model option
C Version  5: May  6, 1982   Fix a bug in BUVDATA, change array sizes
C Version  6: June 8, 1982   Add model type 6 (Optically thin Sphere)
C Version  7: July 7, 1983   Extend to include Merge format
C        (D. Meier)                     version 2 files; change BUVDATA
C                                       to BWTDATA.
C             Aug  3, 1983   Added W to version 2 files.
C             Oct 17, 1983   Corrected bug in BVIS when QQ=0.0
C                            Don't bother doing VIS calc if ERR <= 0
C Version  8: 1985 Feb 7   [TJP] Fix bug found by VMS4.0:
C                          attempting to read SYS$OUTPUT.
C Version  9: 1988 Apr 18  [TJP] Convex version.
C Version 10: 1989 Mar 29  [TJP] Improved performance (vectorized).
C Version 11: 1990 Nov  5  [TJP] Fix serious bug: it would give the 
C                          wrong answer on files containing an
C                          incomplete set of baselines or triangles.
C Version 12: 1992 Jun 24  [TJP] generate new file names for output
C                          (Unix only; VMS does it automatically).
C Version 13: 1992 Aug 10  [TJP] Correct bug prompting for new file
C                          name. Close files asap to allow other
C                          programs to read them.
C Version 14: 1992 Dec  7  [TJP] Calculate closure phases if needed.
C Version 15: 1993 Jan 10  [RC]  Added MODPLOT style graphics.  Slightly
C                          improved error checking in BGETMOD.
C Version 15.1: 1993 Mar 2 [TJP] Change check that data have been
C                          calibrated from fatal to warning.
C Version 16: 1993 Mar 25  [TJP] Relative weighting of amplitude and
C                          closure-phases.
C Version 16.1: 1993 Jul 15 [TJP/RC] Correct initialization error.
C Version 16.2: 1993 Oct 26 [TJP] Fix bug that stopped interrupt from
C                          working.
C Version 16.3: 1993 Oct 30 [TJP] Modify GRIDLS to prevent component
C                          flux from becoming negative.
C-----------------------------------------------------------------------
      INCLUDE 'modelfit20.inc'
C
      INTEGER EDIT, IER, L, J
      INTEGER VLBOPE, GETIN
      LOGICAL QUERY
      CHARACTER*80 INTXT
C
      CALL SCLOCK
      CALL PUTOUT('Program MODELFIT  --  1993 Oct 30 -- 20 stations')
      CALL PUTOUT(' ')
C
C Get the file name for data file:
C
   10 IER = GETIN(INDSN, 'Input data file name: ', L)
      IF (IER.NE.1) STOP
      IER = VLBOPE(INDAT, INDSN(1:L), 'MERGE', 'OLD', INDSN)
      IF (IER.NE.1) GOTO 10
C
C Get the input data and compute the arrays UDATA and VDATA:
C
      CALL BGETDAT
      CLOSE (UNIT=INDAT)
C
C Determine which triangles are independent for each data record:
C
      CALL BTRIANGLE
C
C Compute the non-zero data points and the weights for the data.
C
      CALL BWTDATA
C
C Get underlying (constant) part of input model, if any:
C
      CALL UNDERMOD
C
C Open graphics device, if wanted.
C
      CALL PUTOUT(' ')
      DOPLOT = QUERY('Do you want to view modelfitting graphically? '//
     +  '(Y or N): ')
      IF (DOPLOT) CALL OPENGR
C
      FRSTRY = .TRUE.
C
C Line to come back to when trying a new model:
C
  100 CONTINUE
C
C Get relative weighting of amplitudes and closure phases.
C
  101 IER = GETIN(INTXT, 'Weighting of amplitudes in total AF '//
     1                   '(0.0...1.0, default 0.5): ', L)
      IF (IER.NE.1) STOP
      IF (L.LT.1) THEN
          AWEIGT = 0.5
      ELSE
          READ (INTXT(:L),*,ERR=101) AWEIGT
      END IF
      IF (AWEIGT.LT.0.0 .OR. AWEIGT.GT.1.0) GOTO 101
      CWEIGT = 1.0 - AWEIGT
C
C Read in and edit the model file: returns EDIT=1 if file was
C edited, 0 otherwise.
C
      CALL BGETMOD(EDIT)
C
C Warn user if model has non Gaussian components.
C
      IF (DOPLOT) CALL WARNGR
C
C Flag so as to set new graphics window limits on next plot.
C
      WINSET = .FALSE.
C
C Compute and display the initial fit.
C
      METHOD = 'Initial model'
      JITER = 0
      DO J=1,NCOMP
          CALL BVIS(J)
      END DO
      CALL BMODAP
      CALL BAGRFAC
      CALL BAFOUT(EDIT)
C
C Set up so that Control C's will be recognized.
C
      IF (FRSTRY) CALL CTRLCEN
C
C Fit the model:
C
      CALL PUTOUT(' ')
      IF (QUERY('Well, should we try a gradient search? (Y or N): ')) 
     1    THEN
          CALL GRADLS
      END IF
C
      CALL PUTOUT(' ')
      IF (QUERY('Well, should we try a brute force (grid)'//
     1         ' fit? (Y or N): ')) THEN
          CALL GRIDLS
      END IF
C
C Write the agreement factors and model to the output units:
C
      CALL BAFOUT(1)
C
C Let the user try again, if desired:
C
      IF (QUERY('Would you like to try again? (Y or N): ')) THEN
          FRSTRY = .FALSE.
          GO TO 100
      END IF
      IF (DOPLOT) CALL PGEND
C-----------------------------------------------------------------------
      END
      SUBROUTINE BAFOUT(SAVE)
      INTEGER SAVE
C
C       By Richard S. Simon
C
C       This prints the results from program MODELFIT to unit INMOD
C       (e.g., a model with comments), and a summary to unit OUTC.
C   SAVE = 1: save in file and summarize
C   SAVE = 0: save only
C-----------------------------------------------------------------------
      INCLUDE 'modelfit20.inc'
      CHARACTER*64 MODNAM, OLDMOD, TEMP
      CHARACTER*24 MESSGE
      CHARACTER*80 HISTRY
      CHARACTER*8  ELAPSE
      REAL CPU
      REAL STAR
      PARAMETER (STAR='*   ')
      CHARACTER*1 VARIED,FIXED,FLAG(6,MXMOD)
      PARAMETER (VARIED='v',FIXED=' ')
      INTEGER VLBOPE, GETIN, LEN1
      INTEGER IER, L, I1, I2, I, IFORM, J, K
      REAL    ZFREQ
C
C Close the input model file.
C
      IF (SAVE.EQ.0) GOTO 200
      MODNAM = MODDSN
      OLDMOD = MODNAM
C
C Open the output model file.
C
   17 CALL NEWFIL(MODNAM, TEMP)
      MODNAM = TEMP
      IER = VLBOPE(INMOD, MODNAM, 'MODEL', 'NEW', MODNAM)
      IF (IER.NE.1) THEN
          IER = GETIN(MODNAM, 'Name for output model file: ', L)
          IF (IER.NE.1) STOP
          GOTO 17
      END IF
C
C Write the header in the output model file.
C
      CALL MKHIST('MODELFIT', '30-Oct-1993', HISTRY)
      L = LEN1(HISTRY)
      WRITE (INMOD,1000) '! ',HISTRY(1:L)
      L = LEN1(INDSN)
      WRITE (INMOD,1000) '! Input data file:   ',INDSN(1:L)
      L = LEN1(OLDMOD)
      WRITE (INMOD,1000) '! Input model file:  ',OLDMOD(1:L)
      IF (UNDMOD) THEN
C         There was an underlying model file, so write it out:
          INQUIRE (FILE=UNDDSN,NAME=TEMP)
          L = LEN1(TEMP)
          WRITE (INMOD,1000) '!'
          WRITE (INMOD,1000) '! An underlying (constant) model was ',
     1    'used for this fit.'
          WRITE (INMOD,1000) '! Name of underlying file: ',TEMP(1:L)
      ENDIF
C
      ZFREQ = FREQ / 1.E6
      WRITE (INMOD,1008) SOURCE,NDAY,NYEAR,ZFREQ
      WRITE (INMOD,1000) '!'
      WRITE (INMOD,1000) '! Baseline       Agreement          ',
     1'        Triangle       Agreement'
      WRITE (INMOD,1000) '!                   Factor          ',
     1'                          Factor'
C
      I1 = MIN(NUMBAS,NUMTRI)
      I2 = I1 + 1
      DO 100 I=1,I1
          IF (VISAF(I) .GT. 0. .AND. CLOAF(I) .GT. 0.) THEN
            ASSIGN 1001 TO IFORM
          ELSEIF (VISAF(I) .LT. 0. .AND. CLOAF(I) .LT. 0.) THEN
            ASSIGN 1004 TO IFORM
            VISAF(I) = STAR
            CLOAF(I) = STAR
          ELSEIF (VISAF(I) .LT. 0.) THEN
            ASSIGN 1002 TO IFORM
            VISAF(I) = STAR
          ELSE
            ASSIGN 1003 TO IFORM
            CLOAF(I) = STAR
          ENDIF
          WRITE (INMOD,IFORM) STNAME(NAVIS(I)),STNAME(NBVIS(I)),
     1    VISAF(I), STNAME(NACLO(I)), STNAME(NBCLO(I)),
     1    STNAME(NCCLO(I)), CLOAF(I)
  100 CONTINUE
C
      IF (NUMBAS .GT. NUMTRI) THEN
          DO 120 I = I2, NUMBAS
            IF (VISAF(I) .LT. 0.) THEN
              ASSIGN 1002 TO IFORM
              VISAF(I) = STAR
            ELSE
              ASSIGN 1001 TO IFORM
            ENDIF
            WRITE (INMOD,IFORM) STNAME(NAVIS(I)),STNAME(NBVIS(I)),
     1      VISAF(I)
  120     CONTINUE
      ELSEIF (NUMTRI .GT. NUMBAS) THEN
          DO 140 I = I2, NUMTRI
C           If AF is <0, print a star instead of AF.
            IF (CLOAF(I) .GT. 0.) THEN
              ASSIGN 1005 TO IFORM
            ELSE
              ASSIGN 1006 TO IFORM
              CLOAF(I) = STAR
            ENDIF
            WRITE (INMOD,IFORM) STNAME(NACLO(I)), STNAME(NBCLO(I)),
     1      STNAME(NCCLO(I)), CLOAF(I)
  140     CONTINUE
      ENDIF
      IF (AMPLAF .GE. 0. .AND. PHASAF .GE. 0.) THEN
          ASSIGN 1007 TO IFORM
      ELSEIF (AMPLAF .GE. 0.) THEN
          ASSIGN 1014 TO IFORM
          PHASAF = STAR
      ELSE
          ASSIGN 1015 TO IFORM
          AMPLAF = STAR
      ENDIF
      WRITE (INMOD,IFORM) '! Total amplitude agreement factor:',AMPLAF,
     1    '! Total phase clo agreement factor:',PHASAF,
     2    '! Total agreement factor:',TOTLAF
      WRITE (INMOD,1016) '! Equivalent number of ind. data ',
     1    'points:',TNZDAT
      WRITE (INMOD,1000) '!'
      WRITE (INMOD,1000) '! Starting Model (v => variable parameter):'
      WRITE (INMOD,1000) '! #      Flux     Radius      Theta ',
     1'      Axis      Ratio        Phi       Type'
      WRITE (INMOD,1009) (J,FLSTRT(J),FLAG(1,J),RASTRT(J),FLAG(2,J),
     1    THSTRT(J),FLAG(3,J),AXSTRT(J),FLAG(4,J),ARSTRT(J),FLAG(5,J),
     2    PHSTRT(J),FLAG(6,J),TYPE(J), J=1,NCOMP)
      WRITE (INMOD,1000) '!'
      WRITE (INMOD,1000) '! Final Model:'
      WRITE (INMOD,1000) '!        Flux     Radius      Theta ',
     1'      Axis      Ratio        Phi       Type'
      WRITE (INMOD,1010) (FLUX(J),RADIUS(J),THETA(J),AXIS(J),
     1    RATIO(J),PHI(J),TYPE(J), J=1,NCOMP)
      CLOSE (UNIT=INMOD)
      MODDSN = MODNAM
C
C Write summary to terminal:
C
  200 DO 210 J = 1, NCOMP
          THETA(J) = RTHETA(J)/DEGRAD
          PHI(J)   = RPHI(J)/DEGRAD
          DO 205 K=1,6
            IF (VARY(K,J)) THEN
              FLAG(K,J) = VARIED
            ELSE
              FLAG(K,J) = FIXED
            ENDIF
  205     CONTINUE
  210 CONTINUE
      WRITE (OUTC,1011) SOURCE, JITER, METHOD(1:15)
      WRITE (OUTC,1000) '  #      Flux     Radius      Theta ',
     1'     Axis      Ratio        Phi       Type'
      WRITE (OUTC,1012) (J,FLUX(J),FLAG(1,J),RADIUS(J),FLAG(2,J),
     1THETA(J),FLAG(3,J),AXIS(J),FLAG(4,J),RATIO(J),FLAG(5,J),
     2PHI(J),FLAG(6,J),TYPE(J), J=1,NCOMP)
C
C       Generate comment on quality of fit:
      IF (TOTLAF .LT. 0.5) THEN
          MESSGE = 'Too good to be true!'
      ELSEIF (TOTLAF .LT. 1.) THEN
          MESSGE = 'Astoundingly good!'
      ELSEIF (TOTLAF .LT. 2.) THEN
          MESSGE = 'I''m impressed.'
      ELSEIF (TOTLAF .LT. 3.) THEN
          MESSGE = 'Good'
      ELSE
          MESSGE = ' '
      ENDIF
C
      WRITE (OUTC,1013) AMPLAF, PHASAF, TOTLAF
      IF (MESSGE.NE.' ') WRITE (OUTC,'(1X,A)') MESSGE
      CALL RCLOCK(ELAPSE, CPU)
      WRITE (OUTC,1017) ELAPSE, CPU
      IF (SAVE.EQ.1) THEN
          L = LEN1(MODNAM)
          WRITE (OUTC,'(1X,A)')  '-------------------------------------'
          WRITE (OUTC,'(1X,2A)') 'Model saved in file: ', MODNAM(1:L)
          WRITE (OUTC,'(1X,A)')  '-------------------------------------'
      END IF
      WRITE (OUTC, '(1X)')
C
C Plot the thing graphically (if required) each time we print out the model.
C
      IF (DOPLOT) CALL DRAWIT
C
      RETURN
C
 1000 FORMAT (A,A,A,A)
 1001 FORMAT ('! ',A4,'-',A4,6X,F10.3,12X,A4,'-',A4,'-',A4,6X,F10.3)
 1002 FORMAT ('! ',A4,'-',A4,6X,5X,A4,13X,A4,'-',A4,'-',A4,6X,F10.3)
 1003 FORMAT ('! ',A4,'-',A4,6X,F10.3,12X,A4,'-',A4,'-',A4,6X,5X,A4)
 1004 FORMAT ('! ',A4,'-',A4,6X,5X,A4,13X,A4,'-',A4,'-',A4,6X,5X,A4)
 1005 FORMAT ('!',38X,A4,'-',A4,'-',A4,6X,F10.3)
 1006 FORMAT ('!',38X,A4,'-',A4,'-',A4,6X,5X,A4)
 1007 FORMAT ('!'/2(A,F10.3/),A,F10.3)
 1008 FORMAT ('! Source: ',2A8,' Day:',I4,', Year:',I5,F15.2,' MHz')
 1009 FORMAT(('!',I2,4(F10.3,A),F10.4,A,F10.3,A,I10))
 1010 FORMAT ('  ',4F11.4,F11.5,F11.4,I11)
 1011 FORMAT (80('-')/' Source: ',2A8,'  Iteration #',I3,10X,A)
 1012 FORMAT((' ',I2,4(F10.3,A),F10.4,A,F10.3,A,I10))
 1013 FORMAT (/' Agreement factors (amp, cl-phs, total):',3F10.3)
 1014 FORMAT ('!'/A,F10.3/A,5X,A4/A,F10.3)
 1015 FORMAT ('!'/A,5X,A4/A,F10.3/A,F10.3)
 1016 FORMAT (A,A,F10.2)
 1017 FORMAT (' Elapsed time: ',A,';  cpu time: ',F10.2)
      END
      SUBROUTINE BAGRFAC
C
C       By Richard S. Simon
C
C       This subroutine finds the agreement factors for the fit of a
C       model to VLB data.
C-----------------------------------------------------------------------
      INCLUDE 'modelfit20.inc'
      INTEGER IB, IC, ID
      REAL CSUM(MXTRI)
      REAL AMPSUM, CLOSUM, VSUM, DELVIS, DELCLO
C
C Amplitude agreement factors.
C
      AMPSUM = 0.
      DO 20 IB = 1, NUMBAS
          IF (NZVIS(IB) .GT. 0) THEN
            VSUM = 0.
            DO 10 ID = 1, NUMREC
              IF (WTVIS(ID,IB) .GT. 0.0) THEN
                DELVIS = MODVIS(ID,IB) - OBSVIS(ID,IB)
                VSUM   = VSUM + WTVIS(ID,IB)*DELVIS*DELVIS
              ENDIF
   10       CONTINUE
            VISAF(IB) = SQRT (VSUM / NZVIS(IB))
            AMPSUM = AMPSUM + VSUM
          ELSE
            VISAF(IB) = -1.
          ENDIF
   20 CONTINUE
      IF (NZAMP .GT. 0) THEN
          AMPLAF = SQRT (AMPSUM / NZAMP)
      ELSE
          AMPLAF = -1.
      END IF
C
C Closure phase agreement factors, for each triangle.
C
      DO 25 IC=1,NUMTRI
          CSUM(IC) = 0.0
   25 CONTINUE
      CLOSUM = 0.
      IF (NUMTRI .GT. 0) THEN
          DO 40 ID = 1,NUMREC
            IF (NINTRI(ID) .GT. 0) THEN
              DO 30 IC = 1,NUMTRI
                IF (WTCLO(ID,IC) .GT. 0.) THEN
                  DELCLO = ABS (MODCLO(ID,IC) - OBSCLO(ID,IC))
                  IF (DELCLO .GT. PI) DELCLO = DELCLO - TWOPI
                  CSUM(IC) = CSUM(IC) + WTCLO(ID,IC)*DELCLO*DELCLO
                  CLOSUM = CLOSUM+DOWNWT(ID)*WTCLO(ID,IC)*DELCLO*DELCLO
                ENDIF
   30         CONTINUE
            ENDIF
   40     CONTINUE
      END IF
      DO 50 IC=1,NUMTRI
          IF (NZCLO(IC) .GT. 0) THEN
            CLOAF(IC) = SQRT (CSUM(IC)/NZCLO(IC))
          ELSE
            CLOAF(IC) = -1.
          END IF
   50 CONTINUE
C
      IF (TNZCLO .GT. 0.) THEN
          PHASAF = SQRT (CLOSUM/TNZCLO)
      ELSE
          PHASAF = -1.
      END IF
C
C Total agreement factor for all data.
C
      TOTLAF = SQRT ((AWEIGT*AMPSUM + CWEIGT*CLOSUM) / 
     1         (AWEIGT*NZAMP + CWEIGT*TNZCLO))
C
      RETURN
      END
      SUBROUTINE BGETDAT
C
C       By Richard S. Simon
C
C       Version 1:      27-July-1981
C       Version 2:      16-April-1982   Fix a bug in TAKE dimension
C       Version 3:       7-July-1983    Extend to include Merge
C                               version 2 files and compute U and V
C                                       D. Meier
C                        3-August-1983  Add W to version 2 files
C       Version 4:       7-Dec-1992     Compute closure phases
C                                       if necessary [TJP].
C
C  This subroutine does the following:
C       1) Calls subroutine HEADX20 to get data parameters.
C       2) Reads in amplitude and closure phase data from data file.
C       3) Computes (or reads in) the U's and V's for a data set.
C-----------------------------------------------------------------------
      INCLUDE 'modelfit20.inc'
C
      INTEGER IOUT, IPROUT, IZERO, IDUMMY, IDUM, IUNITS, IFILE
      INTEGER IB, NWORDS, ID, IT, IC, I, K
      REAL    DUMMY, COSDEC, SINDEC, BX, BY, BHA, OBSIHA, RADIHA
      REAL    TAKE(MAXWPB*MXBAS + 2*MXTRI), SIDINT(MXREC), BZ(MXBAS)
      REAL    BXY(MXBAS), XIHA(MXBAS)
      LOGICAL CALCLO
      INTEGER P1,P2,P3, I1, I2, I3, NT
      INTEGER LOOKUP
      INTEGER CONST(MXTRI,3), NBI(MXBAS)
      REAL    CLPHS,CLERR
C
C
C       Call headx20 to get data parameters
      IOUT = 0
      IPROUT = 0
      CALL HEADX20(.FALSE., INDAT, IZERO, NDAY, NYEAR, IDUMMY, IDUM,
     +            IOUT, IUNITS, DUMMY, IFILE, IPROUT)
      IF (IFILE .NE. 0) THEN
          CALL ERROR('Something wrong with input file.')
      END IF
C
C       Certify that data is in flux units before proceeding.
      IF (IUNITS .NE. 1) THEN
          CALL PUTOUT('++WARNING++ Data have not been calibrated.')
      END IF
C
      WRITE (OUTC,1007) SOURCE(1),SOURCE(2)
      LAMBDA = VELC/FREQ
C
C       Load NSTA, NBAS, etc into COMMON
      NUMSTA = NSTA
      NUMBAS = NBAS
      IF (NTRI.EQ.0) THEN
C         -- set up to calculate closure phases
          NT = 0
          CALCLO = .TRUE.
          CALL PUTOUT('Calculating closure phases')
          DO 8 I=1,NBAS
              NBI(I) = 100*NAVIS(I) + NBVIS(I)
    8     CONTINUE
          DO 13 I3=3,NSTA
            DO 12 I2=2,I3-1
              DO 11 I1=1,I2-1
                  P1 = LOOKUP(100*I1 + I2, NBI, NBAS)
                  P2 = LOOKUP(100*I2 + I3, NBI, NBAS)
                  P3 = LOOKUP(100*I1 + I3, NBI, NBAS)
                  IF (P1.NE.0 .AND. P2.NE.0 .AND. P3.NE.0) THEN
                      NT = NT+1
                      NACLO(NT) = I1
                      NBCLO(NT) = I2
                      NCCLO(NT) = I3
                      CONST(NT,1) = P1
                      CONST(NT,2) = P2
                      CONST(NT,3) = P3
C                     write (*,*) i1,i2,i3, p1,p2,p3
                  END IF
  11          CONTINUE
  12        CONTINUE
  13      CONTINUE
          NUMTRI = NT
          WRITE (OUTC, '(1X,I10,1X,A)') NT, 'triangles'
      ELSE
          CALCLO = .FALSE.
          NUMTRI = NTRI
          CALL PUTOUT('Using closure phases from input file')
      END IF
      NINDCL = (NUMSTA-1)*(NUMSTA-2)/2
C
      COSDEC = COS(DECP)
      SINDEC = SIN(DECP)
      IF (NVER.EQ.1) THEN
          DO IB = 1, NUMBAS
            BX       = (STX(NBVIS(IB))-STX(NAVIS(IB))) / LAMBDA
            BY       = (STY(NBVIS(IB))-STY(NAVIS(IB))) / LAMBDA
            BZ(IB)   = (STZ(NBVIS(IB))-STZ(NAVIS(IB))) / LAMBDA
            BHA      = ATAN2(BY,BX)
            BXY(IB)  = SQRT (BX*BX + BY*BY)
            XIHA(IB) = (GASTM-RAP-BHA+HALFPI) * RADHRS
          ENDDO
      ENDIF
C
C       Read in the data, compute UT
      NWORDS = NDWPB*NUMBAS + 2*NTRI
      DO 150 ID = 1, MXREC
          READ (INDAT, END=160)  IT, (TAKE(I), I = 1, NWORDS)
          IF (IT .EQ. -1) GO TO 160
C         Compute UT in hours
          UT(ID) = (IT - IZERO) / 216000.
          SIDINT(ID) = UT(ID) * SIDR
C         Get amplitudes and errors and UV values
          DO 130 IB = 1, NUMBAS
            K = NDWPB*(IB-1)
            OBSVIS(ID,IB) = TAKE(K+1)
            WTVIS(ID,IB)= TAKE(K+2)
            IF (NVER.EQ.1) THEN
              OBSIHA = XIHA(IB) + SIDINT(ID)
              RADIHA = OBSIHA / RADHRS
              UDATA(ID,IB) = (-BXY(IB)*COS(RADIHA)) / CONV1
              VDATA(ID,IB) = (BZ(IB)*COSDEC -
     1          BXY(IB)*SINDEC*SIN(RADIHA)) / CONV1
            ELSEIF (NVER.EQ.2) THEN
              UDATA(ID,IB) = - TAKE(K+5) / CONV1
              VDATA(ID,IB) = - TAKE(K+6) / CONV1
            ENDIF
  130     CONTINUE
C         Get closure phases and errors
          IF (NUMTRI .GT. 0.) THEN
              IF (CALCLO) THEN
C                 -- Calculate closure phases
                  DO 170 IC=1,NUMTRI
                      P1 = NDWPB*CONST(IC,1) - NDWPB + 3
                      P2 = NDWPB*CONST(IC,2) - NDWPB + 3
                      P3 = NDWPB*CONST(IC,3) - NDWPB + 3
                      CLPHS = AMOD(TAKE(P1)+TAKE(P2)-TAKE(P3),360.0)
                      IF (CLPHS.GT.180.0)  CLPHS = CLPHS-360.0
                      IF (CLPHS.LE.-180.0) CLPHS = CLPHS+360.0
                      P1 = P1+1
                      P2 = P2+1
                      P3 = P3+1
                      IF (TAKE(P1).LE.0.0 .OR. TAKE(P2).LE.0.0 .OR.
     1                    TAKE(P3).LE.0.0) THEN
                          CLPHS = 0.0
                          CLERR = 0.0
                      ELSE
                          CLERR = SQRT(TAKE(P1)**2+TAKE(P2)**2+
     1                                 TAKE(P3)**2)
                      END IF
                      OBSCLO(ID,IC) = CLPHS*DEGRAD
                      WTCLO(ID,IC) =  CLERR*DEGRAD
  170              CONTINUE
              ELSE
C                 -- Read closure phases from file
                  DO 140 IC = 1, NUMTRI
                      K = NDWPB*NUMBAS + 2*(IC-1)
                      OBSCLO(ID,IC)  = TAKE(K+1) * DEGRAD
                      WTCLO(ID,IC) = TAKE(K+2)   * DEGRAD
  140             CONTINUE
              END IF
          END IF
  150 CONTINUE
C       Not all the data was read in, so issue a warning:
      WRITE (OUTC,1008) MXREC,MXREC,MXREC
      ID = MXREC + 1
C
  160 CONTINUE
      NUMREC = ID - 1
C
 1007 FORMAT (/' Source:',2A8)
 1008 FORMAT(/' Well, it looks like you have blown it.  This version',
     + ' of the program will',
     +/' only handle',I4,' separate data records, i.e.,',I4,
     + ' separate UT''s.  Thus,',
     +/' only the first',I4,' times will be used in the fit.  I',
     + ' suggest you use the',
     +/' program AVERAGE to average your data somewhat.  Good luck.')
C
      RETURN
      END
      SUBROUTINE BGETMOD(EDIT)
      INTEGER EDIT
C
C       By Richard S. Simon
C  This subroutine interacts with the user to:
C       1) Get input file name for existing model, or open a new file
C          if necessary.
C       2) Allows changes to the model to be made.
C       3) Allows user to specify which parameters to be varied
C          during model fit.
C  Returns
C     EDIT = 1 if file was edited (changed)
C     EDIT = 0 otherwise
C-----------------------------------------------------------------------
      INCLUDE 'modelfit20.inc'
      CHARACTER*64 NEWDSN,OLDDSN, TEMP
      CHARACTER*1  ANSWER
      DOUBLE PRECISION     NAMES(7), VALUES(7), ENDMRK
      INTEGER IVARY(6)
      INTEGER IER, L, LENGTH, ITYPE, I, J, MODE, JSTOP, K, ISTART
      REAL FLXSUM
      LOGICAL NEWMOD
      INTEGER GETIN, VLBOPE
      LOGICAL QUERY
C
      DATA    ENDMRK/'***EOL'/
      DATA    NAMES/'FLux','RAdius','THeta','AXis','ARatio','PHi',
     1              'TYpe'/
C
      EDIT = 0
      CALL PUTOUT(' ')
      IF (FRSTRY) THEN
   20     IER = GETIN(MODDSN, 'Input model file name (default=create '//
     1                'new model file): ', L)
          IF (IER.NE.1) STOP
          IF (L.LT.1) THEN
            NEWMOD = .TRUE.
          ELSE
            IER = VLBOPE(INMOD, MODDSN, 'MODEL', 'OLD', MODDSN)
            IF (IER.NE.1) GOTO 20
            NEWMOD = .FALSE.
          ENDIF
      ELSE
          OLDDSN = MODDSN
          LENGTH = INDEX(OLDDSN,' ') - 1
  120     IER = GETIN(NEWDSN, 'New input model file name (default='//
     1                OLDDSN(1:LENGTH) // '): ',L)
          IF (IER.NE.1) STOP
          IF (L.LE.0) NEWDSN = OLDDSN(1:LENGTH)
          IER = VLBOPE(INMOD, NEWDSN, 'MODEL', 'OLD', MODDSN)
          IF (IER.NE.1) GOTO 120
          NEWMOD = .FALSE.
      ENDIF
      IF (NEWMOD) THEN
   25     IER = GETIN(MODDSN, 'Name of new model file to be created: ', 
     1                L)
          IF (IER.NE.1) STOP
          CALL NEWFIL(MODDSN, TEMP)
          TEMP = MODDSN
          IER = VLBOPE(INMOD, MODDSN, 'MODEL', 'NEW', MODDSN)
          IF (IER.NE.1) GOTO 25
C         Type in the new model.
          WRITE (OUTC,1003)
          ITYPE = 0
          I = 0
   40     I = I+1
          IF (I .GT. MXMOD) GO TO 45
          CALL GTMODL(INC,ITYPE,FLUX(I),RADIUS(I),
     1    THETA(I),AXIS(I),RATIO(I),PHI(I), *45, 0)
          IF (FLUX(I) .NE. 0) THEN
            IF (ITYPE .EQ. 0) ITYPE = 1
            IF (ITYPE .LT. 1  .OR.  ITYPE .GT. 5) THEN
              CALL PUTOUT('Illegal component type in input model.'//
     +        '  Please repair!')
            ENDIF
            TYPE(I) = ITYPE
            IF (ITYPE .EQ. 5 .AND. RATIO(I) .NE. 0.) THEN
              WRITE (OUTC,1017)
              RATIO(I) = 0.
            END IF
            WRITE(INMOD,1004) FLUX(I),RADIUS(I),
     1      THETA(I),AXIS(I),RATIO(I),PHI(I),ITYPE
            GO TO 40
          END IF
   45     CONTINUE
      ELSE
          I = 0
   50     I = I+1
          IF (I .GT. MXMOD) GO TO 55
          CALL GTMODL(INMOD,ITYPE,FLUX(I),RADIUS(I),
     1    THETA(I),AXIS(I),RATIO(I),PHI(I), *55, 0)
          IF (FLUX(I) .NE. 0.) THEN
            IF (ITYPE .EQ. 0) ITYPE = 1
            IF (ITYPE .LT. 1  .OR.  ITYPE .GT. 5) THEN
              CALL PUTOUT('Illegal component type in input model.'//
     +        '  Please repair!')
            ENDIF
            TYPE(I) = ITYPE
            IF (ITYPE .EQ. 5 .AND. RATIO(I) .NE. 0.) THEN
              WRITE (OUTC,1017)
              RATIO(I) = 0.
            END IF
            GO TO 50
          END IF
   55     CONTINUE
      END IF
      CLOSE (UNIT=INMOD)
C
      NCOMP = I-1
      IF (NCOMP .LT. 1) CALL ERROR('No model components!')
   65 WRITE (OUTC,1006)
      WRITE (OUTC,1005)
      WRITE  (OUTC,1013) (I,FLUX(I),RADIUS(I),THETA(I),
     1AXIS(I),RATIO(I),PHI(I),TYPE(I), I = 1, NCOMP)
C
      IF (.NOT.QUERY('Is this acceptable? (Y or N): ')) THEN
          WRITE (OUTC,1008)
          READ  (INC,1000) ANSWER
          CALL UPCASE (ANSWER)
          IF (ANSWER .EQ. 'C' .OR. ANSWER .EQ. 'B') THEN
C           Changes to model
            WRITE (OUTC,1010)
            DO 80 I = 1, NCOMP
              DO 70 J = 1, 7
                VALUES(J) = -1000.
   70         CONTINUE
              WRITE(OUTC,1009) I,FLUX(I),RADIUS(I),
     1        THETA(I),AXIS(I),RATIO(I),PHI(I),TYPE(I)
              MODE = 2
              CALL KEYIN(NAMES, VALUES, 7, ENDMRK, MODE, INC, OUTC)
              IF (VALUES(1) .NE. -1000.)  FLUX(I) = VALUES(1)
              IF (VALUES(2) .NE. -1000.)RADIUS(I) = VALUES(2)
              IF (VALUES(3) .NE. -1000.) THETA(I) = VALUES(3)
              IF (VALUES(4) .NE. -1000.)  AXIS(I) = VALUES(4)
              IF (VALUES(5) .NE. -1000.) RATIO(I) = VALUES(5)
              IF (VALUES(6) .NE. -1000.)   PHI(I) = VALUES(6)
              IF (VALUES(7) .NE. -1000.)  TYPE(I) = VALUES(7)
              IF (TYPE(I) .EQ. 5 .AND. RATIO(I) .NE. 0.) THEN
                RATIO(I) = 0.
                WRITE (OUTC,1018)
              END IF
   80       CONTINUE
C
C           Remove components with FLUX=0 from model
   81       JSTOP = NCOMP-1
            DO 83 J = 1, JSTOP
              IF (FLUX(J) .EQ. 0.) GO TO 82
   83       CONTINUE
            GO TO 86
   82       DO 84 K = J, NCOMP-1
                FLUX(K) = FLUX(K+1)
                RADIUS(K)  = RADIUS(K+1)
                THETA(K) = THETA(K+1)
                AXIS(K)   = AXIS(K+1)
                RATIO(K)   = RATIO(K+1)
                PHI(K) = PHI(K+1)
                TYPE(K) = TYPE(K+1)
   84       CONTINUE
            NCOMP = NCOMP-1
            GO TO 81
   86       IF (FLUX(NCOMP) .EQ. 0.) NCOMP = NCOMP-1
C
          END IF
          IF (ANSWER .EQ. 'A' .OR. ANSWER .EQ. 'B') THEN
C           Additions:
            WRITE (OUTC,1011)
            WRITE (OUTC,1013) (I,FLUX(I),RADIUS(I),
     1      THETA(I),AXIS(I),RATIO(I),PHI(I),TYPE(I), I = 1, NCOMP)
            ITYPE = 0
            I = NCOMP
   90       I = I+1
            IF (I .LE. MXMOD) THEN
              WRITE (OUTC,1012) I
              CALL GTMODL (INC,ITYPE,FLUX(I),RADIUS(I),
     1        THETA(I),AXIS(I),RATIO(I),PHI(I), *95, 0)
              IF (FLUX(I) .NE. 0) THEN
                IF (ITYPE .EQ. 0) ITYPE = 1
                IF (ITYPE .LT. 1  .OR.  ITYPE .GT. 5) THEN
                  CALL PUTOUT('Illegal component type in input '//
     +            'model.  Please repair!')
                ENDIF
                TYPE(I) = ITYPE
                IF (ITYPE .EQ. 5 .AND. RATIO(I) .NE. 0.) THEN
                  WRITE (OUTC,1017)
                  RATIO(I) = 0.
                END IF
                GO TO 90
              END IF
            ELSE
              CALL PUTOUT('That is the maximum number of model'//
     1        ' components, at present.')
            ENDIF
   95       NCOMP = I-1
          END IF
          IF (NCOMP .LT. 1) CALL ERROR('There are no model '//
     +  'components left!')
C
          ISTART = NCOMP + 1
          DO 105 I = ISTART, MXMOD
            FLUX(I) = 0.
  105     CONTINUE
C
C         -- The model has been edited
          EDIT = 1
          GO TO 65
      END IF
C
 110  NVPAR = 0
      WRITE (OUTC,1014)
      WRITE (OUTC,1005)
      DO 150 I = 1, NCOMP
  115     WRITE(OUTC,1113)I,FLUX(I),RADIUS(I),
     1      THETA(I),AXIS(I),RATIO(I),PHI(I),TYPE(I)
          DO 130 J = 1, 6
            IVARY(J) = 0
            VARY(J,I) = .FALSE.
  130     CONTINUE
          READ (INC,1015,ERR=115)IVARY
C         Make sure AR not varied for line component; correct if necessary.
          IF (TYPE(I) .EQ. 5) THEN
            DO 145 J = 1, 6
              IF (IVARY(J) .EQ. 5) THEN
                IVARY(J) = 0
                WRITE (OUTC,1016)
              ENDIF
  145       CONTINUE
          END IF
C
          DO 140 J = 1, 6
            IF (IVARY(J) .GT. 0 .AND. IVARY(J) .LT. 7) THEN
               IF (VARY(IVARY(J), I)) THEN
                  WRITE (OUTC, 1019) IVARY(J)
                  GO TO 110
               ELSE
                  NVPAR = NVPAR+1
                  VARY(IVARY(J),I) = .TRUE.
               END IF
            END IF
  140     CONTINUE
  150 CONTINUE
      IF (NVPAR .LE. 0) THEN
         CALL PUTOUT(' ')
         CALL PUTOUT('No varying parameters selected!  Please try '//
     +  'again.')
         GO TO 110
      END IF
C
C
      FLXSUM = 0.0
      DO 108 J = 1, NCOMP
          FLXSUM = FLXSUM + FLUX(J)
C         Save the initial model.
          FLSTRT(J) = FLUX(J)
          RASTRT(J) = RADIUS(J)
          THSTRT(J) = THETA(J)
          AXSTRT(J) = AXIS(J)
          ARSTRT(J) = RATIO(J)
          PHSTRT(J) = PHI(J)
          RTHETA(J) = DEGRAD*THETA(J)
          RPHI(J)   = DEGRAD*PHI(J)
  108 CONTINUE
C
 1000 FORMAT (A,A,A,A)
 1003 FORMAT (' Enter new model, 1 component (6 parameters) per line,'/
     +        ' Flux, Rad., Theta, Axis, Ratio, Phi, all separated ',
     +        'by one or more blanks;'/
     +        ' End input with ctrl-D (UNIX) or ctrl-Z (VMS):')
 1004 FORMAT (6F10.3, I5)
 1005 FORMAT (12X,'Flux',3X,'Radius',4X,'Theta',5X,'Axis',4X,'Ratio',
     +          6X,'Phi','  Type')
 1006 FORMAT (//' Starting Model:')
 1008 FORMAT (' Do you wish to make Changes/Deletions, Additions,',
     +          ' or Both (C, A, or B) ? ',$)
 1009 FORMAT (' Changes to ',I2,')',4X,6F9.3,I5)
 1010 FORMAT (//' Enter changes, 1 component at a time using Free',
     +        ' Format (e.g. "FL=3.1  TH=113").'
     +        /' To delete a component, change the flux to 0.0'
     +        /' To go to next component, hit RETURN key.'
     +        /23X,'FL',7X,'RA',7X,'TH',7X,'AX',7X,'AR',7X,'PH',7X,'TY')
 1011 FORMAT (/' Enter new components (separate parameter values by',
     +         ' blanks);'
     +        /' End input with ctrl-D (UNIX) or ctrl-Z (VMS):')
 1012 FORMAT (1X,I5,')',$)
 1013 FORMAT (1X,3X,I2,')',6F9.3,I5)
 1113 FORMAT (1X,3X,I2,')',6F9.3,I5,'   ',$)
 1014 FORMAT (//' Enter the numbers of the parameters to be',
     +          ' varied:'/' (e.g. 125 varies Flux, Radius, and ',
     +          'Axial Ratio.)')
 1015 FORMAT (6I1)
 1016 FORMAT ( ' Sorry. You cannot vary A.R. for line comp. It will be',
     +          ' held fixed.')
 1017 FORMAT (' Sorry! For line component, A.R. will be set = 0.00 .')
 1018 FORMAT (' Note: A.R. set=0 for line component.')
 1019 FORMAT (/' Parameter', I2, ' specified more than once.  ',
     +  'Please try again.')
      RETURN
      END
      SUBROUTINE BMODAP
C
C By Richard S. Simon
C
C Version 1:  27-Jul-1981
C Version 2:  04-Feb-1982         Add underlying model, if any
C
C This subroutine calculates model amplitudes and closure phases,
C given that IMVIS and REVIS have been found by BVIS.
C
C Reorganized for vectorization by TJP, 15-Mar-1988.
C modified for new BTRIANGLE by TJP, 5-Nov-1990.
C-----------------------------------------------------------------------
      INCLUDE 'modelfit20.inc'
C
      REAL MODPHA(MXREC,MXBAS)
      REAL TVISR(MXREC,MXBAS), TVISI(MXREC,MXBAS)
      REAL AMP, X, Y
      INTEGER IB, ID, J, IC
C
      AMP(X,Y) = SQRT(X*X + Y*Y)
C
C Calculate the complex visibilities of the model by summing the
C contributions of the components; convert to polar representation.
C
C First add in the contribution from the underlying model.
C
      IF (UNDMOD) THEN
          DO IB=1,NUMBAS
          DO ID=1, NUMREC
              TVISR(ID,IB) = UNREVIS(ID,IB)
              TVISI(ID,IB) = UNIMVIS(ID,IB)
          END DO
          END DO
      ELSE
          DO IB=1,NUMBAS
          DO ID=1, NUMREC
              TVISR(ID,IB) = 0.0
              TVISI(ID,IB) = 0.0
          END DO
          END DO
      END IF
C
C Add in the contributions from individual components.
C
      DO J=1, NCOMP
          DO IB=1,NUMBAS
          DO ID=1, NUMREC
              TVISR(ID,IB) = TVISR(ID,IB) + REVIS(J,ID,IB)
              TVISI(ID,IB) = TVISI(ID,IB) + IMVIS(J,ID,IB)
          END DO
          END DO
      END DO
C
C Convert to polar representation.
C
      DO IB=1,NUMBAS
      DO ID=1, NUMREC
          MODVIS(ID,IB) = AMP(TVISR(ID,IB), TVISI(ID,IB)) 
          IF (MODVIS(ID,IB) .EQ. 0.0) THEN
              MODPHA(ID,IB) = 0.
          ELSE
              MODPHA(ID,IB) = -ATAN2(TVISI(ID,IB),TVISR(ID,IB))
          END IF
      END DO
      END DO
C
C Calculate the closure phases of the model.
C
      DO IC=1,NUMTRI
      DO ID=1,NUMREC
          MODCLO(ID,IC) = MODPHA(ID,ICBSNO(1,IC)) +
     1                    MODPHA(ID,ICBSNO(2,IC)) - 
     2                    MODPHA(ID,ICBSNO(3,IC))
          IF (MODCLO(ID,IC) .LT. -PI .OR. MODCLO(ID,IC) .GT. PI) THEN
              MODCLO(ID,IC) = AMOD( MODCLO(ID,IC), TWOPI )
              IF (MODCLO(ID,IC) .LT. -PI) THEN
                  MODCLO(ID,IC) = MODCLO(ID,IC) + TWOPI
              ELSE IF (MODCLO(ID,IC) .GT. PI) THEN
                  MODCLO(ID,IC) = MODCLO(ID,IC) - TWOPI
              END IF
          END IF
      END DO
      END DO
C
      RETURN
      END
      SUBROUTINE BTRIANGLE
C-----------------------------------------------------------------------
C Finds which triangles are independent in a VLB data set.
C
C Revised by TJP 1990 Nov 5.
C     The program was incorrectly assuming the baselines were in
C     canonical order. It now uses the header information to determine
C     which baselines must be summed to get each closure phase.
C Revised by TJP 1990 Nov 5.
C     New algorithm for determining number of independent closure
C     phases. Each triangle which makes use of a baseline that has not
C     been used in a previous triangle is independent; otherwise it
C     can be expressed as a combination of the previous triangles.
C-----------------------------------------------------------------------
      INCLUDE 'modelfit20.inc'
C
      INTEGER   BASE(MXBAS), IB, IT, ID, LOOKUP
      LOGICAL   BUSED(MXBAS)
C
C Make baseline labels for the NBAS baselines.
C
      DO IB=1,NBAS
          BASE(IB) = 100*NAVIS(IB) + NBVIS(IB)
      END DO
C
C Find the baseline numbers ICBSNO(*,NUMTRI) for 
C triangle NUMTRI by making baseline labels and looking them up in
C the list of available baselines.
C
      DO IT=1,NUMTRI
          ICBSNO(1,IT) = LOOKUP(100*NACLO(IT) + NBCLO(IT), BASE, NBAS)
          ICBSNO(2,IT) = LOOKUP(100*NBCLO(IT) + NCCLO(IT), BASE, NBAS)
          ICBSNO(3,IT) = LOOKUP(100*NACLO(IT) + NCCLO(IT), BASE, NBAS)
          IF (ICBSNO(1,IT).EQ.0 .OR. ICBSNO(2,IT).EQ.0 .OR.
     +        ICBSNO(3,IT).EQ.0) CALL ERROR('improper triangle list')
      END DO
C=    do i=1,numtri
C=      type *,icbsno(1,i),icbsno(2,i),icbsno(3,i)
C=    end do
C
C Find the independent triangles in each record, NINTRI(ID).
C
      DO ID=1,NUMREC
          DO IB=1,NBAS
              BUSED(IB) = .FALSE.
          END DO
          NINTRI(ID) = 0
          DO IT=1,NUMTRI
              IF (WTCLO(ID,IT) .GT. 0.0) THEN
                  IF (BUSED(ICBSNO(1,IT)) .AND.
     +                BUSED(ICBSNO(2,IT)) .AND.
     +                BUSED(ICBSNO(3,IT))) THEN
C                       -- not independent
                  ELSE
C                       -- independent
                      BUSED(ICBSNO(1,IT)) = .TRUE.
                      BUSED(ICBSNO(2,IT)) = .TRUE.
                      BUSED(ICBSNO(3,IT)) = .TRUE.
                      NINTRI(ID) = NINTRI(ID)+1
                  END IF
              END IF
          END DO
C=        type *, id, nintri(id)
      END DO
C
      RETURN
      END
      SUBROUTINE BWTDATA
C
C       By Richard S. Simon
C
C       This subroutine is part of program MODELFIT, and does the following:
C       1)  Compute the weights for each data point.
C       2)  Find the number of non-zero points in the data set.
C
C       Version N+1:    27-Jul-1981
C-----------------------------------------------------------------------
      INCLUDE 'modelfit20.inc'
      INTEGER COUNT, ID, IB, IC
C
      DO 30 ID = 1, NUMREC
          NUMCLO(ID) = 0
   30 CONTINUE
C
C Find the amplitude weights.
C
      NZAMP = 0
      DO 60 IB = 1, NUMBAS
          COUNT = 0
          DO 50 ID = 1, NUMREC
            IF (WTVIS(ID,IB) .GT. 0.0) THEN
              COUNT = COUNT + 1
              WTVIS(ID,IB) = 1. / (WTVIS(ID,IB))**2
            ENDIF
   50     CONTINUE
          NZVIS(IB) = COUNT
          NZAMP = NZAMP + COUNT
   60 CONTINUE
C
C Find the weights for the closure phase data, if any.
C
      DO 80 IC = 1, NUMTRI
          COUNT = 0
          DO 70 ID = 1, NUMREC
            IF (WTCLO(ID,IC) .GT. 0.0) THEN
              COUNT = COUNT + 1
              NUMCLO(ID) = NUMCLO(ID) + 1
              WTCLO(ID,IC) = 1./ (WTCLO(ID,IC))**2
            ENDIF
   70     CONTINUE
          NZCLO(IC) = COUNT
   80 CONTINUE
      TNZCLO = 0.
      DO 90 ID = 1,NUMREC
          IF (NUMCLO(ID) .GT. 0) THEN
            DOWNWT(ID) = REAL(NINTRI(ID)) / REAL(NUMCLO(ID))
            TNZCLO = TNZCLO + NINTRI(ID)
          ELSE
            DOWNWT(ID) = 0
          ENDIF
   90 CONTINUE
      TNZDAT = NZAMP + TNZCLO
C
      RETURN
      END
      SUBROUTINE BVIS(J)
      INTEGER J
C
C By Richard S. Simon
C
C Finds the contribution to the complex visibility from a single model
C component.  Most of MODELFIT's cpu time is used in this routine, which
C must therefore be coded carefully. Recoded 14-Mar-1989 by TJP to
C put the loops inside the switch on component type; this may improve
C vectorization.
C-----------------------------------------------------------------------
      INCLUDE 'modelfit20.inc'
C
      REAL C1
      PARAMETER (C1=-3.559707)
      INTEGER IB, ID
      REAL SPHI, CPHI, SX, CX, COMVIS, COMPHA, QQ, XX
      REAL BESJ0, BESJ1
      REAL Q, A, R, U, V
C
      Q(A,SPHI,CPHI,R,U,V)= TWOPI*SQRT((V*CPHI+U*SPHI)**2
     1+ (R*(U*CPHI-V*SPHI))**2) * A
C
      SPHI = SIN(RPHI(J))
      CPHI = COS(RPHI(J))
      SX = TWOPI*RADIUS(J)*SIN(RTHETA(J))
      CX = TWOPI*RADIUS(J)*COS(RTHETA(J))
      IF (AXIS(J).EQ.0.0) THEN
          DO IB=1,NUMBAS
          DO ID=1,NUMREC
              COMPHA = SX*UDATA(ID,IB) + CX*VDATA(ID,IB)
              COMVIS = FLUX(J)
              REVIS(J,ID,IB)  = COMVIS*COS(COMPHA)
              IMVIS(J,ID,IB)  = COMVIS*SIN(COMPHA)
          END DO
          END DO
C
      ELSE IF (TYPE(J) .EQ. 1) THEN
          DO IB=1,NUMBAS
          DO ID=1,NUMREC
              COMPHA = SX*UDATA(ID,IB) + CX*VDATA(ID,IB)
              COMVIS = FLUX(J) * (EXP (C1 * AXIS(J)**2 *
     1          ((VDATA(ID,IB)*CPHI + UDATA(ID,IB)*SPHI)**2 +
     2          (RATIO(J) * (UDATA(ID,IB)*CPHI-VDATA(ID,IB)*SPHI))**2)))
              REVIS(J,ID,IB)  = COMVIS*COS(COMPHA)
              IMVIS(J,ID,IB)  = COMVIS*SIN(COMPHA)
          END DO
          END DO
C
      ELSE IF (TYPE(J) .EQ. 2) THEN
          DO IB=1,NUMBAS
          DO ID=1,NUMREC
              COMPHA = SX*UDATA(ID,IB) + CX*VDATA(ID,IB)
              QQ = 0.5 * Q(AXIS(J),SPHI,CPHI,RATIO(J),
     1               UDATA(ID,IB),VDATA(ID,IB))
              IF (ABS(QQ).LT.0.01) THEN
                  COMVIS = FLUX(J)*(1.0 - QQ*QQ/8. + QQ*QQ*QQ*QQ/192.)
              ELSE
                  COMVIS = 2. * FLUX(J) * BESJ1(QQ)/QQ
              END IF
              REVIS(J,ID,IB)  = COMVIS*COS(COMPHA)
              IMVIS(J,ID,IB)  = COMVIS*SIN(COMPHA)
          END DO
          END DO
C
      ELSE IF (TYPE(J) .EQ. 3) THEN
          DO IB=1,NUMBAS
          DO ID=1,NUMREC
              COMPHA = SX*UDATA(ID,IB) + CX*VDATA(ID,IB)
              QQ = 0.5 * Q(AXIS(J),SPHI,CPHI,RATIO(J),UDATA(ID,IB),
     1             VDATA(ID,IB))
              IF (ABS(QQ).LT.0.01) THEN
                  COMVIS = FLUX(J)*(1.0 - QQ*QQ/10. + QQ*QQ*QQ*QQ/280.)
              ELSE
                  COMVIS = 3. * FLUX(J) * (SIN(QQ)-QQ*COS(QQ)) /
     1               (QQ*QQ*QQ)
              END IF
              REVIS(J,ID,IB)  = COMVIS*COS(COMPHA)
              IMVIS(J,ID,IB)  = COMVIS*SIN(COMPHA)
          END DO
          END DO

      ELSE IF (TYPE(J) .EQ. 4) THEN
          DO IB=1,NUMBAS
          DO ID=1,NUMREC
              COMPHA = SX*UDATA(ID,IB) + CX*VDATA(ID,IB)
              COMVIS = FLUX(J) * BESJ0(Q(AXIS(J),SPHI,
     1          CPHI,RATIO(J),UDATA(ID,IB),VDATA(ID,IB))/2.0)
              REVIS(J,ID,IB)  = COMVIS*COS(COMPHA)
              IMVIS(J,ID,IB)  = COMVIS*SIN(COMPHA)
          END DO
          END DO
C
      ELSE IF (TYPE(J) .EQ. 5) THEN
          DO IB=1,NUMBAS
          DO ID=1,NUMREC
              COMPHA = SX*UDATA(ID,IB) + CX*VDATA(ID,IB)
              XX = (UDATA(ID,IB)*SPHI + VDATA(ID,IB)*CPHI)*PI*AXIS(J)
              IF (ABS(XX).LT.0.01) THEN
                  COMVIS = FLUX(J)*(1.0 - XX*XX/6. + XX*XX*XX*XX/120.)
              ELSE
                  COMVIS = FLUX(J) * SIN(XX)/XX
              END IF
              REVIS(J,ID,IB)  = COMVIS*COS(COMPHA)
              IMVIS(J,ID,IB)  = COMVIS*SIN(COMPHA)
          END DO
          END DO
C
      ELSE
          CALL ERROR ('Internal error in MODELFIT (BVIS).')
      END IF
C
      RETURN
      END
      SUBROUTINE FIXMOD
C
C       By Richard S. Simon
C-----------------------------------------------------------------------
      INCLUDE 'modelfit20.inc'
      INTEGER J
C
      DO 10 J=1,NCOMP
          IF (RADIUS(J) .LT. 0.) THEN
              RADIUS(J) = -RADIUS(J)
              RTHETA(J) =  RTHETA(J) + PI
          END IF
          RTHETA(J) = AMOD(RTHETA(J),TWOPI)
          THETA(J) = RTHETA(J)/DEGRAD
          AXIS(J) = ABS(AXIS(J))
          RATIO(J) = ABS(RATIO(J))
          IF (RATIO(J) .GT. 1.) THEN
              AXIS(J)  = AXIS(J)* RATIO(J)
              RATIO(J) = 1. / RATIO(J)
              RPHI(J)  = RPHI(J) + HALFPI
          END IF
          RPHI(J) = AMOD(RPHI(J),PI)
          PHI(J) = RPHI(J)/DEGRAD
  10  CONTINUE
C
      RETURN
      END
      SUBROUTINE GRADLS
C
C By Richard S. Simon             July 29, 1981
C
C This subroutine is loosely based on the subroutine GRADLS in
C "Data Reduction and Error Analysis for the Physical Sciences" by
C P. R. Bevington (1969), pg. 221.
C By moving along gradient of A.F., it minimizes the A.F.
C-----------------------------------------------------------------------
      INCLUDE 'modelfit20.inc'
      REAL GRAD(6,MXMOD)
      LOGICAL QUERY
      REAL AFLIM, OLDAF, AF1, AF2, SUM, DELTA, SAVE
      INTEGER J, ITER, K, NBACK
C
      METHOD = 'Gradient search'
      AFLIM = 0.01
      DO 10 J=1,NCOMP
          DELTAP(1,J) = 0.05*FLUX(J)
          DELTAP(2,J) = 0.05*RADIUS(J) + 0.05
          DELTAP(3,J) = 5.*DEGRAD
          DELTAP(4,J) = 0.05*AXIS(J) + 0.05
          IF (RATIO(J) .GT. 0.5) THEN
              DELTAP(5,J) = -0.05
          ELSE
              DELTAP(5,J) =  0.05
          END IF
          DELTAP(6,J) = 5.*DEGRAD
   10 CONTINUE
C
      DO 210 ITER = 1, MAXIT
          OLDAF = TOTLAF
          AF1   = TOTLAF
          SUM   = 0.
C
C         Find components of gradient:
          DO 39 J=1,NCOMP
            DO 38 K=1,6
              IF(VARY(K,J)) THEN
                DELTA = 0.1 * DELTAP(K,J)
                PARM(J,K) = PARM(J,K) + DELTA
                CALL BVIS(J)
                CALL BMODAP
                CALL BAGRFAC
                PARM(J,K) = PARM(J,K) - DELTA
                GRAD(K,J) = AF1 - TOTLAF
                SUM = SUM + GRAD(K,J)*GRAD(K,J)
              ENDIF
   38       CONTINUE
   39     CONTINUE
          IF (SQRT(SUM) .EQ. 0.) GO TO 220
C
C         Normalize the gradient components and change each variable parameter.
          DO 52 J = 1,NCOMP
            DO 50 K = 1,6
              IF (VARY(K,J)) THEN
                GRAD(K,J) = DELTAP(K,J)*GRAD(K,J)/SQRT(SUM)
C               Move to a new point.
                PARM(J,K) = PARM(J,K) + GRAD(K,J)
              ENDIF
   50       CONTINUE
            CALL BVIS(J)
   52     CONTINUE
C
          CALL BMODAP
          CALL BAGRFAC
C
C         Make sure AF decreases.
          IF (AF1 .LE. TOTLAF) THEN
            DO 68 NBACK = 1, 15
C             Moved too far, so decrease step size.
C             Moved too far, so backup in steps of 1/2, until AF has decreased
C             to less than its original value, then rearrange parameters so that
C             it looks like step size was correct.
              AF2 = TOTLAF
              DO 64 J=1,NCOMP
                DO 62 K=1,6
                  IF (VARY(K,J)) THEN
                    GRAD(K,J) = GRAD(K,J) / 2.
                    PARM(J,K) = PARM(J,K) - GRAD(K,J)
                    DELTAP(K,J) = DELTAP(K,J) / 2.
                  ENDIF
   62           CONTINUE
                CALL BVIS(J)
   64         CONTINUE
              CALL BMODAP
              CALL BAGRFAC
              IF (TOTLAF .LT. AF1) THEN
                SAVE = AF2
                AF2 = TOTLAF
                TOTLAF = SAVE
                DO 66 J = 1,NCOMP
                  DO 65 K = 1, 6
                    IF (VARY(K,J)) PARM(J,K) = PARM(J,K) + GRAD(K,J)
   65             CONTINUE
   66           CONTINUE
                GO TO 80
              ENDIF
   68       CONTINUE
            GO TO 200
          ENDIF
C
C         Increment parameters until AF starts to increase.
          AF2 = TOTLAF
   70     DO 72 J = 1, NCOMP
            DO 71 K = 1,6
              IF (VARY(K,J)) PARM(J,K) = PARM(J,K) + GRAD(K,J)
   71       CONTINUE
            CALL BVIS(J)
   72     CONTINUE
          CALL BMODAP
          CALL BAGRFAC
          IF (TOTLAF .LT. AF2) THEN
            AF1 = AF2
            AF2 = TOTLAF
            GO TO 70
          ENDIF
C
C         Find minimum of parabola defined by last three points
   80     IF (TOTLAF .GT. AF2) THEN
            DELTA = 1./ (1.+(AF1-AF2)/(TOTLAF-AF2)) + 0.5
          ELSE
C           Backup if AF hasn't decreased
            DELTA = 1.
          ENDIF
          DO 93 J = 1,NCOMP
            DO 92 K = 1,6
              IF (VARY(K,J)) PARM(J,K) = PARM(J,K)-GRAD(K,J)*DELTA
   92       CONTINUE
            CALL BVIS(J)
   93     CONTINUE
          CALL BMODAP
          CALL BAGRFAC
C
          IF (TOTLAF .GT. AF2) THEN
C           Model diverged, so backup to lowest minimum.
            DO 102 J = 1, NCOMP
              DO 101 K = 1,6
                IF(VARY(K,J)) PARM(J,K) = PARM(J,K)+GRAD(K,J)*(DELTA-1.)
  101         CONTINUE
              CALL BVIS(J)
  102       CONTINUE
            CALL BMODAP
            CALL BAGRFAC
          ENDIF
C
  200     CONTINUE
C         -- Print current model parameters and fit
          JITER = ITER
          CALL BAFOUT(0)
C
C         If AF doesn't change significantly by 2nd iteration, quit here
          IF(ITER.GT.1 .AND. (OLDAF-TOTLAF)/OLDAF .LT. AFLIM .AND.
     +    OLDAF .GT. TOTLAF) THEN
            IF (QUERY('Convergence is slow. Continue? (Y or N): ')) THEN
                AFLIM = AFLIM/2.
            ELSE
                GOTO 220
            END IF
          ENDIF
          IF (CCFLAG) GO TO 225
  210 CONTINUE
      CALL PUTOUT(' ')
      CALL PUTOUT('Iteration halted after maximum no. '//
     1            'of iterations along gradient.')
      GO TO 230
  220 CALL PUTOUT(' ')
      CALL PUTOUT('Iteration halted;  model seems to have '//
     1            'converged along gradient.')
      GO TO 230
  225 CALL PUTOUT(' ')
      CALL PUTOUT('Iteration halted by user.')
      CALL CTRLCEN
  230 CALL FIXMOD
      RETURN
C
      END
      SUBROUTINE GRIDLS
C
C       By Richard S. Simon             July 27, 1981
C
C       This subroutine is based on the subroutine GRIDLS in
C       "Data Reduction and Error Analysis for the Physical Sciences" by
C       P. R. Bevington (1969), pg. 212.
C       By trial and error, it minimizes the A.F.
C
C Modified to prevent Flux going negative; TJP 30-Oct-93.
C-----------------------------------------------------------------------
      INCLUDE 'modelfit20.inc'
      LOGICAL QUERY
      INTEGER J, K, ITER
      REAL AFLIM, OLDAF, DELTA, AF1, PREAF, PREPRM, PREDEL, AF2
      REAL SAVE, FN
C
      METHOD = 'BRUTE FORCE !!!'
      AFLIM = 0.003
      DO 10 J=1,NCOMP
          DELTAP(1,J) = 0.1*FLUX(J)
          DELTAP(2,J) = 0.1*RADIUS(J) + 0.1
          DELTAP(3,J) = 5.0*DEGRAD
          DELTAP(4,J) = 0.1*AXIS(J)
          IF (RATIO(J) .GT. 0.5) THEN
              DELTAP(5,J) = -0.1
          ELSE
              DELTAP(5,J) =  0.1 * RATIO(J) + .05
          END IF
          DELTAP(6,J) = 10.*DEGRAD
   10 CONTINUE
C
      DO 210 ITER = 1,MAXIT
          OLDAF = TOTLAF
C
          DO 200 J=1,NCOMP
            DO 190 K=1,6
              IF(VARY(K,J)) THEN
                DELTA = DELTAP(K,J)
                AF1 = TOTLAF
                PREAF  = TOTLAF
                PREPRM = PARM(J,K)
                PREDEL = DELTA
                PARM(J,K) = PREPRM + DELTA
                CALL BVIS(J)
                CALL BMODAP
                CALL BAGRFAC
                AF2 = TOTLAF
C               -- Reverse direction of search if AF increasing
                IF (AF2 .GT. AF1) THEN
                  DELTA = -DELTA
                  PARM(J,K) = PARM(J,K) + DELTA
                  SAVE = AF1
                  AF1 = AF2
                  AF2 = SAVE
                ENDIF
C               -- Increment PARM(J,K) until AF increases
                FN = 0.
   60           FN = FN + 1.
                PARM(J,K) = PARM(J,K) + DELTA
C               -- Flux may not be negative
                IF (K.EQ.1) PARM(J,K) = ABS(PARM(J,K))
                CALL BVIS(J)
                CALL BMODAP
                CALL BAGRFAC
                IF (TOTLAF .LT. AF2) THEN
                  AF1 = AF2
                  AF2 = TOTLAF
                  GO TO 60
                ELSEIF (TOTLAF .GT. AF2) THEN
C                 Find minimum of parabola defined by last three points
                  DELTA = DELTA * (1./(1.+(AF1-AF2)/(TOTLAF-AF2))+.5)
                  PARM(J,K) = PARM(J,K) - DELTA
                  DELTAP(K,J) = DELTAP(K,J) * FN/3.
C                 SIGMAP(K,J) = DELTAP(K,J) *
C     1           SQRT(2./((TNZDAT-NVPAR)*(TOTLAF-2.*AF2+AF1)))
                ENDIF
                CALL BVIS(J)
C               -- AF for final set of parameters
                CALL BMODAP
                CALL BAGRFAC
                IF (TOTLAF .GE. PREAF) THEN
C                 Model diverged, so reset model and decrease step size.
                  PARM(J,K) = PREPRM
                  DELTAP(K,J) = PREDEL/5.
                  CALL BVIS(J)
                  CALL BMODAP
                  CALL BAGRFAC
                ENDIF
              ENDIF
              IF (CCFLAG) GO TO 225
  190       CONTINUE
  200     CONTINUE
C         -- Print current model parameters and fit
          JITER = ITER
          CALL BAFOUT(0)
C
C If AF doesn't change significantly by 4th iteration, quit here.
C
          IF (ITER.GT.3 .AND. (OLDAF-TOTLAF)/OLDAF .LT. AFLIM .AND.
     +        OLDAF .GE. TOTLAF) THEN
            IF (QUERY('Convergence is slow. Continue? (Y or N): ')) THEN
                  AFLIM = AFLIM/2.
              ELSE
                  GOTO 220
              END IF
          END IF
          OLDAF = TOTLAF
  210 CONTINUE
C
      CALL PUTOUT(' ')
      CALL PUTOUT('Iteration halted after maximum no. of iterations.')
      GOTO 230
C
  220 CALL PUTOUT(' ')
      CALL PUTOUT('Iteration halted;  model seems to have converged.')
      GOTO 230
C
  225 CALL PUTOUT(' ')
      CALL PUTOUT('Iteration halted by user.')
      CALL CTRLCEN
      GOTO 230
C
  230 CALL FIXMOD
      RETURN
C
      END
      SUBROUTINE UNDERMOD
C
C By Richard S. Simon
C
C This subroutine is used by modelfit to calculate, store, and
C retrieve the contribution to a model fit from an underlying model.
C This underlying model is held constant while a few additional
C components may be varied in the usual way.
C
C Version 01:  04-Feb-1982
C-----------------------------------------------------------------------
      INCLUDE 'modelfit20.inc'
C
      REAL C1
      PARAMETER (C1=-3.559707)
      REAL  Q, A, CPHI, SPHI, R, U, V, FLXSUM, UTHETA, UPHI, COMPHA
      REAL  COMVIS, QQ, XX
      REAL  UFLUX(MXUNDM),URADIUS(MXUNDM),URTHETA(MXUNDM)
      REAL  UAXIS(MXUNDM),URATIO(MXUNDM),URPHI(MXUNDM)
      INTEGER I, IER, J, L, IREC, NUNDM, IB, ID
      INTEGER ITYPE(MXUNDM)
      LOGICAL QUERY
      INTEGER GETIN
      REAL BESJ0, BESJ1
C
C The following equivalence is strictly to save storage space
C
C     EQUIVALENCE (UFLUX(1),IMVIS(1)),(URADIUS(1),IMVIS(MXUNDM+1)),
C    1(URTHETA(1),IMVIS(2*MXUNDM+1)),(UAXIS(1),IMVIS(3*MXUNDM+1)),
C    2(URATIO(1),IMVIS(4*MXUNDM+1)),(URPHI(1),IMVIS(5*MXUNDM+1)),
C    3(ITYPE(1),IMVIS(6*MXUNDM+1))
C
      Q(A,SPHI,CPHI,R,U,V) = TWOPI*SQRT((V*CPHI+U*SPHI)**2
     1                      + (R*(U*CPHI-V*SPHI))**2) * A
C
C Find out if underlying model is wanted:
C
   10 CALL PUTOUT(' ')
      UNDMOD = QUERY('Should part of input model be '//
     1               'a separate (constant) model? (Y or N): ')
      IF (.NOT.UNDMOD) RETURN
C
C Determine if underlying model has already been computed;  then
C get that, or a 'new' underlying model.
C
      CALL PUTOUT(' ')
      IF (QUERY('Has the underlying model been computed '//
     1          'before?  (Y or N): ')) THEN
C         -- Try and read in contribution from underlying model:
          CALL PUTOUT(' ')
   20     IER = GETIN(UNDDSN, 'Filename for contribution from '//
     1                'underlying model: ', L)
          IF (IER.NE.1) STOP
          OPEN  (UNIT=INUND,READONLY,FORM='UNFORMATTED',ERR=20,
     1    STATUS='OLD',FILE=UNDDSN)
          DO 30 I=1,MXREC
            READ (INUND,ERR=50,END=40) (UNIMVIS(I,J), J=1,NBAS)
            READ (INUND,ERR=50,END=50) (UNREVIS(I,J), J=1,NBAS)
   30     CONTINUE
          I = MXREC + 1
          CALL PUTOUT('Warning:  Maximum number of records being'//
     1                ' used.')
   40     IREC = I-1
          IF (IREC .EQ. NUMREC) THEN
            CALL PUTOUT('Contribution from underlying model has'//
     1      ' successfully been read in.')
            UNDMOD = .TRUE.
            CLOSE (UNIT=INUND)
            RETURN
          ELSE
            CALL ERROR('Underlying model file does not have the'//
     1      ' right number of data records.')
          ENDIF
C
   50     CLOSE (UNIT=INUND)
          CALL PUTOUT(' ')
          UNDMOD = QUERY('Something''s wrong with the underlying'//
     1                   ' model file.  Try again? (Y or N): ')
          IF (UNDMOD) GOTO 10
C
      END IF
C
C Get a new input model, read it in, and find contribution.
C
      CALL PUTOUT(' ')
   60 IER = GETIN(UNDDSN, 'Filename for underlying model: ', L)
      IF (IER.NE.1) STOP
      OPEN  (UNIT=INUND,READONLY,FORM='FORMATTED',ERR=60,
     1       STATUS='OLD',FILE=UNDDSN)
      CALL PUTOUT(' ')
   70 IER = GETIN(OUTUNDDSN,
     1            'Filename for output from underlying model: ', L)
      IF (IER.NE.1) STOP
      OPEN  (UNIT=OUTUND,FORM='UNFORMATTED',ERR=70,
     1       STATUS='NEW',FILE=OUTUNDDSN)
C
C Read in the the underlying model.
C
      FLXSUM = 0.
      DO 150 I=1,MXUNDM
          CALL GTMODL (INUND,ITYPE(I),UFLUX(I),URADIUS(I),UTHETA,
     1    UAXIS(I),URATIO(I),UPHI, *155, 0)
          IF (UFLUX(I) .EQ. 0.0) GO TO 155
          FLXSUM = FLXSUM + UFLUX(I)
          URTHETA(I) = UTHETA * DEGRAD
          URPHI(I)   = UPHI * DEGRAD
          IF (ITYPE(I) .EQ. 0) ITYPE(I) = 1
  150 CONTINUE
      WRITE (OUTC,650) MXUNDM
      I = MXUNDM + 1
  155 NUNDM = I-1
      IF (FLXSUM .EQ. 0.0) CALL ERROR('Model has zero total flux')
C
C Calculate the contribution from the underlying model, store it in
C the arrays UNIMVIS and UNREVIS, and write UNIMVIS and UNREVIS to
C the output file OUTUNDDSN.
C
      DO 350 IB = 1, NBAS
          DO 325 ID = 1, NUMREC
            IF (WTVIS(ID,IB).LE.0.) GO TO 325
            UNREVIS(ID,IB) = 0.0
            UNIMVIS(ID,IB) = 0.0
            DO 300  J=1,NUNDM
              COMPHA = TWOPI*URADIUS(J) * (UDATA(ID,IB)*SIN(URTHETA(J))
     1        + VDATA(ID,IB)*COS(URTHETA(J)))
C
C             Find the complex visibility amplitude:
              IF (UAXIS(J) .EQ. 0.0) THEN
                COMVIS = UFLUX(J)
              ELSEIF (ITYPE(J) .EQ. 1) THEN
                SPHI   = SIN(URPHI(J))
                CPHI   = COS(URPHI(J))
                COMVIS = UFLUX(J) * (EXP (C1 * UAXIS(J)**2 *
     1          ((VDATA(ID,IB)*CPHI + UDATA(ID,IB)*SPHI)**2 +
     2          (URATIO(J)*(UDATA(ID,IB)*CPHI-VDATA(ID,IB)*SPHI))**2)))
              ELSEIF (TYPE(J) .EQ. 2) THEN
                SPHI   = SIN(URPHI(J))
                CPHI   = COS(URPHI(J))
                QQ = Q(UAXIS(J),SPHI,CPHI,URATIO(J),
     1               UDATA(ID,IB),VDATA(ID,IB))
                QQ = QQ / 2.0
C               -- ! Thus, axis is diameter
                IF (ABS(QQ).LT.0.01) THEN
                  COMVIS = UFLUX(J)*(1.0 - QQ*QQ/8. + QQ*QQ*QQ*QQ/192.)
                ELSE
                  COMVIS = 2. * UFLUX(J) * BESJ1(QQ)/QQ
                END IF
              ELSEIF (ITYPE(J) .EQ. 3) THEN
                SPHI   = SIN(URPHI(J))
                CPHI   = COS(URPHI(J))
                QQ = Q(UAXIS(J),SPHI,CPHI,URATIO(J),
     1               UDATA(ID,IB),VDATA(ID,IB))
                QQ = QQ / 2.0
C               -- ! Thus, axis is diameter
                IF (ABS(QQ).LT.0.01) THEN
                  COMVIS = UFLUX(J)*(1.0 - QQ*QQ/10. + QQ*QQ*QQ*QQ/280.)
                ELSE
                  COMVIS = 3. * UFLUX(J) * (SIN(QQ)-QQ*COS(QQ)) /
     1               (QQ*QQ*QQ)
                END IF
              ELSEIF (ITYPE(J) .EQ. 4) THEN
                SPHI   = SIN(URPHI(J))
                CPHI   = COS(URPHI(J))
                COMVIS = UFLUX(J) * BESJ0(Q(UAXIS(J),SPHI,
     1          CPHI,URATIO(J),UDATA(ID,IB),VDATA(ID,IB))/2.0)
              ELSEIF (TYPE(J) .EQ. 5) THEN
                SPHI   = SIN(URPHI(J))
                CPHI   = COS(URPHI(J))
                XX = (UDATA(ID,IB)*SPHI + VDATA(ID,IB)*CPHI)*PI*UAXIS(J)
                IF (ABS(XX).LT.0.01) THEN
                  COMVIS = UFLUX(J)*(1.0 - XX*XX/6. + XX*XX*XX*XX/120.)
                ELSE
                  COMVIS = UFLUX(J) * SIN(XX)/XX
                END IF
              ELSE
                CALL ERROR (' Illegal component type in model.')
              ENDIF
C
              UNREVIS(ID,IB)  = UNREVIS(ID,IB) + COMVIS*COS(COMPHA)
              UNIMVIS(ID,IB)  = UNIMVIS(ID,IB) + COMVIS*SIN(COMPHA)
  300       CONTINUE
  325     CONTINUE
  350 CONTINUE
C
C Write the contribution from the underlying model to the output file.
C
      CALL PUTOUT(' ')
      CALL PUTOUT('Model calculation done.')
      DO 400 I=1,NUMREC
          WRITE (OUTUND) (UNIMVIS(I,J), J=1,NBAS)
          WRITE (OUTUND) (UNREVIS(I,J), J=1,NBAS)
  400 CONTINUE
      CLOSE (UNIT=INUND)
      CLOSE (UNIT=OUTUND)
      CALL PUTOUT('Contribution from underlying model written '//
     1            'to file '//OUTUNDDSN)
      WRITE (OUTC,1001) '0There were ',NUNDM,' components in the',
     1' underlying model.'
      UNDMOD = .TRUE.
      RETURN
C
  650 FORMAT (' ++WARNING++ Model truncated after ',I5,' components.')
 1001 FORMAT (A,I5,A,A)
      END
      SUBROUTINE OPENGR
C-----------------------------------------------------------------------
C Get plot-FWHM from user, ask if they want the screen split, open the
C graphics device, & find the max color index available.
C-----------------------------------------------------------------------
      IMPLICIT NONE
      INCLUDE 'modelfit20.inc'
      INTEGER PGBEG
      LOGICAL QUERY
C
      INTEGER IER, JUNK
      LOGICAL SPLIT
C
 3    WRITE (OUTC, 5)
 5    FORMAT (' Please choose convolving-beam FWHM for plots [mas]: ',
     +  $)
      READ (INC, *, ERR = 3) BEAM
      IF (BEAM .LT. 0.0) GOTO 3
C
      SPLIT = QUERY('Do you want the screen split to give 4 '//
     +  'displays? (Y or N): ')
 10   IF (SPLIT) THEN
         IER = PGBEG(0, '?', 2, 2)
      ELSE
         IER = PGBEG(0, '?', 1, 1)
      END IF
      IF (IER .NE. 1) GOTO 10
      CALL PGQCOL(JUNK, CIMAX)
C
      RETURN
      END
      SUBROUTINE WARNGR
C-----------------------------------------------------------------------
C Warn the user about any non-Gaussian model components.
C-----------------------------------------------------------------------
      IMPLICIT NONE
      INCLUDE 'modelfit20.inc'
C
      INTEGER IC
C
      DO IC = 1, NCOMP
         IF (TYPE(IC) .NE. 1) THEN
            CALL PUTOUT(' ')
            CALL PUTOUT('Warning: This model contains '//
     +  'non-Gaussian/delta component(s)')
            CALL PUTOUT('         which won''t appear '//
     +  'in the contour map, although')
            CALL PUTOUT('         their major/minor axes '//
     +  'will be sketched in.')
            CALL PUTOUT(' ')
            RETURN
         END IF
      END DO
C
      RETURN
      END
      SUBROUTINE DRAWIT
C-----------------------------------------------------------------------
C Choose appropriate window limits & contour levels & draw a contour
C map of the Gaussian components in the model.  Draw in lines or boxes
C to indicate the FWHM major & minor axes of all components & label
C the plot.
C
C [RC] 1993 Jan 10.
C-----------------------------------------------------------------------
      IMPLICIT NONE
      INCLUDE 'modelfit20.inc'
C
      INTEGER LEN1
      REAL PGRND
C
      REAL EXCESS, XWEST, XSPLIT, MAXTKA, DYPOSN, COLUM2
C
C Choose windows with sides 15% longer than they initially need to be:
      PARAMETER (EXCESS = 1.15)
C
C Set this parameter to 1.0 if you like your x axis labelled +ve to
C the west; or -1.0 for +ve east:
      PARAMETER (XWEST = 1.0)
C
C Where, along x axis of display, to split the screen:
      PARAMETER (XSPLIT = 0.75)
C
C Max no. of major tick spacings along each axis (must be > 5 to
C guarantee >= 2 major tick marks on each axis):
      PARAMETER (MAXTKA = 5.01)
C
C Spacing between lines of text in graphics display.
      PARAMETER (DYPOSN = 0.035)
C
C X coordinate of 2nd column in list of contour levels.
      PARAMETER (COLUM2 = 0.4)
C
      INTEGER SIDEPX, NPCNT, NNCNT, COLAXI, COLPCT, COLNCT, COLMAJ,
     +  COLLAB, WTHMAJ
C
C How many pixels along each axis:
      PARAMETER (SIDEPX = 100)
C
C How many positive, negative contours will we draw:
      PARAMETER (NPCNT = 8, NNCNT = 4)
C
C Which color indices for the plot axes, +ve, -ve contours,
C major/minor axis indicators & textual labelling:
      PARAMETER (COLAXI = 4, COLPCT = 5, COLNCT = 7, COLMAJ = 2,
     +  COLLAB = 3)
C
C The width of the lines used to mark the major/minor axes:
      PARAMETER (WTHMAJ = 6)
C
      CHARACTER*64 NUMBR, HEADING
      INTEGER IX, IY, IC, IL, TIKSBX, TIKSBY, NC, ICONT, NCNT
      LOGICAL NEWWIN, INCOL
      REAL X1, X2, Y1, Y2, XMID, YMID, SIZE, MAP(SIDEPX, SIDEPX),
     +  PKMAP, COSIN, RATM2, XXCOEF, YYCOEF, XYCOEF, PKBRIT,
     +  ECOEF, X, Y, XXTERM, YPCOEF, CNTLEV(NPCNT + NNCNT),
     +  XPOS, YPOS, XMAJ, XMIN, YMAJ, YMIN, TR(6), X0, Y0, CNVAX2,
     +  CNVRT2, BEAMWD, TIKMJX, TIKMJY, YPOSN, NPKMAP, APEAK, BEAMW2
C
C Are enough color indices available to draw the thing in color?
C
      INCOL = CIMAX .GE. MAX(COLAXI, COLPCT, COLNCT, COLMAJ, COLLAB)
C
C Should we set new window limits?  Find min acceptable window & decide.
C
      CALL PIKWIN(0.64 * 0.5 ** (NPCNT - 1), X1, X2, Y1, Y2)
      IF (.NOT. WINSET) THEN
         NEWWIN = .TRUE.
      ELSE
         NEWWIN = (X1 .LT. XWIN1) .OR. (X2 .GT. XWIN2) .OR.
     +            (Y1 .LT. YWIN1) .OR. (Y2 .GT. YWIN2) .OR.
     +            MAX(Y2 - Y1, X2 - X1) .LT.
     +            MAX(YWIN2 - YWIN1, XWIN2 - XWIN1) / EXCESS ** 2
      END IF
C
C If so, expand the window limits out from the smallest tolerable so that
C hopefully we won't have to resize them again.  Also, force window to be
C square.
C
      IF (NEWWIN) THEN
         XMID = 0.5 * (X1 + X2)
         YMID = 0.5 * (Y1 + Y2)
         SIZE = 0.5 * EXCESS * MAX(X2 - X1, Y2 - Y1)
         WINDLT = 2 * SIZE / (SIDEPX - 1)
         XWIN1 = XMID - SIZE
         XWIN2 = XMID + SIZE
         YWIN1 = YMID - SIZE
         YWIN2 = YMID + SIZE
      END IF
      WINSET = .TRUE.
C
C Choose beam width.
C
      IF (BEAM .LT. MINPXB * WINDLT) THEN
         BEAMWD = MINPXB * WINDLT
         WRITE (OUTC, 5) BEAMWD, MINPXB
 5       FORMAT (' (Beam FWHM increased to', F8.1, ' mas =',
     +  F8.1, ' pixel spacings for this plot).')
      ELSE
         BEAMWD = BEAM
      END IF
      BEAMW2 = BEAMWD * BEAMWD
C
C Buffer the output (for speed); setup window & labeling.
C
      CALL PGBBUF
      CALL PGPAGE
      CALL PGSVP(0.1, XSPLIT, 0.1, 0.9)
      CALL PGWNAD(XWEST * XWIN1, XWEST * XWIN2, YWIN1, YWIN2)
      CALL PGSLW(1)
      IF (INCOL) THEN
         CALL PGSCI(COLAXI)
      ELSE
         CALL PGSCI(1)
      END IF
C
      TIKMJX = PGRND((XWIN2 - XWIN1) / MAXTKA, TIKSBX)
      TIKMJY = PGRND((YWIN2 - YWIN1) / MAXTKA, TIKSBY)
      CALL PGBOX('ABCTS', TIKMJX, TIKSBX, 'ABCTS', TIKMJY, TIKSBY)
      IF (INCOL) CALL PGSCI(COLLAB)
      CALL PGBOX('N', TIKMJX, TIKSBX, 'N', TIKMJY, TIKSBY)
      HEADING = METHOD
      IF (JITER .GT. 0) THEN
         CALL PGNUMB(JITER, 0, 1, NUMBR, NC)
         HEADING = HEADING(1:LEN1(HEADING))//' / Iteration # '//
     +  NUMBR(1:NC)
      END IF
      CALL PGLAB('mas', 'mas', HEADING)
C
C Now make the map (in brightness units: Jy / beam).
C
      DO IX = 1, SIDEPX
         DO IY = 1, SIDEPX
            MAP(IX, IY) = 0.0
         END DO
      END DO
      PKMAP = 0.0
      NPKMAP = 0.0
      DO IC = 1, NCOMP
         IF (TYPE(IC) .EQ. 1) THEN
C
C Get coefs for evaluating the Gaussian convolved with the beam -
C convolving Gaussians with Gaussians is purely a matter of obesity.
C
            CNVAX2 = AXIS(IC) ** 2 + BEAMW2
            CNVRT2 = ((AXIS(IC) * RATIO(IC)) ** 2 + BEAMW2) / CNVAX2
            COSIN = COS(2 * RPHI(IC))
            RATM2 = 1 / CNVRT2
            ECOEF = -4 * LOG(2.0) / CNVAX2
            XXCOEF = 0.5 * (1 + RATM2 + COSIN * (RATM2 - 1)) * ECOEF
            YYCOEF = 0.5 * (1 + RATM2 - COSIN * (RATM2 - 1)) * ECOEF
            XYCOEF = SIN(2 * RPHI(IC)) * (RATM2 - 1) * ECOEF
            PKBRIT = BEAMW2 * FLUX(IC) / (SQRT(CNVRT2) * CNVAX2)
            X0 = XWIN1 - WINDLT + RADIUS(IC) * SIN(RTHETA(IC))
            Y0 = YWIN1 - WINDLT - RADIUS(IC) * COS(RTHETA(IC))
C
C & add this one to the map.
C
            DO IX = 1, SIDEPX
               X = X0 + IX * WINDLT
               XXTERM = XXCOEF * X * X
               YPCOEF = XYCOEF * X
               DO IY = 1, SIDEPX
                  Y = Y0 + IY * WINDLT
                  MAP(IX, IY) = MAP(IX, IY) + PKBRIT *
     +                          EXP(XXTERM + (YPCOEF + YYCOEF * Y) * Y)
                  PKMAP = MAX(PKMAP, MAP(IX, IY))
                  NPKMAP = MIN(NPKMAP, MAP(IX, IY))
               END DO
            END DO
         END IF
      END DO
C
C Setup the coordinate transformation.
C
      TR(1) = XWEST * (XWIN1 - WINDLT)
      TR(2) = XWEST * WINDLT
      TR(3) = 0.0
      TR(4) = YWIN1 - WINDLT
      TR(5) = 0.0
      TR(6) = WINDLT
C
C Now decide on some +ve contour levels (64%, 32%, 16%,... of peak),
C & draw them.
C
      IF (PKMAP .NE. 0.0) THEN
         CNTLEV(1) = 0.64 * PKMAP
         DO IL = 2, NPCNT
            CNTLEV(IL) = 0.5 * CNTLEV(IL - 1)
         END DO
         IF (INCOL) CALL PGSCI(COLPCT)
         CALL PGCONS(MAP, SIDEPX, SIDEPX, 1, SIDEPX, 1, SIDEPX,
     +  CNTLEV, NPCNT, TR)
         NCNT = NPCNT
         CNTLEV(NPCNT + 1) = -CNTLEV(NPCNT)
C
C APEAK is the peak level used in converting the contour levels into
C `percentages of peak'.
C
         APEAK = PKMAP
      ELSE
         NCNT = 0
         CNTLEV(1) = 0.64 * NPKMAP * 0.5**(NNCNT - 1)
         APEAK = -NPKMAP
      END IF
C
C & do some -ve contours...
C
      IF (NPKMAP .NE. 0.0) THEN
         DO IL = NCNT + 2, NCNT + NNCNT
            CNTLEV(IL) = 2.0 * CNTLEV(IL - 1)
         END DO
         IF (INCOL) CALL PGSCI(COLNCT)
         CALL PGCONS(MAP, SIDEPX, SIDEPX, 1, SIDEPX, 1, SIDEPX,
     +  CNTLEV(NCNT + 1), NNCNT, TR)
         NCNT = NCNT + NNCNT
      END IF
C
C Draw in lines or boxes to indicate major/minor axes.
C
      CALL PGSLW(WTHMAJ)
      IF (INCOL) CALL PGSCI(COLMAJ)
      DO IC = 1, NCOMP
         XPOS = -RADIUS(IC) * SIN(RTHETA(IC))
         YPOS =  RADIUS(IC) * COS(RTHETA(IC))
         XMAJ = -0.5 * AXIS(IC) * SIN(RPHI(IC))
         YMAJ =  0.5 * AXIS(IC) * COS(RPHI(IC))
         XMIN = -RATIO(IC) * YMAJ
         YMIN =  RATIO(IC) * XMAJ
C
         CALL PGMOVE(XWEST * (XPOS - XMAJ - XMIN), YPOS - YMAJ - YMIN)
         CALL PGDRAW(XWEST * (XPOS + XMAJ - XMIN), YPOS + YMAJ - YMIN)
         CALL PGDRAW(XWEST * (XPOS + XMAJ + XMIN), YPOS + YMAJ + YMIN)
         CALL PGDRAW(XWEST * (XPOS - XMAJ + XMIN), YPOS - YMAJ + YMIN)
         CALL PGDRAW(XWEST * (XPOS - XMAJ - XMIN), YPOS - YMAJ - YMIN)
      END DO
      CALL PGSLW(1)
C
C Do annotation to the right of the plot.
C
      CALL PGSVP(XSPLIT, 1.0, 0.0, 1.0)
      CALL PGSWIN(0.0, 1.0, 0.0, 1.0)
      IF (INCOL) CALL PGSCI(COLLAB)
      YPOSN = 0.85
C
      CALL PGNUMB(NINT(TOTLAF * 1E3), -3, 1, NUMBR, NC)
      CALL PGTEXT(0.0, YPOSN, 'Total AF =  '//NUMBR(1:NC))
      YPOSN = YPOSN - 2 * DYPOSN
C
      CALL PGNUMB(NINT(BEAMWD * 10), -1, 1, NUMBR, NC)
      CALL PGTEXT(0.0, YPOSN, 'Beam FWHM =')
      CALL PGTEXT(0.0, YPOSN - DYPOSN, '  '//NUMBR(1:NC)//' mas')
      YPOSN = YPOSN - 3 * DYPOSN
C
C List contour levels.
C
      IF (NCNT .GT. 0) THEN
         CALL PGTEXT(0.0, YPOSN, 'Contours:')
         YPOSN = YPOSN - DYPOSN
         CALL PGTEXT(0.0, YPOSN, 'Percent')
         CALL PGTEXT(COLUM2, YPOSN, ' Jy / beam')
         YPOSN = YPOSN - DYPOSN
         DO ICONT = 1, NCNT
            IF (CNTLEV(ICONT) .GE. 0.0) THEN
               IF (INCOL) CALL PGSCI(COLPCT)
            ELSE
               IF (INCOL) CALL PGSCI(COLNCT)
            END IF
            WRITE(NUMBR, '(F6.2)') 100 * CNTLEV(ICONT) / APEAK
            CALL PGTEXT(0.0, YPOSN, NUMBR(1:6))
            WRITE(NUMBR, '(G11.3)') CNTLEV(ICONT)
            CALL PGTEXT(COLUM2, YPOSN, NUMBR(1:11))
            YPOSN = YPOSN - DYPOSN
         END DO
      END IF
      IF (INCOL) CALL PGSCI(1)
      CALL PGEBUF
C
      RETURN
      END
      SUBROUTINE PIKWIN(FRACPK, X1, X2, Y1, Y2)
C-----------------------------------------------------------------------
C Pick graphics window limits (X1, X2, Y1, Y2) large enough to include
C all model components.  Each Gaussian/delta is taken to extend out to
C where its surface brightness has dropped to FRACPK times the peak
C brightness of the max peak-brightness Gaussian.  Additionally, for all
C component types, we must be able to see out to 2 * AXF times the
C lengths of the semi-major/minor axes.
C
C [RC]  1993 Jan 10.
C-----------------------------------------------------------------------
      IMPLICIT NONE
      INCLUDE 'modelfit20.inc'
      REAL FRACPK, X1, X2, Y1, Y2
C
      REAL WINMIN, WINENL, AXF
C
C Min window size [mas] (used in emergencies only!) :
      PARAMETER (WINMIN = 1E-3)
C
C Factor to enlarge window by, if larger than WINMIN:
      PARAMETER (WINENL = 1.2)
C
C We must be able to see at least 2 * AXF times the lengths of major
C & minor axes of all components:
      PARAMETER (AXF = 0.6)
C
      INTEGER IC
      LOGICAL GAUCMP(MXMOD)
      REAL XWIDTH, YWIDTH, COSIN, XPOS, YPOS, MAXBRI, TX1, TX2,
     +  TY1, TY2, XWINWD, YWINWD, XWINCT, YWINCT, NFWHM, CNVAX2(MXMOD),
     +  BRIGAU(MXMOD), MINBRI, CNVRA2(MXMOD), BEAM2, AXIS2, AXRAT2
C
C Find the central brightness of the max peak-brightness Gaussian. 
C
      MAXBRI = 0.0
      BEAM2 = BEAM * BEAM
      DO IC = 1, NCOMP
C
C Will this component be a nonzero-sized Gaussian, when convolved with
C a beam of FWHM=BEAM? If so, treat it as a Gaussian for the purpose of
C finding its extremities.
C
         AXIS2 = AXIS(IC) ** 2
         AXRAT2 = AXIS2 * RATIO(IC) ** 2
         GAUCMP(IC) = TYPE(IC) .EQ. 1 .AND. (BEAM2 .NE. 0.0 .OR.
     +  (AXIS2 .NE. 0.0 .AND. AXRAT2 .NE. 0.0))
         IF (GAUCMP(IC)) THEN
            CNVAX2(IC) = AXIS2 + BEAM2
            CNVRA2(IC) = (AXRAT2 + BEAM2) / CNVAX2(IC)
            BRIGAU(IC) = ABS(FLUX(IC)) / (CNVAX2(IC) * SQRT(CNVRA2(IC)))
            MAXBRI = MAX(MAXBRI, BRIGAU(IC))
         END IF
      END DO
      MINBRI = MAXBRI * FRACPK
      DO IC = 1, NCOMP
         XPOS = -RADIUS(IC) * SIN(RTHETA(IC))
         YPOS =  RADIUS(IC) * COS(RTHETA(IC))
         IF (GAUCMP(IC)) THEN
C
C Find limits for Gaussian: NFWHM is the no. of FWHM we should go out
C to.
C
            IF (BRIGAU(IC) .GT. MINBRI) THEN
               NFWHM = SQRT(LOG(BRIGAU(IC) / MINBRI) / (4 * LOG(2.0)))
               COSIN = COS(2 * RPHI(IC))
               XWIDTH = SQRT(0.5 * (CNVRA2(IC) + 1 +
     +  COSIN * (CNVRA2(IC) - 1)) * CNVAX2(IC)) * NFWHM
               YWIDTH = SQRT(0.5 * (CNVRA2(IC) + 1 -
     +  COSIN * (CNVRA2(IC) - 1)) * CNVAX2(IC)) * NFWHM
            ELSE
               XWIDTH = 0.0
               YWIDTH = 0.0
            END IF
         ELSE
            XWIDTH = 0.0
            YWIDTH = 0.0
         END IF
C
C Make sure the window is big enough for us to see 2 * AXF times the
C lengths of the major/minor axis lines.
C
         XWIDTH = MAX(XWIDTH, AXF * (ABS(AXIS(IC) * SIN(RPHI(IC))) +
     +  ABS(AXIS(IC) * RATIO(IC) * COS(RPHI(IC)))))
         YWIDTH = MAX(YWIDTH, AXF * (ABS(AXIS(IC) * COS(RPHI(IC))) +
     +  ABS(AXIS(IC) * RATIO(IC) * SIN(RPHI(IC)))))
C
         IF (IC .EQ. 1) THEN
            TX1 = XPOS - XWIDTH
            TX2 = XPOS + XWIDTH
            TY1 = YPOS - YWIDTH
            TY2 = YPOS + YWIDTH
         ELSE
            TX1 = MIN(TX1, XPOS - XWIDTH)
            TX2 = MAX(TX2, XPOS + XWIDTH)
            TY1 = MIN(TY1, YPOS - YWIDTH)
            TY2 = MAX(TY2, YPOS + YWIDTH)
         END IF
      END DO
C
C Enlarge window by a factor of WINENL, & make sure window is at least
C minimum size (WINMIN).
C
      XWINWD = MAX(WINENL * (TX2 - TX1), WINMIN) * 0.5
      YWINWD = MAX(WINENL * (TY2 - TY1), WINMIN) * 0.5
      XWINCT = (TX1 + TX2) * 0.5
      YWINCT = (TY1 + TY2) * 0.5
C
      X1 = XWINCT - XWINWD
      X2 = XWINCT + XWINWD
      Y1 = YWINCT - YWINWD
      Y2 = YWINCT + YWINWD
C
      RETURN
      END
