C-----------------------------------------------------------------------
C MODELFIT.INC
C-----------------------------------------------------------------------
C This common block is used by the program MODELFIT and most of its
C associated subroutines, via INCLUDE statements.
C
C Modified 28-Mar-1989 (TJP).
C Modified 24-Jun-1992 (TJP).
C Modified 10-Jan-1993 (RC).
C-----------------------------------------------------------------------
C
C Parametric constants.
C
      INTEGER MXSTA,MXBAS,MXTRI,MXICLO,MAXWPB,MXREC,MXUNDM,MXMOD
      INTEGER MAXIT
      INTEGER INC,OUTC,INDAT,INMOD,INUND,OUTUND
C     -- Maximum number of stations, baselines, triangles
      PARAMETER (MXSTA=20)
      PARAMETER (MXBAS=MXSTA*(MXSTA-1)/2)
      PARAMETER (MXTRI=MXBAS*(MXSTA-2)/3)
C     -- Maximum number of independent triangles
      PARAMETER (MXICLO=(MXSTA-1)*(MXSTA-2)/2)
C     -- Maximum data words per baseline
      PARAMETER (MAXWPB=7)
C     -- Maximum number of data records
      PARAMETER (MXREC=350)
C     -- Maximum number of components in underlying model  ?!!
      PARAMETER (MXUNDM=MXBAS*MXREC)
C     -- Maximum number of model components
      PARAMETER (MXMOD=15)
C     -- Maximum number of iterations in GRIDLS or GRADLS
      PARAMETER (MAXIT=20)
C     -- I/O unit numbers
      PARAMETER (INC=5,OUTC=6,INDAT=8,INMOD=9,INUND=10,OUTUND=11)
C
      REAL PI,DEGRAD,RADHRS,TWOPI,HALFPI,SIDR,VELC,CONV1,MINPXB
C     -- pi
      PARAMETER (PI=3.14159265359)
      PARAMETER (TWOPI=2.*PI)
      PARAMETER (HALFPI=PI/2.)
C     -- # of radians per degree
      PARAMETER (DEGRAD=PI/180.)
C     -- hours of UT per radian
      PARAMETER (RADHRS=1./(DEGRAD*15.))
C     -- seconds of ST per second of UT
      PARAMETER (SIDR=1.0027379093)
C     -- speed of light
      PARAMETER (VELC=2.997924580E8)
C     -- msec of arc per radian
      PARAMETER (CONV1 = 180.0*60.0*60.0*1000.0/PI)
C     -- min # of pixels per beam, when plotting Gaussians.
      PARAMETER (MINPXB = 3.0)
C-----------------------------------------------------------------------
      REAL    FLUX, RADIUS, RTHETA, AXIS, RATIO, RPHI, THETA, PHI
      REAL    DELTAP, SIGMAP, IMVIS, REVIS
      REAL    FLSTRT, RASTRT, THSTRT, AXSTRT, ARSTRT, PHSTRT
      REAL    UNIMVIS, UNREVIS
      REAL    UDATA, VDATA, OBSVIS, WTVIS, MODVIS
      REAL    OBSCLO, WTCLO, MODCLO, DOWNWT
      REAL    TNZCLO
      REAL    LAMBDA, UT, TNZDAT
      REAL    VISAF, CLOAF, AMPLAF, PHASAF, TOTLAF
      REAL    XWIN1, XWIN2, YWIN1, YWIN2, WINDLT, BEAM
      REAL    AWEIGT, CWEIGT
      INTEGER TYPE, NCOMP
      INTEGER ICBSNO, NUMCLO
      INTEGER NINDCL, NUMBAS, NUMREC, NUMSTA, NUMTRI, NZCLO
      INTEGER NZVIS, NZAMP, NINTRI
      INTEGER NVPAR, JITER, NDAY, NYEAR
      INTEGER CIMAX
      LOGICAL FRSTRY, VARY, UNDMOD
      LOGICAL DOPLOT, WINSET
      REAL    PARM(MXMOD,6)
      EQUIVALENCE (FLUX(1),PARM(1,1))
      COMMON /MDLFIT/
C      Model parameters:
     1      FLUX(MXMOD), RADIUS(MXMOD), RTHETA(MXMOD), AXIS(MXMOD),
     2      RATIO(MXMOD), RPHI(MXMOD), TYPE(MXMOD), THETA(MXMOD),
     3      PHI(MXMOD), NCOMP, FRSTRY, VARY(6,MXMOD),
     4      DELTAP(6,MXMOD), SIGMAP(6,MXMOD), IMVIS(MXMOD,MXREC,MXBAS),
     5      REVIS(MXMOD,MXREC,MXBAS), FLSTRT(MXMOD), RASTRT(MXMOD),
     6      THSTRT(MXMOD), AXSTRT(MXMOD), ARSTRT(MXMOD), PHSTRT(MXMOD),
C      Parameters for contribution from underlying model:
     7      UNIMVIS(MXREC,MXBAS), UNREVIS(MXREC,MXBAS), UNDMOD,
C      Amplitude related data:
     8      UDATA(MXREC,MXBAS), VDATA(MXREC,MXBAS), OBSVIS(MXREC,MXBAS),
     9      WTVIS(MXREC,MXBAS), MODVIS(MXREC,MXBAS),
C      Closure phase related variables:
     A   OBSCLO(MXREC,MXTRI), WTCLO(MXREC,MXTRI), MODCLO(MXREC,MXTRI),
     B   ICBSNO(3,MXTRI), NUMCLO(MXREC), DOWNWT(MXREC),
C      Variables to describe limits of data:
     C   NINDCL, NUMBAS, NUMREC, NUMSTA, NUMTRI, NZCLO(MXTRI), TNZCLO,
     D   NZVIS(MXBAS), NZAMP, NINTRI(MXREC), LAMBDA, UT(MXREC),
     E   TNZDAT, NVPAR, JITER, NDAY, NYEAR, AWEIGT, CWEIGT,
C      Variables used in the calculation of agreement factors:
     F   VISAF(MXBAS), CLOAF(MXTRI), AMPLAF, PHASAF, TOTLAF,
C      Variables for graphics handling:
     G   DOPLOT, CIMAX, XWIN1, XWIN2, YWIN1, YWIN2, WINDLT, WINSET,
     H   BEAM
C-----------------------------------------------------------------------
      DOUBLE PRECISION STNAME, STX, STY, STZ, SOURCE, RAP, DECP, GASTM,
     1   FREQ, FLUX1
      COMMON /PARMS/
     1   STNAME(MXSTA), STX(MXSTA), STY(MXSTA), STZ(MXSTA), SOURCE(2),
     2   RAP, DECP, GASTM, FREQ, FLUX1
C-----------------------------------------------------------------------
      INTEGER
     1   NSTA, NBAS, NTRI, NAVIS(MXBAS), NBVIS(MXBAS),
     2   NACLO(MXTRI), NBCLO(MXTRI), NCCLO(MXTRI), NVER, NDWPB
      COMMON /ORDER/
     1   NSTA, NBAS, NTRI, NAVIS, NBVIS,
     2   NACLO, NBCLO, NCCLO, NVER, NDWPB
C-----------------------------------------------------------------------
      CHARACTER*64    INDSN, MODDSN, METHOD, UNDDSN, OUTUNDDSN
      COMMON /STRING/ INDSN, MODDSN, METHOD, UNDDSN, OUTUNDDSN
C-----------------------------------------------------------------------
      LOGICAL         CCFLAG
      COMMON /CONTRC/ CCFLAG
C-----------------------------------------------------------------------
