      PROGRAM MODFIX
C-----------------------------------------------------------------------
C Perform shifting, windowing, rotating, and scaling operations on
C Caltech VLB-format model files.
C
C Author:            T.J. Pearson (TJP)
C Written:           1983 Aug 24.
C Language:          Fortran-77
C
C Modifications:
C Version 1.0:  1983 Aug 24 (TJP).
C Version 1.1:  1983 Aug 25 - add STRETCH option (TJP).
C Version 1.2:  1983 Sep 22 - correct sign convention in SHIFT (TJP).
C Version 1.3:  1985 Mar 27 - correct sign convention in LRTB; attempt
C                             to standardize code (TJP).
C Version 1.4:  1985 Mar 27 - add SQUASH parameter; permit non-Gaussian
C                             components (TJP).
C Version 1.5:  1986 Sep 18 - add SORT parameter (TJP).
C Version 1.6:  1988 Apr  6 - changes for Convex (TJP).
C Version 1.7:  1992 Feb 12 - add NUMBER option (TJP).
C Version 1.8:  1992 Jul 13 - minor bug fix (TJP).
C-----------------------------------------------------------------------
C
C Parameters; MXMOD is max. no. of model components.
C
      CHARACTER*(*) VERSN
      INTEGER INC,OUTC,OUTM,INM,NPARS,MXMOD
      PARAMETER (VERSN='1.8 - 1992 Jul 13')
      PARAMETER (INC=5, OUTC=6, INM=1, OUTM=2)
      PARAMETER (NPARS=33)
      PARAMETER (MXMOD=10000)
      REAL D2R
      PARAMETER (D2R=3.14159265358979/180.0)
C
      CHARACTER*128 INPUT, OUTPUT
      CHARACTER*80  HISTRY
      INTEGER       I, J, L, M, N, IER, MODE, NCOMP, NFINAL
      INTEGER       CTYPE, MAXTYP, TYPE(MXMOD), IND(MXMOD)
      INTEGER       LEN1, VLBOPE
      LOGICAL       WINDOW, NONEG, SQUASH, SORT
      REAL          A, B
      REAL          XSHIFT, YSHIFT, X1, X2, Y1, Y2, ROTAT, SCALE, STRET
      REAL          FLUX, RADIUS, THETA, AXIS, RATIO, PHI, TOTFLX, T
      REAL          FL(MXMOD), AX(MXMOD)
      REAL          AR(MXMOD), PH(MXMOD), XC(MXMOD), YC(MXMOD)
C
      DOUBLE PRECISION PARS(NPARS), VALS(NPARS), ENDMRK
      DATA PARS / 'Input', 9*' ', 'Output', 9*' ', 2*'SHift',
     1            4*'Lrtb', 'Rotate', 'SCale' , 'NOneg', 'STretch',
     2            'SQuash', 'SOrt', 'Number' /
      DATA VALS / '***UNSET', 9*' ', '***UNSET', 9*' ', 2*0D0,
     1            4*0D0, 0D0, 1D0, 1D0, 1D0, 
     2            1D0, 1D0, MXMOD /
      DATA ENDMRK / '/' /
C-----------------------------------------------------------------------
  600 FORMAT (' MODFIX modifies VLB model files'/' (Version ',A,')'//
     1        ' Parameters are: INPUT, OUTPUT, SHIFT, LRTB, ROTATE,'/
     2        ' SCALE, NONEG, STRETCH, SQUASH, SORT, NUMBER.')
  610 FORMAT (' ++WARNING++ Input model truncated after ',I5,
     1        ' components.')
  620 FORMAT (F13.6,F11.3,F11.3,F11.3,F11.4,F11.3,I4)
  630 FORMAT (' ',I5,' components; total flux density = ',1PG10.3,' Jy')
  640 FORMAT (' (All of the components are delta functions.)')
  650 FORMAT (' ++WARNING++ The model contains non-delta-function ',
     1        ' components.'/' It will not be squashed.')
  810 FORMAT ('! ',A)
  820 FORMAT ('!   LRTB    = ',1PG13.6,3(',',1PG13.6))
  830 FORMAT ('!   SHIFT   = ',1PG13.6,',',1PG13.6)
  840 FORMAT ('!   ROTATE  = ',1PG13.6)
  850 FORMAT ('!   SCALE   = ',1PG13.6)
  855 FORMAT ('!   STRETCH = ',1PG13.6)
  860 FORMAT ('!   NONEG')
  865 FORMAT ('!   SQUASH')
  866 FORMAT ('!   SORT')
  867 FORMAT ('!   NUMBER  = ',I8)
  870 FORMAT ('! ---------------------------------------------')
C-----------------------------------------------------------------------
C
C            Prolog.
C
      WRITE (OUTC,600) VERSN
C
C            Read parameters.
C
   10 MODE = 0
      CALL KEYIN(PARS,VALS,NPARS,ENDMRK,MODE,INC,OUTC)
      WRITE (INPUT,'(10A8)') (VALS(I),I=1,10)
      L = MAX(1,LEN1(INPUT))
      WRITE (OUTPUT,'(10A8)') (VALS(I),I=11,20)
      M = MAX(1,LEN1(OUTPUT))
      XSHIFT = VALS(21)
      YSHIFT = VALS(22)
      X1     = VALS(23)
      X2     = VALS(24)
      Y1     = VALS(25)
      Y2     = VALS(26)
      ROTAT  = VALS(27)
      SCALE  = VALS(28)
      NONEG  = VALS(29).EQ.0D0
      STRET  = VALS(30)
      SQUASH = VALS(31).EQ.0D0
      SORT   = VALS(32).EQ.0D0
      N      = NINT(VALS(33))
      IF (N.GT.MXMOD) N = MXMOD
      WINDOW = (X1.NE.0.0 .OR. X2.NE.0.0 .OR. Y1.NE.0.0 .OR. Y2.NE.0.0)
C
C            Make sure X2 > X1, Y1 > Y2.
C
      IF (WINDOW) THEN
          IF (X1.GT.X2) THEN
            T = X2
            X2 = X1
            X1 = T
          END IF
          IF (Y2.GT.Y1) THEN
            T = Y2
            Y2 = Y1
            Y1 = T
          END IF
      END IF
C
C            Open files. If this fails, user may specify
C            new parameters (unless KEYIN has reached an EOF).
C
      IER = VLBOPE(INM, INPUT(1:L), 'MODEL', 'OLD', INPUT)
      IF (IER.NE.1) THEN
          IF (MODE.EQ.1) CALL ERROR('MODFIX file open error')
          GOTO 10
      END IF
      L = LEN1(INPUT)
C
      IER = VLBOPE(OUTM, OUTPUT(1:M), 'MODEL', 'NEW', OUTPUT)
      IF (IER.NE.1) THEN
          CLOSE (UNIT=INM)
          IF (MODE.EQ.1) CALL ERROR('MODFIX file open error')
          GOTO 10
      END IF
      M = LEN1(OUTPUT)
C
C Write header comments in new model file.
C
      CALL MKHIST('MODFIX', VERSN, HISTRY)
      WRITE (OUTM,810) HISTRY(1:LEN1(HISTRY))
      IF (WINDOW)       WRITE (OUTM,820) X1,X2,Y1,Y2
      IF (XSHIFT.NE.0.0 .OR. YSHIFT.NE.0.0 ) 
     1                  WRITE (OUTM,830) XSHIFT,YSHIFT
      IF (ROTAT.NE.0.0) WRITE (OUTM,840) ROTAT
      IF (SCALE.NE.1.0) WRITE (OUTM,850) SCALE
      IF (STRET.NE.1.0) WRITE (OUTM,855) STRET
      IF (NONEG)        WRITE (OUTM,860)
      IF (SQUASH)       WRITE (OUTM,865)
      IF (SORT)         WRITE (OUTM,866)
      IF (N.LT.MXMOD)   WRITE (OUTM,867) N
                        WRITE (OUTM,870)
C
C            Read input model; convert polar coordinates
C            to cartesian (XC increasing to West, YC to North, in
C            accordance with LRTB convention).
C            Report number of components and total flux density.
C
      WRITE (OUTC,'(2A)') ' Input model:  ', INPUT(1:L)
      NCOMP = 0
      TOTFLX = 0.0
      MAXTYP = 0
      DO 20 I=1,N
          CALL GTMODL(INM,CTYPE,FLUX,RADIUS,THETA,AXIS,RATIO,
     1                PHI,*30,OUTM)
          IF (FLUX.EQ.0.0) GOTO 30
          NCOMP = NCOMP+1
          IF (CTYPE.EQ.0) CTYPE = 1
          IF (AXIS.EQ.0.0) CTYPE = 0
          TYPE(NCOMP) = CTYPE
          MAXTYP = MAX(MAXTYP,CTYPE)
          FL(NCOMP) = FLUX
          TOTFLX = TOTFLX+FLUX
          XC(NCOMP) = -RADIUS*SIN(THETA*D2R)
          YC(NCOMP) = RADIUS*COS(THETA*D2R)
          AX(NCOMP) = AXIS
          AR(NCOMP) = RATIO
          PH(NCOMP) = PHI
   20 CONTINUE
      WRITE (OUTC,610) N
   30 WRITE (OUTC,630) NCOMP, TOTFLX
      IF (MAXTYP.EQ.0) WRITE (OUTC,640)
C
C            SQUASH the model if requested, and if it only
C            contains delta-functions.
C
      IF (SQUASH .AND. (MAXTYP.GT.0)) THEN
          WRITE (OUTC,650)
      ELSE IF (SQUASH) THEN
          DO 50 I=2,NCOMP
              DO 40 J=1,I-1
                  IF (XC(I).EQ.XC(J).AND.YC(I).EQ.YC(J)) THEN
                      FL(J) = FL(J) + FL(I)
                      FL(I) = 0.0
                      GOTO 50
                  END IF
   40         CONTINUE
   50     CONTINUE
      END IF
C
C            Apply transformations to model components.
C            STRETCH and ROTATE (which commute) are applied before
C            SHIFT.  Note that STRETCH affects FWHM and ROTATE
C            affects major-axis position angle.
C
      A = STRET*COS(ROTAT*D2R)
      B = STRET*SIN(ROTAT*D2R)
      DO 60 I=1,NCOMP
          FL(I) = FL(I)*SCALE
          T = XC(I)
          XC(I) = T*A - YC(I)*B - XSHIFT
          YC(I) = T*B + YC(I)*A + YSHIFT
          AX(I) = AX(I)*STRET 
          PH(I) = PH(I)+ROTAT
   60 CONTINUE
C
C Apply WINDOW by zeroing components.
C
      IF (WINDOW) THEN
          DO 70 I=1,NCOMP
              IF ((XC(I).LT.X1 .OR. XC(I).GT.X2 .OR.
     1             YC(I).GT.Y1 .OR. YC(I).LT.Y2)) FL(I) = 0.0
   70     CONTINUE
      END IF
C
C Sort into decreasing flux order.
C
      DO 80 I=1,NCOMP
          IND(I) = I
   80 CONTINUE
      IF (SORT) THEN
          CALL DRSORT(FL, NCOMP, IND)
      END IF
C
C Write out model, ignoring zero components, and if NONEG was requested,
C negative ones as well.
C
      NFINAL = NCOMP
      TOTFLX = 0.0
      DO 90 J=1,NCOMP
          I = IND(J)
          IF (FL(I) .EQ. 0.0) THEN
              NFINAL = NFINAL-1
          ELSE IF (NONEG .AND. (FL(I).LT.0.0)) THEN
              NFINAL = NFINAL-1
          ELSE
              TOTFLX = TOTFLX + FL(I)
              RADIUS = SQRT(XC(I)**2 + YC(I)**2)
              IF (RADIUS.GT.0.0) THEN
                  THETA = ATAN2(-XC(I),YC(I))/D2R
              ELSE
                  THETA = 0.0
              END IF
              IF (TYPE(I) .EQ. 0) THEN
                  WRITE (OUTM,620) FL(I),RADIUS,THETA
              ELSE
                  WRITE (OUTM,620) FL(I),RADIUS,THETA,AX(I),AR(I),PH(I),
     1                             TYPE(I)
              END IF
          END IF
   90 CONTINUE
C
C Report remaining number of components and total flux density.
C
      WRITE (OUTC,'(2A)') ' Output model: ', OUTPUT(1:M)
      WRITE (OUTC,630) NFINAL,TOTFLX
      END
