      PROGRAM MODGEN
C-----------------------------------------------------------------------
C MODGEN generates surface brightness models of astrophysical sources
C  based on input component temperatures, radii, distances, etc.
C
C D. L. Meier,  VAX-11 Fortran
C
C Version 1.0:  1991 Feb  1.  New program.  (DLM)
C Version 1.1:  1991 Feb 11. Add DISKTYPE and SPHTYPE parameters to
C                            allow control over model type of dust shells.
C                            Limit flattened disk dust emissivity to unity.
C                            Update physical constants in UCONV (DLM).
C-----------------------------------------------------------------------
      IMPLICIT NONE
      CHARACTER*(*)  PROGNM, VERSN
      PARAMETER  (PROGNM='MODGEN')
      PARAMETER  (VERSN='1.1 - 1991 Feb 11')
C
      INTEGER    MAXC,MAXC1,NPARS,INC,OUTC,OUTMOD,NSHELLS
      PARAMETER  (MAXC=40)
C     ! maximum number of input model components
      PARAMETER  (MAXC1=MAXC-1)
      PARAMETER  (NPARS=6*MAXC+23)
C     ! number of input parameters
      PARAMETER  (INC=5)
C     ! unit for input
      PARAMETER  (OUTC=6)
C     ! unit for output
      PARAMETER  (OUTMOD=10)
C     ! unit for output MODEL file
      PARAMETER  (NSHELLS=30)
C
      REAL*8     PI,TWOPI,RPDEG,RPMAS,VELC
      REAL*8     DEFTEMP,DEFDIAM,DEFRAD,DEFTHET,DEFTYPE,DEFNSPT
      PARAMETER  (PI=3.141592653589793 D0)
      PARAMETER  (TWOPI=2D0*PI)
      PARAMETER  (RPDEG=PI/180.D0)
      PARAMETER  (RPMAS=RPDEG/3.6D6)
      PARAMETER  (VELC=2.99792458D8)
      PARAMETER  (DEFTEMP=0D0)
      PARAMETER  (DEFDIAM=0D0)
      PARAMETER  (DEFRAD=0D0)
      PARAMETER  (DEFTHET=-1D0)
      PARAMETER  (DEFTYPE=2D0)
      PARAMETER  (DEFNSPT=0D0)
C
      CHARACTER*8  UTEMP, UDIAM, URAD, UDIST, ULAMB, UTMELT
      CHARACTER*80 OUTFIL,HISTRY
      INTEGER LEN1
      INTEGER I, IER, MODE, NC, NCOMP, NCENT, NS, N, ISEED
      INTEGER TYPE(MAXC), NSPT(MAXC) 
      INTEGER LASTTYPE, DISKTYPE, SPHTYPE
      REAL   RAN
      REAL*8 UCONV, BLKBDY
      REAL*8 TEMP(MAXC), DIAM(MAXC), RAD(MAXC), THET(MAXC)
      REAL*8 FREQ, DIST, INCL, ARATIO, ROT, XX, YY
      REAL*8 RADMIN, RADMAX, DLOGR, TCENT, DCENT, DISKDUST, SPHDUST
      REAL*8 TMELT, MELTFACT
      REAL*8 LASTTEMP, LASTDIAM
      REAL*8 OMEGA, FLUX, FLUX0, RADIUS, THETA, RSHELL, RINN, ROUT
      REAL*8 OMINN, OMOUT, SURFBR, TSHELL, FLINN, FLOUT, DPA
      REAL*8 PARS(NPARS), VALS(NPARS), ENDMRK
C
      DATA ISEED/3456757/
      DATA TMELT/1000D0/
      DATA ENDMRK/'/'/
      DATA PARS /MAXC*'TEMP', MAXC*'DIAMeter', 
     1       MAXC*'RADius', MAXC*'THETA', MAXC*'TYPE', MAXC*'NSPOTS', 
     2       'DISTance', 'LAMBDA', 
     3       'UTEMP', 'UDIAM', 'URAD', 'UDIST', 'ULAMB', 
     4       'INCL', 'ROTate', 'OUTMOD',9*' ', 'DISKDUST', 'SPHDUST',
     5       'DISKTYPe', 'SPHTYPe' /
      DATA VALS /'***UNSET',MAXC1*DEFTEMP, '***UNSET',MAXC1*DEFDIAM, 
     1       MAXC*DEFRAD, MAXC*DEFTHET, MAXC*DEFTYPE, MAXC*DEFNSPT, 
     2       '***UNSET','***UNSET', 
     3       'K', 'DSUN', 'AU', 'PC', 'MICRONS', 
     4       0D0, 0D0, 'OUTMOD', 9*' ', 0D0, 0D0,
     5       2D0, 2D0 /
C
C-----------------------------------------------------------------------
C Formats
C-----------------------------------------------------------------------
  620 FORMAT (1P,E13.6,E11.3,0P,F11.3,1P,E11.3,0P,F11.4,F11.3,I4)
  810 FORMAT ('! ',A)
  820 FORMAT ('! ',A,I4)
 6000 FORMAT(' ',A,' generates a MODEL file from component temps,',
     1         ' sizes, positions, and model distance'/
     2         ' (Version ',A,')'/)
 6010 FORMAT(A)
 6020 FORMAT (A8,2X,F10.1,2(F16.10,9X),F5.1)
 6030 FORMAT (/1X,A,/1X,A)
C
C---------------------------------------------------------------------
C Read control parameters
C---------------------------------------------------------------------
C
      WRITE (OUTC,6000) PROGNM, VERSN
    5 MODE = 0
      WRITE (OUTC,6010) ' Control parameters (end with /):'
      CALL KEYIN(PARS,VALS,NPARS,ENDMRK,MODE,INC,OUTC)
C
C---------------------------------------------------------------------
C     Deterimine number of components in model and fill in missing
C     information
C---------------------------------------------------------------------
      LASTTEMP = DEFTEMP
      LASTDIAM = DEFDIAM
      LASTTYPE = DEFTYPE
      DO 10 NC=1,MAXC
        TEMP(NC) = VALS(NC)
        IF (TEMP(NC) .NE. DEFTEMP) THEN
          LASTTEMP = TEMP(NC)
          NCOMP = NC
        END IF
        DIAM(NC) = VALS(MAXC+NC)
        IF (DIAM(NC) .NE. DEFDIAM) THEN
          LASTDIAM = DIAM(NC)
          NCOMP = NC
        END IF
        RAD(NC)  = VALS(2*MAXC+NC)
        THET(NC) = VALS(3*MAXC+NC)
        TYPE(NC) = VALS(4*MAXC+NC) + 0.1
        IF (TYPE(NC) .NE. DEFTYPE) THEN
          LASTTYPE = TYPE(NC)
          NCOMP = NC
        END IF
        NSPT(NC) = VALS(5*MAXC+NC) + 0.1
   10 CONTINUE
      RADMAX = 0D0
      RADMIN = 1D38
      DO 20 NC=1,NCOMP
        IF (TEMP(NC) .EQ. DEFTEMP) THEN
          TEMP(NC) = LASTTEMP
        END IF
        IF (TEMP(NC) .LT. 0D0) CALL ERROR('Negative temperatures')
        IF (DIAM(NC) .EQ. DEFDIAM) THEN
          DIAM(NC) = LASTDIAM
        END IF
        IF (RAD(NC) .GT. RADMAX) RADMAX = RAD(NC)
        IF ((RAD(NC).GT.0D0) .AND. (RAD(NC).LT.RADMIN)) RADMIN = RAD(NC)
        IF (TYPE(NC) .EQ. DEFTYPE) THEN
          TYPE(NC) = LASTTYPE
        END IF
   20 CONTINUE
C
C---------------------------------------------------------------------
C     Read remaining parameters and convert all into standard units
C---------------------------------------------------------------------
      WRITE (UTEMP, '(A8)') VALS(6*MAXC+3)
      CALL UPCASE(UTEMP)
      WRITE (UDIAM, '(A8)') VALS(6*MAXC+4)
      CALL UPCASE(UDIAM)
      WRITE (URAD,  '(A8)') VALS(6*MAXC+5)
      CALL UPCASE(URAD)
      WRITE (UDIST, '(A8)') VALS(6*MAXC+6)
      CALL UPCASE(UDIST)
      WRITE (ULAMB, '(A8)') VALS(6*MAXC+7)
      CALL UPCASE(ULAMB)
C     -- Convert frequency to Hz
      FREQ = VELC/(VALS(6*MAXC+2)*UCONV(ULAMB))
C     -- Convert distance into meters
      DIST = VALS(6*MAXC+1)*UCONV(UDIST)
C     -- Convert component radii extrema into milliarcseconds and extend
C     -- dust region interior to and beyond min and max radii
      RADMAX = UCONV(URAD)*RADMAX*DSQRT(10D0)/(DIST*RPMAS)
      RADMIN = UCONV(URAD)*RADMIN/(DSQRT(10D0)*DIST*RPMAS)
      DLOGR  = (DLOG(RADMAX)-DLOG(RADMIN))/NSHELLS
C     -- Convert inclination and rotation into radians and compute axial
C     -- ratio
      INCL = VALS(6*MAXC+8)*RPDEG
      ARATIO = DCOS(INCL)
      ROT  = VALS(6*MAXC+9)*RPDEG
C
      NCENT = 0
      TCENT = 0D0
      DO 30 NC=1,NCOMP
C       -- Convert temperatures to Hz
        TEMP(NC) = TEMP(NC)*UCONV(UTEMP)
C       -- Convert diameters and radii into meters and then milliarcseconds
        DIAM(NC) = DIAM(NC)*UCONV(UDIAM)/(DIST*RPMAS)
        RAD(NC)  = RAD(NC)*UCONV(URAD)/(DIST*RPMAS)
C       -- Determine if there is a central object and find the hottest one
        IF ((RAD(NC).EQ.0D0) .AND. (TEMP(NC).GT.TCENT)) THEN
          NCENT = NC
          TCENT = TEMP(NC)
          DCENT = DIAM(NC)
        END IF
C       -- If specified, choose angles of component positions randomly
        IF ((THET(NC).LT.0D0) .AND. (RAD(NC).GT.0D0))
     1     THET(NC)  = 360D0*RAN(ISEED)
C       -- Flatten model by the inclination angle INCL and rotate by ROT
        IF (RAD(NC) .NE. 0D0) THEN
          THET(NC) = THET(NC)*RPDEG
          XX = RAD(NC)*DSIN(THET(NC))
          YY = RAD(NC)*DCOS(THET(NC))*ARATIO
          RAD(NC) = DSQRT(XX*XX+YY*YY)
          THET(NC) = DATAN2(XX,YY) + ROT
C         -- Try to keep position angle of component between -PI and PI
          IF (THET(NC) .LT. -1D0*PI) THET(NC) = THET(NC) + TWOPI
C         -- Convert angles back to degrees
          THET(NC) = THET(NC)/RPDEG
        END IF
   30 CONTINUE
      WRITE (OUTFIL,  '(10A8)') (VALS(I),I=6*MAXC+10,6*MAXC+19)
      DISKDUST = VALS(6*MAXC+20)
C
C     Check if disk dust becomes optically thick when flattened
C     and limit emissivity accordingly
      IF (DISKDUST .GT. 0D0) THEN
        IF ((ARATIO/DISKDUST) .LT. 1D0) THEN
          WRITE (OUTC,'(/,1X,A)') 
     1        '*** WARNING *** Dust disk is so highly flattened that'
          WRITE (OUTC,'(1X,A)') 
     1        'implied emissivity is greater than unity.'
          DISKDUST = ARATIO
          WRITE (OUTC,'(1X,A,1PE10.3)') 
     1        'Setting DISKDUST parameter to axial ratio = ',DISKDUST
          WRITE (OUTC,'(/,1X,A)') 
     1        '*** WARNING *** Dust disk is now optically thick.'
          WRITE (OUTC,'(1X,2A,/)') 
     1        'Model may be incorrect as extinction is not taken into',
     2        ' account.'
        END IF
      END IF
      SPHDUST  = VALS(6*MAXC+21)
      DISKTYPE = VALS(6*MAXC+22) + 0.1
      SPHTYPE  = VALS(6*MAXC+23) + 0.1
C
C---------------------------------------------------------------------
C     Create the model components, plus any "star spots"
C---------------------------------------------------------------------
      OPEN(UNIT=OUTMOD,NAME=OUTFIL,STATUS='NEW',FORM='FORMATTED',
     1  CARRIAGECONTROL='LIST',IOSTAT=IER)
      IF (IER.NE.0) THEN
          WRITE (OUTC,'(1X,2A)') 
     1      '*** WARNING *** Cannot create file OUTMOD '//OUTFIL
          GOTO 5
      END IF
      CALL MKHIST('MODGEN', VERSN, HISTRY)
      WRITE (OUTMOD,810) HISTRY(1:LEN1(HISTRY))
      DO 50 NC=1,NCOMP
C
C     -- Compute surface brightness of component and compute flux in Jy by
C     -- multiplying by its solid angle 
        WRITE (OUTMOD,810) ' '
        WRITE (OUTMOD,820) 'Component #',NC
        OMEGA = 0.25D0*PI*((DIAM(NC)*RPMAS)**2)
        FLUX = BLKBDY(FREQ,TEMP(NC))*OMEGA/1.D-26
        WRITE (OUTMOD,620) FLUX,RAD(NC),THET(NC),DIAM(NC),1.0,0.0,
     1    TYPE(NC)
        IF (NSPT(NC) .GT. 0) THEN
          WRITE (OUTMOD,810) 'Spots for above component:'
          FLUX0=-0.10*FLUX/NSPT(NC)
          DO 40 N=1,NSPT(NC)
            FLUX  = FLUX0*RAN(ISEED)
C
C           -- Compute spot radius such that d(r**2/2) is constant 
C           -- and 0<r<0.45
            RADIUS= 0.10125D0*RAN(ISEED)
            RADIUS= DSQRT(2D0*RADIUS)
            THETA = 360.D0*RAN(ISEED)
            WRITE (OUTMOD,620) FLUX,RADIUS,THETA,0.1*DIAM(NC),1.0,0.0,2
   40     CONTINUE
        END IF
   50 CONTINUE
C
C     -- Create NSHELLS dust shells around model from ~1/3 smallest
C     -- radius to ~3 times largest component radius, taking into
C     -- account thermal equilibrium with central comonent
      IF (((DISKDUST.GT.0D0) .OR. (SPHDUST.GT.0D0)) .AND. 
     1   (NCENT.GT.0)) THEN
        WRITE (OUTMOD,810) 'Disk and Spherical Dust Shells:'
        DPA = 90.0 + ROT/RPDEG
C       -- Convert dust melting temperature to Hz
        UTMELT = 'K'
        TMELT = TMELT*UCONV(UTMELT)
        DO 60 NS=1,NSHELLS
C         -- Compute radii of this shell
          RINN = RADMIN*DEXP((NS-1)*DLOGR)
          ROUT = RADMIN*DEXP(NS*DLOGR)
          RSHELL = DSQRT(RINN*ROUT)
C         -- Compute shell's surface brightness and flux
          TSHELL = 0.5D0*TCENT*DSQRT(DCENT/RSHELL)
          OMINN = PI*((RINN*RPMAS)**2)
          OMOUT = PI*((ROUT*RPMAS)**2)
          MELTFACT = 1.0D0
          IF ((TSHELL/TMELT) .GT. 200D0) THEN
            MELTFACT = 0D0
          ELSE
            MELTFACT = DEXP(-TSHELL/TMELT)
          END IF
          SURFBR = BLKBDY(FREQ,TSHELL)*MELTFACT
          FLINN = -SURFBR*OMINN/1.D-26
          FLOUT =  SURFBR*OMOUT/1.D-26
C         -- Output spherical shells
          IF (SPHDUST .GT. 0D0) THEN
            WRITE (OUTMOD,620) SPHDUST*FLINN,0.0,0.0,RINN,1.0,0.0,
     1        SPHTYPE
            WRITE (OUTMOD,620) SPHDUST*FLOUT,0.0,0.0,ROUT,1.0,0.0,
     1        SPHTYPE
          END IF
C         -- Output disk shells, flattening and rotating properly
          IF (DISKDUST .GT. 0D0) THEN
            WRITE (OUTMOD,620) DISKDUST*FLINN,0.0,0.0,RINN,ARATIO,DPA,
     1        DISKTYPE
            WRITE (OUTMOD,620) DISKDUST*FLOUT,0.0,0.0,ROUT,ARATIO,DPA,
     1        DISKTYPE
          END IF
   60   CONTINUE
      END IF
C
      CLOSE(UNIT=OUTMOD)
C
C---------------------------------------------------------------------
      STOP
      END

      REAL*8 FUNCTION UCONV(UNITT)
      CHARACTER*8 UNITT
      INTEGER L, LEN1
C
      L = LEN1(UNITT)
C
C     Convert Temperature Units to Frequency (Hz)
      IF (UNITT(1:L) .EQ. 'K') THEN
        UCONV = 2.0836738D10
      ELSE IF (UNITT(1:L) .EQ. 'EV') THEN
        UCONV = 2.417988D14
      ELSE IF (UNITT(1:L) .EQ. 'KEV') THEN
        UCONV = 2.417988D17
      ELSE IF (UNITT(1:L) .EQ. 'MEV') THEN
        UCONV = 2.417988D20
      ELSE IF (UNITT(1:L) .EQ. 'GEV') THEN
        UCONV = 2.417988D23
      ELSE IF (UNITT(1:L) .EQ. 'TEV') THEN
        UCONV = 2.417988D26
C
C     Convert Frequency Units to Hz
      ELSE IF (UNITT(1:L) .EQ. 'HZ') THEN
        UCONV = 1D0
      ELSE IF (UNITT(1:L) .EQ. 'KHZ') THEN
        UCONV = 1D3
      ELSE IF (UNITT(1:L) .EQ. 'MHZ') THEN
        UCONV = 1D6
      ELSE IF (UNITT(1:L) .EQ. 'GHZ') THEN
        UCONV = 1D9
      ELSE IF (UNITT(1:L) .EQ. 'THZ') THEN
        UCONV = 1D12
      ELSE IF (UNITT(1:L) .EQ. 'PHZ') THEN
        UCONV = 1D15
      ELSE IF (UNITT(1:L) .EQ. 'EHZ') THEN
        UCONV = 1D18
C
C     Convert Length Units to Meters
      ELSE IF (UNITT(1:5) .EQ. 'ANGST') THEN
        UCONV = 1D-8
      ELSE IF (UNITT(1:6) .EQ. 'MICRON') THEN
        UCONV = 1D-6
      ELSE IF (UNITT(1:L) .EQ. 'MM') THEN
        UCONV = 1D-3
      ELSE IF (UNITT(1:L) .EQ. 'CM') THEN
        UCONV = 1D-2
      ELSE IF (UNITT(1:L) .EQ. 'M') THEN
        UCONV = 1D0
      ELSE IF (UNITT(1:5) .EQ. 'METER') THEN
        UCONV = 1D0
      ELSE IF (UNITT(1:L) .EQ. 'KM') THEN
        UCONV = 1D3
      ELSE IF (UNITT(1:L) .EQ. 'IN') THEN
        UCONV = 2.54D-2
      ELSE IF (UNITT(1:L) .EQ. 'FT') THEN
        UCONV = 3.048D-1
      ELSE IF (UNITT(1:L) .EQ. 'FEET') THEN
        UCONV = 3.048D-1
      ELSE IF (UNITT(1:L) .EQ. 'YD') THEN
        UCONV = 9.144D-1
      ELSE IF (UNITT(1:4) .EQ. 'YARD') THEN
        UCONV = 9.144D-1
      ELSE IF (UNITT(1:L) .EQ. 'MI') THEN
        UCONV = 1.609D3
      ELSE IF (UNITT(1:4) .EQ. 'MILE') THEN
        UCONV = 1.609D3
      ELSE IF (UNITT(1:L) .EQ. 'REARTH') THEN
        UCONV = 6.371D6
      ELSE IF (UNITT(1:L) .EQ. 'RSUN') THEN
        UCONV = 6.960D8
      ELSE IF (UNITT(1:L) .EQ. 'DSUN') THEN
        UCONV = 1.392D9
      ELSE IF (UNITT(1:L) .EQ. 'AU') THEN
        UCONV = 1.495985D11
      ELSE IF (UNITT(1:L) .EQ. 'LY') THEN
        UCONV = 9.4605D15
      ELSE IF (UNITT(1:L) .EQ. 'PC') THEN
        UCONV = 3.0856D16
      ELSE IF (UNITT(1:6) .EQ. 'PARSEC') THEN
        UCONV = 3.0856D16
      ELSE IF (UNITT(1:L) .EQ. 'KPC') THEN
        UCONV = 3.0856D19
      ELSE IF (UNITT(1:L) .EQ. 'MPC') THEN
        UCONV = 3.0856D22
      ELSE IF (UNITT(1:L) .EQ. 'GPC') THEN
        UCONV = 3.0856D25
      ELSE IF (UNITT(1:L) .EQ. 'Z') THEN
C       Redshift, assuming Ho=75 km/s/Mpc
        UCONV = 12.3339D25
      ELSE IF (UNITT(1:5) .EQ. 'REDSH') THEN
        UCONV = 12.3339D25
      ELSE 
        WRITE(6,'(A,A)') '*** WARNING *** UNKNOWN UNIT: ',UNITT(1:L)
        WRITE(6,'(A)')   'CONVERSION FACTOR TO HZ OR METERS SET TO 1.0!'
        UCONV = 1D0
      END IF
      RETURN
      END

      REAL*8 FUNCTION BLKBDY(FREQ,TEMP)
      REAL*8 FREQ, TEMP
C
C     Compute surface brightness of a black body of a given Temperature
C     (in units of Hz!) at a given frequency (also in Hz)
C
      REAL*8     VELC, PLANCK
      PARAMETER  (VELC=2.99792458D8)
      PARAMETER  (PLANCK=6.6260755D-34)
C                (Planck's constant in J/Hz)
C
      REAL*8 HNUOKT
      HNUOKT = FREQ/TEMP
      IF (HNUOKT .LE. 1.D-5) THEN
        BLKBDY = 1.0D0
      ELSE IF (HNUOKT .GT. 200.D0) THEN
        BLKBDY = 0.0D0
      ELSE
        BLKBDY = HNUOKT/(DEXP(HNUOKT)-1.0D0)
      END IF
C
C     Compute total black body surface brightness; note that TEMP
C     is in Hz
      BLKBDY = 2D0*PLANCK*TEMP*(FREQ*FREQ)*BLKBDY/(VELC*VELC)
      RETURN
      END
