      PROGRAM MODMAP
C-----------------------------------------------------------------------
C Generate a standard FITS-format map by convolution of a Caltech-format
C model file with a gaussian beam (like MODPLOT).  Note: only gaussian
C and delta-function components are admitted.
C
C Author:               T.J. Pearson (TJP)
C Language:             Fortran-77 (extensions: some external names
C                                   are longer than 6 char)
C Written:              1983 Aug 29.
C
C Modifications:
C Version 1.0:  1983 Aug 29 - T. J. Pearson (TJP).
C Version 1.1:  1984 Jan 20 - use new FITS_ routines; remove
C                             FORMAT="MODPLOT" option.
C Version 1.2:  1986 Oct 31 - add TELESCOP and DATE-OBS parameters
C                             (Tasso Tzioumis).
C Version 1.3:  1988 Jun 24 - use REAL instead of DOUBLE (error found
C                             on Convex) and changes for Fortran-77
C                             standard (TJP).
C Version 1.4:  1988 Jul  7 - put pixel (1,1) in lower left corner (TJP)
C-----------------------------------------------------------------------
      CHARACTER*(*) VERSN
      PARAMETER (VERSN='1.4 - 1988 Jul 7')
C
      INTEGER INC, OUTC, INMOD
      PARAMETER (INC=5, OUTC=6, INMOD=1)
C     -- Number of KEYIN parameters.
      INTEGER NPARS
      PARAMETER (NPARS=38)
C     -- Maximum number of components.
      INTEGER MAXCMP
      PARAMETER (MAXCMP=10000)
C     -- Maximum number of map pixels.
      INTEGER MAXPIX
      PARAMETER (MAXPIX=512*512)
      REAL PI, DTR
      PARAMETER (PI=3.14159265359)
      PARAMETER (DTR=PI/180.0)
C
      DOUBLE PRECISION PARS(NPARS),VALS(NPARS),ENDMRK
      REAL    MMAX, MMIN, JYBEAM
      CHARACTER*128 FILEIN, FILEOU
      CHARACTER*16 SOURCE, TELESC, DATEOB
      INTEGER NAXISN(4), BITPIX, IER
      DOUBLE PRECISION AMP, TOTAL
      REAL    SUM(MAXPIX)
      REAL    FCPT(MAXCMP)
      REAL    C(MAXCMP)
      REAL    RAD(MAXCMP), THETA(MAXCMP), A(MAXCMP)
      REAL    R(MAXCMP), PHI(MAXCMP)
      REAL    B(MAXCMP), XO(MAXCMP), YO(MAXCMP), COSPHI(MAXCMP)
      REAL    SINPHI(MAXCMP), AC(MAXCMP)
      REAL    EXPON(3000)
      REAL    BSCALE,BZERO
C
      INTEGER VLBOPE, LEN1
C
      DATA PARS / 'MOdfile', 9*' ',
     1            'MApfile', 9*' ',
     2            'BItpix',
     3            4*'LRtb',
     4            'XYint',
     5            'ROtate',
     6            3*'BEam',
     7            'FRequenc',
     8            'SOurce', ' ',
     9            'TElescop', ' ',
     A            'DAte-obs', ' ',
     B            'Number' /
      DATA VALS / '***UNSET', 9*' ',
     1            10*' ',
     2            16D0,
     3            4*'***UNSET',
     4            '***UNSET',
     5            0D0,
     6            '***UNSET', 0D0, 0D0,
     7            '***UNSET',
     8            'MODEL', ' ',
     9            2*' ',
     A            2*' ',
     B            0D0 /
      DATA ENDMRK /'/'/
C-----------------------------------------------------------------------
  600 FORMAT (' MODMAP generates maps from VLB model files'/
     1        ' (Version ',A,')'/)
 2003 FORMAT (' Total flux density in model:',F8.2,' Jy.'/
     1        ' Total flux density in map:  ',F8.2,' Jy.'/
     2        ' Max =',F9.4,' Jy/beam.'/
     3        ' Min =',F9.4,' Jy/beam.')
 2004 FORMAT (' Frequency =',F8.2,' MHz.')
 4003 FORMAT (' Beam:  ',F6.2,',',F6.2,',',F6.1)
 4013 FORMAT (' Map size: ',I5,' pixels (x) by ',I5,' pixels (y).')
 4016 FORMAT (' RMS value for map= ',1PE9.2,' Jy/beam'/
     +        '                  =',0PF8.3,' % of peak')
 4018 FORMAT (' Number of components in map: ',I6)
 5001 FORMAT (' Window (LRTB) = ',3(F9.2,','),F9.2)
 4027 FORMAT (' ++WARNING++ Illegal value specified for BITPIX.',
     1        ' BITPIX=32 assumed.')
 4017 FORMAT (' ++WARNING++ Maximum number of model components is',
     1        I6,'.')
C-----------------------------------------------------------------------
C
C Prolog.
C
      CALL SCLOCK
      WRITE (OUTC,600) VERSN
C
      DO 10 I=1,3000
          X = (I-1.) * (-2.77259)/500.0
          EXPON(I) = EXP(X)
   10 CONTINUE
C
C Read parameters.
C
   20 MODE = 0
      CALL KEYIN (PARS, VALS, NPARS, ENDMRK, MODE, INC, OUTC)
      WRITE (FILEIN,'(10A8)') (VALS(I),I=1,10)
      L = LEN1(FILEIN)
      WRITE (FILEOU,'(10A8)') (VALS(I),I=11,20)
      M = LEN1(FILEOU)
      BITPIX = VALS(21)
      WINDL  = VALS(22)
      WINDR  = VALS(23)
      WINDT  = VALS(24)
      WINDB  = VALS(25)
      DEL    = VALS(26)
      ROTANG = VALS(27)
      FREQ   = VALS(31)
      WRITE (SOURCE, '(2A8)') VALS(32),VALS(33)
      WRITE (TELESC, '(2A8)') VALS(34),VALS(35)
      WRITE (DATEOB, '(2A8)') VALS(36),VALS(37)
      NUMBER = VALS(38)
C
C Check parameters.
C
      IF (BITPIX.NE.16 .AND. BITPIX.NE.32) THEN
          BITPIX = 32
          WRITE (OUTC,4027)
      END IF
      IF (NUMBER.LE.0) THEN
          NUMBER = MAXCMP
      ELSE IF (NUMBER .GT. MAXCMP) THEN
          NUMBER = MAXCMP
          WRITE (OUTC,4017) NUMBER
      END IF
C
C Beam parameters: FWHM major, minor (mas); pa of major axis (degrees).
C
      BMAJ = VALS(28)
      BMIN = VALS(29)
      BPA  = VALS(30)
      IF (BMIN.EQ.0D0) BMIN = BMAJ
      IF (BMIN.GT.BMAJ) THEN
          TEMP = BMAJ
          BMAJ = BMIN
          BMIN = TEMP
      END IF
      AXIS = BMAJ * 0.001
      RATIO = BMIN / BMAJ
      ANGLE = (BPA + ROTANG) * DTR
      COSANG = COS(ANGLE)
      SINANG = SIN(ANGLE)
C
C Round window so that point (0,0) lies exactly on a gridpoint.
C The map array will be NPIX = NX x NY pixels.
C
      IF (DEL.LE.0.0) THEN
          WRITE (OUTC,'(A)') ' ++WARNING++ XYINT must be > 0. '//
     1          'Try again with new XYINT.'
          IF (MODE.EQ.1) CALL ERROR('MODMAP error')
          GOTO 20
      END IF
      NX1 = NINT(WINDL/DEL)
      NX2 = NINT(WINDR/DEL)
      NY1 = NINT(WINDT/DEL)
      NY2 = NINT(WINDB/DEL)
      NX = NX2-NX1+1
      NY = NY1-NY2+1
      NPIX = NX*NY
      WINDL = NX1*DEL
      WINDR = NX2*DEL
      WINDT = NY1*DEL
      WINDB = NY2*DEL
      IF (NPIX.GT.MAXPIX) THEN
          WRITE (OUTC,'(A)') ' ++WARNING++ The output map is too '//
     1          'big. Try again with new LRTB or XYINT.'
          IF (MODE.EQ.1) CALL ERROR('MODMAP error')
          GOTO 20
      END IF
C
C Check window: in LRTB, L<R and T>B.
C
      IF (WINDT.LE.WINDB .OR. WINDL.GE.WINDR) THEN
          WRITE (OUTC,'(A)') ' ++WARNING++ Illegal window. '//
     1          'Try again with new LRTB.'
          IF (MODE.EQ.1) CALL ERROR('MODMAP error')
          GOTO 20
      END IF
C
C Open files. If this fails, user may specify new parameters 
C (unless KEYIN has reached an EOF).
C
      IER = VLBOPE(INMOD, FILEIN(1:L), 'MODEL', 'OLD', FILEIN)
      IF (IER.NE.1) THEN
          IF (MODE.EQ.1) CALL ERROR('MODMAP error')
          GOTO 20
      END IF
      L = LEN1(FILEIN)
C
      CALL FITS_INIT(FILEOU(1:M),FILEOU,M,IER)
      IF (IER.NE.0) THEN
          WRITE (OUTC,'(1X,3A)')
     1          '++WARNING Cannot create file MAPFILE="',
     2          FILEOU(1:M),'"'
          CLOSE (UNIT=INMOD)
          IF (MODE.EQ.1) CALL ERROR('MODMAP error')
          GOTO 20
      END IF
C
      WRITE (OUTC,'(2A)') ' Input model:  ', FILEIN(1:L)
      WRITE (OUTC,'(2A)') ' Output map:   ', FILEOU(1:M)
C
C Read the model, but not more than min(NUMBER,MAXCMP) components.
C
      LMAX = 0
      ISDELT = 0
      NCOMP  = 0
      DO 30 I=1,NUMBER
          FCPT(I) = 0.
          CALL GTMODL(INMOD,L,FCPT(I),RAD(I),THETA(I),A(I),R(I),
     1                  PHI(I),*40,0)
          IF (FCPT(I).EQ.0.) GOTO 40
          LMAX = MAX(LMAX,L)
          NCOMP = NCOMP + 1
          IF (A(I).EQ.0.0) ISDELT = ISDELT + 1
   30 CONTINUE
      WRITE (OUTC,'(A,I7,A)') ' Only first',NUMBER,
     1          ' model components will be used.'
   40 CLOSE (UNIT=INMOD)
      IF (LMAX.GT.1) THEN
          WRITE (OUTC,'(A)')
     1          ' ++WARNING++ Model contains non-gaussian components.'
      END IF
      ISGAUS = NCOMP - ISDELT
      FLUX = 0.
      DO 50 I=1,NCOMP
          THET = (THETA(I) + ROTANG) * DTR
          RADI = RAD(I) * 0.001
          YO(I) =  RADI*COS(THET)
          XO(I) = -RADI * SIN(THET)
          FLUX = FLUX + FCPT(I)
   50 CONTINUE
C
C Display parameters.
C
      WRITE (OUTC,4013) NX,NY
      WRITE (OUTC,5001) WINDL,WINDR,WINDT,WINDB
      WRITE (OUTC,4003) BMAJ,BMIN,BPA
      WRITE (OUTC,4018) NCOMP
C
C Compute the map.
C
      IF (ISGAUS .GT. 0) THEN
C           This routine sorts a model so that gaussians are first,
C           followed by delta fns
            MOVE = 0
            DO 500 I = 1, NCOMP
              IF (A(I) .NE. 0.) THEN
                MOVE = MOVE + 1
                TEMP = FCPT(I)
                FCPT(I) = FCPT(MOVE)
                FCPT(MOVE) = TEMP
                TEMP = A(I)
                A(I) = A(MOVE)
                A(MOVE) = TEMP
                TEMP = R(I)
                R(I) = R(MOVE)
                R(MOVE) = TEMP
                TEMP = PHI(I)
                PHI(I) = PHI(MOVE)
                PHI(MOVE) = TEMP
                TEMP = XO(I)
                XO(I) = XO(MOVE)
                XO(MOVE) = TEMP
                TEMP = YO(I)
                YO(I) = YO(MOVE)
                YO(MOVE) = TEMP
              ENDIF
              IF (MOVE .GE. ISGAUS) GO TO 510
  500       CONTINUE
  510       CONTINUE
            DO 11 K = 1,ISGAUS
              PHIK = (PHI(K)+ROTANG) * DTR
              AK = A(K) * 0.001
C             Convolve the gaussian components with beam using GCONVL
              CALL GCONVL(AXIS,RATIO,ANGLE,AK,R(K),PHIK,AC(K),RC,PHIC)
              COSPHI(K) = COS(PHIC)
              SINPHI(K) = SIN(PHIC)
              B(K) = AC(K) * RC
              C(K) = FCPT(K) * 2.77259 / (PI*AC(K)*B(K))
   11       CONTINUE
      END IF
      IF (ISDELT .GT. 0) THEN
          BK = AXIS * RATIO
          TEMP = 2.77259 / (PI * AXIS * BK)
          KSTART = ISGAUS + 1
          DO 101 K = KSTART, NCOMP
              C(K) = FCPT(K) * TEMP
  101     CONTINUE
      END IF
C
      XX = WINDL / 1000.
      YY = WINDT / 1000.
      DEL = DEL * 0.001
      DO 102 I=1,NPIX
          SUM(I) = 0.0
  102 CONTINUE
C
C Evaluate the contributions of gaussians.
C
      IF (ISGAUS.GT.0) THEN
          X = XX-DEL
          DO 17 I = 1,NX
              X = X+DEL
              Y = YY+DEL
              DO 18 J = 1,NY
                  Y = Y-DEL
                  DO 19 K = 1,ISGAUS
                      XP =  (X-XO(K))*COSPHI(K)   + (Y-YO(K))*SINPHI(K)
C                     -- For each gaussian, find all contributions
C                     -- within 2 FWHMs
                      IF (ABS(XP) .GT. B(K)*1.6557) GO TO 19
                      YP = -(X-XO(K))*SINPHI(K)   + (Y-YO(K))*COSPHI(K)
                      IF (ABS(YP) .GT. AC(K)*1.6557) GO TO 19
                      AMP = C(K)*EXP(-2.77259*(XP/B(K))**2 - 
     1                                2.77259*(YP/AC(K))**2)
                      SUM(I+(J-1)*NX) = SUM(I+(J-1)*NX) + AMP
   19             CONTINUE
   18         CONTINUE
   17     CONTINUE
      END IF
C
C Evaluate the contributions of gaussians.
C
      IF (ISDELT.GT.0) THEN
          SLIMIT = 1.6557 * AXIS
          SINBK = SINANG/BK
          COSBK = COSANG/BK
          SINAX = SINANG/AXIS
          COSAX = COSANG/AXIS
          DO 117 K = KSTART, NCOMP
              NXLOW = (XO(K) - SLIMIT -XX) / DEL - 0.5
              NXHI  = (XO(K) + SLIMIT -XX) / DEL + 0.5
              NYHI  = (YY - YO(K) + SLIMIT) / DEL + 0.5
              NYLOW = (YY - YO(K) - SLIMIT) / DEL - 0.5
              IF (NXLOW .GT. NX .OR. NYLOW .GT. NY) GO TO 117
              IF (NXHI  .LT. 1  .OR. NYHI  .LT. 1 ) GO TO 117
              IF (NXLOW .LT. 1) NXLOW = 1
              IF (NXHI  .GT. NX) NXHI = NX
              IF (NYLOW .LT. 1) NYLOW = 1
              IF (NYHI .GT. NY) NYHI = NY
              X = XX + (NXLOW-2.) * DEL
              DO 118 I = NXLOW, NXHI
                X = X + DEL
                Y = YY - (NYLOW-2.) * DEL
                DO 119 J = NYLOW, NYHI
                Y = Y - DEL
                XP = (X-XO(K)) * COSBK + (Y-YO(K)) * SINBK
                IF (ABS(XP) .GT. 1.6557) GO TO 119
                YP = (Y-YO(K)) * COSAX - (X-XO(K)) * SINAX
                XRAD = 500. * (XP**2 + YP**2) + 1.5
                IF(XRAD .LT. 3000)
     1            SUM(I+(J-1)*NX) = SUM(I+(J-1)*NX)  + C(K)*EXPON(XRAD)
  119           CONTINUE
  118         CONTINUE
  117       CONTINUE
      END IF
C
C Convert map from 'Jy/sq arcsec' to 'Jy/beam'.
C
      JYBEAM = (PI*BMAJ*BMIN/(4.E6*LOG(2.)))
      DO 200 I=1,NPIX
          SUM(I) = SUM(I)*JYBEAM
  200 CONTINUE
C
C Find map total, maximum, minimum and rms. Units are 'Jy/sq arcsec'.
C
      TOTAL = SUM(1)
      MMAX = SUM(1)
      MMIN = SUM(1)
      RMS = SUM(1)**2
      DO 210 I=2,NPIX
          TOTAL = TOTAL + SUM(I)
          MMAX = MAX(MMAX,SUM(I))
          MMIN = MIN(MMIN,SUM(I))
          RMS = RMS + SUM(I)**2
  210 CONTINUE
      RMS = SQRT(RMS/(NX*NY))
      RMSP = 100. * RMS/MMAX
C
      TOTAL = TOTAL*(DEL**2)/JYBEAM
      WRITE (OUTC,2003) FLUX,TOTAL,MMAX,MMIN
      WRITE (OUTC,4016) RMS,RMSP
      WRITE (OUTC,2004) FREQ
C
C Write out map file in FITS format.
C
      NAXIS = 4
      NAXISN(1) = NX
      NAXISN(2) = NY
      NAXISN(3) = 1
      NAXISN(4) = 1
      BZERO = 0.0
      RMAX = 32000.
      IF (BITPIX.EQ.32) RMAX = 2E9
      BSCALE = MAX(ABS(MMAX),ABS(MMIN))/RMAX
      CALL FITS_HSTAN(BITPIX,NAXIS,NAXISN,BSCALE,BZERO,IER)
      CALL FITS_WSTR ('BUNIT',    'JY/BEAM',
     1                'Brightness units', IER)
      CALL FITS_WSTR ('OBJECT',   SOURCE,    ' ', IER)
      IF (TELESC.NE.' ') CALL FITS_WSTR ('TELESCOP', TELESC, ' ',
     1                                     IER)
      CALL FITS_WSTR ('INSTRUME', 'MODEL', ' ', IER)
      IF (DATEOB.NE.' ') CALL FITS_WSTR ('DATE-OBS', DATEOB, ' ',
     1                                    IER)
      CALL FITS_WREAL('DATAMAX', MMAX, 'Map maximum (Jy/beam)', IER)
      CALL FITS_WREAL('DATAMIN', MMIN, 'Map minimum (Jy/beam)', IER)
      CALL FITS_WREAL('BMAJ', BMAJ/3600000.,
     1                'Beam major axis FHWM (deg)', IER)
      CALL FITS_WREAL('BMIN', BMIN/3600000.,
     1                'Beam minor axis FHWM (deg)', IER)
      CALL FITS_WREAL('BPA', BPA,
     1                'Beam major axis position angle (deg)', IER)
C
C Axis 1.
C
      PIX1 = NINT(1-WINDL/DEL/1000.)
      DEL1 = -DEL/3600D0
      CALL FITS_WREAL('CRVAL1',   0.0,       ' ', IER)
      CALL FITS_WREAL('CRPIX1',   PIX1,      ' ', IER)
      CALL FITS_WSTR ('CTYPE1',   'RA---SIN',' ', IER)
      CALL FITS_WREAL('CDELT1',   DEL1,      ' ', IER)
      CALL FITS_WREAL('CROTA1',   0.0,       ' ', IER)
C
C Axis 2.
C
      PIX2 = NINT(1-WINDB/DEL/1000.)
      DEL2 = DEL/3600D0
      CALL FITS_WREAL('CRVAL2',   0.0,       ' ', IER)
      CALL FITS_WREAL('CRPIX2',   PIX2,      ' ', IER)
      CALL FITS_WSTR ('CTYPE2',   'DEC--SIN',' ', IER)
      CALL FITS_WREAL('CDELT2',   DEL2,      ' ', IER)
      CALL FITS_WREAL('CROTA2',   0.0,       ' ', IER)
C
C Axis 3 - Frequency (degenerate).
C
      CALL FITS_WREAL('CRVAL3',   FREQ*1E6,  ' ', IER)
      CALL FITS_WREAL('CRPIX3',   1.0,       ' ', IER)
      CALL FITS_WSTR ('CTYPE3',   'FREQ',    ' ', IER)
      CALL FITS_WREAL('CDELT3',   0.0,       ' ', IER)
      CALL FITS_WREAL('CROTA3',   0.0,       ' ', IER)
C
C Axis 4 - Stokes parameter (degenerate).
C
      CALL FITS_WREAL('CRVAL4',   1.0,       ' ', IER)
      CALL FITS_WREAL('CRPIX4',   1.0,       ' ', IER)
      CALL FITS_WSTR ('CTYPE4',   'STOKES',  ' ', IER)
      CALL FITS_WREAL('CDELT4',   1.0,       ' ', IER)
      CALL FITS_WREAL('CROTA4',   0.0,       ' ', IER)
C
C History.
C
      CALL FITS_WCMT('HISTORY', 'MODMAP version:  '//VERSN, IER)
      CALL FITS_HIS(IER)
      CALL FITS_WCMT('HISTORY', 'Model file:      '//FILEIN, IER)
C
C End FITS header; write FITS data (from bottom up) and close file.
C
      CALL FITS_WEND(IER)
      DO 250 I=1,NY
          CALL FITS_WRAYF(SUM(1+(NY-I)*NX),NX,BITPIX,BSCALE,BZERO,IER)
  250 CONTINUE
      CALL FITS_CLOSE(IER)
C
C And we're done.
C
      WRITE (OUTC, '(1X)')
      CALL REPORT(OUTC)
      CALL REPCPU(OUTC)
C-----------------------------------------------------------------------
      END
