      PROGRAM MRGCOP
C-----------------------------------------------------------------------
C MRGCOPY: to copy one MERGE-format file to another. This is a dummy
C program which can be modified for special applications: see the sample
C code commented out by "C#". 
C
C Parameters:
C      INPUT = input file name
C      OUTPUT = output file name
C      HISTORY = text to be written in header
C
C T.J. Pearson, standard Fortran 77.
C
C Subroutines required:
C      KEYIN, LEN1, MKHIST, PUTOUT, VLBOPE.
C
C Version 1.0:  1984 May 2.
C Version 1.1:  1985 Jul 24. Add HISTORY parameter, and remove the
C       "sample code" that erroneously changed UT in each record (TJP).
C Version 1.2:  1986 Mar 18. Improve code for writing new history.
C Version 1.3:  1987 Dec  4. Improve code for writing new history.
C Version 1.4:  1988 Jul 12. Use VLBOPE routine.
C version 1.5:  1990 Nov 13. Remove unused variables.
C-----------------------------------------------------------------------
C
C Constants.
C
      CHARACTER*(*) PROGNM, VERSN
      INTEGER INC,OUTC,INDAT,OUTDAT,MAXS,MAXB,MAXT,NPARS
      DOUBLE PRECISION PI,SIDR,VELC
C     -- Program version and date:
      PARAMETER (PROGNM='MRGCOPY')
      PARAMETER (VERSN='1.5 - 1990 Nov 13')
C     -- Fortran unit numbers used:
      PARAMETER (INC=5,OUTC=6,INDAT=8,OUTDAT=9)
C     -- Maximum numbers of stations, baselines, and triangles:
      PARAMETER (MAXS=20)
      PARAMETER (MAXB=(MAXS-1)*MAXS/2)
      PARAMETER (MAXT=MAXB*(MAXS-2)/3)
C     -- Number of input parameters:
      PARAMETER (NPARS=30)
C     -- Pi:
      PARAMETER (PI=3.141592653589793D0)
C     -- Radians of ST per sec of UT:
      PARAMETER (SIDR=(PI/2592000.0D0)*1.0027379093D0)
C     -- Speed of light:
      PARAMETER (VELC=2.997925D8)
C-----------------------------------------------------------------------
C Variables.
C
      CHARACTER*8  STNAME(MAXS)
      CHARACTER*16 SOURCE
      INTEGER   IUT, IS, IB, IT, NS, NB, NT, I, J, IER, MODE
      INTEGER   IZ, IY, IS1, IS2, NHIST
      INTEGER   NNEW, NDWPB
      INTEGER   NBI(MAXB), NB1(MAXB), NB2(MAXB)
      INTEGER   NTI(MAXT)
      INTEGER   REC1(20), IDUM(8)
      REAL      INREC(7*MAXB+2*MAXT)
      DOUBLE PRECISION STX(MAXS), STY(MAXS), STZ(MAXS),
     1          RA, DEC, RAP, DECP, GASTM, FREQ, FLUX,
     2          WAVE,BX,BY,BZ(MAXB),BOFF(MAXB),BXY(MAXB),
     3          COSDEC,SINDEC, TINTG, AVG, BW,
     4          PARS(NPARS),VALS(NPARS),ENDMRK
      CHARACTER*80 INDSN, OUTDSN, OLDHIS, HISTRY, ADDHIS(3)
C-----------------------------------------------------------------------
C Functions called.
C
      INTEGER   LEN1, VLBOPE
C-----------------------------------------------------------------------
C Names and default values of KEYIN parameters.
C
      DATA  PARS/ 'Input',   9*' ', 'Output',  9*' ', 'History',9*' ' /
      DATA  VALS/ '***UNSET',9*' ', '***UNSET',9*' ', 10*' ' /
      DATA  ENDMRK/ '/' /
C-----------------------------------------------------------------------
C
C Introduction.
C
      CALL PUTOUT(PROGNM//' copies MERGE files')
      CALL PUTOUT('(Version '//VERSN//')')
C
C Control parameters.
C
      MODE = 0
  100 IF (MODE.NE.0) CALL ERROR('Bad input parameters in '//PROGNM)
      CALL PUTOUT(' ')
      CALL PUTOUT('Parameters (INPUT, OUTPUT, HISTORY):')
      CALL KEYIN(PARS,VALS,NPARS,ENDMRK,MODE,INC,OUTC)
      WRITE (INDSN,'(10A8)') (VALS(I),I=1,10)
      WRITE (OUTDSN,'(10A8)') (VALS(I),I=11,20)
      WRITE (HISTRY,'(10A8)') (VALS(I),I=21,30)
C
C Open input Merge file.
C
      IER = VLBOPE(INDAT, INDSN, 'MERGE', 'OLD', INDSN)
      IF (IER.NE.1) GOTO 100
C
C Open output Merge file.
C
      IER = VLBOPE(OUTDAT, OUTDSN, 'MERGE', 'NEW', OUTDSN)
      IF (IER.NE.1) THEN
          CLOSE (UNIT=INDAT)
          GOTO 100
      END IF
C
C Make the new history records.
C
      CALL MKHIST(PROGNM, VERSN, ADDHIS(1))
      IF (HISTRY.NE.' ') THEN
          NNEW = 2
          ADDHIS(2) = '- '//HISTRY(1:78)
      ELSE
          NNEW = 1
      END IF
      DO 105 I=1,NNEW
          CALL PUTOUT(ADDHIS(I)(1:LEN1(ADDHIS(I))))
  105 CONTINUE
C
C Read and re-write header. Merge format level must be 1.
C
      READ (INDAT,END=82,ERR=82) (REC1(J),J=1,20)
      IF (REC1(1).NE.1) GOTO 82
      NDWPB = 4
      NHIST = REC1(2)
      NS    = REC1(4)
      NB    = REC1(5)
      NT    = REC1(6)
      IF (NS.LT.2) GOTO 82
      IF (NS.GT.MAXS) GOTO 82
      REC1(2) = REC1(2) + NNEW
      REC1(3) = REC1(3) + NNEW
      WRITE (OUTDAT) (REC1(J),J=1,20)
C
      IF (NB.GT.0) THEN
          READ (INDAT,END=82,ERR=82) (NBI(IB),IB=1,NB)
          WRITE (OUTDAT) (NBI(IB),IB=1,NB)
          DO 110 IB=1,NB
              NB1(IB) = NBI(IB)/100
              NB2(IB) = NBI(IB) -  NB1(IB)*100
  110     CONTINUE
      END IF
C
      IF (NT.GT.0) THEN
          READ (INDAT,END=82,ERR=82) (NTI(IT),IT=1,NT)
          WRITE (OUTDAT) (NTI(IT),IT=1,NT)
      END IF
C
      DO 120 I=1,NHIST
          READ (INDAT,END=82,ERR=82) OLDHIS
          WRITE (OUTDAT) OLDHIS
  120 CONTINUE
      DO 125 I=1,NNEW
          WRITE (OUTDAT) ADDHIS(I)
  125 CONTINUE
C
      READ (INDAT,END=82,ERR=82) SOURCE,RA,DEC,RAP,DECP,FLUX
      WRITE (OUTDAT) SOURCE,RA,DEC,RAP,DECP,FLUX
C
      DO 130 IS=1,NS
          READ (INDAT,END=82,ERR=82) STNAME(IS),STX(IS),STY(IS),STZ(IS)
          WRITE(OUTDAT) STNAME(IS),STX(IS),STY(IS),STZ(IS),
     1            (0.D0,J=5,10),0
  130 CONTINUE
C
      READ (INDAT,END=82,ERR=82) IY, IZ, GASTM, FREQ, TINTG,
     1            AVG, BW, (IDUM(I),I=1,8)
      WRITE (OUTDAT) IY, IZ, GASTM, FREQ, TINTG,
     1            AVG, BW, (IDUM(I),I=1,8)
C
C Baseline parameters (BXY and BZ are components in wavelengths; 
C BOFF is "hour angle").
C
      WAVE   = VELC/FREQ
      COSDEC = DCOS(DECP)
      SINDEC = DSIN(DECP)
      DO 140 IB=1,NB
          IS1 = NB1(IB)
          IS2 = NB2(IB)
          BX  = (STX(IS1)-STX(IS2))/WAVE
          BY  = (STY(IS1)-STY(IS2))/WAVE
          BZ(IB) = (STZ(IS1)-STZ(IS2))/WAVE
          BXY(IB) = DSQRT(BX*BX+BY*BY)
          IF (BXY(IB).EQ.0D0) THEN
            CALL PUTOUT('Station coordinates missing.')
            GOTO 82
          END IF
          BOFF(IB) = GASTM-RAP-PI/2.0-DATAN2(BY,BX)
  140 CONTINUE
C
C Read and re-write data records.
C
  300 READ(INDAT,END=400) IUT,(INREC(J),J=1,NDWPB*NB+2*NT)
      IF (IUT.LT.0) GOTO 400
C#----------------------------------------------------------------------
C# Example code: the following adds 1 day to all the
C# UT times recorded in the data records.
C#
C#	IUT = IUT + 24*60*60*60		! add one day
C#----------------------------------------------------------------------
C#----------------------------------------------------------------------
C# Example code: the following loop computes (u,v,w)
C# for each data point.
C#
C#	DO IB=1,NB
C#	    TIHA = (IUT-IZ)*SIDR + BOFF(IB)
C#	    U = BXY(IB)*DCOS(TIHA)
C#	    V = BZ(IB)*COSDEC + BXY(IB)*SINDEC*DSIN(TIHA)
C#	    W = BZ(IB)*SINDEC - BXY(IB)*COSDEC*DSIN(TIHA)
C#	END DO
C#----------------------------------------------------------------------
      WRITE (OUTDAT) IUT,(INREC(J),J=1,NDWPB*NB+2*NT)
      GOTO 300
C
C Write final record and quit.
C
  400 WRITE (OUTDAT) -1,(0.0,J=1,NDWPB*NB+2*NT)
      CLOSE (UNIT=INDAT)
      CLOSE (UNIT=OUTDAT)
      STOP
C
C Error exit. Delete the nascent output file.
C
   82 CLOSE (UNIT=OUTDAT, STATUS='DELETE')
      CLOSE (UNIT=INDAT)
      CALL ERROR('Something wrong with input MERGE file.')
      STOP
C-----------------------------------------------------------------------
      END
