      PROGRAM MRGSUB
C-----------------------------------------------------------------------
C MRGSUB: to subtract a model from a MERGE-format file.
C
C Parameters:
C      INPUT = input file name
C      OUTPUT = output file name
C      MODFILE = model file name
C      HISTORY = text to be written in header
C
C T.J. Pearson, standard Fortran 77.
C
C Version 1.0:  1989 Mar 20.
C Version 1.1:  1990 Nov 13: remove unused variables.
C Version 2.0:  1991 Jul 17: vector subtraction.
C Version 2.1:  1993 Apr  1: accept all types of model component;
C                            compute new phase errors.
C-----------------------------------------------------------------------
C
C Constants.
C
      CHARACTER*(*) PROGNM, VERSN
      INTEGER INC,OUTC,INDAT,OUTDAT,INMOD,MAXS,MAXB,MAXT,NPARS
      DOUBLE PRECISION PI,RPDEG,SIDR,VELC
C     -- Program version and date:
      PARAMETER (PROGNM='MRGSUB')
      PARAMETER (VERSN='2.1 - 1993 Apr 1')
C     -- Fortran unit numbers used:
      PARAMETER (INC=5,OUTC=6,INDAT=8,OUTDAT=9,INMOD=10)
C     -- Maximum numbers of stations, baselines, and triangles:
      PARAMETER (MAXS=40)
      PARAMETER (MAXB=(MAXS-1)*MAXS/2)
      PARAMETER (MAXT=MAXB*(MAXS-2)/3)
C     -- Number of input parameters:
      PARAMETER (NPARS=42)
C     -- Pi:
      PARAMETER (PI=3.141592653589793D0)
C     -- Radians per degree
      PARAMETER (RPDEG=PI/180D0)
C     -- Radians of ST per sec of UT:
      PARAMETER (SIDR=(PI/2592000.0D0)*1.0027379093D0)
C     -- Speed of light:
      PARAMETER (VELC=2.997925D8)
C-----------------------------------------------------------------------
C Variables.
C
      CHARACTER*8  STNAME(MAXS)
      CHARACTER*16 SOURCE, OPCODE, CODES(2)
      INTEGER   IUT, IS, IB, IT, NS, NB, NT, I, J, IER, IPOS, MODE
      INTEGER   IZ, IY, IS1, IS2, NHIST
      INTEGER   NNEW, NDWPB
      INTEGER   NBI(MAXB), NB1(MAXB), NB2(MAXB)
      INTEGER   NTI(MAXT)
      INTEGER   REC1(20), IDUM(8)
      LOGICAL   VECTOR
      REAL      INREC(7*MAXB)
      REAL      U, V, PHSERR, AMPERR
      REAL      OAMP, OPHS, OVISI, OVISR
      REAL      MAMP, MPHS, MVISI, MVISR
      REAL      NAMP, NPHS, NVISI, NVISR
      REAL      T
      DOUBLE PRECISION STX(MAXS), STY(MAXS), STZ(MAXS),
     1          RA, DEC, RAP, DECP, GASTM, FREQ, FLUX,
     2          WAVE,BX,BY,BZ(MAXB),BOFF(MAXB),BXY(MAXB),TIHA,
     3          COSDEC,SINDEC, TINTG, AVG, BW,
     4          PARS(NPARS),VALS(NPARS),ENDMRK
      CHARACTER*80 INDSN, OUTDSN, MODDSN, OLDHIS, HISTRY, ADDHIS(4)
C-----------------------------------------------------------------------
C Functions called.
C
      INTEGER   LEN1, VLBOPE
C-----------------------------------------------------------------------
C Names and default values of KEYIN parameters.
C
      DATA  PARS/ 'Input',   9*' ', 'Output',  9*' ',
     1            'Modfile', 9*' ', 'History', 9*' ',
     2            'Opcode',    ' ' /
      DATA  VALS/ '***UNSET',9*' ', '***UNSET',9*' ', 
     1            '***UNSET',9*' ', 10*' ',
     2            'VECTOR', ' ' /
      DATA  ENDMRK/ '/' /
      DATA  CODES/ 'VECTOR', 'SCALAR'/
C-----------------------------------------------------------------------
C
C Introduction.
C
      CALL PUTOUT(PROGNM//' subtracts a model from a MERGE file')
      CALL PUTOUT('(Version '//VERSN//')')
C
C Control parameters.
C
      MODE = 0
  100 IF (MODE.NE.0) CALL ERROR('Bad input parameters in '//PROGNM)
      CALL PUTOUT(' ')
      CALL PUTOUT('Parameters (INPUT, OUTPUT, MODFILE, HISTORY):')
      CALL KEYIN(PARS,VALS,NPARS,ENDMRK,MODE,INC,OUTC)
      WRITE (INDSN,'(10A8)') (VALS(I),I=1,10)
      WRITE (OUTDSN,'(10A8)') (VALS(I),I=11,20)
      WRITE (MODDSN,'(10A8)') (VALS(I),I=21,30)
      WRITE (HISTRY,'(10A8)') (VALS(I),I=31,40)
      WRITE (OPCODE,'(2A8)')  (VALS(I),I=41,42)
      CALL KEYFND(OPCODE, CODES, 2, IPOS, IER)
      IF (IPOS.EQ.1) THEN
          VECTOR = .TRUE.
          OPCODE = 'VECTOR'
      ELSE IF (IPOS.EQ.2) THEN
          VECTOR = .FALSE.
          OPCODE = 'SCALAR'
      ELSE
          CALL PUTOUT('Bad OPCODE (should be VECTOR or SCALAR)')
          GOTO 100
      END IF
C
C Open input Merge file.
C
      IER = VLBOPE(INDAT, INDSN, 'MERGE', 'OLD', INDSN)
      IF (IER.NE.1) GOTO 100
C
C Open input Model file and read it.
C
      IER = VLBOPE(INMOD, MODDSN, 'MODEL', 'OLD', MODDSN)
      IF (IER.NE.1) GOTO 100
      CALL GETMOD(INMOD)
      CLOSE (UNIT=INMOD)
C
C Open output Merge file.
C
      IER = VLBOPE(OUTDAT, OUTDSN, 'MERGE', 'NEW', OUTDSN)
      IF (IER.NE.1) THEN
          CLOSE (UNIT=INDAT)
          GOTO 100
      END IF
C
C Make the new history records.
C
      CALL MKHIST(PROGNM, VERSN, ADDHIS(1))
      ADDHIS(2) = '- Model: '//MODDSN
      ADDHIS(3) = '- Opcode: '//OPCODE
      IF (HISTRY.NE.' ') THEN
          NNEW = 4
          ADDHIS(4) = '- '//HISTRY(1:78)
      ELSE
          NNEW = 3
      END IF
      DO 105 I=1,NNEW
          CALL PUTOUT(ADDHIS(I)(1:LEN1(ADDHIS(I))))
  105 CONTINUE
C
C Read and re-write header. Merge format level must be 1.
C
      READ (INDAT,END=82,ERR=82) (REC1(J),J=1,20)
      IF (REC1(1).NE.1) GOTO 82
      NDWPB = 4
      NHIST = REC1(2)
      NS    = REC1(4)
      NB    = REC1(5)
      NT    = REC1(6)
C     -- remove closure phases
      REC1(6) = 0
      IF (NS.LT.2) GOTO 82
      IF (NS.GT.MAXS) GOTO 82
      REC1(2) = REC1(2) + NNEW
      REC1(3) = REC1(3) + NNEW
      WRITE (OUTDAT) (REC1(J),J=1,20)
C
      IF (NB.GT.0) THEN
          READ (INDAT,END=82,ERR=82) (NBI(IB),IB=1,NB)
          WRITE (OUTDAT) (NBI(IB),IB=1,NB)
          DO 110 IB=1,NB
              NB1(IB) = NBI(IB)/100
              NB2(IB) = NBI(IB) -  NB1(IB)*100
  110     CONTINUE
      END IF
C
      IF (NT.GT.0) THEN
          READ (INDAT,END=82,ERR=82) (NTI(IT),IT=1,NT)
      END IF
C
      DO 120 I=1,NHIST
          READ (INDAT,END=82,ERR=82) OLDHIS
          WRITE (OUTDAT) OLDHIS
  120 CONTINUE
      DO 125 I=1,NNEW
          WRITE (OUTDAT) ADDHIS(I)
  125 CONTINUE
C
      READ (INDAT,END=82,ERR=82) SOURCE,RA,DEC,RAP,DECP,FLUX
      WRITE (OUTDAT) SOURCE,RA,DEC,RAP,DECP,FLUX
C
      DO 130 IS=1,NS
          READ (INDAT,END=82,ERR=82) STNAME(IS),STX(IS),STY(IS),STZ(IS)
          WRITE(OUTDAT) STNAME(IS),STX(IS),STY(IS),STZ(IS),
     1            (0.D0,J=5,10),0
  130 CONTINUE
C
      READ (INDAT,END=82,ERR=82) IY, IZ, GASTM, FREQ, TINTG,
     1            AVG, BW, (IDUM(I),I=1,8)
      WRITE (OUTDAT) IY, IZ, GASTM, FREQ, TINTG,
     1            AVG, BW, (IDUM(I),I=1,8)
C
C Baseline parameters (BXY and BZ are components in wavelengths; 
C BOFF is "hour angle").
C
      WAVE   = VELC/FREQ
      COSDEC = COS(DECP)
      SINDEC = SIN(DECP)
      DO 140 IB=1,NB
          IS1 = NB1(IB)
          IS2 = NB2(IB)
          BX  = (STX(IS1)-STX(IS2))/WAVE
          BY  = (STY(IS1)-STY(IS2))/WAVE
          BZ(IB) = (STZ(IS1)-STZ(IS2))/WAVE
          BXY(IB) = SQRT(BX*BX+BY*BY)
          IF (BXY(IB).EQ.0D0) THEN
            CALL PUTOUT('Station coordinates missing.')
            GOTO 82
          END IF
          BOFF(IB) = GASTM-RAP-PI/2.0-ATAN2(BY,BX)
  140 CONTINUE
C
C Read and re-write data records.
C
  300 READ(INDAT,END=400) IUT,(INREC(J),J=1,NDWPB*NB)
      IF (IUT.LT.0) GOTO 400
      DO 320 IB=1,NB
C         -- compute u,v
          TIHA = (IUT-IZ)*SIDR + BOFF(IB)
          U = BXY(IB)*COS(TIHA)
          V = BZ(IB)*COSDEC + BXY(IB)*SINDEC*SIN(TIHA)
C         -- compute model
          CALL UVMODX(U,V,MVISR,MVISI)
          OAMP  = INREC(1 + NDWPB*(IB-1))
          OPHS  = INREC(3 + NDWPB*(IB-1))*RPDEG
          AMPERR = INREC(2 + NDWPB*(IB-1))
          PHSERR = INREC(4 + NDWPB*(IB-1))
C         -- vector subtraction
          IF (VECTOR) THEN
              OVISR = OAMP*COS(OPHS)
              OVISI = OAMP*SIN(OPHS)
              NVISR = OVISR - MVISR
              NVISI = OVISI - MVISI
              NAMP  = SQRT(NVISR*NVISR + NVISI*NVISI)
              NPHS  = ATAN2(NVISI,NVISR)
              PHSERR =  180.0
              IF (NAMP.GT.0.0) PHSERR = (AMPERR/NAMP)/RPDEG
C         -- scalar subtraction
          ELSE
              MAMP = SQRT(MVISI*MVISI + MVISR*MVISR)
              IF (MAMP.EQ.0.0) THEN
                  MPHS = 0.0
              ELSE
                  MPHS = ATAN2(MVISI,MVISR)
              END IF
              NAMP = OAMP - MAMP
              NPHS = OPHS - MPHS
          END IF
          INREC(1 + NDWPB*(IB-1)) = NAMP
          T = NPHS/RPDEG
          IF (T.LE.-180.0) THEN
              T = T + 360.0
          ELSE IF (T.GT.180.0) THEN
              T = T - 360.0
          END IF
          INREC(3 + NDWPB*(IB-1)) = T
          INREC(4 + NDWPB*(IB-1)) = PHSERR
  320 CONTINUE
      WRITE (OUTDAT) IUT,(INREC(J),J=1,NDWPB*NB)
      GOTO 300
C
C Write final record and quit.
C
  400 WRITE (OUTDAT) -1,(0.0,J=1,NDWPB*NB)
      CLOSE (UNIT=INDAT)
      CLOSE (UNIT=OUTDAT)
      STOP
C
C Error exit. Delete the nascent output file.
C
   82 CLOSE (UNIT=OUTDAT, STATUS='DELETE')
      CLOSE (UNIT=INDAT)
      CALL ERROR('Something wrong with input MERGE file.')
      STOP
C-----------------------------------------------------------------------
      END

C*GETMOD -- read model file
C+
      SUBROUTINE GETMOD(INMOD)
      INTEGER INMOD
C
C INMOD: (in) : Fortran unit for input file
C NMDCMP (in) : maximum number of model components to read from file
C NC     (out): actual number of components read
C TFLUX  (out): total flux in NC components
C-----------------------------------------------------------------------
C     -- Max. no. of model components
      INTEGER MXMOD
      PARAMETER (MXMOD=5000) 
C
      REAL PI, RPDEG, RPMAS
      PARAMETER (PI=3.1415927)
      PARAMETER (RPDEG=PI/180.)
      PARAMETER (RPMAS=PI/180./3600./1000.)
C
      INTEGER NMDCMP
      INTEGER NC
      REAL TFLUX
C
      INTEGER I, ITYPE
      REAL FLXSUM, RAD, THETA, PHI
      CHARACTER*80 TEXT
      INTEGER LEN1
C
C Model parameters:
C
      REAL
     1  FLUX(MXMOD), XC(MXMOD), YC(MXMOD), AA(MXMOD),
     2  RATIO(MXMOD), CPHI(MXMOD), SPHI(MXMOD)
      INTEGER
     1   TYPE(MXMOD), NCOMP
      LOGICAL
     1   ALLDEL
      COMMON /MODEL/ FLUX, XC, YC, AA, RATIO, CPHI, SPHI,
     1               TYPE, NCOMP, ALLDEL
C-----------------------------------------------------------------------
C
      NMDCMP = MXMOD
      FLXSUM = 0.0
      ALLDEL = .TRUE.
      DO 150 I=1,NMDCMP
          CALL GTMODL(INMOD, ITYPE, FLUX(I), RAD, THETA,
     1                AA(I), RATIO(I), PHI, *155, 0)
          IF (FLUX(I) .EQ. 0.0) GOTO 155
          FLXSUM = FLXSUM + FLUX(I)
          THETA  = THETA * RPDEG
          XC(I)  = RPMAS*RAD*SIN(THETA)
          YC(I)  = RPMAS*RAD*COS(THETA)
          AA(I)  = AA(I)*RPMAS
          PHI    = PHI*RPDEG
          CPHI(I)= COS(PHI)
          SPHI(I)= SIN(PHI)
          IF (ITYPE .EQ. 0) ITYPE = 1
          IF (ITYPE .LT. 1 .OR. ITYPE .GT. 5) THEN
              WRITE (TEXT,660) ITYPE
              CALL PUTOUT(TEXT(1:LEN1(TEXT)))
              ITYPE = 1
          END IF
          IF (AA(I).EQ. 0.0) ITYPE = 0
          TYPE(I) = ITYPE
          IF (ITYPE.NE.0) ALLDEL = .FALSE.
  150 CONTINUE
      WRITE (TEXT,650) NMDCMP
      CALL PUTOUT(TEXT(1:LEN1(TEXT)))
      I = NMDCMP + 1
  155 NCOMP = I-1
      IF (FLXSUM .EQ. 0.0) CALL PUTOUT('Model has zero total flux')
      NC = NCOMP
      TFLUX = FLXSUM
      RETURN
C
  650 FORMAT ('++WARNING++ Model truncated after ',I5,' components.')
  660 FORMAT ('++WARNING++ Invalid component type for component ',I4,
     1'; Gaussian used')
C
      END

C*UVMODX -- compute model visibility
C+
      SUBROUTINE UVMODX(U,V,VISR,VISI)
      REAL U, V, VISR, VISI
C
C Calculate model complex visibility (VISR,VISI)
C at point (U,V)
C-----------------------------------------------------------------------
C     -- Max. no. of model components
      INTEGER MXMOD
      PARAMETER (MXMOD=5000) 
      REAL C1, PI, TWOPI
C     -- C1 = 1 / (4 ln 2)
      PARAMETER (C1=0.360673760)
      PARAMETER (PI=3.141592654)
      PARAMETER (TWOPI=2.0*PI)
C
      INTEGER J
      REAL Q, Q2, UROT, VROT
      REAL COMPHA, COMVIS
      REAL BESJ0, BESJ1
      EXTERNAL BESJ0, BESJ1
C
C Model parameters:
C
      REAL
     1  FLUX(MXMOD), XC(MXMOD), YC(MXMOD), AA(MXMOD),
     2  RATIO(MXMOD), CPHI(MXMOD), SPHI(MXMOD)
      INTEGER
     1   TYPE(MXMOD), NCOMP
      LOGICAL
     1   ALLDEL
      COMMON /MODEL/ FLUX, XC, YC, AA, RATIO, CPHI, SPHI,
     1               TYPE, NCOMP, ALLDEL
C-----------------------------------------------------------------------
C
C Compute delta-function model.
C
      IF (ALLDEL) THEN
          VISR = 0.0
          VISI = 0.0
          DO 100 J=1,NCOMP
              COMPHA = TWOPI*(U*XC(J) + V*YC(J))
              VISR   = VISR + FLUX(J)*COS(COMPHA)
              VISI   = VISI + FLUX(J)*SIN(COMPHA)
  100     CONTINUE
C
C Compute general model.
C
      ELSE
          VISR = 0.0
          VISI = 0.0
          DO 120 J=1,NCOMP
              COMPHA = TWOPI*(U*XC(J) + V*YC(J))
              UROT = (U*CPHI(J) - V*SPHI(J))*RATIO(J)
              VROT =  U*SPHI(J) + V*CPHI(J)
              IF (AA(J) .EQ. 0.0) THEN
C                 -- delta function
                  COMVIS = 1.0
              ELSE IF (TYPE(J) .EQ. 1) THEN
C                 -- gaussian
                  Q2   = (PI*AA(J))**2 * (UROT**2 + VROT**2)
                  COMVIS = EXP(-C1*Q2)
              ELSE IF (TYPE(J) .EQ. 2) THEN
C                 -- uniform disk
                  Q = (PI*AA(J)) * SQRT(UROT**2 + VROT**2)
                  IF (Q.LT.0.01) THEN
                      Q2 = Q*Q
                      COMVIS = 1.0 - Q2/8.0 + Q2*Q2/192.0
                  ELSE
                      COMVIS = 2.0*BESJ1(Q) / Q
                  END IF
              ELSE IF (TYPE(J) .EQ. 3) THEN
C                 -- optically thin sphere
                  Q = (PI*AA(J)) * SQRT(UROT**2 + VROT**2)
                  IF (Q.LT.0.01) THEN
                      Q2 = Q*Q
                      COMVIS = 1.0 - Q2/10.0 + Q2*Q2/280.0
                  ELSE
                      COMVIS = 3.0*(SIN(Q) - Q*COS(Q))/(Q*Q*Q)
                  END IF
              ELSE IF (TYPE(J) .EQ. 4) THEN
C                 -- ring
                  Q = (PI*AA(J)) * SQRT(UROT**2 + VROT**2)
                  COMVIS = BESJ0(Q)
              ELSE
                  CALL ERROR ('Illegal component type in model')
              ENDIF
              VISR  = VISR + FLUX(J)*COMVIS*COS(COMPHA)
              VISI  = VISI + FLUX(J)*COMVIS*SIN(COMPHA)
  120     CONTINUE
      END IF
      END
