* Subroutines used by MERGE and MK3MERGE
*
* MRG_FFIND
* MRG_HISTORY
* MRG_LOC
* MRG_OUTPUT
* MRG_PARAM

      SUBROUTINE MRG_FFIND(NF,FILES,MAXFIL,INC,OUTC)
C-----------------------------------------------------------------------
C VLBMERGE: Obtain list of input file names.
C
C Author:               T.J. Pearson (TJP)
C Language:             VAX-11 Fortran
C
C Modifications:
C
C Arguments:
C-----------------------------------------------------------------------
      INTEGER NF,MAXFIL,INC,OUTC, GETIN, IER, L, NFOUND
      CHARACTER*(*) FILES(MAXFIL)
      CHARACTER*128 INPUT
C
      NF = 0
   20 IER = GETIN(INPUT, 'Input file name: ', L)
      IF (IER.NE.1) GOTO 40
      IF (INPUT.EQ.' ') GOTO 40
      CALL FFILES(INPUT, MAXFIL-NF, FILES(NF+1), NFOUND)
      NF = NF+NFOUND
      IF (NF.GE.MAXFIL) GOTO 30
      GOTO 20
   30 WRITE (OUTC,110) MAXFIL
   40 RETURN
C
  110 FORMAT(' ++WARNING++ Input file list truncated at ',I3,' files')
      END

      SUBROUTINE MRG_HISTORY (TEXT, PR)
C-----------------------------------------------------------------------
C MERGE: Save a history record for the output dataset; send a copy to
C the log file if requested.
C
C Author:            T.J. Pearson (TJP)
C Language:          VAX-11 Fortran
C
C Modifications:
C   1984 Nov 12 - new subroutine (TJP).
C   1986 Feb 12 - split lines longer than 80 characters (TJP).
C
C Arguments:
C   TEXT (input, character):  text of history record.
C   PR (input, integer):  Fortran unit of log file; if 0, no log
C            output will be generated.
C-----------------------------------------------------------------------
      INCLUDE 'merge62.inc'
      INTEGER PR, LEN1, L, L1, L2
      CHARACTER*(*) TEXT
C-----------------------------------------------------------------------
      L = LEN1(TEXT)
      L1 = 1
      L2 = 80
   10 L2 = MIN(L2,L)
      NHIS = NHIS + 1
      IF (NHIS.LE.MAXHIS) HIS(NHIS) = TEXT(L1:L2)
      IF (PR.GT.0) WRITE (PR,'(1X,A)') TEXT(L1:L2)
      IF (L2.LT.L) THEN
          L1 = L2+1
          L2 = L1+80
          GOTO 10
      END IF
      END

      INTEGER FUNCTION MRG_LOC(UT)
C-----------------------------------------------------------------------
C VLBMERGE: Given 'UT', function returns location 'MRG_LOC' at which
C data corresponding to this UT is stored. 'MRG_LOC' is a
C subscript of the storage array 'STORE'. If there is
C no existing entry for 'UT', a new entry is created.
C-----------------------------------------------------------------------
      INCLUDE  'merge64.inc'
      INTEGER HIGH, UT, LOW, MID, I, J
C
C               Special cases: first entry, or new entry at either end
C
      HIGH = 1
      IF (NFR.EQ.0) GOTO 30
      IF (UT.LT.FRINDX(1)) GOTO 30
      HIGH = NFR+1
      IF (UT.GT.FRINDX(NFR)) GOTO 30
C
C               Entry in range: binary search of 'FRINDX'
C
      LOW = 1
      HIGH = NFR
   10 IF (HIGH-LOW.LE.1) GOTO 20
            MID = (HIGH+LOW)/2
            IF (UT.LE.FRINDX(MID)) HIGH = MID
            IF (UT.GE.FRINDX(MID)) LOW = MID
            GOTO 10
   20 IF (UT.EQ.FRINDX(LOW)) HIGH = LOW
      IF (UT.EQ.FRINDX(HIGH)) LOW = HIGH
C
C               Entry already exists: at 'LOW' & 'HIGH'
C
      IF (LOW.EQ.HIGH) GOTO 70
C
C               New entry: between 'LOW' and 'HIGH'
C
   30 IF (NFR.GE.MAXIND) CALL ERROR('Not enough memory: '//
     1    'increase parameter MAXIND in MERGE4.INC and recompile.')
      J = NFR
   40 IF (J.LT.HIGH) GOTO 50
      FRINDX(J+1) = FRINDX(J)
      SPINDX(J+1) = SPINDX(J)
      J = J-1
      GOTO 40
   50 NFR = NFR+1
C
C               Allocate space for new entry
C
      FRINDX(HIGH) = UT
      SPINDX(HIGH) = LAST+1
      IF (LAST+NCELLS.GT.MAXSTR) CALL ERROR('Not enough memory: '//
     1    'increase parameter MAXSTR in MERGE4.INC and recompile.')
      DO 60 I=1,NCELLS
      LAST = LAST+1
   60 STORE(LAST) = 0.0
C
   70 MRG_LOC = SPINDX(HIGH)
      RETURN
      END

      SUBROUTINE MRG_OUTPUT(OUTFIL,OUT,PR)
C-----------------------------------------------------------------------
C MERGE: Generate output file.
C
C Author:               T.J. Pearson (TJP)
C Language:             VAX-11 Fortran
C
C Modifications:
C   1984 Nov 12 - use new INCLUDE file (TJP).
C   1985 May 26 - compute reference date and GST (TJP).
C
C Arguments:
C   OUTFIL (input, character):  name of output file.
C   OUT (input, integer):  Fortran unit number for output; the output
C               file is assumed to be open on this unit, for
C               unformatted output.
C   PR (input, integer):  Fortran unit number for messages, assumed to
C               be open with Fortran carriage control.
C-----------------------------------------------------------------------
      INCLUDE   'merge61.inc'
      INCLUDE   'merge62.inc'
      INCLUDE   'merge63.inc'
      INCLUDE   'merge64.inc'
      REAL*8    PI
      PARAMETER (PI = 3.1415 92653 58979 32384 62643 D0)
C
      CHARACTER OUTFIL*(*),TEXT*40
      INTEGER   I,J,L1,L2,L3,LOC1,LOC2,OUT,PR,DOY,MONTH
      REAL      CLPHS(MAXT),CLERR(MAXT)
      REAL*8    DATE
C-----------------------------------------------------------------------
C
C               Add final records to History.
C
      CALL MRG_HISTORY('- Output file: '//OUTFIL,PR)
      WRITE (TEXT,'(''- '',I6,'' data records'')') NFR
      CALL MRG_HISTORY(TEXT,PR)
      DO I=1,NS
            IF (HX(I)*HY(I)*HZ(I).EQ.0D0)
     1          CALL MRG_HISTORY('++WARNING++ No data for station '//
     2                           NSTATN(I),PR)
      END DO
C
C               Write Header records:
C               ... introductory record:
C
      NHIS = MIN(MAXHIS,NHIS)
      NHEAD = 2+NHIS+NS
      IF (NB.GT.0) NHEAD = NHEAD+1
      IF (NT.GT.0) NHEAD = NHEAD+1
      WRITE (OUT) NVER,NHIS,NHEAD,NS,NB,NT,LENG,KFLAG,(0,I=1,12)
C
C               ... baseline and triangle identifiers:
C
      IF (NB.GT.0) WRITE (OUT) (LABELB(I),I=1,NB)
      IF (NT.GT.0) WRITE (OUT) (LABELT(I),I=1,NT)
C
C               ... History records:
C
      DO I=1,NHIS
            WRITE (OUT) HIS(I)
      END DO
C
C               ... Source description record:
C
      WRITE (OUT) HSRC,HRA50,HDEC50,HRAP,HDECP,HFLUX,(0D0,I=8,10)
C
C               ... Station description records:
C
      DO 20 I=1,NS
            WRITE (OUT) NSTATN(I),HX(I),HY(I),HZ(I),(0D0,J=5,10)
   20 CONTINUE
C
C               ... miscellaneous parameters record; the reference
C               date and GAST are computed from the first data record
C
      DOY = (FRINDX(1)/(24*60*60*60)) + 1
      HUTF = (DOY-1)*(24*60*60*60)
      MONTH = 1
      CALL JULDA(HYEAR,MONTH,DOY,DATE,HGST)
      HGST = PI * HGST/(12*60*60)
      WRITE (OUT) HYEAR,HUTF,HGST,HFREQ,HINTEG,HAVG,HBW,(0D0,I=7,10)
C
C               Data records, no closure phases, or original closure
C               phases
C
      IF (KLOSE.EQ.2 .OR. KLOSE.EQ.3) THEN
            DO 30 I=1,NFR
                LOC1 = SPINDX(I)
                LOC2 = LOC1+NCELLS-1
                WRITE (OUT)FRINDX(I),(STORE(J),J=LOC1,LOC2)
   30       CONTINUE
C
C               Data records, with closure phases
C
      ELSE
            DO 50 I=1,NFR
                LOC1 = SPINDX(I)
                LOC2 = LOC1+NCELLS-1
                DO 40 J=1,NT
                L1 = LOC1 + 4*CONST(J,1) - 2
                L2 = LOC1 + 4*CONST(J,2) - 2
                L3 = LOC1 + 4*CONST(J,3) - 2
                CLPHS(J) = AMOD(STORE(L1)+STORE(L2)-STORE(L3),360.0)
                IF (CLPHS(J).GT.180.0)  CLPHS(J) = CLPHS(J)-360.0
                IF (CLPHS(J).LE.-180.0) CLPHS(J) = CLPHS(J)+360.0
                L1 = L1+1
                L2 = L2+1
                L3 = L3+1
                CLERR(J) = SQRT(STORE(L1)**2+STORE(L2)**2+STORE(L3)**2)
                IF (STORE(L1).LE.0. .OR. STORE(L2).LE.0. .OR.
     1                  STORE(L3).LE.0.0) THEN
                    CLPHS(J) = 0.0
                    CLERR(J) = 0.0
                END IF
   40           CONTINUE
                WRITE (OUT) FRINDX(I),(STORE(J),J=LOC1,LOC2),(CLPHS(J),
     1                  CLERR(J),J=1,NT)
   50       CONTINUE
      END IF
C
C               End record
C
      WRITE (OUT) (-1,J=1,LENG)
C
      RETURN
C-----------------------------------------------------------------------
      END

      SUBROUTINE MRG_PARAM(MODE,INC,PR,VERSN)
C-----------------------------------------------------------------------
C MERGE: Use user's control parameters to set up header.
C
C Author:               T.J. Pearson (TJP)
C Language:             VAX-11 Fortran
C
C Modifications:
C   1984 Oct 30 - add FIXPHS parameter (TJP).
C   1984 Nov 12 - use new INCLUDE file (TJP).
C
C Arguments:
C   MODE
C   INC
C   VERSN
C-----------------------------------------------------------------------
      INCLUDE   'merge61.inc'
      INCLUDE   'merge63.inc'
      INCLUDE   'merge64.inc'
C
      INTEGER   MAXS2,NPARS
      PARAMETER (MAXS2=MAXS-2)
      PARAMETER (NPARS=21+2*MAXS)
C
      REAL*8    PARS(NPARS),VALS(NPARS),ENDMRK
      CHARACTER T*8,VERSN*(*),TEXT*80
      INTEGER*4 PR,I,MODE,INMODE,INC,I1,I2,I3
C
      DATA    PARS/'SOURCE',' ','ALIAS',' ','FLUX','FRINGE??',
     1          'CLOSE',MAXS*'STAT????','TITLE',9*' ',
     2          'CHAN???','FREQ????','BAND????',MAXS*'NEWSTAT?',
     3          'NOFIXPH?'/
      DATA    VALS/2*' ',2*'********',0D0,-1D0,
     1          'CALCULAT',2*'***UNSET',MAXS2*' ',10*' ',
     2          1D0,0D0,10D0,MAXS*' ',-1D0/
      DATA    ENDMRK/'/'/
C-----------------------------------------------------------------------
C
C               Read parameters.
C
      WRITE (PR,440)
      INMODE = MODE
      CALL KEYIN(PARS,VALS,NPARS,ENDMRK,INMODE,INC,PR)
C
C               How many stations?
C
      NS = 0
      DO I=1,MAXS
            WRITE (OSTATN(I),91) VALS(I+7)
            IF (OSTATN(I).EQ.' ') GOTO 20
            CALL UPCASE(OSTATN(I))
            NS = NS+1
      END DO
   20 NB = NS*(NS-1)/2
      NT = NB*(NS-2)/3
C
C               What are the new station names, if any?
C
      DO I=1,NS
            WRITE (NSTATN(I),91) VALS(I+20+MAXS)
            IF (NSTATN(I).EQ.' ') NSTATN(I) = OSTATN(I)
            CALL UPCASE(NSTATN(I))
      END DO
C
C               CLOSE parameter:
C                  "C" : 1 : calculate closure phases
C                  "O" : 2 : original closure phases
C                  "N" : 3 : no closure phases
C
      WRITE (T,'(A8)') VALS(7)
      CALL UPCASE(T)
      IF (T(1:1).EQ.'C') THEN
            KLOSE = 1
      ELSE IF (T(1:1).EQ.'O') THEN
            KLOSE = 2
      ELSE IF (T(1:1).EQ.'N') THEN
            KLOSE = 3
            NT = 0
      ELSE
            CALL ERROR('Illegal value for parameter CLOSE')
      END IF
      LENG = 1+4*NB+2*NT
C
C               History: the first record comes from TITLE,
C               if specified; the next gives MERGE version and date.
C
      NVER = 1
      WRITE (TEXT,91) (VALS(I),I=8+MAXS,17+MAXS)
      IF (TEXT.NE.' ') CALL MRG_HISTORY(TEXT,PR)
      CALL MKHIST('MERGE', VERSN, TEXT)
      CALL MRG_HISTORY(TEXT,PR)
C
C               Save parameter values in common block.
C               Parameters SOURCE, ALIAS, FLUX:
C
      WRITE (HSRC,91) VALS(1),VALS(2)
      WRITE (HALIAS, 91) VALS(3),VALS(4)
      CALL UPCASE(HSRC)
      CALL UPCASE(HALIAS)
      HFLUX = VALS(5)
C
C               Parameters BANDWIDTH, FREQUENCY, CHANNEL:
C
      TBAND  = VALS(20+MAXS)*1D6
      TFREQ  = VALS(19+MAXS)*1D6
      NCHANL = VALS(18+MAXS)
C
C               Parameters FRINGERATES, FIXPHS:
C
      GETFR = (VALS(6).NE.-1D0)
      KFLAG = 0
      IF (GETFR) KFLAG = 2
      FIXPHS = VALS(21+2*MAXS).EQ.-1D0
C
C               Initialize storage for subr. MRG_LOC
C
      NFR = 0
      LAST = 0
      NCELLS = 4*NB
      IF (KLOSE.EQ.2) NCELLS = NCELLS + 2*NT
C
C               Initialize array configuration: baseline and traingle
C               labels LABELB, LABELT, and CONST
C               (it is assumed that the array is 'complete': all
C               possible baselines are required - this could be changed)
C
      I = 0
      DO I2=2,NS
            DO I1=1,I2-1
                I = I+1
                LABELB(I) = 100*I1 + I2
            END DO
      END DO
C
      I = 0
      DO I3=3,NS
            DO I2=2,I3-1
                DO I1=1,I2-1
                    I = I+1
                    LABELT(I) = 10000*I1 + 100*I2 + I3
                    CONST(I,1) = I1+(I2-1)*(I2-2)/2
                    CONST(I,2) = I2+(I3-1)*(I3-2)/2
                    CONST(I,3) = I1+(I3-1)*(I3-2)/2
                END DO
            END DO
      END DO
C
      RETURN
C
C               Formats
C
   91 FORMAT(10A8)
  440 FORMAT(' Control parameters (SOURCE, STATIONS, TITLE, etc):')
      END
