      PROGRAM MTANAL
*-----------------------------------------------------------------------
* MTANAL - Magnetic Tape Dump
*
* T.J. Pearson 23-Dec-1982  (Caltech/JPL), after
* D. Pearce     9-Sep-1981  (Starlink - Chilton)
* Updated       8-Dec-1984 to recognize some file types (TJP).
* Updated      17-Nov-1987 to recognize blocked FITS files (TJP).
* Updated      23-Apr-1988 for Convex compatibility; uses MTPCKG (TJP).
*-----------------------------------------------------------------------
      CHARACTER UNIT*64,ANSWER*128,FILE*128
      LOGICAL   MTSEOF,EBCDIC,EOF
      INTEGER   LPR,BUFSIZ,N,NFILE,NFSKIP,NFANAL, IER,
     1          CHAN, STATUS, BLKLEN, MINBLK,MAXBLK, NERROR, L,
     2          NBLOCK, NEOF, NEOFQ, LEN1, NDONE, GETIN, VLBOPE
      PARAMETER (LPR=7)
      PARAMETER (BUFSIZ=65534)
      BYTE      BUFFER(BUFSIZ)
      CHARACTER HEADER*19,DESCRP*80,BUFCHR*80
      EQUIVALENCE (BUFFER,BUFCHR)
*-----------------------------------------------------------------------
 1000 FORMAT ('MTANAL - Tape analysis utility',25X,A/)
 2000 FORMAT (A/)
 2500 FORMAT ('Tape translated from EBCDIC to ASCII'//)
 4000 FORMAT (/'Start of file number',I5/
     +        'Block 1  (length',I6,' bytes)  starts:')
 5000 FORMAT ('End of file number',I7/'File contained',I6,
     +        ' blocks, shortest block',I6,' bytes, longest block',I6,
     +        ' bytes'/82('-'))
 6000 FORMAT(/'File number',I5,' is empty')
 2001 FORMAT('File ',I10,' contains ',I10,' blocks of length ',I10,
     1       ' bytes; ',I10,' errors')
 2002 FORMAT('File ',I10,' contains ',I10,' blocks, min ',I10,
     1       ', max ',I10,' bytes; ',I10,' errors')
 2003 FORMAT('+++ Parity error in block ',I10,' of file ',I10)
*-----------------------------------------------------------------------
*
* Prompt user for tape unit (no default)
*
   10 IER = GETIN(UNIT,'Tape Unit (device name) = ',N)
      IF (IER.NE.1) STOP
      IF (N.EQ.0) STOP
*
* Open tape stream 
*
      CALL MTOPID(UNIT(1:N), CHAN, STATUS)
      IF (STATUS.EQ.0) CALL MTREWI(CHAN, STATUS)
      IF (STATUS.NE.0) THEN
          CALL PUTOUT('Unable to read tape '//UNIT(1:N))
          GOTO 10
      END IF
*
* Prompt user for any required file skips
*
   20 IER= GETIN(ANSWER,
     1            'Number of files to be skipped [0] = ',N)
      IF (IER.NE.1) STOP
      NFSKIP = 0
      IF (N.NE.0) READ (ANSWER(:N), *, ERR=20) NFSKIP
*
* Prompt user for number of files to be analysed
*
   30 IER = GETIN(ANSWER,
     1            'Number of files to be analysed [32768] = ',N)
      IF (IER.NE.1) STOP
      NFANAL = 32768
      IF (N.NE.0) READ (ANSWER(:N), *, ERR=30) NFANAL
*
* Prompt user for number of file marks at end of tape
*
   35 IER = GETIN(ANSWER,
     1    'Number of consecutive EOFs to terminate scan [2] = ',N)
      IF (IER.NE.1) STOP
      NEOFQ = 2
      IF (N.NE.0) READ (ANSWER(:N), *, ERR=35) NEOFQ
*
* Prompt user for output file name
*
    5 IER =GETIN(ANSWER,
     1            'File name for listing [MTANAL.LIS] = ',N)
      IF (IER.NE.1) STOP
      FILE = 'MTANAL.LIS'
      IF (N.NE.0) FILE = ANSWER(:N)
*
* Open LPR, get time and date and print header
*
      IER = VLBOPE(LPR, FILE, 'TEXT', 'NEW', FILE)
      IF (IER.NE.1) GOTO 5
      CALL DATTIM(HEADER)
      WRITE (LPR,1000) HEADER
*
* Prompt user for any title
*
      IER = GETIN(ANSWER,'Title [optional] = ',N)
      IF (IER.NE.1) STOP
      IF (N.NE.0) WRITE (LPR,2000) ANSWER(:N)
*
* Prompt user for EBCDIC translation option
*
      IER = GETIN(ANSWER,'Translate from EBCDIC [no]? ',N)
      IF (IER.NE.1) STOP
      EBCDIC = (ANSWER(1:1).EQ.'Y'.OR.ANSWER(1:1).EQ.'y')
      IF (EBCDIC) WRITE (LPR,2500)
*
* Skip any leading files if requested
*
      CALL MTSKFI(CHAN,NFSKIP,NDONE,STATUS)
      CALL ERRCHK(STATUS,'MTSKFI','Error skipping input tape')
*
* Initialise file counter
*
      NFILE = NFSKIP+1
      NEOF = 0
*
* Scan tape for number of files requested or until error,
* or until an empty file is encountered
*
      DO WHILE (NFANAL.GT.0)
*
* Read physical block and translate if necessary
*
          CALL MTREAD(CHAN,BUFSIZ,BUFFER,BLKLEN,STATUS)
          IF (MTSEOF(STATUS)) THEN
            WRITE (LPR,6000) NFILE
            NEOF = NEOF + 1
            IF (NEOF.GE.NEOFQ) STOP
          ELSE
            NEOF = 1
          END IF
          IF (EBCDIC) CALL EBCASC(BUFFER,BLKLEN)
*
* Display first block (if possible)
*
          WRITE (LPR,4000) NFILE,BLKLEN
          DESCRP = ' '
          IF (BLKLEN.EQ.80) THEN
            IF (BUFCHR(1:4).EQ.'VOL1') THEN
                DESCRP = ' - VOL label: '//BUFCHR(5:10)
            ELSE IF (BUFCHR(1:4).EQ.'HDR1') THEN
                DESCRP = ' - HDR label: '//BUFCHR(5:21)
            ELSE IF (BUFCHR(1:4).EQ.'EOF1') THEN
                DESCRP = ' - EOF label: '//BUFCHR(5:21)
            END IF
          ELSE IF (MOD(BLKLEN,2880).EQ.0) THEN
            IF (BUFCHR(1:6).EQ.'SIMPLE') THEN
                DESCRP = ' - FITS file'
            END IF
          END IF
          L = MIN(BLKLEN,128)
          IF (L.GT.0) CALL HEXDMP(LPR,BUFFER,L)
*
* Initialise block counter and min/max values
*
          NBLOCK = 0
          NERROR = 0
          MINBLK = BLKLEN
          MAXBLK = BLKLEN
*
* Scan through file until EOF (or error)
*
          EOF=.FALSE.
          DO WHILE (.NOT.EOF)
*
* Check for EOF from previous read
*
            IF (MTSEOF(STATUS)) THEN
                WRITE (LPR,5000) NFILE,NBLOCK,MINBLK,MAXBLK
                IF (MINBLK.EQ.MAXBLK) THEN
                    WRITE (ANSWER, 2001) NFILE, NBLOCK, MAXBLK, NERROR
                ELSE
                    WRITE (ANSWER, 2002) NFILE, NBLOCK, MINBLK,
     1                                   MAXBLK, NERROR
                END IF
                CALL SSPACE(ANSWER,L)
                IF (NERROR.EQ.0) L = INDEX(ANSWER, ';')-1
                IF (DESCRP.NE.' ') THEN
                    CALL PUTOUT(ANSWER(:L)//DESCRP(:LEN1(DESCRP)))
                ELSE
                    CALL PUTOUT(ANSWER(:L))
                END IF
                NFILE = NFILE+1
                EOF = .TRUE.
            ELSE IF (STATUS.EQ.1) THEN
                NBLOCK = NBLOCK + 1
                NERROR = NERROR + 1
                WRITE (ANSWER, 2003) NBLOCK, NFILE
                CALL SSPACE(ANSWER,L)
                CALL PUTOUT(ANSWER(:L))
                CALL MTREAD(CHAN,BUFSIZ,BUFFER,BLKLEN,STATUS)
            ELSE
                CALL ERRCHK(STATUS,'MTREAD','Error reading input tape')
                MINBLK = MIN(MINBLK,BLKLEN)
                MAXBLK = MAX(MAXBLK,BLKLEN)
                NBLOCK = NBLOCK+1
                CALL MTREAD(CHAN,BUFSIZ,BUFFER,BLKLEN,STATUS)
            ENDIF
*
          END DO
*
* Decrement file count
*
          NFANAL = NFANAL-1
*
      ENDDO
*
* End of job
*
      END

      SUBROUTINE HEXDMP(LPR,BUFFER,NDUMP)
*-----------------------------------------------------------------------
*      Dump buffer(1:ndump) in hex and ascii
*-----------------------------------------------------------------------
      INTEGER LPR, NDUMP
      BYTE BUFFER(NDUMP)
      CHARACTER*113 OUTPUT
      INTEGER I, I2, J, K, L
*
      DO 500 I = 1,NDUMP,32
          OUTPUT = ' '
          I2 = MIN(NDUMP,I+31)
          L = 1
          DO 400 J=I,I2
            K = J+1-I
            WRITE (OUTPUT(L:L+1),1010) BUFFER(J)
            L = L+2
            IF (MOD(J,4).EQ.0) L = L+1
            OUTPUT(80:80) = '>'
            IF (BUFFER(J).LT.32 .OR. BUFFER(J).GT.126) THEN
                OUTPUT(80+K:80+K) = '.'
            ELSE
                OUTPUT(80+K:80+K) = CHAR(BUFFER(J))
            END IF
  400     CONTINUE
          OUTPUT(82+I2-I:82+I2-I) = '<'
          WRITE (LPR,1000) I,OUTPUT
  500 CONTINUE
      RETURN
*
 1000 FORMAT (I8,':  ',A)
 1010 FORMAT (Z2.2)
      END

      SUBROUTINE ERRCHK(STATUS, ROUTIN, USRMSG)
*-----------------------------------------------------------------------
* TIO_ERRCHK - Check status from tape I/O operation
*-----------------------------------------------------------------------
      CHARACTER*(*) ROUTIN, USRMSG
      INTEGER       STATUS
*
      INTEGER       LEN1
      CHARACTER*76  ERRMSG
*
*
      IF (STATUS.NE.0) THEN
            CALL MTMESS(STATUS,ROUTIN,ERRMSG)
            CALL PUTOUT(USRMSG)
            CALL PUTOUT(ERRMSG(:LEN1(ERRMSG)))
            STOP
      END IF
*
      END
