      PROGRAM MTDUMP
*-----------------------------------------------------------------------
* MTDUMP - Magnetic Tape Dump
*
* T.J. Pearson 23-Dec-1982  (Caltech/JPL), after
* D. Pearce     9-Sep-1981  (Starlink - Chilton)
* Updated       6-Feb-1990 for Convex compatibility; uses MTPCKG (TJP).
*-----------------------------------------------------------------------
      INTEGER   PR, BUFSIZ
      PARAMETER (PR=7)
      PARAMETER (BUFSIZ=65534)
*
      CHARACTER UNIT*64, ANSWER*128, FILE*128
      LOGICAL   MTSEOF, EBCDIC
      BYTE      BUFFER(BUFSIZ)
      CHARACTER HEADER*19
      INTEGER   BLKLEN, CHAN, N, NBDUMP, NBLOCK, NBSKIP, NFILE
      INTEGER   NFSKIP, STATUS, GETIN, VLBOPE, IER, NDONE
*-----------------------------------------------------------------------
 1000 FORMAT (5X,'MTDUMP - Tape dumping utility',25X,A/)
 2000 FORMAT (5X,A/)
 2500 FORMAT (5X,'Tape translated from EBCDIC to ASCII')
 5000 FORMAT (/5X,25('*')/5X,'* FILE MARK ENCOUNTERED *'/
     +        5X,25('*')/)
 6000 FORMAT (/5X,'File number = ',I5,'   Block number =',I5,
     +        '   Block length =',I5,' bytes')
*-----------------------------------------------------------------------
* Prompt user for tape unit (no default)
*
   10 IER = GETIN(UNIT,'Tape Unit (device name) = ',N)
      IF (IER.NE.1) STOP
      IF (N.EQ.0) STOP
*
* Open tape stream
*
      CALL MTOPID(UNIT(1:N), CHAN, STATUS)
      IF (STATUS.EQ.0) CALL MTREWI(CHAN, STATUS)
      IF (STATUS.NE.0) THEN
          CALL PUTOUT('Unable to read tape '//UNIT(1:N))
          GOTO 10
      END IF
*
* Prompt user for any required file skips
*
   20 IER = GETIN(ANSWER,
     1           'Number of files to be skipped [0] = ',N)
      IF (IER.NE.1) STOP
      NFSKIP = 0
      IF (N.NE.0) READ (ANSWER(:N), *, ERR=20) NFSKIP
*
* Prompt user for any required block skips
*
   30 IER = GETIN(ANSWER,
     1           'Number of blocks to be skipped [0] = ',N)
      IF (IER.NE.1) STOP
      NBSKIP = 0
      IF (N.NE.0) READ (ANSWER(:N), *, ERR=30) NBSKIP
*
* Prompt user for number of blocks to be dumped
*
   40 IER = GETIN(ANSWER,
     1           'Number of blocks to be dumped [32768] = ',N)
      IF (IER.NE.1) STOP
      NBDUMP = 32768
      IF (N.NE.0) READ (ANSWER(:N), *, ERR=40) NBDUMP
*
* Prompt user for output file name
*
    5 IER = GETIN(ANSWER,
     1           'File name for listing [MTDUMP.LIS] = ',N)
      IF (IER.NE.1) STOP
      FILE = 'MTDUMP.LIS'
      IF (N.NE.0) FILE = ANSWER(:N)
*
* Open PR, get time and date and print header
*
      IER = VLBOPE(PR, FILE, 'TEXT', 'NEW', FILE)
      IF (IER.NE.1) GOTO 5
      CALL DATTIM(HEADER)
      WRITE (PR,1000) HEADER
*
* Prompt user for any title
*
      IER = GETIN(ANSWER,'Title [optional] = ',N)
      IF (IER.NE.1) STOP
      IF (N.NE.0) WRITE (PR,2000) ANSWER(:N)
*
* Prompt user for EBCDIC translation option
*
      IER = GETIN(ANSWER,'Translate from EBCDIC [no]? ',N)
      IF (IER.NE.1) STOP
      EBCDIC = (ANSWER(1:1).EQ.'Y'.OR.ANSWER(1:1).EQ.'y')
      IF (EBCDIC) WRITE (PR,2500)
*
* Skip any leading files if requested
*
      CALL MTSKFI(CHAN,NFSKIP,NDONE,STATUS)
      CALL ERRCHK(STATUS,'MTFSKI','Error skipping input tape')
*
* Initialise block counter and file counter
*
      NBLOCK = 0
      NFILE = NFSKIP+1
*
* Skip any leading blocks if requested
*
      DO WHILE (NBSKIP.GT.0)
          CALL MTREAD(CHAN,BUFSIZ,BUFFER,BLKLEN,STATUS)
          IF (MTSEOF(STATUS)) THEN
              NFILE = NFILE+1
              NBLOCK = 0
          ELSE
              CALL ERRCHK(STATUS,'MTREAD','Error skipping input tape')
              NBSKIP = NBSKIP-1
              NBLOCK = NBLOCK+1
          ENDIF
      ENDDO
*
* Scan tape for number of blocks requested or until error
*
      DO WHILE (NBDUMP.GT.0)
*
* Read physical block
*
          CALL MTREAD(CHAN,BUFSIZ,BUFFER,BLKLEN,STATUS)
*
* Test for End-Of-File status
*
          IF (MTSEOF(STATUS)) THEN
              WRITE (PR,5000)
              NFILE = NFILE+1
              NBLOCK = 0
*
* ...else increment block counter and print buffer
*
          ELSE
              CALL ERRCHK(STATUS,'MTREAD','Error reading input tape')
              NBLOCK = NBLOCK+1
              WRITE (PR,6000) NFILE,NBLOCK,BLKLEN
              IF (EBCDIC) CALL EBCASC(BUFFER,BLKLEN)
              CALL HEXDMP(PR,BUFFER,BLKLEN)
              NBDUMP = NBDUMP-1
          ENDIF
      ENDDO
*
* End of job
*
      END

      SUBROUTINE HEXDMP(LPR,BUFFER,NDUMP)
*-----------------------------------------------------------------------
*      Dump buffer(1:ndump) in hex and ascii
*-----------------------------------------------------------------------
      INTEGER LPR, NDUMP
      BYTE BUFFER(NDUMP)
      CHARACTER*113 OUTPUT
      INTEGER I, I2, J, K, L
*
      DO 500 I = 1,NDUMP,32
          OUTPUT = ' '
          I2 = MIN(NDUMP,I+31)
          L = 1
          DO 400 J=I,I2
            K = J+1-I
            WRITE (OUTPUT(L:L+1),1010) BUFFER(J)
            L = L+2
            IF (MOD(J,4).EQ.0) L = L+1
            OUTPUT(80:80) = '>'
            IF (BUFFER(J).LT.32 .OR. BUFFER(J).GT.126) THEN
                OUTPUT(80+K:80+K) = '.'
            ELSE
                OUTPUT(80+K:80+K) = CHAR(BUFFER(J))
            END IF
  400     CONTINUE
          OUTPUT(82+I2-I:82+I2-I) = '<'
          WRITE (LPR,1000) I,OUTPUT
  500 CONTINUE
      RETURN
*
 1000 FORMAT (I8,':  ',A)
 1010 FORMAT (Z2.2)
      END

      SUBROUTINE ERRCHK(STATUS, ROUTIN, USRMSG)
*-----------------------------------------------------------------------
* TIO_ERRCHK - Check status from tape I/O operation
*-----------------------------------------------------------------------
      CHARACTER*(*) ROUTIN, USRMSG
      INTEGER       STATUS
*
      INTEGER       LEN1
      CHARACTER*76  ERRMSG
*
*
      IF (STATUS.NE.0) THEN
            CALL MTMESS(STATUS,ROUTIN,ERRMSG)
            CALL PUTOUT(USRMSG)
            CALL PUTOUT(ERRMSG(:LEN1(ERRMSG)))
            STOP
      END IF
*
      END
