      PROGRAM INVERT
C-----------------------------------------------------------------------
C INVERT: main program. This routine determines the array sizes,
C allocates the arrays dynamically (VAX specific) and calls subroutine
C INVER1 to do the work.
C 
C Original author:            T.J. Pearson
C 
C Version 7.0: 1982 Jan 25
C Version 8.0: 1983 Mar 15 - add PLOTFILE parameter, and change format
C                            of gray-scale plots (TJP).
C Version 9.0: 1983 Jul 28 - extend to read MERGE version 2 files (DLM).
C Version 9.1: 1983 Aug  4 - add W to MERGE version 2 (DLM).
C Version 9.2: 1983 Aug 15 - sort uvdata before weighting (TJP).
C Version 9.3: 1983 Aug 18 - add FAST weighting option; label uvplot
C                            in wavelengths (TJP).
C Version 9.4: 1984 Dec 10 - add noise calculations; make UVWEIGHT='U'
C                            equivalent to UVWEIGHT='F'.
C Version 9.5: 1985 Jun 26 - pass antennas and epoch through to CLEAN.
C Version 9.6: 1986 Apr  4 - pass map center through to CLEAN.
C Version 9.7: 1986 Aug 28 - increase to 20 stations.
C Version 9.8: 1988 Jan 11 - Add SNR weighting (WTPOWER) (RCW);
C                            minor changes for Convex (TJP).
C Version 9.9: 1988 Feb 25 - Add beam size calculation (TJP).
C Version 10.0: 1988 Apr 10 - Convex-compatible version (TJP).
C Version 10.1: 1988 Dec 19 - Minor changes to plotting; issue warning
C                            if oversampling is < 2.5 (TJP).
C Version 10.2: 1989 Jul 13 - Two input files (TJP).
C Version 11.0: 1989 Feb 13 - Write out FITS files (TJP).
C Version 11.1: 1989 Jul 13 - Merge 10.2 and 11.0 (TJP).
C-----------------------------------------------------------------------
      CHARACTER*(*) VERSN
      PARAMETER (VERSN='11.1 - 1989 Jul 13')
      INTEGER   INC, OUTC, LIST, NPARS
      PARAMETER (INC=5, OUTC=6, LIST=1, NPARS=103)
C 
      CHARACTER*255 LISFIL
      INTEGER   I, IDIM, MAXP, MODE, ICENT, IER
      INTEGER   LIB$GET_VM, LEN1
      INTEGER   NBYTES(7),LOC(7)
      INTEGER   VALID(8)
        DATA VALID/16,32,64,128,256,512,1024,2048/
C 
C Input control parameter definitions.
C 
      DOUBLE PRECISION PARS(NPARS),VALS(NPARS),ENDMRK
      DATA  ENDMRK/ '/' /
      DATA  PARS  / '*', 4*' ', 'FLUX', 'PRINTUV',
     1              'PRINTMAP', 'PLOTBeam', 'PLOTMap',
     2              'NOBEAM', 'PRINTBea', 'MAPSize', 'ROTate',
     3              'SIGma','MAPFile',4*' ', 'BEAMFile',4*' ',
     4              'UVWeight','UVRad',2*'UVTaper','UVCut',
     5              'Xyint','ARCSec','MILLIARC','NOMAP','PRINTCF',
     6              20*'CONtours', 2*'SHIft', 3*'SUBsourc',
     7              'PLOTFile', 9*' ','WTpower',
     8              'IDIM', 'MAXP',
     9              'LISTfile', 9*' ',
     A              'INput', 9*' ', 'INput', 9*' '/
      DATA  VALS  / 5*' ',  0D0, 6*-1D0,
     1              0D0, 0D0, 0.7D0,
     2              'MAP',4*' ', 'BEAM',4*' ',
     4              'UNIFORM',1D0,1D0,0D0,1D37,
     5              0D0,2*-1D0,-1D0,-1D0,
     6              20*1D37, 2*0D0, 3*0D0,
     7              'PLOTOUT', 9*' ',0.D0,
     8              256D0, 5000D0,
     9              'invert.l', 'is', 8*' ',
     A              'FROM', 9*' ', 10*' ' /
C-----------------------------------------------------------------------
C Format statements.
  630 FORMAT('Program INVERT -- makes maps from interferometer data'/
     1       '(Version ',A,')'//
     2       'Control parameters:'/)
C 
C Introduction.
C 
      CALL PUTOUT('INVERT makes maps from interferometer data')
      CALL PUTOUT('(Version '//VERSN//')')
      CALL PUTOUT(' ')
C 
C Read the control parameters.
C 
      CALL PUTOUT('Control parameters:')
      MODE = 0
      CALL KEYIN(PARS,VALS,NPARS,ENDMRK,MODE,INC,OUTC)
C 
C Evaluate the parameters used for memory allocation.
C 
      IDIM = VALS(72)
      MAXP = VALS(73)
      DO 10 I=1,8
          IF (IDIM.EQ.VALID(I)) GOTO 20
   10 CONTINUE
      CALL ERROR('IDIM must be a power of 2 between 16 and 2048')
   20 CONTINUE
      ICENT = IDIM/2+1
C 
C Allocate memory for 7 arrays:
C     COMPLEX UVARR(ICENT,IDIM)
C     REAL*4 U(MAXP)
C     REAL*4 V(MAXP)
C     REAL*4 WT(MAXP)
C     COMPLEX VISIB(MAXP)
C     REAL*4 FMULT(IDIM)
C     REAL*4 VISERR(MAXP)
C 
      NBYTES(1) = ICENT*IDIM*8
      NBYTES(2) = MAXP*4
      NBYTES(3) = MAXP*4
      NBYTES(4) = MAXP*4
      NBYTES(5) = MAXP*8
      NBYTES(6) = IDIM*4
      NBYTES(7) = MAXP*4
      DO 30 I=1,7
          IER = LIB$GET_VM(NBYTES(I),LOC(I))
          IF (IER.NE.1) CALL
     1        ERROR('Insufficient virtual memory for dynamic arrays.')
   30 CONTINUE
C 
C Open the listing file.
C 
      WRITE (LISFIL, '(10A8)') (VALS(I),I=74,83)
      OPEN (UNIT=LIST, FILE=LISFIL, STATUs='NEW',
     1      CARRIAGECONTROL='LIST', IOSTAT=IER)
      IF (IER.NE.0) CALL ERROR('Cannot open LISTFILE='//
     1                         LISFIL(1:LEN1(LISFIL)))
      WRITE (LIST, 630) VERSN
      CALL KEYOUT(PARS, VALS, NPARS, LIST)
C 
C Call a subroutine to do the work.
C 
      CALL INVER1(IDIM,ICENT,MAXP,%VAL(LOC(1)),%VAL(LOC(1)),
     1            %VAL(LOC(2)),%VAL(LOC(3)),%VAL(LOC(4)),%VAL(LOC(5)),
     2            %VAL(LOC(6)),%VAL(LOC(7)), VALS, NPARS, LIST, VERSN)
C 
C Close the listing file.
C 
      INQUIRE (UNIT=LIST, NAME=LISFIL)
      CLOSE (UNIT=LIST)
      CALL PUTOUT('INVERT completed; listing in '//
     1            LISFIL(1:LEN1(LISFIL)))
C-----------------------------------------------------------------------
      END
  
      SUBROUTINE INVER1(IDIM,ICENT,MAXP,UVARR,MAPARR,U,V,WT,VISIB,
     1                    FMULT,VISERR, VALS, NPARS, LIST, VERSN)
      INTEGER NPARS
      DOUBLE PRECISION VALS(NPARS)
      INTEGER LIST
      CHARACTER*(*) VERSN
C-----------------------------------------------------------------------
C INVERT - make maps from aperture-synthesis data.
C 
C Author:               T.J. Pearson (TJP), D.L.Meier (DLM)
C Language:             VAX-11 Fortran
C 
C-----------------------------------------------------------------------
      INTEGER    NTERP,ISAMP
      REAL       FLOG2,PI,RPDEG,RPSA,RPMAS
      PARAMETER  (NTERP=301)          ! size of interpolation function
      PARAMETER  (ISAMP=100)          ! oversampling of interp. funct.
      PARAMETER  (FLOG2=0.693147181)  ! ln(2)
      PARAMETER  (PI=3.1415926536)
      PARAMETER  (RPDEG=PI/180.0)
      PARAMETER  (RPSA=PI/648000.0)
      PARAMETER  (RPMAS=RPSA/1000.0)
C 
C               uv/map array
C               NB UVARR and MAPARR are equivalent arrays
C 
      INTEGER IDIM            ! array size for FFT
      INTEGER ICENT           ! position of center = IDIM/2+1
      COMPLEX UVARR(ICENT,IDIM)
      REAL    MAPARR(IDIM,IDIM), MAPMAX,MAPMIN
      INTEGER NDIM(2)
C 
C               Arrays for input visibilities
C 
      INTEGER MAXP            ! maximum number of points
      REAL    U(MAXP),V(MAXP),WT(MAXP),VISERR(MAXP)
      COMPLEX VISIB(MAXP)
C 
C               Other variables
C 
      REAL    FTERP(NTERP),FMULT(IDIM)
      REAL    CPUW,CPUI(2),CPUF(2)
      REAL    TR(6)
      CHARACTER*7 WTPRT
      CHARACTER*8 PASS
      CHARACTER*20 TODAY
      CHARACTER*16 SNAME
      CHARACTER*12 ELAPSE
      DIMENSION VCN(20),VCNPC(20)
      INTEGER NAXISN(4), BITPIX
      INTEGER PGBEGIN
      LOGICAL PRTUV,PRTMAP,PRTBM,PLTMAP,PLTBM,DOFLX,NOBEAM,NOMAP
      LOGICAL BFLG,DOPLOT,PRTCF
      CHARACTER*64  FMAP,FBEAM,STRING
      CHARACTER*128 INPUT1, INPUT2
      CHARACTER*80  FPLOT, MESSAGE
      CHARACTER*40  XYUNIT
      CHARACTER*8   WTTYP
      CHARACTER*32  TELESC
      CHARACTER*8   DATOBS,DATMAP
      CHARACTER*12  OBSERVER
      REAL          EPOCH
      REAL*8        RA0, DEC0
C-----------------------------------------------------------------------
C Set the clock.
C 
      CALL SCLOCK
      CALL DATTIM(TODAY)
      NDIM(1) = IDIM
      NDIM(2) = IDIM
C 
C Decode control parameters.
C 
C File names.
C 
      WRITE (INPUT1,'(10A8)') (VALS(I),I=84,93)
      WRITE (INPUT2,'(10A8)') (VALS(I),I=94,103)
      WRITE (FMAP,610)  (VALS(I),I=16,20)
      WRITE (FBEAM,610) (VALS(I),I=21,25)
      WRITE (FPLOT,610) (VALS(I),I=61,70)
C 
C Printing/plotting switches.
C 
      PRTUV  = VALS(7).NE.-1D0      ! PRINTUV
      PRTMAP = VALS(8).NE.-1D0      ! PRINTMAP
      PRTCF  = VALS(35).NE.-1D0     ! PRINTCF
      PRTBM  = VALS(12).NE.-1D0     ! PRINTBEAM
      PLTMAP = VALS(10).NE.-1D0     ! PLOTMAP
      PLTBM  = VALS(9).NE.-1D0      ! PLOTBEAM
      PMRAD = VALS(10)
      PBRAD = VALS(9)
      ZFLUX  = VALS(6)
      DOFLX = ZFLUX.NE.0.0            ! zero spacing flux supplied
      NOBEAM = VALS(11).NE.-1D0      ! NOBEAM
      NOMAP  = VALS(34).NE.-1D0      ! NOMAP
      IF (NOBEAM) THEN
          PRTBM = .FALSE.
          PLTBM = .FALSE.
          PRTUV = .FALSE.
      END IF
      IF (NOMAP) THEN
          PRTMAP = .FALSE.
          PLTMAP = .FALSE.
      END IF
      DOPLOT = PLTBM.OR.PLTMAP.OR.PRTUV.OR.PRTBM.OR.PRTMAP
      IF (DOPLOT) THEN
          IF (PGBEGIN(0,FPLOT,1,1).NE.1) THEN
            PRTBM = .FALSE.
            PLTBM = .FALSE.
            PRTUV = .FALSE.
            PRTMAP = .FALSE.
            PLTMAP = .FALSE.
            DOPLOT = .FALSE.
          ELSE
            CALL PGQINF('DEV/TYPE',FPLOT,L)
          END IF
      END IF
      ROTATE = VALS(14)            ! rotation
      RSIN = SIN(ROTATE*RPDEG)
      RCOS = COS(ROTATE*RPDEG)
      MAPSIZE = VALS(13)
      IF (MAPSIZE.LE.0) MAPSIZE = IDIM/2        ! default MAPSIZE
      IF (MAPSIZE.GT.IDIM) CALL ERROR('MAPSIZE too large.')
      MAPW = (MAPSIZE-1)/2
      SIGMA = VALS(15)          ! Half FWHM of Gaussian interp.
C                                       ! fn., in units of UVINT
C 
C               XYINT (default milliarcsec, could be arcsec)
C 
      XYINT = VALS(31)
C 
C               Shift by dx,dy (default milliarcsec, could be arcsec)
C 
      DX = VALS(56)
      DY = VALS(57)
C 
C               Set up to use VIS errors in weights.
C 
      WTPOW = VALS(71)
C 
C               Subtract SSUB Jy at XSUB,YSUB
C 
      SSUB = VALS(58)
      XSUB = VALS(59)
      YSUB = VALS(60)
C 
      IF (VALS(32).NE.-1D0) THEN
                XYINT = XYINT*1000.0
                PMRAD = PMRAD*1000.0
                PBRAD = PBRAD*1000.0
                DX = DX*1000.0
                DY = DY*1000.0
                XSUB = XSUB*1000.0
                YSUB = YSUB*1000.0
      END IF
C 
C               UVWEIGHT: value 'N', 'U', 'R' or 'F';
C               UVRAD = radius of search circle for 'U', in UVINTs
C               UVTAPER = val,width: gaussian taper to 'val' at 'width'
C                         wavelengths
C               UVCUT = u,v cutoff in wavelengths
C 
      WRITE (WTTYP,610) VALS(26)
      CALL UPCASE(WTTYP)
      IF (WTTYP(1:1).EQ.'N') THEN
C         -- NATURAL weighting
          NWT = 1
          WTPRT = 'Natural'
      ELSE IF ((WTTYP(1:1).EQ.'U') .OR. (WTTYP(1:1).EQ.'F'))  THEN
C         -- UNIFORM weighting
          NWT = 4
          WTPRT = 'Uniform'
      ELSE IF (WTTYP(1:1).EQ.'R') THEN
C         -- RADIAL weighting
          NWT = 3
          WTPRT = 'Radial '
      ELSE
          CALL ERROR('Invalid value for UVWEIGHT')
      END IF
      UVRAD = VALS(27)
      UVTVAL = VALS(28)
      UVTWID = VALS(29)
      UVCUT  = VALS(30)
C 
C Contours.
C 
      NCONS = 0
      DO 3 ICON=1,20
            IF (VALS(35+ICON).GE.1D30) GOTO 6
            NCONS = NCONS+1
    3       VCNPC(ICON) = VALS(35+ICON)
    6 IF (NCONS.EQ.0) THEN
          NCONS = 15
          DO 200 ICON=1,NCONS
              VCNPC(ICON) = (10*ICON-55)
  200     CONTINUE
      END IF
C 
C-----------------------------------------------------------------------
C 
C               Get u,v and visibility values
C
      NP = 0
      UVMAX = 0.0
      TELESC= ' '
      IF (INPUT1. NE. ' ')
     1    CALL FINDUV(LIST, INPUT1,U,V,VISIB,VISERR,MAXP-1,NP,UVCUT,
     2                UVMAX, SNAME,FREQ,TELESC,DATOBS,EPOCH,RA0,DEC0)
      IF (INPUT2. NE. ' ')
     1    CALL FINDUV(LIST, INPUT2,U,V,VISIB,VISERR,MAXP-1,NP,UVCUT,
     2                UVMAX, SNAME,FREQ,TELESC,DATOBS,EPOCH,RA0,DEC0)
      CALL PUTOUT('INVERT - input file(s) read successfully')
C 
C               Source subtraction
C 
      IF (SSUB.NE.0.0) THEN
          CALL SHIFT(U,V,VISIB,NP,-XSUB*RPMAS,-YSUB*RPMAS)
          DO 90 IP=1,NP
   90     VISIB(IP) = VISIB(IP) - SSUB
          CALL SHIFT(U,V,VISIB,NP,XSUB*RPMAS,YSUB*RPMAS)
          WRITE (LIST,670) SSUB,XSUB,YSUB
      END IF
C 
C               Shift by phase-rotation
C 
      CALL SHIFT(U,V,VISIB,NP,DX*RPMAS,DY*RPMAS)
C 
C               Choose uv interval etc.
C               xy dimensions in milliarcsec, uv in wavelengths
C               Default XYINT -> OVSAMP=2
C 
      IF (XYINT.LE.0D0) XYINT = 1.0/(RPMAS*4.0*UVMAX)
      UVINT = 1.0/(IDIM*XYINT*RPMAS)
      OVSAMP = IDIM*UVINT/UVMAX/2.0
      KLMAX = UVMAX/UVINT+0.55
      KLLO = ICENT-KLMAX-5
      KLHI = ICENT+KLMAX+6
      SIGSQ = FLOG2*(1.0/ISAMP/SIGMA)**2
      WRITE (LIST,620) XYINT,UVINT,WTPRT,WTPOW,OVSAMP,SIGMA,IDIM
      IF (KLLO.LT.1 .OR. KLHI.GT.IDIM) CALL ERROR('XYINT too large.')
      IF (OVSAMP.LT.2.5) THEN
          WRITE (LIST,619)
          CALL PUTOUT('++WARNING++ Oversampling factor too small!')
      END IF
      IF (ROTATE.NE.0.0) WRITE (LIST,621) ROTATE
      IF (DX*DY.NE.0.0)  WRITE (LIST,622) DX,DY
C 
C               Calculate interpolation function (Gaussian)
C 
      DO 30 ITERP=1,NTERP
   30 FTERP(ITERP) = EXP(-SIGSQ*(1-ITERP)**2)
      IF (PRTCF) WRITE (LIST,623) (FTERP(ITERP),ITERP=1,NTERP)
C 
C               Calculate 1/transform of convolving function, FMULT
C 
      FCENT = FTERP(1)/2.0
      DO 50 ITERP=2,NTERP
   50 FCENT = FCENT+FTERP(ITERP)
      PHFACT = 2.0*PI*XYINT*RPMAS*UVINT/ISAMP
      DO 70 IX=1,ICENT-1
      FMULT(IX) = FTERP(1)/2.0
      DO 60 ITERP=2,NTERP
   60 FMULT(IX) = FMULT(IX) + FTERP(ITERP)*
     1                          COS(PHFACT*(IX-ICENT)*(ITERP-1))
   70 FMULT(IX) = FCENT/FMULT(IX)
      FMULT(ICENT) = 1.0
      DO 80 IX=ICENT+1,IDIM
   80 FMULT(IX) = FMULT(2*ICENT-IX)
      IF (PRTCF) WRITE (LIST,624) (FMULT(IX),IX=1,ICENT)
C 
C               Scale the u's and v's by XYINT; rotate if necessary
C               Positive ROTATE should rotate the source
C               anticlockwise
C 
      DO 100 IP=1,NP
      UP = U(IP)*RCOS + V(IP)*RSIN
      VP = -U(IP)*RSIN + V(IP)*RCOS
      U(IP) = UP/UVINT
  100 V(IP) = VP/UVINT
      UVMAX = UVMAX/UVINT
      UVTWID = UVTWID/UVINT
      IF (UVTWID.LE.0.0) UVTWID = UVMAX
C 
C Determine weights.
C 
      CALL RCLOCK(ELAPSE,CPU1)
      IF (NWT.EQ.4) THEN
          CALL FASTWT(LIST,UVRAD,NWT,UVTVAL,UVTWID,UVMAX,U,V,WT,NP,WT0,
     1                MAPARR,IDIM)
      ELSE
          CALL FINDWT(LIST,UVRAD,NWT,UVTVAL,UVTWID,UVMAX,U,V,WT,NP,WT0)
      END IF
C 
C Apply signal-to-noise weighting.
C 
      IF (WTPOW.NE.0.0) THEN
            DO I = 1,NP
                WT(I) = WT(I)/VISERR(I)**WTPOW
            END DO
      END IF
C 
C               Estimate noise level for map.
C 
      SUMERR = 0.0
      DO I=1,NP
            SUMERR = SUMERR + VISERR(I)**2
      END DO
      ESTNOI = 1000.0*SQRT(SUMERR)/FLOAT(NP)
      WRITE (LIST,628) ESTNOI
      SUMERR = 0.0
      SUMWT = 0.0
      DO I=1,NP
            SUMERR = SUMERR + (WT(I)*VISERR(I))**2
            SUMWT = SUMWT + WT(I)
      END DO
      ESTNOI = 1000.0*SQRT(SUMERR)/SUMWT
      WRITE (LIST,627) ESTNOI
C 
C               Estimate beam size.
C 
      CALL UVBEAM(U, V, WT, NP, LIST, UVINT)
C 
      CALL RCLOCK(ELAPSE,CPU2)
      CPUW = CPU2-CPU1
      CALL PUTOUT('INVERT - weights computed')
C 
C               Add in zero-baseline, if requested
C 
      IF (DOFLX) THEN
            NP = NP+1
            VISIB(NP) = CMPLX(ZFLUX,0.0)
            WT(NP) = WT0
      END IF
C 
C               Form dirty map and dirty beam
C               First pass for BEAM, second pass for MAP
C 
      DO 33 IPASS=1,2
      BFLG = IPASS.EQ.1
      IF (BFLG) THEN
                IF (NOBEAM) GOTO 33
                PASS = 'BEAM'
                MSLO = MAX0(ICENT-2*MAPW,1)
                MSHI = MIN0(ICENT+2*MAPW,IDIM)
      ELSE
                IF (NOMAP) GOTO 33
                PASS = 'MAP'
                MSLO = MAX0(ICENT-MAPW,1)
                MSHI = MIN0(ICENT+MAPW,IDIM)
      END IF
      NSLO = MSLO
      NSHI = MSHI
C 
C               Zero visibility array
C 
      DO 40 IV=1,IDIM
      DO 40 IU=1,ICENT
   40       UVARR(IU,IV) = (0.0,0.0)
      WSUM = 0.0
C 
C               Add in visibilities.
C 
      CALL RCLOCK(ELAPSE,CPU1)
      CALL TERPUV(UVARR,ICENT,IDIM,U,V,VISIB,WT,NP,FTERP,NTERP,ISAMP,
     1              BFLG,WSUM)
      CALL RCLOCK(ELAPSE,CPU2)
      CPUI(IPASS) = CPU2-CPU1
      CALL PUTOUT('INVERT - interpolation completed')
C 
C               Gray-scale plot of visibility array.
C 
      IF (PRTUV.AND.BFLG) THEN
            CALL PLOTUV(UVARR,ICENT,KLLO,ICENT,KLLO,KLHI,UVINT)
            CALL PUTOUT('INVERT - visibility array plotted')
      END IF
C 
C               Do the Fourier Transform.
C               FOUR2 is Norman Brenner's routine, here used for
C               transforming conjugate symmetric complex data to real.
C               The sign changes shift the phase center from point
C               (1,1) to (ICENT,ICENT).
C 
      CALL RCLOCK(ELAPSE,CPU1)
      DO 120 IV=1,IDIM,2
      DO 110 IU=1,ICENT,2
  110     UVARR(IU,IV) = -UVARR(IU,IV)
      DO 120 IU=2,ICENT,2
  120     UVARR(IU,IV+1) = -UVARR(IU,IV+1)
      CALL FOUR2(UVARR,NDIM,2,-1,-1)
      DO 140 IY=1,IDIM,2
      DO 130 IX=1,IDIM,2
  130     MAPARR(IX,IY) = -MAPARR(IX,IY)
      DO 140 IX=2,IDIM,2
  140     MAPARR(IX,IY+1) = -MAPARR(IX,IY+1)
      CALL RCLOCK(ELAPSE,CPU2)
      CPUF(IPASS) = CPU2-CPU1
C 
C               Correct map for convolution taper
C               and locate maximum and minimum
C 
      MAPMAX = -1E37
      MAPMIN =  1E37
      DO 155 N=1, IDIM
      DO 155 M=1, IDIM
          MAPARR(M,N) = MAPARR(M,N)*FMULT(M)*FMULT(N)/WSUM/2.0
          IF (MAPARR(M,N).GT.MAPMAX) THEN
                MAPMAX = MAPARR(M,N)
                NMAX = N
                MMAX = M
          END IF
          IF (MAPARR(M,N).LT.MAPMIN) THEN
                MAPMIN = MAPARR(M,N)
                NMIN = N
                MMIN = M
          END IF
  155 CONTINUE
      WRITE (LIST,660) PASS,MSLO,MSHI,NSLO,NSHI,ICENT,ICENT,WSUM,
     1                  MAPMAX,MMAX,NMAX, MAPMIN,MMIN,NMIN
      CALL PUTOUT('INVERT - FFT completed')
C 
C               Write Map and Beam to their
C               respective output files FMAP and FBEAM.
C 
      IF (.NOT.BFLG) THEN
          CALL FITS_INIT(FMAP,FMAP,JUNK,IER)
          IF (IER.NE.0) CALL ERROR('Can''t open file MAPFILE: '//FMAP)
      ELSE
          CALL FITS_INIT(FBEAM,FBEAM,JUNK,IER)
          IF (IER.NE.0) CALL ERROR('Can''t open file BEAMFILE: '//FBEAM)
      END IF
C
C Write clean map to disk in FITS format.
C
      BITPIX = 32
      NAXIS = 4
      NAXISN(1) = IDIM
      NAXISN(2) = IDIM
      NAXISN(3) = 1
      NAXISN(4) = 1
      BZERO = 0.0
      RMAX = 32000.
      IF (BITPIX.EQ.32) RMAX = 2E9
      BSCALE = MAX(ABS(MAPMAX),ABS(MAPMIN))/RMAX
      CALL USERNM(OBSERVER)
      CALL IDATE(IMON,ID,IY)
      WRITE (DATMAP,1000) ID,IMON,IY
 1000 FORMAT(I2.2,'/',I2.2,'/',I2.2)
      CALL FITS_HSTAN(BITPIX,NAXIS,NAXISN,0D0,0D0,IER)
C?    CALL FITS_WLOG ('EXTEND',.TRUE.,
C?   1                'Clean components follow',IER)
      CALL FITS_WLOG ('BLOCKED',.TRUE.,
     1                'Tape may be blocked',IER)
      CALL FITS_WREAL('BSCALE', BSCALE,
     1                'Real = Tape*BSCALE + BZERO',IER)
      CALL FITS_WREAL('BZERO', BZERO, ' ', IER)
      CALL FITS_WSTR ('BUNIT',    'JY/BEAM',
     1                'Brightness units', IER)
      CALL FITS_WSTR ('OBJECT', SNAME, 'Object name', IER)
      CALL FITS_WSTR ('INSTRUME', 'VLBI', ' ', IER)
      L = LEN1(TELESC)
      CALL FITS_WSTR ('TELESCOP', TELESC(1:L), ' ', IER)
      CALL FITS_WSTR ('DATE-OBS', DATOBS,
     1                'Date of observation dd/mm/yy', IER)
      CALL FITS_WSTR ('DATE-MAP', DATMAP, 'Date map created', IER)
      CALL FITS_WSTR ('DATE',     DATMAP, 'Date map created', IER)
      CALL FITS_WSTR ('OBSERVER', OBSERVER, ' ', IER)
      CALL FITS_WSTR ('ORIGIN', 'INVERT (Caltech)',
     1                'Version '//VERSN, IER)
      CALL FITS_WREAL('DATAMAX', MAPMAX, 'Map maximum (Jy/beam)',
     1                IER)
      CALL FITS_WREAL('DATAMIN', MAPMIN, 'Map minimum (Jy/beam)',
     1                IER)
      CALL FITS_WREAL('EPOCH',   EPOCH, 'Epoch of RA/Dec', IER)
C?    CALL FITS_WREAL('BMAJ', BMAJ/3600000.,
C?   1                'Beam major axis FHWM (deg)', IER)
C?    CALL FITS_WREAL('BMIN', BMIN/3600000.,
C?   1                'Beam minor axis FHWM (deg)', IER)
C?    CALL FITS_WREAL('BPA', BPA,
C?   1                'Beam major axis position angle (deg)', IER)
C?    CALL FITS_WINT ('NITER', ITER,
C?   1                'Number of CLEAN iterations', IER)
C
C               Axis 1 - RA [shift and rotation ignored!]
C
       PIX1 = ICENT
       DEL1 = -XYINT/3600E3
       STRING = 'RA  = '
       CALL PANGLE('H', RA0, STRING(6:23), 4)
       CALL FITS_WSTR ('CTYPE1',   'RA---SIN',
     1 '--- Axis 1: Right Ascension ---', IER)
       CALL FITS_WDBLE('CRVAL1',   RA0/RPDEG, STRING, IER)
       CALL FITS_WREAL('CRPIX1',   PIX1,      ' ', IER)
       CALL FITS_WREAL('CDELT1',   DEL1,      ' ', IER)
       CALL FITS_WREAL('CROTA1',   0.0,       ' ', IER)
C
C               Axis 2 - DEC
C
       PIX2 = ICENT
       DEL2 = -XYINT/3600E3
       STRING = 'Dec = '
       CALL PANGLE('D', DEC0, STRING(6:22), 3)
       CALL FITS_WSTR ('CTYPE2',   'DEC--SIN',
     1 '--- Axis 2: Declination ---', IER)
       CALL FITS_WDBLE('CRVAL2',   DEC0/RPDEG, STRING, IER)
       CALL FITS_WREAL('CRPIX2',   PIX2,      ' ', IER)
       CALL FITS_WREAL('CDELT2',   DEL2,      ' ', IER)
       CALL FITS_WREAL('CROTA2',   0.0,       ' ', IER)
C
C               Axis 3 - Frequency (degenerate).
C
       CALL FITS_WSTR ('CTYPE3',   'FREQ',
     1 '--- Axis 3: Frequency ---', IER)
       CALL FITS_WREAL('CRVAL3',   FREQ,      ' ', IER)
       CALL FITS_WREAL('CRPIX3',   1.0,       ' ', IER)
       CALL FITS_WREAL('CDELT3',   0.0,       ' ', IER)
       CALL FITS_WREAL('CROTA3',   0.0,       ' ', IER)
C
C               Axis 4 - Stokes parameter (degenerate).
C
       CALL FITS_WSTR ('CTYPE4',   'STOKES',
     1 '--- Axis 4: Stokes parameter ---', IER)
       IF (BFLG) THEN
           CALL FITS_WREAL('CRVAL4',   0.0,   'Dirty beam', IER)
       ELSE
           CALL FITS_WREAL('CRVAL4',   1.0,   'Dirty map', IER)
       END IF
       CALL FITS_WREAL('CRPIX4',   1.0,       ' ', IER)
       CALL FITS_WREAL('CDELT4',   1.0,       ' ', IER)
       CALL FITS_WREAL('CROTA4',   0.0,       ' ', IER)
C
C               History.
C
       DO I=1,LEN(MESSAGE)
                MESSAGE(I:I) = '-'
       END DO
       CALL FITS_WCMT('HISTORY', MESSAGE, IER)
       CALL FITS_WCMT('HISTORY', 'INVERT version:  '//VERSN, IER)
       CALL FITS_HIS(IER)
C?     CALL FITS_WCMT('HISTORY', 'Dirty map file:  '//FMAP,  IER)
C?     CALL FITS_WCMT('HISTORY', 'Dirty beam file: '//FBEAM, IER)
C?     CALL FITS_WCMT('HISTORY', 'Clean map file:  '//FCMAP, IER)
C?     CALL FITS_WCMT('HISTORY', 'Model file:      '//FMODEL,IER)
C
C               Hidden parameters for AIPS.
C
C?    IF (NOREST) CALL FITS_WCMT('HISTORY',
C?   1               'AIPS PRODUCT=3   / Residuals', IER)
C
C               End FITS header; write FITS data.
C
      CALL FITS_WEND(IER)
      DO 180 N=1,IDIM
          CALL FITS_WRAYF(MAPARR(1,N), IDIM, BITPIX,
     1                    BSCALE, BZERO, IER)
  180 CONTINUE
      CALL FITS_CLOSE(IER)
      IF (IER.NE.0) CALL ERROR(
     1          'Something went wrong writing FITS file.')
C 
C               Plot contour or gray-scale map if requested.
C 
      IRAD = NINT(PMRAD/XYINT)
      IF (BFLG) IRAD = NINT(PBRAD/XYINT)
      IF (IRAD.GT.0) THEN
            MSLO = MAX0(MSLO,ICENT-IRAD)
            MSHI = MIN0(MSHI,ICENT+IRAD)
            NSLO = MAX0(NSLO,ICENT-IRAD)
            NSHI = MIN0(NSHI,ICENT+IRAD)
      END IF
      X1 = (MSLO-ICENT)*XYINT
      X2 = (MSHI-ICENT)*XYINT
      Y1 = (ICENT-NSHI)*XYINT
      Y2 = (ICENT-NSLO)*XYINT
      IF (AMAX1(X2-X1,Y2-Y1).LT.1000.0) THEN
            XYUNIT = '(milliarcsec)'
            FACTOR = 1.0
      ELSE
            XYUNIT = '(arcsec)'
            FACTOR = 0.001
      END IF
      X1 = X1*FACTOR
      Y1 = Y1*FACTOR
      X2 = X2*FACTOR
      Y2 = Y2*FACTOR
      TR(1) = -ICENT*XYINT*FACTOR
      TR(2) = XYINT*FACTOR
      TR(3) = 0.0
      TR(4) = ICENT*XYINT*FACTOR
      TR(5) = 0.0
      TR(6) = -XYINT*FACTOR
      IF ((BFLG.AND.PLTBM).OR.(.NOT.BFLG.AND.PLTMAP)) THEN
          WRITE (LIST,680) (VCNPC(ICON),ICON=1,NCONS)
          DO ICON=1,NCONS
              VCN(ICON) = VCNPC(ICON)*(MAPMAX/100.0)
          END DO
          CALL PGADVANCE
          CALL PGVSTAND
          CALL PGWNAD(X1,X2,Y1,Y2)
          CALL PGSCI(1)
          CALL PGCONT(MAPARR,IDIM,IDIM,MSLO,MSHI,NSLO,NSHI,
     1                VCN,NCONS,TR)
          CALL PGSCI(9)
          CALL PGBOX('BCNST', 0.0, 0, 'BCNST', 0.0, 0)
          CALL PGLABEL('x '//XYUNIT,'y '//XYUNIT,' ')
          CALL PGMTEXT('T',2.5,0.0,0.0,PASS)
          CALL PGMTEXT('T',0.8,0.0,0.0,SNAME)
C#        CALL PGMTEXT('T',0.8,1.0,1.0,LEGEND)
          CALL PGSCI(1)
      END IF
      IF ((BFLG.AND.PRTBM).OR.(.NOT.BFLG.AND.PRTMAP)) THEN
          CALL PGADVANCE
          CALL PGVSTAND
          CALL PGWNAD(X1,X2,Y1,Y2)
          CALL PGSCI(1)
          CALL PGGRAY(MAPARR,IDIM,IDIM,MSLO,MSHI,NSLO,NSHI,
     1                1.3*MAPMAX,0.0,TR)
          CALL PGSCI(9)
          CALL PGBOX('BCNST', 0.0, 0, 'BCNST', 0.0, 0)
          CALL PGLABEL('x '//XYUNIT,'y '//XYUNIT,' ')
          CALL PGMTEXT('T',2.5,0.0,0.0,PASS)
          CALL PGMTEXT('T',0.8,0.0,0.0,SNAME)
C#        CALL PGMTEXT('T',0.8,1.0,1.0,LEGEND)
          CALL PGSCI(1)
      END IF
      CALL PUTOUT('INVERT - plotting completed')
C 
   33 CONTINUE
C 
C               Close plot file
C 
      IF (DOPLOT) CALL PGEND
      IF (NOMAP) FMAP = 'NOMAP requested'
      IF (NOBEAM) FBEAM = 'NOBEAM requested'
      IF (.NOT.DOPLOT) FPLOT = 'none'
C 
C               Read the clock
C 
      WRITE (LIST,690) CPUW,CPUI,CPUF
C 
C Execution summary.
C 
      WRITE (LIST,686)
      WRITE (LIST,685) 'INVERT version:',VERSN
      CALL REPORT(LIST)
      CALL REPCPU(LIST)
      WRITE (LIST,685) 'MAPFILE:',FMAP(1:LEN1(FMAP))
      WRITE (LIST,685) 'BEAMFILE:',FBEAM(1:LEN1(FBEAM))
      WRITE (LIST,685) 'PLOTFILE:',FPLOT(1:LEN1(FPLOT))
C 
      CALL PUTOUT('INVERT - completed')
      RETURN
C-----------------------------------------------------------------------
C       Format statements
C-----------------------------------------------------------------------
  610 FORMAT(10A8)
  620 FORMAT(//' Cellsize (XYINT):  ',F10.3,' milliarcsec'/
     1           ' u,v interval (UVINT):  ',F16.2,' wavelengths'/
     2           ' Weighting type:  ',A,/,
     3           ' Gridding weights divided by (data errors)**',F5.2,/
     4           ' Oversampling factor:  ',F10.3/
     5           ' Half FWHM of interpolation fn. (SIGMA): ',F8.4,
     6           ' * UVINT'/
     7           ' Array size for Fourier transform: ',I6)
  619 FORMAT(' *****************************************************'/
     1       ' **WARNING** Oversampling factor is too small (<2.5) *'/
     2       ' *****************************************************')
  621 FORMAT(  ' *** Map rotated by ',F8.2,' degrees')
  622 FORMAT(  ' *** Map shifted by ',2F9.3,' milliarcsec')
  623 FORMAT(//' Convolving function:'//(10F8.4))
  624 FORMAT(//' Sensitivity correction function:'//(10F8.2))
  628 FORMAT(' Estimated noise level for natural weighting',
     1          ' (no SNR scaling): ',F10.3,' mJy')
  627 FORMAT(' Estimated noise level for actual weighting:  ',
     2         F10.3,' mJy')
  660 FORMAT(//10X,A8//
     1           ' X index range: ',2I5/
     2           ' Y index range: ',2I5/
     3           ' Center at ',I5,',',I5/
     4           ' Sum of weights: ',1PE12.3/
     5           ' Maximum ',1PE12.3,'  in cell',2I5/
     6           ' Minimum ',1PE12.3,'  in cell',2I5)
  670 FORMAT(//' Source of ',F8.3,' Jy subtracted at',2F10.3,
     1           ' milliarcsec')
  680 FORMAT(//' Contour levels (percent of peak):'//
     1           (10F8.2))
  685 FORMAT (1X,A,T20,A)
  686 FORMAT(//' INVERT execution summary:'/)
  690 FORMAT(//' CPU time for weighting =     ',F7.1/
     3           '              interpolation = ',2F7.1/
     4           '              FFT =           ',2F7.1)
      END

      SUBROUTINE FINDUV(OUTC,FILE,U,V,VISIB,VISERR,MAXP,NP,
     1                  UVCUT,UVMAX,SNAME,FRQHZ,TELESC,
     2                  DATOBS,EPOCH,RA0,DEC0)
      INTEGER OUTC
C-----------------------------------------------------------------------
C INVERT: Read input data file and determine all u,v coordinates and
C visibilities. u,v in wavelengths; visibilities in Jy. Ignore points
C beyond UVCUT.
C 
C Author:               T.J. Pearson (TJP), D.L. Meier (DLM)
C Language:             VAX-11 Fortran
C 
C Modifications:
C 1983 Jul 28 - extend to read MERGE version 2 files (DLM).
C 1983 Aug  4 - add W to MERGE version 2 (DLM).
C 1984 Dec 10 - return VISERR (TJP).
C 1985 Jun 25 - return TELESC, DATOBS, EPOCH.
C 1986 Apr  4 - return RA0, DEC0 (map center) [TJP].
C 1986 Aug 28 - increase MAXS from 15 to 20 [TJP].
C 1989 Jul 13 - allow for more than one file [TJP].
C-----------------------------------------------------------------------
        INTEGER   INDAT,MAXS,MAXB,MAXT,MAXWPB
        REAL*8    PI,SIDR,VELC,RPDEG
        PARAMETER (INDAT=8,MAXS=20,MAXB=(MAXS-1)*MAXS/2)
        PARAMETER (MAXT=MAXB*(MAXS-2)/3)
        PARAMETER (MAXWPB=7)
        PARAMETER (PI = 3.141592653589793 D0)
        PARAMETER (SIDR = (PI/2592000.0D0)*1.0027379093 D0)
C               (radians of ST per sec/60 of UT)
        PARAMETER (VELC = 2.997925 D8)
        PARAMETER (RPDEG = PI/180D0)
C 
        REAL    U(MAXP),V(MAXP),UVMAX,VISERR(MAXP)
        COMPLEX VISIB(MAXP)
        CHARACTER*(*) FILE
        CHARACTER*16 SNAME
        CHARACTER*32 TELESC
        CHARACTER*8  DATOBS,STNAME
        CHARACTER*1  STNABR
        REAL    EPOCH
        LOGICAL ORBITERS
        INTEGER STATYP
        REAL    REC(MAXWPB*MAXB)
        REAL*8  STX,STY,STZ,SOURCE,RA,DEC,GASTM,FREQ,FLUX,
     1          WAVE,BX,BY,BZ(MAXB),BOFF(MAXB),BXY(MAXB),TIHA,
     2          COSDEC,SINDEC,TINTG,AVG,BW,AEIOWM, RA0, DEC0
        COMMON /PARMS/ STNAME(MAXS),STX(MAXS),STY(MAXS),STZ(MAXS),
     1          SOURCE(2),RA,DEC,GASTM,FREQ,FLUX,TINTG,AVG,BW,
     2          AEIOWM(6,MAXS),STATYP(MAXS),ORBITERS
        COMMON /ORDER/ NS,NB,NT, NB1(MAXB),NB2(MAXB),NC1(MAXT),
     1          NC2(MAXT),NC3(MAXT),NVER,NDWPB
C 
        OPEN (UNIT=INDAT, NAME=FILE, READONLY, STATUS='OLD',
     1       FORM='UNFORMATTED', ERR=120)
        INQUIRE (UNIT=INDAT, NAME=FILE)
C 
        WRITE (OUTC,600) FILE(1:LEN1(FILE))
        CALL HEADX20(.FALSE.,INDAT,IZ,ID,IY,DUM1,0,0,KFLAG,DUM3,IFEOF,0)
        IF (NS.LT.2 .OR. NB.LT.1) CALL ERROR('Not enough stations.')
        IF (NS.GT.MAXS) CALL ERROR('Too many stations.')
C*      The following check is probably best left out:
C*      IF (KFLAG.NE.1) CALL ERROR('Data have not been calibrated.')
        LENG = NDWPB*NB
        WRITE (SNAME,620) SOURCE
        FRQHZ = FREQ
        RA0 = RA
        DEC0 = DEC
        L = LEN1(TELESC)
        IF (L.NE.0) THEN
            L = L+1
            TELESC(L:L) = '+'
        END IF
        DO IS=1,NS
            TELESC(L+IS:L+IS) = STNABR(STNAME(IS))
        END DO
        IMON = 1
        CALL TDATE(IY,IMON,ID,JD,MONTH,DUM1)
        WRITE (DATOBS,640) ID,IMON,MOD(IY,100)
        EPOCH = 2000.0 + ((JD-2451545)-0.5D0)/365.25D0
        WRITE (OUTC,630) SOURCE, FREQ*1E-9,
     1                   TELESC(L+1:L+NS), DATOBS, EPOCH,
     2                   (STNAME(J),J=1,NS)
C 
C               Baseline parameters
C 
        WAVE = VELC/FREQ
        COSDEC = DCOS(DEC)
        SINDEC = DSIN(DEC)
        DO 210 IB=1,NB
        IF (NVER.EQ.1) THEN
                IS1 = NB1(IB)
                IS2 = NB2(IB)
                BX = (STX(IS1)-STX(IS2))/WAVE
                BY = (STY(IS1)-STY(IS2))/WAVE
                BZ(IB) = (STZ(IS1)-STZ(IS2))/WAVE
                BXY(IB) = DSQRT(BX*BX+BY*BY)
                IF (BXY(IB).EQ.0D0)
     1                  CALL ERROR('Station coordinates missing.')
                BOFF(IB) = GASTM-RA-PI/2D0-DATAN2(BY,BX)
        ELSE
                BZ(IB)   = 0.D0
                BXY(IB)  = 0.D0
                BOFF(IB) = 0.D0
        END IF
210     CONTINUE
C 
C               Determine all (u,v) values
C               and complex visiblities
C 
        AMPMAX = 0.0
        IP = NP
        IREJEC = 0
C 
C               (start loop)
  300   READ (INDAT,END=400) IUT,(REC(J),J=1,LENG)
        IF (IUT.LT.0) GOTO 400
        DO 320 IB=1,NB
        IF (REC(NDWPB*(IB-1)+2).LE.0.0 .OR. REC(NDWPB*(IB-1)+4).LE.0.0)
     1     GOTO 320
C               ( skip deleted data )
        IF (NVER.EQ.1) THEN
                TIHA = (IUT-IZ)*SIDR + BOFF(IB)
                UP = BXY(IB)*DCOS(TIHA)
                VP = BZ(IB)*COSDEC + BXY(IB)*SINDEC*DSIN(TIHA)
        ELSE IF (NVER.EQ.2) THEN
                UP = REC(NDWPB*(IB-1)+5)
                VP = REC(NDWPB*(IB-1)+6)
        END IF
        UVP = SQRT(UP*UP+VP*VP)
        IF (UVP.GT.UVCUT) THEN
                IREJEC = IREJEC+1
        ELSE
                IP = IP+1
                IF (IP.GT.MAXP) CALL ERROR('Too many points.')
                U(IP) = UP
                V(IP) = VP
                UVMAX = AMAX1(UVMAX,UVP)
                PHASE = REC(NDWPB*(IB-1)+3)*RPDEG
                VISIB(IP) = REC(NDWPB*(IB-1)+1)*CEXP(CMPLX(0.0,PHASE))
                AMPMAX = AMAX1(AMPMAX,REC(NDWPB*(IB-1)+1))
                VISERR(IP) = REC(NDWPB*(IB-1)+2)
        END IF
  320   CONTINUE
        GOTO 300
C               (end loop)
C 
  400   NP = IP
        CLOSE (UNIT=INDAT)
        IF (NP.LE.0) CALL ERROR('No visibility data.')
        WRITE (OUTC,610) NP,UVMAX,AMPMAX,IREJEC
        RETURN
C 
  120   CALL ERROR('Can''t open file '//FILE)
C 
  600 FORMAT(//' Input file: ',A)
  610 FORMAT(  '    Number of u,v points accepted: ',I6
     1       / '    UVmax = ',F16.2,' wavelengths'
     2       / '    Maximum correlated flux density = ',F9.3,' Jy'
     3       / '    Number of points rejected (baseline > UVCUT): ',I6)
  620 FORMAT(2A8)
  630 FORMAT(  '    Source: ',2A8
     1       / '    Frequency: ',F10.3,' GHz'
     2       / '    Array: ',A
     3       / '    Observation date: ',A,F10.2
     4       / '    Stations: ',(1X,8A9))
  640 FORMAT(I2.2,'/',I2.2,'/',I2.2)
      END

      SUBROUTINE FASTWT(OUTC,UVRAD,NWT,UVTVAL,UVTWID,UVMAX,U,V,WT,
     1                  NP,WT0, MAPARR,IDIM)
      INTEGER OUTC
C-----------------------------------------------------------------------
C INVERT: Choose weights, using FAST_UNIFORM algorithm. We simply count
C the number of points in square cells in the (u,v) plane. The cellsize
C is UVRAD*UVINT.
C 
C Author:               T.J. Pearson (TJP)
C Language:             VAX-11 Fortran
C Written:              18-Aug-1983
C 
C Modifications:
C    09-Jan-1988  Declare NWT integer rather than logical (RCW).
C-----------------------------------------------------------------------
        REAL    U(NP),V(NP),WT(NP)
        INTEGER NWT
        INTEGER MAPARR(*)
C 
        IF (NWT.NE.4) CALL ERROR('Bug in FASTWT. Report to TJP.')
        WRITE (OUTC,100) UVRAD
        IF (UVRAD.LT.1.0) THEN
            CALL ERROR('UVRAD must not be < 1 for FAST weighting.')
        ELSE IF (UVRAD.GT.1.0) THEN
            DO IP=1,NP
                U(IP) = U(IP)/UVRAD
                V(IP) = V(IP)/UVRAD
            END DO
        END IF
C 
C               Zero the uvplane array.
C 
        ICENT = IDIM/2
        DO K=1,IDIM*IDIM
            MAPARR(K) = 0
        END DO
        MAPARR(ICENT+(ICENT-1)*IDIM) = 1
C 
C               Find which cell each (u,v) point belongs in.
C               Center of array is at (IDIM/2,IDIM/2).
C 
        DO IP=1,NP
            I = ICENT - U(IP) + 0.5
            J = ICENT - V(IP) + 0.5
            K = I + (J-1)*IDIM
            MAPARR(K) = MAPARR(K) + 1
            I = ICENT + U(IP) + 0.5
            J = ICENT + V(IP) + 0.5
            K = I + (J-1)*IDIM
            MAPARR(K) = MAPARR(K) + 1
        END DO
C 
C               Compute weights.
C 
        NMAX = MAPARR(ICENT+(ICENT-1)*IDIM)
        WT0 = 1.0/NMAX
        DO IP=1,NP
            I = ICENT - U(IP) + 0.5
            J = ICENT - V(IP) + 0.5
            K = I + (J-1)*IDIM
            NMAX = MAX(NMAX,MAPARR(K))
            WT(IP) = 1.0/MAPARR(K)
        END DO
        WRITE (OUTC,120) NMAX
        IF (UVRAD.GT.1.0) THEN
            DO IP=1,NP
                U(IP) = U(IP)*UVRAD
                V(IP) = V(IP)*UVRAD
            END DO
        END IF
C 
C               Apply Gaussian taper: exp[(u*u+v*v)*gauss]
C 
        IF (UVTVAL.LT.1.0.AND.UVTVAL.GT.0.0) THEN
            GAUSS = ALOG(UVTVAL)/UVTWID/UVTWID
            DO IP=1,NP
                WT(IP) = WT(IP)*EXP(GAUSS*(U(IP)**2+V(IP)**2))
            END DO
            WRITE (OUTC,140) 1.0/SQRT(-GAUSS),UVTVAL,UVTWID
        END IF
C 
        RETURN
C 
  100   FORMAT(//' FAST uv-weighting: weights inversely proportional'
     1          /6X,'to number of points within each cell of size ',
     2          F6.3,' * UVINT')
  120   FORMAT(' Maximum density of points per cell in u,v-plane =',I6)
  140   FORMAT(' Gaussian taper applied: falling to 1/e at baseline',
     1         F16.2,' * UVINT'/' UVTAPER = ',F8.3,F16.2,' * UVINT')
        END

      SUBROUTINE FINDWT(OUTC,UVRAD,NWT,UVTVAL,UVTWID,UVMAX,U,V,WT,
     1                  NP,WT0)
      INTEGER OUTC
C-----------------------------------------------------------------------
C INVERT: Choose weights.  For NATURAL (NWT=1), all weights are 1; for
C UNIFORM (NWT=2), weight is inversely proportional to the local
C density of (u,v) points, defined as the number of points within a
C distance UVRAD.  This is rather arbitrary. WT0 is the weight for the
C zero-spacing.
C 
C Author:               T.J. Pearson (TJP)
C Language:             VAX-11 Fortran
C Written:
C 
C Modifications:
C 28-Nov-1981 - faster: eliminate unnecesary SQRTs (TJP).
C 15-Aug-1983 - assume uv data are sorted by u (TJP).
C 09-Jan-1988 - declare NWT integer rather than logical (RCW).
C-----------------------------------------------------------------------
        REAL    U(NP),V(NP),WT(NP)
        INTEGER NWT
        INTEGER LOW,HIGH,MID
C 
        DO IP=1,NP
            WT(IP) = 1.0
        END DO
        WT0 = 1.0
        UVRAD2 = UVRAD**2
C 
C               NATURAL weighting (all weights are 1).
C 
        IF (NWT.EQ.1) THEN
            WRITE (OUTC,100)
C 
C               RADIAL weighting.
C 
        ELSE IF (NWT.EQ.3) THEN
            WRITE (OUTC,130)
            DO IP=1,NP
                WT(IP) = SQRT(U(IP)**2+V(IP)**2)
            END DO
            WT0 = 0.0
        END IF
C 
C               Apply Gaussian taper: exp[(u*u+v*v)*gauss]
C 
        IF (UVTVAL.LT.1.0.AND.UVTVAL.GT.0.0) THEN
            GAUSS = ALOG(UVTVAL)/UVTWID/UVTWID
            DO IP=1,NP
                WT(IP) = WT(IP)*EXP(GAUSS*(U(IP)**2+V(IP)**2))
            END DO
            WRITE (OUTC,140) 1.0/SQRT(-GAUSS),UVTVAL,UVTWID
        END IF
C 
        RETURN
C 
  100   FORMAT(//' NATURAL uv-weighting: all points given equal weight')
  130   FORMAT(//' RADIAL  uv-weighting: weights proportional to r')
  140   FORMAT(' Gaussian taper applied: falling to 1/e at baseline',
     1         F16.2,' * UVINT'/' UVTAPER = ',F8.3,F16.2,' * UVINT')
        END

        SUBROUTINE SHIFT(U,V,VISIB,NP,DX,DY)
C-----------------------------------------------------------------------
C INVERT: Apply phase rotation to shift map by (dx,dy) (radians).
C 
C Author:               T.J. Pearson (TJP)
C Language:             VAX-11 Fortran
C 
C Modifications:
C-----------------------------------------------------------------------
        REAL       PI
        PARAMETER  (PI=3.1415926536)
        REAL       U(NP),V(NP)
        COMPLEX    VISIB(NP)
C 
        IF(DX.EQ.0.0 .AND. DY.EQ.0.0) RETURN
        X = 2.0*PI*DX
        Y = 2.0*PI*DY
        DO 100 IP=1,NP
  100   VISIB(IP) = VISIB(IP)*CEXP(CMPLX(0.0,U(IP)*X+V(IP)*Y))
        RETURN
        END

        SUBROUTINE TERPUV(UVARR,ICENT,IDIM, U,V,VISIB,WT,NP,
     1                    FTERP,NTERP,ISAMP, BFLG, WSUM)
C-----------------------------------------------------------------------
C INVERT: interpolate visibility data into UV array. The symmetric
C interpolation function FTERP(1..NTERP) is sampled at intervals of
C UVINT/ISAMP out to a  maximum of 3*ISAMP+1.
C 
C Author:               T.J. Pearson (TJP)
C Language:             VAX-11 Fortran
C 
C Modifications:
C 18-Aug-1983 - trivial changes for speed (TJP).
C-----------------------------------------------------------------------
        COMPLEX UVARR(ICENT,IDIM)
C               (conjugate-symmetric array)
        COMPLEX VISIB(NP)
        REAL    U(NP),V(NP),WT(NP)
C               (visibility data and weights;
C                u and v in units of cells)
        REAL    FTERP(NTERP)
C               (tabulated interpolation function)
        LOGICAL BFLG
        COMPLEX VDATA,WDATA
C 
        DO IP=1,NP
            UP = ICENT - U(IP)
            VP = ICENT - V(IP)
            IU1 = INT(UP)-2
            IU2 = IU1+5
            IV1 = INT(VP)-2
            IV2 = IV1+5
            IF (BFLG) THEN
                VDATA = WT(IP)
                WDATA = VDATA
            ELSE
                VDATA = VISIB(IP)*WT(IP)
                WDATA = CONJG(VDATA)
            END IF
            DO IU=IU1,IU2
                IUT = ISAMP*ABS(UP-IU)+1.5
                FU  = FTERP(IUT)
                JU  = 2*ICENT-IU
                DO IV=IV1,IV2
                    IVT = ISAMP*ABS(VP-IV)+1.5
                    FUV = FTERP(IVT)*FU
                    IF (IU.GT.ICENT) THEN
                        JV = 2*ICENT-IV
                        UVARR(JU,JV) = UVARR(JU,JV) + WDATA*FUV
                    ELSE
                        UVARR(IU,IV) = UVARR(IU,IV) + VDATA*FUV
                    END IF
                    WSUM = WSUM + WT(IP)*FUV
                END DO
            END DO
        END DO
C 
C               Conjugate the center line
C 
        DO IV=2,ICENT-1
            IVT = 2*ICENT-IV
            VDATA = UVARR(ICENT,IV)+CONJG(UVARR(ICENT,IVT))
            UVARR(ICENT,IV) = VDATA
            UVARR(ICENT,IVT) = CONJG(VDATA)
        END DO
C 
        RETURN
        END

        SUBROUTINE PLOTUV(UVARR,ICENT,I1,I2,J1,J2,UVINT)
C-----------------------------------------------------------------------
C INVERT: plot gray-scale map of interpolated (u,v)-plane coverage.
C Array UVARR is complex, and has to be rearranged for plotting by
C PGPLOT, and restored after plotting. The imaginary part is assumed
C to be zero!
C 
C Author:               T.J. Pearson (TJP)
C Language:             VAX-11 Fortran
C Written:
C 
C Modifications:
C 17-Aug-1983 - label plot in wavelengths (TJP).
C-----------------------------------------------------------------------
        REAL TR(6)
        REAL UVARR(2*ICENT,*)
C 
C               Convert from complex to real format.
C 
        TMAX = 0.0
        DO J=J1,J2
            DO I=1,ICENT
                UVARR(I,J) = UVARR(2*I-1,J)
                TMAX = MAX(TMAX,UVARR(I,J))
            END DO
        END DO
C 
C               Plot.
C 
        TR(1) = ICENT*UVINT*1E-6
        TR(2) = -UVINT*1E-6
        TR(3) = 0.0
        TR(4) = ICENT*UVINT*1E-6
        TR(5) = 0.0
        TR(6) = -UVINT*1E-6
        X1 = TR(1) + TR(2)*I1
        X2 = TR(1) + TR(2)*I2
        Y1 = TR(4) + TR(6)*J1
        Y2 = TR(4) + TR(6)*J2
        CALL PGADVANCE
        CALL PGVSTAND
        CALL PGWNAD(X1,X2,Y2,Y1)
        CALL PGSCI(1)
        CALL PGGRAY(UVARR,ICENT*2,2*(ICENT-1),I1,I2,J1,J2,
     1          TMAX, 0.0, TR)
        CALL PGSCI(9)
        CALL PGBOX('BCNST', 0.0, 0, 'BCNST', 0.0, 0)
        CALL PGLABEL('u (M\gl)','v (M\gl)',
     1          'Interpolated Visibility Array')
        CALL PGSCI(1)
C 
C               Restore.
C 
        DO J=J1,J2
            DO I=ICENT,1,-1
                UVARR(2*I-1,J) = UVARR(I,J)
                UVARR(2*I,J) = 0.0
            END DO
        END DO
        RETURN
        END

      SUBROUTINE UVBEAM(U, V, WT, N, OUTC, UVINT)
      INTEGER N, OUTC
      REAL U(N), V(N), WT(N)
      REAL UVINT
C 
C Estimate FWHM of dirty beam by computing the second moments of the
C distribution of uv points.
C 
C TJP 1988 Feb 25
C--
      INTEGER I
      REAL MUU, MUV, MVV, S, A, B, PA, T, SCALE
      REAL PI, RPMAS
      PARAMETER (PI=3.141592653589793)
      PARAMETER (RPMAS=PI/180.0/60.0/60.0/1000.0)
C 
      MUU = 0.0
      MUV = 0.0
      MVV = 0.0
      T = 0.0
      DO 10 I=1,N
          MUU = MUU + U(I)*U(I)*WT(I)
          MUV = MUV + U(I)*V(I)*WT(I)
          MVV = MVV + V(I)*V(I)*WT(I)
          T = T + WT(I)
   10 CONTINUE
      MUU = MUU/T
      MUV = MUV/T
      MVV = MVV/T
      PA = -0.5*ATAN2D(2.0*MUV, (MUU-MVV))
      S = SQRT((MUU-MVV)*(MUU-MVV) + 4.0*MUV*MUV)
C 
C Widths of uv distribution in wavelengths:
C 
      A = SQRT(2.0*(MUU+MVV) + 2.0*S)*UVINT
      B = SQRT(2.0*(MUU+MVV) - 2.0*S)*UVINT
C 
C The following scale factor to convert the A and B values
C to FWHM beamwidths in mas is empirical.
C 
      SCALE = 0.70/RPMAS
      WRITE (OUTC,100) SCALE/B, SCALE/A, PA, B/A
  100 FORMAT(/' Estimated beam size:'/
     1        '   Major axis (mas): ',F10.3/
     2        '   Minor axis (mas): ',F10.3/
     3        '   Position angle of major axis: ',F8.1/
     4        '   Axial ratio:      ',F9.2)
      END
