C* Copied from BIGDISK:[CORSYS.SOURCE]PCRDEF.FOR
C PCRDEF  !Version 17OCT88 11:00
C Block-2 correlator PCR structured definitions
C Thomas G. Lockhart, JPL m/s 138-208, Pasadena CA 91109
C written March 26, 1986
C modified Sept 23, 1988 by DNF
C - include new definitions of TNSTBL and TDOTBL (WNDTBL and WDOTBL)
C modified June 11, 1987
C - modify format of phase model table
C - define geometric model table dimensions using parameter MAXGME
C - repair typo in station name field definition

C Table identification labels.

	INTEGER*2 PRCLBL, GRPLBL, ICMLBL	! global configuration tables
	INTEGER*2 TONLBL, TMLLBL, TMOLBL	! tone (station) tables
	INTEGER*2 DLYLBL, GEOLBL, PHSLBL	! cross correlation tables
	INTEGER*2 TNSLBL, TCOLBL, TDOLBL	! TENSOR-specific tables
	INTEGER*2 WNDLBL, WDOLBL		! WINDOW-specific tables (DNF)
	INTEGER*2 OPRLBL
	INTEGER*2 CDOLBL, CSOLBL		! correlator output tables
	INTEGER*2 SDOLBL, SSOLBL		! station output tables
	INTEGER*2 IAOLBL			! mark-2 interface table
	INTEGER*2 FDOLBL			! digital filter output

	PARAMETER (PRCLBL = '10'X)	! process configuration table
	PARAMETER (GRPLBL = '20'X)	! group configuration table
	PARAMETER (ICMLBL = '22'X)	! mark-2 interface configuration

C station tone (phase cal) tables

	PARAMETER (TONLBL = '30'X)	! tone model
	PARAMETER (TMLLBL = '32'X)	! tone model lookup table header
	PARAMETER (TMOLBL = '35'X)	! tone (phase cal) model output

C cross-correlation model tables

	PARAMETER (DLYLBL = '40'X)	! delay model
	PARAMETER (GEOLBL = '45'X)	! geometric model output
	PARAMETER (PHSLBL = '50'X)	! phase model

C tensor configuration tables

	PARAMETER (TNSLBL = '60'X)	! tensor configuration
	PARAMETER (WNDLBL = '61'X)	! window configuration (DNF)
	PARAMETER (TCOLBL = '65'X)	! tensor current configuration output

C output tables

	PARAMETER (CDOLBL = '70'X)	! cross-correlation data
	PARAMETER (CSOLBL = '71'X)	! cross-correlation status
	PARAMETER (SDOLBL = '72'X)	! tone (phase cal) data
	PARAMETER (SSOLBL = '73'X)	! tone (phase cal) status
	PARAMETER (IAOLBL = '76'X)	! interface ancillary data
	PARAMETER (OPRLBL = '78'X)	! operator comment
	PARAMETER (TDOLBL = '80'X)	! tensor (segment) data
	PARAMETER (WDOLBL = '81'X)	! window data (DNF)
	PARAMETER (FDOLBL = '90'X)	! digital filter data

C Basic definitions.
C
C The least significant longword of a quadword integer should be handled
C	as an unsigned quantity, which is an unsupported data type
C	in VAX FORTRAN.  There is some support for extended integers
C	in the VAX Run Time Library.

	STRUCTURE /SYS_PC/  ! VAX system time
	    INTEGER*4 L0			! least significant longword
	    INTEGER*4 L1			! most significant longword
	END STRUCTURE ! /SYS_PC/

	STRUCTURE /INT_8/  ! 8 byte integer
	    INTEGER*4 L0			! least significant longword
	    INTEGER*4 L1			! most significant longword
	END STRUCTURE ! /INT_8/

	STRUCTURE /STR_8/  ! 8 byte fixed length string
	    BYTE BODY(8)
	END STRUCTURE ! /STR_8/

	STRUCTURE /STR_16/  ! 16 byte counted string
	    BYTE LENGTH
	    BYTE BODY(15)
	END STRUCTURE ! /STR_16/

	STRUCTURE /STR_256/  ! 256 byte counted string
	    BYTE LENGTH
	    BYTE BODY(255)
	END STRUCTURE ! /STR_256/

C Generic table definition.

	STRUCTURE /TID_PC/  ! generic identification table

C	required fields

	    INTEGER*2 GROUPPC			! process group #
	    INTEGER*2 CLASSPC			! table type #

C	optional fields

	    INTEGER*4 MODELPC			! model #
	    RECORD /INT_8/ SYSTMPC		! model sync time
	    REAL*8 RTIMEPC			! information reference time
	    BYTE %FILL(8)			! (reserved)
	END STRUCTURE ! /TID_PC/

C Process table definition.

	STRUCTURE /PRC_SO/  ! process configuration station table
	    INTEGER*4 PSTNSSO			! physical station #
	    BYTE %FILL(28)			! (reserved)
	END STRUCTURE ! /PRC_SO/

	STRUCTURE /PRC_GO/  ! process configuration table

C	table identification

	    INTEGER*2 GROUPGO			! process group #
	    INTEGER*2 CLASSGO			! table type #
	    BYTE %FILL(28)			! (reserved)

C	FORTH code configuration and table dimensions

	    INTEGER*4 PRVIDGO			! program version ID (yyyyddd)
	    INTEGER*4 MXCHNGO			! maximum # processor channels
	    INTEGER*4 MXTMCGO			! maximum # tone model coefficients
	    INTEGER*4 MXCYCGO			! maximum # time mux cycles
	    BYTE %FILL(16)			! (reserved)

C	model table configuration

	    INTEGER*4 NSTNSGO			! # of video data stations
	    BYTE %FILL(12)			! (reserved)
	    INTEGER*4 NTMODGO			! # of tone models
	    INTEGER*4 NTCOEGO			! # of tone model coefficients
	    INTEGER*4 NDMODGO			! # of delay models
	    INTEGER*4 NPMODGO			! # of phase models

C	H/W configuration

	    BYTE %FILL(16)
C	    LOGICAL*4 STNFLGO			! use station h/w?
C	    LOGICAL*4 DFOFLGO			! use tone extraction digital filter?
	    BYTE %FILL(16)
C	    LOGICAL*4 CORFLGO			! use cross correlator h/w?
C	    LOGICAL*4 TNSFLGO			! use tensor h/w?
	    LOGICAL*4 INFLAGO			! use mark-2 interface?
	    BYTE %FILL(28)			! (reserved)

C	file names for sub-system downloaded object code

	    RECORD /STR_256/ TCFNMGO		! TENSOR download code file name
	    RECORD /STR_256/ ICFNMGO		! mark-2 interface download code name
	    BYTE %FILL(64)			! (reserved)

C	stations information

	    RECORD /PRC_SO/ STATION(MAXSTN)
	END STRUCTURE ! /PRC_GO/

C Group table definition.
C
C To determine the source of a given channel/data stream use the ECLCHSO
C	fields in the stations section of the group table.  Each zero-based
C	field contains an input stream (channel) MUX setting.  If a value is
C	27 or less, the source is a mark-3 tape track; 28 or 29 indicates
C	that the source is a mark-2 VCR.  A value of greater than 29 is
C	reserved for future use, and may indicate that the source is
C	an internal H/W test data stream.

C To determine the correspondence between mark-2 interface MUX settings
C	and VCR numbers, consult both the logical/physical VCR mapping
C	fields in the group table and the mark-2 interface/mark-3 station
C	mapping table (not yet implemented).

	STRUCTURE /GRP_CO/	! channel model assignment table entry
	    BYTE DMODEL				! delay model #
	    BYTE PMODEL				! phase model #
	    BYTE TMODEL(MAXTEX)			! tone model #'s
	    BYTE %FILL(2)			! (reserved)
	END STRUCTURE ! /GRP_CO/

	STRUCTURE /GRP_SO/	! group configuration station table

C hardware configuration buffer

	    INTEGER*2 ECLCHSO(MAXCHN+2)		! ECL track select
	    INTEGER*2 ECLBSSO			! ECL bit synch frequency select
	    INTEGER*2 ANABDSO			! analog board bit track select
	    INTEGER*2 TSTMDSO			! test pattern mode
	    BYTE %FILL(2)			! (reserved)
	    INTEGER*4 PCINTSO(MAXTEX)		! phase cal integration lengths

C station operation modes

	    LOGICAL*4 TMONFSO			! tone extractors enabled?
	    BYTE %FILL(8)			! (reserved)

C tape sequencing information

	    RECORD /STR_16/ MTSEQSO		! expected tape name
	    RECORD /STR_16/ MTTAGSO		! expected tape ID code
	    BYTE %FILL(32)			! (reserved)
	END STRUCTURE ! /GRP_SO/

	STRUCTURE /GRP_GO/	! group configuration table

C table identification

	    INTEGER*2 GROUPGO			! process group #
	    INTEGER*2 CLASSGO			! table type #
	    BYTE %FILL(4)			! (reserved)
	    RECORD /INT_8/ MSYSTGO		! model synchronization time
	    BYTE %FILL(16)			! (reserved)

C conversion factors (constants)

	    REAL*8 BITSCGO			! bit rate (bits/sec)
	    REAL*8 BITRIGO			! Time Read Interval (bits/TRI)
	    REAL*8 BTCOIGO			! Correlator Interval (bits/COI)
	    REAL*8 BTTFRGO			! Tape Frame Interval (bits/FRI)
	    REAL*8 BTHFRGO			! h/w Frame Interval (bits/HFI)
	    REAL*8 BITMTGO			! speed of light (bits/meter)
	    BYTE %FILL(16)			! (reserved)

C model information

	    INTEGER*4 MYEARGO			! model year (since 1900)
	    BYTE %FILL(4)			! (reserved)
	    REAL*8 ETA00GO			! earth rotation rate (rad/bit)
	    REAL*8 HODELGO			! hardware offset delay (bits)
	    BYTE %FILL(8)			! (reserved)

C model configuration

	    LOGICAL*4 CPFLAGO			! phase calibration from tape?
	    LOGICAL*4 TMLFLGO			! tone model table lookup?
	    INTEGER*4 TMMUIGO			! tone model update int (TRIs)
	    INTEGER*4 GMMUIGO			! geom model update int (TRIs)
	    BYTE %FILL(16)			! (reserved)

C output data configuration

	    LOGICAL*4 LAGFLGO			! correlator output in lag mode?
	    INTEGER*4 SDADIGO			! station data dump int (TRIs)
	    INTEGER*4 CDADIGO			! correlator data dump int (TRIs)
	    INTEGER*4 DFRFCGO			! dig filter reduction factor
	    LOGICAL*4 GDIAGGO			! geom model diagnostics (#488)
	    LOGICAL*4 EODLAGO			! even/odd lags per log chan
	    BYTE %FILL(8)			! (reserved)

C channel configuration

	    INTEGER*4 NCHANGO			! # correlator channels
	    INTEGER*4 L2NPTGO			! log2(# lags per channel)
	    INTEGER*4 SYCHNGO			! time sync channel #
	    BYTE %FILL(12)			! (reserved)
	    INTEGER*4 NCHCHGO			! # phys chans per log chan
	    INTEGER*4 NTECHGO			! # tones per logical channel
	    RECORD /GRP_CO/ PCTABGO(MAXCHN)	! channel model assignments

C mark-2 interface configuration

	    BYTE ICTABGO(MAXCYC)		! channel cycle table
	    BYTE %FILL(2)			! (reserved)
	    LOGICAL*4 RERRSGO			! read tape errors?
	    LOGICAL*4 STEMPGO			! read ancillary VCR data?
	    BYTE COTABGO(MAXCYC)		! cycle order table
	    BYTE %FILL(6)			! (reserved)
	    INTEGER*4 INVCRGO			! # logical streams
	    INTEGER*4 IPVCRGO(MAXVCR)		! data stream MUX setting

C run time information

	    RECORD /STR_16/ OPRIDGO		! operator ID
	    BYTE %FILL(16)			! (reserved)
	    RECORD /STR_256/ CCRNMGO		! CCR file name
	    RECORD /STR_256/ PCRNMGO		! PCR file name
	    RECORD /STR_256/ TMLNMGO		! tone model lookup file name
	    BYTE %FILL(64)			! (reserved)

C stations information

	    RECORD /GRP_SO/ STATION(MAXSTN)	! stations information
	END STRUCTURE ! /GRP_GO/

C Mark-2 interface channel mapping table definition.
C
C This table gives the correspondence between block-2 station
C	channel streams and mark-2 tape input streams.
C
C VCRID is the logical VCR number for the specified channel in each
C	block-2 station.  An entry of zero indicates that the channel
C	is not mapped.
C Note that some VCRs are mapped to multiple channels within a station.
C
C	STRUCTURE /ICM_IC/	! interface channel mapping table
CC table identification
C	INTEGER*2 GROUPIC		! process group #
C	INTEGER*2 CLASSIC		! table type #
C	BYTE %FILL(4)			! (reserved)
C	RECORD /INT_8/ ISYSTIC		! table creation date
C	BYTE %FILL(16)			! (reserved)
C
C	INTEGER*4 NSTNSIC		! # of block-2 stations in table
C	INTEGER*4 NMODEIC		! # of MUX modes in table
C	BYTE %FILL(24)			! (reserved)
C	INTEGER*4 VCRIDIC(MAXCHN,MAXICM,MAXSTN)
C	END STRUCTURE ! /ICM_IC/

C Operator comment/status table definition.

	STRUCTURE /OPR_OP/	! operator comments table

C Table identification

	INTEGER*2 GROUPOP		! process group #
	INTEGER*2 CLASSOP		! table type #
	BYTE %FILL(4)			! (reserved)
	RECORD /INT_8/ OSYSTOP		! system time (VAX format)
	REAL*8 RTIMEOP			! reference time (bits)
	BYTE %FILL(8)			! (reserved)

	RECORD /STR_16/ OPNAMOP		! operator name
	BYTE %FILL(16)			! (reserved)
	RECORD /STR_256/ OPCOMOP	! operator comment
	END STRUCTURE ! /OPR_OP/

C Tone model table definition.

	STRUCTURE /TON_TO/	! tone model station table
C tone (additive) model information
	REAL*8 TCPH0TO			! cal phase (cyc)
	REAL*8 TCPD0TO			! tone cal frequency (cyc)
	BYTE %FILL(16)			! (reserved)
C tone (extraction) model information
	REAL*8 TFREQTO			! tone frequency (Hz)
	REAL*8 TFQOFTO			! tone frequency offset (Hz)
	BYTE %FILL(16)			! (reserved)
	REAL*8 TPOLYTO(MAXTMC)		! tone model polynomial
	BYTE %FILL(24)			! (reserved)
	END STRUCTURE ! /TON_TO/

	STRUCTURE /TON_TM/	! tone model table
C table identification
	INTEGER*2 GROUPTM		! process group #
	INTEGER*2 CLASSTM		! table type #
	INTEGER*4 TMODNTM		! tone model #
	RECORD /INT_8/ TSYSTTM		! model synchronization time
	BYTE %FILL(16)			! (reserved)

	BYTE %FILL(32)			! (reserved)
C stations information
	RECORD /TON_TO/ STATION(MAXSTN)	! station information
	END STRUCTURE ! /TON_TM/

C Tone model lookup table (header) definition.

	INTEGER MAXTL4

	PARAMETER (MAXTL4 = MAXSTN*MAXTON)

	STRUCTURE /TML_TM/	! tone model lookup table (header only)
C table identification
	INTEGER*2 GROUPTM		! process group #
	INTEGER*2 CLASSTM		! table type #
	BYTE %FILL(4)			! (reserved)
	RECORD /INT_8/ TSYSTTM		! table creation time (VAX)
	REAL*8 TMBEGTM			! table start time (bits)
	REAL*8 TMENDTM			! table stop time (bits)
C header configuration information
	INTEGER*4 HDLENTM		! actual header length (bytes)
	BYTE %FILL(12)			! (reserved)
	INTEGER*4 NMSTNTM		! # stations mapped
	INTEGER*4 NTMDLTM		! # tone models mapped
	INTEGER*4 NTMENTM		! # tone model entries per interval
	INTEGER*4 TMUPDTM		! tone model update interval (TRIs)
C table contents identification
	RECORD /STR_16/ ENAMETM		! experiment name
	INTEGER*4 SCNUMTM		! scan #
	BYTE %FILL(12)			! (reserved)
	BYTE %FILL(32)			! (reserved)
C tone model entry mapping information
	INTEGER*4 MODELTM(MAXTL4)	! tone entry map (NMSTN,NTMDL)
	END STRUCTURE ! /TML_TM/

C Delay model table definition.

	STRUCTURE /DLY_DO/	! delay model station table
	RECORD /STR_16/ STNAMDO		! station name
	BYTE %FILL(16)			! (reserved)
C geometric model information
	REAL*8 RSPINDO			! equatorial component (meters)
	REAL*8 ZPROJDO			! height above equator (meters)
	REAL*8 WLONGDO			! (west) longitude (rad)
	REAL*8 ZTROPDO			! zenith troposphere
C antenna axis offsets (if AXTYPDO=0 then no correction applied)
	INTEGER*4 AXTYPDO		! antenna type (0-5)
	BYTE %FILL(4)			! (reserved)
	REAL*8 AXOF1DO			! (meters)
	REAL*8 AXOF2DO			! (meters)
	BYTE %FILL(8)			! (reserved)
C timing model information
	REAL*8 CLCORDO			! station clock correction (usec)
	REAL*8 CLRATDO			! clock rate (usec/hour)
	REAL*8 PCCORDO			! tone clock correction (usec)
	REAL*8 PCRATDO			! tone clock rate (usec)
C sequencing information
	REAL*8 STARTDO			! scan start time (bits)
	REAL*8 STOPTDO			! scan stop time (bits)
	BYTE %FILL(16)			! (reserved)
	END STRUCTURE ! /DLY_DO/

	STRUCTURE /DLY_DM/	! delay model table
C table identification
	INTEGER*2 GROUPDM		! process group #
	INTEGER*2 CLASSDM		! table type #
	INTEGER*4 DMODNDM		! tone model #
	RECORD /INT_8/ DSYSTDM		! model synchronization time
	BYTE %FILL(16)			! (reserved)
C data identification
	RECORD /STR_16/ EXNAMDM		! experiment name
	RECORD /STR_16/ SNAMEDM		! source name
	INTEGER*4 SCANNDM		! scan #
	BYTE %FILL(28)			! (reserved)
C orientation model information
	LOGICAL*4 RSFLGDM		! retarded station model?
	BYTE %FILL(12)			! (reserved)
	REAL*8 XPOLEDM			! X-axis polar motion (asec)
	REAL*8 YPOLEDM			! Y-axis polar motion (asec)
C source model information
	INTEGER*4 PDATEDM		! position reference date (years)
	BYTE %FILL(4)			! (reserved)
	REAL*8 RTAS0DM			! right ascension (rad)
	REAL*8 DECL0DM			! declination (rad)
	BYTE %FILL(8)			! (reserved)
	REAL*8 RTASCDM			! right ascension of date (rad)
	REAL*8 DECLNDM			! declination of date (rad)
	BYTE %FILL(16)			! (reserved)
C timing model information
	REAL*8 EXTIMDM			! experiment reference time (bits)
	REAL*8 UTMTCDM			! UT1-UTC (sec)
	REAL*8 EQOEQDM			! equation of equinox (sec)
	BYTE %FILL(8)			! (reserved)
C sequencing information
	INTEGER*4 ODATEDM		! observation date (years)
	BYTE %FILL(12)			! (reserved)
	REAL*8 BEGTMDM			! earliest start time (bits)
	REAL*8 ENDTMDM			! latest stop time (bits)
	BYTE %FILL(32)			! (reserved)
	RECORD /DLY_DO/ STATION(MAXSTN)
	END STRUCTURE ! /DLY_DM/

C Phase model table definition.

	STRUCTURE /PHS_PO/	! phase model station table
	REAL*8 FQSLOPO			! station LO frequency (MHz)
	REAL*8 FQOFFPO			! LO offset for model (Hz)
	REAL*8 FQPSRPO			! Pulsar reptition frequency (Hz) (DNF)
	BYTE %FILL(8)			! (reserved)

	REAL*8 PHREPPO			! phase replacement (cyc)
	REAL*8 PDREPPO			! freq. replacement (cyc/bit)
	REAL*8 PDDREPO			! freq' replacement (cyc/bit/FRI)
	REAL*8 PDDDRPO			! freq'' replacement (cyc/bit/FRI/FRI)
	REAL*8 DPREPPO			! delta phase replacement (cyc/bit)
	BYTE %FILL(24)			! (reserved)
	END STRUCTURE ! /PHS_PO/

	STRUCTURE /PHS_PM/	! phase model table
C table identification
	INTEGER*2 GROUPPM		! process group #
	INTEGER*2 CLASSPM		! table type #
	INTEGER*4 PMODNPM		! tone model #
	RECORD /INT_8/ PSYSTPM		! model synchronization time
	BYTE %FILL(16)			! (reserved)
C global model information
	REAL*8 FQCENPM			! nominal centroid freqency (MHz)
	REAL*8 FQNLOPM			! station LO frequency (MHz)
	REAL*8 FQDELPM			! shift delta frequency (+/-MHz)
	BYTE %FILL(8)			! (reserved)
	LOGICAL*4 FQSBFPM		! upper sideband?
	BYTE %FILL(28)			! (reserved)
	RECORD /PHS_PO/ STATION(MAXSTN)
	END STRUCTURE ! /PHS_PM/

C Tensor configuration table definition.

	INTEGER MAXTCB, MAXTCW

	PARAMETER (MAXTCB = MAXBND*4+MAXTCH*3)
	PARAMETER (MAXTCW = MAXWND*7)

	STRUCTURE /TNS_TC/	! tensor configuration table
C table identification
	INTEGER*2 GROUPTC		! process group #
	INTEGER*2 CLASSTC		! table type #
	BYTE %FILL(4)
	RECORD /INT_8/ TSYSTTC		! model sync time
	INTEGER*4 STNN1TC
	INTEGER*4 STNN2TC
C table configuration
	INTEGER*4 MXCHNTC		! maximum # channels allowed
	INTEGER*4 MXBNDTC		! maximum # bands allowed
	INTEGER*4 MXWNDTC		! maximum # windows allowed
C global parameters
	INTEGER*4 INVLDTC		! invalid threshold (bits/2)
	LOGICAL*4 NRMFLTC		! normalize data?
	INTEGER*4 LBSFTTC		! internal left shift count
	LOGICAL*4 ACCFLTC		! high precision normalization?
	LOGICAL*4 VAXFPTC		! VAX F-format floating point output?
	LOGICAL*4 SCNERTC		! single channel rejection mode?
C data configuration
	INTEGER*4 NOCOITC		! # COI's per boxcar accumulation
	INTEGER*4 DFACTTC		! digital filter factor
	INTEGER*4 BITIMTC		! correlator acc. interval (bits)
	LOGICAL*4 GLMAXTC		! locate global maxima?
	BYTE %FILL(4)
C band configuration
	LOGICAL*4 FQFLGTC		! frequency-delay FFT?
	LOGICAL*4 FQDIRTC		! positive FFT coefficients?
	INTEGER*4 NTCHNTC		! # logical channels
	INTEGER*4 NPTCHTC		! # lags per channel
	INTEGER*4 FQBANTC		! # frequency bands
	INTEGER*4 FQPARTC(MAXTCB)	! band parameters
C band information
C	INTEGER*4 FQBINTC		! FFT size (bins)
C	INTEGER*4 FQCHNTC		! # channels in band
C	RECORD /INT_8/ FQREFTC		! band reference frequency
C channel information
C	INTEGER*4 CHNUMTC		! physical channel #
C	INTEGER*4 FQOSETC		! channel offset in FFT
C	INTEGER*4 PNOSETC		! first bin in channel
C accumulation configuration
	LOGICAL*4 TMFLGTC		! time-rate FFT?
	LOGICAL*4 TMDIRTC		! positive FFT coefficients?
	INTEGER*4 TMFFTTC		! time FFT size (bins)
	INTEGER*4 TMACCTC		! # averages to accumulate (bins)
C output window configuration
	LOGICAL*4 OWSTATC		! output window changed?
	LOGICAL*4 OWMAXTC		! locate window maxima?
	INTEGER*4 NOWINTC		! # output windows
	INTEGER*4 OWPARTC(MAXTCW)
C window information
C	INTEGER*4 OWBANTC		! band # for window
C frequency-delay window parameters
C	INTEGER*4 OWDOFTC		! window starting offset (bins)
C	INTEGER*4 OWDSZTC		! window size (bins)
C	INTEGER*4 OWDTSTC		! peak location tracking size (bins)
C time-rate window parameters
C	INTEGER*4 OWFOFTC		! window starting offset (bins)
C	INTEGER*4 OWFSZTC		! window size (bins)
C	INTEGER*4 OWFTSTC		! peak location tracking size (bins)
	END STRUCTURE ! /TNS_TC/

C Window configuration table definition.  Version 28NOV88 13:00 (DNF)

C   The table gives the configuration of a single window.

C   Channel Information structure

	STRUCTURE /WND_CC/
	    INTEGER*4 LNCHNWC			! logical channel #
	    INTEGER*4 POFFTWC			! position offset in FFT (bins)
	    REAL*8 FQCHNWC			! channel reference frequency
	    INTEGER*4 NMCBNWC			! number of bins in channel
	    INTEGER*4 NMCLGWC			! number of lags in channel
	    INTEGER*4 PFBINWC			! position of first bin
	    BYTE %FILL(4)			! (reserved)
	END STRUCTURE ! /WND_CC/

C Window Configuration Table

	STRUCTURE /WND_WC/

C   Table identification

	    INTEGER*2 GROUPWC			! process group #
	    INTEGER*2 CLASSWC			! table type #
	    INTEGER*4 TWNDNWC			! window ID #
	    RECORD /INT_8/ TSYSTWC		! model sync time
	    BYTE %FILL(16)			! (reserved)

C   Index numbers

	    INTEGER*4 FVERNWC			! FORTH version #
	    INTEGER*4 WVERNWC			! WINDOW version #
	    INTEGER*4 LNST1WC			! station 1 #
	    INTEGER*4 LNST2WC			! station 2 #
	    INTEGER*4 LNBASWC			! baseline #
	    INTEGER*4 LNBNDWC			! band #
	    INTEGER*4 LNWNDWC			! window #
	    BYTE %FILL(4)			! (reserved)

C   Maximum numbers

	    INTEGER*4 MXTWDWC			! maximum # of windows total
	    INTEGER*4 MXBASWC			! maximum # of baselines
	    INTEGER*4 MXBNDWC			! maximum # of bands
	    INTEGER*4 MXWNDWC			! maximum # of windows per band
	    INTEGER*4 MXCHNWC			! maximum # of channels per band
	    INTEGER*4 MXDWSWC			! maximum # of delay window bins
	    INTEGER*4 MXRWSWC			! maximum # of drate window bins
	    INTEGER*4 MXDFTWC			! maximum # of delay bins in FFT
	    INTEGER*4 MXRFTWC			! maximum # of drate bins in FFT
	    INTEGER*4 MXBINWC			! maximum # of bins per channel
	    INTEGER*4 MXDMPWC			! maximum # of dumps per segment
	    BYTE %FILL(20)			! (reserved)

C   Actual numbers

	    INTEGER*4 NMTWDWC			! actual # of windows total
	    INTEGER*4 NMBASWC			! actual # of baselines
	    INTEGER*4 NMBNDWC			! actual # of bands
	    INTEGER*4 NMWNDWC			! actual # of windows this band
	    INTEGER*4 NMCHNWC			! actual # of channels this band
	    INTEGER*4 DWSIZWC			! actual # of delay window bins
	    INTEGER*4 RWSIZWC			! actual # of drate window bins
	    INTEGER*4 DFSIZWC			! actual # of delay bins in FFT
	    INTEGER*4 RFSIZWC			! actual # of drate bins in FFT
	    INTEGER*4 NMFRQWC			! actual # of spanned freqs
	    INTEGER*4 NMDMPWC			! actual # of spanned dumps
	    BYTE %FILL(20)			! (reserved)

C   Necessary information

	    REAL*8 FQBNDWC			! band reference frequency (MHz)
	    REAL*4 SZDBNWC			! size of delay bin (usecs)
	    REAL*4 SZRBNWC			! size of drate bin (psecs/sec)
	    LOGICAL*4 DRMAXWC			! searched for peak
	    BYTE %FILL(12)			! (reserved)

C   Additional information

	    REAL*8 SSTRTWC			! scan start time (bits)
	    REAL*8 SSTOPWC			! scan stop time (bits)
	    INTEGER*4 NMBITWC			! length of sample (bits)
	    REAL*4 NMSECWC			! length of sample (secs)
	    REAL*4 INVLDWC			! invalid threshold (fraction)
	    INTEGER*4 SZDSRWC			! delay srch window size (bins)
	    INTEGER*4 SZRSRWC			! drate srch window size (bins)
	    INTEGER*4 PSDSRWC			! delay srch window posn (bins)
	    INTEGER*4 PSRSRWC			! drate srch window posn (bins)
	    BYTE %FILL(20)			! (reserved)

C   Channel information

	    RECORD /WND_CC/ CHANNEL(MAXCHN)

C   End of table marker

	    INTEGER*4 ENDOFWC			! mark end of table for length

	END STRUCTURE ! /WND_WC/

C Tone model output table definition.

	STRUCTURE /TMO_TO/	! tone model output table entry
	REAL*8 TPREVTO
	REAL*8 TMID1TO
	REAL*8 TMID2TO
	REAL*8 TNEXTTO
	END STRUCTURE ! /TMO_TO/

	STRUCTURE /TMO_TM/	! tone model output table
C table identification
	INTEGER*2 GROUPTM		! process group #
	INTEGER*2 CLASSTM		! table type #
	BYTE %FILL(4)			! (reserved)
	RECORD /INT_8/ TSYSTTM		! model sync time
	REAL*8 RTIMETM			! model start time
	BYTE %FILL(8)			! (reserved)
C derived tone model information
	RECORD /TMO_TO/ MODELTM(MAXTME)
	END STRUCTURE ! /TMO_TM/

C Geometric model output table definition.

	STRUCTURE /GMO_GO/	! geometric model output table entry
C total delay models
	REAL*8 DPREVDO
	REAL*8 DMID1DO
	REAL*8 DMID2DO
	REAL*8 DNEXTDO
C geometry-only delay models
	REAL*8 GPREVDO
	REAL*8 GMID1DO
	REAL*8 GMID2DO
	REAL*8 GNEXTDO
C phase models
	REAL*8 PPREVPO
	REAL*8 PMID1PO
	REAL*8 PMID2PO
	REAL*8 PNEXTPO
	END STRUCTURE ! /GMO_GO/

	STRUCTURE /GMO_GM/	! geometric model output table
C table identification
	INTEGER*2 GROUPGM		! process group #
	INTEGER*2 CLASSGM		! table type #
	BYTE %FILL(4)
	RECORD /INT_8/ GSYSTGM		! model sync time
	REAL*8 RTIMEGM			! model start time
	BYTE %FILL(8)			! (reserved)
C derived geometric model information
	RECORD /GMO_GO/ MODELGM(MAXGME)
	END STRUCTURE ! /GMO_GM/

C Correlator data output table definition.
C
C COFFLCD is a bit mask used to determine if a particular baseline's
C	data should be considered valid.  Bit 1 (numbering 0-31)
C	corresponds to baseline 1.  COFFLCD is set whenever correlation
C	has not been performed; this could be because a tape is out-
C	of-sync (indicated by NSYNCGO), or because no valid models
C	have been sent to the hardware.

	INTEGER*4 MAXSS4, MAXCC4, MAXCN4, MAXCD4, MAXCR4
	INTEGER*4 MAXSS2

	PARAMETER (MAXSS4 = (2+MAXCHN+2))	! station status area
	PARAMETER (MAXCC4 = ((MAXCHN+4)*8*2))	! xc coefficients
 	PARAMETER (MAXCN4 = (MAXCHN*3))		! xc sums
	PARAMETER (MAXCD4 = (MAXCC4+MAXCN4))	! xc data for one baseline
	PARAMETER (MAXCR4 = (MAXSTN*MAXSS4)+(MAXBAS*MAXCD4))
						! maximum total size
	PARAMETER (MAXSS2 = (2*MAXSS4))

	STRUCTURE /CDO_SO/	! correlator data output station table
	INTEGER*4 NSYNCSO		! station out-of-sync count (TRIs)
	BYTE %FILL(4)			! (reserved)
	INTEGER*4 ERRORSO(MAXCHN)	! frame error counts
	BYTE %FILL(8)			! (reserved)
	END STRUCTURE ! /CDO_SO/

	STRUCTURE /CDO_CO/	! correlator data output count table
	INTEGER*4 COUNTCO(MAXCC4)
	INTEGER*4 CCSUMCO(MAXCN4)
	END STRUCTURE ! /CDO_CO/

	STRUCTURE /CDO_CD/	! correlator data output table
C table identification
	INTEGER*2 GROUPCD		! process group #
	INTEGER*2 CLASSCD		! table type #
	INTEGER*4 COFFLCD		! baseline off flags (bit mask)
	BYTE %FILL(8)			! (reserved)
	REAL*8 RTIMECD			! data reference time
	BYTE %FILL(8)			! (reserved)
C cross correlation status and data fields
	UNION
	MAP				! station status area
	RECORD /CDO_SO/ STATION(MAXSTN)
	END MAP
	MAP				! correlation counts and sums
	INTEGER*4 COUNTCD(MAXCR4)
	END MAP
	END UNION
	END STRUCTURE ! /CDO_CD/

C Correlator status table definition.

	STRUCTURE /CSO_CS/	! correlator status table
C table identification
	INTEGER*2 GROUPCS		! process group #
	INTEGER*2 CLASSCS		! table type #
	BYTE %FILL(12)			! (reserved)
	REAL*8 RTIMECS			! status reference time
	BYTE %FILL(8)			! (reserved)
C status information
	INTEGER*2 CSTATCS(16)		! correlator status
C serial numbers
	BYTE SERIDCS(64)		! board identification #s
C error information
	INTEGER*2 ERRORCS(640)		! error FIFOs as dumped
	END STRUCTURE ! /CSO_CS/

C Station data table definition.
C
C Include room for related information in dimension definitions,
C	to simplify correct calculations of locations.

	INTEGER*4 MAXSC4, MAXSN4, MAXSP4, MAXSD4, MAXSR4
	INTEGER*4 MAXSC2, MAXSN2, MAXSP2, MAXSD2, MAXSR2

	PARAMETER (MAXSC4 = (MAXTEX*MAXCHN*2))		! accumulations
	PARAMETER (MAXSN4 = (MAXTEX*MAXCHN+1))		! normalization sums
	PARAMETER (MAXSP4 = (MAXTEX*MAXCHN/2+2))	! "snapshot" phases
	PARAMETER (MAXSD4 = (MAXSC4+MAXSN4+MAXSP4))	! data for one station
	PARAMETER (MAXSR4 = (MAXSTN*(MAXSS4+MAXSC4+MAXSN4+MAXSP4)))
							! total size
	PARAMETER (MAXSC2 = (2*MAXSC4))
	PARAMETER (MAXSN2 = (2*MAXSN4))
	PARAMETER (MAXSP2 = (2*MAXSP4))
	PARAMETER (MAXSD2 = (2*MAXSD4))
	PARAMETER (MAXSR2 = (2*MAXSR4))

	STRUCTURE /SDO_SO/	! station data output station table
	INTEGER*4 NSYNCSO		! station out-of-sync count (TRIs)
	BYTE %FILL(4)			! (reserved)
	INTEGER*4 ERRORSO(MAXCHN)	! frame error counts
	BYTE %FILL(8)			! (reserved)
	END STRUCTURE ! /SDO_SO/

	STRUCTURE /SDO_DO/	! station data output count table
C Tone extractor accumulators
	INTEGER*4 COUNTDO(MAXSC4)		! correlation counts
	INTEGER*4 INVLDDO(MAXSN4-1)		! invalid counts
	BYTE %FILL(2)				! (reserved)
	INTEGER*2 PACRCDO			! phase accumulator CRC
C Tone extractor "snapshot" phases
	INTEGER*2 PHASEDO(MAXSP2-4)		! "snapshot" phases
	INTEGER*4 PDSTSDO			! phase proc. delay status
	BYTE %FILL(2)				! (reserved)
	INTEGER*2 PSCRCDO			! "snapshot" CRC
	END STRUCTURE ! /SDO_DO/

	STRUCTURE /SDO_SD/	! station data output table
C table identification
	INTEGER*2 GROUPSD		! process group #
	INTEGER*2 CLASSSD		! table type #
	INTEGER*4 SOFFLSD		! station off flags (bit mask)
	BYTE %FILL(8)			! (reserved)
	REAL*8 RTIMESD			! data reference time
	BYTE %FILL(8)			! (reserved)

	UNION
	MAP				! station status area
	RECORD /SDO_SO/ STATION(MAXSTN)
	END MAP
	MAP
	INTEGER*4 COUNTSD(MAXSR4)	! correlation counts and sums
	END MAP
	MAP
	INTEGER*2 PHASESD(MAXSR2)	! "snapshot" phases
	END MAP
	END UNION
	END STRUCTURE ! /SDO_SD/

C Station status table definition.

C This table contains error and configuration information for
C	the station section of the block-2 processor.

	STRUCTURE /SSO_SS/	! station status table
C table identification
	INTEGER*2 GROUPSS		! process group #
	INTEGER*2 CLASSSS		! table type #
	BYTE %FILL(12)			! (reserved)
	REAL*8 RTIMESS			! status reference time
	BYTE %FILL(8)			! (reserved)
C status information
	INTEGER*2 STERRSS(324*MAXSTN)	! status information
	END STRUCTURE ! /SSO_SS/

C Mark-2 interface ancillary information table definition.
C
C ITIME is the reference time for the record derived from the
C	correlator master clock.
C VTIME is a 6 character variable containing ASCII format HHMMSS
C	from the interface.
C VSTAT is a 4 byte variable, with each byte containing a constant
C	3 bit field (set to 010) and a variable 5 bit field.
C	Bits 0-4 in bytes 0-2 and bit 0 in byte 3 indicate whether
C	logical VCRs 1-16 are on and are decoding valid times.
C	Bits 1-3 of byte 3 indicate valid VTIME, all recorders on,
C	and run mode, respectively.
C VDATA contains a set of 4 byte fields from the error summary
C	in the interface subsystem and/or from the ancillary data
C	on each mark-2 tape.  If RERRS is non-zero, then the first
C	INVCR entries of VDATA contain the frame errors from each
C	logical VCR.  Following those fields, if RANCV is non-zero
C	the next INVCR entries of VDATA contain ancillary data placed
C	on the tapes at record time; if system temperatures have been
C	recorded, then these fields are ASCII character strings.

	INTEGER MAXIAF

	PARAMETER (MAXIAF = 2*MAXVCR)

	STRUCTURE /IAO_IA/	! mark-2 interface ancillary output table
C table identification
	INTEGER*2 GROUPIA		! process group #
	INTEGER*2 CLASSIA		! table type #
	BYTE %FILL(12)			! (reserved)
	REAL*8 RTIMEIA			! data reference time
	BYTE %FILL(8)			! (reserved)
C ancillary information
	BYTE %FILL(20)			! (reserved)
	BYTE %FILL(2)			! (reserved)
	BYTE VTIMEIA(6)			! interface VCR clock (HHMMSS)
	INTEGER*4 VSTATIA		! status word
	BYTE VDATAIA(4*MAXIAF)		! ancillary data fields
	END STRUCTURE ! /IAO_IA/

C Tensor data output table definition.

	STRUCTURE /TDO_TD/
C table identification
	INTEGER*2 GROUPTD		! process group #
	INTEGER*2 CLASSTD		! table type #
	INTEGER*2 TOTBUF
	INTEGER*4 TOTBYT
	INTEGER*2 BUFNUM
	INTEGER*4 BUFBYT
	INTEGER*2 STA1
	INTEGER*2 STA2
	RECORD /INT_8/ SYSTIM
	RECORD /INT_8/ START
	RECORD /INT_8/ STOP
C table status
	INTEGER*2 ERRWRD
	LOGICAL*1 IMHAPY
	BYTE	%FILL(5)
C baseline header
	INTEGER*2 HRDVER
	RECORD /STR_8/ HRDDAY
	INTEGER*2 SFTVER
	RECORD /STR_8/ SFTDAY
	INTEGER*2 OUTVER
	RECORD /STR_8/ OUTDAY
	RECORD /INT_8/ PRMTIM
C configuration information
	LOGICAL*1 NEWPRM
	LOGICAL*1 FRQFFT
	LOGICAL*1 FFFTDR
	LOGICAL*1 TIMFFT
	INTEGER*2 NUMPTS
	INTEGER*2 NUMTIM
	LOGICAL*1 TFFTDR
	LOGICAL*1 NORMAL
	LOGICAL*1 FFLOAT
	LOGICAL*1 ACCFLG
	LOGICAL*1 CERFLG
	BYTE %FILL(1)
	INTEGER*2 BINPNT
	INTEGER*2 NOCOI
	INTEGER*2 DFACT
	INTEGER*4 BITTIM
	INTEGER*4 INVLVL
	LOGICAL*1 MAXFLG
	LOGICAL*1 WMAXFL
	INTEGER*2 PTSCHN
C band information
	INTEGER*2 NUMBND
	END STRUCTURE ! /TDO_TD/

C Window data output table definition.  Version 29NOV88 14:30 (DNF)

	INTEGER*4 MAXDAT,MAXWSZ,MAXWLN

	PARAMETER (MAXDAT = 2048)
	PARAMETER (MAXWSZ = 10)
	PARAMETER (MAXWLN = MAXWSZ*MAXWSZ)

	STRUCTURE /WDO_WD/

C   Table identification

	    INTEGER*2 GROUPWD			! process group #
	    INTEGER*2 CLASSWD			! table type #
	    INTEGER*4 TWNDNWD			! window number identifier
	    RECORD /INT_8/ TSYSTWD		! system time (serial #)
	    REAL*8 NSTRTWD			! nominal segment start time
	    REAL*8 NSTOPWD			! nominal segment stop time

C   Index numbers

	    INTEGER*4 FVERNWD			! FORTH version #
	    INTEGER*4 WVERNWD			! WINDOW version #
	    INTEGER*4 LNST1WD			! station 1 #
	    INTEGER*4 LNST2WD			! station 2 #
	    INTEGER*4 LNBASWD			! baseline #
	    INTEGER*4 LNBNDWD			! band #
	    INTEGER*4 LNWNDWD			! window #
	    BYTE %FILL(4)			! reserved

C   Necessary information

	    REAL*8 TSTRTWD			! actual segment start time
	    REAL*8 TSTOPWD			! actual segment stop time
	    INTEGER*4 DLYSZWD			! size of the delay FFT
	    INTEGER*4 DLRSZWD			! size of the delay rate FFT
	    BYTE %FILL(8)			! (reserved)
	    REAL*8 NOMFQWD			! nominal frequency
	    REAL*8 AVGFQWD			! average frequency
	    REAL*8 DLYFQWD			! delay frequency
	    REAL*8 FAZFQWD			! phase delay frequency
	    REAL*8 DLRFQWD			! delay rate frequency
	    REAL*8 AVGTMWD			! average time
	    BYTE %FILL(16)			! (reserved)

C   Additional information

	    REAL*4 AMPMXWD			! amplitude of max in FFT
	    REAL*4 FAZMXWD			! phase of max in FFT (deg)
	    INTEGER*4 DLYMXWD			! delay of max in FFT (bins)
	    INTEGER*4 DLRMXWD			! drate of max in FFT (bins)
	    REAL*4 AMPESWD			! amplitude estimate
	    REAL*4 FAZESWD			! phase estimate (deg)
	    REAL*4 DLYESWD			! delay estimate (bins)
	    REAL*4 DLRESWD			! delay rate estimate (bins)
	    BYTE %FILL(32)			! (reserved)

C   Data

	    INTEGER*4 NDBINWD			! # of delay bins in window
	    INTEGER*4 NRBINWD			! # of drate bins in window
	    INTEGER*4 NMWDPWD			! window delay position (bins)
	    INTEGER*4 NMWRPWD			! window drate position (bins)
	    BYTE %FILL(16)			! (reserved)
	    COMPLEX*8 DRWNDWD(MAXWLN)		! actual data

C   Data flags

	    INTEGER*4 NMCHNWD			! # of channels
	    INTEGER*4 NMDMPWD			! # of time dumps
	    UNION
		MAP
		    REAL*4 NSUMSWD(MAXCHN+2*MAXDAT)
		ENDMAP
		MAP
		    INTEGER*4 FLAGSWD(MAXCHN+2*MAXDAT)
		ENDMAP
	    END UNION
	    					! NMCHNTD norm sums over dumps
	    					! NMDMPTD norm sums over chans
						! NMDMPTD bit flags

c   End of table marker

	    INTEGER*4 ENDOFWD			! Mark end of table for length

	END STRUCTURE ! /WDO_WD/

C Digital filter output table definition.
C
C	PARAMETER (MAXFD4 = ?)
C
C	STRUCTURE /FDO_FD/
CC table identification
C	INTEGER*2 GROUPFD		! process group #
C	INTEGER*2 CLASSFD		! table type #
C	INTEGER*4 SEQNMFD		! dump sequence #
C	%FILL(8)			! (reserved)
C	REAL*8 FTIMEFD			! reference time
C	%FILL(8)			! (reserved)
C
C	INTEGER*4 FDATAFD(MAXFD4)	! filtered bit stream
C	END STRUCTURE ! /FDO_FD/

C Aggregate PCR table definition.
C
C This table can be used for program I/O.

	STRUCTURE /PCR_PC/	! I/O table
	UNION
C table identification (required fields)
	MAP			! generic table identification
	INTEGER*2 GROUPPC		! group #
	INTEGER*2 CLASSPC		! table type #
C optional fields
	INTEGER*4 MODELPC		! model #
	RECORD /INT_8/ SYSTMPC		! synchronization time
	REAL*8 RTIMEPC			! reference time
	BYTE %FILL(8)			! (reserved)
	END MAP
	MAP
	RECORD /TID_PC/ TIDLBL	! generic table identification
	END MAP
	MAP
	RECORD /PRC_GO/ PRCTBL	! process table
	END MAP
	MAP
	RECORD /GRP_GO/ GRPTBL	! group table
	END MAP
	MAP
	RECORD /OPR_OP/ OPRTBL	! operator table
	END MAP
C	MAP
C	RECORD /ICM_IC/ ICMTBL	! mark-2 interface channel map
C	END MAP
	MAP
	RECORD /TON_TM/ TONTBL	! tone model table
	END MAP
C	MAP
C	RECORD /TML_TM/ TMLTBL	! tone model lookup table (header)
C	END MAP
	MAP
	RECORD /TMO_TM/ TMOTBL	! tone model output
	END MAP
	MAP
	RECORD /DLY_DM/ DLYTBL	! delay model table
	END MAP
	MAP
	RECORD /GMO_GM/ GMOTBL	! geometric model output
	END MAP
	MAP
	RECORD /PHS_PM/ PHSTBL	! phase model table
	END MAP
	MAP
	RECORD /TNS_TC/ TNSTBL	! tensor configuration table
	END MAP
	MAP
	RECORD /WND_WC/ WNDTBL	! window configuration table
	END MAP
C	MAP
C	RECORD /TCO_TC/ TCOTBL	! tensor current configuration output
C	END MAP
	MAP
	RECORD /CDO_CD/ CDOTBL	! raw xc data output
	END MAP
	MAP
	RECORD /CSO_CS/ CSOTBL	! xc status output
	END MAP
	MAP
	RECORD /SDO_SD/ SDOTBL	! station phase cal data output
	END MAP
	MAP
	RECORD /SSO_SS/ SSOTBL	! station status output
	END MAP
	MAP
	RECORD /IAO_IA/ IAOTBL	! mark-2 interface ancillary data output
	END MAP
	MAP
	RECORD /TDO_TD/ TDOTBL	! tensor data output
	END MAP
	MAP
	RECORD /WDO_WD/ WDOTBL	! window data output
	END MAP
C	MAP
C	RECORD /FDO_TD/ FDOTBL	! digital filter data output
C	END MAP
	END UNION
	INTEGER	BEYOND
	END STRUCTURE ! /PCR_PC/
