      PROGRAM QFIT
C-----------------------------------------------------------------------
C Written by Richard S. Simon           June 1981
C
C This program will determine the goodness-of-fit of a model to
C VLB data.  The agreement factor that the program produces is
C a reduced Chi-Square estimate. The smaller the agreement factor,
C the better the fit;  an agreement factor of 1 implies that the
C model fits the data within the errors.
C
C This program carefully takes into account which closure phase
C triangles are independent when computing the A.F. !
C
C Version 1.0: 1981 Jun 30
C Version 2.0: 1983 Jul 20 : extend to include MERGE format Version 2
C                            files (D. Meier)
C              1983 Aug  3 : added W to version 2 files
C              1983 Oct 17 : corrected bug in QMODAP when QQ=0.0;
C                            don't bother doing VIS calc if ERR <= 0
C Version 2.1: 1984 Mar 9  : Fix bugs in visibility calc when
C                            TYPE.GT.1:  QGETMF, QMODAP
C Version 3.0: 1988 Apr 19 : changes for Convex [TJP].
C Version 3.1: 1988 Jul 14 : correct dimension error for array CSUM.
C Version 3.2: 1989 Mar  6 : correct bug: compute all visibilities in
C                            case closure phases need them [TJP].
C Version 3.3: 1989 Jun 21 : correct bug in TRIANGLE. Program computed
C                            closure phase incorrectly if baselines not
C                            in canonical order. New algorithm for
C                            finding number of independent closure 
C                            phases [TJP].
C Version 3.4: 1990 Nov  5 : another algorithm for finding number of
C                            independent closure phases [TJP].
C Version 3.5: 1994 Mar 17 : increase to 20 stations; standardize for
C                            portability [TJP].
C-----------------------------------------------------------------------
C     IMPLICIT NONE
      CHARACTER*(*) PROGNM, VERSN
      PARAMETER (PROGNM='QFIT')
      PARAMETER (VERSN='3.5 - 1994 Mar 17')
C
C       Common blocks, etc. are contained in a separate file:
      INCLUDE 'qfit.inc'
C
      LOGICAL NEWMOD, QUERY
      CHARACTER*80 HISTRY
      INTEGER L, IER
      INTEGER  GETIN, LEN1, VLBOPE
      EXTERNAL GETIN, LEN1, VLBOPE
C
      CALL PUTOUT (PROGNM//' finds the agreement factors for the '//
     1                   'fit of a model to VLB data.')
      CALL PUTOUT ('(Version '//VERSN//')')
      CALL PUTOUT (' ')
C
C Get the file names for data, model, and output:
C
   10 IER = GETIN (INDSN, 'Input data file name: ', L)
      IF (IER.NE.1) STOP
      IER = VLBOPE(INDAT, INDSN, 'MERGE', 'OLD', INDSN)
      IF (IER.NE.1) GOTO 10
C
   20 IER = GETIN (MODDSN, 'Input model file name: ', L)
      IF (IER.NE.1) STOP
      IER = VLBOPE(INMOD, MODDSN, 'MODEL', 'OLD', MODDSN)
      IF (IER.NE.1) GOTO 20
C
   30 IER = GETIN (OUTDSN,
     :  'Listing file name [default: standard output]: ', L)
      IF (IER.NE.1) STOP
      IF (OUTDSN .EQ. ' ') THEN
          OUTPR = OUTC
      ELSE
          OUTPR = PR
          IER = VLBOPE(OUTPR, OUTDSN, 'TEXT', 'NEW', OUTDSN)
          IF (IER.NE.1) GOTO 30
      END IF
      CALL MKHIST(PROGNM, VERSN, HISTRY)
      WRITE (OUTPR,'(A)') HISTRY(1:LEN1(HISTRY))
C
C Get the input data and compute the arrays UDATA and VDATA:
C
      CALL QGETDF
C
C Determine which triangles are independent for each data record:
C
      CALL TRIANG
C
C Compute the non-zero data points and the weights for the data.
C
      CALL WTDATA
C
C Line to come back to when trying a new model:
C
      NEWMOD = .FALSE.
  100 IF (NEWMOD) THEN
          CALL PUTOUT(' ')
  120     IER = GETIN(MODDSN, 'New input model file name: ', L)
          IF (IER.NE.1) STOP
          CLOSE (UNIT=INMOD)
          IER = VLBOPE(INMOD, MODDSN, 'MODEL', 'OLD', MODDSN)
          IF (IER.NE.1) GOTO 120
      END IF
C
C Read in the model file:
C
      CALL QGETMF
C
C Compute the model amplitudes and closure phases:
C
      CALL QMODAP
C
C Compute the agreement factors:
C
      CALL AGRFAC
C
C Write the agreement factors to the output unit:
C
      CALL AFOUT
      IF (OUTDSN.NE.' ') CALL PUTOUT('Agreement factors to file '//
     1                OUTDSN(1:LEN1(OUTDSN)))
C
C Let the user try again, if desired:
C
      CALL PUTOUT(' ')
      IF (QUERY('Try again with a different model (Y or N)?  ')) THEN
          NEWMOD = .TRUE.
          GOTO 100
      END IF
C
      END

      SUBROUTINE AFOUT
C-----------------------------------------------------------------------
C       By Richard S. Simon
C
C       This prints the results from program QFIT to unit OUTPR
C-----------------------------------------------------------------------
      INCLUDE 'qfit.inc'
      REAL STAR
      PARAMETER (STAR='*   ')
      INTEGER  I, I1, I2, IFORM
      INTEGER  LEN1
      EXTERNAL LEN1
C
      WRITE (OUTPR,'()')
      WRITE (OUTPR,1000) 'Input data file:   ',INDSN(1:LEN1(INDSN))
      WRITE (OUTPR,1000) 'Input model file:  ',MODDSN(1:LEN1(MODDSN))
      WRITE (OUTPR,'()')
      WRITE (OUTPR,1000) '  Baseline       Agreement          ',
     1'        Triangle       Agreement'
      WRITE (OUTPR,1000) '                    Factor          ',
     1'                          Factor'
C
      I1 = MIN(NUMBAS,NUMTRI)
      I2 = I1 + 1
      DO 100 I=1,I1
          IF (VISAF(I) .GT. 0. .AND. CLOAF(I) .GT. 0.) THEN
            ASSIGN 1001 TO IFORM
          ELSEIF (VISAF(I) .LT. 0. .AND. CLOAF(I) .LT. 0.) THEN
            ASSIGN 1004 TO IFORM
            VISAF(I) = STAR
            CLOAF(I) = STAR
          ELSEIF (VISAF(I) .LT. 0.) THEN
            ASSIGN 1002 TO IFORM
            VISAF(I) = STAR
          ELSE
            ASSIGN 1003 TO IFORM
            CLOAF(I) = STAR
          ENDIF
          WRITE (OUTPR,IFORM) STNAME(NAVIS(I)),STNAME(NBVIS(I)),
     1    VISAF(I), STNAME(NACLO(I)), STNAME(NBCLO(I)),
     1    STNAME(NCCLO(I)), CLOAF(I)
  100 CONTINUE
C
      IF (NUMBAS .GT. NUMTRI) THEN
          DO 120 I = I2, NUMBAS
            IF (VISAF(I) .LT. 0.) THEN
              ASSIGN 1002 TO IFORM
              VISAF(I) = STAR
            ELSE
              ASSIGN 1001 TO IFORM
            ENDIF
            WRITE (OUTPR,IFORM) STNAME(NAVIS(I)),STNAME(NBVIS(I)),
     1      VISAF(I)
  120     CONTINUE
      ELSEIF (NUMTRI .GT. NUMBAS) THEN
          DO 140 I = I2, NUMTRI
C           If AF is <0, print a star instead of AF.
            IF (CLOAF(I) .GT. 0.) THEN
              ASSIGN 1005 TO IFORM
            ELSE
              ASSIGN 1006 TO IFORM
              CLOAF(I) = STAR
            ENDIF
            WRITE (OUTPR,IFORM) STNAME(NACLO(I)), STNAME(NBCLO(I)),
     1      STNAME(NCCLO(I)), CLOAF(I)
  140     CONTINUE
      ENDIF
      WRITE (OUTPR,1007) AMPLAF, PHASAF, TOTLAF
C
      RETURN
C
 1000 FORMAT (A,A,A,A)
 1001 FORMAT (' ',A4,'-',A4,6X,F10.3,12X,A4,'-',A4,'-',A4,6X,F10.3)
 1002 FORMAT (' ',A4,'-',A4,6X,5X,A4,13X,A4,'-',A4,'-',A4,6X,F10.3)
 1003 FORMAT (' ',A4,'-',A4,6X,F10.3,12X,A4,'-',A4,'-',A4,6X,5X,A4)
 1004 FORMAT (' ',A4,'-',A4,6X,5X,A4,13X,A4,'-',A4,'-',A4,6X,5X,A4)
 1005 FORMAT (38X,A4,'-',A4,'-',A4,6X,F10.3)
 1006 FORMAT (38X,A4,'-',A4,'-',A4,6X,5X,A4)
 1007 FORMAT (/'Total amplitude agreement factor:',F10.3
     1          /'Total phase clo agreement factor:',F10.3,
     2          /'Total agreement factor:          ',F10.3)
      END

      SUBROUTINE AGRFAC
C-----------------------------------------------------------------------
C       This subroutine finds the agreement factors for the fit of a
C       modle to VLB data.
C-----------------------------------------------------------------------
      INCLUDE 'qfit.inc'
      INTEGER IB, IC, ID
      REAL    AMPSUM, CLOSUM, VSUM, DELVIS, DELCLO, CSUM(MXTRI)
C
C       Amplitude agreement factors:
      AMPSUM = 0.
C=    type *, 'Amplitudes'
      DO 20 IB = 1, NUMBAS
          IF (NZVIS(IB) .GT. 0) THEN
            VSUM = 0.
            DO 10 ID = 1, NUMREC
              IF (WTVIS(ID,IB) .GT. 0.0) THEN
                DELVIS = MODVIS(ID,IB) - OBSVIS(ID,IB)
C=              type *,modvis(id,ib),obsvis(id,ib),wtvis(id,ib),
C=   &                WTVIS(ID,IB)*DELVIS*DELVIS
                VSUM   = VSUM + WTVIS(ID,IB)*DELVIS*DELVIS
              ENDIF
   10       CONTINUE
            VISAF(IB) = SQRT (VSUM / NZVIS(IB))
            AMPSUM = AMPSUM + VSUM
          ELSE
            VISAF(IB) = -1.
          ENDIF
   20 CONTINUE
      IF (NZAMP .GT. 0) THEN
          AMPLAF = SQRT (AMPSUM / NZAMP)
      ELSE
          AMPLAF = -1.
      ENDIF
C
C       Closure phase agreement factors, for each triangle
      DO 25 IC = 1, NUMTRI
          CSUM(IC) = 0.
   25 CONTINUE
      CLOSUM = 0.
C=    type *, 'Closure phases'
      IF (NUMTRI .GT. 0) THEN
          DO 40 ID = 1,NUMREC
            IF (NINTRI(ID) .GT. 0) THEN
              DO 30 IC = 1,NUMTRI
                IF (WTCLO(ID,IC) .GT. 0.) THEN
                  DELCLO = ABS (MODCLO(ID,IC) - OBSCLO(ID,IC))
C=                type *,modclo(id,ic)*57.29578,obsclo(id,ic)*57.29578,
C=   &                   wtclo(id,ic),
C=   &                   WTCLO(ID,IC)*DELCLO*DELCLO
                  IF (DELCLO .GT. PI) DELCLO = DELCLO - TWOPI
                  CSUM(IC) = CSUM(IC) + WTCLO(ID,IC)*DELCLO*DELCLO
                  CLOSUM = CLOSUM+DOWNWT(ID)*WTCLO(ID,IC)*DELCLO*DELCLO
                ENDIF
   30         CONTINUE
            ENDIF
   40     CONTINUE
      ENDIF
      DO 50 IC = 1,NUMTRI
          IF (NZCLO(IC) .GT. 0) THEN
            CLOAF(IC) = SQRT (CSUM(IC)/NZCLO(IC))
          ELSE
            CLOAF(IC) = -1.
          ENDIF
   50 CONTINUE
C
      IF (TNZCLO .GT. 0.) THEN
          PHASAF = SQRT (CLOSUM/TNZCLO)
      ELSE
          PHASAF = -1.
      ENDIF
C
C       Total agreement factor for all data:
      TOTLAF = SQRT ((AMPSUM+CLOSUM) / (NZAMP+TNZCLO))
C
      RETURN
      END

      SUBROUTINE QGETMF
C-----------------------------------------------------------------------
C By Richard S. Simon
C
C This routine, based on TJP's GETMOD in UVPLOT, retrieves a model.
C-----------------------------------------------------------------------
      INCLUDE 'qfit.inc'
      INTEGER  I, ITYPE
      REAL     FLXSUM
C
      FLXSUM = 0.
      DO 150 I = 1, MXMOD
          CALL GTMODL (INMOD, ITYPE, FLUX(I), RADIUS(I), THETA(I),
     1    AXIS(I), RATIO(I), PHI(I), *155, 0)
          IF (FLUX(I) .EQ. 0.0) GO TO 155
          FLXSUM = FLXSUM + FLUX(I)
          RTHETA(I) = THETA(I) * DEGRAD
          RPHI(I)   = PHI(I) * DEGRAD
          TYPE(I)   = ITYPE
          IF (TYPE(I) .EQ. 0) THEN
              TYPE(I) = 1
          ELSE IF (TYPE(I) .LT. 1 .OR. TYPE(I) .GT. 5) THEN
              WRITE (OUTC,660) TYPE(I)
              TYPE(I) = 1
          END IF
  150 CONTINUE
      WRITE (OUTC,650) MXMOD
      I = MXMOD + 1
  155 NCOMP = I-1
      IF (FLXSUM .EQ. 0.0) CALL ERROR('Model has zero total flux')
      RETURN
C
  650 FORMAT (' ++WARNING++ Model truncated after ',I5,' components.')
  660 FORMAT (' ++WARNING++ Invalid component type for component ',I4,
     1'; Gaussian used')
C
      END

      SUBROUTINE QGETDF
C-----------------------------------------------------------------------
C By Richard S. Simon
C
C Version 1:      ??-June-1980
C Version 2:      20-July-1983    Extend to include Merge
C                               version 2 files and compute U and V;
C                               fix bug in TAKE dimension
C
C  This subroutine does the following:
C       1) Reads header parameters.
C       2) Reads in amplitude and closure phase data from data file.
C       3) Computes (or reads in) the U's and V's for a data set.
C-----------------------------------------------------------------------
      INCLUDE 'qfit.inc'
      INTEGER  IOUT, IPROUT, IZERO, IDAY, IYEAR, IDUMMY
      INTEGER  IDUM, IUNITS, IFILE, IB, NWORDS, ID, IT
      INTEGER  I, K, IC
      REAL     DUMMY, COSDEC, SINDEC, BX, BY, BHA, OBSIHA, RADIHA
      REAL     TAKE(MAXWPB*MXBAS + 2*MXTRI), SIDINT(MXREC), BZ(MXBAS)
      REAL     BXY(MXBAS), XIHA(MXBAS)
      CHARACTER*16 SNAME
C
C Get header parameters
C
      IOUT = 0
      IPROUT = 0
      CALL HEADX20(.FALSE., INDAT, IZERO, IDAY, IYEAR, IDUMMY, IDUM,
     +            IOUT, IUNITS, DUMMY, IFILE, IPROUT)
      IF (IFILE .NE. 0) THEN
          CALL ERROR('Data file does not have header record at'//
     1    ' start of file')
      END IF
C
C       Certify that data is in flux units before proceeding.
      IF (IUNITS .NE. 1) THEN
          CALL ERROR('Data not in flux units')
      END IF
C
      WRITE (SNAME,'(2A8)') SOURCE(1),SOURCE(2)
      CALL PUTOUT(' ')
      CALL PUTOUT('Source: '//SNAME)
      LAMBDA = VELC/FREQ
C
C       Load NSTA, NBAS, etc into COMMON
      NUMSTA = NSTA
      NUMBAS = NBAS
      NUMTRI = NTRI
      NINDCL = (NUMSTA-1)*(NUMSTA-2)/2
C
      COSDEC = COS(DECP)
      SINDEC = SIN(DECP)
      IF (NVER.EQ.1) THEN
          DO 100 IB=1,NUMBAS
            BX       = (STX(NBVIS(IB))-STX(NAVIS(IB))) / LAMBDA
            BY       = (STY(NBVIS(IB))-STY(NAVIS(IB))) / LAMBDA
            BZ(IB)   = (STZ(NBVIS(IB))-STZ(NAVIS(IB))) / LAMBDA
            BHA      = ATAN2(BY,BX)
            BXY(IB)  = SQRT (BX*BX + BY*BY)
            XIHA(IB) = (GASTM-RAP-BHA+HALFPI) * RADHRS
  100     CONTINUE
      END IF
C
C       Read in the data, compute UT
      NWORDS = NDWPB*NUMBAS + 2*NUMTRI
      DO 150 ID=1,MXREC
          READ (INDAT, END=160)  IT, (TAKE(I), I = 1, NWORDS)
          IF (IT .EQ. -1) GO TO 160
C         Compute UT in hours
          UT(ID) = (IT - IZERO) / 216000.
          SIDINT(ID) = UT(ID) * SIDR
C         Get amplitudes and errors and UV values
          DO 130 IB = 1, NUMBAS
            K = NDWPB*(IB-1)
            OBSVIS(ID,IB) = TAKE(K+1)
            WTVIS(ID,IB)= TAKE(K+2)
            IF (NVER.EQ.1) THEN
              OBSIHA = XIHA(IB) + SIDINT(ID)
              RADIHA = OBSIHA / RADHRS
              UDATA(ID,IB) = (-BXY(IB)*COS(RADIHA)) / CONV1
              VDATA(ID,IB) = (BZ(IB)*COSDEC -
     1          BXY(IB)*SINDEC*SIN(RADIHA)) / CONV1
            ELSEIF (NVER.EQ.2) THEN
              UDATA(ID,IB) = - TAKE(K+5) / CONV1
              VDATA(ID,IB) = - TAKE(K+6) / CONV1
            ENDIF
  130     CONTINUE
C         Get closure phases and errors
          IF (NUMTRI .GT. 0.) THEN
            DO 140 IC = 1, NUMTRI
              K = NDWPB*NUMBAS + 2*(IC-1)
              OBSCLO(ID,IC)  = TAKE(K+1) * DEGRAD
              WTCLO(ID,IC) = TAKE(K+2)   * DEGRAD
  140       CONTINUE
          ENDIF
  150 CONTINUE
C       Not all the data was read in, so issue a warning:
      WRITE (OUTC,1008) MXREC,MXREC,MXREC
      ID = MXREC + 1
C
  160 CONTINUE
      NUMREC = ID - 1
C
 1008 FORMAT(/' Well, it looks like you have blown it.  This version',
     +' of the program will',
     +/' only handle',I4,' separate data records, i.e.,',I4,
     +' separate UT''s.  Thus,',
     +/' only the first',I4,' times will be used in the fit.'/)
C
      RETURN
      END

      SUBROUTINE QMODAP
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
      INCLUDE 'qfit.inc'
C
      REAL C1
      PARAMETER (C1=-3.559707)
      INTEGER IB, IC, ID, J
      REAL COMPHA, COMVIS, CPHI, SPHI, IMVIS, REVIS, MODPHA(MXREC,MXBAS)
      REAL Q, A, R, U, V, QQ, XX
      REAL BESJ0, BESJ1
      EXTERNAL BESJ0, BESJ1
C
      Q(A,SPHI,CPHI,R,U,V)= TWOPI*SQRT((V*CPHI+U*SPHI)**2
     1+ (R*(U*CPHI-V*SPHI))**2) * A
C
      DO 105 IB = 1, NUMBAS
          DO 100 ID = 1, NUMREC
C$$         IF (WTVIS(ID,IB).LE.0.) GO TO 100 ! removed 1989 Mar 6
            REVIS = 0.0
            IMVIS = 0.0
            DO 5  J=1,NCOMP
              COMPHA = TWOPI*RADIUS(J) * (UDATA(ID,IB)*SIN(RTHETA(J))
     1        + VDATA(ID,IB)*COS(RTHETA(J)))
C
C             Find the complex visibility amplitude:
              IF (AXIS(J) .EQ. 0.0) THEN
                COMVIS = FLUX(J)
              ELSEIF (TYPE(J) .EQ. 1) THEN
                SPHI   = SIN(RPHI(J))
                CPHI   = COS(RPHI(J))
                COMVIS = FLUX(J) * (EXP (C1 * AXIS(J)**2 *
     1          ((VDATA(ID,IB)*CPHI + UDATA(ID,IB)*SPHI)**2 +
     2          (RATIO(J) * (UDATA(ID,IB)*CPHI-VDATA(ID,IB)*SPHI))**2)))
              ELSEIF (TYPE(J) .EQ. 2) THEN
                SPHI   = SIN(RPHI(J))
                CPHI   = COS(RPHI(J))
                QQ = Q(AXIS(J),SPHI,CPHI,RATIO(J),
     1                 UDATA(ID,IB),VDATA(ID,IB))
                QQ = QQ / 2.0
C                  Thus, axis is diameter
                IF (ABS(QQ).LT.0.01) THEN
                  COMVIS = FLUX(J)*(1.0 - QQ*QQ/8. + QQ*QQ*QQ*QQ/192.)
                ELSE
                  COMVIS = 2. * FLUX(J) * BESJ1(QQ)/QQ
                END IF
              ELSEIF (TYPE(J) .EQ. 3) THEN
                SPHI   = SIN(RPHI(J))
                CPHI   = COS(RPHI(J))
                QQ = Q(AXIS(J),SPHI,CPHI,RATIO(J),
     1                 UDATA(ID,IB),VDATA(ID,IB))
                QQ = QQ / 2.0 
C                  ! Thus, axis is diameter
                IF (ABS(QQ).LT.0.01) THEN
                  COMVIS = FLUX(J)*(1.0 - QQ*QQ/10. + QQ*QQ*QQ*QQ/280.)
                ELSE
                  COMVIS = 3. * FLUX(J) * (SIN(QQ)-QQ*COS(QQ)) /
     1               (QQ*QQ*QQ)
                END IF
              ELSEIF (TYPE(J) .EQ. 4) THEN
                SPHI   = SIN(RPHI(J))
                CPHI   = COS(RPHI(J))
                COMVIS = FLUX(J) * BESJ0(Q(AXIS(J),SPHI,
     1          CPHI,RATIO(J),UDATA(ID,IB),VDATA(ID,IB)))
              ELSEIF (TYPE(J) .EQ. 5) THEN
                SPHI   = SIN(RPHI(J))
                CPHI   = COS(RPHI(J))
                XX = (UDATA(ID,IB)*SPHI + VDATA(ID,IB)*CPHI)*PI*AXIS(J)
                IF (ABS(XX).LT.0.01) THEN
                  COMVIS = FLUX(J)*(1.0 - XX*XX/6. + XX*XX*XX*XX/120.)
                ELSE
                  COMVIS = FLUX(J) * SIN(XX)/XX
                END IF
              ELSE
                CALL ERROR ('Illegal component type in model')
              ENDIF
C
              REVIS  = REVIS + COMVIS*COS(COMPHA)
              IMVIS  = IMVIS + COMVIS*SIN(COMPHA)
    5       CONTINUE
            MODVIS(ID,IB) = SQRT(REVIS*REVIS + IMVIS*IMVIS)
            IF (REVIS .EQ. 0. .AND. IMVIS .EQ. 0.) THEN
              MODPHA(ID,IB) = 0.
            ELSE
              MODPHA(ID,IB) = -ATAN2(IMVIS,REVIS)
            ENDIF
  100     CONTINUE
  105 CONTINUE
C
C       Calculate the closure phases for the model:
      DO 120 IC = 1,NUMTRI
          DO 110 ID = 1,NUMREC
            MODCLO(ID,IC) = MODPHA(ID,ICBSNO(1,IC)) +
     1      MODPHA(ID,ICBSNO(2,IC)) - MODPHA(ID,ICBSNO(3,IC))
            IF (MODCLO(ID,IC) .LT. -PI .OR. MODCLO(ID,IC) .GT. PI) THEN
              MODCLO(ID,IC) = AMOD( MODCLO(ID,IC), TWOPI )
              IF (MODCLO(ID,IC) .LT. -PI) MODCLO(ID,IC) = MODCLO(ID,IC)
     +        + TWOPI
              IF (MODCLO(ID,IC) .GT.  PI) MODCLO(ID,IC) = MODCLO(ID,IC)
     +        - TWOPI
            ENDIF
  110     CONTINUE
  120 CONTINUE
C
      RETURN
      END

      SUBROUTINE TRIANG
C-----------------------------------------------------------------------
C Finds which triangles are independent in a VLB data set.
C
C Revised by TJP 1989 Jun 21.
C     The program was incorrectly assuming the baselines were in
C     canonical order. It now uses the header information to determine
C     which baselines must be summed to get each closure phase.
C Revised by TJP 1990 Nov 5.
C     New algorithm for determining number of independent closure
C     phases. Each triangle which makes use of a baseline that has not
C     been used in a previous triangle is independent; otherwise it
C     can be expressed as a combination of the previous triangles.
C-----------------------------------------------------------------------
      INCLUDE 'qfit.inc'
C
      INTEGER   BASE(MXBAS), IB, ID, IT
      LOGICAL   BUSED(MXBAS)
      INTEGER   LOOKUP
      EXTERNAL  LOOKUP
C
C Make baseline labels for the NBAS baselines.
C
      DO 100 IB=1,NBAS
          BASE(IB) = 100*NAVIS(IB) + NBVIS(IB)
  100 CONTINUE
C
C Find the baseline numbers baseline numbers ICBSNO(*,NTRI) for 
C triangle NTRI by making baseline labels and looking them up in
C the list of available baselines.
C
      DO 110 IT=1,NTRI
          ICBSNO(1,IT) = LOOKUP(100*NACLO(IT) + NBCLO(IT), BASE, NBAS)
          ICBSNO(2,IT) = LOOKUP(100*NBCLO(IT) + NCCLO(IT), BASE, NBAS)
          ICBSNO(3,IT) = LOOKUP(100*NACLO(IT) + NCCLO(IT), BASE, NBAS)
          IF (ICBSNO(1,IT).EQ.0 .OR. ICBSNO(2,IT).EQ.0 .OR.
     :        ICBSNO(3,IT).EQ.0) CALL ERROR('improper triangle list')
  110 CONTINUE
C
C Find the independent triangles in each record, NINTRI(ID).
C
      DO 140 ID=1,NUMREC
          DO 120 IB=1,NBAS
              BUSED(IB) = .FALSE.
  120     CONTINUE
          NINTRI(ID) = 0
          DO 130 IT=1,NTRI
              IF (WTCLO(ID,IT) .GT. 0.0) THEN
                  IF (BUSED(ICBSNO(1,IT)) .AND.
     :                BUSED(ICBSNO(2,IT)) .AND.
     :                BUSED(ICBSNO(3,IT))) THEN
C                       -- not independent
                  ELSE
C                       -- independent
                      BUSED(ICBSNO(1,IT)) = .TRUE.
                      BUSED(ICBSNO(2,IT)) = .TRUE.
                      BUSED(ICBSNO(3,IT)) = .TRUE.
                      NINTRI(ID) = NINTRI(ID)+1
                  END IF
              END IF
  130     CONTINUE
  140 CONTINUE
C
      RETURN
      END

      SUBROUTINE WTDATA
C-----------------------------------------------------------------------
C By Richard S. Simon
C
C This subroutine is part of program QFIT, and does the following:
C 1)  Compute the weights for each data point.
C 2)  Find the number of non-zero points in the data set.
C
C Version N+1:    ??-Jun-1981
C Version N+2:    Extend to include MERGE format Version 2 files
C-----------------------------------------------------------------------
      INCLUDE 'qfit.inc'
      INTEGER IB, IC, ID
C
      DO 30 ID = 1, NUMREC
          NUMCLO(ID) = 0
   30 CONTINUE
C
C       Amplitude weights:
      NZAMP = 0
      DO 60 IB = 1, NUMBAS
          NZVIS(IB) = 0
          DO 50 ID = 1, NUMREC
            IF (WTVIS(ID,IB) .GT. 0.0) THEN
              NZVIS(IB) = NZVIS(IB) + 1
              WTVIS(ID,IB) = 1. / (WTVIS(ID,IB))**2
            ENDIF
   50     CONTINUE
          NZAMP = NZAMP + NZVIS(IB)
   60 CONTINUE
C
C       Find the weights for the closure phase data, if any:
      DO 80 IC = 1, NUMTRI
          NZCLO(IC) = 0
          DO 70 ID = 1, NUMREC
            IF (WTCLO(ID,IC) .GT. 0.0) THEN
              NZCLO(IC) = NZCLO(IC) + 1
              NUMCLO(ID) = NUMCLO(ID) + 1
              WTCLO(ID,IC) = 1./ (WTCLO(ID,IC))**2
            ENDIF
   70     CONTINUE
   80 CONTINUE
      TNZCLO = 0.
      DO 90 ID = 1,NUMREC
          IF (NUMCLO(ID) .GT. 0) THEN
            DOWNWT(ID) = REAL(NINTRI(ID)) / REAL(NUMCLO(ID))
            TNZCLO = TNZCLO + NINTRI(ID)
          ELSE
            DOWNWT(ID) = 0
          ENDIF
   90 CONTINUE
C
      RETURN
      END
