      PROGRAM SUNCON
C-----------------------------------------------------------------------
C SUNCON converts a  Merge-format  file   from   the   Convex   "Native"
C floating-point format to  "IEEE" floating-point format, or vice versa.
C Some Convexes and most Unix workstations (including Sun SPARCstations)
C use the IEEE  floating-point  format.   The difference between the two
C formats amounts to a factor of 4 in all  single  or  double  precision
C floating-point numbers in  most cases.  Note that the program does not
C check that the operation requested is appropriate or not:  it is up to
C the user to apply the correction, using SUNCON, when moving files from
C one environment to another.
C
C Example
C
C % suncon
C input  = 3c179cvx.mrg
C output = 3c179sun.mrg
C target = sun
C /
C
C Parameters
C
C End the list of parameters with a slash (/). 
C
C INPUT = filename :  specifies the name of the input file.
C 
C OUTPUT = filename :  specifies the name for the output file.
C
C TARGET = CONVEX or SUN :  specify SUN to convert from Convex format to
C         IEEE  (SUN)  format; specify CONVEX to convert from IEEE (SUN)
C         format to CONVEX format.
C
C HISTORY= "text" :  the specified text (up to 80 characters)  will  get
C         written in the history of the output file.
C
C T.J. Pearson, standard Fortran 77.
C
C Version 1.0:  1992 Apr 06: based on MRGCOPY v 1.5.
C-----------------------------------------------------------------------
C
C Constants.
C
      CHARACTER*(*) PROGNM, VERSN
      INTEGER INC,OUTC,INDAT,OUTDAT,MAXS,MAXB,MAXT,NPARS
C     -- Program version and date:
      PARAMETER (PROGNM='SUNCON')
      PARAMETER (VERSN='1.0 - 1992 Apr 6')
C     -- Fortran unit numbers used:
      PARAMETER (INC=5,OUTC=6,INDAT=8,OUTDAT=9)
C     -- Maximum numbers of stations, baselines, and triangles:
      PARAMETER (MAXS=40)
      PARAMETER (MAXB=(MAXS-1)*MAXS/2)
      PARAMETER (MAXT=MAXB*(MAXS-2)/3)
C     -- Number of input parameters:
      PARAMETER (NPARS=32)
C-----------------------------------------------------------------------
C Variables.
C
      CHARACTER*8  STNAME(MAXS)
      CHARACTER*16 SOURCE, TARGET
      INTEGER   IUT, IS, IB, IT, NS, NB, NT, I, J, IER, MODE
      INTEGER   IZ, IY, NHIST
      INTEGER   NNEW, NDWPB, LENG
      INTEGER   NBI(MAXB), NB1(MAXB), NB2(MAXB)
      INTEGER   NTI(MAXT)
      INTEGER   REC1(20), IDUM(8)
      REAL      INREC(7*MAXB+2*MAXT), RFACT
      DOUBLE PRECISION STX(MAXS), STY(MAXS), STZ(MAXS),
     1          RA, DEC, RAP, DECP, GASTM, FREQ, FLUX,
     2          TINTG, AVG, BW, DFACT,
     4          PARS(NPARS),VALS(NPARS),ENDMRK
      CHARACTER*80 INDSN, OUTDSN, OLDHIS, HISTRY, ADDHIS(4)
C-----------------------------------------------------------------------
C Functions called.
C
      INTEGER   LEN1, VLBOPE
C-----------------------------------------------------------------------
C Names and default values of KEYIN parameters.
C
      DATA  PARS/ 'Input',   9*' ', 'Output',  9*' ', 'History',9*' ',
     1            'Target', ' '/
      DATA  VALS/ '***UNSET',9*' ', '***UNSET',9*' ', 10*' ', 
     1            '***UNSET', ' '/
      DATA  ENDMRK/ '/' /
C-----------------------------------------------------------------------
C
C Introduction.
C
      CALL PUTOUT(PROGNM//
     1 ' converts MERGE-format files from Convex native')
      CALL PUTOUT('floating-point format to IEEE floating-point'//
     1 ' format, or vice versa.')
      CALL PUTOUT('(Version '//VERSN//')')
C
C Control parameters.
C
      MODE = 0
  100 IF (MODE.NE.0) CALL ERROR('Bad input parameters in '//PROGNM)
      CALL PUTOUT(' ')
      CALL PUTOUT(
     1 'Parameters (INPUT, OUTPUT, TARGET=SUN or TARGET=CONVEX:')
      CALL KEYIN(PARS,VALS,NPARS,ENDMRK,MODE,INC,OUTC)
      WRITE (INDSN, '(10A8)') (VALS(I),I=1,10)
      WRITE (OUTDSN,'(10A8)') (VALS(I),I=11,20)
      WRITE (HISTRY,'(10A8)') (VALS(I),I=21,30)
      WRITE (TARGET,'(2A8)')  (VALS(I),I=31,32)
      CALL UPCASE(TARGET)
      IF (TARGET.EQ.'SUN') THEN
          RFACT = 0.25
          DFACT = 0.25D0
          ADDHIS(2)='- Convert Convex Native to IEEE fp format (*0.25)'
      ELSE IF (TARGET.EQ.'CONVEX') THEN
          RFACT = 4.0
          DFACT = 4.0D0
          ADDHIS(2)='- Convert IEEE to Convex Native fp format (*4.0)'
      ELSE
          CALL PUTOUT('TARGET must be "SUN" or "CONVEX"')
          GOTO 100
      END IF
C
C Open input Merge file.
C
      IER = VLBOPE(INDAT, INDSN, 'MERGE', 'OLD', INDSN)
      IF (IER.NE.1) GOTO 100
C
C Open output Merge file.
C
      IER = VLBOPE(OUTDAT, OUTDSN, 'MERGE', 'NEW', OUTDSN)
      IF (IER.NE.1) THEN
          CLOSE (UNIT=INDAT)
          GOTO 100
      END IF
C
C Make the new history records.
C
      CALL MKHIST(PROGNM, VERSN, ADDHIS(1))
      IF (HISTRY.NE.' ') THEN
          NNEW = 3
          ADDHIS(3) = '- '//HISTRY(1:78)
      ELSE
          NNEW = 2
      END IF
      DO 105 I=1,NNEW
          CALL PUTOUT(ADDHIS(I)(1:LEN1(ADDHIS(I))))
  105 CONTINUE
C
C Read and re-write header. Merge format level must be 1.
C
      READ (INDAT,END=82,ERR=82) (REC1(J),J=1,20)
      IF (REC1(1).NE.1) GOTO 82
      NDWPB = 4
      NHIST = REC1(2)
      NS    = REC1(4)
      NB    = REC1(5)
      NT    = REC1(6)
      IF (NS.LT.2) GOTO 82
      IF (NS.GT.MAXS) GOTO 82
      REC1(2) = REC1(2) + NNEW
      REC1(3) = REC1(3) + NNEW
      WRITE (OUTDAT) (REC1(J),J=1,20)
C
      IF (NB.GT.0) THEN
          READ (INDAT,END=82,ERR=82) (NBI(IB),IB=1,NB)
          WRITE (OUTDAT) (NBI(IB),IB=1,NB)
          DO 110 IB=1,NB
              NB1(IB) = NBI(IB)/100
              NB2(IB) = NBI(IB) -  NB1(IB)*100
  110     CONTINUE
      END IF
C
      IF (NT.GT.0) THEN
          READ (INDAT,END=82,ERR=82) (NTI(IT),IT=1,NT)
          WRITE (OUTDAT) (NTI(IT),IT=1,NT)
      END IF
C
      DO 120 I=1,NHIST
          READ (INDAT,END=82,ERR=82) OLDHIS
          WRITE (OUTDAT) OLDHIS
  120 CONTINUE
      DO 125 I=1,NNEW
          WRITE (OUTDAT) ADDHIS(I)
  125 CONTINUE
C
      READ (INDAT,END=82,ERR=82) SOURCE,RA,DEC,RAP,DECP,FLUX
      RA = RA*DFACT
      DEC = DEC*DFACT
      RAP = RAP*DFACT
      DECP = DECP*DFACT
      FLUX = FLUX*DFACT
      WRITE (OUTDAT) SOURCE,RA,DEC,RAP,DECP,FLUX
C
      DO 130 IS=1,NS
          READ (INDAT,END=82,ERR=82) STNAME(IS),STX(IS),STY(IS),STZ(IS)
          STX(IS) = STX(IS)*DFACT
          STY(IS) = STY(IS)*DFACT
          STZ(IS) = STZ(IS)*DFACT
          WRITE(OUTDAT) STNAME(IS),STX(IS),STY(IS),STZ(IS),
     1            (0.D0,J=5,10),0
  130 CONTINUE
C
      READ (INDAT,END=82,ERR=82) IY, IZ, GASTM, FREQ, TINTG,
     1            AVG, BW, (IDUM(I),I=1,8)
      GASTM = GASTM*DFACT
      FREQ  = FREQ*DFACT
      TINTG = TINTG*DFACT
      AVG   = AVG*DFACT
      BW    = BW*DFACT
      WRITE (OUTDAT) IY, IZ, GASTM, FREQ, TINTG, AVG, BW, 
     1               (IDUM(I),I=1,8)
C
C Read and re-write data records.
C
      LENG = NDWPB*NB+2*NT
  300 READ(INDAT,END=400) IUT,(INREC(J),J=1,LENG)
      IF (IUT.LT.0) GOTO 400
C     -- Convert the data
      DO 320 J=1,LENG
          INREC(J) = INREC(J)*RFACT
  320 CONTINUE
      WRITE (OUTDAT) IUT,(INREC(J),J=1,LENG)
      GOTO 300
C
C Write final record and quit.
C
  400 WRITE (OUTDAT) -1,(0.0,J=1,LENG)
      CLOSE (UNIT=INDAT)
      CLOSE (UNIT=OUTDAT)
      STOP
C
C Error exit. Delete the nascent output file.
C
   82 CLOSE (UNIT=OUTDAT, STATUS='DELETE')
      CLOSE (UNIT=INDAT)
      CALL ERROR('Something wrong with input MERGE file.')
      STOP
C-----------------------------------------------------------------------
      END
