      PROGRAM TSYS
C-----------------------------------------------------------------------
C Program to read OVRO log file, HP log file and/or VLA monitor data
C file for MkII VLBI; to extract system temperatures; and to write them
C in a format suitable for use with VLB:CAL.  Based upon TSYSOVRO by
C T.J. Pearson, TSYSMK3D by S.C. Unwin C and G.L. Dvorak, and TSYSVLA
C by J.M. Wrobel.
C       Version 0.0  31May84 Original form - J.M. Wrobel
C       Version 1.0  20Jun84 Bypass odd HP TSYS values, warn user - jmw
C       Version 1.1  8Nov84  Avoid TSYS truncation for > 100 K - jmw
C       Version 1.2  28Jan85 Bypass odd VLA TSYS values, warn user - jmw
C       Version 1.3  6Jun85  Better test to increment VLA DAYNUM  - jmw
C       Version 1.4  11Dec87 Add programs for Jodrell Bank logs - akt
C       Version 1.5  2Apr88  Minor changes for Convex-UNIX - tjp
C       Version 1.6  28Oct88 Change in VLA file format - tjp
C       Version 1.7  18Nov88 New OVRO option - tjp
C-----------------------------------------------------------------------
      CHARACTER*(*) VERSN
      PARAMETER     (VERSN='1.7 - 1988 Nov 18')
      CHARACTER*128 INFILE, OUTFIL
      CHARACTER*80  COMMNT
      CHARACTER*1   TYPE
      INTEGER       IER, L, GETIN
C-----------------------------------------------------------------------
C                                               Introduction.
C
      CALL PUTOUT('TSYS extracts system temperature data from antenna'//
     1            ' log files')
      CALL PUTOUT('(Version '//VERSN//')')
C                                               Get file type.
 100  CALL PUTOUT
     1     ('---------------------------------------------------')
      CALL PUTOUT ('Legal types of tsys input files are:')
      CALL PUTOUT ('      O - OVRO logs')
      CALL PUTOUT ('      H - HP logs')
      CALL PUTOUT ('      V - VLA monitor files')
      CALL PUTOUT ('      J - JODRELL log files')
  99  IER = GETIN (TYPE,    'Enter O, H, V, J, or E to exit: ', L)
      IF (IER.NE.1) CALL EXIT(IER)
      IF (TYPE.EQ.'E' .OR. TYPE.EQ.'e') GOTO 999
      IF (TYPE.NE.'O' .AND. TYPE.NE.'o' .AND.
     1      TYPE.NE.'H' .AND. TYPE.NE.'h' .AND.
     2      TYPE.NE.'J' .AND. TYPE.NE.'j' .AND.
     3      TYPE.NE.'V' .AND. TYPE.NE.'v') THEN
            CALL PUTOUT ('Invalid response.  Try again.')
            GOTO 99
      ENDIF
C
C                                               Get input file name,
C                                               open it.
  10  IER = GETIN (INFILE,  'Input tsys file name      : ', L)
      IF (IER.NE.1) CALL EXIT(IER)
      OPEN (UNIT=8, FILE=INFILE(1:L), STATUS='OLD', READONLY,
     1        IOSTAT=IER)
      IF (IER.NE.0) THEN
            CALL PUTOUT ('Cannot open file '//INFILE(1:L))
            GOTO 10
      END IF
      INQUIRE (UNIT=8, NAME=INFILE)
C
C                                               Get output file name,
C                                               open it.
  20  CALL PUTOUT ('Output tsys file name')
      IER = GETIN (OUTFIL, '        (default tsys.dat): ', L)
      IF (IER.NE.1) CALL EXIT(IER)
      IF (L.LT.1 .OR. OUTFIL(1:L).EQ.' ') THEN
            OUTFIL = 'tsys.dat'
            L = LEN('tsys.dat')
      END IF
      OPEN (UNIT=9, FILE=OUTFIL(1:L), STATUS='NEW',
     1        FORM='FORMATTED', CARRIAGECONTROL='LIST', IOSTAT=IER)
      IF (IER.NE.0) THEN
           CALL PUTOUT ('Cannot open file '//OUTFIL(1:L))
           GOTO 20
      ENDIF
      INQUIRE (UNIT=9, NAME=OUTFIL)
C
C                                               Get comment.
      IER = GETIN (COMMNT, 'Comment with !            : ', L)
      IF (IER.NE.1) CALL EXIT(IER)
C
C                                               Fill outfile.
      WRITE (9,'(A)') COMMNT(1:L)
      IF (TYPE.EQ.'O' .OR. TYPE.EQ.'o') CALL TSYSOV
      IF (TYPE.EQ.'H' .OR. TYPE.EQ.'h') CALL TSYSHP
      IF (TYPE.EQ.'V' .OR. TYPE.EQ.'v') CALL TSYSVL
      IF (TYPE.EQ.'J' .OR. TYPE.EQ.'j') CALL TSYSJB
      WRITE (9, '('' /'')')
C
C                                               Tidy up, restart.
      CLOSE (UNIT=8, STATUS='KEEP')
      CLOSE (UNIT=9, STATUS='KEEP')
      GOTO 100
C-----------------------------------------------------------------------
 999  CONTINUE
      END

      SUBROUTINE TSYSOV
C-----------------------------------------------------------------------
C Modification of T.J. Pearson's TSYSOVRO.
C Extracts TSYS records from an OVRO 40-m log file and generates a table
C which may be used by VLB:CAL.  For 40-m version 15-Aug-1982 or later.
C-----------------------------------------------------------------------
      CHARACTER*80 RECORD
      CHARACTER*8  SOURCE, RESULT
      INTEGER      OVCHEK, EXTRN, EXTRS, LEN1, IER
      INTEGER      L, LS, ID, IH, IM, IS, NREC, NOUT
      INTEGER      GETIN
      LOGICAL      USEDB
      REAL         DB, RM, TCAL
C-----------------------------------------------------------------------
      CALL PUTOUT
     1     ('---------- Handling OVRO log file -----------------')
C
C                                               dB's or TSYS?
    5 IER = GETIN(RECORD,'Use TSYS values or dB values? [T, D] ', L)
      CALL UPCASE(RECORD)
      IF (RECORD(1:1).EQ.'D') THEN
          USEDB = .TRUE.
      ELSE IF (RECORD(1:1).EQ.'T') THEN
          USEDB = .FALSE.
      ELSE
          GOTO 5
      END IF
C
C                                               Get Tcal value.
      IF (USEDB) THEN
   10     IER = GETIN(RECORD, 'Tcal value [K]: ', L)
          IF (IER.NE.1) CALL ERROR('End of file')
          READ (RECORD, *, ERR=10) TCAL
          WRITE (9, '(''TSYS OVRO TCAL='',F6.2,'' /'')', ERR=850) TCAL
      ELSE
          WRITE (9, '(''TSYS OVRO FT=1.00 /'')', ERR=850)
      END IF
C
C                                               Write outfile header,
C                                               initialize.
      IER = 0
      NREC = 0
      NOUT = 0
      SOURCE = ' '
      LS = 1
C
C                                               Read log record.
 100  READ (8, '(A)', ERR=800, END=200) RECORD
      NREC = NREC+1
C
C                                               Get source name from
C                                               OBS or ACQD record.
      IF (RECORD(12:15).EQ.'OBS:' .OR. RECORD(12:16).EQ.'ACQD:') THEN
            IER = EXTRS (RECORD(13:), 'SOURCE', RESULT, L)
            IF (IER.EQ.1) THEN
                SOURCE = RESULT
                LS = L
            END IF
C
C                                               Get DB or TSYS field
C                                               from TSYS record.
      ELSE IF (RECORD(12:16) .EQ. 'TSYS:') THEN
            IF (USEDB) THEN
                 IER = EXTRN (RECORD(13:), 'DB', DB)
            ELSE
                 IER = EXTRN (RECORD(13:), 'TSYS', DB)
            END IF
                 IF (IER.EQ.1)
     1               IER = OVCHEK (RECORD(1:10),ID,IH,IM,IS)
                 IF (IER.EQ.1) THEN
                     RM = FLOAT(IS)/60.0
                     WRITE (9,
     1                  '(I3,1X,I2.2,'':'',I2.2,F3.2,F8.3,'' ! '',A)',
     2                  ERR=850) ID, IH, IM, RM, DB, SOURCE(1:LS)
                     NOUT = NOUT+1
                 END IF
      END IF
      GOTO 100
C
C                                               Successful end. Summary.
 200  CALL PUTOUT('Successful end.')
      WRITE (RECORD, '(I6,A)') NREC, ' OVRO log records read.'
      CALL PUTOUT (RECORD(1:LEN1(RECORD)))
      WRITE (RECORD, '(I6,A)') NOUT, ' OVRO TSYS records written.'
      CALL PUTOUT (RECORD(1:LEN1(RECORD)))
      GOTO 999
C
C                                               Fatal read error.
  800 CALL PUTOUT ('Error reading input file.  Stopping.')
      GOTO 999
C
C                                               Fatal write error.
  850 CALL PUTOUT ('Error writing output file.  Stopping.')
  999 RETURN
C-----------------------------------------------------------------------
      END

      SUBROUTINE TSYSHP
C-----------------------------------------------------------------------
C Modification of S.C. Unwin and G.L. Dvorak's TSYSMK3D.
C Extracts /TSYS1/ values from an HP log file and generates a table
C which may be used by VLB:CAL.  User must supply posn. in the /TSYS1/
C list.  Usually this is the second of the four values given (POS=2).
C-----------------------------------------------------------------------
      CHARACTER*132 LINE
      CHARACTER*80  RECORD
      CHARACTER*8   STN
      INTEGER       C(10), NREC, NOUT, POS, ID, IH, IM, IS, IER, LEN1
      INTEGER       GETIN, CPOS1, CPOS2, J, L
      REAL          RM, TSYS
C-----------------------------------------------------------------------
      CALL PUTOUT
     1     ('---------- Handling HP log file -------------------')
C
C                                               Get tsys position.
C                                               Answer ok?
   20 IER = GETIN(RECORD, 'Position of tsys value in list: ', L)
      IF (IER.NE.1) CALL ERROR('End of file')
      READ  (RECORD,*, ERR=20) POS
C
C                                               Bad tsys pos?
      IF (POS.GT.9) THEN
           CALL PUTOUT
     1        ('Position of tsys value too large.  Try again.')
           GOTO 20
      ENDIF
C
C                                               Station name?
C                                               Write outfile header.
      IER = GETIN (STN,
     1        'Station name                  : ', L)
              IF (IER.NE.1) CALL EXIT(IER)
      WRITE (9, '(''TSYS '',A,'' /'')', ERR=850) STN(1:L)
C
C                                               Begin reading.
      NREC = 0
  30  READ (8, '(A)', END=50, ERR=800) LINE
      C(1) = INDEX ( LINE, '/TSYS1/') + 6
      NREC = NREC+1
C
C                                               String /TSYS1/ found?
C                                               No => read new line.
      IF (C(1).EQ.6) GOTO 30
C
C                                               Yes => step thru line.
      DO 40 J=2,10
           C(J) = INDEX (LINE(C(J-1)+1:), ',') + C(J-1)
           IF (C(J).EQ.C(J-1)) C(J) = LEN1(LINE)+1
   40 CONTINUE
      CPOS1 = C(POS)
      CPOS2 = C(POS+1)
C
C                                               Tsys value is
C                                               LINE(CPOS1+1:CPOS2-1).
C                                               Tsys found?
      IER = 0
      IF (CPOS1.EQ.0 .OR. CPOS2-CPOS1.LE.1) THEN
C
C                                               No.
            WRITE (6,'(A)')
     1             ' Warning.  TSYS value not found at '
     2             // LINE(1:3) // ' ' // LINE(4:5) // ':'
     3             // LINE(6:7) // ':' // LINE(8:9) // '.'
      ELSE
C
C                                               Yes.
            READ (LINE(1:3), '(I3)') ID
            READ (LINE(4:5), '(I2)') IH
            READ (LINE(6:7), '(I2)') IM
            READ (LINE(8:9), '(I2)') IS
            RM = FLOAT(IS)/60.0
            READ (LINE(CPOS1+1:CPOS2-1), '(F5.1)', IOSTAT=IER) TSYS
C                                               Check for read error.
            IF (IER.GT.0) THEN
               WRITE (6,'(A)')
     1         ' Warning.  Skipping TSYS value at '
     2         // LINE(1:3) // ' ' // LINE(4:5) // ':'
     3         // LINE(6:7) // ':' // LINE(8:9) // '.'
            ELSE
               WRITE (9, '(I3,1X,I2.2,'':'',I2.2,F3.2,F8.3)', ERR=850)
     1                ID, IH, IM, RM, TSYS
               NOUT = NOUT+1
            ENDIF
      ENDIF
      GOTO 30
C
C                                               Successful end. Summary.
  50  CALL PUTOUT('Successful end.')
      WRITE (RECORD, '(I6,A)') NREC, ' HP log records read.'
      CALL PUTOUT (RECORD(1:LEN1(RECORD)))
      WRITE (RECORD, '(I6,1X,A,A)') NOUT, STN(1:L),
     1         ' TSYS records written.'
      CALL PUTOUT (RECORD(1:LEN1(RECORD)))
      GOTO 999
C
C                                               Fatal read error.
  800 CALL PUTOUT ('Error reading input file.  Stopping.')
      GOTO 999
C
C                                               Fatal write error.
  850 CALL PUTOUT ('Error writing output file.  Stopping.')
  999 RETURN
C-----------------------------------------------------------------------
      END

      SUBROUTINE TSYSJB
C-----------------------------------------------------------------------
C Modification of T.J. Pearson's TSYSOVRO to do Jodrell Bank log files.
C Extracts TSYS records from a JODRELL log file and generates a table
C which may be used by VLB:CAL.
C-----------------------------------------------------------------------
C     IMPLICIT     NONE
      CHARACTER*80 RECORD
      CHARACTER*8  SOURCE, RESULT
      INTEGER      GETIN, LEN1, IER, L, LS, ID, IH, IM, IS, NREC, NOUT
      REAL         RM, TSYS, FT
C-----------------------------------------------------------------------
      CALL PUTOUT
     1     ('---------- Handling JODRELL log file -----------------')
C
C
C                                ! Get fudge factor value.
   20 IER = GETIN(RECORD, 'Factor FT: ', L)
      IF (IER.NE.1) CALL ERROR('End of file')
      IF (L.LT.1 .OR. RECORD.EQ.' ') THEN
          FT = 1.0
      ELSE
          READ (RECORD,*, ERR=20) FT
          IF (FT.LE.0.0) FT = 1.0
      END IF
C
C                                               Write outfile header,
C                                               initialize.
      WRITE (9, '(''TSYS JODRELL  FT='',F5.3,'' /'')', ERR=850) FT
      IER = 0
      NREC = 0
      NOUT = 0
      SOURCE = ' '
      LS = 1
C
C                                               Read log record.
 100  READ (8, '(A)', ERR=800, END=200) RECORD
      NREC = NREC+1
C
C  Ignore comments
      IF (RECORD(1:1).EQ.'!'.OR. LEN1(RECORD).LE.0) GOTO 100
C
C  Get date and time
      READ (RECORD(7:19),'(I3,2X,3(I2,1X))') ID,IH,IM,IS
      RM = FLOAT(IS)/60.0
C  Parse source name
      IF(RECORD(56:56).EQ.' ') RECORD(56:56)='0'      
      IF(RECORD(56:56).EQ.'-') THEN
          RECORD(56:56)='0'          
          RECORD(55:55)='-'
      END IF
      IF(RECORD(55:55).EQ.' ') RECORD(55:55)='+'      
      RESULT=RECORD(45:46)//RECORD(48:49)//RECORD(55:57)//RECORD(59:59) 
C  Check and read tsys value
      IF (INDEX(RECORD(23:31),'**') .EQ. 0) THEN
          READ(RECORD(23:31),*) TSYS
          IF (RESULT .NE. SOURCE) THEN
           SOURCE=RESULT
           WRITE (9,'(I3,1X,I2.2,'':'',I2.2,F3.2,F8.3,'' ! '',A8)',
     1             ERR=850) ID, IH, IM, RM, TSYS, SOURCE
           NOUT = NOUT+1
          ELSE
           WRITE (9,'(I3,1X,I2.2,'':'',I2.2,F3.2,F8.3,'' ! '')',
     1             ERR=850) ID, IH, IM, RM, TSYS
           NOUT = NOUT+1
          ENDIF
      ENDIF
      GOTO 100
C
C                                               Successful end. Summary.
 200  CALL PUTOUT('Successful end.')
      WRITE (RECORD, '(I6,A)') NREC, ' JODRELL log records read.'
      CALL PUTOUT (RECORD(1:LEN1(RECORD)))
      WRITE (RECORD, '(I6,A)') NOUT, ' JODRELL TSYS records written.'
      CALL PUTOUT (RECORD(1:LEN1(RECORD)))
      GOTO 999
C
C                                               Fatal read error.
  800 CALL PUTOUT ('Error reading input file.  Stopping.')
      GOTO 999
C
C                                               Fatal write error.
  850 CALL PUTOUT ('Error writing output file.  Stopping.')
  999 RETURN
C-----------------------------------------------------------------------
      END

      SUBROUTINE TSYSVL
C-----------------------------------------------------------------------
C Modification of J.M. Wrobel's TSYSVLA.
C Extracts TSYS values from a VLA monitor data file provided by P. Crane
C and generates a table which may be used by VLB:CAL.  User must supply
C initial day number and handedness of feed used (R or L).  For VLA
C monitor data file format in use 1988.
C-----------------------------------------------------------------------
      CHARACTER*80 RECORD
      CHARACTER*1  RORLCP
      INTEGER      NOUT, ACHMS(3), UHMS(3), DAYNUM, IER, LEN1, L, I
      INTEGER      GETIN, IPOL
      REAL         ACDATE, EL, TSYS(2), RM
C-----------------------------------------------------------------------
      CALL PUTOUT
     1     ('-------- Handling VLA monitor file ----------------')
C
C                                               Get daynum, rcp or lcp.
      IER = GETIN(RECORD, 'Initial day number: ', L)
      READ  (RECORD,*) DAYNUM
  20  IER = GETIN (RORLCP, 'R or L cp?        : ', L)
      IF (RORLCP.EQ.'R' .OR. RORLCP.EQ.'r') THEN
          IPOL = 1
      ELSE IF (RORLCP.EQ.'L' .OR. RORLCP.EQ.'l') THEN
          IPOL = 2
      ELSE
          CALL PUTOUT ('Invalid response.  Try again.')
          GOTO 20
      ENDIF
C
C                                               Write outfile header.
C                                               Skip infile header.
C                                               Handle in, out files
C                                               line by line.
      WRITE (9, '(''TSYS VLA /'')', ERR=850)
      DO 25 I=1,8
          READ (8, '(1X)', ERR=800)
   25 CONTINUE
      NOUT = 0
C
  30  CONTINUE
          READ (8, '(1X,A7,6(1X,I2),F9.0,F12.0,1X,F12.0)',
     1        IOSTAT=IER, END=50) ACDATE, ACHMS, UHMS, EL, TSYS
          IF (IER.GT.0) THEN
              WRITE (6,1000) UHMS
              GOTO 30
          END IF
C
C                                            Change daynum? Assume delta
C                                           time = 80s. Decimal minutes.
          IF (UHMS(1).EQ.0 .AND.
     1       (UHMS(2)*60)+UHMS(3) .LT. 80) THEN
              WRITE (6,'(A)') ' Incrementing day number.'
              DAYNUM = DAYNUM+1
          ENDIF
          RM = FLOAT(UHMS(3))/60.0
          WRITE (9, '(I3,1X,I2.2,'':'',I2.2,F3.2,F8.2)', ERR=850)
     1             DAYNUM, UHMS(1), UHMS(2), RM, TSYS(IPOL)
          NOUT = NOUT+1
      GOTO 30
C
C                                               Successful end. Summary.
  50  CALL PUTOUT('Successful end.')
      WRITE (RECORD, '(I6,A)') NOUT,
     1         ' VLA TSYS records read and written.'
      CALL PUTOUT (RECORD(1:LEN1(RECORD)))
      GOTO 999
C
C                                               Fatal read error.
  800 CALL PUTOUT ('Error reading input file.  Stopping.')
      GOTO 999
C
C                                               Fatal write error.
  850 CALL PUTOUT ('Error writing output file.  Stopping.')
  999 RETURN
C
 1000 FORMAT (' Warning.  Skipping TSYS value at UT ',I2,2(':',I2))
C-----------------------------------------------------------------------
      END

      INTEGER FUNCTION OVCHEK (RECORD, ID, IH, IM, IS)
C-----------------------------------------------------------------------
C Check syntax of a 40-m log file time code (10-character string,
C "ddd/hhmmss").
C Arguments:
C   RECORD (input, character*10): time code to be parsed
C   ID     (output, integer)    : day number "ddd"
C   IH     (output, integer)    : hour "hh"
C   IM     (output, integer)    : minute "mm"
C   IS     (output, integer)    : second "ss"
C Returns:
C   1     if syntax is ok
C   not 1 if syntax is not ok (in which case ID, IH, IM, IS may not
C         be set).
C Stolen from tjp.
C-----------------------------------------------------------------------
      CHARACTER*(*) RECORD
      CHARACTER*1   C
      INTEGER       I, ID, IH, IM, IS
      LOGICAL       NUMBER
C-----------------------------------------------------------------------
      NUMBER(C) = '0'.LE.C .AND. C.LE.'9'
C-----------------------------------------------------------------------
      IF (LEN(RECORD).NE.10)  GOTO 100
      IF (RECORD(4:4).NE.'/') GOTO 100
      DO 10 I=1,3
           IF (.NOT.NUMBER(RECORD(I:I))) GOTO 100
   10 CONTINUE
      DO 20 I=5,10
           IF (.NOT.NUMBER(RECORD(I:I))) GOTO 100
   20 CONTINUE
      READ (RECORD(1:10), '(I3,1X,3I2)') ID, IH, IM, IS
      IF (ID.LT.1 .OR. ID.GT.366) GOTO 100
      IF (IH.LT.0 .OR. IH.GT.23)  GOTO 100
      IF (IM.LT.0 .OR. IM.GT.59)  GOTO 100
      IF (IS.LT.0 .OR. IS.GT.59)  GOTO 100
      OVCHEK = 1
      RETURN
C
 100  OVCHEK = 384
      RETURN
C-----------------------------------------------------------------------
      END

      INTEGER FUNCTION EXTRN(C1,C2,RESULT)
C-----------------------------------------------------------------------
C Extracts a numeric value from a 40-m log record (C1), given a keyword
C (C2). It returns the real value RESULT, and a status code of 0
C (failure - keyword not found, or not a numeric value) or 1 (success).
C Stolen from tjp.
C-----------------------------------------------------------------------
      CHARACTER*(*) C1, C2
      CHARACTER*32  STRING
      INTEGER       EXTRS, L, IER
      REAL          RESULT
C-----------------------------------------------------------------------
      EXTRN = EXTRS(C1, C2, STRING, L)
      IF (EXTRN.NE.1) RETURN
      READ (STRING(1:L), '(F32.0)', IOSTAT=IER) RESULT
      IF (IER.NE.0) EXTRN = 0
      RETURN
C-----------------------------------------------------------------------
      END

      INTEGER FUNCTION EXTRS(C1,C2,RESULT,LENG)
C-----------------------------------------------------------------------
C Extracts a numeric value from a 40-m log record (C1), given a keyword
C (C2). It returns the real value RESULT, and a status code of 0
C (failure - keyword not found, or not a numeric value) or 1 (success).
C LENG receives the number of characters in the string.
C Stolen from tjp.
C-----------------------------------------------------------------------
      CHARACTER*(*) C1, C2, RESULT
      INTEGER       I, L1, L2, LENG
C-----------------------------------------------------------------------
C                                               Assume failure.
      RESULT = ' '
      EXTRS = 0
      LENG = 0
C
C                                               Look for keyword.
C                                               L1 points to it.
      L1 = INDEX (C1, C2)
      IF (L1.LE.0) RETURN
C
C                                               L1 points to 1st char
C                                               after keyward.
      L1 = L1 + LEN(C2)
C
C                                             Skip >= 0 blanks, optional
C                                             '=' and >= 0 blanks. Error
C                                             if neither blank nor '='
C                                             follows keyword.
      I = L1
      DO WHILE (I.LE.LEN(C1) .AND. C1(I:I).EQ.' ')
           I = I+1
      END DO
      IF (I.LE.LEN(C1) .AND. C1(I:I).EQ.'=') I = I+1
      DO WHILE (I.LE.LEN(C1) .AND. C1(I:I).EQ.' ')
           I = I+1
      END DO
      IF (I.EQ.L1 .OR. I.GT.LEN(C1)) RETURN
C
C                                               Find terminating blank.
      L1 = I
      L2 = I+1
      DO WHILE (L2.LE.LEN(C1) .AND. C1(L2:L2).NE.' ')
           L2 = L2+1
      END DO
C
C                                               Success.
      RESULT = C1(L1:L2-1)
      EXTRS = 1
      LENG = MIN(LEN(RESULT),L2-L1)
      RETURN
C-----------------------------------------------------------------------
      END
