      PROGRAM UPTIME
C-----------------------------------------------------------------------
C UPTIME: Computes the rise and set times of a group of sources at a set
C of stations.
C
C T.J. Pearson,  Fortran-77.
C
C Version 1.0:  1981 Mar  6.
C Version 1.1:  1981 May  6 - compare names in upper case.
C Version 2.1:  1982 May 12 - add plots.
C Version 2.2:  1982 Jun  8 - new graphics; no questions.
C Version 2.3:  1983 Apr 19 - correct pagination; add UT.
C Version 2.4:  1984 Jun 20 - add PARALLAX option to plot parallactic
C                             angle.
C Version 2.5:  1986 Feb  9 - add day of year to listing.
C Version 2.6:  1987 Mar 16 - add a station LST and a local time (RCW).
C Version 2.7:  1987 Dec 16 - remove Convex incompatibilities (TJP).
C Version 2.8:  1988 Feb  4 - less verbose stations list (TJP).
C Version 2.9:  1988 Apr  2 - change list file format for Unix (TJP).
C Version 3.0:  1988 Dec 20 - Add Doppler calculation, precession,
C                             new catalog formats, and
C                             convert for Microsoft FORTRAN.  (RCW)
C Version 3.1   1989 Apr 13 - Add user input of catalog file names (RCW)
C Version 3.2   1990 Apr 10 - Labels for plots, fix use of UTLOC (AKT)
C Version 3.3   1990 Jul 30 - Change sign of UTLOC for consistency with
C                             v3.1 (TJP).
C Version 3.4   1990 Aug 18 - Up to 40 stations (TJP).
C Version 3.5   1991 Jan 11 - Changes to catalog include file (RCW/TJP).
C Version 3.6   1991 Jan 23 - Fix pagination; add LPP parameter (TJP).
C Version 3.7   1991 Jun 22 - Correct precession (TJP).
C Version 3.8   1991 Dec 16 - Add AIRMASS option (RCV).
C Version 3.9   1992 Mar  4 - Allow 4-digit year in date.
C Version 3.10  1992 Sep 15 - Default listfile is stdout (TJP).
C Version 3.11  1993 Apr 22 - Correct bug in pagination (TJP).
C Version 3.12  1993 Nov 11 - Increase to 200 sources (TJP).
C version 3.13  1994 Nov 17 - 12-char source names; aliases (TJP).
C Version 3.14  1995 Jan 30 - use RVLSRK instead or RVLSR (TJP).
C-----------------------------------------------------------------------
      CHARACTER*(*) VERSN
      INTEGER OUTC,INC,INDAT,PRUNIT,PR2,NPARS,MAXSRC,MAXSR2,MAXS
      DOUBLE PRECISION VELC,SIDR,PI,RPDEG,RPHR
      PARAMETER (VERSN='3.14 - 1995 Jan 30')
C     ! terminal output unit
      PARAMETER (OUTC=6)
C     ! terminal input unit
      PARAMETER (INC=5)
C     ! unit for catalogs
      PARAMETER (INDAT=90)
C     ! printer output
      PARAMETER (PRUNIT=8)
C     ! scratch file
      PARAMETER (PR2=9)
C     ! maximum number of sources
      PARAMETER (MAXSRC=200)
      PARAMETER (MAXSR2=2*MAXSRC)
C     ! maximum no of stations
      PARAMETER (MAXS=40)
      PARAMETER (NPARS=19+MAXS+MAXSR2+3+MAXS+22)
C     ! velocity of light m/s
      PARAMETER (VELC=2.99792458D8)
C     ! sidereal-solar ratio
      PARAMETER (SIDR=1.0027379093D0)
      PARAMETER (PI=3.14159265359D0)
      PARAMETER (RPDEG=PI/180D0)
      PARAMETER (RPHR=PI/12D0)
      CHARACTER*1 FORM, CHAR
      PARAMETER (FORM=CHAR(12))
C
      LOGICAL ALL,OK,FOUND(MAXSRC), PLOT,PARAL,DOAIRM
C
      CHARACTER*12  SNAME(MAXSRC),XSNAME
      CHARACTER*8   STNAME(MAXS),XSTNAM
      CHARACTER*5   EPOCH(MAXSRC)
      CHARACTER*8   LSTSTA, CBLANK
      CHARACTER*14  NOTE
      CHARACTER*25  VNOTE
      CHARACTER*64  PLTFIL,LSTFIL,SCRFIL
      CHARACTER*121 BAR,FRAME,RACK,UTFRAM,LSTFRM
      CHARACTER*215 LEGEND
      CHARACTER*2   LABEL(MAXS)
      CHARACTER*132 RECORD
      CHARACTER*13  LRA, LDEC, CRAJ, CDECJ, CRAB, CDECB, TFORM
      CHARACTER*255 RESULT
      CHARACTER*80  STAFIL, SRCFIL
C
      INTEGER MNAME,DNAME,PGBEG,J1(2),J2(2),DOY,ALIAS
      INTEGER VLBOPE, IER, RDSRC, RDSTA
      INTEGER I, ID, IM, IS, ISRC
      INTEGER IUT1, IUT2, IX, IY, J, JD, JSRC, JSTA, K, KODE, KSEG
      INTEGER L, LEN1, LL, LPP, LST1, LST2, M, MDATE, MODE, NPLOT
      INTEGER NS, NSEG, NSRC, NSRCPP, LINES, PR, LR
      INTEGER NMNAIR(2)

C
      REAL    HALIM(MAXS),ZALIM(MAXS)
      REAL    T(MAXS),P(MAXS)
      REAL    CDEC(MAXSRC),SDEC(MAXSRC)
      REAL    XP(121),YP(121),ZP(121)
      REAL    VSUN, VEARTH, TL, FLINE, BBC, SLA_RVLSRK
      REAL    VEL(MAXSRC), LO1(MAXS), LINEFR, CONT
      REAL    COSHAL, COSZAL, HAL, HARAD, RISE, SET, SUBLST, UTLOC
      REAL    UTM, XLSTM
      REAL    AIRMAS,MINAIR(2)
C
      DOUBLE PRECISION    DATE,DATE0,GMST,GMST0,GMSTR,RR,TT,PP
      DOUBLE PRECISION    RA(MAXSRC),DEC(MAXSRC)
      DOUBLE PRECISION    R1950(MAXSRC), D1950(MAXSRC)
      DOUBLE PRECISION    R2000(MAXSRC), D2000(MAXSRC)
      DOUBLE PRECISION    CLAT(MAXS),SLAT(MAXS),X(MAXS),Y(MAXS),Z(MAXS)
      DOUBLE PRECISION    PARS(NPARS),VALS(NPARS),ENDMRK,BLANK
      DOUBLE PRECISION    PEPOCH, DR, DD, AMPRMS(21)
C
C     Include file for catalog reading.
C
      INCLUDE 'rdcat.inc'
      DATA   PARS/ 'ALl', 'Date', 'PLotfile', 7*' ', 'LIStfile', 7*' ',
     1             'PAralla?', MAXS*'STATions', MAXSR2*'?',
     2             'LStsta', 'Utloc', 'LINefreq',
     4             MAXS*'LO1', 'STAFile', 9*' ', 'SRCfile', 9*' ',
     5             'LPP','AIrmass' /

      DATA   VALS/ -1D0, 0D0, 8*' ', 8*' ',
     1             -1D0, MAXS*' ', MAXSR2*' ', ' ', 0.0D0, 0.0D0,
     2             MAXS*0.D0, 'VLB_STAT','IONS', 8*' ',
     3             'VLB_SOUR','CES', 8*' ', 60D0, -1D0 /
      DATA   ENDMRK/ '/' /,BLANK/' '/, CBLANK/' '/
      DATA   CONT / 'CONT' /
C
C Format statements.
C
  500 FORMAT(' UPTIME computes rise and set times'/
     1       ' (Version ',A,')'/)
  601 FORMAT(' Program UPTIME'/)
  610 FORMAT(//5X,'Date: ',I4,1X,A3,I3,' (',A3,')'/
     1       5X,'Day of year: ',I3/
     2       5X,'Julian date: ',F12.1/
     3       5X,'Greenwich Mean S.T. at 0h U.T.: ',A)
  611 FORMAT(5X,'Frequency of spectral line:',F9.2,//,
     1 10X,'Doppler calculation is for Earth center at 0 hr UT ',
     2   '(non-relativistic)')
  630 FORMAT(/' Source',8X,'Station ',
     1       ' ZAlim', ' HAlim',3X,'GST rise, set    Vel. ',
     2        'Cen. freq.   -LO1 ')
  640 FORMAT(/1X,A12,2X,A8,F6.1,F6.2,2X,A,2X,A) 
  642 FORMAT(F7.2,F9.2)
  643 FORMAT(F9.2)
  631 FORMAT(A1)
  650 FORMAT(15X,A8,F6.1,F6.2,2X,A,2X,A)
  641 FORMAT(' +++ERROR+++ Source not found in catalog: ',A)
  618 FORMAT(//' Sources:'//4X,'Name',10X,
     1          'RA(B1950)    Dec(B1950)     ',
     2          'RA(J2000)    Dec(J2000)     ',
     3          'RA/Dec (geocentric apparent)'/)
  619 FORMAT(4X,A12,2X,A,A,2X,A,A,2X,A,A)
  620 FORMAT(//' Stations:'//4X,'Name',13X,'Radius',
     1       6X,'Latitude',3X,'(Geocentric)',2X,'Longitude',
     2       9X,'X',12X,'Y',12X,'Z',7X,'HA/ZA limits'/)
  621 FORMAT(4X,A8,8X,F9.1,F14.6,10X,F14.6,3X,3F13.1,T111,
     1       2F6.1)
  624 FORMAT(' +++ERROR+++ Station not found in catalog: ',A8)
  660 FORMAT(T60,'Greenwich Sidereal Time'/)
  662 FORMAT(T40,'Upper: Local Sidereal Time at ',A,' /  Lower:',
     1       ' Greenwich Sidereal Time'/)
  663 FORMAT(1X,A5,A,24(I5))
  661 FORMAT(/   T60,'UT (',I4,1X,A3,I3,')')
  665 FORMAT(3X,'GST',1X,25(I5))
  667 FORMAT(/  T39,'Middle: UT (',I4,1X,A3,I3,')  /  Lower: ',
     1       'Local time (UT',F5.1,')')
  668 FORMAT(/  T39,'Middle: UT (',I4,1X,A3,I3,')  /  Lower: ',
     1       'Local time (UT+',F4.1,')')
  670 FORMAT(11X,A121)
  680 FORMAT(A11,A121)
  740 FORMAT(2X,F5.2,2X,F5.2)
  810 FORMAT('  **** Error  ****  Station for LST (',A,
     1       ') not among requested STATIONS')
C-----------------------------------------------------------------------
C            Read file names and parameters
C-----------------------------------------------------------------------
C
      WRITE (OUTC,500) VERSN
      CALL SCRNAM('UPTIME', SCRFIL, L)
      IER = VLBOPE(PR2, SCRFIL(:L), 'TEXT', 'NEW', RESULT)
      IF (IER.NE.1) CALL ERROR('Cannot open scratch file '//SCRFIL(:L))
C
      WRITE (OUTC,'(A)') ' Control parameters (end with /)'
      WRITE (OUTC,'(A)') ' eg SOURCES (or ALL), STATIONS, DATE, '//
     1                 ' PARALLAX, PLOTFILE, LISTFILE'
      MODE = 0
      DO 1 ISRC=1,MAXSRC
         PARS(19+MAXS+2*ISRC-1) = 'Source'
         PARS(19+MAXS+2*ISRC)   = ' '
 1    CONTINUE
      CALL KEYIN(PARS,VALS,NPARS,ENDMRK,MODE,INC,OUTC)
      ALL = VALS(1).NE.-1D0
      PARAL = VALS(19).NE.-1D0
      WRITE (PLTFIL,'(8A8)') (VALS(J),J=3,10)
      WRITE (LSTFIL,'(8A8)') (VALS(J),J=11,18)
      PLOT = PLTFIL.NE.' '
      IF (PLOT) THEN
          IER = PGBEG(0,PLTFIL,1,1)
          IF (IER.NE.1) CALL ERROR('Cannot open PLOTFILE')
      END IF
C
      LINEFR = VALS(19+MAXS+MAXSR2+3)
      IF (LSTFIL.EQ.' ') THEN
C         -- standard output
          PR = OUTC
      ELSE
          PR = PRUNIT
          IER = VLBOPE(PR, LSTFIL, 'TEXT', 'NEW', RESULT)
          IF (IER.NE.1) CALL ERROR('Cannot open LISTFILE')
      END IF
      LPP = VALS(43+2*MAXS+MAXSR2)
      DOAIRM = VALS(44+2*MAXS+MAXSR2).NE.-1D0
C        ! Switch: airmass inst.of elev.
      IF (DOAIRM) THEN
         IF (PARAL)
     1    CALL ERROR('Options PARAL and AIRMASS are mutually exclusive')
         AIRMAS = VALS(44+2*MAXS+MAXSR2)
C           ! User-specified Y scale
         IF (AIRMAS.LT.1.001) AIRMAS = 3.
C           ! Default Y scale
      ENDIF
C
      WRITE(PR,601)
C
C-----------------------------------------------------------------------
C U.T. - G.S.T. conversion; if date not supplied, use today's date.
C-----------------------------------------------------------------------
C
      MDATE = VALS(2)
      IY = MOD(MDATE/3600,100)
      IM = MOD(MDATE,3600)/60
      ID = MOD(MDATE,60)
      IF (MDATE.LE.0) CALL IDATE(IM,ID,IY)
      IF (IY.LE.50) THEN
          IY = 2000 + IY
      ELSE IF (IY.LT.100) THEN
          IY = 1900 + IY
      END IF
      CALL TDATE(IY,IM,ID,JD,MNAME,DNAME)
      CALL JULDA(IY,IM,ID,DATE,GMST)
      CALL JULDA(IY,1,0,DATE0,GMST0)
      DOY = DATE-DATE0
      PEPOCH = IY + DOY / 365.25D0
C            DATE = Julian Date
C            DATE0 = Julian Date of zeroth day of year
C            GMST = Greenwich Mean S.T. at 0 U.T. (sec)
      GMSTR = GMST*RPHR/3600.0
      CALL PANGLE('H',GMSTR,LRA,3)
      WRITE(PR,610) IY,MNAME,ID,DNAME,DOY,DATE,LRA
      IF (LINEFR.NE.0.0) WRITE(PR,611) LINEFR
      CALL SLA_MAPPA(2000D0, DATE-2400000.5D0, AMPRMS)
C
C-----------------------------------------------------------------------
C Sources: look up source in catalog.  
C-----------------------------------------------------------------------
C
      NSRC = 0
      IF (ALL) GOTO 1050
      DO 1040 ISRC=1,MAXSRC
         J = 19+MAXS+2*ISRC-1
         IF (VALS(J).EQ.BLANK) GOTO 1040
         NSRC = NSRC+1
         WRITE(SNAME(NSRC),'(A8,A4)') VALS(J),VALS(J+1)
         CALL UPCASE(SNAME(NSRC))
         FOUND(NSRC) = .FALSE.
 1040 CONTINUE
C
 1050 CONTINUE
      JSRC = 19+MAXS+MAXSR2+3+MAXS+11
      WRITE(SRCFIL, '(10A8)') (VALS(I), I=JSRC, JSRC+9)
 1100 IER = RDSRC(INDAT, .TRUE., SRCFIL)
      IF (IER .EQ. 1) GO TO 1800
      IF (ALL) THEN
         XSNAME = SRCNAM(1)
         CALL UPCASE(XSNAME)
         NSRC = NSRC+1
         IF (NSRC.GT.MAXSRC) CALL ERROR('Too many sources')
         SNAME(NSRC) = XSNAME
         RA(NSRC) = SRCRA
         DEC(NSRC) = SRCDEC
         EPOCH(NSRC) = SRCEPO
         VEL(NSRC) = SRCVEL(1)
         FOUND(NSRC) = .TRUE.
      ELSE
         DO 1150 ISRC=1,NSRC
            DO 1140 ALIAS=1,5
               XSNAME = SRCNAM(ALIAS)
               CALL UPCASE(XSNAME)
               IF (SNAME(ISRC).EQ.XSNAME) THEN
                  RA(ISRC) = SRCRA
                  DEC(ISRC) = SRCDEC
                  EPOCH(ISRC) = SRCEPO
                  VEL(ISRC) = SRCVEL(1)
                  FOUND(ISRC) = .TRUE.
               END IF
 1140       CONTINUE
 1150    CONTINUE
      END IF
      GOTO 1100
 1800 CLOSE (UNIT=INDAT)
C
      WRITE (PR,618)
      OK = .TRUE.
      DO 1250 ISRC=1,NSRC
          IF (.NOT.FOUND(ISRC)) THEN
              WRITE(OUTC,641) SNAME(ISRC)
              OK = .FALSE.
          ELSE
C
C            Precession.
C
             IF (EPOCH(ISRC) .EQ. 'DATE') THEN
C               -- Catalog coordinates are geocentric apparent of
C                  date. No precession required.
                R1950(ISRC) = 0.0
                D1950(ISRC) = 0.0
                R2000(ISRC) = 0.0
                D2000(ISRC) = 0.0
             ELSE IF (EPOCH(ISRC) .EQ. 'B1950') THEN
C               -- Catalog coordinates are B1950. Convert to J2000
C                  and compute geocentric apparent coordinates.
                R1950(ISRC) = RA(ISRC)
                D1950(ISRC) = DEC(ISRC)
                CALL SLA_FK45Z(R1950(ISRC), D1950(ISRC), 1975D0,
     1                         R2000(ISRC), D2000(ISRC))
                CALL SLA_MAPQKZ(R2000(ISRC), D2000(ISRC), AMPRMS,
     1                          RA(ISRC), DEC(ISRC))
             ELSE IF (EPOCH(ISRC) .EQ. 'J2000') THEN
C               -- Catalog coordinates are J2000. Convert to B1950
C                  and compute geocentric apparent coordinates.
                R2000(ISRC) = RA(ISRC)
                D2000(ISRC) = DEC(ISRC)
                CALL SLA_FK54Z(R2000(ISRC), D2000(ISRC), 1975D0,
     1                         R1950(ISRC), D1950(ISRC), DR, DD)
                CALL SLA_MAPQKZ(R2000(ISRC), D2000(ISRC), AMPRMS,
     1                          RA(ISRC), DEC(ISRC))
             ELSE
                CALL ERROR(' Invalid coordinate epoch for '//
     1                       SNAME(ISRC))
             END IF
             CRAJ  = TFORM(R2000(ISRC), 'T',0,2,5,'   ')
             CDECJ = TFORM(D2000(ISRC), ' ',1,2,4,'   ')
             LRA   = TFORM(R1950(ISRC), 'T',0,2,5,'   ')
             LDEC  = TFORM(D1950(ISRC), ' ',1,2,4,'   ')
             CRAB  = TFORM(RA(ISRC),    'T',0,2,5,'   ')
             CDECB = TFORM(DEC(ISRC),   ' ',1,2,4,'   ')
             WRITE (PR, 619) SNAME(ISRC), LRA, LDEC, CRAJ, CDECJ,
     1                       CRAB, CDECB
          END IF
 1250 CONTINUE
C
C-----------------------------------------------------------------------
C Station coordinates.
C-----------------------------------------------------------------------
C
      NS = 0
      DO 2100 I=1,MAXS
          IF (VALS(19+I).EQ.BLANK) GOTO 2100
          NS = NS+1
          WRITE(STNAME(NS),'(A)') VALS(19+I)
          CALL UPCASE(STNAME(NS))
C         Get first lo.
          LO1(NS) = VALS(19+MAXS+MAXSR2+3+I)
          FOUND(NS) = .FALSE.
          IF (PLOT) WRITE(LABEL(I),'(I2)') I
 2100 CONTINUE
      IF (NS.LT.1) CALL ERROR('No stations')
      IF (PLOT) THEN
        LEGEND = ' '
        IX = 2
        DO 2150 IS=1,NS
        IF (IS.GT.9) IX = 1
        I = LEN1(LEGEND)+2
        LEGEND(I:) = STNAME(IS)(1:LEN1(STNAME(IS)))
     1               //'\u'//LABEL(IS)(IX:2)//'\d'
 2150   CONTINUE
      END IF  
C
      WRITE (PR,620)
C
      JSTA = 19+MAXS+MAXSR2+3+MAXS+1
      WRITE(STAFIL, '(10A8)') (VALS(I), I=JSTA, JSTA+9)
 2200 IER = RDSTA(INDAT, .TRUE., STAFIL)
      IF (IER .EQ. 1) GO TO 2400
      XSTNAM = STANAM
      CALL UPCASE(XSTNAM)
      DO 2300 M=1,NS
          IF (XSTNAM.NE.STNAME(M)) GOTO 2300
          ZALIM(M) = STAZAL
          HALIM(M) = STAHAL
          IF (HALIM(M).EQ.0.0) HALIM(M)=12.0
          HALIM(M) = ABS(HALIM(M))
          IF (ZALIM(M).EQ.0.0) ZALIM(M)=90.0
C         ! latitude (deg)
          TT = STALAT/RPDEG  
C         ! w longitude (deg)
          PP = STALON/RPDEG  
          RR = STAEL
          IF (STAEL.LT.6D6) CALL GEOID(RR,TT)
          SLAT(M) = DSIN(STALAT)
          CLAT(M) = DCOS(STALAT)
          T(M) = TT
          P(M) = PP/15D0
          X(M) = RR*CLAT(M)*DCOS(STALON)
          Y(M) = RR*CLAT(M)*DSIN(STALON)
          Z(M) = RR*SLAT(M)
          WRITE(PR,621) XSTNAM,RR,TT,PP,X(M),Y(M),Z(M),HALIM(M),ZALIM(M)
          FOUND(M) = .TRUE.
 2300 CONTINUE
      GOTO 2200
C
 2400 CLOSE(UNIT=INDAT)
      DO 2500 M=1,NS
          IF (FOUND(M)) GOTO 2500
          WRITE (OUTC,624) STNAME(M)
          OK = .FALSE.
 2500 CONTINUE
      IF (.NOT.OK) CALL
     1            ERROR('Unknown source(s) or unknown station(s)')
C
C     Check request for LST station.
C
      WRITE(LSTSTA,'(A)') VALS(19+MAXS+MAXSR2+1)
      CALL UPCASE(LSTSTA)
      SUBLST = 0.0
      DO 2700 M=1,NS
          IF (LSTSTA.EQ.STNAME(M)) THEN
              SUBLST = P(M)
          END IF
 2700 CONTINUE
C
C       If LST station is not being processed, give warning.
      IF (SUBLST.EQ.0.0 .AND. LSTSTA.NE.CBLANK) THEN
          WRITE(PR,810) LSTSTA
      END IF
C
C       Get  (Local time - UT) difference for local time labels.
      UTLOC = -1D0*VALS(19+MAXS+MAXSR2+2)
C
C-----------------------------------------------------------------------
C Begin loop for sources.
C-----------------------------------------------------------------------
C
      WRITE (PR,630)
      RACK = ' '
      DO 2710 J=1,121
          FRAME(J:J) = '-'
 2710 CONTINUE
      UTFRAM = FRAME
      LSTFRM = FRAME
      DO 2720 J=1,121,5
          FRAME(J:J) = '+'
          RACK(J:J)  = '|'
 2720 CONTINUE
      UTM = 24.0 - GMST/3600.0
      IUT1 = 1 - MOD(NINT(5.0*UTM),5)
      IF (IUT1.LT.1) IUT1 = IUT1+5
      IUT2 = (NINT(5.0*UTM)+IUT1-1)/5
      DO 2730 J=IUT1,121,5
          UTFRAM(J:J) = '+'
 2730 CONTINUE
      IF (SUBLST.NE.0.0) THEN
         XLSTM = 24.0 - SUBLST
         LST1 = 1 - MOD(NINT(5.0*XLSTM),5)
         IF (LST1.LT.1) LST1 = LST1 + 5
         LST2 = (NINT(5.0*XLSTM)+LST1-1)/5
         DO 2740 J=LST1,121,5
            LSTFRM(J:J) = '+'
 2740    CONTINUE
      END IF
C
C     How many sources per page? Header requires 5 lines (including
C     Formfeed line) + 2 lines if SUBLST requested. Footer requires
C     7 lines + 2 lines if UTLOC requested. Each source requires
C     NS+1 lines.
C
      LINES = LPP - 5 - 7
      IF (SUBLST.NE.0.0) LINES = LINES - 2
      IF (UTLOC.NE.0.0) LINES = LINES - 2
      NSRCPP = MAX(1,LINES/(NS+1))
C     WRITE(*,*) LINES, NS, NSRCPP
C
      DO 8000 ISRC=1,NSRC
C
          CRAJ  = TFORM(R2000(ISRC), 'T',0,2,5,'   ')
          CDECJ = TFORM(D2000(ISRC), ' ',1,2,4,'   ')
          LRA   = TFORM(R1950(ISRC), 'T',0,2,5,'   ')
          LDEC  = TFORM(D1950(ISRC), ' ',1,2,4,'   ')
C
C         Find the Doppler calibration for the source.  Use value
C         for Earth orbit and sun wrt LSR only ignore Earth rotation.
C         The routine that reads the catalog sets VEL to 'CONT' if
C         no velocity is given.
C
          IF (VEL(ISRC) .NE. CONT) THEN
             VSUN = SLA_RVLSRK(REAL(R2000(ISRC)),REAL(D2000(ISRC)))
             CALL SLA_ECOR(REAL(RA(ISRC)), REAL(DEC(ISRC)), 
     1                     IY, DOY, 0.5, VEARTH, TL)
             FLINE = LINEFR * (1.0 - (VEL(ISRC)+VSUN+VEARTH)*1.0E3/VELC)
             WRITE(VNOTE, 642) VEL(ISRC), FLINE
          ELSE
             VNOTE = ' '
          END IF
C
          CDEC(ISRC) = DCOS(DEC(ISRC))
          SDEC(ISRC) = DSIN(DEC(ISRC))
          IF (MOD(ISRC-1,NSRCPP).EQ.0) THEN
C           ! do page header
            WRITE (PR2,631) FORM
            IF (SUBLST.EQ.0.0) THEN
                WRITE (PR2,660)
            ELSE
                WRITE (PR2,662) LSTSTA
                WRITE (PR2,663) '  LST',CBLANK(1:LST1), 
     1                          (MOD(J,24),J=LST2,LST2+23)
                WRITE (PR2,670) LSTFRM
            END IF
            WRITE (PR2,665) (J,J=0,24)
            WRITE (PR2,670) FRAME
          END IF
          IF (PLOT) THEN
            CALL PGBBUF
            CALL PGSCI(1)
            IF (PARAL) THEN
                CALL PGENV(0.,24.,-180.,180.,0,1)
                CALL PGLAB('G.S.T.',
     1                  'Elevation / Parallactic angle',SNAME(ISRC))
            ELSE IF (DOAIRM) THEN
                CALL PGENV(0.,24.,0.999,AIRMAS,0,0)
C                  ! Plot scale
                CALL PGLAB('G.S.T.','Airmass',SNAME(ISRC))
C                  ! Label
            ELSE
                CALL PGENV(0.,24.,0.,99.,0,0)
                CALL PGLAB('G.S.T.','Elevation',SNAME(ISRC))
            END IF
          END IF
C
C            Determine rise and set time at each station
C
      DO 4200 IS=1,NS
          COSZAL = DCOS(ZALIM(IS)*RPDEG)
          COSHAL = (COSZAL-SLAT(IS)*SDEC(ISRC))/(CLAT(IS)*CDEC(ISRC))
          IF (COSHAL.GE.1.0) THEN
            NOTE = ' Never visible'
            KODE = 1
            HAL = 0.0
          ELSE IF (COSHAL.LE.-1.0 .AND. HALIM(IS).GE.12.0) THEN
            NOTE = '  Circumpolar '
            KODE = -1
            HAL = 12.0
          ELSE IF (COSHAL.LE.-1.0) THEN
            KODE = 0
            HAL = HALIM(IS)
          ELSE
            KODE = 0
            HAL = ACOS(COSHAL)/RPDEG/15.0
            HAL = AMIN1(HAL,HALIM(IS))
          END IF
          RISE = DMOD(RA(ISRC)/RPHR - HAL + P(IS), 24D0)
          IF (RISE.LT.0.0) RISE = RISE + 24.0
          SET  = DMOD(RA(ISRC)/RPHR + HAL + P(IS), 24D0)
          IF (SET.LE.0.0) SET = SET + 24.0
          IF (KODE.EQ.0) WRITE(NOTE,740) RISE,SET
          IF (KODE.EQ.1) THEN
            NSEG = 0
          ELSE IF (KODE.EQ.-1) THEN
            NSEG = 1
            J1(1) = 1
            J2(1) = 121
          ELSE IF (RISE.LT.SET) THEN
            NSEG = 1
            J1(1) = NINT(5*RISE) + 1
            J2(1) = NINT(5*SET) + 1
          ELSE
            NSEG = 2
            J1(1) = 1
            J2(1) = NINT(5*SET) + 1
            J1(2) = NINT(5*RISE) + 1
            J2(2) = 121
          END IF
          BAR = RACK
          LL = LEN1(STNAME(IS))
C         ! Loop count may be zero !
          DO 4100 KSEG=1,NSEG
            K = 0
            NPLOT = 0
            NMNAIR(KSEG) = NPLOT
C              ! Init. pointer to lowest airmass
            MINAIR(KSEG) = AIRMAS+1.
C              ! Init. value of lowest airmass
            DO 4050 J=J1(KSEG),J2(KSEG)
                K= MOD(K,LL)+1
                BAR(J:J) = STNAME(IS)(K:K)
                NPLOT = NPLOT+1
                XP(NPLOT) = 0.2*(J-1)
                HARAD = (XP(NPLOT)-P(IS))*RPHR - RA(ISRC)
                YP(NPLOT) = ASIN(SLAT(IS)*SDEC(ISRC) +
     1                  CLAT(IS)*CDEC(ISRC)*COS(HARAD))/RPDEG
                IF (DOAIRM) THEN
C                  -- Replace elev. by airmass in YP
                   IF (YP(NPLOT).GT.1.E-5) THEN
                      YP(NPLOT) = 1./SIN(RPDEG*YP(NPLOT))
                   ELSE
                      YP(NPLOT) = 6.E6
                   ENDIF
                   IF (YP(NPLOT).LT.MINAIR(KSEG)) THEN
                      NMNAIR(KSEG) = NPLOT
                      MINAIR(KSEG) = YP(NPLOT)
                   ENDIF
                ENDIF
                ZP(NPLOT) = ATAN2(CLAT(IS)*SIN(HARAD),
     1                  SLAT(IS)*CDEC(ISRC) -
     2                  CLAT(IS)*SDEC(ISRC)*COS(HARAD))/RPDEG
 4050       CONTINUE
            IF (PLOT) THEN
              CALL PGSCI(2+MOD(IS,6))
              CALL PGLINE(NPLOT,XP,YP)
              CALL PGSCH(0.5)
              IF (KSEG.EQ.1) THEN
                 IF (DOAIRM) THEN
                    IF (IS.GT.9)
     1                CALL PGTEXT(XP(NMNAIR(1)),
     2                  YP(NMNAIR(1))-.0125*(AIRMAS-1.),LABEL(IS))
                    IF (IS.LT.9)
     1                CALL PGTEXT(XP(NMNAIR(1)),
     2                  YP(NMNAIR(1))-.0125*(AIRMAS-1.),LABEL(IS)(2:2))
                 ELSE
                    IF (IS.GT.9)
     1                CALL PGTEXT(XP(NPLOT),YP(NPLOT),LABEL(IS))
                    IF (IS.LT.9) 
     1                CALL PGTEXT(XP(NPLOT),YP(NPLOT),LABEL(IS)(2:2))
                 ENDIF
              ELSE IF (KSEG.EQ.2) THEN
                 IF (DOAIRM) THEN
                    IF (IS.GT.9)
     1                CALL PGTEXT(XP(NMNAIR(2)),
     2                  YP(NMNAIR(2))-.0125*(AIRMAS-1.),LABEL(IS))
                    IF (IS.LT.9)
     1                CALL PGTEXT(XP(NMNAIR(2)),
     2                  YP(NMNAIR(2))-.0125*(AIRMAS-1.),LABEL(IS)(2:2))
                 ELSE
                    CALL PGPTXT(XP(1),YP(1),0.0,1.0,LABEL(IS))
                 ENDIF
              END IF
              IF (PARAL) THEN
                CALL PGSLS(3)
                CALL PGLINE(NPLOT,XP,ZP)
                IF (KSEG.EQ.1) THEN
                 IF (IS.GT.9) CALL PGTEXT(XP(NPLOT),ZP(NPLOT),LABEL(IS))
                 IF (IS.LT.9) 
     1             CALL PGTEXT(XP(NPLOT),ZP(NPLOT),LABEL(IS)(2:2))
                ELSE IF (KSEG.EQ.2) THEN
                 CALL PGPTXT(XP(1),ZP(1),0.0,1.0,LABEL(IS))
                END IF
                CALL PGSLS(1)
              END IF
              CALL PGSCH(1.0)
            END IF
 4100     CONTINUE
C
C         Get second LO for center of line.
C
          IF (VEL(ISRC) .NE. CONT) THEN
             BBC = FLINE - LO1(IS)
             WRITE(VNOTE(17:25), 643) BBC
          END IF
C
          IF (IS.EQ.1) THEN
             WRITE(PR,640) SNAME(ISRC), 
     :                    STNAME(IS),ZALIM(IS),HAL,NOTE,VNOTE
             WRITE(PR2,670) RACK
             WRITE(PR2,680) SNAME(ISRC),BAR
          ELSE
             WRITE(PR,650) STNAME(IS),ZALIM(IS),HAL, NOTE
             WRITE(PR2,670) BAR
          END IF
 4200  CONTINUE
C
          IF (ISRC.EQ.NSRC .OR. MOD(ISRC,NSRCPP).EQ.0) THEN
C           ! do page footer
            WRITE(PR2,670) RACK
            WRITE(PR2,670) FRAME
            WRITE(PR2,665) (J,J=0,24)
            WRITE(PR2,670) UTFRAM
            WRITE(PR2,663) '   UT',CBLANK(1:IUT1), 
     1                     (MOD(J,24),J=IUT2,IUT2+23)
            IF (UTLOC.NE.0.0) THEN
               WRITE(PR2,670) UTFRAM
               WRITE(PR2,663) 'Local',CBLANK(1:IUT1), 
     1              (MOD(J,24),J=24+IUT2+UTLOC,24+IUT2+23+UTLOC)
               IF (UTLOC.LT.0.0) WRITE(PR2,667) IY,MNAME,ID,UTLOC
               IF (UTLOC.GT.0.0) WRITE(PR2,668) IY,MNAME,ID,UTLOC
            ELSE
               WRITE(PR2,661) IY,MNAME,ID
            END IF
          END IF
C
      IF (PLOT) THEN
         CALL PGSCH(0.6)
         CALL PGMTXT('T',1.2,0.5,0.5,LEGEND)
         CALL PGSCH(1.0)
         CALL PGEBUF
      END IF
C
 8000 CONTINUE
C
C Append scratch file to listing.
C
      REWIND (UNIT=PR2)
 8010 READ (PR2,'(A)',END=8020) RECORD
      LR = MAX(1,LEN1(RECORD))
      WRITE (PR,'(A)') RECORD(1:LR)
      GOTO 8010
 8020 CLOSE (UNIT=PR2, STATUS='DELETE')
      CLOSE (UNIT=PR)
      IF (PLOT) CALL PGEND
C-----------------------------------------------------------------------
      END
