      PROGRAM UVCOPY
C-----------------------------------------------------------------------
C UVCOPY: to copy one MERGE-format file to another. A subset of the 
C input data can be selected, by UVRANGE, TIMERANGE, or AMPRANGE.
C
C Parameters:
C      INPUT = input file name
C      OUTPUT = output file name
C      HISTORY = text to be written in header
C      UVRANGE = uvmin, uvmax (wavelengths)
C      TIMERANGE = day1, ut1, day2, ut2
C      AMPRANGE = amin, amax
C
C T.J. Pearson, standard Fortran 77.
C
C Subroutines required:
C      KEYIN, LEN1, MKHIST, PUTOUT, VLBOPE, TIMEC.
C
C Version 1.0:  1989 Jul 13.
C Version 1.1:  1989 Oct 31: correct error in dimension of ADDHIS.
C Version 1.2:  1990 Nov 13: remove unused variables.
C Version 1.3:  1993 May 14: accept level 2 merge files; add AMPRANGE;
C                            maximum 40 stations.
C-----------------------------------------------------------------------
C
C Constants.
C
      CHARACTER*(*) PROGNM, VERSN
      INTEGER INC,OUTC,INDAT,OUTDAT,MAXS,MAXB,MAXT,NPARS
      DOUBLE PRECISION PI,SIDR,VELC
C     -- Program version and date:
      PARAMETER (PROGNM='UVCOPY')
      PARAMETER (VERSN='1.3 - 1993 May 14')
C     -- Fortran unit numbers used:
      PARAMETER (INC=5,OUTC=6,INDAT=8,OUTDAT=9)
C     -- Maximum numbers of stations, baselines, and triangles:
      PARAMETER (MAXS=40)
      PARAMETER (MAXB=(MAXS-1)*MAXS/2)
      PARAMETER (MAXT=MAXB*(MAXS-2)/3)
C     -- Number of input parameters:
      PARAMETER (NPARS=38)
C     -- Pi:
      PARAMETER (PI=3.141592653589793D0)
C     -- Radians of ST per sec of UT:
      PARAMETER (SIDR=(PI/2592000.0D0)*1.0027379093D0)
C     -- Speed of light:
      PARAMETER (VELC=2.997925D8)
C-----------------------------------------------------------------------
C Variables.
C
      CHARACTER*8  STNAME(MAXS)
      CHARACTER*16 SOURCE
      INTEGER   IUT, IS, IB, IT, NS, NB, NT, I, J, IER, MODE
      INTEGER   IZ, IY, IS1, IS2, NHIST
      INTEGER   NNEW, NDWPB
      INTEGER   NBI(MAXB), NB1(MAXB), NB2(MAXB)
      INTEGER   NTI(MAXT)
      INTEGER   REC1(20), IDUM(8), LEVEL
      REAL      INREC(7*MAXB+2*MAXT)
      REAL      U, V, UV, UVMIN, UVMAX, AMIN, AMAX, AMP
      INTEGER   UTMIN, UTMAX
      DOUBLE PRECISION STX(MAXS), STY(MAXS), STZ(MAXS),
     1          RA, DEC, RAP, DECP, GASTM, FREQ, FLUX,
     2          WAVE,BX,BY,BZ(MAXB),BOFF(MAXB),BXY(MAXB),TIHA,
     3          COSDEC,SINDEC, TINTG, AVG, BW,
     4          PARS(NPARS),VALS(NPARS),ENDMRK
      CHARACTER*80 INDSN, OUTDSN, OLDHIS, HISTRY, ADDHIS(4)
C-----------------------------------------------------------------------
C Functions called.
C
      CHARACTER*8 TIMEC
      INTEGER   LEN1, VLBOPE
C-----------------------------------------------------------------------
C Names and default values of KEYIN parameters.
C
      DATA  PARS/ 'Input',   9*' ',
     1            'Output',  9*' ',
     2            'History', 9*' ',
     3            2*'UVrange', 
     4            4*'Timerang',
     5            2*'Amprange'/
      DATA  VALS/ '***UNSET',9*' ',
     1            '***UNSET',9*' ',
     2            10*' ',
     3            0D0, 1D38,
     4            1D0, 0D0, 367D0, 0D0,
     5            0D0, 1D38 /
      DATA  ENDMRK/ '/' /
C-----------------------------------------------------------------------
C
C Introduction.
C
      CALL PUTOUT(PROGNM//' copies part of a MERGE-format uv file')
      CALL PUTOUT('(Version '//VERSN//')')
C
C Control parameters.
C
      MODE = 0
  100 IF (MODE.NE.0) CALL ERROR('Bad input parameters in '//PROGNM)
      CALL PUTOUT(' ')
      CALL PUTOUT('Parameters (INPUT, OUTPUT, '//
     1            'HISTORY, UVRANGE, TIMERANGE, AMPRANGE):')
      CALL KEYIN(PARS,VALS,NPARS,ENDMRK,MODE,INC,OUTC)
      WRITE (INDSN,'(10A8)') (VALS(I),I=1,10)
      WRITE (OUTDSN,'(10A8)') (VALS(I),I=11,20)
      WRITE (HISTRY,'(10A8)') (VALS(I),I=21,30)
      UVMIN = VALS(31)
      UVMAX = VALS(32)
      UTMIN = 24*3600*60*NINT(VALS(33)-1) + 60*VALS(34)
      UTMAX = 24*3600*60*NINT(VALS(35)-1) + 60*VALS(36)
      AMIN  = VALS(37)
      AMAX  = VALS(38)
C
C Open input Merge file.
C
      IER = VLBOPE(INDAT, INDSN, 'MERGE', 'OLD', INDSN)
      IF (IER.NE.1) GOTO 100
C
C Open output Merge file.
C
      IER = VLBOPE(OUTDAT, OUTDSN, 'MERGE', 'NEW', OUTDSN)
      IF (IER.NE.1) THEN
          CLOSE (UNIT=INDAT)
          GOTO 100
      END IF
C
C Make the new history records.
C
      CALL MKHIST(PROGNM, VERSN, ADDHIS(1))
      IF (HISTRY.NE.' ') THEN
          NNEW = 5
          ADDHIS(5) = '- '//HISTRY(1:78)
      ELSE
          NNEW = 4
      END IF
      WRITE (ADDHIS(2), '(''- UVRANGE = '',1PE12.4,'', '',E12.4)')
     1        UVMIN, UVMAX
      WRITE (ADDHIS(3), '(''- TIMERANGE = '',I3,''/'',A8,
     1                                 '','',I3,''/'',A8)')
     2        UTMIN/(60*86400)+1, TIMEC(MOD(UTMIN/60,86400)),
     3        UTMAX/(60*86400)+1, TIMEC(MOD(UTMAX/60,86400))
      WRITE (ADDHIS(4), '(''- AMPRANGE = '',1PE12.4,'', '',E12.4)')
     1        AMIN, AMAX
      DO 105 I=1,NNEW
          CALL PUTOUT(ADDHIS(I)(1:LEN1(ADDHIS(I))))
  105 CONTINUE
C
C Read and re-write header. 
C
      READ (INDAT,END=82,ERR=82) (REC1(J),J=1,20)
      LEVEL = REC1(1)
      IF (LEVEL.NE.1 .AND. LEVEL.NE.2) GOTO 82
      NDWPB = 4
      IF (LEVEL.EQ.2) NDWPB = 7
      NHIST = REC1(2)
      NS    = REC1(4)
      NB    = REC1(5)
      NT    = REC1(6)
      IF (NS.LT.2) GOTO 82
      IF (NS.GT.MAXS) GOTO 82
      REC1(2) = REC1(2) + NNEW
      REC1(3) = REC1(3) + NNEW
      WRITE (OUTDAT) (REC1(J),J=1,20)
C
      IF (NB.GT.0) THEN
          READ (INDAT,END=82,ERR=82) (NBI(IB),IB=1,NB)
          WRITE (OUTDAT) (NBI(IB),IB=1,NB)
          DO 110 IB=1,NB
              NB1(IB) = NBI(IB)/100
              NB2(IB) = NBI(IB) -  NB1(IB)*100
  110     CONTINUE
      END IF
C
      IF (NT.GT.0) THEN
          READ (INDAT,END=82,ERR=82) (NTI(IT),IT=1,NT)
          WRITE (OUTDAT) (NTI(IT),IT=1,NT)
      END IF
C
      DO 120 I=1,NHIST
          READ (INDAT,END=82,ERR=82) OLDHIS
          WRITE (OUTDAT) OLDHIS
  120 CONTINUE
      DO 125 I=1,NNEW
          WRITE (OUTDAT) ADDHIS(I)
  125 CONTINUE
C
      READ (INDAT,END=82,ERR=82) SOURCE,RA,DEC,RAP,DECP,FLUX
      WRITE (OUTDAT) SOURCE,RA,DEC,RAP,DECP,FLUX
C
      DO 130 IS=1,NS
          READ (INDAT,END=82,ERR=82) STNAME(IS),STX(IS),STY(IS),STZ(IS)
          WRITE(OUTDAT) STNAME(IS),STX(IS),STY(IS),STZ(IS),
     1            (0.D0,J=5,10),0
  130 CONTINUE
C
      READ (INDAT,END=82,ERR=82) IY, IZ, GASTM, FREQ, TINTG,
     1            AVG, BW, (IDUM(I),I=1,8)
      WRITE (OUTDAT) IY, IZ, GASTM, FREQ, TINTG,
     1            AVG, BW, (IDUM(I),I=1,8)
C
C Baseline parameters (BXY and BZ are components in wavelengths; 
C BOFF is "hour angle").
C
      WAVE   = VELC/FREQ
      COSDEC = DCOS(DECP)
      SINDEC = DSIN(DECP)
      DO 140 IB=1,NB
          IS1 = NB1(IB)
          IS2 = NB2(IB)
          BX  = (STX(IS1)-STX(IS2))/WAVE
          BY  = (STY(IS1)-STY(IS2))/WAVE
          BZ(IB) = (STZ(IS1)-STZ(IS2))/WAVE
          BXY(IB) = DSQRT(BX*BX+BY*BY)
          IF (BXY(IB).EQ.0D0) THEN
            CALL PUTOUT('Station coordinates missing.')
            GOTO 82
          END IF
          BOFF(IB) = GASTM-RAP-PI/2.0-DATAN2(BY,BX)
  140 CONTINUE
C
C Read and re-write data records.
C
  300 READ(INDAT,END=400) IUT,(INREC(J),J=1,NDWPB*NB+2*NT)
      IF (IUT.LT.0) GOTO 400
C     -- Skip records outside TIMERANGE.
      IF (IUT.LT.UTMIN .OR. IUT.GT.UTMAX) GOTO 300
C     -- Compute (u,v,w) for each data point; flag data outside UVRANGE.
C        and AMPRANGE
      DO 310 IB=1,NB
          AMP = INREC(NDWPB*(IB-1)+1)
          IF (LEVEL.EQ.1) THEN
              TIHA = (IUT-IZ)*SIDR + BOFF(IB)
              U = BXY(IB)*DCOS(TIHA)
              V = BZ(IB)*COSDEC + BXY(IB)*SINDEC*DSIN(TIHA)
C%            W = BZ(IB)*SINDEC - BXY(IB)*COSDEC*DSIN(TIHA)
          ELSE
              U = INREC(NDWPB*(IB-1)+5)
              V = INREC(NDWPB*(IB-1)+6)
C%            W = INREC(NDWPB*(IB-1)+7)
          END IF
          UV = SQRT(U*U + V*V)
          IF (UV.LT.UVMIN .OR. UV.GT.UVMAX .OR.
     1        AMP.LT.AMIN .OR. AMP.GT.AMAX) THEN
              INREC(NDWPB*(IB-1)+2) = -ABS(INREC(NDWPB*(IB-1)+2))
              INREC(NDWPB*(IB-1)+4) = -ABS(INREC(NDWPB*(IB-1)+4))
          END IF
  310 CONTINUE
      WRITE (OUTDAT) IUT,(INREC(J),J=1,NDWPB*NB+2*NT)
      GOTO 300
C
C Write final record and quit.
C
  400 WRITE (OUTDAT) -1,(0.0,J=1,NDWPB*NB+2*NT)
      CLOSE (UNIT=INDAT)
      CLOSE (UNIT=OUTDAT)
      STOP
C
C Error exit. Delete the nascent output file.
C
   82 CLOSE (UNIT=OUTDAT, STATUS='DELETE')
      CLOSE (UNIT=INDAT)
      CALL ERROR('Something wrong with input MERGE file.')
      STOP
C-----------------------------------------------------------------------
      END
