   	PROGRAM UVCROSS
	IMPLICIT NONE
C-----------------------------------------------------------------------
C UVCROSS: To determine corrections to the station gains based on U,V
C          crossing points or close neighbours.
C
C VAX-11 Fortran.
C
C Original program by M.W. Hodges.
C Substantially changed version 2 by R.C. Vermeulen
C
C History:
C
C Version 1.0:  1984 Nov 12 - initial program written
C Version 1.1:  1986 Jul 23 - general clean up; add keyin of parameters
C Version 1.2:  1986 Jul 31 - include SNR's in the solution
C Version 1.3:  1986 Aug 21 - fix MAXV bug; add UT's for crossings
C		1986 Aug 22 - declare MAXV as real (SCU)
C Version 1.4:  1987 Jan 26 - fix GETFLUX to look for multiple days
C Version 1.5:  1989 Feb 23 - fix it with maximum crossing points upto 1000.
C               From: ST%"zhang%buasta@buenga.BU.EDU" 23-FEB-1989 15:52
C Version 1.6:  1989 Aug 30 - correct bug in computing number of 
C                             stations (TJP).
C
C Version 2.0:  1990 Apr 02   Overhaul: Program substantially rewritten. 
C                             - Main program; search for close points:
C                               - Emphasis now on UV constraints, not time
C                                 limits. No theoretical crossings calculated;
C                                 "near misses" generalised so "true
C                                 crossings" no longer special case.
C                               - Data weighting options more elaborate.
C                             - SOLVE subroutine (see comments there too):
C                               - Closer to true unbiased least-squares.
C                               - Solve in clusters of dependent stations.
C                               - NOSOLVE options added.
C                                                Rene C. Vermeulen
C Version 2.1   1990 May 04 - Proper handling of baselines/stations without
C                             data.
C                           - Add EDITFILE, MSGLEVEL.
C                           - Minor bugfixes and cleanup of output. 
C                                                Rene C. Vermeulen
C Version 2.2   1990 Dec 07 - CPU-time saved: V sorting, bracketing, faster sort
C                           - Remove infinite loop on file-opening errors,
C                             by introduction of error-count NERR.
C                           - ZALIM and UVPOWER parameters added.
C                           - Annotation expanded, NCPRINT introduced.
C                           - Write # records rather than points in editfile.
C                           - More real*8 calculations.
C                                                Rene C. Vermeulen
C Version 2.3   1991 Apr 23 - Bug-fix in errorbar weighting scheme.
C                           - UVTAPER now depends on UVPOWER.
C                           - Fix inconsistency in handling of "4-way" ratios.
C                           - Expand and clarify handling of 4-way ratios.
C                           - Add TRADIUS and UVWINDOW options.
C                           - Change default MSGLEVEL from 0 to 1.
C                           - Print minimum crossing distances in listfile.
C                                                Rene C. Vermeulen
C Version 2.4   1992 Sep 14 - Add "errmult" lines to EDITFILE.
C                                                Rene C. Vermeulen
C Version 2.5   1993 Dec 13 - Increase from 20 to 30 stations: TJP.
C This code already contains some foundations for the next major update,
C which should be invisible to the user.
C
C-----------------------------------------------------------------------
	CHARACTER VERSION*(*)
	INTEGER   INC,OUTC,INDAT,PR,PR2,MXNERR
	INTEGER   MAXS,MAXB,MAXT,MAXBC,MAXWPB,MAXP,NPARS,NPARS1
	PARAMETER (VERSION='2.5 - 1993 Dec 13')
	PARAMETER (INC=5,OUTC=6,INDAT=8,PR=10,PR2=11)
	PARAMETER (MAXS=30)	! maximum number of stations
	PARAMETER (MAXB=(MAXS-1)*MAXS/2)	! Max. no. baselines
	PARAMETER (MAXT=MAXB*(MAXS-2)/3)	! Max. no. triangles
	PARAMETER (MAXBC=MAXB*(MAXB-1)/2)	! Max. no. basel. crossings
	PARAMETER (MAXWPB=7)	! maximum words per baseline
 	PARAMETER (MAXP=65536)	! maximum number of U,V points overall
	PARAMETER (NPARS1=39)	! Input parameters excluding "NOSolve"
	PARAMETER (NPARS=NPARS1+MAXS)	! All input parameters
	PARAMETER (MXNERR=3)	! Maximum number of (input) errors before abort
C
	REAL*8    PI,HALFPI,SIDR,VELC,RPDEG
	PARAMETER (PI=3.141592653589793D0)	! pi
	PARAMETER (HALFPI=PI/2.0D0)
	PARAMETER (SIDR=(PI/2592000.0D0)*1.0027379093D0)
C				! radians of ST per 1/60 sec of UT
	PARAMETER (VELC=2.997925D8)		! speed of light
	PARAMETER (RPDEG=PI/180.0D0)	! Radians per degree
C
	CHARACTER*32 OPSYS
	LOGICAL VMS
C
	EXTERNAL TIMEC,LEN1
	CHARACTER*8 TIMEC
	INTEGER LEN1
C
	INTEGER IUNITS
C
	CHARACTER*8  STNAME(MAXS)
	CHARACTER*16 SOURCE
	LOGICAL   ORBIT
	INTEGER   NS,NB,NT,NB1,NB2,NC1,NC2,NC3,NVER,NDWPB,STATYP
        REAL*8    STX,STY,STZ,RA,DEC,GASTM,FREQ,FLUX,TINTG,AVG,BW,AEIOWM
C
        COMMON    /PARMS/ STNAME,STX(MAXS),STY(MAXS),STZ(MAXS),
     1            SOURCE,RA,DEC,GASTM,FREQ,FLUX,TINTG,AVG,BW,
     2            AEIOWM(6,MAXS),STATYP,ORBIT
	COMMON    /ORDER/ NS,NB,NT,NB1(MAXB),NB2(MAXB),NC1(MAXT),
     1	          NC2(MAXT),NC3(MAXT), NVER, NDWPB
C
	CHARACTER*8 TMP1,TMP2
	CHARACTER*40 LEGEN1
	CHARACTER*(9*MAXS) LEGEN2
	CHARACTER*20 NOW
	CHARACTER*17 BNAME(MAXB)
	CHARACTER*16 DOTS,BLANK
	LOGICAL   NEWK,SORTU,WEIRDPAR
	INTEGER   IUT,MODE,I,J,K,L,IZ,IF,ID,IY,DAY,NERR
	INTEGER   LDOTS1,LDOTS2,LDOTS3,LDOTS4
	INTEGER   IMON,JD,IS,IB,IS1,IS2,L1,L2,MONTH,MXLBN
	INTEGER   LSTART,LSTARTR,FIRSTK,LASTK,FIRSTL,LASTL
	INTEGER   IST,IEN,JST,JEN
	INTEGER   ISTU,IENU,JSTU,JENU,ISTV,IENV,JSTV,JENV
	INTEGER   IND1,IND2,NREC,NREJEC,NPOINT,NBCLOSE,NTOT,NTOTR
	INTEGER   CLOSTOT,CLOSTOTR
	INTEGER   TREF,CTIM
	INTEGER   TIME(MAXP)
	INTEGER   BASEL(MAXP),CNTOT(MAXP),INDXU(MAXP),INDXV(MAXP)
	INTEGER   NBPOINT(MAXB),BSTART(MAXB)
	INTEGER   BAS1(MAXBC),BAS2(MAXBC)
	REAL      URAD,VRAD,UTAP,VTAP,CLOSUV,CLOSUVA
	REAL 	  UP,VP,DU,DV,DUV,UREF,VREF,UBEG,UEND,VBEG,VEND,WT
	REAL      DUM1,DUM2,DUM3
	REAL      LAT(MAXS),LON(MAXS)
	REAL      SINLAT(MAXS),COSLAT(MAXS),SINLON(MAXS),COSLON(MAXS)
	REAL      REC(MAXWPB*MAXB)
	REAL 	  AMP(MAXP),ERR(MAXP),U(MAXP),V(MAXP)
	REAL*8    RATIO,SIGMA
	REAL*8    AMPTOT,AMPTOTR,ER2TOT,ER2TOTR,WTTOT,WTTOTR
	REAL*8    PAIRWT,PAIRWTR,PAIRER2,PAIRER2R
	REAL*8    TIMEP,WAVE,BX,BY,COSDEC,SINDEC,HA,ZA
	REAL*8    BZ(MAXB),BOFF(MAXB),BXY(MAXB)
	REAL*8    GAIN(MAXS)
	REAL*8    CAMP(MAXP),CER2(MAXP)
	REAL*8    BRATIO(MAXBC),BWTTOT(MAXBC)
C
	CHARACTER*64 INDSN,LISTDSN,EDITDSN
	LOGICAL   EDIF,LIST,SQUARE,ABSERR,NOSOLVE(MAXS)
	INTEGER   TDIFF, MSGLEVEL, NCPRINT,TRADIUS
        REAL      URADB,VRADB,WTPOWER,UTAPB,VTAPB,ZALIM
	REAL      UVPOWER,UWINDOW,VWINDOW
C
	REAL*8    PARS(NPARS),VALS(NPARS),ENDMRK
	DATA  PARS/ 
     #              'Input   ', 7*'        ',	! 64 characters
     #              'Editfile', 7*'        ',	! 64 characters
     #              'Listfile', 7*'        ',	! 64 characters
     #              'Msglevel',
     #              'NCprint ',
     #              'UVRadius',   'UVRadius',	! array of 2
     #              'UVPower ',
     #              'DORtangl',
     #              'DOAbserr',
     #              'Wtpower ',
     #              'UVTaper ',   'UVTaper ',	! array of 2
     #              'TDiff   ',
     #              'Zalimit ',
     #              'TRadius ',
     #              'UVWindow',   'UVWindow',	! array of 2
     #              MAXS*'NOSolve '		! array of MAXS
     #              /
	DATA  VALS/ '***UNSET', 7*'        ',	! Input filename must be given
     #              '        ', 7*'        ',	! No editfile by default
     #              'UVCROSS.',   'LIS     ', 6*'        ',	! Listfile-name
     #              1D0,			! DO make a listfile 
     #              0D0,			! NCprint=0 means all !!
     #              '***UNSET',   0D0,		! UVRadius must be given
     #              0D0,			! UVRadius independent of U,V
     #              -1D0,			! DORtangl=False
     #              -1D0,			! DOAbserr=False
     #              -1D0,			! Wtpower=-1
     #              0D0,          0D0,		! No UVTAPER
     #              0D0,			! No time difference limit
     #              90D0,			! Full horizon-horizon tracks 
     #              0D0,			! No time window limit
     #              0D0,          0D0,		! No UV-window limit
     #              MAXS*'        '		! No stations left out
     #              /
	DATA  ENDMRK/ '/' /
C
	DATA  BLANK  / '                ' /
	DATA  DOTS   / '................' /
C
C
	NERR=0		! No errors as yet
	WEIRDPAR=.FALSE.
C
C		Are we on VMS or UNIX ?
C
	CALL SYSVER(OPSYS)
	VMS = INDEX(OPSYS,'VMS').NE.0
C
C		Introduction.
C
	CALL DATTIM(NOW)
	WRITE (OUTC,'(A)')' '
	WRITE (OUTC,'(A)')
     #   ' UVCROSS calculates gain correction factors from MERGE files.'
	WRITE(OUTC,'(A)')' Version '//VERSION
	WRITE (OUTC,'(A)')' '
C
C		Read control parameters.
C
100	CONTINUE
	WRITE (OUTC,'(A)') '   Please specify: Input and UVRadius '
     #                   //'(in MegaLamda).'
	WRITE (OUTC,'(A)') '   Optionally:     Editfile, Listfile, '
     #                   //'Msglevel, NCprint, UVPower, DORtangl,'
	WRITE (OUTC,'(A)') '                   DOAbserr, Wtpower, '
     #                   //'UVTaper (MegaLamda), TDiff (min),'
	WRITE (OUTC,'(A)') '                   Zalimit (deg), '
     #                   //'TRadius (min), UVWindow (MegaLamda), '
     #                   //'NOSolve.'
	WRITE(OUTC,'(A)') ' '
	MODE = 0
	CALL KEYIN(PARS,VALS,NPARS,ENDMRK,MODE,INC,OUTC)
C
	WRITE(INDSN,'(8A8)')(VALS(I),I=1,8)
	WRITE(EDITDSN,'(8A8)')(VALS(I),I=9,16)
	WRITE(LISTDSN,'(8A8)')(VALS(I),I=17,24)
	MSGLEVEL = VALS(25)
	NCPRINT = ABS(VALS(26))
	URADB = ABS(VALS(27))		! Millions of wavelengths
	VRADB = ABS(VALS(28))		! Millions of wavelengths
	IF (URADB.EQ.0.) URADB=VRADB	! Order of these two IF statements
	IF (VRADB.EQ.0.) VRADB=URADB	! is important !!!
	IF (URADB.EQ.0.) THEN
	   WRITE(OUTC,'(A)')' '
	   WRITE(OUTC,'(A)')' UVRadius cannot be zero !'
	   WRITE(OUTC,'(A)')' '
	   NERR=NERR+1
	   IF (NERR.GT.MXNERR)
     #        CALL ERROR('ABORTING; ERROR COUNT EXCEEDED.')
	   GOTO 100
	ENDIF
	UVPOWER= VALS(29)
	IF (UVPOWER.EQ.0.) THEN
	   URAD=URADB
	   VRAD=VRADB
	ELSEIF (.NOT.(WEIRDPAR)) THEN
	   IF ((URADB.GE.1.).OR.(VRADB.GE.1.)) THEN
	      WRITE(OUTC,'(A)')' '
	      WRITE(OUTC,'(A)')' Uncommon combination of UVRADIUS and'
     #                         //' UVPOWER !!!'
	      WRITE(OUTC,'(A)')' If you really want this, then just'
     #                         //' restart the run...'
	      WRITE(OUTC,'(A)')' '
	      WEIRDPAR=.TRUE.
	      GOTO 100
	   ENDIF
	ENDIF
	SQUARE = VALS(30).NE.-1D0	! UVRADIUS in rectangl. grid (save time)
	ABSERR = VALS(31).NE.-1D0	! Weights from abs. not fractional err.
	WTPOWER= VALS(32)
	UTAPB = ABS(VALS(33))	! Millions of wavelengths
	VTAPB = ABS(VALS(34))	! Millions of wavelengths
	IF (VTAPB.EQ.0.) VTAPB=UTAPB	! Order of these two statements
	IF (UTAPB.EQ.0.) UTAPB=VTAPB	! is important !!!
	IF (UVPOWER.EQ.0.) THEN
	   UTAP=UTAPB
	   VTAP=VTAPB
	ENDIF
	TDIFF = NINT(VALS(35)*60.*60.)	! Max.dif.in obs.time betw.basel.in pair
	ZALIM = ABS(VALS(36))*RPDEG	! Zenith angle limit (deg -> radians)
	TRADIUS = NINT(ABS(VALS(37))*60.*60.)	! Max timespan along baselines
	UWINDOW = VALS(38)	! Millions of wavelengths
	VWINDOW = VALS(39)	! Millions of wavelengths
	IF (VWINDOW.EQ.0.) VWINDOW=UWINDOW	! Order of these two statements
	IF (UWINDOW.EQ.0.) UWINDOW=VWINDOW	! is important !!!
	IF (UWINDOW*VWINDOW.LT.0.) THEN
	   WRITE(OUTC,'(A)')' '
	   WRITE(OUTC,'(A)')' U-window and V-window have '
     #                      //'opposite signs !!!'
	   WRITE(OUTC,'(A)')' '
	   NERR=NERR+1
	   IF (NERR.GT.MXNERR)
     #        CALL ERROR('ABORTING; ERROR COUNT EXCEEDED.')
	   GOTO 100
	ENDIF
C
C		NOSOLVE can only be decoded with the station names in hand.
C
C               Open input Merge file.
C
	OPEN (UNIT=INDAT, NAME=INDSN, READONLY, STATUS='OLD',
     1       FORM='UNFORMATTED', ERR=1010)
	INQUIRE (UNIT=INDAT, NAME=INDSN)
C
C		Open output files (if required).
C
	EDIF = EDITDSN.NE.' '
	IF (EDIF) THEN
	   OPEN (UNIT=PR2, NAME=EDITDSN, STATUS='NEW',FORM='FORMATTED',
     1	      CARRIAGECONTROL='LIST',ERR=1025)
	   INQUIRE (UNIT=PR2, NAME=EDITDSN)
	   IF (VMS) THEN
	      WRITE(PR2,'(A,A)')'$! Command file for VAX/VMS to run '
     #                        //'VLBEDIT'
	      WRITE(PR2,'(A,A)')'$!    Prepared by UVCROSS Version ',
     #                          VERSION
	      WRITE(PR2,'(A,A)')'$!    Run ',NOW
	      WRITE(PR2,'(A)')'$ VLBEDIT'
	      WRITE(PR2,'(A)')INDSN
	      WRITE(PR2,'(A)')INDSN(:INDEX(INDSN,';')-1)
	   ELSE
	      WRITE(PR2,'(A,A)')'# Command file for UNIX to run '
     #                        //'VLBEDIT'
	      WRITE(PR2,'(A,A)')'#    Prepared by UVCROSS Version ',
     #                          VERSION
	      WRITE(PR2,'(A,A)')'#    Run ',NOW
	      WRITE(PR2,'(A)')'vlbedit  <<-END-'
	      WRITE(PR2,'(A)')INDSN
	      WRITE(PR2,'(A)')INDSN(:LEN1(INDSN))//'.UVCROSSED'
	   ENDIF
	   WRITE(PR2,'(A,A,A,A,A)')'HISTORY="Gains found by UVCROSS ',
     #                             VERSION,', Run ',NOW,'"'
	   WRITE(PR2,'(A)')'/'
	ENDIF
C
	LIST = MSGLEVEL.GT.0
	IF (LIST) THEN
	   OPEN (UNIT=PR, NAME=LISTDSN, STATUS='NEW',FORM='FORMATTED',
     1	      CARRIAGECONTROL='LIST',ERR=1030)
	   INQUIRE (UNIT=PR, NAME=LISTDSN)
	   WRITE (PR,'(A)') ' UVCROSS calculates gain correction '//
     #        'factors from MERGE files.'
	   WRITE(PR,'(A)')' (Version '//VERSION//')'
	   WRITE (PR,'(A)')' '
	   WRITE (PR,'(A,A)') ' Run ',NOW
	   WRITE (PR,'(A,A)') ' Input file: ',INDSN(:LEN1(INDSN))
	   IF (EDIF) THEN
	      WRITE (PR,'(A,A)') ' Editfile: ',EDITDSN(:LEN1(EDITDSN))
	   ELSE
	      WRITE(PR,'(A)') ' No Editfile requested'
	   ENDIF
	   WRITE (PR,'(A,A)') ' Listfile: ',LISTDSN(:LEN1(LISTDSN))
	   WRITE (PR,'(A,I1)')'    Message level requested: ',MSGLEVEL
	   IF (NCPRINT.EQ.0) THEN
	      WRITE(PR,'(A)')'    All crossing ratios will be printed'
	   ELSE
	      WRITE(PR,'(A,I5,A)')'    Only the first ',NCPRINT,
     #                            ' ratios will be printed'
	   ENDIF
	   WRITE (PR,'(A,F7.3,A,F7.3,A)') ' U,V Radius given: ',URADB,
     #        ' , ',VRADB,' Megalamda'
	   IF (SQUARE) THEN
	      WRITE(PR,'(A)')'    U,V radii apply separately '
     #           //'(they define a RECTANGLE, not an ellipse)'
	   ELSE
	      WRITE(PR,'(A)')'    U,V radii taken to '
     #           //'define an ellipse'
	   ENDIF
	   IF (UVPOWER.EQ.0.) THEN
	      WRITE(PR,'(A)')'    U,V radii are constant over '
     #                       //'the u,v-plane'
	   ELSE 
	      WRITE(PR,'(A,F6.2)')'    The above numbers '
     #                          //'are multiplied by U,V distance '
     #                          //'to the power of ',UVPOWER
	   ENDIF
	   IF (WTPOWER.EQ.0.) THEN
	      WRITE(PR,'(A)')' No error weighting'
	   ELSE
	      IF (ABSERR) THEN
	         WRITE(PR,'(A,F6.2)')' Weighting based on '
     #              //'ABSOLUTE values of errors raised to the '
     #              //'power of ',WTPOWER
	      ELSE
	         WRITE(PR,'(A,F6.2)')' Weighting based on FRACTIONAL '
     #              //'errors raised to the power of ',WTPOWER
	      ENDIF
	   ENDIF
	   IF (UTAPB.EQ.0.) THEN
	      WRITE(PR,'(A)') ' No weighting (tapering) with U,V '
     #                        //'separation'
	   ELSE
	      WRITE(PR,'(A,/A,F8.3,A,F8.3,A)')
     #         ' Weighting ("tapering") with U,V separation: ',
     #         '    1/e width of ',UTAPB,' MegaLamda in U, and ',
     #         VTAPB,' MegaLamda in V'
	      IF (UVPOWER.EQ.0.) THEN
	         WRITE(PR,'(A)')'    U,V taper has constant width '
     #                          //'over the u,v-plane'
	      ELSE 
	         WRITE(PR,'(A,F6.2)')'    The above numbers '
     #                             //'are multiplied by U,V distance '
     #                             //'to the power of ',UVPOWER
	      ENDIF
	   ENDIF
	   IF (TDIFF.EQ.0.) THEN
	      WRITE(PR,'(A)') ' No time difference limit between '
     #           //'baselines'
	   ELSE
	      WRITE(PR,'(A,F10.3,A)')' Maximum allowed time'
     #           //' difference between the points from different '
     #           //'baselines is ',FLOAT(TDIFF)/(60.*60.),' minutes'
	   ENDIF
	   IF (ZALIM.GE.HALFPI) THEN
	      WRITE(PR,'(A)') ' No zenith angle limit on the stations'
	   ELSE
	      WRITE(PR,'(A,F4.1,A)')' Data discarded for zenith'
     #           //' angles larger than ',ZALIM/RPDEG,' degrees'
	   ENDIF
	   IF (TRADIUS.EQ.0) THEN
	      WRITE(PR,'(A)') ' No time radius imposed on crossing '
     #                        //'points'
	   ELSE
	      WRITE(PR,'(A,F10.3,A)') ' All crossing ponts will have a'
     #                   //' time radius of ',
     #                   FLOAT(TRADIUS)/(60.*60.),' minutes'
	   ENDIF
	   IF (UWINDOW.EQ.0.) THEN
	      WRITE(PR,'(A)') ' No U,V search window will be applied'
	   ELSEIF (UWINDOW.GT.0.) THEN
	      WRITE (PR,'(A,/A,F8.3,A,/A,F8.3,A)')
     #           ' Crossing points used only if '
     #           //'INSIDE the following window:',
     #           '    U = ',UWINDOW,' MegaLamda',
     #           '    V = ',VWINDOW,' Megalamda'
	   ELSE
	      WRITE (PR,'(A,/A,F8.3,A,/A,F8.3,A)')
     #           ' Crossing points used only if '
     #           //'OUTSIDE the following window:',
     #           '    U = ',UWINDOW,' MegaLamda',
     #           '    V = ',VWINDOW,' Megalamda'
	   ENDIF
	   WRITE (PR,'(A)')' '
	ENDIF 
C
C		Read header, check a few trivia.
C
	CALL HEADX30(.FALSE.,INDAT,IZ,ID,IY,DUM1,0,0,IUNITS,DUM3,IF,0)
C
	READ(INDAT,END=1020) IUT,(REC(J),J=1,NDWPB*NB)
	IF (IUT.LT.0) THEN
	   WRITE(OUTC,'(A)')' '
	   WRITE(OUTC,'(A)')' Cannot handle multiple dataset'
     #                      //' MERGE files'
	   WRITE(OUTC,'(A)')' '
	   NERR=NERR+1
	   IF (NERR.GT.MXNERR)
     #        CALL ERROR('ABORTING; ERROR COUNT EXCEEDED.')
	   CLOSE(UNIT=INDAT)
	   IF (LIST) CLOSE(UNIT=PR,DISP='DELETE')
	   IF (EDIF) CLOSE(UNIT=PR2,DISP='DELETE')
	   GOTO 100
	ENDIF
C
	IF ((NVER.NE.1).AND.(NVER.NE.2)) THEN
	   WRITE(OUTC,'(A)')' '
	   WRITE(OUTC,'(A)')' Unsupported MERGE format !!'
	   WRITE(OUTC,'(A)')' '
	   NERR=NERR+1
	   IF (NERR.GT.MXNERR)
     #        CALL ERROR('ABORTING; ERROR COUNT EXCEEDED.')
	   CLOSE(UNIT=INDAT)
	   IF (LIST) CLOSE(UNIT=PR,DISP='DELETE')
	   IF (EDIF) CLOSE(UNIT=PR2,DISP='DELETE')
	   GOTO 100
	ENDIF
C
        IF (NS.LT.2) THEN
	   WRITE(OUTC,'(A)')' '
	   WRITE(OUTC,'(A)')'Not enough stations.'
	   WRITE(OUTC,'(A)')' '
	   NERR=NERR+1
	   IF (NERR.GT.MXNERR)
     #        CALL ERROR('ABORTING; ERROR COUNT EXCEEDED.')
	   CLOSE(UNIT=INDAT)
	   IF (LIST) CLOSE(UNIT=PR,DISP='DELETE')
	   IF (EDIF) CLOSE(UNIT=PR2,DISP='DELETE')
	   GOTO 100
	ENDIF
        IF (NS.GT.MAXS) THEN
	   WRITE(OUTC,'(A)')' '
	   WRITE(OUTC,'(A)')'Too many stations.'
	   WRITE(OUTC,'(A)')' '
	   NERR=NERR+1
	   IF (NERR.GT.MXNERR)
     #        CALL ERROR('ABORTING; ERROR COUNT EXCEEDED.')
	   CLOSE(UNIT=INDAT)
	   IF (LIST) CLOSE(UNIT=PR,DISP='DELETE')
	   IF (EDIF) CLOSE(UNIT=PR2,DISP='DELETE')
	   GOTO 100
	ENDIF
	IF (ORBIT.AND.(ZALIM.LT.HALFPI)) THEN
	   WRITE(OUTC,'(A)')' '
	   WRITE(OUTC,'(A)')' Cannot handle zenith angle limits for'
     #                      //' orbiters'
	   WRITE(OUTC,'(A)')' '
	   NERR=NERR+1
	   IF (NERR.GT.MXNERR)
     #        CALL ERROR('ABORTING; ERROR COUNT EXCEEDED.')
	   CLOSE(UNIT=INDAT)
	   IF (LIST) CLOSE(UNIT=PR,DISP='DELETE')
	   IF (EDIF) CLOSE(UNIT=PR2,DISP='DELETE')
	   GOTO 100
	ENDIF
C
	IF (IUNITS.NE.1) THEN
	   WRITE(OUTC,'(A)')' '
	   WRITE(OUTC,'(A)')' WARNING: Amplitudes are not in flux units'
	   WRITE(OUTC,'(A)')' '
	   IF (LIST) THEN
	      WRITE(PR,'(A)')' '
	      WRITE(PR,'(A)')' WARNING: Amplitudes are not in flux '
     #                         //'units'
	      WRITE(PR,'(A)')' '
	   ENDIF
	ENDIF
C
C		Assemble legends.
C
	IMON = 1
	CALL TDATE(IY,IMON,ID,JD,MONTH,DAY)
	LEGEN1 = SOURCE
	I = LEN1(LEGEN1)+3
	WRITE (LEGEN1(I:),'(A,A3,I5,A,F5.2,A)') '(',MONTH,
     #     IY,', ',FREQ*1E-9,' GHz)'
	LEGEN2 = STNAME(1)
	DO IS=2,NS
	    I = LEN1(LEGEN2)+2
	    LEGEN2(I:) = STNAME(IS)
	ENDDO
	WRITE(OUTC,'(A)') ' '//LEGEN1(1:LEN1(LEGEN1))
	WRITE(OUTC,'(A)') ' '//LEGEN2(1:LEN1(LEGEN2))
	WRITE(OUTC,'(A)')' '
	IF (LIST) THEN
	   WRITE(PR,'(X,A)') LEGEN1(1:LEN1(LEGEN1))
	   WRITE(PR,'(X,A)') LEGEN2(1:LEN1(LEGEN2))
	   WRITE(PR,'(A)')' '
	ENDIF
C
C		Set NOSOLVE flags false,
C		and calculate station lat.-long. if needed.
C
	DO IS=1,NS
C
	   NOSOLVE(IS)=.FALSE.
C
	   IF (ZALIM.LT.HALFPI) THEN
C		Next IF's could have been handled with ATAN2 instead of ATAN
	      IF (STX(IS).EQ.0.) THEN
	         IF (STY(IS).GT.0.) THEN
	            LON(IS)=-HALFPI
	            SINLON(IS)=-1.
	            COSLON(IS)=0.
	         ELSEIF (STY(IS).LT.0.) THEN
	            CALL ERROR('Cannot handle a station exactly at'
     #                         //' the pole !!')
	         ELSE
	            LON(IS)=HALFPI
	            SINLON(IS)=1.
	            COSLON(IS)=0.
	         ENDIF
	         LAT(IS)=ATAN(-STZ(IS)/STY(IS))	! Y=0 already trapped
	      ELSE
	         LON(IS)=ATAN(-STY(IS)/STX(IS))
	         IF (STX(IS).LT.0.) LON(IS)=LON(IS)+PI
	         SINLON(IS)=SIN(LON(IS))
	         COSLON(IS)=COS(LON(IS))
	         LAT(IS)=ATAN(STZ(IS)*COSLON(IS)/STX(IS))
	      ENDIF
	      SINLAT(IS)=SIN(LAT(IS))
	      COSLAT(IS)=COS(LAT(IS))
	   ENDIF
	ENDDO
C		Now decode the NOSOLVE requests.
	DO I=1,MAXS
	   WRITE(TMP1,'(A8)')VALS(I+NPARS1)
	   IF (TMP1.NE.'        ') THEN
	      CALL UPCASE(TMP1)
	      DO J=1,NS
	         TMP2=STNAME(J)
	         CALL UPCASE(TMP2)
	         IF (TMP1.EQ.TMP2) THEN
	            IF (NOSOLVE(J)) THEN
	               WRITE(OUTC,'(A)') ' '
	               WRITE(OUTC,'(A,A)')
     #                    ' WARNING: NOSOLVE specified more than '//
     #                    'once for ',STNAME(J)
	               WRITE(OUTC,'(A)') ' '
	               IF (LIST) THEN
	                  WRITE(PR,'(A)')' '
	                  WRITE(PR,'(A,A)')
     #                    ' WARNING: NOSOLVE specified more than '//
     #                    'once for ',STNAME(J)
	                  WRITE(PR,'(A)')' '
	               ENDIF
	            ELSE
	               NOSOLVE(J)=.TRUE.
	            ENDIF
	            GOTO 115
	         ENDIF
	      ENDDO
	      WRITE(OUTC,'(A)')' '
	      WRITE(OUTC,'(A,A8)')
     #           ' NOSOLVE specified for unrecognised station: ',
     #           VALS(I+NPARS1)
	      WRITE(OUTC,'(A)')' '
	      NERR=NERR+1
	      IF (NERR.GT.MXNERR)
     #           CALL ERROR('ABORTING; ERROR COUNT EXCEEDED.')
	      CLOSE(UNIT=INDAT)
	      IF (LIST) CLOSE(UNIT=PR,DISP='DELETE')
	      IF (EDIF) CLOSE(UNIT=PR2,DISP='DELETE')
	      GOTO 100
	   ENDIF
115	   CONTINUE
	ENDDO
C
C		Baseline parameters.
C
	WAVE=VELC/FREQ*1D6	! 1 million wavelengths
	COSDEC=DCOS(DEC)
	SINDEC=DSIN(DEC)
	MXLBN=5			! Minimum for proper printout
	DO IB=1,NB
	   IS1 = NB1(IB)
	   IS2 = NB2(IB)
	   L1 = LEN1(STNAME(IS1))
	   L2 = LEN1(STNAME(IS2))
	   BNAME(IB) = STNAME(IS1)(1:L1)//'-'//STNAME(IS2)(1:L2)
	   MXLBN=MAX(MXLBN,LEN1(BNAME(IB)))
	   BX=(STX(IS1)-STX(IS2))/WAVE
	   BY=(STY(IS1)-STY(IS2))/WAVE
	   BZ(IB) = (STZ(IS1)-STZ(IS2))/WAVE
	   BXY(IB) = DSQRT(BX*BX+BY*BY)
	   IF (BXY(IB).EQ.0D0) THEN
	      WRITE (OUTC,'(A)')' '
	      WRITE (OUTC,'(A)')
     #             ' WARNING: Zero-length baseline: '//BNAME(IB)
	      WRITE (OUTC,'(A)')' '
	      IF (LIST) THEN
	         WRITE (PR,'(A)')' '
	         WRITE (PR,'(A)')
     #             'WARNING: Zero-length baseline: '//BNAME(IB)
	         WRITE (PR,'(A)')' '
	      ENDIF
	      BOFF(IB) = 0.0
	   ELSE
	      BOFF(IB) = GASTM-RA-HALFPI-DATAN2(BY,BX)
	   ENDIF
	ENDDO
C
C		Read the data.
C
	WRITE(OUTC,'(A)')' UVCROSS - Reading data...'
	NPOINT=0
	NREJEC=0
	NREC=0
120	CONTINUE	! A new record has just been read at this point.
	NREC=NREC+1
        DO IB=1,NB
           IF ( (REC(NDWPB*(IB-1)+2).LE.0.0)
     #        .OR. (REC(NDWPB*(IB-1)+4).LE.0.0)) GOTO 130
C               ( skip deleted data )
C
	   IF (NPOINT.GT.MAXP) CALL ERROR('Too many points.')
           TIMEP = (IUT-IZ)*SIDR + BOFF(IB)
C
           IF (NVER.EQ.1) THEN		! Calculate U,V
	      UP = BXY(IB)*DCOS(TIMEP)
	      VP = BZ(IB)*COSDEC + 
     #                    BXY(IB)*SINDEC*DSIN(TIMEP)
           ELSE IF (NVER.EQ.2) THEN
	      UP = REC(NDWPB*(IB-1)+5) / 1E6	! In Mlambda
	      VP = REC(NDWPB*(IB-1)+6) / 1E6
           ENDIF
C
C	   Use UVWINDOW to reject points, allow a UVRADIIUS margin.
C
	   IF (UWINDOW.NE.0.) THEN
	      IF (UVPOWER.NE.0.) THEN
	         DUM2 = SQRT( UP*UP + VP*VP ) ** UVPOWER
	         URAD = URADB * DUM2
	         VRAD = VRADB * DUM2
	      ENDIF
	      IF (UWINDOW.GT.0.) THEN
	         IF (     ( (ABS(UP) - URAD) .GT.  UWINDOW)
     #               .OR. ( (ABS(VP) - VRAD) .GT.  VWINDOW) ) THEN
	            NREJEC=NREJEC+1
	            GOTO 130	
	         ENDIF
	      ELSE
	         IF (     ( (ABS(UP) + URAD) .LT. -UWINDOW)
     #               .OR. ( (ABS(VP) + VRAD) .LT. -VWINDOW) ) THEN
	            NREJEC=NREJEC+1
	            GOTO 130	
	         ENDIF
	      ENDIF
	   ENDIF
C
C	   Use ZA-limit to reject points.
C
	   IF (ZALIM.LT.HALFPI) THEN
	      HA = (IUT-IZ)*SIDR + GASTM + LON(NB1(IB)) - RA
	      ZA = ACOS( SINLAT(NB1(IB))*SINDEC +
     #                   COSLAT(NB1(IB))*COSDEC*COS(HA) )
	      IF (ZA.GT.ZALIM) THEN
	         NREJEC=NREJEC+1
	         GOTO 130
	      ENDIF
	      HA = (IUT-IZ)*SIDR + GASTM + LON(NB2(IB)) - RA
	      ZA = ACOS( SINLAT(NB2(IB))*SINDEC +
     #                   COSLAT(NB2(IB))*COSDEC*COS(HA) )
	      IF (ZA.GT.ZALIM) THEN
	         NREJEC=NREJEC+1
	         GOTO 130
	      ENDIF
	   ENDIF
C
	   NPOINT=NPOINT+1		! Point now accepted
C
	   IF (UP.LT.0.) THEN		! Use only 1 half-plane
	      UP=-UP
	      VP=-VP
	   ENDIF
C				Fill data arrays
	   INDXU(NPOINT)=NPOINT	! INDXV will be filled after baseline sort.
	   U(NPOINT) = UP
	   V(NPOINT) = VP
	   TIME(NPOINT) = IUT
	   AMP(NPOINT) = REC(NDWPB*(IB-1)+1)
	   ERR(NPOINT) = REC(NDWPB*(IB-1)+2)
	   BASEL(NPOINT)=IB
	   NBPOINT(IB)=NBPOINT(IB)+1
C
130	   CONTINUE	! end of this point
C
        ENDDO
	READ(INDAT,END=140) IUT,(REC(J),J=1,NDWPB*NB)
        IF (IUT.LT.0) GOTO 140
	GOTO 120
C			All points now read.
140	CONTINUE
        CLOSE (UNIT=INDAT)
C
        IF (NPOINT.LE.0) CALL ERROR('No visibility data.')
        WRITE (OUTC,'(13X,I5,A)') NREC,' data records read'
        WRITE (OUTC,'(13X,I5,A,I5,A)') NPOINT,' U,V points accepted, ',
     #                                NREJEC,' rejected'
        IF (LIST) THEN
           WRITE (PR,'(X,I5,A)') NREC,' data records read'
           WRITE (PR,'(X,I5,A)') NREJEC,' U,V points rejected'
	   IF (MSGLEVEL.GE.2) THEN
	      WRITE (PR,'(X,I5,A)')NPOINT,' U,V points accepted, of which'
	   ELSE
	      WRITE (PR,'(X,I5,A)')NPOINT,' U,V points accepted'
	      WRITE(PR,'(A)')' '
	   ENDIF
	ENDIF
C
C		Sort data points by baseline first, and then
C		within each baseline set up both U and V index arrays in
C		descending order. These INDXU and INDXV arrays can be
C		used to acces BASEL, U, V, TIME, AMP, ERR, CNTOT, CAMP, CER2.
C
	WRITE(OUTC,'(A)')' UVCROSS - Sorting data...'
C			Set up space needed per baseline in indexed sort
C			and provide printout.
	BSTART(1)=NPOINT+1
	IF (MSGLEVEL.GE.2) WRITE(PR,'(A,I6,A,A)')' ',NBPOINT(1),
     #                     ' points are on baseline ',BNAME(1)
	DO I=2,NB
	   BSTART(I)=BSTART(I-1)-NBPOINT(I-1)	!BSTARTS are in fact at end now
	   IF (MSGLEVEL.GE.2)
     #        WRITE(PR,'(A,I6,A,A)')' ',NBPOINT(I),
     #                              ' points are on baseline ',BNAME(I)
	ENDDO
	IF (MSGLEVEL.GE.2) WRITE(PR,'(A)')' '
C			Now sort by baseline.
	DO I=1,NPOINT
	   BSTART(BASEL(I))=BSTART(BASEL(I))-1
	   INDXU(BSTART(BASEL(I)))=I
	   INDXV(BSTART(BASEL(I)))=I
	ENDDO
C			Then sort baseline subsets in U and V
	DO I=NB,1,-1
	   IF (NBPOINT(I).NE.0) THEN
	      CALL DRSORT(U,NPOINT,NBPOINT(I),INDXU(BSTART(I)))
	      CALL DRSORT(V,NPOINT,NBPOINT(I),INDXV(BSTART(I)))
	   ENDIF
	ENDDO
C
C		Now try to find amplitude ratios per baseline pair from
C		close neighbours. This search is not restricted to circles 
C		around theoretical crossings, but allows all sorts of 
C		near misses. The procedure ensures that as many U,V
C		points as possible can enter into the calculation,
C		without having to average over a wide area of the U,V
C		plane (useful e.g. for a baseline pair which runs
C		almost parallel).
C
C		Each pair of baselines is treated separately.
C		For each pair, the two baselines are treated in turn
C		as the "reference" and the "neighbour" baseline. By
C		looking at all reference points in turn, a set of
C		ratios is obtained. Each is determined as follows:
C
C		Points on the neighbour baseline are sought within a
C		given U,V ellipse of the reference point. A
C		"neighbour" amplitude is derived as the weighted
C		(ABSERR, WTPOWER, UVTAPER applied to error bar of data
C		points) average of all neighbour-baseline amplitudes
C		within the ellipse. A "reference" amplitude is
C		obtained likewise. The error of an amplitude is given by
C               the square root of the weighted (as above) average of
C		the square of the error bars of the data points. The two 
C		average amplitudes determine a ratio, which is taken to
C		"belong" to a specific point on the reference baseline. 
C		Its relative error is given by the square root of the sum 
C		of the squares of the relative errors of the two amplitudes.
C
C		All these ratios are averaged (weighted by their
C		error) to form the ratio applicable to the pair of
C		baselines as a whole. The error estimate for the final
C		ratio on the baseline pair is not determined from the
C		errors of the contributing ratios, since these are
C		typically formed using many overlapping points: they
C		are far from independent. Instead, the square root is
C		taken of the weighted average of the squares of the 
C		relative errors of all the the points actually used as 
C		reference (i.e. those which have close neighbours on the 
C		other baseline). For this weighting, ABSERR and WTPOWER 
C		are taken into account, but UVTAPER is meaningless (the
C		reference point is always at the centre of the circle). 
C		Clearly, the error obtained by this procedure is not 
C		exactly correct, as not all points will enter equally 
C		frequently into the calculation of the ratio, or with 
C		equal "tapering". However, I couldn't think of a better 
C		way which retains the advantage of forming a ratio from 
C		as much information as possible. While not proper in
C		absolute value, the errors obtained for different
C		baseline pairs should still be reasonably accurate to 
C		provide relative weights for these data in the least
C		squares gain solutions, which is all they're used for.
C
	WRITE(OUTC,'(A)')' UVCROSS - Searching for close points...'
	IF (NPOINT.GT.10000) THEN
	   WRITE(OUTC,'(10X,A,/,10X,A)')
     #       ' ---> If this takes TOO LONG, then average your data, or',
     #       '      use more restrictive UVRadius, UVPOWER, UVWindow, '
     #        //'or Zalimit.'
	   WRITE(OUTC,'(A)')' '
	ENDIF
C
	IF (MSGLEVEL.GE.2) THEN		! Make table caption
	   WRITE(PR,'(A)')' '
	   WRITE(PR,'(A)')' Results of close point search:'
	   WRITE(PR,'(A)')' '
	   LDOTS1=INT((31+MXLBN-18)/2)
	   LDOTS3=INT(( 6+MXLBN- 9)/2)
	   LDOTS2=31+MXLBN-18-LDOTS1
	   LDOTS4= 6+MXLBN- 9-LDOTS3
	   WRITE(PR,'(X,A,A18,A,4X,A,A9,A,X,A7,3X,A,4X,A)')
     #        DOTS(:LDOTS1),'Reference Baseline',DOTS(:LDOTS2),
     #        DOTS(:LDOTS3),'Neighbour',         DOTS(:LDOTS4),
     #        'Minimum','Ratio','Error'
	   WRITE(PR,'(X,2X,A4,7X,A1,8X,A1,4X,A4,X,A4,A,2X,A4,X,
     #                A4,A,X,A7)')
     #       'Time','U','V','#pts','Name',BLANK(:MXLBN-3),'#pts',
     #       'Name',BLANK(:MXLBN-3),'UV-dist'
	   WRITE(PR,'(A)')' '
	   WRITE(PR,'(A)')' * = Inverted ratio and error'
	   WRITE(PR,'(A)')' '
	ENDIF
	NBCLOSE=0		! Counter for no. basel. pairs which come close
C
C
C			The main loops over baselines
C
C
	DO I=1,NB-1				! Each baseline
C
	   IF (NBPOINT(I).EQ.0) GOTO 211	! (if it has any points)
C
C	   Is any part of the baseline in the UV-Window ?
C
	   IF (UWINDOW.NE.0.) THEN
	      IST=BSTART(I)
	      IEN=BSTART(I)+NBPOINT(I)-1
	      IF (UWINDOW.GT.0.) THEN
	         IF (    (U(INDXU(IST)).LT.-UWINDOW)
     #               .OR.(U(INDXU(IEN)).GT. UWINDOW)  )   GOTO 211
	         IF (    (V(INDXV(IST)).LT.-VWINDOW)
     #               .OR.(V(INDXV(IEN)).GT. VWINDOW)  )   GOTO 211
	      ELSE
	         IF (     (U(INDXU(IST)).LT.-UWINDOW)
     #               .AND.(U(INDXU(IEN)).GT. UWINDOW)  )   GOTO 211
	         IF (     (V(INDXV(IST)).LT.-VWINDOW)
     #               .AND.(V(INDXV(IEN)).GT. VWINDOW)  )   GOTO 211
	      ENDIF
	   ENDIF
C
	   DO J=I+1,NB				! versus higher baselines
C
C				Do we need to bother with this combination ?
	      IF (NBPOINT(J).EQ.0) GOTO 210	! Are there any points ?
C
	      IF (NB1(I).NE.NB1(J)) THEN	! Is there a "NOSOLVE" station ?
	         IF ((NB1(I).NE.NB2(J)) .AND. NOSOLVE(NB1(I))) GOTO 210
	         IF ((NB2(I).NE.NB1(J)) .AND. NOSOLVE(NB1(J))) GOTO 210
	      ENDIF
	      IF (NB2(I).NE.NB2(J)) THEN
	         IF ((NB2(I).NE.NB1(J)) .AND. NOSOLVE(NB2(I))) GOTO 210
	         IF ((NB1(I).NE.NB2(J)) .AND. NOSOLVE(NB2(J))) GOTO 210
	      ENDIF
C
	      IST=BSTART(I)
	      IEN=BSTART(I)+NBPOINT(I)-1
	      JST=BSTART(J)
	      JEN=BSTART(J)+NBPOINT(J)-1
C
C	      Upper limits to actual U,V RAD for this reference baseline,
C	      only needed if variable (UVPOWER.NE.0.)
C
	      IF (UVPOWER.GT.0.) THEN
	         DUM1 = MAX( ABS(V(INDXV(IST))), ABS(V(INDXV(IEN))) )
	         DUM2 = SQRT( U(INDXU(IST))**2. + DUM1**2. ) ** UVPOWER
	         URAD = URADB * DUM2
	         VRAD = VRADB * DUM2
	      ELSEIF (UVPOWER.LT.0.) THEN
	         DUM1 = MIN( ABS(V(INDXV(IST))), ABS(V(INDXV(IEN))) )
	         DUM2 = SQRT( U(INDXU(IEN))**2. + DUM1**2. ) ** UVPOWER
	         URAD = URADB * DUM2
	         VRAD = VRADB * DUM2
	      ENDIF
C
	      UBEG = MIN( U(INDXU(IST)), U(INDXU(JST)) ) + URAD
	      UEND = MAX( U(INDXU(IEN)), U(INDXU(JEN)) ) - URAD
	      IF (UEND.GT.UBEG) GOTO 210	! Baselines not close in U
C
	      VBEG = MIN( V(INDXV(IST)), V(INDXV(JST)) ) + VRAD
	      VEND = MAX( V(INDXV(IEN)), V(INDXV(JEN)) ) - VRAD
	      IF (VEND.GT.VBEG) GOTO 210	! Baselines not close in V
C
C			Find first and last points on the baselines
C			which are within U,V RAD of eachother; i.e. find
C			the extent to which the two baselines overlap.
C
	      ISTU=IST
	      ISTV=IST
	      IENU=IEN
	      IENV=IEN
	      JSTU=JST
	      JSTV=JST
	      JENU=JEN
	      JENV=JEN
	      CALL BRACKET(U,INDXU,UBEG,UEND,ISTU,IENU)
	      CALL BRACKET(U,INDXU,UBEG,UEND,JSTU,JENU)
	      CALL BRACKET(V,INDXV,VBEG,VEND,ISTV,IENV)
	      CALL BRACKET(V,INDXV,VBEG,VEND,JSTV,JENV)
C
C			Is any of this stretch inside the UVWindow ?
C
	      IF (UWINDOW.GT.0.) THEN
	         IF (    (U(INDXU(ISTU)).LT.-UWINDOW)
     #               .OR.(U(INDXU(IENU)).GT. UWINDOW)  ) GOTO 210
	         IF (    (U(INDXU(JSTU)).LT.-UWINDOW)
     #               .OR.(U(INDXU(JENU)).GT. UWINDOW)  ) GOTO 210
	         IF (    (V(INDXV(ISTV)).LT.-VWINDOW)
     #               .OR.(V(INDXV(IENV)).GT. VWINDOW)  ) GOTO 210
	         IF (    (V(INDXV(JSTV)).LT.-VWINDOW)
     #               .OR.(V(INDXV(JENV)).GT. VWINDOW)  ) GOTO 210
	      ELSEIF (UWINDOW.LT.0.) THEN
	         IF (     (U(INDXU(ISTU)).LT.-UWINDOW)
     #               .AND.(U(INDXU(IENU)).GT. UWINDOW)  ) GOTO 210
	         IF (     (U(INDXU(JSTU)).LT.-UWINDOW)
     #               .AND.(U(INDXU(JENU)).GT. UWINDOW)  ) GOTO 210
	         IF (     (V(INDXV(ISTV)).LT.-VWINDOW)
     #               .AND.(V(INDXV(IENV)).GT. VWINDOW)  ) GOTO 210
	         IF (     (V(INDXV(JSTV)).LT.-VWINDOW)
     #               .AND.(V(INDXV(JENV)).GT. VWINDOW)  ) GOTO 210
	      ENDIF
C
C			Determine whether baseline pair will be run
C			in U or V order. Choose the one for which the
C			second (J) baseline has the largest range, such that
C			you run outside U,V RAD as quickly as possible.
C
	      IF ( VRAD * ( U(INDXU(JST)) - U(INDXU(JEN)) )
     #            .GT.
     #             URAD * ( V(INDXV(JST)) - V(INDXV(JEN)) )
     #           ) THEN
	         SORTU=.TRUE.
	         IST=ISTU
	         IEN=IENU
	         JST=JSTU
	         JEN=JENU
	      ELSE
	         SORTU=.FALSE.
	         IST=ISTV
	         IEN=IENV
	         JST=JSTV
	         JEN=JENV
	      ENDIF
C
C				Setup
C
	      NBCLOSE=NBCLOSE+1		! Down again later if pair not close.
C
	      BRATIO(NBCLOSE)=0.	! Average for the full baseline
	      BWTTOT(NBCLOSE)=0.	! pair will be accumulated here.
	      CLOSTOT=0		! Quantities needed for the 
	      CLOSTOTR=0	! average ratio and error on 
	      PAIRER2=0.	! each baseline, but not for 
	      PAIRER2R=0.	! the final least-squares solution 
	      PAIRWT=0.		! in SOLVE, so there is no need 
	      PAIRWT=0.		! to store them in arrays. 
	      CLOSUVA=1.E37	! Should be larger than any UV-value in Mlamda
	      FIRSTK=IEN			! Timesavers for second 
	      LASTK=IST				! part, when ref. and 
	      FIRSTL=JEN			! neighb. basel. are 
	      LASTL=JST				! swapped.
	      LSTARTR=IST	! Timesavers in later points on each baseline,
	      LSTART=JST	! making use of the sorting in U and V.
C
C			Start search
C
	      DO K=IST,IEN				! all relevant pts on 
C							! the reference basel.
C			Is the reference point in the UV-Window ?
C
	         IF (UWINDOW.GT.0.) THEN
	            IF (    (ABS(U(INDXU(K))).GT.UWINDOW)
     #                  .OR.(ABS(V(INDXV(K))).GT.VWINDOW)  ) GOTO 160
	         ELSEIF (UWINDOW.LT.0.) THEN
	            IF (    (ABS(U(INDXU(K))).LT.-UWINDOW)
     #                  .OR.(ABS(V(INDXV(K))).LT.-VWINDOW)  ) GOTO 160
	         ENDIF
C
C			Setup
C
	         NTOT=0
	         AMPTOT=0.
	         WTTOT=0.
	         ER2TOT=0.
	         CLOSUV=1.E37	! Should be larger than any UV-value in Mlamda
	         NEWK=.TRUE.
C
	         IF (SORTU) THEN
	            IND1=INDXU(K)
	         ELSE
	            IND1=INDXV(K)
	         ENDIF
	         TREF=TIME(IND1)
	         UREF=U(IND1)
	         VREF=V(IND1)
	         IF (UVPOWER.NE.0.) THEN
	            DUM1 = SQRT (UREF**2. + VREF**2.) ** UVPOWER
	            URAD = URADB * DUM1
	            VRAD = VRADB * DUM1
	            UTAP = UTAPB * DUM1
	            VTAP = VTAPB * DUM1
	         ENDIF

C
C	         Determine TIME of CLOSEST point on basel. J if needed.
C
	         IF (TRADIUS.NE.0.)
     #              CALL CLOSTIM(TIME,U,V,INDXU,INDXV,MAXP,LSTART,JEN,
     #                           SORTU,UREF,VREF,CTIM)
C
	         DO L=LSTART,JEN		! all relevant pts on basel. J,
C
	            IF (SORTU) THEN
	               IND2=INDXU(L)	! Index the neighbour baseline
	               DU=U(IND2)-UREF	! U distance between K and L points
	               IF (DU.LT.-URAD) GOTO 160 ! Done K pt (for this J basel.)
	               IF (DU.GT.URAD) GOTO 150  ! This L pt not yet in range.
	               IF (NEWK) THEN
	                  NEWK=.FALSE.
	                  LSTART=L	! For next K, need not check earlier pts
	               ENDIF
	               DV=ABS(V(IND2)-VREF)	! V dist. between K and L points
	               IF (DV.GT.VRAD) GOTO 150
	               DU=ABS(DU)
	            ELSE		! Same, but U and V interchanged.
	               IND2=INDXV(L)
	               DV=V(IND2)-VREF
	               IF (DV.LT.-VRAD) GOTO 160
	               IF (DV.GT.VRAD) GOTO 150
	               IF (NEWK) THEN
	                  NEWK=.FALSE.
	                  LSTART=L
	               ENDIF
	               DU=ABS(U(IND2)-UREF)
	               IF (DU.GT.URAD) GOTO 150
	               DV=ABS(DV)
	            ENDIF
	            IF (.NOT.SQUARE) THEN	! Full ellipse UVRAD limits
	               IF ((DU/URAD)**2+(DV/VRAD)**2.GT.1.) GOTO 150
	            ENDIF
C			Got a close point in U,V; is it close in time too ?
	            IF (TDIFF.NE.0) THEN
	               IF (ABS(TREF-TIME(IND2)).GT.TDIFF) GOTO 150
	            ENDIF
	            IF (TRADIUS.NE.0) THEN
	               IF (ABS(CTIM-TIME(IND2)).GT.TRADIUS) GOTO 150
	            ENDIF
C
C			Got a point, find its weight.
C			First, errror weighting.
	            IF (WTPOWER.EQ.0.) THEN
	               WT=1.
	            ELSE
	               WT=ERR(IND2)
	               IF (.NOT.ABSERR) THEN
	                  IF (AMP(IND2).EQ.0.) GOTO 150	! Cannot divide by 0
	                  WT=WT/AMP(IND2)
	               ENDIF
	               WT=WT**WTPOWER
	            ENDIF
C			Additional U,V distance weighting.
	            IF (UTAP.GT.0.)		! If <= 0, then no taper
     #                 WT=WT*EXP(-(DU/UTAP)**2-(DV/VTAP)**2)
C
C			Finally add this point to the totals.
	            NTOT=NTOT+1
	            AMPTOT=AMPTOT+DBLE(WT*AMP(IND2))
	            WTTOT=WTTOT+DBLE(WT)
	            ER2TOT=ER2TOT+DBLE(WT*ERR(IND2)*ERR(IND2))
	            DUV=SQRT(DU*DU+DV*DV)
	            CLOSUV=MIN(CLOSUV,DUV)
	            FIRSTL=MIN(L,FIRSTL)	! Will save time when ref. and
	            LASTL=MAX(L,LASTL)		! neighb. basel. are swapped
C
C			For skipped points, you end up here
150	            CONTINUE
C			End of this L point
	         ENDDO
C
C			Done with J (neighbour) baseline for this K (ref.) point
160	         CONTINUE
C
	         IF (WTTOT.EQ.0.) GOTO 170	! No close points
C
C			We have close points.
	         AMPTOT=AMPTOT/WTTOT		! Convert sums to averages
	         ER2TOT=ER2TOT/WTTOT
	         CLOSUVA=MIN(CLOSUVA,CLOSUV)
C
C			Need amplitude on reference baseline as well.
	         IF (CAMP(IND1).NE.0.) THEN	! We've calculated this before
	            NTOTR=CNTOT(IND1)
	            AMPTOTR=CAMP(IND1)
	            ER2TOTR=CER2(IND1)
	         ELSE				! Use same code as for neighbour
	            NTOTR=0			! baseline and store result .
	            AMPTOTR=0.		! NOTE: No close endpoint test needed,
	            WTTOTR=0.		! as we are on 1 and the same baseline.
	            ER2TOTR=0.
	            NEWK=.TRUE.
	            DO L=LSTARTR,IEN	! NOTE: I (K) baseline now
	               IF (SORTU) THEN
	                  IND2=INDXU(L)
	                  DU=U(IND2)-UREF
	                  IF (DU.LT.-URAD) GOTO 165
	                  IF (DU.GT.URAD) GOTO 163
	                  IF (NEWK) THEN
	                     NEWK=.FALSE.
	                     LSTARTR=L		! LSTARTR now
	                  ENDIF
	                  DV=ABS(V(IND2)-VREF)
	                  IF (DV.GT.VRAD) GOTO 163
	                  DU=ABS(DU)
	               ELSE
	                  IND2=INDXV(L)
	                  DV=V(IND2)-VREF
	                  IF (DV.LT.-VRAD) GOTO 165
	                  IF (DV.GT.VRAD) GOTO 163
	                  IF (NEWK) THEN
	                     NEWK=.FALSE.
	                     LSTARTR=L
	                  ENDIF
	                  DU=ABS(U(IND2)-UREF)
	                  IF (DU.GT.URAD) GOTO 163
	                  DV=ABS(DV)
	               ENDIF
	               IF (.NOT.SQUARE) THEN
	                  IF((DU/URAD)**2+(DV/VRAD)**2.GT.1.)GOTO 163
	               ENDIF
	               IF((TDIFF.NE.0)
     #                    .AND.(ABS(TREF-TIME(IND2)).GT.TDIFF))	! This should
     #                    GOTO 163	! not happen on ref. basel. for proper
C					  choice of parameters
	               IF (TRADIUS.NE.0) THEN
	                  IF (ABS(TREF-TIME(IND2)).GT.TRADIUS) GOTO 163 ! Note
C					      TREF, not CTIM, on the ref. basel.
	               ENDIF
	               IF (WTPOWER.EQ.0.) THEN
	                  WT=1.
	               ELSE
	                  WT=ERR(IND2)
	                  IF (.NOT.ABSERR) THEN
	                     IF (AMP(IND2).EQ.0.) GOTO 163
	                     WT=WT/AMP(IND2)
	                  ENDIF
	                  WT=WT**WTPOWER
	               ENDIF
	               IF (UTAP.GT.0.)
     #                    WT=WT*EXP(-(DU/UTAP)**2-(DV/VTAP)**2)
	               NTOTR=NTOTR+1
	               AMPTOTR=AMPTOTR+DBLE(WT*AMP(IND2))
	               WTTOTR=WTTOTR+DBLE(WT)
	               ER2TOTR=ER2TOTR+DBLE(WT*ERR(IND2)*ERR(IND2))
163	               CONTINUE
	            ENDDO
165	            CONTINUE
	            IF (WTTOTR.EQ.0.) GOTO 170	! There should always be 
C			at least 1 point (the reference), but it's AMP
C			might just be 0, so still need this test.
	            AMPTOTR=AMPTOTR/WTTOTR
	            ER2TOTR=ER2TOTR/WTTOTR
C			Store to avoid multiple calculations (you go into 
	            CNTOT(IND1)=NTOTR	! the THEN clause of this IF statement
	            CAMP(IND1)=AMPTOTR	! next time around).
	            CER2(IND1)=ER2TOTR
	         ENDIF
C
C			Now we have an average amp and a variance on both basel.
C
C			Normalise variance, convert to sigma, and then 
	         ER2TOT=SQRT(ER2TOT/DBLE(NTOT))/AMPTOT	! to fractional error
	         ER2TOTR=SQRT(ER2TOTR/DBLE(NTOTR))/AMPTOTR
C
C			Work out the ratio 
	         RATIO=AMPTOTR/AMPTOT
	         SIGMA=RATIO*SQRT(ER2TOTR*ER2TOTR+ER2TOT*ER2TOT)
C			Add to baseline pair total, weighted by sigma
	         BRATIO(NBCLOSE)=BRATIO(NBCLOSE) + RATIO/SIGMA
	         BWTTOT(NBCLOSE)=BWTTOT(NBCLOSE) + 1./SIGMA ! Track total weight
C
C			Count 1 independent point: the reference point.
	         IF (AMP(IND1).NE.0.) THEN
	            IF (WTPOWER.EQ.0.) THEN
	               WT=1.
	            ELSE
	               WT=ERR(IND1)
	               IF (.NOT.ABSERR) WT=WT/AMP(IND1)
	               WT=WT**WTPOWER
	            ENDIF
	         ENDIF
	         CLOSTOTR=CLOSTOTR+1
	         PAIRWTR=PAIRWTR+DBLE(WT)
	         PAIRER2R=PAIRER2R+DBLE(WT*( (ERR(IND1)*ERR(IND1))  /
     #                                  (AMP(IND1)*AMP(IND1))    )   )
C
C			Bookkeeping to save time in second half
	         FIRSTK=MIN(K,FIRSTK)		! when reference and neighbour 
	         LASTK=MAX(K,LASTK)		! baselines are swapped
C
	         IF ((MSGLEVEL.GE.3).AND.(    (NBCLOSE.LE.NCPRINT)
     #                                    .OR.(NCPRINT.EQ.0)      ) )
     #              WRITE(PR,'(X,A8,X,F7.2,X,F8.2,X,I4,X,
     #                         A,4X,I4,X,A,2X,F6.2,2X,F7.3,A1,
     #                         X,F8.4)')	! Print particulars for ratio
     #                 TIMEC(MOD(TREF/60,86400)),UREF,VREF,NTOTR,
     #                 BNAME(I)(:MXLBN),NTOT,BNAME(J)(:MXLBN),CLOSUV,
     #                 RATIO,' ',SIGMA
C
170	         CONTINUE	! No close points for this K goes to 170
C				End of this K reference point
	      ENDDO
C
C			End first half. Do we need to bother with second half ?
C
	      IF (CLOSTOTR.EQ.0) THEN
	         NBCLOSE=NBCLOSE-1	! Pair of baselines is nowhere close
	         GOTO 210		! Skip second half of this pair
	      ENDIF
C
C			Second half: swap roles of I (now neighbour) 
C			                       and J (now reference)
C
	         IF ((MSGLEVEL.GE.3).AND.(    (NBCLOSE.LE.NCPRINT)
     #                                    .OR.(NCPRINT.EQ.0)      ) )
     #              WRITE(PR,'(A)')' '
	      LSTART=FIRSTK		! NOTE: swap between
	      LSTARTR=JST		! baselines
	      DO K=FIRSTL,LASTL ! NOTE: obtained from former L points
C					(=J baseline), and now used 
C					for reference baseline.
	         IF (UWINDOW.GT.0.) THEN
	            IF (    (ABS(U(INDXU(K))).GT.UWINDOW)
     #                  .OR.(ABS(V(INDXV(K))).GT.VWINDOW)  ) GOTO 190
	         ELSEIF (UWINDOW.LT.0.) THEN
	            IF (    (ABS(U(INDXU(K))).LT.-UWINDOW)
     #                  .OR.(ABS(V(INDXV(K))).LT.-VWINDOW)  ) GOTO 190
	         ENDIF
	         NTOT=0
	         AMPTOT=0.
	         WTTOT=0.
	         ER2TOT=0.
	         CLOSUV=1.E37
	         NEWK=.TRUE.
	         IF (SORTU) THEN
	            IND1=INDXU(K)
	         ELSE
	            IND1=INDXV(K)
	         ENDIF
	         TREF=TIME(IND1)
	         UREF=U(IND1)
	         VREF=V(IND1)
	         IF (UVPOWER.NE.0.) THEN
	            DUM1 = SQRT (UREF**2. + VREF**2.) ** UVPOWER
	            URAD = URADB * DUM1
	            VRAD = VRADB * DUM1
	            UTAP = UTAPB * DUM1
	            VTAP = VTAPB * DUM1
	         ENDIF
C
C	         Determine TIME of CLOSEST point on basel. J if needed.
C
	         IF (TRADIUS.NE.0.)
     #              CALL CLOSTIM(TIME,U,V,INDXU,INDXV,MAXP,LSTART,LASTK,
     #                           SORTU,UREF,VREF,CTIM)
C
	         DO L=LSTART,LASTK	!	NOTE swapped baselines
	            IF (SORTU) THEN
	               IND2=INDXU(L)
	               DU=U(IND2)-UREF
	               IF (DU.LT.-URAD) GOTO 190
	               IF (DU.GT.URAD) GOTO 180
	               IF (NEWK) THEN
	                  NEWK=.FALSE.
	                  LSTART=L
	               ENDIF
	               DV=ABS(V(IND2)-VREF)
	               IF (DV.GT.VRAD) GOTO 180
	               DU=ABS(DU)
	            ELSE
	               IND2=INDXV(L)
	               DV=V(IND2)-VREF
	               IF (DV.LT.-VRAD) GOTO 190
	               IF (DV.GT.VRAD) GOTO 180
	               IF (NEWK) THEN
	                  NEWK=.FALSE.
	                  LSTART=L
	               ENDIF
	               DU=ABS(U(IND2)-UREF)
	               IF (DU.GT.URAD) GOTO 180
	               DV=ABS(DV)
	            ENDIF
	            IF (.NOT.SQUARE) THEN
	               IF ((DU/URAD)**2+(DV/VRAD)**2.GT.1.) GOTO 180
	            ENDIF
	            IF (TDIFF.NE.0) THEN
	               IF (ABS(TREF-TIME(IND2)).GT.TDIFF) GOTO 180
	            ENDIF
	            IF (TRADIUS.NE.0) THEN
	               IF (ABS(CTIM-TIME(IND2)).GT.TRADIUS) GOTO 180
	            ENDIF
	            IF (WTPOWER.EQ.0.) THEN
	               WT=1.
	            ELSE
	               WT=ERR(IND2)
	               IF (.NOT.ABSERR) THEN
	                  IF (AMP(IND2).EQ.0.) GOTO 180
	                  WT=WT/AMP(IND2)
	               ENDIF
	               WT=WT**WTPOWER
	            ENDIF
	            IF (UTAP.GT.0.)
     #                 WT=WT*EXP(-(DU/UTAP)**2-(DV/VTAP)**2)
                    NTOT=NTOT+1
	            AMPTOT=AMPTOT+DBLE(WT*AMP(IND2))
	            WTTOT=WTTOT+DBLE(WT)
	            ER2TOT=ER2TOT+DBLE(WT*ERR(IND2)*ERR(IND2))
	            DUV=SQRT(DU*DU+DV*DV)
	            CLOSUV=MIN(CLOSUV,DUV)
180	            CONTINUE
	         ENDDO
190	         CONTINUE
	         IF (WTTOT.EQ.0.) GOTO 200
	         AMPTOT=AMPTOT/WTTOT
	         ER2TOT=ER2TOT/WTTOT
	         IF (CAMP(IND1).NE.0.) THEN
	            NTOTR=CNTOT(IND1)
	            AMPTOTR=CAMP(IND1)
	            ER2TOTR=CER2(IND1)
	         ELSE
                    NTOTR=0
	            AMPTOTR=0.
	            WTTOTR=0.
	            ER2TOTR=0.
	            NEWK=.TRUE.
	            DO L=LSTARTR,JEN		! NOTE swapped baselines
	               IF (SORTU) THEN
	                  IND2=INDXU(L)
	                  DU=U(IND2)-UREF
	                  IF (DU.LT.-URAD) GOTO 195
	                  IF (DU.GT.URAD) GOTO 193
	                  IF (NEWK) THEN
	                     NEWK=.FALSE.
	                     LSTARTR=L
	                  ENDIF
	                  DV=ABS(V(IND2)-VREF)
	                  IF (DV.GT.VRAD) GOTO 193
	                  DU=ABS(DU)
	               ELSE
	                  IND2=INDXV(L)
	                  DV=V(IND2)-VREF
	                  IF (DV.LT.-VRAD) GOTO 195
	                  IF (DV.GT.VRAD) GOTO 193
	                  IF (NEWK) THEN
	                     NEWK=.FALSE.
	                     LSTARTR=L
	                  ENDIF
	                  DU=ABS(U(IND2)-UREF)
	                  IF (DU.GT.URAD) GOTO 193
	                  DV=ABS(DV)
	               ENDIF
	               IF (.NOT.SQUARE) THEN
	                  IF((DU/URAD)**2+(DV/VRAD)**2.GT.1.)GOTO 193
	               ENDIF
	               IF((TDIFF.NE.0)
     #                    .AND.(ABS(TREF-TIME(IND2)).GT.TDIFF))
     #                    GOTO 193
	               IF (TRADIUS.NE.0) THEN
	                  IF (ABS(TREF-TIME(IND2)).GT.TRADIUS) GOTO 193
	               ENDIF
	               IF (WTPOWER.EQ.0.) THEN
	                  WT=1.
	               ELSE
	                  WT=ERR(IND2)
	                  IF (.NOT.ABSERR) THEN
	                     IF (AMP(IND2).EQ.0.) GOTO 193
	                     WT=WT/AMP(IND2)
	                  ENDIF
	                  WT=WT**WTPOWER
	               ENDIF
	               IF (UTAP.GT.0.)
     #                    WT=WT*EXP(-(DU/UTAP)**2-(DV/VTAP)**2)
                       NTOTR=NTOTR+1
	               AMPTOTR=AMPTOTR+DBLE(WT*AMP(IND2))
	               WTTOTR=WTTOTR+DBLE(WT)
	               ER2TOTR=ER2TOTR+DBLE(WT*ERR(IND2)*ERR(IND2))
193	               CONTINUE
	            ENDDO
195	            CONTINUE
	            AMPTOTR=AMPTOTR/WTTOTR
	            ER2TOTR=ER2TOTR/WTTOTR
	            CNTOT(IND1)=NTOTR
	            CAMP(IND1)=AMPTOTR
	            CER2(IND1)=ER2TOTR
	         ENDIF
	         ER2TOT=SQRT(ER2TOT/DBLE(NTOT))/AMPTOT
	         ER2TOTR=SQRT(ER2TOTR/DBLE(NTOTR))/AMPTOTR
	         RATIO=AMPTOT/AMPTOTR	! NOTE reversal of ref. and neighb.
	         SIGMA=RATIO*SQRT(ER2TOTR*ER2TOTR+ER2TOT*ER2TOT)
	         BRATIO(NBCLOSE)=BRATIO(NBCLOSE) + RATIO/SIGMA
	         BWTTOT(NBCLOSE)=BWTTOT(NBCLOSE) + 1./SIGMA
	         IF (AMP(IND1).NE.0.) THEN
	            IF (WTPOWER.EQ.0.) THEN
	               WT=1.
	            ELSE
	               WT=ERR(IND1)
	               IF (.NOT.ABSERR) WT=WT/AMP(IND1)
	               WT=WT**WTPOWER
	            ENDIF
	         ENDIF
	         CLOSTOT=CLOSTOT+1		! NOTE: total on other baseline
	         PAIRWT=PAIRWT+DBLE(WT)
	         PAIRER2=PAIRER2+DBLE(WT*( (ERR(IND1)*ERR(IND1))  /
     #                                (AMP(IND1)*AMP(IND1))    )   )
	         IF ((MSGLEVEL.GE.3).AND.(    (NBCLOSE.LE.NCPRINT)
     #                                    .OR.(NCPRINT.EQ.0)      ) )
     #              WRITE(PR,'(X,A8,X,F7.2,X,F8.2,X,I4,X,
     #                         A,4X,I4,X,A,2X,F6.2,2X,F7.3,A1,
     #                         X,F8.4)')
     #                 TIMEC(MOD(TREF/60,86400)),UREF,VREF,NTOTR,
     #                 BNAME(J)(:MXLBN),NTOT,BNAME(I)(:MXLBN),CLOSUV,
     #                 RATIO,' ',SIGMA
C				NOTE: reversed I and J indicated in print by '*'
200	         CONTINUE
	      ENDDO
C
	      IF (CLOSTOT.EQ.0) THEN
	         NBCLOSE=NBCLOSE-1
	         GOTO 210
	      ENDIF
C
C			Done both halves of pair, now calculate values 
C			for the pair as a whole
	      BAS1(NBCLOSE)=I
	      BAS2(NBCLOSE)=J
	      BRATIO(NBCLOSE)=BRATIO(NBCLOSE)/BWTTOT(NBCLOSE) ! Sum to average
C			Relative error determined by sum of squares
C			of relative errors of all independent points.
	      BWTTOT(NBCLOSE)=SQRT(   PAIRER2 /(PAIRWT *DBLE(CLOSTOT) )
     #                             + PAIRER2R/(PAIRWTR*DBLE(CLOSTOTR)) )
	      BWTTOT(NBCLOSE)=BRATIO(NBCLOSE)*BWTTOT(NBCLOSE)
	      IF (LIST) THEN
	         IF ((MSGLEVEL.GE.3).AND.(    (NBCLOSE.LE.NCPRINT)
     #                                    .OR.(NCPRINT.EQ.0)      ) )
     #              WRITE(PR,'(A)')' '
	         IF ((MSGLEVEL.GE.2).AND.(    (NBCLOSE.LE.NCPRINT)
     #                                    .OR.(NCPRINT.EQ.0)      ) )
     #                      WRITE(PR,'(X,
     #                      ''- - - A V E R A G E - - -'',X,I4,X,A,
     #                      4X,I4,X,A,2X,F6.2,2X,F7.3,2X,
     #                      F8.4)')
     #              CLOSTOTR,BNAME(I)(:MXLBN),
     #              CLOSTOT,BNAME(J)(:MXLBN),CLOSUVA,BRATIO(NBCLOSE),
     #              BWTTOT(NBCLOSE)
	         IF ((MSGLEVEL.GE.3).AND.(    (NBCLOSE.LE.NCPRINT)
     #                                    .OR.(NCPRINT.EQ.0)      ) )
     #              THEN
	            WRITE(PR,'(A)')' '
	            WRITE(PR,'(A)')' '
	         ENDIF
	      ENDIF
C
210	      CONTINUE	! 210: No close points found within UV-window
	   ENDDO	! End baseline J
211	   CONTINUE	! 211: Baseline I unusable
	ENDDO		! End baseline I
C
C
C		Solve for the station gain corrections (if possible)
C
	IF (NBCLOSE.EQ.0) THEN
	   WRITE(OUTC,'(A)')' NO CROSSING POINTS FOUND.'
	   IF (LIST) WRITE(PR,'(A)')' NO CROSSING POINTS FOUND.'
	   IF (EDIF) THEN
	      WRITE(PR2,'(A)')'! NO CROSSING POINTS FOUND.'
	      WRITE(PR2,'(A)')'/'
	      IF (VMS) THEN
	         WRITE(PR2,'(A)')'$ EXIT'
	      ELSE
	         WRITE(PR2,'(A)')'-END-'
	      ENDIF
	   ENDIF
	   GOTO 1000
	ELSE
	   WRITE(OUTC,'(13X,I5,A)')NBCLOSE,' baseline "crossings" found'
	   IF (LIST) THEN
	      WRITE(PR,'(A)')' '
	      IF ((MSGLEVEL.GE.2).AND.(     (NBCLOSE.GT.NCPRINT)
     #                                 .AND.(NCPRINT.NE.0)      ) )
     #           WRITE(PR,'(A)')' FURTHER PRINTING HAS BEEN SUPPRESSED'
     #                          //' BY NCPRINT SETTING !!!'
	      WRITE(PR,'(A,I5)')
     #           ' Total number of baseline "crossings" found: ',NBCLOSE
	      WRITE(PR,'(A)')' '
	   ENDIF
	   WRITE(OUTC,'(A)')' UVCROSS - Solving for gains...'
	   CALL CLUSSOL (BAS1,BAS2,BRATIO,BWTTOT,NBCLOSE,NS,STNAME,
     1		       GAIN,NOSOLVE,NB1,NB2,NB,LIST,PR,OUTC,NCPRINT)
C		"Full" printout occurs inside CLUSSOL.
C		Now do EDITFILE if requested.
C
	   IF (EDIF) THEN
	      DO I=1,NS
	         IF (.NOT.NOSOLVE(I)) THEN
                    WRITE(PR2,'(A,F6.3,A,A,A,I5,A)')'MULTIPLY ',
     #                GAIN(I),'  ',STNAME(I),'  AMP  INT = 1,',NREC,' /'
                    WRITE(PR2,'(A,F6.3,A,A,A,I5,A)')' ERRMULT ',
     #                GAIN(I),'  ',STNAME(I),'  AMP  INT = 1,',NREC,' /'
	         ENDIF
	      ENDDO
	      WRITE(PR2,'(A)')'/'
	      IF (VMS) THEN
	         WRITE(PR2,'(A)')'$ EXIT'
	      ELSE
	         WRITE(PR2,'(A)')'-END-'
	      ENDIF
	   ENDIF
	ENDIF
C
C		The end.
C
1000	CONTINUE
	IF (EDIF) CLOSE(UNIT=PR2)
	IF (LIST) CLOSE(UNIT=PR)
	STOP
C
C	Crude traps for file errors.
C
1010	CONTINUE
	WRITE(OUTC,'(A)')' '
	WRITE(OUTC,'(A)')' Could not open input file !!!'
	WRITE(OUTC,'(A)')' '
	NERR=NERR+1
	IF (NERR.GT.MXNERR)
     #     CALL ERROR('ABORTING; ERROR COUNT EXCEEDED.')
	GOTO 100
C
1020	CONTINUE
	WRITE(OUTC,'(A)')' '
	WRITE(OUTC,'(A)')' Empty input file !!!'
	WRITE(OUTC,'(A)')' '
	CLOSE(UNIT=INDAT)
	NERR=NERR+1
	IF (NERR.GT.MXNERR)
     #     CALL ERROR('ABORTING; ERROR COUNT EXCEEDED.')
	GOTO 100
C
1025	CONTINUE
	WRITE(OUTC,'(A)')' '
	WRITE(OUTC,'(A)')' Could not open edit-file !!!'
	WRITE(OUTC,'(A)')' '
	CLOSE(UNIT=INDAT)
	NERR=NERR+1
	IF (NERR.GT.MXNERR)
     #     CALL ERROR('ABORTING; ERROR COUNT EXCEEDED.')
	GOTO 100
C
1030	CONTINUE
	WRITE(OUTC,'(A)')' '
	WRITE(OUTC,'(A)')' Could not open listing file !!!'
	WRITE(OUTC,'(A)')' '
	IF (EDIF) CLOSE(UNIT=PR2,DISP='DELETE')
	CLOSE(UNIT=INDAT)
	NERR=NERR+1
	IF (NERR.GT.MXNERR)
     #     CALL ERROR('ABORTING; ERROR COUNT EXCEEDED.')
	GOTO 100
C
	END
C
	SUBROUTINE CLUSSOL(BAS1,BAS2,RATIO,SIGMA,NCLOSE,NSTA,
     1			 STNAME,GAIN,NOSOLVE,NB1,NB2,NB,LIST,
     2                   PR,OUTC,NCPRINT)
	IMPLICIT NONE
C-----------------------------------------------------------------
C
C	This subroutine determines clusters of dependent station gains
C	from a set of flux density ratios at the (u,v) crossing points.
C
C	It calls SOLVE to make the actual gain solutions.
C
C History:
C
C Originally called SOLVE, written 13Nov84 MWH
C
C Version 2.0:	1990 Apr 02 - Better FUNCTN (see comments there) to avoid bias.
C                           - Prevent overall gain from drifting.
C                           - Solve in clusters of dependent stations.
C                           - Introduce NOSOLVE flags.
C                           - Beneficial side-effect of all of the above: 
C                             results no longer dependent on order of stations.
C                           - Correct small bug in parabolic minimum search.
C                           - Change declarations to agree with main program. 
C                                        Rene C. Vermeulen.
C Version 2.1   1990 May 04 - Tidy up output.
C                                        Rene C. Vermeulen.
C Version 2.2   1990 Dec 04 - Add NCPRINT to restrict ratio output.
C                           - More real*8 calculations.
C                                        Rene C. Vermeulen.
C Version 3.0   1991 Apr 23 - Changed to CLUSSOL, which calls new routine SOLVE.
C                           - Fix inconsistency in handling of "4-way" ratios.
C                           - Expand and clarify handling of 4-way ratios.
C                           - Do double round of solutions, forks first,
C                             with 4-way ratios added in second round.
C                           - Loop back over clusters.
C                                        Rene C. Vermeulen.
C
C-----------------------------------------------------------------
C
	INTEGER  NCLOSE,NSTA,NB,OUTC,PR,NCPRINT
	LOGICAL  LIST
C
	CHARACTER*8  STNAME(NSTA)
	LOGICAL  NOSOLVE(NSTA)
	INTEGER  BAS1(NCLOSE),BAS2(NCLOSE)
	INTEGER  NB1(NB),NB2(NB)
	REAL*8   RATIO(NCLOSE),SIGMA(NCLOSE)
	REAL*8   GAIN(NSTA)
C
	INTEGER LEN1
	EXTERNAL LEN1
C
	INTEGER   MAXS,MAXB,MAXBC
	PARAMETER (MAXS=30)	! maximum number of stations
	PARAMETER (MAXB=(MAXS-1)*MAXS/2)	! Max. no. baselines
	PARAMETER (MAXBC=MAXB*(MAXB-1)/2)	! Max. no. basel. crossings
C
	LOGICAL  FORK(MAXBC)
	INTEGER  GRID(MAXS,MAXS), CLNO(MAXS), NSTACL(MAXS)
	INTEGER  CROSGR(MAXBC), IST(4,MAXBC)
	REAL*8   DELTA(MAXS)
C
	CHARACTER*37 CNAME
	CHARACTER*11 USESTR
	LOGICAL  NEWINFO,NONOPT,LESSPR,UNCONST
	INTEGER  I,J,K,L1,L2,L3,L4,N4WAY,NUNDEF
	INTEGER  MAXLS,MAXLB,NEXTCLUS,MAXCLUS,CURCLUS
	INTEGER  M(4)
C				Setup
	NONOPT=.FALSE.
	NEXTCLUS=1
	DO I=1,NSTA
	   GAIN(I)=1.0		! Initialize gains to 1
	   CLNO(I)=0		! Initialise cluster
	   NSTACL(I)=0		! membership to 0
	   DELTA(I)=.05		! Set initial stepsizes in gain to 5%
	   DO J=1,I-1
	      GRID(J,I)=0	! Stations not related by any a-priori ratios
	   ENDDO
	ENDDO
C			  Figure out how the stations are related by
	MAXLS=0			! the available ratios; build GRID.
	MAXLB=0
	N4WAY=0
	DO I=1,NCLOSE			! Loop over ratios
	   IST(1,I) = NB1(BAS1(I))		! Pointers
	   IST(2,I) = NB2(BAS1(I))
	   IST(3,I) = NB1(BAS2(I))
	   IST(4,I) = NB2(BAS2(I))
	   CROSGR(I) = 0		! Ratio does not constrain any group yet
	   FORK(I) = .FALSE.
	   L1 = LEN1(STNAME(IST(1,I)))		! Useful for nice printout
	   L2 = LEN1(STNAME(IST(2,I)))
	   L3 = LEN1(STNAME(IST(3,I)))
	   L4 = LEN1(STNAME(IST(4,I)))
	   MAXLS=MAX(MAXLS,L1)
	   MAXLS=MAX(MAXLS,L2)
	   MAXLS=MAX(MAXLS,L3)
	   MAXLS=MAX(MAXLS,L4)
	   MAXLB=MAX(MAXLB,L1+L2+L3+L4+5)
C				Find out how this ratio is configured.
	   DO J=1,2		! Numerator stations
	      DO K=3,4			! Denominator stations
	         IF (IST(J,I).EQ.IST(K,I)) THEN	! FORK RATIO (2 stations +dummy)
	            FORK(I)=.TRUE.
	            M(1)=IST(3-J,I)
	            M(2)=IST(7-K,I)
	            IF (.NOT.(NOSOLVE(M(1)).OR.NOSOLVE(M(2)))) THEN
	               IF (CLNO(M(1)).NE.0) THEN
	                  IF (CLNO(M(2)).EQ.0) THEN
	                     CALL CLADD(CLNO,NSTACL,NSTA,CROSGR(I),
     #                                  M(2),CLNO(M(1)))
	                  ELSE
	                     CALL CLCOMB(CLNO,NSTACL,NSTA,CROSGR,NCLOSE,
     #                                  CLNO(M(1)),CLNO(M(2)),
     #                                  NEXTCLUS,I)
	                  ENDIF
	               ELSE
	                  IF (CLNO(M(2)).EQ.0) THEN
	                     CALL CLNEW(CLNO,NSTACL,NSTA,CROSGR(I),
     #                                  M(1),M(2),NEXTCLUS)
	                  ELSE
	                     CALL CLADD(CLNO,NSTACL,NSTA,CROSGR(I),
     #                                  M(1),CLNO(M(2)))
	                  ENDIF
	               ENDIF
	            ENDIF
	         ENDIF
	      ENDDO
	   ENDDO
	   IF (     (.NOT.FORK(I))
     #         .AND.
     #              (.NOT.(    NOSOLVE(IST(1,I)) .OR. NOSOLVE(IST(2,I))
     #                     .OR.NOSOLVE(IST(3,I)) .OR. NOSOLVE(IST(4,I))
     #                    )
     #              )
     #        )
     #        N4WAY=N4WAY+1	! Count as tentative 4-way relation
	ENDDO
C
	MAXCLUS=NEXTCLUS-1
C
C		Do a first-pass solution, based purely on fork ratios ?
C
C	CALL SOLVE(RATIO,SIGMA,CROSGR,FORK,IST,NCLOSE,
C     #			 CLNO,GAIN,DELTA,NOSOLVE,NSTA,NSTACL,MAXCLUS)
C
C		Further passes: are any 4-way ratios useful ?
C
C		This part is no doubt incomplete, because it is very tedious !!!
C
C		Basically, only those cases are admitted in which a single 
C		4-way ratio is IN ITSELF sufficient:
C		      - to add a new member to a cluster, or
C		      - to tie 2 clusters together, or
C		      - to create a new cluster.
C
C		If 3 or 4 stations are not in a cluster the ratio is ignored.
C
C		If 2 stations are not in a cluster, the ratio is ignored
C		unless the other 2 belong to the same cluster, and 1 member
C		is in the numerator, and the other in the denominator.
C		In that case, a new separate cluster is made from the
C		2 formerly single stations.
C
C		If 1 station is not in a cluster, the ratio is ignored
C		unless there are at least 2 others that belong to the
C		same cluster, and at least 1 member is in the numerator,
C		and at least 1 in the denominator. The 4-th station is
C		also in a cluster, but that may be a different one. In
C		this case, the formerly single station is tied in with
C		the cluster of the 4-th station. 
C
C		If all stations are already in clusters, then the ratio
C		is ignored if they belong to 4 different ones.
C		Otherwise, if they belong to 3 different clusters, the
C		ratio is still ignored unless the cluster with 2 members
C		is represented both in the numerator and the denominator;
C		in that case, the 2 clusters represented by single members
C		are tied. If the stations belong to 2 clusters, these
C		clusters are tied. The case where all belong to the same 
C		cluster, finally, is trivial.
C
C		All 4-way ratios are checked in turn. If any
C		re-clustering occurs, the process is repeated.
C		If ignored ratios remain in the end, a warning message
C		is printed.
C
	NEWINFO=.FALSE.		! Indicates if 4-way ratios could be useful.
C
	IF (N4WAY.NE.0) NEWINFO=.TRUE.
C
C			  Loop: each time new stations may get
221	CONTINUE	! "frozen"into clusters (indicated by NEWINFO), thus 
C			  allowing yet more links in the next iteration.
	IF (NEWINFO) THEN ! Don't bother if nothing changed any more.
	   NEWINFO=.FALSE.
	   NONOPT=.FALSE.	! Warning flag about non-optimal algorithm
C
	   DO I=1,NCLOSE	! Reconsider all ratios, that is,
C
	      IF ( ( FORK(I) .OR. NOSOLVE(IST(1,I)) ! if they are 4-way
     #                       .OR. NOSOLVE(IST(2,I))
     #                       .OR. NOSOLVE(IST(3,I))
     #                       .OR. NOSOLVE(IST(4,I)) ) ) GOTO 261
C		Determine cluster membership.
	      NUNDEF=0
	      DO J=1,4
	         M(J)=CLNO(IST(J,I))
	         IF (M(J).EQ.0) NUNDEF=NUNDEF+1
	      ENDDO
C
	      IF     (NUNDEF.EQ.4) THEN
	         NONOPT=.TRUE.
C
	      ELSEIF (NUNDEF.EQ.3) THEN
	         NONOPT=.TRUE.
C
	      ELSEIF (NUNDEF.EQ.2) THEN
	         IF (M(1).NE.0) THEN
	            IF     (M(3).EQ.M(1)) THEN
	               CALL CLNEW(CLNO,NSTACL,NSTA,CROSGR(I),
     #                            IST(2,I),IST(4,I),NEXTCLUS)
	               NEWINFO=.TRUE.
	               FORK(I)=.TRUE.	! Effectively no longer 4-way
	            ELSEIF (M(4).EQ.M(1)) THEN
	               CALL CLNEW(CLNO,NSTACL,NSTA,CROSGR(I),
     #                            IST(2,I),IST(3,I),NEXTCLUS)
	               NEWINFO=.TRUE.
	               FORK(I)=.TRUE.
	            ELSEIF (M(2).NE.0) THEN
	               IF     (M(3).EQ.M(2)) THEN
	                  CALL CLNEW(CLNO,NSTACL,NSTA,CROSGR(I),
     #                               IST(1,I),IST(4,I),NEXTCLUS)
	                  NEWINFO=.TRUE.
	                  FORK(I)=.TRUE.
	               ELSEIF (M(4).EQ.M(2)) THEN
	                  CALL CLNEW(CLNO,NSTACL,NSTA,CROSGR(I),
     #                               IST(1,I),IST(3,I),NEXTCLUS)
	                  NEWINFO=.TRUE.
	                  FORK(I)=.TRUE.
	               ELSE
	                  NONOPT=.TRUE.
	               ENDIF
	            ELSE
	               NONOPT=.TRUE.
	            ENDIF
	         ELSEIF (M(2).NE.0) THEN
	            IF     (M(3).EQ.M(2)) THEN
	               CALL CLNEW(CLNO,NSTACL,NSTA,CROSGR(I),
     #                            IST(1,I),IST(4,I),NEXTCLUS)
	               NEWINFO=.TRUE.
	               FORK(I)=.TRUE.
	            ELSEIF (M(4).EQ.M(2)) THEN
	               CALL CLNEW(CLNO,NSTACL,NSTA,CROSGR(I),
     #                            IST(1,I),IST(3,I),NEXTCLUS)
	               NEWINFO=.TRUE.
	               FORK(I)=.TRUE.
	            ELSE
	               NONOPT=.TRUE.
	            ENDIF
	         ELSE
	            NONOPT=.TRUE.
	         ENDIF
C
	      ELSEIF (NUNDEF.EQ.1) THEN
	         IF (M(1).NE.0) THEN
	            IF     (M(3).EQ.M(1)) THEN
	               IF (M(2).EQ.0) THEN
	                  CALL CLADD(CLNO,NSTACL,NSTA,CROSGR(I),
     #                               IST(2,I),M(4))
	               ELSE
	                  CALL CLADD(CLNO,NSTACL,NSTA,CROSGR(I),
     #                               IST(4,I),M(2))
	               ENDIF
	               NEWINFO=.TRUE.
	               FORK(I)=.TRUE.
	            ELSEIF (M(4).EQ.M(1)) THEN
	               IF (M(2).EQ.0) THEN
	                  CALL CLADD(CLNO,NSTACL,NSTA,CROSGR(I),
     #                               IST(2,I),M(3))
	               ELSE
	                  CALL CLADD(CLNO,NSTACL,NSTA,CROSGR(I),
     #                               IST(3,I),M(2))
	               ENDIF
	               NEWINFO=.TRUE.
	               FORK(I)=.TRUE.
	            ELSEIF (M(2).NE.0) THEN
	               IF     (M(3).EQ.M(2)) THEN
	                  IF (M(1).EQ.0) THEN
	                     CALL CLADD(CLNO,NSTACL,NSTA,CROSGR(I),
     #                                  IST(1,I),M(4))
	                  ELSE
	                     CALL CLADD(CLNO,NSTACL,NSTA,CROSGR(I),
     #                                  IST(4,I),M(1))
	                  ENDIF
	                  NEWINFO=.TRUE.
	                  FORK(I)=.TRUE.
	               ELSEIF (M(4).EQ.M(2)) THEN
	                  IF (M(1).EQ.0) THEN
	                     CALL CLADD(CLNO,NSTACL,NSTA,CROSGR(I),
     #                                  IST(1,I),M(3))
	                  ELSE
	                     CALL CLADD(CLNO,NSTACL,NSTA,CROSGR(I),
     #                                  IST(3,I),M(1))
	                  ENDIF
	                  NEWINFO=.TRUE.
	                  FORK(I)=.TRUE.
	               ELSE
	                  NONOPT=.TRUE.
	               ENDIF
	            ELSE
	               NONOPT=.TRUE.
	            ENDIF
	         ELSE
	            IF     (M(3).EQ.M(2)) THEN
	               IF (M(1).EQ.0) THEN
	                  CALL CLADD(CLNO,NSTACL,NSTA,CROSGR(I),
     #                               IST(1,I),M(4))
	               ELSE
	                  CALL CLADD(CLNO,NSTACL,NSTA,CROSGR(I),
     #                               IST(4,I),M(1))
	               ENDIF
	               NEWINFO=.TRUE.
	               FORK(I)=.TRUE.
	            ELSEIF (M(4).EQ.M(2)) THEN
	               IF (M(1).EQ.0) THEN
	                  CALL CLADD(CLNO,NSTACL,NSTA,CROSGR(I),
     #                               IST(1,I),M(3))
	               ELSE
	                  CALL CLADD(CLNO,NSTACL,NSTA,CROSGR(I),
     #                               IST(3,I),M(1))
	               ENDIF
	               NEWINFO=.TRUE.
	               FORK(I)=.TRUE.
	            ELSE
	               NONOPT=.TRUE.
	            ENDIF
	         ENDIF
C
	      ELSE				! NUNDEF = 0
	         IF     (M(1).EQ.M(2)) THEN
	            IF     (M(1).EQ.M(3)) THEN
	               IF (M(1).EQ.M(4)) THEN
	                   FORK(I)=.TRUE.		! Trivial case
	               ELSE
	                  CALL CLCOMB(CLNO,NSTACL,NSTA,CROSGR,NCLOSE,
     #                                M(1),M(4),NEXTCLUS,I)
	                  NEWINFO=.TRUE.		! Only #4 different
	                  FORK(I)=.TRUE.
	               ENDIF
	            ELSEIF (M(1).EQ.M(4)) THEN
	               CALL CLCOMB(CLNO,NSTACL,NSTA,CROSGR,NCLOSE,
     #                             M(1),M(3),NEXTCLUS,I)
	               NEWINFO=.TRUE.		! Only #3 diferent
	               FORK(I)=.TRUE.
	            ELSEIF (M(3).EQ.M(4)) THEN
	               CALL CLCOMB(CLNO,NSTACL,NSTA,CROSGR,NCLOSE,
     #                             M(1),M(3),NEXTCLUS,I)
	               NEWINFO=.TRUE.		! #1,2 and #3,4 are pairs
	               FORK(I)=.TRUE.
	            ELSE
	               NONOPT=.TRUE.		! #1,2 pair, 3 and 4 different.
	            ENDIF
	         ELSEIF (M(1).EQ.M(3)) THEN
	            FORK(I)=.TRUE.
	            IF (M(1).EQ.M(4)) THEN
	               CALL CLCOMB(CLNO,NSTACL,NSTA,CROSGR,NCLOSE,
     #                             M(1),M(2),NEXTCLUS,I)
	               NEWINFO=.TRUE.		! Only #2 diferent
	            ELSEIF (M(2).NE.M(4)) THEN
	               CALL CLCOMB(CLNO,NSTACL,NSTA,CROSGR,NCLOSE,
     #                             M(2),M(4),NEXTCLUS,I)
	               NEWINFO=.TRUE.		! Superfluous otherwise
	            ENDIF
	         ELSEIF (M(1).EQ.M(4)) THEN
	            FORK(I)=.TRUE.
	            IF (M(2).NE.M(3)) THEN
	               CALL CLCOMB(CLNO,NSTACL,NSTA,CROSGR,NCLOSE,
     #                             M(2),M(4),NEXTCLUS,I)
	               NEWINFO=.TRUE.		! Superfluous otherwise
	            ENDIF
	         ELSEIF (M(2).EQ.M(3)) THEN
	            FORK(I)=.TRUE.
	               CALL CLCOMB(CLNO,NSTACL,NSTA,CROSGR,NCLOSE,
     #                             M(1),M(4),NEXTCLUS,I)
	            NEWINFO=.TRUE.
	         ELSEIF (M(2).EQ.M(4)) THEN
	            FORK(I)=.TRUE.
	            CALL CLCOMB(CLNO,NSTACL,NSTA,CROSGR,NCLOSE,
     #                          M(1),M(3),NEXTCLUS,I)
	            NEWINFO=.TRUE.
	         ELSE
	            NONOPT=.TRUE.	! #1 and #2 different from eachother
	         ENDIF			! and from #3 and #4
	      ENDIF
261	      CONTINUE		! Unchecked ratios end up here.
	   ENDDO
	   GOTO 221
	ENDIF
C
	MAXCLUS=NEXTCLUS-1	      
C
	IF (NONOPT) THEN
C	                  Print warning if complicated situations arose
	   WRITE(OUTC,'(X,/,A,/,A,X)')
     #        ' WARNING: Clustering algorithm may have failed to make',
     #        '          optimal use of all 4-way ratios available !'
	   IF (LIST) WRITE(PR,'(X,/,A,/,A,X)')
     #        ' WARNING: Clustering algorithm may have failed to make',
     #        '          optimal use of all 4-way ratios available !'
	ENDIF
C
C			Start on the solution, cluster by cluster
C
	CALL SOLVE(RATIO,SIGMA,CROSGR,FORK,IST,NCLOSE,
     #			 CLNO,GAIN,DELTA,NOSOLVE,NSTA,NSTACL,MAXCLUS)
C
C	print solution per cluster.
	DO CURCLUS=1,MAXCLUS
	   WRITE(OUTC,535)CURCLUS
	   IF (LIST) WRITE(PR,535)CURCLUS
	   DO I=1,NSTA
	      IF (CLNO(I).EQ.CURCLUS) THEN
	         WRITE(OUTC,540) STNAME(I)(:MAXLS),GAIN(I)
	         IF (LIST) WRITE(PR,540) STNAME(I)(:MAXLS),GAIN(I)
	      ENDIF
	   ENDDO
	ENDDO
C			Done: print some wrap-up information
C				! Warn if not all stations were solved.
	UNCONST=.FALSE.
	DO I=1,NSTA
	   IF (CLNO(I).EQ.0) UNCONST=.TRUE.
	ENDDO
	IF (UNCONST) THEN
	   WRITE(OUTC,'(X,/,A,/,X)')
     #        ' The gain of the following station(s) is not'
     #        //' constrained:'
	   IF (LIST) WRITE(PR,'(X,/,A,/,X)')
     #        ' The gain of the following station(s) is not'
     #        //' constrained:'
	   DO I=1,NSTA
	      IF (CLNO(I).EQ.0) THEN
	         IF (NOSOLVE(I)) THEN
	            WRITE(OUTC,'(10X,A,A)')STNAME(I)(:MAXLS),
     #                                   ' (NOSOLVE requested)'
	            IF (LIST) WRITE(PR,'(10X,A,A)')STNAME(I)(:MAXLS),
     #                                   ' (NOSOLVE requested)'
	         ELSE
	            WRITE(OUTC,'(10X,A  )')STNAME(I)(:MAXLS)
	            IF (LIST) WRITE(PR,'(10X,A  )')STNAME(I)(:MAXLS)
	         ENDIF
	      ENDIF
	   ENDDO
	ENDIF
C
	IF (LIST) THEN		! Summary: ratios and corrected ratios
	   WRITE (PR,516)
	   LESSPR=.FALSE.
	   IF ((NCPRINT.NE.0).AND.(NCLOSE.GT.NCPRINT)) THEN
	      LESSPR=.TRUE.
	      NCLOSE=NCPRINT
	   ENDIF
	   DO I=1,NCLOSE
	      L1 = LEN1(STNAME(IST(1,I)))
	      L2 = LEN1(STNAME(IST(2,I)))
	      L3 = LEN1(STNAME(IST(3,I)))
	      L4 = LEN1(STNAME(IST(4,I)))
	      CNAME=STNAME(IST(1,I))(1:L1)//'-'//
     #              STNAME(IST(2,I))(1:L2)//' / '//
     #              STNAME(IST(3,I))(1:L3)//'-'//
     #              STNAME(IST(4,I))(1:L4)
	      USESTR=' '
	      IF (.NOT.FORK(I)) USESTR='(NOT USED)'
	      WRITE(PR,501) CNAME(:MAXLB),RATIO(I),SIGMA(I),
     #           RATIO(I)*GAIN(IST(1,I))*GAIN(IST(2,I)) / 
     #                    GAIN(IST(3,I))/GAIN(IST(4,I)),USESTR
	   ENDDO
	   IF (LESSPR) THEN
	      WRITE (PR,'(A)')' '
	      WRITE (PR,'(A)')' FURTHER PRINTING HAS BEEN SUPPRESSED'
     #                        //' BY NCPRINT SETTING !!!'
	   ENDIF
	ENDIF
C			Bye-bye !
	RETURN
C
  501   FORMAT(1X,'Ratio ',A,' (',F7.3,
     1		  ' +/- ',F8.4,' ) would become ',F7.3,2X,A)
  516   FORMAT(1X,/,1X,/,1X,
     1         'This solution would have the following effect:',/)
  535	FORMAT(1X,/,1X,
     #     'The UVCROSS "least-squares" solution for cluster ',I2,':',/)
  540   FORMAT(1X,'Multiply ',A,' amplitudes by ',F6.3)
C
	END
C
	SUBROUTINE SOLVE(RATIO,SIGMA,CROSGR,FORK,IST,NCLOSE,
     #			 CLNO,GAIN,DELTA,NOSOLVE,NSTA,NSTACL,MAXCLUS)
	IMPLICIT NONE
C-----------------------------------------------------------------
C
C	This subroutine makes gain solutions based on flux density
C	ratios at uv crossing points. Clusters of dependent stations must
C	be indicated.
C
C History:
C
C Loosely Based on SOLVE (Nov 16 1984) by Mark W. Hodges.
C
C Version  1.0  23 Apr 1991  Written by Rene C. Vermeulen.
C
C
	INTEGER  NCLOSE
	INTEGER  CROSGR(NCLOSE)
	INTEGER  IST(4,NCLOSE)
	REAL*8   RATIO(NCLOSE),SIGMA(NCLOSE),FORK(NCLOSE)
C
	INTEGER  NSTA
	INTEGER  CLNO(NSTA)
	REAL*8   GAIN(NSTA),DELTA(NSTA)
	LOGICAL  NOSOLVE(NSTA)
C
	INTEGER  MAXCLUS
	INTEGER  NSTACL(MAXCLUS)
C
	INTEGER  I,J
	INTEGER  CURCLUS,ITER
	REAL*8   PREV,DEL,TOT1,TOT2,TOT3,TOTERR,FN,SAVE
	REAL*8   GAICOR



C			Start on the solution, cluster by cluster
C
	DO CURCLUS=1,MAXCLUS
	   IF (NSTACL(CURCLUS).NE.0) THEN
	      PREV=1.0E10		! Old value of chisquared
  31	      DO I=1,NSTA			! loop through each station
	         IF (CLNO(I).EQ.CURCLUS) THEN	! which is a clustermember
C
C				Next part adapted from Version 1 of UVCROSS
C				It works, but may not be optimal or efficient.
C
	            DEL=DELTA(I)
	            CALL FUNCTN (GAIN,RATIO,SIGMA,CROSGR,CURCLUS,I,
     #                           IST,NSTA,NCLOSE,TOT1)
	            FN=0
	            ITER=0
  41	            CONTINUE
	            GAIN(I)=GAIN(I)+DEL	! increment the station gain
	            CALL FUNCTN (GAIN,RATIO,SIGMA,CROSGR,CURCLUS,I,
     #                           IST,NSTA,NCLOSE,TOT2)
	            ITER=ITER+1
	            IF(ITER.GE.25) GO TO 61
	            IF(TOT1-TOT2) 51,41,61
  51	            CONTINUE
	            DEL=-DEL
	            GAIN(I)=GAIN(I)+DEL
	            SAVE=TOT1
	            TOT1=TOT2
	            TOT2=SAVE
  61                CONTINUE
	            FN=FN+1
	            GAIN(I)=GAIN(I)+DEL
	            CALL FUNCTN (GAIN,RATIO,SIGMA,CROSGR,CURCLUS,I,
     #                           IST,NSTA,NCLOSE,TOT3)
	            IF(TOT3-TOT2) 71,91,81
  71                CONTINUE
	            TOT1=TOT2
	            TOT2=TOT3
	            GO TO 61
  81                CONTINUE
	            DEL=DEL*(1./(1.+(TOT1-TOT2)/(TOT3-TOT2))+0.5)
	            GOTO 101
  91	            CONTINUE
	            DEL=0.5*DEL
 101                CONTINUE
	            GAIN(I)=GAIN(I)-DEL
	            DELTA(I)=DELTA(I)*FN/3.
	         ENDIF
	      ENDDO
	      CALL FUNCTN (GAIN,RATIO,SIGMA,CROSGR,CURCLUS,0,
     #                     IST,NSTA,NCLOSE,TOTERR)
	      IF (TOTERR.NE.0.) THEN
	         IF(ABS((PREV-TOTERR)/TOTERR).GT.0.001) THEN
	            PREV=TOTERR
	            GO TO 31
	         ENDIF
	      ENDIF
C			Converged for this cluster.
C
C	                Prevent gain scale from floating.
C
	      GAICOR=1.
	      DO J=1,NSTA
	         IF (CLNO(J).EQ.CURCLUS) GAICOR=GAIN(J)*GAICOR
	      ENDDO
	      GAICOR=GAICOR**(-1./FLOAT(NSTACL(CURCLUS)))
	      DO J=1,NSTA
	         IF (CLNO(J).EQ.CURCLUS) GAIN(J)=GAIN(J)*GAICOR
	      ENDDO
C
	   ENDIF
	ENDDO
C			Done.
	RETURN
	END
C
	SUBROUTINE FUNCTN (GAIN,RATIO,SIGMA,CROSGR,CURCLUS,CURST,
     #                     IST,NSTA,NCLOSE,TOTERR)
	IMPLICIT NONE
C---------------------------------------------------------------
C
C	This subroutine calculates the total error due to station
C	gain errors. The total error is returned as TOTERR.
C
C History:
C
C Version 1.0	1984 Nov 16   Mark W. Hodges
C
C Version 2.0   1990 Mar 29   Rene C. Vermeulen
C Version 2.1   1990 Dec 04   More real*8 calculations. RCV
C Version 3.0   1991 Apr 23   Add CROSGR, CURST selection for each ratio. RCV
C
C Discussion:
C
C In version 1.0: 
C
C            sum      1       G1 G2                2
C  TOTERR =  over    ---  * ( ----- * RATIO  -  1 )
C           ratios  SIGMA     G3 G4
C
C Call the ratio times the four gains the "corrected ratio".
C
C The above function indicates by WHAT AMOUNT the corrected ratios
C differ from 1.
C
C However, since the corrections are made by MULTIPLICATION, one should
C instead test by WHAT FACTOR the ratios differ from 1. In other words,
C a ratio of 0.1 should be considered equally bad as a ratio of 10, not 
C as a ratio of 1.9 ! 
C
C Arguments for a better function are then as follows: 
C The product of all corrected ratios should ideally be 1. Thus, the sum 
C of the logs should be as close as possible to 0. To enforce least SQUARES, 
C minimise the sum of the squared logs. Furthermore, each term in the sum
C of a least squares function should be weighted by a variance, not by a
C standard deviation. This gives: 
C
C            sum          1                G1 G2            2
C  TOTERR =  over     ---------  * [ log ( ----- * RATIO ) ]
C           ratios   SIGMA*SIGMA           G3 G4
C
C At the end of the routine, TOTERR is normalised by the sum of the
C variances, and the square root is taken.
C
C                                   Rene Vermeulen.
C
C---------------------------------------------------------------
	INTEGER   NCLOSE,NSTA,CURCLUS,CURST
	INTEGER   CROSGR(NCLOSE)
	INTEGER   IST(4,NCLOSE)
	REAL*8    RATIO(NCLOSE),SIGMA(NCLOSE),GAIN(NSTA),TOTERR
C
	INTEGER   I
	REAL*8    ERROR,TOTERR1,TOTSIG
C
	TOTERR1=0.
	TOTSIG=0.
	DO I=1,NCLOSE
	   IF ((CURCLUS.EQ.0).OR.(CROSGR(I).EQ.CURCLUS)) THEN
	      IF (    (CURCLUS.EQ.0) .OR. (CURST.EQ.0)
     #            .OR.(IST(1,I).EQ.CURST)
     #            .OR.(IST(2,I).EQ.CURST)
     #            .OR.(IST(3,I).EQ.CURST)
     #            .OR.(IST(4,I).EQ.CURST)) THEN
     #            
C
C		Original code MWH in version 1:
C
C	          TOTERR=TOTERR+(GAIN(IST(I,1))*GAIN(IST(I,2))/
C     1	          GAIN(IST(I,3))/GAIN(IST(I,4))*RATIO(I)-1.)**2/SIGMA(I)
C
C		Version 2.0 and later, Rene Vermeulen:
C
	         ERROR=GAIN(IST(1,I))*GAIN(IST(2,I)) /
     #                 (GAIN(IST(3,I))*GAIN(IST(4,I)))
	         ERROR=ERROR*RATIO(I)
	         ERROR=LOG(ERROR)
	         ERROR=ERROR*ERROR
	         ERROR=ERROR/(SIGMA(I)*SIGMA(I))
C
	         TOTERR1=TOTERR1+ERROR
	         TOTSIG=TOTSIG+1./(SIGMA(I)*SIGMA(I))
	      ENDIF
	   ENDIF
	ENDDO
C
	IF (TOTSIG.NE.0.) THEN
	   TOTERR=SQRT(TOTERR1/TOTSIG)
	ELSE
	   TOTERR=1.E37
	ENDIF
C
	RETURN
	END
C
C 	SUBROUTINE GNSCLE(GAIN,CLNO,NSTA,NSTACL,MAXCLUS,CURCLUS)
C 	IMPLICIT NONE
C 
C 	Serves SOLVE.
C 	Adjusts the GAIN of all stations in cluster CURCLUS
C 	such that their product is 1.
C 
C 	History:
C 
C 	Version  1.0  25 Mar 1991   Written by Rene C. Vermeulen
C 
C 
C 	INTEGER NSTA,MAXCLUS,CURCLUS
C 	INTEGER CLNO(NSTA),NSTACL(MAXCLUS)
C 	REAL*8  GAIN(NSTA)
C 
C 	INTEGER I
C 	REAL*8  GAICOR
C 
C 	GAICOR = 1.
C 
C 	DO I=1,NSTA
C 	   IF (CLNO(I).EQ.CURCLUS) GAICOR = GAICOR*GAIN(I)
C 	ENDDO
C 
C 	GAICOR=GAICOR**(1./FLOAT(NSTACL(CURCLUS)))
C  
C 	DO I=1,NSTA
C 	   IF (CLNO(I).EQ.CURCLUS) GAIN(I) = GAIN(I)/GAICOR
C 	ENDDO
C 
C 	RETURN
C 	END
C
C 	SUBROUTINE GNREST(GAIN,CLNO,NSTA,ST,NSTACL,MAXCLUS,CURCLUS,
C     #                    DEL,SIGN)
C 	IMPLICIT NONE
C 
C 	Serves SOLVE.
C 
C 	Performs the inverse gain adjustment of GNSCLE;
C 	when these routines are called in succession, they
C 	should cancel out EXCEPT when GAIN(st) < 0.05 or > 20.0.
C 
C 	History:
C 
C 	Version  1.0  22 Mar 1991   Written by Rene C. Vermeulen
C 
C 
C 	INTEGER NSTA,ST,MAXCLUS,CURCLUS
C 	INTEGER CLNO(NSTA),NSTACL(MAXCLUS)
C 	REAL*8  DEL,SIGN
C 	REAL*8  GAIN(NSTA)
C 
C 	INTEGER I
C 	REAL*8  GAICOR
C 
C 	GAICOR=GAIN(ST)
C 
C 	GAIN(ST)=GAIN(ST)/(1.+SIGN*DEL)! reset the station gain
C 
C 	GAICOR=GAICOR/GAIN(ST)
C 	GAICOR=GAICOR**(1./FLOAT(NSTACL(CURCLUS)-1))
C 	DO I=1,NSTA		! overall gain scale may not drift.
C 	   IF ((CLNO(I).EQ.CURCLUS).AND.(I.NE.ST)) 
C     #        GAIN(I)=GAIN(I)*GAICOR
C 	ENDDO
C 
C 	RETURN
C 	END
C
      SUBROUTINE DRSORT(A,M,N,IND)
C-----------------------------------------------------------------------
C Indexed sort of a real array A into decreasing order. Shell sort.
C Array IND(N) is sorted so that IND(I) is the index of array A(N).
C
C Adapted by R.C. Vermeulen from a DRSORT provided by T.J. Pearson.
C Dec 07 1990
C
C Subroutines required:
C   None.
C-----------------------------------------------------------------------
      INTEGER M,N
      REAL A(M)
      INTEGER IND(N)
C
      INTEGER IGAP, IMAX, I, J, JPLUSG
      LOGICAL IEX
C
      IGAP=N
      DO WHILE (IGAP.GT.1)
         IGAP=IGAP/2
         IMAX=N-IGAP
         IEX=.TRUE.
         DO WHILE(IEX)
            IEX=.FALSE.
            DO I=1,IMAX
               J=IND(I)
               JPLUSG=IND(I+IGAP)
               IF(A(J).LT.A(JPLUSG)) THEN
                  IND(I)=JPLUSG
                  IND(I+IGAP)=J
                  IEX=.TRUE.
               ENDIF
            ENDDO
         ENDDO
      ENDDO
      RETURN
      END
C
	SUBROUTINE CLNEW(CLUSTER,NSTACL,NSTA,CROSGR1,ST1,ST2,NEXTCLUS)
	IMPLICIT NONE
C
C	Cluster creation routine to serve SOLVE
C
C	Makes station number ST1 and ST2 the first 2 members of 
C	a new cluster, number NEXTCLUS.
C	NEXTCLUS is then incremented by 1.
C
C	Rene Vemeulen, 18 Mar 1991
C
	INTEGER NSTA
	INTEGER CLUSTER(NSTA),NSTACL(NSTA)
	INTEGER CROSGR1
	INTEGER ST1,ST2,NEXTCLUS
C
	IF ((NEXTCLUS.LT.1).OR.(NEXTCLUS.GT.NSTA))
     #     CALL ERROR('IMPROPER CALL TO CLNEW')
	IF (NSTACL(NEXTCLUS).NE.0) CALL ERROR('IMPROPER CALL TO CLNEW')
C
	CALL CLADD(CLUSTER,NSTACL,NSTA,CROSGR1,ST1,NEXTCLUS)
	CALL CLADD(CLUSTER,NSTACL,NSTA,CROSGR1,ST2,NEXTCLUS)
C
	NEXTCLUS = NEXTCLUS+1
C
	RETURN
C
	END
C
	SUBROUTINE CLADD(CLUSTER,NSTACL,NSTA,CROSGR1,ST1,CLUSNO)
	IMPLICIT NONE
C
C	Cluster extension routine to serve SOLVE
C
C	Makes station number ST1 a member of 
C	cluster number CLUSNO.
C
C	Rene Vemeulen, 19 Mar 1991
C
	INTEGER NSTA
	INTEGER CLUSTER(NSTA),NSTACL(NSTA)
	INTEGER CROSGR1,ST1,CLUSNO
C
	IF ((ST1.LT.1).OR.(ST1.GT.NSTA)) 
     #     CALL ERROR('IMPROPER CALL TO CLADD')
C
	IF ((CLUSNO.LT.1).OR.(CLUSNO.GT.NSTA))
     #     CALL ERROR('IMPROPER CALL TO CLADD')
C
	IF (CLUSTER(ST1).EQ.0) THEN
	   CLUSTER(ST1) = CLUSNO
	   NSTACL(CLUSNO) = NSTACL(CLUSNO) + 1
	ELSEIF (CLUSTER(ST1).NE.CLUSNO) THEN
	   CALL ERROR('IMPROPER CALL TO CLADD')
	ENDIF
C
	CROSGR1 = CLUSNO
C
	RETURN
C
	END
C
	SUBROUTINE CLCOMB(CLUSTER,NSTACL,NSTA,CROSGR,NCROSS,
     #                    CL1,CL2,NEXTCLUS,CROSNO)
	IMPLICIT NONE
C
C	Cluster combination routine to serve SOLVE
C
C	Let MN be the lesser of CL1 and CL2, and MX the greater.
C	This routine sets all array elements of CLUSTER(NSTA) which have
C	value MX to value MN.
C	This routine also adds NSTACL(MX) to NSTACL(MN), then sets
C	NSTACL(MX) to 0.
C
C	Rene C. Vemeulen, Apr 02 1990
C
C	Mar 18 1991   Modified to renumber clusters. RCV
C
	INTEGER NSTA,NCROSS,CL1,CL2,NEXTCLUS,CROSNO
	INTEGER CLUSTER(NSTA),NSTACL(NSTA),CROSGR(NCROSS)
C
	INTEGER I,MN,MX
C
	IF (CROSNO.GT.NCROSS) CALL ERROR('IMPROPER CALL TO CLCOMB')
	IF (CROSNO.LT.1) CALL ERROR('IMPROPER CALL TO CLCOMB')
C
	MN = MIN(CL1,CL2)
	MX = MAX(CL1,CL2)
C
	IF (MN.EQ.0) CALL ERROR('IMPROPER CALL TO CLCOMB')
	IF (MX.GT.NSTA) CALL ERROR('IMPROPER CALL TO CLCOMB')
C
	CROSGR(CROSNO) = MN
C
C
	IF (MN.NE.MX) THEN
C
	   DO I=1,NSTA
	      IF (CLUSTER(I).EQ.MX) CLUSTER(I) = MN
	      IF (CLUSTER(I).GT.MX) CLUSTER(I) = CLUSTER(I)-1
	   ENDDO
C
	   DO I=1,NCROSS
	      IF (CROSGR(I).EQ.MX) CROSGR(I) = MN
	      IF (CROSGR(I).GT.MX) CROSGR(I) = CROSGR(I) - 1
	   ENDDO
C
	   NSTACL(MN) = NSTACL(MN)+NSTACL(MX)
	   DO I=MX,NEXTCLUS-1
	      NSTACL(I) = NSTACL(I+1)
	   ENDDO
C
	   NEXTCLUS = NEXTCLUS-1
C
	ENDIF
C
C
	RETURN
C
	END
C
	SUBROUTINE BRACKET(A,IND,BEG,END,S,E)
	IMPLICIT NONE
C
C	Service routine for UVCROSS.
C
C	Assumes that IND provides down-sorted access to A.
C
C	Determines S to show the first element in A which
C	is not larger than BEG; this will be A(IND(S)).
C
C	Also determines E to show the last element in A which
C	is not smaller than END; this will be A(IND(E)).
C
C	Rene Vermeulen, Nov 19 1990.
C
	REAL A(*),BEG,END
	INTEGER IND(*),S,E
C
	INTEGER SS,EE,T,TT,Y,YY
C
	IF (A(IND(S)).LE.BEG) THEN
	   T=S
	ELSE
	   T=(S+E)/2
	   TT=S-1	! Just to make sure the while-loop can get started
	   SS=S
	   EE=E
	   DO WHILE (T.NE.TT)
	      IF (A(IND(T)).LE.BEG) THEN
	         EE=T
	         TT=T
	         T=(T+SS)/2
	      ELSE
	         SS=T
	         TT=T
	         T=(T+EE)/2
	      ENDIF
	   ENDDO
	ENDIF
C
	IF (A(IND(E)).GE.END) THEN
	   Y=E
	ELSE
	   Y=(S+E+1)/2
	   YY=E+1	! Just to make sure the while-loop can get started
	   SS=S
	   EE=E
	   DO WHILE (Y.NE.YY)
	      IF (A(IND(Y)).LT.END) THEN
	         EE=Y
	         YY=Y
	         Y=(Y+SS+1)/2
	      ELSE
	         SS=Y
	         YY=Y
	         Y=(Y+EE+1)/2
	      ENDIF
	   ENDDO
	ENDIF
C
	S=T
	E=Y
C
	RETURN
C
	END
C
	SUBROUTINE CLOSTIM(TIME,U,V,INDXU,INDXV,MAXP,START,END,SORTU,
     #                           UREF,VREF,CTIM)
	IMPLICIT NONE
C
C	Service routine for UVCROSS
C
C	Enters in CTIM the time of that data point which is nearest
C	the coordinates UREF, VREF in the uv-plane.
C
C	The data are in arrays TIME, U, V, all of length MAXP.
C	The range to be searched is indicated by START and END, which
C	are in turn entries into an index array; either INDXU (if SORTU
C	is true) or INDXV (if SORTU is false).
C
C	Written 1991 Feb 27     Rene C. Vermeulen
C
	INTEGER  MAXP, START, END, CTIM
	INTEGER  TIME(MAXP), INDXU(MAXP), INDXV(MAXP)
	REAL     UREF, VREF
	REAL     U(MAXP), V(MAXP)
	LOGICAL  SORTU
C
	INTEGER I,IND
	REAL    UVMIN,UVD
C
	IF (SORTU) THEN
	   IND=INDXU(START)
	 ELSE
	   IND=INDXV(START)
	ENDIF
	UVMIN=(U(IND)-UREF)**2.+(V(IND)-VREF)**2.
	CTIM=TIME(IND)
C
	DO I=START+1,END
	   IF (SORTU) THEN
	      IND=INDXU(I)
	   ELSE
	      IND=INDXV(I)
	   ENDIF
	   UVD=(U(IND)-UREF)**2.+(V(IND)-VREF)**2.
	   IF (UVD.LT.UVMIN) THEN
	      UVMIN=UVD
	      CTIM=TIME(IND)
	   ENDIF
	ENDDO
C
	RETURN
	END
