      PROGRAM UVDIST
C-----------------------------------------------------------------------
C UVDIST: to plot amplitude versus uv distance for data in a MERGE-
C format visibility file.
C
C Version 1.0:  1983 Jun  5 - T.J. Pearson, VAX-11 Fortran.
C Version 2.0:  1984 Jan  3 - Add ERRORS parameter; add optional x-axis
C                             in m or km; plot optional gaussian model;
C                             upgrade to MERGE format 2 (TJP/RLM/DLM).
C Version 2.1:  1985 May 24 - Do not die if station coordinates are
C                             missing (TJP).
C Version 3.0:  1985 Jun  4 - Add XAXIS="UVPA" option (TJP).
C Version 3.2:  1987 Mar  7 - Add XAXIS="BAPA" option (APM/TJP).
C Version 3.3:  1987 Aug 15 - Fix bug plotting Gaussians (A K Tzioumis);
C                             correct errors in labelling (TJP).
C Version 3.4:  1987 Sep  5 - remove all GRPCKG calls (TJP).
C-----------------------------------------------------------------------
      CHARACTER*(*) VERSN
      INTEGER INC,OUTC,INDAT,MAXS,MAXB,MAXT,MAXWPB
      INTEGER MAXP,NPARS, NPTS
      DOUBLE PRECISION PI,SIDR,VELC
      PARAMETER (VERSN='3.4 - 1987 Sep 5')
      PARAMETER (INC=5,OUTC=6,INDAT=8)
C     -- maximum number of stations
      PARAMETER (MAXS=20)
      PARAMETER (MAXB=(MAXS-1)*MAXS/2)
      PARAMETER (MAXT=MAXB*(MAXS-2)/3)
C     -- maximum words per baseline
      PARAMETER (MAXWPB=7)  
C     -- maximum number of u,v-points  
      PARAMETER (MAXP=50000)
C     -- input parameters
      PARAMETER (NPARS=36)
C     -- number of points for gaussian
      PARAMETER (NPTS=200)    
      PARAMETER (PI=3.141592653589793D0)
C     -- radians of ST per sec of UT
      PARAMETER (SIDR=(PI/2592000.0D0)*1.0027379093D0)
C     -- speed of light, m/s
      PARAMETER (VELC=2.997925D8)
C
      CHARACTER*8  STNAME(MAXS)
      CHARACTER*16 SOURCE
      CHARACTER*17 BNAME(MAXB)
      REAL      REC(MAXWPB*MAXB)
      REAL      AMP(MAXP),AMP1(MAXP),AMP2(MAXP),R(MAXP),ANGPRO(MAXP)
      REAL      XUV(NPTS), G(NPTS)
      REAL      FLUX1, FLUX2, FWHM1, FWHM2
      DOUBLE PRECISION STX,STY,STZ,RA,DEC,GASTM,FREQ,FLUX,
     1          WAVE,BX,BY,BZ(MAXB),BOFF(MAXB),BXY(MAXB),TIHA,
     2          COSDEC,SINDEC
      DOUBLE PRECISION PARS(NPARS),VALS(NPARS),ENDMRK
      LOGICAL   ERRORS, GPLOT, ANGPLT, PAPROJ
      INTEGER   PGBEGIN
      CHARACTER*4  PLTYPE
      CHARACTER*255 INDSN,PLDSN
      CHARACTER*40 LEGEN1
      CHARACTER*(9*MAXS) LEGEN2
      CHARACTER*80 GLABEL, XAXIS, YAXIS
      CHARACTER*20 NOW
      COMMON    /PARMS/ STNAME,STX(MAXS),STY(MAXS),STZ(MAXS),
     1          SOURCE,RA,DEC,GASTM,FREQ,FLUX
      COMMON    /ORDER/ NS,NB,NT, NB1(MAXB),NB2(MAXB),
     2          NC1(MAXT), NC2(MAXT), NC3(MAXT), NVER, NDWPB
      DATA      PARS/ 'XAxis', 'YAxis', 'BAPA',2*'XRange',2*'YRange',
     1                'Mark', 'Linewidt', 'FOnt', 'Dotsize',
     2                'Input', 9*' ', 'Plotfile', 9*' ',
     3                'Errors', 2*'FLux', 2*'FWhm' /
      DATA      VALS/ 'UVDIST', 'AMP', 0, 2*0D0, 2*0D0,
     1                1D0, 1D0, 1D0, 0D0,
     2                '***UNSET', 9*' ',
     3                '?', 9*' ', -1D0, 4*0D0 /
      DATA      ENDMRK/ '/' /
C-----------------------------------------------------------------------
  900 FORMAT(' UVDIST plots amplitudes versus uv distance'/
     1       ' (Version ',A,')'/)
  910 FORMAT('(',A3,I5,', ',F5.2,' GHz)')
  920 FORMAT('Gaussian model:  FLUX =',F7.3,'  FWHM =',F7.3)
  921 FORMAT('Gaussian model:  FLUX =',F7.3,',',F7.3,
     1       '  FWHM =',F7.3,',',F7.3)
  930 FORMAT(' ++WARNING++ Station coordinates for ',A,' are missing')
  940 FORMAT(' ++WARNING++ Zero-length baseline: ',A)
  950 FORMAT(' ++WARNING++ unrecognized value for XAXIS="',A,
     1       '" replaced by XAXIS="UVDIST"')
C-----------------------------------------------------------------------
C
C Introduction.
C
      CALL DATTIM(NOW)
      WRITE (OUTC,900) VERSN
C
C Control parameters.
C
  100 WRITE (OUTC,'(A)')
     1        ' Parameters (eg: INPUT,PLOTFILE,XRANGE,YRANGE):'
      MODE = 0
      CALL KEYIN(PARS,VALS,NPARS,ENDMRK,MODE,INC,OUTC)
      WRITE (XAXIS,'(A8)') VALS(1)
      CALL UPCASE(XAXIS)
      BAPA   = VALS(3)
      XMIN   = VALS(4)
      XMAX   = VALS(5)
      YMIN   = VALS(6)
      YMAX   = VALS(7)
      MARK   = VALS(8)
      LINEWD = VALS(9)
      NFONT  = VALS(10)
      NDOTSZ = VALS(11)
      IF (NDOTSZ.LT.1) NDOTSZ = LINEWD
      WRITE (INDSN,'(10A8)') (VALS(I),I=12,21)
      WRITE (PLDSN,'(10A8)') (VALS(I),I=22,31)
      ERRORS = VALS(32).NE.-1D0
      GPLOT  = (VALS(33).NE.0D0) .OR. (VALS(34).NE.0D0)
      FLUX1 = VALS(33)
      FLUX2 = VALS(34)
      FWHM1 = VALS(35)
      FWHM2 = VALS(36)
C
C Open input Merge file.
C
      OPEN (UNIT=INDAT, FILE=INDSN, READONLY, STATUS='OLD',
     1      FORM='UNFORMATTED', IOSTAT=IER)
      IF (IER.NE.0) THEN
          IF (MODE.EQ.1) CALL ERROR('Cannot open INPUT')
          WRITE (OUTC,'(A)') ' Cannot open INPUT: try again.'
          GOTO 100
      END IF
      INQUIRE (UNIT=INDAT, NAME=INDSN)
C
C Open graphics device.
C
      IER = PGBEGIN(0,PLDSN,1,1)
      IF (IER.NE.1) THEN
          IF (MODE.EQ.1) CALL ERROR('Cannot open PLOTFILE')
          WRITE (OUTC,'(A)') ' Cannot create PLOTFILE: try again.'
          GOTO 100
      END IF
C
C Read header and assemble legends for plot.
C
      CALL HEADX20(.FALSE.,INDAT,IZ,ID,IY,DUM1,0,0,IUNITS,DUM3,IF,0)
      IF (NS.LT.2) CALL ERROR('Not enough stations.')
      IF (NS.GT.MAXS) CALL ERROR('Too many stations.')
      IF (IUNITS.EQ.0) THEN
          YAXIS = 'Correlation Coefficient'
      ELSE IF (IUNITS.EQ.1) THEN
          YAXIS = 'Correlated Flux Density (Jy)'
      ELSE
          YAXIS = 'Amplitude'
      END IF
      IMON = 1
      CALL TDATE(IY,IMON,ID,JD,MONTH,DAY)
      LEGEN1 = SOURCE
      I = LEN1(LEGEN1)+3
      WRITE (LEGEN1(I:),910) MONTH,IY,FREQ*1E-9
      LEGEN2 = STNAME(1)
      DO 120 IS=2,NS
          I = LEN1(LEGEN2)+2
          LEGEN2(I:) = STNAME(IS)
  120 CONTINUE
      WRITE (OUTC,'(1X,A)') LEGEN1(1:LEN1(LEGEN1))
      WRITE (OUTC,'(1X,A)') LEGEN2(1:LEN1(LEGEN2))
C
C XAXIS type.
C
      ANGPLT = .FALSE.
      PAPROJ = .FALSE.
      IF (XAXIS(1:3).EQ.'UVM') THEN
          WAVE = 1D0
          XAXIS = 'Projected baseline (m)'
      ELSE IF (XAXIS(1:3).EQ.'UVK') THEN
          WAVE = 1D3
          XAXIS = 'Projected baseline (km)'
      ELSE IF (XAXIS.EQ.'UVDIST') THEN
          WAVE = (VELC*1D6)/FREQ
          XAXIS = 'Projected baseline (10\u6\d \gl)'
      ELSE IF (XAXIS.EQ.'UVPA') THEN
          WAVE = (VELC*1D6)/FREQ
          XAXIS = 'uv position angle (degrees)'
          GPLOT = .FALSE.
          ANGPLT = .TRUE.
      ELSE IF (XAXIS.EQ.'PROJ') THEN
          WAVE = (VELC*1D6)/FREQ
          WRITE (XAXIS,'(A,I4,A)') 'Baseline projected onto PA',
     1      NINT(BAPA), '^  (10\u6\d \gl)'
          PAPROJ= .TRUE.
      ELSE
          WRITE (OUTC,950) XAXIS(1:8)
          WAVE = (VELC*1D6)/FREQ
          XAXIS = 'Projected baseline (10\u6\d \gl)'
      END IF
C
C Baseline parameters (BXY and BZ are components
C im millions of wavelengths; BOFF is "hour angle").
C
      FACTOR = WAVE*FREQ/(VELC*1D6)
      V2FAC  = VELC/FREQ/WAVE
      COSDEC = DCOS(DEC)
      SINDEC = DSIN(DEC)
      DO 140 IS=1,NS
          IF (STX(IS)*STY(IS)*STZ(IS).EQ.0D0)
     1        WRITE (OUTC,930) STNAME(IS)(1:LEN1(STNAME(IS)))
  140 CONTINUE
      DO 150 IB=1,NB
          IS1 = NB1(IB)
          IS2 = NB2(IB)
          L1 = LEN1(STNAME(IS1))
          L2 = LEN1(STNAME(IS2))
          BNAME(IB) = STNAME(IS1)(1:L1)//'-'//STNAME(IS2)(1:L2)
          BX  = (STX(IS1)-STX(IS2))/WAVE
          BY  = (STY(IS1)-STY(IS2))/WAVE
          BZ(IB) = (STZ(IS1)-STZ(IS2))/WAVE
          BXY(IB) = DSQRT(BX*BX+BY*BY)
          IF (BXY(IB).EQ.0D0) THEN
              WRITE (OUTC,940) BNAME(IB)
              BOFF(IB) = 0.0
          ELSE
              BOFF(IB) = GASTM-RA-PI/2.0-DATAN2(BY,BX)
          END IF
  150 CONTINUE
C
C Determine all (u,v) values.
C
      AMPMAX = 0.0
      RMAX = 0.0
      IP = 0
C
C     (start loop)
C
  300 READ(INDAT,END=400) IUT,(REC(J),J=1,NDWPB*NB)
      IF (IUT.LT.0) GOTO 400
      DO 320 IB=1,NB
C         -- skip deleted data
          IF (REC(NDWPB*(IB-1)+2).LE.0.0) GOTO 320 
          IP = IP+1
          IF (IP.GT.MAXP) CALL ERROR('Too many points.')
          IF (NVER.EQ.1) THEN
              TIHA = (IUT-IZ)*SIDR + BOFF(IB)
              U = BXY(IB)*DCOS(TIHA)
              V = BZ(IB)*COSDEC + BXY(IB)*SINDEC*DSIN(TIHA)
          ELSE IF (NVER.EQ.2) THEN
              U = REC(NDWPB*(IB-1)+5)*V2FAC
              V = REC(NDWPB*(IB-1)+6)*V2FAC
          END IF
          AMP(IP) = REC(NDWPB*(IB-1)+1)
          IF (ERRORS) THEN
              AMP1(IP) = AMP(IP) - REC(NDWPB*(IB-1)+2)
              AMP2(IP) = AMP(IP) + REC(NDWPB*(IB-1)+2)
              AMPMAX = AMAX1(AMP2(IP),AMPMAX)
          ELSE
              AMPMAX = AMAX1(AMP(IP),AMPMAX)
          END IF
          IF (ANGPLT) THEN
              R(IP) = ATAN2D(U,V)
              IF (R(IP).LT.0.0) R(IP) = R(IP) + 180.0
          ELSE
              R(IP) = SQRT(U*U+V*V)
              RMAX = AMAX1(R(IP),RMAX)
          END IF
          IF (PAPROJ) THEN
              ANGPRO(IP) = ATAN2D(U,V) - BAPA
              R(IP) =  ABS(R(IP)*COSD(ANGPRO(IP)))
          END IF
320   CONTINUE
      GOTO 300
C
C     (end loop)
C
  400 CONTINUE
C
C Plotting limits: autoscale if not specified.
C
      IF (XMIN.NE.XMAX) THEN
          RMIN = XMIN
          RMAX = XMAX
      ELSE
          RMIN = 0.0
          RMAX = RMAX*1.2
          IF (ANGPLT) RMAX = 180.0
      END IF
      IF (YMIN.NE.YMAX) THEN
          AMPMIN = YMIN
          AMPMAX = YMAX
      ELSE
          AMPMIN = 0.0
          AMPMAX = AMPMAX*1.2
      END IF
C
C Calculate gaussian model.
C
      IF (GPLOT) THEN
          DO 420 J=1,NPTS
              XUV(J) = RMIN + (J-1)*(RMAX-RMIN)/(NPTS-1)
              ARG1 = 0.009147*XUV(J)*FWHM1*FACTOR
              ARG2 = 0.009147*XUV(J)*FWHM2*FACTOR
              G(J) = FLUX1*EXP(-ARG1**2) + FLUX2*EXP(-ARG2**2)
  420     CONTINUE
          IF (FLUX2.EQ.0.0) THEN
              WRITE (GLABEL,920) FLUX1, FWHM1
          ELSE
              WRITE (GLABEL,921) FLUX1, FLUX2, FWHM1, FWHM2
          END IF
          WRITE (OUTC, '(1X,A)') GLABEL(1:LEN1(GLABEL))
      END IF
C
C Plot and label axes.
C
      CALL PGSLW(LINEWD)
      CALL PGSCF(NFONT)
      CALL PGSCI(5)
      CALL PGENV(RMIN,RMAX,AMPMIN,AMPMAX,0,0)
      CALL PGLABEL(XAXIS, YAXIS, ' ')
      CALL PGMTEXT('T',2.5,0.5,0.5,LEGEN1)
      CALL PGMTEXT('T',0.8,0.5,0.5,LEGEN2)
C
C Plot data points (green).
C
      CALL PGSCI(3)
      CALL PGSLW(NDOTSZ)
      IF (ERRORS) CALL PGERRY(IP,R,AMP1,AMP2,0.0)
      IF (MARK.GE.0 .OR. (.NOT.ERRORS)) CALL PGPOINT(IP,R,AMP,MARK)
C
C Plot gaussian (red).
C
      CALL PGSCI(2)
      CALL PGSLW(LINEWD)
      IF (GPLOT) CALL PGLINE(NPTS,XUV,G)
C
C Write file name on plot (hardcopy only).
C
      CALL PGQINF('HARDCOPY',PLTYPE,L)
      IF (PLTYPE.EQ.'YES') THEN
          CALL PGSLW(1)
          CALL PGSCI(1)
          CALL PGSCF(1)
          CALL PGSCH(0.6)
          CALL PGMTEXT('R',3.5,0.0,0.0,INDSN)
          CALL PGMTEXT('R',3.5,1.0,1.0,NOW)
          IF (GPLOT) CALL PGMTEXT('R',2.0,0.0,0.0,GLABEL)
          CALL PGSCH(1.0)
          CALL PGQINF('DEV/TYPE', PLDSN, L)
          WRITE (OUTC,'(2A)') ' Created plot file: ', PLDSN(1:L)
      END IF
C
C Close plot file and exit.
C
      CALL PGEND
C-----------------------------------------------------------------------
      END
