      PROGRAM UVGEN
C-----------------------------------------------------------------------
C UVGEN: to convert a MERGE-format file from "format 1" to "format 2",
C by computing the (u,v)-coordinates.
C
C T.J. Pearson, Fortran-77.
C
C Version 1.0:  1983 Jun 10.
C Version 1.1:  1983 Jul 19 - Fixed bug in comp. of U and V (RA->RAP).
C                             Changed output data records so U and V
C                             are at end of each baseline section.
C Version 1.2:  1983 Aug  3 - Add W to version 2 files.
C Version 1.3:  1987 Mar 16 - Force conversion of reference IUT and GST 
C                             in header to those at 0 hr UT on 
C                             reference day [DLM].
C Version 1.4:  1987 Apr 27 - add history record; 20 stations.
C Version 1.5:  1987 Dec  3 - use MKHIST routine.
C Version 1.6:  1988 Apr 13 - minor changes for Convex.
C Version 1.7:  1990 Nov 13 - remove unused variables.
C-----------------------------------------------------------------------
C     IMPLICIT NONE
      CHARACTER*(*) VERSN
      INTEGER INC,OUTC,INDAT,OUTDAT,MAXS,MAXB,MAXT,NPARS,MAXWPB,ICON
      DOUBLE PRECISION PI,TWOPI,SIDR,VELC
      PARAMETER (VERSN='1.7 - 1990 Nov 13')
      PARAMETER (INC=5,OUTC=6,INDAT=8,OUTDAT=9)
      PARAMETER (MAXS=20)
      PARAMETER (MAXB=(MAXS-1)*MAXS/2)
      PARAMETER (MAXT=MAXB*(MAXS-2)/3)
      PARAMETER (MAXWPB=7)
      PARAMETER (NPARS=20)
      PARAMETER (PI=3.141592653589793D0)
      PARAMETER (TWOPI=PI*2D0)
C     -- radians of ST per sec of UT
      PARAMETER (SIDR=(PI/2592000.0D0)*1.0027379093D0)
C     -- speed of light
      PARAMETER (VELC=2.997925D8)
C     -- 60ths of seconds per day
      PARAMETER (ICON=5184000)
C
      CHARACTER*8  STNAME(MAXS)
      CHARACTER*16 SOURCE
      INTEGER   IUT, IS, IB, IT, NS, NB, NT, I, J, IER, MODE
      INTEGER   IZ, IY, IS1, IS2, NHIST, JMAX, LENG
      INTEGER   NBI(MAXB), NB1(MAXB), NB2(MAXB)
      INTEGER   NTI(MAXT)
      INTEGER   REC1(20), IDUM(8)
      REAL      INREC(4*MAXB+2*MAXT)
      REAL      OUTREC(MAXWPB*MAXB+2*MAXT)
      REAL      U, V, W
      DOUBLE PRECISION STX(MAXS), STY(MAXS), STZ(MAXS)
      DOUBLE PRECISION RA, DEC, RAP, DECP, GASTM, GST, FREQ, FLUX,
     1          WAVE,BX,BY,BZ(MAXB),BOFF(MAXB),BXY(MAXB),TIHA,
     2          COSDEC,SINDEC, TINTG, AVG, BW
      DOUBLE PRECISION PARS(NPARS),VALS(NPARS),ENDMRK
      CHARACTER*80 INDSN, OUTDSN, HISTRY, ADDHIS
      INTEGER   VLBOPE
C
      DATA      PARS/ 'Input', 9*' ',
     1                'Output', 9*' ' /
      DATA      VALS/ '***UNSET', 9*' ',
     1                '***UNSET', 9*' ' /
      DATA      ENDMRK/ '/' /
C-----------------------------------------------------------------------
C
C Introduction.
C
      CALL PUTOUT(
     1      'UVGEN converts MERGE files from format 1 to format 2')
      CALL PUTOUT('(Version '//VERSN//')')
C
C Control parameters.
C
  100 CALL PUTOUT(' ')
      CALL PUTOUT('Parameters (INPUT,OUTPUT):')
      MODE = 0
      CALL KEYIN(PARS,VALS,NPARS,ENDMRK,MODE,INC,OUTC)
      WRITE (INDSN,'(10A8)') (VALS(I),I=1,10)
      WRITE (OUTDSN,'(10A8)') (VALS(I),I=11,20)
C
C Make the new history record.
C
      CALL MKHIST('UVGEN', VERSN, ADDHIS)
C
C Open input Merge file.
C
      IER = VLBOPE(INDAT, INDSN, 'MERGE', 'OLD', INDSN)
      IF (IER.NE.1) GOTO 100
C
C Open output Merge file.
C
      IER = VLBOPE(OUTDAT, OUTDSN, 'MERGE', 'NEW', OUTDSN)
      IF (IER.NE.1) THEN
          CLOSE (UNIT=INDAT)
          GOTO 100
      END IF
C
C Read and re-write header.
C
      READ (INDAT,END=82,ERR=82) (REC1(J),J=1,20)
      IF (REC1(1).NE.1) GOTO 82
      REC1(1) = 2
      NHIST = REC1(2)
      NS    = REC1(4)
      NB    = REC1(5)
      NT    = REC1(6)
      JMAX = MAXWPB*NB+2*NT
      LENG = JMAX+1
      REC1(7) = LENG
      IF (NS.LT.2) CALL ERROR('Not enough stations.')
      IF (NS.GT.MAXS) CALL ERROR('Too many stations.')
      REC1(2) = REC1(2)+1
      REC1(3) = REC1(3)+1
      WRITE (OUTDAT) (REC1(J),J=1,20)
C
      IF (NB.GT.0) THEN
          READ (INDAT,END=82,ERR=82) (NBI(IB),IB=1,NB)
          WRITE (OUTDAT) (NBI(IB),IB=1,NB)
          DO 210 IB=1,NB
              NB1(IB) = NBI(IB)/100
              NB2(IB) = NBI(IB) -  NB1(IB)*100
  210     CONTINUE
      END IF
C
      IF (NT.GT.0) THEN
          READ (INDAT,END=82,ERR=82) (NTI(IT),IT=1,NT)
          WRITE (OUTDAT) (NTI(IT),IT=1,NT)
      END IF
C
      DO 220 I=1,NHIST
          READ (INDAT,END=82,ERR=82) HISTRY
          WRITE (OUTDAT) HISTRY
  220 CONTINUE
      WRITE (OUTDAT) ADDHIS
C
      READ (INDAT,END=82,ERR=82) SOURCE,RA,DEC,RAP,DECP,FLUX
      WRITE (OUTDAT) SOURCE,RA,DEC,RAP,DECP,FLUX
C
      DO 230 IS=1,NS
          READ (INDAT,END=82,ERR=82) STNAME(IS),STX(IS),STY(IS),STZ(IS)
          WRITE(OUTDAT) STNAME(IS),STX(IS),STY(IS),STZ(IS),
     1            (0.D0,J=5,10),0
  230 CONTINUE
C
      READ (INDAT,END=82,ERR=82) IY, IUT, GST, FREQ, TINTG,
     1            AVG, BW, (IDUM(I),I=1,8)
C
C Convert reference time to 0 hr UT on reference day.
C
      IZ = (IUT/ICON) * ICON
      GASTM = GST - (IUT-IZ)*SIDR
      GASTM = DMOD(GASTM,TWOPI)
      IF (GASTM.LT.0.D0) GASTM = TWOPI + GASTM
      WRITE (OUTDAT) IY, IZ, GASTM, FREQ, TINTG,
     1            AVG, BW, (IDUM(I),I=1,8)
C
C Baseline parameters (BXY and BZ are components
C in wavelengths; BOFF is "hour angle").
C
      WAVE   = VELC/FREQ
      COSDEC = DCOS(DECP)
      SINDEC = DSIN(DECP)
      DO 240 IB=1,NB
          IS1 = NB1(IB)
          IS2 = NB2(IB)
          BX  = (STX(IS1)-STX(IS2))/WAVE
          BY  = (STY(IS1)-STY(IS2))/WAVE
          BZ(IB) = (STZ(IS1)-STZ(IS2))/WAVE
          BXY(IB) = DSQRT(BX*BX+BY*BY)
          IF (BXY(IB).EQ.0D0) THEN
            CALL PUTOUT('Station coordinates missing.')
            GOTO 82
          END IF
          BOFF(IB) = GASTM-RAP-PI/2.0-DATAN2(BY,BX)
  240 CONTINUE
C
C Read and re-write data records.
C
  300 READ(INDAT,END=400) IUT,(INREC(J),J=1,4*NB+2*NT)
      IF (IUT.LT.0) GOTO 400
      DO 310 IB=1,NB
          TIHA = (IUT-IZ)*SIDR + BOFF(IB)
          U = BXY(IB)*DCOS(TIHA)
          V = BZ(IB)*COSDEC + BXY(IB)*SINDEC*DSIN(TIHA)
          W = BZ(IB)*SINDEC - BXY(IB)*COSDEC*DSIN(TIHA)
          OUTREC(MAXWPB*IB-6) = INREC(4*IB-3)
          OUTREC(MAXWPB*IB-5) = INREC(4*IB-2)
          OUTREC(MAXWPB*IB-4) = INREC(4*IB-1)
          OUTREC(MAXWPB*IB-3) = INREC(4*IB)
          OUTREC(MAXWPB*IB-2) = U
          OUTREC(MAXWPB*IB-1) = V
          OUTREC(MAXWPB*IB)   = W
  310 CONTINUE
      DO 320 IT=1,NT
          OUTREC(MAXWPB*NB + 2*IT - 1) = INREC(4*NB + 2*IT - 1)
          OUTREC(MAXWPB*NB + 2*IT)     = INREC(4*NB + 2*IT)
  320 CONTINUE
      WRITE (OUTDAT) IUT,(OUTREC(J),J=1,MAXWPB*NB+2*NT)
      GOTO 300
C
C Error exit.
C
   82 CALL PUTOUT('Something wrong with input MERGE file.')
      CLOSE (UNIT=OUTDAT, STATUS='DELETE')
      CLOSE (UNIT=INDAT)
      GOTO 100
C
C Write final record and quit.
C
  400 WRITE (OUTDAT) -1,(0.0,J=1,MAXWPB*NB+2*NT)
      CLOSE (UNIT=OUTDAT)
C-----------------------------------------------------------------------
      END
