      PROGRAM UVPLOT
C-----------------------------------------------------------------------
C UVPLOT: to plot and list u,v coverage of a standard MERGE-format
C visibility file.
C
C T.J. Pearson, VAX-11 Fortran.
C
C Version 1.0:  1979 Sep 22.
C Version 1.1:  1980 Mar 13.
C Version 1.3:  1980 Jun 30 - Extended to 15 stations.
C Version 1.4:  1980 Oct 17 - Parameters UMAX, MARK added.
C Version 2.0:  1981 May 21 - Adopt Fortran 77 standards; new plot
C      format (uses PGPLOT package); new parameter NOINVERT.
C Version 3.0:  1981 Jun 12 - Add model amplitude contours.
C Version 3.1:  1982 Jun  8 - New graphics devices.
C Version 3.2:  1982 Oct  7 - LINEWIDTH parameter.
C Version 3.3:  1983 Apr 15 - Split plot header into two lines.
C Version 3.4:  1983 Jun  1 - add FONT and DOTSIZE parameters.
C Version 4.0:  1983 Jul  5 - Extended to include Merge format version
C                             2 (D. L. Meier).
C Version 4.1:  1983 Aug  3 - Include W in version 2 files (D.L. Meier).
C Version 4.2:  1983 Aug 19 - Add per-baseline count to output (TJP).
C Version 4.3:  1983 Sep 20 - Increase MAXS to 20; increase MAXP to 
C                               50000.
C Version 4.4:  1984 Mar 29 - Make missing coordinates non-fatal (TJP).
C Version 4.5:  1984 Oct  1 - Issue an explicit message if there are no
C                             data in the dataset, instead of crashing
C                             (TJP).
C Version 4.6:  1984 Nov 28 - Add COLORS parameter (TJP).
C Version 4.7:  1986 Jan  7 - Update to reflect latest PGPLOT (TJP).
C Version 4.8:  1986 Feb 17 - Show file modification date on plot (TJP).
C Version 5.0:  1986 Nov 14 - Use parameters for file names (TJP).
C Version 5.1:  1987 Jan 14 - Fix bug in LIST option (TJP).
C Version 5.2:  1987 Aug 29 - Prompt before drawing plot if plotting
C                             on user's terminal (TJP).
C Version 5.3:  1988 Apr  2 - Minor changes for Convex (TJP).
C Version 5.4:  1988 Jun 24 - Add NOTEXT parameter (Tasso Tzioumis) and
C                             use VLBOPE routine (TJP).
C Version 5.5:  1988 Jul 19 - Choose frequency unit intelligently; use
C                             GETCOM routine (TJP).
C Version 6.0:  1989 Apr 12 - Remove LISTFILE option; add LABEL option
C                             (TJP).
C Version 6.1:  1989 Dec 20 - Correct dimension error (TJP).
C Version 6.2:  1991 Mar 21 - Increase to 40 stations (TJP).
C-----------------------------------------------------------------------
      CHARACTER*(*) PROGNM, VERSN
      INTEGER INC,OUTC,INDAT,INMOD,PL,MAXS,MAXB,MAXT,MAXP,NPARS
      INTEGER MAXWPB, NDIM
      DOUBLE PRECISION PI,SIDR,VELC
      PARAMETER (PROGNM='UVPLOT')
      PARAMETER (VERSN='6.2 - 1991 Mar 21')
      PARAMETER (INC=5,OUTC=6,INDAT=8,INMOD=9,PL=7)
C   Maximum number of stations/baselines/triangles
      PARAMETER (MAXS=40)
      PARAMETER (MAXB=(MAXS-1)*MAXS/2)
      PARAMETER (MAXT=MAXB*(MAXS-2)/3)
C   Maximum words per baseline
      PARAMETER (MAXWPB=7)
C   Maximum number of u,v-points
      PARAMETER (MAXP=50000)
C   Input parameters
      PARAMETER (NPARS=73)
C   Size of amplitude array
      PARAMETER (NDIM=50)
      PARAMETER (PI=3.141592653589793D0)
C   Radians of ST per sec of UT
      PARAMETER (SIDR=(PI/2592000.0D0)*1.0027379093D0)
C   Speed of light
      PARAMETER (VELC=2.997925D8)
C
      CHARACTER*8  STNAME(MAXS)
      CHARACTER*16 SOURCE
      REAL      REC(MAXWPB*MAXB),U(MAXP),V(MAXP)
      REAL      AMPL(NDIM+1,2*NDIM+1)
      REAL      CONTS(20), TR(6)
      DOUBLE PRECISION STX,STY,STZ,RA,DEC,GASTM,FREQ,FLUX,
     1          WAVE,BX,BY,BZ(MAXB),BOFF(MAXB),BXY(MAXB),TIHA,
     2          COSDEC,SINDEC, PARS(NPARS),VALS(NPARS),ENDMRK,
     3          FACTOR
      INTEGER   BCOUNT(MAXB), COLORS(5), IER
      INTEGER   PGBEGIN, VLBOPE, GETCOM
      LOGICAL   INVERT, MODEL, COUNT, FILEID, DOLAB
      CHARACTER*3  FUNIT
      CHARACTER*4  STRING
      CHARACTER*255 INDSN, PLTDSN, MODDSN
      CHARACTER*80 LEGEN1
      CHARACTER*(9*MAXS) LEGEN2
      CHARACTER*20 NOW
      CHARACTER*17 BNAME(MAXB), REVTIM
      CHARACTER*2  BLAB(MAXB)
      CHARACTER*2  STNABR
      REAL       ULABP(MAXB), VLABP(MAXB)
      LOGICAL    LABEL(MAXB)
      COMMON    /PARMS/ STNAME,STX(MAXS),STY(MAXS),STZ(MAXS),
     1                  SOURCE,RA,DEC,GASTM,FREQ,FLUX
      COMMON    /ORDER/ NS,NB,NT, NB1(MAXB),NB2(MAXB), NC1(MAXT),
     1                  NC2(MAXT),NC3(MAXT), NVER, NDWPB
C-----------------------------------------------------------------------
      DATA      PARS/ 'Input', 9*' ', 
     1                'Plotfile', 9*' ',
     2                10*' ',
     3                'Umax', 'Vmax', 'MArk', 'NOInvert',
     4                'Model',7*' ', 20*'CONtours', 'LINewidt',
     5                'Font', 'Dotsize', 'COUnt', 5*'COLors',
     6                'NOText', 'Label' /
      DATA      VALS/ '***UNSET', 9*' ',
     1                '?', 9*' ',
     2                10*' ',
     3                0D0,    0D0,    -1D0,      -1D0,
     4                8*' ', 20*-1D0 , 1D0 , 1D0, 0D0 , -1D0,
     5                5D0, 5D0, 3D0, 2D0, 4D0, -1D0, -1D0 /
      DATA      ENDMRK/ '/' /
C-----------------------------------------------------------------------
  600 FORMAT(' Umax = ',F10.2,'       Vmax = ',F10.2,
     1       '  million wavelengths'/
     2       ' Number of (u,v)-points = ',I6/
     3       ' Mean epoch = ',F7.2)
  601 FORMAT(' Number of (u,v)-points in plot window = ',I6)
  605 FORMAT(1X,A,I6,' points')
  611 FORMAT(' ++WARNING++ Station coordinates for ',A,' are missing')
  612 FORMAT(' ++WARNING++ Zero-length baseline: ',A)
  680 FORMAT('(',A3,I5,', ',F7.3, 1X, A,')')
C-----------------------------------------------------------------------
C
C Introduction.
C
      CALL DATTIM(NOW)
      CALL PUTOUT(PROGNM//' plots u-v coordinates from MERGE files')
      CALL PUTOUT('(Version '//VERSN//')')
C
C Control parameters.
C
      MODE = 0
   10 IF (MODE.NE.0) CALL ERROR('Bad input parameters in '//PROGNM)
      CALL PUTOUT(' ')
      CALL PUTOUT(
     1  'Parameters (INPUT, PLOTFILE; type SHOW for complete list):')
      MODE = 0
      CALL KEYIN(PARS,VALS,NPARS,ENDMRK,MODE,INC,OUTC)
      WRITE (INDSN, '(10A8)') (VALS(J),J= 1,10)
      WRITE (PLTDSN,'(10A8)') (VALS(J),J=11,20)
C    Umaxp, Vmaxp are in millions of wavelengths 
      UMAXP = VALS(31)*1D-6
      VMAXP = VALS(32)*1D-6
      MARK = VALS(33)
      INVERT = VALS(34).EQ.-1D0
      WRITE (MODDSN,'(8A8)') (VALS(J),J=35,42)
      NCONTS = 0
      DO J=1,20
          CONTS(J) = VALS(42+J)
          IF (CONTS(J).LT.0.0) GOTO 115
          NCONTS =NCONTS+1
      END DO
  115 MODEL = MODDSN.NE.' '.AND.NCONTS.GT.0
      LINEWD = VALS(63)
      NFONT  = VALS(64)
      NDOTSZ = VALS(65)
      IF (NDOTSZ.LT.1) NDOTSZ = LINEWD
      COUNT = VALS(66).NE.-1D0
      DO J=1,5
          COLORS(J) = VALS(66+J)
      END DO
C     -- NOTEXT parameter suppresses file identification
      FILEID = VALS(72).EQ.-1D0
C     -- LABEL parameter requests baseline labels
      DOLAB  = VALS(73).NE.-1D0
C
C Open input Merge file.
C
      IER = VLBOPE(INDAT, INDSN, 'MERGE', 'OLD', INDSN)
      IF (IER.NE.1) GOTO 10
C
C Open graphics device.
C
      IER = PGBEGIN(PL,PLTDSN,1,1)
      IF (IER.NE.1) THEN
          CALL PUTOUT('++WARNING++ Cannot open PLOTFILE')
          GOTO 10
      END IF
C
C Read model data.
C
      IF (MODEL) THEN
          IER = VLBOPE(INMOD, MODDSN, 'MODEL', 'OLD', MODDSN)
          IF (IER.NE.1) GOTO 10
          CALL GETMOD(INMOD)
          CLOSE (UNIT=INMOD)
      END IF
C
C Read header and assemble legends for plot.
C
      NREC = 0
      CALL HEADX40(.FALSE.,INDAT,IZ,ID,IY,DUM1,0,0,DUM2,DUM3,IF,0)
      IF (NS.LT.2) CALL ERROR('Not enough stations.')
      IF (NS.GT.MAXS) CALL ERROR('Too many stations.')
      IF (NVER.NE.1 .AND. NVER.NE.2) 
     1                CALL ERROR('Illegal format version.')
      IMON = 1
      CALL TDATE(IY,IMON,ID,JD,MONTH,DAY)
      LEGEN1 = SOURCE
      I = LEN1(LEGEN1)+3
      CALL PRFREQ(FREQ, FACTOR, FUNIT)
      WRITE (LEGEN1(I:),680) MONTH, IY, FREQ/FACTOR, FUNIT
      LEGEN2 = STNAME(1)
      DO IS=2,NS
          I = LEN1(LEGEN2)+2
          LEGEN2(I:) = STNAME(IS)
      END DO
      CALL PUTOUT(' ')
      CALL PUTOUT(LEGEN1(1:LEN1(LEGEN1)))
      CALL PUTOUT(LEGEN2(1:LEN1(LEGEN2)))
C
C Baseline parameters.
C
C   WAVE is 1 million wavelengths.
      WAVE=VELC/FREQ*1D6
      COSDEC=DCOS(DEC)
      SINDEC=DSIN(DEC)
      DO IS=1,NS
          IF (STX(IS).EQ.0D0 .AND. STY(IS).EQ.0D0) 
     1            WRITE (OUTC,611) STNAME(IS)(1:LEN1(STNAME(IS)))
      END DO
      DO IB=1,NB
          IS1 = NB1(IB)
          IS2 = NB2(IB)
          L1 = LEN1(STNAME(IS1))
          L2 = LEN1(STNAME(IS2))
          BNAME(IB) = STNAME(IS1)(1:L1)//'-'//STNAME(IS2)(1:L2)
          BLAB(IB)(1:1) = STNABR(STNAME(IS1))
	  BLAB(IB)(2:2) = STNABR(STNAME(IS2))
          LABEL(IB) = .FALSE.
          BX = (STX(IS1)-STX(IS2))/WAVE
          BY = (STY(IS1)-STY(IS2))/WAVE
          BZ(IB) = (STZ(IS1)-STZ(IS2))/WAVE
          BXY(IB) = DSQRT(BX*BX+BY*BY)
          IF (BXY(IB).EQ.0D0) THEN
              WRITE (OUTC,612) BNAME(IB)
              BOFF(IB) = 0.0
          ELSE
              BOFF(IB) = GASTM-RA-PI/2.0-DATAN2(BY,BX)
          END IF
      END DO
C
C Determine all (u,v) values.
C
      UMAX = 0.0
      VMAX = 0.0
      EPOCH = 0.0
      IP = 0
      IPFULL = 0
      DO IB=1,NB
          BCOUNT(IB) = 0
      END DO
C
C (start loop)
C
  300 READ(INDAT,END=400) IUT,(REC(J),J=1,NDWPB*NB)
      IF (IUT.LT.0) GOTO 400
      DO 320 IB=1,NB
C   Skip deleted data
      IF (REC(NDWPB*(IB-1)+2).LE.0.0) GOTO 320
          IF (NVER.EQ.1) THEN
            TIHA = (IUT-IZ)*SIDR + BOFF(IB)
            UP = BXY(IB)*DCOS(TIHA)
            VP = BZ(IB)*COSDEC + BXY(IB)*SINDEC*DSIN(TIHA)
          ELSE IF (NVER.EQ.2) THEN
C           Convert to millions of wavelengths.
            UP = REC(NDWPB*(IB-1)+5)/1E6
            VP = REC(NDWPB*(IB-1)+6)/1E6
          END IF
          IPFULL = IPFULL+1
          EPOCH = EPOCH + FLOAT(IUT)/(365.25*24*3600*60)
          UMAX = AMAX1(ABS(UP),UMAX)
          VMAX = AMAX1(ABS(VP),VMAX)
          IF (UMAXP.GT.0.0 .AND. ABS(UP).GT.UMAXP) GOTO 320
          IF (VMAXP.GT.0.0 .AND. ABS(VP).GT.VMAXP) GOTO 320
          IP = IP+1
          IF (IP.GT.MAXP) CALL ERROR('Too many points.')
          BCOUNT(IB) = BCOUNT(IB)+1
          U(IP) = UP
          V(IP) = VP
          IF (.NOT.LABEL(IB)) THEN
              LABEL(IB) = .TRUE.
              ULABP(IB) = U(IP)
              VLABP(IB) = V(IP)
          END IF
  320 CONTINUE
      NREC = NREC+1
      GOTO 300
C
C (end loop)
C
  400 IF (IP.LE.0) CALL ERROR('No data in window')
      EPOCH = EPOCH/FLOAT(IP) + FLOAT(IY)
      WRITE (OUTC,600) UMAX,VMAX,IPFULL,EPOCH
      IF (IP.NE.IPFULL) WRITE (OUTC,601) IP
      IF (COUNT) WRITE(OUTC,605) (BNAME(IB),BCOUNT(IB),IB=1,NB)
      UMAX = UMAX*1.2
      VMAX = VMAX*1.2
      IF (UMAXP.GT.0.0) UMAX = UMAXP
      IF (VMAXP.GT.0.0) VMAX = VMAXP
      UINT = UMAX/FLOAT(NDIM)
      VINT = VMAX/FLOAT(NDIM)
C
C Compute model amplitudes.
C
      IF (MODEL) THEN
          AMMAX = 0.0
          AMMIN = 1E38
          DO I=1,NDIM+1
            DO J=1,2*NDIM+1
                CALL UVMOD(1E6*UINT*(I-1),1E6*VINT*(J-1-NDIM),
     1                  AMPL(I,J),PH)
                AMMAX = AMAX1(AMPL(I,J),AMMAX)
                AMMIN = AMIN1(AMPL(I,J),AMMIN)
            END DO
          END DO
      END IF
C
C Plotting Section.
C
      CALL PGQINF('DEV/TYPE', PLTDSN, L)
      CALL PUTOUT('Plot device/file: '//PLTDSN(1:L))
      CALL PGQINF('TERMINAL', STRING, L)
      IF (STRING.EQ.'YES') IER =
     1    GETCOM(STRING, 'Type <CR> to see plot: ', L)
      CALL PGSLW(LINEWD)
      CALL PGSCF(NFONT)
      CALL PGSCI(COLORS(1))
      CALL PGENV(UMAX,-UMAX,-VMAX,VMAX,1,1)
      CALL PGSCI(COLORS(2))
      CALL PGLABEL('u  (10\u6\d \gl)','v  (10\u6\d \gl)',' ')
      CALL PGMTEXT('T',2.5,0.5,0.5,LEGEN1)
      CALL PGMTEXT('T',0.8,0.5,0.5,LEGEN2)
      CALL PGSCI(COLORS(3))
      CALL PGSLW(NDOTSZ)
      CALL PGPOINT(IP,U,V,MARK)
      IF (INVERT) THEN
          DO I=1,IP
              U(I) = -U(I)
              V(I) = -V(I)
          END DO
          CALL PGSCI(COLORS(4))
          CALL PGPOINT(IP,U,V,MARK)
      END IF
C
C Contours of model.
C
      CALL PGSLW(LINEWD)
      IF (MODEL) THEN
          TR(1) = -UINT
          TR(2) = UINT
          TR(3) = 0.0
          TR(4) = (-1.0-NDIM)*VINT
          TR(5) = 0.0
          TR(6) = VINT
          CALL PGSCI(COLORS(5))
          CALL PGCONT(AMPL,NDIM+1,2*NDIM+1,1,NDIM+1,1,
     1                  2*NDIM+1,CONTS,NCONTS,TR)
          TR(1) = -TR(1)
          TR(2) = -TR(2)
          TR(4) = -TR(4)
          TR(6) = -TR(6)
          CALL PGCONT(AMPL,NDIM+1,2*NDIM+1,1,NDIM+1,1,
     1                  2*NDIM+1,CONTS,NCONTS,TR)
      END IF
C
C Label baselines.
C
      CALL PGSLW(1)
      CALL PGSCI(COLORS(2))
      CALL PGSCF(1)
      CALL PGSCH(0.5)
      IF (DOLAB) THEN
          DO IB=1,NB
              IF (LABEL(IB))
     1            CALL PGTEXT(ULABP(IB), VLABP(IB), BLAB(IB))
          END DO
      END IF
C
C Annotation.
C
      CALL PGQINF('HARDCOPY', STRING, L)
      IF (STRING.EQ.'YES' .AND. FILEID) THEN
          CALL PGSCH(0.6)
          CALL PGMTEXT('R',3.5,0.0,0.0,INDSN)
          CALL FRDT(INDSN, REVTIM)
          CALL PGMTEXT('R',3.5,1.0,1.0,REVTIM)
      END IF
C
C Close plot file.
C
      CALL PGEND
C
      END

      SUBROUTINE UVMOD(U,V,AMPL,PHASE)
C-----------------------------------------------------------------------
C Calculate model amplitude (AMPL) and phase (PHASE)
C at point (U,V)
C-----------------------------------------------------------------------
      PARAMETER (MAXMOD = 2001)
      REAL    FLUX(MAXMOD),XC(MAXMOD),YC(MAXMOD),AA(MAXMOD),
     1          R(MAXMOD),PHI(MAXMOD)
      COMMON /MODEL/ NCOMP,FLUX,XC,YC,AA,R,PHI
C
      VISI = 0.0
      VISR = 0.0
      DO 10 J=1,NCOMP
          COMPHA = 6.283185*(U*XC(J)+V*YC(J))
          IF (AA(J).NE.0.0) THEN
              SPHI = SIN(PHI(J))
              CPHI = COS(PHI(J))
              ARG = (-3.559707*AA(J)**2*((V*CPHI+U*SPHI)**2 +
     1              (R(J)*(U*CPHI-V*SPHI))**2 ))
              COMVIS = EXP(ARG)*FLUX(J)
          ELSE
              COMVIS = FLUX(J)
          END IF
          VISI = VISI+COMVIS*SIN(COMPHA)
          VISR = VISR+COMVIS*COS(COMPHA)
   10 CONTINUE
C
      AMPL = SQRT(VISI*VISI + VISR*VISR)
      IF (AMPL.EQ.0.0) THEN
          PHASE = 0.0
      ELSE
          PHASE = ATAN2(VISI,VISR)
      END IF
C
      RETURN
      END

      SUBROUTINE GETMOD(INMOD)
      INTEGER INMOD
C-----------------------------------------------------------------------
C Read the model parameters from unit INMOD
C-----------------------------------------------------------------------
      REAL PI,RPDEG,RPMAS
      PARAMETER (PI=3.14159265359)
      PARAMETER (RPDEG=PI/180.0)
      PARAMETER (RPMAS=RPDEG/3600000.0)
      INTEGER MAXMOD
      PARAMETER (MAXMOD = 2001)
C-----------------------------------------------------------------------
      INTEGER I, NCOMP, NCS, TYPE
      REAL    FLXSUM, THETA, RAD
      REAL    FLUX(MAXMOD), XC(MAXMOD), YC(MAXMOD), AA(MAXMOD)
      REAL    R(MAXMOD), PHI(MAXMOD)
      COMMON  /MODEL/ NCOMP, FLUX, XC, YC, AA, R, PHI
C-----------------------------------------------------------------------
 1000 FORMAT(' ++WARNING++ Model truncated after ',I4,' components')
 1010 FORMAT(' ++WARNING++ Invalid component type: Gaussian used')
C-----------------------------------------------------------------------
      NCS = 0
      FLXSUM = 0.0
      DO 10 I=1,MAXMOD
          CALL GTMODL(INMOD,TYPE,FLUX(I),RAD,THETA,AA(I),R(I),
     1                PHI(I),*20,0)
          IF (FLUX(I).EQ.0.0) GOTO 20
          FLXSUM = FLXSUM+FLUX(I)
          THETA = THETA*RPDEG
          XC(I) = RPMAS*RAD*SIN(THETA)
          YC(I) = RPMAS*RAD*COS(THETA)
          AA(I) = AA(I)*RPMAS
          PHI(I) = PHI(I)*RPDEG
          IF (TYPE.NE.0 .AND. TYPE.NE.1) WRITE (6,1010)
   10 CONTINUE
      WRITE (6,1000) MAXMOD
      I = MAXMOD+1
   20 NCOMP = I-1
      IF (FLXSUM.EQ.0.0) CALL ERROR('Model has zero total flux')
      RETURN
      END
