      PROGRAM VLBEXP
C-----------------------------------------------------------------------
C VLBEXP: export data from a standard MERGE-format visibility file in
C ASCII text.
C
C T.J. Pearson, Fortran-77.
C
C Version 1.0:  1990 Feb 6.
C-----------------------------------------------------------------------
      CHARACTER*(*) PROGNM, VERSN
      INTEGER INC,OUTC,INDAT,OUTDAT,MAXS,MAXB,MAXT,NPARS
      INTEGER MAXWPB
      DOUBLE PRECISION PI,SIDR,VELC
      PARAMETER (PROGNM='VLBEXP')
      PARAMETER (VERSN='1.0 - 1990 Feb 6')
      PARAMETER (INC=5,OUTC=6,INDAT=8,OUTDAT=9)
C   Maximum number of stations/baselines/triangles
      PARAMETER (MAXS=20)
      PARAMETER (MAXB=(MAXS-1)*MAXS/2)
      PARAMETER (MAXT=MAXB*(MAXS-2)/3)
C   Maximum words per baseline
      PARAMETER (MAXWPB=7)
C   Input parameters
      PARAMETER (NPARS=20)
      PARAMETER (PI=3.141592653589793D0)
C   Radians of ST per sec of UT
      PARAMETER (SIDR=(PI/2592000.0D0)*1.0027379093D0)
C   Speed of light
      PARAMETER (VELC=2.997925D8)
C
      CHARACTER*8  STNAME(MAXS)
      CHARACTER*16 SOURCE
      REAL      REC(MAXWPB*MAXB)
      DOUBLE PRECISION STX(MAXS),STY(MAXS),STZ(MAXS),
     1          RA,DEC,GASTM,FREQ,FLUX,
     1          WAVE,BX,BY,BZ(MAXB),BOFF(MAXB),BXY(MAXB),TIHA,
     2          COSDEC,SINDEC, PARS(NPARS),VALS(NPARS),ENDMRK,
     3          FACTOR, DA, MJDATE
      INTEGER   IER, VLBOPE
      INTEGER   NS,NB,NT, NB1(MAXB),NB2(MAXB), NC1(MAXT)
      INTEGER   NC2(MAXT),NC3(MAXT), NVER, NDWPB
      CHARACTER*3  FUNIT
      CHARACTER*255 INDSN, OUTDSN
      CHARACTER*80 LEGEN1
      CHARACTER*(9*MAXS) LEGEN2
      CHARACTER*17 BNAME(MAXB)
      COMMON    /PARMS/ STNAME,STX,STY,STZ,SOURCE,RA,DEC,GASTM,FREQ,FLUX
      COMMON    /ORDER/ NS,NB,NT, NB1,NB2, NC1,NC2,NC3, NVER, NDWPB
C-----------------------------------------------------------------------
      DATA      PARS/ 'Input', 9*' ',   'Output', 9*' '/
      DATA      VALS/ '***UNSET', 9*' ','***UNSET', 9*' '/
      DATA      ENDMRK/ '/' /
C-----------------------------------------------------------------------
  600 FORMAT(' Output file: ',A/
     2       ' Number of (u,v)-points = ',I6)
  611 FORMAT(' ++WARNING++ Station coordinates for ',A,' are missing')
  612 FORMAT(' ++WARNING++ Zero-length baseline: ',A)
  680 FORMAT('(',A3,I5,', ',F7.3, 1X, A,')')
  700 FORMAT(F15.8,',',I3,2(',',F10.4),2(',',F9.2),3(',',F9.3))
C-----------------------------------------------------------------------
C
C Introduction.
C
      CALL PUTOUT(PROGNM//' exports data from MERGE files as text')
      CALL PUTOUT('(Version '//VERSN//')')
C
C Control parameters.
C
      MODE = 0
   10 IF (MODE.NE.0) CALL ERROR('Bad input parameters in '//PROGNM)
      CALL PUTOUT(' ')
      CALL PUTOUT(
     1  'Parameters (INPUT, OUTPUT):')
      MODE = 0
      CALL KEYIN(PARS,VALS,NPARS,ENDMRK,MODE,INC,OUTC)
      WRITE (INDSN, '(10A8)') (VALS(J),J= 1,10)
      WRITE (OUTDSN,'(10A8)') (VALS(J),J=11,20)
C
C Open input Merge file.
C
      IER = VLBOPE(INDAT, INDSN, 'MERGE', 'OLD', INDSN)
      IF (IER.NE.1) GOTO 10
C
C Open output text file.
C
      IER = VLBOPE(OUTDAT, OUTDSN, 'TEXT', 'NEW', OUTDSN)
      IF (IER.NE.1) THEN
          CLOSE (UNIT=INDAT)
          GOTO 10
      END IF
C
C Read header.
C
      CALL HEADX20(.FALSE.,INDAT,IZ,ID,IY,DUM1,0,0,DUM2,DUM3,IF,0)
      IF (NS.LT.2) CALL ERROR('Not enough stations.')
      IF (NS.GT.MAXS) CALL ERROR('Too many stations.')
      IF (NVER.NE.1 .AND. NVER.NE.2) 
     1                CALL ERROR('Illegal format version.')
      IMON = 1
      CALL TDATE(IY,IMON,ID,JD,MONTH,DAY)
      LEGEN1 = SOURCE
      I = LEN1(LEGEN1)+3
      CALL PRFREQ(FREQ, FACTOR, FUNIT)
      WRITE (LEGEN1(I:),680) MONTH, IY, FREQ/FACTOR, FUNIT
      LEGEN2 = STNAME(1)
      DO 100 IS=2,NS
          I = LEN1(LEGEN2)+2
          LEGEN2(I:) = STNAME(IS)
  100 CONTINUE
      CALL PUTOUT(' ')
      CALL PUTOUT(LEGEN1(1:LEN1(LEGEN1)))
      CALL PUTOUT(LEGEN2(1:LEN1(LEGEN2)))
C
C Baseline parameters.
C
C   WAVE is 1 million wavelengths.
      WAVE=VELC/FREQ*1D6
      COSDEC=DCOS(DEC)
      SINDEC=DSIN(DEC)
      DO 110 IS=1,NS
          IF (STX(IS).EQ.0D0 .AND. STY(IS).EQ.0D0) 
     1            WRITE (OUTC,611) STNAME(IS)(1:LEN1(STNAME(IS)))
  110 CONTINUE
      DO 120 IB=1,NB
          IS1 = NB1(IB)
          IS2 = NB2(IB)
          L1 = LEN1(STNAME(IS1))
          L2 = LEN1(STNAME(IS2))
          BNAME(IB) = STNAME(IS1)(1:L1)//'-'//STNAME(IS2)(1:L2)
          BX = (STX(IS1)-STX(IS2))/WAVE
          BY = (STY(IS1)-STY(IS2))/WAVE
          BZ(IB) = (STZ(IS1)-STZ(IS2))/WAVE
          BXY(IB) = DSQRT(BX*BX+BY*BY)
          IF (BXY(IB).EQ.0D0) THEN
              WRITE (OUTC,612) BNAME(IB)
              BOFF(IB) = 0.0
          ELSE
              BOFF(IB) = GASTM-RA-PI/2.0-DATAN2(BY,BX)
          END IF
  120 CONTINUE
C
C Modified Julian Date at beginning of year.
C
      CALL SLA_CLDJ(IY, 1, 1, MJDATE, IER)
C
C Read through dataset.
C
      IP = 0
  300 READ (INDAT,END=400) IUT,(REC(J),J=1,NDWPB*NB)
      IF (IUT.LT.0) GOTO 400
C     -- days since beginning of year
      DA = DBLE(IUT)/(60D0*86400D0)
      DO 320 IB=1,NB
          AMP    = REC(NDWPB*(IB-1)+1)
          AMPERR = REC(NDWPB*(IB-1)+2)
          PHS    = REC(NDWPB*(IB-1)+3)
          PHSERR = REC(NDWPB*(IB-1)+4)
C         -- Skip deleted data
          IF (AMPERR.GT.0.0 .AND. PHSERR.GT.0.0) THEN
              IF (NVER.EQ.1) THEN
                  TIHA = (IUT-IZ)*SIDR + BOFF(IB)
                  UP = BXY(IB)*DCOS(TIHA)
                  VP = BZ(IB)*COSDEC + BXY(IB)*SINDEC*DSIN(TIHA)
                  WP = BZ(IB)*SINDEC - BXY(IB)*COSDEC*DSIN(TIHA)
              ELSE IF (NVER.EQ.2) THEN
C                 -- Convert to millions of wavelengths.
                  UP = REC(NDWPB*(IB-1)+5)/1E6
                  VP = REC(NDWPB*(IB-1)+6)/1E6
                  WP = REC(NDWPB*(IB-1)+7)/1E6
              END IF
              IP = IP+1
              WRITE (OUTDAT,700) MJDATE+DA, IB, AMP, AMPERR, PHS,
     1                           PHSERR, UP,VP,WP 
          END IF
  320 CONTINUE
      GOTO 300
C
C (end loop)
C
  400 WRITE (OUTC,600) OUTDSN(1:LEN1(OUTDSN)),IP
      CLOSE (UNIT=INDAT)
      CLOSE (UNIT=OUTDAT)
C
      END
