      PROGRAM VPLOT
*-----------------------------------------------------------------------
* VPLOT: program for plotting VLBI visibility data.
*
* Author:            T.J. Pearson (TJP)
* Language:          Fortran-77
*
* Modifications:
* Version 1.0:  1990 Jun  7 - new program (TJP).
* Version 1.1:  1990 Aug 27 - add NOERR parameter, increase MAXP (TJP).
* Version 1.2:  1990 Sep  6 - add PHASE plot (TJP).
* Version 1.3:  1990 Sep 25 - add GRID, GST, ALL options (TJP).
* Version 1.4:  1990 Oct 14 - add second model and MODF command (TJP).
* Version 1.5:  1990 Nov 12 - remove unused variables (TJP).
* Version 1.6:  1991 Mar 22 - allow up to 40 stations (TJP).
* Version 1.7:  1991 Apr 18 - PROJ and CLO plots (TJP).
* Version 1.8:  1991 Apr 20 - HIGHLIGHT command (TJP).
* Version 1.9:  1991 Jun  6 - add TI plot, increase array sizes (TJP).
* Version 1.10: 1991 Oct 23 - add AMPLOT (TJP).
* Version 1.11: 1991 Oct 31 - new UVPLOT options, annotation (TJP).
* Version 1.12: 1991 Nov 18 - HIGHLIGHT works for one station or two.
* Version 1.13: 1992 Jan  7 - make BL and CL work with UT.
* Version 1.14: 1992 Jan 18 - STACK, PLOTFILE commands.
* Version 1.15: 1992 Feb 10 - bug fix.
* Version 1.16: 1992 Jul  6 - prompt for new input if file not found.
* Version 1.17: 1992 Aug  6 - fix bug in CL plots; more annotation; add
*                             STACK x PHASE and VISPLOT commands.
* Version 1.18: 1992 Oct 26 - VISPLOT CLOS command; REFANT command.
* Version 1.19: 1992 Nov 11 - correct some minor errors.
* Version 1.20: 1993 Sep 22 - add ANNOT parameter.
* Version 1.21: 1994 Mar 24 - correct calculation of agreement factors.
* Version 1.22: 1994 Jul  6 - improved algorithm for paplot.
*-----------------------------------------------------------------------
      CHARACTER*(*) VERSN
      PARAMETER (VERSN='1.22 - 1994 Jul 6')
      INTEGER NARRAY
      PARAMETER (NARRAY=12)
      INTEGER NPARS, NCOMMS
      PARAMETER (NPARS=65)
      PARAMETER (NCOMMS=22)
      INTEGER UMOCK, UMRG, UMOD
      PARAMETER (UMOCK=1, UMRG=2, UMOD=3)
      INTEGER MAXB
      PARAMETER (MAXB=780)
*
      CHARACTER*128 FILE, RESULT, TEXT, LEGEND, MEMFIL
      CHARACTER*128 MRGFIL, MODFIL, PLTFIL, PLTOLD
      CHARACTER*120 COM
      CHARACTER*8 ELAPSE
      CHARACTER*20 WORDS(80)
      CHARACTER*16 COMMS(NCOMMS), VCOMMS(3)
      INTEGER I, IER, NP, L, NS, NB, NT, NREC, NBYTES, LOC(NARRAY)
      INTEGER IB, JB, MEMNP, NPX, NPY, MODE, NDAY, NCOMP, IP, NWORDS
      INTEGER GETIN, VLBOPE, LEN1, VALLOC, PGBEG, LOOKUP, KCTOI
      INTEGER BCOUNT(MAXB), IB1, IB2, IB3, IS(3), L1, L2, L3, HLB
      INTEGER NMDCMP, NPP, JUNK, REFANT
      INTEGER KFLAG
      LOGICAL MEM, MODEL, NOERR, GST, INTER
      REAL UTMIN, UTMAX, GSTMIN, GSTMAX, TFLUX, CPU, PA, SCALE
      DOUBLE PRECISION KCTOR
      REAL AFAC(MAXB), CFAC(MAXB), AMPLAF, VECTAF
      CHARACTER*17 BNAME(MAXB)
      INTEGER BLABEL(MAXB)
      CHARACTER*26 TNAME
      CHARACTER*40 TELESC
      CHARACTER*1  AXIS
*
      CHARACTER*26 HBNAME
      COMMON /CPLT/ HBNAME
      LOGICAL GRID
      REAL U1, U2, V1, V2, ASCALE, PSCALE, CSCALE
      INTEGER DOTSIZ
      COMMON /PLT/ GRID, U1, U2, V1, V2, DOTSIZ, ASCALE, PSCALE, CSCALE
      INTEGER P(2)
      COMMON /STORE/ P
      CHARACTER*8 STNAME
      COMMON /PARMS/ STNAME(40)
      COMMON /LABEL/ BLABEL
      INTEGER PAGE, ANNLEV
      COMMON /ANN/ PAGE, ANNLEV
*
      DOUBLE PRECISION PARS(NPARS), VALS(NPARS), ENDMRK
      DATA PARS   / 'Input   ', 9*' ',
     1              'MODfile ', 9*' ',
     2              'MOCkdata', 9*' ',
     3              'Plotfile', 9*' ',
     4              2*'Npxy',
     5              'NOErrors',
     6              'GRid', 'GSt',
     7              2*'Urange', 2*'Vrange',
     8              'Dotsize',
     9              'NUmber',
     A              'AScale', 'PScale', 'CScale',
     B              'ANnot', 'COmmand', 9*' ' /
      DATA VALS   / 10*' ',
     1              10*' ',
     2              10*' ',
     3              '?', 9*' ',
     4              1D0, 1D0,
     5              -1D0,
     6              -1D0, -1D0,
     7              2*0D0, 2*0D0,
     8              1D0,
     9              5000D0,
     A              0D0, 180D0, 180D0,
     B              1D0, 10* ' ' /
      DATA ENDMRK / '/' /
      DATA COMMS / 'ALL', 'BACK', 'BL', 'CL', 'COLOR', 'EXIT', 'GST',
     1             'HIGHLIGHT', 'LIST', 'MODFILE', 'NEXT', 'PHASE', 
     2             'PLOTFILE', 'PROJECT', 'QUIT', 'RADIUS', 'REFANT',
     3             'STACK', 'TIMELINE', 'UT', 'UVPLOT', 'VISPLOT'/
      DATA VCOMMS / 'AMPLITUDES', 'PHASES', 'CLOSURE-PHASES' /
C-----------------------------------------------------------------------
*
* Read parameters
*
      CALL PUTOUT('VPLOT plots VLBI visibility data')
      CALL PUTOUT('(Version '//VERSN//')')
      MODE = 0
      PLTOLD = '****'
  10  IF (MODE.NE.0) STOP
      CALL PUTOUT(' ')
      CALL PUTOUT('VPLOT control parameters:')
      CALL KEYIN(PARS,VALS,NPARS,ENDMRK,MODE,5,6)
      WRITE (MRGFIL,'(10A8)') (VALS(I),I= 1,10)
      WRITE (MODFIL,'(10A8)') (VALS(I),I=11,20)
      WRITE (MEMFIL,'(10A8)') (VALS(I),I=21,30)
      WRITE (PLTFIL,'(10A8)') (VALS(I),I=31,40)
      WRITE (COM,   '(10A8)') (VALS(I),I=56,65)
      NPX = VALS(41)
      NPY = VALS(42)
      NOERR = VALS(43) .GE. 0D0
      GRID  = VALS(44) .GE. 0D0
      GST   = VALS(45) .GE. 0D0
      U1    = MIN(VALS(46),VALS(47))
      U2    = MAX(VALS(46),VALS(47))
      V1    = MIN(VALS(48),VALS(49))
      V2    = MAX(VALS(48),VALS(49))
      DOTSIZ= NINT(VALS(50))
      NMDCMP= NINT(VALS(51))
      ASCALE= VALS(52)
      PSCALE= VALS(53)
      CSCALE= VALS(54)
      IF (PSCALE.LE.0.0 .OR. PSCALE.GT.180.0) PSCALE = 180.0
      IF (CSCALE.LE.0.0 .OR. CSCALE.GT.180.0) CSCALE = 180.0
      ANNLEV= NINT(VALS(55))
*
* Open plot file.
*
      IF (PLTFIL.NE.PLTOLD) THEN
          IER = PGBEG(0, PLTFIL, NPX, NPY)
          PLTOLD = '****'
          IF (IER.NE.1) GOTO 10
          PLTOLD = PLTFIL
          PAGE = 0
      END IF
      CALL PGASK(.FALSE.)
*
* Open the MEM file and read the header to get the name of the
* associated Merge file. Otherwise use the specified Merge file.
*
      IF (MEMFIL.NE.' ') THEN
          L = LEN1(MEMFIL)
          OPEN (UNIT=UMOCK, FILE=MEMFIL(1:L), STATUS='OLD', READONLY,
     *      FORM='UNFORMATTED', IOSTAT=IER)
          READ (UMOCK,IOSTAT=IER) FILE
          READ (UMOCK,IOSTAT=IER) MEMNP
          IF (IER.NE.0) CALL ERROR(
     *        'MOCKDATA file does not exist or is invalid')
          IF (MRGFIL.NE.' ') FILE = MRGFIL
          MEM = .TRUE.
      ELSE
          FILE = MRGFIL
          MEMNP = 0
          MEM = .FALSE.
      END IF
*
* Find size of input merge file.
*
      IER = VLBOPE(UMRG, FILE, 'MERGE', 'OLD', RESULT)
      IF (IER.NE.1) GOTO 10
      CALL PUTOUT('Data file (MERGE): '//RESULT(1:LEN1(RESULT)))
      CALL MRGSIZ(UMRG, NS, NB, NT, NREC, NP)
      IF (MEM .AND. NP.NE.MEMNP) THEN
          WRITE (*,*) 'NP=',NP,'     MEMNP=',MEMNP
          CALL ERROR('Merge file doesn''t match MEM file.')
      END IF
*
* Allocate memory. We need NARRAY arrays of dimension NP for the
* following: 
*   1. time, 
*   2. baseline,
*   3. u,
*   4. v, 
*   5. amp,
*   6. phase,
*   7. err,
*   8. modamp,
*   9. modphase,
*  10. work array,
*  11. GST,
*  12. Work array.
*
      NBYTES = NP*4
      WRITE (TEXT, 640) NP, REAL(NBYTES*NARRAY)/(1024.0*1024.0)
  640 FORMAT ('Allocating memory for',I7,' data points;',
     *        F8.2,' Mbytes.')
      CALL PUTOUT(TEXT(1:LEN1(TEXT)))
      DO 30 I=1,NARRAY
          LOC(I) = VALLOC(NBYTES/4, P)
   30 CONTINUE
*
* Read the Merge data into memory.
*
      REWIND (UNIT=UMRG)
      CALL MRGRD(UMRG, NP, P(LOC(1)), P(LOC(2)), P(LOC(3)),
     *  P(LOC(4)), P(LOC(5)), P(LOC(6)), P(LOC(7)),
     *  P(LOC(11)),
     *  LEGEND,BNAME,BLABEL,TELESC,BCOUNT,KFLAG)
      CLOSE (UNIT=UMRG)
*
* Determine time range for plotting. AXIS is returned as 'D' for
* plotting against UT days; 'H' for plotting against UT hms. 
* NDAY is the reference day number for AXIS='H'.
* Determine GST range for plotting and convert GST to
* hours in this range.
*
      CALL UTRNGE(NP, P(LOC(1)), UTMIN, UTMAX, AXIS, NDAY)
      CALL GSTRNG(NP, P(LOC(11)), GSTMIN, GSTMAX)
*
* Read the MEM mock-data into memory, or compute model.
*
      IF (MEM) THEN
          MODEL = .TRUE.
          CALL GETMEM(UMOCK, NP, P(LOC(8)), P(LOC(9)))
          CLOSE (UNIT=UMOCK)
      ELSE IF (MODFIL.EQ.' ') THEN
          MODEL = .FALSE.
      ELSE
          MODEL = .TRUE.
          IER = VLBOPE(UMOD, MODFIL, 'MODEL', 'OLD', MODFIL)
          IF (IER.NE.1) THEN
              MODEL = .FALSE.
              CALL PUTOUT('+++ Problem with MODFILE')
              NCOMP = 0
              MODFIL = ' '
          ELSE
              MODEL = .TRUE.
              CALL PUTOUT(' ')
              CALL PUTOUT('Reading model file '//MODFIL(1:LEN1(MODFIL)))
              CALL GETMOD(UMOD, NMDCMP, NCOMP, TFLUX)
              CLOSE (UNIT=UMOD)
              WRITE (TEXT,'(A,I5,A,F8.3,A)') 'Computing model... ',
     *               NCOMP, ' components,', TFLUX, ' Jy'
              CALL PUTOUT(TEXT(:LEN1(TEXT)))
              CALL SCLOCK
              CALL CMPMOD(NP, P(LOC(3)), P(LOC(4)),
     *                        P(LOC(8)), P(LOC(9)))
              CALL RCLOCK(ELAPSE, CPU)
              WRITE (TEXT,'(A,F8.2,A)')
     *               'CPU time for model calculation =', CPU, ' s'
              CALL PUTOUT(TEXT(:LEN1(TEXT)))
          END IF
      END IF
*
* Evaluate agreement factors.
*
      IF (MODEL) CALL AGRFAC(NP, NB, P(LOC(2)), P(LOC(5)), 
     *            P(LOC(6)), P(LOC(7)), P(LOC(8)), 
     *            P(LOC(9)), AFAC, CFAC, AMPLAF, VECTAF)
*
* Interactive mode.
*
      INTER = COM.EQ.' '
      IB = 0
      HLB = 0
      NPP = 5
      IF (INTER) CALL HELP
      IF (.NOT.INTER) GOTO 201
*
  200 IF (INTER) THEN
          IER = GETIN(COM, 'VPLOT> ',L)
      ELSE
          IER = 0
      END IF
      IF (IER.NE.1) THEN
          COM = 'EXIT'
          L = 4
      ELSE IF (L.EQ.0) THEN
          COM = 'N'
          L = 1
      END IF
*
* Parse the command and check for minimum match.
*
 201  CALL PARSEB(COM, WORDS, NWORDS)
      IF (NWORDS.LT.1) THEN
          WORDS(1) = 'NEXT'
          NWORDS = 1
      END IF
      CALL KEYFND(WORDS(1), COMMS, NCOMMS, L, IER)
      IF (IER.NE.0) THEN
          CALL PUTOUT('Invalid command')
          CALL HELP
          GOTO 200
      END IF
      WORDS(1) = COMMS(L)
*
*- GST command --
      IF (WORDS(1).EQ.'GST') THEN
          GST = .TRUE.
          CALL PUTOUT('Time axis is now GST')
          GOTO 200
*- UT command --
      ELSE IF (WORDS(1).EQ.'UT') THEN
          GST = .FALSE.
          CALL PUTOUT('Time axis is now UT')
          GOTO 200
*- COLOR command --
      ELSE IF (WORDS(1).EQ.'COLOR') THEN
          CALL COLOR
          GOTO 200
*- HIGHLIGHT command --
      ELSE IF (WORDS(1).EQ.'HIGHLIGHT') THEN
        IF (NWORDS.EQ.2) THEN
*         -- Find one station name or number
          CALL KEYFND(WORDS(2), STNAME, NS, IS(1), IER)
          IP = 1
          IF (IER.NE.0) IS(1) = KCTOI(WORDS(2),IP)
          IF (IS(1).LT.1 .OR. IS(1).GT.NS) THEN
              CALL PUTOUT('Bad station name: '//WORDS(1+I))
              HLB = 0
              GOTO 200
          END IF
          HBNAME = STNAME(IS(1))
          CALL PUTOUT(HBNAME)
          HLB = -IS(1)
        ELSE IF (NWORDS.EQ.3) THEN  
*         -- Find two station names and numbers
          DO 196 I=1,2
            CALL KEYFND(WORDS(1+I), STNAME, NS, IS(I), IER)
            IP = 1
            IF (IER.NE.0) IS(I) = KCTOI(WORDS(1+I),IP)
            IF (IS(I).LT.1 .OR. IS(I).GT.NS) THEN
              CALL PUTOUT('Bad station name: '//WORDS(1+I))
              HLB = 0
              GOTO 200
            END IF
  196     CONTINUE
*         -- Order the stations
          CALL ISORT(IS, 2)
*         -- Baseline name
          L1 = LEN1(STNAME(IS(1)))
          L2 = LEN1(STNAME(IS(2)))
          HBNAME = STNAME(IS(1))(:L1)//'-'//STNAME(IS(2))(:L2)
          CALL PUTOUT(HBNAME)
*         -- Find the baseline number
          HLB = LOOKUP(100*IS(1)+IS(2), BLABEL, NB)
        ELSE
          CALL PUTOUT('Specify one or two antenna names or numbers')
        END IF
        GOTO 200
*- REFANT command --
      ELSE IF (WORDS(1).EQ.'REFANT') THEN
        IF (NWORDS.EQ.2) THEN
*         -- Find one station name or number
          CALL KEYFND(WORDS(2), STNAME, NS, IS(1), IER)
          IP = 1
          IF (IER.NE.0) IS(1) = KCTOI(WORDS(2),IP)
          IF (IS(1).LT.1 .OR. IS(1).GT.NS) THEN
              CALL PUTOUT('Bad station name: '//WORDS(2))
              REFANT = 0
              GOTO 200
          END IF
          CALL PUTOUT('REFANT '//STNAME(IS(1)))
          REFANT = IS(1)
        ELSE 
          REFANT = 0
          CALL PUTOUT('REFANT cleared')
        END IF
        GOTO 200
*- STACK command --
      ELSE IF (WORDS(1).EQ.'STACK') THEN
*         -- Amplitudes or phases?
          IF (NWORDS.LT.2 .OR. NWORDS.GT.3) THEN
              IER = 1
          ELSE
*             -- Find keyword
              IF (NWORDS.GT.2) THEN
                  CALL KEYFND(WORDS(3), VCOMMS, 2, JUNK, IER)
              ELSE
                  IER = 0
                  JUNK =1
              END IF
          END IF
          IF (IER.NE.0) THEN
              CALL PUTOUT('Usage: STACK station AMPL|PHASE')
              GOTO 200
          END IF
*         -- Find a station name
          CALL KEYFND(WORDS(2), STNAME, NS, IS(1), IER)
          IP = 1
          IF (IER.NE.0) IS(1) = KCTOI(WORDS(2),IP)
          IF (IS(1).LT.1 .OR. IS(1).GT.NS) THEN
              CALL PUTOUT('Bad station name: '//WORDS(2))
              GOTO 200
          END IF
          L1 = LEN1(STNAME(IS(1)))
          TNAME = STNAME(IS(1))(:L1)
          CALL PUTOUT(TNAME)
          IF (JUNK.EQ.1) THEN
*             -- amplitude plot
              CALL SAPLOT(NS-1, NP, NS, NB, 
     *                STNAME, BLABEL,
     *                UTMIN-0.5/24.0, UTMAX+0.5/24.0, P(LOC(1)), 
     *                GSTMIN, GSTMAX, P(LOC(11)),
     *                P(LOC(2)), P(LOC(5)), P(LOC(7)),
     *                P(LOC(8)), LEGEND, MODEL, AXIS, NDAY, NOERR, GST,
     *                RESULT, MODFIL, NCOMP, TFLUX, IS(1), KFLAG)
          ELSE IF (JUNK.EQ.2) THEN
*             -- phase plot
              CALL SPPLOT(NS-1, NP, NS, NB, 
     *                STNAME, BLABEL,
     *                UTMIN-0.5/24.0, UTMAX+0.5/24.0, P(LOC(1)), 
     *                GSTMIN, GSTMAX, P(LOC(11)),
     *                P(LOC(2)), P(LOC(5)), P(LOC(6)), P(LOC(7)),
     *                P(LOC(9)), LEGEND, MODEL, AXIS, NDAY, NOERR, GST,
     *                RESULT, MODFIL, NCOMP, TFLUX, IS(1))
          END IF
          GOTO 200
*- MODFILE command --
      ELSE IF (WORDS(1).EQ.'MODFILE') THEN
          IF (NWORDS.LT.2) THEN
              IER = GETIN(MODFIL, 'Model file name: ',L)
          ELSE
              MODFIL = WORDS(2)
              L = LEN1(MODFIL)
          END IF
          IER = VLBOPE(UMOD, MODFIL(:L), 'MODEL', 'OLD', MODFIL)
          IF (IER.NE.1) THEN
              MODEL = .FALSE.
              CALL PUTOUT('+++ Problem with MODFILE')
              NCOMP = 0
              MODFIL = ' '
          ELSE
              MODEL = .TRUE.
              CALL PUTOUT(' ')
              CALL PUTOUT('Reading model file '//MODFIL(1:LEN1(MODFIL)))
              CALL GETMOD(UMOD, NMDCMP, NCOMP, TFLUX)
              CLOSE (UNIT=UMOD)
              WRITE (TEXT,'(A,I5,A,F8.3,A)') 'Computing model... ',
     *               NCOMP, ' components,', TFLUX, ' Jy'
              CALL PUTOUT(TEXT(:LEN1(TEXT)))
              CALL SCLOCK
*             -- compute new model
              CALL CMPMOD(NP, P(LOC(3)), P(LOC(4)),
     *                        P(LOC(8)), P(LOC(9)))
              CALL RCLOCK(ELAPSE, CPU)
              WRITE (TEXT,'(A,F8.2,A)')
     *               'CPU time for model calculation =', CPU, ' s'
              CALL PUTOUT(TEXT(:LEN1(TEXT)))
              CALL AGRFAC(NP, NB, P(LOC(2)), P(LOC(5)), 
     *            P(LOC(6)), P(LOC(7)), P(LOC(8)), 
     *            P(LOC(9)), AFAC, CFAC, AMPLAF, VECTAF)
          END IF
          GOTO 200
*- EDMODEL command --
      ELSE IF (WORDS(1).EQ.'EDMODEL') THEN
          IF (NCOMP.GT.10) THEN
              CALL PUTOUT('Too many components to edit')
          ELSE
              CALL EDMOD
          END IF
          GOTO 200
*- NEXT command --
      ELSE IF (WORDS(1).EQ.'NEXT') THEN
  175     IB = IB+1
          IF (IB.GT.NB) THEN
              CALL PUTOUT('No more baselines')
              IB = NB
              GOTO 200
          END IF
          IF (BCOUNT(IB).LE.0) GOTO 175
*- BACK command --
      ELSE IF (WORDS(1).EQ.'BACK') THEN
  176     IB = IB-1
          IF (IB.LT.1) THEN
              CALL PUTOUT('No earlier baseline')
              IB = 1
              GOTO 200
          END IF
          IF (BCOUNT(IB).LE.0) GOTO 176
*- ALL command --
      ELSE IF (WORDS(1).EQ.'ALL') THEN
          DO 181 JB=1,NB
            IF (BCOUNT(JB).GT.0) THEN
              CALL BLPLOT(NP, JB,
     *              UTMIN-0.5/24.0, UTMAX+0.5/24.0, P(LOC(1)), 
     *              GSTMIN, GSTMAX, P(LOC(11)), P(LOC(2)),
     *              P(LOC(5)), P(LOC(6)), P(LOC(7)),
     *              P(LOC(8)), P(LOC(9)), 
     *              LEGEND, BNAME(JB), MODEL, 
     *              AXIS, NDAY, NOERR,GST)
              CALL ANNOT(RESULT, MODFIL, NCOMP, TFLUX)
              CALL PGASK(.TRUE.)
            END IF
  181     CONTINUE
          CALL PGASK(.FALSE.)
          GOTO 200
*- UVPLOT command --
      ELSE IF (WORDS(1).EQ.'UVPLOT') THEN
        IF (NWORDS.GE.2) CALL UPCASE(WORDS(2))
        IF (NWORDS.LT.2) THEN
*         -- Simple uv plot          
          CALL UVPLOT(NP, LEGEND, TELESC, P(LOC(3)), P(LOC(4)),
     *                P(LOC(2)), HLB)
          CALL ANNOT(RESULT, ' ', NCOMP, TFLUX)
        ELSE IF (NWORDS.GT.2) THEN
        ELSE IF (WORDS(2)(1:1).EQ.'A') THEN
*         -- Display amplitude
          CALL AMPLOT(NP, LEGEND, TELESC, P(LOC(3)), P(LOC(4)),
     *                P(LOC(5)), P(LOC(12)), 'Amplitude')
          CALL ANNOT(RESULT, ' ', NCOMP, TFLUX)
        ELSE IF (WORDS(2)(1:1).EQ.'M') THEN
*         -- Display model amplitude
          IF (MODEL) THEN
              CALL AMPLOT(NP, LEGEND, TELESC, P(LOC(3)), P(LOC(4)),
     *                P(LOC(8)), P(LOC(12)), 'Model Ampl')
              CALL ANNOT(RESULT, MODFIL, NCOMP, TFLUX)
          ELSE
              CALL PUTOUT('No model file selected')
          END IF
        ELSE IF (WORDS(2)(1:1).EQ.'R') THEN
*         -- Display ampl/model
          IF (MODEL) THEN
              CALL RATIO(NP, P(LOC(5)), P(LOC(8)), P(LOC(10)))
              CALL AMPLOT(NP, LEGEND, TELESC, P(LOC(3)), P(LOC(4)),
     *                P(LOC(10)), P(LOC(12)), 'Ampl/Model')
              CALL ANNOT(RESULT, MODFIL, NCOMP, TFLUX)
          ELSE
              CALL PUTOUT('No model file selected')
          END IF
        ELSE IF (WORDS(2)(1:1).EQ.'E') THEN
*         -- Display amplitude error
          IF (MODEL) THEN
              CALL QERROR(NP, P(LOC(5)), P(LOC(8)), P(LOC(7)), 
     *                P(LOC(10)))
              CALL AMPLOT(NP, LEGEND, TELESC, P(LOC(3)), P(LOC(4)),
     *                P(LOC(10)), P(LOC(12)), 'Ampl error')
              CALL ANNOT(RESULT, MODFIL, NCOMP, TFLUX)
          ELSE
              CALL PUTOUT('No model file selected')
          END IF
        END IF
        GOTO 200
*- RAD command --
      ELSE IF (WORDS(1).EQ.'RADIUS') THEN
          CALL RPLOT(NP, LEGEND, TELESC, P(LOC(2)), P(LOC(3)), 
     *       P(LOC(4)), P(LOC(5)), P(LOC(8)), P(LOC(10)), MODEL, HLB)
          CALL ANNOT(RESULT, MODFIL, NCOMP, TFLUX)
          GOTO 200
*- PHASE command --
      ELSE IF (WORDS(1).EQ.'PHASE') THEN
          CALL PPLOT(NP, LEGEND, TELESC, P(LOC(2)), P(LOC(3)), 
     *       P(LOC(4)), P(LOC(6)), P(LOC(9)), P(LOC(10)), MODEL, HLB)
          CALL ANNOT(RESULT, MODFIL, NCOMP, TFLUX)
          GOTO 200
*- TIMELINE command --
      ELSE IF (WORDS(1).EQ.'TIMELINE') THEN
          CALL TIPLOT(NS, NP, LEGEND, TELESC, STNAME, BLABEL,
     *       GST, GSTMIN, GSTMAX, UTMIN-0.5/24.0, UTMAX+0.5/24.0,
     *       AXIS, NDAY, 
     *       P(LOC(1)), P(LOC(2)), P(LOC(11)))
          CALL ANNOT(RESULT, ' ', NCOMP, TFLUX)
          GOTO 200
*- PROJECT command --
      ELSE IF (WORDS(1).EQ.'PROJECT') THEN
          IP = 1
          PA = KCTOR(WORDS(2), IP)
          PA = MOD(PA,360.0)
          IF (PA.GT.180.0) PA = PA-360.0
          CALL PAPLOT(NP, LEGEND, TELESC, P(LOC(2)), P(LOC(3)), 
     *       P(LOC(4)), P(LOC(5)), P(LOC(8)), P(LOC(10)), MODEL, PA,
     *       HLB)
          CALL ANNOT(RESULT, MODFIL, NCOMP, TFLUX)
          GOTO 200
*- CLPLOT command --
      ELSE IF (WORDS(1).EQ.'CL') THEN
*         -- Find three station names and numbers
          DO 197 I=1,3
            CALL KEYFND(WORDS(1+I), STNAME, NS, IS(I), IER)
            IP = 1
            IF (IER.NE.0) IS(I) = KCTOI(WORDS(1+I),IP)
            IF (IS(I).LT.1 .OR. IS(I).GT.NS) THEN
              CALL PUTOUT('Bad station name: '//WORDS(1+I))
              GOTO 200
            END IF
  197     CONTINUE
*         -- Order the stations
          CALL ISORT(IS, 3)
*         -- Triangle name
          L1 = LEN1(STNAME(IS(1)))
          L2 = LEN1(STNAME(IS(2)))
          L3 = LEN1(STNAME(IS(3)))
          TNAME = STNAME(IS(1))(:L1)//'-'//
     *            STNAME(IS(2))(:L2)//'-'//STNAME(IS(3))(:L3)
          CALL PUTOUT(TNAME)
*         -- Find the baseline numbers
          IB1 = LOOKUP(100*IS(1)+IS(2), BLABEL, NB)
          IB2 = LOOKUP(100*IS(2)+IS(3), BLABEL, NB)
          IB3 = LOOKUP(100*IS(1)+IS(3), BLABEL, NB)
          CALL CLPLOT(NP, IB1, IB2, IB3, 
     *              UTMIN-0.5/24.0, UTMAX+0.5/24.0, P(LOC(1)), 
     *              GSTMIN, GSTMAX, P(LOC(11)), P(LOC(2)),
     *              P(LOC(5)), P(LOC(6)), P(LOC(7)),
     *              P(LOC(8)), P(LOC(9)), 
     *              LEGEND, TNAME, MODEL, AXIS, NDAY, NOERR, GST)
          CALL ANNOT(RESULT, MODFIL, NCOMP, TFLUX)
          GOTO 200
*- BLPLOT command --
      ELSE IF (WORDS(1).EQ.'BL') THEN
*         -- Find two station names and numbers
          DO 198 I=1,2
            CALL KEYFND(WORDS(1+I), STNAME, NS, IS(I), IER)
            IP = 1
            IF (IER.NE.0) IS(I) = KCTOI(WORDS(1+I),IP)
            IF (IS(I).LT.1 .OR. IS(I).GT.NS) THEN
              CALL PUTOUT('Bad station name: '//WORDS(1+I))
              GOTO 200
            END IF
  198     CONTINUE
*         -- Order the stations
          CALL ISORT(IS, 2)
*         -- Baseline name
          L1 = LEN1(STNAME(IS(1)))
          L2 = LEN1(STNAME(IS(2)))
          TNAME = STNAME(IS(1))(:L1)//'-'//STNAME(IS(2))(:L2)
          CALL PUTOUT(TNAME)
*         -- Find the baseline number
          IB = LOOKUP(100*IS(1)+IS(2), BLABEL, NB)
          CALL BLPLOT(NP, IB,
     *              UTMIN-0.5/24.0, UTMAX+0.5/24.0, P(LOC(1)), 
     *              GSTMIN, GSTMAX, P(LOC(11)), P(LOC(2)),
     *              P(LOC(5)), P(LOC(6)), P(LOC(7)),
     *              P(LOC(8)), P(LOC(9)), 
     *              LEGEND, BNAME(IB), MODEL, 
     *              AXIS, NDAY, NOERR,GST)
          CALL ANNOT(RESULT, MODFIL, NCOMP, TFLUX)
          GOTO 200
*- VISPLOT command --
      ELSE IF (WORDS(1).EQ.'VISPLOT') THEN
          IF (NWORDS.LT.2) THEN
              IER = 1
          ELSE
*             -- Find keyword
              CALL KEYFND(WORDS(2), VCOMMS, 3, JUNK, IER)
          END IF
          IF (IER.NE.0) THEN
              CALL PUTOUT(
     :       'Usage: VISPLOT AMPL|PHAS|CLOS [plots/page] [yrange]')
              GOTO 200
          END IF
          IF (NWORDS.GE.3) THEN
              IP = 1
              NPP = KCTOI(WORDS(3),IP)
              IF (NPP.LT.1) NPP = 5
          END IF
          SCALE = 0.0
          IF (NWORDS.GE.4) THEN
              IP = 1
              SCALE = KCTOR(WORDS(4),IP)
          END IF
          IF (JUNK.EQ.1) THEN
              IF (NWORDS.LT.4)  SCALE = ASCALE
              IF (SCALE.LT.0.0) SCALE = 0.0
          ELSE IF (JUNK.EQ.2) THEN
              IF (SCALE.EQ.0.0) SCALE = PSCALE
              IF (SCALE.LT.0.0 .OR. SCALE.GT.180.0) SCALE = 180.0
          ELSE IF (JUNK.EQ.3) THEN
              IF (SCALE.EQ.0.0) SCALE = CSCALE
              IF (SCALE.LT.0.0 .OR. SCALE.GT.180.0) SCALE = 180.0
          END IF
          WRITE (TEXT, '(A,1X,A,1X,I3,1X,F8.2)') 'VISPLOT',
     :             VCOMMS(JUNK), NPP, SCALE        
          CALL PUTOUT(TEXT(1:LEN1(TEXT)))
          IF (JUNK.EQ.1) THEN
*             -- amplitude plot
              ASCALE = SCALE
              CALL SAPLOT(NPP, NP, NS, NB, 
     *                STNAME, BLABEL,
     *                UTMIN-0.5/24.0, UTMAX+0.5/24.0, P(LOC(1)), 
     *                GSTMIN, GSTMAX, P(LOC(11)),
     *                P(LOC(2)), P(LOC(5)), P(LOC(7)),
     *                P(LOC(8)), LEGEND, MODEL, AXIS, NDAY, NOERR, GST,
     *                RESULT, MODFIL, NCOMP, TFLUX, REFANT, KFLAG)
          ELSE IF (JUNK.EQ.2) THEN
*             -- phase plot
              PSCALE = SCALE
              CALL SPPLOT(NPP, NP, NS, NB, 
     *                STNAME, BLABEL,
     *                UTMIN-0.5/24.0, UTMAX+0.5/24.0, P(LOC(1)), 
     *                GSTMIN, GSTMAX, P(LOC(11)),
     *                P(LOC(2)), P(LOC(5)), P(LOC(6)), P(LOC(7)),
     *                P(LOC(9)), LEGEND, MODEL, AXIS, NDAY, NOERR, GST,
     *                RESULT, MODFIL, NCOMP, TFLUX, REFANT)
          ELSE IF (JUNK.EQ.3) THEN
*             -- closure-phase plot
              CSCALE = SCALE
              CALL SCPLOT(NPP, NP, NS, NB, 
     *                STNAME, BLABEL,
     *                UTMIN-0.5/24.0, UTMAX+0.5/24.0, P(LOC(1)), 
     *                GSTMIN, GSTMAX, P(LOC(11)),
     *                P(LOC(2)), P(LOC(5)), P(LOC(6)), P(LOC(7)),
     *                P(LOC(9)), LEGEND, MODEL, AXIS, NDAY, NOERR, GST,
     *                RESULT, MODFIL, NCOMP, TFLUX, REFANT)
          END IF
          GOTO 200
*- PLOTFILE command --
      ELSE IF (WORDS(1).EQ.'PLOTFILE') THEN
          IF (NWORDS.LT.2) PLTFIL = '?'
          IF (NWORDS.GE.2) PLTFIL = WORDS(2)
          IER = PGBEG(0, PLTFIL, NPX, NPY)
          IF (IER.NE.1) IER = PGBEG(0, '?', NPX, NPY)
          IF (IER.NE.1) GOTO 300
          PAGE = 0
          CALL PGASK(.FALSE.)
          GOTO 200
*- EXIT command --
      ELSE IF (WORDS(1).EQ.'EXIT' .OR. WORDS(1).EQ.'QUIT') THEN
          CALL PGEND
          GOTO 300
*- LIST command --
      ELSE IF (WORDS(1).EQ.'LIST') THEN
          WRITE (TEXT,'(7X,A17,2X,A5,2X,A10,2X,A10)') 'Baseline    ',
     *                               'Pts', 'Ampl AF', 'Vect AF'
          CALL PUTOUT(TEXT(1:LEN1(TEXT)))
          DO 165 JB=1,NB
              IF (MODEL) THEN
                  WRITE (TEXT, '(I5,2X,A17,2X,I5,2F12.3)') 
     *                JB, BNAME(JB), BCOUNT(JB), AFAC(JB), CFAC(JB)
              ELSE
                  WRITE (TEXT, '(I5,2X,A17,2X,I5)')
     *                JB, BNAME(JB), BCOUNT(JB)
              END IF
              IF (BCOUNT(JB).GT.0) CALL PUTOUT(TEXT(1:LEN1(TEXT)))
  165     CONTINUE
          IF (MODEL) THEN
              WRITE (TEXT, '(7X,A17,2X,I5,2F12.3)') 
     *            'Total', NP, AMPLAF, VECTAF
          ELSE
              WRITE (TEXT, '(7X,A17,2X,I5)')
     *            'Total', NP
          END IF
          CALL PUTOUT(TEXT(1:LEN1(TEXT)))
          GOTO 200
*- default command (baseline number) --
      ELSE
          IP = 1
          IB = KCTOI(WORDS(1), IP)
          IF (IB.EQ.0) THEN
              CALL HELP
              GOTO 200
          END IF
      END IF
      IF (IB.GT.NB .OR. IB.LT.1) THEN
          CALL PUTOUT('No such baseline')
      ELSE
          CALL BLPLOT(NP, IB,
     *              UTMIN-0.5/24.0, UTMAX+0.5/24.0, P(LOC(1)), 
     *              GSTMIN, GSTMAX, P(LOC(11)), P(LOC(2)),
     *              P(LOC(5)), P(LOC(6)), P(LOC(7)),
     *              P(LOC(8)), P(LOC(9)), 
     *              LEGEND, BNAME(IB), MODEL, 
     *              AXIS, NDAY, NOERR,GST)
          CALL ANNOT(RESULT, MODFIL, NCOMP, TFLUX)
      END IF
      GOTO 200
*
* Exit from program
*
  300 CONTINUE
*
      END

C*AGRFAC -- compute agreement factors for model and data
C+
      SUBROUTINE AGRFAC(NP, NB, BASE, AMP, PHS, ERR, MODAMP, MODPHS, 
     *                  AFAC, CFAC, AMPLAF, VECTAF)
C     -- Input
      INTEGER NP, NB, BASE(NP)
      REAL AMP(NP), PHS(NP), ERR(NP), MODAMP(NP), MODPHS(NP)
C     -- Output
      REAL AFAC(NB), CFAC(NB), AMPLAF, VECTAF
C-----------------------------------------------------------------------
      REAL PI, RPDEG
      PARAMETER (PI = 3.141592653589793)
      PARAMETER (RPDEG = PI/180.0)
C
      INTEGER MAXB
      PARAMETER (MAXB=780)
      INTEGER IP, IB, NPT(MAXB)
      REAL P, Q, PH, MODPH, ASUM, VSUM, BSUM(MAXB), CSUM(MAXB)
C
      VSUM = 0.0
      ASUM = 0.0
      DO 10 IP=1,NP
          IB = BASE(IP)
          P = ((AMP(IP) - MODAMP(IP))/ERR(IP))**2
          ASUM = ASUM + P
          BSUM(IB) = BSUM(IB) + P
          PH = PHS(IP)*RPDEG
          MODPH = MODPHS(IP)*RPDEG
C;          Q = ((AMP(IP)*COS(PH) - MODAMP(IP)*COS(MODPH))/ERR(IP))**2 +
C;     *        ((AMP(IP)*SIN(PH) - MODAMP(IP)*SIN(MODPH))/ERR(IP))**2
C;          VSUM = VSUM + Q*0.5
C;          CSUM(IB) = CSUM(IB) + Q*0.5  
          Q = (AMP(IP)**2 + MODAMP(IP)**2 - 2.0*AMP(IP)*MODAMP(IP)*
     *         COS(PH-MODPH))/(2.0*ERR(IP)**2)
          VSUM = VSUM + Q
          CSUM(IB) = CSUM(IB) + Q
          NPT(IB) = NPT(IB)+1
   10 CONTINUE
      DO 20 IB=1,NB
          IF (NPT(IB).GT.0) THEN
              AFAC(IB) = SQRT(BSUM(IB)/REAL(NPT(IB)))
              CFAC(IB) = SQRT(CSUM(IB)/REAL(NPT(IB)))
          ELSE
              AFAC(IB) = 0.0
              CFAC(IB) = 0.0
          END IF
   20 CONTINUE
      AMPLAF = SQRT(ASUM/REAL(NP))
      VECTAF = SQRT(VSUM/REAL(NP))
      WRITE (*,600) AMPLAF, VECTAF
  600 FORMAT(' Amplitude agreement factor = ',G20.5/
     *       ' Vector agreement factor    = ',G20.5)
      END

C*AMPLOT -- plot uv coverage, weighted by amplitude
C+
      SUBROUTINE AMPLOT (NP, LEGEND, TELESC, U, V, AMP, WORK, TYPE)
      INTEGER NP, WORK(NP)
      REAL U(NP), V(NP), AMP(NP)
      CHARACTER*(*) LEGEND, TELESC, TYPE
C-----------------------------------------------------------------------
*
      CHARACTER*26 HBNAME
      COMMON /CPLT/ HBNAME
      INTEGER BLABEL(780)
      COMMON /LABEL/ BLABEL
      LOGICAL GRID
      REAL U1, U2, V1, V2, ASCALE, PSCALE, CSCALE
      INTEGER DOTSIZ
      COMMON /PLT/ GRID, U1, U2, V1, V2, DOTSIZ, ASCALE, PSCALE, CSCALE
*
      REAL UMAX, D, VPX1, VPX2, VPY1, VPY2, AMAX, ASCFAC, BSCALE
      REAL AMIN, CL, R, G, B
      REAL UW1, UW2, VW1, VW2, X, Y
      INTEGER IP, JP, SAVELW, IC, CI1, CI2
      LOGICAL GREY
*
* Find the maximum baseline and amplitude.
*
      UMAX = 0.0
      AMAX = 0.0
      AMIN = AMP(1)
      DO 100 IP=1,NP
          UMAX = MAX(UMAX,ABS(U(IP)),ABS(V(IP)))
          AMAX = MAX(AMAX,AMP(IP))
          AMIN = MIN(AMIN,AMP(IP))
  100 CONTINUE
      UMAX = 1.2*UMAX
      ASCFAC = 100.0/AMAX
      BSCALE = 16.0/(AMAX-AMIN)
      UW1 = -UMAX
      UW2 = UMAX
      VW1 = -UMAX
      VW2 = UMAX
      IF (U1.NE.U2) THEN
          UW1 = U1
          UW2 = U2
      END IF
      IF (V1.NE.V2) THEN
          VW1 = V1
          VW2 = V2
      END IF
*
* Create a sorted index to plot data in order of amplitude.
*
      DO 105 IP=1,NP
          WORK(IP) = IP
  105 CONTINUE
      CALL ARSORT(AMP, NP, WORK)
*
* Set up the viewport and window
*
      CALL PGBBUF
      CALL PGPAGE
      CALL PGSVP(0.0, 1.0, 0.0, 1.0)
      CALL PGQVP(1, VPX1, VPX2, VPY1, VPY2)
      D = MIN(VPX2-VPX1, VPY2-VPY1)/40.0
      VPX1 = VPX1 + 5.0*D
      VPX2 = VPX2 - 2.0*D
      VPY1 = VPY1 + 9.0*D
      VPY2 = VPY2 - 4.0*D
      CALL PGVSIZ(VPX1, VPX2, VPY1, VPY2)
      CALL PGWNAD(UW2,UW1,VW1,VW2)
      CALL PGMTXT('T',2.5,0.0,0.0,LEGEND)
      CALL PGMTXT('T',0.8,0.0,0.0,TELESC)
      CALL PGMTXT('T',0.8,1.0,1.0,TYPE)
      IF (GRID) THEN
          CALL PGSCI(14)
          CALL PGBOX('G', 0.0, 0, 'G', 0.0, 0)
      END IF
      CALL PGSCI(1)
      CALL PGBOX('BCNST', 0.0, 0, 'BCNSTV', 0.0, 0)
      CALL PGMTXT('B', 3.2, 0.5, 0.5, 
     1             '\(2263) East : u (wavelengths)')
      CALL PGMTXT('L', 3.2, 0.5, 0.5, 
     1            'v (wavelengths) : North \(2261)')
*
* Decide whether to use greyscale or point size
*
      CALL PGQCOL(CI1,CI2)
      GREY = CI2.GE.31
      IF (GREY) THEN
          DO 110 IC=16,31
              R = MAX(0.0,(IC-23)/8.0)
              B = MAX(0.0,(23-IC)/8.0)
              G = 1.0 - ABS(IC-23)/8.0
              CL = (0.5 + 0.5*(IC-15)/16.0)
              CALL PGSCR(IC, R, G, B)
  110     CONTINUE
      END IF
*
* Plot data
*
      CALL PGQLW(SAVELW)
      IF (GREY) THEN
          CALL PGSLW(DOTSIZ)
          DO 120 JP=1,NP
              IP = WORK(JP)
              CALL PGSCI(15+MAX(1,NINT((AMP(IP)-AMIN)*BSCALE)))
              CALL PGPT(1, U(IP), V(IP), -1)
              CALL PGPT(1, -U(IP), -V(IP), -1)
  120     CONTINUE
          DO 121 IC=16,31
              CALL PGSCI(IC)
              X = UW2 - (UW2-UW1)*(IC-15.5)/16.0
              Y = VW2 - 0.95*(VW2-VW1)
              CALL PGPT(1, X, Y, -1)
  121     CONTINUE
      ELSE
          CALL PGSCI(3)
          DO 130 IP=1,NP
              CALL PGSLW(MAX(1,NINT(SQRT(AMP(IP)*ASCFAC))))
              CALL PGPT(1, U(IP), V(IP), -1)
              CALL PGPT(1, -U(IP), -V(IP), -1)
  130     CONTINUE
      END IF
      CALL PGSCI(1)
      CALL PGSLW(SAVELW)
      CALL PGEBUF
*      
      END

      SUBROUTINE ANNOT(DNAME, MNAME, NCOMP, FLUX)
      CHARACTER*(*) DNAME, MNAME
      INTEGER NCOMP
      REAL FLUX
*-----------------------------------------------------------------------
* Annotation:
*       ANNLEV controls elevel of annotation:
*	    -1: none
*	     0: reduced
*            1: full
*	DNAME = data file name
*	MNAME = model file name
*       NCOMP = number of model components
*       FLUX  = total model flux (Jy)
*-----------------------------------------------------------------------
      REAL CH, DY, Y
      INTEGER CF, LW, CI, L, PAGE, ANNLEV
      CHARACTER*17 REVTIM
      CHARACTER*80 TEXT
      INTEGER LEN1
      COMMON /ANN/ PAGE, ANNLEV
*
      IF (ANNLEV.LT.0) RETURN
*
      CALL PGBBUF
      CALL PGQCH(CH)
      CALL PGQCF(CF)
      CALL PGQLW(LW)
      CALL PGQCI(CI)
*
      CALL PGSCH(0.6)
      CALL PGSCF(1)
      CALL PGSLW(1)
      CALL PGSCI(1)
*
      DY = 1.5
      Y = 7.5
C     -- Line 1: data file name
      L = LEN1(DNAME)
      CALL FRDT(DNAME(1:L), REVTIM)
      Y = Y+DY
      CALL PGMTXT('B', Y,0.0,0.0,'Visibility file: '//DNAME(1:L)//
     1            '  ('//REVTIM//')')
C     -- Line 2: model file name
      IF (MNAME.NE.' ') THEN
          L = LEN1(MNAME)
          CALL FRDT(MNAME(1:L), REVTIM)
          Y = Y+DY
          CALL PGMTXT('B', Y,0.0,0.0,'Model file: '//MNAME(1:L)//
     1            '  ('//REVTIM//')')
          Y = Y+DY
          WRITE (TEXT, '(''Model: '',I8,'' cmpts, '',F12.3,'' Jy'')')
     1       NCOMP, FLUX
          CALL SSPACE(TEXT, L)
          CALL PGMTXT('B', Y,0.0,0.0, TEXT(1:L))
      END IF
*     -- Line 3: page number
      IF (ANNLEV.GE.1) THEN
          Y = Y+DY
          PAGE = PAGE+1
          WRITE (TEXT,'(''Page:'',I6)') PAGE
          CALL SSPACE(TEXT,L)
          CALL PGMTXT('B', Y,0.0,0.0, TEXT(1:L))
      END IF
*     --
      IF (ANNLEV.GE.1) CALL PGIDEN
*
      CALL PGSCH(CH)
      CALL PGSCF(CF)
      CALL PGSLW(LW)
      CALL PGSCI(CI)
      CALL PGEBUF
*
      END

C*ARSORT -- indexed sort of real array into ascending order
C+
      SUBROUTINE ARSORT(A,N,IND)
      INTEGER N
      REAL A(N)
      INTEGER IND(N)
C
C Indexed sort of a real array A into increasing order. Shell sort.
C
C Array IND(N) is sorted so that IND(I) is the index of array A(N).
C-----------------------------------------------------------------------
      INTEGER IGAP, IMAX, I, J, JPLUSG, IEX
C
      IGAP=N
    5 IF(IGAP.LE.1) RETURN
      IGAP=IGAP/2
      IMAX=N-IGAP
   10     IEX=0
          DO 20 I=1,IMAX
          J=IND(I)
          JPLUSG=IND(I+IGAP)
          IF(A(J).GT.A(JPLUSG)) GOTO 40
          GOTO 20
   40     IND(I)=JPLUSG
          IND(I+IGAP)=J
          IEX=IEX+1
   20   CONTINUE
      IF(IEX.GT.0) GOTO 10
      GOTO 5
      END

C*BLPLOT   -- plot one baseline, amp and phase versus UT or GST.
C+
      SUBROUTINE BLPLOT(NP, IB, 
     *                UTMIN, UTMAX, TIME, 
     *                GSTMIN, GSTMAX, GST,
     *                BASE, AMP, PHASE, ERR, MAMP, MPHS,
     *                LEGEND, BNAME, MODEL, AXIS, NDAY, NOERR, DOGST)
      INTEGER NP, IB, BASE(NP), NDAY
      REAL UTMIN, UTMAX, TIME(NP)
      REAL GSTMIN, GSTMAX, GST(NP)
      REAL AMP(NP), PHASE(NP), ERR(NP)
      REAL MAMP(NP), MPHS(NP)
      CHARACTER*(*) LEGEND, BNAME, AXIS
      LOGICAL MODEL, NOERR, DOGST
C-----------------------------------------------------------------------
      INTEGER MAXP
      PARAMETER (MAXP=8000)
      INTEGER N, IP
      REAL X(MAXP), Y1(MAXP), Y2(MAXP), Y3(MAXP), Y4(MAXP)
      REAL Y5(MAXP), Y6(MAXP)
      REAL AMAX, PHSERR
      REAL TMIN, TMAX
      CHARACTER*20 STR
*
      LOGICAL GRID
      REAL U1, U2, V1, V2, ASCALE, PSCALE, CSCALE
      INTEGER DOTSIZ
      COMMON /PLT/ GRID, U1, U2, V1, V2, DOTSIZ, ASCALE, PSCALE, CSCALE
*
* Set up
*
      IF (DOGST) THEN
          TMIN = GSTMIN
          TMAX = GSTMAX
      ELSE
          TMIN = UTMIN
          TMAX = UTMAX
      END IF
*
* Get the data for the requested baseline
*
      N = 0
      AMAX = 0.0
      DO 100 IP=1,NP
          IF (BASE(IP).EQ.IB) THEN
              N = N+1
              IF (N.GT.MAXP) THEN
                  STR = BNAME
                  CALL PUTOUT('Too many data points on baseline '
     :                        //STR)
                  RETURN
              END IF
              IF (DOGST) THEN
                  X(N) = GST(IP)
              ELSE
                  X(N) = TIME(IP)
              END IF
              IF (NOERR) THEN
                  Y1(N) = AMP(IP)
                  AMAX = MAX(AMAX,Y1(N))
                  Y3(N) = PHASE(IP)
              ELSE
                  Y1(N) = AMP(IP)-ERR(IP)
                  Y2(N) = AMP(IP)+ERR(IP)
                  AMAX = MAX(AMAX,Y2(N))
                  PHSERR = 180.0
                  IF (AMP(IP).NE.0.0) 
     :                PHSERR = (ERR(IP)/AMP(IP))*57.295780
                  Y3(N) = PHASE(IP)-PHSERR
                  Y4(N) = PHASE(IP)+PHSERR
              END IF
              Y5(N) = MAMP(IP)
              Y6(N) = MPHS(IP)
          END IF
  100 CONTINUE
      IF (N.LT.1) THEN
          STR = BNAME
          CALL PUTOUT('No data for baseline '//STR)
          RETURN
      END IF
*
* Plot it.
*
      CALL PLOT1(N, X, Y1, Y2, Y3, Y4, Y5, Y6,
     *    TMIN, TMAX, AMAX, NDAY, DOGST, NOERR, MODEL, GRID,
     *    AXIS, LEGEND, BNAME, IB, 'Amplitude (Jy)', 'Phase')
*
      END

C*CLPLOT  -- plot one closure-phase triangle (UT or GST).
C+
      SUBROUTINE CLPLOT(NP, IB1, IB2, IB3, 
     *                UTMIN, UTMAX, TIME,
     *                GSTMIN, GSTMAX, GST, BASE, 
     *                AMP, PHASE, ERR, MAMP, MPHS, 
     *                LEGEND, BNAME, MODEL, AXIS, NDAY, NOERR, DOGST)
      INTEGER NP, IB1,IB2,IB3, BASE(NP), NDAY
      REAL UTMIN, UTMAX, TIME(NP)
      REAL GSTMIN, GSTMAX, GST(NP)
      REAL AMP(NP), PHASE(NP), ERR(NP)
      REAL MAMP(NP), MPHS(NP)
      CHARACTER*(*) LEGEND, BNAME, AXIS
      LOGICAL MODEL, NOERR, DOGST
C-----------------------------------------------------------------------
      INTEGER MAXP
      PARAMETER (MAXP=8000)
      INTEGER N, IP, IP1, IP2, IP3
      REAL X(MAXP), Y1(MAXP), Y2(MAXP), Y3(MAXP), Y4(MAXP)
      REAL Y5(MAXP), Y6(MAXP)
      REAL AMAX, REFTIM, REFUT
      REAL CLA, CLP, CLAERR, CLPERR, PHSE1, PHSE2, PHSE3
      REAL TMIN, TMAX
      CHARACTER*20 STR
*
      LOGICAL GRID
      REAL U1, U2, V1, V2, ASCALE, PSCALE, CSCALE
      INTEGER DOTSIZ
      COMMON /PLT/ GRID, U1, U2, V1, V2, DOTSIZ, ASCALE, PSCALE, CSCALE
*
* Set up
*
      IF (DOGST) THEN
          TMIN = GSTMIN
          TMAX = GSTMAX
      ELSE
          TMIN = UTMIN
          TMAX = UTMAX
      END IF
*
* Get the data for the requested triangle. Data are assumed to be
* in time order.
*
      N = 0
      AMAX = 0.0
      REFTIM = 0.0
      REFUT  = 0.0
      IP1 = 0
      IP2 = 0
      IP3 = 0
      DO 100 IP=1,NP
          IF (GST(IP).NE.REFTIM) THEN
*             -- start of new integration; save result from last
*             -- integration, if any. IP1-IP3 are indices of the
*             -- data for the requested triangle.
              IF (IP1.NE.0 .AND. IP2.NE.0 .AND. IP3.NE.0) THEN
                  N = N+1
                  IF (N.GT.MAXP) THEN
                      STR = BNAME
                      CALL PUTOUT('Too many data points on triangle '
     :                            //STR)
                      RETURN
                  END IF
                  IF (DOGST) THEN
                     X(N) = REFTIM
                  ELSE
                     X(N) = REFUT
                  END IF
*                 -- observed data
                  CLA = AMP(IP1)*AMP(IP2)*AMP(IP3)
                  CLP = MOD(PHASE(IP1)+PHASE(IP2)-PHASE(IP3),360.0)
                  IF (CLP.LE.-180.0) CLP = CLP+360.0
                  IF (CLP.GT.180.0)  CLP = CLP-360.0
                  IF (NOERR) THEN
                      Y1(N) = CLA
                      AMAX = MAX(AMAX,Y1(N))
                      Y3(N) = CLP
                  ELSE
                      PHSE1 = 180.0
                      PHSE2 = 180.0
                      PHSE3 = 180.0
                      IF (AMP(IP1).NE.0.0) PHSE1=(ERR(IP1)/AMP(IP1))**2
                      IF (AMP(IP2).NE.0.0) PHSE2=(ERR(IP2)/AMP(IP2))**2
                      IF (AMP(IP3).NE.0.0) PHSE3=(ERR(IP3)/AMP(IP3))**2
                      CLAERR = CLA*SQRT(PHSE1+PHSE2+PHSE3)
                      CLPERR = 57.295780*SQRT(PHSE1+PHSE2+PHSE3)
                      Y1(N) = CLA-CLAERR
                      Y2(N) = CLA+CLAERR
                      AMAX = MAX(AMAX,Y2(N))
                      Y3(N) = CLP-CLPERR
                      Y4(N) = CLP+CLPERR
                  END IF
*                 -- model data
                  Y5(N) = MAMP(IP1)*MAMP(IP2)*MAMP(IP3)
                  CLP = MOD(MPHS(IP1)+MPHS(IP2)-MPHS(IP3),360.0)
                  IF (CLP.LE.-180.0) CLP = CLP+360.0
                  IF (CLP.GT.180.0)  CLP = CLP-360.0
                  Y6(N) = CLP
              END IF
              REFTIM = GST(IP)
              REFUT  = TIME(IP)
              IP1 = 0
              IP2 = 0
              IP3 = 0
          END IF
          IF (BASE(IP).EQ.IB1) IP1 = IP
          IF (BASE(IP).EQ.IB2) IP2 = IP
          IF (BASE(IP).EQ.IB3) IP3 = IP
  100 CONTINUE
      IF (N.LT.1) THEN
          STR = BNAME
          CALL PUTOUT('No data for triangle '//STR)
          RETURN
      END IF
*
* Plot it.
*
      CALL PLOT1(N, X, Y1, Y2, Y3, Y4, Y5, Y6,
     *    TMIN, TMAX, AMAX, NDAY, DOGST, NOERR, MODEL, GRID,
     *    AXIS, LEGEND, BNAME, 0, 'Bispectrum (Jy\u3\d)',
     *    ' Closure Phase')
*      
      END

C*CMPMOD -- compute model visibilities
C+
      SUBROUTINE CMPMOD(NP, U, V, AMP, PHS)
C     -- Input: number of uv-points
      INTEGER NP
C     -- Input: u,v coordinates (wavelengths)
      REAL U(NP), V(NP)
C     -- Output: visibility amplitude and phase (degrees)
      REAL AMP(NP), PHS(NP)
C-----------------------------------------------------------------------
C     -- Max. no. of model components
      INTEGER MXMOD
      PARAMETER (MXMOD=5000) 
      REAL C1, PI, TWOPI
C     -- C1 = 1 / (4 ln 2)
      PARAMETER (C1=0.360673760)
      PARAMETER (PI=3.141592654)
      PARAMETER (TWOPI=2.0*PI)
C
      INTEGER IP, J
      REAL Q, Q2, UROT, VROT
      REAL REVIS, IMVIS, COMPHA, COMVIS
      REAL BESJ0, BESJ1
      EXTERNAL BESJ0, BESJ1
C
C Model parameters:
C
      REAL
     1  FLUX(MXMOD), XC(MXMOD), YC(MXMOD), AA(MXMOD),
     2  RATIO(MXMOD), CPHI(MXMOD), SPHI(MXMOD)
      INTEGER
     1   TYPE(MXMOD), NCOMP
      LOGICAL
     1   ALLDEL
      COMMON /MODEL/ FLUX, XC, YC, AA, RATIO, CPHI, SPHI,
     1               TYPE, NCOMP, ALLDEL
C-----------------------------------------------------------------------
C
C Compute delta-function model.
C
      IF (ALLDEL) THEN
        DO 110 IP=1,NP
          REVIS = 0.0
          IMVIS = 0.0
          DO 100 J=1,NCOMP
              COMPHA = TWOPI*(U(IP)*XC(J) + V(IP)*YC(J))
              REVIS  = REVIS + FLUX(J)*COS(COMPHA)
              IMVIS  = IMVIS + FLUX(J)*SIN(COMPHA)
  100     CONTINUE
          AMP(IP) = SQRT(REVIS*REVIS + IMVIS*IMVIS)
          IF (AMP(IP).EQ.0.0) THEN
              PHS(IP) = 0.0
          ELSE
              PHS(IP) = ATAN2(IMVIS,REVIS)*(180.0/PI)
          END IF
  110   CONTINUE
C
C Compute general model.
C
      ELSE
        DO 130 IP=1,NP
          REVIS = 0.0
          IMVIS = 0.0
          DO 120 J=1,NCOMP
              COMPHA = TWOPI*(U(IP)*XC(J) + V(IP)*YC(J))
              UROT = (U(IP)*CPHI(J) - V(IP)*SPHI(J))*RATIO(J)
              VROT =  U(IP)*SPHI(J) + V(IP)*CPHI(J)
              IF (AA(J) .EQ. 0.0) THEN
C                 -- delta function
                  COMVIS = 1.0
              ELSE IF (TYPE(J) .EQ. 1) THEN
C                 -- gaussian
                  Q2   = (PI*AA(J))**2 * (UROT**2 + VROT**2)
                  COMVIS = EXP(-C1*Q2)
              ELSE IF (TYPE(J) .EQ. 2) THEN
C                 -- uniform disk
                  Q = (PI*AA(J)) * SQRT(UROT**2 + VROT**2)
                  IF (Q.LT.0.01) THEN
                      Q2 = Q*Q
                      COMVIS = 1.0 - Q2/8.0 + Q2*Q2/192.0
                  ELSE
                      COMVIS = 2.0*BESJ1(Q) / Q
                  END IF
              ELSE IF (TYPE(J) .EQ. 3) THEN
C                 -- optically thin sphere
                  Q = (PI*AA(J)) * SQRT(UROT**2 + VROT**2)
                  IF (Q.LT.0.01) THEN
                      Q2 = Q*Q
                      COMVIS = 1.0 - Q2/10.0 + Q2*Q2/280.0
                  ELSE
                      COMVIS = 3.0*(SIN(Q) - Q*COS(Q))/(Q*Q*Q)
                  END IF
              ELSE IF (TYPE(J) .EQ. 4) THEN
C                 -- ring
                  Q = (PI*AA(J)) * SQRT(UROT**2 + VROT**2)
                  COMVIS = BESJ0(Q)
              ELSE
                  CALL ERROR ('Illegal component type in model')
              ENDIF
              REVIS  = REVIS + FLUX(J)*COMVIS*COS(COMPHA)
              IMVIS  = IMVIS + FLUX(J)*COMVIS*SIN(COMPHA)
  120     CONTINUE
          AMP(IP) = SQRT(REVIS*REVIS + IMVIS*IMVIS)
          IF (AMP(IP).EQ.0.0) THEN
              PHS(IP) = 0.0
          ELSE
              PHS(IP) = ATAN2(IMVIS,REVIS)*(180.0/PI)
          END IF
  130   CONTINUE
      END IF
      END

*COLOR  -- change color scheme
C+
      SUBROUTINE COLOR
C-----------------------------------------------------------------------
      INTEGER NCOLS, NPARS
      PARAMETER (NCOLS=6, NPARS=3*NCOLS)
      INTEGER MODE, I, CI(NCOLS)
      DOUBLE PRECISION RED, GREEN, BLUE
      DOUBLE PRECISION PARS(NPARS), VALS(NPARS), ENDMRK
      CHARACTER*30 TEXT
      SAVE PARS, VALS, ENDMRK
*
      DATA CI/ 0, 1, 3, 2, 5, 14 /
      DATA PARS/3*'Backgrou', 3*'Frame', 3*'Data', 3*'Model',
     *          3*'Highligh', 3*'Grid'/
      DATA VALS/ 3*0D0, 
     *           3*1D0,
     *           0D0, 1D0, 0D0,
     *           1D0, 0D0, 0D0,
     *           0D0, 1D0, 1D0,
     *           3*0.33D0 /
      DATA ENDMRK/'/'/
*
      CALL PUTOUT('Current color representation (R,G,B values):')
      DO 10 I=1,NCOLS
          WRITE (TEXT,'(1X,A8,''='',F6.3,'','',F6.3,'','',F6.3)')
     *       PARS(3*I), VALS(3*I-2), VALS(3*I-1), VALS(3*I)
          CALL PUTOUT(TEXT)
   10 CONTINUE
      CALL PUTOUT('New color representation (end with /):')
      MODE = 0
      CALL KEYIN(PARS,VALS,NPARS,ENDMRK,MODE,5,6)
      DO 20 I=1,NCOLS
          RED   = MIN(1D0,MAX(0D0,VALS(3*I-2)))
          GREEN = MIN(1D0,MAX(0D0,VALS(3*I-1)))
          BLUE  = MIN(1D0,MAX(0D0,VALS(3*I)))
          VALS(3*I-2) = RED
          VALS(3*I-1) = GREEN
          VALS(3*I)   = BLUE
          CALL PGSCR(CI(I), REAL(RED), REAL(GREEN), REAL(BLUE))
   20 CONTINUE
      CALL PGUPDT
      END

C*EDMOD  -- edit model (not yet implemented)
C+
      SUBROUTINE EDMOD
C-----------------------------------------------------------------------
      END

C*GETMEM -- read MEM mock data.
C+
      SUBROUTINE GETMEM(UNIT, NP, MAMP, MPHS)
      INTEGER UNIT,NP
      REAL MAMP(NP), MPHS(NP)
C-----------------------------------------------------------------------
      REAL PI, DEG
      PARAMETER (PI = 3.141592653589793)
      PARAMETER (DEG=180.0/PI)
*
      INTEGER IP
      REAL AMP, PHS
*
* Read data (complex numbers)
*
      READ (UNIT=UNIT) (MAMP(IP),MPHS(IP), IP=1,NP)
*
* Convert to amplitude, phase (degrees) range -180 to +180.
*
      DO 10 IP=1,NP
         AMP = SQRT(MAMP(IP)**2 + MPHS(IP)**2)
         IF (AMP.EQ.0) THEN
             PHS = 0.0
         ELSE
             PHS = ATAN2(MPHS(IP), MAMP(IP)) * DEG
         END IF
         MAMP(IP) = AMP
         MPHS(IP) = PHS
   10 CONTINUE
      END

C*GETMOD -- read model file
C+
      SUBROUTINE GETMOD(INMOD, NMDCMP, NC, TFLUX)
      INTEGER INMOD
      INTEGER NMDCMP
      INTEGER NC
      REAL TFLUX
C
C INMOD: (in) : Fortran unit for input file
C NMDCMP (in) : maximum number of model components to read from file
C NC     (out): actual number of components read
C TFLUX  (out): total flux in NC components
C-----------------------------------------------------------------------
C     -- Max. no. of model components
      INTEGER MXMOD
      PARAMETER (MXMOD=5000) 
C
      REAL PI, RPDEG, RPMAS
      PARAMETER (PI=3.1415927)
      PARAMETER (RPDEG=PI/180.)
      PARAMETER (RPMAS=PI/180./3600./1000.)
C
      INTEGER I, ITYPE
      REAL FLXSUM, RAD, THETA, PHI
      CHARACTER*80 TEXT
      INTEGER LEN1
C
C Model parameters:
C
      REAL
     1  FLUX(MXMOD), XC(MXMOD), YC(MXMOD), AA(MXMOD),
     2  RATIO(MXMOD), CPHI(MXMOD), SPHI(MXMOD)
      INTEGER
     1   TYPE(MXMOD), NCOMP
      LOGICAL
     1   ALLDEL
      COMMON /MODEL/ FLUX, XC, YC, AA, RATIO, CPHI, SPHI,
     1               TYPE, NCOMP, ALLDEL
C-----------------------------------------------------------------------
C
      IF (NMDCMP.GT.MXMOD) NMDCMP = MXMOD
      FLXSUM = 0.0
      ALLDEL = .TRUE.
      DO 150 I=1,NMDCMP
          CALL GTMODL(INMOD, ITYPE, FLUX(I), RAD, THETA,
     1                AA(I), RATIO(I), PHI, *155, 0)
          IF (FLUX(I) .EQ. 0.0) GOTO 155
          FLXSUM = FLXSUM + FLUX(I)
          THETA  = THETA * RPDEG
          XC(I)  = RPMAS*RAD*SIN(THETA)
          YC(I)  = RPMAS*RAD*COS(THETA)
          AA(I)  = AA(I)*RPMAS
          PHI    = PHI*RPDEG
          CPHI(I)= COS(PHI)
          SPHI(I)= SIN(PHI)
          IF (ITYPE .EQ. 0) ITYPE = 1
          IF (ITYPE .LT. 1 .OR. ITYPE .GT. 5) THEN
              WRITE (TEXT,660) ITYPE
              CALL PUTOUT(TEXT(1:LEN1(TEXT)))
              ITYPE = 1
          END IF
          IF (AA(I).EQ. 0.0) ITYPE = 0
          TYPE(I) = ITYPE
          IF (ITYPE.NE.0) ALLDEL = .FALSE.
  150 CONTINUE
      WRITE (TEXT,650) NMDCMP
      CALL PUTOUT(TEXT(1:LEN1(TEXT)))
      I = NMDCMP + 1
  155 NCOMP = I-1
      IF (FLXSUM .EQ. 0.0) CALL PUTOUT('Model has zero total flux')
      NC = NCOMP
      TFLUX = FLXSUM
      RETURN
C
  650 FORMAT ('++WARNING++ Model truncated after ',I5,' components.')
  660 FORMAT ('++WARNING++ Invalid component type for component ',I4,
     1'; Gaussian used')
C
      END

C*GSTRNG -- determine GST range of data.
C+
      SUBROUTINE GSTRNG(NP, GST, GSTMIN, GSTMAX)
      INTEGER NP
      REAL GST(NP), GSTMIN, GSTMAX
C-----------------------------------------------------------------------
      REAL PI, TWOPI
      PARAMETER (PI = 3.141592653589793)
      PARAMETER (TWOPI = PI*2.0)
*
      INTEGER IP
      REAL C, S, T, GST0
*
* GST is a cyclic quantity. Determine mean.
*
      C = 0.0
      S = 0.0
      DO 10 IP=1,NP
          C = C + COS(GST(IP))
          S = S + SIN(GST(IP))
   10 CONTINUE
      GST0 = ATAN2(S, C)
      IF (GST0.LT.0.0) GST0 = GST0+TWOPI
*
* Convert supplied GST's to range (GST0-pi, GST0+pi), convert to
* hours, and find max and min.
*
      GSTMIN = GST0+24.0
      GSTMAX = GST0-24.0
      DO 20 IP=1,NP
         T = MOD(GST(IP) - GST0, TWOPI)
         IF (T.GT.PI) T = T-TWOPI
         IF (T.LT.-PI) T = T+TWOPI
         GST(IP) = (GST0 + T)*(12.0/PI)
         GSTMIN = MIN(GSTMIN,GST(IP))
         GSTMAX = MAX(GSTMAX,GST(IP))
   20 CONTINUE
      WRITE (*,650) GSTMIN, GSTMAX
  650 FORMAT (' GST range of data (hours): ',F6.2,' to ',F6.2)
      GSTMIN = INT(GSTMIN)
      IF (GSTMIN.LT.0.0) GSTMIN = GSTMIN-1
      GSTMAX = INT(GSTMAX)+1
      IF (GSTMAX.LT.0.0) GSTMAX = GSTMAX-1
      END

C*HELP   -- show list of commands
C+
      SUBROUTINE HELP
C-----------------------------------------------------------------------
      CALL PUTOUT(' ')
      CALL PUTOUT('Commands are:')
      CALL PUTOUT('  Next           (next baseline) [default]')
      CALL PUTOUT('  BAck           (previous baseline)')
      CALL PUTOUT('  All            (display all baselines)')
      CALL PUTOUT('  BL  A B        (plot amp/phase for baseline AB)')
      CALL PUTOUT('  CL  A B C      (plot closure for triangle ABC)')
      CALL PUTOUT('            (A,B,C are station names or numbers)')
      CALL PUTOUT('  COlor          (change color scheme)')
      CALL PUTOUT('  EXIT, QUIT     (exit from program)')
      CALL PUTOUT('  GST            (switch to GST)')
      CALL PUTOUT('  Highlight A    (highlight stn A in global plots)')
      CALL PUTOUT('  Highlight A  B (highlight bl AB in global plots)')
      CALL PUTOUT('  List           (list agreement factors)')
      CALL PUTOUT('  MODfile file   (read a new model file)')
      CALL PUTOUT('  PHase          (plot phase versus uv radius)')
      CALL PUTOUT('  PLOTfile file/type (change plotting device)')
      CALL PUTOUT('  PRoj pa        (plot amp projected onto given PA)')
      CALL PUTOUT('  Rad            (plot amp versus uv radius)')
      CALL PUTOUT('  Refant A       (set ref antenna for VISPLOT)')
      CALL PUTOUT('  Stack A ampl/ph (all baselines involving A)')
      CALL PUTOUT('  TImeline       (plot sampling versus time)')
      CALL PUTOUT('  UT             (switch to UT, default)')
      CALL PUTOUT('  UVplot         (plot v versus u)')
      CALL PUTOUT('  UVplot Amp     (color indicates amplitude)')
      CALL PUTOUT('  UVplot Model   (color indicates model amplitude)')
      CALL PUTOUT('  UVplot Ratio   (color indicates ampl/model)')
      CALL PUTOUT('  UVplot Error   (color indicates (AMP-MOD)/error)')
      CALL PUTOUT('  Visplot Ampl|Phas N scale (plot amplitude or phase'
     : //' for all baselines, with N per page')
      CALL PUTOUT('  Visplot Clos N scale (plot all closure phases, '
     : //' with N per page')
      CALL PUTOUT('  ?              (display list of commands)')
      END

C*ISORT -- sort integer array into ascending order
C+
      SUBROUTINE ISORT (RA, N)
      INTEGER N
      INTEGER RA(N)
C
C Sort array RA(1..N) into ascending order.
C-----------------------------------------------------------------------
      INTEGER I, J, L, IR
      INTEGER RRA
C
      L = N/2 + 1
      IR = N
   10 CONTINUE
          IF (L.GT.1) THEN
              L = L-1
              RRA = RA(L)
          ELSE
              RRA = RA(IR)
              RA(IR) = RA(1)
              IR = IR-1
              IF (IR.EQ.1) THEN
                  RA(1) = RRA
                  RETURN
              END IF
          END IF
          I = L
          J = L+L
   20     IF (J.LE.IR) THEN
              IF (J.LT.IR) THEN
                  IF (RA(J).LT.RA(J+1)) J = J+1
              END IF
              IF (RRA.LT.RA(J)) THEN
                  RA(I) = RA(J)
                  I = J
                  J = J+J
              ELSE
                  J = IR+1
              END IF
          GOTO 20
          END IF
          RA(I) = RRA
      GOTO 10
      END

C*MRGRD -- Read Merge file into memory.
C+
      SUBROUTINE MRGRD(INDAT,NP,
     * TIME, BASE, U, V, AMP, PHASE, ERR, GST,
     *  LEGEND, BNAME, BLABEL, TELESC, BCOUNT, KFLAG)
      INTEGER INDAT, NP
      INTEGER BASE(NP)
      REAL TIME(NP), U(NP), V(NP), AMP(NP), PHASE(NP), ERR(NP), GST(NP)
      CHARACTER*(*) LEGEND, BNAME(*), TELESC
      INTEGER BLABEL(*), BCOUNT(*), KFLAG
C
C Arguments:
C  INDAT (input)   : Fortran unit on which the Merge file is open
C                    (use routine VLBOPE to open the file).
C  NP (input)      : number of complex visibilities.
C  TIME
C  BASE
C  U
C  V
C  AMP
C  PHASE
C  ERR
C  GST
C  KFLAG (output) : 0 => correl coeff (x10^5); 1=> Jy
C
C Subroutines required:
C  ERROR   (CITVLB)
C  HEADX40 (CITVLB)
C  LEN1    (CITVLB)
C  STNABR  (CITVLB)
C  TDATE   (CITVLB)
C
C History:
C  1989 Dec 11 - TJP
C  1991 Mar 22 - 40 stations (TJP)
C-----------------------------------------------------------------------
      INTEGER   MAXS,MAXB,MAXT,MAXWPB
      PARAMETER (MAXS=40,MAXB=(MAXS-1)*MAXS/2)
      PARAMETER (MAXT=MAXB*(MAXS-2)/3)
      PARAMETER (MAXWPB=7)
      DOUBLE PRECISION PI,SIDR,VELC,RPDEG,TWOPI
      PARAMETER (PI = 3.141592653589793 D0)
      PARAMETER (TWOPI = 2D0*PI)
      PARAMETER (SIDR = (PI/2592000.0D0)*1.0027379093 D0)
C             (radians of ST per sec/60 of UT)
      PARAMETER (VELC = 2.997925 D8)
      PARAMETER (RPDEG = PI/180D0)
C
      CHARACTER*16 SNAME
      CHARACTER*8  DATOBS,STNAME
      CHARACTER*1  STNABR
      LOGICAL ORBITR, VLA
      INTEGER STATYP, LEN1, IZ, ID, IY, IMON, JD, MONTH, LENG
      INTEGER IS, IB, NS, NB, NT, IFEOF, NC1, NC2, NC3, IS1, IS2, NVER
      INTEGER IP, NDWPB, J, NB1, NB2, IUT, INBUF
      INTEGER L1, L2
      REAL    EPOCH, DUM1, DUM3, AMPMAX, UP, VP, UVP, UVMAX
      REAL    REC(MAXWPB*MAXB)
      DOUBLE PRECISION  STX,STY,STZ,RA,DEC,GASTM,FREQ,FLUX,
     1        WAVE,BX,BY,BZ(MAXB),BOFF(MAXB),BXY(MAXB),TIHA,
     2        COSDEC,SINDEC,TINTG,AVG,BW,AEIOWM, RA0, DEC0
      COMMON /PARMS/ STNAME(MAXS),STX(MAXS),STY(MAXS),STZ(MAXS),
     1        SNAME,RA,DEC,GASTM,FREQ,FLUX,TINTG,AVG,BW,
     2        AEIOWM(6,MAXS),STATYP(MAXS),ORBITR
      COMMON /ORDER/ NS,NB,NT, NB1(MAXB),NB2(MAXB),NC1(MAXT),
     1        NC2(MAXT),NC3(MAXT),NVER,NDWPB
C-----------------------------------------------------------------------
  610 FORMAT( ' Number of u,v points accepted: ',I6
     1       /' UVmax = ',F16.2,' wavelengths'
     2       /' Maximum correlated flux density = ',F9.3,' Jy')
  630 FORMAT(/' Source: ',A
     1       /' Frequency: ',F10.3,' GHz'
     2       /' Array: ',A
     3       /' Observation date: ',A,F10.2
     4       /' Stations: ',7A9/(11X,7A9))
  631 FORMAT(A,1X,F10.3,' GHz',F9.2)
  640 FORMAT(I2.2,'/',I2.2,'/',I2.2)
C-----------------------------------------------------------------------
C
      CALL HEADX40(.FALSE.,INDAT,IZ,ID,IY,DUM1,0,0,KFLAG,DUM3,IFEOF,0)
      IF (NS.LT.2 .OR. NB.LT.1) CALL ERROR('Not enough stations.')
      IF (NS.GT.MAXS) CALL ERROR('Too many stations.')
      LENG = NDWPB*NB
      RA0 = RA
      DEC0 = DEC
      TELESC = ' '
      VLA = .TRUE.
      DO 100 IS=1,NS
          TELESC(IS:IS) = STNABR(STNAME(IS))
          IF (TELESC(IS:IS).NE.'Y') VLA = .FALSE.
  100 CONTINUE
      IF (VLA) TELESC = 'VLA'
      DO 110 IB=1,NB
          IS1 = NB1(IB)
          IS2 = NB2(IB)
          L1 = LEN1(STNAME(IS1))
          L2 = LEN1(STNAME(IS2))
          BNAME(IB) = STNAME(IS1)(1:L1)//'-'//STNAME(IS2)(1:L2)
          BLABEL(IB) = 100*IS1+IS2
          BCOUNT(IB) = 0
  110 CONTINUE
      IMON = 1
      CALL TDATE(IY,IMON,ID,JD,MONTH,DUM1)
      WRITE (DATOBS,640) ID,IMON,MOD(IY,100)
      EPOCH = 2000.0 + ((JD-2451545)-0.5D0)/365.25D0
      WRITE (*,630) SNAME, FREQ*1E-9, TELESC(1:NS), DATOBS, EPOCH,
     1                 (STNAME(J),J=1,NS)
      WRITE (LEGEND,631) SNAME(1:LEN1(SNAME)), FREQ*1E-9, EPOCH
C 
C Baseline parameters.
C 
      WAVE = VELC/FREQ
      COSDEC = COS(DEC)
      SINDEC = SIN(DEC)
      DO 210 IB=1,NB
      IF (NVER.EQ.1) THEN
            IS1 = NB1(IB)
            IS2 = NB2(IB)
            BX = (STX(IS1)-STX(IS2))/WAVE
            BY = (STY(IS1)-STY(IS2))/WAVE
            BZ(IB) = (STZ(IS1)-STZ(IS2))/WAVE
            BXY(IB) = SQRT(BX*BX+BY*BY)
            IF (BXY(IB).EQ.0D0)
     1              CALL ERROR('Station coordinates missing.')
            BOFF(IB) = GASTM-RA-PI/2D0-ATAN2(BY,BX)
      ELSE
            BZ(IB)   = 0.D0
            BXY(IB)  = 0.D0
            BOFF(IB) = 0.D0
      END IF
  210 CONTINUE
C 
C Determine all (u,v) values and complex visibilities.
C 
      AMPMAX = 0.0
      IP = 0
      INBUF = 0
C 
C Loop through data records.
C
  300 READ (INDAT,END=400) IUT,(REC(J),J=1,LENG)
      IF (IUT.LT.0) GOTO 400
C         -- Loop through baselines
          DO 320 IB=1,NB
C             -- Skip deleted data
              IF (REC(NDWPB*(IB-1)+2).LE.0.0 .OR.
     1            REC(NDWPB*(IB-1)+4).LE.0.0) GOTO 320
C             -- Count the data point
              IP = IP+1
              IF (IP.GT.NP) CALL ERROR('Fatal error in MRGRD')
C             -- Compute (u,v) if necessary
              IF (NVER.EQ.1) THEN
                  TIHA = (IUT-IZ)*SIDR + BOFF(IB)
                  UP = REAL(BXY(IB)*COS(TIHA))
                  VP = REAL(BZ(IB)*COSDEC + BXY(IB)*SINDEC*SIN(TIHA))
              ELSE IF (NVER.EQ.2) THEN
                  UP = REC(NDWPB*(IB-1)+5)
                  VP = REC(NDWPB*(IB-1)+6)
              END IF
              UVP       = UP*UP + VP*VP
              UVMAX     = MAX(UVMAX,UVP)
              BCOUNT(IB) = BCOUNT(IB)+1
C             -- Compute complex visibility and error
C                Units of TIME are days (Jan 1 = day 1)
C                Units of GST are radians
              TIME(IP)  = 1.0 + REAL(IUT)/(24.0*60.0*60.0*60.0)
              GST(IP)   = MOD((IUT-IZ)*SIDR + GASTM,TWOPI)
              BASE(IP)  = IB
              U(IP)     = UP
              V(IP)     = VP
              AMP(IP)   = REC(NDWPB*(IB-1)+1)
              PHASE(IP) = MOD(REC(NDWPB*(IB-1)+3),360.0)
              IF (PHASE(IP).LT.-180.0) PHASE(IP) = PHASE(IP)+360.0
              IF (PHASE(IP).GT. 180.0) PHASE(IP) = PHASE(IP)-360.0
              ERR(IP)   = REC(NDWPB*(IB-1)+2)
              IF (KFLAG.EQ.0) THEN
                  AMP(IP) = AMP(IP)*1E5
                  ERR(IP) = ERR(IP)*1E5
              END IF
              AMPMAX    = MAX(AMPMAX,AMP(IP))
  320     CONTINUE
      GOTO 300
C
C (end loop)
C 
  400 WRITE (*,610) IP, SQRT(UVMAX), AMPMAX
C
      END

C*PAPLOT   -- plot amplitude versus projection in uv plane
*
      SUBROUTINE PAPLOT(NP, LEGEND, TELESC, BASE, U, V, AMP, MAMP, 
     *                 WORK, MODEL, PA, HLB)
      INTEGER NP, HLB, BASE(NP)
      REAL U(NP), V(NP), AMP(NP), MAMP(NP), WORK(NP), PA
      CHARACTER*(*) LEGEND, TELESC
      LOGICAL MODEL
*
      CHARACTER*26 HBNAME
      COMMON /CPLT/ HBNAME
      INTEGER BLABEL(780)
      COMMON /LABEL/ BLABEL
      LOGICAL GRID
      REAL U1, U2, V1, V2, ASCALE, PSCALE, CSCALE
      INTEGER DOTSIZ
      COMMON /PLT/ GRID, U1, U2, V1, V2, DOTSIZ, ASCALE, PSCALE, CSCALE
*
      REAL AMAX, UMAX, D, VPX1, VPX2, VPY1, VPY2, COSA, SINA
      INTEGER IP, IS1, IS2, SAVELW
      CHARACTER*80 TXT
*
      REAL PI, RPDEG
      PARAMETER (PI = 3.141592653589793)
      PARAMETER (RPDEG = PI/180.0)
*
* Compute the uv projections; find the maximum baseline.
*
      UMAX = 0.0
      COSA = COS(PA*RPDEG)
      SINA = SIN(PA*RPDEG)
      DO 40 IP=1,NP
          WORK(IP) = ABS(U(IP)*SINA + V(IP)*COSA)
          UMAX = MAX(UMAX,WORK(IP))
   40 CONTINUE
      UMAX = 1.1*UMAX
*
* Find the maximum amplitude.
*
      AMAX = 0.0
      DO 50 IP=1,NP
          AMAX = MAX(AMAX,AMP(IP),MAMP(IP))
   50 CONTINUE
      AMAX = 1.2*AMAX
*
* Set up the viewport and window
*
      CALL PGBBUF
      CALL PGPAGE
      CALL PGSVP(0.0, 1.0, 0.0, 1.0)
      CALL PGQVP(1, VPX1, VPX2, VPY1, VPY2)
      D = MIN(VPX2-VPX1, VPY2-VPY1)/40.0
      VPX1 = VPX1 + 5.0*D
      VPX2 = VPX2 - 2.0*D
      VPY1 = VPY1 + 9.0*D
      VPY2 = VPY2 - 4.0*D
      CALL PGVSIZ(VPX1, VPX2, VPY1, VPY2)
      CALL PGSWIN(0.0,UMAX,0.0,AMAX)
      CALL PGMTXT('T',2.5,0.0,0.0,LEGEND)
      CALL PGMTXT('T',0.8,0.0,0.0,TELESC)
      IF (GRID) THEN
          CALL PGSCI(14)
          CALL PGBOX('G', 0.0, 0, 'G', 0.0, 0)
      END IF
      CALL PGSCI(1)
      CALL PGBOX('BCNST', 0.0, 0, 'BCNSTV', 0.0, 0)
      WRITE (TXT, '(A,I4,A)') 'Baseline projected onto PA',
     *                        NINT(PA), '^  (wavelengths)'
      CALL PGMTXT('B', 3.2, 0.5, 0.5, TXT)
      CALL PGMTXT('L', 3.2, 0.5, 0.5, 'Amplitude (Jy)')
*
* Plot data
*
      CALL PGQLW(SAVELW)
      CALL PGSLW(DOTSIZ)
      CALL PGSCI(3)
      CALL PGPT(NP, WORK, AMP, -1)
      IF (MODEL) THEN
          CALL PGSCI(2)
          CALL PGPT(NP, WORK, MAMP, -1)
      END IF
*
* Highlight selected data
*
      IF (HLB.GT.0) THEN
          CALL PGSCI(5)
          DO 300 IP=1,NP
              IF (BASE(IP).EQ.HLB) 
     *            CALL PGPT(1, WORK(IP), AMP(IP), -1)
  300     CONTINUE
          CALL PGMTXT('T', 0.8, 1.0, 1.0, HBNAME)
      ELSE IF (HLB.LT.0) THEN
          CALL PGSCI(5)
          DO 310 IP=1,NP
              IS1 = BLABEL(BASE(IP))/100
              IS2 = BLABEL(BASE(IP))-100*IS1
              IF ((-IS1.EQ.HLB) .OR. (-IS2.EQ.HLB)) 
     *            CALL PGPT(1, WORK(IP), AMP(IP), -1)
  310     CONTINUE
          CALL PGMTXT('T', 0.8, 1.0, 1.0, HBNAME)
      END IF
*
      CALL PGSCI(1)
      CALL PGSLW(SAVELW)
      CALL PGEBUF
*      
      END

C*PLOT1  -- plotting routine used by BLPLOT and CLPLOT
C+
      SUBROUTINE PLOT1(N, X, Y1, Y2, Y3, Y4, Y5, Y6,
     *    TMIN, TMAX, AMAX, NDAY, DOGST, NOERR, MODEL, GRID,
     *    AXIS, LEGEND, TLEFT, IB, ALABEL, PLABEL)
      INTEGER N, NDAY, IB
      REAL X(N), Y1(N), Y2(N), Y3(N), Y4(N), Y5(N), Y6(N)
      REAL TMIN, TMAX, AMAX
      LOGICAL DOGST, NOERR, MODEL, GRID
      CHARACTER*(*) AXIS, LEGEND, TLEFT, ALABEL, PLABEL
C-----------------------------------------------------------------------
      REAL D, VPX1, VPX2, VPY1, VPY2, YLO, YHI
      INTEGER I
      CHARACTER*20 STR
*
* Set up the amplitude viewport and window
*
      CALL PGBBUF
      CALL PGPAGE
      CALL PGSVP(0.0, 1.0, 0.0, 1.0)
      CALL PGQVP(1, VPX1, VPX2, VPY1, VPY2)
      D = MIN(VPX2-VPX1, VPY2-VPY1)/40.0
      VPX1 = VPX1 + 5.0*D
      VPX2 = VPX2 - 2.0*D
      VPY1 = VPY1 + 9.0*D
      VPY2 = VPY2 - 4.0*D
      CALL PGVSIZ(VPX1, VPX2, VPY1+0.5*(VPY2-VPY1), VPY2)
      CALL PGRNGE(0.0, AMAX, YLO, YHI)
      CALL PGSWIN(TMIN,TMAX,YLO,YHI)
      CALL PGMTXT('T',2.5,0.0,0.0,LEGEND)
      CALL PGMTXT('T',0.8,0.0,0.0,TLEFT)
      IF (IB.NE.0) THEN
          WRITE (STR,'(I8)') IB
          CALL PGMTXT('T',0.8,1.0,1.0,STR)
      END IF
      IF (DOGST .OR. AXIS.EQ.'D') THEN
          IF (GRID) THEN
              CALL PGSCI(14)
              CALL PGBOX('G', 0.0, 0, 'G', 0.0, 0)
              CALL PGSCI(1)
          END IF
          CALL PGBOX('BCST', 0.0, 0, 'BCNSTV', 0.0, 0)
      ELSE IF (AXIS.EQ.'H') THEN
          IF (GRID) THEN
              CALL PGSCI(14)
              CALL PGTBOX('GZ', 0.0, 0, 'G', 0.0, 0)
              CALL PGSCI(1)
          END IF
          CALL PGTBOX('BCSTZYH', 0.0, 0, 'BCNSTV', 0.0, 0)
      END IF
      CALL PGMTXT('L', 4.0, 0.5, 0.5, ALABEL)
*
* Plot amplitude data
*
      CALL PGSCI(3)
      IF (NOERR) THEN
          CALL PGPT(N, X, Y1, -1)
      ELSE
          CALL PGERRY(N, X, Y1, Y2, 0.0)
      END IF
      IF (MODEL) THEN
          CALL PGSCI(2)
          IF (DOGST) THEN
              CALL PGMOVE(X(1),Y5(1))
              DO 15 I=2,N
                  IF (X(I).LT.X(I-1)) THEN
                      CALL PGMOVE(X(I),Y5(I))
                  ELSE
                      CALL PGDRAW(X(I),Y5(I))
                  END IF
   15         CONTINUE
          ELSE
              CALL PGLINE(N, X, Y5)
          END IF
      END IF
      CALL PGSCI(1)
*
* Set up the phase viewport and window
*
      CALL PGVSIZ(VPX1, VPX2, VPY1, VPY2-0.5*(VPY2-VPY1))
      CALL PGSWIN(TMIN,TMAX,-180.0,180.0)
      IF (DOGST .OR. AXIS.EQ.'D') THEN
          IF (GRID) THEN
              CALL PGSCI(14)
              CALL PGBOX('G', 0.0, 0, 'G', 90.0, 3)
              CALL PGSCI(1)
          END IF
          CALL PGBOX('BCNST', 0.0, 0, 'BCNSTV', 90.0, 3)
          IF (DOGST) THEN
              CALL PGMTXT('B', 3.2, 0.5, 0.5, 'GST (hours)')
          ELSE
              CALL PGMTXT('B', 3.2, 0.5, 0.5, 'UT (day of year)')
          END IF
      ELSE IF (AXIS.EQ.'H') THEN
          IF (GRID) THEN
              CALL PGSCI(14)
              CALL PGTBOX('GZ', 0.0, 0, 'G', 90.0, 3)
              CALL PGSCI(1)
          END IF
          CALL PGTBOX('BCNSTZYH', 0.0, 0, 'BCNSTV', 90.0, 3)
          WRITE (STR,'(''UT on day '',I3)') NDAY
          CALL PGMTXT('B', 3.2, 0.5, 0.5, STR)
      END IF
      CALL PGMTXT('L', 4.0, 0.5, 0.5, PLABEL)
      IF (.NOT.GRID) THEN
          CALL PGMOVE(TMIN, 0.0)
          CALL PGDRAW(TMAX, 0.0)
      END IF
*
* Plot phase data, taking care of phase wrapping
*
      CALL PGSCI(3)
      IF (NOERR) THEN
          CALL PGPT(N, X, Y3, -1)
      ELSE
          CALL PGERRY(N, X, Y3, Y4, 0.0)
      END IF
      IF (MODEL) THEN
          CALL PGSCI(2)
          CALL PGMOVE(X(1),Y6(1))
          DO 150 I=2,N
              IF (X(I).LT.X(I-1)) THEN
                  CALL PGMOVE(X(I),Y6(I))
              ELSE IF (ABS(Y6(I)-Y6(I-1)).LE.180.0) THEN
                  CALL PGDRAW(X(I),Y6(I))
              ELSE IF (Y6(I)-Y6(I-1).GT.180.0) THEN
                  CALL PGDRAW(X(I),Y6(I)-360.0)
                  CALL PGMOVE(X(I-1),Y6(I-1)+360.0)
                  CALL PGDRAW(X(I),Y6(I))
              ELSE
                  CALL PGDRAW(X(I),Y6(I)+360.0)
                  CALL PGMOVE(X(I-1),Y6(I-1)-360.0)
                  CALL PGDRAW(X(I),Y6(I))
              END IF
  150     CONTINUE
      END IF
      CALL PGSCI(1)
      CALL PGEBUF
*
      END

C*PPLOT  -- plot phase versus uv radius.
C+
      SUBROUTINE PPLOT(NP, LEGEND, TELESC, BASE, U, V, PHS, MPHS, WORK,
     *                 MODEL, HLB)
      INTEGER NP, HLB, BASE(NP)
      REAL U(NP), V(NP), PHS(NP), MPHS(NP), WORK(NP)
      CHARACTER*(*) LEGEND, TELESC
      LOGICAL MODEL
C-----------------------------------------------------------------------
      CHARACTER*26 HBNAME
      COMMON /CPLT/ HBNAME
      INTEGER BLABEL(780)
      COMMON /LABEL/ BLABEL
      LOGICAL GRID
      REAL U1, U2, V1, V2, ASCALE, PSCALE, CSCALE
      INTEGER DOTSIZ
      COMMON /PLT/ GRID, U1, U2, V1, V2, DOTSIZ, ASCALE, PSCALE, CSCALE
*
      REAL UMAX, D, VPX1, VPX2, VPY1, VPY2
      INTEGER IP, IS1, IS2, SAVELW
*
* Compute the uv radii.
* Find the maximum baseline.
*
      UMAX = 0.0
      DO 40 IP=1,NP
          WORK(IP) = SQRT(U(IP)**2 + V(IP)**2)
          UMAX = MAX(UMAX,WORK(IP))
   40 CONTINUE
      UMAX = 1.1*UMAX
*
* Set up the viewport and window
*
      CALL PGBBUF
      CALL PGPAGE
      CALL PGSVP(0.0, 1.0, 0.0, 1.0)
      CALL PGQVP(1, VPX1, VPX2, VPY1, VPY2)
      D = MIN(VPX2-VPX1, VPY2-VPY1)/40.0
      VPX1 = VPX1 + 5.0*D
      VPX2 = VPX2 - 2.0*D
      VPY1 = VPY1 + 9.0*D
      VPY2 = VPY2 - 4.0*D
      CALL PGVSIZ(VPX1, VPX2, VPY1, VPY2)
      CALL PGSWIN(0.0,UMAX,-180.0,180.0)
      CALL PGMTXT('T',2.5,0.0,0.0,LEGEND)
      CALL PGMTXT('T',0.8,0.0,0.0,TELESC)
      IF (GRID) THEN
          CALL PGSCI(14)
          CALL PGBOX('G', 0.0, 0, 'G', 60.0, 3)
      END IF
      CALL PGSCI(1)
      CALL PGBOX('BCNST', 0.0, 0, 'BCNSTV', 60.0, 3)
      CALL PGMTXT('B', 3.2, 0.5, 0.5, 'uv radius (wavelengths)')
      CALL PGMTXT('L', 3.2, 0.5, 0.5, 'Phase')
*
* Plot data
*
      CALL PGQLW(SAVELW)
      CALL PGSLW(DOTSIZ)
      CALL PGSCI(3)
      CALL PGPT(NP, WORK, PHS, -1)
      IF (MODEL) THEN
          CALL PGSCI(2)
          CALL PGPT(NP, WORK, MPHS, -1)
      END IF
*
* Highlight selected data
*
      IF (HLB.GT.0) THEN
          CALL PGSCI(5)
          DO 300 IP=1,NP
              IF (BASE(IP).EQ.HLB) 
     *            CALL PGPT(1, WORK(IP), PHS(IP), -1)
  300     CONTINUE
          CALL PGMTXT('T', 0.8, 1.0, 1.0, HBNAME)
      ELSE IF (HLB.LT.0) THEN
          CALL PGSCI(5)
          DO 310 IP=1,NP
              IS1 = BLABEL(BASE(IP))/100
              IS2 = BLABEL(BASE(IP))-100*IS1
              IF ((-IS1.EQ.HLB) .OR. (-IS2.EQ.HLB)) 
     *            CALL PGPT(1, WORK(IP), PHS(IP), -1)
  310     CONTINUE
          CALL PGMTXT('T', 0.8, 1.0, 1.0, HBNAME)
      END IF
*
      CALL PGSCI(1)
      CALL PGSLW(SAVELW)
      CALL PGEBUF
*      
      END

      SUBROUTINE QERROR(N,AMP,MOD,ERR,RES)
      INTEGER N
      REAL AMP(N), MOD(N), ERR(N), RES(N)
*-----------------------------------------------------------------------
* Absolute value of AMP minus MOD, in units of error, clipped at 3.0
*-----------------------------------------------------------------------
      INTEGER I
*
      DO 10 I=1,N
          RES(I) = MIN(3.0, ABS(AMP(I)-MOD(I))/ERR(I))
   10 CONTINUE
*
      END

      SUBROUTINE RATIO(N,TOP,BOT,RES)
      INTEGER N
      REAL TOP(N), BOT(N), RES(N)
*-----------------------------------------------------------------------
* Ratio of two arrays
*-----------------------------------------------------------------------
      INTEGER I
*
      DO 10 I=1,N
          RES(I) = TOP(I)/BOT(I)
   10 CONTINUE
*
      END

C*RPLOT   -- plot amplitude versus uv radius
*
      SUBROUTINE RPLOT(NP, LEGEND, TELESC, BASE, U, V, AMP, MAMP, 
     *                 WORK, MODEL, HLB)
      INTEGER NP, HLB, BASE(NP)
      REAL U(NP), V(NP), AMP(NP), MAMP(NP), WORK(NP)
      CHARACTER*(*) LEGEND, TELESC
      LOGICAL MODEL
*
      CHARACTER*26 HBNAME
      COMMON /CPLT/ HBNAME
      INTEGER BLABEL(780)
      COMMON /LABEL/ BLABEL
      LOGICAL GRID
      REAL U1, U2, V1, V2, ASCALE, PSCALE, CSCALE
      INTEGER DOTSIZ
      COMMON /PLT/ GRID, U1, U2, V1, V2, DOTSIZ, ASCALE, PSCALE, CSCALE
*
      REAL AMAX, UMAX, D, VPX1, VPX2, VPY1, VPY2
      INTEGER IP, IS1, IS2, SAVELW
*
* Compute the uv radii.
* Find the maximum baseline.
*
      UMAX = 0.0
      DO 40 IP=1,NP
          WORK(IP) = SQRT(U(IP)**2 + V(IP)**2)
          UMAX = MAX(UMAX,WORK(IP))
   40 CONTINUE
      UMAX = 1.1*UMAX
*
* Find the maximum amplitude.
*
      AMAX = 0.0
      DO 50 IP=1,NP
          AMAX = MAX(AMAX,AMP(IP),MAMP(IP))
   50 CONTINUE
      AMAX = 1.2*AMAX
*
* Set up the viewport and window
*
      CALL PGBBUF
      CALL PGPAGE
      CALL PGSVP(0.0, 1.0, 0.0, 1.0)
      CALL PGQVP(1, VPX1, VPX2, VPY1, VPY2)
      D = MIN(VPX2-VPX1, VPY2-VPY1)/40.0
      VPX1 = VPX1 + 5.0*D
      VPX2 = VPX2 - 2.0*D
      VPY1 = VPY1 + 9.0*D
      VPY2 = VPY2 - 4.0*D
      CALL PGVSIZ(VPX1, VPX2, VPY1, VPY2)
      CALL PGSWIN(0.0,UMAX,0.0,AMAX)
      CALL PGMTXT('T',2.5,0.0,0.0,LEGEND)
      CALL PGMTXT('T',0.8,0.0,0.0,TELESC)
      IF (GRID) THEN
          CALL PGSCI(14)
          CALL PGBOX('G', 0.0, 0, 'G', 0.0, 0)
      END IF
      CALL PGSCI(1)
      CALL PGBOX('BCNST', 0.0, 0, 'BCNSTV', 0.0, 0)
      CALL PGMTXT('B', 3.2, 0.5, 0.5, 'uv radius (wavelengths)')
      CALL PGMTXT('L', 3.2, 0.5, 0.5, 'Amplitude (Jy)')
*
* Plot data
*
      CALL PGQLW(SAVELW)
      CALL PGSLW(DOTSIZ)
      CALL PGSCI(3)
      CALL PGPT(NP, WORK, AMP, -1)
      IF (MODEL) THEN
          CALL PGSCI(2)
          CALL PGPT(NP, WORK, MAMP, -1)
      END IF
*
* Highlight selected data
*
      IF (HLB.GT.0) THEN
          CALL PGSCI(5)
          DO 300 IP=1,NP
              IF (BASE(IP).EQ.HLB) 
     *            CALL PGPT(1, WORK(IP), AMP(IP), -1)
  300     CONTINUE
          CALL PGMTXT('T', 0.8, 1.0, 1.0, HBNAME)
      ELSE IF (HLB.LT.0) THEN
          CALL PGSCI(5)
          DO 310 IP=1,NP
              IS1 = BLABEL(BASE(IP))/100
              IS2 = BLABEL(BASE(IP))-100*IS1
              IF ((-IS1.EQ.HLB) .OR. (-IS2.EQ.HLB)) 
     *            CALL PGPT(1, WORK(IP), AMP(IP), -1)
  310     CONTINUE
          CALL PGMTXT('T', 0.8, 1.0, 1.0, HBNAME)
      END IF
*
      CALL PGSCI(1)
      CALL PGSLW(SAVELW)
      CALL PGEBUF
*      
      END

C*TIPLOT -- plot time coverage by station
C+
      SUBROUTINE TIPLOT(NS, NP, LEGEND, TELESC, STNAME, BLABEL,
     *       DOGST, GSTMIN, GSTMAX, UTMIN, UTMAX,
     *       AXIS, NDAY, 
     *       TIME, BASE, GST)
      INTEGER NS, NP, BASE(NP), NDAY, BLABEL(780)
      LOGICAL DOGST
      REAL GSTMIN, GSTMAX, UTMIN, UTMAX
      REAL GST(NP), TIME(NP)
      CHARACTER*(*) LEGEND, TELESC, AXIS, STNAME(40)
C-----------------------------------------------------------------------
      REAL D, VPX1, VPX2, VPY1, VPY2, YLO, YHI, X, REFTIM, REFGST
      INTEGER IP, IS1, IS2, IS
      LOGICAL S(40)
      CHARACTER*80 STR
*
      CHARACTER*26 HBNAME
      COMMON /CPLT/ HBNAME
      LOGICAL GRID
      REAL U1, U2, V1, V2, ASCALE, PSCALE, CSCALE
      INTEGER DOTSIZ
      COMMON /PLT/ GRID, U1, U2, V1, V2, DOTSIZ, ASCALE, PSCALE, CSCALE
*
* Set up the viewport and window
*
      CALL PGBBUF
      CALL PGPAGE
      CALL PGSVP(0.0, 1.0, 0.0, 1.0)
      CALL PGQVP(1, VPX1, VPX2, VPY1, VPY2)
      D = MIN(VPX2-VPX1, VPY2-VPY1)/40.0
      VPX1 = VPX1 + 5.0*D
      VPX2 = VPX2 - 2.0*D
      VPY1 = VPY1 + 9.0*D
      VPY2 = VPY2 - 4.0*D
      CALL PGVSIZ(VPX1, VPX2, VPY1, VPY2)
      YLO = NS+1
      YHI = 0
*
      IF (DOGST) THEN
*         -- abscissa is GST
          CALL PGSWIN(GSTMIN,GSTMAX,YLO,YHI)
          X = GSTMIN-0.01*(GSTMAX-GSTMIN)
          IF (GRID) THEN
              CALL PGSCI(14)
              CALL PGBOX('G', 0.0, 0, ' ', 0.0, 0)
          END IF
          CALL PGSCI(1)
          CALL PGBOX('BCNST', 0.0, 0, 'BCTV', 1.0, 1)
          CALL PGMTXT('B', 3.2, 0.5, 0.5, 'GST (hours)')
      ELSE IF (AXIS.EQ.'D') THEN
*         -- abscissa is UT (days)
          CALL PGSWIN(UTMIN,UTMAX,YLO,YHI)
          X = UTMIN - 0.01*(UTMAX-UTMIN)
          IF (GRID) THEN
              CALL PGSCI(14)
              CALL PGBOX('G', 0.0, 0, ' ', 0.0, 0)
          END IF
          CALL PGSCI(1)
          CALL PGBOX('BCNST', 0.0, 0, 'BCTV', 1.0, 1)
          CALL PGMTXT('B', 3.2, 0.5, 0.5, 'UT (day of year)')
      ELSE IF (AXIS.EQ.'H') THEN
*         -- abscissa is UT (hours)
          CALL PGSWIN(UTMIN,UTMAX,YLO,YHI)
          X = UTMIN - 0.01*(UTMAX-UTMIN)
          IF (GRID) THEN
              CALL PGSCI(14)
              CALL PGTBOX('GZ', 0.0, 0, ' ', 0.0, 0)
          END IF
          CALL PGSCI(1)
          CALL PGTBOX('BCNSTZYH', 0.0, 0, 'BCTV', 1.0, 1)
          WRITE (STR,'(''UT on day '',I3)') NDAY
          CALL PGMTXT('B', 3.2, 0.5, 0.5, STR)
      END IF
*
* Annotation
*
      CALL PGMTXT('T',2.5,0.0,0.0,LEGEND)
      CALL PGMTXT('T',0.8,0.0,0.0,TELESC)
      CALL PGSCH(0.6)
      DO 50 IS=1,NS
          CALL PGPTXT(X, REAL(IS), 0.0, 1.0, STNAME(IS))
   50 CONTINUE
      CALL PGSCH(1.0)
*
* Plot data
*
      CALL PGSCI(3)
      REFTIM = 0.0
      DO 10 IS=1,NS
          S(IS) = .FALSE.
   10 CONTINUE
      DO 100 IP=1,NP
          IF (TIME(IP).NE.REFTIM) THEN
C             -- plot
              IF (DOGST) THEN
                  DO 20 IS=1,NS
                      IF(S(IS)) CALL PGPT(1, REFGST, REAL(IS), -1)
   20             CONTINUE
              ELSE
                  DO 30 IS=1,NS
                      IF(S(IS)) CALL PGPT(1, REFTIM, REAL(IS), -1)
   30             CONTINUE
              END IF
              DO 40 IS=1,NS
                 S(IS) = .FALSE.
   40         CONTINUE
              REFTIM = TIME(IP)
              REFGST = GST(IP)
          END IF
          IS1 = BLABEL(BASE(IP))/100
          IS2 = MOD(BLABEL(BASE(IP)),100)
          S(IS1) = .TRUE.
          S(IS2) = .TRUE.
  100 CONTINUE
      IF (DOGST) THEN
          DO 120 IS=1,NS
              IF(S(IS)) CALL PGPT(1, REFGST, REAL(IS), -1)
  120     CONTINUE
      ELSE
          DO 130 IS=1,NS
              IF(S(IS)) CALL PGPT(1, REFTIM, REAL(IS), -1)
  130     CONTINUE
      END IF
*
      CALL PGSCI(1)
      CALL PGEBUF
*      
      END

C*UTRNGE -- determine UT range of data.
*
      SUBROUTINE UTRNGE(NP, TIME, UTMIN, UTMAX, AXIS, NDAY)
      INTEGER NP
      REAL TIME(NP), UTMIN, UTMAX
      CHARACTER*(*) AXIS
      INTEGER NDAY
*
      INTEGER I,I1,I2,D1,D2, ND
*
      UTMIN = TIME(1)
      UTMAX = TIME(1)
      DO 10 I=2,NP
          UTMIN = MIN(UTMIN, TIME(I))
          UTMAX = MAX(UTMAX, TIME(I))
   10 CONTINUE
      I1 = INT(UTMIN*24.0)
      I2 = INT(UTMAX*24.0)
      IF (REAL(I2).LT.(UTMAX*24.0)) I2 = I2+1
      UTMIN = REAL(I1)/24.0
      UTMAX = REAL(I2)/24.0
      D1 = I1/24
      I1 = I1 - 24*D1
      D2 = I2/24
      I2 = I2 - 24*D2
      WRITE (*,650) D1,I1, D2, I2
  650 FORMAT (' UT range of data: ',I3.3,'/',I2.2, '0000 to ',
     *        I3.3,'/',I2.2, '0000')
      IF (UTMAX-UTMIN.LT.1.0) THEN
          AXIS = 'H'
          ND = UTMIN
          UTMIN = (UTMIN - REAL(ND))*24.0*3600.0
          UTMAX = (UTMAX - REAL(ND))*24.0*3600.0
          DO 20 I=1,NP
              TIME(I) = (TIME(I) - REAL(ND))*24.0*3600.0
   20     CONTINUE
          NDAY = ND
      ELSE
          AXIS = 'D'
          NDAY = 0
      END IF
      END

C*UVPLOT -- plot uv coverage
C+
      SUBROUTINE UVPLOT (NP, LEGEND, TELESC, U, V, BASE, HLB)
      INTEGER NP, HLB, BASE(NP)
      REAL U(NP), V(NP)
      CHARACTER*(*) LEGEND, TELESC
C-----------------------------------------------------------------------
*
      CHARACTER*26 HBNAME
      COMMON /CPLT/ HBNAME
      INTEGER BLABEL(780)
      COMMON /LABEL/ BLABEL
      LOGICAL GRID
      REAL U1, U2, V1, V2, ASCALE, PSCALE, CSCALE
      INTEGER DOTSIZ
      COMMON /PLT/ GRID, U1, U2, V1, V2, DOTSIZ, ASCALE, PSCALE, CSCALE
*
      REAL UMAX, D, VPX1, VPX2, VPY1, VPY2
      REAL UW1, UW2, VW1, VW2
      INTEGER IP, IS1, IS2, SAVELW
*
* Find the maximum baseline.
*
      UMAX = 0.0
      DO 100 IP=1,NP
          UMAX = MAX(UMAX,ABS(U(IP)),ABS(V(IP)))
  100 CONTINUE
      UMAX = 1.2*UMAX
      UW1 = -UMAX
      UW2 = UMAX
      VW1 = -UMAX
      VW2 = UMAX
      IF (U1.NE.U2) THEN
          UW1 = U1
          UW2 = U2
      END IF
      IF (V1.NE.V2) THEN
          VW1 = V1
          VW2 = V2
      END IF
*
* Set up the viewport and window
*
      CALL PGBBUF
      CALL PGPAGE
      CALL PGSVP(0.0, 1.0, 0.0, 1.0)
      CALL PGQVP(1, VPX1, VPX2, VPY1, VPY2)
      D = MIN(VPX2-VPX1, VPY2-VPY1)/40.0
      VPX1 = VPX1 + 5.0*D
      VPX2 = VPX2 - 2.0*D
      VPY1 = VPY1 + 9.0*D
      VPY2 = VPY2 - 4.0*D
      CALL PGVSIZ(VPX1, VPX2, VPY1, VPY2)
      CALL PGWNAD(UW2,UW1,VW1,VW2)
      CALL PGMTXT('T',2.5,0.0,0.0,LEGEND)
      CALL PGMTXT('T',0.8,0.0,0.0,TELESC)
      IF (GRID) THEN
          CALL PGSCI(14)
          CALL PGBOX('G', 0.0, 0, 'G', 0.0, 0)
      END IF
      CALL PGSCI(1)
      CALL PGBOX('BCNST', 0.0, 0, 'BCNSTV', 0.0, 0)
      CALL PGMTXT('B', 3.2, 0.5, 0.5, 
     1             '\(2263) East : u (wavelengths)')
      CALL PGMTXT('L', 3.2, 0.5, 0.5, 
     1            'v (wavelengths) : North \(2261)')
*
* Plot data
*
      CALL PGQLW(SAVELW)
      CALL PGSLW(DOTSIZ)
      CALL PGSCI(3)
      CALL PGPT(NP, U, V, -1)
*
* Plot conjugate data
*
      DO 120 IP=1,NP
          U(IP) = -U(IP)
          V(IP) = -V(IP)
  120 CONTINUE
      CALL PGPT(NP, U, V, -1)
      DO 140 IP=1,NP
          U(IP) = -U(IP)
          V(IP) = -V(IP)
  140 CONTINUE
*
* Highlight selected data
*
      IF (HLB.GT.0) THEN
          CALL PGSCI(5)
          DO 160 IP=1,NP
              IF (HLB.EQ.BASE(IP)) THEN
                  CALL PGPT(1, U(IP), V(IP), -1)
                  CALL PGPT(1, -U(IP), -V(IP), -1)
              END IF
  160     CONTINUE
          CALL PGMTXT('T', 0.8, 1.0, 1.0, HBNAME)
      ELSE IF (HLB.LT.0) THEN
          CALL PGSCI(5)
          DO 310 IP=1,NP
              IS1 = BLABEL(BASE(IP))/100
              IS2 = BLABEL(BASE(IP))-100*IS1
              IF ((-IS1.EQ.HLB) .OR. (-IS2.EQ.HLB)) THEN
                  CALL PGPT(1, U(IP), V(IP), -1)
                  CALL PGPT(1, -U(IP), -V(IP), -1)
              END IF
  310     CONTINUE
          CALL PGMTXT('T', 0.8, 1.0, 1.0, HBNAME)
      END IF
      CALL PGSCI(1)
      CALL PGSLW(SAVELW)
      CALL PGEBUF
*      
      END

C*SCPLOT   -- plot stack of closure-phases versus UT or GST.
C+
      SUBROUTINE SCPLOT(NPP, NP, NS, NB, 
     *                STNAME, BLABEL,
     *                UTMIN, UTMAX, TIME, 
     *                GSTMIN, GSTMAX, GST,
     *                BASE, AMP, PHASE, ERR, MPHS,
     *                LEGEND, MODEL, AXIS, NDAY, NOERR, DOGST,
     *                RESULT, MODFIL, NCOMP, TFLUX, REFANT)
      INTEGER NPP, NP, NS, NB, BLABEL(NB), BASE(NP), NDAY, REFANT
      REAL UTMIN, UTMAX, TIME(NP)
      REAL GSTMIN, GSTMAX, GST(NP)
      REAL AMP(NP), PHASE(NP), ERR(NP)
      REAL MPHS(NP)
      CHARACTER*(*) STNAME(NS), LEGEND, AXIS
      LOGICAL MODEL, NOERR, DOGST
      CHARACTER*(*) RESULT, MODFIL
      INTEGER NCOMP
      REAL TFLUX
C
C All parameters are input.
C NPP: number of plots per page
C NP:  number of visibility data
C NS:  number of stations
C NB:  number of baselines
C STNAME(NS): station names
C BLABEL(NB): baseline labels
C REFANT: if non-zero, plot only triangles involving this antenna
C-----------------------------------------------------------------------
      INTEGER MAXP
      PARAMETER (MAXP=8000)
      INTEGER N, I, K, L1,L2,L3, IB1,IB2,IB3, IS1,IS2,IS3
      INTEGER IER, L, NSUB
      INTEGER LEN1, LOOKUP, GETIN
      REAL X(MAXP), CLP(MAXP), CLPERR(MAXP), MCLP(MAXP)
      REAL TMIN, TMAX, TEMP1, TEMP2
      REAL VPX1, VPX2, VPY1, VPY2, D, HT, TICK
      CHARACTER*20 STR, COM
      CHARACTER*26 TNAME
      LOGICAL FIRST
*
      LOGICAL GRID
      REAL U1, U2, V1, V2, ASCALE, PSCALE, CSCALE
      INTEGER DOTSIZ
      COMMON /PLT/ GRID, U1, U2, V1, V2, DOTSIZ, ASCALE, PSCALE, CSCALE
*
* Set up
*
      IF (DOGST) THEN
          TMIN = GSTMIN
          TMAX = GSTMAX
      ELSE
          TMIN = UTMIN
          TMAX = UTMAX
      END IF
*
* Subdivision of phase axis.
*
      TICK = 90.0
      NSUB = 3
      IF (CSCALE.LT.100.0) THEN
          TICK = 0.0
          NSUB =0
      END IF
*
* Loop through the triangles.
*
      K = NPP
      FIRST = .TRUE.
      DO 1003 IS3=3,NS
        DO 1002 IS2=2,IS3-1
          DO 1001 IS1=1,IS2-1
C           -- skip triangles not involving reference antenna
            IF (REFANT.NE.0 .AND. REFANT.NE.IS1. AND. REFANT.NE.IS2
     :          .AND. REFANT.NE.IS3) GOTO 1001
C           -- Triangle name
            L1 = LEN1(STNAME(IS1))
            L2 = LEN1(STNAME(IS2))
            L3 = LEN1(STNAME(IS3))
            TNAME = STNAME(IS1)(:L1)//'-'//
     *            STNAME(IS2)(:L2)//'-'//STNAME(IS3)(:L3)
C           -- Find the baseline numbers
            IB1 = LOOKUP(100*IS1+IS2, BLABEL, NB)
            IB2 = LOOKUP(100*IS2+IS3, BLABEL, NB)
            IB3 = LOOKUP(100*IS1+IS3, BLABEL, NB)
            IF (IB1.EQ.0 .OR. IB2.EQ.0 .OR. IB3.EQ.0) GOTO 1001
C           -- Get data for this triangle
            CALL GETCL(NP, GST, TIME, BASE, AMP, PHASE, ERR, MPHS,
     :           DOGST, MAXP, IB1, IB2, IB3, N, X, CLP, CLPERR, MCLP)
            IF (N.LT.1) THEN
                CALL PUTOUT('No data for triangle '//TNAME)
                GOTO 1001
            ELSE IF (N.GT.MAXP) THEN
                CALL PUTOUT('Too many data points for triangle '//TNAME)
            END IF
*           -- Plot the data
            K = K+1
            IF (K.GT.NPP) THEN
                IF (.NOT.FIRST) THEN
                  COM = ' '
                  IER = GETIN(COM,
     :            'Type <return> for next page, Q to quit> ',L)
                  IF (COM(1:1).EQ.'Q' .OR. COM(1:1).EQ.'q' 
     :              .OR. IER.NE.1) GOTO 999
                ELSE
                  FIRST = .FALSE.
                END IF
                CALL PGPAGE
                CALL PGSVP(0.0, 1.0, 0.0, 1.0)
                CALL PGQVP(1, VPX1, VPX2, VPY1, VPY2)
                D = MIN(VPX2-VPX1, VPY2-VPY1)/40.0
                VPX1 = VPX1 + 5.0*D
                VPX2 = VPX2 - 2.0*D
                VPY1 = VPY1 + 9.0*D
                VPY2 = VPY2 - 4.0*D
                CALL PGSCH(0.6)
                K = 1
            END IF
            CALL PGBBUF
            HT = (VPY2-VPY1)/REAL(NPP)
            CALL PGVSIZ(VPX1, VPX2, VPY2-K*HT, VPY2-(K-1)*HT)
            CALL PGSWIN(TMIN,TMAX,-CSCALE,CSCALE)
            IF (K.EQ.1) THEN
                CALL PGSCH(1.0)
                CALL PGMTXT('T',2.5,0.0,0.0,LEGEND)
                CALL PGMTXT('T',0.8,0.0,0.0, 'Closure Phase (degrees)')
                CALL PGSCH(0.6)
            END IF
            IF (DOGST .OR. AXIS.EQ.'D') THEN
                IF (GRID) THEN
                    CALL PGSCI(14)
                    CALL PGBOX('G', 0.0, 0, 'G', TICK, NSUB)
                    CALL PGSCI(1)
                END IF
                CALL PGBOX('BCST', 0.0, 0, 'BCNSTV', TICK, NSUB)
            ELSE IF (AXIS.EQ.'H') THEN
                IF (GRID) THEN
                     CALL PGSCI(14)
                     CALL PGTBOX('GZ', 0.0, 0, 'G', TICK, NSUB)
                     CALL PGSCI(1)
                END IF
                CALL PGTBOX('BCSTZYH', 0.0, 0, 'BCNSTV', TICK, NSUB)
            END IF
            IF (K.EQ.NPP) THEN
                IF (DOGST) THEN
                    CALL PGMTXT('B', 3.2, 0.5, 0.5, 'GST (hours)')
                    CALL PGBOX('N', 0.0, 0, ' ', TICK, NSUB)
                ELSE IF (AXIS.EQ.'D') THEN
                    CALL PGMTXT('B', 3.2, 0.5, 0.5, 
     :                                           'UT (day of year)')
                    CALL PGBOX('N', 0.0, 0, ' ', TICK, NSUB)
                ELSE 
                    WRITE (STR,'(''UT on day '',I3)') NDAY
                    CALL PGMTXT('B', 3.2, 0.5, 0.5, STR)
                    CALL PGTBOX('ZHN', 0.0, 0, ' ', TICK, NSUB)
                END IF
            END IF
            CALL PGMTXT('T',-1.5,0.05,0.0,TNAME)
*
            CALL PGSCI(3)
            IF (NOERR) THEN
                CALL PGPT(N, X, CLP, -1)
            ELSE
                DO 57 I=1,N
                   TEMP1 = CLP(I)
                   TEMP2 = CLPERR(I)
                   CLP(I) = TEMP1-TEMP2
                   CLPERR(I) = TEMP1+TEMP2
   57           CONTINUE
                CALL PGERRY(N, X, CLP, CLPERR, 0.0)
            END IF
            IF (MODEL) THEN
                CALL PGSCI(2)
                CALL PGMOVE(X(1),MCLP(1))
                DO 150 I=2,N
                    IF (X(I).LT.X(I-1)) THEN
                        CALL PGMOVE(X(I),MCLP(I))
                    ELSE IF (ABS(MCLP(I)-MCLP(I-1)).LE.180.0) THEN
                        CALL PGDRAW(X(I),MCLP(I))
                    ELSE IF (MCLP(I)-MCLP(I-1).GT.180.0) THEN
                        CALL PGDRAW(X(I),MCLP(I)-360.0)
                        CALL PGMOVE(X(I-1),MCLP(I-1)+360.0)
                        CALL PGDRAW(X(I),MCLP(I))
                    ELSE
                        CALL PGDRAW(X(I),MCLP(I)+360.0)
                        CALL PGMOVE(X(I-1),MCLP(I-1)-360.0)
                        CALL PGDRAW(X(I),MCLP(I))
                    END IF
  150           CONTINUE
            END IF
            CALL PGSCI(1)
            CALL PGEBUF
            IF (K.EQ.NPP) CALL ANNOT(RESULT, MODFIL, NCOMP, TFLUX)
 1001     CONTINUE
 1002   CONTINUE
 1003 CONTINUE
C     -- Label bottom axis if necessary
      IF (K.NE.NPP) THEN
                IF (DOGST) THEN
                    CALL PGMTXT('B', 3.2, 0.5, 0.5, 'GST (hours)')
                    CALL PGBOX('N', 0.0, 0, ' ', TICK, NSUB)
                ELSE IF (AXIS.EQ.'D') THEN
                    CALL PGMTXT('B', 3.2, 0.5, 0.5, 
     :                                           'UT (day of year)')
                    CALL PGBOX('N', 0.0, 0, ' ', TICK, NSUB)
                ELSE 
                    WRITE (STR,'(''UT on day '',I3)') NDAY
                    CALL PGMTXT('B', 3.2, 0.5, 0.5, STR)
                    CALL PGTBOX('ZHN', 0.0, 0, ' ', TICK, NSUB)
                END IF
                CALL ANNOT(RESULT, MODFIL, NCOMP, TFLUX)
      END IF
C
  999 CONTINUE
      CALL PGSCI(1)
      CALL PGSCH(1.0)
      END

C*GETCL  -- Get closure phases for a single triangle
C+
      SUBROUTINE GETCL(NP, GST, TIME, BASE, AMP, PHASE, ERR, MPHS,
     :           DOGST, MAXP, IB1, IB2, IB3, N, X, CLP, CLPERR, MCLP)
      INTEGER NP, BASE(NP), MAXP, IB1,IB2,IB3, N
      LOGICAL DOGST
      REAL GST(NP), TIME(NP), AMP(NP), PHASE(NP), ERR(NP), MPHS(NP)
      REAL X(MAXP), CLP(MAXP), CLPERR(MAXP), MCLP(MAXP)
C
C input
C     INTEGER NP
C     REAL GST(NP), AMP(NP), PHASE(NP), MPHS(NP)
C     INTEGER MAXP	! input: array size
C     INTEGER IB1, IB2, IB3	! input: requested baseline triple
C output
C     INTEGER N		! output: number of samples
C     REAL X(MAXP)	! output UT or GST
C     REAL CLS(MAXP)	! output: closure phase
C     REAL CLSERR(MAXP)	! output: error on closure phase
C     REAL MCLS(MAXP)	! output: model closure phase
C
C Get the data for the requested triangle. Data are assumed to be
C in time order.
C-----------------------------------------------------------------------
      REAL REFTIM, REFUT
      INTEGER IP, IP1, IP2, IP3
      REAL PHSE1, PHSE2, PHSE3
C
      N = 0
      REFTIM = 0.0
      REFUT  = 0.0
      IP1 = 0
      IP2 = 0
      IP3 = 0
      DO 100 IP=1,NP
          IF (GST(IP).NE.REFTIM) THEN
*             -- start of new integration; save result from last
*             -- integration, if any. IP1-IP3 are indices of the
*             -- data for the requested triangle.
              IF (IP1.NE.0 .AND. IP2.NE.0 .AND. IP3.NE.0) THEN
                  N = N+1
                  IF (N.GT.MAXP) THEN
                      RETURN
                  END IF
                  IF (DOGST) THEN
                     X(N) = REFTIM
                  ELSE
                     X(N) = REFUT
                  END IF
*                 -- observed data
                  CLP(N) = MOD(PHASE(IP1)+PHASE(IP2)-PHASE(IP3),360.0)
                  IF (CLP(N).LE.-180.0) CLP(N) = CLP(N)+360.0
                  IF (CLP(N).GT.180.0)  CLP(N) = CLP(N)-360.0
                  PHSE1 = 180.0
                  PHSE2 = 180.0
                  PHSE3 = 180.0
                  IF (AMP(IP1).NE.0.0) PHSE1 = (ERR(IP1)/AMP(IP1))**2
                  IF (AMP(IP2).NE.0.0) PHSE2 = (ERR(IP2)/AMP(IP2))**2
                  IF (AMP(IP3).NE.0.0) PHSE3 = (ERR(IP3)/AMP(IP3))**2
                  CLPERR(N) = 57.295780*SQRT(PHSE1+PHSE2+PHSE3)
*                 -- model data
                  MCLP(N) = MOD(MPHS(IP1)+MPHS(IP2)-MPHS(IP3),360.0)
                  IF (MCLP(N).LE.-180.0) MCLP(N) = MCLP(N)+360.0
                  IF (MCLP(N).GT.180.0)  MCLP(N) = MCLP(N)-360.0
              END IF
              REFTIM = GST(IP)
              REFUT  = TIME(IP)
              IP1 = 0
              IP2 = 0
              IP3 = 0
          END IF
          IF (BASE(IP).EQ.IB1) IP1 = IP
          IF (BASE(IP).EQ.IB2) IP2 = IP
          IF (BASE(IP).EQ.IB3) IP3 = IP
  100 CONTINUE
*
      RETURN
      END

C*SPPLOT   -- plot stack of phases versus UT or GST.
C+
      SUBROUTINE SPPLOT(NPP, NP, NS, NB, 
     *                STNAME, BLABEL,
     *                UTMIN, UTMAX, TIME, 
     *                GSTMIN, GSTMAX, GST,
     *                BASE, AMP, PHASE, ERR, MPHS,
     *                LEGEND, MODEL, AXIS, NDAY, NOERR, DOGST,
     *                RESULT, MODFIL, NCOMP, TFLUX, REFANT)
      INTEGER NPP, NP, NS, NB, BLABEL(NB), BASE(NP), NDAY, REFANT
      REAL UTMIN, UTMAX, TIME(NP)
      REAL GSTMIN, GSTMAX, GST(NP)
      REAL AMP(NP), PHASE(NP), ERR(NP)
      REAL MPHS(NP)
      CHARACTER*(*) STNAME(NS), LEGEND, AXIS
      LOGICAL MODEL, NOERR, DOGST
      CHARACTER*(*) RESULT, MODFIL
      INTEGER NCOMP
      REAL TFLUX
C
C All parameters are input.
C NPP: number of plots per page
C NP:  number of visibility data
C NS:  number of stations
C NB:  number of baselines
C STNAME(NS): station names
C BLABEL(NB): baseline labels
C REFANT: if non-zero, plot only baselines involving this antenna
C-----------------------------------------------------------------------
      INTEGER MAXP
      PARAMETER (MAXP=8000)
      INTEGER N, I, K, L1,L2, IB, IS1,IS2
      INTEGER IER, L, NSUB
      INTEGER LEN1, LOOKUP, GETIN
      REAL X(MAXP), CLP(MAXP), CLPERR(MAXP), MCLP(MAXP)
      REAL TMIN, TMAX, TEMP1, TEMP2
      REAL VPX1, VPX2, VPY1, VPY2, D, HT, TICK
      CHARACTER*20 STR, COM
      CHARACTER*26 TNAME
      LOGICAL FIRST
*
      LOGICAL GRID
      REAL U1, U2, V1, V2, ASCALE, PSCALE, CSCALE
      INTEGER DOTSIZ
      COMMON /PLT/ GRID, U1, U2, V1, V2, DOTSIZ, ASCALE, PSCALE, CSCALE
*
* Set up
*
      IF (DOGST) THEN
          TMIN = GSTMIN
          TMAX = GSTMAX
      ELSE
          TMIN = UTMIN
          TMAX = UTMAX
      END IF
*
* Subdivision of phase axis.
*
      TICK = 90.0
      NSUB = 3
      IF (PSCALE.LT.100.0) THEN
          TICK = 0.0
          NSUB =0
      END IF
*
* Loop through the triangles.
*
      K = NPP
      FIRST = .TRUE.
      DO 1002 IS2=2,NS
          DO 1001 IS1=1,IS2-1
C           -- skip baselines not involving reference antenna
            IF (REFANT.NE.0 .AND. REFANT.NE.IS1
     :          .AND. REFANT.NE.IS2) GOTO 1001
C           -- Baseline name
            L1 = LEN1(STNAME(IS1))
            L2 = LEN1(STNAME(IS2))
            TNAME = STNAME(IS1)(:L1)//'-'//STNAME(IS2)(:L2)
C           -- Find the baseline number
            IB = LOOKUP(100*IS1+IS2, BLABEL, NB)
            IF (IB.EQ.0) GOTO 1001
C           -- Get data for this baseline
            CALL GETPH(NP, GST, TIME, BASE, AMP, PHASE, ERR, MPHS,
     :           DOGST, MAXP, IB, N, X, CLP, CLPERR, MCLP)
            IF (N.LT.1) THEN
                CALL PUTOUT('No data for baseline '//TNAME)
                GOTO 1001
            ELSE IF (N.GT.MAXP) THEN
                CALL PUTOUT('Too many data points for baseline '//TNAME)
            END IF
*           -- Plot the data
            K = K+1
            IF (K.GT.NPP) THEN
                IF (.NOT.FIRST) THEN
                  COM = ' '
                  IER = GETIN(COM,
     :            'Type <return> for next page, Q to quit> ',L)
                  IF (COM(1:1).EQ.'Q' .OR. COM(1:1).EQ.'q' 
     :              .OR. IER.NE.1) GOTO 999
                ELSE
                  FIRST = .FALSE.
                END IF
                CALL PGPAGE
                CALL PGSVP(0.0, 1.0, 0.0, 1.0)
                CALL PGQVP(1, VPX1, VPX2, VPY1, VPY2)
                D = MIN(VPX2-VPX1, VPY2-VPY1)/40.0
                VPX1 = VPX1 + 5.0*D
                VPX2 = VPX2 - 2.0*D
                VPY1 = VPY1 + 9.0*D
                VPY2 = VPY2 - 4.0*D
                CALL PGSCH(0.6)
                K = 1
            END IF
            CALL PGBBUF
            HT = (VPY2-VPY1)/REAL(NPP)
            CALL PGVSIZ(VPX1, VPX2, VPY2-K*HT, VPY2-(K-1)*HT)
            CALL PGSWIN(TMIN,TMAX,-PSCALE,PSCALE)
            IF (K.EQ.1) THEN
                CALL PGSCH(1.0)
                CALL PGMTXT('T',2.5,0.0,0.0,LEGEND)
                CALL PGMTXT('T',0.8,0.0,0.0, 'Phase (degrees)')
                CALL PGSCH(0.6)
            END IF
            IF (DOGST .OR. AXIS.EQ.'D') THEN
                IF (GRID) THEN
                    CALL PGSCI(14)
                    CALL PGBOX('G', 0.0, 0, 'G', TICK, NSUB)
                    CALL PGSCI(1)
                END IF
                CALL PGBOX('BCST', 0.0, 0, 'BCNSTV', TICK, NSUB)
            ELSE IF (AXIS.EQ.'H') THEN
                IF (GRID) THEN
                     CALL PGSCI(14)
                     CALL PGTBOX('GZ', 0.0, 0, 'G', TICK, NSUB)
                     CALL PGSCI(1)
                END IF
                CALL PGTBOX('BCSTZYH', 0.0, 0, 'BCNSTV', TICK, NSUB)
            END IF
            IF (K.EQ.NPP) THEN
                IF (DOGST) THEN
                    CALL PGMTXT('B', 3.2, 0.5, 0.5, 'GST (hours)')
                    CALL PGBOX('N', 0.0, 0, ' ', TICK, NSUB)
                ELSE IF (AXIS.EQ.'D') THEN
                    CALL PGMTXT('B', 3.2, 0.5, 0.5, 
     :                                           'UT (day of year)')
                    CALL PGBOX('N', 0.0, 0, ' ', TICK, NSUB)
                ELSE 
                    WRITE (STR,'(''UT on day '',I3)') NDAY
                    CALL PGMTXT('B', 3.2, 0.5, 0.5, STR)
                    CALL PGTBOX('ZHN', 0.0, 0, ' ', TICK, NSUB)
                END IF
            END IF
            CALL PGMTXT('T',-1.5,0.05,0.0,TNAME)
*
            CALL PGSCI(3)
            IF (NOERR) THEN
                CALL PGPT(N, X, CLP, -1)
            ELSE
                DO 57 I=1,N
                   TEMP1 = CLP(I)
                   TEMP2 = CLPERR(I)
                   CLP(I) = TEMP1-TEMP2
                   CLPERR(I) = TEMP1+TEMP2
   57           CONTINUE
                CALL PGERRY(N, X, CLP, CLPERR, 0.0)
            END IF
            IF (MODEL) THEN
                CALL PGSCI(2)
                CALL PGMOVE(X(1),MCLP(1))
                DO 150 I=2,N
                    IF (X(I).LT.X(I-1)) THEN
                        CALL PGMOVE(X(I),MCLP(I))
                    ELSE IF (ABS(MCLP(I)-MCLP(I-1)).LE.180.0) THEN
                        CALL PGDRAW(X(I),MCLP(I))
                    ELSE IF (MCLP(I)-MCLP(I-1).GT.180.0) THEN
                        CALL PGDRAW(X(I),MCLP(I)-360.0)
                        CALL PGMOVE(X(I-1),MCLP(I-1)+360.0)
                        CALL PGDRAW(X(I),MCLP(I))
                    ELSE
                        CALL PGDRAW(X(I),MCLP(I)+360.0)
                        CALL PGMOVE(X(I-1),MCLP(I-1)-360.0)
                        CALL PGDRAW(X(I),MCLP(I))
                    END IF
  150           CONTINUE
            END IF
            CALL PGSCI(1)
            CALL PGEBUF
            IF (K.EQ.NPP) CALL ANNOT(RESULT, MODFIL, NCOMP, TFLUX)
 1001     CONTINUE
 1002 CONTINUE
C     -- Label bottom axis if necessary
      IF (K.NE.NPP) THEN
                IF (DOGST) THEN
                    CALL PGMTXT('B', 3.2, 0.5, 0.5, 'GST (hours)')
                    CALL PGBOX('N', 0.0, 0, ' ', TICK, NSUB)
                ELSE IF (AXIS.EQ.'D') THEN
                    CALL PGMTXT('B', 3.2, 0.5, 0.5, 
     :                                           'UT (day of year)')
                    CALL PGBOX('N', 0.0, 0, ' ', TICK, NSUB)
                ELSE 
                    WRITE (STR,'(''UT on day '',I3)') NDAY
                    CALL PGMTXT('B', 3.2, 0.5, 0.5, STR)
                    CALL PGTBOX('ZHN', 0.0, 0, ' ', TICK, NSUB)
                END IF
                CALL ANNOT(RESULT, MODFIL, NCOMP, TFLUX)
      END IF
C
  999 CONTINUE
      CALL PGSCI(1)
      CALL PGSCH(1.0)
      END

C*GETPH  -- Get phases for a single baseline
C+
      SUBROUTINE GETPH(NP, GST, TIME, BASE, AMP, PHASE, ERR, MPHS,
     :           DOGST, MAXP, IB, N, X, CLP, CLPERR, MCLP)
      INTEGER NP, BASE(NP), MAXP, IB, N
      LOGICAL DOGST
      REAL GST(NP), TIME(NP), AMP(NP), PHASE(NP), ERR(NP), MPHS(NP)
      REAL X(MAXP), CLP(MAXP), CLPERR(MAXP), MCLP(MAXP)
C
C input
C     INTEGER NP
C     REAL GST(NP), AMP(NP), PHASE(NP), MPHS(NP)
C     INTEGER MAXP	! input: array size
C     INTEGER IB	! input: requested baseline
C output
C     INTEGER N		! output: number of samples
C     REAL X(MAXP)	! output UT or GST
C     REAL CLS(MAXP)	! output: phase
C     REAL CLSERR(MAXP)	! output: error on phase
C     REAL MCLS(MAXP)	! output: model phase
C
C Get the data for the requested baseline.
C-----------------------------------------------------------------------
      INTEGER IP
C
      N = 0
      DO 100 IP=1,NP
          IF (BASE(IP).EQ.IB) THEN
              N = N+1
              IF (N.GT.MAXP) THEN
                  RETURN
              END IF
              IF (DOGST) THEN
                  X(N) = GST(IP)
              ELSE
                  X(N) = TIME(IP)
              END IF
*             -- observed data
              CLP(N) = MOD(PHASE(IP),360.0)
              IF (CLP(N).LE.-180.0) CLP(N) = CLP(N)+360.0
              IF (CLP(N).GT.180.0)  CLP(N) = CLP(N)-360.0
              CLPERR(N) = 180.0
              IF (AMP(IP).NE.0.0) CLPERR(N) = 57.295780*ERR(IP)/AMP(IP) 
*             -- model data
              MCLP(N) = MOD(MPHS(IP),360.0)
              IF (MCLP(N).LE.-180.0) MCLP(N) = MCLP(N)+360.0
              IF (MCLP(N).GT.180.0)  MCLP(N) = MCLP(N)-360.0
          END IF
  100 CONTINUE
*
      RETURN
      END

C*SAPLOT   -- plot stack of amplitudes versus UT or GST.
C+
      SUBROUTINE SAPLOT(NPP, NP, NS, NB, 
     *                STNAME, BLABEL,
     *                UTMIN, UTMAX, TIME, 
     *                GSTMIN, GSTMAX, GST,
     *                BASE, AMP, ERR, MAMP,
     *                LEGEND, MODEL, AXIS, NDAY, NOERR, DOGST,
     *                RESULT, MODFIL, NCOMP, TFLUX, REFANT, KFLAG)
      INTEGER NPP, NP, NS, NB, BLABEL(NB), BASE(NP), NDAY, REFANT, KFLAG
      REAL UTMIN, UTMAX, TIME(NP)
      REAL GSTMIN, GSTMAX, GST(NP)
      REAL AMP(NP), ERR(NP), MAMP(NP)
      CHARACTER*(*) STNAME(NS), LEGEND, AXIS
      LOGICAL MODEL, NOERR, DOGST
      CHARACTER*(*) RESULT, MODFIL
      INTEGER NCOMP
      REAL TFLUX
C
C All parameters are input.
C NPP: number of plots per page
C NP:  number of visibility data
C NS:  number of stations
C NB:  number of baselines
C STNAME(NS): station names
C BLABEL(NB): baseline labels
C REFANT: if non-zero, plot only baselines involving this antenna
C KFLAG: 0 for correlation coeff, 1 for Jy
C-----------------------------------------------------------------------
      INTEGER MAXP
      PARAMETER (MAXP=8000)
      INTEGER N, I, K, L1,L2, IB, IS1,IS2
      INTEGER IER, L, NSUB
      INTEGER LEN1, LOOKUP, GETIN
      REAL X(MAXP), DAT(MAXP), DATERR(MAXP), MDAT(MAXP)
      REAL TMIN, TMAX, TEMP1, TEMP2
      REAL AMAX, AMPMAX
      REAL VPX1, VPX2, VPY1, VPY2, D, HT, TICK
      CHARACTER*20 STR, COM
      CHARACTER*26 TNAME
      LOGICAL FIRST
*
      LOGICAL GRID
      REAL U1, U2, V1, V2, ASCALE, PSCALE, CSCALE
      INTEGER DOTSIZ
      COMMON /PLT/ GRID, U1, U2, V1, V2, DOTSIZ, ASCALE, PSCALE, CSCALE
*
* Set up
*
      IF (DOGST) THEN
          TMIN = GSTMIN
          TMAX = GSTMAX
      ELSE
          TMIN = UTMIN
          TMAX = UTMAX
      END IF
*
* Subdivision of amplitude axis.
*
      TICK = 0.0
      NSUB = 0
*
* Loop through the baselines.
*
      K = NPP
      FIRST = .TRUE.
      DO 1002 IS2=2,NS
          DO 1001 IS1=1,IS2-1
C           -- skip baselines not involving reference antenna
            IF (REFANT.NE.0 .AND. REFANT.NE.IS1
     :          .AND. REFANT.NE.IS2) GOTO 1001
C           -- Baseline name
            L1 = LEN1(STNAME(IS1))
            L2 = LEN1(STNAME(IS2))
            TNAME = STNAME(IS1)(:L1)//'-'//STNAME(IS2)(:L2)
C           -- Find the baseline number
            IB = LOOKUP(100*IS1+IS2, BLABEL, NB)
            IF (IB.EQ.0) GOTO 1001
C           -- Get data for this baseline
            CALL GETAMP(NP, GST, TIME, BASE, AMP, ERR, MAMP, DOGST,
     :           MAXP, IB, N, X, DAT, DATERR, MDAT, NOERR, AMPMAX)
            IF (N.LT.1) THEN
                CALL PUTOUT('No data for baseline '//TNAME)
                GOTO 1001
            ELSE IF (N.GT.MAXP) THEN
                CALL PUTOUT('Too many data points for baseline '//TNAME)
            END IF
*           -- Plot the data
            K = K+1
            IF (K.GT.NPP) THEN
                IF (.NOT.FIRST) THEN
                  COM = ' '
                  IER = GETIN(COM,
     :            'Type <return> for next page, Q to quit> ',L)
                  IF (COM(1:1).EQ.'Q' .OR. COM(1:1).EQ.'q' 
     :              .OR. IER.NE.1) GOTO 999
                ELSE
                  FIRST = .FALSE.
                END IF
                CALL PGPAGE
                CALL PGSVP(0.0, 1.0, 0.0, 1.0)
                CALL PGQVP(1, VPX1, VPX2, VPY1, VPY2)
                D = MIN(VPX2-VPX1, VPY2-VPY1)/40.0
                VPX1 = VPX1 + 5.0*D
                VPX2 = VPX2 - 2.0*D
                VPY1 = VPY1 + 9.0*D
                VPY2 = VPY2 - 4.0*D
                CALL PGSCH(0.6)
                K = 1
            END IF
            CALL PGBBUF
            HT = (VPY2-VPY1)/REAL(NPP)
            CALL PGVSIZ(VPX1, VPX2, VPY2-K*HT, VPY2-(K-1)*HT)
            AMAX = ASCALE
            IF (ASCALE.EQ.0.0) AMAX = 1.1*AMPMAX
            CALL PGSWIN(TMIN,TMAX,0.0,AMAX)
            IF (K.EQ.1) THEN
                CALL PGSCH(1.0)
                CALL PGMTXT('T',2.5,0.0,0.0,LEGEND)
                IF (KFLAG.EQ.0) THEN
                    CALL PGMTXT('T',0.8,0.0,0.0,
     :                  'Correlation Coeff (\x10\u5\d)')
                ELSE
                    CALL PGMTXT('T',0.8,0.0,0.0, 'Amplitude (Jy)')
                END IF
                CALL PGSCH(0.6)
            END IF
            IF (DOGST .OR. AXIS.EQ.'D') THEN
                IF (GRID) THEN
                    CALL PGSCI(14)
                    CALL PGBOX('G', 0.0, 0, 'G', TICK, NSUB)
                    CALL PGSCI(1)
                END IF
                CALL PGBOX('BCST', 0.0, 0, 'BCNSTV', TICK, NSUB)
            ELSE IF (AXIS.EQ.'H') THEN
                IF (GRID) THEN
                     CALL PGSCI(14)
                     CALL PGTBOX('GZ', 0.0, 0, 'G', TICK, NSUB)
                     CALL PGSCI(1)
                END IF
                CALL PGTBOX('BCSTZYH', 0.0, 0, 'BCNSTV', TICK, NSUB)
            END IF
            IF (K.EQ.NPP) THEN
                IF (DOGST) THEN
                    CALL PGMTXT('B', 3.2, 0.5, 0.5, 'GST (hours)')
                    CALL PGBOX('N', 0.0, 0, ' ', TICK, NSUB)
                ELSE IF (AXIS.EQ.'D') THEN
                    CALL PGMTXT('B', 3.2, 0.5, 0.5, 
     :                                           'UT (day of year)')
                    CALL PGBOX('N', 0.0, 0, ' ', TICK, NSUB)
                ELSE 
                    WRITE (STR,'(''UT on day '',I3)') NDAY
                    CALL PGMTXT('B', 3.2, 0.5, 0.5, STR)
                    CALL PGTBOX('ZHN', 0.0, 0, ' ', TICK, NSUB)
                END IF
            END IF
            CALL PGMTXT('T',-1.5,0.05,0.0,TNAME)
*
            CALL PGSCI(3)
            IF (NOERR) THEN
                CALL PGPT(N, X, DAT, -1)
            ELSE
                DO 57 I=1,N
                   TEMP1 = DAT(I)
                   TEMP2 = DATERR(I)
                   DAT(I) = TEMP1-TEMP2
                   DATERR(I) = TEMP1+TEMP2
   57           CONTINUE
                CALL PGERRY(N, X, DAT, DATERR, 0.0)
            END IF
            IF (MODEL) THEN
                CALL PGSCI(2)
                IF (DOGST) THEN
                    CALL PGMOVE(X(1),MDAT(1))
                    DO 15 I=2,N
                        IF (X(I).LT.X(I-1)) THEN
                            CALL PGMOVE(X(I),MDAT(I))
                        ELSE
                            CALL PGDRAW(X(I),MDAT(I))
                        END IF
   15               CONTINUE
                ELSE
                    CALL PGLINE(N, X, MDAT)
                END IF
            END IF
            CALL PGSCI(1)
            CALL PGEBUF
            IF (K.EQ.NPP) CALL ANNOT(RESULT, MODFIL, NCOMP, TFLUX)
 1001     CONTINUE
 1002 CONTINUE
C     -- Label bottom axis if necessary
      IF (K.NE.NPP) THEN
                IF (DOGST) THEN
                    CALL PGMTXT('B', 3.2, 0.5, 0.5, 'GST (hours)')
                    CALL PGBOX('N', 0.0, 0, ' ', TICK, NSUB)
                ELSE IF (AXIS.EQ.'D') THEN
                    CALL PGMTXT('B', 3.2, 0.5, 0.5, 
     :                                           'UT (day of year)')
                    CALL PGBOX('N', 0.0, 0, ' ', TICK, NSUB)
                ELSE 
                    WRITE (STR,'(''UT on day '',I3)') NDAY
                    CALL PGMTXT('B', 3.2, 0.5, 0.5, STR)
                    CALL PGTBOX('ZHN', 0.0, 0, ' ', TICK, NSUB)
                END IF
                CALL ANNOT(RESULT, MODFIL, NCOMP, TFLUX)
      END IF
C
  999 CONTINUE
      CALL PGSCI(1)
      CALL PGSCH(1.0)
      END

C*GETAMP  -- Get amplitudes for a single baseline
C+
      SUBROUTINE GETAMP(NP, GST, TIME, BASE, AMP, ERR, MAMP,
     :           DOGST, MAXP, IB, N, X, DAT, DATERR, MDAT, NOERR, 
     :           AMPMAX)
      INTEGER NP, BASE(NP), MAXP, IB, N
      LOGICAL DOGST, NOERR
      REAL GST(NP), TIME(NP), AMP(NP), ERR(NP), MAMP(NP)
      REAL X(MAXP), DAT(MAXP), DATERR(MAXP), MDAT(MAXP), AMPMAX
C
C input
C     INTEGER NP
C     REAL GST(NP), AMP(NP), MAMP(NP)
C     INTEGER MAXP	! input: array size
C     INTEGER IB	! input: requested baseline
C     LOGICAL NOERR	! input: if true, don't include error
C output
C     INTEGER N		! output: number of samples
C     REAL X(MAXP)	! output UT or GST
C     REAL CLS(MAXP)	! output: amplitude
C     REAL CLSERR(MAXP)	! output: error on amplitude
C     REAL MCLS(MAXP)	! output: model amplitude
C     REAL AMPMAX       ! output: maximum amplitude
C
C Get the data for the requested baseline.
C-----------------------------------------------------------------------
      INTEGER IP
C
      N = 0
      AMPMAX = 0.0
      DO 100 IP=1,NP
          IF (BASE(IP).EQ.IB) THEN
              N = N+1
              IF (N.GT.MAXP) THEN
                  RETURN
              END IF
              IF (DOGST) THEN
                  X(N) = GST(IP)
              ELSE
                  X(N) = TIME(IP)
              END IF
*             -- observed data
              DAT(N) = AMP(IP)
              DATERR(N) = ERR(IP)
              IF (NOERR) THEN
                  AMPMAX = MAX(AMPMAX,AMP(IP))
              ELSE
                  AMPMAX = MAX(AMPMAX,AMP(IP)+ERR(IP))
              END IF
*             -- model data
              MDAT(N) = MAMP(IP)
          END IF
  100 CONTINUE
*
      RETURN
      END
