$!----------------------------------------------------------------------
$! This procedure tests the sequence: FAKE, AMPHI, INVERT, CLEAN.
$! The model used in AMPHI is the starting model (i.e. cheating).
$! Programs LIST, UVPLOT, VISPLOT, FITSLIST and MAPPLOT are also tested.
$!
$! Revised 1986 Nov 17 - new syntax for LIST, VISPLOT, UVPLOT (TJP).
$! Revised 1988 Jun 23 - new syntax for AMPHI, INVERT, CLEAN (TJP).
$! Revised 1989 Aug 17 - change YRANGE in MAPPLOT for new CLEAN (TJP).
$! REvised 1992 Jul  6 - add dirty map plot in MAPPLOT; use PostScript.
$!----------------------------------------------------------------------
$!
$ PROC = F$ENVIRONMENT("PROCEDURE")
$ PROC := 'PROC'
$ NAME = F$PARSE(PROC,,,"NAME","SYNTAX_ONLY")
$!
$ SET DEFAULT SYS$SCRATCH
$ ON WARNING THEN GOTO EXIT
$!
$! Change the following definitions to match your printer:
$!
$ PLOTTER = "VPS"
$ LW = 1    ! line-width for plotting
$!----------------------------------------------------------------------
$ CREATE DAISYG.MOD
! Daisy model with Gaussian components (TJP, 15-Nov-82)
!
     1.000000      0.0          0.0          0.9      1.0      0.0   1
     0.050000      1.99991     90.00219      1.0      1.0      0.0   1
     0.050000      3.99995     90.00040      1.0      1.0      0.0   1
     0.050000      5.99995     90.00027      1.0      1.0      0.0   1
     0.050000      8.0         90.00055      1.0      1.0      0.0   1
     0.050000      9.99990     90.0          1.0      1.0      0.0   1
     0.050000      2.00015     59.99922      1.0      1.0      0.0   1
     0.050000      4.00019     59.99779      1.0      1.0      0.0   1
     0.050000      5.99975     60.00362      1.0      1.0      0.0   1
     0.050000      7.99989     59.99959      1.0      1.0      0.0   1
     0.050000      9.99971     59.99938      1.0      1.0      0.0   1
     0.050000      2.00015     30.00078      1.0      1.0      0.0   1
     0.050000      4.00019     30.00222      1.0      1.0      0.0   1
     0.050000      5.99975     29.99638      1.0      1.0      0.0   1
     0.050000      7.99989     30.00041      1.0      1.0      0.0   1
     0.050000      9.99971     30.00063      1.0      1.0      0.0   1
     0.050000      1.99991     -0.00219      1.0      1.0      0.0   1
     0.050000      3.99995     -0.00040      1.0      1.0      0.0   1
     0.050000      5.99995     -0.00026      1.0      1.0      0.0   1
     0.050000      8.0         -0.00055      1.0      1.0      0.0   1
     0.050000      9.99990      0.0          1.0      1.0      0.0   1
$!----------------------------------------------------------------------
$ ASSIGN/USER FAKEFAKE.LIS FOR006
$ FAKE
  OUTPUT        = DAISYG.MRG
  MODEL         = DAISYG.MOD
  RA            = 18:03:39.177
  DEC           = 78:27:54.21
  YEAR          = 1982
  DAY           = 95
  START         = 06:00:00
  STOP          = 18:00:00
  INTEG         = 600
  FREQ          = 4989
  SOURCE        = DAISY_G
  NUP           = 3
  STATIONS      = BONN, HSTK, NRAO, FDVS, OVRO
    EFFICIENCY  = 0.5,  0.5,  0.5,  0.5,  0.5
    DIAMETER    = 100,  34,   43,   25,   40
    POINTING    =   1,   1,    1,    1,    1
    TSYS        =  75,  120,  40,   120,  70
  BANDWIDTH     = 1.8
  ERRMULT       = 0.01
  /
$!----------------------------------------------------------------------
$ LIST
  INPUT = DAISYG.MRG
  LISTFILE = FAKELIST.LIS
  HEADER 
  /
$!----------------------------------------------------------------------
$ VISPLOT
  INPUT = DAISYG.MRG
  PLOTFILE = FAKESTRT.PLT/{PLOTTER}
  MODFILE = DAISYG.MOD
  NP=10 LINEWIDTH={LW} HTLET=0.06 IHA MODEL=200 
  /
$!----------------------------------------------------------------------
$ UVPLOT
  INPUT = DAISYG.MRG
  PLOTFILE = FAKEUVP.PLT/{PLOTTER}
  LINEWIDTH={LW} 
  /
$!----------------------------------------------------------------------
$ AMPHI
  INPUT   = DAISYG.MRG
  OUTPUT  = DAISYG.ADJ
  MODFILE = DAISYG.MOD
  LISTFILE= FAKEAMP.LIS
  /
$!----------------------------------------------------------------------
$ INVERT
  INPUT   = DAISYG.ADJ
  MAPFILE = DAISYG.MAP
  BEAMFILE= DAISYG.BMM
  PLOTFILE= FAKEINV.PLT/{PLOTTER}
  LISTFILE= FAKEINV.LIS
  XYINT   = 0.25  MILLIARCSEC
  FLUX    = 2.0
  PRINTUV
  PLOTMAP
  PLOTBEAM = 5
  /
$!----------------------------------------------------------------------
$ CLEAN
  MAPFILE  = DAISYG.MAP
  BEAMFILE = DAISYG.BMM
  PLOTFILE = FAKECLN.PLT/{PLOTTER}
  LISTFILE = FAKECLN.LIS
  BEAM     = 0.0	! use calculated beam
  PRINTRESIDUALS
  NITER    = 400
  LRTB     = -12,2,6,-2,  -6,2,12,-2
  MODEL    = DAISYG.MDL
  CLEANMAP = DAISYG.CLM
  PLOTWINDOW
  PLOTMAP
  TITLE    = "{name}"
  CONTOURS = -5,-4,-3,-2,-1,1,2,3,4,5,10,20,30,40,50
  /
$!----------------------------------------------------------------------
$ VISPLOT
  INPUT    = DAISYG.MRG
  PLOTFILE = FAKEFIT.PLT/{PLOTTER}
  MODFILE  = DAISYG.MDL
  NP       = 10
  LINEWIDTH= {LW}
  HTLET    = 0.06
  IHA
  MODEL    = 200
  /
$!----------------------------------------------------------------------
$ FITSLIST
  INPUT   = DAISYG.CLM
  LIST    = FAKEFITS.LIS
  /
$!----------------------------------------------------------------------
$ MAPPLOT
  MAPFILE = DAISYG.CLM
  PLOTFILE= FAKEMAP.PLT/{PLOTTER}
  PLEV    = 1
  LEVS    = -5,-4,-3,-2,-1,-0.5,-0.25,0.25,0.5,1,2,3,4,5,10,20,30,40,50
  BEAM
  XRANGE  = 10,85
  YRANGE  = 43,118
  LINEWIDT= {LW}
  /
  MAPFILE = DAISYG.MAP
  FORMAT  = INVERT
  /
$!----------------------------------------------------------------------
$ EXIT: EXIT
