$!----------------------------------------------------------------------
$!  This procedure tests the sequence: FAKE, AMPHI, INVERT, CLEAN.
$!  The model used in AMPHI is a point source.
$!  Programs LIST and VISPLOT are also tested.
$!
$! Revised 1986 Dec 23 - new syntax for LIST, VISPLOT, UVPLOT (TJP).
$! Revised 1988 Jun 23 - new syntax for AMPHI, CLEAn, INVERT (TJP).
$!----------------------------------------------------------------------
$!
$ PROC = F$ENVIRONMENT("PROCEDURE")
$ NAME = F$PARSE(PROC,,,"NAME","SYNTAX_ONLY")
$!
$ SET DEFAULT SYS$SCRATCH
$ ON WARNING THEN GOTO EXIT
$!
$! Change the following definitions to match your printer:
$!
$ PLOTTER = "PRINTRONIX"
$ LW = 1    ! line-width for plotting
$!
$ CREATE DAISYG.MOD
! Daisy model with Gaussian components (TJP, 15-Nov-82)
!
     1.000000      0.0          0.0          0.9      1.0      0.0   1
     0.050000      1.99991     90.00219      1.0      1.0      0.0   1
     0.050000      3.99995     90.00040      1.0      1.0      0.0   1
     0.050000      5.99995     90.00027      1.0      1.0      0.0   1
     0.050000      8.0         90.00055      1.0      1.0      0.0   1
     0.050000      9.99990     90.0          1.0      1.0      0.0   1
     0.050000      2.00015     59.99922      1.0      1.0      0.0   1
     0.050000      4.00019     59.99779      1.0      1.0      0.0   1
     0.050000      5.99975     60.00362      1.0      1.0      0.0   1
     0.050000      7.99989     59.99959      1.0      1.0      0.0   1
     0.050000      9.99971     59.99938      1.0      1.0      0.0   1
     0.050000      2.00015     30.00078      1.0      1.0      0.0   1
     0.050000      4.00019     30.00222      1.0      1.0      0.0   1
     0.050000      5.99975     29.99638      1.0      1.0      0.0   1
     0.050000      7.99989     30.00041      1.0      1.0      0.0   1
     0.050000      9.99971     30.00063      1.0      1.0      0.0   1
     0.050000      1.99991     -0.00219      1.0      1.0      0.0   1
     0.050000      3.99995     -0.00040      1.0      1.0      0.0   1
     0.050000      5.99995     -0.00026      1.0      1.0      0.0   1
     0.050000      8.0         -0.00055      1.0      1.0      0.0   1
     0.050000      9.99990      0.0          1.0      1.0      0.0   1
$ CREATE POINT.MOD
! Point source model for use by AMPHI
     2.000000      0.0          0.0          0.0      0.0      0.0   0
$ ASSIGN/USER FAKEFAKE.LIS FOR006
$ FAKE
  OUTPUT        = DAISYG.MRG
  MODEL         = DAISYG.MOD
  RA            = 18:03:39.177
  DEC           = 78:27:54.21
  YEAR          = 1982
  DAY           = 95
  START         = 06:00:00
  STOP          = 18:00:00
  INTEG         = 600
  FREQ          = 4989
  SOURCE        = DAISY_G
  NUP           = 3
  STATIONS      = BONN, HSTK, NRAO, FDVS, OVRO
    EFFICIENCY  = 0.5,  0.5,  0.5,  0.5,  0.5
    DIAMETER    = 100,  34,   43,   25,   40
    POINTING    =   1,   1,    1,    1,    1
    TSYS        =  75,  120,  40,   120,  70
  BANDWIDTH     = 1.8
  ERRMULT       = 0.01
  /
$!
$ LIST
  INPUT = DAISYG.MRG
  LIST  = FAKELIST.LIS
  HEADER 
  /
$!
$ VISPLOT
  INPUT = DAISYG.MRG
  NP=10 LINEWIDTH={LW} HTLET=0.06 
  IHA 
  PLOTFILE = FAKESTRT.PLT/{PLOTTER}
  MODFILE = DAISYG.MOD  MODEL=200
  /
$ AMPHI
  INPUT   = DAISYG.MRG
  OUTPUT  = DAISYG.ADJ
  MODFILE = POINT.MOD
  LISTFILE= FAKEAMP.LIS
  /
$ INVERT
  INPUT  = DAISYG.ADJ
  MAPFILE = DAISYG.MAP
  BEAMFILE= DAISYG.BMM
  PLOTFILE= FAKEINV.PLT/{PLOTTER}
  LISTFILE= FAKEINV.LIS
  XYINT   = 0.25  MILLIARCSEC
  FLUX   = 2.0
  PRINTUV
  PLOTMAP
  PLOTBEAM = 5
  /
$ CLEAN
  MAPFILE  = DAISYG.MAP
  BEAMFILE= DAISYG.BMM
  PLOTFILE= FAKECLN.PLT/{PLOTTER}
  LISTFILE= FAKECLN.LIS
  BEAM   = 1.0  ! 10% smaller than dirty beam
  PRINTRESIDUALS
  NITER   = 800
  MODEL   = DAISYG.MDL
  CLEANMAP= DAISYG.CLM
  PLOTWINDOW
  PLOTMAP
  TITLE   = "  ({name})"
  CONTOURS= -5,-4,-3,-2,-1,1,2,3,4,5,10,20,30,40,50
  /
$ VISPLOT
  INPUT = DAISYG.MRG
  PPRINT NP=10 LINEWIDTH={LW} HTLET=0.06 
  IHA 
  PLOTFILE = FAKEFIT.PLT/{PLOTTER}  MODEL=200 
  MODFILE = DAISYG.MDL
$!
$ PRINT -
  'PROC',-
  SYS$SCRATCH:FAKEFAKE.LIS,-
  FAKELIST.LIS,-
  FAKESTRT.PLT/PASSALL,-
  FAKEAMP.LIS,-
  FAKEINV.LIS,-
  FAKEINV.PLT/PASSALL,-
  FAKECLN.LIS,-
  FAKECLN.PLT/PASSALL,-
  FAKEFIT.PLT/PASSALL
$ EXIT: EXIT
