C*GCONVL -- convolve two elliptical Gaussians
C+
      SUBROUTINE GCONVL (AX1,AR1,PHI1, AX2,AR2,PHI2, AX0,AR0,PHI0)
      REAL     AX1, AR1, PHI1, AX2, AR2, PHI2, AX0, AR0, PHI0
C
C GCONVL computes the parameters of the elliptical Gaussian formed by
C convolving two given elliptical Gaussians (e.g., a model component
C and an elliptical Gaussian beam). The Gaussians are defined by major
C axis AX (usually the FWHM, but the convention is irrelevant in this
C subroutine; the units are arbitrary), the axial ratio AR (<= 1), and
C the major-axis position angle PHI (radians). 
C
C Reference:
C  J. P. Wild, Aust. J. Phys, v23, pp113-115 (1970).
C
C Arguments:
C  AX1, AR1, PHI1 (input, real): parameters of first Gaussian.
C  AX2, AR2, PHI2 (input, real): parameters of second Gaussian.
C  AX0, AR0, PHI0 (output, real): parameters of resultant Gaussian.
C
C History:
C  1985 Jan  2 - T. J. Pearson.
C-----------------------------------------------------------------------
      REAL     S1, D1, D1SIN, D1COS, S2, D2, D2SIN, D2COS
      REAL     S0, D0, D0SIN, D0COS
C
C Compute the (S,D) parameters of the input Gaussians. S is the Sum and
C D is the Difference of the squares of the major and minor axes.
C
      S1    = (AX1**2)*(1.0+(AR1**2))
      D1    = (AX1**2)*(1.0-(AR1**2))
      D1SIN = D1*SIN(2.0*PHI1)
      D1COS = D1*COS(2.0*PHI1)
      S2    = (AX2**2)*(1.0+(AR2**2))
      D2    = (AX2**2)*(1.0-(AR2**2))
      D2SIN = D2*SIN(2.0*PHI2)
      D2COS = D2*COS(2.0*PHI2)
C
C Add the parameters to get the (S,D) of the resultant.
C The extension to more than two Gaussians is obvious.
C
      S0    = S1 + S2
      D0SIN = D1SIN + D2SIN
      D0COS = D1COS + D2COS
C
C Convert to (AX, AR, PHI) representation.
C
      D0    = SQRT(D0SIN**2 + D0COS**2)
      IF (D0.EQ.0) THEN
          PHI0 = 0.0
      ELSE
          PHI0 = ATAN2(D0SIN,D0COS)/2.0
      END IF
      AX0   = SQRT((S0+D0)/2.0)
      AR0   = SQRT((S0-D0)/2.0)/AX0
C
C And that's all there is to it.
C
      END
