C*HEADX30 -- read header from Merge-format file (up to 30 stations)
C+
      SUBROUTINE HEADX30 (SKIPH,INUNIT,IZERO,IDAY,IYEAR,
     1                    NEWHIS,NHN,IOUT,IUNITS,XVALS,IFILE,IPROUT)
      LOGICAL SKIPH
      INTEGER INUNIT, IZERO, IDAY, IYEAR
      INTEGER NEWHIS(20,1)
      INTEGER NHN, IOUT, IUNITS
      DOUBLE PRECISION XVALS(170)
      INTEGER IFILE, IPROUT
C
C HEADX30 is the same as routine HEADX20, but dimensioned for 30 
C stations. See HEADX20 for details.
C-----------------------------------------------------------------------
C
C Constants.
C
      INTEGER   NS, NB, NC, ICON
C     -- Number of stations
      PARAMETER (NS=30)
C     -- Number of baselines
      PARAMETER (NB=NS*(NS-1)/2)
C     -- Number of closure phases
      PARAMETER (NC=NB*(NS-2)/3)
C     -- 60ths of seconds per day
      PARAMETER (ICON=5184000)
      DOUBLE PRECISION PI, TWOPI, RPDEG, RPAS, RPSEC, HRAD, SIDR
C     -- pi
      PARAMETER (PI=3.14159265359D0)
C     -- two times pi
      PARAMETER (TWOPI=PI*2D0)
C     -- radians per degree
      PARAMETER (RPDEG=PI/180D0)
C     -- radians per arcsecond
      PARAMETER (RPAS=RPDEG/3600D0)
C     -- radians per second of time
      PARAMETER (RPSEC=PI/12D0/3600D0)
C     -- Hours of time per radian
      PARAMETER (HRAD=12D0/PI)
C     -- sidereal rate / solar rate
      PARAMETER (SIDR=1.00273791)
C-----------------------------------------------------------------------
C
C Variables.
C
      DOUBLE PRECISION GST,GASTM,FREQ,TINTG,AVG,BW
      DOUBLE PRECISION STNAME,STX,STY,STZ,WORD(9),AEIOWM
      DOUBLE PRECISION SOURCE(2),RA,DEC,RAP,DECP,FLUX
      INTEGER   AA(20)
      INTEGER   IDUM(20),STATYP
      INTEGER   NHIST, NHREC, LREC, NHRO, NHOUT, IUT
      INTEGER   I, J, K
      INTEGER   LEN1
      CHARACTER*16 IA,JA,KA,LA
      LOGICAL ORBIT,EXTHIS
      CHARACTER*128 FILENM
C
      COMMON /PARMS/STNAME(NS),STX(NS),STY(NS),STZ(NS),SOURCE,RAP,
     +              DECP,GASTM,FREQ,FLUX,TINTG,AVG,BW,AEIOWM(6,NS),
     +              STATYP(NS),ORBIT
      COMMON /PARM2/RA,DEC
      INTEGER NSTA,NBAS,NCLO,NB1(NB),NB2(NB),NC1(NC),NC2(NC),
     +              NC3(NC),IVERSN,NDWPB
      COMMON /ORDER/NSTA,NBAS,NCLO,NB1,NB2,NC1,NC2,NC3,IVERSN,NDWPB
C-----------------------------------------------------------------------
C
C Read first header record. Check that the format
C version is 1 or 2, and compute the number of
C data words per baseline (NDWPB).
C
      READ (INUNIT, END=40, ERR=20) IVERSN,NHIST,NHREC,NSTA,NBAS,NCLO,
     1                                    LREC,IUNITS,(IDUM(I),I=1,12)
      IF (IVERSN.EQ.1) THEN
          NDWPB = 4
      ELSE IF (IVERSN.EQ.2) THEN
          NDWPB = 7
      ELSE
          NDWPB = 0
          WRITE (6,601) IVERSN
          GOTO 30
      END IF
      IF (NSTA.GT.NS) THEN
          WRITE (6,604) NSTA,NS
          GOTO 30
      END IF
C
C Skip over header records if SKIPH = .TRUE., and then
C return immediately. The number of records to skip is
C parameter NHREC from the first header record.
C
      IF (SKIPH) THEN
          DO 210 I=1,NHREC
            READ (INUNIT, END=20, ERR=20)
  210     CONTINUE
          GOTO 10
      END IF
C
C Parameter changes for first record; this includes
C updating the total number of header records and the
C number of history records to include the new
C history records from NEWHIS and XVALS.
C
      IF (IOUT.NE.0) THEN
          IF (XVALS(1).NE.0D0)  IUNITS = XVALS(1)
          EXTHIS = .TRUE.
          DO 220 I=2,11
            IF (XVALS(I).EQ.0D0) EXTHIS = .FALSE.
  220     CONTINUE
          NHRO = NHREC + NHN
          NHOUT = NHIST + NHN
          IF (EXTHIS) NHRO = NHRO + 1
          IF (EXTHIS) NHOUT = NHOUT + 1
          WRITE (IOUT) IVERSN,NHOUT,NHRO,NSTA,NBAS,NCLO,
     +                        LREC,IUNITS,(IDUM(I),I=1,12)
      END IF
C
C Listing of first record.
C
      IF (IPROUT.NE.0) THEN
          WRITE (IPROUT,1099) IVERSN,NSTA,NHIST,NBAS,NHREC,NCLO,LREC
          IF (IUNITS.EQ.1) WRITE (IPROUT,1097)
          IF (IUNITS.EQ.0) WRITE (IPROUT,1098)
          IF (IUNITS.EQ.2) WRITE (IPROUT,1096)
      END IF
C
C Baseline and Triangle indices.
C
      IF (NBAS.GT.0) THEN
          READ (INUNIT, END=20, ERR=20) (NB2(J),J=1,NBAS)
          IF (IOUT.NE.0) WRITE (IOUT) (NB2(J),J=1,NBAS)
          DO 230 K=1,NBAS
            NB1(K) = NB2(K) / 100
            NB2(K) = NB2(K) -  NB1(K)*100
  230     CONTINUE
          IF (IPROUT.NE.0) WRITE(IPROUT,1031) (NB1(K),NB2(K),K=1,NBAS)
      END IF
      IF (NCLO.GT.0) THEN
          READ (INUNIT, END=20, ERR=20) (NC3(J),J=1,NCLO)
          IF (IOUT.NE.0) WRITE (IOUT) (NC3(J),J=1,NCLO)
          DO 240 K = 1,NCLO
            NC1(K) = NC3(K) / 10000
            NC2(K) = NC3(K) / 100   -  NC1(K) * 100
            NC3(K) = NC3(K) - NC1(K)*10000 - NC2(K)*100
  240     CONTINUE
          IF (IPROUT.NE.0) WRITE (IPROUT,1036) 
     1                                 (NC1(K),NC2(K),NC3(K),K=1,NCLO)
      END IF
C
C History records.
C
      IF (IPROUT.NE.0) WRITE(IPROUT,1000)
      DO 250 I=1,NHIST
          READ (INUNIT,END=20, ERR=20) AA
          IF (IPROUT.NE.0) WRITE (IPROUT,100) AA
          IF (IOUT.NE.0)   WRITE (IOUT) AA
  250 CONTINUE
      IF (IOUT.NE.0 .AND. IPROUT.NE.0) THEN
          WRITE (IPROUT,101) ((NEWHIS(I,J),I=1,20),J=1,NHN)
          IF (EXTHIS) WRITE (IPROUT,201) (XVALS(I),I=2,11)
      END IF
      IF (IOUT.NE.0) THEN
          DO 260 J=1,NHN
            WRITE (IOUT) (NEWHIS(I,J),I=1,20)
  260     CONTINUE
          IF (EXTHIS) WRITE(IOUT) (XVALS(I),I=2,11)
      END IF
C
C Source information record.
C
      READ (INUNIT,END=20, ERR=20) SOURCE,RA,DEC,RAP,DECP,FLUX
      IF (IOUT.NE.0) THEN
          IF (XVALS(43).NE.0D0 .AND. XVALS(44).NE.0D0) THEN
            SOURCE(1) = XVALS(43)
            SOURCE(2) = XVALS(44)
          END IF
          IF (XVALS(13).NE.0D0) RA   = XVALS(13) * RPSEC
          IF (XVALS(14).NE.0D0) DEC  = XVALS(14) * RPAS
          IF (XVALS(15).NE.0D0) RAP  = XVALS(15) * RPSEC
          IF (XVALS(16).NE.0D0) DECP = XVALS(16) * RPAS
          IF (XVALS(17).NE.0D0) FLUX = XVALS(17)
          WRITE (IOUT) SOURCE,RA,DEC,RAP,DECP,FLUX
      END IF
      IF (IPROUT.NE.0) THEN
          CALL PANGLE('H',RA,  IA, 3)
          CALL PANGLE('D',DEC, JA, 3)
          CALL PANGLE('H',RAP, KA, 3)
          CALL PANGLE('D',DECP,LA, 3)
          WRITE (IPROUT,1001) SOURCE,IA,JA,KA,LA,FLUX
      END IF
C
C Station records.
C
      IF (IPROUT.NE.0) WRITE (IPROUT,102)
      ORBIT = .FALSE.
      DO 300 I=1,NSTA
          IF (IVERSN.EQ.1) THEN
              READ (INUNIT, END=20, ERR=20) STNAME(I),STX(I),STY(I),
     1                                          STZ(I)
            STATYP(I) = 0
          ELSE IF (IVERSN.EQ.2) THEN
              READ (INUNIT, END=20, ERR=20) STNAME(I),(WORD(J),J=1,9),
     1                                            STATYP(I)
            IF (STATYP(I).LE.1) THEN
                STX(I) = WORD(1)
                STY(I) = WORD(2)
                STZ(I) = WORD(3)
                DO 270 J=1,6
                  AEIOWM(J,I) = 0D0
  270           CONTINUE
            ELSE IF (STATYP(I).EQ.2) THEN
                ORBIT = .TRUE.
                STX(I) = 0D0
                STY(I) = 0D0
                STZ(I) = 0D0
                DO 280 J=1,6
                  AEIOWM(J,I) = WORD(J)
  280           CONTINUE
            END IF
          END IF
          IF (IOUT.NE.0) THEN
            IF (XVALS(50+I).NE.0D0)   STNAME(I) = XVALS(50+I)
            IF (XVALS(50+NS+I).NE.0D0)   STX(I) = XVALS(50+NS+I)
            IF (XVALS(50+2*NS+I).NE.0D0) STY(I) = XVALS(50+2*NS+I)
            IF (XVALS(50+3*NS+I).NE.0D0) STZ(I) = XVALS(50+3*NS+I)
            IF (IVERSN.EQ.1) THEN
                WRITE (IOUT) STNAME(I),STX(I),STY(I),STZ(I)
            ELSE IF (IVERSN.EQ.2) THEN
                IF (STATYP(I).LE.1) WRITE(IOUT) STNAME(I),STX(I),
     1                  STY(I),STZ(I),(0D0,J=5,10),STATYP(I)
                IF (STATYP(I).EQ.2) WRITE(IOUT) STNAME(I),
     1                  (AEIOWM(J,I),J=1,6),(0D0,J=8,10),STATYP(I)
            END IF
          END IF
          IF (IPROUT.NE.0 .AND. STATYP(I).LE.1)
     1            WRITE(IPROUT,104) STNAME(I),STX(I),STY(I),STZ(I)
C
  300 CONTINUE
      IF (IPROUT.NE.0 .AND. ORBIT) THEN
          WRITE (IPROUT,106)
          DO 320 I=1,NSTA
            IF (STATYP(I).EQ.2) THEN
                DO 310 J=3,6
                  WORD(J) = AEIOWM(J,I)/RPDEG
  310           CONTINUE
                WRITE(IPROUT,107) STNAME(I),(AEIOWM(J,I),J=1,2),
     1                  (WORD(J),J=3,6)
            END IF
  320     CONTINUE
      END IF
C
C            Final record.
C
      READ (INUNIT, END=20, ERR=20)  IYEAR,IUT,GST,FREQ,TINTG,
     1                               AVG,BW,(IDUM(I),I=1,8)
      IF (IOUT.NE.0) THEN
          IF (XVALS(38).NE.0D0)  IYEAR  = XVALS(38)
          IF (XVALS(12).NE.0D0 .OR. 
     1              XVALS(39).NE.0D0)  IUT    = XVALS(12)*ICON + 
     2                                          XVALS(39)*RPSEC
          IF (XVALS(40).NE.0D0)  GST    = XVALS(40)*RPSEC
          IF (XVALS(41).NE.0D0)  FREQ   = XVALS(41)
          IF (XVALS(42).NE.0D0)  TINTG  = XVALS(42)
          IF (XVALS(45).NE.0D0)  AVG    = XVALS(45)
          IF (XVALS(18).NE.0D0)  BW     = XVALS(18)
          WRITE (IOUT) IYEAR,IUT,GST,FREQ,TINTG,AVG,BW,(IDUM(I),I=1,8)
      END IF
      IDAY = IUT/ICON + 1
      IZERO = (IDAY-1) * ICON
      GASTM = GST - (IUT-IZERO)*SIDR/(60.*3600.*HRAD)
      GASTM = DMOD(GASTM,TWOPI)
      IF (GASTM.LT.0.D0) GASTM = TWOPI + GASTM
      IF (IPROUT.NE.0) THEN
          CALL PANGLE('H', GASTM, IA, 3)
          WRITE (IPROUT,105) IYEAR,IDAY,IA,FREQ,TINTG,AVG,BW
      END IF
C
C            Normal exit. Set IFILE=0 to indicate success.
C
   10 IFILE = 0
      RETURN
C
C            Error exit. We get here if any of the READ statements
C            takes an ERR= or END= exit.
C
   20 WRITE (6,603)
   30 INQUIRE (UNIT=INUNIT, NAME=FILENM)
      WRITE (6,602) FILENM(1:LEN1(FILENM))
C
C            Error exit. We get here if the first READ
C            encounters an end-of-file, or if any fatal error
C            occurs. Set IFILE to 1 to indicate this.
C
   40 IFILE = 1
      RETURN
C-----------------------------------------------------------------------
C Format statements
C-----------------------------------------------------------------------
 1099 FORMAT (T15,'==========  COUNTERS  =========='/
     1        T10,'Version ',I5,T40,I5,' stations'/
     2        T10,I5,' history records',T40,I5,' baselines'/
     3        T10,I5,' header records',T40,I5,' closure phases'/
     4        T10,'Record length = ',I5)
 1098 FORMAT (T10,'Amplitudes are correlation coefficients')
 1097 FORMAT (T10,'Amplitudes are in Janskys')
 1096 FORMAT (T10,'Amplitudes are actually fringe rates in Hz')
 1031 FORMAT ('    Baselines'/
     1        5(1X,10('(',I2,',',I2,')',:,1X)/))
 1036 FORMAT ('    Closure phases'/
     1        24(1X,5('(',I2,',',I2,',',I2,')',:,1X)/))
 1000 FORMAT (T15,'==========  HISTORY RECORDS  =============='/)
  100 FORMAT (1X,20A4)
  101 FORMAT (T15,'==========  NEW HISTORY RECORDS  =========='/
     1        (1X,20A4))
  201 FORMAT (1X,10A8)
 1001 FORMAT (T15,'==========  SOURCE PARAMETERS  ============'/
     1        T17,'Name  = ',2A8/
     2        T17,'RA    = ',A16,'  (1950)'/
     3        T17,'Dec   = ',A16/
     4        T17,'RA    = ',A16,'  (precessed)'/
     5        T17,'Dec   = ',A16/
     6        T17,'Flux  = ',F10.3,' Jy')
  102 FORMAT (T15,'==========  STATION PARAMETERS  ==========='/
     1        T5,'Station name',T29,'X',T50,'Y',T69,'Z')
  104 FORMAT (1X,5X,A8,T17,3(8X,F12.3))
  105 FORMAT (T15,'==========  OTHER PARAMETERS =============='/
     1        T17,'Year = ',I6/
     2        T17,'GST at 0 hr UT on day',I4,' is ',A16/
     3        T17,'Frequency = ',F15.2/
     4        T17,'Coherent integration time = ',F8.1,' seconds'/
     5        T17,'Average time = ',F8.1,' seconds'/
     6        T17,'Bandwidth = ',F15.2)
  106 FORMAT (T15,'==========  ORBITER PARAMETERS  ==========='/
     1        T5,'Orbiter name',T29,'A',T45,'E',T58,'I',T71,'O',T84,
     2        'W',T97,'M')
  107 FORMAT (1X,5X,A8,T17,8X,F12.3,5X,F6.3,4(5X,F8.3))
  601 FORMAT (' *** Invalid format for MERGE file: version = ',I10)
  603 FORMAT (' *** File has wrong format for a MERGE file:')
  602 FORMAT (' *** file: ',A)
  604 FORMAT (' *** File has ',I2,' stations; maximum allowed is ',I2)
C-----------------------------------------------------------------------
      END
