C*MRGSIZ -- determine size of Merge file
C+
      SUBROUTINE MRGSIZ(INDAT, NS, NB, NT, NREC, NP)
      INTEGER INDAT
      INTEGER NS, NB, NT, NREC, NP
C
C Read a Merge file to determine the size parameters needed for
C subsequent analysis.
C
C Arguments:
C  INDAT (input)   : Fortran unit on which the Merge file is open
C                    (use routine VLBOPE to open the file).
C  NS (output)     : number of stations (antennas).
C  NB (output)     : number of baselines (max NS*(NS-1)/2).
C  NT (output)     : number of closure triangles.
C  NREC (output)   : number of data records (distinct times).
C  NP (output)     : number of complex visibilities.
C
C Subroutines required:
C  ERROR   (CITVLB)
C
C History:
C  1989 Dec 13 - TJP
C  1990 Feb 13 - correct error for format=2 files (TJP).
C  1991 Jan  7 - remove undeclared variables (TJP).
C  1991 Mar 22 - allow up to 40 stations (TJP).
C-----------------------------------------------------------------------
C
C Constants.
C
      INTEGER       MAXS,MAXB,MAXT
      PARAMETER     (MAXS=40)
      PARAMETER     (MAXB=(MAXS-1)*MAXS/2)
      PARAMETER     (MAXT=MAXB*(MAXS-2)/3)
C
C Variables.
C
      CHARACTER*8  STNAME(MAXS)
      CHARACTER*16 SOURCE
      CHARACTER*80 HISTRY
      INTEGER   IUT, IS, IB, IT, I, J, NDWPB
      INTEGER   IZ, IY, NHIST
      INTEGER   NBI(MAXB), NB1(MAXB), NB2(MAXB)
      INTEGER   NTI(MAXT)
      INTEGER   REC1(20), IDUM(8)
      REAL      INREC(7*MAXB)
      DOUBLEPRECISION STX(MAXS), STY(MAXS), STZ(MAXS)
      DOUBLEPRECISION RA, DEC, RAP, DECP, GASTM, FREQ, FLUX
      DOUBLEPRECISION TINTG, AVG, BW
C-----------------------------------------------------------------------
C
C Read input header; check format level.
C
      READ (INDAT,END=180,ERR=180) (REC1(J),J=1,20)
      IF (REC1(1).EQ.1) THEN
          NDWPB = 4
      ELSE IF (REC1(1).EQ.2) THEN
          NDWPB = 7
      ELSE
          GOTO 180
      END IF
      NHIST = REC1(2)
      NS    = REC1(4)
      NB    = REC1(5)
      NT    = REC1(6)
      IF (NS.LT.2) CALL ERROR('Not enough stations.')
      IF (NS.GT.MAXS) CALL ERROR('Too many stations.')
      IF (NB.GT.0) THEN
          READ (INDAT,END=180,ERR=180) (NBI(IB),IB=1,NB)
          DO 110 IB=1,NB
              NB1(IB) = NBI(IB)/100
              NB2(IB) = NBI(IB) -  NB1(IB)*100
  110     CONTINUE
      END IF
      IF (NT.GT.0) THEN
          READ (INDAT,END=180,ERR=180) (NTI(IT),IT=1,NT)
      END IF
      DO 120 I=1,NHIST
          READ (INDAT,END=180,ERR=180) HISTRY
  120 CONTINUE
      READ (INDAT,END=180,ERR=180) SOURCE,RA,DEC,RAP,DECP,FLUX
      DO 130 IS=1,NS
          READ (INDAT,END=180,ERR=180)
     1          STNAME(IS),STX(IS),STY(IS),STZ(IS)
  130 CONTINUE
      READ (INDAT,END=180,ERR=180) IY, IZ, GASTM, FREQ, TINTG,
     1            AVG, BW, (IDUM(I),I=1,8)
C
C Read data records and count data.
C
      NREC = 0
      NP = 0
  160 READ (INDAT,ERR=180,END=190) IUT,(INREC(J),J=1,NB*NDWPB)
      IF (IUT.LT.0) GOTO 190
      NREC = NREC + 1
      DO 170 IB=1,NB
          IF (INREC(NDWPB*(IB-1)+2).GT.0.0 .AND.
     *        INREC(NDWPB*(IB-1)+4).GT.0.0) NP = NP+1
  170 CONTINUE
      GOTO 160
C
C Error return.
C
  180 NS = 0
      NB = 0
      NT = 0
      NREC = 0
      NP = 0
      RETURN
C
C Normal return.
C
  190 RETURN
C
      END
