      INTEGER FUNCTION RDSRC( IUNIT, OPENIT, FILE )
C
C    RDSRC is a routine to read an entry of the source catalog.
C    The catalog is assumed to be in a KEYIN readable format.
C
C    Input parameters:
      INTEGER        IUNIT    ! Unit number read from.
      LOGICAL        OPENIT   ! Open file on first call.  See below.
      CHARACTER*(*)  FILE     ! File name of catalog.
C
C    The value of RDSRC on return is:
C         0  if a line was successfully read.
C         1  if an end-of-file or an "ENDCAT" is encountered.  All catalog
C            entries should terminate with a '/' so if RDSRC=1, assume that
C            no new data has been read.
C
C    Data from the catalog is returned via COMMON's specified in the 
C    include file, rdcat.inc.  The calling program should use that 
C    file.  If new parameters are added to the catalog, it should 
C    only be necessary to relink old programs that don't use them.
C
C    OPENIT controls whether or not the catalog file is opened on
C    the first read and closed when an EOF or parameter ENDCAT is reached.
C    Set OPENIT=.TRUE. for normal reading of external catalog files.
C    Set OPENIT=.FALSE. if no open or close is desired - eg. if the 
C    catalog information is imbedded in the calling program's main input.
C    Use ENDCAT to terminate catalog input in such cases.
C
C    The number of variables should be kept small for efficient reading
C    of large catalogs.
C
C    R. C. Walker, Dec. 19, 1988.
C
C    Include file.
C
      INCLUDE 'rdcat.inc'
C
C     Constants:
C
      REAL*8         PI, RADDS, RADHS, UNSET
      PARAMETER      (PI=3.141592653589793238D0)
      PARAMETER      (RADDS=PI/180.D0/3600.D0)
      PARAMETER      (RADHS=PI/12.D0/3600.D0)
      PARAMETER      (UNSET=-9999.D0)
C
C     Internal variables.
      INTEGER        MODE, IER, VLBOPE, I
      REAL           CONT
      REAL*8         BLANK1, NONAME
      LOGICAL        FOPEN
      CHARACTER*50   CNAME
      CHARACTER*255  RESULT
C
C     Catalog input parameters.
C
      INTEGER        NSRP
      PARAMETER      (NSRP=20)
      REAL*8         RNAMES(NSRP), RVALUE(NSRP), ENDMRK
      DATA   RNAMES(1)            / 'Source' /
      DATA   (RNAMES(I),I=2,3)    / 'Ra',  'Dec' /
      DATA   (RNAMES(I),I=4,5)    / 'Epoch', 'Calcode' /
      DATA   (RNAMES(I),I=6,14)   / 8*'Velocity', 'Endcat' /
      DATA   (RNAMES(I),I=15,18)  / 4*'PMEPOCH' / 
      DATA   (RNAMES(I),I=19,20)  / 'DRA', 'DDEC' /
      DATA   RVALUE(4)            /  'UNSET' /
      DATA   ENDMRK               / '/' /
C
      DATA   BLANK1               / ' ' /
      DATA   NONAME               / 'NONAME' /
      DATA   FOPEN                / .FALSE. /
      DATA   CONT                 / 'CONT' /
C
C     Open the file if requested.
C
      IF( OPENIT .AND. .NOT. FOPEN ) THEN
         IER = VLBOPE( IUNIT, FILE, 'TEXT', 'OLD', RESULT )
         IF( IER .EQ. 0 ) THEN
            CALL PUTOUT( RESULT )
            CNAME = FILE
            CALL ERROR( 'Source catalog: '//CNAME//' not opened' )
         END IF
         FOPEN = .TRUE. 
C         
      END IF
C
C     Set parameters defaults before reading new line.  They are:
C       SOURCE='NONAME', RA=0, DEC=-90deg, and CALCODE=' '.
C       EPOCH defaults to the previous value so it 
C       need not be given for every source, although that is safer 
C       in case a source of different epoch is inserted into the catalog.
C
      MODE = 0
      RVALUE(1) = NONAME
      RVALUE(2) = 0.D0
      RVALUE(3) = -324000.D0      
      RVALUE(5) = BLANK1
      DO I = 1, 8
         RVALUE(I+5) = UNSET
      END DO
      RVALUE(14) = UNSET
      DO I = 15, 20
         RVALUE(I) = 0.D0
      END DO
C
C     Read the catalog line with KEYIN.  
C
      CALL KEYIN( RNAMES, RVALUE, NSRP, ENDMRK, MODE, IUNIT, 6 )
C
      IF( MODE .EQ. 1 .OR. RVALUE(14) .EQ. 0.D0 ) THEN
         IF( OPENIT .AND. FOPEN ) CLOSE( UNIT=IUNIT )
         FOPEN = .FALSE.
         RDSRC = 1
         GO TO 990
      END IF
C
      IF( RVALUE(4) .EQ. UNSET ) CALL ERROR( 
     1       'Epoch of source catalog not specified' )
C
      RDSRC = 0
      WRITE( SRCNAM, '(A8)' ) RVALUE(1)
      SRCRA = RVALUE(2) * RADHS
      SRCDEC = RVALUE(3) * RADDS
      WRITE( SRCEPO, '(A5)' ) RVALUE(4)
      CALL UPCASE( SRCEPO ) 
      WRITE( SRCCAL, '(A1)' ) RVALUE(5)
      CALL UPCASE( SRCCAL )
      DO I = 1, 8
         SRCVEL(I) = RVALUE(I+5)
         IF( RVALUE(I+5) .EQ. UNSET ) SRCVEL(I) = CONT
      END DO
      SRCPMY = RVALUE(15)
      SRCPMM = RVALUE(16)
      SRCPMD = RVALUE(17)
      SRCPMT = RVALUE(18)*RADHS
      SRCDRA = RVALUE(19)
      SRCDDEC = RVALUE(20)
C
C     Make sure epoch is valid.
C
      IF( SRCEPO .NE. 'J2000' .AND. SRCEPO .NE. 'B1950' .AND. 
     1    SRCEPO .NE. 'DATE' )  THEN
         CALL PUTOUT( 'Source catalog contains invalid epoch: '//
     1                 SRCEPO )
         CALL ERROR( ' Epoch must be J2000, B1950, or DATE ' )
      END IF
C
  990 RETURN
      END
