      SUBROUTINE PGELLIP (N,XC,YC,A,B,ANGLE,NHATCH)
C-----------------------------------------------------------------------
C PGPLOT: Draw an ellipse in the current window.
C
C Arguments:
C
C N (input, integer) : number of points defining the ellipse; the
C      more points, the better the ellipse.
C XC (input, real) : world x-coordinate of the center of the ellipse.
C YC (input, real) : world y-coordinate of the center of the ellipse.
C A (input, real) : semi-major axis of ellipse (world coordinates).
C B (input, real) : semi-minor axis of ellipse (world coordinates).
C ANGLE (input, real) : the angle in radians that the major axis makes
C      with the x-axis of world coordinates; if x=0 or pi, the
C      major axis is parallel to the x-axis; if x = pi/2, the major
C      axis is parallel to the y-axis.
C NHATCH (input, integer) : number of cross-hatching lines required.
C      If NHATCH=1, the cross-hatching lines will coincide with the
C      ellipse's axes. If NHATCH < 1, the ellipse will not be hatched.
C
C Side effects:
C
C The pen position is changed to (XC,YC) in world coordinates.
C
C (7-Jan-1991)
C-----------------------------------------------------------------------
      IMPLICIT NONE
      INTEGER  I, NHATCH, N
      REAL     XC, YC, XP, YP, A, B, ANGLE, CA, SA, DA
      REAL PI
      PARAMETER (PI=3.14159265359)
C
      IF (A.LT.0.0 .OR. B.LT.0.0 .OR. A.LT.B .OR. N.LT.2) RETURN
C
      CA = COS(ANGLE)
      SA = SIN(ANGLE)
      DA = 2*PI/N
C
C Draw outline.
C
      CALL PGBBUF
      CALL PGMOVE(XC+A*CA,YC+A*SA)
      DO I=1,N
          XP = A*COS(I*DA)
          YP = B*SIN(I*DA)
          CALL PGDRAW(XC+XP*CA-YP*SA,YC+YP*CA+XP*SA)
      END DO
C
C Draw cross-hatching.
C
      IF (A.LE.0.0 .OR. B.LE.0.0) GOTO 100
      DA = 2*A/(NHATCH+1)
      DO I=1,NHATCH
          XP = -A + I*DA
          YP = B*SQRT(1.0-(XP/A)**2)
          CALL PGMOVE(XC+XP*CA-YP*SA,YC+YP*CA+XP*SA)
          CALL PGDRAW(XC+XP*CA+YP*SA,YC-YP*CA+XP*SA)
      END DO
      DO I=1,NHATCH
          YP = -A + I*DA
          IF (ABS(YP).LT.B) THEN
              XP = A*SQRT(1.0-(YP/B)**2)
            CALL PGMOVE(XC+XP*CA-YP*SA,YC+YP*CA+XP*SA)
            CALL PGDRAW(XC-XP*CA-YP*SA,YC+YP*CA-XP*SA)
          END IF
      END DO
C
  100 CALL PGMOVE(XC,YC)
      CALL PGEBUF
      RETURN
      END
