C*PHSAMP -- compute model at point in uv-plane
C+
      SUBROUTINE PHSAMP(UU,VV,AMOD,PMOD)
      REAL UU, VV
      REAL AMOD, PMOD
C
C Calculate model amplitude and phase at point (UU,VV)
C S.C. Unwin  --  1981 Sept 15
C Modified to use lookup tables for sin/cos, 1982 May 3
C Run time <50% of accurate evaluation of sin/cos
C Use a single 4096-point table. 1982 June 2
C Allow disks etc. in 'slow' calc. - V4.5.   1982 June 2
C Redefine AA to be 'diameter' of all types. 1982 June 8
C Fix bug in model reading: TYPE*2 -> TYPE*4.1982 Aug 28
C Trap ATAN2(0,0)                            1983 Mar 24
C Fixed SIN/COS lookup for Convex        JER 1990 Sep 20
C-----------------------------------------------------------------------
      INTEGER   MAXMOD
      PARAMETER (MAXMOD = 10000)
C
      INTEGER J, MASK
C     INTEGER ITEMP, INDEXS, INDEXC
      REAL    VISI, VISR, COMPHA, COMVIS
      REAL    BESJ0, BESJ1
      REAL    CPHI, SPHI, Q, QQ, XX
      REAL    FLUX(MAXMOD),XC(MAXMOD),YC(MAXMOD),AA(MAXMOD)
      REAL    R(MAXMOD),PHI(MAXMOD),LOOK(4096)
      INTEGER ITYPE(MAXMOD),NCOMP
      LOGICAL USELOO
      DATA MASK /4095/
      COMMON /MODEL/ NCOMP,FLUX,XC,YC,AA,R,PHI,ITYPE
      COMMON /LOOKUP/ USELOO,LOOK
C
      VISI = 0.0
      VISR = 0.0
C
C Use lookup table for sin/cos?  (Only for delta functions)
C Note: LOOKUP tables eliminated. This loop retained for vectorization.
C
      IF (USELOO) THEN
        DO 31 J=1,NCOMP
          COMPHA = 1.533980788E-3*(UU*XC(J)+VV*YC(J))
          VISI = VISI+FLUX(J)*SIN(COMPHA)
          VISR = VISR+FLUX(J)*COS(COMPHA)
   31   CONTINUE
C
C Compute sin/cos explicitly
C
      ELSE
        DO 30 J=1,NCOMP
C         -- Radians (see MAIN)
          COMPHA = 1.533980788E-3*(UU*XC(J)+VV*YC(J))
          IF (ABS(AA(J)).LT.5.0E-12) THEN
C             -- Delta function = < 0.001 mas
              COMVIS = FLUX(J)
          ELSE
              SPHI = SIN(PHI(J))
              CPHI = COS(PHI(J))
              Q = (VV*CPHI+UU*SPHI)**2 + (R(J)*(UU*CPHI-VV*SPHI))**2
              QQ = 3.141592654*AA(J)*SQRT(Q)
C             -- 2*pi*(diameter/2)*(radius/lambda), in radians
              IF     (ITYPE(J).LE.1)  THEN
C                 -- Gaussian (AA=FWHM)
                  COMVIS = FLUX(J)*EXP(-3.559707*Q*AA(J)**2)
              ELSE IF (ITYPE(J).EQ.2)  THEN
C                 -- Uniform disk (AA = diameter)
                  COMVIS = 2.0*FLUX(J)*BESJ1(QQ)/QQ
              ELSE IF (ITYPE(J).EQ.3)  THEN
C                 -- Tapered disk (AA=diameter)
                  COMVIS = 3.0*FLUX(J)*(SIN(QQ)-QQ*COS(QQ))/(QQ*QQ*QQ)
              ELSE IF (ITYPE(J).EQ.4)  THEN
C                 -- Ring (AA=diameter)
                  COMVIS = FLUX(J)*BESJ0(QQ)
              ELSE IF (ITYPE(J).EQ.5)  THEN
C                 -- Line (AA=length)
                  XX = AA(J)*3.141592654*(UU*SPHI+VV*CPHI)
                  IF(ABS(XX).LT.0.001) THEN
                      COMVIS = FLUX(J)
                  ELSE
                      COMVIS = FLUX(J)*SIN(XX)/XX
                  END IF
              ELSE
                  CALL ERROR(' Illegal component type in model. ')
              END IF
          END IF
          VISI = VISI+COMVIS*SIN(COMPHA)
          VISR = VISR+COMVIS*COS(COMPHA)
   30 CONTINUE
      END IF
C
      AMOD = SQRT(VISI*VISI+VISR*VISR)
      PMOD = 0.0
      IF (AMOD.NE.0.0) PMOD = 57.29577951*ATAN2(VISI,VISR)
      RETURN
      END
