      INTEGER FUNCTION RDSTA( IUNIT, OPENIT, FILE )
C
C    RDSTA is a routine to read an entry of the station catalog.
C    The catalog is assumed to be in a KEYIN readable format.
C
C    Input parameters:
      INTEGER        IUNIT    ! Unit number read from.
      LOGICAL        OPENIT   ! Open file on first call.  See below.
      CHARACTER*(*)  FILE     ! File name of catalog.
C
C    The value of RDSTA on return is:
C         0  if a line was successfully read.
C         1  if an end-of-file or an "ENDCAT" is encountered.  All catalog
C            entries should terminate with a '/' so RDSTA=1, assume that
C            no new data has been read.
C
C    Data from the catalog is returned via COMMON's specified in the 
C    include file, rdcat.inc.  The calling program should use that 
C    file.  If new parameters are added to the catalog, it should 
C    only be necessary to relink old programs that don't use them.
C
C    OPENIT controls whether or not the catalog file is opened on
C    the first read and closed when an EOF or parameter ENDCAT is reached.
C    Set OPENIT=.TRUE. for normal reading of external catalog files.
C    Set OPENIT=.FALSE. if no open or close is desired - eg. if the 
C    catalog information is imbedded in the calling program's main input.
C    Use ENDCAT to terminate catalog input in such cases.
C
C    R. C. Walker, Dec. 19, 1988.
C
C    Include file. 
C
      INCLUDE 'rdcat.inc'
C
C     Constants.
C
      REAL*8         PI, RADDEG, UNSET
      PARAMETER      (PI=3.141592653589793238D0)
      PARAMETER      (RADDEG=PI/180.D0)
      PARAMETER      (UNSET=-9999.D0)
C
C     Internal variables.
C
      INTEGER        MODE, IER, VLBOPE
      LOGICAL        FOPEN
      REAL*8         NONAME, XX, NONE
      CHARACTER*50   CNAME
      CHARACTER*255  RESULT
C
C     Catalog input parameters.
C
      INTEGER       NSP, I
      PARAMETER     (NSP=23)      ! Number of KEYIN parameters.
      REAL*8        CNAMES(NSP), CVALUE(NSP), ENDMRK
      DATA   (CNAMES(I),I=1,3) / 7HSTAtion, 6HSTCode, 7HCONtrol /
      DATA   (CNAMES(I),I=4,6) / 4HELev,    3HLAT,    4HLONG   /
      DATA   (CNAMES(I),I=7,8) / 5HHALim, 5HZALim /
      DATA   CNAMES(9)         / 6HENDCAT /
      DATA   (CNAMES(I),I=10,12) / 1HX, 1HY, 1HZ /
      DATA   (CNAMES(I),I=13,22) / 7HDEScrip, 9*1H  /
      DATA   CNAMES(23)          / 7HNDRIVES /
C
      DATA   ENDMRK               / 1H/ /
      DATA   NONAME, XX, NONE     / 6HNONAME, 2HXX, 4HNONE /
      DATA   FOPEN                / .FALSE. /
C
C     Open the file if requested.
C
      IF( OPENIT .AND. .NOT. FOPEN ) THEN
         IER = VLBOPE( IUNIT, FILE, 'TEXT', 'OLD', RESULT )
         IF( IER .EQ. 0 ) THEN
            CALL PUTOUT( RESULT )
            CNAME = FILE
            CALL ERROR( 'Station catalog: '//CNAME//' not opened' )
         END IF
         FOPEN = .TRUE. 
      END IF
C
C     Read the catalog line with KEYIN.  Reset all parameters before
C     start.
C
      MODE = 0
      CVALUE(1) = NONAME
      CVALUE(2) = XX
      CVALUE(3) = NONE
      CVALUE(4) = 0.0D0
      CVALUE(5) = 0.0D0
      CVALUE(6) = 0.0D0
      CVALUE(7) = 12.0D0
      CVALUE(8) = 90.D0      
      CVALUE(9) = UNSET
      CVALUE(10) = 0.0D0
      CVALUE(11) = 0.0D0
      CVALUE(12) = 0.0D0
      CVALUE(23) = 1.0D0
C
      CALL KEYIN( CNAMES, CVALUE, NSP, ENDMRK, MODE, IUNIT, 6 )
C
      IF( MODE .EQ. 1 .OR. CVALUE(9) .EQ. 0.D0 ) THEN
         IF( OPENIT .AND. FOPEN ) CLOSE( UNIT=IUNIT )
         FOPEN = .FALSE.
         RDSTA = 1
         GO TO 990
      END IF
C
C           Decode input
C
      RDSTA = 0
      WRITE( STANAM, '(A8)' ) CVALUE(1)
      WRITE( STACOD, '(A3)' ) CVALUE(2)
      WRITE( STACTL, '(A4)' ) CVALUE(3)
      STAEL  = CVALUE(4)
      STALAT = CVALUE(5) * RADDEG / 3600.D0
      STALON = CVALUE(6) * RADDEG / 3600.D0
      STAHAL = CVALUE(7)
      STAZAL = CVALUE(8)
      STAX   = CVALUE(10)
      STAY   = CVALUE(11)
      STAZ   = CVALUE(12)
      STANDR = CVALUE(23)
C
  990 RETURN
      END
