C*STNABR -- find NUG abbreviation of VLBI station name
C+
      CHARACTER*(*) FUNCTION STNABR(NAME)
      CHARACTER*(*) NAME
C
C Generate a one- or two-letter abbreviation for a station name
C (according to US Network Handbook, where applicable).
C
C Argument:
C  NAME   (input) : station name.
C
C Returns:
C  STNABR         : abbreviation.
C
C Subroutines required:
C  UPCASE
C
C History:
C  1983 Apr 25 - TJP.
C  1984 Oct 18 - List updated (SCU)
C  1985 Mar 13 - List updated (SCU)
C  1986 Feb 13 - add DUMMY (TJP).
C  1987 Nov 12 - convert to standard Fortran-77 (TJP).
C  1990 Aug  7 - make 2-character abbreviations, and list updated (SCU).
C  1991 Oct  8 - change declaration from CHAR*2 to CHAR(*), to allow
C                function to be called with arbitrary length.
C-----------------------------------------------------------------------
      CHARACTER*8   UPNAME
      CHARACTER*2   ABR
C
      UPNAME = NAME
      CALL UPCASE(UPNAME)
      IF (UPNAME(1:4).EQ.'HRAS') THEN
          ABR  = 'F '
      ELSE IF (UPNAME(1:4).EQ.'GRAS') THEN
          ABR = 'F '
      ELSE IF (UPNAME(1:4).EQ.'NRAO') THEN
          ABR = 'G '
      ELSE IF (UPNAME(1:3).EQ.'HAY') THEN
          ABR = 'K '
      ELSE IF (UPNAME(1:3).EQ.'HST') THEN
          ABR = 'K '
      ELSE IF (UPNAME(1:3).EQ.'ILL') THEN
          ABR = 'V '
      ELSE IF (UPNAME(1:3).EQ.'EFF') THEN
          ABR = 'B '
      ELSE IF (UPNAME(1:3).EQ.'ALG') THEN
          ABR = 'C '
      ELSE IF (UPNAME.EQ.'ARO')      THEN
          ABR = 'C '
      ELSE IF (UPNAME(1:2).EQ.'PU')  THEN
          ABR = 'PU'
      ELSE IF (UPNAME(1:3).EQ.'CRI') THEN
          ABR = 'R '
      ELSE IF (UPNAME(1:3).EQ.'KRI') THEN
          ABR = 'R '
      ELSE IF (UPNAME(1:3).EQ.'SIM') THEN
          ABR = 'R '
      ELSE IF (UPNAME(1:3).EQ.'ONS') THEN
          ABR = 'S '
      ELSE IF (UPNAME(1:3).EQ.'CHI') THEN
          ABR = 'U '
      ELSE IF (UPNAME(1:3).EQ.'DWI') THEN
          ABR = 'W '
      ELSE IF (UPNAME(1:3).EQ.'SAF') THEN
          ABR = 'E '
      ELSE IF (UPNAME(1:3).EQ.'HAR') THEN
          ABR = 'E '
      ELSE IF (UPNAME(1:5).EQ.'WESTF') THEN
          ABR = 'K '
      ELSE IF (UPNAME(1:3).EQ.'MARY') THEN
          ABR = 'N '
      ELSE IF (UPNAME(1:3).EQ.'VLA') THEN
          ABR = 'Y '
      ELSE IF (UPNAME(1:5).EQ.'WSRT0') THEN
          ABR = 'W0'
      ELSE IF (UPNAME(1:3).EQ.'MED') THEN
          ABR = 'L '
      ELSE IF (UPNAME(1:4).EQ.'BGNA') THEN
          ABR = 'L '
      ELSE IF (UPNAME(1:3).EQ.'BOL') THEN
          ABR = 'L '
      ELSE IF (UPNAME(1:3).EQ.'NOB') THEN
          ABR = 'M '
      ELSE IF (UPNAME(1:3).EQ.'NRO') THEN
          ABR = 'M '
      ELSE IF (UPNAME(1:3).EQ.'JAP') THEN
          ABR = 'M '
      ELSE IF (UPNAME(1:4).EQ.'NOTO') THEN
          ABR = 'NO'
      ELSE IF (UPNAME(1:3).EQ.'TOR') THEN
          ABR = 'Z '
      ELSE IF (UPNAME(1:3).EQ.'DEF') THEN
          ABR = 'DF'
      ELSE IF (UPNAME(1:3).EQ.'CAM') THEN
          ABR = 'CB'
      ELSE IF (UPNAME(1:3).EQ.'LOV') THEN
          ABR = 'J1'
      ELSE IF (UPNAME(1:5).EQ.'JBNK1') THEN
          ABR = 'J1'
      ELSE IF (UPNAME(1:5).EQ.'JBNK2') THEN
          ABR = 'J2'
      ELSE IF (UPNAME(1:3).EQ.'MET') THEN
          ABR = 'V '
      ELSE IF (UPNAME(1:3).EQ.'FIN') THEN
          ABR = 'V '
      ELSE IF (UPNAME(1:3).EQ.'ITA') THEN
          ABR = 'X '
      ELSE IF (UPNAME(1:3).EQ.'ATI') THEN
          ABR = 'X '
      ELSE IF (UPNAME(1:4).EQ.'OOTY') THEN
          ABR = 'OO'
      ELSE IF (UPNAME(1:3).EQ.'SHA') THEN
          ABR = 'SH'
      ELSE IF (UPNAME(1:4).EQ.'KASH') THEN
          ABR = 'KA'
      ELSE IF (UPNAME(1:3).EQ.'PER') THEN
          ABR = 'PR'
      ELSE IF (UPNAME(1:3).EQ.'ALI') THEN
          ABR = 'AS'
      ELSE IF (UPNAME(1:3).EQ.'PAR') THEN
          ABR = 'PK'
      ELSE IF (UPNAME(1:3).EQ.'PKS') THEN
          ABR = 'PK'
      ELSE IF (UPNAME(1:3).EQ.'CUL') THEN
          ABR = 'CG'
      ELSE IF (UPNAME(1:3).EQ.'HOB') THEN
          ABR = 'HB'
      ELSE IF (UPNAME(1:3).EQ.'NAN') THEN
          ABR = 'NC'
      ELSE IF (UPNAME(1:3).EQ.'MAD') THEN
          ABR = 'D '
      ELSE IF (UPNAME(1:5).EQ.'DSS13') THEN
          ABR = 'DV'
      ELSE IF (UPNAME(1:5).EQ.'DSS14') THEN
          ABR = 'DM'
      ELSE IF (UPNAME(1:5).EQ.'DSS15') THEN
          ABR = 'DG'
      ELSE IF (UPNAME(1:4).EQ.'DSS6') THEN
          ABR = 'DS'
      ELSE IF (UPNAME(1:4).EQ.'DSS4') THEN
          ABR = 'DT'
      ELSE IF (UPNAME(1:3).EQ.'PIE') THEN
          ABR = 'PT'
      ELSE IF (UPNAME(1:3).EQ.'KIT') THEN
          ABR = 'KP'
      ELSE IF (UPNAME(1:3).EQ.'LOS') THEN
          ABR = 'LA'
      ELSE IF (UPNAME(1:7).EQ.'VLBA_PT') THEN
          ABR = 'PT'
      ELSE IF (UPNAME(1:7).EQ.'VLBA_KP') THEN
          ABR = 'KP'
      ELSE IF (UPNAME(1:7).EQ.'VLBA_LA') THEN
          ABR = 'LA'
      ELSE IF (UPNAME(1:7).EQ.'VLBA_FD') THEN
          ABR = 'FD'
      ELSE IF (UPNAME(1:7).EQ.'VLBA_NL') THEN
          ABR = 'NL'
      ELSE IF (UPNAME(1:7).EQ.'VLBA_BR') THEN
          ABR = 'BR'
      ELSE IF (UPNAME(1:7).EQ.'VLBA_OV') THEN
          ABR = 'OV'
      ELSE IF (UPNAME(1:7).EQ.'VLBA_SC') THEN
          ABR = 'SC'
      ELSE IF (UPNAME(1:7).EQ.'VLBA_HN') THEN
          ABR = 'HN'
      ELSE IF (UPNAME(1:7).EQ.'VLBA_MK') THEN
          ABR = 'MK'
      ELSE IF (UPNAME(1:5).EQ.'DUMMY') THEN
          ABR = '* '
      ELSE
          ABR = UPNAME(1:1)
      END IF
          STNABR = ABR
      END
