C*VLBOPE -- open a VLB data file [MSDOS/MICROSOFT FORTRAN]
C+
      INTEGER FUNCTION VLBOPE (UNIT, NAME, TYPE, STATUS, RESULT)
      INTEGER UNIT
      CHARACTER*(*) NAME
      CHARACTER*(*) TYPE
      CHARACTER*(*) STATUS
      CHARACTER*(*) RESULT
C
C This routine encapsulates all system-dependent OPEN parameters
C like READONLY, CARRIAGECONTROL in a single subroutine.  Moving to
C a different operating system should require only this routine to be 
C changed, assuming the operating system can cope with Fortran formatted
C and unformatted read and write.
C
C Input arguments:
C   UNIT : Fortran unit number to be used
C   NAME : file name
C   TYPE : file type, one of the strings
C      'MERGE'  merge format
C      'FITS'   disk FITS format (image or uvfits)
C      'MODEL'  model file (text)
C      'TEXT'   arbitrary text file
C   STATUS :
C      'OLD' to open a file for reading (file must already exist)
C      'NEW' to open a new file for writing (file must not already
C            exist, unless the operating system can create a new
C            version)
C      'SCRATCH'  Open new scratch file.  Close on termination of
C            program.
C
C Output arguments:
C   VLBOPE : receives 1 if the file was opened successfully,
C      0 if the open failed
C   RESULT : receives the full file name, if the file
C      was opened successfully, or am error message (text) if the
C      open failed; the declared length of this variable should
C      be big enough to accommodate the result; 255 bytes should be
C      enough.
C
C Subroutines required:
C   ERROR
C   LEN1
C   PUTOUT
C   ERRSNS (VMS)
C   SYS$GETMSG (VMS)
C
C History:
C   1988 Jun 9 - TJP.  VAX/VMS
C   1988 Nov 29 - RCW.  MSDOS/MICROSOFT
C        Some modes not tested.  Parameters RECORDTYPE and CARRIAGECONTROL
C        removed and not replaced with any equivalent.
C        No detailed information on file open failure obtained.  How?
C        
C-----------------------------------------------------------------------
      INTEGER ITYPE, ISTAT, IER
      INTEGER LEN1
      CHARACTER*11 FMT
      CHARACTER*5  TYPEC
      CHARACTER*3  STATUC
      CHARACTER*50 NAMEC
C
C Verify the TYPE argument.
C
      IF (TYPE.EQ.'MERGE') THEN
          ITYPE = 1
          FMT = 'UNFORMATTED'
      ELSE IF (TYPE.EQ.'FITS') THEN
          ITYPE = 2
          FMT = 'UNFORMATTED'
      ELSE IF (TYPE.EQ.'MODEL' .OR. TYPE.EQ.'TEXT') THEN
          ITYPE = 3
          FMT = 'FORMATTED'
      ELSE
          TYPEC = TYPE
          CALL ERROR('VLBOPE: invalid argument TYPE='//TYPEC)
      END IF
C
C Verify the STATUS argument.
C
      IF (STATUS.EQ.'OLD') THEN
          ISTAT = 1
      ELSE IF (STATUS.EQ.'NEW') THEN
          ISTAT = 2
      ELSE IF (STATUS.EQ.'SCRATCH') THEN
          ISTAT = 3
      ELSE
          STATUC = STATUS
          CALL ERROR('VLBOPE: invalid argument STATUS='//STATUC)
      END IF
C
C Attempt to open the file.
C
      IF (ISTAT.EQ.1) THEN
C         -- 'OLD' file
          IF (ITYPE.EQ.2) THEN
              OPEN (UNIT=UNIT, FILE=NAME, MODE='READ', STATUS='OLD',
     1              RECL=2880,
     2              FORM=FMT, IOSTAT=IER)
          ELSE
              OPEN (UNIT=UNIT, FILE=NAME, MODE='READ', STATUS='OLD',
     1              FORM=FMT, IOSTAT=IER)
          END IF
      ELSE IF (ISTAT.EQ.2) THEN
C         -- 'NEW' file
          OPEN (UNIT=UNIT, FILE=NAME, STATUS='NEW',
     1          FORM=FMT, IOSTAT=IER)
      ELSE
C         -- 'SCRATCH' file
          OPEN (UNIT=UNIT, FILE=NAME, STATUS='SCRATCH',
     1          FORM=FMT, IOSTAT=IER)
      END IF
C
C Success: find the complete file name.
C
      IF (IER.EQ.0) THEN
          INQUIRE (UNIT=UNIT, NAME=RESULT)
          VLBOPE = 1
C
C Failure: determine the error, and issue a message.
C
      ELSE
          TYPEC = TYPE
          NAMEC = NAME(1:LEN1(NAME))
          IF (ISTAT.EQ.1) THEN
              CALL PUTOUT('++ Cannot find '//TYPEC//' file: '//
     1                NAMEC)
          ELSE
              CALL PUTOUT(' ++ Cannot create '//TYPEC//' file: '//
     1                NAMEC) 
              CALL PUTOUT(' ++ Does it already exist?')
          END IF
C         I don't know how to get details of error
          RESULT = ' File open error'
          VLBOPE = 0
      END IF
C-----------------------------------------------------------------------
      END
