      INTEGER FUNCTION NARROW ()
C-----------------------------------------------------------------------
C NARROW reads a single character from the terminal (SYS$COMMAND).
C Normal ASCII characters are returned as the corresponding integer 
C codes. Escape sequences generated by the VT100 and I100 terminals are
C recognised and returned as negative integer codes.  
C Some characters (eg control-U,
C control-R, delete, control-C, control-Y) are normally intercepted
C by the operating system and cannot be read with NARROW.  The type-
C ahead buffer is purged on each call to narrow; thus characters
C placed in the type-ahead buffer before the call to NARROW are
C ignored.
C
C Arguments: none.
C
C Returns:
C
C NARROW (integer): either a positive number (0-127) equal to
C      the ASCII code of the character read, or a negative number, as
C      follows, if one of the VT100 escape sequences has been read:
C       UP ARROW, DOWN ARROW, RIGHT ARROW, LEFT ARROW: -1,-2,-3,-4;
C       PF1, PF2, PF3, PF4: -11,-12,-13,-14.
C      If the terminal is in "alternate keypad mode", the following
C      codes are generated by typing the keypad keys:
C       Keypad digits 0 through 9: -20 through -29;
C       Keypad ENTER: -8;
C       Keypad comma, minus, period: -16,-17,-18.
C      If an unrecognized escape sequence is received, NARROW is set
C       to zero.
C
C T.J. Pearson, 8-Nov-1983
C-----------------------------------------------------------------------
      IMPLICIT  NONE
      INTEGER   IER, J
      INTEGER   CODE_TABLE(22)
      CHARACTER VALID_TABLE(22)
      CHARACTER CH, CH1, CH2
      INTEGER   FGETCHAR
C
C Valid escape sequences are <esc> O or <esc> [
C followed by one of the characters in VALID_TABLE;
C CODE_TABLE contains the corresponding integer codes
C returned by NARROW.
C
      DATA VALID_TABLE/ 'A','B','C','D',  'P','Q','R','S',
     1            'p','q','r','s','t','u','v','w','x','y',
     2            'm','l','n', 'M'/
      DATA CODE_TABLE/ -1,-2,-3,-4, -11,-12,-13,-14,
     1            -20,-21,-22,-23,-24,-25,-26,-27,-28,-29,
     2            -17,-16,-18, -8/
C
C Read a single character from Fortran unit 5 using Convex
C routine FGETCHAR.
C
      IER = FGETCHAR(CH)
      IF (IER.NE.0) GOTO 100
C
C If character is not <escape>, return it.
C
      IF (CH .NE. CHAR(27)) THEN
          NARROW = ICHAR(CH)
          RETURN
C
C If character is <escape>, read the rest of the escape
C sequence, assumed to be two more characters.
C
      ELSE 
          IER = FGETCHAR(CH1)
          IF (IER.NE.0) GOTO 100
          IF (CH1.NE.'O' .AND. CH1.NE.'[') THEN
              NARROW = ICHAR(CH1)
              RETURN
          ELSE
              IER = FGETCHAR(CH2)
              IF (IER.NE.0) GOTO 100
              DO J=1,22
                  IF (CH2.EQ.VALID_TABLE(J)) THEN
                      NARROW = CODE_TABLE(J)
                      RETURN
                  END IF
              END DO
              NARROW = 0
              RETURN
          END IF
      END IF
C
  100 CALL ERROR('reading character from terminal')
      NARROW = 0
      RETURN
      END
