C*NEWFIL -- create a unique file name for a new file
C+
      SUBROUTINE NEWFIL (NAME, NEW)
      CHARACTER*(*) NAME, NEW
C
C This routine is for operating systems that do not allow multiple
C versions of a file. Parameter NAME is the desired name for the
C file. If it already exists, the string returned in NEW is a name
C derived from NAME by adding a period and a numerical suffix (or 
C incrementing the suffix if it already exists); otherwise NEW = NAME.
C This process is iterated until the file named NEW does not exist.
C
C Examples:
C       NAME			NEW (if file NAME exists)
C       2021.mod		2021.mod.1
C	2021.mod.3		2021.mod.4
C	2021.mod.99		2021.mod.100
C
C Arguments:
C  NAME   (input)  : desired file name.
C  NEW    (input)  : derived file name.
C
C History:
C  1992 Jun 24 - New routine (T. J. Pearson).
C-----------------------------------------------------------------------
      LOGICAL EXIST
      INTEGER I, LNAME, LBASE, LNEW, VERSN
      INTEGER KCTOI, LEN1
      CHARACTER*4 SUFFIX
C
C If file does not exist, return the requested name.
C
      LNAME = LEN1(NAME)
      INQUIRE (FILE=NAME(1:LNAME), EXIST=EXIST)
C     WRITE (*,*) 'NAME=',NAME(1:LNAME), ' EXIST=',EXIST
      IF (.NOT.EXIST) THEN
          NEW = NAME
          RETURN
      END IF
C
C Get numerical suffix of input name and remove it to get base name.
C The numerical suffix is "." followed by one or more decimal digits.
C
      LBASE = LNAME
   10 IF (INDEX('0123456789', NAME(LBASE:LBASE)) .GT. 0) THEN
          LBASE = LBASE-1
      GOTO 10
      END IF
      IF (NAME(LBASE:LBASE).EQ.'.') THEN
          I = LBASE+1
          LBASE = LBASE-1
          VERSN = KCTOI(NAME, I)
      ELSE
          LBASE = LNAME
          VERSN = 0
      END IF
C     WRITE (*,*) 'BASE=',NAME(1:LBASE), ' VERSION=', VERSN
C
C Increment the suffix and test whether the file exists. If not, exit.
C Otherwise, loop. A maximum version number of 9999 is specified to
C prevent infinite loops.
C
      NEW(1:LBASE) = NAME(1:LBASE)
      NEW(LBASE+1:LBASE+1) = '.'
   20 CONTINUE
          VERSN = VERSN+1
          WRITE (SUFFIX,'(I4)') VERSN
          I = 1
   30     IF (SUFFIX(I:I).EQ.' ') THEN
              I = I+1
          GOTO 30
          END IF
          NEW(LBASE+2:) = SUFFIX(I:)
          LNEW = LBASE+LEN(SUFFIX)-I+2
          INQUIRE (FILE=NEW, EXIST=EXIST)
C         WRITE (*,*) 'NAME=',NEW(1:LNEW), ' EXIST=',EXIST
          IF (EXIST.AND.VERSN.LT.9999) GOTO 20
      RETURN
C
      END
