	INTEGER FUNCTION NARROW ()
C-----------------------------------------------------------------------
C NARROW reads a single character from the terminal (SYS$COMMAND).
C Normal ASCII characters are returned as the corresponding integer 
C codes. Escape sequences generated by the VT100 and I100 terminals are
C recognised and returned as negative integer codes.  
C Some characters (eg control-U,
C control-R, delete, control-C, control-Y) are normally intercepted
C by the operating system and cannot be read with NARROW.  The type-
C ahead buffer is purged on each call to narrow; thus characters
C placed in the type-ahead buffer before the call to NARROW are
C ignored.
C
C Arguments: none.
C
C Returns:
C
C NARROW (integer): either a positive number (0-127) equal to
C	the ASCII code of the character read, or a negative number, as
C	follows, if one of the VT100 escape sequences has been read:
C	 UP ARROW, DOWN ARROW, RIGHT ARROW, LEFT ARROW: -1,-2,-3,-4;
C	 PF1, PF2, PF3, PF4: -11,-12,-13,-14.
C	If the terminal is in "alternate keypad mode", the following
C	codes are generated by typing the keypad keys:
C	 Keypad digits 0 through 9: -20 through -29;
C	 Keypad ENTER: -8;
C	 Keypad comma, minus, period: -16,-17,-18.
C	If an unrecognized escape sequence is received, NARROW is set
C	 to zero.
C
C T.J. Pearson, 8-Nov-1983
C-----------------------------------------------------------------------
	IMPLICIT  NONE
	INCLUDE   '($IODEF)'
	INCLUDE   '($SSDEF)'
	INTEGER   IOFUNC
	PARAMETER (IOFUNC=IO$_READVBLK.OR.IO$M_NOECHO.OR.IO$M_ESCAPE.OR.
     1	           IO$M_PURGE)
	INTEGER   CHAN, IER, SYS$ASSIGN, SYS$QIOW, TERMSK(2), J
	INTEGER*2 IOSB(4), CODE_TABLE(22)
	LOGICAL   FIRST
	BYTE	  TERMCH(32),BUFFER(20), VALID_TABLE(22)
	DATA      TERMCH/32*'FF'X/
	DATA	  TERMSK(1) / 32 /
	DATA      FIRST/.TRUE./
	SAVE      FIRST,CHAN
C
C		Valid escape sequences are <esc> O or <esc> [
C		followed by one of the characters in VALID_TABLE;
C		CODE_TABLE contains the corresponding integer codes
C		returned by NARROW.
C
	DATA VALID_TABLE/ 'A','B','C','D',  'P','Q','R','S',
     1		'p','q','r','s','t','u','v','w','x','y',
     2		'm','l','n', 'M'/
	DATA CODE_TABLE/ -1,-2,-3,-4, -11,-12,-13,-14,
     1		-20,-21,-22,-23,-24,-25,-26,-27,-28,-29,
     2		-17,-16,-18, -8/
C
C		On the first call, assign an I/O channel to SYS$COMMAND.
C
	IF (FIRST) THEN
	    FIRST = .FALSE.
	    IER = SYS$ASSIGN('SYS$COMMAND',CHAN,,)
	    IF (.NOT.IER) CALL EXIT(IER)
	END IF
C
C		Read a single character; all characters are
C		terminators, and escape sequences are recognised.
C
	TERMSK(2) = %LOC(TERMCH)
	IER = SYS$QIOW(, %VAL(CHAN), %VAL(IOFUNC), IOSB, , ,
     1	              BUFFER, 	! P1 (buffer address)
     2		      %VAL(20),	! P2 (buffer size)
     3		      ,		! P3 (timeout count)
     4		      TERMSK,	! P4 (read terminator descr.block)
     5		      ,		! P5 (prompt buffer address)
     6		      )		! P6 (prompt buffer size)
	IF (.NOT.IER) CALL EXIT(IER)
C
C		If terminator is a single character, return it.
C
	IF (IOSB(4).EQ.1) THEN
	    NARROW = BUFFER(1)
	    RETURN
C
C		If terminator is a VT100 escape sequence, interpret it.
C
	ELSE IF ((BUFFER(1).EQ.27) .AND.
     1	        (BUFFER(2).EQ.79 .OR. BUFFER(2).EQ.91)) THEN
	    DO J=1,22
		IF (BUFFER(3).EQ.VALID_TABLE(J)) THEN
		    NARROW = CODE_TABLE(J)
		    RETURN
		END IF
	    END DO
C
C		If terminator is an I100 escape sequence, interpret it.
C
	ELSE IF (BUFFER(1).EQ.27) THEN
	    DO J=1,4
		IF (BUFFER(2).EQ.VALID_TABLE(J)) THEN
		    NARROW = CODE_TABLE(J)
		    RETURN
		END IF
	    END DO
	END IF
C
C		Otherwise, return zero.
C
	NARROW = 0
	RETURN
	END
