C*NEWFIL -- create a unique file name for a new file [VMS]
C+
      SUBROUTINE NEWFIL (NAME, NEW)
      CHARACTER*(*) NAME, NEW
C
C This routine is for operating systems that do not allow multiple
C versions of a file. Parameter NAME is the desired name for the
C file. If it already exists, the string returned in NEW is a name
C derived from NAME by adding a period and a numerical suffix (or 
C incrementing the suffix if it already exists); otherwise NEW = NAME.
C This process is iterated until the file named NEW does not exist.
C
C Examples:
C       NAME			NEW (if file NAME exists)
C       2021.mod		2021.mod.1
C	2021.mod.3		2021.mod.4
C	2021.mod.99		2021.mod.100
C
C Arguments:
C  NAME   (input)  : desired file name.
C  NEW    (input)  : derived file name.
C
C History:
C  1992 Jun 24 - New routine (T. J. Pearson).
C-----------------------------------------------------------------------
      INTEGER I, LNAME, LBASE, LNEW, VERSN
      INTEGER KCTOI, LEN1
      CHARACTER*4 SUFFIX
C
C Get VMS version number of input name and remove it to get base name.
C The numerical suffix is ";" followed by one or more decimal digits.
C
      LNAME = LEN1(NAME)
      LBASE = LNAME
   10 IF (INDEX('0123456789', NAME(LBASE:LBASE)) .GT. 0) THEN
          LBASE = LBASE-1
      GOTO 10
      END IF
      IF (NAME(LBASE:LBASE).EQ.'.' .OR. NAME(LBASE:LBASE).EQ.';') THEN
          I = LBASE+1
          LBASE = LBASE-1
          VERSN = KCTOI(NAME, I)
      ELSE
          LBASE = LNAME
          VERSN = 0
      END IF
C     WRITE (*,*) 'BASE=',NAME(1:LBASE), ' VERSION=', VERSN
C
      NEW = NAME(1:LBASE)
      RETURN
C
      END
