      SUBROUTINE MEM3
     * (METHOD,LEVEL,MEMRUN,NRAND,SEED, AIMX,RATEX,DEFX,ACCX,TOLX,
     *  ALPHAX,BETAX, SX,TESTX,CHISQX,SIGMAX,PROBX,GOODX,OMEGAX,ISTAT)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*                                                                      *
*           -----------------------------------------------            *
*          |                                               |           *
*          | "MEMSYS 3"  Quantified Maximum Entropy System |           *
*          |                                               |           *
*           -----------------------------------------------            *
*                                                                      *
*                 Conceptualised and programmed by                     *
*                 Stephen Gull and John Skilling                       *
*                                                                      *
*         (C)   Copyright 1989                                         *
*               Maximum Entropy Data Consultants Ltd                   *
*               33 North End, Meldreth, Royston, England               *
*                                                                      *
*               Version 2.02            14 Sep 1989                    *
************************************************************************
*
* Purpose:
*        Perform one iterate of data-space maximum entropy,
*        update the Lagrange multipliers and find the new map.
*
*   Enter with:
*              <21> = data D
*              <23> = Lagrange multipliers w  (except first iterate)
*     and optionally
*              <20> = default model
*              <22> = accuracies = 1/sigma 
*
*    Exit with:
*              <23> = new Lagrange mutipliers
*              <1>  = new map f
*              <24> = normalised residuals = (D-Transform(old f))/sigma
*     and optionally
*              <22> = Poisson accuracies = sqrt(D+tol)/Transform(old f)
*
*    Workspace:
*              <2> , <25> , <25> , <26> , <27> , <28>
*     and optionally
*              <29> if transform is nonlinear
*
* Parameters:
*   ARGUMENT TYPE  I/O  DIMENSION   DESCRIPTION
*    METHOD  I     I      -         Composite switch for stop criterion,
*                                   variable default, and variable acc.
*    LEVEL   I     I      -         Composite switch for diagnostics
*                                   including flowcharting
*    MEMRUN  I     I      -         0 is  start,  1 is continue,
*                                   2 is restart, 3 is diagnostics
*    NRAND   I     I      -         Number of random vectors to be used
*    SEED    I     I      -         Seed for reproducible random vectors
*    AIMX    R     I      -         Ultimate target of constraint >= 0
*    RATEX   R     I      -         Dimensionless distance limit, O(1)
*    DEFX    R     I      -         Default level (if constant)
*    ACCX    R     I      -         Accuracy      (if constant)
*    TOLX    R     I      -         Tolerance (dimensionless O(.1)) used
*                                   for maximisations & to define CODEs
*    ALPHAX  R     I O    -         alpha used to calculate new map
*    BETAX   R     I O    -         alpha * (1 + dist penalty)
*    SX      R       O    -         Entropy of entry map
*    TESTX   R       O    -         1 - cos(angle( w , D-Transform(f) ))
*                                   for entry map
*    CHISQX  R       O    -         Chisquared of entry map
*    SIGMAX  R       O    -         estimate of overall scaling of
*                                   sigma(data)  (=1 if not being found)
*    PROBX   R       O    -         LogProb(Data)       (base e)
*    GOODX   R       O    -         number of good degrees of freedom
*    OMEGAX  R       O    -         termination variable, 
*                                   should climb to 1.00
*    ISTAT   I       O    -         Return code
*
* Globals:
*   VARIABLE  COMMON BLOCK  I/O  DESCRIPTION
*    NJ          MECOMP     I    Number of buffers in map-space area
*    MJ          MECOMP     I    Map-space buffer length in words
*    NK          MECOMP     I    Number of buffers in data-space area
*    MK          MECOMP     I    Data-space buffer length in words
*
*    KA          MECOMP     I    Allocation pointers for areas
*    KB          MECOMP     I    Base addresses in core for buffers
*    KC          MECOMP      (O) Core addresses for current buffers
*    KD          MECOMP      (O) Disc or upper core addresses of buffers
*
*    IOUT        MECOMP     I    Fortran output stream for diagnostics
*
*    L0          MECOMP       O  Flag for progress diagnostics
*    L1          MECOMP       O  Flag for numerical diagnostics
*    M0          MECOMP       O  Flag for type of constraint
*    M1          MECOMP       O  Flag for variable or constant default
*    M2          MECOMP       O  Flag for constant or variable errors
*    M3          MECOMP       O  Flag for linear or nonlinear data
*
*    NTRANS      MECOMP       O  Accumulated number of transforms
*
*    ST          MECOMS     I O  Storage vector
*
*    PR          MECOMC      (O) Read/write diagnostic flags
*
* External calls:
*            Map-Data
*    MEMOP     f d     Call user transform routine OPUS
*    MEMTR     f d     Call user transpose routine TROPUS
*   (MEMX      f d     Call user transform routine MEMEX)
*
*    MEM0      f d     Initialise on first call
*    MEMA      f       New map and entropy
*    MEMB        d     Likelihood and gradient
*    MEMC        d     Scalars for gradS, gradL
*    MEME        d     gradQ
*    MEMI        d     Data and units
*
*    UINIT             Initialise pointers and enable diagnostics
*    UEXIT             Error exit
*    MEMCG     f d     Conjugate gradient
*    MEML              Control alpha and distance penalty
*    MEPROB    f d     LogProb and Good degrees of freedom
*
* Key:
*
*    METHOD = 1000*M3 + 100*M2 + 10*M1 + M0
*
*        M3 = 0 is linear data
*        M3 = 1 is nonlinear data
*
*        M2 = 0 is 'Variable acc in <22>'
*        M2 = 1 is 'Constant acc=1/sigma'
*        M2 = 2 is 'Poisson statistics'
*
*        M1 = 0 is 'Constant default'
*        M1 = 1 is 'Variable def in <20>'
*        M1 = 2 is 'Constant default with positive/negative'
*        M1 = 3 is 'Variable def in <20> with positive/negative'
*
*        M0 = 0 is 'Chisquared=Ndata'
*        M0 = 1 is 'Classic with known noise'
*        M0 = 2 is 'Classic with unknown noise'
*        M0 = 3 is 'Alpha given'
*
*   LEVEL = 10*L1 + L0
*
*        L1 = 0 gives no numerical diagnostics
*        L1 = 1 gives major numerical diagnostics 
*        L1 = 2 gives additional technical diagnostics
*       (L1 = 3 gives further development diagnostics)
*
*        L0 = 0 gives no progress diagnostics
*        L0 = 1 gives names of main routines
*        L0 = 2 gives additionally read/write flowchart
*        L0 = 3 additionally calls udiag after each main routine
*
*  Normally,  ISTAT = 1000000*code6 + 100000*code5 + 10000*code4
*                     + 1000*code3 + 100*code2 + 10*code1 +code0
*
*     code6 = 0    is "LogProb returns acceptably"
*     code6 = 1    is "max number LogProb iterations reached"
*     code6 = 2,3  is "LogProb failure to maximise accurately"
*
*     code5 = 0    is "Good returns acceptably"
*     code5 = 1    is "max number Good iterations reached"
*     code5 = 2,3  is "Good failure to maximise accurately"
*
*     code4 = 0    is "conjugate gradient returns acceptably"
*     code4 = 1    is "max number conjgrad iterations reached"
*     code4 = 2,3  is "conjgrad failure to maximise accurately"
*
*     code3 = 0 is "test <= 1"
*     code3 = 1 is "test >  1 , unacceptable"
*
*     code2 = 0 is "Control is satisfied by current alpha"
*     code2 = 1 is "Control changes alpha"
*     code2 = 2 is "Control imposes distance penalty"
*
*     code1 = 0 is "termination condition omega = 1 reached, within TOL"
*     code1 = 1 is "termination condition on omega not reached"
*
*     code0 = 0 is "bubble overlaps true bubble(alpha) to within TOL"
*     code0 = 1 is "bubble not overlapping true bubble(alpha)"
*
*  but if Omega > 1 on setup, ISTAT = 0 regardless of the code values.
*
* History:
*   John Skilling    7 Dec 1988     Version 1.21
*   JS              16 Dec 1988     (L1=3)
*   David Robinson   8 Mar 1989     Poisson statistics
*   JS              31 Mar 1989     Version 1.72
*   JS              25 May 1989     Version 1.80, with MEMTRQ, not ALF
*   JS              17 Jun 1989     Version 2.00 with pos/neg & Poisson
*   JS              14 Sep 1989     Version 2.02
*
* Notes:
* (1) ALPHAX and BETAX, and the globals in /MECOMP/ and /MECOML/,
*     must be preserved between calls.
* (2) All relevant parameters are initialised by "MEMRUN=0".
*-----------------------------------------------------------------------
      IMPLICIT CHARACTER*1 (A-Z)
      INTEGER METHOD,LEVEL,MEMRUN,ISTAT,NRAND,SEED
      REAL AIMX,RATEX,DEFX,ACCX,TOLX, SX,TESTX,CHISQX
      REAL ALPHAX,BETAX,SIGMAX,OMEGAX,GOODX,PROBX
      INTEGER CODE0,CODE1,CODE2,CODE3,CODE4,CODE5,CODE6
      REAL*8 AIM,RATE,DEF,ACC,TOL,EPS,BIG,ZERO,DIV,xxx,yyy
      REAL*8 ALF,ALPHA,BETA,CHISQ,DATA,DD,GOOD,H,LHOOD,LOH
      REAL*8 OMEGA,PROB,S,SIGMA,SUMF,TEST,UNITS,UPH,VAR,WD,WW
      PARAMETER (ZERO=0., EPS=1.D-20, BIG=1.D20)
      INCLUDE 'MECOMP.INC'
      INCLUDE 'MECOML.INC'
      DIV(xxx,yyy) = xxx/MAX(yyy,EPS)
* Input flags and parameters
      M0=METHOD
      M1=M0/10
      M2=M1/10
      M3=M2/10
      M0=M0-(M0/10)*10
      M1=M1-(M1/10)*10
      M2=M2-(M2/10)*10
      M3=M3-(M3/10)*10
      IF(   ( M0.NE.0 .AND. M0.NE.1 .AND. M0.NE.2 .AND. M0.NE.3 )
     * .OR. ( M1.NE.0 .AND. M1.NE.1 .AND. M1.NE.2 .AND. M1.NE.3 )
     * .OR. ( M2.NE.0 .AND. M2.NE.1 .AND. M2.NE.2 )
     * .OR. ( M3.NE.0 .AND. M3.NE.1 ) )
     *  CALL UEXIT('Illegal method')
      IF(M2.EQ.2) THEN
        IF( M0.EQ.2 .OR. M1.EQ.2 .OR. M1.EQ.3 .OR. M3.EQ.1 )
     *    CALL UEXIT('Method incompatible with Poisson statistics M2=2')
      ENDIF
      L0=LEVEL
      L1=L0/10
      L0=L0-(L0/10)*10
      L1=L1-(L1/10)*10
      IF(   ( L0.NE.0 .AND. L0.NE.1 .AND. L0.NE.2 .AND. L0.NE.3 )
     * .OR. ( L1.NE.0 .AND. L1.NE.1 .AND. L1.NE.2 .AND. L1.NE.3 ) )
     *  CALL UEXIT('Illegal level')
      IF(L0.GE.1) WRITE(IOUT,51) M3,M2,M1,M0
      AIM=AIMX
      RATE=RATEX
      DEF=DEFX
      ACC=ACCX
      TOL=TOLX
      GOOD=0.
      PROB=0.
      UPH=0.
      CODE0=0
      CODE1=0
      CODE2=0
      CODE3=0
      CODE4=0
      CODE5=0
      CODE6=0
      CALL UINIT
      IF(MEMRUN.EQ.0) THEN
* Initialisation of new run
        CALL MEM0
        NTRANS=0
        NTABLE=0
        ALPHA=BIG
        BETA=ALPHA
      ELSEIF(MEMRUN.GT.0 .AND. MEMRUN.LE.3) THEN
* Continue existing run
* (Forget lookup table of Omega(Alpha) if restart)
        IF(MEMRUN.EQ.2) NTABLE=0
        ALPHA=ALPHAX
        BETA=BETAX
        CALL MEMDIV(23,25,ACC)
        CALL MEME(ALPHA)
* If iterating, new dimensional multipliers <23> incremented by
* <23> += effectiveaccuracy B**-1 (-alpha<23>/effectiveaccuracy + <24>)
* where B = beta*I + A,
*    A = effectiveaccuracy Opus {f} Tropus effectiveaccuracy
*    effective accuracy in ACC or <22>, or <29> if nonlinear,
        IF(MEMRUN.EQ.1)
     *     CALL MEMCG(1,BETA,ZERO,TOL,DEF,ACC, LOH,H,UPH,CODE4)
        IF(MEMRUN.EQ.3)
     *     CALL MEMCG(0,BETA,ZERO,TOL,DEF,ACC, LOH,H,UPH,CODE4)
      ELSE
        CALL UEXIT('Illegal MemRun')
      ENDIF

* Begin problem-dependent interaction with image-space .................
* Enter with dimensional multipliers in <23>
      CALL MEMK(23,25)
      CALL MEMTR(25,2)
      CALL MEMA(DEF,SUMF,S)
      CALL MEMOP(2,25)
*  Nonlinear?
      IF(M3.EQ.1) CALL MEMX(ACC)
      CALL MEMB(ACC,LHOOD)
* Exit with scalars and the following areas:
*    f in <1> = default * exp(Tropus<23>),
*    effective accuracy in ACC or <22>, or <29> if nonlinear,
*         defined by d2LHOOD/dfdf = Tropus . effectiveaccuracy**2 . Opus
*    <24> defined by dLHOOD/df  = - Tropus . effectiveaccuracy . <24>
*.......................................end interaction with image-space

      CALL MEMDIV(23,25,ACC)
      CALL MEMC(WW,WD,DD)
      TEST=1.-DIV(WD,SQRT(WW)*SQRT(DD))
      CALL MEMI(ACC,DATA,UNITS)
      IF(M0.EQ.2) THEN
        SIGMA=SQRT(2.*(LHOOD-ALPHA*S)/DATA)
      ELSE
        SIGMA=1.
      ENDIF
      CHISQ=DIV(2.*LHOOD,SIGMA*SIGMA)
      IF(L1.GE.1) WRITE(IOUT,52) S,TEST,CHISQ
* Termination criterion OMEGA
      IF(M0.EQ.0) THEN
        OMEGA=DIV(DATA*AIM,CHISQ)
        VAR=DIV(TEST,1.-TEST)+TOL*TOL/12.
        CODE5=0
        CODE6=0
      ELSEIF(M0.EQ.1 .OR. M0.EQ.2) THEN
        CALL MEPROB(MEMRUN,NRAND,SEED,
     *              DEF,ACC,TOL,ALPHA,S,CHISQ,SIGMA,AIM,
     *              GOOD,PROB,OMEGA,CODE6,CODE5)
        PROB=UNITS-DATA*LOG(SIGMA)+PROB
        IF(L1.GE.1) WRITE(IOUT,53) PROB,CODE6,CODE5,GOOD
        VAR=DIV(TEST,1.-TEST)+TOL*TOL/12.
      ELSEIF(M0.EQ.3) THEN
        OMEGA=DIV(AIM,ALPHA)
        VAR=TOL*TOL/12.
        CODE5=0
        CODE6=0
      ENDIF
* Control on alpha and beta (if iterating)
      IF(MEMRUN.LT.3)
     *   CALL MEML(ALPHA,OMEGA,VAR,RATE,SUMF,WW,WD,DD,ALF,BETA,CODE2)
* Return codes, with immediate termination as overriding special case
      IF(M0.EQ.1 .OR. M0.EQ.2) THEN
        IF(UPH/(SIGMA*SIGMA).GT.TOL*GOOD) CODE0=1
      ENDIF
      IF(ABS(OMEGA-1.).GT.TOL)          CODE1=1
      IF(TEST.GT.1.00001)               CODE3=1
      ISTAT=CODE6*1000000+CODE5*100000+
     *      CODE4*10000+CODE3*1000+CODE2*100+CODE1*10+CODE0
      IF(L1.GE.1) THEN
        IF(M0.EQ.2) WRITE(IOUT,54) SIGMA
        WRITE(IOUT,55) OMEGA,CODE4,CODE3,CODE2,CODE1,CODE0,ALPHA
      ENDIF
      IF(MEMRUN.EQ.0 .AND. OMEGA.GE.1.) ISTAT=0
* Exit
      SX=S
      TESTX=TEST
      CHISQX=CHISQ
      ALPHAX=ALF
      BETAX=BETA
      SIGMAX=SIGMA
      OMEGAX=OMEGA
      GOODX=GOOD
      PROBX=PROB
   51 FORMAT('  MemSys3 Version 2.02  14 Sep 1989   Method ',4I1)
   52 FORMAT('  Entropy === ',1PE12.4,
     *       '    Test ===',0PF7.4,'    Chisq ===',1PE12.4)
   53 FORMAT('  LogProb ===',1PE13.4,
     *       '    Code ===  ',2I1,'       Good  ===',1PE12.4)
   54 FORMAT(49X,'Sigma ===',1PE12.4)
   55 FORMAT('  Omega   === ',0PF10.6,
     *       '      Code ===  ',5I1,'    Alpha ===',1PE12.4)
      END


      SUBROUTINE MEML(ALPHA,OMEGA,VAR,RATE,SUMF,WW,WD,DD,ALF,BETA,CODE)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*
* Purpose:
*         Find values of alpha and distance penalty by inserting new
*         value of OMEGA into a table of values and accuracies, and 
*         binary chopping within the table to set alpha to the closest
*         accessible value (within the distance limit) to OMEGA=1.
*
* Parameters:
*   ARGUMENT TYPE  I/O  DIMENSION   DESCRIPTION
*    ALPHA   R*8   I      -         Input alpha
*    OMEGA   R*8   I      -         New value of ending criterion
*    VAR     R*8   I      -         Intrinsic variance of (OMEGA)
*    RATE    R*8   I      -         Dimensionless distance limit
*    SUMF    R*8   I      -         Total flux (dimensional)
*    WW      R*8   I      -         gradS.gradS
*    WD      R*8   I      -         gradS.gradC
*    DD      R*8   I      -         gradC.gradC
*    ALF     R*8   I      -         Output alpha
*    BETA    R*8     O    -         Alpha * (Distance penalty + 1)
*    CODE    I       O    -         Return code
*                                  0 = OK
*                                  1 = alpha changed from input value
*                                  2 = distance penalty invoked
*
* Globals:
*   VARIABLE  COMMON BLOCK  I/O  DESCRIPTION
*    L0,L1       MECOMP     I    Diagnostic flags
*    IOUT        MECOMP     I    Diagnostic stream
*    ....        MECOML     I O  Table of OMEGA(alpha) with variances
*
* External calls:
*    MEML0       Insert new value of OMEGA into table
*    MEML1       Estimate OMEGA(alpha)
*
* History:
*   John Skilling    7 Dec 1988     MEMSYS3 version
*   JS              10 Dec 1988     Routine name changed
*   JS               8 Jan 1989     BETA parameter
*   JS              19 Jan 1989     RATE protected slightly
*   JS              17 Jun 1989     Input & output alpha separated
*
* Notes:
*-----------------------------------------------------------------------
      IMPLICIT CHARACTER*1 (A-Z)
      INTEGER I,CODE
      REAL*8 OMEGA,VAR,RATE,SUMF,WW,WD,DD,ALPHA,ALF,BETA,PEN1
      REAL*8 ALF1,ALF2,R,SIGMA,Y1,Y2,YNEW,EPS,ONE,DIV,xxx,yyy
      PARAMETER (EPS=1.D-20, ONE=1.)
      INCLUDE 'MECOMP.INC'
      INCLUDE 'MECOML.INC'
      DIV(xxx,yyy) = xxx/MAX(yyy,EPS)
      IF(L0.GE.1) WRITE(IOUT,51)
      CALL MEML0(ALPHA,OMEGA,VAR)
      IF(L1.GE.2) THEN
        WRITE(IOUT,52)
        DO 1 I=1,NTABLE
          WRITE(IOUT,53) I,XTABLE(I),YTABLE(I),VTABLE(I)
    1   CONTINUE
        WRITE(IOUT,54) RATE
        WRITE(IOUT,55) SUMF
        WRITE(IOUT,56) WW
        WRITE(IOUT,57) WD
        WRITE(IOUT,58) DD
        WRITE(IOUT,59) ALPHA
      ENDIF
      CALL MEML1(ALPHA,YNEW,SIGMA)
      ALF=ALPHA
      IF(ABS(YNEW).LT.SIGMA) THEN
        CODE=0
      ELSE
        CODE=1
        R=RATE*SUMF/2.002+WD
        IF(R*ABS(R).GT.WW*DD) THEN
          R=R+SQRT(R*R-WW*DD)
          ALF1=DIV(R,WW)
          ALF2=DIV(DD,R)
        ELSE
          ALF1=DIV(SQRT(DD),SQRT(WW))
          ALF2=ALF1
        ENDIF
        ALF1=MAX(ALF1,ALF)
        ALF2=MIN(ALF2,ALF)
        CALL MEML1(ALF1,Y1,SIGMA)
        IF(Y1.GT.0.) THEN
          ALF=ALF1
        ELSE
          CALL MEML1(ALF2,Y2,SIGMA)
          IF(Y2.LE.0.) THEN
            ALF=ALF2
          ELSE
            DO 2 I=1,30
              ALF=(ALF1+ALF2)/2.
              CALL MEML1(ALF,YNEW,SIGMA)
              IF(YNEW.LE.0.) THEN
                ALF1=ALF
                Y1=YNEW
              ELSE
                ALF2=ALF
                Y2=YNEW
              ENDIF
    2       CONTINUE
            IF(Y1-Y2.NE.0.) THEN
              R=Y1/(Y1-Y2)
              ALF=ALF1+(ALF2-ALF1)*R
            ENDIF
          ENDIF
        ENDIF
      ENDIF
      R=(WW*ALF-2.*WD+DIV(DD,ALF))/(RATE*SUMF)
      IF(R.GT.1.) CODE=2
      PEN1=MAX(ONE,R)
      BETA=ALF*PEN1
      IF(L1.GE.2) THEN
        WRITE(IOUT,60) ALF
        WRITE(IOUT,61) PEN1
      ENDIF
   51 FORMAT('   Control')
   52 FORMAT('   Control on  log alpha',
     *       '               log omega               var omega')
   53 FORMAT(' ',I4,1PE24.15,2E24.15)
   54 FORMAT('     rate ',1PE24.15)
   55 FORMAT('     sumf ',1PE24.15)
   56 FORMAT('      ww  ',1PE24.15)
   57 FORMAT('      wd  ',1PE24.15)
   58 FORMAT('      dd  ',1PE24.15)
   59 FORMAT('     alpha',1PE24.15)
   60 FORMAT('     alpha',1PE24.15)
   61 FORMAT('     pen1 ',1PE24.15)
      END


      SUBROUTINE MEML0(ALPHA,OMEGA,VAR)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*
* Purpose:
*         Insert new entry into table of OMEGA(alpha) with variances,
*         discarding least significant earlier entry if table is full.
*
* Parameters:
*   ARGUMENT TYPE  I/O  DIMENSION   DESCRIPTION
*    ALPHA   R*8   I      -         New alpha
*    OMEGA   R*8   I      -         New omega(alpha)
*    VAR     R*8   I      -         Intrinsic variance of log(OMEGA)
*
* Globals:
*   VARIABLE  COMMON BLOCK  I/O  DESCRIPTION
*    SIZE        MECOML     I    Maximum length of table
*    NTABLE      MECOML     I O  Current length of table
*    XTABLE      MECOML     I O  Values of log(alpha)
*    YTABLE      MECOML     I O  Values of log(omega)
*    VTABLE      MECOML     I O  Values of intrinsic variance
*
* External calls:
*      -
*
* History:
*   John Skilling    7 Dec 1988     MEMSYS3 version
*   JS              10 Dec 1988     Routine name changed
*   JS              10 Dec 1988     Deletion algorithm simplified
*   JS              16 Dec 1988     "L1=3" development code
*   JS               4 Jan 1989     MECOML addressing corrected
*   JS              15 Feb 1989     Ordered table without overlap
*
* Notes:
*   (1) Deletion algorithm is ad hoc.
*   (2) "Development code" can be used to test control of the tables.
*-----------------------------------------------------------------------
      IMPLICIT CHARACTER*1 (A-Z)
      REAL*8 ALPHA,OMEGA,VAR
      INTEGER I,J
      REAL*8 D,S,X,XNEW,YNEW,VNEW,EPS
      PARAMETER (EPS=1.D-20)
      INCLUDE 'MECOML.INC'
      INCLUDE 'MECOMP.INC'
      XNEW=LOG(ALPHA)
      YNEW=LOG(MAX(OMEGA,EPS))
      VNEW=MAX(VAR,EPS)
C Development.... 
C..Write alpha, omega, var to Fortran 3 to be read by separate
C.. graphical routine to display operation of control
           if(l1.eq.3) then
             write(3,*) alpha
             write(3,*) omega
             write(3,*) var
           endif
C ....development
      DO 1 I=1,NTABLE
        IF(XNEW.EQ.XTABLE(I)) THEN
          YTABLE(I)=(YNEW*VTABLE(I)+YTABLE(I)*VNEW)/(VNEW+VTABLE(I))
          VTABLE(I)=VNEW*VTABLE(I)/(VNEW+VTABLE(I))
          RETURN
        ENDIF
    1 CONTINUE
      IF(NTABLE.GE.SIZE) THEN
* Delete worst alpha, preserving order
        S=MAX(VAR,EPS)
        J=-1
        DO 2 I=1,NTABLE
          X=LOG(ALPHA)-XTABLE(I)
          D=VTABLE(I)+X*X*X*X
          IF( D.GT.S ) THEN
            S=D
            J=I
          ENDIF
    2   CONTINUE
        IF(J.EQ.-1) RETURN
        NTABLE=NTABLE-1
        DO 3 I=J,NTABLE
          XTABLE(I)=XTABLE(I+1)
          YTABLE(I)=YTABLE(I+1)
          VTABLE(I)=VTABLE(I+1)
    3   CONTINUE
      ENDIF
* Insert alpha
      J=1
      DO 4 I=1,NTABLE
        IF(XNEW.GT.XTABLE(I)) J=I+1
    4 CONTINUE
      DO 5 I=NTABLE,J,-1
        XTABLE(I+1)=XTABLE(I)
        YTABLE(I+1)=YTABLE(I)
        VTABLE(I+1)=VTABLE(I)
    5 CONTINUE
      XTABLE(J)=XNEW
      YTABLE(J)=YNEW
      VTABLE(J)=VNEW
      NTABLE=NTABLE+1
      END


      SUBROUTINE MEML1(ALPHA,YVAL,SIGMA)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*
* Purpose:
*         Interpolate table to estimate log(omega(alpha)) and its error
*
* Parameters:
*   ARGUMENT TYPE  I/O  DIMENSION   DESCRIPTION
*    ALPHA   R*8   I      -         alpha
*    YVAL    R*8     O    -         Estimate of log(omega(alpha))
*    SIGMA   R*8     O    -         Standard deviation of estimate
*
* Globals:
*   VARIABLE  COMMON BLOCK  I/O  DESCRIPTION
*    SIZE        MECOML     I    Maximum length of table
*    NTABLE      MECOML     I    Current length of table
*    XTABLE      MECOML     I    Values of log(alpha)
*    YTABLE      MECOML     I    Values of log(omega)
*    VTABLE      MECOML     I    Values of intrinsic variance
*
* External calls:
*      -
*
* History:
*   John Skilling    7 Dec 1988     MEMSYS3 version
*   JS              10 Dec 1988     Routine name changed
*   JS               4 Jan 1989     MECOML addressing corrected
*
* Notes:
*-----------------------------------------------------------------------
      IMPLICIT CHARACTER*1 (A-Z)
      REAL*8 ALPHA,YVAL,SIGMA
      INTEGER I
      REAL*8 X,Y,WT,W,WX,WXX,WY,WXY,XBAR,YBAR,EPS,ONE,DIV,xxx,yyy
      PARAMETER (EPS=1.D-20, ONE=1.)
      INCLUDE 'MECOML.INC'
      DIV(xxx,yyy) = xxx/MAX(yyy,EPS)
      W=0.
      WX=0.
      WXX=0.
      WY=0.
      WXY=0.
      IF(NTABLE.EQ.1) THEN
        YVAL=YTABLE(1)
        SIGMA=0.
      ELSE
        DO 1 I=1,NTABLE
          X=LOG(ALPHA)-XTABLE(I)
          Y=YTABLE(I)
          WT=DIV(ONE,VTABLE(I)+X*X*X*X)
          W=W+WT
          WX=WX+WT*X
          WY=WY+WT*Y
    1   CONTINUE
        XBAR=WX/W
        YBAR=WY/W
        DO 2 I=1,NTABLE
          X=LOG(ALPHA)-XTABLE(I)
          Y=YTABLE(I)
          WT=DIV(ONE,VTABLE(I)+X*X*X*X)
          X=X-XBAR
          Y=Y-YBAR
          WXX=WXX+WT*X*X
          WXY=WXY+WT*X*Y
    2   CONTINUE
        YVAL=YBAR-XBAR*DIV(WXY,WXX)
        SIGMA=1./SQRT(W)
      ENDIF
      END


      SUBROUTINE MEPROB(MEMRUN,NRAND,SEED,
     *  DEF,ACC,TOL,ALPHA,S,CHISQ,SDEV,AIM, GOOD,PROB,OMEGA,CODEP,CODEG)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*   Purpose:
*        Calculate      Number of good degrees of freedom
*              and      const + log( prob ( Data | form of transform ) )
*              and      termination criterion.
*
*   Enter with:
*              <1>  = new map f
*     and optionally
*              <22> = accuracies = 1/sigma
*              <29> = nonlinear responsivity
*
*   Workspace:  <2>, <25>,<26>,<27>,<28> 
*
* Parameters:
*   ARGUMENT TYPE  I/O  DIMENSION   DESCRIPTION
*    MEMRUN  I     I      -         0 if initial iterate, 1 if not
*    NRAND   I     I      -         Number random vectors to be used
*    SEED    I     I      -         Seed for reproducible random vectors
*    DEF     R*8   I      -         Default (if constant,& if using +/-)
*    ACC     R*8   I      -         Accuracy      (if constant)
*    TOL     R*8   I      -         Maximisation tolerance 
*                                   (dimensionless O(.1) )
*    ALPHA   R*8   I      -         alpha as returned by MEM3
*    S       R*8   I      -         Entropy as returned by MEM3
*    CHISQ   R*8   I      -         Chisquared as returned by MEM3
*    SDEV    R*8   I      -         scaling of sigma(data)
*                                    as returned by MEM3
*    AIM     R*8   I      -         Scale factor for termination
*    GOOD    R*8     O    -         Number good degrees freedom
*    PROB    R*8     O    -         log prob(Data | form of transform)
*    OMEGA   R*8     O    -         Termination criterion
*    CODEP   I       O    -         Return code for LogProb
*    CODEG   I       O    -         Return code for Good
*         Return codes  =  min(codes returned by calls to MEMDET)
*          code = 0    is "estimated accurately"
*          code = 1    is "max number iterations reached"
*          code = 2,3  is "failure to estimate accurately"
*
* Globals:
*   VARIABLE  COMMON BLOCK  I/O  DESCRIPTION
*    ...         MECOMP     I    MEMSYS pointers
*
* External calls:
*    VRAND       Seed random generator
*    MEMD        Vector random
*    MEMDET      Calculate log determinant(B) and good degrees freedom
*    MEMK        Copy area
*    MEMQ        Apply matrix A
*
* History:
*   John Skilling    7 Dec 1988     MEMSYS3 version
*   JS               9 Dec 1988     Parameter list reordered
*   JS              10 Dec 1988     Routine name changed
*   JS              13 Dec 1988     VRAND calling parameter altered
*   JS               8 Jan 1989     Chisquared entry parameter 
*   JS               9 Jan 1989     Omega exit parameter
*   JS              19 Jan 1989     Invariant use of additive constant C
*   JS              29 Jan 1989     Data and Units removed
*   JS               2 Feb 1989     Initial OMEGA calculation corrected
*   JS              17 Jun 1989     Version 2.00 with pos/neg
*
*  Notes:
*  (1) Each call to MEPROB can be as expensive as a MaxEnt iterate
*  (2) Inaccurate estimates are weighted down severely
*-----------------------------------------------------------------------
      IMPLICIT CHARACTER*1 (A-Z)
      INTEGER I,CG,CP,CODE,CODEG,CODEP,MEMRUN,NRAND,SEED
      REAL*8 ACC,AIM,ALPHA,DEF,GOOD,S,CHISQ,SDEV,TOL,W,WP,WG,OMEGA
      REAL*8 C,PROB,R,LOWG,VALG,UPPG,LOWP,VALP,UPPP
      INCLUDE 'MECOMP.INC'
      IF(L0.GE.1) WRITE(IOUT,51)
   51 FORMAT('   LogProb')
      C=CHISQ-2.*ALPHA*S/(SDEV*SDEV)
      CALL VRAND(SEED,0)
      IF(MEMRUN.EQ.0) THEN
        VALG=0.
        DO 1 I=1,NRAND
          CALL MEMD
          CALL MEMQ(DEF,ACC,W)
          VALG=VALG+W
    1   CONTINUE
        VALG=VALG/NRAND
        CALL MEMK(24,26)
        CALL MEMQ(DEF,ACC,R)
        OMEGA=VALG*SDEV*SDEV*AIM/R
        GOOD=0.
        PROB=-C/2.
        CODEP=0
        CODEG=0
        RETURN
      ENDIF
      PROB=0.
      WG=0.
      CODEG=10
      GOOD=0.
      WP=0.
      CODEP=10
      DO 2 I=1,NRAND
        CALL MEMD
        CALL MEMDET
     *    (ALPHA,C,TOL,DEF,ACC, LOWP,VALP,UPPP,CP,LOWG,VALG,UPPG,CG)
* Include new logprob
        IF(CP.EQ.0) THEN
          W=1.
        ELSE
          W=1.E-6
        ENDIF
        CODEP=MIN(CODEP,CP)
        PROB=PROB+VALP*W
        WP=WP+W
* Include new good
        IF(CG.EQ.0) THEN
          W=1.
        ELSE
          W=1.E-6
        ENDIF
        CODEG=MIN(CODEG,CG)
        GOOD=GOOD+VALG*W
        WG=WG+W
    2 CONTINUE
* Normalise and exit
      PROB=-PROB/(2.*WP)
      GOOD=GOOD/(WG*ALPHA)
      OMEGA=GOOD*SDEV*SDEV*AIM/(-2.*ALPHA*S)
      END


      SUBROUTINE MEMASK(DEFX,ACCX,TOLX,ALPHAX,SDEVX, MEANX,ERRX,CODE)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*   Purpose:
*               Estimate  SUM  f * mask    with error bars
*                          i    i      i
*   Enter with:
*              <1>  = map f
*              <2>  = mask
*     and optionally
*              <20> = default if using pos/neg
*              <22> = accuracies = 1/sigma
*              <29> = nonlinear responsivity
*
*   Workspace:  <2>, <25>,<26>,<27>,<28> 
*
* Parameters:
*   ARGUMENT TYPE  I/O  DIMENSION   DESCRIPTION
*    DEFX    R     I      -         Default (if constant,& if using +/-)
*    ACCX    R     I      -         Accuracy (if constant)
*    TOLX    R     I      -         Maximisation tolerance 
*                                   (dimensionless O(.1) )
*    ALPHAX  R     I      -         alpha as returned by MEM3
*    SDEVX   R     I      -         scaling of sigma(data)
*                                    as returned by MEM3
*    MEANX   R       O    -         mean of required sum
*    ERRX    R       O    -         standard deviation of required sum
*    CODE    I       O    -         Return code passed up by MEMCG
*              0    is "standard deviation was estimated accurately"
*              1    is "max number iterations reached in conjgrad"
*              2,3  is "failure to estimate standard dev. accurately"
*
* Globals:
*     -
*
* External calls:
*    MEMOP       Call user transform routine OPUS
*    MEMQ        Application of matrix A
*    MEMU        Contravariant mask
*    MEMV        Initialise mask calculation
*    MEMW        Initialise mask calculation
*    MEMCG       Conjugate gradient
*
* History:
*   John Skilling    7 Dec 1988     MEMSYS3 version
*   JS               9 Dec 1988     Parameter list reordered
*   JS              10 Dec 1988     Routine name changed
*   JS               8 Jan 1989     Version 1.5
*   JS              17 Jun 1989     Version 2.00 with pos/neg
*
*  Notes:
*  (1) MEM3 must be called before MEMASK.
*  (2) A call to MEMASK may be more expensive than a MaxEnt iterate.
*  (3) Calculation of standard deviation can be very ill-conditioned,
*      involving inverse of B without a factor A, and is prone to
*      serious rounding error.  The value of the return code should be
*      checked (zero indicates correct operation of the procedure).
*      The only remedy for serious error may be full double precision
*      operation.
*-----------------------------------------------------------------------
      IMPLICIT CHARACTER*1 (A-Z)
      INTEGER CODE
      REAL DEFX,ACCX,TOLX,ALPHAX,SDEVX,MEANX,ERRX
      REAL*8 DEF,ALPHA,SDEV,TOL,ACC,MEAN,ERR
      REAL*8 C,PFP,P,Q,R,U,V,LO,UP,EPS
      PARAMETER (EPS=1.D-20)
      DEF=DEFX
      ACC=ACCX
      TOL=TOLX
      ALPHA=ALPHAX
      SDEV=SDEVX
* Begin exit from image-space ..........................................
      CALL MEMU(DEF, MEAN,PFP)
      IF(PFP.LT.EPS) THEN
        MEANX=MEAN
        ERRX=0.
        CODE=0
        RETURN
      ENDIF
      CALL MEMOP(2,25)
      CALL MEMMUL(25,26,ACC)
*..............................................end exit from image-space
      CALL MEMV(P)
* Preliminary jump along <26> to reduce effect of rounding errors
      CALL MEMQ(DEF,ACC,Q)
      U=P/(ALPHA*P+Q)
      V=Q/(ALPHA*P+Q)
      CALL MEMW(U,V,R)
* Main conjugate gradient iteration
      C=ALPHA*PFP-R-ALPHA*U*P
      CALL MEMCG(0,ALPHA,C,TOL,DEF,ACC, LO,R,UP,CODE)
      ERR=SQRT(R)*SDEV/ALPHA
      MEANX=MEAN
      ERRX=ERR
      END
