* REAL*8 is for external controlling routines
* REAL is for called vector routines

* The only routines which interact with image-space are:-
*  a) the transform routines themselves,
*  b) MEMA, setting new map and its entropy
*  c) MEMQ, applying the MaxEnt matrix  A = acc.Opus.{f}.Tropus.acc
*  d) MEMU, giving statistics of the user's image-space MASK.

      SUBROUTINE MEINIT
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*
* Purpose:
*             Initialise MEMSYS pointers
*
* Parameters:
*   ARGUMENT TYPE  I/O  DIMENSION   DESCRIPTION
*    -
*
* Globals:
*   VARIABLE  COMMON BLOCK  I/O  DESCRIPTION
*    ...         MECOMP       O  Sizes and flags
*    L0,L1       MECOMP       O  Diagnostic levels (set to 0)
*    NTRANS      MECOMP       O  Transform counter (set to 0)
*    PR          MECOMC       O  Flowchart flags
*
* External calls:
*    URESET      Reset individual pointers
*
* History:
*   John Skilling    9 Dec 1988     First MEMSYS3 version
*   JS              10 Dec 1988     Transform counter
*   JS              13 Dec 1988     L0,L1 set to 0
*
* Notes:
*-----------------------------------------------------------------------
      IMPLICIT CHARACTER*1 (A-Z)
      INTEGER I
      INCLUDE 'MECOMP.INC'
      INCLUDE 'MECOMC.INC'
      L0=0
      L1=0
      DO 1 I=1,40
        PR(I)='.'
        CALL URESET(I)
    1 CONTINUE
      NTRANS=0
      END

      SUBROUTINE MEM0
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*
* Purpose:
*          <23>  := 0
*
*
* Parameters:
*   ARGUMENT TYPE  I/O  DIMENSION   DESCRIPTION
*     -
*
* Globals:
*   VARIABLE  COMMON BLOCK  I/O  DESCRIPTION
*    ...         MECOMP     I    Sizes and flags
*
* External calls:
*    UWRITE      Write area
*    UINIT       Initialise area
*    VFILL       Vector fill
*
* History:
*   John Skilling    7 Dec 1988     First MEMSYS3 version
*   JS              29 Jan 1989     Version 1.6
*
* Notes:
*-----------------------------------------------------------------------
      IMPLICIT CHARACTER*1 (A-Z)
      REAL ZERO
      PARAMETER (ZERO=0.)
      INTEGER I
      INCLUDE 'MECOMP.INC'
      IF(L0.GE.1) WRITE(IOUT,5)
    5 FORMAT('   Mem0')
      DO 1 I=1,NK
        CALL VFILL(KC(23),ZERO,MK)
        CALL UWRITE(23)
    1 CONTINUE
      CALL UINIT
      END

      SUBROUTINE MEMA(DEFX,SUMFX,SX)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*
* Purpose:
*          <1>  :=  <2>   := def * exp<2>
*          S    := Entropy = SUM( <1> - def - <1> log(<1>/def) )
*          SUMF := Flux    = SUM( <1> )
*
*     or the equivalent for positive/negative
*          S    := Entropy = SUM( 2*def * ( cosh<2> - 1 - <2> sinh<2> )
*          SUMF := TotalFlux = SUM( 2*def * cosh<2> )
*          <1>  :=  <2>  := def * 2*sinh<2>
*
* Parameters:
*   ARGUMENT TYPE  I/O  DIMENSION   DESCRIPTION
*    DEFX    R*8   I      -         Default level (if constant)
*    SUMFX   R*8     O    -         Flux
*    SX      R*8     O    -         Entropy
*
* Globals:
*   VARIABLE  COMMON BLOCK  I/O  DESCRIPTION
*    ...         MECOMP     I    Sizes and flags
*    M1          MECOMP     I    Type of default
*
* External calls:
*    UREAD       Read area
*    UPDATE      Update area
*    UWRITE      Write area
*    UINIT       Initialise area
*    VMOV        Vector move
*    VFILL       Vector fill
*    VSUM        Vector sum
*    VSUB        Vector subtract
*    VMUL        Vector multiply
*    VDIV        Vector divide
*    VDOT        Vector dot product
*    VSMUL       Vector scalar multiply
*    VENT1       Covariant entropy vector
*    VENT2       Covariant entropy vector for pos/neg
*
* History:
*   John Skilling    7 Dec 1988     First MEMSYS3 version
*   JS              17 Jun 1989     Version 2.00 with pos/neg
*   JS               7 Sep 1989     Version 2.01 reconfigured loops
*
* Notes:
*-----------------------------------------------------------------------
      IMPLICIT CHARACTER*1 (A-Z)
      REAL*8 DEFX,SUMFX,SX
      REAL DEF,SUMF,SUM1,SUM2,S,ONE
      INTEGER I
      INCLUDE 'MECOMP.INC'
      PARAMETER (ONE=1.)
      IF(L0.GE.1) WRITE(IOUT,5)
    5 FORMAT('   MemA')
      DEF=DEFX
      SUMFX=0.
      SX=0.
      IF(M1.EQ.0) THEN
        DO 1 I=1,NJ
          CALL UPDATE(2)
          CALL VENT1(KC(1), KC(2),MJ)
          CALL VSUM(KC(2), S,MJ)
          CALL VSUM(KC(1), SUMF,MJ)
          CALL VSMUL(KC(1),DEF, KC(1),MJ)
          S=S*DEF
          SUMF=SUMF*DEF
          CALL VMOV(KC(1),KC(2),MJ)
          CALL UWRITE(1)
          CALL UWRITE(2)
          SUMFX=SUMFX+SUMF
          SX=SX+S
    1   CONTINUE
      ELSEIF(M1.EQ.1) THEN
        DO 2 I=1,NJ
          CALL UPDATE(2)
          CALL UREAD(20)
          CALL VENT1(KC(1), KC(2),MJ)
          CALL VDOT(KC(2),KC(20), S,MJ)
          CALL VMUL(KC(1),KC(20), KC(1),MJ)
          CALL VSUM(KC(1), SUMF,MJ)
          CALL VMOV(KC(1),KC(2),MJ)
          CALL UWRITE(1)
          CALL UWRITE(2)
          SUMFX=SUMFX+SUMF
          SX=SX+S
    2   CONTINUE
      ELSEIF(M1.EQ.2) THEN
        DO 3 I=1,NJ
          CALL UPDATE(2)
          CALL VENT2(KC(1), KC(2),MJ)
          CALL VSUM(KC(2), S,MJ)
          CALL VSUM(KC(1), SUM1,MJ)
          CALL VFILL(KC(2),ONE,MJ)
          CALL VDIV(KC(2),KC(1), KC(2),MJ)
          CALL VSUM(KC(2), SUM2,MJ)
          CALL VSUB(KC(1),KC(2), KC(1),MJ)
          CALL VSMUL(KC(1),DEF, KC(1),MJ)
          S=S*DEF
          SUMF=(SUM1+SUM2)*DEF
          CALL VMOV(KC(1),KC(2),MJ)
          CALL UWRITE(1)
          CALL UWRITE(2)
          SUMFX=SUMFX+SUMF
          SX=SX+S
    3   CONTINUE
      ELSEIF(M1.EQ.3) THEN
        DO 4 I=1,NJ
          CALL UPDATE(2)
          CALL UREAD(20)
          CALL VENT2(KC(1), KC(2),MJ)
          CALL VDOT(KC(2),KC(20), S,MJ)
          CALL VDOT(KC(1),KC(20), SUM1,MJ)
          CALL VFILL(KC(2),ONE,MJ)
          CALL VDIV(KC(2),KC(1), KC(2),MJ)
          CALL VDOT(KC(2),KC(20), SUM2,MJ)
          CALL VSUB(KC(1),KC(2), KC(1),MJ)
          CALL VMUL(KC(1),KC(20), KC(1),MJ)
          SUMF=SUM1+SUM2
          CALL VMOV(KC(1),KC(2),MJ)
          CALL UWRITE(1)
          CALL UWRITE(2)
          SUMFX=SUMFX+SUMF
          SX=SX+S
    4   CONTINUE
      ENDIF
      CALL UINIT
      END

      SUBROUTINE MEMB(ACCX,LHOODX)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*
* Purpose:
*         <24>  := ( <21> - <25> ) * acc
*                = ( Data - Mockdata ) /sigma = Scaled residuals
*
*         LHOOD := <24>.<24> / 2          ( >= 0)
*
*     or the equivalent for Poisson statistics:
*
*         <22>  := sqrt(<21>+1) / <25>
*                = sqrt(gradgrad(-log(lhood))
*                = sqrt(data) / mockdata = accuracies
*
*         <24>  := ( <21> - <25> ) / sqrt(<21>+1)
*                = - grad (-log Lhood) / sqrt(gradgrad(-log Lhood))
*                = Scaled residuals
*
*         LHOOD := SUM ( <25> - <21> - <21> log(<25>/<21>) )
*
*            for   pr(Data|Mock) = UNITS - LHOOD
*
* Parameters:
*   ARGUMENT TYPE  I/O  DIMENSION   DESCRIPTION
*    ACCX    R*8   I      -         Accuracy = 1/sigma
*    LHOODX  R*8     O    -         L = - log likelihood
*
* Globals:
*   VARIABLE  COMMON BLOCK  I/O  DESCRIPTION
*    ...         MECOMP     I    Sizes and flags
*    M2          MECOMP     I    Type of accuracy
*
* External calls:
*    UREAD       Read area
*    UWRITE      Write area
*    UINIT       Initialise area
*    VSUB        Vector subtract
*    VMUL        Vector multiply
*    VDIV        Vector divide
*    VSADD       Vector scalar add
*    VSMUL       Vector scalar multiply
*    VSUM        Vector sum
*    VDOT        Vector dot product
*    VLOG        Vector natural logarithm
*    VSQRT       Vector square root
*
* History:
*   John Skilling    7 Dec 1988     First MEMSYS3 version
*   JS               9 Dec 1988     Unscaled mock data
*   JS              10 Dec 1988     Scaling correction
*   David Robinson   8 Mar 1988     First Poisson version
*   JS              17 Jun 1989     Redefinition of LHOOD (see MEMI)
*   JS               7 Sep 1989     Version 2.01 reconfigured loops
*
* Notes:
*  (1)  These forms of LHOOD have a zero minimum at F=D.
*  (2)  Adding 1 to D when setting Poisson accuracies can be justified
*       by averaging gradgradL over a reasonable range of values of F.
*-----------------------------------------------------------------------
      IMPLICIT CHARACTER*1 (A-Z)
      REAL*8 ACCX,LHOODX
      REAL ACC,LHOOD,A,B,ONE
      INTEGER I
      INCLUDE 'MECOMP.INC'
      PARAMETER (ONE=1.)
      IF(L0.GE.1) WRITE(IOUT,5)
    5 FORMAT('   MemB')
      ACC=ACCX
      LHOODX=0.
      IF(M2.EQ.0) THEN
        DO 1 I=1,NK
          CALL UREAD(21)
          CALL UREAD(25)
          CALL VSUB(KC(21),KC(25),KC(24),MK)
          CALL UREAD(22)
          CALL VMUL(KC(24),KC(22),KC(24),MK)
          CALL VDOT(KC(24),KC(24),A,MK)
          LHOOD=A/2.
          CALL UWRITE(24)
          LHOODX=LHOODX+LHOOD
    1   CONTINUE
      ELSEIF(M2.EQ.1) THEN
        DO 2 I=1,NK
          CALL UREAD(21)
          CALL UREAD(25)
          CALL VSUB(KC(21),KC(25),KC(24),MK)
          CALL VSMUL(KC(24),ACC,KC(24),MK)
          CALL VDOT(KC(24),KC(24),A,MK)
          LHOOD=A/2.
          CALL UWRITE(24)
          LHOODX=LHOODX+LHOOD
    2   CONTINUE
      ELSEIF(M2.EQ.2) THEN
        DO 3 I=1,NK
          CALL UREAD(21)
          CALL UREAD(25)
          CALL VSUB(KC(21),KC(25),KC(24),MK)
          CALL VDIV(KC(25),KC(21), KC(22),MK)
          CALL VLOG(KC(22),        KC(22),MK)
          CALL VDOT(KC(21),KC(22), A,MK)
          CALL VSUB(KC(25),KC(21), KC(22),MK)
          CALL VSUM(KC(22),        B,MK)
          LHOOD=B-A
          CALL VSADD(KC(21),ONE,   KC(22),MK)
          CALL VSQRT(KC(22),       KC(22),MK)
          CALL VDIV(KC(24),KC(22), KC(24),MK)
          CALL VDIV(KC(22),KC(25), KC(22),MK)
          CALL UWRITE(22)
          CALL UWRITE(24)
          LHOODX=LHOODX+LHOOD
    3   CONTINUE
      ENDIF
      CALL UINIT
      END

      SUBROUTINE MEMC(WWX,WDX,DDX)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*
* Purpose:
*         Trigonometric relations between gradS = <25> and gradC = <24>
*
*          ww := <25>.<25>  ,  wd := <25>.<24>  ,  dd := <24>.<24>
*
* Parameters:
*   ARGUMENT TYPE  I/O  DIMENSION   DESCRIPTION
*    WWX     R*8     O    -         gradS.gradS
*    WDX     R*8     O    -         gradS.gradC
*    DDX     R*8     O    -         gradC.gradC
*
* Globals:
*   VARIABLE  COMMON BLOCK  I/O  DESCRIPTION
*    ...         MECOMP     I    Sizes and flags
*
* External calls:
*    UREAD       Read area
*    UINIT       Initialise area
*    VDOT        Vector dot product
*
* History:
*   John Skilling    7 Dec 1988     First MEMSYS3 version
*   JS              26 Jan 1989     Version 1.6
*
* Notes:
*-----------------------------------------------------------------------
      IMPLICIT CHARACTER*1 (A-Z)
      REAL*8 WWX,WDX,DDX
      REAL WW,WD,DD
      INTEGER I
      INCLUDE 'MECOMP.INC'
      IF(L0.GE.1) WRITE(IOUT,5)
    5 FORMAT('   MemC')
      WWX=0.
      WDX=0.
      DDX=0.
      DO 2 I=1,NK
        CALL UREAD(25)
        CALL UREAD(24)
        CALL VDOT(KC(25),KC(25),WW,MK)
        CALL VDOT(KC(25),KC(24),WD,MK)
        CALL VDOT(KC(24),KC(24),DD,MK)
        WWX=WWX+WW
        WDX=WDX+WD
        DDX=DDX+DD
    2 CONTINUE
      CALL UINIT
      END

      SUBROUTINE MEMD
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*
* Purpose:
*           <26> := Random vector with components from Normal(0,1)
*
* Parameters:
*   ARGUMENT TYPE  I/O  DIMENSION   DESCRIPTION
*     -
*
* Globals:
*   VARIABLE  COMMON BLOCK  I/O  DESCRIPTION
*    ...         MECOMP     I    Sizes and flags
*
* External calls:
*    UWRITE      Write area
*    UINIT       Initialise area
*    VRAND       Vector random
*
* History:
*   John Skilling    7 Dec 1988     First MEMSYS3 version
*
* Notes:
*-----------------------------------------------------------------------
      IMPLICIT CHARACTER*1 (A-Z)
      INTEGER I
      INCLUDE 'MECOMP.INC'
      IF(L0.GE.1) WRITE(IOUT,5)
    5 FORMAT('   MemD')
      DO 1 I=1,NK
        CALL VRAND(KC(26),MK)
        CALL UWRITE(26)
    1 CONTINUE
      CALL UINIT
      END

      SUBROUTINE MEME(ALPHAX)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*
* Purpose:
*                      <26> :=  <24> - alpha*<25>  
*
*      (Interpret as   <26>  = gradL - alpha*gradS  =  gradQ   )
*
* Parameters:
*   ARGUMENT TYPE  I/O  DIMENSION   DESCRIPTION
*    ALPHAX  R*8   I      -         Lagrange multiplier alpha
*
* Globals:
*   VARIABLE  COMMON BLOCK  I/O  DESCRIPTION
*    ...         MECOMP     I    Sizes and flags
*
* External calls:
*    UREAD       Read area
*    UWRITE      Write area
*    UINIT       Initialise area
*    VSMULA      Vector scalar multiply and add
*
* History:
*   John Skilling    7 Dec 1988     First MEMSYS3 version
*   JS              26 Jan 1989     Version 1.6
*
* Notes:
*-----------------------------------------------------------------------
      IMPLICIT CHARACTER*1 (A-Z)
      REAL*8 ALPHAX
      REAL ALPHA
      INTEGER I
      INCLUDE 'MECOMP.INC'
      IF(L0.GE.1) WRITE(IOUT,5)
    5 FORMAT('   MemE')
      ALPHA=ALPHAX
      DO 2 I=1,NK
        CALL UREAD(25)
        CALL UREAD(24)
        CALL VSMULA(KC(25),-ALPHA,KC(24),KC(26),MK)
        CALL UWRITE(26)
    2 CONTINUE
      CALL UINIT
      END

      SUBROUTINE MEMI(ACCX, DATAX,UNITSX)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*
* Purpose:
*          DATA  := Number of nonzero entries in acc
*
*          UNITS := Sum log ( nonzero entries in acc / sqrt(2*pi) )
*            for   pr(Data|Mock) = UNITS - LHOOD
*
*
* Parameters:
*   ARGUMENT TYPE  I/O  DIMENSION   DESCRIPTION
*    ACCX    R*8   I      -         Accuracy = 1/sigma
*    DATAX   R*8     O    -         Number of data
*    UNITSX  R*8     O    -         Sum log accuracy
*
* Globals:
*   VARIABLE  COMMON BLOCK  I/O  DESCRIPTION
*    ...         MECOMP     I    Sizes and flags
*    M2          MECOMP     I    Type of accuracy
*
* External calls:
*    UREAD       Read area
*    UINIT       Initialise area
*    VSUML       Vector sum logarithms
*
* History:
*   John Skilling     29 Jan 1989     Version 1.6
*   David Robinson     8 Mar 1989     Poisson stats added
*   DR/JS             14 Jun 1989     Poisson stats corrected
*   JS                 7 Sep 1989     Version 2.01 reconfigured loops
*
* Notes:
*  (1) The Poisson (M2=2) contribution ( D log D - D - log D! ) to UNITS
*      is here approximated by  -(1/2) log( 1 + 2*pi*D ) .
*      Importantly, error = 0 for D=0,  error -> 0 as D -> infinity.
*      Max error = 0.007 (less than 1%) at D=1.
*-----------------------------------------------------------------------
      IMPLICIT CHARACTER*1 (A-Z)
      REAL*8 ACCX,DATAX,UNITSX,PILOG
      REAL ACC,DATA,UNITS,ONE,PI2
      INTEGER I
      INCLUDE 'MECOMP.INC'
*  PI2 = 1/(2*pi) , PILOG = log(sqrt(2*pi))
      PARAMETER (PI2=.1591549430918953D0,PILOG=.9189385332046728D0)
      PARAMETER (ONE=1.)
      IF(L0.GE.1) WRITE(IOUT,5)
    5 FORMAT('   MemI')
      ACC=ACCX
      DATAX=0.
      UNITSX=0.
      IF(M2.EQ.0) THEN
        DO 1 I=1,NK
          CALL UREAD(22)
          CALL VSUML(KC(22),DATA,UNITS,MK)
          DATAX=DATAX+DATA
          UNITSX=UNITSX+UNITS
    1   CONTINUE
      ELSEIF(M2.EQ.1) THEN
        DO 2 I=1,NK
          DATA=FLOAT(MK)
          UNITS=DATA*LOG(ACC)
          DATAX=DATAX+DATA
          UNITSX=UNITSX+UNITS
    2   CONTINUE
      ELSEIF(M2.EQ.2) THEN
        DO 3 I=1,NK
          CALL UREAD(21)
          CALL VSADD(KC(21),PI2, KC(25),MK)
          CALL VSUML(KC(25),DATA,UNITS,MK)
          UNITS=-UNITS/2.
          DATAX=DATAX+DATA
          UNITSX=UNITSX+UNITS
    3   CONTINUE
      ENDIF
      UNITSX=UNITSX-DATAX*PILOG
      CALL UINIT
      END

      SUBROUTINE MEMJ(M,N)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*
* Purpose:
*          Copy map-space area M to N   ,   <N> := <M>
*
* Parameters:
*   ARGUMENT TYPE  I/O  DIMENSION   DESCRIPTION
*    M       I     I      -         Input area (map-space)
*    N       I     I      -         Output area
*
* Globals:
*   VARIABLE  COMMON BLOCK  I/O  DESCRIPTION
*    ...         MECOMP     I    Sizes and flags
*
* External calls:
*    UREAD       Read area
*    UWRITE      Write area
*    UINIT       Initialise area
*    VMOV        Vector move
*
* History:
*   John Skilling    7 Dec 1988     First MEMSYS3 version
*
* Notes:
*-----------------------------------------------------------------------
      IMPLICIT CHARACTER*1 (A-Z)
      INTEGER I,M,N
      INCLUDE 'MECOMP.INC'
      IF(L0.GE.1) WRITE(IOUT,5)
    5 FORMAT('   MemJ')
      IF( KA(M).NE.KA(N) .OR. KB(M).NE.KB(N) ) THEN
        DO 1 I=1,NJ
          CALL UREAD(M)
          CALL VMOV(KC(M),KC(N),MJ)
          CALL UWRITE(N)
    1   CONTINUE
        CALL UINIT
      ENDIF
      END

      SUBROUTINE MEMK(M,N)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*
* Purpose:
*          Copy data-space area M to N   ,   <N> := <M>
*
* Parameters:
*   ARGUMENT TYPE  I/O  DIMENSION   DESCRIPTION
*    M       I     I      -         Input area (data-space)
*    N       I     I      -         Output area
*
* Globals:
*   VARIABLE  COMMON BLOCK  I/O  DESCRIPTION
*    ...         MECOMP     I    Sizes and flags
*
* External calls:
*    UREAD       Read area
*    UWRITE      Write area
*    UINIT       Initialise area
*    VMOV        Vector move
*
* History:
*   John Skilling    7 Dec 1988     First MEMSYS3 version
*
* Notes:
*-----------------------------------------------------------------------
      IMPLICIT CHARACTER*1 (A-Z)
      INTEGER I,M,N
      INCLUDE 'MECOMP.INC'
      IF(L0.GE.1) WRITE(IOUT,5)
    5 FORMAT('   MemK')
      IF( KA(M).NE.KA(N) .OR. KB(M).NE.KB(N) ) THEN
        DO 1 I=1,NK
          CALL UREAD(M)
          CALL VMOV(KC(M),KC(N),MK)
          CALL UWRITE(N)
    1   CONTINUE
        CALL UINIT
      ENDIF
      END

      SUBROUTINE MEMP
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*
* Purpose:
*         Initialise conjugate gradient work areas
*
*           <27> :=  <28> := 0 
*
* Parameters:
*   ARGUMENT TYPE  I/O  DIMENSION   DESCRIPTION
*     -
*
* Globals:
*   VARIABLE  COMMON BLOCK  I/O  DESCRIPTION
*    ...         MECOMP     I    Sizes and flags
*
* External calls:
*    UWRITE      Write area
*    UINIT       Initialise area
*    VFILL       Vector fill
*
* History:
*   John Skilling    7 Dec 1988     First MEMSYS3 version
*
* Notes:
*-----------------------------------------------------------------------
      IMPLICIT CHARACTER*1 (A-Z)
      REAL ZERO
      PARAMETER (ZERO=0.)
      INTEGER I
      INCLUDE 'MECOMP.INC'
      IF(L0.GE.1) WRITE(IOUT,5)
    5 FORMAT('   MemP')
      DO 1 I=1,NK
        CALL VFILL(KC(27),ZERO,MK)
        CALL VFILL(KC(28),ZERO,MK)
        CALL UWRITE(27)
        CALL UWRITE(28)
    1 CONTINUE
      CALL UINIT
      END

      SUBROUTINE MEMQ(DEFX,ACCX,PX)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*
* Purpose:
*            Apply matrix  A = acc.Opus.{metric}.Tropus.acc
*             where   metric = f = <1>              (positive only)
*             or      metric = sqrt(f*f+4*m*m)      (positive/negative)
*
*             <25> :=  A.<26>
*             p    :=  <26>.A.<26>
*
* Parameters:
*   ARGUMENT TYPE  I/O  DIMENSION   DESCRIPTION
*    DEFX    R*8   I      -         Default (if constant, & if pos/neg)
*    ACCX    R*8   I      -         Accuracy (if constant)
*    PX      R*8     O    -         Magnitude-squared
*
* Globals:
*   VARIABLE  COMMON BLOCK  I/O  DESCRIPTION
*    ...         MECOMP     I    Sizes and flags
*    M1          MECOMP     I    Type of metric
*
* External calls:
*    MEMTR       Scaled transpose transform
*    MEMOP       Scaled transform
*    MEMMUL      Multiply by accuracy
*    UREAD       Read area
*    UPDATE      Update area
*    UWRITE      Write area
*    UINIT       Initialise area
*    VDOT1       Vector apply metric
*    VDOT2       Vector apply metric
*    VDOT3       Vector apply metric
*
* History:
*   John Skilling    7 Dec 1988     First MEMSYS3 version
*   JS              26 Jan 1989     Version 1.6
*   JS              17 Jun 1989     Version 2.00 with pos/neg
*   JS               7 Sep 1989     Version 2.01 reconfigured loops
*
* Notes:
*-----------------------------------------------------------------------
      IMPLICIT CHARACTER*1 (A-Z)
      REAL*8 DEFX,ACCX,PX
      REAL DEF,P
      INTEGER I
      INCLUDE 'MECOMP.INC'
      DEF=DEFX
      CALL MEMMUL(26,25,ACCX)
      CALL MEMTR(25,2)
      IF(L0.GE.1) WRITE(IOUT,5)
    5 FORMAT('   MemQ')
      PX=0.
      IF(M1.LE.1) THEN
        DO 1 I=1,NJ
          CALL UREAD(1)
          CALL UPDATE(2)
          CALL VDOT1(KC(2),KC(1), P,MJ)
          CALL UWRITE(2)
          PX=PX+P
    1   CONTINUE
      ELSEIF(M1.EQ.2) THEN
        DO 2 I=1,NJ
          CALL UREAD(1)
          CALL UPDATE(2)
          CALL VDOT2(KC(2),KC(1),DEF, P,MJ)
          CALL UWRITE(2)
          PX=PX+P
    2   CONTINUE
      ELSEIF(M1.EQ.3) THEN
        DO 3 I=1,NJ
          CALL UREAD(1)
          CALL UPDATE(2)
          CALL UREAD(20)
          CALL VDOT3(KC(2),KC(1),KC(20), P,MJ)
          CALL UWRITE(2)
          PX=PX+P
    3   CONTINUE
      ENDIF
      CALL UINIT
      CALL MEMOP(2,25)
      CALL MEMMUL(25,25,ACCX)
      END

      SUBROUTINE MEMR(ALFX,PX, RX,SX,TX)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*
* Purpose:
*         Conjugate gradient update
*            <27> += p * <26>
*            <28> += p * <25>
*            <25> := <28> + alpha * <27>
*              r  := <26>.<26>
*              s  := <27>.<28>
*              t  := <28>.<28>
*
* Parameters:
*   ARGUMENT TYPE  I/O  DIMENSION   DESCRIPTION
*    PX      R*8   I      -         Multiplicative factor
*    ALFX    R*8   I      -         alpha
*    RX      R*8     O    -         Scalar
*    SX      R*8     O    -         Scalar
*    TX      R*8     O    -         Scalar
*
* Globals:
*   VARIABLE  COMMON BLOCK  I/O  DESCRIPTION
*    ...         MECOMP     I    Sizes and flags
*
* External calls:
*    UREAD       Read area
*    UPDATE      Update area
*    UWRITE      Write area
*    UINIT       Initialise area
*    VSMULA      Vector scalar multiply and add
*    VDOT        Vector scalar product
*
* History:
*   John Skilling    7 Dec 1988     First MEMSYS3 version
*
* Notes:
*-----------------------------------------------------------------------
      IMPLICIT CHARACTER*1 (A-Z)
      REAL*8 ALFX,PX,RX,SX,TX
      REAL ALF,P,R,S,T
      INTEGER I
      INCLUDE 'MECOMP.INC'
      IF(L0.GE.1) WRITE(IOUT,5)
    5 FORMAT('   MemR')
      ALF=ALFX
      P=PX
      RX=0.
      SX=0.
      TX=0.
      DO 1 I=1,NK
        CALL UPDATE(25)
        CALL UREAD(26)
        CALL UPDATE(27)
        CALL UPDATE(28)
        CALL VSMULA(KC(26),P,KC(27),KC(27),MK)
        CALL VSMULA(KC(25),P,KC(28),KC(28),MK)
        CALL VSMULA(KC(27),ALF,KC(28),KC(25),MK)
        CALL VDOT(KC(26),KC(26),R,MK)
        CALL VDOT(KC(27),KC(28),S,MK)
        CALL VDOT(KC(28),KC(28),T,MK)
        CALL UWRITE(25)
        CALL UWRITE(27)
        CALL UWRITE(28)
        RX=RX+R
        SX=SX+S
        TX=TX+T
    1 CONTINUE
      CALL UINIT
      END

      SUBROUTINE MEMS(QX)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*
* Purpose:
*         Update memory <26> for conjugate gradient algorithm
*
*            <26> -= q * <25>
*
* Parameters:
*   ARGUMENT TYPE  I/O  DIMENSION   DESCRIPTION
*    QX      R*8   I      -         Multiplicative fcator
*
* Globals:
*   VARIABLE  COMMON BLOCK  I/O  DESCRIPTION
*    ...         MECOMP     I    Sizes and flags
*
* External calls:
*    UREAD       Read area
*    UPDATE      Update area
*    UWRITE      Write area
*    UINIT       Initialise area
*    VSMULA      Vector scalar multiply and add
*
* History:
*   John Skilling    7 Dec 1988     First MEMSYS3 version
*   JS              26 Jan 1989     Version 1.6
*
* Notes:
*-----------------------------------------------------------------------
      IMPLICIT CHARACTER*1 (A-Z)
      REAL*8 QX
      REAL Q
      INTEGER I
      INCLUDE 'MECOMP.INC'
      IF(L0.GE.1) WRITE(IOUT,5)
    5 FORMAT('   MemS')
      Q=QX
      DO 1 I=1,NK
        CALL UREAD(25)
        CALL UPDATE(26)
        CALL VSMULA(KC(25),-Q,KC(26),KC(26),MK)
        CALL UWRITE(26)
    1 CONTINUE
      CALL UINIT
      END

      SUBROUTINE MEMT(QX)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*
* Purpose:
*       Update Lagrange multipliers <23> in conjugate gradient algorithm
*
*            <23> += q * <25>
*
* Parameters:
*   ARGUMENT TYPE  I/O  DIMENSION   DESCRIPTION
*    QX      R*8   I      -         Multiplicative fcator
*
* Globals:
*   VARIABLE  COMMON BLOCK  I/O  DESCRIPTION
*    ...         MECOMP     I    Sizes and flags
*
* External calls:
*    UREAD       Read area
*    UPDATE      Update area
*    UWRITE      Write area
*    UINIT       Initialise area
*    VSMULA      Vector scalar multiply and add
*
* History:
*   John Skilling    7 Dec 1988     First MEMSYS3 version
*   JS              26 Jan 1989     Version 1.6
*
* Notes:
*-----------------------------------------------------------------------
      IMPLICIT CHARACTER*1 (A-Z)
      REAL*8 QX
      REAL Q
      INTEGER I
      INCLUDE 'MECOMP.INC'
      IF(L0.GE.1) WRITE(IOUT,5)
    5 FORMAT('   MemT')
      Q=QX
      DO 2 I=1,NK
        CALL UREAD(25)
        CALL UPDATE(23)
        CALL VSMULA(KC(25),Q,KC(23),KC(23),MK)
        CALL UWRITE(23)
    2 CONTINUE
      CALL UINIT
      END

      SUBROUTINE MEMU(DEFX,MX,PX)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*
* Purpose:
*            Flux, magnitude, and contravariant form of covariant map
*
*             m  :=  <1>.<2>
*             p  :=  <2>.{metric}.<2>
*            <2> :=  {metric} <2>
*             where   metric = f = <1>              (positive only)
*             or      metric = sqrt(f*f+4*m*m)      (positive/negative)
*
* Parameters:
*   ARGUMENT TYPE  I/O  DIMENSION   DESCRIPTION
*    DEFX    R*8   I      -         Default (if constant, & if pos/neg)
*    MX      R*8     O    -         Flux
*    PX      R*8     O    -         Magnitude-squared
*
* Globals:
*   VARIABLE  COMMON BLOCK  I/O  DESCRIPTION
*    ...         MECOMP     I    Sizes and flags
*
* External calls:
*    UREAD       Read area
*    UPDATE      Update area
*    UWRITE      Write area
*    UINIT       Initialise area
*    VDOT        Vector scalar product
*    VDOT1       Vector apply metric
*    VDOT2       Vector apply metric
*    VDOT3       Vector apply metric
*
* History:
*   John Skilling    7 Dec 1988     First MEMSYS3 version
*   JS              17 Jun 1989     Version 2.00 with pos/neg
*   JS               7 Sep 1989     Version 2.01 reconfigured loops
*
* Notes:
*-----------------------------------------------------------------------
      IMPLICIT CHARACTER*1 (A-Z)
      REAL*8 DEFX,MX,PX
      REAL DEF,M,P
      INTEGER I
      INCLUDE 'MECOMP.INC'
      IF(L0.GE.1) WRITE(IOUT,5)
    5 FORMAT('   MemU')
      DEF=DEFX
      MX=0.
      PX=0.
      IF(M1.LE.1) THEN
        DO 1 I=1,NJ
          CALL UREAD(1)
          CALL UPDATE(2)
          CALL VDOT(KC(2),KC(1),M,MJ)
          CALL VDOT1(KC(2),KC(1), P,MJ)
          CALL UWRITE(2)
          MX=MX+M
          PX=PX+P
    1   CONTINUE
      ELSEIF(M1.EQ.2) THEN
        DO 2 I=1,NJ
          CALL UREAD(1)
          CALL UPDATE(2)
          CALL VDOT(KC(2),KC(1),M,MJ)
          CALL VDOT2(KC(2),KC(1),DEF, P,MJ)
          CALL UWRITE(2)
          MX=MX+M
          PX=PX+P
    2   CONTINUE
      ELSEIF(M1.EQ.3) THEN
        DO 3 I=1,NJ
          CALL UREAD(1)
          CALL UPDATE(2)
          CALL VDOT(KC(2),KC(1),M,MJ)
          CALL UREAD(20)
          CALL VDOT3(KC(2),KC(1),KC(20), P,MJ)
          CALL UWRITE(2)
          MX=MX+M
          PX=PX+P
    3   CONTINUE
      ENDIF
      CALL UINIT
      END

      SUBROUTINE MEMV(RX)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*
* Purpose:
*             r := <26>.<26>
*                = gradient magnitude 
*                  for conjugate gradient "mask" evaluation
*
* Parameters:
*   ARGUMENT TYPE  I/O  DIMENSION   DESCRIPTION
*    RX      R*8     O    -         Magnitude squared
*
* Globals:
*   VARIABLE  COMMON BLOCK  I/O  DESCRIPTION
*    ...         MECOMP     I    Sizes and flags
*
* External calls:
*    UREAD       Read area
*    UINIT       Initialise area
*    VDOT        Vector scalar product
*
* History:
*   John Skilling    7 Dec 1988     First MEMSYS3 version
*
* Notes:
*-----------------------------------------------------------------------
      IMPLICIT CHARACTER*1 (A-Z)
      REAL*8 RX
      REAL R
      INTEGER I
      INCLUDE 'MECOMP.INC'
      IF(L0.GE.1) WRITE(IOUT,5)
    5 FORMAT('   MemV')
      RX=0.
      DO 1 I=1,NK
        CALL UREAD(26)
        CALL VDOT(KC(26),KC(26),R,MK)
        RX=RX+R
    1 CONTINUE
      CALL UINIT
      END

      SUBROUTINE MEMW(UX,VX, RX)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*
* Purpose:
*         Update gradient <26> for conjugate gradient "mask" evaluation
*
*            <26> := v * <26> - u * <25>
*             r   := <26>.<26>
*
* Parameters:
*   ARGUMENT TYPE  I/O  DIMENSION   DESCRIPTION
*    UX      R*8   I      -         Coefficient
*    VX      R*8   I      -         Coefficient
*    RX      R*8     O    -         Scalar product
*
* Globals:
*   VARIABLE  COMMON BLOCK  I/O  DESCRIPTION
*    ...         MECOMP     I    Sizes and flags
*
* External calls:
*    UREAD       Read area
*    UPDATE      Update area
*    UWRITE      Write area
*    UINIT       Initialise area
*    VDOT        Vector scalar product
*    VSMUL       Vector scalar multiply
*    VSMULA      Vector scalar multiply and add
*
* History:
*   John Skilling    7 Dec 1988     First MEMSYS3 version
*
* Notes:
*-----------------------------------------------------------------------
      IMPLICIT CHARACTER*1 (A-Z)
      REAL*8 UX,VX,RX
      REAL U,V,R
      INTEGER I
      INCLUDE 'MECOMP.INC'
      IF(L0.GE.1) WRITE(IOUT,5)
    5 FORMAT('   MemW')
      U=UX
      V=VX
      RX=0.
      DO 1 I=1,NK
        CALL UREAD(25)
        CALL UPDATE(26)
        CALL VSMUL(KC(26),V,KC(26),MK)
        CALL VSMULA(KC(25),-U,KC(26),KC(26),MK)
        CALL VDOT(KC(26),KC(26),R,MK)
        CALL UWRITE(26)
        RX=RX+R
    1 CONTINUE
      CALL UINIT
      END

      SUBROUTINE MEMX(ACCX)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*
* Purpose:
*    Mock data and their differential responsivity for nonlinear data
*    Set
*               <29> := Function'(<25> ) * acc = scaled differential
*               <25> := Function( <25> )       = Mock data
*
* Parameters:
*   ARGUMENT TYPE  I/O  DIMENSION   DESCRIPTION
*    ACCX    R*8   I      -         Accuracy = 1/sigma
*
* Globals:
*   VARIABLE  COMMON BLOCK  I/O  DESCRIPTION
*    ...         MECOMP     I    Sizes and flags
*    M2          MECOMP     I    Type of accuracy
*
* External calls:
*    UREAD       Read area
*    UPDATE      Update area
*    UWRITE      Write area
*    UINIT       Initialise area
*    VMEMX       Responsivity and its differential
*    VMUL        Vector multiply
*    VSMUL       Vector scalar multiply
*
* History:
*   John Skilling    7 Dec 1988     First MEMSYS3 version
*   JS              26 Jan 1989     Version 1.6
*   JS              23 Feb 1989     Version 1.7
*   JS               7 Sep 1989     Version 2.01 reconfigured loops
*
* Notes:
*-----------------------------------------------------------------------
      IMPLICIT CHARACTER*1 (A-Z)
      REAL*8 ACCX
      REAL ACC
      INTEGER I
      INCLUDE 'MECOMP.INC'
      INCLUDE 'MECOMC.INC'
      IF(L0.GE.1) WRITE(IOUT,5)
    5 FORMAT('   MemX')
      ACC=ACCX
      IF(M2.EQ.1) THEN
        DO 1 I=1,NK
          CALL UPDATE(25)
          CALL VMEMX(KC(25),KC(29),MK)
          CALL VSMUL(KC(29),ACC,KC(29),MK)
          CALL UWRITE(25)
          CALL UWRITE(29)
    1   CONTINUE
      ELSE
        DO 2 I=1,NK
          CALL UPDATE(25)
          CALL VMEMX(KC(25),KC(29),MK)
          CALL UREAD(22)
          CALL VMUL(KC(29),KC(22),KC(29),MK)
          CALL UWRITE(25)
          CALL UWRITE(29)
    2   CONTINUE
      ENDIF
      CALL UINIT
      END

      SUBROUTINE MEMMUL(K,L,ACCX)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*
* Purpose:
*           <L> := <K> * acc 
*
* Parameters:
*   ARGUMENT TYPE  I/O  DIMENSION   DESCRIPTION
*    K       I     I      -         Input area
*    L       I     I      -         Output area
*    ACCX    R*8   I      -         Accuracy = 1/sigma
*
* Globals:
*   VARIABLE  COMMON BLOCK  I/O  DESCRIPTION
*    ...         MECOMP     I    Sizes and flags
*    M2          MECOMP     I    Type of accuracy
*
* External calls:
*    UREAD       Read area
*    UPDATE      Update area
*    UWRITE      Write area
*    UINIT       Initialise area
*    VMUL        Vector multiply
*    VSMUL       Vector scalar multiply
*
* History:
*   John Skilling   26 Jan 1989     Version 1.6
*   David Robinson  14 Jun 1989     M2 options generalised
*   JS               7 Sep 1989     Version 2.01 reconfigured loops
*
* Notes:
*   (1)  "acc" is in <22> if M2=0or2 and M3=0
*                    ACCX if M2=1    and M3=0
*                    <29> if M3=1
*-----------------------------------------------------------------------
      IMPLICIT CHARACTER*1 (A-Z)
      INTEGER I,K,L
      REAL*8 ACCX
      REAL ACC
      INCLUDE 'MECOMP.INC'
      ACC=ACCX
      IF(L0.GE.1) WRITE(IOUT,5)
    5 FORMAT('   MemMUL')
      IF(M3.EQ.0 .AND. M2.EQ.1) THEN
        DO 1 I=1,NK
          IF(K.NE.L) THEN
            CALL UREAD(K)
          ELSE
            CALL UPDATE(K)
          ENDIF
          CALL VSMUL(KC(K),ACC,KC(L),MK)
          CALL UWRITE(L)
    1   CONTINUE
      ELSEIF(M3.EQ.0 .AND. M2.NE.1) THEN
        DO 2 I=1,NK
          IF(K.NE.L) THEN
            CALL UREAD(K)
          ELSE
            CALL UPDATE(K)
          ENDIF
          CALL UREAD(22)
          CALL VMUL(KC(22),KC(K),KC(L),MK)
          CALL UWRITE(L)
    2   CONTINUE
      ELSEIF(M3.EQ.1) THEN
        DO 3 I=1,NK
          IF(K.NE.L) THEN
            CALL UREAD(K)
          ELSE
            CALL UPDATE(K)
          ENDIF
          CALL UREAD(29)
          CALL VMUL(KC(K),KC(29),KC(L),MK)
          CALL UWRITE(L)
    3   CONTINUE
      ENDIF
      CALL UINIT
      END

      SUBROUTINE MEMDIV(K,L,ACCX)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*
* Purpose:
*           <L> := <K> / acc 
*
* Parameters:
*   ARGUMENT TYPE  I/O  DIMENSION   DESCRIPTION
*    K       I     I      -         Input area
*    L       I     I      -         Output area
*    ACCX    R*8   I      -         Accuracy = 1/sigma
*
* Globals:
*   VARIABLE  COMMON BLOCK  I/O  DESCRIPTION
*    ...         MECOMP     I    Sizes and flags
*    M2          MECOMP     I    Type of accuracy
*
* External calls:
*    UREAD       Read area
*    UPDATE      Update area
*    UWRITE      Write area
*    UINIT       Initialise area
*    VDIV        Vector divide
*    VSMUL       Vector scalar multiply
*
* History:
*   John Skilling   26 Jan 1989     Version 1.6
*   David Robinson  14 Jun 1989     M2 options generalised
*
* Notes:
*   (1) This should only be called with input area already incorporating
*       a factor of "acc".
*-----------------------------------------------------------------------
      IMPLICIT CHARACTER*1 (A-Z)
      INTEGER I,K,L
      REAL*8 ACCX
      REAL ACC
      INCLUDE 'MECOMP.INC'
      ACC=ACCX
      IF(L0.GE.1) WRITE(IOUT,5)
    5 FORMAT('   MemDIV')
      IF(M3.EQ.0 .AND. M2.EQ.1) THEN
        DO 1 I=1,NK
          IF(K.NE.L) THEN
            CALL UREAD(K)
          ELSE
            CALL UPDATE(K)
          ENDIF
          CALL VSMUL(KC(K),1./ACC,KC(L),MK)
          CALL UWRITE(L)
    1   CONTINUE
      ELSEIF(M3.EQ.0 .AND. M2.NE.1) THEN
        DO 2 I=1,NK
          IF(K.NE.L) THEN
            CALL UREAD(K)
          ELSE
            CALL UPDATE(K)
          ENDIF
          CALL UREAD(22)
          CALL VDIV(KC(K),KC(22),KC(L),MK)
          CALL UWRITE(L)
    2   CONTINUE
      ELSEIF(M3.EQ.1) THEN
        DO 3 I=1,NK
          IF(K.NE.L) THEN
            CALL UREAD(K)
          ELSE
            CALL UPDATE(K)
          ENDIF
          CALL UREAD(29)
          CALL VDIV(KC(K),KC(29),KC(L),MK)
          CALL UWRITE(L)
    3   CONTINUE
      ENDIF
      CALL UINIT
      END

      SUBROUTINE MEMOP(J,K)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*
* Purpose:
*            Differential response
*
*            <K> := Opus(<J>)
*
* Parameters:
*   ARGUMENT TYPE  I/O  DIMENSION   DESCRIPTION
*    J       I     I      -         Input map area
*    K       I     I      -         Output data area
*
* Globals:
*   VARIABLE  COMMON BLOCK  I/O  DESCRIPTION
*    ...         MECOMP     I    Sizes and flags
*    NTRANS      MECOMP     I O  Transform counter
*    PR          MECOMC       O  Flowchart flags
*
* External calls:
*    OPUS        Map-to-Data differential transform
*    UINIT       Initialise area
*
* History:
*   John Skilling    7 Dec 1988     First MEMSYS3 version
*   JS              26 Jan 1989     Version 1.6
*
* Notes:
*   (1)  Opus must NOT change frequently between iterates
*-----------------------------------------------------------------------
      IMPLICIT CHARACTER*1 (A-Z)
      INTEGER J,K
      INCLUDE 'MECOMP.INC'
      INCLUDE 'MECOMC.INC'
      IF(L0.GE.1) WRITE(IOUT,5)
    5 FORMAT('   Opus')
      CALL OPUS(J,K)
      NTRANS=NTRANS+1
      PR(J)='R'
      PR(K)='W'
      CALL UINIT
      END

      SUBROUTINE MEMTR(K,J)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*
* Purpose:
*            Transpose differential response
*
*            <J> := Tropus(<K>)
*
* Parameters:
*   ARGUMENT TYPE  I/O  DIMENSION   DESCRIPTION
*    K       I     I      -         Input data area
*    J       I     I      -         Output map area
*
* Globals:
*   VARIABLE  COMMON BLOCK  I/O  DESCRIPTION
*    ...         MECOMP     I    Sizes and flags
*    NTRANS      MECOMP     I O  Transform counter
*    PR          MECOMC       O  Flowchart flags
*
* External calls:
*    TROPUS      Data-to-Map differential transform
*    UINIT       Initialise area
*
* History:
*   John Skilling    7 Dec 1988     First MEMSYS3 version
*   JS              26 Jan 1989     Version 1.6
*
* Notes:
*   (1)  Tropus must NOT change frequently between iterates (see Opus)
*-----------------------------------------------------------------------
      IMPLICIT CHARACTER*1 (A-Z)
      INTEGER J,K
      INCLUDE 'MECOMP.INC'
      INCLUDE 'MECOMC.INC'
      IF(L0.GE.1) WRITE(IOUT,5)
    5 FORMAT('   Tropus')
      CALL TROPUS(K,J)
      NTRANS=NTRANS+1
      PR(K)='R'
      PR(J)='W'
      CALL UINIT
      END

      SUBROUTINE MEMTRQ(ISEED, ERR)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*
* Purpose:
*         Check for   OPUS / TROPUS   inconsistencies.
*
*         Dimensionless return parameter ERR should be O(rounding error)
*         if OPUS and TROPUS are each other's transposes.
*
* Method:
*         IF( ISEED > 0 ) THEN
*           <1> := random   ,   <26> := random
*         ELSE
*           use <1> and <26> as set by user.
*
*         ERR := ABS(  <1> . Tropus<26>  -  <26> . Opus<1>  )
*                                       in dimensionless form
*                               
*
* Parameters:
*   ARGUMENT TYPE  I/O  DIMENSION   DESCRIPTION
*     ISEED   I    I      -         Seed for random vector (or 0)
*     ERR     R      O    -         Fractional error
*
* Globals:
*   VARIABLE  COMMON BLOCK  I/O  DESCRIPTION
*    ...         MECOMP     I    Sizes and flags
*
* External calls:
*    UREAD       Read  area
*    UWRITE      Write area
*    UINIT       Initialise area
*    VRAND       Random vector generator
*    VMOV        Vector move
*    VDOT        Vector dot product
*    OPUS        Map-to-Data differential transform
*    TROPUS      Data-to-Map differential transform
*
* History:
*   John Skilling   23 May 1989     Second MEMSYS3 version
*
* Notes:
*-----------------------------------------------------------------------
      IMPLICIT CHARACTER*1 (A-Z)
      INTEGER I,ISEED
      REAL U,V,ERR
      REAL*8 UU,VV,UTV,VRU,URRU,VTTV
      INCLUDE 'MECOMP.INC'
* Entry protection
      L0=0
      L1=0
      CALL UINIT
      IF(ISEED.GT.0) THEN
*   Optional setup source areas <1> and <26>
        CALL VRAND(ISEED,0)
        DO 1 I=1,NJ
          CALL VRAND(KC(1),MJ)
          CALL UWRITE(1)
    1   CONTINUE
        DO 2 I=1,NK
          CALL VRAND(KC(26),MK)
          CALL UWRITE(26)
    2   CONTINUE
        CALL UINIT
      ENDIF
* <2> := Tropus <26>
*  VV := <26>.<26>
      VV=0.
      DO 3 I=1,NK
        CALL UREAD(26)
        CALL VDOT(KC(26),KC(26),V,MK)
        CALL VMOV(KC(26),KC(25),MK)
        CALL UWRITE(25)
        VV=VV+V
    3 CONTINUE
      CALL UINIT
      CALL TROPUS(25,2)
      CALL UINIT
* UTV  := <1>.<2>
* VTTV := <2>.<2>
      UTV=0.
      VTTV=0.
      DO 4 I=1,NJ
        CALL UREAD(1)
        CALL UREAD(2)
        CALL VDOT(KC(1),KC(2),U,MJ)
        CALL VDOT(KC(2),KC(2),V,MJ)
        UTV=UTV+U
        VTTV=VTTV+V
    4 CONTINUE
      CALL UINIT
* <25> := Opus <1>
*   UU := <1>.<1>
      UU=0.
      DO 5 I=1,NJ
        CALL UREAD(1)
        CALL VDOT(KC(1),KC(1),U,MJ)
        CALL VMOV(KC(1),KC(2),MJ)
        CALL UWRITE(2)
        UU=UU+U
    5 CONTINUE
      CALL UINIT
      CALL OPUS(2,25)
      CALL UINIT
* VRU  := <26>.<25>
* URRU := <25>.<25>
      VRU=0.
      URRU=0.
      DO 6 I=1,NK
        CALL UREAD(25)
        CALL UREAD(26)
        CALL VDOT(KC(25),KC(26),U,MK)
        CALL VDOT(KC(25),KC(25),V,MK)
        VRU=VRU+U
        URRU=URRU+V
    6 CONTINUE
      CALL UINIT
* Fractional error
      U=SQRT(UU)*SQRT(VV)
      V=SQRT(URRU)*SQRT(VTTV)
      ERR=ABS(UTV-VRU)/SQRT(U*V)
      END
