**<uinit>**********************************************
 
      SUBROUTINE UINIT
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*
* Purpose:
*   Re-initialise all pointers, with optional diagnostics
*
* Parameters:
*   ARGUMENT TYPE  I/O  DIMENSION   DESCRIPTION
*     -
*
* Globals:
*   VARIABLE  COMMON BLOCK  I/O  DESCRIPTION
*    IOUT,L0     MECOMP     I    Diagnostic stream and flag
*    PR          MECOMC     I    Read/Write flags
*
* External calls:
*    URESET      Reset pointers
*    UDIAG       User's diagnostic routine
*
* History:
*   John Skilling    8 Nov 1985     Initial release
*                    7 Dec 1988     MEMSYS3 version
*
* Notes:
*-----------------------------------------------------------------------
      INCLUDE 'MECOMP.INC'
      INCLUDE 'MECOMC.INC'
      IF(L0.GE.2) WRITE(IOUT,4) PR
      DO 1 I=1,40
        PR(I)='.'
        CALL URESET(I)
    1 CONTINUE
      IF(L0.GE.3) CALL UDIAG(' ')
    4 FORMAT(11X,20A1,2X,20A1)
      END
 
**<ureset>*********************************************
 
      SUBROUTINE URESET(I)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*
* Purpose:
*   Reset pointers for given area
*
* Parameters:
*   ARGUMENT TYPE  I/O  DIMENSION   DESCRIPTION
*    I       I     I      -         Area number
*
* Globals:
*   VARIABLE  COMMON BLOCK  I/O  DESCRIPTION
*    KA          MECOMP     I    Allocation pointers
*    KB          MECOMP     I    Base addresses
*    KC          MECOMP       O  Core addresses
*    KD          MECOMP       O  External addresses
*
* External calls:
*
* History:
*   John Skilling    8 Nov 1985     Initial release
*                    7 Dec 1988     MEMSYS3 version
*                   22 Feb 1989     KD redefined
*
* Notes:
*-----------------------------------------------------------------------
      INCLUDE 'MECOMP.INC'
      KC(I)=KB(I)
      IF(KA(I).GT.0) THEN
* External       
        KD(I)=KA(I)
      ELSE
* Core
        KD(I)=KB(I)
      ENDIF
      END
 
**<uread>**********************************************
 
      SUBROUTINE UREAD(I)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*
* Purpose:
*   Read a buffer from given area and increment pointers appropriately
*
* Parameters:
*   ARGUMENT TYPE  I/O  DIMENSION   DESCRIPTION
*    I       I     I      -         Input area number
*
* Globals:
*   VARIABLE  COMMON BLOCK  I/O  DESCRIPTION
*    MJ,MK       MECOMP     I    Buffer sizes (map and data space)
*    KA          MECOMP     I    Allocation pointers
*    KB          MECOMP      -   Base addresses
*    KC          MECOMP     I O  Core addresses
*    KD          MECOMP     I O  External addresses       
*    PR          MECOMC       O  Read/Write flag
*
* External calls:
*    UGET        Get buffer from "disc"       
*
* History:
*   John Skilling    8 Nov 1985     Initial release
*                    7 Dec 1988     MEMSYS3 version
*                   22 Feb 1989     KD redefined
*
* Notes:
*    (1) Use UPDATE instead of UREAD if area to be modified in-place
*-----------------------------------------------------------------------
      INCLUDE 'MECOMP.INC'
      INCLUDE 'MECOMC.INC'
      PR(I)='R'
      IF(I.LE.20) THEN
        M=MJ
      ELSE
        M=MK
      ENDIF
      IF(KA(I).GT.0) THEN
* External       
        CALL UGET(KD(I),KC(I),M)
        KD(I)=KD(I)+M
      ELSE
* Core
        KC(I)=KD(I)
        KD(I)=KD(I)+M
      ENDIF
      END
 
**<update>*********************************************
 
      SUBROUTINE UPDATE(I)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*
* Purpose:
*   Read a buffer from given area, ready for in-place writing
*
* Parameters:
*   ARGUMENT TYPE  I/O  DIMENSION   DESCRIPTION
*    I       I     I      -         Input area number
*
* Globals:
*   VARIABLE  COMMON BLOCK  I/O  DESCRIPTION
*    MJ,MK       MECOMP     I    Buffer sizes (map and data space)
*    KA          MECOMP     I    Allocation pointers
*    KB          MECOMP      -   Base addresses
*    KC          MECOMP     I    Core addresses
*    KD          MECOMP      -   External addresses       
*    PR          MECOMC       O  Read/Write flag
*
* External calls:
*    UGET        Get buffer from "disc"       
*
* History:
*   John Skilling    8 Nov 1985     Initial release
*                    7 Dec 1988     MEMSYS3 version
*                   22 Feb 1989     KD redefined
*
* Notes:
*    (1) Use UREAD instead of UPDATE if area is read-only
*    (2) This routine implies withdrawal of support for
*        sequential access disc files.
*-----------------------------------------------------------------------
      INCLUDE 'MECOMP.INC'
      INCLUDE 'MECOMC.INC'
      PR(I)='U'
      IF(I.LE.20) THEN
        M=MJ
      ELSE
        M=MK
      ENDIF
* External       
      IF(KA(I).GT.0) CALL UGET(KD(I),KC(I),M)
* Core
      END

**<uwrite>*********************************************
 
      SUBROUTINE UWRITE(I)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*
* Purpose:
*   Write a buffer to given area and increment pointers appropriately
*
* Parameters:
*   ARGUMENT TYPE  I/O  DIMENSION   DESCRIPTION
*    I       I     I      -         Output area number
*
* Globals:
*   VARIABLE  COMMON BLOCK  I/O  DESCRIPTION
*    MJ,MK       MECOMP     I    Buffer sizes (map and data space)
*    KA          MECOMP     I    Allocation pointers
*    KB          MECOMP      -   Base addresses
*    KC          MECOMP     I O  Core addresses
*    KD          MECOMP     I O  External addresses       
*    PR          MECOMC       O  Read/Write flag
*
* External calls:
*    UPUT        Put buffer to "disc"       
*
* History:
*   John Skilling    8 Nov 1985     Initial release
*                    7 Dec 1988     MEMSYS3 version
*                   22 Feb 1989     KD redefined
*
* Notes:
*-----------------------------------------------------------------------
      INCLUDE 'MECOMP.INC'
      INCLUDE 'MECOMC.INC'
      IF( PR(I).NE.'U' )  PR(I)='W'
      IF(I.LE.20) THEN
        M=MJ
      ELSE
        M=MK
      ENDIF
      IF(KA(I).GT.0) THEN
* External       
        CALL UPUT(KD(I),KC(I),M)
        KD(I)=KD(I)+M
      ELSE
* Core
        KD(I)=KD(I)+M
        KC(I)=KD(I)
      ENDIF
      END
 
**<udiag>**********************************************
 
      SUBROUTINE UDIAG(STRING)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*
* Purpose:
*   User's diagnostic routine, to inspect any part of any area in core.
*
* Parameters:
*   ARGUMENT TYPE  I/O  DIMENSION   DESCRIPTION
*    STRING  C *   I      -         Message to show calling location
*
* Globals:
*   VARIABLE  COMMON BLOCK  I/O  DESCRIPTION
*    ST          MECOMS     I    Storage vector
*    NJ,MJ       MECOMP     I    Sizes  (map space)
*    NK,MK       MECOMP     I    Sizes (data space)
*    KA          MECOMP     I    Initialised allocation pointers
*    KB          MECOMP     I    Initialised base addresses
*
* External calls:
*     -
*
* History:
*   John Skilling    8 Nov 1985     Initial release
*                    7 Dec 1988     MEMSYS3 version
*                   22 Feb 1989     External handling redefined
*
* Notes:
*    (1)  This implementation is numerical, and interrogates the user
*         via the default Fortran stream (*).
*-----------------------------------------------------------------------
      CHARACTER*(*) STRING
      INCLUDE 'MECOMP.INC'
      INCLUDE 'MECOMS.INC'
      WRITE(*,*) ' UDIAG:- ',STRING
    1 CONTINUE
        WRITE(*,*) ' Which area would you like?  (0 exits)  '
        READ(*,*,END=1,ERR=1) N
        IF(N.LT.0 .OR. N.GT.40) GOTO 1
        IF(N.EQ.0) RETURN
        IF(N.LE.20) THEN
          M=MJ
          IF(KA(N).EQ.0) M=M*NJ
        ELSE
          M=MK
          IF(KA(N).EQ.0) M=M*NK
        ENDIF
        IF(KA(N).EQ.0) THEN
          WRITE(*,5) KA(N),KB(N)
        ELSE
          WRITE(*,6) KA(N),KB(N)
        ENDIF
        WRITE(*,7) KB(N)+M-1
        WRITE(*,*) ' First point ?  '
        READ(*,*) J
        J=KB(N)+J-1
        WRITE(*,*) '  Last point ?  '
        READ(*,*) K
        K=KB(N)+K-1
        WRITE(*,8) (ST(I),I=J,K)
      GOTO 1
    5 FORMAT(' KA =',I3,'  (core area)              ',
     *       ' KB =',I7,' = base address')
    6 FORMAT(' KA =',I8,'  (external area)     ',
     *       ' KB =',I7,' = base address')
    7 FORMAT(    40X,I7,' = last address')
    8 FORMAT(1PE14.4,3E14.4)
      END

      SUBROUTINE UEXIT(STRING)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*
* Purpose:
*   Error exit
*
* Parameters:
*   ARGUMENT TYPE  I/O  DIMENSION   DESCRIPTION
*    STRING  C *   I      -         Error message
*
* Globals:
*   VARIABLE  COMMON BLOCK  I/O  DESCRIPTION
*    IOUT       MECOMP      I    Output stream for messages
*
* External calls:
*    UDIAG         Diagnostic routine
*
* History:
*   John Skilling   18 Nov 1988     Initial release
*
* Notes:
*-----------------------------------------------------------------------
      CHARACTER*(*) STRING
      INCLUDE 'MECOMP.INC'
      WRITE(IOUT,*) ' Error exit from MEMSYS:'
      CALL UDIAG(STRING)
      STOP
      END
