*
*             Vector Library
* Contains: 
*    VFILL (U,A,N)        U[] = A
*    VMOV  (U,V,N)        V[] = U[]
*    VADD  (U,V,W,N)      W[] = U[] + V[]
*    VSUB  (U,V,W,N)      W[] = U[] - V[]
*    VMUL  (U,V,W,N)      W[] = U[] * V[]
*    VDIV  (U,V,W,N)      W[] = U[] / V[]
*    VSUM  (U,A,N)        A = Sum(U[])
*    VDOT  (U,V,A,N)      A = U[].V[]
*    VSADD (U,A,V,N)      V[] = U[] + A
*    VSMUL (U,A,V,N)      V[] = U[] * A
*    VSMULA(U,A,V,W,N)    W[] = U[] * A + V[]
*    VSQRT (U,V,N)        V[] = SQRT U[]
*    VLOG  (U,V,N)        V[] = LOG U[]
*    VSUML (U,A,B,N)      A = Number U[], B = Sum(LOG(U[])) for U[] != 0
*    VDOT1 (U,V,A,N)      A = U[].metric.U[] ,  U[] = metric*U[]
*                         metric = V[]
*    VDOT2 (U,V,D,A,N)    A = U[].metric.U[] ,  U[] = metric*U[]
*                         metric = SQRT( V[]**2 + 4*D**2 )
*    VDOT3 (U,V,D,A,N)    A = U[].metric.U[] ,  U[] = metric*U[]
*                         metric = SQRT( V[]**2 + 4*D[]**2 )
*    VENT1 (U,V,N)        U[] = EXP V[]
*                         V[] = U[] - 1 - V[]*U[]
*    VENT2 (U,V,N)        U[] = EXP V[]
*                         V[] = 2*(COSH V[] - 1 - V[] SINH V[])
*    VMEMX (F,D)          D[] = Phi'(F[]) ,  F[] = Phi(F[])
*
* Note: Exception handling can affect operation of MEMSYS3

      SUBROUTINE VFILL(U,A,N)
      INTEGER U
      REAL A
      INCLUDE 'MECOMS.INC'
      DO 1 I=0,N-1
        ST(U+I)=A
    1 CONTINUE
      END

      SUBROUTINE VMOV(U,V,N)
      INTEGER U,V
      INCLUDE 'MECOMS.INC'
      DO 1 I=0,N-1
        ST(V+I)=ST(U+I)
    1 CONTINUE
      END

      SUBROUTINE VADD(U,V,W,N)
      INTEGER U,V,W
      INCLUDE 'MECOMS.INC'
      DO 1 I=0,N-1
        ST(W+I)=ST(U+I)+ST(V+I)
    1 CONTINUE
      END

      SUBROUTINE VSUB(U,V,W,N)
      INTEGER U,V,W
      INCLUDE 'MECOMS.INC'
      DO 1 I=0,N-1
        ST(W+I)=ST(U+I)-ST(V+I)
    1 CONTINUE
      END

      SUBROUTINE VMUL(U,V,W,N)
      INTEGER U,V,W
      INCLUDE 'MECOMS.INC'
      DO 1 I=0,N-1
        ST(W+I)=ST(U+I)*ST(V+I)
    1 CONTINUE
      END

      SUBROUTINE VDIV(U,V,W,N)
      INTEGER U,V,W
      REAL EPS
      INCLUDE 'MECOMS.INC'
      PARAMETER (EPS=1.D-25)
      DO 1 I=0,N-1
        IF(ABS(ST(V+I)).GT.EPS) THEN
          ST(W+I)=ST(U+I)/ST(V+I)
        ELSE
          ST(W+I)=0.
        ENDIF
    1 CONTINUE
      END

      SUBROUTINE VSUM(U,A,N)
      INTEGER U
      REAL A
      INCLUDE 'MECOMS.INC'
      A=0.
      DO 1 I=0,N-1
        A=A+ST(U+I)
    1 CONTINUE
      END

      SUBROUTINE VDOT(U,V,A,N)
      INTEGER U,V
      REAL A
      INCLUDE 'MECOMS.INC'
      A=0.
      DO 1 I=0,N-1
        A=A+ST(U+I)*ST(V+I)
    1 CONTINUE
      END

      SUBROUTINE VSADD(U,A,V,N)
      INTEGER U,V
      REAL A
      INCLUDE 'MECOMS.INC'
      DO 1 I=0,N-1
        ST(V+I)=A+ST(U+I)
    1 CONTINUE
      END

      SUBROUTINE VSMUL(U,A,V,N)
      INTEGER U,V
      REAL A
      INCLUDE 'MECOMS.INC'
      DO 1 I=0,N-1
        ST(V+I)=A*ST(U+I)
    1 CONTINUE
      END

      SUBROUTINE VSMULA(U,A,V,W,N)
      INTEGER U,V,W
      REAL A
      INCLUDE 'MECOMS.INC'
      DO 1 I=0,N-1
        ST(W+I)=A*ST(U+I)+ST(V+I)
    1 CONTINUE
      END

      SUBROUTINE VSQRT(U,V,N)
      INTEGER U,V
      INCLUDE 'MECOMS.INC'
      DO 1 I=0,N-1
        IF( ST(U+I).GE.0. ) THEN
          ST(V+I)=SQRT(ST(U+I))
        ELSE
          ST(V+I)=0.
        ENDIF
    1 CONTINUE
      END

      SUBROUTINE VLOG(U,V,N)
      INTEGER U,V
      INCLUDE 'MECOMS.INC'
      DO 1 I=0,N-1
        IF(ST(U+I).GT.0.) THEN
          ST(V+I)=LOG(ST(U+I))
        ELSE
          ST(V+I)=0.
        ENDIF
    1 CONTINUE
      END

      SUBROUTINE VSUML(U,A,B,N)
      INTEGER U
      REAL A,B
      INCLUDE 'MECOMS.INC'
      A=0.
      B=0.
      DO 1 I=0,N-1
        IF(ST(U+I).GT.0.) THEN
          A=A+1.
          B=B+LOG(ST(U+I))
        ENDIF
    1 CONTINUE
      END

      SUBROUTINE VDOT1(U,V,A,N)
      INTEGER U,V
      REAL A,B
      INCLUDE 'MECOMS.INC'
      A=0.
      DO 1 I=0,N-1
        B=ST(V+I)*ST(U+I)
        A=A+ST(U+I)*B
        ST(U+I)=B
    1 CONTINUE
      END

      SUBROUTINE VDOT2(U,V,D,A,N)
      INTEGER U,V
      REAL D,A,B,C
      INCLUDE 'MECOMS.INC'
      C=4.*D*D
      A=0.
      DO 1 I=0,N-1
        B=ST(U+I)*SQRT(ST(V+I)**2+C)
        A=A+ST(U+I)*B
        ST(U+I)=B
    1 CONTINUE
      END

      SUBROUTINE VDOT3(U,V,D,A,N)
      INTEGER U,V,D
      REAL A,B
      INCLUDE 'MECOMS.INC'
      A=0.
      DO 1 I=0,N-1
        B=ST(U+I)*SQRT(ST(V+I)**2+4.*ST(D+I)**2)
        A=A+ST(U+I)*B
        ST(U+I)=B
    1 CONTINUE
      END

      SUBROUTINE VENT1(U,V, N)
      INTEGER U,V
      REAL ZERO
      INCLUDE 'MECOMS.INC'
      PARAMETER (ZERO=0.)
      DO 1 I=0,N-1
        IF( ST(V+I).GT.-40. ) THEN
          V2=ST(V+I)/2.
          E=EXP(V2)
          ST(V+I)=MIN(2.*(SINH(V2)-V2*E)*E,ZERO)
          ST(U+I)=E*E
        ELSE
          ST(V+I)=-1.
          ST(U+I)=0.
        ENDIF
    1 CONTINUE
      END

      SUBROUTINE VENT2(U,V, N)
      INTEGER U,V
      INCLUDE 'MECOMS.INC'
      DO 1 I=0,N-1
        V2=ST(V+I)/2.
        S=SINH(V2)
        C=COSH(V2)
        ST(V+I)=4.*S*(S-ST(V+I)*C)
        ST(U+I)=(S+C)**2
    1 CONTINUE
      END

      SUBROUTINE VMEMX(F,D,N)
      INTEGER F,D
      INCLUDE 'MECOMS.INC'
      DO 1 I=0,N-1
        X=ST(F+I)
        CALL MEMEX(X,PHI,DPHI)
        ST(F+I)=PHI
        ST(D+I)=DPHI
    1 CONTINUE
      END
