C*MEMMIN -- MEM Merge input.
C+
      SUBROUTINE MEMMIN(INDAT,NP)
      INTEGER INDAT, NP
C
C MEM Merge INput. Read a Caltech-Merge format file and pass the data
C to MEMSYS. The complex visibilities are stored in MEMSYS area <21>,
C the accuracies (inverse standard error) in area <22>, and the (u,v)
C coordinates (in wavelengths) in area <40>.
C
C Arguments:
C  INDAT (input)   : Fortran unit on which the Merge file is open
C                    (use routine VLBOPE to open the file).
C  NP (output)     : number of complex visibilities.
C
C Subroutines required:
C  ERROR   (CITVLB)
C  HEADX20 (CITVLB)
C  LEN1    (CITVLB)
C  STNABR  (CITVLB)
C  TDATE   (CITVLB)
C  UWRITE  (MEMSYS)
C  UINIT   (MEMSYS)
C
C History:
C  1990 Jan  4 - TJP
C-----------------------------------------------------------------------
      INTEGER   OUTC
      PARAMETER (OUTC=6)
      INTEGER   MAXS,MAXB,MAXT,MAXWPB
      PARAMETER (MAXS=20,MAXB=(MAXS-1)*MAXS/2)
      PARAMETER (MAXT=MAXB*(MAXS-2)/3)
      PARAMETER (MAXWPB=7)
      DOUBLE PRECISION PI,SIDR,VELC,RPDEG
      PARAMETER (PI = 3.141592653589793 D0)
      PARAMETER (SIDR = (PI/2592000.0D0)*1.0027379093 D0)
C             (radians of ST per sec/60 of UT)
      PARAMETER (VELC = 2.997925 D8)
      PARAMETER (RPDEG = PI/180D0)
C
      CHARACTER*128 FILE
      CHARACTER*16 SNAME
      CHARACTER*32 TELESC
      CHARACTER*8  DATOBS,STNAME
      CHARACTER*1  STNABR
      LOGICAL ORBITR
      INTEGER STATYP, LEN1, IZ, ID, IY, IMON, JD, MONTH, LENG, I
      INTEGER IS, IB, NS, NB, NT, IFEOF, NC1, NC2, NC3, IS1, IS2, NVER
      INTEGER IP, KFLAG, NDWPB, J, L, NB1, NB2, IUT, INBUF
      REAL    EPOCH, DUM1, DUM3, AMP, AMPMAX, PHASE, UP, VP, UVP, UVMAX
      REAL    A
      REAL    REC(MAXWPB*MAXB)
      DOUBLE PRECISION  STX,STY,STZ,RA,DEC,GASTM,FREQ,FLUX,
     1        WAVE,BX,BY,BZ(MAXB),BOFF(MAXB),BXY(MAXB),TIHA,
     2        COSDEC,SINDEC,TINTG,AVG,BW,AEIOWM, RA0, DEC0
      COMMON /PARMS/ STNAME(MAXS),STX(MAXS),STY(MAXS),STZ(MAXS),
     1        SNAME,RA,DEC,GASTM,FREQ,FLUX,TINTG,AVG,BW,
     2        AEIOWM(6,MAXS),STATYP(MAXS),ORBITR
      COMMON /ORDER/ NS,NB,NT, NB1(MAXB),NB2(MAXB),NC1(MAXT),
     1        NC2(MAXT),NC3(MAXT),NVER,NDWPB
C------------------------------------------------------------------------ 
C MEMSYS common blocks. 
      INCLUDE 'MECOMS.INC'
      INCLUDE 'MECOMP.INC'
C FITS header common blocks.
      INCLUDE 'MEFITS.INC'
C-----------------------------------------------------------------------
  600 FORMAT(/' (MEMMIN) Input file: ',A)
  610 FORMAT( '   Number of u,v points accepted: ',I6
     1       /'   UVmax = ',F16.2,' wavelengths'
     2       /'   Maximum correlated flux density = ',F9.3,' Jy')
  630 FORMAT( '   Source: ',A
     1       /'   Frequency: ',F10.3,' GHz'
     2       /'   Array: ',A
     3       /'   Observation date: ',A,F10.2
     4       /'   Stations: ',7A9/(13X,7A9))
  640 FORMAT(I2.2,'/',I2.2,'/',I2.2)
C-----------------------------------------------------------------------
C 
      INQUIRE (UNIT=INDAT, NAME=FILE)
C 
      WRITE (OUTC,600) FILE(1:LEN1(FILE))
      CALL HEADX20(.FALSE.,INDAT,IZ,ID,IY,DUM1,0,0,KFLAG,DUM3,IFEOF,0)
      IF (NS.LT.2 .OR. NB.LT.1) CALL ERROR('Not enough stations.')
      IF (NS.GT.MAXS) CALL ERROR('Too many stations.')
      LENG = NDWPB*NB
      RA0 = RA
      DEC0 = DEC
      L = 0
      TELESC = ' '
      DO 100 IS=1,NS
          TELESC(L+IS:L+IS) = STNABR(STNAME(IS))
  100 CONTINUE
      IMON = 1
      CALL TDATE(IY,IMON,ID,JD,MONTH,DUM1)
      WRITE (DATOBS,640) ID,IMON,MOD(IY,100)
      EPOCH = 2000.0 + ((JD-2451545)-0.5D0)/365.25D0
      WRITE (OUTC,630) SNAME, FREQ*1E-9, TELESC(1:NS), DATOBS, EPOCH,
     1                 (STNAME(J),J=1,NS)
C 
C             Baseline parameters
C 
      WAVE = VELC/FREQ
      COSDEC = COS(DEC)
      SINDEC = SIN(DEC)
      DO 210 IB=1,NB
      IF (NVER.EQ.1) THEN
            IS1 = NB1(IB)
            IS2 = NB2(IB)
            BX = (STX(IS1)-STX(IS2))/WAVE
            BY = (STY(IS1)-STY(IS2))/WAVE
            BZ(IB) = (STZ(IS1)-STZ(IS2))/WAVE
            BXY(IB) = SQRT(BX*BX+BY*BY)
            IF (BXY(IB).EQ.0D0)
     1              CALL ERROR('Station coordinates missing.')
            BOFF(IB) = GASTM-RA-PI/2D0-ATAN2(BY,BX)
      ELSE
            BZ(IB)   = 0.D0
            BXY(IB)  = 0.D0
            BOFF(IB) = 0.D0
      END IF
  210 CONTINUE
C 
C Determine all (u,v) values and complex visibilities.
C 
      AMPMAX = 0.0
      IP = 0
      INBUF = 0
C 
C Loop through data records.
C
  300 READ (INDAT,END=400) IUT,(REC(J),J=1,LENG)
      IF (IUT.LT.0) GOTO 400
C         -- Loop through baselines
          DO 320 IB=1,NB
C             -- Skip deleted data
              IF (REC(NDWPB*(IB-1)+2).LE.0.0 .OR.
     1            REC(NDWPB*(IB-1)+4).LE.0.0) GOTO 320
C             -- Count the data point
              IP = IP+1
C             -- Compute (u,v) if necessary
              IF (NVER.EQ.1) THEN
                  TIHA = (IUT-IZ)*SIDR + BOFF(IB)
                  UP = REAL(BXY(IB)*COS(TIHA))
                  VP = REAL(BZ(IB)*COSDEC + BXY(IB)*SINDEC*SIN(TIHA))
              ELSE IF (NVER.EQ.2) THEN
                  UP = REC(NDWPB*(IB-1)+5)
                  VP = REC(NDWPB*(IB-1)+6)
              END IF
              UVP = UP*UP + VP*VP
              UVMAX = MAX(UVMAX,UVP)
C             -- Compute complex visibility and error
              AMP =   REC(NDWPB*(IB-1)+1)
              PHASE = REC(NDWPB*(IB-1)+3)*RPDEG
              AMPMAX = MAX(AMPMAX,AMP)
C             -- Put U and V in area <40>
              ST(KC(40)+INBUF)   = UP
              ST(KC(40)+INBUF+1) = VP
C             -- Put real and imaginary parts of visibility in area <21>
              ST(KC(21)+INBUF)   = REC(NDWPB*(IB-1)+1)*COS(PHASE)
              ST(KC(21)+INBUF+1) = REC(NDWPB*(IB-1)+1)*SIN(PHASE)
C             -- Put accuracy (1/sigma) in area <22>
              A = 1.0/REC(NDWPB*(IB-1)+2)
              ST(KC(22)+INBUF)   = A
              ST(KC(22)+INBUF+1) = A
C             -- If the buffer is full, send it. Otherwise, increment
C                pointer.
              IF (INBUF+2 .GT. MK) THEN
                  CALL UWRITE(21)
                  CALL UWRITE(22)
                  CALL UWRITE(40)
                  INBUF = 0
              ELSE
                  INBUF = INBUF+2
              END IF
  320     CONTINUE
      GOTO 300
C
C (end loop)
C 
  400 NP = IP
      WRITE (OUTC,610) NP,SQRT(UVMAX),AMPMAX
C
C Send final buffer to MEMSYS if required. First fill it with dummy
C data of zero weight.
C
      IF (INBUF .GT. 1) THEN
          DO 410 I=INBUF+1,MK
              ST(KC(21)+I) = 0.0
              ST(KC(22)+I) = 0.0
              ST(KC(40)+I) = 0.0
  410     CONTINUE
          CALL UWRITE(21)
          CALL UWRITE(22)
          CALL UWRITE(40)
      END IF
      CALL UINIT
C
C Save parameters for FITS header.
C
      CALL USERNM(HOBSER)
      HTELES = TELESC
      HSNAME = SNAME
      HDATOB = DATOBS
      HEPOCH = EPOCH
      HFREQ  = FREQ
      HRA0   = RA0
      HDEC0  = DEC0
C
      END
