*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*
*  This file [VLBGRD.FOR] contains VLB Gridding and degridding               *
*  routines for full, transposed  complex plane                              *
*  (Modified from [USER.DEVINDER.VLBI]VLBGRD.FOR -- 10th Oct., 1986.)        *
*  (Modified from NMR complex version -- 4 Dec.,1989 - SF Gull CIT           *
*  (Modified from conjugate symmetric version for IKAROS, using              *
*  extensive tabling for multiple gridding/degridding 9 Dec 1989             *
*                                                                            *
* The gridder uses a small convolution in the u-v plane NCELS eavh side      *
* of the given u-v point. The gridding function is tabled in PSF             *
* NPCEL sub-divisions to a grid point of u-v. The grid correction function(s)*
* are tabled in HOOK(KBCORX) and HOOK(KBCORY)                                *
* Extensixe tabling of the interior loops of the gridder is then done on the *
* first pass through GRDSET. The tables LOOK, LOOKPU, LOOKPV provide the     *
* necessary information to GRID and DEGRID                                   *
* I do not advise changing this code without discussing it with me first     *
* S.F. Gull 13 December 1989, CIT. This is not solely because:               *
*                                                                            *
*  These routines are the Copyright (c) property of:                         *
*        Maximum Entropy Data Consultants Ltd.                               *
*        33 North End, Meldreth, Royston SG8 6NR, U.K.                       *
*----------------------------------------------------------------------------*

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*
* Externally callable routines                                            *
*-------------------------------------------------------------------------*
      SUBROUTINE GRIDST(NX,NY)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*
* Purpose:                                                                *
*         --  Generates the convolution function ("Sze Optl. 0.4") &      *
*             the grid-correction function for a map size of NXxNY        *
*             Generates the look-up tables  for  GRID and DEGRID          *
*		                                                          *
* Input:      NX    : 1st dimension of the  map array                     *
*             NY    : 2nd     "      "  "    "    "                       *
*                                                                         *
* Output:     The convolution and grid correction functions are passed    *
*             down to the other routines by an internal common block.     *
* External calls:                                                         *
*             VALLOC: Dynamic memory allocator                            *
*                     Requests storage for NX+NY reals                    *
*-------------------------------------------------------------------------*
      INCLUDE 'VLBGRD.INC'
      INTEGER VALLOC
      EXTERNAL VALLOC
* Gridding function
      CALL CONVFN(PSF,NPTS)
* Grid corrector
* Memory allocation for grid correction function
      KBCORX=VALLOC(NX,HOOK)
      CALL CORFUN(PSF,NCELS,NPCEL,NPTS,NX,HOOK(KBCORX))
      IF (NX.NE.NY) THEN
        KBCORY=VALLOC(NY,HOOK)
        CALL CORFUN(PSF,NCELS,NPCEL,NPTS,NY,HOOK(KBCORY))
      ELSE
        KBCORY=KBCORX
      ENDIF
* Look-up tables for internal gridding/degridding loop
      L=1
      DO 1 I=0,2*NCELS-1
        DO 2 J=0,2*NCELS-1
          LOOKPU(L)=I*NPCEL
          LOOKPV(L)=J*NPCEL
          LOOK(L)=J+NY*I
          L=L+1
    2   CONTINUE
    1 CONTINUE    
      END

      SUBROUTINE GRDSET(UV,LUV,LPUV,WEIGHT,NDATA,NU,NV)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*
* Purpose:     Sets up look-up tables for the gridder/degridder for
*              a transposed, complex Fourier grid UVGRID(NV,NU)
*              centred at u=NU/2+1,v=NV/2+1
*
*      INPUT:  UV     : 'u' and 'v' co-ordinates of a visibility (grid-pts.)
*              NDATA  : Number of u-v points given
*              NU     : u-dimension of UVGRID 
*              NV     : v-dimension of UVGRID 
*
*     OUTPUT:  LUV    : look-up table to store the first grid point of  
*                       UVGRID accessed by this u-v point (LUV(2,I) not used)
*              LPUV   : look-up table for gridding function used by this
*                       u-v point (LPUV(1,I)=>u ; LPUV(2,I)=>v)
*              WEIGHT : Accuracy flag - no gridding is this is .LE. 0. 
*
*       NOTE:  Make sure that U,V does not lie within 3 grid-points
*              of the edge of the Fourier grid:
*              I'm sure I don't have to tell you how important 
*              this remark is!
*                ie.   ABS(U) < NU/2-3    and    ABS(V) < NV/2-3
*--------------------------------------------------------------------------*
      REAL WEIGHT(2,NDATA)
      INCLUDE 'VLBGRD.INC'
      REAL UV(2,NDATA)
      INTEGER LUV(2,NDATA),LPUV(2,NDATA)
      DO 1 I=1,NDATA
        IF(WEIGHT(1,I).NE.0.) THEN
          CALL GRID0(UV(1,I),LPUV(1,I),IU0)
          IU0=IU0+NU/2
          CALL GRID0(UV(2,I),LPUV(2,I),JV0)
          JV0=JV0+NV/2
          LUV(1,I)=1+JV0+IU0*NV
          LUV(2,I)=0
        ENDIF
    1 CONTINUE
      END

      SUBROUTINE GRID(UV,DATA,LUV,LPUV,WEIGHT,UVGRID,NDATA)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*
* Purpose:     Splodges  visibilities onto the rectangular,
*              transposed, complex Fourier grid UVGRID(NV,NU)
*              centred at u=NU/2+1,v=NV/2+1
*
*      INPUT:  UV     : 'u' and 'v' co-ordinates of a visibility (grid-pts.)
*              DATA   : array of visibilities (COMPLEX)
*              LUV    : look-up table to store the first grid point of  
*                       UVGRID accessed by this u-v point (LUV(2,I) not used)
*              LPUV   : look-up table for gridding function used by this
*                       u-v point (LPUV(1,I)=>u ; LPUV(2,I)=>v)
*              WEIGHT : Accuracy flag - no gridding if this is .LE. 0. 
*              UVGRID : COMPLEX (square) array in the 'uv' plane
*              NDATA  : Number of u-v points given
*
*     OUTPUT:  UVGRID : returns with visibilities suitably injected
*
*       NOTE:  Make sure that U,V does not lie within 3 grid-points
*              of the edge of the Fourier grid:
*              I'm sure I don't have to tell you how important 
*              this remark is!
*                ie.   ABS(U) < NU/2-3    and    ABS(V) < NV/2-3
*--------------------------------------------------------------------------
      COMPLEX UV(NDATA),DATA(NDATA),UVGRID(*)
      REAL WEIGHT(2,NDATA)
      INTEGER LUV(2,NDATA),LPUV(2,NDATA)
      INCLUDE 'VLBGRD.INC'
      DO 1 I=1,NDATA
        IF(WEIGHT(1,I).GT.0.) THEN
C$DIR NO_RECURRENCE
          DO 2 J=1,N2CELS
            UVGRID(LUV(1,I)+LOOK(J))=UVGRID(LUV(1,I)+LOOK(J))+
     *      PSF(LPUV(1,I)+LOOKPU(J))*PSF(LPUV(2,I)+LOOKPV(J))*DATA(I)
    2     CONTINUE
        ENDIF
    1 CONTINUE
      END

      SUBROUTINE DEGRID(UV,DATA,LUV,LPUV,WEIGHT,UVGRID,NDATA)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*
* Purpose:     Degrids visibilities from the rectangular,
*              transposed, complex Fourier grid UVGRID(NV,NU)
*              centred at u=NU/2+1,v=NV/2+1
*
*      INPUT:  UV     : 'u' and 'v' co-ordinates of a visibility (grid-pts.)
*              LUV    : look-up table to store the first grid point of  
*                       UVGRID accessed by this u-v point (LUV(2,I) not used)
*              LPUV   : look-up table for gridding function used by this
*                       u-v point (LPUV(1,I)=>u ; LPUV(2,I)=>v)
*              WEIGHT : Accuracy flag - no degridding if this is .LE. 0. 
*              UVGRID : COMPLEX (square) array in the 'uv' plane
*              NDATA  : Number of u-v points given
*
*     OUTPUT:  DATA   : array of visibilities degridded (COMPLEX)
*
*       NOTE:  Make sure that U,V does not lie within 3 grid-points
*              of the edge of the Fourier grid:
*                ie.   ABS(U) < NU/2-3    and    ABS(V) < NV/2-3
*--------------------------------------------------------------------------
      COMPLEX UV(NDATA),DATA(NDATA),UVGRID(*)
      REAL WEIGHT(2,NDATA)
      INTEGER LUV(2,NDATA),LPUV(2,NDATA)
      INCLUDE 'VLBGRD.INC'
      DO 1 I=1,NDATA
        IF(WEIGHT(1,I).NE.0.) THEN
          DATA(I)=(0.,0.)
          DO 2 J=1,N2CELS
             DATA(I)=DATA(I)+UVGRID(LUV(1,I)+LOOK(J))*
     *       PSF(LPUV(1,I)+LOOKPU(J))*PSF(LPUV(2,I)+LOOKPV(J))
    2     CONTINUE
        ENDIF
    1 CONTINUE
      END

*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*
* Purpose:             Applies the grid correction function to the map.
*
*      INPUT:  MAP    : REAL array of NXxNY --  map with the
*                       map-centre at (NX/2+1,NY/2+1)
*              NX     : 1st dimension of the array MAP
*              NY     : 2nd     "      "  "    "    " 
*      Internal Common Block /VLBGRD/
*              CORRX  : REAL x-grid correction
*              CORRY  : REAL y-grid correction
*
*     OUTPUT:  MAP    : Map with the grid correction applied.
*------------------------------------------------------------------------*
      SUBROUTINE GRIDCR(MAP,NX,NY)
      REAL  MAP(*)
      INCLUDE 'VLBGRD.INC'
      CALL GRDCR1(MAP,NX,NY,HOOK(KBCORX),HOOK(KBCORY))
      END

      SUBROUTINE GRDCR1(MAP,NX,NY,CORRX,CORRY)
* Indirection to use dynamic memory
      REAL  MAP(*),CORRX(NX),CORRY(NY)
      K=1
      DO 20 I=1,NY
        DO 10 J=1,NX
          MAP(K)=MAP(K)*CORRX(J)*CORRY(I)
          K=K+1
  10    CONTINUE
  20  CONTINUE
      END

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Internal routines used to set up the gridder
* Not for external use!
*-----------------------------------------------------------------------
      SUBROUTINE GRID0(X,IXPSF,IX0)
* Internal routine for GRDSET
      INCLUDE 'VLBGRD.INC'

      XIN=NINT(X-0.5)
      XOFSET=1.-X+XIN
      IXPSF=INT(XOFSET*NPCEL)+1
      IX0=XIN-NCELS+1
      END

      SUBROUTINE CONVFN(PSF,NPTS)
* Make Gridding function via rational function approximation
* Store in array PSF
      REAL                        PSF(-NPTS:NPTS)
      REAL                        XNUM(5,3),XDEN(3,3)
      EXTERNAL                    PSFVAL
      DATA (XNUM(I,1),I=1,5) / 5.1006074E-01,1.7719075E-01,
     *                        -2.3778674E-01,6.8586842E-02,
     *                        -2.1577950E-02/
      DATA (XDEN(I,1),I=1,3) / 1.0000000E+00,1.0418408E+00,
     *                         4.6759292E-02/
      DATA (XNUM(I,2),I=1,5) / 5.2176148E-02,-2.9427656E-02,
     *                         6.5252421E-03,5.5823299E-05,
     *                        -2.9986344E-05/
      DATA (XDEN(I,2),I=1,3) / 1.0000000E+00,2.7330452E-01,
     *                         2.2523453E-02/
      DATA (XNUM(I,3),I=1,5) / 2.8390020E-05,-3.6229083E-04,
     *                         2.9081245E-05,-9.7011646E-05,
     *                        -3.0482903E-06/
      DATA (XDEN(I,3),I=1,3) / 1.0000000E+00,2.1539373E-01,
     *                         1.2881806E-02/
C
      XNORM=1.0/FLOAT(NPTS)
      DO 10 I=0,NPTS
        TEMP=3.0*FLOAT(I)*XNORM
        T2=TEMP*TEMP
        IF (TEMP.GE.0.0 .AND. TEMP.LT.1.0) THEN
          Y1=PSFVAL(XNUM(1,1),T2-1.0,5)
          Y2=PSFVAL(XDEN(1,1),T2-1.0,3)
        ELSEIF (TEMP.GE.1.0 .AND. TEMP.LT.2.0) THEN
          Y1=PSFVAL(XNUM(1,2),T2-4.0,5)
          Y2=PSFVAL(XDEN(1,2),T2-4.0,3)
        ELSE
          Y1=PSFVAL(XNUM(1,3),T2-9.0,5)
          Y2=PSFVAL(XDEN(1,3),T2-9.0,3)
        ENDIF
        TEMP=Y1/Y2
        PSF(I)=TEMP
        PSF(-I)=TEMP
  10  CONTINUE
      END

      FUNCTION PSFVAL(PSFW,XX,N)
* Internal routine used by CONVFN
      REAL PSFW(*)
      Y=0.0
      DO 10 J=N,1,-1
  10   Y=Y*XX+PSFW(J)
      PSFVAL=Y
      END

      SUBROUTINE CORFUN(PSF,NCELS,NPCEL,NPTS,NCORR,CORR)
* Make the grid correction function as the inverse of the DFT 
* of the Gridding function
      REAL            COSDIF(0:10),PSF(-NPTS:NPTS),CORR(*)
      DATA   TWOPI   /6.283185307/
 
      NCORR2=NCORR/2
      NPCEL2=NPCEL/2
      XNORM=TWOPI/FLOAT(NCORR)
      DO 60 J=0,NCORR2
        TWOPIT=FLOAT(J)*XNORM
        DO 10 I=0,2*NCELS-1
  10      COSDIF(I)=COS(TWOPIT*I)
        SUMN=0.0
        SUMD=0.0
        DO 50 K=-NCELS+1,NCELS
          KCEL=K*NPCEL
          DO 30 L=-NCELS+1,NCELS
            LCEL=L*NPCEL
            TEMP1=0.5*(PSF(KCEL)*PSF(LCEL)+
     *                 PSF(KCEL-NPCEL2)*PSF(LCEL-NPCEL2))
            DO 20 IDELTA=1,NPCEL2-1
  20          TEMP1=TEMP1+PSF(KCEL-IDELTA)*PSF(LCEL-IDELTA)
            SUMD=SUMD+COSDIF(ABS(K-L))*TEMP1
  30      CONTINUE
          TEMP2=0.5*(PSF(KCEL)*COS(K*TWOPIT)+
     *               PSF(KCEL-NPCEL2)*COS((K-0.5)*TWOPIT))
          DO 40 IDELTA=1,NPCEL2-1
            DELTA=FLOAT(IDELTA)/FLOAT(NPCEL)
            TEMP2=TEMP2+PSF(KCEL-IDELTA)*COS((K-DELTA)*TWOPIT)
  40      CONTINUE
          SUMN=SUMN+TEMP2
  50    CONTINUE
        TEMP=SUMN/SUMD
        CORR(NCORR2-J+1)=TEMP
        IF (J.NE.NCORR2) CORR(NCORR2+J+1)=TEMP
  60  CONTINUE
      END
