*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*
* Transformation routines for VLBMEM                                         *
* SF Gull December 1989 CIT                                                  *
* Copyright (c) of this code rests with:                                     *
*                Dr. S.F. Gull                                               *
*        MRAO, Cavendish Laboratory, Madingley Road                          *
*              Cambridge CB3 0HE, U.K.                                       *
*                                                                            *
* The copyright owner grants the Owens Valley Radio Observatory, CalTech      *
* permission to modify this code, provided that the owner is notified        *
* of any such changes made, and that ownership of the modified code          *
* is granted to the original copyright owner.                                *
*----------------------------------------------------------------------------*

*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*
*                                                                            *
* Plan of OPUS:                                                              *
*        1) Pad map with zeroes to double size                               *
*        2) Grid-correct the map (for what you are about to do!)             *
*        3) FFT using FOURSG to conjugate symmetric transposed format        *
*        4) Make a full complex array in files <15> + <16>                   *
*        5) Degrid from the complex plane, the centre being at the           *
*           of the first row of <16>                                         *
* Plan of TROPUS:                                                            *
*        1) Zero the files <15> and <16>>                                    *
*        2) Grid the data into the full complex plane                        *
*        3) Collect up the full complex plane into packed format             *
*        4) Fourier transform to a real map                                  *
*        5) Grid-correction                                                  *
*        6) Collect the central part that you actually need                  *
*        NB: the full map is left in <16>                                    *
*----------------------------------------------------------------------------*
      SUBROUTINE OPSET
* Set up gridding functions and look-up tables
      INCLUDE 'MECOMP.INC'
      INCLUDE 'MECOMS.INC'
      INCLUDE 'OPCOMP.INC'
      NX=ND(1)
      NY=ND(2)
      CALL GRIDST(NX,NY)
      DO 1 I=1,NK
* Data files can be blocked
        CALL UREAD(22)
        CALL UREAD(40)
        CALL GRDSET(ST(KC(40)),ST(KC(39)),ST(KB(38)),ST(KC(22)),
     *              MK/2,NX,NY)
        CALL UWRITE(38)
        CALL UWRITE(39)
    1 CONTINUE
      CALL UINIT
      END

      SUBROUTINE OPUS(J,K)
      INCLUDE 'MECOMP.INC'
      INCLUDE 'MECOMS.INC'
      INCLUDE 'OPCOMP.INC'
      NX=ND(1)
      NY=ND(2)
* Copy to File 16 for degridding
* Pad with zeroes
      CALL VFILL(KB(16),0.,NX*NY)
      CALL OP3(ST(KB(J)),ST(KB(16)),NX,NY)
* Grid correction
      CALL GRIDCR(ST(KB(16)),NX,NY)
* Change phase centre after grid correction
* Slightly subtle - the phase centre was at the x=NX/2 and y=NY/2
* but is changed to be centred on x=0, y=NY/2 for convenience of gridding
* that accounts for a missing RXZAP before transformation
      CALL RYZAP(ST(KB(16)),NX,NY)
* Fourier transform 
      CALL FOURSG(ST(KB(16)),ST(KB(16)),ST(KB(15)),ND,1,0)
      CALL CYZAP(ST(KB(16)),NX,NY)
      CALL CXZAP(ST(KB(16)),NX,NY)
* Expand to full complex from transposed, packed format
* Requires unit 15 to preceed unit 16
      CALL FTCNJG(ST(KB(16)),ST(KB(15)),NX,NY)
* Blocked degridding
      DO 1 I=1,NK
* Data files can be blocked, but <15> can't be
        CALL UREAD(22)
        CALL UREAD(38)
        CALL UREAD(39)
        CALL UREAD(40)
        CALL DEGRID(ST(KC(40)),ST(KC(K)),ST(KC(39)),ST(KC(38)),
     *           ST(KC(22)),ST(KB(15)),MK/2)
        CALL UWRITE(K)
    1 CONTINUE
      CALL UINIT
      END

      SUBROUTINE OP3(X,Y,NX,NY)
* Pad a map to double size with zeroes
      REAL X(NX/2,NY/2),Y(NX,NY)
      DO 1 I=1,NY/2
        I1=I+NY/4
        DO 2 J=1,NX/2
          J1=J+NX/4
          Y(J1,I1)=X(J,I)
    2   CONTINUE
    1 CONTINUE
      END

      SUBROUTINE TROPUS(J,K)
      INCLUDE 'MECOMP.INC'
      INCLUDE 'MECOMS.INC'
      INCLUDE 'OPCOMP.INC'
      NX=ND(1)
      NY=ND(2)
* Zero the gridding array (UNIT 15), which must be in memory
      CALL VFILL(KB(15),0.,2*NX*NY)
      DO 1 I=1,NK
* Blocked gridding
        CALL UREAD(22)
        CALL UREAD(38)
        CALL UREAD(39)
        CALL UREAD(40)
        CALL UREAD(J)
        CALL GRID(ST(KC(40)),ST(KC(J)),ST(KB(39)),ST(KB(38)),
     *    ST(KC(22)),ST(KB(15)),MK/2)
    1 CONTINUE
      CALL UINIT
* Fix up full complex into transposed, packed format
      CALL FTPACK(ST(KB(16)),ST(KB(15)),NX,NY)
* Fourier transform, changing phase centre for grid correction (see OPUS)
      CALL CYZAP(ST(KB(16)),NX,NY)
      CALL CXZAP(ST(KB(16)),NX,NY)
      CALL FOURSG(ST(KB(16)),ST(KB(16)),ST(KB(15)),ND,-1,-1)
      CALL RYZAP(ST(KB(16)),NX,NY)
* Grid correction
      CALL GRIDCR(ST(KB(16)),NX,NY)
* Correct for Fourier inverse, rather than transpose
      CALL VSMUL(KB(16),0.5,KB(16),NX*NY)
* Copy central 1/2 of map to destination
      CALL TROP3(ST(KB(16)),ST(KB(K)),NX,NY)
      END

      SUBROUTINE TROP3(Y,X,NX,NY)
* Select the central part of a map
      REAL X(NX/2,NY/2),Y(NX,NY)
      DO 1 I=1,NY/2
        I1=I+NY/4
        DO 2 J=1,NX/2
          J1=J+NX/4
          X(J,I)=Y(J1,I1)
    2   CONTINUE
    1 CONTINUE
      END
C
      SUBROUTINE MEMEX(X,Y,Z)
* Dummy routine for nonlinear data
      END

      SUBROUTINE FTCNJG(X,Y,NX,NY)
* Make a full complex array from a packed conjugate symmetric one.
      COMPLEX X(NY,*),Y(NY,*)
* The fully complex lines x=1,NX/2-1
      DO 1 I=2,NX/2
        I2=NX/2+2-I
        Y(1,I2)=CONJG(X(1,I))
        DO 2 J=2,NY
          Y(NY+2-J,I2)=CONJG(X(J,I))
    2   CONTINUE
    1 CONTINUE
* The x=NX/2 line
      DO 3 J=2,NY/2
        Y(J,1)=X(J+NY/2,1)
        Y(NY+2-J,1)=CONJG(X(J+NY/2,1))
    3 CONTINUE
* The x=0 line
      DO 4 J=2,NY/2
        X(NY+2-J,1)=CONJG(X(J,1))
    4 CONTINUE
* and finally the reals
      Y(1,1)=CMPLX(REAL(X(NY/2+1,1)),0.)
      Y(NY/2+1,1)=CMPLX(AIMAG(X(NY/2+1,1)),0.)
      X(NY/2+1,1)=CMPLX(AIMAG(X(1,1)),0.)
      X(1,1)=CMPLX(REAL(X(1,1)),0.)
      END

      SUBROUTINE FTPACK(X,Y,NX,NY)
* Collect a full complex array into packed format
      COMPLEX X(NY,*),Y(NY,*)
* The fully complex lines x=1,NX/2-1
      DO 1 I=2,NX/2
        I2=NX/2+2-I
        X(1,I)=X(1,I)+CONJG(Y(1,I2))
        DO 2 J=2,NY
          X(J,I)=X(J,I)+CONJG(Y(NY+2-J,I2))
    2   CONTINUE
    1 CONTINUE
* The x=0 line
      DO 4 J=2,NY/2
        X(J,1)=X(J,1)+CONJG(X(NY+2-J,1))
    4 CONTINUE
* The x=NX/2 line
      DO 3 J=2,NY/2
        X(J+NY/2,1)=Y(J,1)+CONJG(Y(NY+2-J,1))
    3 CONTINUE
* and finally the reals
      X(1,1)=2.*CMPLX(REAL(X(1,1)),REAL(X(NY/2+1,1)))
      X(NY/2+1,1)=2.*CMPLX(REAL(Y(1,1)),REAL(Y(NY/2+1,1)))
      END

      SUBROUTINE RXZAP(X,NX,NY)
* Change phase centre in x (real array)
      REAL X(*)
      DO 1 I=2,NX*NY,2
        X(I)=-X(I)
    1 CONTINUE
      END

      SUBROUTINE RYZAP(X,NX,NY)
* Change phase centre in y (real array)
      REAL X(*)
      DO 1 I=NX+1,NX*NY,2*NX
        DO 2 J=I,I+NX-1
          X(J)=-X(J)
    2 CONTINUE
    1 CONTINUE
      END

      SUBROUTINE CYZAP(X,NX,NY)
* Change phase centre in y (complex array)
* Note that the array is transposed
      COMPLEX X(*)
      DO 1 I=2,NX*NY/2,2
        X(I)=-X(I)
    1 CONTINUE
      END

      SUBROUTINE CXZAP(X,NX,NY)
* Change phase centre in x (complex array)
* Note that the array is transposed
      COMPLEX X(*)
      DO 1 I=NY+1,NX*NY/2,2*NY
        DO 2 J=I,I+NY-1
          X(J)=-X(J)
    2 CONTINUE
    1 CONTINUE
      END
