      SUBROUTINE WRFITS(TYPE, NX, NY, RDATA, FILE)
      INTEGER TYPE, NX, NY
      REAL RDATA(NX*NY)
      CHARACTER*(*) FILE
C
C Write a data array as a FITS image.
C
C Arguments:
C  TYPE (input)   : 0 => VLBMEM image
C                   1 => dirty map
C                   2 => dirty beam
C                   3 => dirty residuals
C  NX, NY (input) : number of pixels
C  RDATA (input)  : data array
C  FILE  (in/out) : file name
C-----------------------------------------------------------------------
      REAL       PI,RPDEG
      PARAMETER  (PI=3.1415926536D0)
      PARAMETER  (RPDEG=PI/180D0)
C
      INTEGER JUNK, IER, IPIX, NPIX, BITPIX, NAXIS, NAXISN(4)
      INTEGER ID, IMON, IY, I, L
      INTEGER LEN1
      REAL MAPMAX, MAPMIN, RMAX, BSCALE, BZERO, PIX1, PIX2
      CHARACTER*8  DATMAP, BUNIT
      CHARACTER*80 MESSAG, STRING
      CHARACTER*255 ACTUAL, FILEV
      INCLUDE      'MEFITS.INC'
C-----------------------------------------------------------------------
C Invent a new file name if necessary.
C
      CALL NEWFIL(FILE, FILEV)
C
C Open output file.
C
      CALL FITS_INIT(FILEV,ACTUAL,L,IER)
      IF (IER.NE.0) THEN
          CALL PUTOUT('Error writing file: '//FILEV)
          RETURN
      ELSE
          CALL PUTOUT('Writing file '//ACTUAL(1:L))
      END IF
C
C Find map maximum.
C
      NPIX = NX*NY
      MAPMAX = RDATA(1)
      MAPMIN = RDATA(1)
      DO 155 IPIX=1,NPIX
          MAPMAX = MAX(MAPMAX, RDATA(IPIX))
          MAPMIN = MIN(MAPMIN, RDATA(IPIX))
  155 CONTINUE
C
C Header and scaling parameters.
C
      BITPIX = 32
      NAXIS = 4
      NAXISN(1) = NX
      NAXISN(2) = NY
      NAXISN(3) = 1
      NAXISN(4) = 1
      BZERO = 0.0
      RMAX = 32000.
      IF (BITPIX.EQ.32) RMAX = 2E9
      BSCALE = MAX(ABS(MAPMAX),ABS(MAPMIN))/RMAX
      CALL IDATE(IMON,ID,IY)
      WRITE (DATMAP,1000) ID,IMON,IY
 1000 FORMAT(I2.2,'/',I2.2,'/',I2.2)
      IF (TYPE.EQ.0) THEN
          BUNIT = 'JY'
      ELSE
          BUNIT = 'JY/BEAM'
      END IF
      CALL FITS_HSTAN(BITPIX,NAXIS,NAXISN,0D0,0D0,IER)
      CALL FITS_WLOG ('BLOCKED', .TRUE., 'Tape may be blocked',IER)
      CALL FITS_WREAL('BSCALE', BSCALE, 
     1                'Real = Tape*BSCALE + BZERO',IER)
      CALL FITS_WREAL('BZERO', BZERO, ' ', IER)
      CALL FITS_WSTR ('BUNIT', BUNIT, 'Brightness units', IER)
      CALL FITS_WSTR ('OBJECT', HSNAME, 'Object name', IER)
      CALL FITS_WSTR ('INSTRUME', 'VLBI', ' ', IER)
      L = LEN1(HTELES)
      CALL FITS_WSTR ('TELESCOP', HTELES(1:L), ' ', IER)
      CALL FITS_WSTR ('DATE-OBS', HDATOB,
     1                'Date of observation dd/mm/yy', IER)
      CALL FITS_WSTR ('DATE-MAP', DATMAP, 'Date map created', IER)
      CALL FITS_WSTR ('DATE',     DATMAP, 'Date map created', IER)
      CALL FITS_WSTR ('OBSERVER', HOBSER, ' ', IER)
      CALL FITS_WSTR ('ORIGIN', 'VLBMEM (Caltech)', ' ', IER)
      CALL FITS_WREAL('DATAMAX', MAPMAX, 'Map maximum '//BUNIT,
     1                IER)
      CALL FITS_WREAL('DATAMIN', MAPMIN, 'Map minimum '//BUNIT,
     1                IER)
      CALL FITS_WREAL('EPOCH',   HEPOCH, 'Epoch of RA/Dec', IER)
C
C Axis 1 - RA.
C
       PIX1 = NX/2+1
       STRING = 'RA  = '
       CALL PANGLE('H', HRA0, STRING(6:23), 4)
       CALL FITS_WSTR ('CTYPE1',   'RA---SIN',
     1 '--- Axis 1: Right Ascension ---', IER)
       CALL FITS_WDBLE('CRVAL1',   HRA0/RPDEG, STRING, IER)
       CALL FITS_WREAL('CRPIX1',   PIX1,      ' ', IER)
       CALL FITS_WREAL('CDELT1',   -HXINT,    ' ', IER)
       CALL FITS_WREAL('CROTA1',   0.0,       ' ', IER)
C
C Axis 2 - DEC.
C
       PIX2 = NY/2+1
       STRING = 'Dec = '
       CALL PANGLE('D', HDEC0, STRING(6:22), 3)
       CALL FITS_WSTR ('CTYPE2',   'DEC--SIN',
     1 '--- Axis 2: Declination ---', IER)
       CALL FITS_WDBLE('CRVAL2',   HDEC0/RPDEG, STRING, IER)
       CALL FITS_WREAL('CRPIX2',   PIX2,      ' ', IER)
       CALL FITS_WREAL('CDELT2',   HYINT,     ' ', IER)
       CALL FITS_WREAL('CROTA2',   0.0,       ' ', IER)
C
C Axis 3 - Frequency (degenerate).
C
       CALL FITS_WSTR ('CTYPE3',   'FREQ',
     1 '--- Axis 3: Frequency ---', IER)
       CALL FITS_WREAL('CRVAL3',   HFREQ,      ' ', IER)
       CALL FITS_WREAL('CRPIX3',   1.0,       ' ', IER)
       CALL FITS_WREAL('CDELT3',   0.0,       ' ', IER)
       CALL FITS_WREAL('CROTA3',   0.0,       ' ', IER)
C
C Axis 4 - Stokes parameter (degenerate).
C
      CALL FITS_WSTR ('CTYPE4',   'STOKES',
     1    '--- Axis 4: Stokes parameter ---', IER)
      IF (TYPE.EQ.0) THEN
          CALL FITS_WREAL('CRVAL4',   1.0,   'VLBMEM image', IER)
      ELSE IF (TYPE.EQ.1) THEN
          CALL FITS_WREAL('CRVAL4',   1.0,   'Dirty map', IER)
      ELSE IF (TYPE.EQ.2) THEN
          CALL FITS_WREAL('CRVAL4',   0.0,   'Dirty beam', IER)
      ELSE
          CALL FITS_WREAL('CRVAL4',   1.0,   'Dirty residuals', IER)
          CALL FITS_WCMT('HISTORY', 'AIPS PRODUCT=3   / Residuals',
     1         IER)
      END IF
      CALL FITS_WREAL('CRPIX4',   1.0,       ' ', IER)
      CALL FITS_WREAL('CDELT4',   1.0,       ' ', IER)
      CALL FITS_WREAL('CROTA4',   0.0,       ' ', IER)
C
C History.
C
      DO 200 I=1,LEN(MESSAG)
          MESSAG(I:I) = '-'
  200 CONTINUE
      CALL FITS_WCMT('HISTORY', MESSAG, IER)
      CALL FITS_HIS(IER)
C
C End FITS header; write FITS data.
C
      CALL FITS_WEND(IER)
      CALL FITS_WRAYF(RDATA, NPIX, BITPIX, BSCALE, BZERO, IER)
      CALL FITS_CLOSE(IER)
      IF (IER.NE.0) CALL ERROR(
     1          'Something went wrong writing FITS file.')
C-----------------------------------------------------------------------
      END
