      SUBROUTINE FITIN (INUNIT,AMAP,MAXPIX,STATUS,ERROR)
C-----------------------------------------------------------------------
C FITIN: read a FITS-format disk file.
C
C     Parameters -   (">" input, "<" output )
C
C     (>) INUNIT  (Integer) Fortran unit number for the input file.
C                 The file must have been opened using a Fortran
C                 open statement, and FITIN assumes the file is
C                 positioned at the start of the header record(s).
C     (<) AMAP    (Real array) Array to contain the map pixels.
C     (>) MAXPIX  (Integer) Dimension of AMAP.
C     (<) STATUS  (Integer) A status code for the read.
C                 0 => OK, 1 => error
C     (<) ERROR   (Character) Returns with a message describing
C                 the error.
C
C     Input -
C The file must be FITS standard, and must begin 'SIMPLE=T'.
C It must specify NAXIS, and BITPIX, which must be one of 8, 16, 32,
C -32.
C
C     The program is fairly tolerant - it will ignore 'cards'
C     that it cannot understand.
C
C Revised: 23-Dec-1991
C Revised: 06 Sep 2000 RLM . Add byte swap for BITPIX = -32
C-----------------------------------------------------------------------
C     IMPLICIT NONE
C
      INTEGER INUNIT,MAXPIX,STATUS
      REAL AMAP(MAXPIX)
      CHARACTER*(*) ERROR
      INCLUDE 'mapcom.inc'
      LOGICAL BITP, SIMPLE
      REAL BSCALE,BZERO
      REAL*8 RVAL
      INTEGER I,N,ISTAT,BITPIX,BYTPIX,TYPE,IPIX
      INTEGER PIXELS,PIXREC,ITEMS,IVAL
      CHARACTER HEADER*2880, KEYW*8, CVAL*72
C
C            BUFFER, BUFFL, and BUFFW are the mag tape input buffer.
C
      BYTE BUFFER(2880)
      INTEGER*2 BUFFW(1440)
      INTEGER*4 BUFFL(720)
      REAL*4    BUFFR(720)
      EQUIVALENCE (BUFFER, BUFFW, BUFFL, BUFFR)
C-----------------------------------------------------------------------
C
C            Initial values
C
      STATUS = 1
      MPNDM = 0
      MPNIT = 0
      MPOBJ = ' '
      MPTEL = ' '
      MPINS = ' '
      MPOBS = ' '
      MPDOB = ' '
      MPDMP = ' '
      MPBUN = ' '
      MPTYP = 'MAP'
      MPCRT = ' '
      MPBMJ = 0D0
      MPBMN = 0D0
      MPBPA = 0D0
      MPEPO = 0D0
      MPFRQ = 0D0
      MPBWD = 0D0
      DO 100 I=1,7
          MPSIZ(I) = 1
          MPCTY(I) = ' '
          MPRV(I) = 0D0
          MPRP(I) = 0D0
          MPIN(I) = 1D0
          MPRO(I) = 0D0
  100 CONTINUE
      MPCTY(1) = 'x'
      MPCTY(2) = 'y'
      MPCTY(3) = 'z'
      BSCALE = 1.
      BZERO = 0.
      BITP = .FALSE.
      SIMPLE = .FALSE.
      MPNDM = -1
C
C            Read in the next header record.
C
  110 CALL RDFITC(INUNIT, ISTAT, HEADER)
      IF (ISTAT.NE.0) THEN
          ERROR = 'File has wrong format for a FITS file'
          GOTO 550
      END IF
C
C            Decode the header records.
C
      DO 120 I=0,35
          CALL FITKEY(HEADER(80*I+1:80*I+80), KEYW, TYPE, RVAL, CVAL)
          IF (KEYW.EQ.'END') GOTO 320
          IF (KEYW.EQ.'SIMPLE') THEN
            IF (CVAL.NE.'T') GOTO 560
            SIMPLE = .TRUE.
          ELSE IF (KEYW.EQ.'BITPIX') THEN
            BITPIX = RVAL
            BYTPIX = ABS(BITPIX)/8
            IF ((BITPIX.NE.8) .AND. (BITPIX.NE.16) .AND.
     1          (BITPIX.NE.32) .AND. (BITPIX.NE.-32) ) GOTO 560
            BITP = .TRUE.
          ELSE IF (KEYW.EQ.'NAXIS') THEN
            MPNDM = RVAL
            IF (MPNDM.LT.0 .OR. MPNDM.GT.7) GOTO 560
          ELSE IF (KEYW(1:5).EQ.'NAXIS') THEN
            N = ICHAR(KEYW(6:6)) - ICHAR('0')
            IF (N.LT.1 .OR. N.GT.MPNDM) GOTO 560
            MPSIZ(N) = RVAL
          ELSE IF (KEYW(1:5).EQ.'CRVAL') THEN
            N = ICHAR(KEYW(6:6)) - ICHAR('0')
            IF (N.LT.1 .OR. N.GT.MPNDM) GOTO 560
            MPRV(N) = RVAL
          ELSE IF (KEYW(1:5).EQ.'CRPIX') THEN
            N = ICHAR(KEYW(6:6)) - ICHAR('0')
            IF (N.LT.1 .OR. N.GT.MPNDM) GOTO 560
            MPRP(N) = RVAL
          ELSE IF (KEYW(1:5).EQ.'CDELT') THEN
            N = ICHAR(KEYW(6:6)) - ICHAR('0')
            IF (N.LT.1 .OR. N.GT.MPNDM) GOTO 560
            MPIN(N) = RVAL
          ELSE IF (KEYW(1:5).EQ.'CTYPE') THEN
            N = ICHAR(KEYW(6:6)) - ICHAR('0')
            IF (N.LT.1 .OR. N.GT.MPNDM) GOTO 560
            MPCTY(N) = CVAL
          ELSE IF (KEYW(1:5).EQ.'CROTA') THEN
            N = ICHAR(KEYW(6:6)) - ICHAR('0')
            IF (N.LT.1 .OR. N.GT.MPNDM) GOTO 560
            MPRO(N) = RVAL
          ELSE IF (KEYW.EQ.'BSCALE') THEN
            BSCALE = RVAL
          ELSE IF (KEYW.EQ.'BZERO') THEN
            BZERO = RVAL
          ELSE IF (KEYW.EQ.'OBJECT') THEN
            MPOBJ = CVAL
          ELSE IF (KEYW.EQ.'TELESCOP') THEN
            MPTEL = CVAL
          ELSE IF (KEYW.EQ.'INSTRUME') THEN
            MPINS = CVAL
          ELSE IF (KEYW.EQ.'OBSERVER') THEN
            MPOBS = CVAL
          ELSE IF (KEYW.EQ.'DATE-OBS') THEN
            MPDOB = CVAL
          ELSE IF (KEYW.EQ.'DATE-MAP') THEN
            MPDMP = CVAL
          ELSE IF (KEYW.EQ.'BUNIT') THEN
            MPBUN = CVAL
          ELSE IF (KEYW.EQ.'BMAJ') THEN
            MPBMJ = RVAL
          ELSE IF (KEYW.EQ.'BMIN') THEN
            MPBMN = RVAL
          ELSE IF (KEYW.EQ.'BPA') THEN
            MPBPA = RVAL
          ELSE IF (KEYW.EQ.'EPOCH') THEN
            MPEPO = RVAL
          ELSE IF (KEYW.EQ.'NITER') THEN
            MPNIT = RVAL
          ELSE IF (KEYW.EQ.'ORIGIN') THEN
            MPCRT = CVAL
          END IF
  120 CONTINUE
      GOTO 110
C
C            Check for required parameters.
C
  320 CONTINUE
      IF (.NOT.SIMPLE) THEN
          KEYW = 'SIMPLE'
          GOTO 570
      END IF
      IF (.NOT.BITP) THEN
          KEYW = 'BITPIX'
          GOTO 570
      END IF
      IF (MPNDM.LT.0) THEN
          KEYW = 'NAXIS'
          GOTO 550
      END IF
C
C            Look for frequency on an axis.
C
      DO 140 I=1,MPNDM
          IF (MPCTY(I).EQ.'FREQ') MPFRQ = MPRV(I)*1E-6
  140 CONTINUE
C
C            Compute size of map.
C
      PIXELS = 0
      IF (MPNDM.GT.0) THEN
          PIXELS = 1
          DO 150 I=1,MPNDM
            PIXELS = PIXELS*MPSIZ(I)
  150     CONTINUE
      END IF
      IF (PIXELS.LE.0) THEN
          ERROR = 'Map contains no pixels'
          GOTO 550
      ELSE IF (PIXELS.GT.MAXPIX) THEN
          ERROR = 'Map too big'
          GOTO 550
      END IF
C
C            Read in the data records, converting to REAL*4 format.
C            Number of pixels to be read = PIXELS, each consisting of
C            BYTPIX bytes.
C
      IPIX = 0
      PIXREC = 2880/BYTPIX
C
C            So long as there are pixels left to be read in, read in
C            a record. Swap bytes as they are read in and convert to
C            REAL. Note kluge (cloodge?) for dealing with BITPIX=8
C            because VAX treats BYTE variables as SIGNED.
C
C
C     Add Byte swap for BITPIX = -32 for Linux/PC (RLM, 5 Sep 2000)
      DO WHILE (PIXELS.GT.0)
          CALL RDFITS(INUNIT, ISTAT, BUFFER)
          IF (ISTAT.NE.0) THEN
            ERROR = 'File has wrong format for a FITS file'
            GO TO 550
          END IF
          IF (BITPIX.EQ.16) THEN
            CALL FITS_BSWAP(BUFFL,1440)
          ELSE IF (BITPIX.EQ.32) THEN
            CALL FITS_WBSWAP(BUFFW,720)
          ELSE IF (BITPIX.EQ.-32) THEN
          CALL FITS_IEEER4(BUFFR,720)
          CALL FITS_WBSWAP(BUFFW,720)
          END IF
          ITEMS = MIN(PIXELS,PIXREC)
          IF (BITPIX.EQ.16) THEN
            DO 170 I=1,ITEMS
                IPIX = IPIX+1
                AMAP(IPIX) = BUFFW(I)*BSCALE + BZERO
  170       CONTINUE
          ELSE IF (BITPIX.EQ.32) THEN
            DO 180 I=1,ITEMS
                IPIX = IPIX+1
                AMAP(IPIX) = BUFFL(I)*BSCALE + BZERO
  180       CONTINUE
          ELSE IF (BITPIX.EQ.-32) THEN
            DO 190 I=1,ITEMS
                IPIX = IPIX+1
                AMAP(IPIX) = BUFFR(I)*BSCALE + BZERO
  190       CONTINUE
          ELSE IF (BITPIX.EQ.8) THEN
            DO 200 I=1,ITEMS
                IVAL = BUFFER(I)
                IF (IVAL.LT.0) IVAL = IVAL+256
                IPIX = IPIX+1
                AMAP(IPIX) = IVAL*BSCALE + BZERO
  200       CONTINUE
          END IF
          PIXELS = PIXELS-PIXREC
      END DO
C
C            All done: return.  STATUS=0 implies success,
C            STATUS=1 implies failure.
C
      STATUS = 0
      ERROR = ' '
  550 RETURN
C
C            Error returns.
C
  560 ERROR = 'Invalid parameter: '//KEYW//'='//CVAL
      GOTO 550
  570 ERROR = 'Required header parameter missing: '//KEYW
      GOTO 550
      END
